create schema if not exists sm_sc;
create schema if not exists sm_dat;
drop type if exists typ_arr_point;
create type typ_arr_point as
(
  point_id    varchar(64)      ,
  point_arr   float[]
);
-- 创建高维数据点类型，用于聚类计算
-- create extension cube
drop type if exists typ_arr_point_cube;
create type typ_arr_point_cube as
(
  point_id    varchar(64)      ,
  point_arr   cube
);

-- 1. 定义类型
drop type if exists sm_sc.typ_l_complex;
create type sm_sc.typ_l_complex as
(
  m_re         float,
  m_im         float
);

-- 2. 定义类型转换 --
-- 2.1. float
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_float_to_typ_l_complex(float);
create or replace function sm_sc.fv_cast_float_to_typ_l_complex
(
  i_re      float
)
returns sm_sc.typ_l_complex
as
$$
begin
  return (i_re, 0.0)::sm_sc.typ_l_complex;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_float_to_typ_l_complex(32.6 :: float)
-- ------------------------------------------------------------------------
drop cast if exists (float as sm_sc.typ_l_complex);
create cast (float as sm_sc.typ_l_complex) with function sm_sc.fv_cast_float_to_typ_l_complex(float) as implicit;
-- select 32.5 :: float :: sm_sc.typ_l_complex
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_typ_l_complex_to_float(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_cast_typ_l_complex_to_float
(
  i_complex      sm_sc.typ_l_complex
)
returns float
as
$$
begin
  if abs(i_complex.m_im) < 0.0000001
  then
    return i_complex.m_re;
  else
    return null;
    -- raise exception 'error!  can''t cast complex number to real number, while m_im of the complex number <> 0.0! ';
  end if;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_typ_l_complex_to_float((32.6 :: float, -56.2 :: float)::sm_sc.typ_l_complex)
-- select sm_sc.fv_cast_typ_l_complex_to_float((32.6 :: float, 0 :: float)::sm_sc.typ_l_complex)
-- ------------------------------------------------------------------------
drop cast if exists (sm_sc.typ_l_complex as float);
create cast (sm_sc.typ_l_complex as float) with function sm_sc.fv_cast_typ_l_complex_to_float(sm_sc.typ_l_complex) as assignment;
-- -- select (32.5 :: float, 54.2 :: float)::sm_sc.typ_l_complex::float
-- select (32.5 :: float, 0 :: float)::sm_sc.typ_l_complex::float
-- ------------------------------------------------------------------------
-- 2.2. decimal
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_decimal_to_typ_l_complex(decimal);
create or replace function sm_sc.fv_cast_decimal_to_typ_l_complex
(
  i_re      decimal
)
returns sm_sc.typ_l_complex
as
$$
begin
  return (i_re, 0.0)::sm_sc.typ_l_complex;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_decimal_to_typ_l_complex(32.6 :: decimal)
-- ------------------------------------------------------------------------
drop cast if exists (decimal as sm_sc.typ_l_complex);
create cast (decimal as sm_sc.typ_l_complex) with function sm_sc.fv_cast_decimal_to_typ_l_complex(decimal) as implicit;
-- select 32.5 :: decimal :: sm_sc.typ_l_complex
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_typ_l_complex_to_decimal(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_cast_typ_l_complex_to_decimal
(
  i_complex      sm_sc.typ_l_complex
)
returns decimal
as
$$
begin
  if abs(i_complex.m_im) < 0.0000001
  then
    return i_complex.m_re;
  else
    return null;
    -- raise exception 'error!  can''t cast complex number to real number, while m_im of the complex number <> 0.0! ';
  end if;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_typ_l_complex_to_decimal((32.6 :: decimal, -56.2 :: decimal)::sm_sc.typ_l_complex)
-- select sm_sc.fv_cast_typ_l_complex_to_decimal((32.6 :: decimal, 0 :: decimal)::sm_sc.typ_l_complex)
-- ------------------------------------------------------------------------
drop cast if exists (sm_sc.typ_l_complex as decimal);
create cast (sm_sc.typ_l_complex as decimal) with function sm_sc.fv_cast_typ_l_complex_to_decimal(sm_sc.typ_l_complex) as assignment;
-- -- select (32.5 :: decimal, 54.2 :: decimal)::sm_sc.typ_l_complex::decimal
-- select (32.5 :: decimal, 0 :: decimal)::sm_sc.typ_l_complex::decimal
-- ------------------------------------------------------------------------
-- 2.3. bigint
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_bigint_to_typ_l_complex(bigint);
create or replace function sm_sc.fv_cast_bigint_to_typ_l_complex
(
  i_re      bigint
)
returns sm_sc.typ_l_complex
as
$$
begin
  return (i_re, 0.0)::sm_sc.typ_l_complex;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_bigint_to_typ_l_complex(32 :: bigint)
-- ------------------------------------------------------------------------
drop cast if exists (bigint as sm_sc.typ_l_complex);
create cast (bigint as sm_sc.typ_l_complex) with function sm_sc.fv_cast_bigint_to_typ_l_complex(bigint) as implicit;
-- select 32 :: bigint:: sm_sc.typ_l_complex
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_typ_l_complex_to_bigint(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_cast_typ_l_complex_to_bigint
(
  i_complex      sm_sc.typ_l_complex
)
returns bigint
as
$$
begin
  if i_complex.m_im = 0.0
  then
    return i_complex.m_re;
  else
    raise exception 'error!  can''t cast complex number to real number, while m_im of the complex number <> 0.0! ';
  end if;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_typ_l_complex_to_bigint((32 :: bigint, -56 :: bigint)::sm_sc.typ_l_complex)
-- select sm_sc.fv_cast_typ_l_complex_to_bigint((32 :: bigint, 0 :: bigint)::sm_sc.typ_l_complex)
-- ------------------------------------------------------------------------
drop cast if exists (sm_sc.typ_l_complex as bigint);
create cast (sm_sc.typ_l_complex as bigint) with function sm_sc.fv_cast_typ_l_complex_to_bigint(sm_sc.typ_l_complex) as assignment;
-- -- select (32 :: bigint, 54 :: bigint)::sm_sc.typ_l_complex::bigint
-- select (32 :: bigint, 0)::sm_sc.typ_l_complex::bigint
-- ------------------------------------------------------------------------
-- 2.4. int
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_int_to_typ_l_complex(int);
create or replace function sm_sc.fv_cast_int_to_typ_l_complex
(
  i_re      int
)
returns sm_sc.typ_l_complex
as
$$
begin
  return (i_re, 0.0)::sm_sc.typ_l_complex;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_int_to_typ_l_complex(32 :: int)
-- ------------------------------------------------------------------------
drop cast if exists (int as sm_sc.typ_l_complex);
create cast (int as sm_sc.typ_l_complex) with function sm_sc.fv_cast_int_to_typ_l_complex(int) as implicit;
-- select 32 :: int:: sm_sc.typ_l_complex
-- ------------------------------------------------------------------------
drop function if exists sm_sc.fv_cast_typ_l_complex_to_int(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_cast_typ_l_complex_to_int
(
  i_complex      sm_sc.typ_l_complex
)
returns int
as
$$
begin
  if i_complex.m_im = 0.0
  then
    return i_complex.m_re;
  else
    raise exception 'error!  can''t cast complex number to real number, while m_im of the complex number <> 0.0! ';
  end if;
end
$$
language plpgsql volatile
cost 100;
-- select sm_sc.fv_cast_typ_l_complex_to_int((32 :: int, -56 :: int)::sm_sc.typ_l_complex)
-- select sm_sc.fv_cast_typ_l_complex_to_int((32 :: int, 0 :: int)::sm_sc.typ_l_complex)
-- ------------------------------------------------------------------------
drop cast if exists (sm_sc.typ_l_complex as int);
create cast (sm_sc.typ_l_complex as int) with function sm_sc.fv_cast_typ_l_complex_to_int(sm_sc.typ_l_complex) as assignment;
-- -- select (32 :: int, 54 :: int)::sm_sc.typ_l_complex::int
-- select (32 :: int, 0)::sm_sc.typ_l_complex::int
-- ------------------------------------------------------------------------
drop type if exists sm_sc.__typ_arr_float_ex;
create type sm_sc.__typ_arr_float_ex as
(
  m_arr_mid      float[],
  m_cnt          int
);
drop type if exists sm_sc.__typ_array_ex;
create type sm_sc.__typ_array_ex 
as
(
  m_array    float[]
);

-- drop function if exists sm_sc.fv_cast_array_ex;
create or replace function sm_sc.fv_cast_array_ex 
(
  i_array     float[]    
)
returns sm_sc.__typ_array_ex
as
$$
declare -- here
  v_rtn     sm_sc.__typ_array_ex;
begin
  v_rtn.m_array = i_array;
  return v_rtn;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_cast_array_ex(array[1.2, 2.4])
-- select sm_sc.fv_cast_array_ex(array[array[1.2, 2.4], array[3.6, 4.8]])
-- ---------------------------
drop cast if exists (float[] as sm_sc.__typ_array_ex);
create cast (float[] as sm_sc.__typ_array_ex)   
with function sm_sc.fv_cast_array_ex(float[])
as implicit;
drop type if exists sm_sc.__typ_l_complex_ex;
create type sm_sc.__typ_l_complex_ex as
(
  m_arr_mid      sm_sc.typ_l_complex,
  m_cnt          int
);
drop type if exists sm_sc.__typ_arr_cpx_ex;
create type sm_sc.__typ_arr_cpx_ex as
(
  m_arr_mid      sm_sc.typ_l_complex[],
  m_cnt          int
);
-- 创建全局 sequence 
-- 如果是分布式集群环境和并发访问场景，需对全局 sequence: seq_global 做分段规划、各节点设置独立的 minvalue 和 maxvalue
-- 该 seq 只能通过 sm_sc.fv_get_global_seq 间接获取，不能直接 nextval(), setval() 操作

-- -- -- drop sequence if exists sm_sc.__seq_global;
-- -- create sequence if not exists sm_sc.__seq_global increment by 1 minvalue 0 no maxvalue start with 1;
-- -- select nextval('sm_sc.__seq_global');    -- 如果不初始化 seq，会报错：'currval of sequence "sm_sc.__seq_global" is not yet defined in this session'

-- 不采用 pg sequence 特性，而采用自定义 seq 表。
-- pg sequence 的 currval() 只支持 session 范围内的，获得 seq 当前值，导致各个 session 无法获得一致的全局 seq 当前值。
drop table if exists sm_sc.__vt_global_seq;
create unlogged table sm_sc.__vt_global_seq
(
  seq_no       bigint   ,
  cur_val      bigint   ,
  primary key (seq_no)
);

comment on table  sm_sc.__vt_global_seq                          is '全局序列值，用于生成全局唯一 id';
comment on column sm_sc.__vt_global_seq.seq_no                   is '序列编号'                       ;
comment on column sm_sc.__vt_global_seq.cur_val                  is '序列当前值'                     ;

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_array_kv;
create unlogged table sm_sc.__vt_array_kv
(
  arr_key              varchar(64)
, arr_val              float[]    
, create_date          timestamp   default now()
, last_update_date     timestamp   default now()
, primary key(arr_key)
);

comment on table  sm_sc.__vt_array_kv                          is '张量 kv 存储';
comment on column sm_sc.__vt_array_kv.arr_key                  is 'arr_key'     ;
comment on column sm_sc.__vt_array_kv.arr_val                  is 'arr_val'     ;

create index on sm_sc.__vt_array_kv using hash (arr_key) ;
-- alter table sm_sc.__vt_array_kv add constraint constraint_set_kv unique (arr_key);
alter table sm_sc.__vt_array_kv set (autovacuum_enabled = on);
-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_tmp_array_kv;
create unlogged table sm_sc.__vt_tmp_array_kv
(
  arr_key              varchar(64)
, arr_val              float[]    
, primary key(arr_key)
);

comment on table  sm_sc.__vt_tmp_array_kv                          is '张量 kv 存储，用于导入预训练模型时，临时存取张量 kv';
comment on column sm_sc.__vt_tmp_array_kv.arr_key                  is 'arr_key'     ;
comment on column sm_sc.__vt_tmp_array_kv.arr_val                  is 'arr_val'     ;

create index on sm_sc.__vt_tmp_array_kv using hash (arr_key) ;
-- alter table sm_sc.__vt_array_kv add constraint constraint_set_kv unique (arr_key);
alter table sm_sc.__vt_tmp_array_kv set (autovacuum_enabled = on);

-- -------------------------------------------------------------------------------------------
-- drop sequence if exists sm_sc.seq_task;
create sequence if not exists sm_sc.seq_task increment by 1 minvalue 1 no maxvalue start with 1;
drop table if exists sm_sc.tb_classify_task;
create unlogged table sm_sc.tb_classify_task
(
  work_no          bigint    primary key default nextval('sm_sc.seq_task')    ,
  model_code_6     char(6)               default left(md5(random()::text), 6) ,
  learn_cnt        int                   default 0                            ,                     
  loss_fn_type     varchar(32)                                                ,                     
  loss_delta       float                                                      ,                          
  ret_w            float[]                                          
);

comment on table  sm_sc.tb_classify_task                         is '分类任务训练任务表，包含信息：任务会话、任务级配置参数';
comment on column sm_sc.tb_classify_task.work_no                 is '训练任务序号'                                          ;
comment on column sm_sc.tb_classify_task.learn_cnt               is '已完成训练次数'                                        ;
comment on column sm_sc.tb_classify_task.loss_fn_type            is '损失函数选取类型。参看字典表：loss_fn_type'            ;
comment on column sm_sc.tb_classify_task.loss_delta              is '当次训练损失函数值'                                    ;
comment on column sm_sc.tb_classify_task.ret_w                   is '权重，仅用于线性回归、逻辑回归'                        ;

-- drop sequence if exists sm_sc.seq_learn_work;
create sequence if not exists sm_sc.seq_learn_work increment by 1 minvalue 1 no maxvalue start with 1;

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.tb_cluster_task;
create unlogged table sm_sc.tb_cluster_task
(
  work_no               bigint    primary key default nextval('sm_sc.seq_task'),           
  cluster_cnt           int                                              ,                                
  max_point_distance    float                                           ,                                        
  cluster_type          int                                                          
);

comment on table  sm_sc.tb_cluster_task                           is '聚类任务训练任务表，包含信息：任务会话、任务级配置参数';
comment on column sm_sc.tb_cluster_task.work_no                   is '训练任务序号                                          ';
comment on column sm_sc.tb_cluster_task.cluster_cnt               is '聚类数量，用于硬聚类，如 kmeans++                     ';
comment on column sm_sc.tb_cluster_task.max_point_distance        is '相邻样本空间距离设定阈值最大，用于软聚类，如 dbscan++ ';
comment on column sm_sc.tb_cluster_task.cluster_type              is '聚类方法类型。1: kmeans++; 2: dbscan++                ';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.tb_nn_train_input_buff;
create unlogged table sm_sc.tb_nn_train_input_buff
(
  work_no            bigint          , 
  ord_no             int             ,    
  i_depdt_01         float[]         ,     
  i_depdt_02         float[]         ,     
  i_depdt_03         float[]         ,     
  i_depdt_04         float[]         ,    
  i_indepdt_01       float[]         ,      
  i_indepdt_02       float[]         ,     
  i_indepdt_03       float[]         ,     
  i_indepdt_04       float[]         ,   
  i_dataset_dtl      text            ,
  primary key (work_no, ord_no)
)
with
(parallel_workers = 64)
;

comment on table  sm_sc.tb_nn_train_input_buff                is '规约: 训练集数据编码规范后，存放这里'     ;
comment on column sm_sc.tb_nn_train_input_buff.work_no        is '训练任务序号'                             ;
comment on column sm_sc.tb_nn_train_input_buff.ord_no         is '训练集记录序号，insert row_number() 即可' ;
comment on column sm_sc.tb_nn_train_input_buff.i_depdt_01     is '训练集记录输出(分类结果), 比流转的张量少一个维度'             ;
comment on column sm_sc.tb_nn_train_input_buff.i_depdt_02     is '多模态第二路输出, 比流转的张量少一个维度' ;
comment on column sm_sc.tb_nn_train_input_buff.i_depdt_03     is '多模态第三路输出, 比流转的张量少一个维度' ;
comment on column sm_sc.tb_nn_train_input_buff.i_depdt_04     is '多模态第四路输出, 比流转的张量少一个维度' ;
comment on column sm_sc.tb_nn_train_input_buff.i_indepdt_01   is '训练集记录输入(属性), 比流转的张量少一个维度'                 ;
comment on column sm_sc.tb_nn_train_input_buff.i_indepdt_02   is '多模态第二路输入,比流转的张量少一个维度'  ;
comment on column sm_sc.tb_nn_train_input_buff.i_indepdt_03   is '多模态第三路输入,比流转的张量少一个维度'  ;
comment on column sm_sc.tb_nn_train_input_buff.i_indepdt_04   is '多模态第四路输入,比流转的张量少一个维度'  ;
comment on column sm_sc.tb_nn_train_input_buff.i_dataset_dtl  is '该条数据集描述说明'                       ;

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.tb_nn_node;
create unlogged table sm_sc.tb_nn_node
(
  work_no                 bigint                 ,     -- 训练任务序号
  node_no                 bigint                 ,     -- 神经元序号（尽量规划为包含层数信息），前向传播起点， node x值 是 i_indepdt；前向传播终点， node x值 是 i_depdt
  node_type               varchar(64)            ,     -- 神经元类型      enum_name = 'node_type'   规约：每个 work_no 全局的 input, output 节点各只有一个
  node_fn_type            varchar(64)            ,     -- 神经元运算函数  enum_name = 'node_fn_type'
  node_fn_asso_value      float[]                ,     -- 超参数，不限定一维或二维或高维的 float 数组，配置规约：1. 激活函数的协参；2. 寄存必要的多目汇总信息，如 avg 所需的 count(path)；3. 卷积、池化的窗口宽度
  nn_depth_no             int                    ,     -- 前向传播串行步骤序号
  learn_cnt_fore          int     default 0      ,     -- 前向传播训练次数序号
  learn_cnt_back          int     default 0      ,     -- 反向传播训练次数序号
  is_fore_node            boolean default true   ,     -- 前向传播节点起点。缺省 true. 从神经网络的初始起点（包括）开始审计，至 prod_mx, conv_2d, rand 类算子为止（不包括），沿途皆为 false
  is_back_node            boolean default false  ,     -- 反向传播节点终点。缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true
  
                                                       -- https://mp.weixin.qq.com/s?__biz=MzI4MDYzNzg4Mw==&mid=2247496614&idx=3&sn=14b16cfe791f59ec5d2f54385f183096
  dropout_ratio           float   default 0.0    ,     -- 该节点 dropout 概率
  dropout_rescale         float   default 1.0    ,     -- 该节点的来路节点 dropout 后，该节点的因变量缩放倍数。通常参考 fore_node.dropout_rescale = 1.0 / (1 - back_node.dropout_ratio)
  dropout_depdt_val       float   default 0.0    ,     -- 该节点 dropout 后，因变量的填充值
  is_dropout              boolean default false  ,     -- 当次前向传播/反向传播，是否被 dropout
  node_depdt_len          int[]                  ,     -- node_o 的高宽，用于训练前审计
  pick_depdt_idx          int4multirange[]       ,     -- 对算子入参的抽样或切片序号集合，用于 nn 从 i_depdt 中获得 本次训练的 v_indepdt   -- 务必保证算子数据集数量经过各层 nn fn 的第一目（包括聚合的n目）而不是第二目输入，最终抵达输出
                                                       -- 该多段范围用集合包住，是为了支持重复采样。
  node_depdt_vals         float[]                ,     -- 因变量结果
  node_dloss_ddepdt       float[]                ,     -- 损失函数对因变量导数
  p_node_depdt            varchar(64)            ,     -- 因变量矩阵指针
  p_node_dloss_ddepdt     varchar(64)            ,     -- 损失函数对因变量导数矩阵的指针
  cost_delta_l1           float[]                ,     -- Adam 梯度下降 一阶矩
  cost_delta_l2           float[]                ,     -- Adam 梯度下降 二阶矩
  node_grp_no             bigint                 ,     -- 所属分组编号。可以来在 ufv_combine 复合算子的 combine_no
  node_grp_ord_no         bigint                 ,     -- 该节点在复合节点中的顺序编号。可以来在 ufv_combine 复合算子的 combine_node_ord_no
  node_desc               text                   ,
  primary key (work_no, node_no)
)
with
(parallel_workers = 64)
;
-- alter table sm_sc.tb_nn_node set (parallel_workers = 64)

comment on table  sm_sc.tb_nn_node                      is '神经网络的各层节点列表，用于训练'     ;
comment on column sm_sc.tb_nn_node.work_no              is '训练任务序号';
comment on column sm_sc.tb_nn_node.node_no              is '神经元序号（尽量规划为包含层数信息），前向传播起点， node x值 是 i_indepdt；前向传播终点， node x值 是 i_depdt';
comment on column sm_sc.tb_nn_node.node_type            is '神经元类型      enum_name = ''node_type''   规约：每个 work_no 全局的 input, output 节点各只有一个';
comment on column sm_sc.tb_nn_node.node_fn_type         is '神经元运算函数  enum_name = ''node_fn_type''';
comment on column sm_sc.tb_nn_node.node_fn_asso_value   is '超参数，不限定一维或二维或高维的 float 数组，配置规约：1. 激活函数的协参；2. 寄存必要的多目汇总信息，如 avg 所需的 count(path)；3. 卷积、池化的窗口宽度';
comment on column sm_sc.tb_nn_node.nn_depth_no          is '前向传播串行步骤序号';
comment on column sm_sc.tb_nn_node.learn_cnt_fore       is '前向传播训练次数序号';
comment on column sm_sc.tb_nn_node.learn_cnt_back       is '反向传播训练次数序号';
comment on column sm_sc.tb_nn_node.is_fore_node         is '前向传播节点起点。缺省 true. 从神经网络的初始起点（包括）开始审计，至 prod_mx, conv_2d, rand 类算子为止（不包括），沿途皆为 false';
comment on column sm_sc.tb_nn_node.is_back_node         is '反向传播节点终点。缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true';
comment on column sm_sc.tb_nn_node.dropout_ratio        is 'dropout 率';
comment on column sm_sc.tb_nn_node.dropout_rescale      is '该节点的来路节点 dropout 后，该节点的因变量缩放倍数。通常参考 fore_node.dropout_rescale = 1.0 / (1 - back_node.dropout_ratio)';
comment on column sm_sc.tb_nn_node.dropout_depdt_val    is '该节点 dropout 后，因变量的填充值';
comment on column sm_sc.tb_nn_node.is_dropout           is '当次前向传播/反向传播，是否被 dropout';
comment on column sm_sc.tb_nn_node.node_depdt_len       is 'node_o 的高宽，用于训练前审计';
comment on column sm_sc.tb_nn_node.pick_depdt_idx       is '对算子入参的抽样或切片序号集合，用于 nn 从 i_depdt 中获得 本次训练的 v_y   -- 务必保证算子数据集数量经过各层 nn fn 的第一目（包括聚合的n目）而不是第二目输入，最终抵达输出';
comment on column sm_sc.tb_nn_node.node_depdt_vals      is '因变量结果';
comment on column sm_sc.tb_nn_node.node_dloss_ddepdt    is '损失函数对因变量导数';
comment on column sm_sc.tb_nn_node.p_node_depdt         is '因变量张量 kv';
comment on column sm_sc.tb_nn_node.p_node_dloss_ddepdt  is '损失函数对因变量导数 kv';
comment on column sm_sc.tb_nn_node.cost_delta_l1        is 'Adam 梯度下降 一阶矩';
comment on column sm_sc.tb_nn_node.cost_delta_l2        is 'Adam 梯度下降 二阶矩';
comment on column sm_sc.tb_nn_node.node_grp_no          is '所属分组编号。可以来在 ufv_combine 复合算子的 combine_no';
comment on column sm_sc.tb_nn_node.node_grp_ord_no      is '该节点在复合节点中的顺序编号。可以来在 ufv_combine 复合算子的 combine_node_ord_no';
comment on column sm_sc.tb_nn_node.node_desc            is '节点描述';

create index on sm_sc.tb_nn_node(work_no, learn_cnt_fore);
create index on sm_sc.tb_nn_node(work_no, learn_cnt_back);
create index on sm_sc.tb_nn_node(work_no, node_type);
create index on sm_sc.tb_nn_node(work_no, nn_depth_no);
create index on sm_sc.tb_nn_node(work_no, is_fore_node);
create index on sm_sc.tb_nn_node(work_no, is_back_node);
create index on sm_sc.tb_nn_node(work_no, dropout_ratio);
create index on sm_sc.tb_nn_node(work_no, is_dropout);
alter table sm_sc.tb_nn_node add constraint unique_idx_node_grp unique (node_grp_no, node_grp_ord_no);

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_nn_node;
create unlogged table sm_sc.__vt_nn_node
(
  work_no                 bigint       ,         
  node_no                 bigint       ,      
  node_type               varchar(64)  ,  
  node_fn_type            varchar(64)  ,  
  node_fn_asso_value      float[]      ,       
  nn_depth_no             int          ,          
  node_depdt_vals         float[]      ,    
  p_node_depdt            varchar(64)  ,     -- 因变量矩阵指针
  primary key (work_no, node_no)
)
with
(parallel_workers = 64)
;

comment on table  sm_sc.__vt_nn_node                       is '神经网络的各层节点列表，用于测试预测'     ;
comment on column sm_sc.__vt_nn_node.work_no               is '训练任务序号';
comment on column sm_sc.__vt_nn_node.node_no               is '神经元（包含层数信息）序号';
comment on column sm_sc.__vt_nn_node.node_type             is '神经元类型      enum_name = ''node_type''。规约：每个 work_no 全局的 input, output 节点各只有一个';
comment on column sm_sc.__vt_nn_node.node_fn_type          is '神经元运算函数  enum_name = ''node_fn_type''';
comment on column sm_sc.__vt_nn_node.node_fn_asso_value    is '配置规约：参看字典表 enum_name = ''node_fn_asso_value''';
comment on column sm_sc.__vt_nn_node.nn_depth_no           is '前向传播串行步骤序号';
comment on column sm_sc.__vt_nn_node.node_depdt_vals       is '训练权重，只存 node_o.m_vals where node_type = ''weight''';
comment on column sm_sc.__vt_nn_node.p_node_depdt          is '训练权重 kv。张量值与 kv 都保留，便于模型导入导出的同时，不直接操作 __vt_array_kv 表，进而解耦 kv 改造扩展';

create index on sm_sc.__vt_nn_node(work_no, node_type);
create index on sm_sc.__vt_nn_node(work_no, nn_depth_no);

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.tb_nn_path;
create unlogged table sm_sc.tb_nn_path
(
  work_no            bigint              ,     -- 训练任务序号
  fore_node_no       bigint              ,     -- 反向传播发起节点(前向传播目标节点)神经元（包含层数信息）序号
  path_ord_no        int                 ,     -- 对应前向目标（也即抵达节点）的传递顺序，从1起始 w0, w1, w2...。
  back_node_no       bigint              ,     -- 前向传播发起节点(反向传播目标节点)神经元（包含层数信息）序号
  -- ddepdt_dindepdt    float[]             ,     -- 因变量对自变量导数。该导数没有对自变量广播求逆，可能与自变量规格不一致，但符合自变量广播规格。
  p_ddepdt_dindepdt  varchar(64)         ,     -- 因变量对自变量导数矩阵指针
  primary key (work_no, fore_node_no, path_ord_no)
)
with
(parallel_workers = 64)
;

-- create index on sm_sc.tb_nn_path using hash (p_ddepdt_dindepdt) ;

comment on table  sm_sc.tb_nn_path                        is '神经网络传播路径列表，包含信息：边、数据流图'     ;
comment on column sm_sc.tb_nn_path.work_no                is '训练任务序号';
comment on column sm_sc.tb_nn_path.fore_node_no           is '反向传播发起节点(前向传播目标节点)神经元（包含层数信息）序号';
comment on column sm_sc.tb_nn_path.path_ord_no            is '对应前向目标（也即抵达节点）的传递顺序，从1起始 w0, w1, w2...。';
comment on column sm_sc.tb_nn_path.back_node_no           is '前向传播发起节点(反向传播目标节点)神经元（包含层数信息）序号';
-- comment on column sm_sc.tb_nn_path.ddepdt_dindepdt        is '因变量对自变量导数。该导数没有对自变量广播求逆，可能与自变量规格不一致，但符合自变量广播规格。';
comment on column sm_sc.tb_nn_path.p_ddepdt_dindepdt           is '前向传播发起节点(反向传播目标节点)神经元（包含层数信息）序号';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_nn_path;
create unlogged table sm_sc.__vt_nn_path
(
  work_no            bigint              ,     -- 训练任务序号
  fore_node_no       bigint              ,     -- 反向传播发起节点(前向传播目标节点)神经元（包含层数信息）序号
  path_ord_no        int                 ,     -- 对应前向目标（也即抵达节点）的传递顺序，从1起始 w0, w1, w2...。
  back_node_no       bigint              ,     -- 前向传播发起节点(反向传播目标节点)神经元（包含层数信息）序号
  primary key (work_no, fore_node_no, path_ord_no)
)
with
(parallel_workers = 64)
;

comment on table  sm_sc.__vt_nn_path                        is '神经网络传播路径列表，用于部署预测'     ;
comment on column sm_sc.__vt_nn_path.work_no                is '训练任务序号';
comment on column sm_sc.__vt_nn_path.fore_node_no           is '反向传播发起节点(前向传播目标节点)神经元（包含层数信息）序号';
comment on column sm_sc.__vt_nn_path.path_ord_no            is '对应前向目标（也即抵达节点）的传递顺序，从1起始 w0, w1, w2...。';
comment on column sm_sc.__vt_nn_path.back_node_no           is '前向传播发起节点(反向传播目标节点)神经元（包含层数信息）序号';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.tb_dic_enum;
create table sm_sc.tb_dic_enum
(
  enum_name      varchar(64)                ,
  enum_key       varchar(32)                ,
  enum_value     varchar(512)               ,
  enum_group     varchar(64)                ,
  enum_order     int                        ,
  enum_range     numrange                   ,
  primary key (enum_name, enum_key)
);
create unique index on sm_sc.tb_dic_enum(enum_name, enum_group, enum_order);

comment on table  sm_sc.tb_dic_enum             is '字典表，where enum_name = ''node_fn_type'', 存放 算子；where enum_name = ''node_fn_type_delta'', 存放 导数算子；where enum_name = ''node_type'', 存放（输入输出位置）节点类型';
comment on column sm_sc.tb_dic_enum.enum_name   is '字典字段项';
comment on column sm_sc.tb_dic_enum.enum_key    is '枚举 key';
comment on column sm_sc.tb_dic_enum.enum_value  is '枚举 value';
comment on column sm_sc.tb_dic_enum.enum_group  is '枚举分组';
comment on column sm_sc.tb_dic_enum.enum_order  is '枚举顺序号，对于 enum_name = ''node_fn_type_delta'', 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 sub, div, pow, log 四种运算操作敏感，其他运算操作缺省 null 或实际位置。';
comment on column sm_sc.tb_dic_enum.enum_range  is '范围枚举的区间';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_tmp_nn_node;
create unlogged table if not exists sm_sc.__vt_tmp_nn_node
(
  sess_id                 bigint       ,
  work_no                 bigint       ,           
  node_no                 bigint       ,           
  node_type               varchar(64)  ,    
  node_fn_type            varchar(64)  ,    
  node_fn_asso_value      float[]      , 
  nn_depth_no             int          ,  
  node_depdt_vals         float[]      ,
  p_node_depdt            varchar(64)  ,     -- 因变量矩阵指针
  node_grp_no             bigint       ,     -- 所属分组编号。可以来在 ufv_combine 复合算子的 combine_no
  node_grp_ord_no         bigint       ,     -- 该节点在复合节点中的顺序编号。可以来在 ufv_combine 复合算子的 combine_node_ord_no  
  primary key (sess_id, work_no, node_no)
)
;

comment on table  sm_sc.__vt_tmp_nn_node                        is '用于测试集/验证集的输出寄存，被 sm_sc.ft_nn_in_out 做 dml 操作'     ;
comment on column sm_sc.__vt_tmp_nn_node.sess_id                is '请求会话 id';
comment on column sm_sc.__vt_tmp_nn_node.work_no                is '训练任务序号';
comment on column sm_sc.__vt_tmp_nn_node.node_no                is '神经元序号';
comment on column sm_sc.__vt_tmp_nn_node.node_type              is '神经元类型';
comment on column sm_sc.__vt_tmp_nn_node.node_fn_type           is '神经元运算函数';
comment on column sm_sc.__vt_tmp_nn_node.node_fn_asso_value     is '超参数配置';
comment on column sm_sc.__vt_tmp_nn_node.nn_depth_no            is '前向传播串行步骤序号，前向深度';
comment on column sm_sc.__vt_tmp_nn_node.node_depdt_vals        is '训练权重';
comment on column sm_sc.__vt_tmp_nn_node.p_node_depdt           is '前向传播张量 kv';
comment on column sm_sc.__vt_tmp_nn_node.node_grp_no            is '所属分组编号。可以来在 ufv_combine 复合算子的 combine_no';
comment on column sm_sc.__vt_tmp_nn_node.node_grp_ord_no        is '该节点在复合节点中的顺序编号。可以来在 ufv_combine 复合算子的 combine_node_ord_no';

-- -------------------------------------------------------------------------------------------
create sequence if not exists huffman_seq start 1000000000;
drop table if exists sm_sc.__vt_tmp_huffman;
create unlogged table if not exists sm_sc.__vt_tmp_huffman
(
  sess_id         bigint           ,   -- char(32)  ,
  node_no         bigint       default nextval('huffman_seq')       ,    
  is_compared     boolean      default false                        ,    
  node_weight     float                                            ,    
  is_org          boolean                                           ,    
  tree_code       varbit                                            ,    
  father_node_no  bigint                                            ,    
  primary key (sess_id, node_no)
)
;

comment on table  sm_sc.__vt_tmp_huffman                     is '用于实现霍夫曼编码，被 sm_sc.fv_huffman 做 dml 操作'     ;
comment on column sm_sc.__vt_tmp_huffman.sess_id             is '会话 id';
comment on column sm_sc.__vt_tmp_huffman.node_no             is '自然序列编号';
comment on column sm_sc.__vt_tmp_huffman.is_compared         is '是否已经比较入树';
comment on column sm_sc.__vt_tmp_huffman.node_weight         is '权值';
comment on column sm_sc.__vt_tmp_huffman.is_org              is '是否是原始 input 的节点';
comment on column sm_sc.__vt_tmp_huffman.tree_code           is '单次比较后入树结果，左子树为0，右子树为1';
comment on column sm_sc.__vt_tmp_huffman.father_node_no      is '父节点';

create index if not exists __idx_huffman_node_weight
  on sm_sc.__vt_tmp_huffman(sess_id, is_compared, node_weight);  
create index if not exists __idx_huffman_father
  on sm_sc.__vt_tmp_huffman(sess_id, father_node_no);
  
  
-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_kmean_ods_array;
create unlogged table sm_sc.__vt_kmean_ods_array
(
  sess_id                bigint           ,   -- char(32)  ,
  point_id               bigint     ,
  point_arr              float[],
  cluster_point_no       bigint     ,
  primary key(sess_id, point_id)
) with (parallel_workers = 64)
;
create index idx_v_ods_array_kmean on sm_sc.__vt_kmean_ods_array (sess_id, cluster_point_no);

comment on table  sm_sc.__vt_kmean_ods_array                     is '用于 kmean 聚类的数据点信息的临时数据，被 sm_sc.prc_kmeans_pp 做 dml 操作';
comment on column sm_sc.__vt_kmean_ods_array.sess_id             is '会话 id';
comment on column sm_sc.__vt_kmean_ods_array.point_id            is '数据集点 id';
comment on column sm_sc.__vt_kmean_ods_array.point_arr           is '数据集点坐标';
comment on column sm_sc.__vt_kmean_ods_array.cluster_point_no    is '聚类点 id';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_kmean_list_cluster;
create unlogged table sm_sc.__vt_kmean_list_cluster
(
  sess_id            bigint           ,   -- char(32)  ,
  cluster_point_no   bigint       ,
  cluster_point_arr  float[]     ,
  is_loop_done       boolean      ,
  loop_cnt           int          ,
  primary key(sess_id, cluster_point_no)
) with (parallel_workers = 64)
;

comment on table  sm_sc.__vt_kmean_list_cluster                     is '用于 kmean 聚类的类别信息的临时数据，被 sm_sc.prc_kmeans_pp 做 dml 操作'     ;
comment on column sm_sc.__vt_kmean_list_cluster.sess_id             is '会话 id';
comment on column sm_sc.__vt_kmean_list_cluster.cluster_point_no    is '聚类点 id';
comment on column sm_sc.__vt_kmean_list_cluster.cluster_point_arr   is '聚类点坐标';
comment on column sm_sc.__vt_kmean_list_cluster.is_loop_done        is '循环完成标记';
comment on column sm_sc.__vt_kmean_list_cluster.loop_cnt            is '循环次数';

-- -------------------------------------------------------------------------------------------
-- 用 dbscan 聚类的临时数据
drop table if exists sm_sc.__vt_dbscan_ods_array;
create unlogged table sm_sc.__vt_dbscan_ods_array
(
  sess_id                bigint           ,   -- char(32)  ,
  point_id               bigint     ,
  point_arr              float[],
  dbscan_grp             int     ,
  primary key(sess_id, point_id)
) with (parallel_workers = 64)
;

comment on table  sm_sc.__vt_dbscan_ods_array                     is '用于 dbscan 聚类的数据点信息的临时数据，被 sm_sc.prc_dbscan_pp 做 dml 操作'     ;
comment on column sm_sc.__vt_dbscan_ods_array.sess_id             is '会话 id';
comment on column sm_sc.__vt_dbscan_ods_array.point_id            is '数据集点 id';
comment on column sm_sc.__vt_dbscan_ods_array.point_arr           is '数据集点坐标';
comment on column sm_sc.__vt_dbscan_ods_array.dbscan_grp          is '聚类类别标记';

create index idx_v_ods_array_dbscan on sm_sc.__vt_dbscan_ods_array (sess_id, dbscan_grp);

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_dbscan_nearby_point_idx;
create unlogged table sm_sc.__vt_dbscan_nearby_point_idx
(
  sess_id                bigint           ,   -- char(32)  ,
  point_id               bigint       ,
  dimension_no           int               ,
  point_arr_n            float    ,
  point_arr              float[]  ,
  primary key(sess_id, point_id, dimension_no)
) with (parallel_workers = 64)
;
create index idx_v_nearby_idx_arr on sm_sc.__vt_dbscan_nearby_point_idx (sess_id, dimension_no, point_arr_n);

comment on table  sm_sc.__vt_dbscan_nearby_point_idx                       is '用于 dbscan 聚类的数据点临近信息的临时数据，被 sm_sc.prc_dbscan_pp 做 dml 操作'     ;
comment on column sm_sc.__vt_dbscan_nearby_point_idx.sess_id               is '会话 id';
comment on column sm_sc.__vt_dbscan_nearby_point_idx.point_id              is '数据集点 id';
comment on column sm_sc.__vt_dbscan_nearby_point_idx.dimension_no          is '临近维度';
comment on column sm_sc.__vt_dbscan_nearby_point_idx.point_arr_n           is '临近距离';
comment on column sm_sc.__vt_dbscan_nearby_point_idx.point_arr             is '临近坐标';

-- ----------------------------------------------------------------------------------------------------------------------------------------------
-- 表变量 for sm_sc.ft_computational_graph_deserialize
drop table if exists sm_sc._vt_fn_compu_graph_deseri__graph;
create unlogged table sm_sc._vt_fn_compu_graph_deseri__graph
(
  sess_id              bigint           ,   -- char(32)  ,
  out_param            varchar(64)              ,
  in_param             varchar(64)              ,
  in_value             float            ,
  param_loc            int                      ,
  out_opr              varchar(64)              ,
  create_time          timestamp default now()  ,
  unique (sess_id, out_param, param_loc) 
);

comment on table  sm_sc._vt_fn_compu_graph_deseri__graph                       is '用于计算图反序列化的临时数据，被 sm_sc.ft_computational_graph_deserialize 做 dml 操作'     ;
comment on column sm_sc._vt_fn_compu_graph_deseri__graph.sess_id               is '会话 id';
comment on column sm_sc._vt_fn_compu_graph_deseri__graph.out_param             is '因变量';
comment on column sm_sc._vt_fn_compu_graph_deseri__graph.in_param              is '自变量';
comment on column sm_sc._vt_fn_compu_graph_deseri__graph.in_value              is '入参常量';
comment on column sm_sc._vt_fn_compu_graph_deseri__graph.param_loc             is '入参位置';
comment on column sm_sc._vt_fn_compu_graph_deseri__graph.out_opr               is '运算类型';

-- ----------------------------------------------------------------------------------------------------------------------------------------------
-- 表变量 for sm_sc.ft_gradient
drop table if exists sm_sc._vt_fn_grad__graph;
create unlogged table if not exists sm_sc._vt_fn_grad__graph
(
  sess_id              bigint           ,   -- char(32)  ,
  o_out_param          varchar(64)              ,
  o_in_param           varchar(64)              ,
  o_in_value           varchar(64)              ,
  o_param_loc          int                      ,
  o_out_opr            varchar(64)              ,
  is_decimal           boolean                  ,
  create_time          timestamp default now()  ,
  unique (sess_id, o_out_param, o_param_loc) 
);
  
comment on table  sm_sc._vt_fn_grad__graph                       is '用于求导推导的临时数据，被 sm_sc.ft_gradient 做 dml 操作'     ;
comment on column sm_sc._vt_fn_grad__graph.sess_id               is '会话 id';
comment on column sm_sc._vt_fn_grad__graph.o_out_param           is '因变量';
comment on column sm_sc._vt_fn_grad__graph.o_in_param            is '自变量';
comment on column sm_sc._vt_fn_grad__graph.o_in_value            is '入参常量';
comment on column sm_sc._vt_fn_grad__graph.o_param_loc           is '入参位置';
comment on column sm_sc._vt_fn_grad__graph.o_out_opr             is '运算类型';
comment on column sm_sc._vt_fn_grad__graph.is_decimal            is '是否数字';
  
-- -------------------------------------------------------------------------------------------
  -- 表变量 for sm_sc.ft_gradient
  drop table if exists sm_sc._vt_fn_grad__algebra;
  create unlogged table if not exists sm_sc._vt_fn_grad__algebra
  (
    sess_id              bigint           ,   -- char(32)  ,
    o_out_param          varchar(64)              ,
    o_out_algebra        text                     ,
    is_decimal           boolean                  ,
    unique (sess_id, o_out_param) 
  );

comment on table  sm_sc._vt_fn_grad__algebra                       is '用于求导推导代数信息的临时数据，被 sm_sc.ft_gradient 做 dml 操作'     ;
comment on column sm_sc._vt_fn_grad__algebra.sess_id               is '会话 id';
comment on column sm_sc._vt_fn_grad__algebra.o_out_param           is '因变量';
comment on column sm_sc._vt_fn_grad__algebra.o_out_algebra         is '因变量代数信息';
comment on column sm_sc._vt_fn_grad__algebra.is_decimal            is '是否数字';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc._vt_fn_grad__forward;
create unlogged table if not exists sm_sc._vt_fn_grad__forward
(
  sess_id              bigint           ,   -- char(32)  ,
  out_param            varchar(64)              ,
  in_params            varchar(64)[]            ,
  in_values            float[]         ,
  out_opr              varchar(64)              ,
  calcu_val            float           ,
  create_time          timestamp default now()  -- ,
  -- unique (sess_id, out_param)   -- -- 避免锁表，防止并发读写阻塞
);
  
comment on table  sm_sc._vt_fn_grad__forward                   is '用于求导推导分组信息的临时数据，寄存前向传播数据，被 sm_sc.ft_gradient 做 dml 操作'     ;
comment on column sm_sc._vt_fn_grad__forward.sess_id           is '会话 id';
comment on column sm_sc._vt_fn_grad__forward.out_param         is '因变量';
comment on column sm_sc._vt_fn_grad__forward.in_params         is '自变量聚合';
comment on column sm_sc._vt_fn_grad__forward.in_values         is '入参常量集合';
comment on column sm_sc._vt_fn_grad__forward.out_opr           is '运算类型';
comment on column sm_sc._vt_fn_grad__forward.calcu_val         is '计算数值';
  
create index if not exists __idx_grad__forward on sm_sc._vt_fn_grad__forward (sess_id, out_param);
-- -- -- create index if not exists on sm_sc._vt_fn_grad__forward using gist (sess_id, in_params);
  
-- -------------------------------------------------------------------------------------------
-- -- 寄存链式求导数据
drop table if exists sm_sc._vt_fn_grad__chain;
create unlogged table if not exists sm_sc._vt_fn_grad__chain
(
  sess_id              bigint           ,   -- char(32)  ,
  out_param            varchar(64)              ,
  param_loc            int                      ,
  in_param             varchar(64)              ,
  in_value             float           ,
  out_opr              varchar(64)              ,
  calcu_val            float           ,
  grad_val             float           ,
  co_vals              float[]         ,
  create_time          timestamp default now()  ,
  unique (sess_id, out_param, param_loc) 
);

comment on table  sm_sc._vt_fn_grad__chain                   is '用于求导推导的临时数据，寄存链式求导数据，被 sm_sc.ft_gradient 做 dml 操作'     ;
comment on column sm_sc._vt_fn_grad__chain.sess_id           is '会话 id';
comment on column sm_sc._vt_fn_grad__chain.out_param         is '因变量';
comment on column sm_sc._vt_fn_grad__chain.param_loc         is '入参位置';
comment on column sm_sc._vt_fn_grad__chain.in_param          is '自变量';
comment on column sm_sc._vt_fn_grad__chain.in_value          is '入参常量';
comment on column sm_sc._vt_fn_grad__chain.out_opr           is '运算类型';
comment on column sm_sc._vt_fn_grad__chain.calcu_val         is '计算数值';
comment on column sm_sc._vt_fn_grad__chain.grad_val          is '梯度值';
comment on column sm_sc._vt_fn_grad__chain.co_vals           is '协参集合';

create index if not exists _idx_grad__chain on sm_sc._vt_fn_grad__chain (sess_id, in_param);

-- -------------------------------------------------------------------------------------------
-- 用于 sm_sc.fv_mx_determinant
--      sm_sc.fv_mx_inversion
--      sm_sc.fv_mx_rank
--      sm_sc.fv_mx_rows_step_simple
--      sm_sc.fv_mx_row_step
drop table if exists sm_sc.__vt_tmp_matrix;
create unlogged table sm_sc.__vt_tmp_matrix
(
  sess_id             bigint           ,   -- char(32)  ,
  new_row_no          int             ,   
  array_x             float[]    
) with (parallel_workers = 64);

comment on table  sm_sc.__vt_tmp_matrix                   is '用于矩阵运算临时表做 dml 操作'     ;
comment on column sm_sc.__vt_tmp_matrix.sess_id           is '会话 id';
comment on column sm_sc.__vt_tmp_matrix.new_row_no        is '新行号';
comment on column sm_sc.__vt_tmp_matrix.array_x           is '行向量';

create index idx_tmp_matrix on sm_sc.__vt_tmp_matrix (sess_id, new_row_no);
  
-- -------------------------------------------------------------------------------------------
-- 用于 sm_sc.prc_mx_eigen_array_value
drop table if exists sm_sc.__vt_tmp_eigen_arrays;
create unlogged table sm_sc.__vt_tmp_eigen_arrays
(
  sess_id        bigint           ,   -- char(32)  ,
  eigen_value    float     ,
  eigen_array    float[]
) with (parallel_workers = 64);
  
comment on table  sm_sc.__vt_tmp_eigen_arrays                   is '用于特征分解，被 sm_sc.prc_mx_eigen_array_value 做 dml 操作'     ;
comment on column sm_sc.__vt_tmp_eigen_arrays.sess_id           is '会话 id';
comment on column sm_sc.__vt_tmp_eigen_arrays.eigen_value       is '特征值';
comment on column sm_sc.__vt_tmp_eigen_arrays.eigen_array       is '特征向量';

create index idx_tmp_eigen_arrays on sm_sc.__vt_tmp_matrix (sess_id);

-- -------------------------------------------------------------------------------------------
-- 用于 sm_sc.fv_nn_node2node_val
drop table if exists sm_sc.__vt_tmp_node_node2node;
create unlogged table sm_sc.__vt_tmp_node_node2node
(
  sess_id              bigint           ,   -- char(32)  ,
  work_no              bigint           ,
  node_no              bigint           ,
  node_type            varchar(64)      ,
  node_depdt_vals           float[]
);

create index __idx_tmp_node_node2node on sm_sc.__vt_tmp_node_node2node (sess_id, work_no, node_no);
create index __idx_tmp_node_node2node_type on sm_sc.__vt_tmp_node_node2node (sess_id, work_no, node_type);


-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_tmp_bpe_seq;
create unlogged table if not exists sm_sc.__vt_tmp_bpe_seq
(
  sess_id         bigint           ,
  seq_code_no     int              ,
  seq_code        text             ,  
  seq_token       text             ,  
  seq_split_cnt   int              ,
  seq_token_2gram text[]           ,
  -- seq_token_gin   tsvector         ,
  primary key (sess_id, seq_code_no)
)
;

comment on table  sm_sc.__vt_tmp_bpe_seq                     is '用于实现 bpe 编码，被 sm_sc.fv_bpe 做 dml 操作。以 ^ 字符起始，以 $ 字符结尾。原始的序列不能包含"^ $"(起始、空格、结尾)这三个字符'     ;
comment on column sm_sc.__vt_tmp_bpe_seq.sess_id             is '会话 id';
comment on column sm_sc.__vt_tmp_bpe_seq.seq_code_no         is '序列编号。用于 fv_bpe 返回的 array 元素排序';
comment on column sm_sc.__vt_tmp_bpe_seq.seq_code            is '原始的序列';
comment on column sm_sc.__vt_tmp_bpe_seq.seq_token           is '插入空格字符的分割的序列';
comment on column sm_sc.__vt_tmp_bpe_seq.seq_split_cnt       is '原始的序列被分成 token 的数量';
comment on column sm_sc.__vt_tmp_bpe_seq.seq_token_2gram     is '序列 token 的 2gram';
-- comment on column sm_sc.__vt_tmp_bpe_seq.seq_token_gin       is '用于匹配的倒排索引';


-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_tmp_bpe_token_list;
create unlogged table if not exists sm_sc.__vt_tmp_bpe_token_list
(
  sess_id         bigint           ,
  token           text             ,  
  -- token_ex        text             ,
  token_freq      bigint           ,  
  primary key (sess_id, token)
)
;

comment on table  sm_sc.__vt_tmp_bpe_token_list                     is '用于实现 bpe 编码，被 sm_sc.fv_bpe 做 dml 操作'     ;
comment on column sm_sc.__vt_tmp_bpe_token_list.token               is 'token';
-- comment on column sm_sc.__vt_tmp_bpe_token_list.token_ex            is 'token 被去除空格间隔前的 2gram';
comment on column sm_sc.__vt_tmp_bpe_token_list.token_freq          is 'token 出现次数';

-- -------------------------------------------------------------------------------------------
drop table if exists sm_sc.__vt_nn_sess;
create unlogged table if not exists sm_sc.__vt_nn_sess
(
  work_no             bigint           ,
  sess_id             bigint           ,  
  sess_status         varchar(32)      ,  
  create_date         timestamp   default now()      ,
  last_update_date    timestamp   default now()      ,
  primary key (work_no, sess_id)
)
;

comment on table  sm_sc.__vt_nn_sess                      is '为模型部署创建的 session 连接池'     ;
comment on column sm_sc.__vt_nn_sess.work_no              is '模型序号';
comment on column sm_sc.__vt_nn_sess.sess_id              is '连接标识';
comment on column sm_sc.__vt_nn_sess.sess_status          is '连接状态，参看字典表';

create index idx_nn_sess_status on sm_sc.__vt_nn_sess (work_no, sess_status);
-- drop function if exists sm_sc.fv_d_abs(float[]);
create or replace function sm_sc.fv_d_abs
(
  i_indepdt_var float[]
)
returns float[]
as
$$
declare 
begin
  return
    (<>` i_indepdt_var) :: float[]
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_abs(array[1.8, -4.6])
-- select 
--   sm_sc.fv_d_abs(array[[1.8, -4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_abs(array[[[1.8, -4.6], [1.4, 3.6]],[[1.8, -4.6], [1.4, 3.6]]])
-- select 
--   sm_sc.fv_d_abs(array[[[[1.8, -4.6], [1.4, 3.6]],[[1.8, -4.6], [1.4, 3.6]]],[[[1.8, -4.6], [1.4, 3.6]],[[1.8, -4.6], [1.4, 3.6]]]])

-- drop function if exists sm_sc.fv_d_add(int[], int[]);
create or replace function sm_sc.fv_d_add
(
  i_depdt_var_len      int[]
, i_indepdt_var_len    int[]   default null
)
returns float[]
as
$$
declare 
begin
  if i_indepdt_var_len <> i_depdt_var_len
  then 
    return 
      array_fill
      (
        sm_sc.fv_aggr_slice_prod
        (
          i_depdt_var_len 
          / 
          sm_sc.fv_lpad
          (
            i_indepdt_var_len
          , array[1]
          , array_length(i_depdt_var_len, 1) - array_length(i_indepdt_var_len, 1)
          )
        ) :: float
      , i_indepdt_var_len
      )
    ;
  else 
    return array_fill(1.0 :: float, i_depdt_var_len);
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_add(array[2, 3])
-- select 
--   sm_sc.fv_d_add(array[2, 3, 4])
-- select 
--   sm_sc.fv_d_add(array[2, 3, 4, 5])
-- select 
--   sm_sc.fv_d_add(array[   3, 1, 5], array[2, 3, 4, 5])
-- drop function if exists sm_sc.fv_d_div_1(float[], int[]);
create or replace function sm_sc.fv_d_div_1
(
  i_co_value           float[] 
, i_indepdt_var_len    int[]  default null
)
returns float[]
as
$$
declare 
  v_co_var_len      int[]  := (select array_agg(array_length(i_co_value, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_co_value)) tb_a_cur_dim(a_cur_dim));
  v_depdt_var_len   int[]  :=  
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_indepdt_var_len, a_co_len)))
      from unnest(sm_sc.__fv_mirror_y(i_indepdt_var_len), sm_sc.__fv_mirror_y(v_co_var_len)) tb_a_len(a_indepdt_var_len, a_co_len)
    )
  ;
  v_ret         float[];
begin
  -- 求导
  v_ret := 1.0 :: float /` i_co_value;
  
  -- 求导结果对齐维度至因变量
  if array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1) = 3
  then 
    v_ret := array[[[v_ret]]];
  elsif array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1) = 2
  then 
    v_ret := array[[v_ret]];
  elsif array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1) = 1
  then 
    v_ret := array[v_ret];
  end if;
  
  -- 由协参规格对齐至因变量
  if v_co_var_len <> v_depdt_var_len
  then 
    -- 维度对齐
    if array_length(v_co_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      v_co_var_len := 
        sm_sc.fv_lpad
        (
          v_co_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1)
        );
    end if;
    
    -- 规格对齐
    v_ret := sm_sc.fv_new(v_ret, v_depdt_var_len / v_co_var_len);
  end if;
  
  -- 由因变量对齐至自变量
  if i_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        / 
        sm_sc.fv_lpad
        (
          i_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(i_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(i_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(i_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(i_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(i_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][1][ : ][ : ], array[1, 2], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(i_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_div_1(array[2.4, -1.6])
-- select 
--   sm_sc.fv_d_div_1(array[[2.8, 3.6], [2.4, -1.6]])
-- select 
--   sm_sc.fv_d_div_1(array[[[2.8, 3.6], [2.4, -1.6]],[[2.8, 3.6], [2.4, -1.6]]])
-- select 
--   sm_sc.fv_d_div_1(array[[[[2.8, 3.6], [2.4, -1.6]],[[2.8, 3.6], [2.4, -1.6]]],[[[2.8, 3.6], [2.4, -1.6]],[[2.8, 3.6], [2.4, -1.6]]]])
-- select 
--   sm_sc.fv_d_div_1(sm_sc.fv_new_rand(array[2, 1, 3]), array[3, 3])
-- select 
--   sm_sc.fv_d_div_1(sm_sc.fv_new_rand(array[3, 3]), array[2, 1, 3])
-- select 
--   sm_sc.fv_d_div_1(sm_sc.fv_new_rand(array[2, 1, 3, 5]), array[3, 3, 5])
-- select 
--   sm_sc.fv_d_div_1(sm_sc.fv_new_rand(array[3, 3, 5]), array[2, 1, 3, 5])
-- select 
--   sm_sc.fv_d_div_1(sm_sc.fv_new_rand(array[4, 2, 1, 3]), array[3, 3])
-- select 
--   sm_sc.fv_d_div_1(sm_sc.fv_new_rand(array[3, 3]), array[4, 2, 1, 3])
-- drop function if exists sm_sc.fv_d_div_2(float[], float[]);
create or replace function sm_sc.fv_d_div_2
(
  i_indepdt_var float[] , 
  i_co_value    float[] default array[1.0]
)
returns float[]
as
$$
declare 
  v_ret               float[]    ;
  v_depdt_var_len     int[]      ;
  v_indepdt_var_len   int[]      :=
    (
      select 
        array_agg(array_length(i_indepdt_var, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(i_indepdt_var)) tb_a_dim_cur(a_dim_cur)
    )
  ;
begin
  v_ret           :=  -` (i_co_value /` (i_indepdt_var ^` 2.0 :: float));
  v_depdt_var_len := 
    (
      select 
        array_agg(array_length(v_ret, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(v_ret)) tb_a_dim_cur(a_dim_cur)
    )
  ;

  -- 由因变量对齐至自变量
  if v_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        / 
        sm_sc.fv_lpad
        (
          v_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(v_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(v_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][1][ : ][ : ], array[1, 2], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_div_2(array[2.8, 3.6], array[1.8, -4.6])
-- select 
--   sm_sc.fv_d_div_2(array[[2.8, 3.6], [2.4, -1.6]], array[[1.8, -4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_div_2(array[[[2.8, 3.6], [2.4, -1.6]],[[1.8, -4.6], [1.4, 3.6]]], array[[[1.8, -4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, -1.6]]])
-- select 
--   sm_sc.fv_d_div_2(array[[[[1.8, -4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, -1.6]]],[[[2.8, 3.6], [2.4, -1.6]],[[1.8, -4.6], [1.4, 3.6]]]]
--                   , array[[[[2.8, 3.6], [2.4, -1.6]],[[1.8, -4.6], [1.4, 3.6]]],[[[1.8, -4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, -1.6]]]])
-- select 
--   sm_sc.fv_d_div_2(sm_sc.fv_new_rand(array[2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_div_2(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[2, 1, 3]))
-- select 
--   sm_sc.fv_d_div_2(sm_sc.fv_new_rand(array[2, 1, 3, 5]), sm_sc.fv_new_rand(array[3, 3, 5]))
-- select 
--   sm_sc.fv_d_div_2(sm_sc.fv_new_rand(array[3, 3, 5]), sm_sc.fv_new_rand(array[2, 1, 3, 5]))
-- select 
--   sm_sc.fv_d_div_2(sm_sc.fv_new_rand(array[4, 2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_div_2(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[4, 2, 1, 3]))
-- drop function if exists sm_sc.fv_d_div_2_un_de_broadcast(float[], float[]);
create or replace function sm_sc.fv_d_div_2_un_de_broadcast
(
  i_indepdt_var float[] , 
  i_co_value    float[]  default  array[1.0]
)
returns float[]
as
$$
-- declare 
begin
  return -` (i_co_value /` (i_indepdt_var ^` 2.0 :: float));
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(array[2.8, 3.6], array[1.8, -4.6])
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(array[[2.8, 3.6], [2.4, -1.6]], array[[1.8, -4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(array[[[2.8, 3.6], [2.4, -1.6]],[[1.8, -4.6], [1.4, 3.6]]], array[[[1.8, -4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, -1.6]]])
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(array[[[[1.8, -4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, -1.6]]],[[[2.8, 3.6], [2.4, -1.6]],[[1.8, -4.6], [1.4, 3.6]]]]
--                   , array[[[[2.8, 3.6], [2.4, -1.6]],[[1.8, -4.6], [1.4, 3.6]]],[[[1.8, -4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, -1.6]]]])
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(sm_sc.fv_new_rand(array[2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[2, 1, 3]))
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(sm_sc.fv_new_rand(array[2, 1, 3, 5]), sm_sc.fv_new_rand(array[3, 3, 5]))
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(sm_sc.fv_new_rand(array[3, 3, 5]), sm_sc.fv_new_rand(array[2, 1, 3, 5]))
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(sm_sc.fv_new_rand(array[4, 2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_div_2_un_de_broadcast(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[4, 2, 1, 3]))
-- drop function if exists sm_sc.fv_d_exp(float[], float[]);
create or replace function sm_sc.fv_d_exp
(
  i_indepdt_var float[] , 
  i_depdt_var   float[] default null
)
returns float[]
as
$$
declare 
begin
  return
    case when i_depdt_var is not null then i_depdt_var else ^` i_indepdt_var end      
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_exp(array[1.8, -4.6])
-- select 
--   sm_sc.fv_d_exp(array[[1.8, -4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_exp(array[[[1.8, -4.6], [1.4, 3.6]],[[1.8, -4.6], [1.4, 3.6]]])
-- select 
--   sm_sc.fv_d_exp(array[[[[1.8, -4.6], [1.4, 3.6]],[[1.8, -4.6], [1.4, 3.6]]],[[[1.8, -4.6], [1.4, 3.6]],[[1.8, -4.6], [1.4, 3.6]]]])

-- select 
--   sm_sc.fv_d_exp
--   (
--     array[[1.8, -4.6], [1.4, 3.6]],
--     ^` array[[1.8, -4.6], [1.4, 3.6]]
--   )

-- select 
--   sm_sc.fv_d_exp
--   (
--     null,
--     ^` array[[1.8, -4.6], [1.4, 3.6]]
--   )
-- drop function if exists sm_sc.fv_d_ln(float[]);
create or replace function sm_sc.fv_d_ln
(
  i_indepdt_var float[]
)
returns float[]
as
$$
declare 
begin
  return
    /` i_indepdt_var
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_ln(array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_ln(array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_ln(array[[[1.8, 4.6], [1.4, 3.6]],[[1.8, 4.6], [1.4, 3.6]]])
-- select 
--   sm_sc.fv_d_ln(array[[[[1.8, 4.6], [1.4, 3.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[1.8, 4.6], [1.4, 3.6]]]])
-- drop function if exists sm_sc.fv_d_log_1(float[], float[], float[]);
create or replace function sm_sc.fv_d_log_1
(
  i_indepdt_var float[] , 
  i_co_value    float[] ,
  i_depdt_var   float[] default null
)
returns float[]
as
$$
declare 
  v_ret               float[]    ;
  v_depdt_var_len     int[]      ;
  v_indepdt_var_len   int[]      :=
    (
      select 
        array_agg(array_length(i_indepdt_var, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(i_indepdt_var)) tb_a_dim_cur(a_dim_cur)
    )
  ;
begin
  v_ret           :=  
    case 
      when i_depdt_var is not null 
        then -` (i_depdt_var ^` 2.0 :: float) /` (i_indepdt_var *` (^!` i_co_value :: decimal[]) :: float[])      -- -- sm_sc.fv_nullif(... , 0.0 :: float)
      else (-` ((i_indepdt_var :: decimal[] ^!` i_co_value :: decimal[]) :: float[] ^` 2.0 :: float)) /` (i_indepdt_var *` (^!` i_co_value :: decimal[]) :: float[])      -- -- sm_sc.fv_nullif(... , 0.0 :: float)
    end     -- 优先使用 i_depdt_var
  ;
  v_depdt_var_len := 
    (
      select 
        array_agg(array_length(v_ret, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(v_ret)) tb_a_dim_cur(a_dim_cur)
    )
  ;

  -- 由因变量对齐至自变量
  if v_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        / 
        sm_sc.fv_lpad
        (
          v_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(v_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(v_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][1][ : ][ : ], array[1, 2], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_log_1(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_log_1(array[[2.8, 3.6], [2.4, 1.6]]
--                  , array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_log_1(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_log_1(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])


-- select 
--   sm_sc.fv_d_log_1
--   (
--     array[[2.8, 3.6], [2.4, 1.6]], 
--     array[[1.8, 4.6], [1.4, 3.6]],
--     array[[2.8, 3.6], [2.4, 1.6]] ^!` array[[1.8, 4.6], [1.4, 3.6]]
--   )
-- select 
--   sm_sc.fv_d_log_1(sm_sc.fv_new_rand(array[2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_log_1(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[2, 1, 3]))
-- select 
--   sm_sc.fv_d_log_1(sm_sc.fv_new_rand(array[2, 1, 3, 5]), sm_sc.fv_new_rand(array[3, 3, 5]))
-- select 
--   sm_sc.fv_d_log_1(sm_sc.fv_new_rand(array[3, 3, 5]), sm_sc.fv_new_rand(array[2, 1, 3, 5]))
-- select 
--   sm_sc.fv_d_log_1(sm_sc.fv_new_rand(array[4, 2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_log_1(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[4, 2, 1, 3]))
-- drop function if exists sm_sc.fv_d_log_1_un_de_broadcast(float[], float[], float[]);
create or replace function sm_sc.fv_d_log_1_un_de_broadcast
(
  i_indepdt_var float[] , 
  i_co_value    float[] ,
  i_depdt_var   float[] default null
)
returns float[]
as
$$
declare 
begin
  return
    case 
      when i_depdt_var is not null 
        then -` (i_depdt_var ^` 2.0 :: float) /` (i_indepdt_var *` (^!` i_co_value :: decimal[]) :: float[])      -- -- sm_sc.fv_nullif(... , 0.0 :: float)
      else (-` ((i_indepdt_var :: decimal[] ^!` i_co_value :: decimal[]) :: float[] ^` 2.0 :: float)) /` (i_indepdt_var *` (^!` i_co_value :: decimal[]) :: float[])      -- -- sm_sc.fv_nullif(... , 0.0 :: float)
    end     -- 优先使用 i_depdt_var
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_log_1_un_de_broadcast(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_log_1_un_de_broadcast(array[[2.8, 3.6], [2.4, 1.6]]
--                  , array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_log_1_un_de_broadcast(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_log_1_un_de_broadcast(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])


-- select 
--   sm_sc.fv_d_log_1_un_de_broadcast
--   (
--     array[[2.8, 3.6], [2.4, 1.6]], 
--     array[[1.8, 4.6], [1.4, 3.6]],
--     array[[2.8, 3.6], [2.4, 1.6]] ^!` array[[1.8, 4.6], [1.4, 3.6]]
--   )
-- drop function if exists sm_sc.fv_d_log_2(float[][], float[][]);
create or replace function sm_sc.fv_d_log_2
(
  i_indepdt_var float[][] , 
  i_co_value    float[][]
)
returns float[][]
as
$$
declare 
  v_ret               float[]    ;
  v_depdt_var_len     int[]      ;
  v_indepdt_var_len   int[]      :=
    (
      select 
        array_agg(array_length(i_indepdt_var, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(i_indepdt_var)) tb_a_dim_cur(a_dim_cur)
    )
  ;
begin
  v_ret           :=  /` (i_indepdt_var *` (^!` i_co_value :: decimal[]) :: float[]);
  v_depdt_var_len := 
    (
      select 
        array_agg(array_length(v_ret, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(v_ret)) tb_a_dim_cur(a_dim_cur)
    )
  ;

  -- 由因变量对齐至自变量
  if v_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        /
        sm_sc.fv_lpad
        (
          v_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(v_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(v_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][1][ : ][ : ], array[1, 2], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_log_1(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_log_2
--   (
--     array[[1.8, 4.6], [1.4, 3.6]], 
--     array[[2.8, 3.6], [2.4, 1.6]]
--   )
-- select 
--   sm_sc.fv_d_log_2(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_log_2(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])
-- select 
--   sm_sc.fv_d_log_2(sm_sc.fv_new_rand(array[2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_log_2(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[2, 1, 3]))
-- select 
--   sm_sc.fv_d_log_2(sm_sc.fv_new_rand(array[2, 1, 3, 5]), sm_sc.fv_new_rand(array[3, 3, 5]))
-- select 
--   sm_sc.fv_d_log_2(sm_sc.fv_new_rand(array[3, 3, 5]), sm_sc.fv_new_rand(array[2, 1, 3, 5]))
-- select 
--   sm_sc.fv_d_log_2(sm_sc.fv_new_rand(array[4, 2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_log_2(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[4, 2, 1, 3]))

-- drop function if exists sm_sc.fv_d_log_2_un_de_broadcast(float[][], float[][]);
create or replace function sm_sc.fv_d_log_2_un_de_broadcast
(
  i_indepdt_var float[][] , 
  i_co_value    float[][]
)
returns float[][]
as
$$
declare 
begin
  return
    /` (i_indepdt_var *` (^!` i_co_value :: decimal[]) :: float[])
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_log_2_un_de_broadcast
--   (
--     array[2.8, 3.6]
--   , array[1.8, 4.6]
--   )
-- select 
--   sm_sc.fv_d_log_2_un_de_broadcast
--   (
--     array[[1.8, 4.6], [1.4, 3.6]], 
--     array[[2.8, 3.6], [2.4, 1.6]]
--   )
-- select 
--   sm_sc.fv_d_log_2_un_de_broadcast(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_log_2_un_de_broadcast(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])

-- drop function if exists sm_sc.fv_d_mul(float[], int[]);
create or replace function sm_sc.fv_d_mul
(
  i_co_value           float[] 
, i_indepdt_var_len    int[]  default null
)
returns float[]
as
$$
declare 
  v_co_var_len      int[]  := (select array_agg(array_length(i_co_value, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_co_value)) tb_a_cur_dim(a_cur_dim));
  v_depdt_var_len   int[]  :=  
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_indepdt_var_len, a_co_len)))
      from unnest(sm_sc.__fv_mirror_y(i_indepdt_var_len), sm_sc.__fv_mirror_y(v_co_var_len)) tb_a_len(a_indepdt_var_len, a_co_len)
    )
  ;
  v_ret         float[];
begin
  -- 求导
  v_ret := i_co_value;
  
  -- 求导结果对齐维度至因变量
  if array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1) = 3
  then 
    v_ret := array[[[v_ret]]];
  elsif array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1) = 2
  then 
    v_ret := array[[v_ret]];
  elsif array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1) = 1
  then 
    v_ret := array[v_ret];
  end if;
  
  -- 由协参规格对齐至因变量
  if v_co_var_len <> v_depdt_var_len
  then 
    -- 维度对齐
    if array_length(v_co_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      v_co_var_len := 
        sm_sc.fv_lpad
        (
          v_co_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_co_var_len, 1)
        );
    end if;
    
    -- 规格对齐
    v_ret := sm_sc.fv_new(v_ret, v_depdt_var_len / v_co_var_len);
  end if;
  
  -- 由因变量对齐至自变量
  if i_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        / 
        sm_sc.fv_lpad
        (
          i_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(i_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(i_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(i_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(i_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(i_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][ : ][ : ], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(i_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mul(array[2.4, -1.6])
-- select 
--   sm_sc.fv_d_mul(array[[2.8, 3.6], [2.4, -1.6]])
-- select 
--   sm_sc.fv_d_mul(array[[[2.8, 3.6], [2.4, -1.6]],[[2.8, 3.6], [2.4, -1.6]]])
-- select 
--   sm_sc.fv_d_mul(array[[[[2.8, 3.6], [2.4, -1.6]],[[2.8, 3.6], [2.4, -1.6]]],[[[2.8, 3.6], [2.4, -1.6]],[[2.8, 3.6], [2.4, -1.6]]]])
-- select 
--   sm_sc.fv_d_mul(sm_sc.fv_new_rand(array[2, 1, 3]), array[3, 3])
-- select 
--   sm_sc.fv_d_mul(sm_sc.fv_new_rand(array[3, 3]), array[2, 1, 3])
-- drop function if exists sm_sc.fv_d_pow_1(float[], float[], float[]);
create or replace function sm_sc.fv_d_pow_1
(
  i_indepdt_var float[] , 
  i_co_value    float[] ,
  i_depdt_var   float[] default null
)
returns float[]
as
$$
declare 
  v_ret               float[]    ;
  v_depdt_var_len     int[]      ;
  v_indepdt_var_len   int[]      :=
    (
      select 
        array_agg(array_length(i_indepdt_var, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(i_indepdt_var)) tb_a_dim_cur(a_dim_cur)
    )
  ;
begin
  v_ret           :=  
    case 
      when i_depdt_var is not null 
        then i_co_value *` i_depdt_var /` i_indepdt_var      -- -- sm_sc.fv_nullif(... , 0.0 :: float)
      else i_co_value *` (i_indepdt_var ^` (i_co_value -` 1.0 :: float)) 
    end      -- 优先使用 i_depdt_var
  ;
  
  v_depdt_var_len := 
    (
      select 
        array_agg(array_length(v_ret, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(v_ret)) tb_a_dim_cur(a_dim_cur)
    )
  ;

  -- 由因变量对齐至自变量
  if v_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        / 
        sm_sc.fv_lpad
        (
          v_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(v_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(v_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][1][ : ][ : ], array[1, 2], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_pow_1(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_pow_1(array[[2.8, 3.6], [2.4, 1.6]]
--                  , array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_pow_1(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_pow_1(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])

-- select 
--   sm_sc.fv_d_pow_1
--   (
--     array[[2.8, 3.6], [2.4, 1.6]], 
--     array[[1.8, -4.6], [1.4, 3.6]],
--     array[[2.8, 3.6], [2.4, 1.6]] ^` array[[1.8, -4.6], [1.4, 3.6]]
--   )

-- select 
--   sm_sc.fv_d_pow_1(sm_sc.fv_new_rand(array[2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_pow_1(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[2, 1, 3]))
-- select 
--   sm_sc.fv_d_pow_1(sm_sc.fv_new_rand(array[2, 1, 3, 5]), sm_sc.fv_new_rand(array[3, 3, 5]))
-- select 
--   sm_sc.fv_d_pow_1(sm_sc.fv_new_rand(array[3, 3, 5]), sm_sc.fv_new_rand(array[2, 1, 3, 5]))
-- select 
--   sm_sc.fv_d_pow_1(sm_sc.fv_new_rand(array[4, 2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_pow_1(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[4, 2, 1, 3]))

-- drop function if exists sm_sc.fv_d_pow_1_un_de_broadcast(float[], float[], float[]);
create or replace function sm_sc.fv_d_pow_1_un_de_broadcast
(
  i_indepdt_var float[] , 
  i_co_value    float[] ,
  i_depdt_var   float[] default null
)
returns float[]
as
$$
declare 
begin
  return
    case 
      when i_depdt_var is not null 
        then i_co_value *` i_depdt_var /` i_indepdt_var      -- -- sm_sc.fv_nullif(... , 0.0 :: float)
      else i_co_value *` (i_indepdt_var ^` (i_co_value -` 1.0 :: float)) 
    end      -- 优先使用 i_depdt_var
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_pow_1_un_de_broadcast(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_pow_1_un_de_broadcast(array[[2.8, 3.6], [2.4, 1.6]]
--                  , array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_pow_1_un_de_broadcast(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_pow_1_un_de_broadcast(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])

-- select 
--   sm_sc.fv_d_pow_1_un_de_broadcast
--   (
--     array[[2.8, 3.6], [2.4, 1.6]], 
--     array[[1.8, -4.6], [1.4, 3.6]],
--     array[[2.8, 3.6], [2.4, 1.6]] ^` array[[1.8, -4.6], [1.4, 3.6]]
--   )
-- drop function if exists sm_sc.fv_d_pow_2(float[], float[], float[]);
create or replace function sm_sc.fv_d_pow_2
(
  i_indepdt_var float[] , 
  i_co_value    float[] ,
  i_depdt_var   float[] default null
)
returns float[][]
as
$$
declare 
  v_ret               float[]    ;
  v_depdt_var_len     int[]      ;
  v_indepdt_var_len   int[]      :=
    (
      select 
        array_agg(array_length(i_indepdt_var, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(i_indepdt_var)) tb_a_dim_cur(a_dim_cur)
    )
  ;
begin
  v_ret           :=  
    case 
      when i_depdt_var is not null 
        then i_depdt_var *` (^!` i_co_value) 
      else (i_co_value ^` i_indepdt_var) *` (^!` i_co_value) 
    end     -- 优先使用 i_depdt_var
  ;
  
  v_depdt_var_len := 
    (
      select 
        array_agg(array_length(v_ret, a_dim_cur) order by a_dim_cur)
      from generate_series(1, array_ndims(v_ret)) tb_a_dim_cur(a_dim_cur)
    )
  ;

  -- 由因变量对齐至自变量
  if v_indepdt_var_len <> v_depdt_var_len
  then 
    -- 规格对齐
    v_ret := 
      sm_sc.fv_aggr_slice_sum
      (
        v_ret
      , v_depdt_var_len 
        /
        sm_sc.fv_lpad
        (
          v_indepdt_var_len
        , array[1]
        , array_length(v_depdt_var_len, 1) - array_length(v_indepdt_var_len, 1)
        )
      );
    
    -- 维度对齐
    if array_length(v_indepdt_var_len, 1) < array_length(v_depdt_var_len, 1)
    then 
      if array_length(v_depdt_var_len, 1) = 2 and array_length(v_indepdt_var_len, 1) = 1
      then 
        v_ret := sm_sc.fv_mx_ele_2d_2_1d(v_ret);
      elsif array_length(v_depdt_var_len, 1) = 3 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_3d_2_2d(v_ret[1][ : ][ : ], 1);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 2
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_2d(v_ret[1][1][ : ][ : ], array[1, 2], array[1, 1]);
      elsif array_length(v_depdt_var_len, 1) = 4 and array_length(v_indepdt_var_len, 1) = 3
      then 
        v_ret := sm_sc.fv_mx_slice_4d_2_3d(v_ret[1][ : ][ : ][ : ], 1);
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_pow_2(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_pow_2(array[[2.8, 3.6], [2.4, 1.6]]
--                  , array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_pow_2(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_pow_2(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])

-- select 
--   sm_sc.fv_d_pow_2
--   ( 
--     array[[1.8, -4.6], [1.4, 3.6]],
--     array[[2.8, 3.6], [2.4, 1.6]],
--     array[[2.8, 3.6], [2.4, 1.6]] 
--       ^` array[[1.8, -4.6], [1.4, 3.6]]
--   )

-- select 
--   sm_sc.fv_d_pow_2(sm_sc.fv_new_rand(array[2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_pow_2(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[2, 1, 3]))
-- select 
--   sm_sc.fv_d_pow_2(sm_sc.fv_new_rand(array[2, 1, 3, 5]), sm_sc.fv_new_rand(array[3, 3, 5]))
-- select 
--   sm_sc.fv_d_pow_2(sm_sc.fv_new_rand(array[3, 3, 5]), sm_sc.fv_new_rand(array[2, 1, 3, 5]))
-- select 
--   sm_sc.fv_d_pow_2(sm_sc.fv_new_rand(array[4, 2, 1, 3]), sm_sc.fv_new_rand(array[3, 3]))
-- select 
--   sm_sc.fv_d_pow_2(sm_sc.fv_new_rand(array[3, 3]), sm_sc.fv_new_rand(array[4, 2, 1, 3]))
-- drop function if exists sm_sc.fv_d_pow_2_un_de_broadcast(float[][], float[][], float[][]);
create or replace function sm_sc.fv_d_pow_2_un_de_broadcast
(
  i_indepdt_var float[][] , 
  i_co_value    float[][] ,
  i_depdt_var   float[][] default null
)
returns float[][]
as
$$
declare 
begin
  return
    case 
      when i_depdt_var is not null 
        then i_depdt_var *` (^!` i_co_value) 
      else (i_co_value ^` i_indepdt_var) *` (^!` i_co_value) 
    end     -- 优先使用 i_depdt_var
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_pow_2_un_de_broadcast(array[2.8, 3.6]
--                  , array[1.8, 4.6])
-- select 
--   sm_sc.fv_d_pow_2_un_de_broadcast(array[[2.8, 3.6], [2.4, 1.6]]
--                  , array[[1.8, 4.6], [1.4, 3.6]])
-- select 
--   sm_sc.fv_d_pow_2_un_de_broadcast(array[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]
--                  , array[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]])
-- select 
--   sm_sc.fv_d_pow_2_un_de_broadcast(array[[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]],[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]]]
--                  , array[[[[1.8, 4.6], [1.4, 3.6]],[[2.8, 3.6], [2.4, 1.6]]],[[[2.8, 3.6], [2.4, 1.6]],[[1.8, 4.6], [1.4, 3.6]]]])

-- select 
--   sm_sc.fv_d_pow_2_un_de_broadcast
--   ( 
--     array[[1.8, -4.6], [1.4, 3.6]],
--     array[[2.8, 3.6], [2.4, 1.6]],
--     array[[2.8, 3.6], [2.4, 1.6]] 
--       ^` array[[1.8, -4.6], [1.4, 3.6]]
--   )
-- drop function if exists sm_sc.fv_d_sub_1(int[], int[]);
create or replace function sm_sc.fv_d_sub_1
(
  i_depdt_var_len      int[]
, i_indepdt_var_len    int[]   default null
)
returns float[]
as
$$
declare 
begin
  if i_indepdt_var_len <> i_depdt_var_len
  then 
    return 
      array_fill
      (
        sm_sc.fv_aggr_slice_prod
        (
          i_depdt_var_len 
          / 
          sm_sc.fv_lpad
          (
            i_indepdt_var_len
          , array[1]
          , array_length(i_depdt_var_len, 1) - array_length(i_indepdt_var_len, 1)
          )
        ) :: float
      , i_indepdt_var_len
      )
    ;
  else 
    return array_fill(1.0 :: float, i_depdt_var_len);
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_sub_1(array[2, 3])
-- select 
--   sm_sc.fv_d_sub_1(array[2, 3, 4])
-- select 
--   sm_sc.fv_d_sub_1(array[2, 3, 4, 5])
-- select 
--   sm_sc.fv_d_sub_1(array[   3, 1, 5], array[2, 3, 4, 5])
-- drop function if exists sm_sc.fv_d_sub_2(int[], int[]);
create or replace function sm_sc.fv_d_sub_2
(
  i_depdt_var_len      int[]
, i_indepdt_var_len    int[]   default null
)
returns float[]
as
$$
declare 
begin
  if i_indepdt_var_len <> i_depdt_var_len
  then 
    return 
      array_fill
      (
        -
        sm_sc.fv_aggr_slice_prod
        (
          i_depdt_var_len 
          / 
          sm_sc.fv_lpad
          (
            i_indepdt_var_len
          , array[1]
          , array_length(i_depdt_var_len, 1) - array_length(i_indepdt_var_len, 1)
          )
        ) :: float
      , i_indepdt_var_len
      )
    ;
  else 
    return array_fill(-1.0 :: float, i_depdt_var_len);
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_sub_2(array[2, 3])
-- select 
--   sm_sc.fv_d_sub_2(array[2, 3, 4])
-- select 
--   sm_sc.fv_d_sub_2(array[2, 3, 4, 5])
-- select 
--   sm_sc.fv_d_sub_2(array[   3, 1, 5], array[2, 3, 4, 5])
-- drop function if exists sm_sc.fv_opr_abs(anyarray);
create or replace function sm_sc.fv_opr_abs
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     i_right%type   := case array_ndims(i_right) when 2 then array_fill(i_right[0], array[array_length(i_right, 1), array_length(i_right, 2)]) when 1 then array_fill(null::float, array[array_length(i_right, 1)]) else null end;

begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- abs([][])
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := abs(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- abs []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := abs(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- abs [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            abs(i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- abs [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              abs(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_abs(array[array[12.3, -25.1], array[-2.56, 3.25]])
-- select sm_sc.fv_opr_abs(array[12.3, -25.1, 28.33])
-- select sm_sc.fv_opr_abs(array[]::float[])
-- select sm_sc.fv_opr_abs(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_abs
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_abs
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_add(anyarray, anyarray);
create or replace function sm_sc.fv_opr_add
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_add_py(i_left, i_right);
  else  
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] + []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return i_right;
    -- [] + array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return i_left;
    
    elsif i_left is null or i_right is null
    then
      return i_left;
    
    -- [] + []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        i_left[v_y_cur] := i_left[v_y_cur] + i_right[v_y_cur];
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
    
    -- [][] + [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] + i_right[v_y_cur][v_x_cur];
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
      
    -- [][][] + [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              + i_right[v_y_cur][v_x_cur][v_x3_cur] 
            ;
          end loop;    
        end loop;
      end loop;
      return i_left;
      
    -- [][][][] + [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                + i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return i_left;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_add
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_add
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_add
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_add(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_add(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_add(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_add(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_add
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_add
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_opr_add
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_add(anyarray, anyelement);
create or replace function sm_sc.fv_opr_add
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- [][] + float
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_add_py(i_left, i_right);
    
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] + i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] + float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] + i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] + float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            + i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
  
  -- [][][][] + float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              + i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_add(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_add(array[]::float[], 8.8)
-- select sm_sc.fv_opr_add(array[]::float[], null)
-- select sm_sc.fv_opr_add(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_add(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_add(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_add
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_add
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_add(anyelement, anyarray);
create or replace function sm_sc.fv_opr_add
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_add_py(i_left, i_right);  

  -- float + [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left + i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float + []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left + i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- float + [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            + i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float + [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              + i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_add(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_add(8.8, array[]::float[] )
-- select sm_sc.fv_opr_add(null, array[]::float[])
-- select sm_sc.fv_opr_add(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_add(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_add(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_add
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_add
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_add_py(float[], float[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(float[], float);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     float[]    ,
  i_right    float
)
returns float[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) + i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(float, float[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     float    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_add_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_add_py
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_add_py(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_add_py(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_add_py(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_add_py(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_add_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_add_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_opr_add_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )


-- select sm_sc.fv_opr_add_py(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_add_py(array[]::float[], 8.8)
-- select sm_sc.fv_opr_add_py(array[]::float[], null)
-- select sm_sc.fv_opr_add_py(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_add_py(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_add_py(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_add_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_add_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )


-- select sm_sc.fv_opr_add_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_add_py(8.8, array[]::float[] )
-- select sm_sc.fv_opr_add_py(null, array[]::float[])
-- select sm_sc.fv_opr_add_py(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_add_py(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_add_py(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_add_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_add_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- --------------------------
-- drop function if exists sm_sc.fv_opr_add_py(int[], int[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(int[], int);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     int[]    ,
  i_right    int
)
returns int[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) + i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(int, int[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     int    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- --------------------------
-- drop function if exists sm_sc.fv_opr_add_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns bigint[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) + i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- --------------------------
-- drop function if exists sm_sc.fv_opr_add_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns decimal[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) + i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_add_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_add_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left + arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_add_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_add_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_add_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_add_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_add_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_add_py(array[[12,9,18]],             3::   int)
-- set search_path to sm_sc;

-- -- 逻辑运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_and(boolean[], boolean[]);
create or replace function sm_sc.fv_opr_and
(
  i_left     boolean[]    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] and []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] and array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::boolean[];
  
  -- [] and []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] and i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] and [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] and i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] and [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] and i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] and [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] and i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_and
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   )

-- select 
--   sm_sc.fv_opr_and
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_and
--   (
--     array[array[true, false], array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[true], array[false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true], array[false]]
--   );
-- -- select sm_sc.fv_opr_and
-- --   (
-- --     array[true, false],
-- --     array[array[false, true], array[false, true]]
-- --   );
-- -- select sm_sc.fv_opr_and
-- --   (
-- --     array[array[true, false], array[true, false]],
-- --     array[true, false]
-- --   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true, false]]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[], array []]::boolean[],
--     array[]::boolean[]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[], array []]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[true, false],
--     array[true, false]
--   );
-- select sm_sc.fv_opr_and(array[true], array[true, false]);
-- select sm_sc.fv_opr_and(array[false], array[array[true, false]]);
-- select sm_sc.fv_opr_and(array[array[true]], array[true, false]);
-- select sm_sc.fv_opr_and(array[array[false]], array[array[true, false]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_add
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , array[[[true,false,true],[false,false,true]],[[true,false,false],[false,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_add
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , array[[[[false,false,true],[true,false,false]],[[true,false,true],[false,false,true]]],[[[true,false,false],[false,false,true]],[[false,true,false],[false,false,true]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_and(boolean[], boolean);
create or replace function sm_sc.fv_opr_and
(
  i_left     boolean[]    ,
  i_right    boolean
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] and boolean = null :: boolean[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];

  -- [][] and boolean
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] and i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] and boolean
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] and i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] and boolean
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            and i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] and boolean
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              and i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_and(array[array[true, false], array[false, true]], false)
-- select sm_sc.fv_opr_and(array[true, false], false)
-- select sm_sc.fv_opr_and(array[]::boolean[], true)
-- select sm_sc.fv_opr_and(array[array[], array []]::boolean[], false)
-- select 
--   sm_sc.fv_opr_and
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , true
--   )
-- select 
--   sm_sc.fv_opr_and
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , false
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_and(boolean, boolean[]);
create or replace function sm_sc.fv_opr_and
(
  i_left     boolean    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] and boolean = null :: boolean[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::boolean[];

  -- boolean and [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left and i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- boolean and []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left and i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- boolean and [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            and i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- boolean and [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              and i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_and(false, array[array[false, true], array[true, false]])
-- select sm_sc.fv_opr_and(false, array[true, false])
-- select sm_sc.fv_opr_and(false, array[]::boolean[] )
-- select sm_sc.fv_opr_and(true, array[array[], array []]::boolean[])
-- select 
--   sm_sc.fv_opr_and
--   (
--     true
--   , array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_and
--   (
--     false
--   , array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   )

-- -------------------------------------------------------------------------------------------------------------------------------

-- -- 位运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_and(varbit[], varbit[]);
create or replace function sm_sc.fv_opr_and
(
  i_left     varbit[]    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] + []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] + array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::varbit;
  
  -- [] & []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] & i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] & [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] & i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] & [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] & i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] & [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] & i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_and
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_and
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'101'], array[B'011']]
--   );
-- -- select sm_sc.fv_opr_and
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[B'011', B'101'], array[B'011', B'010']]
-- --   );
-- -- select sm_sc.fv_opr_and
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[B'010', B'011']
-- --   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'010']]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[B'010', B'011']]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[], array []]::varbit[],
--     array[]::varbit[]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[array[], array []]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_and
--   (
--     array[B'101', B'011'],
--     array[B'010', B'011']
--   );
-- select sm_sc.fv_opr_and(array[B'101'], array[B'101', B'011']);
-- select sm_sc.fv_opr_and(array[B'101'], array[array[B'101', B'011']]);
-- select sm_sc.fv_opr_and(array[array[B'101']], array[B'101', B'011']);
-- select sm_sc.fv_opr_and(array[array[B'101']], array[array[B'101', B'011']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_and
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , array[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]]
--   )
-- select 
--   sm_sc.fv_opr_and
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , array[[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]],[[[B'001',B'010',B'100'],[B'011',B'010',B'111']],[[B'010',B'100',B'001'],[B'101',B'110',B'001']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_and(varbit[], varbit);
create or replace function sm_sc.fv_opr_and
(
  i_left     varbit[]    ,
  i_right    varbit
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] & varbit = null :: varbit[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::varbit[];

  -- [][] & varbit
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] & i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] & varbit
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] & i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
  -- [][][] + varbit
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            & i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] + varbit
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              & i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;    

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_and(array[array[B'010', B'011'], array[B'011', B'010']], B'011')
-- select sm_sc.fv_opr_and(array[B'010011', B'011010'], B'011101')
-- select sm_sc.fv_opr_and(array[]::varbit[], B'010')
-- select sm_sc.fv_opr_and(array[array[], array []]::varbit[], B'011')
-- select 
--   sm_sc.fv_opr_and
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , B'010'
--   )
-- select 
--   sm_sc.fv_opr_and
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , B'010'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_and(varbit, varbit[]);
create or replace function sm_sc.fv_opr_and
(
  i_left     varbit    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] & varbit = null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::varbit[];

  -- varbit & [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left & i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- varbit & []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left & i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- varbit & [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            & i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- varbit & [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              & i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_and(B'110', array[array[B'011', B'010'], array[B'101', B'110']])
-- select sm_sc.fv_opr_and(B'011101', array[B'010011', B'011010'])
-- select sm_sc.fv_opr_and(B'011', array[]::varbit[] )
-- select sm_sc.fv_opr_and(B'010', array[array[], array []]::varbit[])
-- select 
--   sm_sc.fv_opr_and
--   (
--     B'010'
--   , array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   )
-- select 
--   sm_sc.fv_opr_and
--   (
--     B'010'
--   , array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   )
-- drop function if exists sm_sc.fv_opr_ceil(anyarray, int);
create or replace function sm_sc.fv_opr_ceil
(
  i_left         anyarray,
  i_num_digits    int
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_balan   decimal  := 5.0 * power(0.1, i_num_digits + 1) :: decimal  ;
begin
  -- ceil(null :: anyarray, int) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- ceil([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        -- -- i_left[v_y_cur][v_x_cur] := 
        -- --   case 
        -- --     when i_left[v_y_cur][v_x_cur] = round(i_left[v_y_cur][v_x_cur], i_num_digits) 
        -- --       then i_left[v_y_cur][v_x_cur]
        -- --     else round(i_left[v_y_cur][v_x_cur] + v_balan, i_num_digits) 
        -- --   end;
        if i_left[v_y_cur][v_x_cur] <> round(i_left[v_y_cur][v_x_cur], i_num_digits)
        then 
          i_left[v_y_cur][v_x_cur] := round(i_left[v_y_cur][v_x_cur] + v_balan, i_num_digits);
        end if;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- ceil []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      -- -- i_left[v_y_cur] := 
      -- --   case 
      -- --     when i_left[v_y_cur] = round(i_left[v_y_cur], i_num_digits) 
      -- --       then i_left[v_y_cur]
      -- --     else round(i_left[v_y_cur] + v_balan, i_num_digits) 
      -- --   end;
      if i_left[v_y_cur] <> round(i_left[v_y_cur], i_num_digits)
      then 
        i_left[v_y_cur] := round(i_left[v_y_cur] + v_balan, i_num_digits);
      end if;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- ceil [][][]
  elsif array_ndims(i_left) = 3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          -- -- i_left[v_y_cur][v_x_cur][v_x3_cur] := 
          -- --   case 
          -- --     when i_left[v_y_cur][v_x_cur][v_x3_cur] = round(i_left[v_y_cur][v_x_cur][v_x3_cur], i_num_digits) 
          -- --       then i_left[v_y_cur][v_x_cur][v_x3_cur]
          -- --     else round(i_left[v_y_cur][v_x_cur][v_x3_cur] + v_balan, i_num_digits) 
          -- --   end;
          if i_left[v_y_cur][v_x_cur][v_x3_cur] <> round(i_left[v_y_cur][v_x_cur][v_x3_cur], i_num_digits)
          then 
            i_left[v_y_cur][v_x_cur][v_x3_cur] := round(i_left[v_y_cur][v_x_cur][v_x3_cur] + v_balan, i_num_digits);
          end if;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- ceil [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            -- -- i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
            -- --   case 
            -- --     when i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_num_digits) 
            -- --       then i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]
            -- --     else round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] + v_balan, i_num_digits) 
            -- --   end;
            if i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_num_digits)
            then 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] + v_balan, i_num_digits);
            end if;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_ceil(array[array[-12.34584, 25.143540], array[2.560454, -3.2504870]], 2)
-- select sm_sc.fv_opr_ceil(array[-12.304725, 25.1874, -28.33407], 0)
-- select sm_sc.fv_opr_ceil(array[0, 1.00003, -2.00005, 0.99993, -1.999995, 1.01, -1.01], 2)
-- select sm_sc.fv_opr_ceil(array[]::decimal[])
-- select sm_sc.fv_opr_ceil(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_ceil(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[], 2)
-- select sm_sc.fv_opr_ceil(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[], 1)

-- ---------------------------------------------------------------------------
-- -- 以下单入参函数是必要的，无法用设置默认值的双入参代替，且一定不能用后者（两者对单入参调用冲突），后者无法支持单目运算符
-- drop function if exists sm_sc.fv_opr_ceil(anyarray);
create or replace function sm_sc.fv_opr_ceil
(
  i_left     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- ceil(null :: anyarray) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- ceil([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := ceil(i_left[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- ceil []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := ceil(i_left[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- ceil [][][]
  elsif array_ndims(i_left) = 3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            ceil(i_left[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- ceil [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ceil(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_ceil(array[array[12.35450, -25.1507], array[-2.56405, 3.2544444446]])
-- select sm_sc.fv_opr_ceil(array[12.3101, -25.711, 28.33920])
-- select sm_sc.fv_opr_ceil(array[0, 1.0 :: decimal, -2.0])
-- select sm_sc.fv_opr_ceil(array[]::decimal[])
-- select sm_sc.fv_opr_ceil(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_ceil(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[])
-- select sm_sc.fv_opr_ceil(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_compare(anyarray, anyarray);
create or replace function sm_sc.fv_opr_compare
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns int[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     int[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  v_ret := array_fill(null::int, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] compare []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return array_fill(null::int, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
  -- [] compare array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return array_fill(null::int, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
  
  elsif i_left is null or i_right is null
  then
    return null::int[];
  
  -- [] compare []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := sign(i_left[v_y_cur] - i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][] compare [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := sign(i_left[v_y_cur][v_x_cur] - i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- [][][] compare [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            sign(i_left[v_y_cur][v_x_cur][v_x3_cur] - i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] compare [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              sign(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] - i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_compare
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_compare
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_compare
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_compare
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_compare
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_compare
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.fv_opr_compare(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_compare(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_compare(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_compare(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_compare
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_compare
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_compare(anyarray, anyelement);
create or replace function sm_sc.fv_opr_compare
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns int[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     int[] ;
begin
  -- compare(null :: float[][], float) = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::int[];

  -- compare([][], float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::int, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sign(i_left[v_y_cur][v_x_cur] - i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- compare([], float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::int, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := sign(i_left[v_y_cur] - i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- compare ([][][], float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::int, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := sign(i_left[v_y_cur][v_x_cur][v_x3_cur] - i_right);
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- compare ([][][][], float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::int, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := sign(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] - i_right);
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_compare(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_compare(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.fv_opr_compare(array[]::float[], 8.8)
-- select sm_sc.fv_opr_compare(array[array[], array []]::float[], 8.8)
-- select 
--   sm_sc.fv_opr_compare
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_compare
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_compare(anyelement, anyarray);
create or replace function sm_sc.fv_opr_compare
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns int[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     int[] ;
begin
  -- compare(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- compare(float + [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::int, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sign(i_left - i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- compare(float, [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::int, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := sign(i_left - i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- compare (float, [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::int, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := sign(i_left - i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- compare(float, [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::int, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := sign(i_left - i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_compare(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_compare(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_compare(8.8, array[]::float[] )
-- select sm_sc.fv_opr_compare(8.8, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_compare
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_compare
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_concat(anyarray, anyarray);
create or replace function sm_sc.fv_opr_concat
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] || []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] || array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return i_left;
  
  -- [] || []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := i_left[v_y_cur] || i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] || [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] || i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] || [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            || i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] || [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              || i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_concat
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: text[]
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) :: text[]
--   )
-- select 
--   sm_sc.fv_opr_concat
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_concat
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'101'], array[B'011']]
--   );
-- -- select sm_sc.fv_opr_concat
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[B'011', B'101'], array[B'011', B'010']]
-- --   );
-- -- select sm_sc.fv_opr_concat
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[B'010', B'011']
-- --   );
-- select sm_sc.fv_opr_concat
--   (
--     array[array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'010']]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[B'010', B'011']]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[array[], array []]::varbit[],
--     array[]::varbit[]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[array[], array []]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_concat
--   (
--     array[B'101', B'011'],
--     array[B'010', B'011']
--   );
-- select sm_sc.fv_opr_concat(array[B'101'], array[B'101', B'011']);
-- select sm_sc.fv_opr_concat(array[B'101'], array[array[B'101', B'011']]);
-- select sm_sc.fv_opr_concat(array[array[B'101']], array[B'101', B'011']);
-- select sm_sc.fv_opr_concat(array[array[B'101']], array[array[B'101', B'011']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_concat
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , array[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]]
--   )
-- select 
--   sm_sc.fv_opr_concat
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , array[[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]],[[[B'001',B'010',B'100'],[B'011',B'010',B'111']],[[B'010',B'100',B'001'],[B'101',B'110',B'001']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_concat(anyarray, anyelement);
create or replace function sm_sc.fv_opr_concat
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- concat(null :: float[][], float = null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- concat([][], float)
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := concat(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- concat([], float)
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := concat(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- concat([][][], float)
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            concat(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right)
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- concat([][][][], float)
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              concat(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_concat(array[array[B'010', B'011'], array[B'011', B'010']], B'011')
-- select sm_sc.fv_opr_concat(array[B'010011', B'011010'], B'011101')
-- select sm_sc.fv_opr_concat(array[]::varbit[], B'010')
-- select sm_sc.fv_opr_concat(array[array[], array []]::varbit[], B'011')
-- select 
--   sm_sc.fv_opr_concat
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , B'010'
--   )
-- select 
--   sm_sc.fv_opr_concat
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , B'010'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_concat(anyelement, anyarray);
create or replace function sm_sc.fv_opr_concat
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- concat(null :: float[][], float = null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- concat(float, [][])
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := concat(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- concat(float, [])
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := concat(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- concat(float, [][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            concat(i_left, i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- concat(float, [][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
            concat(i_left, i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_concat(B'110', array[array[B'011', B'010'], array[B'101', B'110']])
-- select sm_sc.fv_opr_concat(B'011101', array[B'010011', B'011010'])
-- select sm_sc.fv_opr_concat(B'011', array[]::varbit[] )
-- select sm_sc.fv_opr_concat(B'010', array[array[], array []]::varbit[])
-- select 
--   sm_sc.fv_opr_concat
--   (
--     B'010'
--   , array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   )
-- select 
--   sm_sc.fv_opr_concat
--   (
--     B'010'
--   , array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   )

-- drop function if exists sm_sc.fv_opr_conjugate(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_opr_conjugate
(
  i_right     sm_sc.typ_l_complex[]
)
returns sm_sc.typ_l_complex[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;

begin
  -- log(null :: sm_sc.typ_l_complex[][], sm_sc.typ_l_complex) = null :: sm_sc.typ_l_complex[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- ~([][])
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := ~ i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- ~ []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := ~ i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- ~ [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            ~ i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- ~ [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ~ i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_conjugate(array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex])
-- select sm_sc.fv_opr_conjugate(array[array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex], array[(12.3, 0.0 :: float)::sm_sc.typ_l_complex, (0.0 :: float, 3.25)::sm_sc.typ_l_complex]])
-- select sm_sc.fv_opr_conjugate(array[]::sm_sc.typ_l_complex[])
-- select sm_sc.fv_opr_conjugate(array[array[], array []]::sm_sc.typ_l_complex[])
-- select 
--   sm_sc.fv_opr_conjugate
--   (
--     array[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]] :: sm_sc.typ_l_complex[]
--   )
-- select 
--   sm_sc.fv_opr_conjugate
--   (
--     array[[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]],[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]]] :: sm_sc.typ_l_complex[]
--   )
-- drop function if exists sm_sc.fv_opr_conjugate_45(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_opr_conjugate_45
(
  i_right     sm_sc.typ_l_complex[]
)
returns sm_sc.typ_l_complex[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- sm_sc.fv_opr_conjugate_45([][])
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := sm_sc.fv_opr_conjugate_45(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- sm_sc.fv_opr_conjugate_45 []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := sm_sc.fv_opr_conjugate_45(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- sm_sc.fv_opr_conjugate_45 [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_opr_conjugate_45(i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- sm_sc.fv_opr_conjugate_45 [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_opr_conjugate_45(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_conjugate_45(array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_conjugate_45(array[12.3, 25.1, 28.33])
-- select sm_sc.fv_opr_conjugate_45(array[array[(12.3, 10.6)::sm_sc.typ_l_complex, (-25.1, 5.6)::sm_sc.typ_l_complex], array[(-2.56, 4.1)::sm_sc.typ_l_complex, (3.25, 5.2)::sm_sc.typ_l_complex]])
-- select sm_sc.fv_opr_conjugate_45(array[(12.3, 5.1)::sm_sc.typ_l_complex, (-25.1, 7.8)::sm_sc.typ_l_complex, (28.33, 1.9)::sm_sc.typ_l_complex])
-- select sm_sc.fv_opr_conjugate_45(array[]::float[])
-- select sm_sc.fv_opr_conjugate_45(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_conjugate_45
--   (
--     array[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]] :: sm_sc.typ_l_complex[]
--   )
-- select 
--   sm_sc.fv_opr_conjugate_45
--   (
--     array[[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]],[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]]] :: sm_sc.typ_l_complex[]
--   )
-- drop function if exists sm_sc.fv_opr_conjugate_i(sm_sc.typ_l_complex[][]);
create or replace function sm_sc.fv_opr_conjugate_i
(
  i_right     sm_sc.typ_l_complex[][]
)
returns sm_sc.typ_l_complex[][]
as
$$
declare -- here
  v_y_cur   int  := 1  ;
  v_x_cur   int  := 1  ;
  v_x3_cur  int  := 1  ;
  v_x4_cur  int  := 1  ;
  v_ret     sm_sc.typ_l_complex[][]   
    := case array_ndims(i_right) 
         when 2 then array_fill(null::sm_sc.typ_l_complex, array[array_length(i_right, 2), array_length(i_right, 1)]) 
         when 3 then array_fill(null::sm_sc.typ_l_complex, array[array_length(i_right, 1), array_length(i_right, 3), array_length(i_right, 2)]) 
         when 4 then array_fill(null::sm_sc.typ_l_complex, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 4), array_length(i_right, 3)]) 
         else null::sm_sc.typ_l_complex[] 
       end;

begin
  -- log(null :: sm_sc.typ_l_complex[][], sm_sc.typ_l_complex) = null :: sm_sc.typ_l_complex[][]
  if array_length(i_right, 1) is null
  then 
    return i_right;
  end if;

  -- ~([][])
  if array_ndims(i_right) = 2
  then    
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop 
        v_ret[v_x_cur][v_y_cur] := ~ i_right[v_y_cur][v_x_cur];
      end loop;
    end loop;
    return v_ret;
  -- ~([][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop 
      for v_x3_cur in 1 .. array_length(i_right, 3)
      loop 
        for v_x_cur in 1 .. array_length(i_right, 2)
        loop 
          v_ret[v_y_cur][v_x3_cur][v_x_cur] := ~ i_right[v_y_cur][v_x_cur][v_x3_cur];
        end loop;
      end loop;
    end loop;
    return v_ret;
  -- ~([][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop 
        for v_x4_cur in 1 .. array_length(i_right, 4)
        loop 
          for v_x3_cur in 1 .. array_length(i_right, 3)
          loop 
            v_ret[v_y_cur][v_x_cur][v_x4_cur][v_x3_cur] := ~ i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur];
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_conjugate_i(array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex])
-- select sm_sc.fv_opr_conjugate_i(array[array[(12.3, -25.1), (-2.56, 3.25)], array[(12.3, 0.0 :: float), (0.0 :: float, 3.25)]] :: sm_sc.typ_l_complex[])
-- select sm_sc.fv_opr_conjugate_i(array[]::sm_sc.typ_l_complex[])
-- select sm_sc.fv_opr_conjugate_i(array[array[], array []]::sm_sc.typ_l_complex[])
-- drop function if exists sm_sc.fv_div_for_zero(float, float);
create or replace function sm_sc.fv_div_for_zero
(
  i_left       float   ,
  i_right      float
)
returns float
as 
$$
begin 
  return 
    coalesce 
    (
      i_left / nullif(i_right, 0.0 :: float),
      case
        when i_left > 0.0
          then 'Infinity'
        when i_left < 0.0
          then '-Infinity'
        when i_left = 0.0
          then 0.0
      end
    )
  ;
-- exception when others then
--   raise exception 'i_left: %; i_right: %; sqlerrm: %;', i_left, i_right, sqlerrm;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_div_for_zero(anyelement, anyelement);
create or replace function sm_sc.fv_div_for_zero
(
  i_left       anyelement   ,
  i_right      anyelement
)
returns anyelement
as 
$$
begin 
  return 
    i_left / i_right
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- set search_path to sm_sc;
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_div(anyarray, anyarray);
create or replace function sm_sc.fv_opr_div
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_div_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] / []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return i_right;
    -- [] / array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return i_left;
    
    elsif i_left is null or i_right is null
    then
      return i_left;
    
    -- [] / []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        i_left[v_y_cur] := sm_sc.fv_div_for_zero(i_left[v_y_cur], i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
    
    -- [][] / [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          i_left[v_y_cur][v_x_cur] := sm_sc.fv_div_for_zero(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
      
    -- [][][] / [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              sm_sc.fv_div_for_zero(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right[v_y_cur][v_x_cur][v_x3_cur]) 
            ;
          end loop;    
        end loop;
      end loop;
      return i_left;
      
    -- [][][][] / [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                sm_sc.fv_div_for_zero(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return i_left;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_div
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_div
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_div    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-12.3, 12.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_div    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[12.3, -12.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[(1.2, 2.3)::sm_sc.typ_l_complex],
--     array[(2.1, 3.2)::sm_sc.typ_l_complex]
--   );
-- select sm_sc.fv_opr_div
--   (
--     array[array[(1.2, 2.3)::sm_sc.typ_l_complex]],
--     array[array[(2.1, 3.2)::sm_sc.typ_l_complex]]
--   );
-- select sm_sc.fv_opr_div(array[1]::float[], array[1,2,3]::float[]);
-- select sm_sc.fv_opr_div(array[1]::float[], array[array[1,2,3]]::float[]);
-- select sm_sc.fv_opr_div(array[array[1]]::float[], array[1,2,3]::float[]);
-- select sm_sc.fv_opr_div(array[array[1]]::float[], array[array[1,2,3]]::float[]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_div
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_div
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_div(anyarray, anyelement);
create or replace function sm_sc.fv_opr_div
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_val_0   alias for $0;
begin
  v_val_0 := array[0.0 :: float];
  -- null :: float[][] / float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_div_py(i_left, i_right);  

  -- [][] / float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := sm_sc.fv_div_for_zero(i_left[v_y_cur][v_x_cur], i_right);   -- i_left[v_y_cur][v_x_cur] / nullif(i_right, v_val_0[1]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] / float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := sm_sc.fv_div_for_zero(i_left[v_y_cur], i_right);   -- i_left[v_y_cur] / nullif(i_right, v_val_0[1]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] / float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_div_for_zero
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
            , i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] / float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_div_for_zero
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_div(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_div(array[]::float[], 8.8)
-- select sm_sc.fv_opr_div(array[]::float[], null)
-- select sm_sc.fv_opr_div(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_div(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_div(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_div
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_div
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_div(anyelement, anyarray);
create or replace function sm_sc.fv_opr_div
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_val_0   alias for $0;
begin
  v_val_0 := array[0.0 :: float];
  -- null :: float[][] / float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray; 
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_div_py(i_left, i_right);   

  -- float / [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := sm_sc.fv_div_for_zero(i_left, i_right[v_y_cur][v_x_cur]);   -- i_left / nullif(i_right[v_y_cur][v_x_cur], v_val_0[1]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float / []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := sm_sc.fv_div_for_zero(i_left, i_right[v_y_cur]);   -- i_left / nullif(i_right[v_y_cur], v_val_0[1]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float / [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_div_for_zero
            (
              i_left 
            , i_right[v_y_cur][v_x_cur][v_x3_cur] 
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float / [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_div_for_zero
              (
                i_left 
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_div(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_div(8.8, array[]::float[] )
-- select sm_sc.fv_opr_div(null, array[]::float[])
-- select sm_sc.fv_opr_div(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_div(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_div(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_div
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_div
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_div(anyarray);
create or replace function sm_sc.fv_opr_div
(
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_div_py(i_right); 
    
  -- / [][]
  elsif array_ndims(i_right) = 2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := sm_sc.fv_div_for_zero(1.0 :: float, i_right[v_y_cur][v_x_cur]);   -- 1.0 :: float/ nullif(i_right[v_y_cur][v_x_cur], v_val_0[1]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- / []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := sm_sc.fv_div_for_zero(1.0 :: float, i_right[v_y_cur]);   -- 1.0 :: float/ nullif(i_right[v_y_cur], v_val_0[1]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- / [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_div_for_zero
            (
              1.0 :: float 
            , i_right[v_y_cur][v_x_cur][v_x3_cur] 
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- / [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_div_for_zero
              (
                1.0 :: float 
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_div(array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_div(array[12.3, 25.1, 2.56, 3.25])
-- select 
--   sm_sc.fv_opr_div
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_div
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_div_py(float[], float[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) / arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(float[], float);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     float[]    ,
  i_right    float
)
returns float[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) / i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(float, float[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     float    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (i_left / arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_div_py(float[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (1.0 / arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_div_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_div_py
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_div_py(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_div_py(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_div_py(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_div_py(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_div_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_div_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_opr_div_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )


-- select sm_sc.fv_opr_div_py(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_div_py(array[]::float[], 8.8)
-- select sm_sc.fv_opr_div_py(array[]::float[], null)
-- select sm_sc.fv_opr_div_py(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_div_py(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_div_py(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_div_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_div_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )


-- select sm_sc.fv_opr_div_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_div_py(8.8, array[]::float[] )
-- select sm_sc.fv_opr_div_py(null, array[]::float[])
-- select sm_sc.fv_opr_div_py(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_div_py(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_div_py(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_div_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_div_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_div_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) // arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) // i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (i_left // arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_div_py(int[], int[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) // arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(int[], int);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     int[]    ,
  i_right    int
)
returns int[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) // i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(int, int[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     int    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  return (i_left // arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_div_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) / arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) / i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_div_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (i_left / arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_div_py(decimal[]);
create or replace function sm_sc.fv_opr_div_py
(
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  from decimal import Decimal
  return (Decimal(1.0) / arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_div_py(array[0.9])

-- select
--   sm_sc.fv_opr_mul_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mul_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_mul_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mul_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_mul_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_mul_py(array[[12,9,18]],             3::   int)
-- drop function if exists sm_sc.fv_opr_exp(anyarray);
create or replace function sm_sc.fv_opr_exp
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_zero    i_right%type  := array[0.0];
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_exp_py(i_right);
  
  -- exp([][])
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := case when i_right[v_y_cur][v_x_cur] :: float = '-inf' :: float then v_zero[1] else exp(i_right[v_y_cur][v_x_cur]) end;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- exp []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := case when i_right[v_y_cur] :: float = '-inf' :: float then v_zero[1] else exp(i_right[v_y_cur]) end;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- exp([][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            case when i_right[v_y_cur][v_x_cur][v_x3_cur] :: float = '-inf' :: float then v_zero[1] else exp(i_right[v_y_cur][v_x_cur][v_x3_cur]) end
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- exp([][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              case when i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] :: float = '-inf' :: float then v_zero[1] else exp(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) end
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_exp(array[array[12.3, 25.1], array[2.56, '-inf' :: float]])
-- select sm_sc.fv_opr_exp(array[12.3, 25.1, 28.33, '-inf' :: float])
-- select sm_sc.fv_opr_exp(array[]::float[])
-- select sm_sc.fv_opr_exp(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_exp
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_exp
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_exp_py(float[]);
create or replace function sm_sc.fv_opr_exp_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.exp(1) ** np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- select 
--   sm_sc.fv_opr_exp_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5])
--   )
-- drop function if exists sm_sc.fv_opr_floor(anyarray, int);
create or replace function sm_sc.fv_opr_floor
(
  i_left        anyarray,
  i_num_digits    int   -- default 0
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_balan   decimal  := 5.0 * power(0.1, i_num_digits + 1) :: decimal  ;
begin
  -- floor(null :: anyarray, int) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- floor([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        -- -- i_left[v_y_cur][v_x_cur] := 
        -- -- case 
        -- --   when i_left[v_y_cur][v_x_cur] = round(i_left[v_y_cur][v_x_cur], i_num_digits) 
        -- --     then i_left[v_y_cur][v_x_cur]
        -- --   else round(i_left[v_y_cur][v_x_cur] - v_balan, i_num_digits) 
        -- -- end;
        if i_left[v_y_cur][v_x_cur] <> round(i_left[v_y_cur][v_x_cur], i_num_digits)
        then 
          i_left[v_y_cur][v_x_cur] := round(i_left[v_y_cur][v_x_cur] - v_balan, i_num_digits);
        end if;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- floor []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      -- -- i_left[v_y_cur] := 
      -- --   case 
      -- --     when i_left[v_y_cur] = round(i_left[v_y_cur], i_num_digits) 
      -- --       then i_left[v_y_cur]
      -- --     else round(i_left[v_y_cur] - v_balan, i_num_digits) 
      -- --   end;
      if i_left[v_y_cur] <> round(i_left[v_y_cur], i_num_digits)
      then 
        i_left[v_y_cur] := round(i_left[v_y_cur] - v_balan, i_num_digits);
      end if;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- floor [][][]
  elsif array_ndims(i_left) = 3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          -- -- i_left[v_y_cur][v_x_cur][v_x3_cur] := 
          -- --   case 
          -- --     when i_left[v_y_cur][v_x_cur][v_x3_cur] = round(i_left[v_y_cur][v_x_cur][v_x3_cur], i_num_digits) 
          -- --       then i_left[v_y_cur][v_x_cur][v_x3_cur]
          -- --     else round(i_left[v_y_cur][v_x_cur][v_x3_cur] - v_balan, i_num_digits) 
          -- --   end;
          if i_left[v_y_cur][v_x_cur][v_x3_cur] <> round(i_left[v_y_cur][v_x_cur][v_x3_cur], i_num_digits)
          then 
            i_left[v_y_cur][v_x_cur][v_x3_cur] := round(i_left[v_y_cur][v_x_cur][v_x3_cur] - v_balan, i_num_digits);
          end if;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- floor [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            -- -- i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
            -- --   case 
            -- --     when i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_num_digits) 
            -- --       then i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]
            -- --     else round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] - v_balan, i_num_digits) 
            -- --   end;
            if i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_num_digits)
            then 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] - v_balan, i_num_digits);
            end if;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_floor(array[array[-12.34584, 25.143540], array[2.560454, -3.2504870]], 2)
-- select sm_sc.fv_opr_floor(array[-12.304725, 25.1874, -28.33407], 0)
-- select sm_sc.fv_opr_floor(array[0, 1.00003, -2.00005, 0.99993, -1.999995, 1.01, -1.01], 2)
-- select sm_sc.fv_opr_floor(array[]::decimal[])
-- select sm_sc.fv_opr_floor(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_floor(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[], 2)
-- select sm_sc.fv_opr_floor(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[], 1)

-- ---------------------------------------------------------------------------
-- -- 以下单入参函数是必要的，无法用设置默认值的双入参代替，且一定不能用后者（两者对单入参调用冲突），后者无法支持单目运算符
-- drop function if exists sm_sc.fv_opr_floor(anyarray);
create or replace function sm_sc.fv_opr_floor
(
  i_left     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: decimal[][], decimal) = null :: decimal[][]
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- floor([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := floor(i_left[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- floor []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := floor(i_left[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- floor [][][]
  elsif array_ndims(i_left) = 3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            floor(i_left[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- floor [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              floor(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_floor(array[array[12.35450, -25.1507], array[-2.56405, 3.2544444446]])
-- select sm_sc.fv_opr_floor(array[12.3101, -25.711, 28.33920])
-- select sm_sc.fv_opr_floor(array[0, 1.0 :: decimal, -2.0])
-- select sm_sc.fv_opr_floor(array[]::decimal[])
-- select sm_sc.fv_opr_floor(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_floor(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[])
-- select sm_sc.fv_opr_floor(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_greatest(anyarray, anyarray);
create or replace function sm_sc.fv_opr_greatest
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] greatest []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] greatest array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return i_left;
  
  -- [] greatest []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := greatest(i_left[v_y_cur], i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] greatest [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := greatest(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] greatest [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] := greatest(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] greatest [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := greatest(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_greatest
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_greatest    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-12.3, 12.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_greatest    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[12.3, -12.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_greatest
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_greatest(array[2.1], array[1.2, 2.1, 3.2]);
-- select sm_sc.fv_opr_greatest(array[2.1], array[array[1.2, 2.1, 3.2]]);
-- select sm_sc.fv_opr_greatest(array[array[2.1]], array[1.2, 2.1, 3.2]);
-- select sm_sc.fv_opr_greatest(array[array[2.1]], array[array[1.2, 2.1, 3.2]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_greatest
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_greatest
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_greatest(anyarray, anyelement);
create or replace function sm_sc.fv_opr_greatest
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- greatest(null :: float[][], float = null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- greatest([][], float)
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := greatest(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- greatest([], float)
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := greatest(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- greatest([][][], float)
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            greatest
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur]
            , i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- greatest([][][][], float)
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              greatest
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_greatest(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_greatest(array[]::float[], 8.8)
-- select sm_sc.fv_opr_greatest(array[]::float[], null)
-- select sm_sc.fv_opr_greatest(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_greatest(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_greatest(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_greatest
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_greatest
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_greatest(anyelement, anyarray);
create or replace function sm_sc.fv_opr_greatest
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- greatest(null :: float[][], float = null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- greatest(float, [][])
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := greatest(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- greatest(float, [])
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := greatest(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- greatest(float, [][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
              greatest
              (
                i_left
              , i_right[v_y_cur][v_x_cur][v_x3_cur]
              )
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- greatest(float, [][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              greatest
              (
                i_left
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
  
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_greatest(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_greatest(8.8, array[]::float[] )
-- select sm_sc.fv_opr_greatest(null, array[]::float[])
-- select sm_sc.fv_opr_greatest(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_greatest(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_greatest(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_greatest
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_greatest
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_imaginary(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_opr_imaginary
(
  i_right     sm_sc.typ_l_complex[]
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     float[]    ;

begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- sm_sc.fv_opr_imaginary([][])
  if array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.fv_opr_imaginary(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- sm_sc.fv_opr_imaginary ([])
  elsif array_ndims(i_right) = 1
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := sm_sc.fv_opr_imaginary(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- sm_sc.fv_opr_imaginary([][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_opr_imaginary(i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- sm_sc.fv_opr_imaginary([][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_opr_imaginary(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_imaginary(array[array[(12.3, 10.6)::sm_sc.typ_l_complex, (-25.1, 5.6)::sm_sc.typ_l_complex], array[(-2.56, 4.1)::sm_sc.typ_l_complex, (3.25, 5.2)::sm_sc.typ_l_complex]])
-- select sm_sc.fv_opr_imaginary(array[(12.3, 5.1)::sm_sc.typ_l_complex, (-25.1, 7.8)::sm_sc.typ_l_complex, (28.33, 1.9)::sm_sc.typ_l_complex])
-- select sm_sc.fv_opr_imaginary(array[]::float[])
-- select sm_sc.fv_opr_imaginary(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_imaginary
--   (
--     array[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]] :: sm_sc.typ_l_complex[]
--   )
-- select 
--   sm_sc.fv_opr_imaginary
--   (
--     array[[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]],[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]]] :: sm_sc.typ_l_complex[]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_is_equal(anyarray, anyarray);
create or replace function sm_sc.fv_opr_is_equal
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     boolean[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_equal_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    v_ret := array_fill(null::boolean, v_len_depdt);
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] == []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
    -- [] == array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
    
    elsif i_left is null or i_right is null
    then
      return null::boolean[];
    
    -- [] == []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        v_ret[v_y_cur] := (i_left[v_y_cur] = i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- [][] == [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] = i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
      
    -- [][][] == [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur] = i_right[v_y_cur][v_x_cur][v_x3_cur])
            ;
          end loop;    
        end loop;
      end loop;
      return v_ret;
      
    -- [][][][] == [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
                (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return v_ret;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_is_equal
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_is_equal
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_is_equal
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_equal
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.fv_opr_is_equal(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_is_equal(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_is_equal(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_is_equal(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_is_equal
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_equal
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_equal(anyarray, anyelement);
create or replace function sm_sc.fv_opr_is_equal
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] = null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_equal_py(i_left, i_right);  

  -- ([][] = float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] = i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] = float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] = i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] = float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              = i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] = float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                = i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_equal(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_is_equal(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.fv_opr_is_equal(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_is_equal(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_is_equal
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_is_equal
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_equal(anyelement, anyarray);
create or replace function sm_sc.fv_opr_is_equal
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] = null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_equal_py(i_left, i_right);  

  -- (float = [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left = i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float = [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := (i_left = i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float = [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            (
              i_left 
              = i_right[v_y_cur][v_x_cur][v_x3_cur] 
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float = [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (
                i_left
                = i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
  
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_equal(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_is_equal(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_is_equal(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_is_equal(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_is_equal
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_equal
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_is_equal_py(float[], float[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(float, float[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     float    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(float[], float);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     float[]    ,
  i_right    float
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_is_equal_py(array[1,2,3], array[2,2,3])
-- select sm_sc.fv_opr_is_equal_py(3, array[2,2,3])
-- select sm_sc.fv_opr_is_equal_py(array[1,2,3], 2)

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_equal_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_equal_py(int[], int[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(int, int[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     int    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(int[], int);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     int[]    ,
  i_right    int
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_equal_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_equal_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_is_equal_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns boolean[]
as
$$
  import numpy as np
  return np.equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_is_equal_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_equal_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_is_equal_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_equal_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_is_equal_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_is_equal_py(array[[12,9,18]],             3::   int)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_is_greater(anyarray, anyarray);
create or replace function sm_sc.fv_opr_is_greater
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     boolean[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_greater_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    v_ret := array_fill(null::boolean, v_len_depdt);
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] > []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
    -- [] > array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
    
    elsif i_left is null or i_right is null
    then
      return null::boolean[];
    
    -- [] > []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        v_ret[v_y_cur] := (i_left[v_y_cur] > i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- [][] > [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] > i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
      
    -- [][][] > [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur] > i_right[v_y_cur][v_x_cur][v_x3_cur])
            ;
          end loop;    
        end loop;
      end loop;
      return v_ret;
      
    -- [][][][] > [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
                (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] > i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return v_ret;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_is_greater
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_is_greater
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_is_greater
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_greater
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.fv_opr_is_greater(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_is_greater(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_is_greater(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_is_greater(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_is_greater
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_greater
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater(anyarray, anyelement);
create or replace function sm_sc.fv_opr_is_greater
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] >  null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_greater_py(i_left, i_right);  

  -- ([][] > float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] > i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] > float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] > i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] > float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              > i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] > float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                > i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_greater(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_is_greater(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.fv_opr_is_greater(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_is_greater(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_is_greater
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_is_greater
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater(anyelement, anyarray);
create or replace function sm_sc.fv_opr_is_greater
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] <= null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_greater_py(i_left, i_right);  

  -- (float > [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left > i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float > [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := (i_left > i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float > [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            > i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float > [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              > i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_greater(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_is_greater(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_is_greater(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_is_greater(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_is_greater
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_greater
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_is_greater_ex(anyarray, anyarray);
create or replace function sm_sc.fv_opr_is_greater_ex
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     boolean[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_greater_ex_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    v_ret := array_fill(null::boolean, v_len_depdt);
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] >= []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
    -- [] >= array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
    
    elsif i_left is null or i_right is null
    then
      return null::boolean[];
    
    -- [] >= []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        v_ret[v_y_cur] := (i_left[v_y_cur] >= i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- [][] >= [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] >= i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
      
    -- [][][] >= [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur] >= i_right[v_y_cur][v_x_cur][v_x3_cur])
            ;
          end loop;    
        end loop;
      end loop;
      return v_ret;
      
    -- [][][][] >= [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
                (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] >= i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return v_ret;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_is_greater_ex
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_is_greater_ex
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_is_greater_ex
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_greater_ex
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.fv_opr_is_greater_ex(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_is_greater_ex(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_is_greater_ex(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_is_greater_ex(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_is_greater_ex
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_greater_ex
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_ex(anyarray, anyelement);
create or replace function sm_sc.fv_opr_is_greater_ex
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] >=  null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_greater_ex_py(i_left, i_right);  

  -- ([][] >= float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] >= i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] >= float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] >= i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] >= float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              >= i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] >= float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                >= i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_greater_ex(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_is_greater_ex(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.fv_opr_is_greater_ex(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_is_greater_ex(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_is_greater_ex
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_is_greater_ex
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_ex(anyelement, anyarray);
create or replace function sm_sc.fv_opr_is_greater_ex
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] <= null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_greater_ex_py(i_left, i_right);  

  -- (float >= [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left >= i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float >= [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := (i_left >= i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float >= [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            >= i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float >= [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              >= i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_greater_ex(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_is_greater_ex(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_is_greater_ex(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_is_greater_ex(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_is_greater_ex
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_greater_ex
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(float[], float[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(float, float[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     float    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(float[], float);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     float[]    ,
  i_right    float
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_is_greater_ex_py(array[1,2,3], array[2,2,3])
-- select sm_sc.fv_opr_is_greater_ex_py(3, array[2,2,3])
-- select sm_sc.fv_opr_is_greater_ex_py(array[1,2,3], 2)

-- -----------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ----------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(int[], int[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(int, int[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     int    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(int[], int);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     int[]    ,
  i_right    int
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ----------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_ex_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_is_greater_ex_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_is_greater_ex_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_greater_ex_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_is_greater_ex_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_greater_ex_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_is_greater_ex_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_is_greater_ex_py(array[[12,9,18]],             3::   int)
-- drop function if exists sm_sc.fv_opr_is_greater_py(float[], float[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(float, float[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     float    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(float[], float);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     float[]    ,
  i_right    float
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_is_greater_py(array[1,2,3], array[2,2,3])
-- select sm_sc.fv_opr_is_greater_py(3, array[2,2,3])
-- select sm_sc.fv_opr_is_greater_py(array[1,2,3], 2)

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_py(int[], int[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(int, int[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     int    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(int[], int);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     int[]    ,
  i_right    int
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_greater_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_greater_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_is_greater_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns boolean[]
as
$$
  import numpy as np
  return np.greater(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_is_greater_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_greater_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_is_greater_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_greater_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_is_greater_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_is_greater_py(array[[12,9,18]],             3::   int)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_is_less(anyarray, anyarray);
create or replace function sm_sc.fv_opr_is_less
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     boolean[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_less_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    v_ret := array_fill(null::boolean, v_len_depdt);
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] < []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
    -- [] < array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
    
    elsif i_left is null or i_right is null
    then
      return null::boolean[];
    
    -- [] < []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        v_ret[v_y_cur] := (i_left[v_y_cur] < i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- [][] < [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] < i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
      
    -- [][][] < [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur] < i_right[v_y_cur][v_x_cur][v_x3_cur])
            ;
          end loop;    
        end loop;
      end loop;
      return v_ret;
      
    -- [][][][] < [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
                (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] < i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return v_ret;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_is_less
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_is_less
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_is_less
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_less
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.fv_opr_is_less(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_is_less(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_is_less(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_is_less(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_is_less
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_less
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less(anyarray, anyelement);
create or replace function sm_sc.fv_opr_is_less
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] <  null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_less_py(i_left, i_right);  

  -- ([][] < float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] < i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] < float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] < i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] < float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              < i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] < float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                < i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_less(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_is_less(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.fv_opr_is_less(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_is_less(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_is_less
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_is_less
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less(anyelement, anyarray);
create or replace function sm_sc.fv_opr_is_less
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] <= null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_less_py(i_left, i_right);  

  -- (float < [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left < i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float < [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := (i_left < i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float < [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            < i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float < [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              < i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_less(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_is_less(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_is_less(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_is_less(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_is_less
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_less
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_is_less_ex(anyarray, anyarray);
create or replace function sm_sc.fv_opr_is_less_ex
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     boolean[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_less_ex_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    v_ret := array_fill(null::boolean, v_len_depdt);
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] <= []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
    -- [] <= array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return array_fill(null::boolean, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
    
    elsif i_left is null or i_right is null
    then
      return null::boolean[];
    
    -- [] <= []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        v_ret[v_y_cur] := (i_left[v_y_cur] <= i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- [][] <= [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] <= i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
      
    -- [][][] <= [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur] <= i_right[v_y_cur][v_x_cur][v_x3_cur])
            ;
          end loop;    
        end loop;
      end loop;
      return v_ret;
      
    -- [][][][] <= [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
                (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <= i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return v_ret;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_is_less_ex
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_is_less_ex
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_is_less_ex
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_is_less_ex
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.fv_opr_is_less_ex(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_is_less_ex(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_is_less_ex(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_is_less_ex(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_is_less_ex
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_less_ex
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_ex(anyarray, anyelement);
create or replace function sm_sc.fv_opr_is_less_ex
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] <=  null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_less_ex_py(i_left, i_right);  

  -- ([][] <= float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] <= i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] <= float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] <= i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] <= float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              <= i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] <= float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                <= i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_less_ex(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_is_less_ex(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.fv_opr_is_less_ex(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_is_less_ex(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_is_less_ex
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_is_less_ex
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_ex(anyelement, anyarray);
create or replace function sm_sc.fv_opr_is_less_ex
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: float[][] <= null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_is_less_ex_py(i_left, i_right);  

  -- (float <= [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left <= i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float <= [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := (i_left <= i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float <= [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            <= i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float <= [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              <= i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_less_ex(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_is_less_ex(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_is_less_ex(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_is_less_ex(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_is_less_ex
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_is_less_ex
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(float[], float[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(float, float[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     float    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(float[], float);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     float[]    ,
  i_right    float
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_is_less_ex_py(array[1,2,3], array[2,2,3])
-- select sm_sc.fv_opr_is_less_ex_py(3, array[2,2,3])
-- select sm_sc.fv_opr_is_less_ex_py(array[1,2,3], 2)

-- ----------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_ex_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ----------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_ex_py(int[], int[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(int, int[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     int    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(int[], int);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     int[]    ,
  i_right    int
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ----------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_ex_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_ex_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_is_less_ex_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns boolean[]
as
$$
  import numpy as np
  return np.less_equal(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_is_less_ex_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_less_ex_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_is_less_ex_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_less_ex_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_is_less_ex_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_is_less_ex_py(array[[12,9,18]],             3::   int)
-- drop function if exists sm_sc.fv_opr_is_less_py(float[], float[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(float, float[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     float    ,
  i_right    float[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(float[], float);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     float[]    ,
  i_right    float
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_is_less_py(array[1,2,3], array[2,2,3])
-- select sm_sc.fv_opr_is_less_py(3, array[2,2,3])
-- select sm_sc.fv_opr_is_less_py(array[1,2,3], 2)

-- ------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_py(int[], int[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(int, int[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     int    ,
  i_right    int[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(int[], int);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     int[]    ,
  i_right    int
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_less_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_is_less_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_is_less_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns boolean[]
as
$$
  import numpy as np
  return np.less(np.array(i_left), np.array(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_is_less_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_less_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_is_less_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_is_less_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_is_less_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_is_less_py(array[[12,9,18]],             3::   int)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_is_regexp_match(text[], text[]);
create or replace function sm_sc.fv_opr_is_regexp_match
(
  i_left     text[]    ,
  i_right    text[]
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     boolean[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  v_ret := array_fill(null::boolean, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] ~ []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return array_fill(null::boolean, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
  -- [] ~ array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return array_fill(null::boolean, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
  
  elsif i_left is null or i_right is null
  then
    return null::boolean[];
  
  -- [] ~ []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] ~ i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][] ~ [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] ~ i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- [][][] ~ [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] ~ i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] ~ [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] ~ i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_is_regexp_match
--   ( 
--     (<>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)) :: text[]
--   , (<>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)) :: text[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array['abbbbbc122223'], array['abc123']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array[a.c], array[1.*?3]]
--   );
-- -- select sm_sc.fv_opr_is_regexp_match
-- --   (
-- --     array['abbbbbc122223', 'abc123'],
-- --     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
-- --   );
-- -- select sm_sc.fv_opr_is_regexp_match
-- --   (
-- --     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
-- --     array['1.3', 'a.*?c']
-- --   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array['abbbbbc122223', 'abc123']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[]::text[],
--     array[array[], array []]::text[]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array[], array []]::text[],
--     array[]::text[]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array[array[], array []]::text[],
--     array[array[], array []]::text[]
--   );
-- select sm_sc.fv_opr_is_regexp_match
--   (
--     array['abbbbbc122223', 'abc123', 'abc123', 'ac13'],
--     array['a.c', '1.*?3', '1.3', 'a.*?c']
--   );
-- select sm_sc.fv_opr_is_regexp_match(array['1'], array['1','2','3']);
-- select sm_sc.fv_opr_is_regexp_match(array['1'], array[array['1','2','3']]);
-- select sm_sc.fv_opr_is_regexp_match(array[array['1']], array['1','2','3']);
-- select sm_sc.fv_opr_is_regexp_match(array[array['1']], array[array['1','2','3']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_is_regexp_match
--   (
--     array[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]
--   , array[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]
--   )
-- select 
--   sm_sc.fv_opr_is_regexp_match
--   (
--     array[[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]],[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]]
--   , array[[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]],[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_regexp_match(text[], text);
create or replace function sm_sc.fv_opr_is_regexp_match
(
  i_left     text[]    ,
  i_right    text
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: text[][] ~ text) = null :: text[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];

  -- ([][] ~ text)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] ~ i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] ~ text)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] ~ i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][][] ~ text
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            ~ i_right
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] ~ text
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              ~ i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_regexp_match(array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']], 'a.c')
-- select sm_sc.fv_opr_is_regexp_match(array['abbbbbc122223', 'abc123', 'abc123', 'ac13'], 'a.c')
-- select sm_sc.fv_opr_is_regexp_match(array[]::text[], '1.3')
-- select sm_sc.fv_opr_is_regexp_match(array[array[], array []]::text[], '1.3')
-- select 
--   sm_sc.fv_opr_is_regexp_match
--   (
--     array[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]
--   , '1.*?3'
--   )
-- select 
--   sm_sc.fv_opr_is_regexp_match
--   (
--     array[[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]],[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]]
--   , 'a.*?c'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_is_regexp_match(text, text[]);
create or replace function sm_sc.fv_opr_is_regexp_match
(
  i_left     text    ,
  i_right    text[]
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     boolean[]  ;
begin
  -- (null :: text[][] ~ text) = null :: text[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::text[];

  -- (text ~ [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left ~ i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (text ~ [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := (i_left ~ i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- float ~ [][][]
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            ~ i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- float ~ [][][][]
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::boolean, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              ~ i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_is_regexp_match('abbbbbc122223', array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']])
-- select sm_sc.fv_opr_is_regexp_match('abc123', array['a.c', '1.*?3', '1.3', 'a.*?c'])
-- select sm_sc.fv_opr_is_regexp_match('abc123', array[]::text[])
-- select sm_sc.fv_opr_is_regexp_match('abc123', array[array[], array []]::text[])
-- select 
--   sm_sc.fv_opr_is_regexp_match
--   (
--     'abc123'
--   , array[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]
--   )
-- select 
--   sm_sc.fv_opr_is_regexp_match
--   (
--     'abc123'
--   , array[[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]],[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_least(anyarray, anyarray);
create or replace function sm_sc.fv_opr_least
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] least []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] least array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return i_left;
  
  -- [] least []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := least(i_left[v_y_cur], i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] least [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := least(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] least [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] := least(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] least [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := least(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_least
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_least
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_least    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-12.3, 12.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_least    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[12.3, -12.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_least
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_least
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_least(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_least(array[2], array[array[1,2,3]]);
-- select sm_sc.fv_opr_least(array[array[3]], array[1,2,3]);
-- select sm_sc.fv_opr_least(array[array[4]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_least
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_least
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_least(anyarray, anyelement);
create or replace function sm_sc.fv_opr_least
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- least(null :: float[][], float = null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- least([][], float)
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := least(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- least([], float)
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := least(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- least([][][], float)
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            least 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur]
            , i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- least([][][][], float)
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] =
              least 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_least(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_least(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_least(array[]::float[], null)
-- select sm_sc.fv_opr_least(array[1.2, 2.3]::float[], 8.8::float)
-- select sm_sc.fv_opr_least(array[array[], array []]::float[], 8.8::float)
-- select sm_sc.fv_opr_least(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_least
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_least
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_least(anyelement, anyarray);
create or replace function sm_sc.fv_opr_least
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- least(null :: float[][], float = null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- least(float, [][])
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := least(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- least(float, [])
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := least(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- least(float, [][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            least 
            (
              i_left
            , i_right[v_y_cur][v_x_cur][v_x3_cur]
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- least(float, [][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              least 
              (
                i_left
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_least(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_least(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_least(null, array[]::float[])
-- select sm_sc.fv_opr_least(8.8::float, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_least(8.8::float, array[array[], array []]::float[])
-- select sm_sc.fv_opr_least(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_least
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_least
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_ln(anyarray);
create or replace function sm_sc.fv_opr_ln
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_ln_py(i_right);
  
  -- ln([][])
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := ln(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- ln []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := ln(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- ln [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            ln(i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- ln [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ln(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_ln(array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_ln(array[12.3, 25.1, 28.33])
-- select sm_sc.fv_opr_ln(array[]::float[])
-- select sm_sc.fv_opr_ln(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_ln
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_ln
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_ln_py(float[]);
create or replace function sm_sc.fv_opr_ln_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.log(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_ln_py(array[0.4, 1.5, 2.7, 8.2])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_log(anyarray, anyarray);
create or replace function sm_sc.fv_opr_log
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_log_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] ^! []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return i_right;
    -- [] ^! array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return i_left;
    
    elsif i_left is null or i_right is null
    then
      return i_left;
    
    -- [] ^! []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        i_left[v_y_cur] := log(i_left[v_y_cur] :: decimal, i_right[v_y_cur] :: decimal);
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
    
    -- [][] ^! [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          i_left[v_y_cur][v_x_cur] := log(i_left[v_y_cur][v_x_cur] :: decimal, i_right[v_y_cur][v_x_cur] :: decimal);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
      
    -- [][][] ^! [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              log(i_left[v_y_cur][v_x_cur][v_x3_cur] :: decimal, i_right[v_y_cur][v_x_cur][v_x3_cur] :: decimal) 
            ;
          end loop;    
        end loop;
      end loop;
      return i_left;
      
    -- [][][][] ^! [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                log(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] :: decimal, i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] :: decimal)
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return i_left;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_log
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_log
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_log
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_log
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_log(array[12.3], array[45.6, 5.6]);
-- select sm_sc.fv_opr_log(array[12.3], array[array[45.6, 5.6]]);
-- select sm_sc.fv_opr_log(array[array[12.3]], array[45.6, 5.6]);
-- select sm_sc.fv_opr_log(array[array[12.3]], array[array[45.6, 5.6]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_log(anyarray, anyelement);
create or replace function sm_sc.fv_opr_log
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float = null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_log_py(i_left, i_right);  

  -- log([][], float)
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := log(i_left[v_y_cur][v_x_cur] :: decimal, i_right :: decimal);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- log([], float)
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := log(i_left[v_y_cur] :: decimal, i_right :: decimal);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- log([][][], float)
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            log 
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] :: decimal
            , i_right :: decimal
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- log([][][][], float)
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              log 
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] :: decimal
              , i_right :: decimal
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_opr_log(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_log(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_log(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_log(anyelement, anyarray);
create or replace function sm_sc.fv_opr_log
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;

begin
  -- log(null :: float[][], float = null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_log_py(i_left, i_right);  

  -- log(float, [][])
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := log(i_left :: decimal, i_right[v_y_cur][v_x_cur] :: decimal);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- log(float, [])
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := log(i_left :: decimal, i_right[v_y_cur] :: decimal);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- log(float, [][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            log 
            (
              i_left :: decimal
            , i_right[v_y_cur][v_x_cur][v_x3_cur] :: decimal
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- log(float, [][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              log 
              (
                i_left :: decimal
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] :: decimal
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_opr_log(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_log(8.8, array[]::float[] )
-- select sm_sc.fv_opr_log(8.8, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_log
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_log
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_log_py(float[], float[]);
create or replace function sm_sc.fv_opr_log_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.log(np.array(i_right)) / np.log(np.array(i_left))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_log_py(float[], float);
create or replace function sm_sc.fv_opr_log_py
(
  i_left     float[]    ,
  i_right    float
)
returns float[]
as
$$
  import numpy as np
  return (np.log(np.array(i_right)) / np.log(np.array(i_left))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_log_py(float, float[]);
create or replace function sm_sc.fv_opr_log_py
(
  i_left     float    ,
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.log(np.array(i_right)) / np.log(np.array(i_left))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_log
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_log
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.fv_opr_log
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.fv_opr_log
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_log
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_log(array[12.3], array[45.6, 5.6]);
-- select sm_sc.fv_opr_log(array[12.3], array[array[45.6, 5.6]]);
-- select sm_sc.fv_opr_log(array[array[12.3]], array[45.6, 5.6]);
-- select sm_sc.fv_opr_log(array[array[12.3]], array[array[45.6, 5.6]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select sm_sc.fv_opr_log(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_log(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_log(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_log
--   (
--     array[[[[1.1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   , 2.2
--   )

-- select sm_sc.fv_opr_log(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_log(8.8, array[]::float[] )
-- select sm_sc.fv_opr_log(8.8, array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_log
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_log
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mod(anyarray, anyarray);
create or replace function sm_sc.fv_opr_mod
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_val_0   alias for $0;
begin
  -- set search_path to sm_sc;
  v_val_0 := array[0.0];
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] % []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] % array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return i_left;
  
  -- [] % []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := i_left[v_y_cur] % nullif(i_right[v_y_cur], v_val_0[1]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] % [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] % nullif(i_right[v_y_cur][v_x_cur], v_val_0[1]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] % [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] % nullif(i_right[v_y_cur][v_x_cur][v_x3_cur], v_val_0[1])
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] % [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] % nullif(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], v_val_0[1])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_mod
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal(32, 4)[]
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal(32, 4)[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mod
--   (
--     array[array[52.3, -52.3], array[45.6, -45.6]],
--     array[array[-52.3, 52.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-52.3, 52.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[array[12.3, -52.3], array[45.6, -45.6]],
--     array[array[52.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_mod    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-52.3, 52.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_mod    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[52.3, -52.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_mod
--   (
--     array[array[32.5, -9.1]],
--     array[array[-52.3, 52.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[array[52.3, -52.3], array[45.6, -45.6]],
--     array[array[52.5, -19.1]]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_mod
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_mod(array[52.3], array[45.6, -45.6]);
-- select sm_sc.fv_opr_mod(array[52.3], array[array[45.6, -45.6]]);
-- select sm_sc.fv_opr_mod(array[array[-52.3]], array[45.6, -45.6]);
-- select sm_sc.fv_opr_mod(array[array[-52.3]], array[array[45.6, -45.6]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_mod
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_mod
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mod(anyarray, anyelement);
create or replace function sm_sc.fv_opr_mod
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_val_0   alias for $0;
begin
  v_val_0 := array[0.0 :: float];
  -- null :: float[][] % float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- [][] % float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] % nullif(i_right, v_val_0[1]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] % float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] % nullif(i_right, v_val_0[1]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] % float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            % i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] % float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              % i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_mod(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_mod(array[]::float[], 8.8)
-- select sm_sc.fv_opr_mod(array[]::float[], null)
-- select sm_sc.fv_opr_mod(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_mod(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_mod(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_mod
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_mod
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mod(anyelement, anyarray);
create or replace function sm_sc.fv_opr_mod
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_val_0   alias for $0;
begin
  v_val_0 := array[0.0 :: float];
  -- null :: float[][] % float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- float % [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left % nullif(i_right[v_y_cur][v_x_cur], v_val_0[1]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float % []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left % nullif(i_right[v_y_cur], v_val_0[1]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float % [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            % i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float % [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              % i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_mod(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_mod(8.8, array[]::float[] )
-- select sm_sc.fv_opr_mod(null, array[]::float[])
-- select sm_sc.fv_opr_mod(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_mod(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_mod(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_mod
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_mod
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_mod_nega_py(decimal(32, 4)[], decimal(32, 4)[]);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     decimal(32, 4)[]    ,
  i_right    decimal(32, 4)[]
)
returns decimal(32, 4)[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_nega_py(decimal(32, 4)[], decimal(32, 4));
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     decimal(32, 4)[]    ,
  i_right    decimal(32, 4)
)
returns decimal(32, 4)[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_nega_py(decimal(32, 4), decimal(32, 4)[]);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     decimal(32, 4)    ,
  i_right    decimal(32, 4)[]
)
returns decimal(32, 4)[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_mod_nega_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5])  :: decimal(32, 4)[]
--   , sm_sc.fv_new_rand(array[   1, 4, 5])  :: decimal(32, 4)[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[32.5], array[-9.1]] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[array[12.5], array[-19.1]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[32.5, -9.1] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[12.5, -19.1] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[32.5, -9.1]] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[array[12.5, -19.1]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[]::decimal(32, 4)[] :: decimal(32, 4)[],
--     array[array[], array []]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[], array []]::decimal(32, 4)[],
--     array[]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[array[], array []]::decimal(32, 4)[],
--     array[array[], array []]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py
--   (
--     array[1.2, 2.3]::decimal(32, 4)[],
--     array[2.1, 3.2]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_nega_py(array[1], array[1,2,3]) :: decimal(32, 4)[];
-- select sm_sc.fv_opr_mod_nega_py(array[1], array[array[1,2,3]]) :: decimal(32, 4)[];
-- select sm_sc.fv_opr_mod_nega_py(array[array[1]], array[1,2,3]) :: decimal(32, 4)[];
-- select sm_sc.fv_opr_mod_nega_py(array[array[1]], array[array[1,2,3]]) :: decimal(32, 4)[];

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_mod_nega_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]] :: decimal(32, 4)[]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]] :: decimal(32, 4)[]
--   )
-- select 
--   sm_sc.fv_opr_mod_nega_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]] :: decimal(32, 4)[]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]] :: decimal(32, 4)[]
--   )

-- select 
--   sm_sc.fv_opr_mod_nega_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5])  :: decimal(32, 4)[]
--   , sm_sc.fv_new_rand(array[   1, 4, 5])  :: decimal(32, 4)[]
--   )


-- select sm_sc.fv_opr_mod_nega_py(array[array[12.3, 25.1], array[2.56, 3.25]] :: decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_nega_py(array[]::decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_nega_py(array[]::decimal(32, 4)[], null)
-- select sm_sc.fv_opr_mod_nega_py(array[1.2, 2.3]::decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_nega_py(array[array[], array []]::decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_nega_py(array[1.2, 2.3]::decimal(32, 4)[], null::decimal(32, 4))
-- select 
--   sm_sc.fv_opr_mod_nega_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]] :: decimal(32, 4)[]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_mod_nega_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]] :: decimal(32, 4)[]
--   , 2.2
--   )


-- select sm_sc.fv_opr_mod_nega_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]] :: decimal(32, 4)[])
-- select sm_sc.fv_opr_mod_nega_py(8.8, array[]::decimal(32, 4)[] )
-- select sm_sc.fv_opr_mod_nega_py(null, array[]::decimal(32, 4)[])
-- select sm_sc.fv_opr_mod_nega_py(8.8, array[1.2, 2.3]::decimal(32, 4)[] )
-- select sm_sc.fv_opr_mod_nega_py(8.8, array[array[], array []]::decimal(32, 4)[])
-- select sm_sc.fv_opr_mod_nega_py(null::decimal(32, 4), array[1.2, 2.3]::decimal(32, 4)[] )
-- select 
--   sm_sc.fv_opr_mod_nega_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]] :: decimal(32, 4)[]
--   )
-- select 
--   sm_sc.fv_opr_mod_nega_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]] :: decimal(32, 4)[]
--   )

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mod_nega_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_nega_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns bigint[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_nega_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mod_nega_py(int[], int[]);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns int[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_nega_py(int[], int);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     int[]    ,
  i_right    int
)
returns int[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_nega_py(int, int[]);
create or replace function sm_sc.fv_opr_mod_nega_py
(
  i_left     int    ,
  i_right    int[]
)
returns int[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod - (np.greater(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_mod_nega_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mod_nega_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_mod_nega_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mod_nega_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_mod_nega_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_mod_nega_py(array[[12,9,18]],             3::   int)
-- drop function if exists sm_sc.fv_opr_mod_posi_py(decimal(32, 4)[], decimal(32, 4)[]);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     decimal(32, 4)[]    ,
  i_right    decimal(32, 4)[]
)
returns decimal(32, 4)[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_posi_py(decimal(32, 4)[], decimal(32, 4));
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     decimal(32, 4)[]    ,
  i_right    decimal(32, 4)
)
returns decimal(32, 4)[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_posi_py(decimal(32, 4), decimal(32, 4)[]);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     decimal(32, 4)    ,
  i_right    decimal(32, 4)[]
)
returns decimal(32, 4)[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_mod_posi_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5])  :: decimal(32, 4)[]
--   , sm_sc.fv_new_rand(array[   1, 4, 5])  :: decimal(32, 4)[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[32.5], array[-9.1]] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[array[12.5], array[-19.1]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[32.5, -9.1] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[12.5, -19.1] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[32.5, -9.1]] :: decimal(32, 4)[],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]] :: decimal(32, 4)[],
--     array[array[12.5, -19.1]] :: decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[]::decimal(32, 4)[] :: decimal(32, 4)[],
--     array[array[], array []]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[], array []]::decimal(32, 4)[],
--     array[]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[array[], array []]::decimal(32, 4)[],
--     array[array[], array []]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py
--   (
--     array[1.2, 2.3]::decimal(32, 4)[],
--     array[2.1, 3.2]::decimal(32, 4)[]
--   );
-- select sm_sc.fv_opr_mod_posi_py(array[1], array[1,2,3]) :: decimal(32, 4)[];
-- select sm_sc.fv_opr_mod_posi_py(array[1], array[array[1,2,3]]) :: decimal(32, 4)[];
-- select sm_sc.fv_opr_mod_posi_py(array[array[1]], array[1,2,3]) :: decimal(32, 4)[];
-- select sm_sc.fv_opr_mod_posi_py(array[array[1]], array[array[1,2,3]]) :: decimal(32, 4)[];

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_mod_posi_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]] :: decimal(32, 4)[]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]] :: decimal(32, 4)[]
--   )
-- select 
--   sm_sc.fv_opr_mod_posi_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]] :: decimal(32, 4)[]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]] :: decimal(32, 4)[]
--   )

-- select 
--   sm_sc.fv_opr_mod_posi_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5])  :: decimal(32, 4)[]
--   , sm_sc.fv_new_rand(array[   1, 4, 5])  :: decimal(32, 4)[]
--   )


-- select sm_sc.fv_opr_mod_posi_py(array[array[12.3, 25.1], array[2.56, 3.25]] :: decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_posi_py(array[]::decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_posi_py(array[]::decimal(32, 4)[], null)
-- select sm_sc.fv_opr_mod_posi_py(array[1.2, 2.3]::decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_posi_py(array[array[], array []]::decimal(32, 4)[], 8.8)
-- select sm_sc.fv_opr_mod_posi_py(array[1.2, 2.3]::decimal(32, 4)[], null::decimal(32, 4))
-- select 
--   sm_sc.fv_opr_mod_posi_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]] :: decimal(32, 4)[]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_mod_posi_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]] :: decimal(32, 4)[]
--   , 2.2
--   )


-- select sm_sc.fv_opr_mod_posi_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]] :: decimal(32, 4)[])
-- select sm_sc.fv_opr_mod_posi_py(8.8, array[]::decimal(32, 4)[] )
-- select sm_sc.fv_opr_mod_posi_py(null, array[]::decimal(32, 4)[])
-- select sm_sc.fv_opr_mod_posi_py(8.8, array[1.2, 2.3]::decimal(32, 4)[] )
-- select sm_sc.fv_opr_mod_posi_py(8.8, array[array[], array []]::decimal(32, 4)[])
-- select sm_sc.fv_opr_mod_posi_py(null::decimal(32, 4), array[1.2, 2.3]::decimal(32, 4)[] )
-- select 
--   sm_sc.fv_opr_mod_posi_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]] :: decimal(32, 4)[]
--   )
-- select 
--   sm_sc.fv_opr_mod_posi_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]] :: decimal(32, 4)[]
--   )

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mod_posi_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_posi_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns bigint[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_posi_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mod_posi_py(int[], int[]);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns int[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_posi_py(int[], int);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     int[]    ,
  i_right    int
)
returns int[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mod_posi_py(int, int[]);
create or replace function sm_sc.fv_opr_mod_posi_py
(
  i_left     int    ,
  i_right    int[]
)
returns int[]
as
$$
  import numpy as np
  v_mod = np.array(i_left) % np.array(i_right)
  return (v_mod + (np.less(np.sign(v_mod), 0.0) * np.abs(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_mod_posi_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mod_posi_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_mod_posi_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mod_posi_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_mod_posi_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_mod_posi_py(array[[12,9,18]],             3::   int)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mul(anyarray, anyarray);
create or replace function sm_sc.fv_opr_mul
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_mul_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] * []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return i_right;
    -- [] * array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return i_left;
    
    elsif i_left is null or i_right is null
    then
      return i_left;
    
    -- [] * []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        i_left[v_y_cur] := i_left[v_y_cur] * i_right[v_y_cur];
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
    
    -- [][] * [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] * i_right[v_y_cur][v_x_cur];
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
      
    -- [][][] * [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              * i_right[v_y_cur][v_x_cur][v_x3_cur] 
            ;
          end loop;    
        end loop;
      end loop;
      return i_left;
      
    -- [][][][] * [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                * i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return i_left;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_mul
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mul
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_mul    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-12.3, 12.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_mul    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[12.3, -12.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_mul
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_mul
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_mul(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_mul(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_mul(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_mul(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_mul
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_mul
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mul(anyarray, anyelement);
create or replace function sm_sc.fv_opr_mul
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] * float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_mul_py(i_left, i_right);  

  -- [][] * float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] * i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] * float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] * i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] * float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            * i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] * float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              * i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_mul(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_mul(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_mul(array[]::float[], null)
-- select sm_sc.fv_opr_mul(array[1.2, 2.3]::float[], 8.8::float)
-- select sm_sc.fv_opr_mul(array[array[], array []]::float[], 8.8::float)
-- select sm_sc.fv_opr_mul(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_mul
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_mul
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mul(anyelement, anyarray);
create or replace function sm_sc.fv_opr_mul
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] * float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_mul_py(i_left, i_right);  

  -- float * [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left * i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float * []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left * i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float * [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            * i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float * [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              * i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_mul(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_mul(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_mul(null, array[]::float[])
-- select sm_sc.fv_opr_mul(8.8::float, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_mul(8.8::float, array[array[], array []]::float[])
-- select sm_sc.fv_opr_mul(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_mul
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_mul
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_mul_py(float[], float[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(float[], float);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     float[]    ,
  i_right    float
)
returns float[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) * i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(float, float[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     float    ,
  i_right    float[]
)
returns float[]
as
$$
  if i_left is None :
    return i_right
  else :
    from numpy import array as arr 
  return (i_left * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_mul_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_mul_py
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_mul_py(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_mul_py(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_mul_py(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_mul_py(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_mul_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_mul_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_opr_mul_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )


-- select sm_sc.fv_opr_mul_py(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_mul_py(array[]::float[], 8.8)
-- select sm_sc.fv_opr_mul_py(array[]::float[], null)
-- select sm_sc.fv_opr_mul_py(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_mul_py(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_mul_py(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_mul_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_mul_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )


-- select sm_sc.fv_opr_mul_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_mul_py(8.8, array[]::float[] )
-- select sm_sc.fv_opr_mul_py(null, array[]::float[])
-- select sm_sc.fv_opr_mul_py(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_mul_py(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_mul_py(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_mul_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_mul_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mul_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns bigint[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) * i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mul_py(int[], int[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(int[], int);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     int[]    ,
  i_right    int
)
returns int[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) * i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(int, int[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     int    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_mul_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  if arr(i_left).size == 0 :
    return i_right
  elif arr(i_right).size == 0 :
    return i_left
  else :
    return (arr(i_left) * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns decimal[]
as
$$
  from numpy import array as arr 
  if i_right is None :
    return i_left
  else :
    return (arr(i_left) * i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mul_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_mul_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  if i_left is None :
    return i_right
  else :
    return (i_left * arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_mul_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mul_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_mul_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_mul_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_mul_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_mul_py(array[[12,9,18]],             3::   int)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_norm(float[]);
create or replace function sm_sc.fv_opr_norm
(
  i_right     float[]
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_opr_abs(i_right);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_norm
--   (
--     array[[-45.6], [-45.6]]
--   );
-- select sm_sc.fv_opr_norm
--   (
--     array[100.0]
--   );

-- drop function if exists sm_sc.fv_opr_norm(decimal[]);
create or replace function sm_sc.fv_opr_norm
(
  i_right     decimal[]
)
returns decimal[]
as
$$
-- declare 
begin
  return sm_sc.fv_opr_abs(i_right);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_norm
--   (
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_norm
--   (
--     100.0
--   );
-- -- 逻辑运算
-- drop function if exists sm_sc.fv_opr_not(boolean[]);
create or replace function sm_sc.fv_opr_not
(
  i_right     boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- not (null :: boolean[][], boolean) = null :: boolean[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- not ([][])
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := (not (i_right[v_y_cur][v_x_cur]));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
  -- not ([])
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := (not (i_right[v_y_cur]));
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- not ([][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            (not(i_right[v_y_cur][v_x_cur][v_x3_cur]))
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- not ([][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (not(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]))
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_not(array[[true, false], [false, true]])
-- select sm_sc.fv_opr_not(array[true, false, false, true])
-- select sm_sc.fv_opr_not(array[]::boolean[])
-- select sm_sc.fv_opr_not(array[[], []]::boolean[])
-- select sm_sc.fv_opr_not(array[[[true, false],[false, true]],[[true, false],[false, true]]])
-- select sm_sc.fv_opr_not(array[[[[true, false],[false, true]],[[true, false],[false, true]]],[[[true, false],[false, true]],[[true, false],[false, true]]]])

-- -------------------------------------------------------------------------------------------

-- -- 位运算
-- drop function if exists sm_sc.fv_opr_not(varbit[]);
create or replace function sm_sc.fv_opr_not
(
  i_right     varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- ~(null :: varbit[][], varbit) = null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- ~([][], varbit)
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := (~ (i_right[v_y_cur][v_x_cur]));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
  -- ~([])
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := (~ (i_right[v_y_cur]));
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- ~ ([][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            ~ (i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- ~ ([][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ~ (i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_not(array[[B'101', B'010'], [B'100', B'001']])
-- select sm_sc.fv_opr_not(array[B'101', B'010', B'100', B'001'])
-- select sm_sc.fv_opr_not(array[]::varbit[])
-- select sm_sc.fv_opr_not(array[[],  []]::varbit[])
-- select sm_sc.fv_opr_not(array[[[B'101', B'010'], [B'100', B'001']],[[B'101', B'010'], [B'100', B'001']]])
-- select sm_sc.fv_opr_not(array[[[[B'101', B'010'], [B'100', B'001']],[[B'101', B'010'], [B'100', B'001']]],[[[B'101', B'010'], [B'100', B'001']],[[B'101', B'010'], [B'100', B'001']]]])
-- set search_path to sm_sc;

-- -- 逻辑运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_or(boolean[], boolean[]);
create or replace function sm_sc.fv_opr_or
(
  i_left     boolean[]    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] or []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] or array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::boolean[];
  
  -- [] or []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] or i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] or [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] or i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] or [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] or i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] or [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] or i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_or
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   )

-- select 
--   sm_sc.fv_opr_or
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_or
--   (
--     array[array[true, false], array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[true], array[false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true], array[false]]
--   );
-- -- select sm_sc.fv_opr_or
-- --   (
-- --     array[true, false],
-- --     array[array[false, true], array[false, true]]
-- --   );
-- -- select sm_sc.fv_opr_or
-- --   (
-- --     array[array[true, false], array[true, false]],
-- --     array[true, false]
-- --   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true, false]]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[], array []]::boolean[],
--     array[]::boolean[]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[], array []]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[true, false],
--     array[true, false]
--   );
-- select sm_sc.fv_opr_or(array[true], array[true, false]);
-- select sm_sc.fv_opr_or(array[false], array[array[true, false]]);
-- select sm_sc.fv_opr_or(array[array[true]], array[true, false]);
-- select sm_sc.fv_opr_or(array[array[false]], array[array[true, false]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , array[[[true,false,true],[false,false,true]],[[true,false,false],[false,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , array[[[[false,false,true],[true,false,false]],[[true,false,true],[false,false,true]]],[[[true,false,false],[false,false,true]],[[false,true,false],[false,false,true]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_or(boolean[], boolean);
create or replace function sm_sc.fv_opr_or
(
  i_left     boolean[]    ,
  i_right    boolean
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] or boolean = null :: boolean[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];

  -- [][] or boolean
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] or i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] or boolean
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] or i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] or boolean
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            or i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] or boolean
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              or i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_or(array[array[true, false], array[false, true]], false)
-- select sm_sc.fv_opr_or(array[true, false], false)
-- select sm_sc.fv_opr_or(array[]::boolean[], true)
-- select sm_sc.fv_opr_or(array[array[], array []]::boolean[], false)
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , true
--   )
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , false
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_or(boolean, boolean[]);
create or replace function sm_sc.fv_opr_or
(
  i_left     boolean    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] or boolean = null :: boolean[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::boolean[];

  -- boolean or [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left or i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- boolean or []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left or i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- boolean or [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            or i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- boolean or [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              or i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_or(false, array[array[false, true], array[true, false]])
-- select sm_sc.fv_opr_or(false, array[true, false])
-- select sm_sc.fv_opr_or(false, array[]::boolean[] )
-- select sm_sc.fv_opr_or(true, array[array[], array []]::boolean[])
-- select 
--   sm_sc.fv_opr_or
--   (
--     true
--   , array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_or
--   (
--     false
--   , array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   )

-- -------------------------------------------------------------------------------------------------------------------------------

-- -- 位运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_or(varbit[], varbit[]);
create or replace function sm_sc.fv_opr_or
(
  i_left     varbit[]    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] | []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] | array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::varbit;
  
  -- [] | []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] | i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] | [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] | i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] | [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] | i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] | [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] | i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_or
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_or
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'101'], array[B'011']]
--   );
-- -- select sm_sc.fv_opr_or
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[B'011', B'101'], array[B'011', B'010']]
-- --   );
-- -- select sm_sc.fv_opr_or
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[B'010', B'011']
-- --   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'010']]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[B'010', B'011']]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[], array []]::varbit[],
--     array[]::varbit[]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[array[], array []]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_or
--   (
--     array[B'101', B'011'],
--     array[B'010', B'011']
--   );
-- select sm_sc.fv_opr_or(array[B'101'], array[B'101', B'011']);
-- select sm_sc.fv_opr_or(array[B'101'], array[array[B'101', B'011']]);
-- select sm_sc.fv_opr_or(array[array[B'101']], array[B'101', B'011']);
-- select sm_sc.fv_opr_or(array[array[B'101']], array[array[B'101', B'011']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , array[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]]
--   )
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , array[[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]],[[[B'001',B'010',B'100'],[B'011',B'010',B'111']],[[B'010',B'100',B'001'],[B'101',B'110',B'001']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_or(varbit[], varbit);
create or replace function sm_sc.fv_opr_or
(
  i_left     varbit[]    ,
  i_right    varbit
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] & varbit = null :: varbit[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::varbit[];

  -- [][] | varbit
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] | i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] | varbit
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] | i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
  -- [][][] | varbit
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            | i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] | varbit
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              | i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;    

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_or(array[array[B'010', B'011'], array[B'011', B'010']], B'011')
-- select sm_sc.fv_opr_or(array[B'010011', B'011010'], B'011101')
-- select sm_sc.fv_opr_or(array[]::varbit[], B'010')
-- select sm_sc.fv_opr_or(array[array[], array []]::varbit[], B'011')
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , B'010'
--   )
-- select 
--   sm_sc.fv_opr_or
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , B'010'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_or(varbit, varbit[]);
create or replace function sm_sc.fv_opr_or
(
  i_left     varbit    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] | varbit = null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::varbit[];

  -- varbit | [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left | i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- varbit | []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left | i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- varbit | [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            | i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- varbit | [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              | i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_or(B'110', array[array[B'011', B'010'], array[B'101', B'110']])
-- select sm_sc.fv_opr_or(B'011101', array[B'010011', B'011010'])
-- select sm_sc.fv_opr_or(B'011', array[]::varbit[] )
-- select sm_sc.fv_opr_or(B'010', array[array[], array []]::varbit[])
-- select 
--   sm_sc.fv_opr_or
--   (
--     B'010'
--   , array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   )
-- select 
--   sm_sc.fv_opr_or
--   (
--     B'010'
--   , array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_pow(anyarray, anyarray);
create or replace function sm_sc.fv_opr_pow
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_pow_py(i_left, i_right);
  else    
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] ^ []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return i_right;
    -- [] ^ array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return i_left;
    
    elsif i_left is null or i_right is null
    then
      return i_left;
    
    -- [] ^ []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        i_left[v_y_cur] := i_left[v_y_cur] ^ i_right[v_y_cur];
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
    
    -- [][] ^ [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] ^ i_right[v_y_cur][v_x_cur];
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
      
    -- [][][] ^ [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur] ^ i_right[v_y_cur][v_x_cur][v_x3_cur]
            ;
          end loop;    
        end loop;
      end loop;
      return i_left;
      
    -- [][][][] ^ [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] ^ i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return i_left;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_pow
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_pow
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_pow    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-12.3, 12.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_pow    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[12.3, -12.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_pow
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_pow
--   (
--     array[0.2, 2.3]::float[],
--     array['inf', '-inf']::float[]
--   );
-- select sm_sc.fv_opr_pow(array[2], array[1,2,3]);
-- select sm_sc.fv_opr_pow(array[2], array[array[1,2,3]]);
-- select sm_sc.fv_opr_pow(array[array[2.5]], array[1,2,3]::float[]);
-- select sm_sc.fv_opr_pow(array[array[2.5]], array[array[1,2,3]]::float[]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_pow
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_pow
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_pow(anyarray, anyelement);
create or replace function sm_sc.fv_opr_pow
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] ^ float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_pow_py(i_left, i_right);  

  -- [][] ^ float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := case when i_left[v_y_cur][v_x_cur] > 1.0 and i_right = '-inf' :: float 
                                           or i_left[v_y_cur][v_x_cur] < 1.0 and i_right = 'inf' :: float 
                                         then 0.0
                                         else i_left[v_y_cur][v_x_cur] ^ i_right end;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] ^ float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := case when i_left[v_y_cur] > 1.0 and i_right = '-inf' :: float 
                                or i_left[v_y_cur] < 1.0 and i_right = 'inf' :: float 
                              then 0.0
                              else i_left[v_y_cur] ^ i_right end;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] ^ float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              case 
                when i_left[v_y_cur][v_x_cur][v_x3_cur] > 1.0 and i_right = '-inf' :: float
                  or i_left[v_y_cur][v_x_cur][v_x3_cur] < 1.0 and i_right = 'inf' :: float 
                  then 0.0
                else i_left[v_y_cur][v_x_cur][v_x3_cur] ^ i_right
              end
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] ^ float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              case 
                when i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] > 1.0 and i_right = '-inf' :: float
                  or i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] < 1.0 and i_right = 'inf' :: float 
                  then 0.0
                else i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] ^ i_right
              end
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_pow(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_pow(array[array[12.3, 25.1], array[2.56, 3.25]] :: float[], '-inf' :: float)
-- select sm_sc.fv_opr_pow(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_pow(array[]::float[], null)
-- select sm_sc.fv_opr_pow(array[1.2, 2.3]::float[], 8.8::float)
-- select sm_sc.fv_opr_pow(array[array[], array []]::float[], 8.8::float)
-- select sm_sc.fv_opr_pow(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_pow
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_pow
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[0.5,1.7,0.8],[2.3,0.4,2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_pow(anyelement, anyarray);
create or replace function sm_sc.fv_opr_pow
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] ^ float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_pow_py(i_left, i_right);  

  -- float ^ [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := case when i_left > 1.0 and i_right[v_y_cur][v_x_cur] = '-inf' :: float 
                                            or i_left < 1.0 and i_right[v_y_cur][v_x_cur] = 'inf' :: float 
                                          then 0.0
                                          else i_left ^ i_right[v_y_cur][v_x_cur] end;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float ^ []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := case when i_left > 1.0 and i_right[v_y_cur] = '-inf' :: float 
                                 or i_left < 1.0 and i_right[v_y_cur] = 'inf' :: float 
                               then 0.0
                               else i_left ^ i_right[v_y_cur] end;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float ^ [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            case 
              when i_left > 1.0 and i_right[v_y_cur][v_x_cur][v_x3_cur] = '-inf' :: float
                or i_left < 1.0 and i_right[v_y_cur][v_x_cur][v_x3_cur] = 'inf' :: float 
                then 0.0
              else i_left ^ i_right[v_y_cur][v_x_cur][v_x3_cur] 
            end
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float ^ [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              case 
                when i_left > 1.0 and i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = '-inf' :: float
                  or i_left < 1.0 and i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 'inf' :: float 
                  then 0.0
                else i_left ^ i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              end
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_pow(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_pow(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_pow(null, array[]::float[])
-- select sm_sc.fv_opr_pow(8.8::float, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_pow(8.8::float, array[array[], array []]::float[])
-- select sm_sc.fv_opr_pow(null::float, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_pow(8.8::float, array['-inf'::float, 2.3])
-- select sm_sc.fv_opr_pow(0.8::float, array['inf'::float, 2.3])
-- select 
--   sm_sc.fv_opr_pow
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_pow
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_pow_py(float[], float[]);
create or replace function sm_sc.fv_opr_pow_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) ** arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_pow_py(float[], float);
create or replace function sm_sc.fv_opr_pow_py
(
  i_left     float[]    ,
  i_right    float
)
returns float[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) ** i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_pow_py(float, float[]);
create or replace function sm_sc.fv_opr_pow_py
(
  i_left     float    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (i_left ** arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_pow_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_pow_py
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_pow_py(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_pow_py(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_pow_py(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_pow_py(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_pow_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_pow_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_opr_pow_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )


-- select sm_sc.fv_opr_pow_py(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_pow_py(array[]::float[], 8.8)
-- select sm_sc.fv_opr_pow_py(array[]::float[], null)
-- select sm_sc.fv_opr_pow_py(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_pow_py(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_pow_py(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_pow_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_pow_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )


-- select sm_sc.fv_opr_pow_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_pow_py(8.8, array[]::float[] )
-- select sm_sc.fv_opr_pow_py(null, array[]::float[])
-- select sm_sc.fv_opr_pow_py(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_pow_py(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_pow_py(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_pow_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_pow_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_real(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_opr_real
(
  i_right     sm_sc.typ_l_complex[]
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     float[]    ;

begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- sm_sc.fv_opr_real([][])
  if array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.fv_opr_real(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- sm_sc.fv_opr_real ([])
  elsif array_ndims(i_right) = 1
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := sm_sc.fv_opr_real(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- sm_sc.fv_opr_real([][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_opr_real(i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- sm_sc.fv_opr_real([][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::float, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_opr_real(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_real(array[array[(12.3, 10.6)::sm_sc.typ_l_complex, (-25.1, 5.6)::sm_sc.typ_l_complex], array[(-2.56, 4.1)::sm_sc.typ_l_complex, (3.25, 5.2)::sm_sc.typ_l_complex]])
-- select sm_sc.fv_opr_real(array[(12.3, 5.1)::sm_sc.typ_l_complex, (-25.1, 7.8)::sm_sc.typ_l_complex, (28.33, 1.9)::sm_sc.typ_l_complex])
-- select sm_sc.fv_opr_real(array[]::float[])
-- select sm_sc.fv_opr_real(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_real
--   (
--     array[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]] :: sm_sc.typ_l_complex[]
--   )
-- select 
--   sm_sc.fv_opr_real
--   (
--     array[[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]],[[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]],[[(12.3, -25.1), (-2.56, 3.25)],[(12.3, -25.1), (-2.56, 3.25)]]]] :: sm_sc.typ_l_complex[]
--   )
-- drop function if exists sm_sc.fv_opr_round(anyarray, int);
create or replace function sm_sc.fv_opr_round
(
  i_left         anyarray,
  i_num_digits    int
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- round(null :: anyarray, int) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- round([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := round(i_left[v_y_cur][v_x_cur], i_num_digits);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- round []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := round(i_left[v_y_cur], i_num_digits);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- round [][][]
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            round(i_left[v_y_cur][v_x_cur][v_x3_cur], i_num_digits)
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- round [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_num_digits)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_round(array[array[12.3, 25.1], array[2.56, 3.25]], 1)
-- select sm_sc.fv_opr_round(array[12.3, 25.1, 28.33], 0)
-- select sm_sc.fv_opr_round(array[]::decimal[])
-- select sm_sc.fv_opr_round(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_round(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[], 2)
-- select sm_sc.fv_opr_round(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[], 1)

-- ---------------------------------------------------------------------------
-- -- 以下单入参函数是必要的，无法用设置默认值的双入参代替，且一定不能用后者（两者对单入参调用冲突），后者无法支持单目运算符
-- drop function if exists sm_sc.fv_opr_round(anyarray);
create or replace function sm_sc.fv_opr_round
(
  i_left     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- round(null :: anyarray) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- round([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := round(i_left[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- round []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := round(i_left[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- round [][][]
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            round(i_left[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- round [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              round(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_round(array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_round(array[12.3, 25.1, 28.33])
-- select sm_sc.fv_opr_round(array[]::decimal[])
-- select sm_sc.fv_opr_round(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_round(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[])
-- select sm_sc.fv_opr_round(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_shift_left(varbit[], int[]);
create or replace function sm_sc.fv_opr_shift_left
(
  i_left     varbit[]    ,
  i_right    int[]
)
returns varbit[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     varbit[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  v_ret := array_fill(null::varbit, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] << []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return array_fill(null::varbit, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
  -- [] << array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return array_fill(null::varbit, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
  
  elsif i_left is null or i_right is null
  then
    return null::varbit[];
  
  -- [] << []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] << i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][] << [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] << i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- [][][] << [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] << i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] << [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] << i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_shift_left
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , ((sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.3 :: float) *` 10 :: float) :: int[]
--   );


-- select 
--   sm_sc.fv_opr_shift_left
--   ( 
--     (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   , ((sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.3 :: float) *` 10 :: float) :: int[]
--   );
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[1, 2], array[-1, -2]]
--   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[1, 2], array[-1, -2]]
--   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[1], array[2]]
--   );
-- -- select sm_sc.fv_opr_shift_left
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[1, 2], array[-1, -2]]
-- --   );
-- -- select sm_sc.fv_opr_shift_left
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[1, -1]
-- --   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[B'101', B'011']],
--     array[array[1, 2], array[-1, -2]]
--   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[1, -1]]
--   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[]::bit[],
--     array[array[], array []]::int[]
--   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[], array []]::bit[],
--     array[]::int[]
--   );
-- select sm_sc.fv_opr_shift_left
--   (
--     array[array[], array []]::bit[],
--     array[array[], array []]::int[]
--   );
-- select sm_sc.fv_opr_shift_left(array[B'101'], array[1, 2]);
-- select sm_sc.fv_opr_shift_left(array[B'101'], array[array[1, 2]]);
-- select sm_sc.fv_opr_shift_left(array[array[B'011']], array[1, 2]);
-- select sm_sc.fv_opr_shift_left(array[array[B'011']], array[array[1, 2]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_shift_left
--   (
--     array[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]
--   , array[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]]
--   )
-- select 
--   sm_sc.fv_opr_shift_left
--   (
--     array[[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]],[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]]
--   , array[[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]],[[[-1,2,1],[1,-2,-1]],[[-1,2,-1],[1,-2,1]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_shift_left(varbit[], int);
create or replace function sm_sc.fv_opr_shift_left
(
  i_left     varbit[]    ,
  i_right    int  default 1
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] << varbit = null :: varbit[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::varbit[];

  -- [][] << varbit
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] << i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] << varbit
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] << i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] << varbit
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            << i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] << varbit
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              << i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_shift_left(array[array[B'010', B'011'], array[B'011', B'010']], 2)
-- select sm_sc.fv_opr_shift_left(array[B'010011', B'011010'], 3)
-- select sm_sc.fv_opr_shift_left(array[]::bit[], 1)
-- select sm_sc.fv_opr_shift_left(array[array[], array []]::bit[], 1)
-- select 
--   sm_sc.fv_opr_shift_left
--   (
--     array[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]
--   , 1
--   )
-- select 
--   sm_sc.fv_opr_shift_left
--   (
--     array[[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]],[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]]
--   , -1
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_shift_left(varbit, int[]);
create or replace function sm_sc.fv_opr_shift_left
(
  i_left     varbit    ,
  i_right    int[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     varbit[]    ;
begin
  -- null :: varbit[][] << varbit = null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::varbit[];

  -- varbit << [][]
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := i_left << i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- varbit << []
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := i_left << i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- varbit << [][][]
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            << i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- varbit << [][][][]
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              << i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_shift_left(B'110', array[array[1, 3], array[-1, 2]])
-- select sm_sc.fv_opr_shift_left(B'011101', array[2, 3])
-- select sm_sc.fv_opr_shift_left(B'011', array[]::int[] )
-- select sm_sc.fv_opr_shift_left(B'010', array[array[], array []]::int[])
-- select 
--   sm_sc.fv_opr_shift_left
--   (
--     B'1011001001010110'
--   , array[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]]
--   )
-- select 
--   sm_sc.fv_opr_shift_left
--   (
--     B'1011001001010110'
--   , array[[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]],[[[-1,2,1],[1,-2,-1]],[[-1,2,-1],[1,-2,1]]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_shift_right(varbit[], int[]);
create or replace function sm_sc.fv_opr_shift_right
(
  i_left     varbit[]    ,
  i_right    int[]
)
returns varbit[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     varbit[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  v_ret := array_fill(null::varbit, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] >> []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return array_fill(null::varbit, array[coalesce(array_length(i_right, 1), 0), coalesce(array_length(i_right, 2), 0)]);
  -- [] >> array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return array_fill(null::varbit, array[coalesce(array_length(i_left, 1), 0), coalesce(array_length(i_left, 2), 0)]);
  
  elsif i_left is null or i_right is null
  then
    return null::varbit[];
  
  -- [] >> []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := (i_left[v_y_cur] >> i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][] >> [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] >> i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- [][][] >> [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] >> i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] >> [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] >> i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_shift_right
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , ((sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.3 :: float) *` 10 :: float) :: int[]
--   );


-- select 
--   sm_sc.fv_opr_shift_right
--   ( 
--     (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   , ((sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.3 :: float) *` 10 :: float) :: int[]
--   );
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[1, 2], array[-1, -2]]
--   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[1, 2], array[-1, -2]]
--   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[1], array[2]]
--   );
-- -- select sm_sc.fv_opr_shift_right
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[1, 2], array[-1, -2]]
-- --   );
-- -- select sm_sc.fv_opr_shift_right
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[1, -1]
-- --   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[B'101', B'011']],
--     array[array[1, 2], array[-1, -2]]
--   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[1, -1]]
--   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[]::bit[],
--     array[array[], array []]::int[]
--   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[], array []]::bit[],
--     array[]::int[]
--   );
-- select sm_sc.fv_opr_shift_right
--   (
--     array[array[], array []]::bit[],
--     array[array[], array []]::int[]
--   );
-- select sm_sc.fv_opr_shift_right(array[B'101'], array[1,2,3]);
-- select sm_sc.fv_opr_shift_right(array[B'101'], array[array[1,2,3]]);
-- select sm_sc.fv_opr_shift_right(array[array[B'101']], array[-1,-2,3]);
-- select sm_sc.fv_opr_shift_right(array[array[B'101']], array[array[1,-2,-3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_shift_right
--   (
--     array[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]
--   , array[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]]
--   )
-- select 
--   sm_sc.fv_opr_shift_right
--   (
--     array[[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]],[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]]
--   , array[[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]],[[[-1,2,1],[1,-2,-1]],[[-1,2,-1],[1,-2,1]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_shift_right(varbit[], int);
create or replace function sm_sc.fv_opr_shift_right
(
  i_left     varbit[]    ,
  i_right    int    default   1
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] >> varbit = null :: varbit[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::varbit[];

  -- [][] >> varbit
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] >> i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] >> varbit
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] >> i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] >> varbit
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            >> i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] >> varbit
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              >> i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_shift_right(array[array[B'010', B'011'], array[B'011', B'010']], 2)
-- select sm_sc.fv_opr_shift_right(array[B'010011', B'011010'], 3)
-- select sm_sc.fv_opr_shift_right(array[]::bit[], 1)
-- select sm_sc.fv_opr_shift_right(array[array[], array []]::bit[], 1)
-- select 
--   sm_sc.fv_opr_shift_right
--   (
--     array[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]
--   , 1
--   )
-- select 
--   sm_sc.fv_opr_shift_right
--   (
--     array[[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]],[[[B'101', B'011', B'001'],[B'101', B'011', B'001']],[[B'101', B'011', B'001'],[B'101', B'011', B'001']]]]
--   , -1
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_shift_right(varbit, int[]);
create or replace function sm_sc.fv_opr_shift_right
(
  i_left     varbit    ,
  i_right    int[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     varbit[]    ;
begin
  -- null :: varbit[][] >> varbit = null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::varbit[];

  -- varbit >> [][]
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := i_left >> i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- varbit >> []
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := i_left >> i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- varbit >> [][][]
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            >> i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- varbit >> [][][][]
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(null::varbit, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              >> i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_shift_right(B'110', array[array[1, 3], array[-1, 2]])
-- select sm_sc.fv_opr_shift_right(B'011101', array[2, 3])
-- select sm_sc.fv_opr_shift_right(B'011', array[]::int[] )
-- select sm_sc.fv_opr_shift_right(B'010', array[array[], array []]::int[])
-- select 
--   sm_sc.fv_opr_shift_right
--   (
--     B'1011001001010110'
--   , array[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]]
--   )
-- select 
--   sm_sc.fv_opr_shift_right
--   (
--     B'1011001001010110'
--   , array[[[[1,2,-1],[1,2,-1]],[[1,2,-1],[1,2,-1]]],[[[-1,2,1],[1,-2,-1]],[[-1,2,-1],[1,-2,1]]]]
--   )

-- drop function if exists sm_sc.fv_opr_sign(anyarray);
create or replace function sm_sc.fv_opr_sign
(
  i_right     anyarray
)
returns smallint[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     smallint[] ;

begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_sign_py(i_right);
  else
    -- sign([][])
    if array_ndims(i_right) =  2
    then
      v_ret := array_fill(null::smallint, array[array_length(i_right, 1), array_length(i_right, 2)]);
      while v_y_cur <= array_length(i_right, 1)
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= array_length(i_right, 2)
        loop
          v_ret[v_y_cur][v_x_cur] := sign(i_right[v_y_cur][v_x_cur]);
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- sign ([])
    elsif array_ndims(i_right) = 1
    then
      v_ret := array_fill(null::smallint, array[array_length(i_right, 1)]);
      while v_y_cur <= array_length(i_right, 1)
      loop
        v_ret[v_y_cur] := sign(i_right[v_y_cur]);
        v_y_cur := v_y_cur + 1;
      end loop;
      return v_ret;
    
    -- sign ([][][])
    elsif array_ndims(i_right) = 3
    then
      v_ret := array_fill(null::smallint, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
      for v_y_cur in 1 .. array_length(i_right, 1)
      loop
        for v_x_cur in 1 .. array_length(i_right, 2)
        loop
          for v_x3_cur in 1 .. array_length(i_right, 3)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
              sign(i_right[v_y_cur][v_x_cur][v_x3_cur]) 
            ;
          end loop;    
        end loop;
      end loop;
      return v_ret;
      
    -- sign ([][][][])
    elsif array_ndims(i_right) = 4
    then
      v_ret := array_fill(null::smallint, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
      for v_y_cur in 1 .. array_length(i_right, 1)
      loop
        for v_x_cur in 1 .. array_length(i_right, 2)
        loop
          for v_x3_cur in 1 .. array_length(i_right, 3)
          loop
            for v_x4_cur in 1 .. array_length(i_right, 4)
            loop
              v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                sign(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return v_ret;
     
    else
      raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_sign(array[array[12.3, -25.1], array[-2.56, 3.25]])
-- select sm_sc.fv_opr_sign(array[12.3, -25.1, 28.33])
-- select sm_sc.fv_opr_sign(array[]::float[])
-- select sm_sc.fv_opr_sign(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_opr_sign
--   (
--     array[[[1,-2,3],[-1.2,2.3,-3.4]],[[-0.5,0.7,0.8],[-0.3,-0.4,-0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_sign
--   (
--     array[[[[1,2,-3],[-1.2,2.3,3.4]],[[0.5,-0.7,0.8],[0.3,0.4,-0.7]]],[[[1.6,-2.7,-3.4],[1.4,2.2,0.8]],[[-0.5,-1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_opr_sign_py(decimal[]);
create or replace function sm_sc.fv_opr_sign_py
(
  i_right    decimal[]
)
returns int[]
as
$$
  import numpy as np
  return np.int8((np.sign(np.array(i_right)))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_sign_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sign_py(float[]);
create or replace function sm_sc.fv_opr_sign_py
(
  i_right    float[]
)
returns int[]
as
$$
  import numpy as np
  return np.int8((np.sign(np.array(i_right)))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_opr_sign_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sign_py(bigint[]);
create or replace function sm_sc.fv_opr_sign_py
(
  i_right    bigint[]
)
returns int[]
as
$$
  import numpy as np
  return np.int8((np.sign(np.array(i_right)))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sign_py(int[]);
create or replace function sm_sc.fv_opr_sign_py
(
  i_right    int[]
)
returns int[]
as
$$
  import numpy as np
  return np.int8((np.sign(np.array(i_right)))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_sign_py(array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_sign_py(array[[3,3,2]]::int[])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_sub(anyarray, anyarray);
create or replace function sm_sc.fv_opr_sub
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_sub_py(i_left, i_right);
  else  
    v_len_depdt :=
      (
        select 
          sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
        from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
      )
    ;
    
    -- 整理入参一
    if v_len_left <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
      then 
        i_left := array[[[i_left]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
      then 
        i_left := array[[i_left]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
      then 
        i_left := array[i_left];
      end if;
      
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_left <> v_len_depdt
      then 
        i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
        v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- 整理入参二
    if v_len_right <> v_len_depdt
    then 
      -- 对齐维度
      if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
      then 
        i_right := array[[[i_right]]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
      then 
        i_right := array[[i_right]];
      elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
      then 
        i_right := array[i_right];
      end if;
      
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      
      -- 对齐维长
      if v_len_right <> v_len_depdt
      then 
        i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
        v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
      end if;
    end if;
    
    -- array[] - []
    if v_len_left is null and i_left is not null -- = 0
    then 
      return i_right;
    -- [] - array[]
    elsif v_len_right is null and i_right is not null -- = 0
    then 
      return i_left;
    
    elsif i_left is null or i_right is null
    then
      return i_left;
    
    -- [] - []
    elsif array_length(v_len_left, 1) = 1
    then 
      while v_y_cur <= v_len_left[1]
      loop
        i_left[v_y_cur] := i_left[v_y_cur] - i_right[v_y_cur];
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
    
    -- [][] - [][]
    elsif array_length(v_len_left, 1) = 2
    then
      while v_y_cur <= v_len_left[1]
      loop 
        v_x_cur := 1  ;
        while v_x_cur <= v_len_left[2]
        loop
          i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] - i_right[v_y_cur][v_x_cur];
          v_x_cur := v_x_cur + 1;
        end loop;
        v_y_cur := v_y_cur + 1;
      end loop;
      return i_left;
      
    -- [][][] - [][][]
    elsif array_length(v_len_left, 1) = 3
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              - i_right[v_y_cur][v_x_cur][v_x3_cur] 
            ;
          end loop;    
        end loop;
      end loop;
      return i_left;
      
    -- [][][][] - [][][][]
    elsif array_length(v_len_left, 1) = 4
    then
      for v_y_cur in 1 .. v_len_left[1]
      loop
        for v_x_cur in 1 .. v_len_left[2]
        loop
          for v_x3_cur in 1 .. v_len_left[3]
          loop
            for v_x4_cur in 1 .. v_len_left[4]
            loop
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                - i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              ;
            end loop;
          end loop;    
        end loop;
      end loop;
      return i_left;
    
    -- 审计二维长度
    else
      raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_sub
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_sub
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- -- select sm_sc.fv_opr_sub    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[32.5, -9.1],
-- --     array[array[-12.3, 12.3], array[-45.6, 45.6]]
-- --   );
-- -- select sm_sc.fv_opr_sub    -- 设计不支持一维二维混合运算
-- --   (
-- --     array[array[12.3, -12.3], array[45.6, -45.6]],
-- --     array[12.5, -19.1]
-- --   );
-- select sm_sc.fv_opr_sub
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_sub
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_sub(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_sub(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_sub(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_sub(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_sub
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_sub
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sub(anyarray, anyelement);
create or replace function sm_sc.fv_opr_sub
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_left) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_sub_py(i_left, i_right);  

  -- [][] - float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] - i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] - float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] - i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] - float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            - i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] - float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              - i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_sub(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_sub(array[]::float[], 8.8::float)
-- select sm_sc.fv_opr_sub(array[]::float[], null)
-- select sm_sc.fv_opr_sub(array[1.2, 2.3]::float[], 8.8::float)
-- select sm_sc.fv_opr_sub(array[array[], array []]::float[], 8.8::float)
-- select sm_sc.fv_opr_sub(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_sub
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_sub
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sub(anyelement, anyarray);
create or replace function sm_sc.fv_opr_sub
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;
  
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_sub_py(i_left, i_right);  

  -- float - [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left - i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float - []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left - i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float - [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            - i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float - [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              - i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_sub(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_sub(8.8::float, array[]::float[] )
-- select sm_sc.fv_opr_sub(null, array[]::float[])
-- select sm_sc.fv_opr_sub(8.8::float, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_sub(8.8::float, array[array[], array []]::float[])
-- select sm_sc.fv_opr_sub(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_sub
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_sub
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sub(anyarray);
create or replace function sm_sc.fv_opr_sub
(
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  if pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_sub_py(i_right);
  
  -- -[][]
  elsif array_ndims(i_right) = 2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := - i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- - []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := - i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- - [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            - i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- - [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              - i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_sub(array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_sub(array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.fv_opr_sub(array[12.3, 25.1])
-- select 
--   sm_sc.fv_opr_sub
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_sub
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- drop function if exists sm_sc.fv_opr_sub_py(float[], float[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     float[]    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(float[], float);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     float[]    ,
  i_right    float
)
returns float[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(float, float[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     float    ,
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (i_left - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_sub_py(float[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_right    float[]
)
returns float[]
as
$$
  from numpy import array as arr 
  return (- arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_sub_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_opr_sub_py
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_opr_sub_py(array[1], array[1,2,3]);
-- select sm_sc.fv_opr_sub_py(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_opr_sub_py(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_opr_sub_py(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_sub_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_opr_sub_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_opr_sub_py
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )


-- select sm_sc.fv_opr_sub_py(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_opr_sub_py(array[]::float[], 8.8)
-- select sm_sc.fv_opr_sub_py(array[]::float[], null)
-- select sm_sc.fv_opr_sub_py(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_opr_sub_py(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_opr_sub_py(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_opr_sub_py
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_opr_sub_py
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )


-- select sm_sc.fv_opr_sub_py(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_sub_py(8.8, array[]::float[] )
-- select sm_sc.fv_opr_sub_py(null, array[]::float[])
-- select sm_sc.fv_opr_sub_py(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_opr_sub_py(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_opr_sub_py(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_opr_sub_py
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_opr_sub_py
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sub_py(bigint[], bigint[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     bigint[]    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(bigint[], bigint);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     bigint[]    ,
  i_right    bigint
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(bigint, bigint[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     bigint    ,
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (i_left - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_sub_py(bigint[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_right    bigint[]
)
returns bigint[]
as
$$
  from numpy import array as arr 
  return (- arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sub_py(int[], int[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     int[]    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(int[], int);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     int[]    ,
  i_right    int
)
returns int[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(int, int[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     int    ,
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  return (i_left - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_sub_py(int[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_right    int[]
)
returns int[]
as
$$
  from numpy import array as arr 
  return (- arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- ---------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_sub_py(decimal[], decimal[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     decimal[]    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(decimal[], decimal);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     decimal[]    ,
  i_right    decimal
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (arr(i_left) - i_right).tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_sub_py(decimal, decimal[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_left     decimal    ,
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (i_left - arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_sub_py(decimal[]);
create or replace function sm_sc.fv_opr_sub_py
(
  i_right    decimal[]
)
returns decimal[]
as
$$
  from numpy import array as arr 
  return (- arr(i_right)).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select
--   sm_sc.fv_opr_sub_py(array[[12,9,18]],array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_sub_py(array[[12,9,18]],array[[3,3,2]]::int[])
-- , sm_sc.fv_opr_sub_py(              12,array[[3,3,2]]::bigint[])
-- , sm_sc.fv_opr_sub_py(array[[12,9,18]],             3::bigint)
-- , sm_sc.fv_opr_sub_py(              12,array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_sub_py(array[[12,9,18]],             3::   int)
-- , sm_sc.fv_opr_sub_py(                 array[[3,3,2]]::   int[])
-- , sm_sc.fv_opr_sub_py(array[[12,9,18]]               ::bigint[])
-- drop function if exists sm_sc.fv_opr_trunc(anyarray, int);
create or replace function sm_sc.fv_opr_trunc
(
  i_left         anyarray,
  i_num_digits    int
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
begin
  -- trunc(null :: anyarray, int) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- trunc([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := trunc(i_left[v_y_cur][v_x_cur], i_num_digits);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- trunc []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := trunc(i_left[v_y_cur], i_num_digits);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- trunc [][][]
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            trunc(i_left[v_y_cur][v_x_cur][v_x3_cur], i_num_digits)
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- trunc [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              trunc(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_num_digits)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_trunc(array[array[12.3, 25.1], array[2.56, 3.25]], 1)
-- select sm_sc.fv_opr_trunc(array[12.3, 25.1, 28.33], 0)
-- select sm_sc.fv_opr_trunc(array[]::decimal[])
-- select sm_sc.fv_opr_trunc(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_trunc(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[], 2)
-- select sm_sc.fv_opr_trunc(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[], 1)


-- ---------------------------------------------------------------------------
-- -- 以下单入参函数是必要的，无法用设置默认值的双入参代替，且一定不能用后者（两者对单入参调用冲突），后者无法支持单目运算符
-- drop function if exists sm_sc.fv_opr_trunc(anyarray);
create or replace function sm_sc.fv_opr_trunc
(
  i_left     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
begin
  -- trunc(null :: anyarray) = null :: anyarray
  if array_length(i_left, array_ndims(i_left)) is null
  then 
    return i_left;
  end if;

  -- trunc([][])
  if array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := trunc(i_left[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- trunc []
  elsif array_ndims(i_left) = 1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := trunc(i_left[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- trunc [][][]
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            trunc(i_left[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- trunc [][][][]
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              trunc(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_trunc(array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_opr_trunc(array[12.3, 25.1, 28.33])
-- select sm_sc.fv_opr_trunc(array[]::decimal[])
-- select sm_sc.fv_opr_trunc(array[array[], array []]::decimal[])
-- select sm_sc.fv_opr_trunc(array[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]::decimal[])
-- select sm_sc.fv_opr_trunc(array[[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]],[[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]],[[12.3101, -25.711, 28.33920],[-12.3101, 25.711, -28.33920]]]]::decimal[])
-- set search_path to sm_sc;

-- -- 逻辑运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_xnor(boolean[], boolean[]);
create or replace function sm_sc.fv_opr_xnor
(
  i_left     boolean[]    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] xnor []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] xnor array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::boolean[];
  
  -- [] xnor []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] = i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] xnor [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] = i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] xnor [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] = i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] xnor [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_xnor
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   )

-- select 
--   sm_sc.fv_opr_xnor
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[true, false], array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[true], array[false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true], array[false]]
--   );
-- -- select sm_sc.fv_opr_xnor
-- --   (
-- --     array[true, false],
-- --     array[array[false, true], array[false, true]]
-- --   );
-- -- select sm_sc.fv_opr_xnor
-- --   (
-- --     array[array[true, false], array[true, false]],
-- --     array[true, false]
-- --   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true, false]]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[], array []]::boolean[],
--     array[]::boolean[]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[], array []]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[true, false],
--     array[true, false]
--   );
-- select sm_sc.fv_opr_xnor(array[true], array[true, false]);
-- select sm_sc.fv_opr_xnor(array[false], array[array[true, false]]);
-- select sm_sc.fv_opr_xnor(array[array[true]], array[true, false]);
-- select sm_sc.fv_opr_xnor(array[array[false]], array[array[true, false]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , array[[[true,false,true],[false,false,true]],[[true,false,false],[false,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , array[[[[false,false,true],[true,false,false]],[[true,false,true],[false,false,true]]],[[[true,false,false],[false,false,true]],[[false,true,false],[false,false,true]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xnor(boolean[], boolean);
create or replace function sm_sc.fv_opr_xnor
(
  i_left     boolean[]    ,
  i_right    boolean
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] xnor null :: boolean[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];

  -- [][] xnor boolean
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] = i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] xnor boolean
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] = i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] xnor boolean
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] = i_right)
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] xnor boolean
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = i_right)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xnor(array[array[true, false], array[false, true]], false)
-- select sm_sc.fv_opr_xnor(array[true, false], false)
-- select sm_sc.fv_opr_xnor(array[]::boolean[], true)
-- select sm_sc.fv_opr_xnor(array[array[], array []]::boolean[], false)
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , true
--   )
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , false
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xnor(boolean, boolean[]);
create or replace function sm_sc.fv_opr_xnor
(
  i_left     boolean    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] xnor null :: boolean[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::boolean[];

  -- boolean xnor [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := (i_left = i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- boolean xnor []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := (i_left = i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- boolean xnor [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left = i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- boolean xnor [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left = i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xnor(false, array[array[false, true], array[true, false]])
-- select sm_sc.fv_opr_xnor(false, array[true, false])
-- select sm_sc.fv_opr_xnor(false, array[]::boolean[] )
-- select sm_sc.fv_opr_xnor(true, array[array[], array []]::boolean[])
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     true
--   , array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     false
--   , array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   )

-- -------------------------------------------------------------------------------------------------------------------------------

-- -- 位运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_xnor(varbit[], varbit[]);
create or replace function sm_sc.fv_opr_xnor
(
  i_left     varbit[]    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] ~# []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] ~# array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::varbit;
  
  -- [] ~# []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := ~ (i_left[v_y_cur] # i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] ~# [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := ~ (i_left[v_y_cur][v_x_cur] # i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] ~# [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            ~ (i_left[v_y_cur][v_x_cur][v_x3_cur] # i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] ~# [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ~ (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] # i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_xnor
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'101'], array[B'011']]
--   );
-- -- select sm_sc.fv_opr_xnor
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[B'011', B'101'], array[B'011', B'010']]
-- --   );
-- -- select sm_sc.fv_opr_xnor
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[B'010', B'011']
-- --   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'010']]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[B'010', B'011']]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[], array []]::varbit[],
--     array[]::varbit[]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[array[], array []]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_xnor
--   (
--     array[B'101', B'011'],
--     array[B'010', B'011']
--   );
-- select sm_sc.fv_opr_xnor(array[B'101'], array[B'101', B'011']);
-- select sm_sc.fv_opr_xnor(array[B'101'], array[array[B'101', B'011']]);
-- select sm_sc.fv_opr_xnor(array[array[B'101']], array[B'101', B'011']);
-- select sm_sc.fv_opr_xnor(array[array[B'101']], array[array[B'101', B'011']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , array[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]]
--   )
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , array[[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]],[[[B'001',B'010',B'100'],[B'011',B'010',B'111']],[[B'010',B'100',B'001'],[B'101',B'110',B'001']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xnor(varbit[], varbit);
create or replace function sm_sc.fv_opr_xnor
(
  i_left     varbit[]    ,
  i_right    varbit
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] & varbit = null :: varbit[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::varbit[];

  -- [][] ~# varbit
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := ~ (i_left[v_y_cur][v_x_cur] # i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] ~# varbit
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := ~ (i_left[v_y_cur] # i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
  -- [][][] ~# varbit
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            ~
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
              # i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] ~# varbit
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ~
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
                # i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;    

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xnor(array[array[B'010', B'011'], array[B'011', B'010']], B'011')
-- select sm_sc.fv_opr_xnor(array[B'010011', B'011010'], B'011101')
-- select sm_sc.fv_opr_xnor(array[]::varbit[], B'010')
-- select sm_sc.fv_opr_xnor(array[array[], array []]::varbit[], B'011')
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , B'010'
--   )
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , B'010'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xnor(varbit, varbit[]);
create or replace function sm_sc.fv_opr_xnor
(
  i_left     varbit    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit # null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::varbit[];

  -- varbit ~# [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := ~ (i_left # i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- varbit ~# []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := ~ (i_left # i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- varbit ~# [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            ~ 
            (
              i_left 
              # i_right[v_y_cur][v_x_cur][v_x3_cur] 
            )
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- varbit ~# [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              ~ 
              (
                i_left
                # i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xnor(B'110', array[array[B'011', B'010'], array[B'101', B'110']])
-- select sm_sc.fv_opr_xnor(B'011101', array[B'010011', B'011010'])
-- select sm_sc.fv_opr_xnor(B'011', array[]::varbit[] )
-- select sm_sc.fv_opr_xnor(B'010', array[array[], array []]::varbit[])
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     B'010'
--   , array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   )
-- select 
--   sm_sc.fv_opr_xnor
--   (
--     B'010'
--   , array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   )
-- set search_path to sm_sc;

-- -- 逻辑运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_xor(boolean[], boolean[]);
create or replace function sm_sc.fv_opr_xor
(
  i_left     boolean[]    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] xor []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] xor array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::boolean[];
  
  -- [] xor []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] <> i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] xor [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] <> i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] xor [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] <> i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] xor [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_xor
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   )

-- select 
--   sm_sc.fv_opr_xor
--   ( 
--     sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[   1, 4, 5]) :: decimal[]) :: int[] :: boolean[]
--   , sm_sc.fv_opr_round(sm_sc.fv_new_rand(array[2, 3, 1, 5]) :: decimal[]) :: int[] :: boolean[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_xor
--   (
--     array[array[true, false], array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[true], array[false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true], array[false]]
--   );
-- -- select sm_sc.fv_opr_xor
-- --   (
-- --     array[true, false],
-- --     array[array[false, true], array[false, true]]
-- --   );
-- -- select sm_sc.fv_opr_xor
-- --   (
-- --     array[array[true, false], array[true, false]],
-- --     array[true, false]
-- --   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[true, false]],
--     array[array[false, true], array[false, true]]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[true, false], array[true, false]],
--     array[array[true, false]]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[], array []]::boolean[],
--     array[]::boolean[]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[], array []]::boolean[],
--     array[array[], array []]::boolean[]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[true, false],
--     array[true, false]
--   );
-- select sm_sc.fv_opr_xor(array[true], array[true, false]);
-- select sm_sc.fv_opr_xor(array[false], array[array[true, false]]);
-- select sm_sc.fv_opr_xor(array[array[true]], array[true, false]);
-- select sm_sc.fv_opr_xor(array[array[false]], array[array[true, false]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , array[[[true,false,true],[false,false,true]],[[true,false,false],[false,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , array[[[[false,false,true],[true,false,false]],[[true,false,true],[false,false,true]]],[[[true,false,false],[false,false,true]],[[false,true,false],[false,false,true]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xor(boolean[], boolean);
create or replace function sm_sc.fv_opr_xor
(
  i_left     boolean[]    ,
  i_right    boolean
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] xor null :: boolean[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::boolean[];

  -- [][] xor boolean
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] <> i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] xor boolean
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] <> i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] xor boolean
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] <> i_right)
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] xor boolean
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> i_right)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xor(array[array[true, false], array[false, true]], false)
-- select sm_sc.fv_opr_xor(array[true, false], false)
-- select sm_sc.fv_opr_xor(array[]::boolean[], true)
-- select sm_sc.fv_opr_xor(array[array[], array []]::boolean[], false)
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   , true
--   )
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   , false
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xor(boolean, boolean[]);
create or replace function sm_sc.fv_opr_xor
(
  i_left     boolean    ,
  i_right    boolean[]
)
returns boolean[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: boolean[][] xor null :: boolean[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::boolean[];

  -- boolean xor [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := (i_left <> i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- boolean xor []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := (i_left <> i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- boolean xor [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left <> i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- boolean xor [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left <> i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xor(false, array[array[false, true], array[true, false]])
-- select sm_sc.fv_opr_xor(false, array[true, false])
-- select sm_sc.fv_opr_xor(false, array[]::boolean[] )
-- select sm_sc.fv_opr_xor(true, array[array[], array []]::boolean[])
-- select 
--   sm_sc.fv_opr_xor
--   (
--     true
--   , array[[[true,false,true],[true,false,true]],[[false,true,false],[true,true,false]]]
--   )
-- select 
--   sm_sc.fv_opr_xor
--   (
--     false
--   , array[[[[true,false,true],[true,false,true]],[[false,true,false],[true,false,false]]],[[[false,false,true],[false,false,true]],[[false,false,true],[false,true,false]]]]
--   )

-- -------------------------------------------------------------------------------------------------------------------------------

-- -- 位运算
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_xor(varbit[], varbit[]);
create or replace function sm_sc.fv_opr_xor
(
  i_left     varbit[]    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] + []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] + array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return null::varbit;
  
  -- [] # []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := (i_left[v_y_cur] # i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] # [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := (i_left[v_y_cur][v_x_cur] # i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] # [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            (i_left[v_y_cur][v_x_cur][v_x3_cur] # i_right[v_y_cur][v_x_cur][v_x3_cur]) 
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] # [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              (i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] # i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_xor
--   ( 
--     (
--       with 
--       cte_x4 as
--       (
--         select 
--           a_y_cur, a_x_cur, a_x3_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x4
--         from generate_series(1, 2) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 1) tb_a_x3_cur(a_x3_cur)
--           , generate_series(1, 5) tb_a_x4_cur(a_x4_cur)
--         group by a_y_cur, a_x_cur, a_x3_cur
--       ),
--       cte_x3 as 
--       (
--         select 
--       	a_y_cur, a_x_cur, array_agg(a_x4) as a_x3
--         from cte_x4
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2 as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2
--     )
--   , (
--       with 
--       cte_x3_ex as
--       (
--         select 
--           a_y_cur, a_x_cur,
--           array_agg(abs(round(random() * 16 - 0.5)) :: int :: bit(3)) as a_x3
--         from generate_series(1, 3) tb_a_y_cur(a_y_cur)
--           , generate_series(1, 3) tb_a_x_cur(a_x_cur)
--           , generate_series(1, 5) tb_a_x3_cur(a_x3_cur)
--         group by a_y_cur, a_x_cur
--       ),
--       cte_x2_ex as 
--       (
--         select 
--       	a_y_cur, array_agg(a_x3) as a_x2
--         from cte_x3_ex
--         group by a_y_cur
--       )
--       select array_agg(a_x2) :: varbit[] from cte_x2_ex
--     )
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_xor
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[B'010'], array[B'011']],
--     array[array[B'011', B'101'], array[B'011', B'101']]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[B'010', B'011'], array[B'101', B'011']],
--     array[array[B'101'], array[B'011']]
--   );
-- -- select sm_sc.fv_opr_xor
-- --   (
-- --     array[B'101', B'011'],
-- --     array[array[B'011', B'101'], array[B'011', B'010']]
-- --   );
-- -- select sm_sc.fv_opr_xor
-- --   (
-- --     array[array[B'101', B'011'], array[B'101', B'011']],
-- --     array[B'010', B'011']
-- --   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[B'101', B'011']],
--     array[array[B'011', B'101'], array[B'011', B'010']]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[B'101', B'011'], array[B'101', B'011']],
--     array[array[B'010', B'011']]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[], array []]::varbit[],
--     array[]::varbit[]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[array[], array []]::varbit[],
--     array[array[], array []]::varbit[]
--   );
-- select sm_sc.fv_opr_xor
--   (
--     array[B'101', B'011'],
--     array[B'010', B'011']
--   );
-- select sm_sc.fv_opr_xor(array[B'101'], array[B'101', B'011']);
-- select sm_sc.fv_opr_xor(array[B'101'], array[array[B'101', B'011']]);
-- select sm_sc.fv_opr_xor(array[array[B'101']], array[B'101', B'011']);
-- select sm_sc.fv_opr_xor(array[array[B'101']], array[array[B'101', B'011']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , array[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]]
--   )
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , array[[[[B'011',B'001',B'100'],[B'011',B'110',B'000']],[[B'001',B'100',B'011'],[B'011',B'100',B'111']]],[[[B'001',B'010',B'100'],[B'011',B'010',B'111']],[[B'010',B'100',B'001'],[B'101',B'110',B'001']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xor(varbit[], varbit);
create or replace function sm_sc.fv_opr_xor
(
  i_left     varbit[]    ,
  i_right    varbit
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit[][] & varbit = null :: varbit[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::varbit[];

  -- [][] # varbit
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := i_left[v_y_cur][v_x_cur] # i_right;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] # varbit
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := i_left[v_y_cur] # i_right;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
  -- [][][] # varbit
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left[v_y_cur][v_x_cur][v_x3_cur] 
            # i_right
          ;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] # varbit
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              # i_right
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;    

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xor(array[array[B'010', B'011'], array[B'011', B'010']], B'011')
-- select sm_sc.fv_opr_xor(array[B'010011', B'011010'], B'011101')
-- select sm_sc.fv_opr_xor(array[]::varbit[], B'010')
-- select sm_sc.fv_opr_xor(array[array[], array []]::varbit[], B'011')
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   , B'010'
--   )
-- select 
--   sm_sc.fv_opr_xor
--   (
--     array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   , B'010'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_opr_xor(varbit, varbit[]);
create or replace function sm_sc.fv_opr_xor
(
  i_left     varbit    ,
  i_right    varbit[]
)
returns varbit[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: varbit # null :: varbit[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::varbit[];

  -- varbit # [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := i_left # i_right[v_y_cur][v_x_cur];
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- varbit # []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := i_left # i_right[v_y_cur];
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- varbit # [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            i_left 
            # i_right[v_y_cur][v_x_cur][v_x3_cur] 
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- varbit # [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              i_left
              # i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_opr_xor(B'110', array[array[B'011', B'010'], array[B'101', B'110']])
-- select sm_sc.fv_opr_xor(B'011101', array[B'010011', B'011010'])
-- select sm_sc.fv_opr_xor(B'011', array[]::varbit[] )
-- select sm_sc.fv_opr_xor(B'010', array[array[], array []]::varbit[])
-- select 
--   sm_sc.fv_opr_xor
--   (
--     B'010'
--   , array[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]]
--   )
-- select 
--   sm_sc.fv_opr_xor
--   (
--     B'010'
--   , array[[[[B'101',B'110',B'001'],[B'101',B'011',B'110']],[[B'011',B'001',B'100'],[B'111',B'100',B'011']]],[[[B'110',B'000',B'011'],[B'010',B'001',B'100']],[[B'001',B'100',B'011'],[B'111',B'010',B'101']]]]
--   )
-- drop function if exists sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1(float[], float[], int[], int[3]);
create or replace function sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1
(
  i_dloss_ddepdt   float[]
, i_co_value       float[]
, i_indepdt_len    int[]
, i_chunk_len      int[3]
)
returns float[]
as
$$
declare 
  v_ret  float[];
begin
  -- 对齐维度
  if array_ndims(i_dloss_ddepdt) - array_ndims(i_co_value) = 1
  then 
    i_co_value := array[i_co_value];
  elsif array_ndims(i_dloss_ddepdt) - array_ndims(i_co_value) = 2
  then 
    i_co_value := array[[i_co_value]];
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    v_ret :=  
    (
      with 
      cte_chunk_width as 
      (
        select 
          a_cur_heigh
        , sm_sc.fa_mx_concat_x
          (
            i_dloss_ddepdt
              [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
              [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
            |**| 
            (
              |^~| 
              (
                i_co_value
                  [i_chunk_len[2] * (a_cur_heigh - 1) + 1 : i_chunk_len[2] * a_cur_heigh]
                  [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
              )
            )
            order by a_cur_width
          ) as a_chunk_width
        from generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) / i_chunk_len[3]) tb_a_cur_width(a_cur_width)
        group by a_cur_heigh
      )
      select 
        sm_sc.fa_mx_concat_y
        (
          a_chunk_width
          order by a_cur_heigh
        )
      from cte_chunk_width
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret :=  
    (
      with 
      cte_chunk_width as 
      (
        select 
          a_cur_heigh
        , sm_sc.fa_mx_concat_x3
          (
            i_dloss_ddepdt
              [ : ]
              [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
              [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
            |**| 
            (
              |^~| 
              (
                i_co_value
                  [ : ]
                  [i_chunk_len[2] * (a_cur_heigh - 1) + 1 : i_chunk_len[2] * a_cur_heigh]
                  [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
              )
            )
            order by a_cur_width
          ) as a_chunk_width
        from generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) / i_chunk_len[3]) tb_a_cur_width(a_cur_width)
        group by a_cur_heigh
      )
      select 
        sm_sc.fa_mx_concat_x
        (
          a_chunk_width
          order by a_cur_heigh
        )
      from cte_chunk_width
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret :=  
    (
      with 
      cte_chunk_width as 
      (
        select 
          a_cur_heigh
        , sm_sc.fa_mx_concat_x4
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
              [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
            |**| 
            (
              |^~| 
              (
                i_co_value
                  [ : ]
                  [ : ]
                  [i_chunk_len[2] * (a_cur_heigh - 1) + 1 : i_chunk_len[2] * a_cur_heigh]
                  [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
              )
            )
            order by a_cur_width
          ) as a_chunk_width
        from generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) / i_chunk_len[3]) tb_a_cur_width(a_cur_width)
        group by a_cur_heigh
      )
      select 
        sm_sc.fa_mx_concat_x3
        (
          a_chunk_width
          order by a_cur_heigh
        )
      from cte_chunk_width
    )
    ;
  else
    raise exception 'unsupport ndims of i_dloss_ddepdt';
  end if;
  
  -- return
  if array_ndims(i_dloss_ddepdt) - array_length(i_indepdt_len, 1) = 1
  then 
    return sm_sc.fv_mx_descend_dim(sm_sc.fv_aggr_chunk_sum(v_ret, array[1] || i_indepdt_len), 1);
  elsif array_ndims(i_dloss_ddepdt) - array_length(i_indepdt_len, 1) = 2
  then 
    return sm_sc.fv_mx_descend_dim(sm_sc.fv_aggr_chunk_sum(v_ret, array[1, 1] || i_indepdt_len), 2);
  elsif array_ndims(i_dloss_ddepdt) = array_length(i_indepdt_len, 1)
  then 
    return v_ret;
  end if;
  
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[2 * 7, 5 * 11])
--   , sm_sc.fv_new_rand(array[3 * 7, 5 * 11])
--   , array[2 * 7, 3 * 11]
--   , array[2, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[3, 2 * 7, 5 * 11])
--   , sm_sc.fv_new_rand(array[3, 3 * 7, 5 * 11])
--   , array[2 * 7, 3 * 11]
--   , array[2, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[2, 3, 2 * 7, 5 * 11])
--   , sm_sc.fv_new_rand(array[2, 3, 3 * 7, 5 * 11])
--   , array[2 * 7, 3 * 11]
--   , array[2, 3, 5]
--   )
-- drop function if exists sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2(float[], float[], int[], int[3]);
create or replace function sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2
(
  i_dloss_ddepdt   float[]
, i_co_value       float[]
, i_indepdt_len    int[]
, i_chunk_len      int[3]
)
returns float[]
as
$$
declare 
  v_ret  float[];
begin
  -- 对齐维度
  if array_ndims(i_dloss_ddepdt) - array_ndims(i_co_value) = 1
  then 
    i_co_value := array[i_co_value];
  elsif array_ndims(i_dloss_ddepdt) - array_ndims(i_co_value) = 2
  then 
    i_co_value := array[[i_co_value]];
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    v_ret :=  
    (
      with 
      cte_chunk_width as 
      (
        select 
          a_cur_heigh
        , sm_sc.fa_mx_concat_x
          (
            (
              |^~| 
              (
                i_co_value
                  [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
                  [i_chunk_len[2] * (a_cur_width - 1) + 1 : i_chunk_len[2] * a_cur_width]
              )
            )
            |**| 
            i_dloss_ddepdt
              [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
              [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
            order by a_cur_width
          ) as a_chunk_width
        from generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) / i_chunk_len[3]) tb_a_cur_width(a_cur_width)
        group by a_cur_heigh
      )
      select 
        sm_sc.fa_mx_concat_y
        (
          a_chunk_width
          order by a_cur_heigh
        )
      from cte_chunk_width
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret :=  
    (
      with 
      cte_chunk_width as 
      (
        select 
          a_cur_heigh
        , sm_sc.fa_mx_concat_x3
          (
            (
              |^~| 
              (
                i_co_value
                  [ : ]
                  [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
                  [i_chunk_len[2] * (a_cur_width - 1) + 1 : i_chunk_len[2] * a_cur_width]
              )
            )
            |**| 
            i_dloss_ddepdt
              [ : ]
              [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
              [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
            order by a_cur_width
          ) as a_chunk_width
        from generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) / i_chunk_len[3]) tb_a_cur_width(a_cur_width)
        group by a_cur_heigh
      )
      select 
        sm_sc.fa_mx_concat_x
        (
          a_chunk_width
          order by a_cur_heigh
        )
      from cte_chunk_width
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret :=  
    (
      with 
      cte_chunk_width as 
      (
        select 
          a_cur_heigh
        , sm_sc.fa_mx_concat_x4
          (
            (
              |^~| 
              (
                i_co_value
                  [ : ]
                  [ : ]
                  [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
                  [i_chunk_len[2] * (a_cur_width - 1) + 1 : i_chunk_len[2] * a_cur_width]
              )
            )
            |**| 
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
              [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width]
            order by a_cur_width
          ) as a_chunk_width
        from generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) / i_chunk_len[3]) tb_a_cur_width(a_cur_width)
        group by a_cur_heigh
      )
      select 
        sm_sc.fa_mx_concat_x3
        (
          a_chunk_width
          order by a_cur_heigh
        )
      from cte_chunk_width
    )
    ;
  else
    raise exception 'unsupport ndims of i_dloss_ddepdt';
  end if;
  
  -- return
  if array_ndims(i_dloss_ddepdt) - array_length(i_indepdt_len, 1) = 1
  then 
    return sm_sc.fv_mx_descend_dim(sm_sc.fv_aggr_chunk_sum(v_ret, array[1] || i_indepdt_len), 1);
  elsif array_ndims(i_dloss_ddepdt) - array_length(i_indepdt_len, 1) = 2
  then 
    return sm_sc.fv_mx_descend_dim(sm_sc.fv_aggr_chunk_sum(v_ret, array[1, 1] || i_indepdt_len), 2);
  elsif array_ndims(i_dloss_ddepdt) = array_length(i_indepdt_len, 1)
  then 
    return v_ret;
  end if;
  
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[2 * 7, 5 * 11])
--   , sm_sc.fv_new_rand(array[2 * 7, 3 * 11])
--   , array[3 * 7, 5 * 11]
--   , array[2, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[3, 2 * 7, 5 * 11])
--   , sm_sc.fv_new_rand(array[3, 2 * 7, 3 * 11])
--   , array[3 * 7, 5 * 11]
--   , array[2, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[2, 3, 2 * 7, 5 * 11])
--   , sm_sc.fv_new_rand(array[2, 3, 2 * 7, 3 * 11])
--   , array[3 * 7, 5 * 11]
--   , array[2, 3, 5]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_chunk_transpose_dloss_dindepdt(anyarray, int[2]);
create or replace function sm_sc.fv_d_chunk_transpose_dloss_dindepdt
(
  i_dloss_ddepdt     anyarray
, i_chunk_len        int[2]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_dloss_ddepdt) is null
  then 
    return i_dloss_ddepdt;
  elsif array_ndims(i_dloss_ddepdt) between 2 and 4
  then
    return 	
      sm_sc.fv_chunk_transpose(i_dloss_ddepdt, array[i_chunk_len[2], i_chunk_len[1]])
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_d_chunk_transpose_dloss_dindepdt
--   (
--     sm_sc.fv_chunk_transpose
--     (
--       sm_sc.fv_new_rand(array[6, 15])
--     , array[3, 5]
--     )
--   , array[3, 5]
--   );
-- select 
--   sm_sc.fv_d_chunk_transpose_dloss_dindepdt
--   (
--     sm_sc.fv_chunk_transpose
--     (
--       sm_sc.fv_new_rand(array[2, 6, 15])
--     , array[3, 5]
--     )
--   , array[3, 5]
--   );
-- select 
--   sm_sc.fv_d_chunk_transpose_dloss_dindepdt
--   (
--     sm_sc.fv_chunk_transpose
--     (
--       sm_sc.fv_new_rand(array[3, 2, 6, 15])
--     , array[3, 5]
--     )
--   , array[3, 5]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_concat_x3_dloss_dindepdt(anyarray, int4range, int[]);
create or replace function sm_sc.fv_d_concat_x3_dloss_dindepdt
(
  i_dloss_ddepdt       anyarray   ,
  i_indepdt_idx_range  int4range  ,
  i_indepdt_len        int[]   default null
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if i_indepdt_len[3] <> upper(i_indepdt_idx_range) - lower(i_indepdt_idx_range)
    then 
      raise exception 'unmatch indepdt idx range at dloss_ddepdt and indepdt len. ';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 4
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[ : ][ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[ : ][ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ]
        , i_indepdt_len
        )
      ;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[ : ][ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[ : ][ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
        , i_indepdt_len
        )
      ;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_d_concat_x3_dloss_dindepdt
-- --   (
-- --     array[array[1, 2], array[3, 4]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x3_dloss_dindepdt
-- --   (
-- --     array[1, 2, 3, 4]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x3_dloss_dindepdt
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x3_dloss_dindepdt
-- --   (
-- --     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- select sm_sc.fv_d_concat_x3_dloss_dindepdt
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_x3_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]   ,
--     int4range(2, 2, '[]')
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_concat_x4_dloss_dindepdt(anyarray, int4range, int[]);
create or replace function sm_sc.fv_d_concat_x4_dloss_dindepdt
(
  i_dloss_ddepdt       anyarray   ,
  i_indepdt_idx_range  int4range  ,
  i_indepdt_len        int[]   default null
)
returns anyarray
as
$$
-- declare 
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if i_indepdt_len[4] <> upper(i_indepdt_idx_range) - lower(i_indepdt_idx_range)
    then 
      raise exception 'unmatch indepdt idx range at dloss_ddepdt and indepdt len. ';
    end if;
  end if;

  -- set search_path to sm_sc;
  if array_ndims(i_dloss_ddepdt) = 4
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[ : ][ : ][ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[ : ][ : ][ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
        , i_indepdt_len
        )
      ;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_d_concat_x4_dloss_dindepdt
-- --   (
-- --     array[array[1, 2], array[3, 4]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x4_dloss_dindepdt
-- --   (
-- --     array[1, 2, 3, 4]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x4_dloss_dindepdt
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x4_dloss_dindepdt
-- --   (
-- --     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- -- select sm_sc.fv_d_concat_x4_dloss_dindepdt
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- select sm_sc.fv_d_concat_x4_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]   ,
--     int4range(2, 2, '[]')
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_concat_x_dloss_dindepdt(anyarray, int4range, int[]);
create or replace function sm_sc.fv_d_concat_x_dloss_dindepdt
(
  i_dloss_ddepdt       anyarray   ,
  i_indepdt_idx_range  int4range  ,
  i_indepdt_len        int[]   default null
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if i_indepdt_len[2] <> upper(i_indepdt_idx_range) - lower(i_indepdt_idx_range)
    then 
      raise exception 'unmatch indepdt idx range at dloss_ddepdt and indepdt len. ';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 4
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ][ : ]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ][ : ]
        , i_indepdt_len
        )
      ;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ]
        , i_indepdt_len
        )
      ;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 2
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[ : ][lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
        , i_indepdt_len
        )
      ;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_concat_x_dloss_dindepdt
--   (
--     array[array[1, 2], array[3, 4]]   ,
--     int4range(2, 2, '[]')
--   );
-- -- select sm_sc.fv_d_concat_x_dloss_dindepdt
-- --   (
-- --     array[1, 2, 3, 4]   ,
-- --     int4range(2, 2, '[]')
-- --   );
-- select sm_sc.fv_d_concat_x_dloss_dindepdt
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_x_dloss_dindepdt
--   (
--     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_x_dloss_dindepdt
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_x_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]   ,
--     int4range(2, 2, '[]')
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_concat_y_dloss_dindepdt(anyarray, int4range, int[]);
create or replace function sm_sc.fv_d_concat_y_dloss_dindepdt
(
  i_dloss_ddepdt       anyarray   ,
  i_indepdt_idx_range  int4range  ,
  i_indepdt_len        int[]   default null
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if i_indepdt_len[1] <> upper(i_indepdt_idx_range) - lower(i_indepdt_idx_range)
    then 
      raise exception 'unmatch indepdt idx range at dloss_ddepdt and indepdt len. ';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 4
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ][ : ][ : ]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ][ : ][ : ]
        , i_indepdt_len
        )
      ;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ][ : ]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ][ : ]
        , i_indepdt_len
        )
      ;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 2
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1][ : ]
        , i_indepdt_len
        )
      ;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 1
  then
    if i_indepdt_len is null 
    then 
      return 	
        i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
      ;
    else 
      -- 广播追踪
      return 
        sm_sc.fv_aggr_chunk_sum
        (
          i_dloss_ddepdt[lower(i_indepdt_idx_range) : upper(i_indepdt_idx_range) - 1]
        , i_indepdt_len
        )
      ;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_concat_y_dloss_dindepdt
--   (
--     array[array[1, 2], array[3, 4]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_y_dloss_dindepdt
--   (
--     array[1, 2, 3, 4]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_y_dloss_dindepdt
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_y_dloss_dindepdt
--   (
--     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]   ,
--     int4range(2, 2, '[]')
--   );
-- select sm_sc.fv_d_concat_y_dloss_dindepdt
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]   ,
--     int4range(2, 2, '[]'),
--     array[1, 3, 2]
--   );
-- select sm_sc.fv_d_concat_y_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]   ,
--     int4range(2, 2, '[]')
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_mirror_dloss_dindepdt(anyarray, int[2]);
create or replace function sm_sc.fv_d_mirror_dloss_dindepdt
(
  i_dloss_ddepdt            anyarray,
  i_dim            int
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;    
  if i_dim = 1
  then 
    return sm_sc.__fv_mirror_y(i_dloss_ddepdt);    
  elsif i_dim = 2
  then
    return sm_sc.__fv_mirror_x(i_dloss_ddepdt);   
  elsif i_dim = 3
  then
    return sm_sc.__fv_mirror_x3(i_dloss_ddepdt);   
  elsif i_dim = 4
  then
    return sm_sc.__fv_mirror_x4(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_mirror_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , 3
--   );
-- drop function if exists sm_sc.fv_d_prod_mx_1(float[][], int[]);
create or replace function sm_sc.fv_d_prod_mx_1
(
  i_co_value    float[], 
  i_indepdt_len int[]    default null
)
returns float[]
as
$$
declare 
begin
  if i_indepdt_len is null 
  then 
    return i_co_value;
  elsif array_ndims(i_co_value) = 2
  then 
    if array_length(i_indepdt_len, 1) = 3
    then 
      return i_co_value *` i_indepdt_len[1] :: float;
    elsif array_length(i_indepdt_len, 1) = 4
    then
      return i_co_value *` (i_indepdt_len[1] * i_indepdt_len[2]) :: float;
    else  -- array_length(i_indepdt_len, 1) = 2
      return i_co_value;
    end if;
  elsif array_ndims(i_co_value) = 3
  then 
    if array_length(i_indepdt_len, 1) = 4
    then 
      return i_co_value *` i_indepdt_len[1] :: float;
    elsif array_length(i_indepdt_len, 1) = 2
    then 
      return 
        sm_sc.fv_mx_slice_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum(i_co_value, array[array_length(i_co_value, 1), 1, 1])
        , 1
        , 1
        )
      ;
    else -- array_length(i_indepdt_len, 1) = 3
      return i_co_value;
    end if;
  elsif array_ndims(i_co_value) = 4
  then 
    if array_length(i_indepdt_len, 1) = 2
    then 
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum(i_co_value, array[array_length(i_co_value, 1), array_length(i_co_value, 2), 1, 1])
        , array[1, 2]
        , array[1, 1]
        )
      ;
    elsif array_length(i_indepdt_len, 1) = 3
    then 
      return 
        sm_sc.fv_mx_slice_4d_2_3d
        (
          sm_sc.fv_aggr_slice_sum(i_co_value, array[array_length(i_co_value, 1), 1, 1, 1])
        , 1
        , 1
        )
      ;
    else -- array_length(i_indepdt_len, 1) = 4
      return i_co_value;
    end if;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_prod_mx_1
--   (
--     sm_sc.fv_new_rand(array[3, 2, 5])
--   , array[6, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_1
--   (
--     sm_sc.fv_new_rand(array[3, 2, 5])
--   , array[3, 6, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_1
--   (
--     sm_sc.fv_new_rand(array[4, 3, 2, 5])
--   , array[6, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_1
--   (
--     sm_sc.fv_new_rand(array[4, 3, 2, 5])
--   , array[3, 6, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_1
--   (
--     sm_sc.fv_new_rand(array[4, 3, 2, 5])
--   , array[4, 3, 6, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_1
--   (
--     sm_sc.fv_new_rand(array[2, 5])
--   , array[6, 5]
--   )
-- drop function if exists sm_sc.fv_d_prod_mx_2(float[][], int[]);
create or replace function sm_sc.fv_d_prod_mx_2
(
  i_co_value    float[][],
  i_indepdt_len int[]    default null
)
returns float[][]
as
$$
declare 
begin
  if i_indepdt_len is null 
  then 
    return |^~| i_co_value;
  elsif array_ndims(i_co_value) = 2
  then 
    if array_length(i_indepdt_len, 1) = 3
    then 
      return (|^~| i_co_value) *` i_indepdt_len[1] :: float;
    elsif array_length(i_indepdt_len, 1) = 4
    then
      return (|^~| i_co_value) *` (i_indepdt_len[1] * i_indepdt_len[2]) :: float;
    else  -- array_length(i_indepdt_len, 1) = 2
      return |^~| i_co_value;
    end if;
  elsif array_ndims(i_co_value) = 3
  then 
    if array_length(i_indepdt_len, 1) = 4
    then 
      return (|^~| i_co_value) *` i_indepdt_len[1] :: float;
    elsif array_length(i_indepdt_len, 1) = 2
    then 
      return 
        |^~|
        sm_sc.fv_mx_slice_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum(i_co_value, array[array_length(i_co_value, 1), 1, 1])
        , 1
        , 1
        )
      ;
    else  -- array_length(i_indepdt_len, 1) = 3
      return |^~| i_co_value;
    end if;
  elsif array_ndims(i_co_value) = 4
  then 
    if array_length(i_indepdt_len, 1) = 2
    then 
      return 
        |^~|
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum(i_co_value, array[array_length(i_co_value, 1), array_length(i_co_value, 2), 1, 1])
        , array[1, 2]
        , array[1, 1]
        )
      ;
    elsif array_length(i_indepdt_len, 1) = 3
    then 
      return 
        |^~|
        sm_sc.fv_mx_slice_4d_2_3d
        (
          sm_sc.fv_aggr_slice_sum(i_co_value, array[array_length(i_co_value, 1), 1, 1, 1])
        , 1
        , 1
        )
      ;
    else  -- array_length(i_indepdt_len, 1) = 4
      return |^~| i_co_value;
    end if;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_prod_mx_2
--   (
--     sm_sc.fv_new_rand(array[3, 2, 5])
--   , array[3, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_2
--   (
--     sm_sc.fv_new_rand(array[3, 2, 5])
--   , array[3, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_2
--   (
--     sm_sc.fv_new_rand(array[4, 3, 2, 5])
--   , array[3, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_2
--   (
--     sm_sc.fv_new_rand(array[4, 3, 2, 5])
--   , array[3, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_2
--   (
--     sm_sc.fv_new_rand(array[4, 3, 2, 5])
--   , array[4, 3, 3, 5]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_2
--   (
--     sm_sc.fv_new_rand(array[2, 5])
--   , array[3, 5]
--   )
-- drop function if exists sm_sc.fv_d_prod_mx_dloss_dindepdt_1(float[], float[], int[]);
create or replace function sm_sc.fv_d_prod_mx_dloss_dindepdt_1
(
  i_dloss_ddepdt   float[]   ,
  i_co_value       float[]   ,
  i_indepdt_len    int[]    default null
)
returns float[]
as
$$
declare 
begin
  -- return i_dloss_ddepdt |**| (|^~| sm_sc.fv_d_prod_mx_1(i_co_value));
  if i_indepdt_len is null 
  then 
    return i_dloss_ddepdt |**| (|^~| i_co_value);
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    if array_length(i_indepdt_len, 1) = 3
    then 
      return 
        sm_sc.fv_mx_slice_4d_2_3d
        (
          sm_sc.fv_aggr_slice_sum
          (
            i_dloss_ddepdt |**| (|^~| i_co_value)
          , array[array_length(i_dloss_ddepdt, 1), 1, 1, 1]
          )
        , 1
        , 1
        )
      ;
    elsif array_length(i_indepdt_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            i_dloss_ddepdt |**| (|^~| i_co_value)
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else -- array_length(i_indepdt_len, 1) = 4
      return i_dloss_ddepdt |**| (|^~| i_co_value);
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    if array_length(i_indepdt_len, 1) = 2
    then 
      return 
        sm_sc.fv_mx_slice_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            i_dloss_ddepdt |**| (|^~| i_co_value)
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , 1
        , 1
        )
      ;
    else -- array_length(i_indepdt_len, 1) = 3
      return i_dloss_ddepdt |**| (|^~| i_co_value);
    end if;
  else 
    return i_dloss_ddepdt |**| (|^~| i_co_value);
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[3, 4])     ,
--     array[2, 3]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[5, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[3, 4])     ,
--     array[5, 2, 3]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[5, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[5, 3, 4])     ,
--     array[2, 3]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[5, 6, 3, 4])     ,
--     array[2, 3]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[3, 4])     ,
--     array[5, 6, 2, 3]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[5, 6, 3, 4])     ,
--     array[6, 2, 3]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_1
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[6, 3, 4])     ,
--     array[5, 6, 2, 3]
--   )
-- drop function if exists sm_sc.fv_d_prod_mx_dloss_dindepdt_2(float[], float[], int[]);
create or replace function sm_sc.fv_d_prod_mx_dloss_dindepdt_2
(
  i_dloss_ddepdt   float[]   ,
  i_co_value       float[]   ,
  i_indepdt_len    int[]    default null
)
returns float[]
as
$$
declare 
begin
  -- return sm_sc.fv_d_prod_mx_2(i_co_value) |**| i_dloss_ddepdt;
  if i_indepdt_len is null 
  then 
    return (|^~| i_co_value) |**| i_dloss_ddepdt;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    if array_length(i_indepdt_len, 1) = 3
    then 
      return 
        sm_sc.fv_mx_slice_4d_2_3d
        (
          sm_sc.fv_aggr_slice_sum
          (
            (|^~| i_co_value) |**| i_dloss_ddepdt
          , array[array_length(i_dloss_ddepdt, 1), 1, 1, 1]
          )
        , 1
        , 1
        )
      ;
    elsif array_length(i_indepdt_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            (|^~| i_co_value) |**| i_dloss_ddepdt
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else -- array_length(i_indepdt_len, 1) = 4
      return (|^~| i_co_value) |**| i_dloss_ddepdt;
    end if;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    if array_length(i_indepdt_len, 1) = 2
    then 
      return 
        sm_sc.fv_mx_slice_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            (|^~| i_co_value) |**| i_dloss_ddepdt
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , 1
        , 1
        )
      ;
    else -- array_length(i_indepdt_len, 1) = 3
      return (|^~| i_co_value) |**| i_dloss_ddepdt;
    end if;
  else 
    return (|^~| i_co_value) |**| i_dloss_ddepdt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[2, 3])     ,
--     array[3, 4]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[5, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[2, 3])     ,
--     array[5, 3, 4]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[5, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[5, 2, 3])     ,
--     array[3, 4]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[5, 6, 2, 3])     ,
--     array[3, 4]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[2, 3])     ,
--     array[5, 6, 3, 4]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[5, 6, 2, 3])     ,
--     array[6, 3, 4]
--   )

-- select 
--   sm_sc.fv_d_prod_mx_dloss_dindepdt_2
--   (
--     sm_sc.fv_new_rand(array[5, 6, 2, 4])     ,    -- dloss_ddepdt
--     sm_sc.fv_new_rand(array[6, 2, 3])     ,
--     array[5, 6, 3, 4]
--   )
-- drop function if exists sm_sc.fv_d_transpose_3d_dloss_dindepdt(anyarray, int[2]);
create or replace function sm_sc.fv_d_transpose_3d_dloss_dindepdt
(
  i_dloss_ddepdt_3d      anyarray      ,
  i_dims                 int[2]        -- array[1, 2] === array[2, 1], array[1, 3] === array[3, 1] or array[2, 3] === array[3, 2]
)
returns anyarray
as
$$
-- declare

begin
  if array_ndims(i_dloss_ddepdt_3d) <> 3
  then 
    raise exception 'ndims of i_dloss_ddepdt_3d should be 3.';
  end if;
  
  if i_dims[1] not in (1, 2, 3) 
    or i_dims[2] not in (1,2,3)
    or i_dims[1] = i_dims[2]
  then 
    raise exception 'illegal i_dims';
  end if;
 
  return sm_sc.fv_opr_transpose_3d(i_dloss_ddepdt_3d, i_dims);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_transpose_3d_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 2]
--   );
-- select sm_sc.fv_d_transpose_3d_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 3]
--   );
-- select sm_sc.fv_d_transpose_3d_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_transpose_dloss_dindepdt(anyarray);
create or replace function sm_sc.fv_d_transpose_dloss_dindepdt
(
  i_dloss_ddepdt     anyarray
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_dloss_ddepdt) is null
  then 
    return i_dloss_ddepdt;
  elsif array_ndims(i_dloss_ddepdt) between 2 and 4
  then
    return 	
      sm_sc.fv_opr_transpose(i_dloss_ddepdt)
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_transpose_dloss_dindepdt
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_d_transpose_dloss_dindepdt
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6], array[1.2, 2.3]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_transpose_i_dloss_dindepdt(anyarray);
create or replace function sm_sc.fv_d_transpose_i_dloss_dindepdt
(
  i_dloss_ddepdt     anyarray
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_dloss_ddepdt) is null
  then 
    return i_dloss_ddepdt;
  elsif array_ndims(i_dloss_ddepdt) between 2 and 4
  then
    return 	
      sm_sc.fv_opr_transpose_i(i_dloss_ddepdt)
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_transpose_i_dloss_dindepdt
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.fv_d_transpose_i_dloss_dindepdt
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- drop function if exists sm_sc.fv_d_transpose_nd_dloss_dindepdt(anyarray, int[2]);
create or replace function sm_sc.fv_d_transpose_nd_dloss_dindepdt
(
  i_dloss_ddepdt_nd      anyarray      ,
  i_dims                 int[2]
)
returns anyarray
as
$$
-- declare

begin 
  return sm_sc.fv_opr_transpose_nd(i_dloss_ddepdt_nd, i_dims);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 2]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 3]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[2, 3]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,2]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,3]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,4]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[2,3]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[2,4]
--   );
-- select sm_sc.fv_d_transpose_nd_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[3,4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_turn_180_dloss_dindepdt(anyarray, int[2]);
create or replace function sm_sc.fv_d_turn_180_dloss_dindepdt
(
  i_dloss_ddepdt            anyarray,
  i_dims           int[2]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if i_dims in (array[1, 2], array[2, 1])
  then 
    return sm_sc.__fv_turn_y_x_180(i_dloss_ddepdt);    
  elsif i_dims in (array[1, 3], array[3, 1])
  then
    return sm_sc.__fv_turn_y_x3_180(i_dloss_ddepdt);   
  elsif i_dims in (array[1, 4], array[4, 1])
  then
    return sm_sc.__fv_turn_y_x4_180(i_dloss_ddepdt);   
  elsif i_dims in (array[2, 3], array[3, 2])
  then
    return sm_sc.__fv_turn_x_x3_180(i_dloss_ddepdt);   
  elsif i_dims in (array[2, 4], array[4, 2])
  then
    return sm_sc.__fv_turn_x_x4_180(i_dloss_ddepdt);   
  elsif i_dims in (array[3, 4], array[4, 3])
  then
    return sm_sc.__fv_turn_x3_x4_180(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_turn_180_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_turn_180_dloss_dindepdt_py(float[], int[2]);
create or replace function sm_sc.fv_d_turn_180_dloss_dindepdt_py
(
  i_dloss_ddepdt            float[],
  i_dims    int[2]
)
returns float[]
as
$$
  import numpy as np
  if i_dims[0] == 1 and i_dims[1] == 2 or i_dims[0] == 2 and i_dims[1] == 1:
    return np.array(i_dloss_ddepdt)[::-1, ::-1].tolist()
  elif i_dims[0] == 1 and i_dims[1] == 3 or i_dims[0] == 3 and i_dims[1] == 1:
    return np.array(i_dloss_ddepdt)[::-1, ::, ::-1].tolist()
  elif i_dims[0] == 1 and i_dims[1] == 4 or i_dims[0] == 4 and i_dims[1] == 1:
    return np.array(i_dloss_ddepdt)[::-1, ::, ::, ::-1].tolist()
  elif i_dims[0] == 2 and i_dims[1] == 3 or i_dims[0] == 3 and i_dims[1] == 2:
    return np.array(i_dloss_ddepdt)[::, ::-1, ::-1].tolist()
  elif i_dims[0] == 2 and i_dims[1] == 4 or i_dims[0] == 4 and i_dims[1] == 2:
    return np.array(i_dloss_ddepdt)[::, ::-1, ::, ::-1].tolist()
  elif i_dims[0] == 3 and i_dims[1] == 4 or i_dims[0] == 4 and i_dims[1] == 3:
    return np.array(i_dloss_ddepdt)[::, ::, ::-1, ::-1].tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_turn_180_dloss_dindepdt_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_turn_90_dloss_dindepdt(anyarray, int[2]);
create or replace function sm_sc.fv_d_turn_90_dloss_dindepdt
(
  i_dloss_ddepdt            anyarray,
  i_dim_from_to    int[2]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;    
  if i_dim_from_to = array[2, 1]
  then 
    return sm_sc.__fv_turn_y_x_90(i_dloss_ddepdt);    
  elsif i_dim_from_to = array[3, 1]
  then
    return sm_sc.__fv_turn_y_x3_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[4, 1]
  then
    return sm_sc.__fv_turn_y_x4_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[3, 2]
  then
    return sm_sc.__fv_turn_x_x3_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[4, 2]
  then
    return sm_sc.__fv_turn_x_x4_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[4, 3]
  then
    return sm_sc.__fv_turn_x3_x4_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[1, 2]
  then 
    return sm_sc.__fv_turn_x_y_90(i_dloss_ddepdt);    
  elsif i_dim_from_to = array[1, 3]
  then
    return sm_sc.__fv_turn_x3_y_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[1, 4]
  then
    return sm_sc.__fv_turn_x4_y_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[2, 3]
  then
    return sm_sc.__fv_turn_x3_x_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[2, 4]
  then
    return sm_sc.__fv_turn_x4_x_90(i_dloss_ddepdt);   
  elsif i_dim_from_to = array[3, 4]
  then
    return sm_sc.__fv_turn_x4_x3_90(i_dloss_ddepdt);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_turn_90_dloss_dindepdt
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_turn_90_dloss_dindepdt_py(float[], int[2]);
create or replace function sm_sc.fv_d_turn_90_dloss_dindepdt_py
(
  i_dloss_ddepdt            float[],
  i_dims_from_to    int[2]
)
returns float[]
as
$$
  import numpy as np
  if i_dims_from_to[0] == 1 :
    return np.array(i_dloss_ddepdt).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::-1].tolist()
  elif i_dims_from_to[0] == 2 :
    return np.array(i_dloss_ddepdt).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::, ::-1].tolist()
  elif i_dims_from_to[0] == 3 :
    return np.array(i_dloss_ddepdt).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::, ::, ::-1].tolist()
  elif i_dims_from_to[0] == 4 :
    return np.array(i_dloss_ddepdt).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::, ::, ::, ::-1].tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_turn_90_dloss_dindepdt_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_mirror_x(anyarray);
create or replace function sm_sc.__fv_mirror_x
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur               int; 
  -- v_ele              alias for $0;
  v_ret               i_right%type;
  v_len_mirror_dim    int  :=   array_length(i_right, 2);
begin
  -- set search_path to sm_sc;
  
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 1
  -- -- then
  -- --   -- -- for v_cur in 1 .. array_length(i_right, 1) / 2
  -- --   -- -- loop
  -- --   -- --   v_ele[1] := i_right[v_cur];
  -- --   -- --   i_right[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   -- --   i_right[array_length(i_right, 1) - v_cur + 1] := v_ele[1];
  -- --   -- -- end loop;
  -- --   -- -- return i_right;
  -- --   v_ret := array_fill(nullif(i_right[1], i_right[1]), array[array_length(i_right, 1)]);
  -- --   for v_cur in 1 .. array_length(i_right, 1)
  -- --   loop 
  -- --     v_ret[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;

  elsif array_ndims(i_right) = 2
  then
    -- -- return 	
    -- -- (
    -- --   select
    -- --     array_agg(array_x_new order by a_cur_y)
    -- --   from 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_x desc) as array_x_new
    -- --     from generate_series(1, array_length(i_right, 1)) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, v_len_mirror_dim) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   ) t_array_x_new
    -- -- )
    -- -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[array_length(i_right, 1), v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][v_cur : v_cur] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[array_length(i_right, 1), v_len_mirror_dim, array_length(i_right, 3)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][v_cur : v_cur][ : ] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1][1][1]), array[array_length(i_right, 1), v_len_mirror_dim, array_length(i_right, 3), array_length(i_right, 4)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][v_cur : v_cur][ : ][ : ] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ][ : ];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dim: %;  len_2: %;', array_dims(i_right),  v_len_mirror_dim;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_mirror_x
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- -- select sm_sc.__fv_mirror_x
-- --   (
-- --     array[1, 2, 3, 4]
-- --   );
-- select sm_sc.__fv_mirror_x
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.__fv_mirror_x
--   (
--     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]
--   );
-- select sm_sc.__fv_mirror_x
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_mirror_x
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_mirror_x3(anyarray);
create or replace function sm_sc.__fv_mirror_x3
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur               int; 
  -- v_ele              alias for $0;
  v_ret               i_right%type;
  v_len_mirror_dim    int  :=   array_length(i_right, 3);
begin
  -- set search_path to sm_sc;
  
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 1
  -- -- then
  -- --   -- -- for v_cur in 1 .. array_length(i_right, 1) / 2
  -- --   -- -- loop
  -- --   -- --   v_ele[1] := i_right[v_cur];
  -- --   -- --   i_right[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   -- --   i_right[array_length(i_right, 1) - v_cur + 1] := v_ele[1];
  -- --   -- -- end loop;
  -- --   -- -- return i_right;
  -- --   v_ret := array_fill(nullif(i_right[1], i_right[1]), array[array_length(i_right, 1)]);
  -- --   for v_cur in 1 .. array_length(i_right, 1)
  -- --   loop 
  -- --     v_ret[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;

  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- -- return 	
  -- --   -- -- (
  -- --   -- --   select
  -- --   -- --     array_agg(array_x_new order by a_cur_y)
  -- --   -- --   from 
  -- --   -- --   (
  -- --   -- --     select 
  -- --   -- --       a_cur_y,
  -- --   -- --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_x desc) as array_x_new
  -- --   -- --     from generate_series(1, array_length(i_right, 1)) tb_a_cur_y(a_cur_y)
  -- --   -- --       , generate_series(1, v_len_mirror_dim) tb_a_cur_x(a_cur_x)
  -- --   -- --     group by a_cur_y
  -- --   -- --   ) t_array_x_new
  -- --   -- -- )
  -- --   -- -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[array_length(i_right, 1), array_length(i_right, 2)]);
  -- --   for v_cur in 1 .. v_len_mirror_dim
  -- --   loop 
  -- --     v_ret[ : ][v_cur : v_cur] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][ : ][v_cur : v_cur] := i_right[ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), v_len_mirror_dim, array_length(i_right, 4)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][ : ][v_cur : v_cur][ : ] := i_right[ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dim: %;  len_2: %;', array_dims(i_right),  v_len_mirror_dim;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_mirror_x3
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_mirror_x3
-- --   (
-- --     array[1, 2, 3, 4]
-- --   );
-- -- select sm_sc.__fv_mirror_x3
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_mirror_x3
-- --   (
-- --     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]
-- --   );
-- select sm_sc.__fv_mirror_x3
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_mirror_x3
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_mirror_x4(anyarray);
create or replace function sm_sc.__fv_mirror_x4
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur               int; 
  -- v_ele              alias for $0;
  v_ret               i_right%type;
  v_len_mirror_dim    int  :=   array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
  
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 1
  -- -- then
  -- --   -- -- for v_cur in 1 .. array_length(i_right, 1) / 2
  -- --   -- -- loop
  -- --   -- --   v_ele[1] := i_right[v_cur];
  -- --   -- --   i_right[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   -- --   i_right[array_length(i_right, 1) - v_cur + 1] := v_ele[1];
  -- --   -- -- end loop;
  -- --   -- -- return i_right;
  -- --   v_ret := array_fill(nullif(i_right[1], i_right[1]), array[array_length(i_right, 1)]);
  -- --   for v_cur in 1 .. array_length(i_right, 1)
  -- --   loop 
  -- --     v_ret[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;

  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- -- return 	
  -- --   -- -- (
  -- --   -- --   select
  -- --   -- --     array_agg(array_x_new order by a_cur_y)
  -- --   -- --   from 
  -- --   -- --   (
  -- --   -- --     select 
  -- --   -- --       a_cur_y,
  -- --   -- --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_x desc) as array_x_new
  -- --   -- --     from generate_series(1, array_length(i_right, 1)) tb_a_cur_y(a_cur_y)
  -- --   -- --       , generate_series(1, v_len_mirror_dim) tb_a_cur_x(a_cur_x)
  -- --   -- --     group by a_cur_y
  -- --   -- --   ) t_array_x_new
  -- --   -- -- )
  -- --   -- -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[array_length(i_right, 1), array_length(i_right, 2)]);
  -- --   for v_cur in 1 .. v_len_mirror_dim
  -- --   loop 
  -- --     v_ret[ : ][v_cur : v_cur] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), v_len_mirror_dim]);
  -- --   for v_cur in 1 .. v_len_mirror_dim
  -- --   loop 
  -- --     v_ret[ : ][ : ][v_cur : v_cur] := i_right[ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][ : ][ : ][v_cur : v_cur] := i_right[ : ][ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dim: %;  len_2: %;', array_dims(i_right),  v_len_mirror_dim;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_mirror_x4
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_mirror_x4
-- --   (
-- --     array[1, 2, 3, 4]
-- --   );
-- -- select sm_sc.__fv_mirror_x4
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_mirror_x4
-- --   (
-- --     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]
-- --   );
-- -- select sm_sc.__fv_mirror_x4
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_mirror_x4
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_mirror_y(anyarray);
create or replace function sm_sc.__fv_mirror_y
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur               int; 
  -- v_ele              alias for $0;
  v_ret               i_right%type;
  v_len_mirror_dim    int  :=   array_length(i_right, 1);
begin
  -- set search_path to sm_sc;
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 1
  then
    -- -- for v_cur in 1 .. v_len_mirror_dim / 2
    -- -- loop
    -- --   v_ele[1] := i_right[v_cur];
    -- --   i_right[v_cur] := i_right[v_len_mirror_dim - v_cur + 1];
    -- --   i_right[v_len_mirror_dim - v_cur + 1] := v_ele[1];
    -- -- end loop;
    -- -- return i_right;
    v_ret := array_fill(nullif(i_right[1], i_right[1]), array[v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[v_cur] := i_right[v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 2
  then
    -- -- return 	
    -- -- (
    -- --   select
    -- --     array_agg(array(select unnest(i_right[a_cur_y : a_cur_y][ : ])) order by a_cur_y desc)
    -- --   from generate_series(1, v_len_mirror_dim) tb_a_cur_y(a_cur_y)
    -- -- )
    -- -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_mirror_dim, array_length(i_right, 2)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[v_cur : v_cur][ : ] := i_right[v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_mirror_dim, array_length(i_right, 2), array_length(i_right, 3)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[v_cur : v_cur][ : ][ : ] := i_right[v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1][1][1]), array[v_len_mirror_dim, array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[v_cur : v_cur][ : ][ : ][ : ] := i_right[v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ][ : ][ : ];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %; len_1: %;', array_dims(i_right), v_len_mirror_dim;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_mirror_y
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.__fv_mirror_y
--   (
--     array[1, 2, 3, 4]
--   );
-- select sm_sc.__fv_mirror_y
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.__fv_mirror_y
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_mirror_y
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x3_x4_180(anyarray);
create or replace function sm_sc.__fv_turn_x3_x4_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x
  -- --     loop 
  -- --       v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
  -- --            [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
  -- --         := i_right[v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x3
  -- --     loop 
  -- --       v_ret[ : ]
  -- --            [v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1]
  -- --            [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1] 
  -- --         := i_right[ : ]
  -- --                   [v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[ : ]
             [ : ] 
             [v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1]
             [v_len_x4 - v_cur_2 + 1 : v_len_x4 - v_cur_2 + 1]
          := i_right[ : ]
                    [ : ]
                    [v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x3_x4_180
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x3_x4_180
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x3_x4_180
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x3_x4_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x3_x4_90(anyarray);
create or replace function sm_sc.__fv_turn_x3_x4_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x desc)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x4, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[ : ][ : ][v_cur_2 : v_cur_2][v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1] := i_right[ : ][ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x3_x4_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x3_x4_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x3_x4_90
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x3_x4_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x3_x_90(anyarray);
create or replace function sm_sc.__fv_turn_x3_x_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x3, v_len_x]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[ : ][v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][v_cur_2 : v_cur_2] := i_right[ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x3, v_len_x, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[ : ][v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][v_cur_2 : v_cur_2][ : ] := i_right[ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x3_x_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x3_x_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- select sm_sc.__fv_turn_x3_x_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_x3_x_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x3_y_90(anyarray);
create or replace function sm_sc.__fv_turn_x3_y_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_y
      loop 
        v_ret[v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][ : ][v_cur_2 : v_cur_2] := i_right[v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_x3, v_len_x, v_len_y, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_y
      loop 
        v_ret[v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][ : ][v_cur_2 : v_cur_2][ : ] := i_right[v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1][ : ];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x3_y_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x3_y_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- select sm_sc.__fv_turn_x3_y_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_x3_y_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x4_x3_90(anyarray);
create or replace function sm_sc.__fv_turn_x4_x3_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x3
  -- --   loop 
  -- --     v_ret[v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][ : ][ : ] := i_right[ : ][ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x4, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x4
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ][ : ][v_len_x4 - v_cur_1 + 1 : v_len_x4 - v_cur_1 + 1][v_cur_2 : v_cur_2] := i_right[ : ][ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x4_x3_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x4_x3_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x4_x3_90
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x4_x3_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x4_x_90(anyarray);
create or replace function sm_sc.__fv_turn_x4_x_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x3
  -- --   loop 
  -- --     v_ret[v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][ : ][ : ] := i_right[ : ][ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x4, v_len_x3, v_len_x]);
    for v_cur_1 in 1 .. v_len_x4
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[ : ][v_len_x4 - v_cur_1 + 1 : v_len_x4 - v_cur_1 + 1][ : ][v_cur_2 : v_cur_2] := i_right[ : ][v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x4_x_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x4_x_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x4_x_90
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x4_x_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x4_y_90(anyarray);
create or replace function sm_sc.__fv_turn_x4_y_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_x3
  -- --   loop 
  -- --     v_ret[v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][ : ][ : ] := i_right[ : ][ : ][v_cur_1 : v_cur_1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_x4, v_len_x, v_len_x3, v_len_y]);
    for v_cur_1 in 1 .. v_len_x4
    loop 
      for v_cur_2 in 1 .. v_len_y
      loop 
        v_ret[v_len_x4 - v_cur_1 + 1 : v_len_x4 - v_cur_1 + 1][ : ][ : ][v_cur_2 : v_cur_2] := i_right[v_cur_2 : v_cur_2][ : ][ : ][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x4_y_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x4_y_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x4_y_90
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x4_y_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x_x3_180(anyarray);
create or replace function sm_sc.__fv_turn_x_x3_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x
  -- --     loop 
  -- --       v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
  -- --            [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
  -- --         := i_right[v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ]
             [v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1]
             [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1] 
          := i_right[ : ]
                    [v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ]
             [v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1]
             [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1]
             [ : ] 
          := i_right[ : ]
                    [v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
                    [ : ]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x_x3_180
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x_x3_180
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- select sm_sc.__fv_turn_x_x3_180
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_x_x3_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x_x3_90(anyarray);
create or replace function sm_sc.__fv_turn_x_x3_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x desc)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x3, v_len_x]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ][v_cur_2 : v_cur_2][v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1] := i_right[ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x3, v_len_x, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ][v_cur_2 : v_cur_2][v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x_x3_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x_x3_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- select sm_sc.__fv_turn_x_x3_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_x_x3_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x_x4_180(anyarray);
create or replace function sm_sc.__fv_turn_x_x4_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x
  -- --     loop 
  -- --       v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
  -- --            [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
  -- --         := i_right[v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
  -- --   for v_cur_1 in 1 .. v_len_x
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x3
  -- --     loop 
  -- --       v_ret[ : ]
  -- --            [v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1]
  -- --            [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1] 
  -- --         := i_right[ : ]
  -- --                   [v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[ : ]
             [v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1]
             [ : ] 
             [v_len_x4 - v_cur_2 + 1 : v_len_x4 - v_cur_2 + 1]
          := i_right[ : ]
                    [v_cur_1 : v_cur_1]
                    [ : ]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x_x4_180
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x_x4_180
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x_x4_180
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x_x4_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x_x4_90(anyarray);
create or replace function sm_sc.__fv_turn_x_x4_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x desc)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x4, v_len_x3, v_len_x]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[ : ][v_cur_2 : v_cur_2][ : ][v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1] := i_right[ : ][v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_x_x4_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_x_x4_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_x_x4_90
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_x_x4_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_x_y_90(anyarray);
create or replace function sm_sc.__fv_turn_x_y_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 2
  then
    -- return 	
    -- (
    --   select
    --     array_agg(array_x_new order by a_cur_x)
    --   from 
    --   (
    --     select 
    --       a_cur_x,
    --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
    --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
    --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
    --     group by a_cur_x
    --   ) t_array_x_new
    -- )
    -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x, v_len_y, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_y
      loop 
        v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][v_cur_2 : v_cur_2][ : ] := i_right[v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_x, v_len_y, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_y
      loop 
        v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][v_cur_2 : v_cur_2][ : ][ : ] := i_right[v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ][ : ];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_turn_x_y_90
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.__fv_turn_x_y_90
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.__fv_turn_x_y_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_x_y_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_y_x3_180(anyarray);
create or replace function sm_sc.__fv_turn_y_x3_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x
  -- --     loop 
  -- --       v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
  -- --            [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
  -- --         := i_right[v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [ : ]
             [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1] 
          := i_right[v_cur_1 : v_cur_1]
                    [ : ]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [ : ]
             [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1]
             [ : ] 
          := i_right[v_cur_1 : v_cur_1]
                    [ : ]
                    [v_cur_2 : v_cur_2]
                    [ : ]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_y_x3_180
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_y_x3_180
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- select sm_sc.__fv_turn_y_x3_180
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_y_x3_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_y_x3_90(anyarray);
create or replace function sm_sc.__fv_turn_y_x3_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x desc)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[v_cur_2 : v_cur_2][ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_x3, v_len_x, v_len_y, v_len_x4]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[v_cur_2 : v_cur_2][ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1][ : ] := i_right[v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2][ : ];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_y_x3_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_y_x3_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- select sm_sc.__fv_turn_y_x3_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_y_x3_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_y_x4_180(anyarray);
create or replace function sm_sc.__fv_turn_y_x4_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x
  -- --     loop 
  -- --       v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
  -- --            [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
  -- --         := i_right[v_cur_1 : v_cur_1]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     for v_cur_2 in 1 .. v_len_x3
  -- --     loop 
  -- --       v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1][ : ]
  -- --            [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
  -- --         := i_right[v_cur_1 : v_cur_1][ : ]
  -- --                   [v_cur_2 : v_cur_2]
  -- --       ;
  -- --     end loop;
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [ : ]
             [ : ]
             [v_len_x4 - v_cur_2 + 1 : v_len_x4 - v_cur_2 + 1] 
          := i_right[v_cur_1 : v_cur_1]
                    [ : ]
                    [ : ]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_y_x4_180
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_y_x4_180
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_y_x4_180
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_y_x4_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_y_x4_90(anyarray);
create or replace function sm_sc.__fv_turn_y_x4_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 2
  -- -- then
  -- --   -- return 	
  -- --   -- (
  -- --   --   select
  -- --   --     array_agg(array_x_new order by a_cur_x desc)
  -- --   --   from 
  -- --   --   (
  -- --   --     select 
  -- --   --       a_cur_x,
  -- --   --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
  -- --   --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
  -- --   --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
  -- --   --     group by a_cur_x
  -- --   --   ) t_array_x_new
  -- --   -- )
  -- --   -- ;
  -- --   v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  -- -- elsif array_ndims(i_right) = 3
  -- -- then
  -- --   v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x3, v_len_x, v_len_y]);
  -- --   for v_cur_1 in 1 .. v_len_y
  -- --   loop 
  -- --     v_ret[ : ][ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ][ : ];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_x4, v_len_x, v_len_x3, v_len_y]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[v_cur_2 : v_cur_2][ : ][ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ][ : ][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.__fv_turn_y_x4_90
-- --   (
-- --     array[array[1, 2], array[3, 4]]
-- --   );
-- -- select sm_sc.__fv_turn_y_x4_90
-- --   (
-- --     array[array[1, 2], array[3, 4], array[5, 6]]
-- --   );
-- -- select sm_sc.__fv_turn_y_x4_90
-- --   (
-- --     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
-- --   );
-- select sm_sc.__fv_turn_y_x4_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_y_x_180(anyarray);
create or replace function sm_sc.__fv_turn_y_x_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 2
  then
    -- return 	
    -- (
    --   select
    --     array_agg(array_x_new order by a_cur_x)
    --   from 
    --   (
    --     select 
    --       a_cur_x,
    --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
    --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
    --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
    --     group by a_cur_x
    --   ) t_array_x_new
    -- )
    -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
          := i_right[v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1]
             [ : ] 
          := i_right[v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
                    [ : ]
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1]
             [ : ]
             [ : ] 
          := i_right[v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
                    [ : ]
                    [ : ]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_turn_y_x_180
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.__fv_turn_y_x_180
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.__fv_turn_y_x_180
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_y_x_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_turn_y_x_90(anyarray);
create or replace function sm_sc.__fv_turn_y_x_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 2
  then
    -- return 	
    -- (
    --   select
    --     array_agg(array_x_new order by a_cur_x desc)
    --   from 
    --   (
    --     select 
    --       a_cur_x,
    --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
    --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
    --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
    --     group by a_cur_x
    --   ) t_array_x_new
    -- )
    -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_x, v_len_y, v_len_x3]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[v_cur_2 : v_cur_2][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1][ : ] := i_right[v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_x, v_len_y, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[v_cur_2 : v_cur_2][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1][ : ][ : ] := i_right[v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ][ : ];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_turn_y_x_90
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.__fv_turn_y_x_90
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.__fv_turn_y_x_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.__fv_turn_y_x_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_chunk_prod_mx(anyarray, anyarray, int[3]);
create or replace function sm_sc.fv_chunk_prod_mx
(
  i_left         anyarray    ,
  i_right        anyarray    ,
  i_chunk_len    int[3]      -- 三个配置分别是：i_chunk_len[1]: 分块矩阵乘法入参一的高；i_chunk_len[2]: 分块矩阵乘法入参一的宽，也即分块矩阵乘法入参二的高；i_chunk_len[3]: 分块矩阵乘法入参二的宽；
)
returns anyarray
as
$$
declare 
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计二维长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_length(i_left, array_ndims(i_left) - 1) % i_chunk_len[1] <> 0
      or array_length(i_left, array_ndims(i_left)) % i_chunk_len[2] <> 0
      or array_length(i_right, array_ndims(i_right) - 1) % i_chunk_len[2] <> 0
      or array_length(i_right, array_ndims(i_right)) % i_chunk_len[3] <> 0
    then 
      raise exception 'unmatched length!';
    elsif array_length(i_left, array_ndims(i_left) - 1) / i_chunk_len[1] <> array_length(i_right, array_ndims(i_right) - 1) / i_chunk_len[2]
      or array_length(i_left, array_ndims(i_left)) / i_chunk_len[2] <> array_length(i_right, array_ndims(i_right)) / i_chunk_len[3]
    then 
      raise exception 'unperfect length for groups of chunk between i_left and i_right';
    end if;
  end if;

  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    -- -- v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    -- -- 
    -- -- -- 对齐维长
    -- -- if v_len_left <> v_len_depdt
    -- -- then 
    -- --   i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
    -- --   v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    -- -- end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    -- -- v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    -- -- 
    -- -- -- 对齐维长
    -- -- if v_len_right <> v_len_depdt
    -- -- then 
    -- --   i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
    -- --   v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    -- -- end if;
  end if;

  if array_ndims(i_left) = 2
  then
    return 
      (
        with
        cte_grp_heigh as 
        (
          select 
            a_cur_heigh,
            sm_sc.fa_mx_concat_x
            (
              i_left
                [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
                [i_chunk_len[2] * (a_cur_width - 1) + 1 : i_chunk_len[2] * a_cur_width] 
              |**| 
              i_right
                [i_chunk_len[2] * (a_cur_heigh - 1) + 1 : i_chunk_len[2] * a_cur_heigh]
                [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width] 
              order by a_cur_width
            ) as a_grp_heigh
          from generate_series(1, array_length(i_left, array_ndims(i_left) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
            , generate_series(1, array_length(i_left, array_ndims(i_left)) / i_chunk_len[2]) tb_a_cur_width(a_cur_width)
          group by a_cur_heigh
        )
        select 
          sm_sc.fa_mx_concat_y(a_grp_heigh order by a_cur_heigh)
        from cte_grp_heigh
      )
    ;
    
  elsif array_ndims(i_left) = 3
  then
    return 
      (
        with
        cte_grp_heigh as 
        (
          select 
            a_cur_heigh,
            sm_sc.fa_mx_concat_x3
            (
              i_left
                [ : ]
                [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
                [i_chunk_len[2] * (a_cur_width - 1) + 1 : i_chunk_len[2] * a_cur_width] 
              |**| 
              i_right
                [ : ]
                [i_chunk_len[2] * (a_cur_heigh - 1) + 1 : i_chunk_len[2] * a_cur_heigh]
                [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width] 
              order by a_cur_width
            ) as a_grp_heigh
          from generate_series(1, array_length(i_left, array_ndims(i_left) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
            , generate_series(1, array_length(i_left, array_ndims(i_left)) / i_chunk_len[2]) tb_a_cur_width(a_cur_width)
          group by a_cur_heigh
        )
        select 
          sm_sc.fa_mx_concat_x(a_grp_heigh order by a_cur_heigh)
        from cte_grp_heigh
      )
    ;
    
  elsif array_ndims(i_left) = 4
  then
    return 
      (
        with
        cte_grp_heigh as 
        (
          select 
            a_cur_heigh,
            sm_sc.fa_mx_concat_x4
            (
              i_left
                [ : ]
                [ : ]
                [i_chunk_len[1] * (a_cur_heigh - 1) + 1 : i_chunk_len[1] * a_cur_heigh]
                [i_chunk_len[2] * (a_cur_width - 1) + 1 : i_chunk_len[2] * a_cur_width] 
              |**| 
              i_right
                [ : ]
                [ : ]
                [i_chunk_len[2] * (a_cur_heigh - 1) + 1 : i_chunk_len[2] * a_cur_heigh]
                [i_chunk_len[3] * (a_cur_width - 1) + 1 : i_chunk_len[3] * a_cur_width] 
              order by a_cur_width
            ) as a_grp_heigh
          from generate_series(1, array_length(i_left, array_ndims(i_left) - 1) / i_chunk_len[1]) tb_a_cur_heigh(a_cur_heigh)
            , generate_series(1, array_length(i_left, array_ndims(i_left)) / i_chunk_len[2]) tb_a_cur_width(a_cur_width)
          group by a_cur_heigh
        )
        select 
          sm_sc.fa_mx_concat_x3(a_grp_heigh order by a_cur_heigh)
        from cte_grp_heigh
      )
    ;
    
  -- 审计二维长度
  else
    raise exception 'no method for such length!  L_Dim: %; R_Dim: %;', array_dims(i_left), array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_chunk_prod_mx
--   (
--     array_fill(random(), array[2 * 7, 3 * 11])
--   , array_fill(random(), array[3 * 7, 5 * 11])
--   , array[2, 3, 5]
--   );  -- 期望规格：array[2 * 7, 5 * 11]
-- select sm_sc.fv_chunk_prod_mx
--   (
--     array_fill(random(), array[3, 2 * 7, 3 * 11])
--   , array_fill(random(), array[3, 3 * 7, 5 * 11])
--   , array[2, 3, 5]
--   );  -- 期望规格：array[3, 2 * 7, 5 * 11]
-- select sm_sc.fv_chunk_prod_mx
--   (
--     array_fill(random(), array[2, 3, 2 * 7, 3 * 11])
--   , array_fill(random(), array[2, 3, 3 * 7, 5 * 11])
--   , array[2, 3, 5]
--   );  -- 期望规格：array[2, 3, 2 * 7, 5 * 11]
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_chunk_transpose(anyarray, int[2]);
create or replace function sm_sc.fv_chunk_transpose
(
  i_right        anyarray
, i_chunk_len    int[2]
)
returns anyarray
as
$$
declare 
  v_ret             i_right%type ;
  v_cur_heigh       int          ;
  v_cur_width       int          ;
begin
  -- set search_path to sm_sc;
  -- 审计二维长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_right) not between 2 and 4
    then
      raise exception 'unsupport ndims!';
    elsif array_length(i_right, array_ndims(i_right) - 1) % i_chunk_len[1] <> 0
      or array_length(i_right, array_ndims(i_right)) % i_chunk_len[2] <> 0
    then 
      raise exception 'unmatch array_len for chunk_len!';
    end if;
  end if;
  
  
    
  if array_ndims(i_right) is null
  then 
    return i_right;
    
  elsif array_ndims(i_right) = 2
  then
    v_ret  :=   
      array_fill
      (
        v_ret[0]
      , array
        [
          array_length(i_right, 1) / i_chunk_len[1] * i_chunk_len[2]
        , array_length(i_right, 2) / i_chunk_len[2] * i_chunk_len[1]
        ]
      );
    for v_cur_heigh in 1 .. array_length(i_right, array_ndims(i_right) - 1) / i_chunk_len[1]
    loop 
      for v_cur_width in 1 .. array_length(i_right, array_ndims(i_right)) / i_chunk_len[2]
      loop 
        v_ret
          [(v_cur_heigh - 1) * i_chunk_len[2] + 1 : v_cur_heigh * i_chunk_len[2]]
          [(v_cur_width - 1) * i_chunk_len[1] + 1 : v_cur_width * i_chunk_len[1]]   
        :=  
          |^~| 
          (
            i_right
              [(v_cur_heigh - 1) * i_chunk_len[1] + 1 : v_cur_heigh * i_chunk_len[1]]
              [(v_cur_width - 1) * i_chunk_len[2] + 1 : v_cur_width * i_chunk_len[2]]
          )
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret  :=   
      array_fill
      (
        v_ret[0]
      , array
        [
          array_length(i_right, 1)
        , array_length(i_right, 2) / i_chunk_len[1] * i_chunk_len[2]
        , array_length(i_right, 3) / i_chunk_len[2] * i_chunk_len[1]
        ]
      );
    for v_cur_heigh in 1 .. array_length(i_right, array_ndims(i_right) - 1) / i_chunk_len[1]
    loop 
      for v_cur_width in 1 .. array_length(i_right, array_ndims(i_right)) / i_chunk_len[2]
      loop 
        v_ret
          [ : ]
          [(v_cur_heigh - 1) * i_chunk_len[2] + 1 : v_cur_heigh * i_chunk_len[2]]
          [(v_cur_width - 1) * i_chunk_len[1] + 1 : v_cur_width * i_chunk_len[1]]   
        :=  
          |^~| 
          (
            i_right
              [ : ]
              [(v_cur_heigh - 1) * i_chunk_len[1] + 1 : v_cur_heigh * i_chunk_len[1]]
              [(v_cur_width - 1) * i_chunk_len[2] + 1 : v_cur_width * i_chunk_len[2]]
          )
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret  :=   
      array_fill
      (
        v_ret[0]
      , array
        [
          array_length(i_right, 1)
        , array_length(i_right, 2)
        , array_length(i_right, 3) / i_chunk_len[1] * i_chunk_len[2]
        , array_length(i_right, 4) / i_chunk_len[2] * i_chunk_len[1]
        ]
      );
    for v_cur_heigh in 1 .. array_length(i_right, array_ndims(i_right) - 1) / i_chunk_len[1]
    loop 
      for v_cur_width in 1 .. array_length(i_right, array_ndims(i_right)) / i_chunk_len[2]
      loop 
        v_ret
          [ : ]
          [ : ]
          [(v_cur_heigh - 1) * i_chunk_len[2] + 1 : v_cur_heigh * i_chunk_len[2]]
          [(v_cur_width - 1) * i_chunk_len[1] + 1 : v_cur_width * i_chunk_len[1]]   
        :=  
          |^~| 
          (
            i_right
              [ : ]
              [ : ]
              [(v_cur_heigh - 1) * i_chunk_len[1] + 1 : v_cur_heigh * i_chunk_len[1]]
              [(v_cur_width - 1) * i_chunk_len[2] + 1 : v_cur_width * i_chunk_len[2]]
          )
        ;
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_chunk_transpose
--   (
--     sm_sc.fv_new_rand(array[6, 15])
--   , array[3, 5]
--   );
-- select sm_sc.fv_chunk_transpose
--   (
--     sm_sc.fv_new_rand(array[2, 6, 15])
--   , array[3, 5]
--   );
-- select sm_sc.fv_chunk_transpose
--   (
--     sm_sc.fv_new_rand(array[3, 2, 6, 15])
--   , array[3, 5]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_concat_py(float[], float[], int);
create or replace function sm_sc.fv_concat_py
(
  i_left      float[]
, i_right     float[]
, i_axis      int
)
returns float[]
as
$$
  import numpy as np
  # np.concat unsupport array_length(, ) == 0
  v_left = np.array(i_left)
  v_right = np.array(i_right)
  if v_left.size == 0 :
    return i_right
  elif v_right.size == 0 :
    return i_left
  else :
    return np.concat((v_left, v_right), axis = - max(v_left.ndim, v_right.ndim) + i_axis - 1).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_concat_py
--   (
--     array[[1, 2], [3, 4]]
--   , array[[1, 2], [3, 4]]
--   , 2
--   );
-- select sm_sc.fv_concat_py
--   (
--     array[1, 2, 3, 4]
--   , array[1, 2, 3, 4]
--   , 1
--   );
-- select sm_sc.fv_concat_py
--   (
--     array[[1, 2], [3, 4], [5, 6]]
--   , array[[1, 2], [3, 4], [5, 6]]
--   , 2
--   );
-- select sm_sc.fv_concat_py
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   , array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   , 0
--   );
-- select sm_sc.fv_concat_py
--   (
--     sm_sc.fv_new_rand(array[2,3,5,4,2])
--   , sm_sc.fv_new_rand(array[2,3,5,4,3])
--   , 1
--   );
-- -- select sm_sc.fv_concat_py
-- --   (
-- --     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
-- --   , array[[[[]]]] :: float[]
-- --   , 1
-- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_concat_x(anyarray, anyarray);
create or replace function sm_sc.fv_concat_x
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_left%type;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_left) <> array_ndims(i_right)
      or 
        array_ndims(i_left) = 3 and array_ndims(i_right) = 3
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 3) <> array_length(i_right, 3)
        )
      or 
        array_ndims(i_left) = 4 and array_ndims(i_right) = 4
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 3) <> array_length(i_right, 3)
          or array_length(i_left, 4) <> array_length(i_right, 4)
        )
      or 
        array_ndims(i_left) = 5 and array_ndims(i_right) = 5
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 3) <> array_length(i_right, 3)
          or array_length(i_left, 4) <> array_length(i_right, 4)
          or array_length(i_left, 5) <> array_length(i_right, 5)
        )
      or array_ndims(i_left) = 2 and array_ndims(i_right) = 2
        and array_length(i_left, 1) <> 1
        and array_length(i_right, 1) <> 1
        and array_length(i_left, 1) <> array_length(i_right, 1)
      or array_ndims(i_left) = 1 and array_ndims(i_right) = 1
    then
      raise exception 'unmatched length!';     
    end if;   
  end if;

  -- 对齐维长
  if (v_len_left[ : 1] || v_len_left[3 : ]) <> (v_len_right[ : 1] || v_len_right[3 : ]) and array_ndims(i_left) <> 1 and array_ndims(i_right) <> 1
  then 
    i_left := sm_sc.fv_new(i_left, (1 @>` (v_len_right[ : 1] / v_len_left[ : 1])) || array[1] || (1 @>` (v_len_right[3 : ] / v_len_left[3 : ])));
    i_right := sm_sc.fv_new(i_right, (1 @>` (v_len_left[ : 1] / v_len_right[ : 1])) || array[1] || (1 @>` (v_len_left[3 : ] / v_len_right[3 : ])));
  end if;
  
  if array_ndims(i_left) is null
  then 
    return i_right;
  elsif array_ndims(i_right) is null
  then
    return i_left;
  -- 同高度 [][] ||| [][] 
  elsif array_ndims(i_left) = 2 and array_ndims(i_right) = 2
  then
    if array_length(i_left, 1) = array_length(i_right, 1)
    then
      -- -- return (select array_agg(array(select unnest(i_left[a_cur:a_cur][:])) || array(select unnest(i_right[a_cur:a_cur][:])) order by a_cur) from generate_series(1, array_length(i_left, 1)) tb_a_cur(a_cur));
      v_ret := array_fill(nullif(i_left[1][1], i_left[1][1]), array[array_length(i_left, 1), array_length(i_left, 2) + array_length(i_right, 2)]);
      v_ret[ : ][ : array_length(i_left, 2)] := i_left;
      v_ret[ : ][array_length(i_left, 2) + 1 : ] := i_right;
      return v_ret;
    elsif array_length(i_left, 1) = 1
    then
      -- -- return (select array_agg(array(select unnest(i_left[1:1][:])) || array(select unnest(i_right[a_cur:a_cur][:])) order by a_cur) from generate_series(1, array_length(i_right, 1)) tb_a_cur(a_cur));
      v_ret := array_fill(nullif(i_left[1][1], i_left[1][1]), array[array_length(i_right, 1), array_length(i_left, 2) + array_length(i_right, 2)]);
      v_ret[ : ][ : array_length(i_left, 2)] := (select sm_sc.fa_mx_concat_y(i_left) from generate_series(1, array_length(i_right, 1)));
      v_ret[ : ][array_length(i_left, 2) + 1 : ] := i_right;
      return v_ret;
    elsif array_length(i_right, 1) = 1
    then
      -- -- return (select array_agg(array(select unnest(i_left[a_cur:a_cur][:])) || array(select unnest(i_right[1:1][:])) order by a_cur) from generate_series(1, array_length(i_left, 1)) tb_a_cur(a_cur));
      v_ret := array_fill(nullif(i_left[1][1], i_left[1][1]), array[array_length(i_left, 1), array_length(i_left, 2) + array_length(i_right, 2)]);
      v_ret[ : ][ : array_length(i_left, 2)] := i_left;
      v_ret[ : ][array_length(i_left, 2) + 1 : ] := (select sm_sc.fa_mx_concat_y(i_right) from generate_series(1, array_length(i_left, 1)));
      return v_ret;
    end if;
  elsif array_ndims(i_left) = 1 and array_ndims(i_right) = 2
  then
    -- -- return (select array_agg(i_left || array(select unnest(i_right[a_cur:a_cur][:])) order by a_cur) from generate_series(1, array_length(i_right, 1)) tb_a_cur(a_cur));
    v_ret := array_fill(nullif(i_right[1][1], i_left[1][1]), array[array_length(i_right, 1), array_length(i_left, 1) + array_length(i_right, 2)]);
    v_ret[ : ][ : array_length(i_left, 2)] := (select array_agg(i_left) from generate_series(1, array_length(i_right, 1)));
    v_ret[ : ][array_length(i_left, 2) + 1 : ] := i_right;
    return v_ret;
  elsif array_ndims(i_right) = 1 and array_ndims(i_left) = 2
  then
    -- -- return (select array_agg(array(select unnest(i_left[a_cur:a_cur][:])) || i_right order by a_cur) from generate_series(1, array_length(i_left, 1)) tb_a_cur(a_cur));
    v_ret := array_fill(nullif(i_left[1][1], i_left[1][1]), array[array_length(i_left, 1), array_length(i_left, 2) + array_length(i_right, 1)]);
    v_ret[ : ][ : array_length(i_left, 2)] := i_left;
    v_ret[ : ][array_length(i_left, 2) + 1 : ] := (select array_agg(i_right) from generate_series(1, array_length(i_left, 1)));
    return v_ret;
  elsif i_left is null
  then
    return i_right;
  elsif i_right is null
  then
    return i_left;
    
  elsif array_ndims(i_left) = 3 and array_ndims(i_right) = 3
  then 
    v_ret := array_fill(nullif(i_left[1][1][1],i_left[1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2) + array_length(i_right, 2), array_length(i_left, 3)]);
    v_ret[ : ][ : array_length(i_left, 2)][ : ] := i_left;
    v_ret[ : ][array_length(i_left, 2) + 1 : ][ : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_left) = 4 and array_ndims(i_right) = 4
  then 
    v_ret := array_fill(nullif(i_left[1][1][1][1],i_left[1][1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2) + array_length(i_right, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    v_ret[ : ][ : array_length(i_left, 2)][ : ][ : ] := i_left;
    v_ret[ : ][array_length(i_left, 2) + 1 : ][ : ][ : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_left) = 5 and array_ndims(i_right) = 5
  then 
    v_ret := array_fill(nullif(i_left[1][1][1][1][1],i_left[1][1][1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2) + array_length(i_right, 2), array_length(i_left, 3), array_length(i_left, 4), array_length(i_left, 5)]);
    v_ret[ : ][ : array_length(i_left, 2)][ : ][ : ][ : ] := i_left;
    v_ret[ : ][array_length(i_left, 2) + 1 : ][ : ][ : ][ : ] := i_right;
    return v_ret;
    
  else
    raise exception 'no method for such length!  L_Dim: %; R_Dim: %;', array_dims(i_left), array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_concat_x
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3, 55.5, -6.66]]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[-12.3, 12.3, 55.5, -6.66]],
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[-12.3, 12.3, 55.5, -6.66]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[-12.3, 12.3, 55.5, -6.66],
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[], array[]]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[],
--     array[array[], array[]]::float[]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[]]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_x
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[],
--     array[array[]]::float[]
--   );
-- select 
--   sm_sc.fv_concat_x
--   (
--     array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]]
--   , array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   )
-- select 
--   sm_sc.fv_concat_x
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]]
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]],[[[-21,-22,-23,-24],[-31,-32,-33,-34],[-121,-122,-123,-124]],[[-25,-26,-27,-28],[-35,-36,-37,-38],[-125,-126,-127,-128]]]]
--   )
-- select 
--   sm_sc.fv_concat_x
--   (
--     array[sm_sc.fv_new_rand(array[2,3,4,5])]
--   , array[sm_sc.fv_new_rand(array[3,3,4,5])]
--   )
-- --------------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_x(anyarray, anyelement);
create or replace function sm_sc.fv_concat_x
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare 
  v_ret   i_left%type  ;
begin
  if array_ndims(i_left) > 5 or array_ndims(i_left) < 2
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
    
  elsif array_ndims(i_left) = 2
  then 
    -- return (select array_agg(array(select unnest(i_left[a_cur:a_cur][:])) || i_right order by a_cur) from generate_series(1, array_length(i_left, 1)) tb_a_cur(a_cur));
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2) + 1]);
    v_ret[ : ][ : array_length(i_left, 2)] := i_left;
    return v_ret;
    
  elsif array_ndims(i_left) = 3
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2) + 1, array_length(i_left, 3)]);
    v_ret[ : ][ : array_length(i_left, 2)][ : ] := i_left;
    return v_ret;
    
  elsif array_ndims(i_left) = 4
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2) + 1, array_length(i_left, 3), array_length(i_left, 4)]);
    v_ret[ : ][ : array_length(i_left, 2)][ : ][ : ] := i_left;
    return v_ret;
    
  elsif array_ndims(i_left) = 5
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2) + 1, array_length(i_left, 3), array_length(i_left, 4), array_length(i_left, 5)]);
    v_ret[ : ][ : array_length(i_left, 2)][ : ][ : ][ : ] := i_left;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_concat_x(array[array[12.3, 25.1], array[2.56, 3.25]], 2.8)
-- select 
--   sm_sc.fv_concat_x
--   (
--     array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]] :: float[]
--   , 5.33 :: float
--   )
-- select 
--   sm_sc.fv_concat_x
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]] :: float[]
--   , 5.33 :: float
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_x(anyelement, anyarray);
create or replace function sm_sc.fv_concat_x
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_ret   i_right%type  ;
begin
  if array_ndims(i_right) > 5 or array_ndims(i_right) < 2
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  elsif array_ndims(i_right) = 2
  then
    -- return (select array_agg(i_left || array(select unnest(i_right[a_cur:a_cur][:])) order by a_cur) from generate_series(1, array_length(i_right, 1)) tb_a_cur(a_cur));
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2) + 1]);
    v_ret[ : ][2 : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2) + 1, array_length(i_right, 3)]);
    v_ret[ : ][2 : ][ : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2) + 1, array_length(i_right, 3), array_length(i_right, 4)]);
    v_ret[ : ][2 : ][ : ][ : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_right) = 5
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2) + 1, array_length(i_right, 3), array_length(i_right, 4), array_length(i_right, 5)]);
    v_ret[ : ][2 : ][ : ][ : ][ : ] := i_right;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_concat_x(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select 
--   sm_sc.fv_concat_x
--   (
--     5.33 :: float
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]] :: float[]
--   )
-- select 
--   sm_sc.fv_concat_x
--   (
--    5.33 :: float
--   , array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]] :: float[]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_concat_x3(anyarray, anyarray);
create or replace function sm_sc.fv_concat_x3
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_left%type;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  
begin
  -- set search_path to sm_sc;
  -- 审计二维长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_left) <> array_ndims(i_right)
      or 
        array_ndims(i_left) = 3 and array_ndims(i_right) = 3
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 2) <> array_length(i_right, 2)
        )
      or 
        array_ndims(i_left) = 4 and array_ndims(i_right) = 4
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 2) <> array_length(i_right, 2)
          or array_length(i_left, 4) <> array_length(i_right, 4)
        )
      or 
        array_ndims(i_left) = 5 and array_ndims(i_right) = 5
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 2) <> array_length(i_right, 2)
          or array_length(i_left, 4) <> array_length(i_right, 4)
          or array_length(i_left, 5) <> array_length(i_right, 5)
        )
    then
      raise exception 'unmatched length!';
    end if;
  end if;

  -- 对齐维长
  if (v_len_left[ : 2] || v_len_left[4 : ]) <> (v_len_right[ : 2] || v_len_right[4 : ])
  then 
    i_left := sm_sc.fv_new(i_left, (1 @>` (v_len_right[ : 2] / v_len_left[ : 2])) || array[1] || (1 @>` (v_len_right[4 : ] / v_len_left[4 : ])));
    i_right := sm_sc.fv_new(i_right, (1 @>` (v_len_left[ : 2] / v_len_right[ : 2])) || array[1] || (1 @>` (v_len_left[4 : ] / v_len_right[4 : ])));
  end if;

  if array_ndims(i_left) is null
  then 
    return i_right;
  elsif array_ndims(i_right) is null
  then
    return i_left;
    
  elsif array_ndims(i_left) = 3 and array_ndims(i_right) = 3
  then 
    v_ret := array_fill(nullif(i_left[1][1][1],i_left[1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3) + array_length(i_right, 3)]);
    v_ret[ : ][ : ][ : array_length(i_left, 3)] := i_left;
    v_ret[ : ][ : ][array_length(i_left, 3) + 1 : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_left) = 4 and array_ndims(i_right) = 4
  then 
    v_ret := array_fill(nullif(i_left[1][1][1][1],i_left[1][1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3) + array_length(i_right, 3), array_length(i_left, 4)]);
    v_ret[ : ][ : ][ : array_length(i_left, 3)][ : ] := i_left;
    v_ret[ : ][ : ][array_length(i_left, 3) + 1 : ][ : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_left) = 5 and array_ndims(i_right) = 5
  then 
    v_ret := array_fill(nullif(i_left[1][1][1][1][1],i_left[1][1][1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3) + array_length(i_right, 3), array_length(i_left, 4), array_length(i_left, 5)]);
    v_ret[ : ][ : ][ : array_length(i_left, 3)][ : ][ : ] := i_left;
    v_ret[ : ][ : ][array_length(i_left, 3) + 1 : ][ : ][ : ] := i_right;
    return v_ret;
    
  -- 审计二维长度
  else
    raise exception 'no method for such length!  L_Dim: %; R_Dim: %;', array_dims(i_left), array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select 
--   sm_sc.fv_concat_x3
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   , array[[[1,2,3],[11,12,13],[111,112,113]],[[5,6,7],[15,16,17],[115,116,117]]]
--   )
-- select 
--   sm_sc.fv_concat_x3
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]],[[[-21,-22,-23,-24],[-31,-32,-33,-34],[-121,-122,-123,-124]],[[-25,-26,-27,-28],[-35,-36,-37,-38],[-125,-126,-127,-128]]]]
--   )
-- select 
--   sm_sc.fv_concat_x3
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5,6])
--   , sm_sc.fv_new_rand(array[2,3,6,5,6])
--   )
-- --------------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_x3(anyarray, anyelement);
create or replace function sm_sc.fv_concat_x3
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare 
  v_ret   i_left%type  ;
begin
  if array_ndims(i_left) > 5 or array_ndims(i_left) < 3
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
    
  elsif array_ndims(i_left) = 3
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3) + 1]);
    v_ret[ : ][ : ][ : array_length(i_left, 3)] := i_left;
    return v_ret;
    
  elsif array_ndims(i_left) = 4
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3) + 1, array_length(i_left, 4)]);
    v_ret[ : ][ : ][ : array_length(i_left, 3)][ : ] := i_left;
    return v_ret;
    
  elsif array_ndims(i_left) = 5
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3) + 1, array_length(i_left, 4), array_length(i_left, 5)]);
    v_ret[ : ][ : ][ : array_length(i_left, 3)][ : ][ : ] := i_left;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- select sm_sc.fv_concat_x3(array[array[12.3, 25.1], array[2.56, 3.25]], 2.8)
-- select 
--   sm_sc.fv_concat_x3
--   (
--     array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]] :: float[]
--   , 5.33 :: float
--   )
-- select 
--   sm_sc.fv_concat_x3
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]] :: float[]
--   , 5.33 :: float
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_x3(anyelement, anyarray);
create or replace function sm_sc.fv_concat_x3
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_ret   i_right%type  ;
begin
  if array_ndims(i_right) > 5 or array_ndims(i_right) < 3
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
    
  elsif array_ndims(i_right) = 3
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3) + 1]);
    v_ret[ : ][ : ][2 : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3) + 1, array_length(i_right, 4)]);
    v_ret[ : ][ : ][2 : ][ : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_right) = 5
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3) + 1, array_length(i_right, 4), array_length(i_right, 5)]);
    v_ret[ : ][ : ][2 : ][ : ][ : ] := i_right;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- select sm_sc.fv_concat_x3(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select 
--   sm_sc.fv_concat_x3
--   (
--     5.33 :: float
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]] :: float[]
--   )
-- select 
--   sm_sc.fv_concat_x3
--   (
--    5.33 :: float
--   , array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]] :: float[]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_concat_x4(anyarray, anyarray);
create or replace function sm_sc.fv_concat_x4
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_left%type;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  
begin
  -- set search_path to sm_sc;
  -- 审计二维长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_left) <> array_ndims(i_right)
      or 
        array_ndims(i_left) = 4 and array_ndims(i_right) = 4
        and
        (
          array_length(i_left, 1) <> array_length(i_right, 1)
          or array_length(i_left, 2) <> array_length(i_right, 2)
          or array_length(i_left, 3) <> array_length(i_right, 3)
        )
    then
      raise exception 'unmatched length!';
    end if;
  end if;

  -- 对齐维长
  if v_len_left[ : 3] <> v_len_right[ : 3]
  then 
    i_left := sm_sc.fv_new(i_left, (1 @>` (v_len_right[ : 3] / v_len_left[ : 3])) || array[1]);
    i_right := sm_sc.fv_new(i_right, (1 @>` (v_len_left[ : 3] / v_len_right[ : 3])) || array[1]);
  end if;

  if array_ndims(i_left) is null
  then 
    return i_right;
  elsif array_ndims(i_right) is null
  then
    return i_left;
    
  elsif array_ndims(i_left) = 4 and array_ndims(i_right) = 4
  then 
    v_ret := array_fill(nullif(i_left[1][1][1][1],i_left[1][1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4) + array_length(i_right, 4)]);
    v_ret[ : ][ : ][ : ][ : array_length(i_left, 4)] := i_left;
    v_ret[ : ][ : ][ : ][array_length(i_left, 4) + 1 : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_left) = 5 and array_ndims(i_right) = 5
  then 
    v_ret := array_fill(nullif(i_left[1][1][1][1][1],i_left[1][1][1][1][1]), array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4) + array_length(i_right, 4), array_length(i_left, 5)]);
    v_ret[ : ][ : ][ : ][ : array_length(i_left, 4)][ : ] := i_left;
    v_ret[ : ][ : ][ : ][array_length(i_left, 4) + 1 : ][ : ] := i_right;
    return v_ret;
    
  -- 审计二维长度
  else
    raise exception 'no method for such length!  L_Dim: %; R_Dim: %;', array_dims(i_left), array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select 
--   sm_sc.fv_concat_x4
--   (
--     array[[[[1,2,3],[11,12,13],[111,112,113]],[[5,6,7],[15,16,17],[115,116,117]]],[[[21,22,23],[31,32,33],[131,132,133]],[[25,26,27],[35,36,37],[135,136,137]]]]
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]],[[[-21,-22,-23,-24],[-31,-32,-33,-34],[-121,-122,-123,-124]],[[-25,-26,-27,-28],[-35,-36,-37,-38],[-125,-126,-127,-128]]]]
--   )

-- select 
--   sm_sc.fv_concat_x4
--   (
--     sm_sc.fv_new_rand(array[2,3,4,3,6])
--   , sm_sc.fv_new_rand(array[2,3,4,7,6])
--   )
-- --------------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_x4(anyarray, anyelement);
create or replace function sm_sc.fv_concat_x4
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare 
  v_ret   i_left%type  ;
begin
  if array_ndims(i_left) <4 or array_ndims(i_left) > 5
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
    
  elsif array_ndims(i_left) = 4
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4) + 1]);
    v_ret[ : ][ : ][ : ][ : array_length(i_left, 4)] := i_left;
    return v_ret;
    
  elsif array_ndims(i_left) = 5
  then 
    v_ret := array_fill(i_right, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4) + 1, array_length(i_left, 5)]);
    v_ret[ : ][ : ][ : ][ : array_length(i_left, 4)][ : ] := i_left;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- select sm_sc.fv_concat_x4(array[array[12.3, 25.1], array[2.56, 3.25]], 2.8)
-- -- select 
-- --   sm_sc.fv_concat_x4
-- --   (
-- --     array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]] :: float[]
-- --   , 5.33 :: float
-- --   )
-- select 
--   sm_sc.fv_concat_x4
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]] :: float[]
--   , 5.33 :: float
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_x4(anyelement, anyarray);
create or replace function sm_sc.fv_concat_x4
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_ret   i_right%type  ;
begin
  if array_ndims(i_right) < 4 or array_ndims(i_right) > 5
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
    
  elsif array_ndims(i_right) = 4
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4) + 1]);
    v_ret[ : ][ : ][ : ][2 : ] := i_right;
    return v_ret;
    
  elsif array_ndims(i_right) = 5
  then 
    v_ret := array_fill(i_left, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4) + 1, array_length(i_right, 5)]);
    v_ret[ : ][ : ][ : ][2 : ][ : ] := i_right;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- select sm_sc.fv_concat_x4(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- -- select 
-- --   sm_sc.fv_concat_x4
-- --   (
-- --     5.33 :: float
-- --   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14]],[[-5,-6,-7,-8],[-15,-16,-17,-18]]] :: float[]
-- --   )
-- select 
--   sm_sc.fv_concat_x4
--   (
--    5.33 :: float
--   , array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]]]] :: float[]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_concat_y(anyarray, anyarray);
create or replace function sm_sc.fv_concat_y
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare 
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  
begin
  -- set search_path to sm_sc;
  -- 审计二维长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_left) <> array_ndims(i_right)
      or 
        array_ndims(i_left) = 3 and array_ndims(i_right) = 3
        and
        (
          array_length(i_left, 2) <> array_length(i_right, 2)
          or array_length(i_left, 3) <> array_length(i_right, 3)
        )
      or 
        array_ndims(i_left) = 4 and array_ndims(i_right) = 4
        and
        (
          array_length(i_left, 2) <> array_length(i_right, 2)
          or array_length(i_left, 3) <> array_length(i_right, 3)
          or array_length(i_left, 4) <> array_length(i_right, 4)
        )
      or 
        array_ndims(i_left) = 5 and array_ndims(i_right) = 5
        and
        (
          array_length(i_left, 2) <> array_length(i_right, 2)
          or array_length(i_left, 3) <> array_length(i_right, 3)
          or array_length(i_left, 4) <> array_length(i_right, 4)
          or array_length(i_left, 5) <> array_length(i_right, 5)
        )
      or array_ndims(i_left) = 2 and array_ndims(i_right) = 2
        and array_length(i_left, 2) <> array_length(i_right, 2) 
        and array_length(i_left, 2) <> 1
        and array_length(i_right, 2) <> 1
    then
      raise exception 'unmatched length!';
    end if;
  end if;

  -- 对齐维长
  if v_len_left[2 : ] <> v_len_right[2 : ] and array_ndims(i_left) <> 1 and array_ndims(i_right) <> 1
  then 
    i_left := sm_sc.fv_new(i_left, array[1] || (1 @>` (v_len_right[2 : ] / v_len_left[2 : ])));
    i_right := sm_sc.fv_new(i_right, array[1] || (1 @>` (v_len_left[2 : ] / v_len_right[2 : ])));
  end if;

  if array_ndims(i_left) is null
  then 
    return i_right;
  elsif array_ndims(i_right) is null
  then
    return i_left;
  -- 同宽度 [][] ||| [][] 
  -- 或 纵向单行拼接 [] ||| [][] 
  --              或 [][] ||| []
  elsif array_length(i_left, 2) = array_length(i_right, 2)
    or array_ndims(i_left) = 1 and array_length(i_left, 1) = array_length(i_right, 2) and array_ndims(i_right) = 2
    or array_ndims(i_right) = 1 and array_length(i_right, 1) = array_length(i_left, 2) and array_ndims(i_left) = 2
  then
    return i_left || i_right;
  -- 横向广播，i_left 需要延拓 [][1] ||| [][]
  elsif array_ndims(i_left) = 2 and array_length(i_left, 2) = 1 and array_ndims(i_right) = 2
  then
    return (select array_agg(array_fill(i_left[cur_sn][1], array[array_length(i_right, 2)]) order by cur_sn) from generate_series(1, array_length(i_left, 1)) cur_sn) || i_right;
  -- 横向广播，i_right 需要延拓 [][] ||| [][1]
  elsif array_ndims(i_right) = 2 and array_length(i_right, 2) = 1 and array_ndims(i_left) = 2
  then
    return i_left || (select array_agg(array_fill(i_right[cur_sn][1], array[array_length(i_left, 2)]) order by cur_sn) from generate_series(1, array_length(i_right, 1)) cur_sn);
  elsif i_left is null
  then 
    return i_right;
  elsif i_right is null
  then 
    return i_left;
    
  elsif array_ndims(i_left) = 3 and array_ndims(i_right) = 3
    or array_ndims(i_left) = 4 and array_ndims(i_right) = 4
    or array_ndims(i_left) = 5 and array_ndims(i_right) = 5
    or array_ndims(i_left) = 1 and array_ndims(i_right) = 1
  then 
    return 
      i_left || i_right
    ;
    
  -- 审计二维长度
  else
    raise exception 'no method for such length!  L_Dim: %; R_Dim: %;', array_dims(i_left), array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_concat_y
--   (
--     array[array[12.3, 12.3], array[45.6, 45.6]],
--     array[array[-2.1, 1.6], array[-1.6, 2.1]]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[32.5], array[9.1]],
--     array[array[2.3, 1.3], array[-5.6, 5.6]]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[12.3, 2.3], array[45.6, 45.6]],
--     array[array[3.5], array[-4.0]]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[32.5, 9.1],
--     array[array[2.3, 1.3], array[-5.6, 5.6]]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[12.3, 12.3], array[45.6, 45.6]],
--     array[2.5, -1.1]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[], array[]]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[],
--     array[array[], array[]]::float[]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[]]::float[],
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[]
--   );
-- select sm_sc.fv_concat_y
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]::float[],
--     array[array[]]::float[]
--   );
-- select 
--   sm_sc.fv_concat_y
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   )
-- select 
--   sm_sc.fv_concat_y
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]],[[25,26,27,28],[35,36,37,38],[135,136,137,138]]]]
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_y(anyarray, anyelement);
create or replace function sm_sc.fv_concat_y
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
-- declare 
begin
  if array_ndims(i_left) > 5
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  elsif array_ndims(i_left) = 5 
  then
    return i_left || array_fill(i_right, array[1, array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4), array_length(i_left, 5)]);
  elsif array_ndims(i_left) = 4 
  then
    return i_left || array_fill(i_right, array[1, array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
  elsif array_ndims(i_left) = 3 
  then
    return i_left || array_fill(i_right, array[1, array_length(i_left, 2), array_length(i_left, 3)]);
  elsif array_ndims(i_left) = 2
  then
    return i_left || array_fill(i_right, array[1, array_length(i_left, 2)]);
  elsif array_ndims(i_left) = 1 
  then
    return i_left || array_fill(i_right, array[1]);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_concat_y(array[array[12.3, 25.1], array[2.56, 3.25]], 2.8)
-- select 
--   sm_sc.fv_concat_y
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   , -3
--   )
-- select 
--   sm_sc.fv_concat_y
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]],[[[21,22,23,24],[31,32,33,34],[131,132,133,134]],[[25,26,27,28],[35,36,37,38],[135,136,137,138]]]]
--   , -3
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_concat_y(anyelement, anyarray);
create or replace function sm_sc.fv_concat_y
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  if array_ndims(i_right) > 5
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  elsif array_ndims(i_right) = 5
  then 
    return array_fill(i_left, array[1, array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4), array_length(i_right, 5)]) || i_right;
  elsif array_ndims(i_right) = 4
  then 
    return array_fill(i_left, array[1, array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]) || i_right;
  elsif array_ndims(i_right) = 3
  then 
    return array_fill(i_left, array[1, array_length(i_right, 2), array_length(i_right, 3)]) || i_right;
  elsif array_ndims(i_right) = 2
  then 
    return array_fill(i_left, array[1, array_length(i_right, 2)]) || i_right;
  elsif array_ndims(i_right) = 1
  then 
    return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_concat_y(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select 
--   sm_sc.fv_concat_y
--   (
--     345
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   )
-- select 
--   sm_sc.fv_concat_y
--   (
--     345
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   )

-- drop function if exists sm_sc.fv_opr_concat_heigh(anyarray, anyarray);
create or replace function sm_sc.fv_opr_concat_heigh
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  if greatest(array_ndims(i_left), array_ndims(i_right)) not between 2 and 4
  then
    raise exception 'no method for such length!  ndims_1: %; ndims_2: %;', array_ndims(i_left), array_ndims(i_right);
  elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 4
  then 
    return sm_sc.fv_concat_x3(i_left, i_right);
  elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 3
  then 
    return sm_sc.fv_concat_x(i_left, i_right);
  elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 2
  then 
    return sm_sc.fv_concat_y(i_left, i_right);
  -- elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 1
  -- then 
  --   return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     array[array[12.3, 25.1], array[2.56, 3.25]]
--   , array[array[12.3, 25.1], array[2.56, 3.25]]
--   )
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   )
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   )


-- -----------------------------------------------------
-- drop function if exists sm_sc.fv_opr_concat_heigh(anyarray, anyelement);
create or replace function sm_sc.fv_opr_concat_heigh
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
-- declare 
begin
  if array_ndims(i_left) not between 2 and 4
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  elsif array_ndims(i_left) = 4
  then 
    return sm_sc.fv_concat_x3(i_left, i_right);
  elsif array_ndims(i_left) = 3
  then 
    return sm_sc.fv_concat_x(i_left, i_right);
  elsif array_ndims(i_left) = 2
  then 
    return sm_sc.fv_concat_y(i_left, i_right);
  -- elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 1
  -- then 
  --   return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     array[array[12.3, 25.1], array[2.56, 3.25]]
--   , 3.8
--   )
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     array[[[-1,-2.9,-3,-4],[-11,-12,-13.1,-14],[-111,-112,-113,-114]]]
--   , 3.8
--   )
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     array[[[[-1,-2,-3,-4.8],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   , 3.8
--   )

-- -----------------------------------------------------
-- drop function if exists sm_sc.fv_opr_concat_heigh(anyelement, anyarray);
create or replace function sm_sc.fv_opr_concat_heigh
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  if array_ndims(i_right) not between 2 and 4
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  elsif array_ndims(i_right) = 4
  then 
    return sm_sc.fv_concat_x3(i_left, i_right);
  elsif array_ndims(i_right) = 3
  then 
    return sm_sc.fv_concat_x(i_left, i_right);
  elsif array_ndims(i_right) = 2
  then 
    return sm_sc.fv_concat_y(i_left, i_right);
  -- elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 1
  -- then 
  --   return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     3.8
--   , array[array[12.3, 25.1], array[2.56, 3.25]]
--   )
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     3
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   )
-- select 
--   sm_sc.fv_opr_concat_heigh
--   (
--     3
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   )

-- drop function if exists sm_sc.fv_opr_concat_width(anyarray, anyarray);
create or replace function sm_sc.fv_opr_concat_width
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  if greatest(array_ndims(i_left), array_ndims(i_right)) not between 2 and 4
  then
    raise exception 'no method for such length!  ndims_1: %; ndims_2: %;', array_ndims(i_left), array_ndims(i_right);
  elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 4
  then 
    return sm_sc.fv_concat_x4(i_left, i_right);
  elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 3
  then 
    return sm_sc.fv_concat_x3(i_left, i_right);
  elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 2
  then 
    return sm_sc.fv_concat_x(i_left, i_right);
  -- elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 1
  -- then 
  --   return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     array[array[12.3, 25.1], array[2.56, 3.25]]
--   , array[array[12.3, 25.1], array[2.56, 3.25]]
--   )
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   )
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   )


-- -----------------------------------------------------
-- drop function if exists sm_sc.fv_opr_concat_width(anyarray, anyelement);
create or replace function sm_sc.fv_opr_concat_width
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
-- declare 
begin
  if array_ndims(i_left) not between 2 and 4
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  elsif array_ndims(i_left) = 4
  then 
    return sm_sc.fv_concat_x4(i_left, i_right);
  elsif array_ndims(i_left) = 3
  then 
    return sm_sc.fv_concat_x3(i_left, i_right);
  elsif array_ndims(i_left) = 2
  then 
    return sm_sc.fv_concat_x(i_left, i_right);
  -- elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 1
  -- then 
  --   return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     array[array[12.3, 25.1], array[2.56, 3.25]]
--   , 3.8
--   )
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     array[[[-1,-2.9,-3,-4],[-11,-12,-13.1,-14],[-111,-112,-113,-114]]]
--   , 3.8
--   )
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     array[[[[-1,-2,-3,-4.8],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   , 3.8
--   )

-- -----------------------------------------------------
-- drop function if exists sm_sc.fv_opr_concat_width(anyelement, anyarray);
create or replace function sm_sc.fv_opr_concat_width
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  if array_ndims(i_right) not between 2 and 4
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  elsif array_ndims(i_right) = 4
  then 
    return sm_sc.fv_concat_x4(i_left, i_right);
  elsif array_ndims(i_right) = 3
  then 
    return sm_sc.fv_concat_x3(i_left, i_right);
  elsif array_ndims(i_right) = 2
  then 
    return sm_sc.fv_concat_x(i_left, i_right);
  -- elsif greatest(array_ndims(i_left), array_ndims(i_right)) = 1
  -- then 
  --   return array_fill(i_left, array[1]) || i_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     3.8
--   , array[array[12.3, 25.1], array[2.56, 3.25]]
--   )
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     3
--   , array[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]]]
--   )
-- select 
--   sm_sc.fv_opr_concat_width
--   (
--     3
--   , array[[[[-1,-2,-3,-4],[-11,-12,-13,-14],[-111,-112,-113,-114]],[[-5,-6,-7,-8],[-15,-16,-17,-18],[-115,-116,-117,-118]]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mirror(anyarray, int);
create or replace function sm_sc.fv_opr_mirror
(
  i_arr            anyarray,
  i_dim            int
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;    
  if pg_typeof(i_arr) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_mirror_py(i_arr, i_dim);
  else
    if i_dim = 1
    then 
      return sm_sc.__fv_mirror_y(i_arr);    
    elsif i_dim = 2
    then
      return sm_sc.__fv_mirror_x(i_arr);   
    elsif i_dim = 3
    then
      return sm_sc.__fv_mirror_x3(i_arr);   
    elsif i_dim = 4
    then
      return sm_sc.__fv_mirror_x4(i_arr);
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_opr_mirror
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , 3
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mirror_heigh(anyarray);
create or replace function sm_sc.fv_opr_mirror_heigh
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur               int; 
  -- v_ele              alias for $0;
  v_ret               i_right%type;
  v_len_mirror_dim    int  :=   array_length(i_right, array_ndims(i_right) - 1);
begin
  -- set search_path to sm_sc;
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 1
  -- -- then
  -- --   -- -- for v_cur in 1 .. v_len_mirror_dim / 2
  -- --   -- -- loop
  -- --   -- --   v_ele[1] := i_right[v_cur];
  -- --   -- --   i_right[v_cur] := i_right[v_len_mirror_dim - v_cur + 1];
  -- --   -- --   i_right[v_len_mirror_dim - v_cur + 1] := v_ele[1];
  -- --   -- -- end loop;
  -- --   -- -- return i_right;
  -- --   v_ret := array_fill(nullif(i_right[1], i_right[1]), array[v_len_mirror_dim]);
  -- --   for v_cur in 1 .. v_len_mirror_dim
  -- --   loop 
  -- --     v_ret[v_cur] := i_right[v_len_mirror_dim - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;
    
  elsif array_ndims(i_right) = 2
  then
    -- -- return 	
    -- -- (
    -- --   select
    -- --     array_agg(array(select unnest(i_right[a_cur_y : a_cur_y][ : ])) order by a_cur_y desc)
    -- --   from generate_series(1, v_len_mirror_dim) tb_a_cur_y(a_cur_y)
    -- -- )
    -- -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_mirror_dim, array_length(i_right, 2)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[v_cur : v_cur][ : ] := i_right[v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[array_length(i_right, 1), v_len_mirror_dim, array_length(i_right, 3)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][v_cur : v_cur][ : ] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), v_len_mirror_dim, array_length(i_right, 4)]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][ : ][v_cur : v_cur][ : ] := i_right[ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1][ : ];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %; len_1: %;', array_dims(i_right), v_len_mirror_dim;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mirror_heigh
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.fv_opr_mirror_heigh
--   (
--     array[1, 2, 3, 4]
--   );
-- select sm_sc.fv_opr_mirror_heigh
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.fv_opr_mirror_heigh
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_mirror_heigh
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mirror_py(float[], int);
create or replace function sm_sc.fv_opr_mirror_py
(
  i_arr            float[],
  i_dim            int
)
returns float[]
as
$$
  from numpy import array
  if i_dim == 1:
    return array(i_arr)[::-1].tolist()
  if i_dim == 2:
    return array(i_arr)[::, ::-1].tolist()
  if i_dim == 3:
    return array(i_arr)[::, ::, ::-1].tolist()
  if i_dim == 4:
    return array(i_arr)[::, ::, ::, ::-1].tolist()
$$
language plpython3u stable
parallel safe
cost 100;


-- drop function if exists sm_sc.fv_opr_mirror_py(decimal[], int);
create or replace function sm_sc.fv_opr_mirror_py
(
  i_arr            decimal[],
  i_dim            int
)
returns decimal[]
as
$$
  from numpy import array
  if i_dim == 1:
    return array(i_arr)[::-1].tolist()
  if i_dim == 2:
    return array(i_arr)[::, ::-1].tolist()
  if i_dim == 3:
    return array(i_arr)[::, ::, ::-1].tolist()
  if i_dim == 4:
    return array(i_arr)[::, ::, ::, ::-1].tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_opr_mirror_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , 3
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mirror_width(anyarray);
create or replace function sm_sc.fv_opr_mirror_width
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur               int; 
  -- v_ele              alias for $0;
  v_ret               i_right%type;
  v_len_mirror_dim    int  :=   array_length(i_right, array_ndims(i_right));
begin
  -- set search_path to sm_sc;
  
  if array_ndims(i_right) is null
  then 
    return i_right;
  -- -- elsif array_ndims(i_right) = 1
  -- -- then
  -- --   -- -- for v_cur in 1 .. array_length(i_right, 1) / 2
  -- --   -- -- loop
  -- --   -- --   v_ele[1] := i_right[v_cur];
  -- --   -- --   i_right[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   -- --   i_right[array_length(i_right, 1) - v_cur + 1] := v_ele[1];
  -- --   -- -- end loop;
  -- --   -- -- return i_right;
  -- --   v_ret := array_fill(nullif(i_right[1], i_right[1]), array[array_length(i_right, 1)]);
  -- --   for v_cur in 1 .. array_length(i_right, 1)
  -- --   loop 
  -- --     v_ret[v_cur] := i_right[array_length(i_right, 1) - v_cur + 1];
  -- --   end loop;
  -- --   return v_ret;

  elsif array_ndims(i_right) = 2
  then
    -- -- return 	
    -- -- (
    -- --   select
    -- --     array_agg(array_x_new order by a_cur_y)
    -- --   from 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_x desc) as array_x_new
    -- --     from generate_series(1, array_length(i_right, 1)) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, v_len_mirror_dim) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   ) t_array_x_new
    -- -- )
    -- -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[array_length(i_right, 1), v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][v_cur : v_cur] := i_right[ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][ : ][v_cur : v_cur] := i_right[ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), v_len_mirror_dim]);
    for v_cur in 1 .. v_len_mirror_dim
    loop 
      v_ret[ : ][ : ][ : ][v_cur : v_cur] := i_right[ : ][ : ][ : ][v_len_mirror_dim - v_cur + 1 : v_len_mirror_dim - v_cur + 1];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dim: %;  len_2: %;', array_dims(i_right),  v_len_mirror_dim;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mirror_width
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- -- select sm_sc.fv_opr_mirror_width
-- --   (
-- --     array[1, 2, 3, 4]
-- --   );
-- select sm_sc.fv_opr_mirror_width
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.fv_opr_mirror_width
--   (
--     array[array[1, 2, 7], array[3, 4, 8], array[5, 6, 9]]
--   );
-- select sm_sc.fv_opr_mirror_width
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_mirror_width
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_prod_inner(anyarray, anyarray);
create or replace function sm_sc.fv_opr_prod_inner
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyelement
as
$$
declare -- here
  -- v_x_cur   int  := 1  ;
  -- v_y_cur   int  := 1  ;
  -- v_return  alias for $0;
begin
  -- set search_path to sm_sc;
  if array_ndims(i_left) = 2 and array_ndims(i_right) = 2 and (array_length(i_left, 1) <> array_length(i_right, 1) or array_length(i_left, 2) <> array_length(i_right, 2))
    or array_ndims(i_left) = 1 and array_ndims(i_right) = 1 and array_length(i_left, 1) <> array_length(i_right, 1)
  then
    raise exception 'unmatched length!';
  end if;
  
  return |@+| (i_left *` i_right);

  -- -- if array_ndims(i_left) = 1 and array_ndims(i_right) = 1
  -- -- then
  -- --   v_return := i_left[1] - i_left[1];
  -- --   while v_y_cur <= array_length(i_right, 1)
  -- --   loop
  -- --     v_return := v_return + (i_left[v_y_cur] * i_right[v_y_cur]);
  -- --     v_y_cur := v_y_cur + 1;
  -- --   end loop;
  -- --   
  -- --   return v_return;
  -- -- elsif array_ndims(i_left) = 2 and array_ndims(i_right) = 2
  -- -- then
  -- --   v_return := i_left[1][1] - i_left[1][1];
  -- --   while v_y_cur <= array_length(i_right, 1)
  -- --   loop 
  -- --     v_x_cur := 1  ;
  -- --     while v_x_cur <= array_length(i_right, 2)
  -- --     loop
  -- --       v_return := v_return + (i_left[v_y_cur][v_x_cur] * i_right[v_y_cur][v_x_cur]);
  -- --       v_x_cur := v_x_cur + 1;
  -- --     end loop;
  -- --     v_y_cur := v_y_cur + 1;
  -- --   end loop;
  -- --   
  -- --   return v_return;
  -- -- else
  -- --   raise exception 'no method for such length!  L_Dim: %; R_Dim: %;', array_dims(i_left), array_dims(i_right);
  -- -- end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_prod_inner
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_opr_prod_inner
--   (
--     array[12.3, -12.3, 45.6, -45.6],
--     array[-12.3, 12.3, -45.6, 45.6]
--   );
-- select sm_sc.fv_opr_prod_inner
--   (
--     array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]],
--     array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]]
--   );
-- select sm_sc.fv_opr_prod_inner
--   (
--     array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex, (2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex],
--     array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex, (2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_prod_inner_pow(anyarray, anyelement);
create or replace function sm_sc.fv_opr_prod_inner_pow
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyelement
as
$$
declare -- here
begin
  return |@+| (i_left ^` i_right);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_prod_inner_pow
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     2.0
--   );
-- select sm_sc.fv_opr_prod_inner_pow
--   (
--     array[12.3, -12.3, 45.6, -45.6],
--     4.0
--   );
-- select sm_sc.fv_opr_prod_inner_pow
--   (
--     array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]],
--     2.0
--   );
-- select sm_sc.fv_opr_prod_inner_pow
--   (
--     array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex, (2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex],
--     3.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_prod_kronecker(anyarray, anyarray);
create or replace function sm_sc.fv_opr_prod_kronecker
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
-- declare -- here
begin
  -- set search_path to sm_sc;
  if array_ndims(i_left) <> 2 or array_ndims(i_right) <> 2
  then
    raise exception 'unsupport ndims!';
  elsif array_ndims(i_left) = 2 and array_ndims(i_right) = 2
  then    
    return 
    (
      with 
      cte_y_grp as
      (
        select 
          a_cur_left_y,
          sm_sc.fa_mx_concat_x(i_left[a_cur_left_y][a_cur_left_x] *` i_right order by a_cur_left_x) as a_y_ele_grp
        from generate_series(1, array_length(i_left, 1)) tb_a_cur_left_y(a_cur_left_y)
          , generate_series(1, array_length(i_left, 2)) tb_a_cur_left_x(a_cur_left_x)
        group by a_cur_left_y
      )
      select 
        sm_sc.fa_mx_concat_y(a_y_ele_grp order by a_cur_left_y)
      from cte_y_grp
    )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_prod_kronecker
--   (
--     array[array[1.0 :: float, -2.0], array[-3.0, 4.0]],
--     array[array[1.0 :: float, -2.0 :: float, 3.0], array[-4.0, 5.0, 6.0], array[7.0, 8.0, -9.0]]
--   );
-- select sm_sc.fv_opr_prod_kronecker
--   (
--     array[array[1.0 :: float, -2.0 :: float, 3.0], array[-4.0, 5.0, 6.0], array[7.0, 8.0, -9.0]],
--     array[array[1.0 :: float, -2.0], array[-3.0, 4.0]]
--   );
-- select sm_sc.fv_opr_prod_kronecker
--   (
--     array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]],
--     array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]]
--   );
-- select sm_sc.fv_opr_prod_kronecker
--   (
--     array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex, (2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex],
--     array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex, (2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_prod_mx_left(float[], float[]);
create or replace function sm_sc.fv_opr_prod_mx_left
(
  i_left     float[]
)
returns float[]
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if array_ndims(i_left) not between 2 and 4
  then
    raise exception 'unsupport ndims!';
  end if;

  return i_left |**| (|^~| i_left);

end
$$
language plpgsql stable
parallel safe;
-- -- cost 100;
-- -- -- -- set search_path to sm_sc;
-- -- -- select sm_sc.fv_opr_prod_mx_left
-- -- --   (
-- -- --     array[array[1.0000,2.0000,3.0000], array[4.0000,5.0000,6.0000]]
-- -- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_prod_mx_pow(float[], int);
create or replace function sm_sc.fv_opr_prod_mx_pow
(
  i_left     float[]    ,
  i_right    int
)
returns float[]
as
$$
declare -- here
  v_right_log_2   int;
  v_arr_pow_part  float[];
begin
    -- i_left 必须方阵，否则报错
    if array_length(i_left, array_ndims(i_left) - 1) <> array_length(i_left, array_ndims(i_left))
    then 
      raise exception 'i_left must be a square. ';
    end if;

    if i_right = 0
    then 
      return sm_sc.fv_eye_unit(array_length(i_left, array_ndims(i_left)), 1.0);
    elsif i_right < 0
    then 
      raise exception 'i_right should be >= 0. ';
    end if;

    -- 算法时间复杂度优化：log 递归
    v_right_log_2   :=  floor(log(2, i_right));

    if v_right_log_2 = 0    -- 此时，v_right = 1
    then
      return i_left;
    else   -- 此时，v_right_log_2 >= 1
      v_arr_pow_part := sm_sc.fv_opr_prod_mx_pow(i_left, power(v_right_log_2 - 1, 2) :: int);
      return v_arr_pow_part |**| v_arr_pow_part |**| sm_sc.fv_opr_prod_mx_pow(i_left, i_right - power(v_right_log_2, 2) :: int);
    end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_prod_mx_pow
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     4   -- 1, 2, 3, 4, 5, 6, 7, 8, 9
--   );
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i http://pypi.douban.com/simple/ --trusted-host pypi.douban.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_opr_prod_mx_py(float[], float[]);
create or replace function sm_sc.fv_opr_prod_mx_py
(
  i_left      float[]  ,
  i_right     float[]
)
returns float[]
as
$$
  import numpy as np
  return np.matmul(np.float64(i_left), np.float64(i_right)).tolist()
$$
language plpython3u stable
parallel safe
;

-- select 
--   sm_sc.fv_opr_prod_mx_py
--   (
--     array[array[1, 2, 3], array[1, 2, 3]] :: float[],
--     array[array[1, 2], array[3, 1], array[2, 3]] :: float[]
--   )
-- select -- 2.328
--   sm_sc.fv_opr_prod_mx_py
--   (
--     sm_sc.fv_new_rand(array[70, 28 * 28]),
--     sm_sc.fv_new_rand(array[28 * 28, 25 * 6])
--   )
-- select -- 2.849
--   sm_sc.fv_opr_prod_mx_py
--   (
--     sm_sc.fv_new_rand(array[70, 10 * 10]),
--     sm_sc.fv_new_rand(array[10 * 10, 25 * 60])
--   )
-- select -- 0.990
--   sm_sc.fv_opr_prod_mx_py
--   (
--     sm_sc.fv_new_rand(array[70, 25 * 16]),
--     sm_sc.fv_new_rand(array[25 * 16, 120])
--   )
-- select -- 0.088
--   sm_sc.fv_opr_prod_mx_py
--   (
--     sm_sc.fv_new_rand(array[70, 120]),
--     sm_sc.fv_new_rand(array[120, 10])
--   )
-- drop function if exists sm_sc.fv_opr_prod_mx_py_cupy(float[], float[]);
create or replace function sm_sc.fv_opr_prod_mx_py_cupy
(
  i_left      float[]  ,
  i_right     float[]
)
returns float[]
as
$$
  import cupy as cp
  with cp.cuda.Device(0):
    return (cp.float64(i_left) @ cp.float64(i_right)).tolist()
$$
language plpython3u stable
parallel safe
;

-- select 
--   sm_sc.fv_opr_prod_mx_py_cupy
--   (
--     array[array[1, 2, 3], array[1, 2, 3]] :: float[],
--     array[array[1, 2], array[3, 1], array[2, 3]] :: float[]
--   )
-- select -- 2.328
--   sm_sc.fv_opr_prod_mx_py_cupy
--   (
--     sm_sc.fv_new_rand(array[70, 28 * 28]),
--     sm_sc.fv_new_rand(array[28 * 28, 25 * 6])
--   )
-- select -- 2.849
--   sm_sc.fv_opr_prod_mx_py_cupy
--   (
--     sm_sc.fv_new_rand(array[70, 10 * 10]),
--     sm_sc.fv_new_rand(array[10 * 10, 25 * 60])
--   )
-- select -- 0.990
--   sm_sc.fv_opr_prod_mx_py_cupy
--   (
--     sm_sc.fv_new_rand(array[70, 25 * 16]),
--     sm_sc.fv_new_rand(array[25 * 16, 120])
--   )
-- select -- 0.088
--   sm_sc.fv_opr_prod_mx_py_cupy
--   (
--     sm_sc.fv_new_rand(array[70, 120]),
--     sm_sc.fv_new_rand(array[120, 10])
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_prod_mx_right(float[], float[]);
create or replace function sm_sc.fv_opr_prod_mx_right
(
  i_right     float[]
)
returns float[]
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if array_ndims(i_right) not between 2 and 4
  then
    raise exception 'unsupport ndims!';
  end if;

  return (|^~| i_right) |**| i_right;

end
$$
language plpgsql stable
parallel safe;
-- -- cost 100;
-- -- -- -- set search_path to sm_sc;
-- -- -- select sm_sc.fv_opr_prod_mx_right
-- -- --   (
-- -- --     array[array[1.0000,2.0000,3.0000], array[4.0000,5.0000,6.0000]]
-- -- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_transpose(anyarray);
create or replace function sm_sc.fv_opr_transpose
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret  i_right%type;
  v_cur  int  ;
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
    
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_transpose_py(i_right);
    
  elsif array_ndims(i_right) = 2
  then
    -- -- return 	
    -- -- (
    -- --   select
    -- --     array_agg(array_x_new order by a_cur_xy)
    -- --   from 
    -- --   (
    -- --     select 
    -- --       a_cur_xy,
    -- --       array_agg(i_right[a_cur_yx][a_cur_xy] order by a_cur_yx) as array_x_new
    -- --     from generate_series(1, array_length(i_right, 1)) tb_a_cur_yx(a_cur_yx)
    -- --       , generate_series(1, array_length(i_right, 2)) tb_a_cur_xy(a_cur_xy)
    -- --     group by a_cur_xy
    -- --   ) t_array_x_new
    -- -- )
    -- -- ;
    v_ret  :=   array_fill(nullif(v_ret[1][1], v_ret[1][1]), array[array_length(i_right, 2), array_length(i_right, 1)]);
    for v_cur in 1 .. array_length(i_right, 2)
    loop 
      v_ret[v_cur : v_cur][ : ]   :=  i_right[ : ][v_cur : v_cur];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret  :=   array_fill(nullif(v_ret[1][1][1], v_ret[1][1][1]), array[array_length(i_right, 1), array_length(i_right, 3), array_length(i_right, 2)]);
    for v_cur in 1 .. array_length(i_right, 3)
    loop 
      v_ret[ : ][v_cur : v_cur][ : ]   :=  i_right[ : ][ : ][v_cur : v_cur];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret  :=   array_fill(nullif(v_ret[1][1][1][1], v_ret[1][1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 4), array_length(i_right, 3)]);
    for v_cur in 1 .. array_length(i_right, 4)
    loop 
      v_ret[ : ][ : ][v_cur : v_cur][ : ]   :=  i_right[ : ][ : ][ : ][v_cur : v_cur];
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_opr_transpose
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6], array[1.2, 2.3]]
--   );
-- select sm_sc.fv_opr_transpose
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_transpose
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- drop function if exists sm_sc.fv_opr_transpose_3d(anyarray, int[2]);
create or replace function sm_sc.fv_opr_transpose_3d
(
  i_array_3d      anyarray      ,
  i_dims          int[2]        -- array[1, 2] === array[2, 1], array[1, 3] === array[3, 1] or array[2, 3] === array[3, 2]
)
returns anyarray
as
$$
declare
  v_ret     i_array_3d%type;
  v_cur_1   int   ;
  v_cur_2   int   ;
begin
  if array_ndims(i_array_3d) <> 3
  then 
    raise exception 'ndims of i_array_3d should be 3.';
  end if;
  
  if i_dims[1] not in (1, 2, 3) 
    or i_dims[2] not in (1,2,3)
    or i_dims[1] = i_dims[2]
  then 
    raise exception 'illegal i_dims';
  end if;
 
  if i_dims in (array[1, 2], array[2, 1])
  then 
    -- -- return 
    -- -- (
    -- --   with 
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_z,
    -- --       a_cur_y,
    -- --       array_agg(i_array_3d[a_cur_z][a_cur_y][a_cur_x] order by a_cur_x) as a_slice_x
    -- --     from generate_series(1, array_length(i_array_3d, 1)) tb_a_cur_z(a_cur_z)
    -- --       , generate_series(1, array_length(i_array_3d, 2)) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array_3d, 3)) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_z, a_cur_y
    -- --   ),
    -- --   cte_slice_z_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y, 
    -- --       array_agg(a_slice_x order by a_cur_z) as a_slice_z_x
    -- --     from cte_slice_x
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_z_x order by a_cur_y)
    -- --   from cte_slice_z_x
    -- -- );
    v_ret := array_fill(nullif(i_array_3d[1][1][1], i_array_3d[1][1][1]), array[array_length(i_array_3d, 2), array_length(i_array_3d, 1), array_length(i_array_3d, 3)]);
    for v_cur_1 in 1 .. array_length(i_array_3d, 2)
    loop 
      for v_cur_2 in 1 .. array_length(i_array_3d, 1)
      loop 
        v_ret[v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ] := i_array_3d[v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ];
      end loop;
    end loop;
    return v_ret;
    
  elsif i_dims in (array[1, 3], array[3, 1])
  then 
    -- -- return 
    -- -- (
    -- --   with 
    -- --   cte_slice_z as 
    -- --   (
    -- --     select 
    -- --       a_cur_x,
    -- --       a_cur_y,
    -- --       array_agg(i_array_3d[a_cur_z][a_cur_y][a_cur_x] order by a_cur_z) as a_slice_z
    -- --     from generate_series(1, array_length(i_array_3d, 1)) tb_a_cur_z(a_cur_z)
    -- --       , generate_series(1, array_length(i_array_3d, 2)) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array_3d, 3)) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_x, a_cur_y
    -- --   ),
    -- --   cte_slice_y_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_x, 
    -- --       array_agg(a_slice_z order by a_cur_y) as a_slice_y_z
    -- --     from cte_slice_z
    -- --     group by a_cur_x
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_y_z order by a_cur_x)
    -- --   from cte_slice_y_x
    -- -- );
    v_ret := array_fill(nullif(i_array_3d[1][1][1], i_array_3d[1][1][1]), array[array_length(i_array_3d, 3), array_length(i_array_3d, 2), array_length(i_array_3d, 1)]);
    for v_cur_1 in 1 .. array_length(i_array_3d, 3)
    loop 
      for v_cur_2 in 1 .. array_length(i_array_3d, 1)
      loop 
        v_ret[v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2] := i_array_3d[v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
    
  elsif i_dims in (array[2, 3], array[3, 2])
  then 
    -- -- return 
    -- -- (
    -- --   with 
    -- --   cte_slice_y as 
    -- --   (
    -- --     select 
    -- --       a_cur_z,
    -- --       a_cur_x,
    -- --       array_agg(i_array_3d[a_cur_z][a_cur_y][a_cur_x] order by a_cur_y) as a_slice_y
    -- --     from generate_series(1, array_length(i_array_3d, 1)) tb_a_cur_z(a_cur_z)
    -- --       , generate_series(1, array_length(i_array_3d, 2)) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array_3d, 3)) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_z, a_cur_x
    -- --   ),
    -- --   cte_slice_x_y as 
    -- --   (
    -- --     select 
    -- --       a_cur_z, 
    -- --       array_agg(a_slice_y order by a_cur_x) as a_slice_x_y
    -- --     from cte_slice_y
    -- --     group by a_cur_z
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x_y order by a_cur_z)
    -- --   from cte_slice_x_y
    -- -- );
    v_ret := array_fill(nullif(i_array_3d[1][1][1], i_array_3d[1][1][1]), array[array_length(i_array_3d, 1), array_length(i_array_3d, 3), array_length(i_array_3d, 2)]);
    for v_cur_1 in 1 .. array_length(i_array_3d, 3)
    loop 
      for v_cur_2 in 1 .. array_length(i_array_3d, 2)
      loop 
        v_ret[ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2] := i_array_3d[ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose_3d
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 2]
--   );
-- select sm_sc.fv_opr_transpose_3d
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 3]
--   );
-- select sm_sc.fv_opr_transpose_3d
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_transpose_i(anyarray);
create or replace function sm_sc.fv_opr_transpose_i
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret  i_right%type;
  v_cur  int  ;
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
    
  elsif pg_typeof(i_right) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_transpose_i_py(i_right);
    
  elsif array_ndims(i_right) = 2
  then
    -- -- return 	
    -- -- (
    -- --   select
    -- --     array_agg(array_x_new order by a_cur_xy desc)
    -- --   from 
    -- --   (
    -- --     select 
    -- --       a_cur_xy,
    -- --       array_agg(i_right[a_cur_yx][a_cur_xy] order by a_cur_yx desc) as array_x_new
    -- --     from generate_series(1, array_length(i_right, 1)) tb_a_cur_yx(a_cur_yx)
    -- --       , generate_series(1, array_length(i_right, 2)) tb_a_cur_xy(a_cur_xy)
    -- --     group by a_cur_xy
    -- --   ) t_array_x_new
    -- -- )
    -- -- ;
    v_ret  :=   array_fill(nullif(v_ret[1][1], v_ret[1][1]), array[array_length(i_right, 2), array_length(i_right, 1)]);
    for v_cur in 1 .. array_length(i_right, 2)
    loop 
      v_ret[v_cur : v_cur][ : ]   :=  i_right[ : ][v_cur : v_cur];
    end loop;
    return sm_sc.fv_opr_mirror(sm_sc.fv_opr_mirror(v_ret, 2), 1);
    
  elsif array_ndims(i_right) = 3
  then
    v_ret  :=   array_fill(nullif(v_ret[1][1][1], v_ret[1][1][1]), array[array_length(i_right, 1), array_length(i_right, 3), array_length(i_right, 2)]);
    for v_cur in 1 .. array_length(i_right, 3)
    loop 
      v_ret[ : ][v_cur : v_cur][ : ]   :=  i_right[ : ][ : ][v_cur : v_cur];
    end loop;
    return sm_sc.fv_opr_mirror(sm_sc.fv_opr_mirror(v_ret, 3), 2);
    
  elsif array_ndims(i_right) = 4
  then
    v_ret  :=   array_fill(nullif(v_ret[1][1][1][1], v_ret[1][1][1][1]), array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 4), array_length(i_right, 3)]);
    for v_cur in 1 .. array_length(i_right, 4)
    loop 
      v_ret[ : ][ : ][v_cur : v_cur][ : ]   :=  i_right[ : ][ : ][ : ][v_cur : v_cur];
    end loop;
    return sm_sc.fv_opr_mirror(sm_sc.fv_opr_mirror(v_ret, 4), 3);
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose_i
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.fv_opr_transpose_i
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.fv_opr_transpose_i
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_transpose_i
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_transpose_i_py(float[]);
create or replace function sm_sc.fv_opr_transpose_i_py
(
  i_right     float[]
)
returns float[]
as
$$
  from numpy import array
  v_right = array(i_right)
  if v_right.size == 0 :
    return v_right
  elif v_right.ndim == 2 :
    return (v_right[::, ::-1].swapaxes(-1,-2))[::, ::-1].tolist()
  elif v_right.ndim == 3 :
    return (v_right[::, ::, ::-1].swapaxes(-1,-2))[::, ::, ::-1].tolist()
  elif v_right.ndim == 4 :
    return (v_right[::, ::, ::-1].swapaxes(-1,-2))[::, ::, ::-1].tolist()
    
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose_i_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_opr_transpose_i_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6], array[1.2, 2.3]]
--   );
-- select sm_sc.fv_opr_transpose_i_py
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_transpose_i_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- drop function if exists sm_sc.fv_opr_transpose_nd(anyarray, int[2]);
create or replace function sm_sc.fv_opr_transpose_nd
(
  i_array_nd      anyarray      ,
  i_dims          int[2]        -- array[1, 2] === array[2, 1], array[1, 3] === array[3, 1] or array[2, 3] === array[3, 2]
)
returns anyarray
as
$$
declare
  v_ret     i_array_nd%type;
  v_cur_1   int   ;
  v_cur_2   int   ;
begin
  if array_ndims(i_array_nd) not in (2, 3, 4)
  then 
    raise exception 'ndims of i_array_nd should be 3 or 4.';
  end if;
  
  if pg_typeof(i_array_nd) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_transpose_nd_py(i_array_nd, i_dims);
  end if;
  
  if array_ndims(i_array_nd) = 2 
      and 
      (
        i_dims[1] not in (1, 2) 
        or i_dims[2] not in (1, 2)
      )
    or array_ndims(i_array_nd) = 3 
      and 
      (
        i_dims[1] not in (1, 2, 3) 
        or i_dims[2] not in (1, 2, 3)
      )
    or array_ndims(i_array_nd) = 4
      and 
      (
        i_dims[1] not in (1, 2, 3, 4) 
        or i_dims[2] not in (1, 2, 3, 4)
      )
    or i_dims[1] = i_dims[2]
  then 
    raise exception 'illegal i_dims';
  end if;
 
  if array_ndims(i_array_nd) = 2
  then 
    v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 2), array_length(i_array_nd, 1)]);
    for v_cur_1 in 1 .. array_length(i_array_nd, 2)
    loop 
      for v_cur_2 in 1 .. array_length(i_array_nd, 1)
      loop 
        v_ret[v_cur_1 : v_cur_1][v_cur_2 : v_cur_2] := i_array_nd[v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_array_nd) = 3
  then
    if i_dims in (array[1, 2], array[2, 1])
    then 
      -- -- return 
      -- -- (
      -- --   with 
      -- --   cte_slice_x as 
      -- --   (
      -- --     select 
      -- --       a_cur_z,
      -- --       a_cur_y,
      -- --       array_agg(i_array_nd[a_cur_z][a_cur_y][a_cur_x] order by a_cur_x) as a_slice_x
      -- --     from generate_series(1, array_length(i_array_nd, 1)) tb_a_cur_z(a_cur_z)
      -- --       , generate_series(1, array_length(i_array_nd, 2)) tb_a_cur_y(a_cur_y)
      -- --       , generate_series(1, array_length(i_array_nd, 3)) tb_a_cur_x(a_cur_x)
      -- --     group by a_cur_z, a_cur_y
      -- --   ),
      -- --   cte_slice_z_x as 
      -- --   (
      -- --     select 
      -- --       a_cur_y, 
      -- --       array_agg(a_slice_x order by a_cur_z) as a_slice_z_x
      -- --     from cte_slice_x
      -- --     group by a_cur_y
      -- --   )
      -- --   select 
      -- --     array_agg(a_slice_z_x order by a_cur_y)
      -- --   from cte_slice_z_x
      -- -- );
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 2), array_length(i_array_nd, 1), array_length(i_array_nd, 3)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 2)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret[v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ] := i_array_nd[v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ];
        end loop;
      end loop;
      return v_ret;
      
    elsif i_dims in (array[1, 3], array[3, 1])
    then 
      -- -- return 
      -- -- (
      -- --   with 
      -- --   cte_slice_z as 
      -- --   (
      -- --     select 
      -- --       a_cur_x,
      -- --       a_cur_y,
      -- --       array_agg(i_array_nd[a_cur_z][a_cur_y][a_cur_x] order by a_cur_z) as a_slice_z
      -- --     from generate_series(1, array_length(i_array_nd, 1)) tb_a_cur_z(a_cur_z)
      -- --       , generate_series(1, array_length(i_array_nd, 2)) tb_a_cur_y(a_cur_y)
      -- --       , generate_series(1, array_length(i_array_nd, 3)) tb_a_cur_x(a_cur_x)
      -- --     group by a_cur_x, a_cur_y
      -- --   ),
      -- --   cte_slice_y_x as 
      -- --   (
      -- --     select 
      -- --       a_cur_x, 
      -- --       array_agg(a_slice_z order by a_cur_y) as a_slice_y_z
      -- --     from cte_slice_z
      -- --     group by a_cur_x
      -- --   )
      -- --   select 
      -- --     array_agg(a_slice_y_z order by a_cur_x)
      -- --   from cte_slice_y_x
      -- -- );
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 3), array_length(i_array_nd, 2), array_length(i_array_nd, 1)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 3)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret[v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2] := i_array_nd[v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1];
        end loop;
      end loop;
      return v_ret;
      
    elsif i_dims in (array[2, 3], array[3, 2])
    then 
      -- -- return 
      -- -- (
      -- --   with 
      -- --   cte_slice_y as 
      -- --   (
      -- --     select 
      -- --       a_cur_z,
      -- --       a_cur_x,
      -- --       array_agg(i_array_nd[a_cur_z][a_cur_y][a_cur_x] order by a_cur_y) as a_slice_y
      -- --     from generate_series(1, array_length(i_array_nd, 1)) tb_a_cur_z(a_cur_z)
      -- --       , generate_series(1, array_length(i_array_nd, 2)) tb_a_cur_y(a_cur_y)
      -- --       , generate_series(1, array_length(i_array_nd, 3)) tb_a_cur_x(a_cur_x)
      -- --     group by a_cur_z, a_cur_x
      -- --   ),
      -- --   cte_slice_x_y as 
      -- --   (
      -- --     select 
      -- --       a_cur_z, 
      -- --       array_agg(a_slice_y order by a_cur_x) as a_slice_x_y
      -- --     from cte_slice_y
      -- --     group by a_cur_z
      -- --   )
      -- --   select 
      -- --     array_agg(a_slice_x_y order by a_cur_z)
      -- --   from cte_slice_x_y
      -- -- );
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 1), array_length(i_array_nd, 3), array_length(i_array_nd, 2)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 3)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 2)
        loop 
          v_ret[ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2] := i_array_nd[ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
        end loop;
      end loop;
      return v_ret;
    end if;
    
  elsif array_ndims(i_array_nd) = 4
  then 
    if i_dims in (array[1, 2], array[2, 1])
    then 
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 2), array_length(i_array_nd, 1), array_length(i_array_nd, 3), array_length(i_array_nd, 4)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 2)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret[v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ][ : ] := i_array_nd[v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ][ : ];
        end loop;
      end loop;
      return v_ret;
    elsif i_dims in (array[1, 3], array[3, 1])
    then 
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 3), array_length(i_array_nd, 2), array_length(i_array_nd, 1), array_length(i_array_nd, 4)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 3)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret[v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2][ : ] := i_array_nd[v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1][ : ];
        end loop;
      end loop;
      return v_ret;
    elsif i_dims in (array[1, 4], array[4, 1])
    then 
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 4), array_length(i_array_nd, 2), array_length(i_array_nd, 3), array_length(i_array_nd, 1)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 4)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret[v_cur_1 : v_cur_1][ : ][ : ][v_cur_2 : v_cur_2] := i_array_nd[v_cur_2 : v_cur_2][ : ][ : ][v_cur_1 : v_cur_1];
        end loop;
      end loop;
      return v_ret;
    elsif i_dims in (array[2, 3], array[3, 2])
    then 
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 1), array_length(i_array_nd, 3), array_length(i_array_nd, 2), array_length(i_array_nd, 4)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 3)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 2)
        loop 
          v_ret[ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2][ : ] := i_array_nd[ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1][ : ];
        end loop;
      end loop;
      return v_ret;
    elsif i_dims in (array[2, 4], array[4, 2])
    then 
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 1), array_length(i_array_nd, 4), array_length(i_array_nd, 3), array_length(i_array_nd, 2)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 4)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 2)
        loop 
          v_ret[ : ][v_cur_1 : v_cur_1][ : ][v_cur_2 : v_cur_2] := i_array_nd[ : ][v_cur_2 : v_cur_2][ : ][v_cur_1 : v_cur_1];
        end loop;
      end loop;
      return v_ret;
    elsif i_dims in (array[3, 4], array[4, 3])
    then 
      v_ret := array_fill(nullif(i_array_nd[0], i_array_nd[0]), array[array_length(i_array_nd, 1), array_length(i_array_nd, 2), array_length(i_array_nd, 4), array_length(i_array_nd, 3)]);
      for v_cur_1 in 1 .. array_length(i_array_nd, 4)
      loop 
        for v_cur_2 in 1 .. array_length(i_array_nd, 3)
        loop 
          v_ret[ : ][ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2] := i_array_nd[ : ][ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
        end loop;
      end loop;
      return v_ret;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose_nd
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 2]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 3]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[2, 3]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,2]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,3]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,4]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[2,3]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[2,4]
--   );
-- select sm_sc.fv_opr_transpose_nd
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[3,4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_transpose_nd_py(float[]);
create or replace function sm_sc.fv_opr_transpose_nd_py
(
  i_right     float[]
, i_dims          int[2]
)
returns float[]
as
$$
  from numpy import array
  v_right = array(i_right)
  if v_right.size == 0 :
    return v_right
  else :
    return v_right.swapaxes(i_dims[0] - 1, i_dims[1] - 1).tolist()
    
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 2]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[1, 3]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4],
--         [5, 6, 7, 8],
--         [9, 10, 11, 12]
--       ],
--       [
--         [21, 22, 23, 24],
--         [25, 26, 27, 28],
--         [29, 30, 31, 32]
--       ]
--     ]
--     , array[2, 3]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,2]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,3]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[1,4]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[2,3]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[2,4]
--   );
-- select sm_sc.fv_opr_transpose_nd_py
--   (
--     sm_sc.fv_new_rand(array[2,3,4,5])
--   , array[3,4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_transpose_py(float[]);
create or replace function sm_sc.fv_opr_transpose_py
(
  i_right     float[]
)
returns float[]
as
$$
  from numpy import array
  v_right = array(i_right)
  if v_right.size == 0 :
    return v_right
  else :
    return v_right.swapaxes(-1,-2).tolist()
    
$$
language plpython3u stable
parallel safe
cost 100;

-- drop function if exists sm_sc.fv_opr_transpose_py(decimal[]);
create or replace function sm_sc.fv_opr_transpose_py
(
  i_right     decimal[]
)
returns decimal[]
as
$$
  from numpy import array
  v_right = array(i_right)
  if v_right.size == 0 :
    return v_right
  else :
    return v_right.swapaxes(-1,-2).tolist()
    
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_transpose_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_opr_transpose_py
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6], array[1.2, 2.3]]
--   );
-- select sm_sc.fv_opr_transpose_py
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_transpose_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_180(anyarray, int[2]);
create or replace function sm_sc.fv_opr_turn_180
(
  i_arr            anyarray,
  i_dims           int[2]     default null
)
returns anyarray
as
$$
-- declare 
begin
  -- 缺省 heigh length 两维度为旋转面
  if i_dims is null 
  then 
    i_dims := 
      array 
      [
        array_ndims(i_arr) - 1
      , array_ndims(i_arr)
      ]
    ;
  end if;
  
  -- set search_path to sm_sc;    
  if pg_typeof(i_arr) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_turn_180_py(i_arr, i_dims);
  else 
    if i_dims in (array[1, 2], array[2, 1])
    then 
      return sm_sc.__fv_turn_y_x_180(i_arr);    
    elsif i_dims in (array[1, 3], array[3, 1])
    then
      return sm_sc.__fv_turn_y_x3_180(i_arr);   
    elsif i_dims in (array[1, 4], array[4, 1])
    then
      return sm_sc.__fv_turn_y_x4_180(i_arr);   
    elsif i_dims in (array[2, 3], array[3, 2])
    then
      return sm_sc.__fv_turn_x_x3_180(i_arr);   
    elsif i_dims in (array[2, 4], array[4, 2])
    then
      return sm_sc.__fv_turn_x_x4_180(i_arr);   
    elsif i_dims in (array[3, 4], array[4, 3])
    then
      return sm_sc.__fv_turn_x3_x4_180(i_arr);
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_opr_turn_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_180_py(float[], int[2]);
create or replace function sm_sc.fv_opr_turn_180_py
(
  i_arr            float[],
  i_dims    int[2]
)
returns float[]
as
$$
  import numpy as np
  if i_dims[0] == 1 and i_dims[1] == 2 or i_dims[0] == 2 and i_dims[1] == 1:
    return np.array(i_arr)[::-1, ::-1].tolist()
  elif i_dims[0] == 1 and i_dims[1] == 3 or i_dims[0] == 3 and i_dims[1] == 1:
    return np.array(i_arr)[::-1, ::, ::-1].tolist()
  elif i_dims[0] == 1 and i_dims[1] == 4 or i_dims[0] == 4 and i_dims[1] == 1:
    return np.array(i_arr)[::-1, ::, ::, ::-1].tolist()
  elif i_dims[0] == 2 and i_dims[1] == 3 or i_dims[0] == 3 and i_dims[1] == 2:
    return np.array(i_arr)[::, ::-1, ::-1].tolist()
  elif i_dims[0] == 2 and i_dims[1] == 4 or i_dims[0] == 4 and i_dims[1] == 2:
    return np.array(i_arr)[::, ::-1, ::, ::-1].tolist()
  elif i_dims[0] == 3 and i_dims[1] == 4 or i_dims[0] == 4 and i_dims[1] == 3:
    return np.array(i_arr)[::, ::, ::-1, ::-1].tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_opr_turn_180_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_90(anyarray, int[2]);
create or replace function sm_sc.fv_opr_turn_90
(
  i_arr            anyarray,
  i_dims_from_to    int[2]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;    
  if pg_typeof(i_arr) = ('double precision[]' :: regtype)
  then 
    return sm_sc.fv_opr_turn_90_py(i_arr, i_dims_from_to);
  else
    if i_dims_from_to = array[1, 2]
    then 
      return sm_sc.__fv_turn_y_x_90(i_arr);    
    elsif i_dims_from_to = array[1, 3]
    then
      return sm_sc.__fv_turn_y_x3_90(i_arr);   
    elsif i_dims_from_to = array[1, 4]
    then
      return sm_sc.__fv_turn_y_x4_90(i_arr);   
    elsif i_dims_from_to = array[2, 3]
    then
      return sm_sc.__fv_turn_x_x3_90(i_arr);   
    elsif i_dims_from_to = array[2, 4]
    then
      return sm_sc.__fv_turn_x_x4_90(i_arr);   
    elsif i_dims_from_to = array[3, 4]
    then
      return sm_sc.__fv_turn_x3_x4_90(i_arr);   
    elsif i_dims_from_to = array[2, 1]
    then 
      return sm_sc.__fv_turn_x_y_90(i_arr);    
    elsif i_dims_from_to = array[3, 1]
    then
      return sm_sc.__fv_turn_x3_y_90(i_arr);   
    elsif i_dims_from_to = array[4, 1]
    then
      return sm_sc.__fv_turn_x4_y_90(i_arr);   
    elsif i_dims_from_to = array[3, 2]
    then
      return sm_sc.__fv_turn_x3_x_90(i_arr);   
    elsif i_dims_from_to = array[4, 2]
    then
      return sm_sc.__fv_turn_x4_x_90(i_arr);   
    elsif i_dims_from_to = array[4, 3]
    then
      return sm_sc.__fv_turn_x4_x3_90(i_arr);
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_opr_turn_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_90_py(float[], int[2]);
create or replace function sm_sc.fv_opr_turn_90_py
(
  i_arr            float[],
  i_dims_from_to    int[2]
)
returns float[]
as
$$
  import numpy as np
  if i_dims_from_to[1] == 1 :
    return np.array(i_arr).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::-1].tolist()
  elif i_dims_from_to[1] == 2 :
    return np.array(i_arr).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::, ::-1].tolist()
  elif i_dims_from_to[1] == 3 :
    return np.array(i_arr).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::, ::, ::-1].tolist()
  elif i_dims_from_to[1] == 4 :
    return np.array(i_arr).swapaxes(i_dims_from_to[0] - 1, i_dims_from_to[1] - 1)[::, ::, ::, ::-1].tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_opr_turn_90_py
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[3, 4]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_heigh_width_180(anyarray);
create or replace function sm_sc.fv_opr_turn_heigh_width_180
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 2
  then
    -- return 	
    -- (
    --   select
    --     array_agg(array_x_new order by a_cur_x)
    --   from 
    --   (
    --     select 
    --       a_cur_x,
    --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
    --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
    --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
    --     group by a_cur_x
    --   ) t_array_x_new
    -- )
    -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_y, v_len_x]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1]
             [v_len_x - v_cur_2 + 1 : v_len_x - v_cur_2 + 1] 
          := i_right[v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ]
             [v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1]
             [v_len_x3 - v_cur_2 + 1 : v_len_x3 - v_cur_2 + 1]
          := i_right[ : ]
                    [v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x3, v_len_x4]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[ : ]
             [ : ]
             [v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1]
             [v_len_x4 - v_cur_2 + 1 : v_len_x4 - v_cur_2 + 1]
          := i_right[ : ]
                    [ : ]
                    [v_cur_1 : v_cur_1]
                    [v_cur_2 : v_cur_2]
        ;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_turn_heigh_width_180
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.fv_opr_turn_heigh_width_180
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.fv_opr_turn_heigh_width_180
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_turn_heigh_width_180
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_heigh_width_90(anyarray);
create or replace function sm_sc.fv_opr_turn_heigh_width_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 2
  then
    -- return 	
    -- (
    --   select
    --     array_agg(array_x_new order by a_cur_x)
    --   from 
    --   (
    --     select 
    --       a_cur_x,
    --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y desc) as array_x_new
    --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
    --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
    --     group by a_cur_x
    --   ) t_array_x_new
    -- )
    -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      v_ret[v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1][ : ] := i_right[ : ][v_cur_1 : v_cur_1];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x3, v_len_x]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x
      loop 
        v_ret[ : ][v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1][v_cur_2 : v_cur_2] := i_right[ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x4, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x4
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ][ : ][v_len_x4 - v_cur_1 + 1 : v_len_x4 - v_cur_1 + 1][v_cur_2 : v_cur_2] := i_right[ : ][ : ][v_cur_2 : v_cur_2][v_cur_1 : v_cur_1];
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_turn_heigh_width_90
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.fv_opr_turn_heigh_width_90
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.fv_opr_turn_heigh_width_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_turn_heigh_width_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_turn_width_heigh_90(anyarray);
create or replace function sm_sc.fv_opr_turn_width_heigh_90
(
  i_right     anyarray
)
returns anyarray
as
$$
declare 
  v_ret      i_right%type   ;
  v_cur_1    int  ;
  v_cur_2    int  ;
  v_len_y    int  := array_length(i_right, 1);
  v_len_x    int  := array_length(i_right, 2);
  v_len_x3   int  := array_length(i_right, 3);
  v_len_x4   int  := array_length(i_right, 4);
begin
  -- set search_path to sm_sc;
    
  if array_ndims(i_right) is null
  then 
    return i_right;
  elsif array_ndims(i_right) = 2
  then
    -- return 	
    -- (
    --   select
    --     array_agg(array_x_new order by a_cur_x desc)
    --   from 
    --   (
    --     select 
    --       a_cur_x,
    --       array_agg(i_right[a_cur_y][a_cur_x] order by a_cur_y) as array_x_new
    --     from generate_series(1, v_len_y) tb_a_cur_y(a_cur_y)
    --       , generate_series(1, v_len_x) tb_a_cur_x(a_cur_x)
    --     group by a_cur_x
    --   ) t_array_x_new
    -- )
    -- ;
    v_ret := array_fill(nullif(i_right[1][1], i_right[1][1]), array[v_len_x, v_len_y]);
    for v_cur_1 in 1 .. v_len_y
    loop 
      v_ret[ : ][v_len_y - v_cur_1 + 1 : v_len_y - v_cur_1 + 1] := i_right[v_cur_1 : v_cur_1][ : ];
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(nullif(i_right[1][1][1], i_right[1][1][1]), array[v_len_y, v_len_x3, v_len_x]);
    for v_cur_1 in 1 .. v_len_x
    loop 
      for v_cur_2 in 1 .. v_len_x3
      loop 
        v_ret[ : ][v_cur_2 : v_cur_2][v_len_x - v_cur_1 + 1 : v_len_x - v_cur_1 + 1] := i_right[ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(nullif(i_right[1][1][1][1], i_right[1][1]), array[v_len_y, v_len_x, v_len_x4, v_len_x3]);
    for v_cur_1 in 1 .. v_len_x3
    loop 
      for v_cur_2 in 1 .. v_len_x4
      loop 
        v_ret[ : ][ : ][v_cur_2 : v_cur_2][v_len_x3 - v_cur_1 + 1 : v_len_x3 - v_cur_1 + 1] := i_right[ : ][ : ][v_cur_1 : v_cur_1][v_cur_2 : v_cur_2];
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_turn_width_heigh_90
--   (
--     array[array[1, 2], array[3, 4]]
--   );
-- select sm_sc.fv_opr_turn_width_heigh_90
--   (
--     array[array[1, 2], array[3, 4], array[5, 6]]
--   );
-- select sm_sc.fv_opr_turn_width_heigh_90
--   (
--     array[[[1,2,3,4],[11,12,13,14],[111,112,113,114]],[[5,6,7,8],[15,16,17,18],[115,116,117,118]]]
--   );
-- select sm_sc.fv_opr_turn_width_heigh_90
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_turn_squr_chunk(anyarray, int8range[], int[2], varchar(8));
create or replace function sm_sc.fv_turn_squr_chunk
(
  i_arr            anyarray
, i_chunk_pos      int8range[]
, i_turn_dims      int[2]
, i_turn_angle     varchar(8)          -- 旋转角度，枚举: '90', '180', '270'.  规约旋转方向描述为从 i_turn_dims[1] 正半轴方向 到 i_turn_dims[2] 正半轴方向
)
returns anyarray
as
$$
declare 
  v_ndims   int   := array_length(i_chunk_pos, 1);
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_arr) not between 2 and 4
    then
      raise exception 'unsupport ndims: %.', array_ndims(i_arr);
    elsif v_ndims <> array_ndims(i_arr)
    then 
      raise exception 'unmatch length of i_chunk_pos: %.', v_ndims;
    elsif exists(select  from generate_series(1, v_ndims) tb_a(a_no) 
                 where lower(i_chunk_pos[a_no]) <= 0 or upper(i_chunk_pos[a_no]) > array_length(i_arr, a_no))
    then 
      raise exception 'overflow length of i_chunk_pos.';
    elsif i_turn_dims[1] = i_turn_dims[2]
      or exists(select  from unnest(i_turn_dims) tb_a(a_dim) where a_dim not between 1 and array_ndims(i_arr))
    then 
      raise exception 'unsupport turn_dims.';
    elsif i_turn_angle not in ('90', '180', '270')
    then 
      raise exception 'unsupport turn_angle.';
    end if;
  end if;
  
  -- 调整优直角为负直角
  if i_turn_angle = '270'
  then 
    i_turn_dims := i_turn_dims[2 : 2] || i_turn_dims[1 : 1];
    i_turn_angle := '90';
  end if;
  
  -- 填充 i_chunk_pos 空值的默认值
  i_chunk_pos :=
    (
      select 
        array_agg
        (
          int8range
          (
            coalesce(lower(i_chunk_pos[a_no]), 1)
          , coalesce(upper(i_chunk_pos[a_no]), array_length(i_arr, a_no) + 1)
          ) 
          order by a_no
        ) 
      from generate_series(1, v_ndims) tb_a(a_no)
    )
  ;
  
  -- 旋转切片
  if i_turn_angle = '180'
  then 
    i_turn_dims := (select array_agg(a_ele order by a_ele) from unnest(i_turn_dims) tb_a(a_ele));
    if i_turn_dims = array[1, 2]
    then 
      if v_ndims = 2
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1] 
          := sm_sc.__fv_turn_y_x_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1]);  
      elsif v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1] 
          := sm_sc.__fv_turn_y_x_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]); 
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]
          := sm_sc.__fv_turn_y_x_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]); 
      end if;      
    elsif i_turn_dims = array[1, 3]
    then
      if v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1] 
          := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
          := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]); 
      end if;      
    elsif i_turn_dims = array[1, 4]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
    elsif i_turn_dims = array[2, 3]
    then
      if v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1] 
          := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
          := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);
      end if;
    elsif i_turn_dims = array[2, 4]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);
    elsif i_turn_dims = array[3, 4]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_y_x3_180(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);
    end if;  
  elsif i_turn_angle = '90'
  then 
    if i_turn_dims = array[1, 2]
    then 
      if v_ndims = 2
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1]
          := sm_sc.__fv_turn_y_x_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1]);    
      elsif v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]
          := sm_sc.__fv_turn_y_x_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]
          := sm_sc.__fv_turn_y_x_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);      
      end if;
    elsif i_turn_dims = array[1, 3]
    then
      if v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]
          := sm_sc.__fv_turn_y_x3_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]
          := sm_sc.__fv_turn_y_x3_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
      end if;
    elsif i_turn_dims = array[1, 4]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_y_x4_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
    elsif i_turn_dims = array[2, 3]
    then
      if v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1] 
          := sm_sc.__fv_turn_x_x3_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
          := sm_sc.__fv_turn_x_x3_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
      end if;
    elsif i_turn_dims = array[2, 4]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_x_x4_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
    elsif i_turn_dims = array[3, 4]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_x3_x4_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
    elsif i_turn_dims = array[2, 1]
    then 
      if v_ndims = 2
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1]
          := sm_sc.__fv_turn_x_y_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1]);    
      elsif v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]
          := sm_sc.__fv_turn_x_y_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]
          := sm_sc.__fv_turn_x_y_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);      
      end if;
    elsif i_turn_dims = array[3, 1]
    then
      if v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]
          := sm_sc.__fv_turn_x3_y_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]
          := sm_sc.__fv_turn_x3_y_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
      end if;
    elsif i_turn_dims = array[4, 1]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_x4_y_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
    elsif i_turn_dims = array[3, 2]
    then
      if v_ndims = 3
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1] 
          := sm_sc.__fv_turn_x3_x_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1]);   
      elsif v_ndims = 4
      then 
        i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
          := sm_sc.__fv_turn_x3_x_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
      end if;
    elsif i_turn_dims = array[4, 2]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_x4_x_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);   
    elsif i_turn_dims = array[4, 3]
    then
      i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1] 
        := sm_sc.__fv_turn_x4_x3_90(i_arr[lower(i_chunk_pos[1]) : upper(i_chunk_pos[1]) - 1][lower(i_chunk_pos[2]) : upper(i_chunk_pos[2]) - 1][lower(i_chunk_pos[3]) : upper(i_chunk_pos[3]) - 1][lower(i_chunk_pos[4]) : upper(i_chunk_pos[4]) - 1]);  
    end if;
  end if;
  
  -- 返回
  return i_arr;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 3]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 4]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 3]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 4]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[3, 4]
--   , '90'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 3]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 4]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 3]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 4]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[3, 4]
--   , '180'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 3]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 4]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 3]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 4]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]],[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]]
--   , array[int8range(1,3), int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[3, 4]
--   , '270'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 3]
--   , '90'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 3]
--   , '90'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 3]
--   , '180'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 3]
--   , '180'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[1, 3]
--   , '270'
--   );
-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--   , array[int8range(1,3), int8range(1,3), int8range(2,4)]
--   , array[2, 3]
--   , '270'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[1,2,3,4],[11,12,13,14]]
--   , array[int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '90'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[1,2,3,4],[11,12,13,14]]
--   , array[int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '180'
--   );

-- select 
--   sm_sc.fv_turn_squr_chunk
--   (
--     array[[1,2,3,4],[11,12,13,14]]
--   , array[int8range(1,3), int8range(2,4)]
--   , array[1, 2]
--   , '270'
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_amask_dloss_dindepdt_1(float[], int[]);
create or replace function sm_sc.fv_d_amask_dloss_dindepdt_1
(
  i_dloss_ddepdt            float[]            ,
  i_co_value                int[]                 -- i_mask_len
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_amask(i_dloss_ddepdt, i_co_value, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_amask_dloss_dindepdt_1
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ]
--   , array[[3, 0, 2, 1, 3]]
--   )

-- select 
--   sm_sc.fv_d_amask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ]
--   , array[[[3, 0, 2, 1, 0]], [[1, 1, 4, 2, 1]], [[3, 2, 2, 1, 4]]]
--   )

-- select 
--   sm_sc.fv_d_amask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ]
--   , array[[[[3, 0, 2, 1, 3]], [[1, 1, 4, 2, 4]], [[3, 3, 2, 1, 2]]],[[[3, 0, 2, 1, 1]], [[1, 1, 4, 2, 0]], [[3, 3, 2, 1, 2]]]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_apad_dloss_dindepdt_1(float[], int);
create or replace function sm_sc.fv_d_apad_dloss_dindepdt_1
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_heigh        int
)
returns float[]
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
    
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return 
      i_dloss_ddepdt[array_length(i_dloss_ddepdt, 1) - i_indepdt_heigh + 1 : ][ : ]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
      i_dloss_ddepdt[ : ][array_length(i_dloss_ddepdt, 2) - i_indepdt_heigh + 1 : ][ : ]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
      i_dloss_ddepdt[ : ][ : ][array_length(i_dloss_ddepdt, 3) - i_indepdt_heigh + 1 : ][ : ]
    ;
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_apad_dloss_dindepdt_1
--   (
--     array[[1, 2, 3, 4, 5],[11, 12, 13, 14, 15],[21, 22, 23, 24, 25],[31, 32, 33, 34, 35],[41, 42, 43, 44, 45]]
--   , 2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_apad_dloss_dindepdt_2(float[], int, int);
create or replace function sm_sc.fv_d_apad_dloss_dindepdt_2
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_heigh        int      ,
  i_times                int        default  1   
)
returns float[]
as
$$
declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[i_indepdt_heigh * (a_cur - 1) + 1 : i_indepdt_heigh * a_cur][ : ])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][i_indepdt_heigh * (a_cur - 1) + 1 : i_indepdt_heigh * a_cur][ : ])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][i_indepdt_heigh * (a_cur - 1) + 1 : i_indepdt_heigh * a_cur][ : ])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_apad_dloss_dindepdt_2
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [11, 12, 13, 14, 15]
--     , [21, 22, 23, 24, 25]
--     , [31, 32, 33, 34, 35]
--     , [41, 42, 43, 44, 45]
--     ],
--     2,
--     2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_bmask_dloss_dindepdt_1(float[], int[]);
create or replace function sm_sc.fv_d_bmask_dloss_dindepdt_1
(
  i_dloss_ddepdt            float[]            ,
  i_co_value                int[]                 -- i_mask_len
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_bmask(i_dloss_ddepdt, i_co_value, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_bmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ]
--   , array[[3, 0, 2, 1, 3]]
--   )

-- select 
--   sm_sc.fv_d_bmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ]
--   , array[[[3, 0, 2, 1, 0]], [[1, 1, 4, 2, 1]], [[3, 2, 2, 1, 4]]]
--   )

-- select 
--   sm_sc.fv_d_bmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ]
--   , array[[[[3, 0, 2, 1, 3]], [[1, 1, 4, 2, 4]], [[3, 3, 2, 1, 2]]],[[[3, 0, 2, 1, 1]], [[1, 1, 4, 2, 0]], [[3, 3, 2, 1, 2]]]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_bpad_dloss_dindepdt_1(float[], int);
create or replace function sm_sc.fv_d_bpad_dloss_dindepdt_1
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_heigh        int
)
returns float[]
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
    
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return 
      i_dloss_ddepdt[1 : i_indepdt_heigh][ : ]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
      i_dloss_ddepdt[ : ][1 : i_indepdt_heigh][ : ]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
      i_dloss_ddepdt[ : ][ : ][1 : i_indepdt_heigh][ : ]
    ;
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_bpad_dloss_dindepdt_1
--   (
--     array[[1, 2, 3, 4, 5],[11, 12, 13, 14, 15],[21, 22, 23, 24, 25],[31, 32, 33, 34, 35],[41, 42, 43, 44, 45]]
--   , 2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_bpad_dloss_dindepdt_2(float[], int, int);
create or replace function sm_sc.fv_d_bpad_dloss_dindepdt_2
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_heigh        int      ,
  i_times                int        default  1   
)
returns float[]
as
$$
declare 
  v_len     int       := array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1);
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[v_len - (a_cur * i_indepdt_heigh) + 1 : v_len - ((a_cur - 1) * i_indepdt_heigh)][ : ])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][v_len - (a_cur * i_indepdt_heigh) + 1 : v_len - ((a_cur - 1) * i_indepdt_heigh)][ : ])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][v_len - (a_cur * i_indepdt_heigh) + 1 : v_len - ((a_cur - 1) * i_indepdt_heigh)][ : ])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_bpad_dloss_dindepdt_2
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [11, 12, 13, 14, 15]
--     , [21, 22, 23, 24, 25]
--     , [31, 32, 33, 34, 35]
--     , [41, 42, 43, 44, 45]
--     ],
--     2,
--     2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_chunk_dloss_dindepdt(float[], int[], int[]);
create or replace function sm_sc.fv_d_chunk_dloss_dindepdt
(
  i_dloss_ddepdt           float[]    ,
  i_indepdt_len            int[]      ,
  i_chunk_pos_per_dim      int[]
)
returns float[]
as
$$
declare 
  v_depdt_len   int[]    := (select array_agg(array_length(i_dloss_ddepdt, a_no) order by a_no) from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a(a_no));
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 1 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif array_length(i_indepdt_len, 1) <> array_ndims(i_dloss_ddepdt)
      or array_length(i_chunk_pos_per_dim, 1) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch i_indepdt_len or i_chunk_pos_per_dim. ';
    elsif true = any(i_indepdt_len <=` 0)
      or true = any(i_chunk_pos_per_dim <=` 0)
      or true = any((i_chunk_pos_per_dim +` i_indepdt_len) >` v_depdt_len)
    then 
      raise exception 'overflow of i_chunk_pos_per_dim + i_indepdt_len. ';
    end if;
  end if;
  
  return sm_sc.fv_area_replace(array_fill(0.0 :: float, i_indepdt_len), i_chunk_pos_per_dim, i_dloss_ddepdt);
  
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_chunk_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[4,3,5,6])
--   , array[8,7,9,8]
--   , array[3,2,1,2]
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_lmask_dloss_dindepdt_1(float[], int[]);
create or replace function sm_sc.fv_d_lmask_dloss_dindepdt_1
(
  i_dloss_ddepdt            float[]            ,
  i_co_value                int[]                 -- i_mask_len
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_lmask(i_dloss_ddepdt, i_co_value, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_lmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ] :: float[]
--   , array[[3], [0], [2], [1]]
--   )

-- select 
--   sm_sc.fv_d_lmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ] :: float[]
--   , array[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]
--   )

-- select 
--   sm_sc.fv_d_lmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ] :: float[]
--   , array[[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]],[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_lower_tri_mx_dloss_dindepdt(float[]);
create or replace function sm_sc.fv_d_lower_tri_mx_dloss_dindepdt
(
  i_dloss_ddepdt            float[]
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_lower_tri_mx(i_dloss_ddepdt, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_lower_tri_mx_dloss_dindepdt
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );

-- select 
--   sm_sc.fv_d_lower_tri_mx_dloss_dindepdt
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );

-- select 
--   sm_sc.fv_d_lower_tri_mx_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt(float[]);
create or replace function sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt
(
  i_dloss_ddepdt            float[]
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_lower_tri_mx_ex(i_dloss_ddepdt, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );

-- select 
--   sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );

-- select 
--   sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_lpad_dloss_dindepdt_1(float[], int);
create or replace function sm_sc.fv_d_lpad_dloss_dindepdt_1
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_width        int
)
returns float[]
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
    
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return 
      i_dloss_ddepdt[ : ][array_length(i_dloss_ddepdt, 1) - i_indepdt_width + 1 : ]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
      i_dloss_ddepdt[ : ][ : ][array_length(i_dloss_ddepdt, 2) - i_indepdt_width + 1 : ]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
      i_dloss_ddepdt[ : ][ : ][ : ][array_length(i_dloss_ddepdt, 3) - i_indepdt_width + 1 : ]
    ;
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_lpad_dloss_dindepdt_1
--   (
--     array[[1, 2, 3, 4, 5],[11, 12, 13, 14, 15],[21, 22, 23, 24, 25],[31, 32, 33, 34, 35],[41, 42, 43, 44, 45]]
--   , 2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_lpad_dloss_dindepdt_2(float[], int, int);
create or replace function sm_sc.fv_d_lpad_dloss_dindepdt_2
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_width        int      ,
  i_times                int        default  1   
)
returns float[]
as
$$
declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][i_indepdt_width * (a_cur - 1) + 1 : i_indepdt_width * a_cur])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][i_indepdt_width * (a_cur - 1) + 1 : i_indepdt_width * a_cur])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][ : ][i_indepdt_width * (a_cur - 1) + 1 : i_indepdt_width * a_cur])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_lpad_dloss_dindepdt_2
--   (
--     array[1, 2, 3, 4, 5, 6],
--     array[7, 8],
--     2
--   );
-- drop function if exists sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(float[], int);
create or replace function sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt
(
  i_dloss_dindepdt         float[]  ,
  i_ascend_time            int
)
returns float[]
as
$$
-- declare
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_dindepdt) > 4
    then 
      raise exception 'unsupport dim > 4.';
    end if;
  end if;
  
  if array_ndims(i_dloss_dindepdt) = 4
  then 
    if i_ascend_time = 1
    then 
      return sm_sc.fv_mx_slice_4d_2_3d(i_dloss_dindepdt, 1, 1);
    elsif i_ascend_time = 2
    then 
      return sm_sc.fv_mx_slice_4d_2_2d(i_dloss_dindepdt, array[1, 2], array[1, 1]);
    elsif i_ascend_time = 3
    then 
      return sm_sc.fv_mx_ele_2d_2_1d(sm_sc.fv_mx_slice_4d_2_2d(i_dloss_dindepdt, array[1, 2], array[1, 1]), 2);
    end if;
  elsif array_ndims(i_dloss_dindepdt) = 3
  then 
    if i_ascend_time = 1
    then 
      return sm_sc.fv_mx_slice_3d_2_2d(i_dloss_dindepdt, 1, 1);
    elsif i_ascend_time = 2
    then 
      return sm_sc.fv_mx_ele_2d_2_1d(sm_sc.fv_mx_slice_3d_2_2d(i_dloss_dindepdt, 1, 1), 2);
    end if;
  elsif array_ndims(i_dloss_dindepdt) = 2  -- and i_ascend_time = 1
  then 
    return sm_sc.fv_mx_ele_2d_2_1d(i_dloss_dindepdt, 2);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(array[[[[1, 2]]]], 1)
-- select sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(array[[[[1, 2]]]], 2)
-- select sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(array[[[[1, 2]]]], 3)
-- select sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(array[[[1, 2]]], 1)
-- select sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(array[[[1, 2]]], 2)
-- select sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(array[[1, 2]], 1)
-- drop function if exists sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(float[], int);
create or replace function sm_sc.fv_d_mx_descend_dim_dloss_dindepdt
(
  i_dloss_dindepdt         float[]  ,
  i_descend_time            int
)
returns float[]
as
$$
-- declare
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_dindepdt) + i_descend_time > 4
    then 
      raise exception 'unsupport dim > 4.';
    end if;
  end if;
  
  if i_descend_time = 3
  then 
    return array[[[i_dloss_dindepdt]]];
  elsif i_descend_time = 2
  then 
    return array[[i_dloss_dindepdt]];
  elsif i_descend_time = 1
  then 
    return array[i_dloss_dindepdt];
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(array[1, 2]::float[], 1)
-- select sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(array[1, 2]::float[], 2)
-- select sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(array[1, 2]::float[], 3)
-- select sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(array[[1, 2]]::float[], 1)
-- select sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(array[[1, 2]]::float[], 2)
-- select sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(array[[[1, 2]]]::float[], 1)
-- drop function if exists sm_sc.fv_d_mx_ele_2d_2_3d_dloss_dindepdt(anyarray, int, int, boolean);
create or replace function sm_sc.fv_d_mx_ele_2d_2_3d_dloss_dindepdt
(
  i_dloss_ddepdt_3d      anyarray
-- , i_cnt_per_grp            int                  -- 新增维度的维度长度(元素个数)。
, i_dim_from               int                  -- 被拆分维度
, i_dim_new                int                  -- 新生维度
, i_dim_pin_ele_on_from    boolean              -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns anyarray
as
$$
-- declare

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    end if;
  end if;

  return 
    sm_sc.fv_mx_ele_3d_2_2d
    (
      i_dloss_ddepdt_3d
    , array[i_dim_new, case when i_dim_from < i_dim_new then i_dim_from else i_dim_from + 1 end]
    , case 
        when i_dim_pin_ele_on_from 
          then case when i_dim_from < i_dim_new then i_dim_from else i_dim_from + 1 end
        else 
          i_dim_new
      end
    )
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- with 
-- cte_ele_2d_2_3d as 
-- (
--   select 
--     a_arr_2d
--   , a_dim_from
--   , a_dim_new
--   , a_dim_pin_ele_on_from
--   , sm_sc.fv_mx_ele_2d_2_3d
--     (
--       a_arr_2d
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     ) as a_ele_2d_2_3d
--   from 
--     (select sm_sc.fv_new_rand(array[6,10]) as a_arr_2d) tb_a_arr_2d(a_arr_2d)
--   , generate_series(1, 2) tb_a_dim_from(a_dim_from)
--   , generate_series(1, 3) tb_a_dim_new(a_dim_new)
--   , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
--   -- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from
-- )
-- select 
--   sm_sc.fv_d_mx_ele_2d_2_3d_dloss_dindepdt
--   (
--     a_ele_2d_2_3d
--   , a_dim_from
--   , a_dim_new
--   , a_dim_pin_ele_on_from
--   ) = a_arr_2d
-- from cte_ele_2d_2_3d
-- drop function if exists sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt(anyarray, int, int[2], int, int[4]);
create or replace function sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt
(
  i_dloss_ddepdt_2d        anyarray
, i_dims_from_to           int[2]     -- 合并维度的原来两个维度。合并后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
                                      -- 枚举项包括，[1, 2] === [2, 1]; [2, 3] === [3, 2]; [1, 3]; [3, 1];
, i_dim_pin_ele            int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
, i_indepdt_len            int[4]
)
returns anyarray
as
$$
-- declare

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt_2d) <> 2
    then 
      raise exception 'ndims should be 2.';
    end if;
  end if;
  
  return 
    sm_sc.fv_mx_ele_2d_2_3d
    (
      i_dloss_ddepdt_2d
    , i_indepdt_len[case when i_dim_pin_ele = i_dims_from_to[1] then i_dims_from_to[1] else i_dims_from_to[2] end]   -- i_cnt_per_grp        
    , case when i_dims_from_to[1] < i_dims_from_to[2] then i_dims_from_to[2] - 1 else i_dims_from_to[2] end   -- i_dim_from           
    , i_dims_from_to[1]   -- i_dim_new            
    , i_dim_pin_ele = i_dims_from_to[2]   -- i_if_dim_pin_ele_on_from
    )
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- with 
-- cte_ele_3d_2_2d as 
-- (
--   select 
--     a_arr_3d
--   , a_dims_from_to
--   , a_dim_pin_ele
--   , sm_sc.fv_mx_ele_3d_2_2d(a_arr_3d, a_dims_from_to, a_dim_pin_ele) as a_out
--   from 
--     (select sm_sc.fv_new_rand(array[2,3,5]) as a_arr_3d) tb_a_arr_3d(a_arr_3d)
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[1, 3] 
--       union all select array[3, 1]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 3) tb_a_dim_pin_ele(a_dim_pin_ele)
--   where a_dim_pin_ele = any(a_dims_from_to)
--   -- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   --   , a_dims_from_to[1]
--   --   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   --   , a_dim_pin_ele
-- )
-- select 
--   array_dims(a_arr_3d) as a_dims_indepdt
-- , array_dims(a_out) a_dims_depdt
-- , a_dims_from_to
-- , a_dim_pin_ele
-- , sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt
--   (
--     a_out
--   , a_dims_from_to
--   , a_dim_pin_ele
--   , (select array_agg(array_length(a_arr_3d, a_no) order by a_no) from generate_series(1, array_ndims(a_arr_3d)) tb_a_no(a_no))
--   ) = a_arr_3d
-- , array_dims(sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt
--   (
--     a_out
--   , a_dims_from_to
--   , a_dim_pin_ele
--   , (select array_agg(array_length(a_arr_3d, a_no) order by a_no) from generate_series(1, array_ndims(a_arr_3d)) tb_a_no(a_no))
--   )) as a_dims_dloss_ddepdt
-- from cte_ele_3d_2_2d
-- drop function if exists sm_sc.fv_d_mx_ele_3d_2_4d_dloss_dindepdt(anyarray, int, int, boolean);
create or replace function sm_sc.fv_d_mx_ele_3d_2_4d_dloss_dindepdt
(
  i_dloss_ddepdt_4d      anyarray
-- , i_cnt_per_grp            int                  -- 新增维度的维度长度(元素个数)。
, i_dim_from               int                  -- 被拆分维度
, i_dim_new                int                  -- 新生维度
, i_dim_pin_ele_on_from    boolean              -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns anyarray
as
$$
-- declare

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt_4d) <> 4
    then 
      raise exception 'ndims should be 4.';
    end if;
  end if;

  return 
    sm_sc.fv_mx_ele_4d_2_3d
    (
      i_dloss_ddepdt_4d
    , array[i_dim_new, case when i_dim_from < i_dim_new then i_dim_from else i_dim_from + 1 end]
    , case 
        when i_dim_pin_ele_on_from 
          then case when i_dim_from < i_dim_new then i_dim_from else i_dim_from + 1 end
        else 
          i_dim_new
      end
    )
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- with 
-- cte_ele_3d_2_4d as 
-- (
--   select 
--     a_arr_3d
--   , a_dim_from
--   , a_dim_new
--   , a_dim_pin_ele_on_from
--   , sm_sc.fv_mx_ele_3d_2_4d
--     (
--       a_arr_3d
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     ) as a_ele_3d_2_4d
--   from 
--     (select sm_sc.fv_new_rand(array[6,8,10]) as a_arr_3d) tb_a_arr_3d(a_arr_3d)
--   , generate_series(1, 3) tb_a_dim_from(a_dim_from)
--   , generate_series(1, 4) tb_a_dim_new(a_dim_new)
--   , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
--   -- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from
-- )
-- select 
--   sm_sc.fv_d_mx_ele_3d_2_4d_dloss_dindepdt
--   (
--     a_ele_3d_2_4d
--   , a_dim_from
--   , a_dim_new
--   , a_dim_pin_ele_on_from
--   ) = a_arr_3d
-- from cte_ele_3d_2_4d
-- drop function if exists sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt(anyarray, int, int[2], int, int[4]);
create or replace function sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt
(
  i_dloss_ddepdt_3d        anyarray
, i_dims_from_to           int[2]     -- 合并维度的原来两个维度。合并后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
                                      -- 枚举项包括，[1, 2] === [2, 1]; [2, 3] === [3, 2]; [3, 4] === [4, 3]; [1, 3]; [3, 1]; [1, 4]; [4, 1]; [2, 4]; [4, 2]
, i_dim_pin_ele            int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
, i_indepdt_len            int[4]
)
returns anyarray
as
$$
-- declare

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    end if;
  end if;
  
  return 
    sm_sc.fv_mx_ele_3d_2_4d
    (
      i_dloss_ddepdt_3d
    , i_indepdt_len[case when i_dim_pin_ele = i_dims_from_to[1] then i_dims_from_to[1] else i_dims_from_to[2] end]   -- i_cnt_per_grp        
    , case when i_dims_from_to[1] < i_dims_from_to[2] then i_dims_from_to[2] - 1 else i_dims_from_to[2] end   -- i_dim_from           
    , i_dims_from_to[1]   -- i_dim_new            
    , i_dim_pin_ele = i_dims_from_to[2]   -- i_if_dim_pin_ele_on_from
    )
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- with 
-- cte_ele_4d_2_3d as 
-- (
--   select 
--     a_arr_4d
--   , a_dims_from_to
--   , a_dim_pin_ele
--   , sm_sc.fv_mx_ele_4d_2_3d(a_arr_4d, a_dims_from_to, a_dim_pin_ele) as a_out
--   from 
--     (select sm_sc.fv_new_rand(array[2,3,5,7]) as a_arr_4d) tb_a_arr_4d(a_arr_4d)
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[3, 4]
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[4, 3]
--       union all select array[1, 3] 
--       union all select array[3, 1]
--       union all select array[1, 4]
--       union all select array[4, 1]
--       union all select array[2, 4]
--       union all select array[4, 2]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 4) tb_a_dim_pin_ele(a_dim_pin_ele)
--   where a_dim_pin_ele = any(a_dims_from_to)
--   -- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   --   , a_dims_from_to[1]
--   --   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   --   , a_dim_pin_ele
-- )
-- select 
--   array_dims(a_arr_4d) as a_dims_indepdt
-- , array_dims(a_out) a_dims_depdt
-- , a_dims_from_to
-- , a_dim_pin_ele
-- , sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt
--   (
--     a_out
--   , a_dims_from_to
--   , a_dim_pin_ele
--   , (select array_agg(array_length(a_arr_4d, a_no) order by a_no) from generate_series(1, array_ndims(a_arr_4d)) tb_a_no(a_no))
--   ) = a_arr_4d
-- , array_dims(sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt
--   (
--     a_out
--   , a_dims_from_to
--   , a_dim_pin_ele
--   , (select array_agg(array_length(a_arr_4d, a_no) order by a_no) from generate_series(1, array_ndims(a_arr_4d)) tb_a_no(a_no))
--   )) as a_dims_dloss_ddepdt
-- from cte_ele_4d_2_3d
-- drop function if exists sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt(anyarray, int[2], int, int[]);
create or replace function sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt
(
  i_dloss_ddepdt_nd        anyarray  
, i_dims_from_to           int[2]     -- 扁平化的维度的原来两个维度。扁平化后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
, i_dim_pin_ele            int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
, i_indepdt_len            int[]
)
returns anyarray
as
$$
declare
  v_ret     i_dloss_ddepdt_nd%type    := array_fill(i_dloss_ddepdt_nd[0], i_indepdt_len);
  v_cur_y   int                       ;
  v_cur_x   int                       ;
  v_cur_x3  int                       ;
  v_cur_x4  int                       ;

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt_nd) not between 2 and 4
    then 
      raise exception 'ndims should be between 2 and 4.';
    end if;
    
    if i_dim_pin_ele <> all(i_dims_from_to)
    then 
      raise exception 'the pin_ele dim must be one of from_to dims.';
    end if;
    
    if array_ndims(i_indepdt_len) <> 1 or array_ndims(i_dloss_ddepdt_nd) <> array_length(i_indepdt_len, 1)
    then 
      raise exception 'wrong i_indepdt_len for i_dloss_ddepdt_nd.';
    end if;
    
    if i_dims_from_to[1] not between 1 and array_ndims(i_dloss_ddepdt_nd) 
      or i_dims_from_to[2] not between 1 and array_ndims(i_dloss_ddepdt_nd) 
      or i_dims_from_to[1] = i_dims_from_to[2]
    then 
      raise exception 'unsupport this i_dims_from_to.';
    end if;
    
    if array_length(i_dloss_ddepdt_nd, i_dims_from_to[1]) <> 1
    then 
      raise exception 'array_length of i_dloss_ddepdt_nd at dims_from should be 1.';
    end if;
    
    if array_length(i_dloss_ddepdt_nd, i_dims_from_to[2]) <> i_indepdt_len[i_dims_from_to[1]] * i_indepdt_len[i_dims_from_to[2]]
    then 
      raise exception 'wrong array_length of i_dloss_ddepdt_nd at dims_to';
    end if;
  end if;
  
  if i_dloss_ddepdt_nd is null 
  then 
    return null;
  end if;  
  
  if array_length(i_indepdt_len, 1) = 2
  then 
    if i_dims_from_to = array[1, 2]
    then 
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          v_ret
            [ : ]
            [v_cur_x : v_cur_x] 
          := 
            i_dloss_ddepdt_nd
              [1 : 1]
              [(v_cur_x - 1) * i_indepdt_len[1] + 1 : v_cur_x * i_indepdt_len[1]] 
          ;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          v_ret
            [v_cur_y : v_cur_y]
            [ : ]
          := 
            i_dloss_ddepdt_nd
              [1 : 1]
              [(v_cur_y - 1) * i_indepdt_len[2] + 1 : v_cur_y * i_indepdt_len[2]] 
          ;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 1]
    then 
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          v_ret
            [v_cur_y : v_cur_y]
            [ : ]
          := 
            i_dloss_ddepdt_nd
              [(v_cur_y - 1) * i_indepdt_len[2] + 1 : v_cur_y * i_indepdt_len[2]] 
              [1 : 1]
          ;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          v_ret
            [ : ]
            [v_cur_x : v_cur_x]
          := 
            i_dloss_ddepdt_nd
              [(v_cur_x - 1) * i_indepdt_len[1] + 1 : v_cur_x * i_indepdt_len[1]] 
              [1 : 1]
          ;
        end loop;
      end if;
    end if;
  elsif array_length(i_indepdt_len, 1) = 3
  then 
    if i_dims_from_to = array[1, 2]
    then 
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [ : ]
              [v_cur_x : v_cur_x]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [1 : 1]
                [(v_cur_x - 1) * i_indepdt_len[1] + 1 : v_cur_x * i_indepdt_len[1]] 
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [ : ]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [1 : 1]
                [(v_cur_y - 1) * i_indepdt_len[2] + 1 : v_cur_y * i_indepdt_len[2]] 
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 1]
    then 
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [ : ]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [(v_cur_y - 1) * i_indepdt_len[2] + 1 : v_cur_y * i_indepdt_len[2]] 
                [1 : 1]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [ : ]
              [v_cur_x : v_cur_x]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [(v_cur_x - 1) * i_indepdt_len[1] + 1 : v_cur_x * i_indepdt_len[1]] 
                [1 : 1]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[1, 3]
    then 
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [ : ]
              [v_cur_x : v_cur_x]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [1 : 1]
                [v_cur_x : v_cur_x]
                [(v_cur_x3 - 1) * i_indepdt_len[1] + 1 : v_cur_x3 * i_indepdt_len[1]] 
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [v_cur_x : v_cur_x]
              [ : ]
            := 
              i_dloss_ddepdt_nd
                [1 : 1]
                [v_cur_x : v_cur_x]
                [(v_cur_y - 1) * i_indepdt_len[3] + 1 : v_cur_y * i_indepdt_len[3]] 
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 1]
    then 
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [v_cur_x : v_cur_x]
              [ : ]
            := 
              i_dloss_ddepdt_nd
                [(v_cur_y - 1) * i_indepdt_len[3] + 1 : v_cur_y * i_indepdt_len[3]] 
                [v_cur_x : v_cur_x]
                [1 : 1]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [ : ]
              [v_cur_x : v_cur_x]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [(v_cur_x3 - 1) * i_indepdt_len[1] + 1 : v_cur_x3 * i_indepdt_len[1]] 
                [v_cur_x : v_cur_x]
                [1 : 1]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 3]
    then 
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [ : ]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [v_cur_y : v_cur_y]
                [1 : 1]
                [(v_cur_x3 - 1) * i_indepdt_len[2] + 1 : v_cur_x3 * i_indepdt_len[2]] 
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [v_cur_x : v_cur_x]
              [ : ]
            := 
              i_dloss_ddepdt_nd
                [v_cur_y : v_cur_y]
                [1 : 1]
                [(v_cur_x - 1) * i_indepdt_len[3] + 1 : v_cur_x * i_indepdt_len[3]] 
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 2]
    then 
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [v_cur_x : v_cur_x]
              [ : ]
            := 
              i_dloss_ddepdt_nd
                [v_cur_y : v_cur_y]
                [(v_cur_x - 1) * i_indepdt_len[3] + 1 : v_cur_x * i_indepdt_len[3]] 
                [1 : 1]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [ : ]
              [v_cur_x3 : v_cur_x3]
            := 
              i_dloss_ddepdt_nd
                [v_cur_y : v_cur_y]
                [(v_cur_x3 - 1) * i_indepdt_len[2] + 1 : v_cur_x3 * i_indepdt_len[2]] 
                [1 : 1]
            ;
          end loop;
        end loop;
      end if;
    end if;
  elsif array_length(i_indepdt_len, 1) = 4
  then 
    if i_dims_from_to = array[1, 2]
    then 
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [1 : 1]
                  [(v_cur_x - 1) * i_indepdt_len[1] + 1 : v_cur_x * i_indepdt_len[1]] 
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [1 : 1]
                  [(v_cur_y - 1) * i_indepdt_len[2] + 1 : v_cur_y * i_indepdt_len[2]] 
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 1]
    then 
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [(v_cur_y - 1) * i_indepdt_len[2] + 1 : v_cur_y * i_indepdt_len[2]] 
                  [1 : 1]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [(v_cur_x - 1) * i_indepdt_len[1] + 1 : v_cur_x * i_indepdt_len[1]] 
                  [1 : 1]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[1, 3]
    then 
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [1 : 1]
                  [v_cur_x : v_cur_x]
                  [(v_cur_x3 - 1) * i_indepdt_len[1] + 1 : v_cur_x3 * i_indepdt_len[1]] 
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [1 : 1]
                  [v_cur_x : v_cur_x]
                  [(v_cur_y - 1) * i_indepdt_len[3] + 1 : v_cur_y * i_indepdt_len[3]] 
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 1]
    then 
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [(v_cur_y - 1) * i_indepdt_len[3] + 1 : v_cur_y * i_indepdt_len[3]] 
                  [v_cur_x : v_cur_x]
                  [1 : 1]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [(v_cur_x3 - 1) * i_indepdt_len[1] + 1 : v_cur_x3 * i_indepdt_len[1]] 
                  [v_cur_x : v_cur_x]
                  [1 : 1]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[1, 4]
    then 
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [1 : 1]
                  [v_cur_x : v_cur_x] 
                  [v_cur_x3 : v_cur_x3]
                  [(v_cur_x4 - 1) * i_indepdt_len[1] + 1 : v_cur_x4 * i_indepdt_len[1]]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x3 in 1 .. i_indepdt_len[3]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [ : ]
              := 
                i_dloss_ddepdt_nd
                  [1 : 1]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [(v_cur_y - 1) * i_indepdt_len[4] + 1 : v_cur_y * i_indepdt_len[4]] 
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[4, 1]
    then 
      if i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x3 in 1 .. i_indepdt_len[3]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [ : ]
              := 
                i_dloss_ddepdt_nd
                  [(v_cur_y - 1) * i_indepdt_len[4] + 1 : v_cur_y * i_indepdt_len[4]] 
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [1 : 1]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. i_indepdt_len[2]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [(v_cur_x4 - 1) * i_indepdt_len[1] + 1 : v_cur_x4 * i_indepdt_len[1]] 
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [1 : 1]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 3]
    then 
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [1 : 1]
                  [(v_cur_x3 - 1) * i_indepdt_len[2] + 1 : v_cur_x3 * i_indepdt_len[2]] 
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [1 : 1]
                  [(v_cur_x - 1) * i_indepdt_len[3] + 1 : v_cur_x * i_indepdt_len[3]] 
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 2]
    then 
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [(v_cur_x - 1) * i_indepdt_len[3] + 1 : v_cur_x * i_indepdt_len[3]] 
                  [1 : 1]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [(v_cur_x3 - 1) * i_indepdt_len[2] + 1 : v_cur_x3 * i_indepdt_len[2]] 
                  [1 : 1]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 4]
    then 
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y] 
                  [1 : 1]
                  [v_cur_x3 : v_cur_x3]
                  [(v_cur_x4 - 1) * i_indepdt_len[2] + 1 : v_cur_x4 * i_indepdt_len[2]]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x3 in 1 .. i_indepdt_len[3]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [ : ]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [1 : 1]
                  [v_cur_x3 : v_cur_x3]
                  [(v_cur_x - 1) * i_indepdt_len[4] + 1 : v_cur_x * i_indepdt_len[4]] 
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[4, 2]
    then 
      if i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x3 in 1 .. i_indepdt_len[3]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [ : ]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [(v_cur_x - 1) * i_indepdt_len[4] + 1 : v_cur_x * i_indepdt_len[4]] 
                  [v_cur_x3 : v_cur_x3]
                  [1 : 1]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x3 in 1 .. i_indepdt_len[3]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [(v_cur_x4 - 1) * i_indepdt_len[2] + 1 : v_cur_x4 * i_indepdt_len[2]] 
                  [v_cur_x3 : v_cur_x3]
                  [1 : 1]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 4]
    then 
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x2 in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x2 : v_cur_x2]
                [ : ]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y] 
                  [v_cur_x2 : v_cur_x2]
                  [1 : 1]
                  [(v_cur_x4 - 1) * i_indepdt_len[3] + 1 : v_cur_x4 * i_indepdt_len[3]]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x3 in 1 .. i_indepdt_len[3]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [ : ]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [1 : 1]
                  [(v_cur_x3 - 1) * i_indepdt_len[4] + 1 : v_cur_x3 * i_indepdt_len[4]] 
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[4, 3]
    then 
      if i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x3 in 1 .. i_indepdt_len[3]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [ : ]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [(v_cur_x3 - 1) * i_indepdt_len[4] + 1 : v_cur_x3 * i_indepdt_len[4]] 
                  [1 : 1]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. i_indepdt_len[1]
        loop 
          for v_cur_x in 1 .. i_indepdt_len[2]
          loop 
            for v_cur_x4 in 1 .. i_indepdt_len[4]
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
                [v_cur_x4 : v_cur_x4]
              := 
                i_dloss_ddepdt_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [(v_cur_x4 - 1) * i_indepdt_len[3] + 1 : v_cur_x4 * i_indepdt_len[3]] 
                  [1 : 1]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    end if;
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_mx_ele_flatten_2dims as 
-- (
--   select 
--     a_arr
--   , a_dim_from
--   , a_dim_to
--   , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx] as a_dim_pin_ele
--   , sm_sc.fv_mx_ele_flatten_2dims
--     (
--       a_arr
--     , array[a_dim_from, a_dim_to]
--     , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx]
--     ) as a_flattened_dims
--   from 
--     (
--                 select sm_sc.fv_new_rand(array[4,3    ]) as a_arr
--       union all select sm_sc.fv_new_rand(array[4,3,5  ]) as a_arr
--       union all select sm_sc.fv_new_rand(array[4,3,5,7]) as a_arr
--     ) tb_a_arr(a_arr)
--   , generate_series(1, 4) tb_a_dim_from(a_dim_from)
--   , generate_series(1, 4) tb_a_dim_to(a_dim_to)
--   , generate_series(1, 2) tb_a_dim_pin_ele_idx(a_dim_pin_ele_idx)
--   where a_dim_from <= array_ndims(a_arr)
--     and a_dim_to <= array_ndims(a_arr)
--     and a_dim_from <> a_dim_to
--   order by array_ndims(a_arr), a_dim_from, a_dim_to, a_dim_pin_ele_idx
-- )
-- select 
--   sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt
--   (
--     a_flattened_dims
--   , array[a_dim_from, a_dim_to]
--   , a_dim_pin_ele
--   , (select array_agg(array_length(a_arr, a_no) order by a_no) from generate_series(1, array_ndims(a_arr)) tb_a_no(a_no))
--   ) = a_arr
-- from cte_mx_ele_flatten_2dims
-- drop function if exists sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt_py(float[], int[2], int, int[]);
create or replace function sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt_py
(
  i_dloss_ddepdt_nd        float[]  
, i_dims_from_to           int[2]     -- 扁平化的维度的原来两个维度。扁平化后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
, i_dim_pin_ele            int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
, i_indepdt_len            int[]
)
returns float[]
as
$$
  import numpy as np
  v_dims_from_to = np.int8(i_dims_from_to) -1
  v_dims_from_to.sort()
  v_dim_pin_ele = i_dim_pin_ele - 1
  v_indepdt_len = np.int8(i_indepdt_len).tolist()
  v_dloss_ddepdt_nd = np.array(i_dloss_ddepdt_nd)
  
  if v_dloss_ddepdt_nd.ndim == 2 :
    if v_dim_pin_ele == 0 :
      return v_dloss_ddepdt_nd.reshape(v_indepdt_len[1], v_indepdt_len[0]).transpose(1, 0).tolist()
    elif v_dim_pin_ele == 1 :
      return v_dloss_ddepdt_nd.reshape(v_indepdt_len[0], v_indepdt_len[1]).tolist()
      
  elif v_dloss_ddepdt_nd.ndim == 3 :
    if np.array_equal(v_dims_from_to, np.array([0, 1])) :
      if v_dim_pin_ele == 0 :
        return v_dloss_ddepdt_nd.transpose(2, 1, 0).reshape(v_indepdt_len[2], v_indepdt_len[1], v_indepdt_len[0]).transpose(2, 1, 0).tolist()
      elif v_dim_pin_ele == 1 :
        return v_dloss_ddepdt_nd.transpose(2, 0, 1).reshape(v_indepdt_len[2], v_indepdt_len[0], v_indepdt_len[1]).transpose(1, 2, 0).tolist()
      
    elif np.array_equal(v_dims_from_to, np.array([0, 2])) :
      if v_dim_pin_ele == 0 :
        return v_dloss_ddepdt_nd.transpose(1, 2, 0).reshape(v_indepdt_len[1], v_indepdt_len[2], v_indepdt_len[0]).transpose(2, 0, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_dloss_ddepdt_nd.transpose(1, 0, 2).reshape(v_indepdt_len[1], v_indepdt_len[0], v_indepdt_len[2]).transpose(1, 0, 2).tolist()
      
    elif np.array_equal(v_dims_from_to, np.array([1, 2])) :
      if v_dim_pin_ele == 1 :
        return v_dloss_ddepdt_nd.reshape(v_indepdt_len[0], v_indepdt_len[2], v_indepdt_len[1]).transpose(0, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_dloss_ddepdt_nd.reshape(v_indepdt_len[0], v_indepdt_len[1], v_indepdt_len[2]).tolist()
    
  elif v_dloss_ddepdt_nd.ndim == 4 :
    if np.array_equal(v_dims_from_to, np.array([0, 1])) :
      if v_dim_pin_ele == 0 :
        return v_dloss_ddepdt_nd.transpose(2, 3, 1, 0).reshape(v_indepdt_len[2], v_indepdt_len[3], v_indepdt_len[1], v_indepdt_len[0]).transpose(3, 2, 0, 1).tolist()
      elif v_dim_pin_ele == 1 :
        return v_dloss_ddepdt_nd.transpose(2, 3, 0, 1).reshape(v_indepdt_len[2], v_indepdt_len[3], v_indepdt_len[0], v_indepdt_len[1]).transpose(2, 3, 0, 1).tolist()
      
    elif np.array_equal(v_dims_from_to, np.array([0, 2])) :
      if v_dim_pin_ele == 0 :
        return v_dloss_ddepdt_nd.transpose(1, 3, 2, 0).reshape(v_indepdt_len[1], v_indepdt_len[3], v_indepdt_len[2], v_indepdt_len[0]).transpose(3, 0, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_dloss_ddepdt_nd.transpose(1, 3, 0, 2).reshape(v_indepdt_len[1], v_indepdt_len[3], v_indepdt_len[0], v_indepdt_len[2]).transpose(2, 0, 3, 1).tolist()
      
    elif np.array_equal(v_dims_from_to, np.array([0, 3])) :
      if v_dim_pin_ele == 0 :
        return v_dloss_ddepdt_nd.transpose(1, 2, 3, 0).reshape(v_indepdt_len[1], v_indepdt_len[2], v_indepdt_len[3], v_indepdt_len[0]).transpose(3, 0, 1, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_dloss_ddepdt_nd.transpose(1, 2, 0, 3).reshape(v_indepdt_len[1], v_indepdt_len[2], v_indepdt_len[0], v_indepdt_len[3]).transpose(2, 0, 1, 3).tolist()
      
    elif np.array_equal(v_dims_from_to, np.array([1, 2])) :
      if v_dim_pin_ele == 1 :
        return v_dloss_ddepdt_nd.transpose(0, 3, 2, 1).reshape(v_indepdt_len[0], v_indepdt_len[3], v_indepdt_len[2], v_indepdt_len[1]).transpose(0, 3, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_dloss_ddepdt_nd.transpose(0, 3, 1, 2).reshape(v_indepdt_len[0], v_indepdt_len[3], v_indepdt_len[1], v_indepdt_len[2]).transpose(0, 2, 3, 1).tolist()

    elif np.array_equal(v_dims_from_to, np.array([1, 3])) :
      if v_dim_pin_ele == 1 :
        return v_dloss_ddepdt_nd.transpose(0, 2, 3, 1).reshape(v_indepdt_len[0], v_indepdt_len[2], v_indepdt_len[3], v_indepdt_len[1]).transpose(0, 3, 1, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_dloss_ddepdt_nd.transpose(0, 2, 1, 3).reshape(v_indepdt_len[0], v_indepdt_len[2], v_indepdt_len[1], v_indepdt_len[3]).transpose(0, 2, 1, 3).tolist()

    elif np.array_equal(v_dims_from_to, np.array([2, 3])) :
      if v_dim_pin_ele == 2 :
        return v_dloss_ddepdt_nd.reshape(v_indepdt_len[0], v_indepdt_len[1], v_indepdt_len[3], v_indepdt_len[2]).transpose(0, 1, 3, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_dloss_ddepdt_nd.reshape(v_indepdt_len[0], v_indepdt_len[1], v_indepdt_len[2], v_indepdt_len[3]).tolist()
        
$$
language plpython3u stable
parallel safe
;

-- with 
-- cte_mx_ele_flatten_2dims as 
-- (
--   select 
--     a_arr
--   , a_dim_from
--   , a_dim_to
--   , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx] as a_dim_pin_ele
--   , sm_sc.fv_mx_ele_flatten_2dims
--     (
--       a_arr
--     , array[a_dim_from, a_dim_to]
--     , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx]
--     ) as a_flattened_dims
--   from 
--     (
--                 select sm_sc.fv_new_rand(array[4,3    ]) as a_arr
--       union all select sm_sc.fv_new_rand(array[4,3,5  ]) as a_arr
--       union all select sm_sc.fv_new_rand(array[4,3,5,7]) as a_arr
--     ) tb_a_arr(a_arr)
--   , generate_series(1, 4) tb_a_dim_from(a_dim_from)
--   , generate_series(1, 4) tb_a_dim_to(a_dim_to)
--   , generate_series(1, 2) tb_a_dim_pin_ele_idx(a_dim_pin_ele_idx)
--   where a_dim_from <= array_ndims(a_arr)
--     and a_dim_to <= array_ndims(a_arr)
--     and a_dim_from <> a_dim_to
--   order by array_ndims(a_arr), a_dim_from, a_dim_to, a_dim_pin_ele_idx
-- )
-- select 
--   sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt_py
--   (
--     a_flattened_dims
--   , array[a_dim_from, a_dim_to]
--   , a_dim_pin_ele
--   , (select array_agg(array_length(a_arr, a_no) order by a_no) from generate_series(1, array_ndims(a_arr)) tb_a_no(a_no))
--   ) = a_arr
-- from cte_mx_ele_flatten_2dims
-- drop function if exists sm_sc.fv_d_mx_slice_3d_2_2d_dloss_dindepdt(float[], int[], int, int);
create or replace function sm_sc.fv_d_mx_slice_3d_2_2d_dloss_dindepdt
(
  i_dloss_dindepdt  float[]  
, i_indepdt_len     int[3]                  -- 自变量规格
, i_dim_sliced      int                     -- 被切片维度
, i_slice_pos       int     default  1      -- 被切片位置序号
)
returns float[]
as
$$
declare
  v_ret      i_dloss_dindepdt%type;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_dim_sliced not between 1 and 3
    then 
      raise exception 'i_dim_sliced is out of length.';
    elsif array_ndims(i_dloss_dindepdt) <> 2
    then 
      raise exception 'ndims of dloss_dindepdt should be 2.';
    elsif array[array_length(i_dloss_dindepdt, 1), array_length(i_dloss_dindepdt, 2)] <> (i_indepdt_len[ : i_dim_sliced - 1] || i_indepdt_len[i_dim_sliced + 1 : ])
    then 
      raise exception 'len of i_dloss_dindepdt should match with i_indepdt_len by i_dim_sliced.';
    elsif i_slice_pos not between 1 and i_indepdt_len[i_dim_sliced]
    then 
      raise exception 'i_slice_pos is out of i_indepdt_len[i_dim_sliced].';
    end if;
  end if;

  v_ret := array_fill(0.0 , i_indepdt_len);
  
  if i_dim_sliced = 1
  then 
    v_ret[i_slice_pos : i_slice_pos][ : ][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = 2
  then 
    v_ret[ : ][i_slice_pos : i_slice_pos][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = 3
  then 
    v_ret[ : ][ : ][i_slice_pos : i_slice_pos] := i_dloss_dindepdt;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_d_mx_slice_3d_2_2d_dloss_dindepdt
--   (
--     (
--       array
--       [
--         [1, 2, 3, 4, 5]     ,
--         [11, 12, 13, 14, 15],
--         [21, 22, 23, 24, 25], 
--         [31, 32, 33, 34, 35] 
--       ]
--     )
--     , array[3, 4, 5]
--     , 1
--     , 2
--   )
-- drop function if exists sm_sc.fv_d_mx_slice_4d_2_2d_dloss_dindepdt(float[], int[], int[], int[]);
create or replace function sm_sc.fv_d_mx_slice_4d_2_2d_dloss_dindepdt
(
  i_dloss_dindepdt  float[]  
, i_indepdt_len     int[4]                  -- 自变量规格
, i_dim_sliced      int[2]                -- 被切片维度
, i_slice_pos       int[2]                -- 被切片在 i_dim_sliced 各对应维度的位置序号。
)
returns float[]
as
$$
declare
  v_ret      i_dloss_dindepdt%type;
  v_buff     int;
begin
  if i_dim_sliced[1] > i_dim_sliced[2]
  then 
    v_buff := i_dim_sliced[1];
    i_dim_sliced[1] := i_dim_sliced[2];
    i_dim_sliced[2] := v_buff;
    
    v_buff := i_slice_pos[1];
    i_slice_pos[1] := i_slice_pos[2];
    i_slice_pos[2] := v_buff;
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_dim_sliced[1] not between 1 and 4 
      or i_dim_sliced[2] not between 1 and 4 
      or i_dim_sliced[1] = i_dim_sliced[2]
    then 
      raise exception 'i_dim_sliced is out of length.';
    elsif array_ndims(i_dloss_dindepdt) <> 2
    then 
      raise exception 'ndims of dloss_dindepdt should be 2.';
    elsif array[array_length(i_dloss_dindepdt, 1), array_length(i_dloss_dindepdt, 2)] <> (i_indepdt_len[ : i_dim_sliced[1] - 1] || i_indepdt_len[i_dim_sliced[1] + 1 : i_dim_sliced[2] - 1] || i_indepdt_len[i_dim_sliced + 1 : ])
    then 
      raise exception 'len of i_dloss_dindepdt should match with i_indepdt_len by i_dim_sliced.';
    elsif i_slice_pos[1] not between 1 and i_indepdt_len[i_dim_sliced[1]]
      or i_slice_pos[2] not between 1 and i_indepdt_len[i_dim_sliced[2]]
    then 
      raise exception 'i_slice_pos is out of i_indepdt_len[i_dim_sliced].';
    end if;
  end if;

  v_ret := array_fill(0.0 , i_indepdt_len);
  
  if i_dim_sliced = array[1, 2]
  then 
    v_ret[i_slice_pos[1] : i_slice_pos[1]][i_slice_pos[2] : i_slice_pos[2]][ : ][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = array[1, 3]
  then 
    v_ret[i_slice_pos[1] : i_slice_pos[1]][ : ][i_slice_pos[2] : i_slice_pos[2]][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = array[1, 4]
  then 
    v_ret[i_slice_pos[1] : i_slice_pos[1]][ : ][ : ][i_slice_pos[2] : i_slice_pos[2]] := i_dloss_dindepdt;
  elsif i_dim_sliced = array[2, 3]
  then 
    v_ret[ : ][i_slice_pos[1] : i_slice_pos[1]][i_slice_pos[2] : i_slice_pos[2]][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = array[2, 4]
  then 
    v_ret[ : ][i_slice_pos[1] : i_slice_pos[1]][ : ][i_slice_pos[2] : i_slice_pos[2]] := i_dloss_dindepdt;
  elsif i_dim_sliced = array[3, 4]
  then 
    v_ret[ : ][ : ][i_slice_pos[1] : i_slice_pos[1]][i_slice_pos[2] : i_slice_pos[2]] := i_dloss_dindepdt;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_d_mx_slice_4d_2_2d_dloss_dindepdt
--   (
--     (
--       array
--       [[
--         [1, 2, 3, 4, 5]     ,
--         [11, 12, 13, 14, 15],
--         [21, 22, 23, 24, 25], 
--         [31, 32, 33, 34, 35] 
--       ]]
--     )
--     , array[2, 3, 4, 5]
--     , array[1, 2]
--     , array[2, 3]
--   )
-- drop function if exists sm_sc.fv_d_mx_slice_4d_2_3d_dloss_dindepdt(float[], int[], int, int);
create or replace function sm_sc.fv_d_mx_slice_4d_2_3d_dloss_dindepdt
(
  i_dloss_dindepdt  float[]  
, i_indepdt_len     int[4]                  -- 自变量规格
, i_dim_sliced      int                     -- 被切片维度
, i_slice_pos       int     default  1      -- 被切片位置序号
)
returns float[]
as
$$
declare
  v_ret      i_dloss_dindepdt%type;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_dim_sliced not between 1 and 4
    then 
      raise exception 'i_dim_sliced is out of length.';
    elsif array_ndims(i_dloss_dindepdt) <> 3
    then 
      raise exception 'ndims of dloss_dindepdt should be 3.';
    elsif array[array_length(i_dloss_dindepdt, 1), array_length(i_dloss_dindepdt, 2), array_length(i_dloss_dindepdt, 3)] <> (i_indepdt_len[ : i_dim_sliced - 1] || i_indepdt_len[i_dim_sliced + 1 : ])
    then 
      raise exception 'len of i_dloss_dindepdt should match with i_indepdt_len by i_dim_sliced.';
    elsif i_slice_pos not between 1 and i_indepdt_len[i_dim_sliced]
    then 
      raise exception 'i_slice_pos is out of i_indepdt_len[i_dim_sliced].';
    end if;
  end if;

  v_ret := array_fill(0.0 , i_indepdt_len);
  
  if i_dim_sliced = 1
  then 
    v_ret[i_slice_pos : i_slice_pos][ : ][ : ][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = 2
  then 
    v_ret[ : ][i_slice_pos : i_slice_pos][ : ][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = 3
  then 
    v_ret[ : ][ : ][i_slice_pos : i_slice_pos][ : ] := i_dloss_dindepdt;
  elsif i_dim_sliced = 4
  then 
    v_ret[ : ][ : ][ : ][i_slice_pos : i_slice_pos] := i_dloss_dindepdt;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_d_mx_slice_4d_2_3d_dloss_dindepdt
--   (
--     (
--       array
--       [[
--         [1, 2, 3, 4, 5]     ,
--         [11, 12, 13, 14, 15],
--         [21, 22, 23, 24, 25], 
--         [31, 32, 33, 34, 35] 
--       ]]
--     )
--     , array[2, 1, 4, 5]
--     , 1
--     , 2
--   )
-- drop function if exists sm_sc.fv_d_new_dloss_dindepdt(float[], int);
create or replace function sm_sc.fv_d_new_dloss_dindepdt
(
  i_dloss_ddepdt             float[]    ,
  i_dims_times               int[]
)
returns float[]    -- 返回值为 2d
as
$$
declare 
  v_depdt_var_len_y   int;
  v_depdt_var_len_x   int;
  v_depdt_var_len_x3  int;
  v_depdt_var_len_x4  int;
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dims_times) <> 1
    then 
      raise exception 'unsupport ndims of i_dims_times.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_dims_times, 1)
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_dims_times.';
    elsif array_ndims(i_dloss_ddepdt) not between 1 and 4
    then
      raise exception 'no method for ndims of this i_dloss_ddepdt!';
    elsif 0 < any(select array_agg(array_length(i_dloss_ddepdt, a_ndim) / i_dims_times order by a_ndim)
                  from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_ndims(a_ndim))
    then 
      raise exception 'i_dims_times is unperfect for len of indepdt_var.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) is null
  then 
    return array[] :: i_dloss_ddepdt % type;
  elsif array_length(i_dims_times, 1) = 1
  then
    v_depdt_var_len_y := array_length(i_dloss_ddepdt, 1) / i_dims_times[1];
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt
          [v_depdt_var_len_y * (a_y_no - 1) + 1 : v_depdt_var_len_y * a_y_no]
        ) 
      from generate_series(1, i_dims_times[1]) tb_a_y(a_y_no)
    );
  elsif array_length(i_dims_times, 1) = 2
  then
    v_depdt_var_len_y := array_length(i_dloss_ddepdt, 1) / i_dims_times[1];
    v_depdt_var_len_x := array_length(i_dloss_ddepdt, 2) / i_dims_times[2];
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt
          [v_depdt_var_len_y * (a_y_no - 1) + 1 : v_depdt_var_len_y * a_y_no]
          [v_depdt_var_len_x * (a_x_no - 1) + 1 : v_depdt_var_len_x * a_x_no]
        ) 
      from generate_series(1, i_dims_times[1]) tb_a_y(a_y_no)
        , generate_series(1, i_dims_times[2]) tb_a_x(a_x_no)
    );
  elsif array_length(i_dims_times, 1) = 3
  then
    v_depdt_var_len_y  := array_length(i_dloss_ddepdt, 1) / i_dims_times[1];
    v_depdt_var_len_x  := array_length(i_dloss_ddepdt, 2) / i_dims_times[2];
    v_depdt_var_len_x3 := array_length(i_dloss_ddepdt, 3) / i_dims_times[3];
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt
          [v_depdt_var_len_y * (a_y_no - 1) + 1 : v_depdt_var_len_y * a_y_no]
          [v_depdt_var_len_x * (a_x_no - 1) + 1 : v_depdt_var_len_x * a_x_no]
          [v_depdt_var_len_x3 * (a_x3_no - 1) + 1 : v_depdt_var_len_x3 * a_x3_no]
        ) 
      from generate_series(1, i_dims_times[1]) tb_a_y(a_y_no)
        , generate_series(1, i_dims_times[2]) tb_a_x(a_x_no)
        , generate_series(1, i_dims_times[3]) tb_a_x3(a_x3_no)
    );
  elsif array_length(i_dims_times, 1) = 4
  then
    v_depdt_var_len_y   := array_length(i_dloss_ddepdt, 1) / i_dims_times[1];
    v_depdt_var_len_x   := array_length(i_dloss_ddepdt, 2) / i_dims_times[2];
    v_depdt_var_len_x3  := array_length(i_dloss_ddepdt, 3) / i_dims_times[3];
    v_depdt_var_len_x4  := array_length(i_dloss_ddepdt, 4) / i_dims_times[4];
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt
          [v_depdt_var_len_y * (a_y_no - 1) + 1 : v_depdt_var_len_y * a_y_no]
          [v_depdt_var_len_x * (a_x_no - 1) + 1 : v_depdt_var_len_x * a_x_no]
          [v_depdt_var_len_x3 * (a_x3_no - 1) + 1 : v_depdt_var_len_x3 * a_x3_no]
          [v_depdt_var_len_x4 * (a_x4_no - 1) + 1 : v_depdt_var_len_x4 * a_x4_no]
        ) 
      from generate_series(1, i_dims_times[1]) tb_a_y(a_y_no)
        , generate_series(1, i_dims_times[2]) tb_a_x(a_x_no)
        , generate_series(1, i_dims_times[3]) tb_a_x3(a_x3_no)
        , generate_series(1, i_dims_times[4]) tb_a_x4(a_x4_no)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_new_dloss_dindepdt
--   (
--     array[[0.5 :: float, -0.2, 0.25, 1.3, -0.9, -0.1], [-0.5 :: float, -0.2, -0.25, 1.3, 0.9, 0.1], [0.5 :: float, 0.2, 0.25, -1.3, 0.9, 0.05]],
--     array[1, 3]
--   );
-- select sm_sc.fv_d_new_dloss_dindepdt
--   (
--     array[[0.5 :: float, -0.2, 0.25, 1.3, -0.9], [-0.5 :: float, -0.2, -0.25, 1.3, 0.9], [0.5 :: float, 0.2, 0.25, -1.3, 0.9], [0.2, 0.25, -1.3, 0.9, 0.05]],
--     array[2, 1]
--   );
-- select sm_sc.fv_d_new_dloss_dindepdt
--   (
--     array[[0.5 :: float, -0.2, 0.25, 1.3, -0.9, -0.25], [-0.5 :: float, -0.2, -0.25, 1.3, 0.9, -0.75], [0.5 :: float, 0.2, 0.25, -1.3, 0.9, -0.65], [0.9, -1.3, -0.25, 1.3, 0.9, 0.75]],
--     array[2, 3]
--   );
-- select sm_sc.fv_d_new_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[4, 6, 9])
--   , array[2, 2, 3]
--   );
-- select sm_sc.fv_d_new_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[10, 4, 6, 9])
--   , array[5, 2, 2, 3]
--   );
-- drop function if exists sm_sc.fv_d_repeat_axis_dloss_dindepdt(float[], int[], int[]);
create or replace function sm_sc.fv_d_repeat_axis_dloss_dindepdt
(
  i_dloss_ddepdt     float[]
, i_dims             int[]  
, i_repeats          int[]  
)
returns float[]    -- 返回值为 2d
as
$$
declare 
  v_dims             int[]    := i_dims %` array_ndims(i_dloss_ddepdt);
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt) not between 1 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif |@|`| ((@|` i_dims) >` array_ndims(i_dloss_ddepdt))   -- 判断 i_dims 是否有越界 i_dloss_ddepdt 的维度
    then 
      raise exception 'i_dims cross the border of i_dloss_ddepdt'' ndims.';
    elsif array_ndims(i_dims) <> array_ndims(i_repeats)
    then 
      raise exception 'unmatch ndims for i_dims and i_repeats';
    end if;
  end if;
  
  return 
    sm_sc.fv_aggr_slice_sum
    (
      i_dloss_ddepdt
    , (
        select 
          array_agg(coalesce(a_repeat, 1) order by a_dim_no)
        from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_dim_no(a_dim_no)
        left join unnest(i_dims, i_repeats) tb_a_axis_no(a_axis_no, a_repeat)
          on tb_a_axis_no.a_axis_no = tb_a_dim_no.a_dim_no
      )
    )
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_d_repeat_axis_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[6,9,10,8])
--   , array[2, 3]
--   , array[3, 5]
--   );
-- select 
--   sm_sc.fv_d_repeat_axis_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[6,10,8])
--   , array[1, 3]
--   , array[2, 4]
--   );
-- select 
--   sm_sc.fv_d_repeat_axis_dloss_dindepdt
--   (
--     sm_sc.fv_new_rand(array[10,8])
--   , array[1]
--   , array[2]
--   );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_rmask_dloss_dindepdt_1(float[], int[]);
create or replace function sm_sc.fv_d_rmask_dloss_dindepdt_1
(
  i_dloss_ddepdt            float[]            ,
  i_co_value                int[]                 -- i_mask_len
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_rmask(i_dloss_ddepdt, i_co_value, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_rmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ] :: float[]
--   , array[[3], [0], [2], [1]]
--   )

-- select 
--   sm_sc.fv_d_rmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ] :: float[]
--   , array[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]
--   )

-- select 
--   sm_sc.fv_d_rmask_dloss_dindepdt_1
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ] :: float[]
--   , array[[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]],[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_rpad_dloss_dindepdt_1(float[], int);
create or replace function sm_sc.fv_d_rpad_dloss_dindepdt_1
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_width        int
)
returns float[]
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
    
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return 
      i_dloss_ddepdt[ : ][1 : i_indepdt_width]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
      i_dloss_ddepdt[ : ][ : ][1 : i_indepdt_width]
    ;
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
      i_dloss_ddepdt[ : ][ : ][ : ][1 : i_indepdt_width]
    ;
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_rpad_dloss_dindepdt_1
--   (
--     array[[1, 2, 3, 4, 5],[11, 12, 13, 14, 15],[21, 22, 23, 24, 25],[31, 32, 33, 34, 35],[41, 42, 43, 44, 45]]
--   , 2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_rpad_dloss_dindepdt_2(float[], int, int);
create or replace function sm_sc.fv_d_rpad_dloss_dindepdt_2
(
  i_dloss_ddepdt         float[]    ,
  i_indepdt_width        int      ,
  i_times                int        default  1   
)
returns float[]
as
$$
declare 
  v_len     int       := array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1);
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][v_len - (a_cur * i_indepdt_width) + 1 : v_len - ((a_cur - 1) * i_indepdt_width)])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][v_len - (a_cur * i_indepdt_width) + 1 : v_len - ((a_cur - 1) * i_indepdt_width)])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][ : ][v_len - (a_cur * i_indepdt_width) + 1 : v_len - ((a_cur - 1) * i_indepdt_width)])
      from generate_series(1, i_times) tb_a(a_cur)
    );
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_rpad_dloss_dindepdt_2
--   (
--     array[1, 2, 3, 4, 5, 6],
--     array[7, 8],
--     2
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_sample_x3_dloss_dindepdt_1(float[], int, int, int4range[]);
create or replace function sm_sc.fv_d_sample_x3_dloss_dindepdt_1
(
  i_dloss_ddepdt   float[]      ,                -- 损失函数对因变量的导数
  i_array_len_x3    int          ,               -- 自变量数组在采样维度的长度
  -- i_period         int          ,                -- 采样周期长度
  i_window_len     int          ,                -- 约束窗口宽度
  i_simp_ranges    int4range[]                   -- 采样(多)区间
  
)
returns float[]
as
$$
declare 
  v_ret            float[]  ;
  v_period         int      ;      -- i_array 采样周期
  v_window_len_ex  int  :=         -- 生成 i_dloss_ddepdt 的周期
    (
      select 
        max(upper(a_range * int4range(1, i_window_len, '[]'))) 
        - min(lower(a_range * int4range(1, i_window_len, '[]')))
      from unnest(i_simp_ranges) tb_a_range(a_range)
    )
  ;
  v_cur_1          int      ;
  v_cur_2          int      ;
  v_arr_idxs       int[]    ;    -- 采样到的切片下标集合
  v_dd_idxss       text[]   ;    -- 采样到的每个单元切片对应导数下标集合，再所有单元切片 array_agg 集合，相当于不整齐的二维 int 数组。
  -- v_zero_slice     float[]  ;    -- 零切片
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 3 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif array_length(i_dloss_ddepdt, 3) % v_window_len_ex > 0
      or i_array_len_x3 / (array_length(i_dloss_ddepdt, 3) % v_window_len_ex) > 0
    then 
      raise exception 'unmatch 3d length of i_dloss_ddepdt, simp_range or i_array_len for period.';
    end if;
  end if;
  
  -- 窗口约束重整采样区间
  for v_cur_1 in 1 .. array_length(i_simp_ranges, 1)
  loop
    i_simp_ranges[v_cur_1] := i_simp_ranges[v_cur_1] * int4range(1, i_window_len, '[]');
  end loop;
  
  -- 换算出 i_array 周期
  v_period  :=   i_array_len_x3 / (array_length(i_dloss_ddepdt, 3) / v_window_len_ex);
  
  if array_ndims(i_dloss_ddepdt) = 3
  then
    v_ret     :=   array_fill(0.0, array[array_length(i_dloss_ddepdt, 1)] || array_length(i_dloss_ddepdt, 2) || i_array_len_x3);
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    v_ret     :=   array_fill(0.0, array[array_length(i_dloss_ddepdt, 1)] || array_length(i_dloss_ddepdt, 2) || i_array_len_x3 || array_length(i_dloss_ddepdt, 4));
  end if;

  with recursive
  -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
  cte_range_dd_idx as 
  (
    select 
      1 as a_range_idx,
      lower(i_simp_ranges[1])                           as a_lower,
      upper(i_simp_ranges[1]) - lower(i_simp_ranges[1]) as a_range_len,
      1 as a_range_dd_pos
    union all 
    select 
      a_range_idx + 1 as a_range_idx,
      lower(i_simp_ranges[a_range_idx + 1])                                         as a_lower,
      upper(i_simp_ranges[a_range_idx + 1]) - lower(i_simp_ranges[a_range_idx + 1]) as a_range_len,
      a_range_dd_pos + a_range_len as a_range_dd_pos
    from cte_range_dd_idx
    where a_range_idx < array_length(i_simp_ranges, 1)
  ),
  -- 算出区间与 i_arr 的下标对应关系
  cte_range_arr_idx as 
  (
    select 
      a_arr_idx,
      a_range_idx
    from generate_series(1, array_length(i_simp_ranges, 1)) tb_a_range_idx(a_range_idx)
      , generate_series
        (
          lower(i_simp_ranges[a_range_idx])
        , upper(i_simp_ranges[a_range_idx]) - 1
        ) tb_a_cur_idx(a_arr_idx)
    -- where a_arr_idx <@ i_simp_ranges[a_range_idx]
  ),
  -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 单位切片为分组，agg后，记录一对多关系，且聚合滤掉未被采样位置。
  cte_map_arr_idx_dd_idx_per_period as
  (
    select 
      tb_a_arr_idx.a_arr_idx,
      array_agg
      (
        tb_a_dd_idx.a_range_dd_pos 
        + tb_a_arr_idx.a_arr_idx 
        - tb_a_dd_idx.a_lower 
        -- order by tb_a_dd_idx.a_range_dd_pos
      ) :: text as a_dd_idxs
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
    having count(tb_a_dd_idx.a_range_dd_pos) > 0
  )
  select 
    array_agg(a_arr_idx order by a_arr_idx),
    array_agg(a_dd_idxs order by a_arr_idx)
  into 
    v_arr_idxs,
    v_dd_idxss
  from cte_map_arr_idx_dd_idx_per_period
  ;
  
  -- 逐个周期，以单位切片为分组，合计 dloss_ddepdt，赋值到 v_ret 对应下标位置，未被采样切片，返回零切片为导数
  for v_cur_1 in 1 .. (array_length(i_dloss_ddepdt, 3) / v_window_len_ex)
  loop 
    for v_cur_2 in 1 .. array_length(v_arr_idxs, 1)
    loop 
      if (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] between 1 and array_length(v_ret, 3)
      then 
        v_ret[ : ][ : ][(v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] : (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2]] :=
        (
          select 
            sm_sc.fa_mx_sum
            (
              i_dloss_ddepdt
              [ : ]
              [ : ]
              [
                (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx] 
              : (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx]
              ]
            )
          from generate_series(1, array_length((v_dd_idxss[v_cur_2] :: int[]), 1)) tb_a_cur_dd_idx(a_cur_dd_idx)
        );
      end if;
    end loop;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_sample_x3_dloss_dindepdt_1
--   (
--     array
--     [[
--       [1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , [11, 12, 13, 14, 15, 16 ,17 ,18 ,19]
--     ]]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );

-- select sm_sc.fv_d_sample_x3_dloss_dindepdt_1
--   (
--     array
--     [[[
--       [1, 2, 3, 4]
--     , [11, 12, 13, 14]
--     , [21, 22, 23, 24]

--     , [31, 32, 33, 34]
--     , [41, 42, 43, 44]

--     , [51, 52, 53, 54]
--     , [61, 62, 63, 64]
--     , [71, 72, 73, 74]

--     , [81, 82, 83, 84]
--     , [91, 92, 93, 94]
--     ]]]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_sample_x4_dloss_dindepdt_1(float[], int, int, int4range[]);
create or replace function sm_sc.fv_d_sample_x4_dloss_dindepdt_1
(
  i_dloss_ddepdt   float[]      ,                -- 损失函数对因变量的导数
  i_array_len_x4    int          ,               -- 自变量数组在采样维度的长度
  -- i_period         int          ,                -- 采样周期长度
  i_window_len     int          ,                -- 约束窗口宽度
  i_simp_ranges    int4range[]                   -- 采样(多)区间
  
)
returns float[]
as
$$
declare 
  v_ret            float[]  ;
  v_period         int      ;      -- i_array 采样周期
  v_window_len_ex  int  :=         -- 生成 i_dloss_ddepdt 的周期
    (
      select 
        max(upper(a_range * int4range(1, i_window_len, '[]'))) 
        - min(lower(a_range * int4range(1, i_window_len, '[]')))
      from unnest(i_simp_ranges) tb_a_range(a_range)
    )
  ;
  v_cur_1          int      ;
  v_cur_2          int      ;
  v_arr_idxs       int[]    ;    -- 采样到的切片下标集合
  v_dd_idxss       text[]   ;    -- 采样到的每个单元切片对应导数下标集合，再所有单元切片 array_agg 集合，相当于不整齐的二维 int 数组。
  -- v_zero_slice     float[]  ;    -- 零切片
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) <> 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif array_length(i_dloss_ddepdt, 4) % v_window_len_ex > 0
      or i_array_len_x4 / (array_length(i_dloss_ddepdt, 4) % v_window_len_ex) > 0
    then 
      raise exception 'unmatch 4d length of i_dloss_ddepdt, simp_range or i_array_len for period.';
    end if;
  end if;
  
  -- 窗口约束重整采样区间
  for v_cur_1 in 1 .. array_length(i_simp_ranges, 1)
  loop
    i_simp_ranges[v_cur_1] := i_simp_ranges[v_cur_1] * int4range(1, i_window_len, '[]');
  end loop;
  
  -- 换算出 i_array 周期
  v_period  :=   i_array_len_x4 / (array_length(i_dloss_ddepdt, 4) / v_window_len_ex);
  
  v_ret     :=   array_fill(0.0, array[array_length(i_dloss_ddepdt, 1)] || array_length(i_dloss_ddepdt, 2) || array_length(i_dloss_ddepdt, 3) || i_array_len_x4);

  with recursive
  -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
  cte_range_dd_idx as 
  (
    select 
      1 as a_range_idx,
      lower(i_simp_ranges[1])                           as a_lower,
      upper(i_simp_ranges[1]) - lower(i_simp_ranges[1]) as a_range_len,
      1 as a_range_dd_pos
    union all 
    select 
      a_range_idx + 1 as a_range_idx,
      lower(i_simp_ranges[a_range_idx + 1])                                         as a_lower,
      upper(i_simp_ranges[a_range_idx + 1]) - lower(i_simp_ranges[a_range_idx + 1]) as a_range_len,
      a_range_dd_pos + a_range_len as a_range_dd_pos
    from cte_range_dd_idx
    where a_range_idx < array_length(i_simp_ranges, 1)
  ),
  -- 算出区间与 i_arr 的下标对应关系
  cte_range_arr_idx as 
  (
    select 
      a_arr_idx,
      a_range_idx
    from generate_series(1, array_length(i_simp_ranges, 1)) tb_a_range_idx(a_range_idx)
      , generate_series
        (
          lower(i_simp_ranges[a_range_idx])
        , upper(i_simp_ranges[a_range_idx]) - 1
        ) tb_a_cur_idx(a_arr_idx)
    -- where a_arr_idx <@ i_simp_ranges[a_range_idx]
  ),
  -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 单位切片为分组，agg后，记录一对多关系，且聚合滤掉未被采样位置。
  cte_map_arr_idx_dd_idx_per_period as
  (
    select 
      tb_a_arr_idx.a_arr_idx,
      array_agg
      (
        tb_a_dd_idx.a_range_dd_pos 
        + tb_a_arr_idx.a_arr_idx 
        - tb_a_dd_idx.a_lower 
        -- order by tb_a_dd_idx.a_range_dd_pos
      ) :: text as a_dd_idxs
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
    having count(tb_a_dd_idx.a_range_dd_pos) > 0
  )
  select 
    array_agg(a_arr_idx order by a_arr_idx),
    array_agg(a_dd_idxs order by a_arr_idx)
  into 
    v_arr_idxs,
    v_dd_idxss
  from cte_map_arr_idx_dd_idx_per_period
  ;
  
  -- 逐个周期，以单位切片为分组，合计 dloss_ddepdt，赋值到 v_ret 对应下标位置，未被采样切片，返回零切片为导数
  for v_cur_1 in 1 .. (array_length(i_dloss_ddepdt, 4) / v_window_len_ex)
  loop 
    for v_cur_2 in 1 .. array_length(v_arr_idxs, 1)
    loop 
      if (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] between 1 and array_length(v_ret, 4)
      then 
        v_ret[ : ][ : ][ : ][(v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] : (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2]] :=
        (
          select 
            sm_sc.fa_mx_sum
            (
              i_dloss_ddepdt
              [ : ]
              [ : ]
              [ : ]
              [
                (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx] 
              : (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx]
              ]
            )
          from generate_series(1, array_length((v_dd_idxss[v_cur_2] :: int[]), 1)) tb_a_cur_dd_idx(a_cur_dd_idx)
        );
      end if;
    end loop;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_sample_x4_dloss_dindepdt_1
--   (
--     array
--     [[[
--       [1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , [11, 12, 13, 14, 15, 16 ,17 ,18 ,19]
--     ]]]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_sample_x_dloss_dindepdt_1(float[], int, int, int4range[]);
create or replace function sm_sc.fv_d_sample_x_dloss_dindepdt_1
(
  i_dloss_ddepdt   float[]      ,                -- 损失函数对因变量的导数
  i_array_len_x    int          ,                -- 自变量数组在采样维度的长度
  -- i_period         int          ,                -- 采样周期长度
  i_window_len     int          ,                -- 约束窗口宽度
  i_simp_ranges    int4range[]                   -- 采样(多)区间
  
)
returns float[]
as
$$
declare 
  v_ret            float[]  ;
  v_period         int      ;      -- i_array 采样周期
  v_window_len_ex  int  :=         -- 生成 i_dloss_ddepdt 的周期
    (
      select 
        max(upper(a_range * int4range(1, i_window_len, '[]'))) 
        - min(lower(a_range * int4range(1, i_window_len, '[]')))
      from unnest(i_simp_ranges) tb_a_range(a_range)
    )
  ;
  v_cur_1          int      ;
  v_cur_2          int      ;
  v_arr_idxs       int[]    ;    -- 采样到的切片下标集合
  v_dd_idxss       text[]   ;    -- 采样到的每个单元切片对应导数下标集合，再所有单元切片 array_agg 集合，相当于不整齐的二维 int 数组。
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif array_length(i_dloss_ddepdt, 2) % v_window_len_ex > 0
      or i_array_len_x / (array_length(i_dloss_ddepdt, 2) % v_window_len_ex) > 0
    then 
      raise exception 'unmatch 2d length of i_dloss_ddepdt, simp_range or i_array_len for period.';
    end if;
  end if;
  
  -- 窗口约束重整采样区间
  for v_cur_1 in 1 .. array_length(i_simp_ranges, 1)
  loop
    i_simp_ranges[v_cur_1] := i_simp_ranges[v_cur_1] * int4range(1, i_window_len, '[]');
  end loop;
  
  -- 换算出 i_array 周期
  v_period  :=   i_array_len_x / (array_length(i_dloss_ddepdt, 2) / v_window_len_ex);
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    v_ret     :=   array_fill(0.0, array_length(i_dloss_ddepdt, 1) || array[i_array_len_x]);
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    v_ret     :=   array_fill(0.0, array_length(i_dloss_ddepdt, 1) || array[i_array_len_x] || array_length(i_dloss_ddepdt, 3));
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    v_ret     :=   array_fill(0.0, array_length(i_dloss_ddepdt, 1) || array[i_array_len_x] || array_length(i_dloss_ddepdt, 3) || array_length(i_dloss_ddepdt, 4));
  end if;

  with recursive
  -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
  cte_range_dd_idx as 
  (
    select 
      1 as a_range_idx,
      lower(i_simp_ranges[1])                           as a_lower,
      upper(i_simp_ranges[1]) - lower(i_simp_ranges[1]) as a_range_len,
      1 as a_range_dd_pos
    union all 
    select 
      a_range_idx + 1 as a_range_idx,
      lower(i_simp_ranges[a_range_idx + 1])                                         as a_lower,
      upper(i_simp_ranges[a_range_idx + 1]) - lower(i_simp_ranges[a_range_idx + 1]) as a_range_len,
      a_range_dd_pos + a_range_len as a_range_dd_pos
    from cte_range_dd_idx
    where a_range_idx < array_length(i_simp_ranges, 1)
  ),
  -- 算出区间与 i_arr 的下标对应关系
  cte_range_arr_idx as 
  (
    select 
      a_arr_idx,
      a_range_idx
    from generate_series(1, array_length(i_simp_ranges, 1)) tb_a_range_idx(a_range_idx)
      , generate_series
        (
          lower(i_simp_ranges[a_range_idx])
        , upper(i_simp_ranges[a_range_idx]) - 1
        ) tb_a_cur_idx(a_arr_idx)
    -- where a_arr_idx <@ i_simp_ranges[a_range_idx]
  ),
  -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 单位切片为分组，agg后，记录一对多关系，且聚合滤掉未被采样位置。
  cte_map_arr_idx_dd_idx_per_period as
  (
    select 
      tb_a_arr_idx.a_arr_idx,
      array_agg
      (
        tb_a_dd_idx.a_range_dd_pos 
        + tb_a_arr_idx.a_arr_idx 
        - tb_a_dd_idx.a_lower 
        -- order by tb_a_dd_idx.a_range_dd_pos
      ) :: text as a_dd_idxs
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
    having count(tb_a_dd_idx.a_range_dd_pos) > 0
  )
  select 
    array_agg(a_arr_idx order by a_arr_idx),
    array_agg(a_dd_idxs order by a_arr_idx)
  into 
    v_arr_idxs,
    v_dd_idxss
  from cte_map_arr_idx_dd_idx_per_period
  ;
  
  -- 逐个周期，以单位切片为分组，合计 dloss_ddepdt，赋值到 v_ret 对应下标位置，未被采样切片，返回零切片为导数
  for v_cur_1 in 1 .. (array_length(i_dloss_ddepdt, 2) / v_window_len_ex)
  loop 
    for v_cur_2 in 1 .. array_length(v_arr_idxs, 1)
    loop 
      if (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] between 1 and array_length(v_ret, 2)
      then 
        v_ret[ : ][(v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] : (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2]] :=
        (
          select 
            sm_sc.fa_mx_sum
            (
              i_dloss_ddepdt
              [ : ]
              [
                (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx] 
              : (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx]
              ]
            )
          from generate_series(1, array_length((v_dd_idxss[v_cur_2] :: int[]), 1)) tb_a_cur_dd_idx(a_cur_dd_idx)
        );
      end if;
    end loop;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_sample_x_dloss_dindepdt_1
--   (
--     array
--     [
--       [1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , [11, 12, 13, 14, 15, 16 ,17 ,18 ,19]
--     ]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );

-- select sm_sc.fv_d_sample_x_dloss_dindepdt_1
--   (
--     array
--     [[
--       [1, 2, 3, 4]
--     , [11, 12, 13, 14]
--     , [21, 22, 23, 24]

--     , [31, 32, 33, 34]
--     , [41, 42, 43, 44]

--     , [51, 52, 53, 54]
--     , [61, 62, 63, 64]
--     , [71, 72, 73, 74]

--     , [81, 82, 83, 84]
--     , [91, 92, 93, 94]
--     ]]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );

-- select sm_sc.fv_d_sample_x_dloss_dindepdt_1
--   (
--     array
--     [[
--       [[1, 2, 3, 4]]
--     , [[11, 12, 13, 14]]
--     , [[21, 22, 23, 24]]

--     , [[31, 32, 33, 34]]
--     , [[41, 42, 43, 44]]

--     , [[51, 52, 53, 54]]
--     , [[61, 62, 63, 64]]
--     , [[71, 72, 73, 74]]

--     , [[81, 82, 83, 84]]
--     , [[91, 92, 93, 94]]
--     ]]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_sample_y_dloss_dindepdt_1(float[], int, int, int4range[]);
create or replace function sm_sc.fv_d_sample_y_dloss_dindepdt_1
(
  i_dloss_ddepdt   float[]      ,                -- 损失函数对因变量的导数
  i_array_len_y    int          ,                -- 自变量数组在采样维度的长度
  -- i_period         int          ,                -- 采样周期长度
  i_window_len     int          ,                -- 约束窗口宽度
  i_simp_ranges    int4range[]                   -- 采样(多)区间
  
)
returns float[]
as
$$
declare 
  v_ret            float[]  ;
  v_period         int      ;      -- i_array 采样周期
  v_window_len_ex  int  :=         -- 生成 i_dloss_ddepdt 的周期
    (
      select 
        max(upper(a_range * int4range(1, i_window_len, '[]'))) 
        - min(lower(a_range * int4range(1, i_window_len, '[]')))
      from unnest(i_simp_ranges) tb_a_range(a_range)
    )
  ;
  v_cur_1          int      ;
  v_cur_2          int      ;
  v_arr_idxs       int[]    ;    -- 采样到的切片下标集合
  v_dd_idxss       text[]   ;    -- 采样到的每个单元切片对应导数下标集合，再所有单元切片 array_agg 集合，相当于不整齐的二维 int 数组。
  -- v_zero_slice     float[]  ;    -- 零切片
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 1 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif array_length(i_dloss_ddepdt, 1) % v_window_len_ex > 0
      or i_array_len_y / (array_length(i_dloss_ddepdt, 1) % v_window_len_ex) > 0
    then 
      raise exception 'unmatch 1d length of i_dloss_ddepdt, simp_range or i_array_len for period.';
    end if;
  end if;
  
  -- 窗口约束重整采样区间
  for v_cur_1 in 1 .. array_length(i_simp_ranges, 1)
  loop
    i_simp_ranges[v_cur_1] := i_simp_ranges[v_cur_1] * int4range(1, i_window_len, '[]');
  end loop;
  
  -- 换算出 i_array 周期
  v_period  :=   i_array_len_y / (array_length(i_dloss_ddepdt, 1) / v_window_len_ex);
  
  if array_ndims(i_dloss_ddepdt) = 1
  then
    v_ret     :=   array_fill(0.0, array[i_array_len_y]);
  elsif array_ndims(i_dloss_ddepdt) = 2
  then
    v_ret     :=   array_fill(0.0, array[i_array_len_y] || array_length(i_dloss_ddepdt, 2));
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    v_ret     :=   array_fill(0.0, array[i_array_len_y] || array_length(i_dloss_ddepdt, 2) || array_length(i_dloss_ddepdt, 3));
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    v_ret     :=   array_fill(0.0, array[i_array_len_y] || array_length(i_dloss_ddepdt, 2) || array_length(i_dloss_ddepdt, 3) || array_length(i_dloss_ddepdt, 4));
  end if;
  
  -- v_zero_slice := v_ret[1 : 1];

  with recursive
  -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
  cte_range_dd_idx as 
  (
    select 
      1 as a_range_idx,
      lower(i_simp_ranges[1])                           as a_lower,
      -- upper(i_simp_ranges[1]) - 1                       as a_upper,
      upper(i_simp_ranges[1]) - lower(i_simp_ranges[1]) as a_range_len,
      1 as a_range_dd_pos
    union all 
    select 
      a_range_idx + 1 as a_range_idx,
      lower(i_simp_ranges[a_range_idx + 1])                                         as a_lower,
      -- upper(i_simp_ranges[a_range_idx + 1]) - 1                                     as a_upper,
      upper(i_simp_ranges[a_range_idx + 1]) - lower(i_simp_ranges[a_range_idx + 1]) as a_range_len,
      a_range_dd_pos + a_range_len as a_range_dd_pos
    from cte_range_dd_idx
    where a_range_idx < array_length(i_simp_ranges, 1)
  ),
  -- 算出区间与 i_arr 的下标对应关系
  cte_range_arr_idx as 
  (
    select 
      a_arr_idx,
      a_range_idx
    from generate_series(1, array_length(i_simp_ranges, 1)) tb_a_range_idx(a_range_idx)
      , generate_series
        (
          lower(i_simp_ranges[a_range_idx])
        , upper(i_simp_ranges[a_range_idx]) - 1
        ) tb_a_cur_idx(a_arr_idx)
    -- where a_arr_idx <@ i_simp_ranges[a_range_idx]
  ),
  -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 单位切片为分组，agg后，记录一对多关系，且聚合滤掉未被采样位置。
  cte_map_arr_idx_dd_idx_per_period as
  (
    select 
      tb_a_arr_idx.a_arr_idx,
      -- sm_sc.fa_mx_sum(i_dloss_ddepdt[a_range_dd_pos + a_arr_idx - a_lower : a_range_dd_pos + a_arr_idx - a_lower]) as a_dloss_dindepdt
      array_agg
      (
        tb_a_dd_idx.a_range_dd_pos 
        + tb_a_arr_idx.a_arr_idx 
        - tb_a_dd_idx.a_lower 
        -- order by tb_a_dd_idx.a_range_dd_pos
      ) :: text as a_dd_idxs
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
    having count(tb_a_dd_idx.a_range_dd_pos) > 0
  )
  select 
    array_agg(a_arr_idx order by a_arr_idx),
    array_agg(a_dd_idxs order by a_arr_idx)
  into 
    v_arr_idxs,
    v_dd_idxss
  from cte_map_arr_idx_dd_idx_per_period
  ;
  
  -- 逐个周期，以单位切片为分组，合计 dloss_ddepdt，赋值到 v_ret 对应下标位置，未被采样切片，返回零切片为导数
  for v_cur_1 in 1 .. (array_length(i_dloss_ddepdt, 1) / v_window_len_ex)
  loop 
    for v_cur_2 in 1 .. array_length(v_arr_idxs, 1)
    loop 
      if (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] between 1 and array_length(v_ret, 1)
      then 
        v_ret[(v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2] : (v_cur_1 - 1) * v_period + v_arr_idxs[v_cur_2]] :=
        (
          select 
            sm_sc.fa_mx_sum
            (
              i_dloss_ddepdt
              [
                (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx] 
              : (v_dd_idxss[v_cur_2] :: int[])[a_cur_dd_idx]
              ]
            )
          from generate_series(1, array_length((v_dd_idxss[v_cur_2] :: int[]), 1)) tb_a_cur_dd_idx(a_cur_dd_idx)
        );
      end if;
    end loop;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_sample_y_dloss_dindepdt_1
--   (
--     array
--     [1, 2, 3, 4, 5, 6, 7, 8, 9]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );

-- select sm_sc.fv_d_sample_y_dloss_dindepdt_1
--   (
--     array
--     [
--       [1, 2, 3, 4]
--     , [11, 12, 13, 14]
--     , [21, 22, 23, 24]

--     , [31, 32, 33, 34]
--     , [41, 42, 43, 44]

--     , [51, 52, 53, 54]
--     , [61, 62, 63, 64]
--     , [71, 72, 73, 74]

--     , [81, 82, 83, 84]
--     , [91, 92, 93, 94]
--     ]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );

-- select sm_sc.fv_d_sample_y_dloss_dindepdt_1
--   (
--     array
--     [
--       [[1, 2, 3, 4]]
--     , [[11, 12, 13, 14]]
--     , [[21, 22, 23, 24]]

--     , [[31, 32, 33, 34]]
--     , [[41, 42, 43, 44]]

--     , [[51, 52, 53, 54]]
--     , [[61, 62, 63, 64]]
--     , [[71, 72, 73, 74]]

--     , [[81, 82, 83, 84]]
--     , [[91, 92, 93, 94]]
--     ]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );

-- select sm_sc.fv_d_sample_y_dloss_dindepdt_1
--   (
--     array
--     [
--       [[[1, 2, 3, 4]]]
--     , [[[11, 12, 13, 14]]]
--     , [[[21, 22, 23, 24]]]

--     , [[[31, 32, 33, 34]]]
--     , [[[41, 42, 43, 44]]]

--     , [[[51, 52, 53, 54]]]
--     , [[[61, 62, 63, 64]]]
--     , [[[71, 72, 73, 74]]]

--     , [[[81, 82, 83, 84]]]
--     , [[[91, 92, 93, 94]]]
--     ]
--   , 15
--   , 3
--   , array[int4range(1, 3, '[]'), int4range(2, 5, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_slice_x3_dloss_dindepdt(float[], int, int4range[]);
create or replace function sm_sc.fv_d_slice_x3_dloss_dindepdt
(
  i_dloss_ddepdt   float[]    ,
  i_arr_len_x3     int      ,
  i_slice_range    int4range[]
)
returns float[]
as
$$
declare 
  v_ret    float[]  ;
  v_cur    record   ;
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 3 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif i_arr_len_x3 < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for i_arr_len_x3.';
    elsif array_length(i_dloss_ddepdt, 3) <> (select sum(upper(a_range) - lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then 
      raise exception 'unmatched 3d length between i_dloss_ddepdt and i_slice_range';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2)] || i_arr_len_x3);
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2)] || i_arr_len_x3 || array_length(i_dloss_ddepdt, 4));
  end if;
  
  for v_cur in 
    with recursive
    -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
    cte_range_dd_idx as 
    (
      select 
        1 as a_range_idx,
        lower(i_slice_range[1])                           as a_lower,
        -- upper(i_slice_range[1]) - 1                       as a_upper,
        upper(i_slice_range[1]) - lower(i_slice_range[1]) as a_range_len,
        1 as a_range_dd_pos
      union all 
      select 
        a_range_idx + 1 as a_range_idx,
        lower(i_slice_range[a_range_idx + 1])                                         as a_lower,
        -- upper(i_slice_range[a_range_idx + 1]) - 1                                     as a_upper,
        upper(i_slice_range[a_range_idx + 1]) - lower(i_slice_range[a_range_idx + 1]) as a_range_len,
        a_range_dd_pos + a_range_len as a_range_dd_pos
      from cte_range_dd_idx
      where a_range_idx < array_length(i_slice_range, 1)
    ),
    -- 算出区间与 i_arr 的下标对应关系
    cte_range_arr_idx as 
    (
      select 
        a_arr_idx,
        a_range_idx
      from generate_series(1, i_arr_len_x3) tb_a_cur_idx(a_arr_idx)
        , generate_series(1, array_length(i_slice_range, 1)) tb_a_range_idx(a_range_idx)
      where a_arr_idx <@ i_slice_range[a_range_idx]
    )
    -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 切片为分组，合计 dloss_ddepdt 切片导数，从而反向传播求导
    select 
      tb_a_arr_idx.a_arr_idx,
      sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][a_range_dd_pos + a_arr_idx - a_lower : a_range_dd_pos + a_arr_idx - a_lower]) as a_dloss_dindepdt
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
  loop 
    v_ret[ : ][ : ][v_cur.a_arr_idx : v_cur.a_arr_idx] := v_cur.a_dloss_dindepdt;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_slice_x3_dloss_dindepdt
--   (
--     array[[[1, 2, 3, 4, 5], [11, 12, 13, 14, 15]], [[21, 22, 23, 24, 25], [31, 32, 33, 34, 35]]]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );

-- select sm_sc.fv_d_slice_x3_dloss_dindepdt
--   (
--     array
--     [
--       [[[1, -1], [2, -2], [3, -3], [4, -4], [5, -5]], [[6, -6], [7, -7], [8, -8], [9.7, -9], [10, -10]]]
--     , [[[1, -1], [2, -2], [3, -3], [4, -4], [5, -5]], [[6, -6], [7, -7], [8, -8], [9.7, -9], [10, -10]]]
--     ]::float[]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_slice_x4_dloss_dindepdt(float[], int, int4range[]);
create or replace function sm_sc.fv_d_slice_x4_dloss_dindepdt
(
  i_dloss_ddepdt   float[]    ,
  i_arr_len_x4     int      ,
  i_slice_range    int4range[]
)
returns float[]
as
$$
declare 
  v_ret    float[]  ;
  v_cur    record   ;
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 3 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif i_arr_len_x4 < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for i_arr_len_x4.';
    elsif array_length(i_dloss_ddepdt, 4) <> (select sum(upper(a_range) - lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then 
      raise exception 'unmatched 3d length between i_dloss_ddepdt and i_slice_range';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), array_length(i_dloss_ddepdt, 3)] || i_arr_len_x4);
  end if;
  
  for v_cur in 
    with recursive
    -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
    cte_range_dd_idx as 
    (
      select 
        1 as a_range_idx,
        lower(i_slice_range[1])                           as a_lower,
        -- upper(i_slice_range[1]) - 1                       as a_upper,
        upper(i_slice_range[1]) - lower(i_slice_range[1]) as a_range_len,
        1 as a_range_dd_pos
      union all 
      select 
        a_range_idx + 1 as a_range_idx,
        lower(i_slice_range[a_range_idx + 1])                                         as a_lower,
        -- upper(i_slice_range[a_range_idx + 1]) - 1                                     as a_upper,
        upper(i_slice_range[a_range_idx + 1]) - lower(i_slice_range[a_range_idx + 1]) as a_range_len,
        a_range_dd_pos + a_range_len as a_range_dd_pos
      from cte_range_dd_idx
      where a_range_idx < array_length(i_slice_range, 1)
    ),
    -- 算出区间与 i_arr 的下标对应关系
    cte_range_arr_idx as 
    (
      select 
        a_arr_idx,
        a_range_idx
      from generate_series(1, i_arr_len_x4) tb_a_cur_idx(a_arr_idx)
        , generate_series(1, array_length(i_slice_range, 1)) tb_a_range_idx(a_range_idx)
      where a_arr_idx <@ i_slice_range[a_range_idx]
    )
    -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 切片为分组，合计 dloss_ddepdt 切片导数，从而反向传播求导
    select 
      tb_a_arr_idx.a_arr_idx,
      sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][ : ][ : ][a_range_dd_pos + a_arr_idx - a_lower : a_range_dd_pos + a_arr_idx - a_lower]) as a_dloss_dindepdt
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
  loop 
    v_ret[ : ][ : ][ : ][v_cur.a_arr_idx : v_cur.a_arr_idx] := v_cur.a_dloss_dindepdt;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_slice_x4_dloss_dindepdt
--   (
--     array[[[[1, 2, 3, 4, 5], [11, 12, 13, 14, 15]], [[21, 22, 23, 24, 25], [31, 32, 33, 34, 35]]]]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );

-- select sm_sc.fv_d_slice_x4_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5],[11, 12, 13, 14, 15]]
--       ]
--     , [
--         [[21, 22, 23, 24, 25],[31, 32, 33, 34, 35]]
--       ]
--     ]::float[]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_slice_x_dloss_dindepdt(float[], int, int4range[]);
create or replace function sm_sc.fv_d_slice_x_dloss_dindepdt
(
  i_dloss_ddepdt   float[]    ,
  i_arr_len_x      int      ,
  i_slice_range    int4range[]
)
returns float[]
as
$$
declare 
  v_ret    float[]  ;
  v_cur    record   ;
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif i_arr_len_x < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for i_arr_len_x.';
    elsif array_length(i_dloss_ddepdt, 2) <> (select sum(upper(a_range) - lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then 
      raise exception 'unmatched 2d length between i_dloss_ddepdt and i_slice_range';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    v_ret := array_fill(0.0, array[array_length(i_dloss_ddepdt, 1)] || i_arr_len_x);
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := array_fill(0.0, array[array_length(i_dloss_ddepdt, 1)] || i_arr_len_x || array[array_length(i_dloss_ddepdt, 3)]);
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := array_fill(0.0, array[array_length(i_dloss_ddepdt, 1)] || i_arr_len_x || array[array_length(i_dloss_ddepdt, 3), array_length(i_dloss_ddepdt, 4)]);
  end if;
  
  for v_cur in 
    with recursive
    -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
    cte_range_dd_idx as 
    (
      select 
        1 as a_range_idx,
        lower(i_slice_range[1])                           as a_lower,
        -- upper(i_slice_range[1]) - 1                       as a_upper,
        upper(i_slice_range[1]) - lower(i_slice_range[1]) as a_range_len,
        1 as a_range_dd_pos
      union all 
      select 
        a_range_idx + 1 as a_range_idx,
        lower(i_slice_range[a_range_idx + 1])                                         as a_lower,
        -- upper(i_slice_range[a_range_idx + 1]) - 1                                     as a_upper,
        upper(i_slice_range[a_range_idx + 1]) - lower(i_slice_range[a_range_idx + 1]) as a_range_len,
        a_range_dd_pos + a_range_len as a_range_dd_pos
      from cte_range_dd_idx
      where a_range_idx < array_length(i_slice_range, 1)
    ),
    -- 算出区间与 i_arr 的下标对应关系
    cte_range_arr_idx as 
    (
      select 
        a_arr_idx,
        a_range_idx
      from generate_series(1, i_arr_len_x) tb_a_cur_idx(a_arr_idx)
        , generate_series(1, array_length(i_slice_range, 1)) tb_a_range_idx(a_range_idx)
      where a_arr_idx <@ i_slice_range[a_range_idx]
    )
    -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 切片为分组，合计 dloss_ddepdt 切片导数，从而反向传播求导
    select 
      tb_a_arr_idx.a_arr_idx,
      sm_sc.fa_mx_sum(i_dloss_ddepdt[ : ][a_range_dd_pos + a_arr_idx - a_lower : a_range_dd_pos + a_arr_idx - a_lower]) as a_dloss_dindepdt
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
  loop 
    v_ret[ : ][v_cur.a_arr_idx : v_cur.a_arr_idx] := v_cur.a_dloss_dindepdt;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_slice_x_dloss_dindepdt
--   (
--     array[[1, 2, 3, 4, 5], [11, 12, 13, 14, 15]]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );

-- select sm_sc.fv_d_slice_x_dloss_dindepdt
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]]]
--     ]::float[]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_slice_y_dloss_dindepdt(float[], int, int4range[]);
create or replace function sm_sc.fv_d_slice_y_dloss_dindepdt
(
  i_dloss_ddepdt   float[]    ,
  i_arr_len_y      int      ,
  i_slice_range    int4range[]
)
returns float[]
as
$$
declare 
  v_ret    float[]  ;
  v_cur    record   ;
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_dloss_ddepdt) not between 1 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt.';
    elsif i_arr_len_y < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for i_arr_len_y.';
    elsif array_length(i_dloss_ddepdt, 1) <> (select sum(upper(a_range) - lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then 
      raise exception 'unmatched 1d length between i_dloss_ddepdt and i_slice_range';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 1
  then 
    v_ret := array_fill(0.0, array[i_arr_len_y]);
  elsif array_ndims(i_dloss_ddepdt) = 2
  then 
    v_ret := array_fill(0.0, i_arr_len_y || array[array_length(i_dloss_ddepdt, 2)]);
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := array_fill(0.0, i_arr_len_y || array[array_length(i_dloss_ddepdt, 2), array_length(i_dloss_ddepdt, 3)]);
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := array_fill(0.0, i_arr_len_y || array[array_length(i_dloss_ddepdt, 2), array_length(i_dloss_ddepdt, 3), array_length(i_dloss_ddepdt, 4)]);
  end if;
  
  for v_cur in 
    with recursive
    -- 迭代 cte 算出区间与 dloss_ddepdt 的位置对应关系
    cte_range_dd_idx as 
    (
      select 
        1 as a_range_idx,
        lower(i_slice_range[1])                           as a_lower,
        -- upper(i_slice_range[1]) - 1                       as a_upper,
        upper(i_slice_range[1]) - lower(i_slice_range[1]) as a_range_len,
        1 as a_range_dd_pos
      union all 
      select 
        a_range_idx + 1 as a_range_idx,
        lower(i_slice_range[a_range_idx + 1])                                         as a_lower,
        -- upper(i_slice_range[a_range_idx + 1]) - 1                                     as a_upper,
        upper(i_slice_range[a_range_idx + 1]) - lower(i_slice_range[a_range_idx + 1]) as a_range_len,
        a_range_dd_pos + a_range_len as a_range_dd_pos
      from cte_range_dd_idx
      where a_range_idx < array_length(i_slice_range, 1)
    ),
    -- 算出区间与 i_arr 的下标对应关系
    cte_range_arr_idx as 
    (
      select 
        a_arr_idx,
        a_range_idx
      from generate_series(1, i_arr_len_y) tb_a_cur_idx(a_arr_idx)
        , generate_series(1, array_length(i_slice_range, 1)) tb_a_range_idx(a_range_idx)
      where a_arr_idx <@ i_slice_range[a_range_idx]
    )
    -- 映射出  i_arr 每一个切片与 dloss_ddepdt 的对应关系，并以 i_arr 切片为分组，合计 dloss_ddepdt 切片导数，从而反向传播求导
    select 
      tb_a_arr_idx.a_arr_idx,
      sm_sc.fa_mx_sum(i_dloss_ddepdt[a_range_dd_pos + a_arr_idx - a_lower : a_range_dd_pos + a_arr_idx - a_lower]) as a_dloss_dindepdt
    from cte_range_arr_idx tb_a_arr_idx
    inner join cte_range_dd_idx tb_a_dd_idx
      on tb_a_dd_idx.a_range_idx = tb_a_arr_idx.a_range_idx
    group by tb_a_arr_idx.a_arr_idx
  loop 
    v_ret[v_cur.a_arr_idx : v_cur.a_arr_idx] := v_cur.a_dloss_dindepdt;
  end loop;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_d_slice_y_dloss_dindepdt
--   (
--     array[[1, 2], [3, 4], [5, 11], [12, 13], [14, 15]]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );

-- select sm_sc.fv_d_slice_y_dloss_dindepdt
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--   , 7
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_upper_tri_mx_dloss_dindepdt(float[]);
create or replace function sm_sc.fv_d_upper_tri_mx_dloss_dindepdt
(
  i_dloss_ddepdt            float[]
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_upper_tri_mx(i_dloss_ddepdt, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_upper_tri_mx_dloss_dindepdt
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );

-- select 
--   sm_sc.fv_d_upper_tri_mx_dloss_dindepdt
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );

-- select 
--   sm_sc.fv_d_upper_tri_mx_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt(float[]);
create or replace function sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt
(
  i_dloss_ddepdt            float[]
)
returns float[]
as
$$
-- declare 
begin
  return sm_sc.fv_upper_tri_mx_ex(i_dloss_ddepdt, 0.0 :: float);

end
$$
language plpgsql stable
cost 100;

-- select 
--   sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );

-- select 
--   sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );

-- select 
--   sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_x(anyarray, int);
create or replace function sm_sc.ft_rand_slice_x
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  elsif array_ndims(i_array) = 2
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_select_cnt)) tb_a_cur(a_cur)
    ;
  elsif array_ndims(i_array) = 3
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_select_cnt)) tb_a_cur(a_cur)
    ;
  elsif array_ndims(i_array) = 4
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_select_cnt)) tb_a_cur(a_cur)
    ;
  else
    raise exception 'no method for such length!  Dim: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select * from sm_sc.ft_rand_slice_x
--   (
--     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
--     , 6
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_x
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 9
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_x
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 5
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_x3(anyarray, int);
create or replace function sm_sc.ft_rand_slice_x3
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  -- -- elsif array_ndims(i_array) = 2
  -- -- then
  -- --   return query
  -- --     select
  -- --       array_agg(a_cur) as o_ord_nos,
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 3), i_select_cnt)) tb_a_cur(a_cur)
  -- --   ;
  elsif array_ndims(i_array) = 3
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 3), i_select_cnt)) tb_a_cur(a_cur)
    ;
  elsif array_ndims(i_array) = 4
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 3), i_select_cnt)) tb_a_cur(a_cur)
    ;
  else
    raise exception 'no method for such length!  Dim: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select * from sm_sc.ft_rand_slice_x3
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 7
-- --     -- , true
-- --   );
-- select * from sm_sc.ft_rand_slice_x3
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 12
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_x3
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 8
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_x3_pick(anyarray, int);
create or replace function sm_sc.ft_rand_slice_x3_pick
(
  i_array          anyarray          ,
  i_pick_cnt     int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  -- -- elsif array_ndims(i_array) = 2
  -- -- then
  -- --   return query
  -- --     select
  -- --       array_agg(a_cur) as o_ord_nos,
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
  -- --   ;
  elsif i_pick_cnt <= array_length(i_array, 3)
  then 
    if array_ndims(i_array) = 3
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    elsif array_ndims(i_array) = 4
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    end if;
  else
    raise exception 'no method for such length!  Dim: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select * from sm_sc.ft_rand_slice_x3_pick
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 2
-- --     -- , true
-- --   );
-- select * from sm_sc.ft_rand_slice_x3_pick
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 2
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_x3_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_x4(anyarray, int);
create or replace function sm_sc.ft_rand_slice_x4
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  -- -- elsif array_ndims(i_array) = 2 and i_select_cnt <= array_length(i_array, 2)
  -- -- then
  -- --   return query
  -- --     select
  -- --       array_agg(a_cur) as o_ord_nos,
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 4), i_select_cnt)) tb_a_cur(a_cur)
  -- --   ;
  -- -- elsif array_ndims(i_array) = 3 and i_select_cnt <= array_length(i_array, 2)
  -- -- then
  -- --   return query
  -- --     select
  -- --       array_agg(a_cur) as o_ord_nos,
  -- --       sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 4), i_select_cnt)) tb_a_cur(a_cur)
  -- --   ;
  elsif array_ndims(i_array) = 4
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x4(i_array[ : ][ : ][ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 4), i_select_cnt)) tb_a_cur(a_cur)
    ;
  else
    raise exception 'no method for such length!  Dim: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select * from sm_sc.ft_rand_slice_x4
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 9
-- --     -- , true
-- --   );
-- -- select * from sm_sc.ft_rand_slice_x4
-- --   (
-- --     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
-- --         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
-- --     , 5
-- --     -- , true
-- --   );
-- select * from sm_sc.ft_rand_slice_x4
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[17, -17], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 7
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_x4_pick(anyarray, int);
create or replace function sm_sc.ft_rand_slice_x4_pick
(
  i_array          anyarray          ,
  i_pick_cnt     int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  -- -- elsif array_ndims(i_array) = 2
  -- -- then
  -- --   return query
  -- --     select
  -- --       array_agg(a_cur) as o_ord_nos,
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
  -- --   ;
  -- -- elsif array_ndims(i_array) = 3
  -- -- then
  -- --   return query
  -- --     select
  -- --       array_agg(a_cur) as o_ord_nos,
  -- --       sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
  -- --   ;
  elsif array_ndims(i_array) = 4 and i_pick_cnt <= array_length(i_array, 4)
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_x4(i_array[ : ][ : ][ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 4), i_pick_cnt)) tb_a_cur(a_cur)
    ;
  else
    raise exception 'no method for such length!  Dim: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select * from sm_sc.ft_rand_slice_x4_pick
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 2
-- --     -- , true
-- --   );
-- -- select * from sm_sc.ft_rand_slice_x4_pick
-- --   (
-- --     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
-- --         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
-- --     , 2
-- --     -- , true
-- --   );
-- select * from sm_sc.ft_rand_slice_x4_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_x_pick(anyarray, int);
create or replace function sm_sc.ft_rand_slice_x_pick
(
  i_array          anyarray          ,
  i_pick_cnt       int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  elsif i_pick_cnt <= array_length(i_array, 2)
  then 
    if array_ndims(i_array) = 2 and i_pick_cnt <= array_length(i_array, 2)
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 2), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    elsif array_ndims(i_array) = 3 and i_pick_cnt <= array_length(i_array, 2)
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    elsif array_ndims(i_array) = 4 and i_pick_cnt <= array_length(i_array, 2)
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    end if;
  else
    raise exception 'no method for such length!  Dim: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select * from sm_sc.ft_rand_slice_x_pick
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
--     , 3
--     -- , true
--   ) tb_a;
-- select * from sm_sc.ft_rand_slice_x_pick
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 3
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_x_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_y(anyarray, int);
create or replace function sm_sc.ft_rand_slice_y
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  elsif array_ndims(i_array) = 2
  then
    return query	
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 1), i_select_cnt)) tb_a_cur(a_cur)
    ;
  elsif array_ndims(i_array) = 3
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 1), i_select_cnt)) tb_a_cur(a_cur)
    ;
  elsif array_ndims(i_array) = 4
  then
    return query
      select
        array_agg(a_cur) as o_ord_nos,
        sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 1), i_select_cnt)) tb_a_cur(a_cur)
    ;
  else
    raise exception 'no method for such length!  Dims: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
    return query select null, null where false; 
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select * from sm_sc.ft_rand_slice_y
--   (
--     array[array[1.0 :: float, 2.0], array[3.0, 4.0], array[5.0, 6.0], array[7.0, 8.0], array[9.0, 10.0], array[11.0 :: float, 12.0]]
--     , 8
--     -- , true
--   ) tb_a;
-- select * from sm_sc.ft_rand_slice_y
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 6
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_y
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 4
--     -- , true
--   );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_rand_slice_y_pick(anyarray, int);
create or replace function sm_sc.ft_rand_slice_y_pick
(
  i_array          anyarray          ,
  i_pick_cnt       int
)
returns table
(
  o_ord_nos   int[],
  o_slices    anyarray
)
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return query
      select null, i_array;
  elsif i_pick_cnt <= array_length(i_array, 1)
  then 
    if array_ndims(i_array) = 2
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 1), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    elsif array_ndims(i_array) = 3
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 1), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    elsif array_ndims(i_array) = 4
    then
      return query
        select
          array_agg(a_cur) as o_ord_nos,
          sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 1), i_pick_cnt)) tb_a_cur(a_cur)
      ;
    end if;
  else 
    raise exception 'no method for such length!  Dims: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
    return query select null, null where false;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select * from sm_sc.ft_rand_slice_y_pick
--   (
--     array[array[1, 2], array[3, 4], array[5, 6], array[7, 8], array[9, 10], array[11, 12]]
--     , 2
--   );
-- select * from sm_sc.ft_rand_slice_y_pick
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 2
--     -- , true
--   );
-- select * from sm_sc.ft_rand_slice_y_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- drop function if exists sm_sc.fv_amask(anyarray, int[], anyelement);
create or replace function sm_sc.fv_amask
(
  i_array               anyarray    ,
  i_mask_len            int[]       ,     -- 该控制参数的维度与 i_array 相同，背景面的 heigh 维度(次高维度)长度为 1，其他维度长度与 i_array 对应维度长度也相同。
  i_mask_element        anyelement
)
returns anyarray
as
$$
declare
  v_pos_range_s         int4range[] ;
  v_background_heigh    int         := array_length(i_array, array_ndims(i_array) - 1);
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 2 and 4 
    then 
      raise exception 'unsupport ndims of i_array';
    elsif array_ndims(i_mask_len) <> array_ndims(i_array)
    then 
      raise exception 'array_ndims of i_mask_len should be the same as i_array''s.';
    elsif array_length(i_mask_len, array_ndims(i_mask_len) - 1) <> 1 
    then 
      raise exception 'width of i_mask_len'' background should be 1.';
    elsif array_ndims(i_mask_len) = 2 
        and array_length(i_mask_len, 2) <> array_length(i_array, 2)
      or array_ndims(i_mask_len) = 3 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 3) <> array_length(i_array, 3)
            )
      or array_ndims(i_mask_len) = 4 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 2) <> array_length(i_array, 2) 
               or array_length(i_mask_len, 4) <> array_length(i_array, 4)
            )
    then 
      raise exception 'unmatched length between i_mask_len and i_array';
    elsif v_background_heigh < any(i_mask_len)
    then
      raise exception 'there should not be any element of i_mask_len > v_background_heigh.';
    end if;
  end if;
  
  if array_ndims(i_mask_len) = 2
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            nullif(int4range(1, i_mask_len[1][a_cur_x] + 1, '[)'), 'empty' :: int4range)
          , int4range(a_cur_x, a_cur_x, '[]')
          ]
        )
      from generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
    );
    
  elsif array_ndims(i_mask_len) = 3
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , nullif(int4range(1, i_mask_len[a_cur_y][1][a_cur_x3] + 1, '[)'), 'empty' :: int4range)
          , int4range(a_cur_x3, a_cur_x3, '[]')
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 3)) tb_a_cur_x3(a_cur_x3)
    );
    
  elsif array_ndims(i_mask_len) = 4 
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , int4range(a_cur_x, a_cur_x, '[]')
          , nullif(int4range(1, i_mask_len[a_cur_y][a_cur_x][1][a_cur_x4] + 1, '[)'), 'empty' :: int4range)
          , int4range(a_cur_x4, a_cur_x4, '[]')
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_mask_len, 4)) tb_a_cur_x4(a_cur_x4)
    );
      
  end if;
  
  return 
    sm_sc.fv_pos_replaces
    (
      i_array
    , v_pos_range_s
    , i_mask_element
    )
  ;
  
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_amask
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ]
--   , array[[3, 0, 2, 1, 3]]
--   , 0
--   )

-- select 
--   sm_sc.fv_amask
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ]
--   , array[[[3, 0, 2, 1, 0]], [[1, 1, 4, 2, 1]], [[3, 2, 2, 1, 4]]]
--   , 0
--   )

-- select 
--   sm_sc.fv_amask
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ]
--   , array[[[[3, 0, 2, 1, 3]], [[1, 1, 4, 2, 4]], [[3, 3, 2, 1, 2]]],[[[3, 0, 2, 1, 1]], [[1, 1, 4, 2, 0]], [[3, 3, 2, 1, 2]]]]
--   , 0
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_apad(anyarray, anyarray, int);
create or replace function sm_sc.fv_apad
(
  i_array            anyarray      ,
  i_fill_values      anyarray      , -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
  i_times            int        default  1   
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- set search_path to sm_sc;
  if i_array is null 
  then 
    return null;
  end if;
  
  -- if v_fill_values is null 
  -- then 
  --   if array_ndims(i_array) = 1
  --   then 
  --     v_fill_values := array[nullif(i_array[1], i_array[1])];
  --   elsif array_ndims(i_array) = 2
  --   then
  --     v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  --   elsif array_ndims(i_array) = 3
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  --   elsif array_ndims(i_array) = 4
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  --   end if;
  -- end if;

  if i_times <= 0
  then
    return i_array;
  elsif array_ndims(i_array) <> array_ndims(v_fill_values)
  then 
    raise exception 'ndims of i_array: % and i_fill_values: % should be the same.', array_ndims(i_array), array_ndims(v_fill_values);
  elsif array_ndims(i_array) = 1
  then 
    return
      (
        select 
          sm_sc.fa_array_concat(v_fill_values)
        from generate_series(1, i_times)
      )
      || i_array
    ;
  elsif array_ndims(i_array) = 2
  then 
    if array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        (
          select 
            sm_sc.fa_mx_concat_y(v_fill_values)
          from generate_series(1, i_times)
        )
        |-|| i_array
      ;
    end if;
  elsif array_ndims(i_array) = 3
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
      or array_length(v_fill_values, 3) not in (1, array_length(i_array, 3))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        (
          select 
            sm_sc.fa_mx_concat_x(v_fill_values)
          from generate_series(1, i_times)
        )
        |-|| i_array
      ;
    end if;
  elsif array_ndims(i_array) = 4
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
      or array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
      or array_length(v_fill_values, 4) not in (1, array_length(i_array, 4))
    then 
      raise exception 'width of 3d i_fill_values should be the same as 3d i_array''s. ';
    else
      return
        (
          select 
            sm_sc.fa_mx_concat_x3(v_fill_values)
          from generate_series(1, i_times)
        )
        |-|| i_array
      ;
    end if;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_apad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     array[7, 8],
--     2
--   );
-- select sm_sc.fv_apad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7], array[8]],
--     2
--   );
-- select sm_sc.fv_apad
--   (
--     array[array[1.0 :: float, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     array[array[7, 8], array[9, 10]] :: float[],
--     2
--   );
-- select sm_sc.fv_apad
--   (
--     array[array[1, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     array[array[7], array[8]],
--     2
--   );
-- select sm_sc.fv_apad
--   (
--     array[array[1.0 :: float, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     array[array[null :: float]],
--     2
--   );
-- select 
--   sm_sc.fv_apad
--   (
--     array[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]
--   , array[[[7, -7],[8, -8],[9, -9],[10, -10],[11, -11],[12, -12]]]
--   , 2
--   );
-- select 
--   sm_sc.fv_apad
--   (
--     array[[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]],[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]]
--   , array[[[[7, -7],[8, -8],[9, -9],[10, -10],[11, -11],[12, -12]]],[[[7, -7],[8, -8],[9, -9],[10, -10],[11, -11],[12, -12]]]]
--   , 2
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_apad(anyarray, int);
create or replace function sm_sc.fv_apad
(
  i_array            anyarray      ,
  i_times            int   default 1
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if array_ndims(i_array) = 1
  then 
    v_fill_values := array[nullif(i_array[1], i_array[1])];
  elsif array_ndims(i_array) = 2
  then
    v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  elsif array_ndims(i_array) = 3
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  elsif array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  end if;

  return sm_sc.fv_apad(i_array, v_fill_values, i_times);

end
$$
language plpgsql stable
cost 100;


-- select sm_sc.fv_apad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     2
--   );
-- select sm_sc.fv_apad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     2
--   );
-- select sm_sc.fv_apad
--   (
--     array[array[1.0 :: float, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     2
--   );
-- select 
--   sm_sc.fv_apad
--   (
--     array[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]
--   , 2
--   );
-- select 
--   sm_sc.fv_apad
--   (
--     array[[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]],[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]]
--   , 2
--   );
-- 2d area_replace
-- drop function if exists sm_sc.fv_area_replace(anyarray, int[], anyarray);
create or replace function sm_sc.fv_area_replace
(
  i_array          anyarray,
  i_area_pos       int[],
  i_sub_tar_array  anyarray
)
returns anyarray
as
$$
-- declare -- here
begin
  -- 审查
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) > 4 
      or array_ndims(i_sub_tar_array) > 4 
      or array_ndims(i_area_pos) <> 1
      or array_ndims(i_sub_tar_array) <> array_length(i_area_pos, 1)
    then
      raise exception 'unsupport ndims!';
    elsif array_length(i_sub_tar_array, 1) > array_length(i_array, 1) - i_area_pos[1] + 1
      or array_length(i_sub_tar_array, 2) > array_length(i_array, 2) - i_area_pos[2] + 1
      or array_length(i_sub_tar_array, 3) > array_length(i_array, 3) - i_area_pos[3] + 1
      or array_length(i_sub_tar_array, 4) > array_length(i_array, 4) - i_area_pos[4] + 1
    then 
      raise exception 'length of i_sub_tar_array overflow beyond i_array.';
    end if;
  end if;

  if array_ndims(i_array) = 1
  then 
    i_array[i_area_pos[1] : i_area_pos[1] + array_length(i_sub_tar_array, 1) - 1]
      := i_sub_tar_array;
  elsif array_ndims(i_array) = 2
  then
    i_array[i_area_pos[1] : i_area_pos[1] + array_length(i_sub_tar_array, 1) - 1]
           [i_area_pos[2] : i_area_pos[2] + array_length(i_sub_tar_array, 2) - 1]
      := i_sub_tar_array;
  elsif array_ndims(i_array) = 3
  then
    i_array[i_area_pos[1] : i_area_pos[1] + array_length(i_sub_tar_array, 1) - 1]
           [i_area_pos[2] : i_area_pos[2] + array_length(i_sub_tar_array, 2) - 1]
           [i_area_pos[3] : i_area_pos[3] + array_length(i_sub_tar_array, 3) - 1]
      := i_sub_tar_array;
  elsif array_ndims(i_array) = 4
  then
    i_array[i_area_pos[1] : i_area_pos[1] + array_length(i_sub_tar_array, 1) - 1]
           [i_area_pos[2] : i_area_pos[2] + array_length(i_sub_tar_array, 2) - 1]
           [i_area_pos[3] : i_area_pos[3] + array_length(i_sub_tar_array, 3) - 1]
           [i_area_pos[4] : i_area_pos[4] + array_length(i_sub_tar_array, 4) - 1]
      := i_sub_tar_array;
  end if;
  
  return i_array;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_area_replace(array[['a', 'b', 'c'], ['e', 'f', 'g'], ['h', 'i', 'j'], ['x', 'y', 'z']], array[3, 2], array[['m', 'n'], ['p', 'q']])
-- select sm_sc.fv_area_replace
--   (
--     array[[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]]
--   , array[1 ,1, 1, 1]
--   , array[[[[7, 8],[9, 0]],[[7, 8],[9, 0]],[[7, 8],[9, 0]]]]
--   )
-- ------------------------------
-- 1d area_replace
-- drop function if exists sm_sc.fv_area_replace(anyarray, int, anyarray);
create or replace function sm_sc.fv_area_replace
(
  i_array          anyarray,
  i_area_pos_1d    int,
  i_sub_tar_array  anyarray
)
returns anyarray
as
$$
-- declare -- here
begin
  return sm_sc.fv_area_replace(i_array, array[i_area_pos_1d], i_sub_tar_array);
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_area_replace(array['a', 'b', 'c', 'e', 'f', 'g', 'h', 'i', 'j', 'x', 'y', 'z'], 6, array['m', 'n', 'p', 'q'])
-- drop function if exists sm_sc.fv_arr_remove_1d(anyarray, anyarray);
create or replace function sm_sc.fv_arr_remove_1d
(
  i_arr                anyarray
, i_remove_eles        anyarray
)
returns anyarray
as
$$
declare -- here
  v_cur    int;
begin
  for v_cur in 1 .. array_length(i_remove_eles, 1)
  loop 
    i_arr := array_remove(i_arr, i_remove_eles[v_cur]);
  end loop;
  
  return i_arr;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_arr_remove_1d(array[1,2,3,4,5,3,4,5,6,7], array[3,5]);
-- drop function if exists sm_sc.fv_array_1d_set_ele(anyarray, int, anyelement);
create or replace function sm_sc.fv_array_1d_set_ele
(
  i_array    anyarray,
  i_pos      int,    -- 可以为负值，由尾部向前追溯位置
  i_element  anyelement
)
returns anyarray
as
$$
-- declare
begin
  if i_pos between 1 and array_length(i_array, 1)
  then
    return i_array[: i_pos - 1] || i_element || i_array[i_pos + 1 : ];
  elsif i_pos between -array_length(i_array, 1) and -1 
  then
    return i_array[: array_length(i_array, 1) + i_pos] || i_element || i_array[array_length(i_array, 1) + i_pos + 2 : ];
  else
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 2, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -2, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 1, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 4, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -1, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -4, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 0, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 5, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -5, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 6, 5)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -6, 5)

-- ------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_array_1d_set_ele(anyarray, int, anyarray);
create or replace function sm_sc.fv_array_1d_set_ele
(
  i_array    anyarray,
  i_pos      int,    -- 可以为负值，由尾部向前追溯位置
  i_elements anyarray
)
returns anyarray
as
$$
-- declare
begin
  if i_pos between 1 and array_length(i_array, 1)
  then
    return i_array[: i_pos - 1] || i_elements || i_array[i_pos + 1 : ];
  elsif i_pos between -array_length(i_array, 1) and -1 
  then
    return i_array[: array_length(i_array, 1) + i_pos] || i_elements || i_array[array_length(i_array, 1) + i_pos + 2 : ];
  else
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 2, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -2, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 1, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 4, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -1, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -4, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 0, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 5, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -5, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], 6, array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], -6, array[5,6,7])

-- ------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_array_1d_set_ele(anyarray, int[], anyarray);
create or replace function sm_sc.fv_array_1d_set_ele
(
  i_array             anyarray  ,
  i_pos_range         int4range ,    -- 可以为负值，由尾部向前追溯位置；约束上下边界同号或为0
  i_elements          anyarray
)
returns anyarray
as
$$
-- declare
begin
  if lower(i_pos_range) < 0 and upper(i_pos_range) - 1 > 0
  then
    raise 'Unsupport!';
  elsif i_pos_range && int4range(1, array_length(i_array, 1), '[]')
  then
    return i_array[: coalesce(lower(i_pos_range), 1) - 1] || i_elements || i_array[coalesce(upper(i_pos_range), 1) : ];
  elsif i_pos_range && int4range(-array_length(i_array, 1), -1, '[]')
  then
    return i_array[: array_length(i_array, 1) + coalesce(lower(i_pos_range), 1)] || i_elements || i_array[array_length(i_array, 1) + coalesce(upper(i_pos_range), 1) + 1 : ];
  else
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[2, 3]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-3, -2] '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[1, 2]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[3, 4]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-2, -1] '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-4, -3] '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-2, 0]  '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[0, 2]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[4, 6]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[5, 6]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[6, 7]   '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-6, -4] '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-6, -5] '::int4range , array[5,6,7])
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-7, -6] '::int4range , array[5,6,7])

-- ------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_array_1d_set_ele(anyarray, int[], anyelement);
create or replace function sm_sc.fv_array_1d_set_ele
(
  i_array             anyarray  ,
  i_pos_range         int4range ,    -- 可以为负值，由尾部向前追溯位置；约束不可以上下边界同号或为0
  i_element           anyelement
)
returns anyarray
as
$$
-- declare
begin
  return sm_sc.fv_array_1d_set_ele(i_array, i_pos_range, array[i_element]);
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[2, 3]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-3, -2] '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[1, 2]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[3, 4]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-2, -1] '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-4, -3] '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-2, 0]  '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[0, 2]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[4, 6]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[5, 6]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[6, 7]   '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-6, -4] '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-6, -5] '::int4range , 6)
-- select sm_sc.fv_array_1d_set_ele(array[1,2,3,4], '[-7, -6] '::int4range , 6)
-- drop function if exists sm_sc.fv_atrim(anyarray, anyelement);
create or replace function sm_sc.fv_atrim
(
  i_array               anyarray,
  i_trim_element        anyelement
)
returns anyarray
as
$$
declare -- here
  v_y_cur                 int  ;
  v_x_cur                 int  ;
  v_x3_cur                int  ;
  v_x4_cur                int  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- []
  if array_ndims(i_array) = 1
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      if i_array[v_y_cur] <> i_trim_element
      then 
        exit;
      else 
        i_array[v_y_cur] = null;
      end if;
    end loop;
    return i_array;

  -- ([][])
  elsif array_ndims(i_array) =  2
  then
    for v_x_cur in 1 .. array_length(i_array, 2)
    loop 
      for v_y_cur in 1 .. array_length(i_array, 1)
      loop 
        if i_array[v_y_cur][v_x_cur] <> i_trim_element
        then 
          exit;
        else 
          i_array[v_y_cur][v_x_cur] = null;
        end if;
      end loop;
    end loop;
    return i_array;

  -- ([][][])
  elsif array_ndims(i_array) =  3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x3_cur in 1 .. array_length(i_array, 3)
      loop 
        for v_x_cur in 1 .. array_length(i_array, 2)
        loop 
          if i_array[v_y_cur][v_x_cur][v_x3_cur] <> i_trim_element
          then 
            exit;
          else 
            i_array[v_y_cur][v_x_cur][v_x3_cur] := null;
          end if;
        end loop;
      end loop;
    end loop;
    return i_array;

  -- ([][][][])
  elsif array_ndims(i_array) =  4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        for v_x4_cur in 1 .. array_length(i_array, 4)
        loop 
          for v_x3_cur in 1 .. array_length(i_array, 3)
          loop 
            if i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> i_trim_element
            then 
              exit;
            else 
              i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := null;
            end if;
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_atrim(array[[0,0,0,0,0,2], [0,0,1,2,0,4], [0,1,0,2,0,0], [0,0,0,0,1,2], [1,0,1,0,0,3]], 0)
-- select sm_sc.fv_atrim(array[0,1,2,3,4,5], 0)
-- drop function if exists sm_sc.fv_augmented(anyarray, int[2], int[2], anyelement);
create or replace function sm_sc.fv_augmented
(
  in i_arr                 anyarray, 
  in i_begin_pos           int[2],   -- 允许为负数
  in i_end_pos             int[2],   -- 不小于 begin_pos
  in i_augmented_val       anyelement   default null
)
  returns anyarray
as
$$
declare -- here
  v_arr_len_heigh      int          :=   array_length(i_arr, array_ndims(i_arr) - 1)    ;
  v_arr_len_width      int          :=   array_length(i_arr, array_ndims(i_arr))        ;
  v_range_arr_heigh    int4range    :=   int4range(1, v_arr_len_heigh + 1, '[)')            ;
  v_range_arr_width    int4range    :=   int4range(1, v_arr_len_width + 1, '[)')            ;
  v_range_tar_heigh    int4range    :=   int4range(i_begin_pos[1], i_end_pos[1] + 1, '[)')  ;
  v_range_tar_width    int4range    :=   int4range(i_begin_pos[2], i_end_pos[2] + 1, '[)')  ;
  v_range_shadow_heigh int4range    :=   v_range_arr_heigh * v_range_tar_heigh          ;
  v_range_shadow_width int4range    :=   v_range_arr_width * v_range_tar_width          ;
  v_ret                i_arr%type   ;
  -- -- v_cur_y          int;
  -- -- v_cur_x          int;

begin
  if array_ndims(i_arr) = 2
  then
    if not (isempty(v_range_shadow_heigh) or isempty(v_range_shadow_heigh))
    then
      -- -- pg 13 不支持数组变量切块（二维切片）赋值的语法，也尚未支持(select [][] into [..:..][..:..])语法  -- pg 14 开始支持数组变量切块（二维切片）赋值的语法
      v_ret := sm_sc.fv_new(i_augmented_val, array[i_end_pos[1] - i_begin_pos[1] + 1, i_end_pos[2] - i_begin_pos[2] + 1]);
      
      v_ret[greatest(1, 1 - i_begin_pos[1] + 1) : least(i_end_pos[1], v_arr_len_heigh) - i_begin_pos[1] + 1]
           [greatest(1, 1 - i_begin_pos[2] + 1) : least(i_end_pos[2], v_arr_len_width) - i_begin_pos[2] + 1]  
        := i_arr[lower(v_range_shadow_heigh) : upper(v_range_shadow_heigh) - 1]
                [lower(v_range_shadow_width) : upper(v_range_shadow_width) - 1];

      -- -- -- -- 使用临时表是pg11的临时方案，有代价：消耗表描述符，消耗oid，多了一次堆栈调用，至少多一次复制；期待未来pg版本对数组变量切块（二维切片）赋值的语法支持。
      -- -- create temp table t_tmp(col_tmp anyarray);
      -- -- insert into t_tmp(col_tmp) select v_ret;
      -- -- 
      -- -- update t_tmp
      -- -- set col_tmp[greatest(1, 1 - i_begin_pos[1] + 1) : least(i_end_pos[1], v_arr_len_heigh) - i_begin_pos[1] + 1]
      -- --            [greatest(1, 1 - i_begin_pos[2] + 1) : least(i_end_pos[2], v_arr_len_width) - i_begin_pos[2] + 1] 
      -- --   = i_arr[lower(v_range_shadow_heigh) : upper(v_range_shadow_heigh) - 1]
      -- --          [lower(v_range_shadow_width) : upper(v_range_shadow_width) - 1];
      -- -- v_ret := (select col_tmp from t_tmp limit 1);
      -- -- drop table if exists t_tmp;
      -- -- -- -- 

      -- for v_cur_y in 0 .. upper(v_range_shadow_heigh) - lower(v_range_shadow_heigh) - 1 by 1
      -- loop
      --   for v_cur_x in 0 .. upper(v_range_shadow_width) - lower(v_range_shadow_width) - 1 by 1
      --   loop
      --     v_ret[greatest(1, 1 - i_begin_pos[1] + 1) + v_cur_y]
      --                [greatest(1, 1 - i_begin_pos[2] + 1) + v_cur_x] 
      --       = i_arr[lower(v_range_shadow_heigh) + v_cur_y]
      --              [lower(v_range_shadow_width) + v_cur_x];
      --   end loop;
      -- end loop;

    end if;
    return v_ret;
  elsif array_ndims(i_arr) = 3
  then
    if not (isempty(v_range_shadow_heigh) or isempty(v_range_shadow_heigh))
    then
      v_ret := sm_sc.fv_new(i_augmented_val, array[array_length(i_arr, 1), i_end_pos[1] - i_begin_pos[1] + 1, i_end_pos[2] - i_begin_pos[2] + 1]);
      
      v_ret[ : ]
           [greatest(1, 1 - i_begin_pos[1] + 1) : least(i_end_pos[1], v_arr_len_heigh) - i_begin_pos[1] + 1]
           [greatest(1, 1 - i_begin_pos[2] + 1) : least(i_end_pos[2], v_arr_len_width) - i_begin_pos[2] + 1]  
        := i_arr[ : ]
                [lower(v_range_shadow_heigh) : upper(v_range_shadow_heigh) - 1]
                [lower(v_range_shadow_width) : upper(v_range_shadow_width) - 1];
    end if;
    return v_ret;
  elsif array_ndims(i_arr) = 4
  then
    if not (isempty(v_range_shadow_heigh) or isempty(v_range_shadow_heigh))
    then
      v_ret := sm_sc.fv_new(i_augmented_val, array[array_length(i_arr, 1), array_length(i_arr, 2), i_end_pos[1] - i_begin_pos[1] + 1, i_end_pos[2] - i_begin_pos[2] + 1]);
      
      v_ret[ : ]
           [ : ]
           [greatest(1, 1 - i_begin_pos[1] + 1) : least(i_end_pos[1], v_arr_len_heigh) - i_begin_pos[1] + 1]
           [greatest(1, 1 - i_begin_pos[2] + 1) : least(i_end_pos[2], v_arr_len_width) - i_begin_pos[2] + 1]  
        := i_arr[ : ]
                [ : ]
                [lower(v_range_shadow_heigh) : upper(v_range_shadow_heigh) - 1]
                [lower(v_range_shadow_width) : upper(v_range_shadow_width) - 1];
    end if;
    return v_ret;
  else
    raise exception 'no method for such range!';
  end if;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- -- 子集
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, 2], array[4, 4], 0)
-- -- 超集
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, -1], array[8, 8], 0)

-- -- 左上相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, -1], array[4, 4], 0)
-- -- 右下相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, 2], array[8, 8], 0)

-- -- 正上相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, 2], array[4, 4], 0)
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, -1], array[4, 8], 0)
-- -- 正下相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, 2], array[8, 4], 0)
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, -1], array[8, 8], 0)

-- -- 正左相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, -1], array[4, 4], 0)
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, 2], array[8, 8], 0)
-- -- 正右相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, 2], array[4, 8], 0)
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, -1], array[8, 8], 0)

-- -- 左下相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[2, -1], array[8, 4], 0)
-- -- 右上相交
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, 2], array[4, 8], 0)

-- -- 左上相离
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, -1], array[0, 0], 0)
-- -- 右下相离
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[8, 8], array[10, 10], 0)

-- -- 左下相离
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[8, -1], array[10, 2], 0)
-- -- 右上相离
-- select sm_sc.fv_augmented(array[array[1, 2, 3, 4, 5], array[6, 7, 8, 9, 10], array[11, 12, 13, 14, 15], array[16, 17, 18, 19, 20], array[21, 22, 23, 24, 25]], array[-1, 8], array[2, 10], 0)

-- -- 左上相交 3d 延拓
-- select sm_sc.fv_augmented(array[[[1, 2, 3, 4, 5], [6, 7, 8, 9, 10], [11, 12, 13, 14, 15], [16, 17, 18, 19, 20], [21, 22, 23, 24, 25]], [[-1, -2, -3, -4, -5], [-6, -7, -8, -9, -10], [-11, -12, -13, -14, -15], [-16, -17, -18, -19, -20], [-21, -22, -23, -24, -25]]], array[-1, -1], array[4, 4], 0)

-- -- 左上相交 4d 延拓
-- select sm_sc.fv_augmented(array[[[[1, 2, 3, 4, 5], [6, 7, 8, 9, 10], [11, 12, 13, 14, 15], [16, 17, 18, 19, 20], [21, 22, 23, 24, 25]], [[-1, -2, -3, -4, -5], [-6, -7, -8, -9, -10], [-11, -12, -13, -14, -15], [-16, -17, -18, -19, -20], [-21, -22, -23, -24, -25]]], [[[31, 32, 33, 34, 35], [36, 37, 38, 39, 40], [41, 42, 43, 44, 45], [46, 47, 48, 49, 50], [51, 52, 53, 54, 55]], [[-31, -32, -33, -34, -35], [-36, -37, -38, -39, -40], [-41, -42, -43, -44, -45], [-46, -47, -48, -49, -50], [-51, -52, -53, -54, -55]]]], array[-1, -1], array[4, 4], 0)
-- drop function if exists sm_sc.fv_bmask(anyarray, int[], anyelement);
create or replace function sm_sc.fv_bmask
(
  i_array               anyarray    ,
  i_mask_len            int[]       ,     -- 该控制参数的维度与 i_array 相同，背景面的 heigh 维度(最高维度)长度为 1，其他维度长度与 i_array 对应维度长度也相同。
  i_mask_element        anyelement
)
returns anyarray
as
$$
declare
  v_pos_range_s         int4range[] ;
  v_background_heigh    int         := array_length(i_array, array_ndims(i_array) - 1);
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 2 and 4 
    then 
      raise exception 'unsupport ndims of i_array';
    elsif array_ndims(i_mask_len) <> array_ndims(i_array)
    then 
      raise exception 'array_ndims of i_mask_len should be the same as i_array''s.';
    elsif array_length(i_mask_len, array_ndims(i_mask_len) - 1) <> 1 
    then 
      raise exception 'width of i_mask_len'' background should be 1.';
    elsif array_ndims(i_mask_len) = 2 
        and array_length(i_mask_len, 2) <> array_length(i_array, 2)
      or array_ndims(i_mask_len) = 3 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 3) <> array_length(i_array, 3)
            )
      or array_ndims(i_mask_len) = 4 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 2) <> array_length(i_array, 2) 
               or array_length(i_mask_len, 4) <> array_length(i_array, 4)
            )
    then 
      raise exception 'unmatched length between i_mask_len and i_array';
    elsif v_background_heigh < any(i_mask_len)
    then
      raise exception 'there should not be any element of i_mask_len > v_background_heigh.';
    end if;
  end if;
  
  if array_ndims(i_mask_len) = 2
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            nullif(int4range(v_background_heigh - i_mask_len[1][a_cur_x] + 1, v_background_heigh + 1, '[)'), 'empty' :: int4range)
          , int4range(a_cur_x, a_cur_x, '[]')
          ]
        )
      from generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
    );
    
  elsif array_ndims(i_mask_len) = 3
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , nullif(int4range(v_background_heigh - i_mask_len[a_cur_y][1][a_cur_x3] + 1, v_background_heigh + 1, '[)'), 'empty' :: int4range)
          , int4range(a_cur_x3, a_cur_x3, '[]')
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 3)) tb_a_cur_x3(a_cur_x3)
    );
    
  elsif array_ndims(i_mask_len) = 4 
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , int4range(a_cur_x, a_cur_x, '[]')
          , nullif(int4range(v_background_heigh - i_mask_len[a_cur_y][a_cur_x][1][a_cur_x4] + 1, v_background_heigh + 1, '[)'), 'empty' :: int4range)
          , int4range(a_cur_x4, a_cur_x4, '[]')
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_mask_len, 4)) tb_a_cur_x4(a_cur_x4)
    );
      
  end if;
  
  return 
    sm_sc.fv_pos_replaces
    (
      i_array
    , v_pos_range_s
    , i_mask_element
    )
  ;
  
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_bmask
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ]
--   , array[[3, 0, 2, 1, 4]]
--   , 0
--   )

-- select 
--   sm_sc.fv_bmask
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ]
--   , array[[[3, 0, 2, 1, 2]], [[1, 1, 4, 2, 3]], [[3, 2, 2, 1, 0]]]
--   , 0
--   )

-- select 
--   sm_sc.fv_bmask
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ]
--   , array[[[[3, 0, 2, 1, 3]], [[1, 1, 4, 2, 1]], [[3, 3, 2, 1, 1]]],[[[3, 0, 2, 1, 2]], [[1, 1, 4, 2, 4]], [[3, 3, 2, 1, 3]]]]
--   , 0
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_bpad(anyarray, anyarray, int);
create or replace function sm_sc.fv_bpad
(
  i_array            anyarray      ,
  i_fill_values      anyarray      , -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
  i_times            int        default  1   
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- set search_path to sm_sc;
  if i_array is null 
  then 
    return null;
  end if;
  
  -- if v_fill_values is null 
  -- then 
  --   if array_ndims(i_array) = 1
  --   then 
  --     v_fill_values := array[nullif(i_array[1], i_array[1])];
  --   elsif array_ndims(i_array) = 2
  --   then
  --     v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  --   elsif array_ndims(i_array) = 3
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  --   elsif array_ndims(i_array) = 4
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  --   end if;
  -- end if;

  if i_times <= 0
  then
    return i_array;
  elsif array_ndims(i_array) <> array_ndims(v_fill_values)
  then 
    raise exception 'ndims of i_array: % and i_fill_values: % should be the same.', array_ndims(i_array), array_ndims(v_fill_values);
  elsif array_ndims(i_array) = 1
  then 
    return
      i_array
      || 
      (
        select 
          sm_sc.fa_array_concat(v_fill_values)
        from generate_series(1, i_times)
      )
    ;
  elsif array_ndims(i_array) = 2
  then 
    if array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        i_array
        |-|| 
        (
          select 
            sm_sc.fa_mx_concat_y(v_fill_values)
          from generate_series(1, i_times)
        )
      ;
    end if;
  elsif array_ndims(i_array) = 3
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
      or array_length(v_fill_values, 3) not in (1, array_length(i_array, 3))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        i_array
        |-|| 
        (
          select 
            sm_sc.fa_mx_concat_x(v_fill_values)
          from generate_series(1, i_times)
        )
      ;
    end if;
  elsif array_ndims(i_array) = 4
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
      or array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
      or array_length(v_fill_values, 4) not in (1, array_length(i_array, 4))
    then 
      raise exception 'width of 3d i_fill_values should be the same as 3d i_array''s. ';
    else
      return
        i_array
        |-|| 
        (
          select 
            sm_sc.fa_mx_concat_x3(v_fill_values)
          from generate_series(1, i_times)
        )
      ;
    end if;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_bpad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     array[7, 8],
--     2
--   );
-- select sm_sc.fv_bpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7], array[8]],
--     2
--   );
-- select sm_sc.fv_bpad
--   (
--     array[array[1.0 :: float, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     array[array[7, 8], array[9, 10]] :: float[],
--     2
--   );
-- select sm_sc.fv_bpad
--   (
--     array[array[1, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     array[array[7], array[8]],
--     2
--   );
-- select sm_sc.fv_bpad
--   (
--     array[array[1.0 :: float, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     array[array[null :: float]],
--     2
--   );
-- select 
--   sm_sc.fv_bpad
--   (
--     array[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]
--   , array[[[7, -7],[8, -8],[9, -9],[10, -10],[11, -11],[12, -12]]]
--   , 2
--   );
-- select 
--   sm_sc.fv_bpad
--   (
--     array[[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]],[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]]
--   , array[[[[7, -7],[8, -8],[9, -9],[10, -10],[11, -11],[12, -12]]],[[[7, -7],[8, -8],[9, -9],[10, -10],[11, -11],[12, -12]]]]
--   , 2
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_bpad(anyarray, int);
create or replace function sm_sc.fv_bpad
(
  i_array            anyarray      ,
  i_times            int   default 1
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if array_ndims(i_array) = 1
  then 
    v_fill_values := array[nullif(i_array[1], i_array[1])];
  elsif array_ndims(i_array) = 2
  then
    v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  elsif array_ndims(i_array) = 3
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  elsif array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  end if;

  return sm_sc.fv_bpad(i_array, v_fill_values, i_times);

end
$$
language plpgsql stable
cost 100;


-- select sm_sc.fv_bpad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     2
--   );
-- select sm_sc.fv_bpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     2
--   );
-- select sm_sc.fv_bpad
--   (
--     array[array[1.0 :: float, -1], array[2, -2], array[3, -3], array[4, -4], array[5, -5], array[6, -6]],
--     2
--   );
-- select 
--   sm_sc.fv_bpad
--   (
--     array[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]
--   , 2
--   );
-- select 
--   sm_sc.fv_bpad
--   (
--     array[[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]],[[[1, -1],[2, -2],[3, -3],[4, -4],[5, -5],[6, -6],[21, -21],[22, -22],[23, -23],[24, -24],[25, -25],[26, -26]]]]
--   , 2
--   );
-- drop function if exists sm_sc.fv_btrim(anyarray, anyelement);
create or replace function sm_sc.fv_btrim
(
  i_array               anyarray,
  i_trim_element        anyelement
)
returns anyarray
as
$$
declare -- here
  v_y_cur                 int  ;
  v_x_cur                 int  ;
  v_x3_cur                int  ;
  v_x4_cur                int  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- ([][])
  if array_ndims(i_array) =  2
  then
    for v_x_cur in 1 .. array_length(i_array, 2)
    loop 
      for v_y_cur in reverse array_length(i_array, 1) .. 1
      loop 
        if i_array[v_y_cur][v_x_cur] <> i_trim_element
        then 
          exit;
        else 
          i_array[v_y_cur][v_x_cur] = null;
        end if;
      end loop;
    end loop;
    return i_array;

  -- []
  elsif array_ndims(i_array) = 1
  then
    for v_y_cur in reverse array_length(i_array, 1) .. 1
    loop 
      if i_array[v_y_cur] <> i_trim_element
      then 
        exit;
      else 
        i_array[v_y_cur] = null;
      end if;
    end loop;
    return i_array;

  -- ([][][])
  elsif array_ndims(i_array) =  3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x3_cur in 1 .. array_length(i_array, 3)
      loop 
        for v_x_cur in reverse array_length(i_array, 2) .. 1
        loop 
          if i_array[v_y_cur][v_x_cur][v_x3_cur] <> i_trim_element
          then 
            exit;
          else 
            i_array[v_y_cur][v_x_cur][v_x3_cur] := null;
          end if;
        end loop;
      end loop;
    end loop;
    return i_array;

  -- ([][][][])
  elsif array_ndims(i_array) =  4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        for v_x4_cur in 1 .. array_length(i_array, 4)
        loop 
          for v_x3_cur in reverse array_length(i_array, 3) .. 1
          loop 
            if i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> i_trim_element
            then 
              exit;
            else 
              i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := null;
            end if;
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_btrim(array[[2,0,0,0,0,0], [1,2,0,4,0,0], [0,0,1,0,2,0], [0,1,2,0,0,0], [1,0,1,0,0,3]], 0)
-- select sm_sc.fv_btrim(array[0,1,2,3,4,5, 0, 0], 0)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_chunk(anyarray, int[2][]);
create or replace function sm_sc.fv_chunk
(
  i_arr            anyarray,
  i_chunk_range    int[2][]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_arr) > 4 
      or array_ndims(i_arr) <> array_length(i_chunk_range, 2)
    then 
      raise exception 'unsupport ndims of i_arr or slice range.';
    elsif true = any(i_chunk_range[1][ : ] <=` 0)
      or true = any(i_chunk_range[1][ : ] >` i_chunk_range[2][ : ])
      or true = any(i_chunk_range[2][ : ] >` (select array[array_agg(array_length(i_arr, a_no) order by a_no)] from generate_series(1, array_ndims(i_arr)) tb_a(a_no)))
    then
      raise exception 'unmatched slice range.';
    end if;
  end if;
  
  if array_ndims(i_arr) = 1
  then 
    return 
      i_arr
        [i_chunk_range[1][1] : i_chunk_range[2][1]]
    ;
  elsif array_ndims(i_arr) = 2
  then 
    return 
      i_arr
        [i_chunk_range[1][1] : i_chunk_range[2][1]]
        [i_chunk_range[1][2] : i_chunk_range[2][2]]
    ;
  elsif array_ndims(i_arr) = 3
  then 
    return 
      i_arr
        [i_chunk_range[1][1] : i_chunk_range[2][1]]
        [i_chunk_range[1][2] : i_chunk_range[2][2]]
        [i_chunk_range[1][3] : i_chunk_range[2][3]]
    ;
  elsif array_ndims(i_arr) = 4
  then 
    return 
      i_arr
        [i_chunk_range[1][1] : i_chunk_range[2][1]]
        [i_chunk_range[1][2] : i_chunk_range[2][2]]
        [i_chunk_range[1][3] : i_chunk_range[2][3]]
        [i_chunk_range[1][4] : i_chunk_range[2][4]]
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_chunk
--   (
--     sm_sc.fv_new_rand(array[4,3,5,6])
--   , array[[2,1,2,1],[3,3,4,4]]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_coalesce(anyarray, anyarray);
create or replace function sm_sc.fv_coalesce
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] + []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] + array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return i_left;
  
  -- [] + []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := coalesce(i_left[v_y_cur], i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] + [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := coalesce(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] + [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = coalesce(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] + [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = coalesce(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_coalesce
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_coalesce
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_coalesce
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_coalesce(array[1], array[1,2,3]);
-- select sm_sc.fv_coalesce(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_coalesce(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_coalesce(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_coalesce
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_coalesce
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_coalesce
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_coalesce(anyarray, anynonarray);
create or replace function sm_sc.fv_coalesce
(
  i_left     anyarray    ,
  i_right    anynonarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- [][] + float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := coalesce(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] + float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := coalesce(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] + float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = coalesce(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right);
        end loop;    
      end loop;
    end loop;
    return i_left;
  
  -- [][][][] + float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = coalesce(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_coalesce(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_coalesce(array[]::float[], 8.8)
-- select sm_sc.fv_coalesce(array[]::float[], null)
-- select sm_sc.fv_coalesce(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_coalesce(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_coalesce(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_coalesce
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_coalesce
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_coalesce(anynonarray, anyarray);
create or replace function sm_sc.fv_coalesce
(
  i_left     anynonarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- float + [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := coalesce(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float + []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := coalesce(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- float + [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = coalesce(i_left, i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float + [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = coalesce(i_left, i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_coalesce(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_coalesce(8.8, array[]::float[] )
-- select sm_sc.fv_coalesce(null, array[]::float[])
-- select sm_sc.fv_coalesce(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_coalesce(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_coalesce(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_coalesce
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_coalesce
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_coalesce_variadic(variadic anyarray);
create or replace function sm_sc.fv_coalesce_variadic
(
  variadic i_array  anyarray
)
  returns anyarray
as
$$
-- declare here

begin
  return 
    (select sm_sc.fa_mx_coalesce(sm_sc.fv_mx_descend_dim(i_array[a_no : a_no]) order by a_no) from generate_series(1, array_length(i_array, 1)) tb_a(a_no))
  ;
  
  -- if array_ndims(i_array) = 2
  -- then
  --   return 
  --   (
  --     with 
  --     cte_y as
  --     (
  --       select 
  --         a_y,
  --         sm_sc.fa_coalesce(i_array[a_variadic][a_y] order by a_variadic) as a_y_ele
  --       from generate_series(1, array_length(i_array, 1)) tb_a_variadic(a_variadic)
  --         , generate_series(1, array_length(i_array, 2)) tb_a_y(a_y)
  --       group by a_y
  --     )
  --     select 
  --       array_agg(a_y_ele order by a_y) 
  --     from cte_y
  --   );
  -- elsif array_ndims(i_array) = 3
  -- then
  --   return
  --   (
  --     with 
  --     cte_yx as
  --     (
  --       select 
  --         a_y,
  --         a_x,
  --         sm_sc.fa_coalesce(i_array[a_variadic][a_y][a_x] order by a_variadic) as a_dims_ele
  --       from generate_series(1, array_length(i_array, 1)) tb_a_variadic(a_variadic)
  --         , generate_series(1, array_length(i_array, 2)) tb_a_y(a_y)
  --         , generate_series(1, array_length(i_array, 3)) tb_a_x(a_x)
  --       group by a_y, a_x
  --     ),
  --     cte_x as
  --     (
  --       select 
  --         a_y,
  --         array_agg(a_dims_ele order by a_x) as a_x_ele
  --       from cte_yx
  --       group by a_y
  --     )
  --     select 
  --       array_agg(a_x_ele order by a_y) 
  --     from cte_x
  --   );
  -- else
  --   raise exception 'unsurpport ndim!';
  -- end if; 
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- -- select sm_sc.fv_coalesce_variadic(1.2, 2.3, 5.6, 52.1)
-- select sm_sc.fv_coalesce_variadic(variadic array[array[null, 2.4, 5.6, 52.3], array[1.2, 2.3, null, 52.1]])
-- select sm_sc.fv_coalesce_variadic(variadic array[array[array[null, 2.4, 5.6, null], array[1.2, 2.3, null, 25.6]], array[array[1, 2, null, 4], array[5, 6, 7, 8]]])

-- drop function if exists sm_sc.fv_ele_replace(anyarray, anyarray, anyelement);
create or replace function sm_sc.fv_ele_replace
(
  i_array            anyarray,
  i_old_elements     anyarray,
  i_new_element      anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur                 int  := 1  ;
  v_y_cur                 int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  -- pg 尚未支持 is not distinct from any(anyarray) 语法。
  v_if_null_old_element   boolean  := (select true from unnest(i_old_elements) tb_a(a_ele) where a_ele is null);
begin
  -- ele_replace(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- ele_replace([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        if i_array[v_y_cur][v_x_cur] = any(i_old_elements) 
          or i_array[v_y_cur][v_x_cur] is null and v_if_null_old_element
        then
          i_array[v_y_cur][v_x_cur] := i_new_element;
        end if;
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- ele_replace []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
        if i_array[v_y_cur] = any(i_old_elements)
          or i_array[v_y_cur] is null and v_if_null_old_element
        then
          i_array[v_y_cur] := i_new_element;
        end if;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- ele_replace [][][]
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          if i_array[v_y_cur][v_x_cur][v_x3_cur] = any(i_old_elements)
            or i_array[v_y_cur][v_x_cur][v_x3_cur] is null and v_if_null_old_element
          then
            i_array[v_y_cur][v_x_cur][v_x3_cur] := i_new_element;
          end if;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- ele_replace [][][][]
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            if i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = any(i_old_elements)
              or i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] is null and v_if_null_old_element
            then
              i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := i_new_element;
            end if;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_ele_replace(array[array[12.3, 25.1], array[3.25, 6.1]], array[12.3, 3.25], 1.111)
-- select sm_sc.fv_ele_replace(array[[[12.3, 25.1], [3.25, 6.1]]], array[12.3, 3.25], 1.111)
-- select sm_sc.fv_ele_replace(array[[[[12.3, 25.1], [3.25, 6.1]]]], array[12.3, 3.25], 1.111)
-- select sm_sc.fv_ele_replace(array[12.3, 25.1, 28.33], array[25.1, 12.3], 2.222)
-- select sm_sc.fv_ele_replace(array[]::float[], array[25.1, 12.3], 2.222)
-- select sm_sc.fv_ele_replace(array[null, 25.1, 28.33], array[null, 12.3], 2.222) 
-- select sm_sc.fv_ele_replace(array[12.3, 25.1, 28.33], array[25.1, 12.3], null)   -- 相当于 slice_nullif
-- select sm_sc.fv_ele_replace(array[array[], array []]::float[], array[25.1, 12.3], 2.222)
-- drop function if exists sm_sc.fv_eye(anyelement, int, variadic anyarray);
create or replace function sm_sc.fv_eye
(
           i_fill_idle_value           anyelement, 
           i_dim                       int       ,
  variadic i_identity_values           anyarray
)
  returns anyarray
as
$$
declare -- here
  v_len    int                         :=    array_length(i_identity_values, 1);
  v_ret    i_identity_values%type      ;
  v_cur    int;
begin
  if i_dim not between 2 and 4
  then 
    raise exception 'unsupport dim > 4';
  end if;
  
  v_ret :=    array_fill(i_fill_idle_value, array_fill(v_len, array[i_dim]));
  
  if i_dim = 2
  then
    for v_cur in 1 .. v_len
    loop 
      v_ret[v_cur][v_cur] := i_identity_values[v_cur];
    end loop;
  elsif i_dim = 3
  then
    for v_cur in 1 .. v_len
    loop 
      v_ret[v_cur][v_cur][v_cur] := i_identity_values[v_cur];
    end loop;
  elsif i_dim = 4
  then
    for v_cur in 1 .. v_len
    loop 
      v_ret[v_cur][v_cur][v_cur][v_cur] := i_identity_values[v_cur];
    end loop;
  end if;
  
  return v_ret;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_eye(0.2 :: float, 3, variadic array[1,2,3,4,5] :: float[])
-- select sm_sc.fv_eye(0.0 :: float, 4, variadic array[1.2, 2.3, 5.6, 52.1] :: float[])

-- -------------------------------------------------------------------------------------------------------------------------
-- -- -- -- 以下部分为对 float 元素类型的支持，对于其他类型，需额外定义多态类型
-- -- -- drop type if exists sm_sc.__typ_array_ex;
-- -- create type sm_sc.__typ_array_ex 
-- -- as
-- -- (
-- --   m_array    float[]
-- -- );
-- ---------------------------
-- -- drop function if exists sm_sc.fv_cast_array_ex;
-- create or replace function sm_sc.fv_cast_array_ex 
-- (
--   i_array     float[]    
-- )
-- returns sm_sc.__typ_array_ex
-- as
-- $$
-- declare -- here
--   v_rtn     sm_sc.__typ_array_ex;
-- begin
--   v_rtn.m_array = i_array;
--   return v_rtn;
-- end
-- $$
-- language plpgsql stable
-- parallel safe
-- cost 100;
-- -- select sm_sc.fv_cast_array_ex(array[1.2, 2.4])
-- -- select sm_sc.fv_cast_array_ex(array[array[1.2, 2.4], array[3.6, 4.8]])
-- -- ---------------------------
-- drop cast if exists (float[] as sm_sc.__typ_array_ex);
-- create cast (float[] as sm_sc.__typ_array_ex)   
-- with function sm_sc.fv_cast_array_ex(float[])
-- as implicit;

-- ---------------------------
-- drop function if exists sm_sc.fv_eye_arr_dense(variadic sm_sc.__typ_array_ex[]);
create or replace function sm_sc.fv_eye_arr_dense
(
  i_fill_idle_value float,
  variadic i_arrays sm_sc.__typ_array_ex[]   -- 可以不是方阵
)
  returns float[]
as
$$
declare -- here
  v_ret_arr        float[];
  v_cur_pos        int    :=  1;
  v_cur_arr        sm_sc.__typ_array_ex;
  v_cur_y          int;
  v_cur_x          int;
  v_ret_len_y      int;
  v_ret_len_x      int;
begin
  if exists (select  from generate_subscripts(i_arrays, 1) tb(col_cur) where array_ndims(i_arrays[col_cur].m_array) <> 2) 
  then
    raise exception 'no method for such ndim!';
  else

    -- 计算结果矩阵的长宽
    -- v_ret_len = (select sum(least(array_length(i_arrays[col_cur].m_array, 1), array_length(i_arrays[col_cur].m_array, 2))) from generate_subscripts(i_arrays, 1) tb(col_cur));
    -- raise notice  'v_ret_len: %', v_ret_len;

    with 
    -- 形状列表
    cte_shape as
    (
      select 
        col_cur
      , array_length(i_arrays[col_cur].m_array, 1) as len_y
      , array_length(i_arrays[col_cur].m_array, 2) as len_x 
      from generate_subscripts(i_arrays, 1) tb(col_cur)
    ),
    -- 每一层方阵大小
    cte_cur_squar as
    (
      select 
        col_cur
      , sum(least(len_y, len_x)) over(order by col_cur) as cur_squar 
      from cte_shape
    ),
    -- 每一层方阵右下角作为下一层起点
    cte_cur_start_pos as
    (
      select 
        col_cur
      , (lag(cur_squar :: int, 1, 0) over(order by col_cur)) + 1 as cur_start_pos 
      from cte_cur_squar
    ),
    -- 本层右下角 yx 坐标
    cte_cur_end_pos as
    (
      select 
        col_cur
      , cur_start_pos + array_length(i_arrays[col_cur].m_array, 1) - 1 as cur_end_pos_y
      , cur_start_pos + array_length(i_arrays[col_cur].m_array, 2) - 1 as cur_end_pos_x 
      from cte_cur_start_pos
    )
    -- 所有层坐标的最大值即返回结果矩阵的形状
    select 
      max(cur_end_pos_y), max(cur_end_pos_x) 
    into v_ret_len_y, v_ret_len_x
    from cte_cur_end_pos;

    -- 以背景元素值初始化矩阵
    v_ret_arr = array_fill(i_fill_idle_value, array[v_ret_len_y, v_ret_len_x]);

    -- 修改对角线
    foreach v_cur_arr in array i_arrays
    loop
      -- pg 13 不支持数组变量切块（二维切片）赋值的语法，也尚未支持(select [][] into [..:..][..:..])语法  -- pg 14 开始支持数组变量切块（二维切片）赋值的语法
      v_ret_arr[v_cur_pos : v_cur_pos + array_length(v_cur_arr.m_array, 1) - 1]
             [v_cur_pos : v_cur_pos + array_length(v_cur_arr.m_array, 2) - 1]
        = v_cur_arr.m_array;
      -- -- for v_cur_y in 0 .. array_length(v_cur_arr.m_array, 1) - 1 by 1
      -- -- loop
      -- --   for v_cur_x in 0 .. array_length(v_cur_arr.m_array, 2) - 1 by 1
      -- --   loop
      -- --     v_ret_arr[v_cur_pos + v_cur_y]
      -- --                [v_cur_pos + v_cur_x] 
      -- --       = v_cur_arr.m_array[v_cur_y + 1]
      -- --                             [v_cur_x + 1];
      -- --   end loop;
      -- -- end loop;

      v_cur_pos = v_cur_pos + least(array_length(v_cur_arr.m_array, 1), array_length(v_cur_arr.m_array, 2));
    end loop;

    return v_ret_arr;
  end if;

end
$$
  language plpgsql stable
parallel safe
  cost 100;
-- select sm_sc.fv_eye_arr_dense(0.5, array[[1, 1], [1, 1.0]] :: float[], array[[2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2]] :: float[], array[[3, 3, 3], [3, 3, 3], [3, 3, 3]] :: float[])
-- select sm_sc.fv_eye_arr_dense(0.8, array[[1.2, 2.4], [11.2, 12.4]] :: float[], array[[1.3, 2.6, 1.4, 2.8], [11.3, 12.6, 11.4, 12.8]] :: float[], array[[1.1, 2.2], [11.1, 12.2], [21.1, 22.2]] :: float[])
-- ---------------------------
-- drop function if exists sm_sc.fv_eye_arr_dense(variadic sm_sc.__typ_array_ex[]);
create or replace function sm_sc.fv_eye_arr_dense
(
  variadic i_arrays sm_sc.__typ_array_ex[]   -- 可以不是方阵
)
  returns float[]
as
$$
-- declare -- here
begin
  return sm_sc.fv_eye_arr_dense(0.0 :: float, variadic i_arrays);
end
$$
  language plpgsql stable
parallel safe
  cost 100;
-- select sm_sc.fv_eye_arr_dense(array[[1.0, 1], [1, 1]] :: float[], array[[2.0, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2]] :: float[], array[[3.0, 3, 3], [3, 3, 3], [3, 3, 3]] :: float[])
-- select sm_sc.fv_eye_arr_dense(array[[1.2, 2.4], [11.2, 12.4]] :: float[], array[[1.3, 2.6, 1.4, 2.8], [11.3, 12.6, 11.4, 12.8]] :: float[], array[[1.1, 2.2], [11.1, 12.2], [21.1, 22.2]] :: float[])
-- -------------------------------------------------------------------------------------------------------------------------
-- -- -- -- 以下部分为对 float 元素类型的支持，对于其他类型，需额外定义多态类型
-- -- -- drop type if exists sm_sc.__typ_array_ex;
-- -- create type sm_sc.__typ_array_ex 
-- -- as
-- -- (
-- --   m_array    float[]
-- -- );
-- ---------------------------
-- -- drop function if exists sm_sc.fv_cast_array_ex;
-- create or replace function sm_sc.fv_cast_array_ex 
-- (
--   i_array     float[]    
-- )
-- returns sm_sc.__typ_array_ex
-- as
-- $$
-- declare -- here
--   v_rtn     sm_sc.__typ_array_ex;
-- begin
--   v_rtn.m_array = i_array;
--   return v_rtn;
-- end
-- $$
-- language plpgsql stable
-- parallel safe
-- cost 100;
-- -- select sm_sc.fv_cast_array_ex(array[1.2, 2.4])
-- -- select sm_sc.fv_cast_array_ex(array[array[1.2, 2.4], array[3.6, 4.8]])
-- -- ---------------------------
-- drop cast if exists (float[] as sm_sc.__typ_array_ex);
-- create cast (float[] as sm_sc.__typ_array_ex)   
-- with function sm_sc.fv_cast_array_ex(float[])
-- as implicit;

-- ---------------------------
-- drop function if exists sm_sc.fv_eye_arr_sparse(variadic sm_sc.__typ_array_ex[]);
create or replace function sm_sc.fv_eye_arr_sparse
(
  i_fill_idle_value float,
  variadic i_arrays sm_sc.__typ_array_ex[]   -- 可以不是方阵
)
  returns float[]
as
$$
declare -- here
  v_ret_arr        float[];
  v_cur_pos        int    :=  1;
  v_cur_arr        sm_sc.__typ_array_ex;
  v_cur_y          int;
  v_cur_x          int;
  v_ret_len_y      int;
  v_ret_len_x      int;
begin
  if exists (select  from generate_subscripts(i_arrays, 1) tb(col_cur) where array_ndims(i_arrays[col_cur].m_array) <> 2) 
  then
    raise exception 'no method for such ndim!';
  else

    -- 计算结果矩阵的长宽
    -- v_ret_len = (select sum(greatest(array_length(i_arrays[col_cur].m_array, 1), array_length(i_arrays[col_cur].m_array, 2))) from generate_subscripts(i_arrays, 1) tb(col_cur));
    -- raise notice  'v_ret_len: %', v_ret_len;

    with 
    -- 形状列表
    cte_shape as
    (
      select 
        col_cur
      , array_length(i_arrays[col_cur].m_array, 1) as len_y
      , array_length(i_arrays[col_cur].m_array, 2) as len_x 
      from generate_subscripts(i_arrays, 1) tb(col_cur)
    ),
    -- 每一层方阵大小
    cte_cur_squar as
    (
      select 
        col_cur
      , sum(greatest(len_y, len_x)) over(order by col_cur) as cur_squar 
      from cte_shape
    ),
    -- 每一层方阵右下角作为下一层起点
    cte_cur_start_pos as
    (
      select 
        col_cur
      , (lag(cur_squar :: int, 1, 0) over(order by col_cur)) + 1 as cur_start_pos 
      from cte_cur_squar
    ),
    -- 本层右下角 yx 坐标
    cte_cur_end_pos as
    (
      select 
        col_cur
      , cur_start_pos + array_length(i_arrays[col_cur].m_array, 1) - 1 as cur_end_pos_y
      , cur_start_pos + array_length(i_arrays[col_cur].m_array, 2) - 1 as cur_end_pos_x 
      from cte_cur_start_pos
    )
    -- 所有层坐标的最大值即返回结果矩阵的形状，且保证返回矩阵为方阵
    select 
      greatest(max(cur_end_pos_y), max(cur_end_pos_x))
    into v_ret_len_y  -- , v_ret_len_x
    from cte_cur_end_pos;
    
    v_ret_len_x := v_ret_len_y;

    -- 以背景元素值初始化矩阵
    v_ret_arr = array_fill(i_fill_idle_value, array[v_ret_len_y, v_ret_len_x]);

    -- 修改对角线
    foreach v_cur_arr in array i_arrays
    loop
      -- pg 13 不支持数组变量切块（二维切片）赋值的语法，也尚未支持(select [][] into [..:..][..:..])语法  -- pg 14 开始支持数组变量切块（二维切片）赋值的语法
      v_ret_arr[v_cur_pos : v_cur_pos + array_length(v_cur_arr.m_array, 1) - 1]
             [v_cur_pos : v_cur_pos + array_length(v_cur_arr.m_array, 2) - 1]
        = v_cur_arr.m_array;
      -- -- for v_cur_y in 0 .. array_length(v_cur_arr.m_array, 1) - 1 by 1
      -- -- loop
      -- --   for v_cur_x in 0 .. array_length(v_cur_arr.m_array, 2) - 1 by 1
      -- --   loop
      -- --     v_ret_arr[v_cur_pos + v_cur_y]
      -- --                [v_cur_pos + v_cur_x] 
      -- --       = v_cur_arr.m_array[v_cur_y + 1]
      -- --                             [v_cur_x + 1];
      -- --   end loop;
      -- -- end loop;

      v_cur_pos = v_cur_pos + greatest(array_length(v_cur_arr.m_array, 1), array_length(v_cur_arr.m_array, 2));
    end loop;

    return v_ret_arr;
  end if;

end
$$
  language plpgsql stable
parallel safe
  cost 100;
-- select sm_sc.fv_eye_arr_sparse(0.5, array[[1, 1], [1, 1.0]] :: float[], array[[2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2]] :: float[], array[[3, 3, 3], [3, 3, 3], [3, 3, 3]] :: float[])
-- select sm_sc.fv_eye_arr_sparse(0.8, array[[1.2, 2.4], [11.2, 12.4]] :: float[], array[[1.3, 2.6, 1.4, 2.8], [11.3, 12.6, 11.4, 12.8]] :: float[], array[[1.1, 2.2], [11.1, 12.2], [21.1, 22.2]] :: float[])
-- ---------------------------
-- drop function if exists sm_sc.fv_eye_arr_sparse(variadic sm_sc.__typ_array_ex[]);
create or replace function sm_sc.fv_eye_arr_sparse
(
  variadic i_arrays sm_sc.__typ_array_ex[]   -- 可以不是方阵
)
  returns float[]
as
$$
-- declare -- here
begin
  return sm_sc.fv_eye_arr_sparse(0.0 :: float, variadic i_arrays);
end
$$
  language plpgsql stable
parallel safe
  cost 100;
-- select sm_sc.fv_eye_arr_sparse(array[[1.0, 1], [1, 1]] :: float[], array[[2.0, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2]] :: float[], array[[3.0, 3, 3], [3, 3, 3], [3, 3, 3]] :: float[])
-- select sm_sc.fv_eye_arr_sparse(array[[1.2, 2.4], [11.2, 12.4]] :: float[], array[[1.3, 2.6, 1.4, 2.8], [11.3, 12.6, 11.4, 12.8]] :: float[], array[[1.1, 2.2], [11.1, 12.2], [21.1, 22.2]] :: float[])
-- drop function if exists sm_sc.fv_eye_unit(float, int, int[]);
create or replace function sm_sc.fv_eye_unit
(
  in i_array_len                 int
, in i_identity_value            float
, in i_array_len_ex_heigh_width  int[] default null
)
  returns float[]
as
$$
declare -- here
  v_arr float[] := array_fill(0.0 :: float, i_array_len_ex_heigh_width || array[i_array_len, i_array_len]);
  v_cur int     ;
  v_arr_identity_value float[]:= array_fill(i_identity_value, i_array_len_ex_heigh_width || array[1,1]);
begin
  if array_length(i_array_len_ex_heigh_width, 1) = 1   -- 3d
  then 
    for v_cur in 1 .. i_array_len
    loop 
      v_arr[ : ][v_cur : v_cur][v_cur : v_cur] := v_arr_identity_value;
    end loop;
  elsif array_length(i_array_len_ex_heigh_width, 1) = 2   -- 4d
  then 
    for v_cur in 1 .. i_array_len
    loop 
      v_arr[ : ][ : ][v_cur : v_cur][v_cur : v_cur] := v_arr_identity_value;
    end loop;
  else -- 2d
    for v_cur in 1 .. i_array_len
    loop 
      v_arr[v_cur][v_cur] := i_identity_value;
    end loop;
  end if;

  return v_arr;
  -- -- return 
  -- -- (
  -- --   select 
  -- --     array_agg
  -- --     (
  -- --       array_x
  -- --       order by v_cur_y
  -- --     )
  -- --   from 
  -- --   (
  -- --     select v_cur_y,
  -- --       array_agg(case when v_cur_x = v_cur_y then i_identity_value else 0.0 end order by v_cur_x) as array_x
  -- --     from 
  -- --       generate_series(1,i_array_len) as v_cur_x
  -- --     , generate_series(1,i_array_len) as v_cur_y
  -- --     group by v_cur_y
  -- --   ) t_a_array_x
  -- -- )	
  -- -- ;

end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_eye_unit(5, 3534.776)
-- select sm_sc.fv_eye_unit(5, 3534.776, array[3])
-- select sm_sc.fv_eye_unit(5, 3534.776, array[3, 4])


-- ---------------------------------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_eye_unit(int);
create or replace function sm_sc.fv_eye_unit
(
  in i_array_len int
)
  returns float[]
as
$$
-- declare here

begin
  return sm_sc.fv_eye_unit(i_array_len, 1.0 :: float);
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_eye_unit(5)
-- drop function if exists sm_sc.fv_lmask(anyarray, int[], anyelement);
create or replace function sm_sc.fv_lmask
(
  i_array               anyarray    ,
  i_mask_len            int[]       ,     -- 该控制参数的维度与 i_array 相同，背景面的 width 维度(最高维度)长度为 1，其他维度长度与 i_array 对应维度长度也相同。
  i_mask_element        anyelement
)
returns anyarray
as
$$
declare
  v_pos_range_s         int4range[] ;
  v_background_width    int         := array_length(i_array, array_ndims(i_array));
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 2 and 4 
    then 
      raise exception 'unsupport ndims of i_array';
    elsif array_ndims(i_mask_len) <> array_ndims(i_array)
    then 
      raise exception 'array_ndims of i_mask_len should be the same as i_array''s.';
    elsif array_length(i_mask_len, array_ndims(i_mask_len)) <> 1 
    then 
      raise exception 'width of i_mask_len'' background should be 1.';
    elsif array_ndims(i_mask_len) = 2 
        and array_length(i_mask_len, 1) <> array_length(i_array, 1)
      or array_ndims(i_mask_len) = 3 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 2) <> array_length(i_array, 2)
            )
      or array_ndims(i_mask_len) = 4 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 2) <> array_length(i_array, 2) 
               or array_length(i_mask_len, 3) <> array_length(i_array, 3)
            )
    then 
      raise exception 'unmatched length between i_mask_len and i_array';
    elsif v_background_width < any(i_mask_len)
    then
      raise exception 'there should not be any element of i_mask_len > v_background_width.';
    end if;
  end if;
  
  if array_ndims(i_mask_len) = 2
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , nullif(int4range(1, i_mask_len[a_cur_y][1] + 1, '[)'), 'empty' :: int4range)
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
    );
    
  elsif array_ndims(i_mask_len) = 3
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , int4range(a_cur_x, a_cur_x, '[]')
          , nullif(int4range(1, i_mask_len[a_cur_y][a_cur_x][1] + 1, '[)'), 'empty' :: int4range)
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
    );
    
  elsif array_ndims(i_mask_len) = 4 
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , int4range(a_cur_x, a_cur_x, '[]')
          , int4range(a_cur_x3, a_cur_x3, '[]')
          , nullif(int4range(1, i_mask_len[a_cur_y][a_cur_x][a_cur_x3][1] + 1, '[)'), 'empty' :: int4range)
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_mask_len, 3)) tb_a_cur_x3(a_cur_x3)
    );
      
  end if;
  
  return 
    sm_sc.fv_pos_replaces
    (
      i_array
    , v_pos_range_s
    , i_mask_element
    )
  ;
  
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_lmask
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ]
--   , array[[3], [0], [2], [1]]
--   , 0
--   )

-- select 
--   sm_sc.fv_lmask
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ]
--   , array[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]
--   , 0
--   )

-- select 
--   sm_sc.fv_lmask
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ]
--   , array[[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]],[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]]
--   , 0
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_lower_tri_mx(anyarray, anyelement);
create or replace function sm_sc.fv_lower_tri_mx
(
  i_array            anyarray      ,
  i_upper_fill_value anyelement        -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare 
  v_upper_fill_value    i_upper_fill_value%type  := i_upper_fill_value;
  v_array_heigh         int                      := array_length(i_array, array_ndims(i_array) - 1);
  v_array_width         int                      := array_length(i_array, array_ndims(i_array));
  v_array_len_y         int                      := array_length(i_array, 1);
  v_array_len_x         int                      := array_length(i_array, 2);
begin
  -- set search_path to sm_sc;
  
  if i_array is null 
  then 
    return null;
  end if;
  
  if v_upper_fill_value is null and array_ndims(i_array) = 1
  then 
    v_upper_fill_value := nullif(i_array[1], i_array[1]);
  elsif v_upper_fill_value is null and array_ndims(i_array) = 2
  then
    v_upper_fill_value := nullif(i_array[1][1], i_array[1][1]);
  elsif v_upper_fill_value is null and array_ndims(i_array) = 3
  then
    v_upper_fill_value := nullif(i_array[1][1][1], i_array[1][1][1]);
  elsif v_upper_fill_value is null and array_ndims(i_array) = 4
  then
    v_upper_fill_value := nullif(i_array[1][1][1][1], i_array[1][1][1][1]);
  end if;

  if array_ndims(i_array) = 2
  then 
    for v_cur in 1 .. case when v_array_heigh >= v_array_width then v_array_width - 1 else v_array_heigh end
    loop 
      i_array
        [v_cur : v_cur]
        [v_cur + 1 : ] 
      := 
        array_fill
        (
          v_upper_fill_value
        , array
          [
            1
          , v_array_width - v_cur
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 3
  then 
    for v_cur in 1 .. case when v_array_heigh >= v_array_width then v_array_width - 1 else v_array_heigh end
    loop 
      i_array
        [ : ]
        [v_cur : v_cur]
        [v_cur + 1 : ] 
      := 
        array_fill
        (
          v_upper_fill_value
        , array
          [
            v_array_len_y
          , 1
          , v_array_width - v_cur
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 4
  then 
    for v_cur in 1 .. case when v_array_heigh >= v_array_width then v_array_width - 1 else v_array_heigh end
    loop 
      i_array
        [ : ]
        [ : ]
        [v_cur : v_cur]
        [v_cur + 1 : ] 
      := 
        array_fill
        (
          v_upper_fill_value
        , array
          [
            v_array_len_y
          , v_array_len_x
          , 1
          , v_array_width - v_cur
          ]
        )
      ;
    end loop;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
  
  return i_array;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   , 0
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_lower_tri_mx(anyarray);
create or replace function sm_sc.fv_lower_tri_mx
(
  i_array            anyarray
)
returns anyarray
as
$$
-- declare 
begin
  return sm_sc.fv_lower_tri_mx(i_array, null);
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   );

-- set search_path to sm_sc;
-- 相比 fv_lower_tri_mx, fv_lower_tri_mx_ex 的入参对角线也被替换
-- drop function if exists sm_sc.fv_lower_tri_mx_ex(anyarray, anyelement);
create or replace function sm_sc.fv_lower_tri_mx_ex
(
  i_array            anyarray      ,
  i_upper_fill_value anyelement        -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare 
  v_upper_fill_value    i_upper_fill_value%type  := i_upper_fill_value;
  v_array_heigh         int                      := array_length(i_array, array_ndims(i_array) - 1);
  v_array_width         int                      := array_length(i_array, array_ndims(i_array));
  v_array_len_y         int                      := array_length(i_array, 1);
  v_array_len_x         int                      := array_length(i_array, 2);
begin
  -- set search_path to sm_sc;
  
  if i_array is null 
  then 
    return null;
  end if;
  
  if v_upper_fill_value is null and array_ndims(i_array) = 1
  then 
    v_upper_fill_value := nullif(i_array[1], i_array[1]);
  elsif v_upper_fill_value is null and array_ndims(i_array) = 2
  then
    v_upper_fill_value := nullif(i_array[1][1], i_array[1][1]);
  elsif v_upper_fill_value is null and array_ndims(i_array) = 3
  then
    v_upper_fill_value := nullif(i_array[1][1][1], i_array[1][1][1]);
  elsif v_upper_fill_value is null and array_ndims(i_array) = 4
  then
    v_upper_fill_value := nullif(i_array[1][1][1][1], i_array[1][1][1][1]);
  end if;

  if array_ndims(i_array) = 2
  then 
    for v_cur in 1 .. least(v_array_heigh, v_array_width)
    loop 
      i_array
        [v_cur : v_cur]
        [v_cur : ] 
      := 
        array_fill
        (
          v_upper_fill_value
        , array
          [
            1
          , v_array_width - v_cur + 1
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 3
  then 
    for v_cur in 1 .. least(v_array_heigh, v_array_width)
    loop 
      i_array
        [ : ]
        [v_cur : v_cur]
        [v_cur : ] 
      := 
        array_fill
        (
          v_upper_fill_value
        , array
          [
            v_array_len_y
          , 1
          , v_array_width - v_cur + 1
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 4
  then 
    for v_cur in 1 .. least(v_array_heigh, v_array_width)
    loop 
      i_array
        [ : ]
        [ : ]
        [v_cur : v_cur]
        [v_cur : ] 
      := 
        array_fill
        (
          v_upper_fill_value
        , array
          [
            v_array_len_y
          , v_array_len_x
          , 1
          , v_array_width - v_cur + 1
          ]
        )
      ;
    end loop;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
  
  return i_array;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   , 0
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_lower_tri_mx_ex(anyarray);
create or replace function sm_sc.fv_lower_tri_mx_ex
(
  i_array            anyarray
)
returns anyarray
as
$$
-- declare 
begin
  return sm_sc.fv_lower_tri_mx_ex(i_array, null);
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_lower_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_lpad(anyarray, anyarray, int);
create or replace function sm_sc.fv_lpad
(
  i_array            anyarray      ,
  i_fill_values      anyarray      , -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
  i_times            int     default   1  
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- set search_path to sm_sc;
  if i_array is null 
  then 
    return null;
  end if;
  
  -- if v_fill_values is null 
  -- then
  --   if array_ndims(i_array) = 1
  --   then 
  --     v_fill_values := array[nullif(i_array[1], i_array[1])];
  --   elsif array_ndims(i_array) = 2
  --   then
  --     v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  --   elsif array_ndims(i_array) = 3
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  --   elsif array_ndims(i_array) = 4
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  --   end if;
  -- end if;

  if i_times <= 0
  then
    return i_array;
  elsif array_ndims(i_array) <> array_ndims(v_fill_values)
  then 
    raise exception 'ndims of i_array: % and i_fill_values: % should be the same.', array_ndims(i_array), array_ndims(v_fill_values);
  elsif array_ndims(i_array) = 1
  then 
    return
      (
        select 
          sm_sc.fa_array_concat(v_fill_values)
        from generate_series(1, i_times)
      )
      || i_array
    ;
  elsif array_ndims(i_array) = 2
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
    then 
      raise exception 'height of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        (
          select 
            sm_sc.fa_mx_concat_x(v_fill_values)
          from generate_series(1, i_times)
        )
        |||| i_array
      ;
    end if;
  elsif array_ndims(i_array) = 3
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1)) 
      or array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        (
          select 
            sm_sc.fa_mx_concat_x3(v_fill_values)
          from generate_series(1, i_times)
        )
        |||| i_array
      ;
    end if;
  elsif array_ndims(i_array) = 4
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1)) 
      or array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
      or array_length(v_fill_values, 3) not in (1, array_length(i_array, 3))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        (
          select 
            sm_sc.fa_mx_concat_x4(v_fill_values)
          from generate_series(1, i_times)
        )
        |||| i_array
      ;
    end if;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_lpad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     array[7, 8],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7], array[8]],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[array[1.0 :: float, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7, 8], array[9, 10]] :: float[],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7, 8]],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[null :: int]],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]
--   , array[[[7, 8],[9, 0]],[[7, 8],[9, 0]],[[7, 8],[9, 0]]]
--   , 2
--   )
-- select sm_sc.fv_lpad
--   (
--     array[[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]]
--   , array[[[[7, 8],[9, 0]],[[7, 8],[9, 0]],[[7, 8],[9, 0]]]]
--   , 2
--   )

-- ------------------------------------------------------------

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_lpad(anyarray, int);
create or replace function sm_sc.fv_lpad
(
  i_array            anyarray      ,
  i_times            int    default  1
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if array_ndims(i_array) = 1
  then
    v_fill_values := array[nullif(i_array[1], i_array[1])];
  elsif array_ndims(i_array) = 2
  then 
    v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  elsif array_ndims(i_array) = 3
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1]);
  elsif array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), 1]);
  end if;

  return sm_sc.fv_lpad(i_array, v_fill_values, i_times);

end
$$
language plpgsql stable
cost 100;


-- select sm_sc.fv_lpad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[array[1.0 :: float, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     2
--   );
-- select sm_sc.fv_lpad
--   (
--     array[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]
--   , 2
--   )
-- select sm_sc.fv_lpad
--   (
--     array[[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]]
--   , 2
--   )

-- drop function if exists sm_sc.fv_ltrim(anyarray, anyelement);
create or replace function sm_sc.fv_ltrim
(
  i_array               anyarray,
  i_trim_element        anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur                 int  ;
  v_y_cur                 int  ;
  v_x3_cur                int  ;
  v_x4_cur                int  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- ([][])
  if array_ndims(i_array) =  2
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        if i_array[v_y_cur][v_x_cur] <> i_trim_element
        then 
          exit;
        else 
          i_array[v_y_cur][v_x_cur] = null;
        end if;
      end loop;
    end loop;
    return i_array;

  -- []
  elsif array_ndims(i_array) = 1
  then
    for v_x_cur in 1 .. array_length(i_array, 1)
    loop 
      if i_array[v_x_cur] <> i_trim_element
      then 
        exit;
      else 
        i_array[v_x_cur] = null;
      end if;
    end loop;
    return i_array;

  -- ([][][])
  elsif array_ndims(i_array) =  3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop 
          if i_array[v_y_cur][v_x_cur][v_x3_cur] <> i_trim_element
          then 
            exit;
          else 
            i_array[v_y_cur][v_x_cur][v_x3_cur] := null;
          end if;
        end loop;
      end loop;
    end loop;
    return i_array;

  -- ([][][][])
  elsif array_ndims(i_array) =  4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop 
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop 
            if i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> i_trim_element
            then 
              exit;
            else 
              i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := null;
            end if;
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_ltrim(array[[0,1,2,3,4,5], [0,0,1,2,0,4], [0,0,0,1,2,3], [0,0,0,0,1,2], [0,0,1,0,0,3]], 0)
-- select sm_sc.fv_ltrim(array[0,1,2,3,4,5], 0)
-- drop function if exists sm_sc.fv_mx_ascend_dim(anyarray, int);
create or replace function sm_sc.fv_mx_ascend_dim
(
  i_array        anyarray  ,
  i_ascend_time  int       default   1
)
returns anyarray
as
$$
-- declare
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) + i_ascend_time > 4
    then 
      raise exception 'unsupport dim > 4.';
    end if;
  end if;
  
  if i_ascend_time = 1
  then 
    return array[i_array];
  elsif i_ascend_time = 2
  then 
    return array[[i_array]];
  elsif i_ascend_time = 3
  then 
    return array[[[i_array]]];
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_ascend_dim(array[1, 2])
-- select sm_sc.fv_mx_ascend_dim(array[1, 2], 1)
-- select sm_sc.fv_mx_ascend_dim(array[1, 2], 2)
-- select sm_sc.fv_mx_ascend_dim(array[1, 2], 3)
-- select sm_sc.fv_mx_ascend_dim(array[[1, 2]])
-- select sm_sc.fv_mx_ascend_dim(array[[1, 2]], 1)
-- select sm_sc.fv_mx_ascend_dim(array[[1, 2]], 2)
-- -- select sm_sc.fv_mx_ascend_dim(array[[1, 2]], 3)
-- select sm_sc.fv_mx_ascend_dim(array[[[1, 2]]])
-- select sm_sc.fv_mx_ascend_dim(array[[[1, 2]]], 1)
-- --select sm_sc.fv_mx_ascend_dim(array[[[1, 2]]], 2)
-- --select sm_sc.fv_mx_ascend_dim(array[[[1, 2]]], 3)
-- drop function if exists sm_sc.fv_mx_descend_dim(anyarray, int);
create or replace function sm_sc.fv_mx_descend_dim
(
  i_array         anyarray  ,
  i_descend_time  int       default   1
)
returns anyarray
as
$$
declare
  v_ret            i_array%type   
    := array_fill
       (
         nullif(i_array[1], i_array[1])
       , (select array_agg(array_length(i_array, a_no) order by a_no) from generate_series(i_descend_time + 1, array_ndims(i_array)) tb_a(a_no))
       )
  ;
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) < i_descend_time + 1
    then 
      raise exception 'unsupport i_descend_time > ndims of i_array - 1.';
    elsif exists (select  from generate_series(2, i_descend_time) tb_a(a_no) where array_length(i_array, a_no) > 1)
    then 
      raise exception 'unsupport length of descend dim > 1';
    end if;
  end if;
  
  if i_descend_time = 0
  then 
    v_ret[:] := i_array;
  elsif i_descend_time = 1
  then 
    v_ret[:] := i_array[1:1];
  elsif i_descend_time = 2
  then 
    v_ret[:] := i_array[1:1][1:1];
  elsif i_descend_time = 3
  then 
    v_ret[:] := i_array[1:1][1:1][1:1];
  elsif i_descend_time = 4
  then 
    v_ret[:] := i_array[1:1][1:1][1:1][1:1];
  elsif i_descend_time = 5
  then 
    v_ret[:] := i_array[1:1][1:1][1:1][1:1][1:1];
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_descend_dim(array[[1, 2]])
-- select sm_sc.fv_mx_descend_dim(array[[1, 2]], 1)
-- select sm_sc.fv_mx_descend_dim(array[[[1, 2]]], 2)
-- select sm_sc.fv_mx_descend_dim(array[[[[1, 2]]]], 3)
-- select sm_sc.fv_mx_descend_dim(array[[[1, 2]]])
-- select sm_sc.fv_mx_descend_dim(array[[[1, 2]]], 1)
-- select sm_sc.fv_mx_descend_dim(array[[[1, 2]]], 2)
-- drop function if exists sm_sc.fv_mx_descend_dim_py(float[], int);
create or replace function sm_sc.fv_mx_descend_dim_py
(
  i_array         float[]  ,
  i_descend_time  int       default   1
)
returns float[]
as
$$
  import numpy as np
  
  if i_descend_time == 1 :
    return np.array(i_array)[0].tolist()
  elif i_descend_time == 2 :
    return np.array(i_array)[0,0].tolist()
  elif i_descend_time == 3 :
    return np.array(i_array)[0,0,0].tolist()
  elif i_descend_time == 4 :
    return np.array(i_array)[0,0,0,0].tolist()
  elif i_descend_time == 5 :
    return np.array(i_array)[0,0,0,0,0].tolist()
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_descend_dim_py(array[[1, 2]])
-- select sm_sc.fv_mx_descend_dim_py(array[[1, 2]], 1)
-- select sm_sc.fv_mx_descend_dim_py(array[[[1, 2]]], 2)
-- select sm_sc.fv_mx_descend_dim_py(array[[[[1, 2]]]], 3)
-- select sm_sc.fv_mx_descend_dim_py(array[[[1, 2]]])
-- select sm_sc.fv_mx_descend_dim_py(array[[[1, 2]]], 1)
-- select sm_sc.fv_mx_descend_dim_py(array[[[1, 2]]], 2)
-- drop function if exists sm_sc.fv_mx_ele_1d_2_2d(anyarray, int);
create or replace function sm_sc.fv_mx_ele_1d_2_2d
(
  i_ele_1d           anyarray     ,
  i_cnt_per_grp      int  -- 对于单行多列，将向下堆砌多行；对于多行单列，将向右堆砌多列
)
returns anyarray
as
$$
-- declare

begin
  if i_ele_1d is null 
  then 
    return null;
  elsif i_cnt_per_grp > 1
    and coalesce(nullif(array_length(i_ele_1d, 1), 1), array_length(i_ele_1d, 2)) % i_cnt_per_grp > 0
  then 
    raise exception 'unsupported count of groups. len_ele_1d_y: %; len_ele_1d_x: %; cnt_per_grp: %; i_ele_1d: %;', array_length(i_ele_1d, 1), array_length(i_ele_1d, 2), i_cnt_per_grp, i_ele_1d;
  elsif array_ndims(i_ele_1d) = 1
  then 
    return 
    (
      select 
        array_agg(i_ele_1d[a_grp_no * i_cnt_per_grp + 1 : (a_grp_no + 1) * i_cnt_per_grp])
      from generate_series(0, (array_length(i_ele_1d, 1) - 1) / i_cnt_per_grp) tb_a_grp(a_grp_no)
    );
  elsif array_ndims(i_ele_1d) = 2 and array_length(i_ele_1d, 1) = 1
  then 
    return 
    (
      select 
        sm_sc.fa_mx_concat_y(i_ele_1d[1 : 1][a_grp_no * i_cnt_per_grp + 1 : (a_grp_no + 1) * i_cnt_per_grp])
      from generate_series(0, (array_length(i_ele_1d, 2) - 1) / i_cnt_per_grp) tb_a_grp(a_grp_no)
    )
    ;
  elsif array_ndims(i_ele_1d) = 2 and array_length(i_ele_1d, 2) = 1
  then 
    return
    (
      select 
        sm_sc.fa_mx_concat_x(i_ele_1d[a_grp_no * i_cnt_per_grp + 1 : (a_grp_no + 1) * i_cnt_per_grp][1 : 1])
      from generate_series(0, (array_length(i_ele_1d, 1) - 2) / i_cnt_per_grp) tb_a_grp(a_grp_no)
    )    
    ;
  else 
    return null;
    -- raise exception 'unsupported ndims or length.';
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_ele_1d_2_2d
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , 3
--   );
-- select sm_sc.fv_mx_ele_1d_2_2d
--   (
--     array[array[1, 2, 3, 4, 5, 6, 7, 8, 9]]
--     , 3
--   );
-- select sm_sc.fv_mx_ele_1d_2_2d
--   (
--     array[array[1], array[2], array[3], array[4], array[5], array[6]]
--     , 3
--   );
-- select sm_sc.fv_mx_ele_1d_2_2d
--   (
--     array[array[1], array[2], array[3], array[4], array[5]]
--     , 2
--   );
-- drop function if exists sm_sc.fv_mx_ele_1d_2_2d_py(float[], int);
create or replace function sm_sc.fv_mx_ele_1d_2_2d_py
(
  i_ele_1d                    float[]
, i_cnt_per_grp               int                          -- 对于单行多列，将向下堆砌多行；对于多行单列，将向右堆砌多列
, i_dim_new                   int        default 1         -- 新生维度
, i_if_dim_pin_ele_on_from    boolean    default true      -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns float[]
as
$$
-- declare

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_ele_1d) not in (1, 2) 
      or cardinality(i_ele_1d) not in (array_length(i_ele_1d, 1), array_length(i_ele_1d, 2))
    then 
      raise exception 'ndims should be 1.';
    elsif i_cnt_per_grp > 1
      and cardinality(i_ele_1d) % i_cnt_per_grp > 0
    then 
      raise exception 'unsupported count of groups. len_ele_1d_y: %; len_ele_1d_x: %; cnt_per_grp: %; i_ele_1d: %;', array_length(i_ele_1d, 1), array_length(i_ele_1d, 2), i_cnt_per_grp, i_ele_1d;
    end if;
  end if;

  if i_ele_1d is null 
  then 
    return null;
  elsif i_dim_new = 1 and i_if_dim_pin_ele_on_from is true
    or i_dim_new = 2 and i_if_dim_pin_ele_on_from is false
  then 
    return 
      sm_sc.fv_opr_reshape_py(i_ele_1d, array[cardinality(i_ele_1d) / i_cnt_per_grp, i_cnt_per_grp])
    ;
  elsif i_dim_new = 1 and i_if_dim_pin_ele_on_from is false
    or i_dim_new = 2 and i_if_dim_pin_ele_on_from is true
  then 
    return 
      |^~| sm_sc.fv_opr_reshape_py(i_ele_1d, array[i_cnt_per_grp, cardinality(i_ele_1d) / i_cnt_per_grp])
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_ele_1d_2_2d_py
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , 3
--   );
-- -- select sm_sc.fv_mx_ele_1d_2_2d_py
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6, 7, 8, 9]]
-- --     , 3
-- --   );
-- -- select sm_sc.fv_mx_ele_1d_2_2d_py
-- --   (
-- --     array[array[1], array[2], array[3], array[4], array[5], array[6]]
-- --     , 3
-- --   );
-- -- -- -- select sm_sc.fv_mx_ele_1d_2_2d_py
-- -- -- --   (
-- -- -- --     array[array[1], array[2], array[3], array[4], array[5]]
-- -- -- --     , 2
-- -- -- --   );
-- drop function if exists sm_sc.fv_mx_ele_2d_2_1d(anyarray, int);
create or replace function sm_sc.fv_mx_ele_2d_2_1d
(
  i_ele_2d        anyarray  ,
  i_dim_pin_ele   int    default  2    -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 1 或 2，不能为其他值。
)
returns anyarray
as
$$
declare
  v_ret   i_ele_2d%type   ;
  v_cur   int             ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_ele_2d) <> 2
    then 
      raise exception 'no method for such i_ele_2d ndims!';
    end if;
  end if;

  if i_ele_2d is null 
  then 
    return null;
  elsif i_dim_pin_ele = 2
  then
    return 
      array(select unnest(i_ele_2d))
    ;
  else 
    v_ret := array_fill(nullif(i_ele_2d[1][1], i_ele_2d[1][1]), array[array_length(i_ele_2d, 1) * array_length(i_ele_2d, 2)]);
    for v_cur in 1 .. array_length(i_ele_2d, 2)
    loop 
      v_ret[array_length(i_ele_2d, 1) * (v_cur - 1) + 1 : array_length(i_ele_2d, 1) * v_cur] := i_ele_2d[ : ][v_cur : v_cur];
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_ele_2d_2_1d
--   (
--     array[array[1, 2, 3, 4]]
--   );
-- select sm_sc.fv_mx_ele_2d_2_1d
--   (
--     array[array[1], array[2], array[3], array[4]]
--   );
-- select sm_sc.fv_mx_ele_2d_2_1d
--   (
--     array[[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11, 12]]
--     , 1
--   );
-- drop function if exists sm_sc.fv_mx_ele_2d_2_1d_py(float[], int);
create or replace function sm_sc.fv_mx_ele_2d_2_1d_py
(
  i_ele_2d        float[]  ,
  i_dim_pin_ele   int    default  2    -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 1 或 2，不能为其他值。
)
returns float[]
as
$$
-- declare
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_ele_2d) <> 2
    then 
      raise exception 'no method for such i_ele_2d ndims!';
    end if;
  end if;

  if i_ele_2d is null 
  then 
    return null;
  elsif i_dim_pin_ele = 2
  then
    return sm_sc.fv_opr_reshape_py(i_ele_2d, array[cardinality(i_ele_2d)]);

  else 
    return sm_sc.fv_opr_reshape_py(|^~| i_ele_2d, array[cardinality(i_ele_2d)]);
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_mx_ele_2d_2_1d_py
--   (  
--     array[array[1, 2, 3, 4]]
--   );
-- select sm_sc.fv_mx_ele_2d_2_1d_py
--   (
--     array[array[1], array[2], array[3], array[4]]
--   );
-- select sm_sc.fv_mx_ele_2d_2_1d_py
--   (
--     array[[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11, 12]]
--     , 1
--   );
-- drop function if exists sm_sc.fv_mx_ele_2d_2_3d(anyarray, int, int, int, boolean);
create or replace function sm_sc.fv_mx_ele_2d_2_3d
(
  i_array_2d               anyarray      
, i_cnt_per_grp            int                  -- 每个切分分组元素个数
, i_dim_from               int                  -- 被拆分维度
, i_dim_new                int                  -- 新生维度
, i_if_dim_pin_ele_on_from boolean              -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns anyarray
as
$$
declare
  v_ret    i_array_2d%type;
  v_cur_y  int     ;
  v_cur_x  int     ;
  v_cur_x3 int     ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_2d) <> 2
    then 
      raise exception 'ndims should be 2.';
    end if;
    
    if array_length(i_array_2d, i_dim_from) % i_cnt_per_grp > 0
      or i_cnt_per_grp not between 1 and array_length(i_array_2d, i_dim_from)
    then 
      raise exception 'unperfect such i_cnt_per_grp.';
    end if;
    
    if i_dim_new not between 1 and 3
    then 
      raise exception 'unsupport such i_dim_new.';
    end if;
  end if;
 
  if i_array_2d is null 
  then 
    return null;
  elsif i_dim_from = 1
  then 
    if i_dim_new in (1, 2)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_2d[0], array[i_cnt_per_grp, array_length(i_array_2d, 1) / i_cnt_per_grp, array_length(i_array_2d, 2)]);
        for v_cur_x3 in 1 .. array_length(i_array_2d, 2)
        loop 
          for v_cur_x in 1 .. array_length(i_array_2d, 1) / i_cnt_per_grp
          loop 
            v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
              := i_array_2d[(v_cur_x - 1) * i_cnt_per_grp + 1 : v_cur_x * i_cnt_per_grp][v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_2d[0], array[array_length(i_array_2d, 1) / i_cnt_per_grp, i_cnt_per_grp, array_length(i_array_2d, 2)]);
        for v_cur_x3 in 1 .. array_length(i_array_2d, 2)
        loop 
          for v_cur_y in 1 .. array_length(i_array_2d, 1) / i_cnt_per_grp
          loop 
            v_ret[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3]
              := i_array_2d[(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp][v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dim_new = 3
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_2d[0], array[i_cnt_per_grp, array_length(i_array_2d, 2), array_length(i_array_2d, 1) / i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_2d, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_2d, 1) / i_cnt_per_grp
          loop 
            v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
              := i_array_2d[(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp][v_cur_x : v_cur_x]
            ;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_2d[0], array[array_length(i_array_2d, 1) / i_cnt_per_grp, array_length(i_array_2d, 2), i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_2d, 2)
        loop 
          for v_cur_y in 1 .. array_length(i_array_2d, 1) / i_cnt_per_grp
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ]
              := i_array_2d[(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp][v_cur_x : v_cur_x]
            ;
          end loop;
        end loop;
      end if;
    end if;
    return v_ret;
  elsif i_dim_from = 2
  then 
    if i_dim_new = 1
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_2d[0], array[array_length(i_array_2d, 2) / i_cnt_per_grp, array_length(i_array_2d, 1), i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_2d, 1)
        loop 
          for v_cur_y in 1 .. array_length(i_array_2d, 2) / i_cnt_per_grp
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ]
              := i_array_2d[v_cur_x : v_cur_x][(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp]
            ;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_2d[0], array[i_cnt_per_grp, array_length(i_array_2d, 1), array_length(i_array_2d, 2) / i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_2d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_2d, 2) / i_cnt_per_grp
          loop 
            v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
              := i_array_2d[v_cur_x : v_cur_x][(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dim_new in (2, 3)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_2d[0], array[array_length(i_array_2d, 1), i_cnt_per_grp, array_length(i_array_2d, 2) / i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_2d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_2d, 2) / i_cnt_per_grp
          loop 
            v_ret[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3]
              := i_array_2d[v_cur_y : v_cur_y][(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp]
            ;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_2d[0], array[array_length(i_array_2d, 1), array_length(i_array_2d, 2) / i_cnt_per_grp, i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_2d, 1)
        loop 
          for v_cur_x2 in 1 .. array_length(i_array_2d, 2) / i_cnt_per_grp
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x2 : v_cur_x2][ : ]
              := i_array_2d[v_cur_y : v_cur_y][(v_cur_x2 - 1) * i_cnt_per_grp + 1 : v_cur_x2 * i_cnt_per_grp]
            ;
          end loop;
        end loop;
      end if;
    end if;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;


-- select 
--   sm_sc.fv_mx_ele_2d_2_3d
--   (
--     array
--     [[1,2,3,4,5,6],[11,12,13,14,15,16],[21,22,23,24,25,26],[31,32,33,34,35,36]]
--   , 2
--   , 2   -- a_dim_from
--   , 1   -- a_dim_new
--   , true   -- a_dim_pin_ele_on_from
--   )
-- from 
--   generate_series(1, 2) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 3) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)

-- select 
--   a_dim_from
-- , a_dim_new
-- , a_dim_pin_ele_on_from
-- , array_dims
--   (
--     sm_sc.fv_mx_ele_2d_2_3d
--     (
--       sm_sc.fv_new_rand(array[6,10])
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
--   )
-- from 
--   generate_series(1, 2) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 3) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
-- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from
-- drop function if exists sm_sc.fv_mx_ele_2d_2_3d_py(float[], int, int, int, boolean);
create or replace function sm_sc.fv_mx_ele_2d_2_3d_py
(
  i_array_2d               float[]      
, i_cnt_per_grp            int                  -- 每个切分分组元素个数
, i_dim_from               int                  -- 被拆分维度
, i_dim_new                int                  -- 新生维度
, i_if_dim_pin_ele_on_from boolean              -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns float[]
as
$$
-- declare
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_2d) <> 2
    then 
      raise exception 'ndims should be 2.';
    end if;
    
    if array_length(i_array_2d, i_dim_from) % i_cnt_per_grp > 0
      or i_cnt_per_grp not between 1 and array_length(i_array_2d, i_dim_from)
    then 
      raise exception 'unperfect such i_cnt_per_grp.';
    end if;
    
    if i_dim_new not between 1 and 3
    then 
      raise exception 'unsupport such i_dim_new.';
    end if;
  end if;
 
  if i_array_2d is null 
  then 
    return null;
  elsif i_dim_from = 1
  then 
    if i_dim_new in (1, 2)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from      -- i_dim_new = 2
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from -- i_dim_new = 1
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1) / i_cnt_per_grp, i_cnt_per_grp, array_length(i_array_2d, 2)]
          )
          |^~| array[1, 2]
        ;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1) / i_cnt_per_grp, i_cnt_per_grp, array_length(i_array_2d, 2)]
          )
        ;
      end if;
    elsif i_dim_new = 3
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d |^~| array[1, 2]
          , array[array_length(i_array_2d, 2), array_length(i_array_2d, 1) / i_cnt_per_grp, i_cnt_per_grp]
          )
          |^~| array[3, 2] |^~| array[2, 1]
        ;
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1) / i_cnt_per_grp, i_cnt_per_grp, array_length(i_array_2d, 2)]
          )
          |^~| array[3, 2]
        ;
      end if;
    end if;
  elsif i_dim_from = 2
  then 
    if i_dim_new = 1
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1), array_length(i_array_2d, 2) / i_cnt_per_grp, i_cnt_per_grp]
          )
          |^~| array[1, 2]
        ;
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1), array_length(i_array_2d, 2) / i_cnt_per_grp, i_cnt_per_grp]
          )
          |^~| array[2, 3] |^~| array[1, 2]
        ;
      end if;
    elsif i_dim_new in (2, 3)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from      -- i_dim_new = 3
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from -- i_dim_new = 2
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1), array_length(i_array_2d, 2) / i_cnt_per_grp, i_cnt_per_grp]
          )
          |^~| array[2, 3]
        ;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from -- i_dim_new = 3
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from       -- i_dim_new = 2
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_2d
          , array[array_length(i_array_2d, 1), array_length(i_array_2d, 2) / i_cnt_per_grp, i_cnt_per_grp]
          )
        ;
      end if;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;


-- select 
--   sm_sc.fv_mx_ele_2d_2_3d_py
--   (
--     array
--     [[1,2,3,4,5,6],[11,12,13,14,15,16],[21,22,23,24,25,26],[31,32,33,34,35,36]]
--   , 2
--   , 2   -- a_dim_from
--   , 1   -- a_dim_new
--   , true   -- a_dim_pin_ele_on_from
--   )
-- from 
--   generate_series(1, 2) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 3) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)

-- select 
--   a_dim_from
-- , a_dim_new
-- , a_dim_pin_ele_on_from
-- , -- array_dims(
--     sm_sc.fv_mx_ele_2d_2_3d_py
--     (
--       a_arr
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
--   -- )
-- = -- array_dims(
--     sm_sc.fv_mx_ele_2d_2_3d
--     (
--       a_arr
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
--   -- )
-- , sm_sc.fv_mx_ele_2d_2_3d_py
--     (
--       a_arr
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
-- , sm_sc.fv_mx_ele_2d_2_3d
--     (
--       a_arr
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
-- from 
--   (select 
--       array 
--       [[ 0, 1, 2, 3, 4, 5]
--       ,[ 6, 7, 8, 9,10,11]
--       ,[12,13,14,15,16,17]
--       ,[18,19,20,21,22,23]
--       ,[24,25,26,27,28,29]
--       ,[30,31,32,33,34,35]] :: float[] as a_arr
--   ) tb_a_arr
-- , generate_series(1, 2) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 3) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
-- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from

-- drop function if exists sm_sc.fv_mx_ele_3d_2_2d(anyarray, int[2], int);
create or replace function sm_sc.fv_mx_ele_3d_2_2d
(
  i_array_3d        anyarray,  
  i_dims_from_to    int[2]  ,  -- 合并维度的原来两个维度。合并后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
                               -- 枚举项包括，[1, 2] === [2, 1]; [2, 3] === [3, 2]; [1, 3]; [3, 1];
  i_dim_pin_ele     int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
)
returns anyarray
as
$$
declare
  v_ret     i_array_3d%type;
  v_cur_y   int            ;
  v_cur_x   int            ;
  v_cur_x3  int            ;

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    end if;
    
    if i_dim_pin_ele <> all(i_dims_from_to)
    then 
      raise exception 'the pin_ele dim must be one of from_to dims.';
    end if;
    
    if i_dims_from_to[1] not between 1 and 3 
      or i_dims_from_to[2] not between 1 and 3
      or i_dims_from_to[1] = i_dims_from_to[2]
    then 
      raise exception 'unsupport this i_dims_from_to.';
    end if;
  end if;

  if i_array_3d is null 
  then 
    return null;
  end if;  
  
  if i_dims_from_to in (array[1, 2], array[2, 1])
  then
    v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1) * array_length(i_array_3d, 2), array_length(i_array_3d, 3)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
      loop 
        for v_cur_x in 1 .. array_length(i_array_3d, 2)
        loop 
          v_ret[(v_cur_x - 1) * array_length(i_array_3d, 1) + 1 : v_cur_x * array_length(i_array_3d, 1)][v_cur_x3 : v_cur_x3]
            := i_array_3d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
          ;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 2
    then
      for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
      loop 
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          v_ret[(v_cur_y - 1) * array_length(i_array_3d, 2) + 1 : v_cur_y * array_length(i_array_3d, 2)][v_cur_x3 : v_cur_x3]
            := i_array_3d[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3]
          ;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[1, 3]
  then
    v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 2), array_length(i_array_3d, 1) * array_length(i_array_3d, 3)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x in 1 .. array_length(i_array_3d, 2)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
        loop 
          v_ret[v_cur_x : v_cur_x][(v_cur_x3 - 1) * array_length(i_array_3d, 1) + 1 : v_cur_x3 * array_length(i_array_3d, 1)] 
            := i_array_3d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
          ;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 3
    then      
      for v_cur_x in 1 .. array_length(i_array_3d, 2)
      loop 
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          v_ret[v_cur_x : v_cur_x][(v_cur_y - 1) * array_length(i_array_3d, 3) + 1 : v_cur_y * array_length(i_array_3d, 3)]
            := i_array_3d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ]
          ;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[3, 1]
  then
    v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1) * array_length(i_array_3d, 3), array_length(i_array_3d, 2)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x in 1 .. array_length(i_array_3d, 2)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
        loop 
          v_ret[(v_cur_x3 - 1) * array_length(i_array_3d, 1) + 1 : v_cur_x3 * array_length(i_array_3d, 1)][v_cur_x : v_cur_x]
            := i_array_3d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
          ;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 3
    then      
      for v_cur_x in 1 .. array_length(i_array_3d, 2)
      loop 
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          v_ret[(v_cur_y - 1) * array_length(i_array_3d, 3) + 1 : v_cur_y * array_length(i_array_3d, 3)][v_cur_x : v_cur_x]
            := i_array_3d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ]
          ;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to in (array[2, 3], array[3, 2])
  then
    v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), array_length(i_array_3d, 2) * array_length(i_array_3d, 3)]);
    if i_dim_pin_ele = 2
    then 
      for v_cur_y in 1 .. array_length(i_array_3d, 1)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
        loop 
          v_ret[v_cur_y : v_cur_y][(v_cur_x3 - 1) * array_length(i_array_3d, 2) + 1 : v_cur_x3 * array_length(i_array_3d, 2)] 
            := i_array_3d[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3]
          ;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 3
    then
      for v_cur_y in 1 .. array_length(i_array_3d, 1)
      loop 
        for v_cur_x in 1 .. array_length(i_array_3d, 2)
        loop 
          v_ret[v_cur_y : v_cur_y][(v_cur_x - 1) * array_length(i_array_3d, 3) + 1 : v_cur_x * array_length(i_array_3d, 3)]
            := i_array_3d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ]
          ;
        end loop;
      end loop;
    end if;
  end if;
  return v_ret; 
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- with cte_arr as
-- (
--   select 
--     array
--     [
--       [
--         [1, 2, 3, 4]
--       , [5, 6, 7, 8]
--       , [9, 10, 11, 12]
--       ]
--     , [
--         [21, 22, 23, 24]
--       , [25, 26, 27, 28]
--       , [29, 30, 31, 32]
--       ]
--     ]
--     as a_arr
-- )
-- select 
--   a_dims_from_to, a_dim_pin_ele,
--   sm_sc.fv_mx_ele_3d_2_2d(a_arr, a_dims_from_to, a_dim_pin_ele) as a_out
-- from cte_arr
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[1, 3] 
--       union all select array[3, 1]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 3) tb_a_dim_pin_ele(a_dim_pin_ele)
-- where a_dim_pin_ele = any(a_dims_from_to)
-- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dims_from_to[1]
--   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dim_pin_ele
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i http://pypi.douban.com/simple/ --trusted-host pypi.douban.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_mx_ele_3d_2_2d_py(float[], int[], int);
create or replace function sm_sc.fv_mx_ele_3d_2_2d_py
(
  i_array_3d        float[]  ,
  i_dims_from_to    int[2]  ,  -- 合并维度的原来两个维度。合并后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
                               -- 枚举项包括，[1, 2] === [2, 1]; [2, 3] === [3, 2]; [1, 3]; [3, 1];
  i_dim_pin_ele     int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
)
returns float[]
as
$$
-- declare
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    end if;
    
    if i_dim_pin_ele <> all(i_dims_from_to)
    then 
      raise exception 'the pin_ele dim must be one of from_to dims.';
    end if;
    
    if i_dims_from_to[1] not between 1 and 3 
      or i_dims_from_to[2] not between 1 and 3
      or i_dims_from_to[1] = i_dims_from_to[2]
    then 
      raise exception 'unsupport this i_dims_from_to.';
    end if;
  end if;

  if i_array_3d is null 
  then 
    return null;
  end if;  
  
  if i_dims_from_to in (array[1, 2], array[2, 1])
  then
    if i_dim_pin_ele = 1
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d |^~| array[1, 2]
        , array
          [
            array_length(i_array_3d, 1) * array_length(i_array_3d, 2)
          , array_length(i_array_3d, 3)
          ]
        )
      ;
      
    elsif i_dim_pin_ele = 2
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d
        , array
          [
            array_length(i_array_3d, 1) * array_length(i_array_3d, 2)
          , array_length(i_array_3d, 3)
          ]
          )
      ;
      
    end if;
  elsif i_dims_from_to = array[1, 3]
  then
    if i_dim_pin_ele = 1
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d |^~| array[1, 2] |^~| array[2, 3]
        , array
          [
            array_length(i_array_3d, 2)
          , array_length(i_array_3d, 1) * array_length(i_array_3d, 3)
          ]
        )
      ;      
      
    elsif i_dim_pin_ele = 3
    then     
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d |^~| array[1, 2]
        , array
          [
            array_length(i_array_3d, 2)
          , array_length(i_array_3d, 1) * array_length(i_array_3d, 3)
          ]
        )
      ;       
      
      
    end if;
  elsif i_dims_from_to = array[3, 1]
  then
    if i_dim_pin_ele = 1
    then     
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d |^~| array[3, 2] |^~| array[2, 1]
        , array
          [
            array_length(i_array_3d, 1) * array_length(i_array_3d, 3)
          , array_length(i_array_3d, 2)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 3
    then      
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d |^~| array[3, 2]
        , array
          [
            array_length(i_array_3d, 1) * array_length(i_array_3d, 3)
          , array_length(i_array_3d, 2)
          ]
        )
      ;   
      
    end if;
  elsif i_dims_from_to in (array[2, 3], array[3, 2])
  then
    if i_dim_pin_ele = 2
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d |^~| array[2, 3]
        , array
          [
            array_length(i_array_3d, 1)
          , array_length(i_array_3d, 2) * array_length(i_array_3d, 3)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 3
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_3d
        , array
          [
            array_length(i_array_3d, 1)
          , array_length(i_array_3d, 2) * array_length(i_array_3d, 3)
          ]
        )
      ;       
      
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
;

-- select 
--   sm_sc.fv_mx_ele_3d_2_2d_py
--   (
--     array[array[1, 2, 3], array[1, 2, 3]] :: float[],
--     array[array[1, 2], array[3, 1], array[2, 3]] :: float[]
--   )

-- with 
-- cte_arr as 
-- (
--   -- select (sm_sc.fv_new_rand(array[2,3,5]) :: decimal[] ~=` 3) :: float[] as a_arr
--   select 
--     array
--     [[[ 0, 1, 2, 3, 4]
--      ,[ 5, 6, 7, 8, 9]
--      ,[10,11,12,13,14]]
--     ,
--      [[15,16,17,18,19]
--      ,[20,21,22,23,24]
--      ,[25,26,27,28,29]]] :: float[]
--     as a_arr
-- ),
-- cte_dims as 
-- (
--   select array[1,2] as a_dims union all
--   select array[2,1]           union all
--   select array[1,3]           union all
--   select array[3,1]           union all
--   select array[2,3]           union all
--   select array[3,2]
-- ),
-- cte_pin as 
-- (
--   select 1 as a_pin union all select 2
-- )
-- select a_dims, a_pin
-- , sm_sc.fv_mx_ele_3d_2_2d_py(a_arr, a_dims, a_dims[a_pin]) = sm_sc.fv_mx_ele_3d_2_2d(a_arr, a_dims, a_dims[a_pin])
-- , sm_sc.fv_mx_ele_3d_2_2d_py(a_arr, a_dims, a_dims[a_pin])
-- , sm_sc.fv_mx_ele_3d_2_2d(a_arr, a_dims, a_dims[a_pin])
-- from cte_arr, cte_dims, cte_pin
-- order by a_dims, a_pin
-- drop function if exists sm_sc.fv_mx_ele_3d_2_4d(anyarray, int, int, int, boolean);
create or replace function sm_sc.fv_mx_ele_3d_2_4d
(
  i_array_3d               anyarray      
, i_cnt_per_grp            int                  -- 每个切分分组元素个数
, i_dim_from               int                  -- 被拆分维度
, i_dim_new                int                  -- 新生维度
, i_if_dim_pin_ele_on_from boolean              -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns anyarray
as
$$
declare
  v_ret    i_array_3d%type;
  v_cur_y  int     ;
  v_cur_x  int     ;
  v_cur_x3 int     ;
  v_cur_x4 int     ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    end if;
    
    if array_length(i_array_3d, i_dim_from) % i_cnt_per_grp > 0
      or i_cnt_per_grp not between 1 and array_length(i_array_3d, i_dim_from)
    then 
      raise exception 'unperfect such i_cnt_per_grp.';
    end if;
    
    if i_dim_new not between 1 and 4
    then 
      raise exception 'unsupport such i_dim_new.';
    end if;
  end if;
 
  if i_array_3d is null 
  then 
    return null;
  elsif i_dim_from = 1
  then 
    if i_dim_new in (1, 2)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_3d[0], array[i_cnt_per_grp, array_length(i_array_3d, 1) / i_cnt_per_grp, array_length(i_array_3d, 2), array_length(i_array_3d, 3)]);
        for v_cur_x3 in 1 .. array_length(i_array_3d, 2)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x in 1 .. array_length(i_array_3d, 1) / i_cnt_per_grp
            loop 
              v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[(v_cur_x - 1) * i_cnt_per_grp + 1 : v_cur_x * i_cnt_per_grp][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1) / i_cnt_per_grp, i_cnt_per_grp, array_length(i_array_3d, 2), array_length(i_array_3d, 3)]);
        for v_cur_x3 in 1 .. array_length(i_array_3d, 2)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_y in 1 .. array_length(i_array_3d, 1) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dim_new = 3
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[i_cnt_per_grp, array_length(i_array_3d, 2), array_length(i_array_3d, 1) / i_cnt_per_grp, array_length(i_array_3d, 3)]);
        for v_cur_x in 1 .. array_length(i_array_3d, 2)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_3d, 1) / i_cnt_per_grp
            loop 
              v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp][v_cur_x : v_cur_x][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1) / i_cnt_per_grp, array_length(i_array_3d, 2), i_cnt_per_grp, array_length(i_array_3d, 3)]);
        for v_cur_x in 1 .. array_length(i_array_3d, 2)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_y in 1 .. array_length(i_array_3d, 1) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
                := i_array_3d[(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp][v_cur_x : v_cur_x][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dim_new = 4
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[i_cnt_per_grp, array_length(i_array_3d, 2), array_length(i_array_3d, 3), array_length(i_array_3d, 1) / i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_3d, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_3d, 1) / i_cnt_per_grp
            loop 
              v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[(v_cur_x4 - 1) * i_cnt_per_grp + 1 : v_cur_x4 * i_cnt_per_grp][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1) / i_cnt_per_grp, array_length(i_array_3d, 2), array_length(i_array_3d, 3), i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_3d, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_y in 1 .. array_length(i_array_3d, 1) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
                := i_array_3d[(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    end if;
    return v_ret;
  elsif i_dim_from = 2
  then 
    if i_dim_new = 1
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 2) / i_cnt_per_grp, array_length(i_array_3d, 1), i_cnt_per_grp, array_length(i_array_3d, 3)]);
        for v_cur_x in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_y in 1 .. array_length(i_array_3d, 2) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_x : v_cur_x][(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[i_cnt_per_grp, array_length(i_array_3d, 1), array_length(i_array_3d, 2) / i_cnt_per_grp, array_length(i_array_3d, 3)]);
        for v_cur_x in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_3d, 2) / i_cnt_per_grp
            loop 
              v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_x : v_cur_x][(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dim_new in (2, 3)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), i_cnt_per_grp, array_length(i_array_3d, 2) / i_cnt_per_grp, array_length(i_array_3d, 3)]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_3d, 2) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_y : v_cur_y][(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), array_length(i_array_3d, 2) / i_cnt_per_grp, i_cnt_per_grp, array_length(i_array_3d, 3)]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x2 in 1 .. array_length(i_array_3d, 2) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x2 : v_cur_x2][ : ][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_y : v_cur_y][(v_cur_x2 - 1) * i_cnt_per_grp + 1 : v_cur_x2 * i_cnt_per_grp][v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dim_new = 4
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), i_cnt_per_grp, array_length(i_array_3d, 3), array_length(i_array_3d, 2) / i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_3d, 2) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_y : v_cur_y][(v_cur_x4 - 1) * i_cnt_per_grp + 1 : v_cur_x4 * i_cnt_per_grp][v_cur_x3 : v_cur_x3]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), array_length(i_array_3d, 2) / i_cnt_per_grp, array_length(i_array_3d, 3), i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 3)
          loop 
            for v_cur_x in 1 .. array_length(i_array_3d, 2) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
                := i_array_3d[v_cur_y : v_cur_y][(v_cur_x - 1) * i_cnt_per_grp + 1 : v_cur_x * i_cnt_per_grp][v_cur_x3 : v_cur_x3]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    end if;
    return v_ret;
  elsif i_dim_from = 3
  then 
    if i_dim_new = 1
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 3) / i_cnt_per_grp, array_length(i_array_3d, 1), array_length(i_array_3d, 2), i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 2)
          loop 
            for v_cur_y in 1 .. array_length(i_array_3d, 3) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
                := i_array_3d[v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][(v_cur_y - 1) * i_cnt_per_grp + 1 : v_cur_y * i_cnt_per_grp]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[i_cnt_per_grp, array_length(i_array_3d, 1), array_length(i_array_3d, 2), array_length(i_array_3d, 3) / i_cnt_per_grp]);
        for v_cur_x in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_3d, 3) / i_cnt_per_grp
            loop 
              v_ret[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][(v_cur_x4 - 1) * i_cnt_per_grp + 1 : v_cur_x4 * i_cnt_per_grp]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dim_new = 2
    then
      if i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), array_length(i_array_3d, 3) / i_cnt_per_grp, array_length(i_array_3d, 2), i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 2)
          loop 
            for v_cur_x in 1 .. array_length(i_array_3d, 3) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
                := i_array_3d[v_cur_y : v_cur_y][v_cur_x3 : v_cur_x3][(v_cur_x - 1) * i_cnt_per_grp + 1 : v_cur_x * i_cnt_per_grp]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), i_cnt_per_grp, array_length(i_array_3d, 2), array_length(i_array_3d, 3) / i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_3d, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_3d, 3) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_y : v_cur_y][v_cur_x3 : v_cur_x3][(v_cur_x4 - 1) * i_cnt_per_grp + 1 : v_cur_x4 * i_cnt_per_grp]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dim_new in (3, 4)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), array_length(i_array_3d, 2), i_cnt_per_grp, array_length(i_array_3d, 3) / i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_3d, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_3d, 3) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
                := i_array_3d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][(v_cur_x4 - 1) * i_cnt_per_grp + 1 : v_cur_x4 * i_cnt_per_grp]
              ;
            end loop;
          end loop;
        end loop;
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        v_ret := array_fill(i_array_3d[0], array[array_length(i_array_3d, 1), array_length(i_array_3d, 2), array_length(i_array_3d, 3) / i_cnt_per_grp, i_cnt_per_grp]);
        for v_cur_y in 1 .. array_length(i_array_3d, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_3d, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_3d, 3) / i_cnt_per_grp
            loop 
              v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
                := i_array_3d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][(v_cur_x3 - 1) * i_cnt_per_grp + 1 : v_cur_x3 * i_cnt_per_grp]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    end if;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;


-- select 
--   sm_sc.fv_mx_ele_3d_2_4d
--   (
--     array
--     [
--       [[1,2,3,4,5,6],[11,12,13,14,15,16],[21,22,23,24,25,26],[31,32,33,34,35,36]]
--     , [[41,42,43,44,45,46],[51,52,53,54,55,56],[61,62,63,64,65,66],[71,72,73,74,75,76]]
--     , [[-1,-2,-3,-4,-5,-6],[11,-12,-13,-14,-15,-16],[21,-22,-23,-24,-25,-26],[-31,-32,-33,-34,-35,-36]]
--     , [[-41,-42,-43,-44,-45,-46],[-51,-52,-53,-54,-55,-56],[-61,-62,-63,-64,-65,-66],[-71,-72,-73,-74,-75,-76]]
--     , [[10,20,30,40,50,60],[110,120,130,140,150,160],[210,220,230,240,250,260],[310,320,330,340,350,360]]
--     , [[410,420,430,440,450,460],[510,520,530,540,550,560],[610,620,630,640,650,660],[710,720,730,740,750,760]]
--     , [[-10,-20,-30,-40,-50,-60],[110,-120,-130,-140,-150,-160],[210,-220,-230,-240,-250,-260],[-310,-320,-330,-340,-350,-360]]
--     , [[-410,-420,-430,-440,-450,-460],[-510,-520,-530,-540,-550,-560],[-610,-620,-630,-640,-650,-660],[-710,-720,-730,-740,-750,-760]]
--     ]
--   , 2
--   , 3   -- a_dim_from
--   , 1   -- a_dim_new
--   , true   -- a_dim_pin_ele_on_from
--   )
-- from 
--   generate_series(1, 3) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)

-- select 
--   a_dim_from
-- , a_dim_new
-- , a_dim_pin_ele_on_from
-- , array_dims
--   (
--     sm_sc.fv_mx_ele_3d_2_4d
--     (
--       sm_sc.fv_new_rand(array[6,8,10])
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
--   )
-- from 
--   generate_series(1, 3) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
-- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from
-- drop function if exists sm_sc.fv_mx_ele_3d_2_4d_py(float[], int, int, int, boolean);
create or replace function sm_sc.fv_mx_ele_3d_2_4d_py
(
  i_array_3d               float[]      
, i_cnt_per_grp            int                  -- 每个切分分组元素个数
, i_dim_from               int                  -- 被拆分维度
, i_dim_new                int                  -- 新生维度
, i_if_dim_pin_ele_on_from boolean              -- 是否在 from 维度保留元素顺序，否则在 new 维度保留元素顺序
)
returns float[]
as
$$
-- declare
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    end if;
    
    if array_length(i_array_3d, i_dim_from) % i_cnt_per_grp > 0
      or i_cnt_per_grp not between 1 and array_length(i_array_3d, i_dim_from)
    then 
      raise exception 'unperfect such i_cnt_per_grp.';
    end if;
    
    if i_dim_new not between 1 and 4
    then 
      raise exception 'unsupport such i_dim_new.';
    end if;
  end if;
 
  if i_array_3d is null 
  then 
    return null;
  elsif i_dim_from = 1
  then 
    if i_dim_new in (1, 2)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from        -- i_dim_new = 2
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from   -- i_dim_new = 1
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3)
            ]
          )
          |^~| array[1, 2]
        ;
        
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3)
            ]
          )
        ;
        
      end if;
    elsif i_dim_new = 3
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d |^~| array[1, 2]
          , array
            [
              array_length(i_array_3d, 2)
            , array_length(i_array_3d, 1) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 3)
            ]
          )
          |^~| array[3, 2] |^~| array[2, 1]
        ;
        
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3)
            ]
          )
          |^~| array[3, 2]
        ;
        
      end if;
    elsif i_dim_new = 4
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d |^~| array[1, 2] |^~| array[2, 3]
          , array
            [
              array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3)
            , array_length(i_array_3d, 1) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[3, 4] |^~| array[2, 3] |^~| array[1, 2]
        ;
        
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d |^~| array[1, 2] |^~| array[2, 3]
          , array
            [
              array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3)
            , array_length(i_array_3d, 1) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[2, 3] |^~| array[1, 2]
        ;
        
      end if;
    end if;
  elsif i_dim_from = 2
  then 
    if i_dim_new = 1
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 3)
            ]
          )
          |^~| array[1, 2]
        ;
        
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 3)
            ]
          )
          |^~| array[2, 3] |^~| array[1, 2]
        ;
        
      end if;
    elsif i_dim_new in (2, 3)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 3)
            ]
          )
          |^~| array[2, 3]
        ;
        
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2) / i_cnt_per_grp
            , i_cnt_per_grp
            , array_length(i_array_3d, 3)
            ]
          )
        ;
        
      end if;
    elsif i_dim_new = 4
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d |^~| array[2, 3]
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 3)
            , array_length(i_array_3d, 2) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[3, 4] |^~| array[2, 3]
        ;
        
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d |^~| array[2, 3]
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 3)
            , array_length(i_array_3d, 2) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[2, 3]
        ;
        
      end if;
    end if;
  elsif i_dim_from = 3
  then 
    if i_dim_new = 1
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[3, 2] |^~| array[2, 1]
        ;
        
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d 
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[4, 3] |^~| array[3, 2] |^~| array[2, 1]
        ;
        
      end if;
    elsif i_dim_new = 2
    then
      if i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[2, 3]
        ;
        
      elsif not i_if_dim_pin_ele_on_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[3, 4] |^~| array[2, 3]
        ;
        
      end if;
    elsif i_dim_new in (3, 4)
    then
      if i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or not i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
          |^~| array[3, 4]
        ;
        
      elsif not i_if_dim_pin_ele_on_from and i_dim_new <> i_dim_from
        or i_if_dim_pin_ele_on_from and i_dim_new = i_dim_from
      then 
        return 
          sm_sc.fv_opr_reshape_py
          (
            i_array_3d
          , array
            [
              array_length(i_array_3d, 1)
            , array_length(i_array_3d, 2)
            , array_length(i_array_3d, 3) / i_cnt_per_grp
            , i_cnt_per_grp
            ]
          )
        ;
        
      end if;
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;


-- select 
--   sm_sc.fv_mx_ele_3d_2_4d_py
--   (
--     array
--     [
--       [[1,2,3,4,5,6],[11,12,13,14,15,16],[21,22,23,24,25,26],[31,32,33,34,35,36]]
--     , [[41,42,43,44,45,46],[51,52,53,54,55,56],[61,62,63,64,65,66],[71,72,73,74,75,76]]
--     , [[-1,-2,-3,-4,-5,-6],[11,-12,-13,-14,-15,-16],[21,-22,-23,-24,-25,-26],[-31,-32,-33,-34,-35,-36]]
--     , [[-41,-42,-43,-44,-45,-46],[-51,-52,-53,-54,-55,-56],[-61,-62,-63,-64,-65,-66],[-71,-72,-73,-74,-75,-76]]
--     , [[10,20,30,40,50,60],[110,120,130,140,150,160],[210,220,230,240,250,260],[310,320,330,340,350,360]]
--     , [[410,420,430,440,450,460],[510,520,530,540,550,560],[610,620,630,640,650,660],[710,720,730,740,750,760]]
--     , [[-10,-20,-30,-40,-50,-60],[110,-120,-130,-140,-150,-160],[210,-220,-230,-240,-250,-260],[-310,-320,-330,-340,-350,-360]]
--     , [[-410,-420,-430,-440,-450,-460],[-510,-520,-530,-540,-550,-560],[-610,-620,-630,-640,-650,-660],[-710,-720,-730,-740,-750,-760]]
--     ]
--   , 2
--   , 3   -- a_dim_from
--   , 1   -- a_dim_new
--   , true   -- a_dim_pin_ele_on_from
--   )
-- from 
--   generate_series(1, 3) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)

-- select 
--   a_dim_from
-- , a_dim_new
-- , a_dim_pin_ele_on_from
-- , array_dims
--   (
--     sm_sc.fv_mx_ele_3d_2_4d_py
--     (
--       sm_sc.fv_new_rand(array[6,8,10])
--     , 2
--     , a_dim_from
--     , a_dim_new
--     , a_dim_pin_ele_on_from
--     )
--   )
-- from 
--   generate_series(1, 3) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
-- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from

-- select 
--   a_dim_from
-- , a_dim_new
-- , a_dim_pin_ele_on_from
-- , sm_sc.fv_mx_ele_3d_2_4d_py
--   (
--     a_arr
--   , 2
--   , a_dim_from
--   , a_dim_new
--   , a_dim_pin_ele_on_from
--   )
-- , sm_sc.fv_mx_ele_3d_2_4d
--   (
--     a_arr
--   , 2
--   , a_dim_from
--   , a_dim_new
--   , a_dim_pin_ele_on_from
--   )
-- from 
--   (select 
--      array 
--      [[[  0,  1,  2,  3,  4,  5,  6,  7,  8,  9]
--       ,[ 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
--       ,[ 20, 21, 22, 23, 24, 25, 26, 27, 28, 29]
--       ,[ 30, 31, 32, 33, 34, 35, 36, 37, 38, 39]
--       ,[ 40, 41, 42, 43, 44, 45, 46, 47, 48, 49]
--       ,[ 50, 51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ,[ 60, 61, 62, 63, 64, 65, 66, 67, 68, 69]
--       ,[ 70, 71, 72, 73, 74, 75, 76, 77, 78, 79]]
--      ,[[ 80, 81, 82, 83, 84, 85, 86, 87, 88, 89]
--       ,[ 90, 91, 92, 93, 94, 95, 96, 97, 98, 99]
--       ,[100,101,102,103,104,105,106,107,108,109]
--       ,[110,111,112,113,114,115,116,117,118,119]
--       ,[120,121,122,123,124,125,126,127,128,129]
--       ,[130,131,132,133,134,135,136,137,138,139]
--       ,[140,141,142,143,144,145,146,147,148,149]
--       ,[150,151,152,153,154,155,156,157,158,159]]
--      ,[[160,161,162,163,164,165,166,167,168,169]
--       ,[170,171,172,173,174,175,176,177,178,179]
--       ,[180,181,182,183,184,185,186,187,188,189]
--       ,[190,191,192,193,194,195,196,197,198,199]
--       ,[200,201,202,203,204,205,206,207,208,209]
--       ,[210,211,212,213,214,215,216,217,218,219]
--       ,[220,221,222,223,224,225,226,227,228,229]
--       ,[230,231,232,233,234,235,236,237,238,239]]
--      ,[[240,241,242,243,244,245,246,247,248,249]
--       ,[250,251,252,253,254,255,256,257,258,259]
--       ,[260,261,262,263,264,265,266,267,268,269]
--       ,[270,271,272,273,274,275,276,277,278,279]
--       ,[280,281,282,283,284,285,286,287,288,289]
--       ,[290,291,292,293,294,295,296,297,298,299]
--       ,[300,301,302,303,304,305,306,307,308,309]
--       ,[310,311,312,313,314,315,316,317,318,319]]
--      ,[[320,321,322,323,324,325,326,327,328,329]
--       ,[330,331,332,333,334,335,336,337,338,339]
--       ,[340,341,342,343,344,345,346,347,348,349]
--       ,[350,351,352,353,354,355,356,357,358,359]
--       ,[360,361,362,363,364,365,366,367,368,369]
--       ,[370,371,372,373,374,375,376,377,378,379]
--       ,[380,381,382,383,384,385,386,387,388,389]
--       ,[390,391,392,393,394,395,396,397,398,399]]
--      ,[[400,401,402,403,404,405,406,407,408,409]
--       ,[410,411,412,413,414,415,416,417,418,419]
--       ,[420,421,422,423,424,425,426,427,428,429]
--       ,[430,431,432,433,434,435,436,437,438,439]
--       ,[440,441,442,443,444,445,446,447,448,449]
--       ,[450,451,452,453,454,455,456,457,458,459]
--       ,[460,461,462,463,464,465,466,467,468,469]
--       ,[470,471,472,473,474,475,476,477,478,479]]]
--      :: float[] as a_arr
--   ) tb_a_arr
-- , generate_series(1, 3) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_new(a_dim_new)
-- , (select true as a_dim_pin_ele_on_from union all select false as a_dim_pin_ele_on_from) tb_a_dim_pin_ele_on_from(a_dim_pin_ele_on_from)
-- order by a_dim_from, a_dim_new, a_dim_pin_ele_on_from
-- drop function if exists sm_sc.fv_mx_ele_4d_2_3d(anyarray, int[2], int);
create or replace function sm_sc.fv_mx_ele_4d_2_3d
(
  i_array_4d        anyarray,  
  i_dims_from_to    int[2]  ,  -- 合并维度的原来两个维度。合并后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
                               -- 枚举项包括，[1, 2] === [2, 1]; [2, 3] === [3, 2]; [3, 4] === [4, 3]; [1, 3]; [3, 1]; [1, 4]; [4, 1]; [2, 4]; [4, 2]
  i_dim_pin_ele     int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
)
returns anyarray
as
$$
declare
  v_ret     i_array_4d%type;
  v_cur_y   int            ;
  v_cur_x   int            ;
  v_cur_x3  int            ;
  v_cur_x4  int            ;

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_4d) <> 4
    then 
      raise exception 'ndims should be 4.';
    end if;
    
    if i_dim_pin_ele <> all(i_dims_from_to)
    then 
      raise exception 'the pin_ele dim must be one of from_to dims.';
    end if;
    
    if i_dims_from_to[1] not between 1 and 4 
      or i_dims_from_to[2] not between 1 and 4
      or i_dims_from_to[1] = i_dims_from_to[2]
    then 
      raise exception 'unsupport this i_dims_from_to.';
    end if;
  end if;

  if i_array_4d is null 
  then 
    return null;
  end if;  
  
  if i_dims_from_to in (array[1, 2], array[2, 1])
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1) * array_length(i_array_4d, 2), array_length(i_array_4d, 3), array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_x in 1 .. array_length(i_array_4d, 2)
          loop 
            v_ret[(v_cur_x - 1) * array_length(i_array_4d, 1) + 1 : v_cur_x * array_length(i_array_4d, 1)][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
              := i_array_4d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 2
    then
      for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_y in 1 .. array_length(i_array_4d, 1)
          loop 
            v_ret[(v_cur_y - 1) * array_length(i_array_4d, 2) + 1 : v_cur_y * array_length(i_array_4d, 2)][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
              := i_array_4d[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[1, 3]
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 2), array_length(i_array_4d, 1) * array_length(i_array_4d, 3), array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
          loop 
            v_ret[v_cur_x : v_cur_x][(v_cur_x3 - 1) * array_length(i_array_4d, 1) + 1 : v_cur_x3 * array_length(i_array_4d, 1)][v_cur_x4 : v_cur_x4] 
              := i_array_4d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 3
    then      
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_y in 1 .. array_length(i_array_4d, 1)
          loop 
            v_ret[v_cur_x : v_cur_x][(v_cur_y - 1) * array_length(i_array_4d, 3) + 1 : v_cur_y * array_length(i_array_4d, 3)][v_cur_x4 : v_cur_x4]
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[3, 1]
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1) * array_length(i_array_4d, 3), array_length(i_array_4d, 2), array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
          loop 
            v_ret[(v_cur_x3 - 1) * array_length(i_array_4d, 1) + 1 : v_cur_x3 * array_length(i_array_4d, 1)][v_cur_x : v_cur_x][v_cur_x4 : v_cur_x4]
              := i_array_4d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 3
    then      
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_y in 1 .. array_length(i_array_4d, 1)
          loop 
            v_ret[(v_cur_y - 1) * array_length(i_array_4d, 3) + 1 : v_cur_y * array_length(i_array_4d, 3)][v_cur_x : v_cur_x][v_cur_x4 : v_cur_x4]
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[1, 4]
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 2), array_length(i_array_4d, 3), array_length(i_array_4d, 1) * array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
          loop 
            v_ret[v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][(v_cur_x4 - 1) * array_length(i_array_4d, 1) + 1 : v_cur_x4 * array_length(i_array_4d, 1)] 
              := i_array_4d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 4
    then
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_y in 1 .. array_length(i_array_4d, 1)
          loop 
            v_ret[v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][(v_cur_y - 1) * array_length(i_array_4d, 4) + 1 : v_cur_y * array_length(i_array_4d, 4)] 
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[4, 1]
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1) * array_length(i_array_4d, 4), array_length(i_array_4d, 2), array_length(i_array_4d, 3)]);
    if i_dim_pin_ele = 1
    then 
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
          loop 
            v_ret[(v_cur_x4 - 1) * array_length(i_array_4d, 1) + 1 : v_cur_x4 * array_length(i_array_4d, 1)][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3] 
              := i_array_4d[ : ][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 4
    then
      for v_cur_x in 1 .. array_length(i_array_4d, 2)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_y in 1 .. array_length(i_array_4d, 1)
          loop 
            v_ret[(v_cur_y - 1) * array_length(i_array_4d, 4) + 1 : v_cur_y * array_length(i_array_4d, 4)][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3] 
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to in (array[2, 3], array[3, 2])
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1), array_length(i_array_4d, 2) * array_length(i_array_4d, 3), array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 2
    then 
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
          loop 
            v_ret[v_cur_y : v_cur_y][(v_cur_x3 - 1) * array_length(i_array_4d, 2) + 1 : v_cur_x3 * array_length(i_array_4d, 2)][v_cur_x4 : v_cur_x4] 
              := i_array_4d[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 3
    then
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
        loop 
          for v_cur_x in 1 .. array_length(i_array_4d, 2)
          loop 
            v_ret[v_cur_y : v_cur_y][(v_cur_x - 1) * array_length(i_array_4d, 3) + 1 : v_cur_x * array_length(i_array_4d, 3)][v_cur_x4 : v_cur_x4] 
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[2, 4]
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1), array_length(i_array_4d, 3), array_length(i_array_4d, 2) * array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 2
    then 
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x3 : v_cur_x3][(v_cur_x4 - 1) * array_length(i_array_4d, 2) + 1 : v_cur_x4 * array_length(i_array_4d, 2)] 
              := i_array_4d[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 4
    then      
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_x in 1 .. array_length(i_array_4d, 2)
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x3 : v_cur_x3][(v_cur_x - 1) * array_length(i_array_4d, 4) + 1 : v_cur_x * array_length(i_array_4d, 4)] 
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to = array[4, 2]
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1), array_length(i_array_4d, 2) * array_length(i_array_4d, 4), array_length(i_array_4d, 3)]);
    if i_dim_pin_ele = 2
    then 
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
          loop 
            v_ret[v_cur_y : v_cur_y][(v_cur_x4 - 1) * array_length(i_array_4d, 2) + 1 : v_cur_x4 * array_length(i_array_4d, 2)][v_cur_x3 : v_cur_x3] 
              := i_array_4d[v_cur_y : v_cur_y][ : ][v_cur_x3 : v_cur_x3][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
    elsif i_dim_pin_ele = 4
    then      
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
        loop 
          for v_cur_x in 1 .. array_length(i_array_4d, 2)
          loop 
            v_ret[v_cur_y : v_cur_y][(v_cur_x - 1) * array_length(i_array_4d, 4) + 1 : v_cur_x * array_length(i_array_4d, 4)][v_cur_x3 : v_cur_x3]
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
            ;
          end loop;
        end loop;
      end loop;
    end if;
  elsif i_dims_from_to in (array[3, 4], array[4, 3])
  then
    v_ret := array_fill(i_array_4d[0], array[array_length(i_array_4d, 1), array_length(i_array_4d, 2), array_length(i_array_4d, 3) * array_length(i_array_4d, 4)]);
    if i_dim_pin_ele = 3
    then 
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x in 1 .. array_length(i_array_4d, 2)
        loop 
          for v_cur_x4 in 1 .. array_length(i_array_4d, 4)
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][(v_cur_x4 - 1) * array_length(i_array_4d, 3) + 1 : v_cur_x4 * array_length(i_array_4d, 3)] 
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][ : ][v_cur_x4 : v_cur_x4]
            ;
          end loop;
        end loop;
      end loop;
       
    elsif i_dim_pin_ele = 4
    then
      for v_cur_y in 1 .. array_length(i_array_4d, 1)
      loop 
        for v_cur_x in 1 .. array_length(i_array_4d, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_4d, 3)
          loop 
            v_ret[v_cur_y : v_cur_y][v_cur_x : v_cur_x][(v_cur_x3 - 1) * array_length(i_array_4d, 4) + 1 : v_cur_x3 * array_length(i_array_4d, 4)] 
              := i_array_4d[v_cur_y : v_cur_y][v_cur_x : v_cur_x][v_cur_x3 : v_cur_x3][ : ]
            ;
          end loop;
        end loop;
      end loop;
        
    end if;
  end if;
  return v_ret; 
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- with cte_arr as
-- (
--   select 
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4]
--         , [5, 6, 7, 8]
--         , [9, 10, 11, 12]
--         ]
--       , [
--           [21, 22, 23, 24]
--         , [25, 26, 27, 28]
--         , [29, 30, 31, 32]
--         ]
--       ]
--     , [
--         [
--           [-1, -2, -3, -4]
--         , [-5, -6, -7, -8]
--         , [-9, -10, -11, -12]
--         ]
--       , [
--           [-21, -22, -23, -24]
--         , [-25, -26, -27, -28]
--         , [-29, -30, -31, -32]
--         ]
--       ]
--     ] as a_arr
-- )
-- select 
--   a_dims_from_to, a_dim_pin_ele,
--   sm_sc.fv_mx_ele_4d_2_3d(a_arr, a_dims_from_to, a_dim_pin_ele) as a_out
-- from cte_arr
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[3, 4]
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[4, 3]
--       union all select array[1, 3] 
--       union all select array[3, 1]
--       union all select array[1, 4]
--       union all select array[4, 1]
--       union all select array[2, 4]
--       union all select array[4, 2]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 4) tb_a_dim_pin_ele(a_dim_pin_ele)
-- where a_dim_pin_ele = any(a_dims_from_to)
-- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dims_from_to[1]
--   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dim_pin_ele
-- drop function if exists sm_sc.fv_mx_ele_4d_2_3d_py(float[], int[2], int);
create or replace function sm_sc.fv_mx_ele_4d_2_3d_py
(
  i_array_4d        float[],  
  i_dims_from_to    int[2]  ,  -- 合并维度的原来两个维度。合并后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
                               -- 枚举项包括，[1, 2] === [2, 1]; [2, 3] === [3, 2]; [3, 4] === [4, 3]; [1, 3]; [3, 1]; [1, 4]; [4, 1]; [2, 4]; [4, 2]
  i_dim_pin_ele     int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
)
returns float[]
as
$$
-- declare
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_4d) <> 4
    then 
      raise exception 'ndims should be 4.';
    end if;
    
    if i_dim_pin_ele <> all(i_dims_from_to)
    then 
      raise exception 'the pin_ele dim must be one of from_to dims.';
    end if;
    
    if i_dims_from_to[1] not between 1 and 4 
      or i_dims_from_to[2] not between 1 and 4
      or i_dims_from_to[1] = i_dims_from_to[2]
    then 
      raise exception 'unsupport this i_dims_from_to.';
    end if;
  end if;

  if i_array_4d is null 
  then 
    return null;
  end if;  
  
  if i_dims_from_to in (array[1, 2], array[2, 1])
  then
    if i_dim_pin_ele = 1
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[1, 2]
        , array
          [
            array_length(i_array_4d, 1) * array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3)
          , array_length(i_array_4d, 4)
          ]
          )
      ;
      
    elsif i_dim_pin_ele = 2
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d
        , array
          [
            array_length(i_array_4d, 1) * array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3)
          , array_length(i_array_4d, 4)
          ]
          )
      ;
      
    end if;
  elsif i_dims_from_to = array[1, 3]
  then
    if i_dim_pin_ele = 1
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[1, 2] |^~| array[2, 3]
        , array
          [
            array_length(i_array_4d, 2)
          , array_length(i_array_4d, 1) * array_length(i_array_4d, 3)
          , array_length(i_array_4d, 4)
          ]
        )
      ;      
      
    elsif i_dim_pin_ele = 3
    then      
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[1, 2]
        , array
          [
            array_length(i_array_4d, 2)
          , array_length(i_array_4d, 1) * array_length(i_array_4d, 3)
          , array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    end if;
  elsif i_dims_from_to = array[3, 1]
  then
    if i_dim_pin_ele = 1
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[3, 2] |^~| array[2, 1]
        , array
          [
            array_length(i_array_4d, 1) * array_length(i_array_4d, 3)
          , array_length(i_array_4d, 2)
          , array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 3
    then      
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[2, 3]
        , array
          [
            array_length(i_array_4d, 1) * array_length(i_array_4d, 3)
          , array_length(i_array_4d, 2)
          , array_length(i_array_4d, 4)
          ]
        )
      ;   
      
    end if;
  elsif i_dims_from_to = array[1, 4]
  then
    if i_dim_pin_ele = 1
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[1, 2] |^~| array[2, 3] |^~| array[3, 4]
        , array
          [
            array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3)
          , array_length(i_array_4d, 1) * array_length(i_array_4d, 4)
          ]
        )
      ;      
      
    elsif i_dim_pin_ele = 4
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[1, 2] |^~| array[2, 3]
        , array
          [
            array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3)
          , array_length(i_array_4d, 1) * array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    end if;
  elsif i_dims_from_to = array[4, 1]
  then
    if i_dim_pin_ele = 1
    then  
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[4, 3] |^~| array[3, 2] |^~| array[2, 1]
        , array
          [
            array_length(i_array_4d, 1) * array_length(i_array_4d, 4)
          , array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 4
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[4, 3] |^~| array[3, 2]
        , array
          [
            array_length(i_array_4d, 1) * array_length(i_array_4d, 4)
          , array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3)
          ]
        )
      ;   
      
    end if;
  elsif i_dims_from_to in (array[2, 3], array[3, 2])
  then
    if i_dim_pin_ele = 2
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[2, 3]
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 2) * array_length(i_array_4d, 3)
          , array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 3
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 2) * array_length(i_array_4d, 3)
          , array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    end if;
  elsif i_dims_from_to = array[2, 4]
  then
    if i_dim_pin_ele = 2
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[2, 3] |^~| array[3, 4]
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 3)
          , array_length(i_array_4d, 2) * array_length(i_array_4d, 4)
          ]
        )
      ;      
      
    elsif i_dim_pin_ele = 4
    then      
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[2, 3]
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 3)
          , array_length(i_array_4d, 2) * array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    end if;
  elsif i_dims_from_to = array[4, 2]
  then
    if i_dim_pin_ele = 2
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[4, 3] |^~| array[3, 2]
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 2) * array_length(i_array_4d, 4)
          , array_length(i_array_4d, 3)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 4
    then      
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[3, 4]
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 2) * array_length(i_array_4d, 4)
          , array_length(i_array_4d, 3)
          ]
        )
      ;   
      
    end if;
  elsif i_dims_from_to in (array[3, 4], array[4, 3])
  then
    if i_dim_pin_ele = 3
    then 
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d |^~| array[3, 4]
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3) * array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    elsif i_dim_pin_ele = 4
    then
      return 
        sm_sc.fv_opr_reshape_py
        (
          i_array_4d
        , array
          [
            array_length(i_array_4d, 1)
          , array_length(i_array_4d, 2)
          , array_length(i_array_4d, 3) * array_length(i_array_4d, 4)
          ]
        )
      ;       
      
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- with cte_arr as
-- (
--   select 
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4]
--         , [5, 6, 7, 8]
--         , [9, 10, 11, 12]
--         ]
--       , [
--           [21, 22, 23, 24]
--         , [25, 26, 27, 28]
--         , [29, 30, 31, 32]
--         ]
--       ]
--     , [
--         [
--           [-1, -2, -3, -4]
--         , [-5, -6, -7, -8]
--         , [-9, -10, -11, -12]
--         ]
--       , [
--           [-21, -22, -23, -24]
--         , [-25, -26, -27, -28]
--         , [-29, -30, -31, -32]
--         ]
--       ]
--     ] as a_arr
-- )
-- select 
--   a_dims_from_to, a_dim_pin_ele,
--   sm_sc.fv_mx_ele_4d_2_3d_py(a_arr, a_dims_from_to, a_dim_pin_ele) as a_out
-- from cte_arr
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[3, 4]
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[4, 3]
--       union all select array[1, 3] 
--       union all select array[3, 1]
--       union all select array[1, 4]
--       union all select array[4, 1]
--       union all select array[2, 4]
--       union all select array[4, 2]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 4) tb_a_dim_pin_ele(a_dim_pin_ele)
-- where a_dim_pin_ele = any(a_dims_from_to)
-- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dims_from_to[1]
--   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dim_pin_ele

-- with 
-- cte_arr as 
-- (
--   -- select (sm_sc.fv_new_rand(array[2,3,5]) :: decimal[] ~=` 3) :: float[] as a_arr
--   select 
--     array
--     [[[[  0,  1,  2,  3,  4,  5,  6]
--       ,[  7,  8,  9, 10, 11, 12, 13]
--       ,[ 14, 15, 16, 17, 18, 19, 20]
--       ,[ 21, 22, 23, 24, 25, 26, 27]
--       ,[ 28, 29, 30, 31, 32, 33, 34]]
--      ,[[ 35, 36, 37, 38, 39, 40, 41]
--       ,[ 42, 43, 44, 45, 46, 47, 48]
--       ,[ 49, 50, 51, 52, 53, 54, 55]
--       ,[ 56, 57, 58, 59, 60, 61, 62]
--       ,[ 63, 64, 65, 66, 67, 68, 69]]
--      ,[[ 70, 71, 72, 73, 74, 75, 76]
--       ,[ 77, 78, 79, 80, 81, 82, 83]
--       ,[ 84, 85, 86, 87, 88, 89, 90]
--       ,[ 91, 92, 93, 94, 95, 96, 97]
--       ,[ 98, 99,100,101,102,103,104]]]
--     ,[[[105,106,107,108,109,110,111]
--       ,[112,113,114,115,116,117,118]
--       ,[119,120,121,122,123,124,125]
--       ,[126,127,128,129,130,131,132]
--       ,[133,134,135,136,137,138,139]]
--      ,[[140,141,142,143,144,145,146]
--       ,[147,148,149,150,151,152,153]
--       ,[154,155,156,157,158,159,160]
--       ,[161,162,163,164,165,166,167]
--       ,[168,169,170,171,172,173,174]]
--      ,[[175,176,177,178,179,180,181]
--       ,[182,183,184,185,186,187,188]
--       ,[189,190,191,192,193,194,195]
--       ,[196,197,198,199,200,201,202]
--       ,[203,204,205,206,207,208,209]]]] :: float[]
--     as a_arr
-- ),
-- cte_dims as 
-- (
--   select array[1,2] as a_dims union all
--   select array[2,1]           union all
--   select array[1,3]           union all
--   select array[3,1]           union all
--   select array[1,4]           union all
--   select array[4,1]           union all
--   select array[2,3]           union all
--   select array[3,2]           union all
--   select array[2,4]           union all
--   select array[4,2]           union all
--   select array[3,4]           union all
--   select array[4,3]
-- ),
-- cte_pin as 
-- (
--   select 1 as a_pin union all select 2
-- )
-- select a_dims, a_pin
-- , sm_sc.fv_mx_ele_4d_2_3d_py(a_arr, a_dims, a_dims[a_pin]) = sm_sc.fv_mx_ele_4d_2_3d(a_arr, a_dims, a_dims[a_pin])
-- , sm_sc.fv_mx_ele_4d_2_3d_py(a_arr, a_dims, a_dims[a_pin])
-- , sm_sc.fv_mx_ele_4d_2_3d(a_arr, a_dims, a_dims[a_pin])
-- from cte_arr, cte_dims, cte_pin
-- order by a_dims, a_pin
-- drop function if exists sm_sc.fv_mx_ele_flatten_2dims(anyarray, int[2], int);
create or replace function sm_sc.fv_mx_ele_flatten_2dims
(
  i_array_nd        anyarray,  
  i_dims_from_to    int[2]  ,  -- 扁平化的维度的原来两个维度。扁平化后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
  i_dim_pin_ele     int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
)
returns anyarray
as
$$
declare
  v_ret     i_array_nd%type;
  v_cur_y   int            ;
  v_cur_x   int            ;
  v_cur_x3  int            ;
  v_cur_x4  int            ;

begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array_nd) not between 2 and 4
    then 
      raise exception 'unsupport ndims outof range [2,4]';
    end if;
    
    if i_dim_pin_ele <> all(i_dims_from_to)
    then 
      raise exception 'the pin_ele dim must be one of from_to dims.';
    end if;
    
    if i_dims_from_to[1] not between 1 and array_ndims(i_array_nd) 
      or i_dims_from_to[2] not between 1 and array_ndims(i_array_nd) 
      or i_dims_from_to[1] = i_dims_from_to[2]
    then 
      raise exception 'unsupport this i_dims_from_to.';
    end if;
  end if;

  if i_array_nd is null 
  then 
    return null;
  end if;  
  
  if array_ndims(i_array_nd) = 2
  then 
    if i_dims_from_to = array[1, 2]
    then
      v_ret := array_fill(i_array_nd[0], array[1, array_length(i_array_nd, 1) * array_length(i_array_nd, 2)]);
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          v_ret
            [1 : 1]
            [(v_cur_x - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x * array_length(i_array_nd, 1)] 
          := 
            i_array_nd
              [ : ]
              [v_cur_x : v_cur_x]
          ;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret
            [1 : 1]
            [(v_cur_y - 1) * array_length(i_array_nd, 2) + 1 : v_cur_y * array_length(i_array_nd, 2)] 
          := 
            i_array_nd
              [v_cur_y : v_cur_y]
              [ : ]
          ;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 1]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1) * array_length(i_array_nd, 2), 1]);
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          v_ret
            [(v_cur_y - 1) * array_length(i_array_nd, 2) + 1 : v_cur_y * array_length(i_array_nd, 2)] 
            [1 : 1]
          := 
            i_array_nd
              [v_cur_y : v_cur_y]
              [ : ]
          ;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          v_ret
            [(v_cur_x - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x * array_length(i_array_nd, 1)] 
            [1 : 1]
          := 
            i_array_nd
              [ : ]
              [v_cur_x : v_cur_x]
          ;
        end loop;
      end if;
    end if;
  elsif array_ndims(i_array_nd) = 3
  then 
    if i_dims_from_to = array[1, 2]
    then
      v_ret := array_fill(i_array_nd[0], array[1, array_length(i_array_nd, 1) * array_length(i_array_nd, 2), array_length(i_array_nd, 3)]);
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [1 : 1]
              [(v_cur_x - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x * array_length(i_array_nd, 1)] 
              [v_cur_x3 : v_cur_x3]
            := 
              i_array_nd
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [1 : 1]
              [(v_cur_y - 1) * array_length(i_array_nd, 2) + 1 : v_cur_y * array_length(i_array_nd, 2)] 
              [v_cur_x3 : v_cur_x3]
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 1]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1) * array_length(i_array_nd, 2), 1, array_length(i_array_nd, 3)]);
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [(v_cur_y - 1) * array_length(i_array_nd, 2) + 1 : v_cur_y * array_length(i_array_nd, 2)] 
              [1 : 1]
              [v_cur_x3 : v_cur_x3]
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [(v_cur_x - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x * array_length(i_array_nd, 1)] 
              [1 : 1]
              [v_cur_x3 : v_cur_x3]
            := 
              i_array_nd
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[1, 3]
    then 
      v_ret := array_fill(i_array_nd[0], array[1, array_length(i_array_nd, 2), array_length(i_array_nd, 1) * array_length(i_array_nd, 3)]);
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [1 : 1]
              [v_cur_x : v_cur_x]
              [(v_cur_x3 - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x3 * array_length(i_array_nd, 1)] 
            := 
              i_array_nd
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            v_ret
              [1 : 1]
              [v_cur_x : v_cur_x]
              [(v_cur_y - 1) * array_length(i_array_nd, 3) + 1 : v_cur_y * array_length(i_array_nd, 3)] 
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 1]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1) * array_length(i_array_nd, 3), array_length(i_array_nd, 2), 1]);
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            v_ret
              [(v_cur_y - 1) * array_length(i_array_nd, 3) + 1 : v_cur_y * array_length(i_array_nd, 3)] 
              [v_cur_x : v_cur_x]
              [1 : 1]
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [(v_cur_x3 - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x3 * array_length(i_array_nd, 1)] 
              [v_cur_x : v_cur_x]
              [1 : 1]
            := 
              i_array_nd
                [ : ]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 3]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), 1, array_length(i_array_nd, 2) * array_length(i_array_nd, 3)]);
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [1 : 1]
              [(v_cur_x3 - 1) * array_length(i_array_nd, 2) + 1 : v_cur_x3 * array_length(i_array_nd, 2)] 
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [1 : 1]
              [(v_cur_x - 1) * array_length(i_array_nd, 3) + 1 : v_cur_x * array_length(i_array_nd, 3)] 
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
            ;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 2]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), array_length(i_array_nd, 2) * array_length(i_array_nd, 3), 1]);
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [(v_cur_x - 1) * array_length(i_array_nd, 3) + 1 : v_cur_x * array_length(i_array_nd, 3)] 
              [1 : 1]
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [ : ]
            ;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            v_ret
              [v_cur_y : v_cur_y]
              [(v_cur_x3 - 1) * array_length(i_array_nd, 2) + 1 : v_cur_x3 * array_length(i_array_nd, 2)] 
              [1 : 1]
            := 
              i_array_nd
                [v_cur_y : v_cur_y]
                [ : ]
                [v_cur_x3 : v_cur_x3]
            ;
          end loop;
        end loop;
      end if;
    end if;
  elsif array_ndims(i_array_nd) = 4
  then 
    if i_dims_from_to = array[1, 2]
    then
      v_ret := array_fill(i_array_nd[0], array[1, array_length(i_array_nd, 1) * array_length(i_array_nd, 2), array_length(i_array_nd, 3), array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [1 : 1]
                [(v_cur_x - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x * array_length(i_array_nd, 1)] 
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [ : ]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [1 : 1]
                [(v_cur_y - 1) * array_length(i_array_nd, 2) + 1 : v_cur_y * array_length(i_array_nd, 2)] 
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [ : ]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 1]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1) * array_length(i_array_nd, 2), 1, array_length(i_array_nd, 3), array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [(v_cur_y - 1) * array_length(i_array_nd, 2) + 1 : v_cur_y * array_length(i_array_nd, 2)] 
                [1 : 1]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [ : ]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [(v_cur_x - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x * array_length(i_array_nd, 1)] 
                [1 : 1]
                [v_cur_x3 : v_cur_x3]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [ : ]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[1, 3]
    then 
      v_ret := array_fill(i_array_nd[0], array[1, array_length(i_array_nd, 2), array_length(i_array_nd, 1) * array_length(i_array_nd, 3), array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [1 : 1]
                [v_cur_x : v_cur_x]
                [(v_cur_x3 - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x3 * array_length(i_array_nd, 1)] 
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [ : ]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [1 : 1]
                [v_cur_x : v_cur_x]
                [(v_cur_y - 1) * array_length(i_array_nd, 3) + 1 : v_cur_y * array_length(i_array_nd, 3)] 
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [ : ]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 1]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1) * array_length(i_array_nd, 3), array_length(i_array_nd, 2), 1, array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [(v_cur_y - 1) * array_length(i_array_nd, 3) + 1 : v_cur_y * array_length(i_array_nd, 3)] 
                [v_cur_x : v_cur_x]
                [1 : 1]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [ : ]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [(v_cur_x3 - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x3 * array_length(i_array_nd, 1)] 
                [v_cur_x : v_cur_x]
                [1 : 1]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [ : ]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[1, 4]
    then 
      v_ret := array_fill(i_array_nd[0], array[1, array_length(i_array_nd, 2), array_length(i_array_nd, 3), array_length(i_array_nd, 1) * array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [1 : 1]
                [v_cur_x : v_cur_x] 
                [v_cur_x3 : v_cur_x3]
                [(v_cur_x4 - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x4 * array_length(i_array_nd, 1)]
              := 
                i_array_nd
                  [ : ]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
            loop 
              v_ret
                [1 : 1]
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [(v_cur_y - 1) * array_length(i_array_nd, 4) + 1 : v_cur_y * array_length(i_array_nd, 4)] 
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [ : ]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[4, 1]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1) * array_length(i_array_nd, 4), array_length(i_array_nd, 2), array_length(i_array_nd, 3), 1]);
      if i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
            loop 
              v_ret
                [(v_cur_y - 1) * array_length(i_array_nd, 4) + 1 : v_cur_y * array_length(i_array_nd, 4)] 
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [1 : 1]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [ : ]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 1
      then 
        for v_cur_x in 1 .. array_length(i_array_nd, 2)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [(v_cur_x4 - 1) * array_length(i_array_nd, 1) + 1 : v_cur_x4 * array_length(i_array_nd, 1)] 
                [v_cur_x : v_cur_x]
                [v_cur_x3 : v_cur_x3]
                [1 : 1]
              := 
                i_array_nd
                  [ : ]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 3]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), 1, array_length(i_array_nd, 2) * array_length(i_array_nd, 3), array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [1 : 1]
                [(v_cur_x3 - 1) * array_length(i_array_nd, 2) + 1 : v_cur_x3 * array_length(i_array_nd, 2)] 
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [ : ]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [1 : 1]
                [(v_cur_x - 1) * array_length(i_array_nd, 3) + 1 : v_cur_x * array_length(i_array_nd, 3)] 
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [ : ]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 2]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), array_length(i_array_nd, 2) * array_length(i_array_nd, 3), 1, array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [(v_cur_x - 1) * array_length(i_array_nd, 3) + 1 : v_cur_x * array_length(i_array_nd, 3)] 
                [1 : 1]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [ : ]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [(v_cur_x3 - 1) * array_length(i_array_nd, 2) + 1 : v_cur_x3 * array_length(i_array_nd, 2)] 
                [1 : 1]
                [v_cur_x4 : v_cur_x4]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [ : ]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[2, 4]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), 1, array_length(i_array_nd, 3), array_length(i_array_nd, 2) * array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y] 
                [1 : 1]
                [v_cur_x3 : v_cur_x3]
                [(v_cur_x4 - 1) * array_length(i_array_nd, 2) + 1 : v_cur_x4 * array_length(i_array_nd, 2)]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [ : ]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [1 : 1]
                [v_cur_x3 : v_cur_x3]
                [(v_cur_x - 1) * array_length(i_array_nd, 4) + 1 : v_cur_x * array_length(i_array_nd, 4)] 
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [ : ]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[4, 2]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), array_length(i_array_nd, 2) * array_length(i_array_nd, 4), array_length(i_array_nd, 3), 1]);
      if i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [(v_cur_x - 1) * array_length(i_array_nd, 4) + 1 : v_cur_x * array_length(i_array_nd, 4)] 
                [v_cur_x3 : v_cur_x3]
                [1 : 1]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [ : ]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 2
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [(v_cur_x4 - 1) * array_length(i_array_nd, 2) + 1 : v_cur_x4 * array_length(i_array_nd, 2)] 
                [v_cur_x3 : v_cur_x3]
                [1 : 1]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [ : ]
                  [v_cur_x3 : v_cur_x3]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[3, 4]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), array_length(i_array_nd, 2), 1, array_length(i_array_nd, 3) * array_length(i_array_nd, 4)]);
      if i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x2 in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y] 
                [v_cur_x2 : v_cur_x2]
                [1 : 1]
                [(v_cur_x4 - 1) * array_length(i_array_nd, 3) + 1 : v_cur_x4 * array_length(i_array_nd, 3)]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x2 : v_cur_x2]
                  [ : ]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [1 : 1]
                [(v_cur_x3 - 1) * array_length(i_array_nd, 4) + 1 : v_cur_x3 * array_length(i_array_nd, 4)] 
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [ : ]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    elsif i_dims_from_to = array[4, 3]
    then 
      v_ret := array_fill(i_array_nd[0], array[array_length(i_array_nd, 1), array_length(i_array_nd, 2), array_length(i_array_nd, 3) * array_length(i_array_nd, 4), 1]);
      if i_dim_pin_ele = 4
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x3 in 1 .. array_length(i_array_nd, 3)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [(v_cur_x3 - 1) * array_length(i_array_nd, 4) + 1 : v_cur_x3 * array_length(i_array_nd, 4)] 
                [1 : 1]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [v_cur_x3 : v_cur_x3]
                  [ : ]
              ;
            end loop;
          end loop;
        end loop;
      elsif i_dim_pin_ele = 3
      then 
        for v_cur_y in 1 .. array_length(i_array_nd, 1)
        loop 
          for v_cur_x in 1 .. array_length(i_array_nd, 2)
          loop 
            for v_cur_x4 in 1 .. array_length(i_array_nd, 4)
            loop 
              v_ret
                [v_cur_y : v_cur_y]
                [v_cur_x : v_cur_x]
                [(v_cur_x4 - 1) * array_length(i_array_nd, 3) + 1 : v_cur_x4 * array_length(i_array_nd, 3)] 
                [1 : 1]
              := 
                i_array_nd
                  [v_cur_y : v_cur_y]
                  [v_cur_x : v_cur_x]
                  [ : ]
                  [v_cur_x4 : v_cur_x4]
              ;
            end loop;
          end loop;
        end loop;
      end if;
    end if;
  end if;
  
  return v_ret; 
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- with cte_arr as
-- (
--   select 
--     array
--     [
--       [
--         [1, 2, 3, 4]
--       , [5, 6, 7, 8]
--       , [9, 10, 11, 12]
--       ]
--     , [
--         [21, 22, 23, 24]
--       , [25, 26, 27, 28]
--       , [29, 30, 31, 32]
--       ]
--     ]
--     as a_arr
-- )
-- select 
--   a_dims_from_to, a_dim_pin_ele,
--   sm_sc.fv_mx_ele_flatten_2dims(a_arr, a_dims_from_to, a_dim_pin_ele) as a_out
-- from cte_arr
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[1, 3] 
--       union all select array[3, 1]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 3) tb_a_dim_pin_ele(a_dim_pin_ele)
-- where a_dim_pin_ele = any(a_dims_from_to)
-- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dims_from_to[1]
--   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dim_pin_ele

-- select 
--   a_dim_from
-- , a_dim_to
-- , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx] as a_dim_pin_ele
-- , array_dims(
--   sm_sc.fv_mx_ele_flatten_2dims
--   (
--     a_arr
--   , array[a_dim_from, a_dim_to]
--   , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx]
--   )) as a_flattened_dims
-- , array_dims(a_arr) as a_arr_dims
-- from 
--   (
--               select sm_sc.fv_new_rand(array[4,3    ]) as a_arr
--     union all select sm_sc.fv_new_rand(array[4,3,5  ]) as a_arr
--     union all select sm_sc.fv_new_rand(array[4,3,5,7]) as a_arr
--   ) tb_a_arr(a_arr)
-- , generate_series(1, 4) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_to(a_dim_to)
-- , generate_series(1, 2) tb_a_dim_pin_ele_idx(a_dim_pin_ele_idx)
-- where a_dim_from <= array_ndims(a_arr)
--   and a_dim_to <= array_ndims(a_arr)
--   and a_dim_from <> a_dim_to
-- order by array_ndims(a_arr), a_dim_from, a_dim_to, a_dim_pin_ele_idx

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_mx_ele_flatten_2dims_py(float[], int[2], int);
create or replace function sm_sc.fv_mx_ele_flatten_2dims_py
(
  i_array_nd        float[]  
, i_dims_from_to    int[2]     -- 扁平化的维度的原来两个维度。扁平化后的新维度在 to 的顺序位置。当 from 与 to 为相邻维度时，[from, to] 等价于 [to, from]。
, i_dim_pin_ele     int        -- 被定住元素顺序的旧维度。该旧维度下的元素顺序，将保留至新维度。i_dim_pin_ele 为 from 或 to，不能为其他值。
)
returns float[]
as
$$
  import numpy as np
  v_dims_from_to = (np.int8(i_dims_from_to) -1).tolist()
  v_dim_pin_ele = i_dim_pin_ele - 1
  
  v_array_nd = np.array(i_array_nd)
  if v_array_nd.ndim == 2 :
    if v_dims_from_to == [0, 1] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 0).reshape(1, v_array_nd.shape[0] * v_array_nd.shape[1]).tolist()
      elif v_dim_pin_ele == 1 :
        return v_array_nd.reshape(1, v_array_nd.shape[0] * v_array_nd.shape[1]).tolist()
    elif v_dims_from_to == [1, 0] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 0).reshape(v_array_nd.shape[0] * v_array_nd.shape[1], 1).tolist()
      elif v_dim_pin_ele == 1 :
        return v_array_nd.reshape(v_array_nd.shape[0] * v_array_nd.shape[1], 1).tolist()
      
  elif v_array_nd.ndim == 3 :
    if v_dims_from_to == [0, 1] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(2, 1, 0).reshape(v_array_nd.shape[2], v_array_nd.shape[1] * v_array_nd.shape[0], 1).transpose(2, 1, 0).tolist()
      elif v_dim_pin_ele == 1 :
        return v_array_nd.transpose(2, 0, 1).reshape(v_array_nd.shape[2], 1, v_array_nd.shape[0] * v_array_nd.shape[1]).transpose(1, 2, 0).tolist()
      
    elif v_dims_from_to == [1, 0] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(2, 1, 0).reshape(v_array_nd.shape[2], 1, v_array_nd.shape[1] * v_array_nd.shape[0]).transpose(2, 1, 0).tolist()
      elif v_dim_pin_ele == 1 :
        return v_array_nd.transpose(2, 0, 1).reshape(v_array_nd.shape[2], v_array_nd.shape[0] * v_array_nd.shape[1], 1).transpose(1, 2, 0).tolist()
      
    elif v_dims_from_to == [0, 2] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 2, 0).reshape(v_array_nd.shape[1], v_array_nd.shape[2] * v_array_nd.shape[0], 1).transpose(2, 0, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.transpose(1, 0, 2).reshape(v_array_nd.shape[1], 1, v_array_nd.shape[0] * v_array_nd.shape[2]).transpose(1, 0, 2).tolist()
      
    elif v_dims_from_to == [2, 0] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 2, 0).reshape(v_array_nd.shape[1], 1, v_array_nd.shape[2] * v_array_nd.shape[0]).transpose(2, 0, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.transpose(1, 0, 2).reshape(v_array_nd.shape[1], v_array_nd.shape[0] * v_array_nd.shape[2], 1).transpose(1, 0, 2).tolist()
      
    elif v_dims_from_to == [1, 2] :
      if v_dim_pin_ele == 1 :
        return v_array_nd.transpose(0, 2, 1).reshape(v_array_nd.shape[0], v_array_nd.shape[2] * v_array_nd.shape[1], 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.reshape(v_array_nd.shape[0], 1, v_array_nd.shape[1] * v_array_nd.shape[2]).tolist()
      
    elif v_dims_from_to == [2, 1] :
      if v_dim_pin_ele == 1 :
        return v_array_nd.transpose(0, 2, 1).reshape(v_array_nd.shape[0], 1, v_array_nd.shape[2] * v_array_nd.shape[1]).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.reshape(v_array_nd.shape[0], v_array_nd.shape[1] * v_array_nd.shape[2], 1).tolist()
    
  elif v_array_nd.ndim == 4 :
    if v_dims_from_to == [0, 1] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(2, 3, 1, 0).reshape(v_array_nd.shape[2], v_array_nd.shape[3], v_array_nd.shape[1] * v_array_nd.shape[0], 1).transpose(3, 2, 0, 1).tolist()
      elif v_dim_pin_ele == 1 :
        return v_array_nd.transpose(2, 3, 0, 1).reshape(v_array_nd.shape[2], v_array_nd.shape[3], 1, v_array_nd.shape[0] * v_array_nd.shape[1]).transpose(2, 3, 0, 1).tolist()
      
    elif v_dims_from_to == [1, 0] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(2, 3, 1, 0).reshape(v_array_nd.shape[2], v_array_nd.shape[3], 1, v_array_nd.shape[1] * v_array_nd.shape[0]).transpose(3, 2, 0, 1).tolist()
      elif v_dim_pin_ele == 1 :
        return v_array_nd.transpose(2, 3, 0, 1).reshape(v_array_nd.shape[2], v_array_nd.shape[3], v_array_nd.shape[0] * v_array_nd.shape[1], 1).transpose(2, 3, 0, 1).tolist()
      
    elif v_dims_from_to == [0, 2] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 3, 2, 0).reshape(v_array_nd.shape[1], v_array_nd.shape[3], v_array_nd.shape[2] * v_array_nd.shape[0], 1).transpose(3, 0, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.transpose(1, 3, 0, 2).reshape(v_array_nd.shape[1], v_array_nd.shape[3], 1, v_array_nd.shape[0] * v_array_nd.shape[2]).transpose(2, 0, 3, 1).tolist()
      
    elif v_dims_from_to == [2, 0] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 3, 2, 0).reshape(v_array_nd.shape[1], v_array_nd.shape[3], 1, v_array_nd.shape[2] * v_array_nd.shape[0]).transpose(3, 0, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.transpose(1, 3, 0, 2).reshape(v_array_nd.shape[1], v_array_nd.shape[3], v_array_nd.shape[0] * v_array_nd.shape[2], 1).transpose(2, 0, 3, 1).tolist()
      
    elif v_dims_from_to == [0, 3] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 2, 3, 0).reshape(v_array_nd.shape[1], v_array_nd.shape[2], v_array_nd.shape[3] * v_array_nd.shape[0], 1).transpose(3, 0, 1, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_array_nd.transpose(1, 2, 0, 3).reshape(v_array_nd.shape[1], v_array_nd.shape[2], 1, v_array_nd.shape[0] * v_array_nd.shape[3]).transpose(2, 0, 1, 3).tolist()
      
    elif v_dims_from_to == [3, 0] :
      if v_dim_pin_ele == 0 :
        return v_array_nd.transpose(1, 2, 3, 0).reshape(v_array_nd.shape[1], v_array_nd.shape[2], 1, v_array_nd.shape[3] * v_array_nd.shape[0]).transpose(3, 0, 1, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_array_nd.transpose(1, 2, 0, 3).reshape(v_array_nd.shape[1], v_array_nd.shape[2], v_array_nd.shape[0] * v_array_nd.shape[3], 1).transpose(2, 0, 1, 3).tolist()
      
    elif v_dims_from_to == [1, 2] :
      if v_dim_pin_ele == 1 :
        return v_array_nd.transpose(0, 3, 2, 1).reshape(v_array_nd.shape[0], v_array_nd.shape[3], v_array_nd.shape[2] * v_array_nd.shape[1], 1).transpose(0, 3, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.transpose(0, 3, 1, 2).reshape(v_array_nd.shape[0], v_array_nd.shape[3], 1, v_array_nd.shape[1] * v_array_nd.shape[2]).transpose(0, 2, 3, 1).tolist()
      
    elif v_dims_from_to == [2, 1] :
      if v_dim_pin_ele == 1 :
        return v_array_nd.transpose(0, 3, 2, 1).reshape(v_array_nd.shape[0], v_array_nd.shape[3], 1, v_array_nd.shape[2] * v_array_nd.shape[1]).transpose(0, 3, 2, 1).tolist()
      elif v_dim_pin_ele == 2 :
        return v_array_nd.transpose(0, 3, 1, 2).reshape(v_array_nd.shape[0], v_array_nd.shape[3], v_array_nd.shape[2] * v_array_nd.shape[1], 1).transpose(0, 2, 3, 1).tolist()

    elif v_dims_from_to == [1, 3] :
      if v_dim_pin_ele == 1 :
        return v_array_nd.transpose(0, 2, 3, 1).reshape(v_array_nd.shape[0], v_array_nd.shape[2], v_array_nd.shape[3] * v_array_nd.shape[1], 1).transpose(0, 3, 1, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_array_nd.transpose(0, 2, 1, 3).reshape(v_array_nd.shape[0], v_array_nd.shape[2], 1, v_array_nd.shape[1] * v_array_nd.shape[3]).transpose(0, 2, 1, 3).tolist()
      
    elif v_dims_from_to == [3, 1] :
      if v_dim_pin_ele == 1 :
        return v_array_nd.transpose(0, 2, 3, 1).reshape(v_array_nd.shape[0], v_array_nd.shape[2], 1, v_array_nd.shape[3] * v_array_nd.shape[1]).transpose(0, 3, 1, 2).tolist()
      elif v_dim_pin_ele == 3 :
        return v_array_nd.transpose(0, 2, 1, 3).reshape(v_array_nd.shape[0], v_array_nd.shape[2], v_array_nd.shape[1] * v_array_nd.shape[3], 1).transpose(0, 2, 1, 3).tolist()

    elif v_dims_from_to == [2, 3] :
      if v_dim_pin_ele == 2 :
        return v_array_nd.transpose(0, 1, 3, 2).reshape(v_array_nd.shape[0], v_array_nd.shape[1], 1, v_array_nd.shape[3] * v_array_nd.shape[2]).tolist()
      elif v_dim_pin_ele == 3 :
        return v_array_nd.reshape(v_array_nd.shape[0], v_array_nd.shape[1], 1, v_array_nd.shape[2] * v_array_nd.shape[3]).tolist()
      
    elif v_dims_from_to == [3, 2] :
      if v_dim_pin_ele == 2 :
        return v_array_nd.transpose(0, 1, 3, 2).reshape(v_array_nd.shape[0], v_array_nd.shape[1], 1, v_array_nd.shape[3] * v_array_nd.shape[2]).tolist()
      elif v_dim_pin_ele == 3 :
        return v_array_nd.reshape(v_array_nd.shape[0], v_array_nd.shape[1], v_array_nd.shape[2] * v_array_nd.shape[3], 1).tolist()

$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- with cte_arr as
-- (
--   select 
--     array
--     [
--       [
--         [1, 2, 3, 4]
--       , [5, 6, 7, 8]
--       , [9, 10, 11, 12]
--       ]
--     , [
--         [21, 22, 23, 24]
--       , [25, 26, 27, 28]
--       , [29, 30, 31, 32]
--       ]
--     ]
--     as a_arr
-- )
-- select 
--   a_dims_from_to, a_dim_pin_ele,
--   sm_sc.fv_mx_ele_flatten_2dims_py(a_arr, a_dims_from_to, a_dim_pin_ele) as a_out
-- from cte_arr
--   , (
--                 select array[1, 2]  
--       union all select array[2, 3] 
--       union all select array[2, 1]  
--       union all select array[3, 2] 
--       union all select array[1, 3] 
--       union all select array[3, 1]
--     ) tb_a_dims_from_to(a_dims_from_to)
--   , generate_series(1, 3) tb_a_dim_pin_ele(a_dim_pin_ele)
-- where a_dim_pin_ele = any(a_dims_from_to)
-- order by least(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dims_from_to[1]
--   , greatest(|@<| a_dims_from_to, |@>| a_dims_from_to)
--   , a_dim_pin_ele

-- select 
--   a_dim_from
-- , a_dim_to
-- , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx] as a_dim_pin_ele
-- , array_dims(
--   sm_sc.fv_mx_ele_flatten_2dims_py
--   (
--     a_arr
--   , array[a_dim_from, a_dim_to]
--   , (array[a_dim_from, a_dim_to])[a_dim_pin_ele_idx]
--   )) as a_flattened_dims
-- , array_dims(a_arr) as a_arr_dims
-- from 
--   (
--               select sm_sc.fv_new_rand(array[4,3    ]) as a_arr
--     union all select sm_sc.fv_new_rand(array[4,3,5  ]) as a_arr
--     union all select sm_sc.fv_new_rand(array[4,3,5,7]) as a_arr
--   ) tb_a_arr(a_arr)
-- , generate_series(1, 4) tb_a_dim_from(a_dim_from)
-- , generate_series(1, 4) tb_a_dim_to(a_dim_to)
-- , generate_series(1, 2) tb_a_dim_pin_ele_idx(a_dim_pin_ele_idx)
-- where a_dim_from <= array_ndims(a_arr)
--   and a_dim_to <= array_ndims(a_arr)
--   and a_dim_from <> a_dim_to
-- order by array_ndims(a_arr), a_dim_from, a_dim_to, a_dim_pin_ele_idx

-- drop function if exists sm_sc.fv_mx_slice_3d_2_2d(anyarray, int, int);
create or replace function sm_sc.fv_mx_slice_3d_2_2d
(
  i_array_3d        anyarray,  
  i_dim_sliced      int     ,               -- 被切片维度
  i_slice_pos       int     default  1      -- 被切片位置序号
)
returns anyarray
as
$$
declare
  v_ret      i_array_3d%type;
  v_len_1    int   :=   array_length(i_array_3d, 1);
  v_len_2    int   :=   array_length(i_array_3d, 2);
  v_len_3    int   :=   array_length(i_array_3d, 3);
  v_cur      int   ;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_dim_sliced not between 1 and 3
    then 
      raise exception 'i_dim_sliced is out of length.';
    elsif array_ndims(i_array_3d) <> 3
    then 
      raise exception 'ndims should be 3.';
    elsif i_slice_pos not between 1 and array_length(i_array_3d, i_dim_sliced)
    then 
      raise exception 'i_slice_pos is out of length of i_array_3d.';
    end if;
  end if;

  if i_array_3d is null 
  then 
    return null;
  elsif i_dim_sliced = 1
  then
    v_ret := array_fill(nullif(i_array_3d[1][1][1], i_array_3d[1][1][1]), array[v_len_2, v_len_3]);
    for v_cur in 1 .. v_len_2
    loop 
      v_ret[v_cur : v_cur][ : ] := i_array_3d[i_slice_pos : i_slice_pos][v_cur : v_cur][ : ];
    end loop;
    return v_ret ;
  elsif i_dim_sliced = 2
  then
    v_ret := array_fill(nullif(i_array_3d[1][1][1], i_array_3d[1][1][1]), array[v_len_1, v_len_3]);
    for v_cur in 1 .. v_len_1
    loop 
      v_ret[v_cur : v_cur][ : ] := i_array_3d[v_cur : v_cur][i_slice_pos : i_slice_pos][ : ];
    end loop;
    return v_ret ;
  elsif i_dim_sliced = 3
  then
    v_ret := array_fill(nullif(i_array_3d[1][1][1], i_array_3d[1][1][1]), array[v_len_1, v_len_2]);
    for v_cur in 1 .. v_len_1
    loop 
      v_ret[v_cur : v_cur][ : ] := i_array_3d[v_cur : v_cur][ : ][i_slice_pos : i_slice_pos];
    end loop;
    return v_ret ;
  end if;
end;
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
-- sm_sc.fv_mx_slice_3d_2_2d
-- (
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]     ,
--         [11, 12, 13, 14, 15],
--         [21, 22, 23, 24, 25], 
--         [31, 32, 33, 34, 35] 
--       ],
--       [
--         [41, 42, 43, 44, 45]     ,
--         [51, 52, 53, 54, 55],
--         [61, 62, 63, 64, 65], 
--         [71, 72, 73, 74, 75] 
--       ],
--       [
--         [81, 82, 83, 84, 85]     ,
--         [91, 92, 93, 94, 95],
--         [101, 102, 103, 104, 105], 
--         [111, 112, 113, 114, 115] 
--       ]
--     ]
--   )
--   , 
-- 1
-- )
-- drop function if exists sm_sc.fv_mx_slice_3d_2_2d_py(float[], int, int);
create or replace function sm_sc.fv_mx_slice_3d_2_2d_py
(
  i_array_3d        float[],  
  i_dim_sliced      int     ,               -- 被切片维度
  i_slice_pos       int     default  1      -- 被切片位置序号
)
returns float[]
as
$$
  import numpy as np
  if i_dim_sliced == 1 :
    return np.array(i_array_3d)[i_slice_pos - 1, :: , ::].tolist()
  elif i_dim_sliced == 2 :
    return np.array(i_array_3d)[:: , i_slice_pos - 1, ::].tolist()
  elif i_dim_sliced == 3 :
    return np.array(i_array_3d)[:: , :: , i_slice_pos - 1].tolist()

$$
language plpython3u stable
parallel safe;

-- -- set search_path to sm_sc;
-- select 
-- sm_sc.fv_mx_slice_3d_2_2d_py
-- (
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]     ,
--         [11, 12, 13, 14, 15],
--         [21, 22, 23, 24, 25], 
--         [31, 32, 33, 34, 35] 
--       ],
--       [
--         [41, 42, 43, 44, 45]     ,
--         [51, 52, 53, 54, 55],
--         [61, 62, 63, 64, 65], 
--         [71, 72, 73, 74, 75] 
--       ],
--       [
--         [81, 82, 83, 84, 85]     ,
--         [91, 92, 93, 94, 95],
--         [101, 102, 103, 104, 105], 
--         [111, 112, 113, 114, 115] 
--       ]
--     ]
--   )
--   , 
-- 1
-- )
-- drop function if exists sm_sc.fv_mx_slice_4d_2_2d(anyarray, int[], int[]);
create or replace function sm_sc.fv_mx_slice_4d_2_2d
(
  i_array_4d        anyarray,  
  i_dim_sliced      int[]     ,          -- 被切片维度
  i_slice_pos       int[]                -- 被切片在 i_dim_sliced 各对应维度的位置序号。
)
returns anyarray
as
$$
declare
  v_ret      i_array_4d%type;
  v_len_1    int   :=   array_length(i_array_4d, 1);
  v_len_2    int   :=   array_length(i_array_4d, 2);
  v_len_3    int   :=   array_length(i_array_4d, 3);
  v_len_4    int   :=   array_length(i_array_4d, 4);
  v_cur      int   ;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 1 > any(i_dim_sliced) or 4 < any(i_dim_sliced) or i_dim_sliced[1] = i_dim_sliced[2]
    then 
      raise exception 'i_dim_sliced is out of length.';
    elsif array_ndims(i_array_4d) <> 4
    then 
      raise exception 'ndims should be 4.';
    elsif i_slice_pos[1] not between 1 and array_length(i_array_4d, i_dim_sliced[1])
      or i_slice_pos[2] not between 1 and array_length(i_array_4d, i_dim_sliced[2])
    then 
      raise exception 'i_slice_pos is out of length of i_array_4d.';
    end if;
  end if;

  if i_array_4d is null 
  then 
    return null;
  elsif i_dim_sliced[1] = 1
  then
    if i_dim_sliced[2] = 2
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_3, v_len_4]);
      for v_cur in 1 .. v_len_3
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[i_slice_pos[1] : i_slice_pos[1]][i_slice_pos[2] : i_slice_pos[2]][v_cur : v_cur][ : ];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 3
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_2, v_len_4]);
      for v_cur in 1 .. v_len_2
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[i_slice_pos[1] : i_slice_pos[1]][v_cur : v_cur][i_slice_pos[2] : i_slice_pos[2]][ : ];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 4
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_2, v_len_3]);
      for v_cur in 1 .. v_len_2
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[i_slice_pos[1] : i_slice_pos[1]][v_cur : v_cur][ : ][i_slice_pos[2] : i_slice_pos[2]];
      end loop;
      return v_ret ;
    end if;
  elsif i_dim_sliced[1] = 2
  then
    if i_dim_sliced[2] = 1
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_3, v_len_4]);
      for v_cur in 1 .. v_len_3
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[i_slice_pos[2] : i_slice_pos[2]][i_slice_pos[1] : i_slice_pos[1]][v_cur : v_cur][ : ];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 3
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_4]);
      for v_cur in 1 .. v_len_1
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[v_cur : v_cur][i_slice_pos[1] : i_slice_pos[1]][i_slice_pos[2] : i_slice_pos[2]][ : ];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 4
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_3]);
      for v_cur in 1 .. v_len_1
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[v_cur : v_cur][i_slice_pos[1] : i_slice_pos[1]][ : ][i_slice_pos[2] : i_slice_pos[2]];
      end loop;
      return v_ret ;
    end if;
  elsif i_dim_sliced[1] = 3
  then
    if i_dim_sliced[2] = 1
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_2, v_len_4]);
      for v_cur in 1 .. v_len_2
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[i_slice_pos[2] : i_slice_pos[2]][v_cur : v_cur][i_slice_pos[1] : i_slice_pos[1]][ : ];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 2
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_4]);
      for v_cur in 1 .. v_len_1
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[v_cur : v_cur][i_slice_pos[2] : i_slice_pos[2]][i_slice_pos[1] : i_slice_pos[1]][ : ];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 4
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_2]);
      for v_cur in 1 .. v_len_1
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[v_cur : v_cur][ : ][i_slice_pos[1] : i_slice_pos[1]][i_slice_pos[2] : i_slice_pos[2]];
      end loop;
      return v_ret ;
    end if;
  elsif i_dim_sliced[1] = 4
  then
    if i_dim_sliced[2] = 1
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_2, v_len_3]);
      for v_cur in 1 .. v_len_2
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[i_slice_pos[2] : i_slice_pos[2]][v_cur : v_cur][ : ][i_slice_pos[1] : i_slice_pos[1]];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 2
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_3]);
      for v_cur in 1 .. v_len_1
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[v_cur : v_cur][i_slice_pos[2] : i_slice_pos[2]][ : ][i_slice_pos[1] : i_slice_pos[1]];
      end loop;
      return v_ret ;
    elsif i_dim_sliced[2] = 3
    then
      v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_2]);
      for v_cur in 1 .. v_len_1
      loop 
        v_ret[v_cur : v_cur][ : ] := i_array_4d[v_cur : v_cur][ : ][i_slice_pos[2] : i_slice_pos[2]][i_slice_pos[1] : i_slice_pos[1]];
      end loop;
      return v_ret ;
    end if;
  end if;
end;
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
-- sm_sc.fv_mx_slice_4d_2_2d
-- (
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]     ,
--           [11, 12, 13, 14, 15],
--           [21, 22, 23, 24, 25], 
--           [31, 32, 33, 34, 35] 
--         ],
--         [
--           [41, 42, 43, 44, 45]     ,
--           [51, 52, 53, 54, 55],
--           [61, 62, 63, 64, 65], 
--           [71, 72, 73, 74, 75] 
--         ],
--         [
--           [81, 82, 83, 84, 85]     ,
--           [91, 92, 93, 94, 95],
--           [101, 102, 103, 104, 105], 
--           [111, 112, 113, 114, 115] 
--         ]
--       ],
--       [
--         [
--           [-1, -2, -3, -4, -5]     ,
--           [-11, -12, -13, -14, -15],
--           [-21, -22, -23, -24, -25], 
--           [-31, -32, -33, -34, -35] 
--         ],
--         [
--           [-41, -42, -43, -44, -45]     ,
--           [-51, -52, -53, -54, -55],
--           [-61, -62, -63, -64, -65], 
--           [-71, -72, -73, -74, -75] 
--         ],
--         [
--           [-81, -82, -83, -84, -85]     ,
--           [-91, -92, -93, -94, -95],
--           [-101, -102, -103, -104, -105], 
--           [-111, -112, -113, -114, -115] 
--         ]
--       ]
--     ]
--   )
--   , array[4, 3]
--   , array[1, 2]
-- )
-- drop function if exists sm_sc.fv_mx_slice_4d_2_2d_py(float[], int[], int[]);
create or replace function sm_sc.fv_mx_slice_4d_2_2d_py
(
  i_array_4d        float[],  
  i_dim_sliced      int[]     ,          -- 被切片维度
  i_slice_pos       int[]                -- 被切片在 i_dim_sliced 各对应维度的位置序号。
)
returns float[]
as
$$
  import numpy as np
  if i_dim_sliced[0] == 1 :
    v_ret_3d = np.array(i_array_4d)[i_slice_pos[0] - 1, :: , :: , ::]
  elif i_dim_sliced[0] == 2 :
    v_ret_3d = np.array(i_array_4d)[:: , i_slice_pos[0] - 1, ::, ::]
  elif i_dim_sliced[0] == 3 :
    v_ret_3d = np.array(i_array_4d)[:: , :: , i_slice_pos[0] - 1, ::]
  elif i_dim_sliced[0] == 4 :
    v_ret_3d = np.array(i_array_4d)[:: , :: , :: , i_slice_pos[0] - 1]

  if i_dim_sliced[0] < i_dim_sliced[1] :
    i_dim_sliced[1] = i_dim_sliced[1] - 1

  if i_dim_sliced[1] == 1 :
    return v_ret_3d[i_slice_pos[1] - 1, :: , ::].tolist()
  elif i_dim_sliced[1] == 2 :
    return v_ret_3d[:: , i_slice_pos[1] - 1, ::].tolist()
  elif i_dim_sliced[1] == 3 :
    return v_ret_3d[:: , :: , i_slice_pos[1] - 1].tolist()
$$
language plpython3u stable
parallel safe;

-- -- set search_path to sm_sc;
-- select 
-- sm_sc.fv_mx_slice_4d_2_2d_py
-- (
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]     ,
--           [11, 12, 13, 14, 15],
--           [21, 22, 23, 24, 25], 
--           [31, 32, 33, 34, 35] 
--         ],
--         [
--           [41, 42, 43, 44, 45]     ,
--           [51, 52, 53, 54, 55],
--           [61, 62, 63, 64, 65], 
--           [71, 72, 73, 74, 75] 
--         ],
--         [
--           [81, 82, 83, 84, 85]     ,
--           [91, 92, 93, 94, 95],
--           [101, 102, 103, 104, 105], 
--           [111, 112, 113, 114, 115] 
--         ]
--       ],
--       [
--         [
--           [-1, -2, -3, -4, -5]     ,
--           [-11, -12, -13, -14, -15],
--           [-21, -22, -23, -24, -25], 
--           [-31, -32, -33, -34, -35] 
--         ],
--         [
--           [-41, -42, -43, -44, -45]     ,
--           [-51, -52, -53, -54, -55],
--           [-61, -62, -63, -64, -65], 
--           [-71, -72, -73, -74, -75] 
--         ],
--         [
--           [-81, -82, -83, -84, -85]     ,
--           [-91, -92, -93, -94, -95],
--           [-101, -102, -103, -104, -105], 
--           [-111, -112, -113, -114, -115] 
--         ]
--       ]
--     ]
--   )
--   , array[4, 3]
--   , array[1, 2]
-- )
-- drop function if exists sm_sc.fv_mx_slice_4d_2_3d(anyarray, int, int);
create or replace function sm_sc.fv_mx_slice_4d_2_3d
(
  i_array_4d        anyarray,  
  i_dim_sliced      int     ,                    -- 被切片维度
  i_slice_pos       int     default  1           -- 被切片位置序号。
)
returns anyarray
as
$$
declare
  v_ret      i_array_4d%type;
  v_len_1    int   :=   array_length(i_array_4d, 1);
  v_len_2    int   :=   array_length(i_array_4d, 2);
  v_len_3    int   :=   array_length(i_array_4d, 3);
  v_len_4    int   :=   array_length(i_array_4d, 4);
  v_cur_m    int   ;
  v_cur_n    int   ;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_dim_sliced not between 1 and 4
    then 
      raise exception 'i_dim_sliced is out of length.';
    elsif array_ndims(i_array_4d) <> 4
    then 
      raise exception 'ndims of i_array_4d should be 4.';
    elsif i_slice_pos not between 1 and array_length(i_array_4d, i_dim_sliced)
    then 
      raise exception 'i_slice_pos is out of length of i_array_4d.';
    end if;
  end if;

  if i_array_4d is null 
  then 
    return null;
  elsif i_dim_sliced = 1
  then
    v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_2, v_len_3, v_len_4]);
    for v_cur_m in 1 .. v_len_2
    loop 
      for v_cur_n in 1 .. v_len_2
      loop 
        v_ret[ : ][ : ][ : ] := i_array_4d[i_slice_pos : i_slice_pos][ : ][ : ][ : ];
      end loop;
    end loop;
    return v_ret ;
  elsif i_dim_sliced = 2
  then
    v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_3, v_len_4]);
    for v_cur_m in 1 .. v_len_1
    loop 
      for v_cur_n in 1 .. v_len_3
      loop 
        v_ret[v_cur_m : v_cur_m][v_cur_n : v_cur_n][ : ] := i_array_4d[v_cur_m : v_cur_m][i_slice_pos : i_slice_pos][v_cur_n : v_cur_n][ : ];
      end loop;
    end loop;
    return v_ret ;
  elsif i_dim_sliced = 3
  then
    v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_2, v_len_4]);
    for v_cur_m in 1 .. v_len_1
    loop 
      for v_cur_n in 1 .. v_len_2
      loop 
        v_ret[v_cur_m : v_cur_m][v_cur_n : v_cur_n][ : ] := i_array_4d[v_cur_m : v_cur_m][v_cur_n : v_cur_n][i_slice_pos : i_slice_pos][ : ];
      end loop;
    end loop;
    return v_ret ;
  elsif i_dim_sliced = 4
  then
    v_ret := array_fill(nullif(i_array_4d[1][1][1][1], i_array_4d[1][1][1][1]), array[v_len_1, v_len_2, v_len_3]);
    for v_cur_m in 1 .. v_len_1
    loop 
      for v_cur_n in 1 .. v_len_2
      loop 
        v_ret[v_cur_m : v_cur_m][v_cur_n : v_cur_n][ : ] := i_array_4d[v_cur_m : v_cur_m][v_cur_n : v_cur_n][ : ][i_slice_pos : i_slice_pos];
      end loop;
    end loop;
    return v_ret ;
  end if;
end;
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
-- sm_sc.fv_mx_slice_4d_2_3d
-- (
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]     ,
--           [11, 12, 13, 14, 15],
--           [21, 22, 23, 24, 25], 
--           [31, 32, 33, 34, 35] 
--         ],
--         [
--           [41, 42, 43, 44, 45]     ,
--           [51, 52, 53, 54, 55],
--           [61, 62, 63, 64, 65], 
--           [71, 72, 73, 74, 75] 
--         ],
--         [
--           [81, 82, 83, 84, 85]     ,
--           [91, 92, 93, 94, 95],
--           [101, 102, 103, 104, 105], 
--           [111, 112, 113, 114, 115] 
--         ]
--       ],
--       [
--         [
--           [-1, -2, -3, -4, -5]     ,
--           [-11, -12, -13, -14, -15],
--           [-21, -22, -23, -24, -25], 
--           [-31, -32, -33, -34, -35] 
--         ],
--         [
--           [-41, -42, -43, -44, -45]     ,
--           [-51, -52, -53, -54, -55],
--           [-61, -62, -63, -64, -65], 
--           [-71, -72, -73, -74, -75] 
--         ],
--         [
--           [-81, -82, -83, -84, -85]     ,
--           [-91, -92, -93, -94, -95],
--           [-101, -102, -103, -104, -105], 
--           [-111, -112, -113, -114, -115] 
--         ]
--       ]
--     ]
--   )
--   , 4, 2
-- )
-- drop function if exists sm_sc.fv_mx_slice_4d_2_3d_py(float[], int, int);
create or replace function sm_sc.fv_mx_slice_4d_2_3d_py
(
  i_array_4d        float[],  
  i_dim_sliced      int     ,               -- 被切片维度
  i_slice_pos       int     default  1      -- 被切片位置序号
)
returns float[]
as
$$
  import numpy as np
  if i_dim_sliced == 1 :
    return np.array(i_array_4d)[i_slice_pos - 1, :: , :: , ::].tolist()
  elif i_dim_sliced == 2 :
    return np.array(i_array_4d)[:: , i_slice_pos - 1, ::, ::].tolist()
  elif i_dim_sliced == 3 :
    return np.array(i_array_4d)[:: , :: , i_slice_pos - 1, ::].tolist()
  elif i_dim_sliced == 4 :
    return np.array(i_array_4d)[:: , :: , :: , i_slice_pos - 1].tolist()

$$
language plpython3u stable
parallel safe;

-- -- set search_path to sm_sc;
-- select 
-- sm_sc.fv_mx_slice_4d_2_3d
-- (
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]     ,
--           [11, 12, 13, 14, 15],
--           [21, 22, 23, 24, 25], 
--           [31, 32, 33, 34, 35] 
--         ],
--         [
--           [41, 42, 43, 44, 45]     ,
--           [51, 52, 53, 54, 55],
--           [61, 62, 63, 64, 65], 
--           [71, 72, 73, 74, 75] 
--         ],
--         [
--           [81, 82, 83, 84, 85]     ,
--           [91, 92, 93, 94, 95],
--           [101, 102, 103, 104, 105], 
--           [111, 112, 113, 114, 115] 
--         ]
--       ],
--       [
--         [
--           [-1, -2, -3, -4, -5]     ,
--           [-11, -12, -13, -14, -15],
--           [-21, -22, -23, -24, -25], 
--           [-31, -32, -33, -34, -35] 
--         ],
--         [
--           [-41, -42, -43, -44, -45]     ,
--           [-51, -52, -53, -54, -55],
--           [-61, -62, -63, -64, -65], 
--           [-71, -72, -73, -74, -75] 
--         ],
--         [
--           [-81, -82, -83, -84, -85]     ,
--           [-91, -92, -93, -94, -95],
--           [-101, -102, -103, -104, -105], 
--           [-111, -112, -113, -114, -115] 
--         ]
--       ]
--     ]
--   )
--   , 4, 2
-- )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_new(anynonarray, int[]);
create or replace function sm_sc.fv_new
(
  i_meta_val                anynonarray    ,
  i_dims_repeat_times       int[]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  if array_ndims(i_dims_repeat_times) > 1
  then
    raise exception 'no method!';
  elsif array_ndims(i_dims_repeat_times) is null
  then 
    return array[] :: i_meta_val%type;
  else
    return array_fill(i_meta_val, i_dims_repeat_times);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_new
--   (
--     12.3,
--     array[5, 6]
--   );

-- drop function if exists sm_sc.fv_new(anyarray, int[]);
create or replace function sm_sc.fv_new
(
  i_meta_val     anyarray    ,
  i_dims_times        int[]
)
returns anyarray
as
$$
declare 
  v_ret     i_meta_val%type      :=   array_fill(nullif(i_meta_val[1], i_meta_val[1]), (select array_agg(array_length(i_meta_val, a_no)) from generate_series(1, array_ndims(i_meta_val)) tb_a(a_no)) * i_dims_times);
  v_cur_1   int                  ;
  v_cur_2   int                  ;
  v_cur_3   int                  ;
  v_cur_4   int                  ;
  v_cur_5   int                  ;
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_ndims(i_meta_val) not between 1 and 5
      or array_ndims(i_dims_times) <> 1 
      or array_length(i_dims_times, 1) > array_ndims(i_meta_val)
    then
      raise exception 'no method!';
    end if;
  end if;
  
  if array_ndims(i_meta_val) is null
  then 
    return array[] :: i_meta_val%type;
  elsif array_ndims(i_meta_val) = 1
  then
    -- -- return (select sm_sc.fa_array_concat(i_meta_val) from generate_series(1, coalesce(i_dims_times[1], 1)) tb_a);
    for v_cur_1 in 1 .. coalesce(i_dims_times[1], 0)
    loop 
      v_ret[array_length(i_meta_val, 1) * (v_cur_1 - 1) + 1 : array_length(i_meta_val, 1) * v_cur_1]   
        := i_meta_val;
    end loop;
    
  elsif array_ndims(i_meta_val) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_y as 
    -- --   (
    -- --     select 
    -- --       sm_sc.fa_mx_concat_x(i_meta_val) as a_x_meta_val
    -- --     from generate_series(1, coalesce(i_dims_times[2], 0)) tb_a_x(a_x_no)
    -- --   )
    -- --   select 
    -- --     sm_sc.fa_mx_concat_y(a_x_meta_val) 
    -- --   from cte_y, generate_series(1, coalesce(i_dims_times[1], 0)) tb_a_y(a_y_no)
    -- -- );
    for v_cur_1 in 1 .. coalesce(i_dims_times[1], 0)
    loop 
      for v_cur_2 in 1 .. coalesce(i_dims_times[2], 0)
      loop 
        v_ret[array_length(i_meta_val, 1) * (v_cur_1 - 1) + 1 : array_length(i_meta_val, 1) * v_cur_1]   
             [array_length(i_meta_val, 2) * (v_cur_2 - 1) + 1 : array_length(i_meta_val, 2) * v_cur_2]
          := i_meta_val;
      end loop;
    end loop;
    
  elsif array_ndims(i_meta_val) = 3
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_x as 
    -- --   (
    -- --     select 
    -- --       sm_sc.fa_mx_concat_x3(i_meta_val) as a_x_meta_val
    -- --     from generate_series(1, coalesce(i_dims_times[3], 0)) tb_a_x3(a_x3_no)
    -- --   ),
    -- --   cte_y_x as 
    -- --   (
    -- --     select 
    -- --       sm_sc.fa_mx_concat_x(a_x_meta_val) as a_y_x_meta_val
    -- --     from cte_x, generate_series(1, coalesce(i_dims_times[2], 0)) tb_a_x(a_x_no)
    -- --   )
    -- --   select 
    -- --     sm_sc.fa_mx_concat_y(a_y_x_meta_val) 
    -- --   from cte_y_x, generate_series(1, coalesce(i_dims_times[1], 0)) tb_a_y(a_y_no)
    -- -- );
    for v_cur_1 in 1 .. coalesce(i_dims_times[1], 0)
    loop 
      for v_cur_2 in 1 .. coalesce(i_dims_times[2], 0)
      loop 
        for v_cur_3 in 1 .. coalesce(i_dims_times[3], 0)
        loop 
          v_ret[array_length(i_meta_val, 1) * (v_cur_1 - 1) + 1 : array_length(i_meta_val, 1) * v_cur_1]   
               [array_length(i_meta_val, 2) * (v_cur_2 - 1) + 1 : array_length(i_meta_val, 2) * v_cur_2] 
               [array_length(i_meta_val, 3) * (v_cur_3 - 1) + 1 : array_length(i_meta_val, 3) * v_cur_3]
            := i_meta_val;
        end loop;
      end loop;
    end loop;
    
  elsif array_ndims(i_meta_val) = 4
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_x3 as 
    -- --   (
    -- --     select 
    -- --       sm_sc.fa_mx_concat_x4(i_meta_val) as a_x3_meta_val
    -- --     from generate_series(1, coalesce(i_dims_times[4], 0)) tb_a_x4(a_x4_no)
    -- --   ),
    -- --   cte_x_x3 as 
    -- --   (
    -- --     select 
    -- --       sm_sc.fa_mx_concat_x3(a_x3_meta_val) as a_x_x3_meta_val
    -- --     from cte_x3, generate_series(1, coalesce(i_dims_times[3], 0)) tb_a_x3(a_x3_no)
    -- --   ),
    -- --   cte_y_x_x3 as 
    -- --   (
    -- --     select 
    -- --       sm_sc.fa_mx_concat_x(a_x_x3_meta_val) as a_y_x_x3_meta_val
    -- --     from cte_x_x3, generate_series(1, coalesce(i_dims_times[2], 0)) tb_a_x(a_x_no)
    -- --   )
    -- --   select 
    -- --     sm_sc.fa_mx_concat_y(a_y_x_x3_meta_val) 
    -- --   from cte_y_x_x3, generate_series(1, coalesce(i_dims_times[1], 0)) tb_a_y(a_y_no)
    -- -- );
    for v_cur_1 in 1 .. coalesce(i_dims_times[1], 0)
    loop 
      for v_cur_2 in 1 .. coalesce(i_dims_times[2], 0)
      loop 
        for v_cur_3 in 1 .. coalesce(i_dims_times[3], 0)
        loop 
          for v_cur_4 in 1 .. coalesce(i_dims_times[4], 0)
          loop 
            v_ret[array_length(i_meta_val, 1) * (v_cur_1 - 1) + 1 : array_length(i_meta_val, 1) * v_cur_1]   
                 [array_length(i_meta_val, 2) * (v_cur_2 - 1) + 1 : array_length(i_meta_val, 2) * v_cur_2] 
                 [array_length(i_meta_val, 3) * (v_cur_3 - 1) + 1 : array_length(i_meta_val, 3) * v_cur_3]
                 [array_length(i_meta_val, 4) * (v_cur_4 - 1) + 1 : array_length(i_meta_val, 4) * v_cur_4]
              := i_meta_val;
          end loop;
        end loop;
      end loop;
    end loop;
    
  elsif array_ndims(i_meta_val) = 5
  then
    for v_cur_1 in 1 .. coalesce(i_dims_times[1], 0)
    loop 
      for v_cur_2 in 1 .. coalesce(i_dims_times[2], 0)
      loop 
        for v_cur_3 in 1 .. coalesce(i_dims_times[3], 0)
        loop 
          for v_cur_4 in 1 .. coalesce(i_dims_times[4], 0)
          loop 
            for v_cur_5 in 1 .. coalesce(i_dims_times[5], 0)
            loop 
              v_ret[array_length(i_meta_val, 1) * (v_cur_1 - 1) + 1 : array_length(i_meta_val, 1) * v_cur_1]   
                   [array_length(i_meta_val, 2) * (v_cur_2 - 1) + 1 : array_length(i_meta_val, 2) * v_cur_2] 
                   [array_length(i_meta_val, 3) * (v_cur_3 - 1) + 1 : array_length(i_meta_val, 3) * v_cur_3]
                   [array_length(i_meta_val, 4) * (v_cur_4 - 1) + 1 : array_length(i_meta_val, 4) * v_cur_4]
                   [array_length(i_meta_val, 5) * (v_cur_5 - 1) + 1 : array_length(i_meta_val, 5) * v_cur_5]
                := i_meta_val;
            end loop;
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_new
--   (
--     array[12.3, 156.6, 7.8, -93.6],
--     array[5]
--   );
-- select sm_sc.fv_new
--   (
--     array[[12.3, 156.6, 7.8, -93.6]],
--     array[5, 2]
--   );
-- select sm_sc.fv_new
--   (
--     array[[12.3, 3.3], [156.6, 3.3], [7.8, 3.3], [-93.6, 3.3]],
--     array[5, 2]
--   );
-- select sm_sc.fv_new
--   (
--     array[[12.3]],
--     array[5, 2]
--   );
-- select sm_sc.fv_new
--   (
--     array[[[1.2, 3.4],[5.6, 7.8]],[[-1.2, -3.4],[-5.6, -7.8]]],
--     array[4, 2, 3]
--   );
-- select sm_sc.fv_new
--   (
--     array[[[[1.2, 3.4],[5.6, 7.8]],[[-1.2, -3.4],[-5.6, -7.8]]],[[[0.2, 0.4],[0.6, 0.8]],[[-0.2, -0.4],[-0.6, -0.8]]]],
--     array[4, 2, 3, 2]
--   );
-- select sm_sc.fv_new
--   (
--     array[[[[[1.2, 3.4],[5.6, 7.8]],[[-1.2, -3.4],[-5.6, -7.8]]],[[[0.2, 0.4],[0.6, 0.8]],[[-0.2, -0.4],[-0.6, -0.8]]]],[[[[1.2, 3.4],[5.6, 7.8]],[[-1.2, -3.4],[-5.6, -7.8]]],[[[0.2, 0.4],[0.6, 0.8]],[[-0.2, -0.4],[-0.6, -0.8]]]]]
--   , array[4, 2, 3, 2, 2]
--   );
-- set search_path to sm_sc;
-- -- -- drop function if exists sm_sc.fv_new_rand(int[]);
-- -- create or replace function sm_sc.fv_new_rand
-- -- (
-- --   i_dims_len       int[]
-- -- )
-- -- returns float[]
-- -- as
-- -- $$
-- -- declare 
-- --   v_ret    float[]   ;
-- --   v_cur_y  int       ;
-- --   v_cur_x  int       ;
-- --   v_cur_x3 int       ;
-- --   v_cur_x4 int       ;
-- --   v_cur_x5 int       ;
-- -- begin
-- --   -- set search_path to sm_sc;
-- --   if current_setting('pg4ml._v_is_debug_check', true) = '1'
-- --   then
-- --     if array_ndims(i_dims_len) <> 1 
-- --       or array_length(i_dims_len, 1) not between 1 and 5
-- --       or 1 > any(i_dims_len)
-- --     then
-- --       raise exception 'unsupport ndims or length!';
-- --     end if;
-- --   end if;
-- -- 
-- --   if array_length(i_dims_len, 1) = 1
-- --   then 
-- --     return 
-- --     (
-- --       select 
-- --         array_agg(random())
-- --       from generate_series(1, i_dims_len[1])
-- --     )
-- --     ;
-- --   elsif array_length(i_dims_len, 1) = 2
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         v_ret[v_cur_y][v_cur_x] = random();
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   elsif array_length(i_dims_len, 1) = 3
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         for v_cur_x3 in 1 .. i_dims_len[3]
-- --         loop 
-- --           v_ret[v_cur_y][v_cur_x][v_cur_x3] = random();
-- --         end loop;
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   elsif array_length(i_dims_len, 1) = 4
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         for v_cur_x3 in 1 .. i_dims_len[3]
-- --         loop 
-- --           for v_cur_x4 in 1 .. i_dims_len[4]
-- --           loop 
-- --             v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] = random();
-- --           end loop;
-- --         end loop;
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   elsif array_length(i_dims_len, 1) = 5
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         for v_cur_x3 in 1 .. i_dims_len[3]
-- --         loop 
-- --           for v_cur_x4 in 1 .. i_dims_len[4]
-- --           loop 
-- --             for v_cur_x5 in 1 .. i_dims_len[5]
-- --             loop 
-- --               v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4][v_cur_x5] = random();
-- --             end loop;
-- --           end loop;
-- --         end loop;
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   end if;
-- -- end
-- -- $$
-- -- language plpgsql volatile
-- -- parallel safe
-- -- cost 100;

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_new_rand(int[]);
create or replace function sm_sc.fv_new_rand
(
  i_dims_len          int[]
)
returns float[]
as
$$
  import numpy as np 
  return (np.random.rand(np.prod(i_dims_len))).reshape(i_dims_len).tolist()
$$
language plpython3u stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_new_rand
--   (
--     array[5, 6]
--   );

-- ------------------------------------------------------------------------

-- drop function if exists sm_sc.fv_new_rand(float, int[]);
create or replace function sm_sc.fv_new_rand
(
  i_meta_val     float    ,
  i_dims_len     int[]
)
returns float[]
as
$$
-- declare 
begin
  return i_meta_val *` sm_sc.fv_new_rand(i_dims_len);
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_new_rand
--   (
--     12.3,
--     array[5, 6]
--   );
-- ------------------------------------------------------------------------



-- drop function if exists sm_sc.fv_new_rand(numrange, int[]);
create or replace function sm_sc.fv_new_rand
(
  i_meta_range   numrange    ,
  i_dims_len     int[]
)
returns float[]
as
$$
-- declare 
begin
  return lower(i_meta_range) :: float +` ((upper(i_meta_range) :: float - lower(i_meta_range) :: float) *` sm_sc.fv_new_rand(i_dims_len));
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_new_rand
--   (
--     numrange(1.2, 4.5, '[]'),
--     array[5, 6]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_new_rand_enum(anyarray, int[2]);
create or replace function sm_sc.fv_new_rand_enum
(
  i_dispersed_array   anyarray    ,
  i_dims_len            int[]
)
returns anyarray
as
$$
declare -- here
  v_enum_dic    name[]  :=  
  (
    select 
      array_agg(distinct enumlabel)
    from unnest(i_dispersed_array) tb_a_enum(enumlabel)
  );
  v_dic_len     int      :=  array_length(v_enum_dic, 1);
  v_ret    i_dispersed_array%type   ;
  v_cur_y  int       ;
  v_cur_x  int       ;
  v_cur_x3 int       ;
  v_cur_x4 int       ;
  v_cur_x5 int       ;
begin
  -- set search_path to sm_sc;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dims_len) <> 1 
      or array_length(i_dims_len, 1) not between 1 and 5
      or 1 <= any(i_dims_len)
    then
      raise exception 'unsupport ndims or length!';
    end if;
  end if;

  if array_length(i_dims_len, 1) = 1
  then
    return 
    (
      select 
        array_agg(v_enum_dic[round(random() * v_dic_len + 0.5 :: float) :: int])
      from generate_series(1, i_dims_len[1])
    );
  elsif array_length(i_dims_len, 1) = 2
  then 
    v_ret := array_fill(null :: float, i_dims_len);
    for v_cur_y in 1 .. i_dims_len[1]
    loop 
      for v_cur_x in 1 .. i_dims_len[2]
      loop 
        v_ret[v_cur_y][v_cur_x] = v_enum_dic[round(random() * v_dic_len + 0.5 :: float) :: int];
      end loop;
    end loop;
    return v_ret;
  elsif array_length(i_dims_len, 1) = 3
  then 
    v_ret := array_fill(null :: float, i_dims_len);
    for v_cur_y in 1 .. i_dims_len[1]
    loop 
      for v_cur_x in 1 .. i_dims_len[2]
      loop 
        for v_cur_x3 in 1 .. i_dims_len[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] = v_enum_dic[round(random() * v_dic_len + 0.5 :: float) :: int];
        end loop;
      end loop;
    end loop;
    return v_ret;
  elsif array_length(i_dims_len, 1) = 4
  then 
    v_ret := array_fill(null :: float, i_dims_len);
    for v_cur_y in 1 .. i_dims_len[1]
    loop 
      for v_cur_x in 1 .. i_dims_len[2]
      loop 
        for v_cur_x3 in 1 .. i_dims_len[3]
        loop 
          for v_cur_x4 in 1 .. i_dims_len[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] = v_enum_dic[round(random() * v_dic_len + 0.5 :: float) :: int];
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  elsif array_length(i_dims_len, 1) = 5
  then 
    v_ret := array_fill(null :: float, i_dims_len);
    for v_cur_y in 1 .. i_dims_len[1]
    loop 
      for v_cur_x in 1 .. i_dims_len[2]
      loop 
        for v_cur_x3 in 1 .. i_dims_len[3]
        loop 
          for v_cur_x4 in 1 .. i_dims_len[4]
          loop 
            for v_cur_x5 in 1 .. i_dims_len[5]
            loop 
              v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4][v_cur_x5] = v_enum_dic[round(random() * v_dic_len + 0.5 :: float) :: int];
            end loop;
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  else
    raise exception 'unsupport ndims!';
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_new_rand_enum
--   (
--     array['abc', 'bcd', 'cde', 'def', 'efg', 'fgh', 'efg', 'fgh', 'cde', 'cde', 'cde'],
--     array[5, 6]
--   );
-- select sm_sc.fv_new_rand_enum
--   (
--     array['abc', 'bcd', 'cde', 'def', 'efg', 'fgh', 'efg', 'fgh', 'cde', 'cde', 'cde'],
--     array[5]
--   );
-- -- -- set search_path to sm_sc;
-- -- -- drop function if exists sm_sc.fv_new_randn(float, float, int[]);
-- -- create or replace function sm_sc.fv_new_randn
-- -- (
-- --   i_meta_val_mean     float    ,
-- --   i_meta_val_stddev   float    ,
-- --   i_dims_len          int[]
-- -- )
-- -- returns float[]
-- -- as
-- -- $$
-- -- declare 
-- --   v_ret    float[]   ;
-- --   v_cur_y  int       ;
-- --   v_cur_x  int       ;
-- --   v_cur_x3 int       ;
-- --   v_cur_x4 int       ;
-- --   v_cur_x5 int       ;
-- -- begin
-- --   -- set search_path to sm_sc;
-- --   if current_setting('pg4ml._v_is_debug_check', true) = '1'
-- --   then
-- --     if array_ndims(i_dims_len) <> 1 
-- --       or array_length(i_dims_len, 1) not between 1 and 5
-- --       or 1 <= any(i_dims_len)
-- --     then
-- --       raise exception 'unsupport ndims or length!';
-- --     end if;
-- --   end if;
-- -- 
-- --   if array_length(i_dims_len, 1) = 1
-- --   then 
-- --     return 
-- --     (
-- --       select 
-- --         array_agg(a_randn)
-- --       from normal_rand(i_dims_len[1], i_meta_val_mean, i_meta_val_stddev) a_randn
-- --     )
-- --     ;
-- --   elsif array_length(i_dims_len, 1) = 2
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         v_ret[v_cur_y][v_cur_x] = normal_rand(1, i_meta_val_mean, i_meta_val_stddev);
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   elsif array_length(i_dims_len, 1) = 3
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         for v_cur_x3 in 1 .. i_dims_len[3]
-- --         loop 
-- --           v_ret[v_cur_y][v_cur_x][v_cur_x3] = normal_rand(1, i_meta_val_mean, i_meta_val_stddev);
-- --         end loop;
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   elsif array_length(i_dims_len, 1) = 4
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         for v_cur_x3 in 1 .. i_dims_len[3]
-- --         loop 
-- --           for v_cur_x4 in 1 .. i_dims_len[4]
-- --           loop 
-- --             v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] = normal_rand(1, i_meta_val_mean, i_meta_val_stddev);
-- --           end loop;
-- --         end loop;
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   elsif array_length(i_dims_len, 1) = 5
-- --   then 
-- --     v_ret := array_fill(null :: float, i_dims_len);
-- --     for v_cur_y in 1 .. i_dims_len[1]
-- --     loop 
-- --       for v_cur_x in 1 .. i_dims_len[2]
-- --       loop 
-- --         for v_cur_x3 in 1 .. i_dims_len[3]
-- --         loop 
-- --           for v_cur_x4 in 1 .. i_dims_len[4]
-- --           loop 
-- --             for v_cur_x5 in 1 .. i_dims_len[5]
-- --             loop 
-- --               v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4][v_cur_x5] = normal_rand(1, i_meta_val_mean, i_meta_val_stddev);
-- --             end loop;
-- --           end loop;
-- --         end loop;
-- --       end loop;
-- --     end loop;
-- --     return v_ret;
-- --   end if;
-- -- end
-- -- $$
-- -- language plpgsql volatile
-- -- parallel safe
-- -- cost 100;

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_new_randn(float, float, int[]);
create or replace function sm_sc.fv_new_randn
(
  i_meta_val_mean     float    ,
  i_meta_val_stddev   float    ,
  i_dims_len          int[]
)
returns float[]
as
$$
  import numpy as np 
  return (np.random.normal(i_meta_val_mean, i_meta_val_stddev, np.prod(i_dims_len))).reshape(i_dims_len).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_new_randn
--   (
--     12.3,
--     2.0 :: float,
--     array[5, 6]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_nullif(anyarray, anyarray);
create or replace function sm_sc.fv_nullif
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] + []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] + array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return i_left;
  
  -- [] + []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      i_left[v_y_cur] := nullif(i_left[v_y_cur], i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][] + [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        i_left[v_y_cur][v_x_cur] := nullif(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;
    
  -- [][][] + [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = nullif(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  -- [][][][] + [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = nullif(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_nullif
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_nullif
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[32.5], array[-9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5], array[-19.1]]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[32.5, -9.1],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[12.5, -19.1]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[32.5, -9.1]],
--     array[array[-12.3, 12.3], array[-45.6, 45.6]]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]],
--     array[array[12.5, -19.1]]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.fv_nullif
--   (
--     array[1.2, 2.3]::float[],
--     array[2.1, 3.2]::float[]
--   );
-- select sm_sc.fv_nullif(array[1], array[1,2,3]);
-- select sm_sc.fv_nullif(array[1], array[array[1,2,3]]);
-- select sm_sc.fv_nullif(array[array[1]], array[1,2,3]);
-- select sm_sc.fv_nullif(array[array[1]], array[array[1,2,3]]);

-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_nullif
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.fv_nullif
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- select 
--   sm_sc.fv_nullif
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , sm_sc.fv_new_rand(array[   1, 4, 5]) 
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_nullif(anyarray, anyelement);
create or replace function sm_sc.fv_nullif
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return i_left;
  -- -- elsif i_right is null
  -- -- then
  -- --   return null::anyarray;

  -- [][] + float
  elsif array_ndims(i_left) =  2
  then
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        i_left[v_y_cur][v_x_cur] := nullif(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [] + float
  elsif array_ndims(i_left) =  1
  then
    while v_y_cur <= array_length(i_left, 1)
    loop
      i_left[v_y_cur] := nullif(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_left;

  -- [][][] + float
  elsif array_ndims(i_left) =  3
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          i_left[v_y_cur][v_x_cur][v_x3_cur] = nullif(i_left[v_y_cur][v_x_cur][v_x3_cur], i_right);
        end loop;    
      end loop;
    end loop;
    return i_left;
  
  -- [][][][] + float
  elsif array_ndims(i_left) = 4
  then
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = nullif(i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_right);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_left;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_nullif(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.fv_nullif(array[]::float[], 8.8)
-- select sm_sc.fv_nullif(array[]::float[], null)
-- select sm_sc.fv_nullif(array[1.2, 2.3]::float[], 8.8)
-- select sm_sc.fv_nullif(array[array[], array []]::float[], 8.8)
-- select sm_sc.fv_nullif(array[1.2, 2.3]::float[], null::float)
-- select 
--   sm_sc.fv_nullif
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.fv_nullif
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_nullif(anyelement, anyarray);
create or replace function sm_sc.fv_nullif
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
begin
  -- null :: float[][] + float = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return i_right;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- float + [][]
  elsif array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := nullif(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- float + []
  elsif array_ndims(i_right) =  1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := nullif(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;
    
  -- float + [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = nullif(i_left, i_right[v_y_cur][v_x_cur][v_x3_cur]);
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- float + [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = nullif(i_left, i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]);
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_nullif(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.fv_nullif(8.8, array[]::float[] )
-- select sm_sc.fv_nullif(null, array[]::float[])
-- select sm_sc.fv_nullif(8.8, array[1.2, 2.3]::float[] )
-- select sm_sc.fv_nullif(8.8, array[array[], array []]::float[])
-- select sm_sc.fv_nullif(null::float, array[1.2, 2.3]::float[] )
-- select 
--   sm_sc.fv_nullif
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_nullif
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i http://pypi.douban.com/simple/ --trusted-host pypi.douban.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_reshape(float[], int[]);
create or replace function sm_sc.fv_opr_reshape_py
(
  i_arr           float[]
, i_new_shape     int[]
)
returns float[]
as
$$
  import numpy as np
  return np.reshape(i_arr, i_new_shape).tolist()
$$
language plpython3u stable
parallel safe
;

-- select 
--   sm_sc.fv_opr_reshape_py
--   (
--     array[[1.2, 2.4, 3.5], [1.6, 2.5, 3.9]] :: float[]
--   , array[2, 3]
--   )

-- select 
--   array_dims
--   (
--     sm_sc.fv_opr_reshape_py
--     (
--       sm_sc.fv_new_rand(array[2, 3, 4])
--     , array[2, 1, 6, 2]  -- array[4, 2, 3]
--     )
--   )

-- select 
--   array_dims
--   (
--     sm_sc.fv_opr_reshape_py
--     (
--       sm_sc.fv_new_rand(array[2, 3, 4, 5])
--     , array[5, 4, 3, 2]   -- array[4, 10, 3]
--     )
--   )

-- drop function if exists sm_sc.fv_pos_replaces(anyarray, int[], anyelement);
create or replace function sm_sc.fv_pos_replaces
(
  i_sour_arr     anyarray,
  i_pos_s        int[],
  i_tar_ele      anyelement
)
returns anyarray
as
$$
declare -- here
  v_cur   int;
begin
  -- 审查
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_sour_arr) > 4
    then
      raise exception 'unsupport ndims of i_sour_arr.';
    elsif array_ndims(i_pos_s) > 2
      or array_ndims(i_sour_arr) = 1 and array_ndims(i_sour_arr) > 1
    then 
      raise exception 'unsupport ndims of i_pos_s.';
    elsif array_length(i_pos_s, 2) <> array_ndims(i_sour_arr)
      or array_ndims(i_pos_s) = 1 and array_length(i_pos_s, 1) <> array_ndims(i_sour_arr)
    then 
      raise exception 'unsupport length of i_pos_s.';
    end if;
  end if;
  
  -- 1d
  if array_ndims(i_sour_arr) = 1
  then
    foreach v_cur in array i_pos_s
    loop
      i_sour_arr[v_cur] := i_tar_ele;
    end loop;
  end if;

  -- 2d
  if array_ndims(i_sour_arr) = 2
  then
    for v_cur in 1 .. array_length(i_pos_s, 1)
    loop
      i_sour_arr[i_pos_s[v_cur][1]][i_pos_s[v_cur][2]] := i_tar_ele;
    end loop;
  elsif array_ndims(i_sour_arr) = 3
  then
    for v_cur in 1 .. array_length(i_pos_s, 1)
    loop
      i_sour_arr[i_pos_s[v_cur][1]][i_pos_s[v_cur][2]][i_pos_s[v_cur][3]] := i_tar_ele;
    end loop;
  elsif array_ndims(i_sour_arr) = 4
  then
    for v_cur in 1 .. array_length(i_pos_s, 1)
    loop
      i_sour_arr[i_pos_s[v_cur][1]][i_pos_s[v_cur][2]][i_pos_s[v_cur][3]][i_pos_s[v_cur][4]] := i_tar_ele;
    end loop;
  end if;

  return i_sour_arr;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_pos_replaces(array['a', 'b', 'c'], array[1, 3], 'd')
-- select sm_sc.fv_pos_replaces(array[['a', 'b', 'c'], ['e', 'f', 'g']], array[[1, 3], [2, 2]], 'd')
-- select sm_sc.fv_pos_replaces(array[[['a', 'b', 'c'], ['e', 'f', 'g']], [['a', 'b', 'c'], ['e', 'f', 'g']]], array[[1, 2, 1], [2, 2, 2]], 'd')
-- select sm_sc.fv_pos_replaces(array[[[['a', 'b', 'c'], ['e', 'f', 'g']], [['a', 'b', 'c'], ['e', 'f', 'g']]], [[['a', 'b', 'c'], ['e', 'f', 'g']], [['a', 'b', 'c'], ['e', 'f', 'g']]]], array[[1, 2, 1, 2], [2, 2, 2, 1]], 'd')

-- ------------------------------------------------------------------
-- drop function if exists sm_sc.fv_pos_replaces(anyarray, int4range[], anyelement);
create or replace function sm_sc.fv_pos_replaces
(
  i_sour_arr     anyarray      ,
  i_pos_range_s  int4range[]   ,
  i_tar_ele      anyelement
)
returns anyarray
as
$$
declare -- here
  v_cur   int;
begin
  -- 审查
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_sour_arr) > 4
    then
      raise exception 'unsupport ndims of i_sour_arr.';
    elsif array_ndims(i_pos_range_s) > 2
      or array_ndims(i_sour_arr) = 1 and array_ndims(i_sour_arr) > 1
    then 
      raise exception 'unsupport ndims of i_pos_range_s.';
    elsif array_length(i_pos_range_s, 2) <> array_ndims(i_sour_arr)
      or array_ndims(i_pos_range_s) = 1 and array_length(i_pos_range_s, 1) <> array_ndims(i_sour_arr)
    then 
      raise exception 'unsupport length of i_pos_range_s.';
    end if;
  end if;
  
  -- 1d
  if array_ndims(i_sour_arr) = 1
  then
    for v_cur in 1 .. array_length(i_pos_range_s, 1)
    loop
      if i_pos_range_s[v_cur] is not null 
      then
        i_sour_arr
          [lower(i_pos_range_s[v_cur]) : upper(i_pos_range_s[v_cur]) -1] 
        := array_fill
           (
             i_tar_ele
           , array[upper(i_pos_range_s[v_cur]) - lower(i_pos_range_s[v_cur])]
           )
        ;
      end if;
    end loop;
  end if;

  -- 2d
  if array_ndims(i_sour_arr) = 2
  then
    for v_cur in 1 .. array_length(i_pos_range_s, 1)
    loop
      if i_pos_range_s[v_cur][1] is not null
        and i_pos_range_s[v_cur][2] is not null 
      then
        i_sour_arr
          [lower(i_pos_range_s[v_cur][1]) : upper(i_pos_range_s[v_cur][1]) -1]
          [lower(i_pos_range_s[v_cur][2]) : upper(i_pos_range_s[v_cur][2]) -1]  
        := array_fill
           (
             i_tar_ele
           , array
             [
               upper(i_pos_range_s[v_cur][1]) - lower(i_pos_range_s[v_cur][1])
             , upper(i_pos_range_s[v_cur][2]) - lower(i_pos_range_s[v_cur][2])
             ]
           )
        ;
      end if;
    end loop;
  elsif array_ndims(i_sour_arr) = 3
  then
    for v_cur in 1 .. array_length(i_pos_range_s, 1)
    loop
      if i_pos_range_s[v_cur][1] is not null
        and i_pos_range_s[v_cur][2] is not null 
        and i_pos_range_s[v_cur][3] is not null 
      then
        i_sour_arr
          [lower(i_pos_range_s[v_cur][1]) : upper(i_pos_range_s[v_cur][1]) -1]
          [lower(i_pos_range_s[v_cur][2]) : upper(i_pos_range_s[v_cur][2]) -1]
          [lower(i_pos_range_s[v_cur][3]) : upper(i_pos_range_s[v_cur][3]) -1]  
        := array_fill
           (
             i_tar_ele
           , array
             [
               upper(i_pos_range_s[v_cur][1]) - lower(i_pos_range_s[v_cur][1])
             , upper(i_pos_range_s[v_cur][2]) - lower(i_pos_range_s[v_cur][2])
             , upper(i_pos_range_s[v_cur][3]) - lower(i_pos_range_s[v_cur][3])
             ]
           )
        ;
      end if;
    end loop;
  elsif array_ndims(i_sour_arr) = 4
  then
    for v_cur in 1 .. array_length(i_pos_range_s, 1)
    loop
      if i_pos_range_s[v_cur][1] is not null
        and i_pos_range_s[v_cur][2] is not null 
        and i_pos_range_s[v_cur][3] is not null 
        and i_pos_range_s[v_cur][4] is not null 
      then
        i_sour_arr
          [lower(i_pos_range_s[v_cur][1]) : upper(i_pos_range_s[v_cur][1]) -1]
          [lower(i_pos_range_s[v_cur][2]) : upper(i_pos_range_s[v_cur][2]) -1]
          [lower(i_pos_range_s[v_cur][3]) : upper(i_pos_range_s[v_cur][3]) -1]
          [lower(i_pos_range_s[v_cur][4]) : upper(i_pos_range_s[v_cur][4]) -1]  
        := array_fill
           (
             i_tar_ele
           , array
             [
               upper(i_pos_range_s[v_cur][1]) - lower(i_pos_range_s[v_cur][1])
             , upper(i_pos_range_s[v_cur][2]) - lower(i_pos_range_s[v_cur][2])
             , upper(i_pos_range_s[v_cur][3]) - lower(i_pos_range_s[v_cur][3])
             , upper(i_pos_range_s[v_cur][4]) - lower(i_pos_range_s[v_cur][4])
             ]
           )
        ;
      end if;
    end loop;
  end if;

  return i_sour_arr;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_pos_replaces
--   (
--     array['a', 'b', 'c', 'e', 'f', 'g', 'h']
--   , array[int4range(1, 2, '[]'), int4range(4, 6, '[]')]
--   , 'd'
--   )

-- select 
--   sm_sc.fv_pos_replaces
--   (
--     array
--     [
--       ['a', 'b', 'c', 'e', 'f', 'g', 'h']
--     , ['a1', 'b1', 'c1', 'e1', 'f1', 'g1', 'h1']
--     ]
--   , array
--     [
--       [int4range(1, 2, '[]'), int4range(1, 2, '[]')]
--     , [int4range(2, 2, '[]'), int4range(4, 6, '[]')]
--     ]
--   , 'd2'
--   )

-- select 
--   sm_sc.fv_pos_replaces
--   (
--     array
--     [
--       [
--         ['a', 'b', 'c', 'e', 'f', 'g', 'h']
--       , ['a1', 'b1', 'c1', 'e1', 'f1', 'g1', 'h1']
--       ]
--     , [
--         ['a2', 'b2', 'c2', 'e2', 'f2', 'g2', 'h2']
--       , ['a3', 'b3', 'c3', 'e3', 'f3', 'g3', 'h3']
--       ]
--     , [
--         ['a4', 'b4', 'c4', 'e4', 'f4', 'g4', 'h4']
--       , ['a5', 'b5', 'c5', 'e5', 'f5', 'g5', 'h5']
--       ]
--     ]
--   , array
--     [
--       [int4range(1, 1, '[]'), int4range(1, 2, '[]'), int4range(1, 2, '[]')]
--     , [int4range(3, 3, '[]'), int4range(2, 2, '[]'), int4range(4, 6, '[]')]
--     ]
--   , 'd2'
--   )

-- select 
--   sm_sc.fv_pos_replaces
--   (
--     array
--     [
--       [
--         [
--           ['a', 'b', 'c', 'e', 'f', 'g', 'h']
--         , ['a1', 'b1', 'c1', 'e1', 'f1', 'g1', 'h1']
--         ]
--       , [
--           ['a2', 'b2', 'c2', 'e2', 'f2', 'g2', 'h2']
--         , ['a3', 'b3', 'c3', 'e3', 'f3', 'g3', 'h3']
--         ]
--       , [
--           ['a4', 'b4', 'c4', 'e4', 'f4', 'g4', 'h4']
--         , ['a5', 'b5', 'c5', 'e5', 'f5', 'g5', 'h5']
--         ]
--       ]
--     , [
--         [
--           ['a0', 'b0', 'c0', 'e0', 'f0', 'g0', 'h0']
--         , ['a6', 'b6', 'c6', 'e6', 'f6', 'g6', 'h6']
--         ]
--       , [
--           ['a7', 'b7', 'c7', 'e7', 'f7', 'g7', 'h7']
--         , ['a8', 'b8', 'c8', 'e8', 'f8', 'g8', 'h8']
--         ]
--       , [
--           ['a9', 'b9', 'c9', 'e9', 'f9', 'g9', 'h9']
--         , ['aj', 'bj', 'cj', 'ej', 'fj', 'gj', 'hj']
--         ]
--       ]
--     ]
--   , array
--     [
--       [int4range(1, 1, '[]'), int4range(1, 1, '[]'), int4range(1, 2, '[]'), int4range(1, 2, '[]')]
--     , [int4range(1, 2, '[]'), int4range(3, 3, '[]'), int4range(2, 2, '[]'), int4range(4, 6, '[]')]
--     ]
--   , 'dk'
--   )
-- drop function if exists sm_sc.fv_rand_1d_ele(int, int);
create or replace function sm_sc.fv_rand_1d_ele
(
  i_all_cnt                int,
  i_pick_cnt               int
)
returns int[]
as
$$
declare -- here

begin
  return 
  (
    select 
      array_agg(ceil(random() * i_all_cnt)::int)
    from generate_series(1, i_pick_cnt)
  );
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select sm_sc.fv_rand_1d_ele(8, 3);
-- -- 从若干个样本中，随机拣取一个
-- drop function if exists sm_sc.fv_rand_ele_pick(anyarray);
create or replace function sm_sc.fv_rand_ele_pick
(
  i_all_eles                anyarray
)
returns table 
(
  o_remain_arr      anyarray   ,
  o_pick_arr        anyarray
)
as
$$
declare -- here
  v_pick_no   int   := round(random() * array_length(i_all_eles, 1) + 0.5 :: float)::int;
begin
  if array_ndims(i_all_eles) > 1
  then
    raise exception 'i_all_eles should be 1-d array';
  end if;
  
  return query
    select 
      i_all_eles[ : v_pick_no - 1] || i_all_eles[v_pick_no + 1 :]   as  o_remain_arr,
      i_all_eles[v_pick_no : v_pick_no]                                        as  o_pick_arr
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select o_remain_arr, o_pick_arr from sm_sc.fv_rand_ele_pick(array[1,2,3,4,5,6,7,8,9]);

-- -----------------------------------------------------

-- drop function if exists sm_sc.fv_rand_1d_ele_pick(anyarray, int);
create or replace function sm_sc.fv_rand_1d_ele_pick
(
  i_all_eles               anyarray,
  i_pick_cnt               int
)
returns table 
(
  o_remain_arr      anyarray   ,
  o_pick_arr        anyarray
)
as
$$
-- declare -- here

begin
  -- 审计是一维
  if array_ndims(i_all_eles) > 1
  then
    raise exception 'i_all_eles should be 1-d array. ';
  end if;

  -- 审计 拣取数量 大于 0
  if i_pick_cnt < 0
  then
    raise exception 'should i_pick_cnt >= 0. ';
  end if;
  
  -- 审计 拣取数量 不大于样本数
  if i_pick_cnt > array_length(i_all_eles, 1)
  then
    raise exception 'pick_cnt must be less than all_cnt.';  
  end if;

  -- 不能采用拣取全部，因为拣取全部无法随机乱序
  -- -- -- 开始排列组合拣取算法
  -- -- -- 拣取数等于样本数，则拣取全部
  -- -- if i_pick_cnt = array_length(i_all_eles, 1)
  -- -- then
  -- --   return query
  -- --     select i_all_eles[0 : 0], i_all_eles
  -- --   ;
  -- 拣取数大于1时，则递归拣取
  if i_pick_cnt <= array_length(i_all_eles, 1) and i_pick_cnt > 1
  then 
    -- -- -- 拣取数大于样本数的一半以上，则反向拣取
    -- -- if i_pick_cnt <= array_length(i_all_eles, 1) / 2
    -- -- then 
    
    -- 不能采用反向拣取，因为反向拣取的剩余无法随机乱序
    return query
      select 
        tb_a_recur.a_remain_arr                           as o_remain_arr    ,
        tb_a_cur.a_pick_arr || tb_a_recur.a_pick_arr      as o_pick_arr  
      from sm_sc.fv_rand_ele_pick(i_all_eles) tb_a_cur(a_remain_arr, a_pick_arr)    -- 本次
        , sm_sc.fv_rand_1d_ele_pick(tb_a_cur.a_remain_arr, i_pick_cnt - 1) tb_a_recur(a_remain_arr, a_pick_arr)
    ;
    -- -- else
    -- --   return query
    -- --     select 
    -- --       tb_a_cur.a_pick_arr || tb_a_recur.a_pick_arr      as o_remain_arr   ,
    -- --       tb_a_recur.a_remain_arr                           as o_pick_arr 
    -- --     from sm_sc.fv_rand_ele_pick(i_all_eles) tb_a_cur(a_remain_arr, a_pick_arr)    -- 本次
    -- --       , sm_sc.fv_rand_1d_ele_pick(tb_a_cur.a_remain_arr, array_length(i_all_eles, 1) - i_pick_cnt - 1) tb_a_recur(a_remain_arr, a_pick_arr)
    --   ;
    -- end if;
  -- 拣取数为1时，则拣取一个
  elsif i_pick_cnt = 1
  then 
    return query
      select 
        tb_a.o_remain_arr    ,
        tb_a.o_pick_arr  
      from sm_sc.fv_rand_ele_pick(i_all_eles) tb_a(o_remain_arr, o_pick_arr)
    ;
  -- 拣取数为0时，则拣取集合为空
  elsif i_pick_cnt = 0
  then 
    return query
      select i_all_eles, i_all_eles[0 : 0]
    ;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select o_remain_arr, o_pick_arr from sm_sc.fv_rand_1d_ele_pick(array[1,2,3,4,5,6,7,8,9], 0);
-- select o_remain_arr, o_pick_arr from sm_sc.fv_rand_1d_ele_pick(array[1,2,3,4,5,6,7,8,9], 1);
-- select o_remain_arr, o_pick_arr from sm_sc.fv_rand_1d_ele_pick(array[1,2,3,4,5,6,7,8,9], 3);
-- select o_remain_arr, o_pick_arr from sm_sc.fv_rand_1d_ele_pick(array[1,2,3,4,5,6,7,8,9], 7);
-- select o_remain_arr, o_pick_arr from sm_sc.fv_rand_1d_ele_pick(array[1,2,3,4,5,6,7,8,9], 9);

-- ---------------------------------------------------------------------------

-- drop function if exists sm_sc.fv_rand_1d_ele_pick(bigint, int);
create or replace function sm_sc.fv_rand_1d_ele_pick
(
  i_all_cnt                int,
  i_pick_cnt               int
)
returns int[]
as
$$
-- declare -- here

begin
  return 
  (
    select 
      o_pick_arr
    from sm_sc.fv_rand_1d_ele_pick((select array_agg(a_no) from generate_series(1, i_all_cnt) tb_a_no(a_no)), i_pick_cnt) tb_a
  )
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select sm_sc.fv_rand_1d_ele_pick(8, 3);


-- -------------------------------------------------------------------------


-- -- -- drop function if exists sm_sc.fv_rand_1d_ele_pick(int, int);
-- -- create or replace function sm_sc.fv_rand_1d_ele_pick
-- -- (
-- --   i_all_cnt                int,
-- --   i_pick_cnt               int
-- -- )
-- -- returns int[]
-- -- as
-- -- $$
-- -- declare -- here
-- --   v_sess_id          bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)   :=    replace(gen_random_uuid()::char(36), '-', '')::char(32);
-- --   v_cur_no           int        :=    i_pick_cnt;
-- --   v_cur_ele_no       int;
-- --   v_remain_final     int        := i_all_cnt - i_pick_cnt;
-- --   v_return           int[];
-- -- 
-- -- begin
-- -- 
-- --   if i_pick_cnt > i_all_cnt
-- --   then
-- --     raise exception 'pick_cnt must be less than all_cnt.';
-- --   end if;
-- -- 
-- --   -- create temp table if not exists sm_sc.__vt_rand_1d_ele
-- --   -- (
-- --   --   sess_id     char(32)    ,     -- 会话 id
-- --   --   arr_picked  int[]       ,     -- 已捡取元素编号
-- --   --   arr_remain  int[]       ,     -- 剩余元素编号
-- --   --   primary key (sess_id)
-- --   -- );
-- -- 
-- --   insert into sm_sc.__vt_rand_1d_ele
-- --   (
-- --     sess_id            ,
-- --     arr_remain         ,
-- --     arr_picked         
-- --   )                    
-- --   select               
-- --     v_sess_id          ,
-- --     array_agg(a_ele_no),
-- --     array[] :: int[]
-- --   from generate_series(1, i_all_cnt) tb_a(a_ele_no)
-- --   ;
-- -- 
-- --   while v_cur_no >= 1
-- --   loop 
-- --     v_cur_ele_no := round(random() * (v_remain_final + v_cur_no) + 0.5 :: float)::int;
-- -- 
-- -- -- -- debug
-- -- -- raise notice 'v_cur_ele_no: %; v_cur_no: %; ', v_cur_ele_no, v_cur_no;
-- -- 
-- --     update sm_sc.__vt_rand_1d_ele
-- --     set 
-- --       arr_picked = array_append(arr_picked, arr_remain[v_cur_ele_no]),
-- --       arr_remain[v_cur_ele_no : greatest(v_cur_ele_no, v_remain_final + v_cur_no - 1)] = arr_remain[least(v_cur_ele_no + 1, i_all_cnt) : least(greatest(v_cur_ele_no + 1, v_remain_final + v_cur_no), i_all_cnt)]
-- --     where sess_id = v_sess_id
-- --     ;
-- -- 
-- --     v_cur_no := v_cur_no - 1;
-- --   end loop;
-- -- 
-- --   v_return := 
-- --   (
-- --     select 
-- --       arr_picked
-- --     from sm_sc.__vt_rand_1d_ele
-- --     where sess_id = v_sess_id
-- --   );
-- -- 
-- --   delete from sm_sc.__vt_rand_1d_ele where sess_id = v_sess_id;
-- --   
-- --   return v_return;
-- -- end
-- -- $$
-- -- language plpgsql volatile
-- -- parallel unsafe
-- -- cost 100;
-- -- 
-- -- -- -- set force_parallel_mode = 'off';
-- -- -- select sm_sc.fv_rand_1d_ele_pick(8, 3);
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_x(anyarray, int);
create or replace function sm_sc.fv_rand_slice_x
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) = 2
  then
    return
    (
      select
        sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  elsif array_ndims(i_array) = 3
  then
    return
    (
      select
        sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  elsif array_ndims(i_array) = 4
  then
    return
    (
      select
        sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  else
    raise exception 'no method for such length!  Dim: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_rand_slice_x
--   (
--     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
--     , 6
--     -- , true
--   );
-- select sm_sc.fv_rand_slice_x
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 9
--     -- , true
--   );
-- select sm_sc.fv_rand_slice_x
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 5
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_x3(anyarray, int);
create or replace function sm_sc.fv_rand_slice_x3
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  -- -- elsif array_ndims(i_array) = 2
  -- -- then
  -- --   return
  -- --   (
  -- --     select
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 3), i_select_cnt)) tb_a_cur(a_cur)
  -- --   )
  -- --   ;
  elsif array_ndims(i_array) = 3
  then
    return
    (
      select
        sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 3), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  elsif array_ndims(i_array) = 4
  then
    return
    (
      select
        sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 3), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  else
    raise exception 'no method for such length!  Dim: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_rand_slice_x3
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 7
-- --     -- , true
-- --   );
-- select sm_sc.fv_rand_slice_x3
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 12
--     -- , true
--   );
-- select sm_sc.fv_rand_slice_x3
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 8
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_x3_pick(anyarray, int);
create or replace function sm_sc.fv_rand_slice_x3_pick
(
  i_array          anyarray          ,
  i_pick_cnt     int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  -- -- elsif array_ndims(i_array) = 2
  -- -- then
  -- --   return
  -- --   (
  -- --     select
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
  -- --   )
  -- --   ;
  elsif i_pick_cnt <= array_length(i_array, 3)
  then 
    if array_ndims(i_array) = 3
    then
      return
      (
        select
          sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    elsif array_ndims(i_array) = 4
    then
      return
      (
        select
          sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    end if;
  else
    raise exception 'no method for such length!  Dim: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_rand_slice_x3_pick
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 2
-- --     -- , true
-- --   );
-- select sm_sc.fv_rand_slice_x3_pick
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 2
--     -- , true
--   );
-- select sm_sc.fv_rand_slice_x3_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_x4(anyarray, int);
create or replace function sm_sc.fv_rand_slice_x4
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  -- -- elsif array_ndims(i_array) = 2 and i_select_cnt <= array_length(i_array, 2)
  -- -- then
  -- --   return
  -- --   (
  -- --     select
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 4), i_select_cnt)) tb_a_cur(a_cur)
  -- --   )
  -- --   ;
  -- -- elsif array_ndims(i_array) = 3 and i_select_cnt <= array_length(i_array, 2)
  -- -- then
  -- --   return
  -- --   (
  --        select
  -- --       sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 4), i_select_cnt)) tb_a_cur(a_cur)
  -- --   )
  -- --   ;
  elsif array_ndims(i_array) = 4
  then
    return 
    (
      select
        sm_sc.fa_mx_concat_x4(i_array[ : ][ : ][ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 4), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  else
    raise exception 'no method for such length!  Dim: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_rand_slice_x4
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 9
-- --     -- , true
-- --   );
-- -- select sm_sc.fv_rand_slice_x4
-- --   (
-- --     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
-- --         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
-- --     , 5
-- --     -- , true
-- --   );
-- select sm_sc.fv_rand_slice_x4
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[17, -17], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 7
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_x4_pick(anyarray, int);
create or replace function sm_sc.fv_rand_slice_x4_pick
(
  i_array          anyarray          ,
  i_pick_cnt     int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  -- -- elsif array_ndims(i_array) = 2
  -- -- then
  -- --   return
  -- --   (
  -- --     select
  -- --       sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
  -- --   )
  -- --   ;
  -- -- elsif array_ndims(i_array) = 3
  -- -- then
  -- --   return
  -- --   (
  -- --     select
  -- --       sm_sc.fa_mx_concat_x3(i_array[ : ][ : ][a_cur : a_cur]) as o_slices
  -- --     from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 3), i_pick_cnt)) tb_a_cur(a_cur)
  -- --   )
  -- --   ;
  elsif array_ndims(i_array) = 4 and i_pick_cnt <= array_length(i_array, 4)
  then
    return
    (
      select
        sm_sc.fa_mx_concat_x4(i_array[ : ][ : ][ : ][a_cur : a_cur]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 4), i_pick_cnt)) tb_a_cur(a_cur)
    )
    ;
  else
    raise exception 'no method for such length!  Dim: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_rand_slice_x4_pick
-- --   (
-- --     array[[1, 2, 3, 4, 5, 6], [7, 8, 9, 10, 11, 12]]
-- --     , 2
-- --     -- , true
-- --   );
-- -- select sm_sc.fv_rand_slice_x4_pick
-- --   (
-- --     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
-- --         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
-- --     , 2
-- --     -- , true
-- --   );
-- select sm_sc.fv_rand_slice_x4_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_x_pick(anyarray, int);
create or replace function sm_sc.fv_rand_slice_x_pick
(
  i_array          anyarray          ,
  i_pick_cnt       int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif i_pick_cnt <= array_length(i_array, 2)
  then 
    if array_ndims(i_array) = 2 and i_pick_cnt <= array_length(i_array, 2)
    then
      return 
      (
        select
          sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 2), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    elsif array_ndims(i_array) = 3 and i_pick_cnt <= array_length(i_array, 2)
    then
      return 
      (
        select
          sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    elsif array_ndims(i_array) = 4 and i_pick_cnt <= array_length(i_array, 2)
    then
      return 
      (
        select
          sm_sc.fa_mx_concat_x(i_array[ : ][a_cur : a_cur][ : ][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 2), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    end if;
  else
    raise exception 'no method for such length!  Dim: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_rand_slice_x_pick
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
--     , 3
--     -- , true
--   ) tb_a;
-- select sm_sc.fv_rand_slice_x_pick
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 3
--     -- , true
--   );
-- select sm_sc.fv_rand_slice_x_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_y(anyarray, int);
create or replace function sm_sc.fv_rand_slice_y
(
  i_array          anyarray          ,
  i_select_cnt     int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) = 2
  then
    return
    (
      select
        sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 1), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  elsif array_ndims(i_array) = 3
  then
    return
    (
      select
        sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 1), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  elsif array_ndims(i_array) = 4
  then
    return
    (
      select
        sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ][ : ]) as o_slices
      from unnest(sm_sc.fv_rand_1d_ele(array_length(i_array, 1), i_select_cnt)) tb_a_cur(a_cur)
    )
    ;
  else
    raise exception 'no method for such length!  Dims: %; select_cnt: %;', array_dims(i_array), i_select_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_rand_slice_y
--   (
--     array[array[1.0 :: float, 2.0], array[3.0, 4.0], array[5.0, 6.0], array[7.0, 8.0], array[9.0, 10.0], array[11.0 :: float, 12.0]]
--   , 8
--   )
-- select 
--   sm_sc.fv_rand_slice_y
--   (
--     array
--     [
--       [[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--     , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]
--     ]
--   , 6
--   );
-- select sm_sc.fv_rand_slice_y
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 4
--     -- , true
--   );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rand_slice_y_pick(anyarray, int);
create or replace function sm_sc.fv_rand_slice_y_pick
(
  i_array          anyarray          ,
  i_pick_cnt       int
)
returns anyarray
as
$$
-- declare

begin
  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif i_pick_cnt <= array_length(i_array, 1)
  then 
    if array_ndims(i_array) = 2
    then
      return
      (
        select
          sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 1), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    elsif array_ndims(i_array) = 3
    then
      return
      (
        select
          sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 1), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    elsif array_ndims(i_array) = 4
    then
      return
      (
        select
          sm_sc.fa_mx_concat_y(i_array[a_cur : a_cur][ : ][ : ][ : ]) as o_slices
        from unnest(sm_sc.fv_rand_1d_ele_pick(array_length(i_array, 1), i_pick_cnt)) tb_a_cur(a_cur)
      )
      ;
    end if;
  else 
    raise exception 'no method for such length!  Dims: %; pick_cnt: %;', array_dims(i_array), i_pick_cnt;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_rand_slice_y_pick
--   (
--     array[array[1, 2], array[3, 4], array[5, 6], array[7, 8], array[9, 10], array[11, 12]]
--     , 2
--   );
-- select sm_sc.fv_rand_slice_y_pick
--   (
--     array[[[1, 2, 3], [11, 21, 31], [12, 22, 32], [13, 23, 33], [14, 24, 34], [15, 25, 35]]
--         , [[16, 26, 36], [17, 27, 37], [18, 28, 38], [19, 29, 39], [61, 62, 63], [81, 82, 83]]]
--     , 2
--     -- , true
--   );
-- select sm_sc.fv_rand_slice_y_pick
--   (
--     array[[[[1, -1], [2, -2], [3, -3]], [[11, -11], [21, -21], [31, -31]], [[12, -12], [22, -22], [32, -32]], [[13, -13], [23, -23], [33, -33]], [[14, -14], [24, -24], [34, -34]], [[15, -15], [25, -25], [35, -35]]]
--         , [[[16, -16], [26, -26], [36, -36]], [[27, -27], [27, -27], [37, -37]], [[18, -18], [28, -28], [38, -38]], [[19, -19], [29, -29], [39, -39]], [[61, -61], [62, -62], [63, -63]], [[81, -81], [82, -82], [83, -83]]]]
--     , 2
--     -- , true
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_regexp_matches(text[], text[], text);
create or replace function sm_sc.fv_regexp_matches
(
  i_array     text[]    ,
  i_pattern    text[]    ,
  i_flags      text   default 'g'
)
returns text[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_pattern, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_pattern)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     text[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  v_ret := array_fill(null::text, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_array := array[[[i_array]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_array := array[[i_array]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_array := array[i_array];
    end if;
    
    v_len_left := (select array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_array := sm_sc.fv_new(i_array, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_pattern := array[[[i_pattern]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_pattern := array[[i_pattern]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_pattern := array[i_pattern];
    end if;
    
    v_len_right := (select array_agg(array_length(i_pattern, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_pattern)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_pattern := sm_sc.fv_new(i_pattern, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_pattern, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_pattern)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- regexp_matches(array[], [])
  if v_len_left is null and i_array is not null -- = 0
  then 
    return array_fill(null::text, array[coalesce(array_length(i_pattern, 1), 0), coalesce(array_length(i_pattern, 2), 0)]);
  -- regexp_matches([] ~ array[]
  elsif v_len_right is null and i_pattern is not null -- = 0
  then 
    return array_fill(null::text, array[coalesce(array_length(i_array, 1), 0), coalesce(array_length(i_array, 2), 0)]);
  
  elsif i_array is null or i_pattern is null
  then
    return null::text[];
  
  -- regexp_matches([], [])
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := (select array_agg(a_matches)::text from regexp_matches(i_array[v_y_cur], i_pattern[v_y_cur], i_flags) tb_a(a_matches));
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_matches([][], [][])
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := (select array_agg(a_matches)::text from regexp_matches(i_array[v_y_cur][v_x_cur], i_pattern[v_y_cur][v_x_cur], i_flags) tb_a(a_matches));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- regexp_matches([][][], [][][])
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
          (
            select array_agg(a_matches)::text 
            from regexp_matches(i_array[v_y_cur][v_x_cur][v_x3_cur], i_pattern[v_y_cur][v_x_cur][v_x3_cur], i_flags) tb_a(a_matches)
          );
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- regexp_matches([][][][], [][][][])
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
            (
              select array_agg(a_matches)::text 
              from regexp_matches(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_pattern[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_flags) tb_a(a_matches)
            );
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_regexp_matches
--   ( 
--     (<>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: text)) :: text[]
--   , (<>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: text)) :: text[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_regexp_matches
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[array['abbbbbc122223'], array['abc123']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array[a.c], array[1.*?3]]
--   );
-- -- select sm_sc.fv_regexp_matches
-- --   (
-- --     array['abbbbbc122223', 'abc123'],
-- --     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
-- --   );
-- -- select sm_sc.fv_regexp_matches
-- --   (
-- --     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
-- --     array['1.3', 'a.*?c']
-- --   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[array['abbbbbc122223', 'abc123']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[]::text[],
--     array[array[], array []]::text[]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[array[], array []]::text[],
--     array[]::text[]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array[array[], array []]::text[],
--     array[array[], array []]::text[]
--   );
-- select sm_sc.fv_regexp_matches
--   (
--     array['abbbbbc122223', 'abc123', 'abc123', 'ac13'],
--     array['a.c', '1.*?3', '1.3', 'a.*?c']
--   );
-- select sm_sc.fv_regexp_matches(array['1'], array['1','2','3']);
-- select sm_sc.fv_regexp_matches(array['1'], array[array['1','2','3']]);
-- select sm_sc.fv_regexp_matches(array[array['1']], array['1','2','3']);
-- select sm_sc.fv_regexp_matches(array[array['1']], array[array['1','2','3']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_regexp_matches
--   (
--     array[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]
--   , array[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]
--   )
-- select 
--   sm_sc.fv_regexp_matches
--   (
--     array[[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]],[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]]
--   , array[[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]],[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_regexp_matches(text[], text, text);
create or replace function sm_sc.fv_regexp_matches
(
  i_array     text[]    ,
  i_pattern    text    ,
  i_flags      text   default 'g'
)
returns text[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     text[]  ;
begin
  -- regexp_matches(null :: text[][] ~ text) = null :: text[][]
  if array_length(i_array, array_ndims(i_array)) is null and i_pattern is not null
  then 
    return v_ret;
  -- -- elsif i_pattern is null
  -- -- then
  -- --   return null::text[];

  -- regexp_matches([][], text)
  elsif array_ndims(i_array) =  2
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1), array_length(i_array, 2)]);
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (select array_agg(a_matches)::text from regexp_matches(i_array[v_y_cur][v_x_cur], i_pattern, i_flags) tb_a(a_matches));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_matches([], text)
  elsif array_ndims(i_array) =  1
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1)]);
    while v_y_cur <= array_length(i_array, 1)
    loop
      v_ret[v_y_cur] := (select array_agg(a_matches)::text from regexp_matches(i_array[v_y_cur], i_pattern, i_flags) tb_a(a_matches));
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_matches([][][], text)
  elsif array_ndims(i_array) =  3
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3)]);
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
          (
            select array_agg(a_matches)::text 
            from regexp_matches(i_array[v_y_cur][v_x_cur][v_x3_cur], i_pattern, i_flags) tb_a(a_matches)
          );
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- regexp_matches([][][][], text)
  elsif array_ndims(i_array) = 4
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), array_length(i_array, 4)]);
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
            (
              select array_agg(a_matches)::text 
              from regexp_matches(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_pattern, i_flags) tb_a(a_matches)
            );
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_regexp_matches(array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']], 'a.c')
-- select sm_sc.fv_regexp_matches(array['abbbbbc122223', 'abc123', 'abc123', 'ac13'], 'a.c')
-- select sm_sc.fv_regexp_matches(array[]::text[], '1.3')
-- select sm_sc.fv_regexp_matches(array[array[], array []]::text[], '1.3')
-- select 
--   sm_sc.fv_regexp_matches
--   (
--     array[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]
--   , '1.*?3'
--   )
-- select 
--   sm_sc.fv_regexp_matches
--   (
--     array[[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]],[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]]
--   , 'a.*?c'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_regexp_matches(text, text[], text);
create or replace function sm_sc.fv_regexp_matches
(
  i_array     text    ,
  i_pattern    text[]    ,
  i_flags      text   default 'g'
)
returns text[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     text[]  ;
begin
  -- regexp_matches(null :: text[][] ~ text) = null :: text[][]
  if array_length(i_pattern, array_ndims(i_pattern)) is null and i_array is not null
  then 
    return v_ret;
  -- -- elsif i_array is null
  -- -- then
  -- --   return null::text[];

  -- regexp_matches(text, [][])
  elsif array_ndims(i_pattern) =  2
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1), array_length(i_pattern, 2)]);
    while v_y_cur <= array_length(i_pattern, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_pattern, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (select array_agg(a_matches)::text from regexp_matches(i_array, i_pattern[v_y_cur][v_x_cur], i_flags) tb_a(a_matches));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_matches(text, [])
  elsif array_ndims(i_pattern) =  1
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1)]);
    while v_y_cur <= array_length(i_pattern, 1)
    loop
      v_ret[v_y_cur] := (select array_agg(a_matches)::text from regexp_matches(i_array, i_pattern[v_y_cur], i_flags) tb_a(a_matches));
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_matches(text, [][][])
  elsif array_ndims(i_pattern) = 3
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1), array_length(i_pattern, 2), array_length(i_pattern, 3)]);
    for v_y_cur in 1 .. array_length(i_pattern, 1)
    loop
      for v_x_cur in 1 .. array_length(i_pattern, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_pattern, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
          (
            select array_agg(a_matches)::text 
            from regexp_matches(i_array, i_pattern[v_y_cur][v_x_cur][v_x3_cur], i_flags) tb_a(a_matches)
          );
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- regexp_matches(text, [][][][])
  elsif array_ndims(i_pattern) = 4
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1), array_length(i_pattern, 2), array_length(i_pattern, 3), array_length(i_pattern, 4)]);
    for v_y_cur in 1 .. array_length(i_pattern, 1)
    loop
      for v_x_cur in 1 .. array_length(i_pattern, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_pattern, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_pattern, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
            (
              select array_agg(a_matches)::text 
              from regexp_matches(i_array, i_pattern[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_flags) tb_a(a_matches)
            );
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_pattern);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_regexp_matches('abbbbbc122223', array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']])
-- select sm_sc.fv_regexp_matches('abc123', array['a.c', '1.*?3', '1.3', 'a.*?c'])
-- select sm_sc.fv_regexp_matches('abc123', array[]::text[])
-- select sm_sc.fv_regexp_matches('abc123', array[array[], array []]::text[])
-- select 
--   sm_sc.fv_regexp_matches
--   (
--     'abc123'
--   , array[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]
--   )
-- select 
--   sm_sc.fv_regexp_matches
--   (
--     'abc123'
--   , array[[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]],[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]]
--   )

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_regexp_replace(text[], text[], text, text);
create or replace function sm_sc.fv_regexp_replace
(
  i_array     text[]    ,
  i_pattern    text[]    ,
  i_replacement   text   default ''    ,
  i_flags      text   default 'g'
)
returns text[]
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_pattern, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_pattern)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt int[]  ;
  v_ret     text[];
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  v_ret := array_fill(null::text, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_array := array[[[i_array]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_array := array[[i_array]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_array := array[i_array];
    end if;
    
    v_len_left := (select array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_array := sm_sc.fv_new(i_array, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_pattern := array[[[i_pattern]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_pattern := array[[i_pattern]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_pattern := array[i_pattern];
    end if;
    
    v_len_right := (select array_agg(array_length(i_pattern, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_pattern)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_pattern := sm_sc.fv_new(i_pattern, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_pattern, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_pattern)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- regexp_replace(array[], [])
  if v_len_left is null and i_array is not null -- = 0
  then 
    return array_fill(null::text, array[coalesce(array_length(i_pattern, 1), 0), coalesce(array_length(i_pattern, 2), 0)]);
  -- regexp_replace([] ~ array[]
  elsif v_len_right is null and i_pattern is not null -- = 0
  then 
    return array_fill(null::text, array[coalesce(array_length(i_array, 1), 0), coalesce(array_length(i_array, 2), 0)]);
  
  elsif i_array is null or i_pattern is null
  then
    return null::text[];
  
  -- regexp_replace([], [])
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := (select array_agg(a_matches)::text from regexp_replace(i_array[v_y_cur], i_pattern[v_y_cur], i_replacement, i_flags) tb_a(a_matches));
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_replace([][], [][])
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := (select array_agg(a_matches)::text from regexp_replace(i_array[v_y_cur][v_x_cur], i_pattern[v_y_cur][v_x_cur], i_replacement, i_flags) tb_a(a_matches));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- regexp_replace([][][], [][][])
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
          (
            select array_agg(a_matches)::text 
            from regexp_replace(i_array[v_y_cur][v_x_cur][v_x3_cur], i_pattern[v_y_cur][v_x_cur][v_x3_cur], i_replacement, i_flags) tb_a(a_matches)
          );
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- regexp_replace([][][][], [][][][])
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
            (
              select array_agg(a_matches)::text 
              from regexp_replace(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_pattern[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_replacement, i_flags) tb_a(a_matches)
            );
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_regexp_replace
--   ( 
--     (<>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: text)) :: text[]
--   , (<>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: text)) :: text[]
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.fv_regexp_replace
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[array['abbbbbc122223'], array['abc123']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array[a.c], array[1.*?3]]
--   );
-- -- select sm_sc.fv_regexp_replace
-- --   (
-- --     array['abbbbbc122223', 'abc123'],
-- --     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
-- --   );
-- -- select sm_sc.fv_regexp_replace
-- --   (
-- --     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
-- --     array['1.3', 'a.*?c']
-- --   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[array['abbbbbc122223', 'abc123']],
--     array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']],
--     array[array['1.3', 'a.*?c']]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[]::text[],
--     array[array[], array []]::text[]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[array[], array []]::text[],
--     array[]::text[]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array[array[], array []]::text[],
--     array[array[], array []]::text[]
--   );
-- select sm_sc.fv_regexp_replace
--   (
--     array['abbbbbc122223', 'abc123', 'abc123', 'ac13'],
--     array['a.c', '1.*?3', '1.3', 'a.*?c']
--   );
-- select sm_sc.fv_regexp_replace(array['1'], array['1','2','3']);
-- select sm_sc.fv_regexp_replace(array['1'], array[array['1','2','3']]);
-- select sm_sc.fv_regexp_replace(array[array['1']], array['1','2','3']);
-- select sm_sc.fv_regexp_replace(array[array['1']], array[array['1','2','3']]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.fv_regexp_replace
--   (
--     array[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]
--   , array[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]
--   )
-- select 
--   sm_sc.fv_regexp_replace
--   (
--     array[[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]],[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]]
--   , array[[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]],[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_regexp_replace(text[], text, text, text);
create or replace function sm_sc.fv_regexp_replace
(
  i_array     text[]    ,
  i_pattern    text    ,
  i_replacement   text   default ''    ,
  i_flags      text   default 'g'
)
returns text[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     text[]  ;
begin
  -- regexp_replace(null :: text[][] ~ text) = null :: text[][]
  if array_length(i_array, array_ndims(i_array)) is null and i_pattern is not null
  then 
    return v_ret;
  -- -- elsif i_pattern is null
  -- -- then
  -- --   return null::text[];

  -- regexp_replace([][], text)
  elsif array_ndims(i_array) =  2
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1), array_length(i_array, 2)]);
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (select array_agg(a_matches)::text from regexp_replace(i_array[v_y_cur][v_x_cur], i_pattern, i_replacement, i_flags) tb_a(a_matches));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_replace([], text)
  elsif array_ndims(i_array) =  1
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1)]);
    while v_y_cur <= array_length(i_array, 1)
    loop
      v_ret[v_y_cur] := (select array_agg(a_matches)::text from regexp_replace(i_array[v_y_cur], i_pattern, i_replacement, i_flags) tb_a(a_matches));
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_replace([][][], text)
  elsif array_ndims(i_array) =  3
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3)]);
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
          (
            select array_agg(a_matches)::text 
            from regexp_replace(i_array[v_y_cur][v_x_cur][v_x3_cur], i_pattern, i_replacement, i_flags) tb_a(a_matches)
          );
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- regexp_replace([][][][], text)
  elsif array_ndims(i_array) = 4
  then
    v_ret := array_fill(null::text, array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), array_length(i_array, 4)]);
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
            (
              select array_agg(a_matches)::text 
              from regexp_replace(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_pattern, i_replacement, i_flags) tb_a(a_matches)
            );
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_regexp_replace(array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']], 'a.c')
-- select sm_sc.fv_regexp_replace(array['abbbbbc122223', 'abc123', 'abc123', 'ac13'], 'a.c')
-- select sm_sc.fv_regexp_replace(array[]::text[], '1.3')
-- select sm_sc.fv_regexp_replace(array[array[], array []]::text[], '1.3')
-- select 
--   sm_sc.fv_regexp_replace
--   (
--     array[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]
--   , '1.*?3'
--   )
-- select 
--   sm_sc.fv_regexp_replace
--   (
--     array[[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]],[[['abc123', 'ac13'],['abc123', 'ac13']],[['abc123', 'ac13'],['abc123', 'ac13']]]]
--   , 'a.*?c'
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.fv_regexp_replace(text, text[], text, text);
create or replace function sm_sc.fv_regexp_replace
(
  i_array     text    ,
  i_pattern    text[]    ,
  i_replacement   text   default ''    ,
  i_flags      text   default 'g'
)
returns text[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     text[]  ;
begin
  -- regexp_replace(null :: text[][] ~ text) = null :: text[][]
  if array_length(i_pattern, array_ndims(i_pattern)) is null and i_array is not null
  then 
    return v_ret;
  -- -- elsif i_array is null
  -- -- then
  -- --   return null::text[];

  -- regexp_replace(text, [][])
  elsif array_ndims(i_pattern) =  2
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1), array_length(i_pattern, 2)]);
    while v_y_cur <= array_length(i_pattern, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_pattern, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := (select array_agg(a_matches)::text from regexp_replace(i_array, i_pattern[v_y_cur][v_x_cur], i_replacement, i_flags) tb_a(a_matches));
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_replace(text, [])
  elsif array_ndims(i_pattern) =  1
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1)]);
    while v_y_cur <= array_length(i_pattern, 1)
    loop
      v_ret[v_y_cur] := (select array_agg(a_matches)::text from regexp_replace(i_array, i_pattern[v_y_cur], i_replacement, i_flags) tb_a(a_matches));
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- regexp_replace(text, [][][])
  elsif array_ndims(i_pattern) = 3
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1), array_length(i_pattern, 2), array_length(i_pattern, 3)]);
    for v_y_cur in 1 .. array_length(i_pattern, 1)
    loop
      for v_x_cur in 1 .. array_length(i_pattern, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_pattern, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] = 
          (
            select array_agg(a_matches)::text 
            from regexp_replace(i_array, i_pattern[v_y_cur][v_x_cur][v_x3_cur], i_replacement, i_flags) tb_a(a_matches)
          );
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- regexp_replace(text, [][][][])
  elsif array_ndims(i_pattern) = 4
  then
    v_ret := array_fill(null::text, array[array_length(i_pattern, 1), array_length(i_pattern, 2), array_length(i_pattern, 3), array_length(i_pattern, 4)]);
    for v_y_cur in 1 .. array_length(i_pattern, 1)
    loop
      for v_x_cur in 1 .. array_length(i_pattern, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_pattern, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_pattern, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
            (
              select array_agg(a_matches)::text 
              from regexp_replace(i_array, i_pattern[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_replacement, i_flags) tb_a(a_matches)
            );
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_pattern);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_regexp_replace('abbbbbc122223', array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']])
-- select sm_sc.fv_regexp_replace('abc123', array['a.c', '1.*?3', '1.3', 'a.*?c'])
-- select sm_sc.fv_regexp_replace('abc123', array[]::text[])
-- select sm_sc.fv_regexp_replace('abc123', array[array[], array []]::text[])
-- select 
--   sm_sc.fv_regexp_replace
--   (
--     'abc123'
--   , array[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]
--   )
-- select 
--   sm_sc.fv_regexp_replace
--   (
--     'abc123'
--   , array[[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]],[[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']],[['1.*?3', 'a.*?c'],['1.*?3', 'a.*?c']]]]
--   )

-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_repeat_axis_py(float[], int, int);
create or replace function sm_sc.fv_repeat_axis_py
(
  i_arr             float[]
, i_dim             int
, i_repeat          int
)
returns float[]
as
$$
  from numpy import array
  return array(i_arr).repeat(i_repeat, i_dim if i_dim < 0 else i_dim - 1).tolist()
$$
language plpython3u stable
parallel safe
;

-- select 
--   sm_sc.fv_repeat_axis_py
--   (
--     array[[1,2,3,4],[5,6,7,8],[9,10,11,12],[13,14,15,16]]
--   , 1
--   , 3
--   )

-- -----------------------------------------------------------------------------------------------------------------------
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_repeat_axis_py(float[], int[], int[]);
create or replace function sm_sc.fv_repeat_axis_py
(
  i_arr              float[]
, i_dims             int[]   -- 规约: 维度顺序从 1 开始递增，从 -1 开始递减，不支持第 0 维度。
, i_repeats          int[]
)
returns float[]
as
$$
  from numpy import array
  if len(i_dims) == 1 :
    return array(i_arr).repeat(i_repeats[0], i_dims[0] if i_dims[0] < 0 else i_dims[0] - 1).tolist()
  if len(i_dims) == 2 :                            
    return array(i_arr).repeat(i_repeats[0], i_dims[0] if i_dims[0] < 0 else i_dims[0] - 1) \
                       .repeat(i_repeats[1], i_dims[1] if i_dims[1] < 0 else i_dims[1] - 1).tolist()
  if len(i_dims) == 3 :                            
    return array(i_arr).repeat(i_repeats[0], i_dims[0] if i_dims[0] < 0 else i_dims[0] - 1) \
                       .repeat(i_repeats[1], i_dims[1] if i_dims[1] < 0 else i_dims[1] - 1) \
                       .repeat(i_repeats[2], i_dims[2] if i_dims[2] < 0 else i_dims[2] - 1).tolist()
  if len(i_dims) == 4 :                            
    return array(i_arr).repeat(i_repeats[0], i_dims[0] if i_dims[0] < 0 else i_dims[0] - 1) \
                       .repeat(i_repeats[1], i_dims[1] if i_dims[1] < 0 else i_dims[1] - 1) \
                       .repeat(i_repeats[2], i_dims[2] if i_dims[2] < 0 else i_dims[2] - 1) \
                       .repeat(i_repeats[3], i_dims[3] if i_dims[3] < 0 else i_dims[3] - 1).tolist()
  if len(i_dims) == 5 :                            
    return array(i_arr).repeat(i_repeats[0], i_dims[0] if i_dims[0] < 0 else i_dims[0] - 1) \
                       .repeat(i_repeats[1], i_dims[1] if i_dims[1] < 0 else i_dims[1] - 1) \
                       .repeat(i_repeats[2], i_dims[2] if i_dims[2] < 0 else i_dims[2] - 1) \
                       .repeat(i_repeats[3], i_dims[3] if i_dims[3] < 0 else i_dims[3] - 1) \
                       .repeat(i_repeats[4], i_dims[4] if i_dims[4] < 0 else i_dims[4] - 1).tolist()
$$
language plpython3u stable
parallel safe
;

-- select 
--   sm_sc.fv_repeat_axis_py
--   (
--     array[[[1,2,3,4],[5,6,7,8],[9,10,11,12],[13,14,15,16]]]
--   , array[1,0]
--   , array[2,3]
--   )
-- drop function if exists sm_sc.fv_rmask(anyarray, int[], anyelement);
create or replace function sm_sc.fv_rmask
(
  i_array               anyarray    ,
  i_mask_len            int[]       ,     -- 该控制参数的维度与 i_array 相同，背景面的 width 维度(最高维度)长度为 1，其他维度长度与 i_array 对应维度长度也相同。
  i_mask_element        anyelement
)
returns anyarray
as
$$
declare
  v_pos_range_s         int4range[] ;
  v_background_width    int         := array_length(i_array, array_ndims(i_array));
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 2 and 4 
    then 
      raise exception 'unsupport ndims of i_array';
    elsif array_ndims(i_mask_len) <> array_ndims(i_array)
    then 
      raise exception 'array_ndims of i_mask_len should be the same as i_array''s.';
    elsif array_length(i_mask_len, array_ndims(i_mask_len)) <> 1 
    then 
      raise exception 'width of i_mask_len'' background should be 1.';
    elsif array_ndims(i_mask_len) = 2 
        and array_length(i_mask_len, 1) <> array_length(i_array, 1)
      or array_ndims(i_mask_len) = 3 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 2) <> array_length(i_array, 2)
            )
      or array_ndims(i_mask_len) = 4 
        and (array_length(i_mask_len, 1) <> array_length(i_array, 1) 
               or array_length(i_mask_len, 2) <> array_length(i_array, 2) 
               or array_length(i_mask_len, 3) <> array_length(i_array, 3)
            )
    then 
      raise exception 'unmatched length between i_mask_len and i_array';
    elsif v_background_width < any(i_mask_len)
    then
      raise exception 'there should not be any element of i_mask_len > v_background_width.';
    end if;
  end if;
  
  if array_ndims(i_mask_len) = 2
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , nullif(int4range(v_background_width - i_mask_len[a_cur_y][1] + 1, v_background_width + 1, '[)'), 'empty' :: int4range)
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
    );
    
  elsif array_ndims(i_mask_len) = 3
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , int4range(a_cur_x, a_cur_x, '[]')
          , nullif(int4range(v_background_width - i_mask_len[a_cur_y][a_cur_x][1] + 1, v_background_width + 1, '[)'), 'empty' :: int4range)
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
    );
    
  elsif array_ndims(i_mask_len) = 4 
  then 
    v_pos_range_s := 
    (
      select 
        array_agg
        (
          array 
          [
            int4range(a_cur_y, a_cur_y, '[]')
          , int4range(a_cur_x, a_cur_x, '[]')
          , int4range(a_cur_x3, a_cur_x3, '[]')
          , nullif(int4range(v_background_width - i_mask_len[a_cur_y][a_cur_x][a_cur_x3][1] + 1, v_background_width + 1, '[)'), 'empty' :: int4range)
          ]
        )
      from generate_series(1, array_length(i_mask_len, 1)) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_mask_len, 2)) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_mask_len, 3)) tb_a_cur_x3(a_cur_x3)
    );
      
  end if;
  
  return 
    sm_sc.fv_pos_replaces
    (
      i_array
    , v_pos_range_s
    , i_mask_element
    )
  ;
  
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_rmask
--   (
--     array
--     [
--       [1, 2, 3, 4, 5]
--     , [2, 3, 4, 5, 6]
--     , [3, 4, 5, 6, 7]
--     , [4, 5, 6, 7, 8]
--     ]
--   , array[[3], [0], [2], [1]]
--   , 0
--   )

-- select 
--   sm_sc.fv_rmask
--   (
--     array
--     [
--       [
--         [1, 2, 3, 4, 5]
--       , [2, 3, 4, 5, 6]
--       , [3, 4, 5, 6, 7]
--       , [4, 5, 6, 7, 8]
--       ]
--     , [
--         [-1, -2, -3, -4, -5]
--       , [-2, -3, -4, -5, -6]
--       , [-3, -4, -5, -6, -7]
--       , [-4, -5, -6, -7, -8]
--       ]
--     , [
--         [11, 12, 13, 14, 15]
--       , [12, 13, 14, 15, 16]
--       , [13, 14, 15, 16, 17]
--       , [14, 15, 16, 17, 18]
--       ]
--     ]
--   , array[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]
--   , 0
--   )

-- select 
--   sm_sc.fv_rmask
--   (
--     array
--     [
--       [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     , [
--         [
--           [1, 2, 3, 4, 5]
--         , [2, 3, 4, 5, 6]
--         , [3, 4, 5, 6, 7]
--         , [4, 5, 6, 7, 8]
--         ]
--       , [
--           [-1, -2, -3, -4, -5]
--         , [-2, -3, -4, -5, -6]
--         , [-3, -4, -5, -6, -7]
--         , [-4, -5, -6, -7, -8]
--         ]
--       , [
--           [11, 12, 13, 14, 15]
--         , [12, 13, 14, 15, 16]
--         , [13, 14, 15, 16, 17]
--         , [14, 15, 16, 17, 18]
--         ]
--       ]
--     ]
--   , array[[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]],[[[3], [0], [2], [1]], [[1], [1], [4], [2]], [[3], [5], [2], [1]]]]
--   , 0
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rpad(anyarray, anyarray, int);
create or replace function sm_sc.fv_rpad
(
  i_array            anyarray      ,
  i_fill_values      anyarray      , -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
  i_times            int     default   1  
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- set search_path to sm_sc;
  if i_array is null 
  then 
    return null;
  end if;
  
  -- if v_fill_values is null 
  -- then
  --   if array_ndims(i_array) = 1
  --   then 
  --     v_fill_values := array[nullif(i_array[1], i_array[1])];
  --   elsif 
  --   array_ndims(i_array) = 2
  --   then
  --     v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  --   elsif array_ndims(i_array) = 3
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  --   elsif array_ndims(i_array) = 4
  --   then 
  --     v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  --   end if;
  -- end if;

  if i_times <= 0
  then
    return i_array;
  elsif array_ndims(i_array) <> array_ndims(v_fill_values)
  then 
    raise exception 'ndims of i_array: % and i_fill_values: % should be the same.', array_ndims(i_array), array_ndims(v_fill_values);
  elsif array_ndims(i_array) = 1
  then 
    return
      i_array
      || 
      (
        select 
          sm_sc.fa_array_concat(v_fill_values)
        from generate_series(1, i_times)
      )
    ;
  elsif array_ndims(i_array) = 2
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
    then 
      raise exception 'height of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        i_array
        |||| 
        (
          select 
            sm_sc.fa_mx_concat_x(v_fill_values)
          from generate_series(1, i_times)
        )
      ;
    end if;
  elsif array_ndims(i_array) = 3
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1)) 
      or array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        i_array
        |||| 
        (
          select 
            sm_sc.fa_mx_concat_x3(v_fill_values)
          from generate_series(1, i_times)
        )
      ;
    end if;
  elsif array_ndims(i_array) = 4
  then 
    if array_length(v_fill_values, 1) not in (1, array_length(i_array, 1)) 
      or array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
      or array_length(v_fill_values, 3) not in (1, array_length(i_array, 3))
    then 
      raise exception 'width of 2d i_fill_values should be 1 or the same as 2d i_array''s. ';
    else
      return
        i_array
        |||| 
        (
          select 
            sm_sc.fa_mx_concat_x4(v_fill_values)
          from generate_series(1, i_times)
        )
      ;
    end if;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_rpad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     array[7, 8],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7], array[8]],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[array[1.0 :: float, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7, 8], array[9, 10]] :: float[],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[7, 8]],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     array[array[null :: int]],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]
--   , array[[[7, 8],[9, 0]],[[7, 8],[9, 0]],[[7, 8],[9, 0]]]
--   , 2
--   )
-- select sm_sc.fv_rpad
--   (
--     array[[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]]
--   , array[[[[7, 8],[9, 0]],[[7, 8],[9, 0]],[[7, 8],[9, 0]]]]
--   , 2
--   )

-- ------------------------------------------------------------

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_rpad(anyarray, int);
create or replace function sm_sc.fv_rpad
(
  i_array            anyarray      ,
  i_times            int    default  1
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if array_ndims(i_array) = 1
  then
    v_fill_values := array[nullif(i_array[1], i_array[1])];
  elsif array_ndims(i_array) = 2
  then 
    v_fill_values := array[[nullif(i_array[1][1], i_array[1][1])]];
  elsif array_ndims(i_array) = 3
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1]);
  elsif array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), 1]);
  end if;

  return sm_sc.fv_rpad(i_array, v_fill_values, i_times);

end
$$
language plpgsql stable
cost 100;


-- select sm_sc.fv_rpad
--   (
--     array[1, 2, 3, 4, 5, 6],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[array[1.0 :: float, 2, 3, 4, 5, 6], array[-1, -2, -3, -4, -5, -6]],
--     2
--   );
-- select sm_sc.fv_rpad
--   (
--     array[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]
--   , 2
--   )
-- select sm_sc.fv_rpad
--   (
--     array[[[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]],[[1, 2, 3, 4, 5, 6],[-1, -2, -3, -4, -5, -6]]]]
--   , 2
--   )

-- drop function if exists sm_sc.fv_rtrim(anyarray, anyelement);
create or replace function sm_sc.fv_rtrim
(
  i_array               anyarray,
  i_trim_element        anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur                 int  ;
  v_y_cur                 int  ;
  v_x3_cur                int  ;
  v_x4_cur                int  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- ([][])
  if array_ndims(i_array) =  2
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in reverse array_length(i_array, 2) .. 1
      loop 
        if i_array[v_y_cur][v_x_cur] <> i_trim_element
        then 
          exit;
        else 
          i_array[v_y_cur][v_x_cur] = null;
        end if;
      end loop;
    end loop;
    return i_array;

  -- []
  elsif array_ndims(i_array) = 1
  then
    for v_x_cur in reverse array_length(i_array, 1) .. 1
    loop 
      if i_array[v_x_cur] <> i_trim_element
      then 
        exit;
      else 
        i_array[v_x_cur] = null;
      end if;
    end loop;
    return i_array;

  -- ([][][])
  elsif array_ndims(i_array) =  3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        for v_x3_cur in reverse array_length(i_array, 3) .. 1
        loop 
          if i_array[v_y_cur][v_x_cur][v_x3_cur] <> i_trim_element
          then 
            exit;
          else 
            i_array[v_y_cur][v_x_cur][v_x3_cur] := null;
          end if;
        end loop;
      end loop;
    end loop;
    return i_array;

  -- ([][][][])
  elsif array_ndims(i_array) =  4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop 
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop 
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop 
          for v_x4_cur in reverse array_length(i_array, 4) .. 1
          loop 
            if i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] <> i_trim_element
            then 
              exit;
            else 
              i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := null;
            end if;
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_rtrim(array[[0,1,2,3,4,5, 0], [4,0,0,1,2,0, 0], [0,1,2,3,0,0, 0], [0,1,2, 0,0,0,0], [0,0,1,0,0,3, 0]], 0)
-- select sm_sc.fv_rtrim(array[0,1,2,3,4,5, 0, 0], 0)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_x(anyarray, int, int, int4range[], anyarray);
create or replace function sm_sc.fv_sample_x   -- 5 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int               ,                                 -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
  i_simp_ranges    int4range[]       ,                                 -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
  i_fill_values    anyarray                                            -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif v_fill_values is not null and i_array is not null and array_ndims(v_fill_values) <> array_ndims(i_array)
    then
      raise exception 'array_ndims of i_fill_values should be the same as i_array''s.  v_fill_values：%', v_fill_values;
    -- 审计填充数值矩阵的宽度
    elsif v_fill_values is not null 
      and 
      (
        array_ndims(v_fill_values) = 2 and array_length(v_fill_values, 1) not in (1, array_length(i_array, 1))
        or array_ndims(v_fill_values) = 3 
          and 
          (
            array_length(v_fill_values, 1) <> array_length(i_array, 1) 
            or array_length(v_fill_values, 3) <> array_length(i_array, 3)
          )
        or array_ndims(v_fill_values) = 4 
          and 
          (
            array_length(v_fill_values, 1) <> array_length(i_array, 1) 
            or array_length(v_fill_values, 3) <> array_length(i_array, 3)
            or array_length(v_fill_values, 4) <> array_length(i_array, 4)
          )
      )
    then
      raise exception 'non-y dim''s width of i_fill_values should the same as i_array''s. ';
    -- 审计采样周期窗口在末尾刚好完整覆盖，不会产生碎片
    elsif array_length(i_array, 2) % i_period > 0
    then
      raise warning 'the slide window with i_period: % and i_window_len: % is not intact at the tail of i_array len: %', i_period, i_window_len, array_length(i_array, 2);
    end if;
  end if;
  
  if v_fill_values is null and array_ndims(i_array) = 2
  then
    v_fill_values := array_fill(nullif(i_array[1][1], i_array[1][1]), array[array_length(i_array, 1), 1]);
  elsif v_fill_values is null and array_ndims(i_array) = 3
  then
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  elsif v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3), array_length(i_array, 4)]);
  end if;

  i_window_len := coalesce(i_window_len, least((select max(upper(a_range)) - 1 from unnest(i_simp_ranges) tb_a(a_range)), array_length(i_array, 2)));

  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  -- elsif array_ndims(i_array) = 1
  -- then
  --   return 
  --   (
  --     select
  --       sm_sc.fa_array_concat
  --       (
  --         (
  --           select 
  --             sm_sc.fa_array_concat(v_fill_values) 
  --           from 
  --             generate_series
  --             (
  --               lower(a_range * int4range(null::int, 0, '[]')), 
  --               upper(a_range * int4range(null::int, 0, '[]')) - 1
  --             ) tb_a_filler_upper(a_upper_idx)
  --         )
  --         ||
  --         sm_sc.fv_rpad
  --         (
  --           i_array
  --           [
  --             a_idx + lower(a_range * int4range(1, i_window_len, '[]'))
  --             : 
  --             a_idx + upper(a_range * int4range(1, i_window_len, '[]')) - 1
  --           ]
  --           , v_fill_values
  --           , upper(a_range * int4range(1, i_window_len, '[]')) - lower(a_range * int4range(1, i_window_len, '[]'))
  --         )
  --         ||
  --         (
  --           select 
  --             sm_sc.fa_array_concat(v_fill_values) 
  --           from 
  --             generate_series
  --             (
  --               lower(a_range * int4range(i_window_len + 1, null::int, '[]')),
  --               upper(a_range * int4range(i_window_len + 1, null::int, '[]')) - 1
  --             ) tb_a_filler_upper(a_upper_idx)
  --         )
  --         order by a_idx, a_no
  --       )
  --     from generate_series(0, array_length(i_array, 1) - 1, i_period) tb_a_idx(a_idx)    -- -- generate_series(0, array_length(i_array, 1) - i_window_len, i_period) tb_a_idx(a_idx)
  --       , (
  --           select 
  --             row_number() over() as a_no
  --             , a_range 
  --           from unnest(i_simp_ranges) tb_a_ranges(a_range)
  --         ) tb_a_range      
  --   )
  --   ;
  elsif array_ndims(i_array) between 2 and 4
  then
    return 	
    (
      select
        sm_sc.fa_mx_concat_x
        (
          sm_sc.fv_concat_x
          (
            sm_sc.fv_concat_x
            (
              (
                select 
                  sm_sc.fa_mx_concat_x(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(null::int, 0, '[]')), 
                    upper(a_range * int4range(null::int, 0, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )
              -- -- -- sm_sc.fv_concat_x() -- -- ||||
              
              -- -- sm_sc.fv_rpad
              -- -- (
              -- --   i_array[ : ]
              -- --   [
              -- --     a_idx + lower(a_range * int4range(1, i_window_len, '[]'))
              -- --     : 
              -- --     a_idx + upper(a_range * int4range(1, i_window_len, '[]')) - 1
              -- --   ]
              -- --   , v_fill_values
              -- --   , upper(a_range * int4range(1, i_window_len, '[]')) - lower(a_range * int4range(1, i_window_len, '[]'))
              -- -- )
            , i_array
              [ : ]
              [
                a_idx + lower(a_range * int4range(1, i_window_len + 1, '[)'))
                : 
                a_idx + upper(a_range * int4range(1, i_window_len + 1, '[)')) - 1
              ] -- [ : ]
            )
            -- -- -- sm_sc.fv_concat_x() -- -- ||||
          , sm_sc.fv_concat_x
            (
              sm_sc.fv_new
              (
                v_fill_values
              , array[1, greatest(0, upper(a_range * int4range(1, i_window_len + 1, '[)')) - greatest(lower(a_range * int4range(1, i_window_len + 1, '[)')), array_length(i_array, 2)))]
                || array_fill(1, array[array_ndims(i_array) - 2])
              )
              
              -- -- -- sm_sc.fv_concat_x() -- -- ||||
            , (
                select 
                  sm_sc.fa_mx_concat_x(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(least(i_window_len + 1, array_length(i_array, 2) - a_idx + 1), null::int, '[]')), 
                    upper(a_range * int4range(least(i_window_len + 1, array_length(i_array, 2) - a_idx + 1), null::int, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )
            )
          )
          order by a_idx, a_no
        )
      from generate_series(0, array_length(i_array, 2) - 1, i_period) tb_a_idx(a_idx)   -- -- generate_series(0, array_length(i_array, 2) - i_window_len, i_period) tb_a_idx(a_idx)
        , (
            select 
              row_number() over() as a_no
              , a_range 
            from unnest(i_simp_ranges) tb_a_ranges(a_range)
          ) tb_a_range    
    )
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;



-- drop function if exists sm_sc.fv_sample_x(anyarray, int, int, int4range[]);
create or replace function sm_sc.fv_sample_x   -- 2 至 4 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int           default null   ,                      -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
  i_simp_ranges    int4range[]   default array[int4range(1, 1, '[]')]  -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if v_fill_values is null and array_ndims(i_array) = 2
  then
    v_fill_values := array_fill(nullif(i_array[1][1], i_array[1][1]), array[array_length(i_array, 1), 1]);
  elsif v_fill_values is null and array_ndims(i_array) = 3
  then
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3)]);
  elsif v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), 1, array_length(i_array, 3), array_length(i_array, 4)]);
  end if;

  return sm_sc.fv_sample_x(i_array, i_period, i_window_len, i_simp_ranges, v_fill_values);

end
$$
language plpgsql stable
parallel safe
cost 100;







-- -- set search_path to sm_sc;
-- select sm_sc.fv_sample_x
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
--     , 3
--     , 2
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- select sm_sc.fv_sample_x
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
--     , 3
--     , 5
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- select sm_sc.fv_sample_x
--   (
--     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
--     , 3
--     , null
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- -- select sm_sc.fv_sample_x
-- --   (
-- --     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
-- --     , 3
-- --     , null :: int
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x
-- --   (
-- --     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
-- --     , 3
-- --   );

-- select sm_sc.fv_sample_x
--   (
--     array[array[1, 2, 3, 4.6, 5, 6], array[7, 8, 9, 10, 11, 12]] :: float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array[array[21, 22], array[23, 24]] :: float[]
--   );

-- select sm_sc.fv_sample_x
--   (
--     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array[[[1, -1]], [[3, -3]], [[5, -5]], [[7, -7]], [[9.7, -9]], [[11, -11]]]::float[]
--   );

-- select sm_sc.fv_sample_x
--   (
--     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     -- , array[[[1, -1]], [[3, -3]], [[5, -5]], [[7, -7]], [[9.7, -9]], [[11, -11]]]::float[]
--   );

-- select sm_sc.fv_sample_x
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array
--       [
--         [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--       , [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--       , [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--       , [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--       ]::float[]
--   );

-- select sm_sc.fv_sample_x
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     -- , array
--     --   [
--     --     [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--     --   , [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--     --   , [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--     --   , [[[1, -1], [2, -2]], [[3, -3], [4, -4]]]
--     --   ]::float[]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_x3(anyarray, int, int, int4range[], anyarray);
create or replace function sm_sc.fv_sample_x3   -- 5 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int               ,                                 -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
  i_simp_ranges    int4range[]       ,                                 -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
  i_fill_values    anyarray                                            -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 3 and 4
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif v_fill_values is not null and i_array is not null and array_ndims(v_fill_values) <> array_ndims(i_array)
    then
      raise exception 'array_ndims of i_fill_values should be the same as i_array''s.  v_fill_values：%', v_fill_values;
    -- 审计填充数值矩阵的宽度
    elsif v_fill_values is not null 
      and 
      (
        array_ndims(v_fill_values) = 3 
          and 
          (
            array_length(v_fill_values, 1) <> array_length(i_array, 1) 
            or array_length(v_fill_values, 2) <> array_length(i_array, 2)
          )
        or array_ndims(v_fill_values) = 4 
          and 
          (
            array_length(v_fill_values, 1) <> array_length(i_array, 1) 
            or array_length(v_fill_values, 2) <> array_length(i_array, 2)
            or array_length(v_fill_values, 4) <> array_length(i_array, 4)
          )
      )
    then
      raise exception 'non-y dim''s width of i_fill_values should the same as i_array''s. ';
    -- 审计采样周期窗口在末尾刚好完整覆盖，不会产生碎片
    elsif array_length(i_array, 3) % i_period > 0
    then
      raise warning 'the slide window with i_period: % and i_window_len: % is not intact at the tail of i_array len: %', i_period, i_window_len, array_length(i_array, 3);
    end if;
  end if;
  
  if v_fill_values is null and array_ndims(i_array) = 3
  then
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1]);
  elsif v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  end if;

  i_window_len := coalesce(i_window_len, least((select max(upper(a_range)) - 1 from unnest(i_simp_ranges) tb_a(a_range)), array_length(i_array, 3)));

  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) between 3 and 4
  then
    return 	
    (
      select
        sm_sc.fa_mx_concat_x3
        (
          sm_sc.fv_concat_x3
          (
            sm_sc.fv_concat_x3
            (
              (
                select 
                  sm_sc.fa_mx_concat_x3(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(null::int, 0, '[]')), 
                    upper(a_range * int4range(null::int, 0, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )          
            , i_array
              [ : ]
              [ : ]
              [
                a_idx + lower(a_range * int4range(1, i_window_len + 1, '[)'))
                : 
                a_idx + upper(a_range * int4range(1, i_window_len + 1, '[)')) - 1
              ] -- [ : ]
            )
          , sm_sc.fv_concat_x3
            (
              sm_sc.fv_new
              (
                v_fill_values
              , array[1, 1, greatest(0, upper(a_range * int4range(1, i_window_len + 1, '[)')) - greatest(lower(a_range * int4range(1, i_window_len + 1, '[)')), array_length(i_array, 3)))]
                || array_fill(1, array[array_ndims(i_array) - 3])
              )
            , (
                select 
                  sm_sc.fa_mx_concat_x3(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(least(i_window_len + 1, array_length(i_array, 3) - a_idx + 1), null::int, '[]')), 
                    upper(a_range * int4range(least(i_window_len + 1, array_length(i_array, 3) - a_idx + 1), null::int, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )
            )
          )
          order by a_idx, a_no
        )
      from generate_series(0, array_length(i_array, 3) - 1, i_period) tb_a_idx(a_idx)   -- -- generate_series(0, array_length(i_array, 3) - i_window_len, i_period) tb_a_idx(a_idx)
        , (
            select 
              row_number() over() as a_no
              , a_range 
            from unnest(i_simp_ranges) tb_a_ranges(a_range)
          ) tb_a_range    
    )
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;



-- drop function if exists sm_sc.fv_sample_x3(anyarray, int, int, int4range[]);
create or replace function sm_sc.fv_sample_x3   -- 2 至 4 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int           default null   ,                      -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
  i_simp_ranges    int4range[]   default array[int4range(1, 1, '[]')]  -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if v_fill_values is null and array_ndims(i_array) = 3
  then
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1]);
  elsif v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), 1, array_length(i_array, 4)]);
  end if;

  return sm_sc.fv_sample_x3(i_array, i_period, i_window_len, i_simp_ranges, v_fill_values);

end
$$
language plpgsql stable
parallel safe
cost 100;







-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_sample_x3
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
-- --     , 3
-- --     , 2
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x3
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
-- --     , 3
-- --     , 5
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x3
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
-- --     , 3
-- --     , null
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x3
-- --   (
-- --     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
-- --     , 3
-- --     , null :: int
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x3
-- --   (
-- --     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
-- --     , 3
-- --   );
   -- 
-- -- select sm_sc.fv_sample_x3
-- --   (
-- --     array[array[1, 2, 3, 4.6, 5, 6], array[7, 8, 9, 10, 11, 12]] :: float[]
-- --     , 3
-- --     , 3
-- --     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
-- --     , array[array[21, 22], array[23, 24]] :: float[]
-- --   );

-- select sm_sc.fv_sample_x3
--   (
--     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array[[[1], [-2]], [[3], [-4]], [[-5], [-6]], [[7], [8]], [[9.7], [10]], [[11], [-12]]]::float[]
--   );

-- select sm_sc.fv_sample_x3
--   (
--     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     -- , array[[[1], [-2]], [[3], [-4]], [[-5], [-6]], [[7], [8]], [[9.7], [10]], [[11], [-12]]]::float[]
--   );

-- select sm_sc.fv_sample_x3
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array
--       [
--         [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       , [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       , [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       , [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       ]::float[]
--   );

-- select sm_sc.fv_sample_x3
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array
--       [
--         [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       , [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       , [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       , [[[32, -42]], [[54, -64]], [[75, -85]], [[97, -97]], [[19.7, -19]], [[110, -116]]]
--       ]::float[]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_x3_py(float[], int, int, int4range, float);
create or replace function sm_sc.fv_sample_x3_py   -- 3 个参数
(
  i_array          float[]           
, i_period         int                                               -- 采样周期
, i_window_len     int                                               -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
, i_simp_range     int4range                                         -- 越界填充，类似于升采样，周期横向插入 i_fill_values 元素稀释原矩阵
                                                                     -- 如果下界小于窗口上边界，则每组样本上方填充 i_fill_values; 如果上界也小于窗口上边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 如果上界大于窗口下边界，那么下方填充 i_fill_values; 如果下界也大于 窗口下边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
, i_fill_value     float                                             -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns float[]
as
$$
declare
  v_range        int4range   := int4range(1, i_window_len, '[]') * i_simp_range;
  v_cnt_window   int         := ((array_length(i_array, 3) - upper(i_simp_range) + 1) / i_period)  + 1;  -- array_length(i_array, x3)
begin 
  if array_ndims(i_array) = 3
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x4
        (
          sm_sc.fv_concat_x4
          (
            array_fill
            (
              i_fill_value
            , array[array_length(i_array, 1), array_length(i_array, 2), v_cnt_window, least(i_window_len, lower(i_simp_range) - 1)]
            )
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                array_length(i_array, 1)
              , array_length(i_array, 2)
              , v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              ]
            , array
              [
                array_length(i_array, 2) * array_length(i_array, 3)
              , array_length(i_array, 3)
              , i_period
              , 1
              ]
            ))[ : ][ : ][ : ][lower(v_range) : (upper(v_range) - 1)]
          )
        , array_fill
          (
            i_fill_value
          , array[array_length(i_array, 1), array_length(i_array, 2), v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1)]
          )
        ))[ : ][ : ][ : ][ : i_window_len]
      , array
        [
          array_length(i_array, 1)
        , array_length(i_array, 2)
        , v_cnt_window
          * 
          i_window_len
        ]
      )
    ;
  elsif array_ndims(i_array) = 4
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x4
        (
          sm_sc.fv_concat_x4
          (
            array_fill
            (
              i_fill_value
            , array[array_length(i_array, 1), array_length(i_array, 2), v_cnt_window, least(i_window_len, lower(i_simp_range) - 1), array_length(i_array, 4)]
            )
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                array_length(i_array, 1)
              , array_length(i_array, 2)
              , v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              , array_length(i_array, 4)
              ]
            , array
              [
                array_length(i_array, 2) * array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 3) * array_length(i_array, 4)
              , i_period * array_length(i_array, 4)
              , array_length(i_array, 4)
              , 1
              ]
            ))[ : ][ : ][ : ][lower(v_range) : (upper(v_range) - 1)][ : ]
          )
        , array_fill
          (
            i_fill_value
          , array[array_length(i_array, 1), array_length(i_array, 2), v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1), array_length(i_array, 4)]
          )
        ))[ : ][ : ][ : ][ : i_window_len][ : ]
      , array
        [
          array_length(i_array, 1)
        , array_length(i_array, 2)
        , v_cnt_window
          * 
          i_window_len
        , array_length(i_array, 4)
        ]
      )
    ;
  end if;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,3,7]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_x3_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_x3(sm_sc.fv_concat_x3(a_arr[ : ][ : ][1: 1 + a_no - 1], a_arr[ : ][ : ][3:3 + a_no - 1]), a_arr[ : ][ : ][5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,3,7,5]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_x3_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_x3(sm_sc.fv_concat_x3(a_arr[ : ][ : ][1: 1 + a_no - 1], a_arr[ : ][ : ][3:3 + a_no - 1]), a_arr[ : ][ : ][5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,3,8]) as a_arr
-- )
-- select 
--   a_no
-- , array_dims(sm_sc.fv_sample_x3_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0))
-- from cte_arr
--   , generate_series(1, 5) tb_a(a_no)
-- 
-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,3,8,6]) as a_arr
-- )
-- select 
--   a_no
-- , array_dims(sm_sc.fv_sample_x3_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0))
-- from cte_arr
--   , generate_series(1, 5) tb_a(a_no)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_x4(anyarray, int, int, int4range[], anyarray);
create or replace function sm_sc.fv_sample_x4   -- 5 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int               ,                                 -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
  i_simp_ranges    int4range[]       ,                                 -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
  i_fill_values    anyarray                                            -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) <> 4
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif v_fill_values is not null and i_array is not null and array_ndims(v_fill_values) <> array_ndims(i_array)
    then
      raise exception 'array_ndims of i_fill_values should be the same as i_array''s.  v_fill_values：%', v_fill_values;
    -- 审计填充数值矩阵的宽度
    elsif v_fill_values is not null 
      and 
      (
        array_ndims(v_fill_values) = 4 
          and 
          (
            array_length(v_fill_values, 1) <> array_length(i_array, 1) 
            or array_length(v_fill_values, 2) <> array_length(i_array, 2)
            or array_length(v_fill_values, 3) <> array_length(i_array, 3)
          )
      )
    then
      raise exception 'non-y dim''s width of i_fill_values should the same as i_array''s. ';
    -- 审计采样周期窗口在末尾刚好完整覆盖，不会产生碎片
    elsif array_length(i_array, 4) % i_period > 0
    then
      raise warning 'the slide window with i_period: % and i_window_len: % is not intact at the tail of i_array len: %', i_period, i_window_len, array_length(i_array, 4);
    end if;
  end if;
  
  if v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 4), 1]);
  end if;

  i_window_len := coalesce(i_window_len, least((select max(upper(a_range)) - 1 from unnest(i_simp_ranges) tb_a(a_range)), array_length(i_array, 4)));

  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  -- elsif array_ndims(i_array) = 4
  -- then
  else
    return 	
    (
      select
        sm_sc.fa_mx_concat_x4
        (
          sm_sc.fv_concat_x4
          (
            sm_sc.fv_concat_x4
            (
              (
                select 
                  sm_sc.fa_mx_concat_x4(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(null::int, 0, '[]')), 
                    upper(a_range * int4range(null::int, 0, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )          
            , i_array
              [ : ]
              [ : ]
              [ : ]
              [
                a_idx + lower(a_range * int4range(1, i_window_len + 1, '[)'))
                : 
                a_idx + upper(a_range * int4range(1, i_window_len + 1, '[)')) - 1
              ]
            )
          , sm_sc.fv_concat_x4
            (
              sm_sc.fv_new
              (
                v_fill_values
              , array[1, 1, 1, greatest(0, upper(a_range * int4range(1, i_window_len + 1, '[)')) - greatest(lower(a_range * int4range(1, i_window_len + 1, '[)')), array_length(i_array, 4)))]
              )
            , (
                select 
                  sm_sc.fa_mx_concat_x4(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(least(i_window_len + 1, array_length(i_array, 4) - a_idx + 1), null::int, '[]')), 
                    upper(a_range * int4range(least(i_window_len + 1, array_length(i_array, 4) - a_idx + 1), null::int, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )
            )
          )
          order by a_idx, a_no
        )
      from generate_series(0, array_length(i_array, 4) - 1, i_period) tb_a_idx(a_idx)   -- -- generate_series(0, array_length(i_array, 4) - i_window_len, i_period) tb_a_idx(a_idx)
        , (
            select 
              row_number() over() as a_no
              , a_range 
            from unnest(i_simp_ranges) tb_a_ranges(a_range)
          ) tb_a_range    
    )
    ;
  -- else
  --   raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;



-- drop function if exists sm_sc.fv_sample_x4(anyarray, int, int, int4range[]);
create or replace function sm_sc.fv_sample_x4   -- 2 至 4 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int           default null   ,                      -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
  i_simp_ranges    int4range[]   default array[int4range(1, 1, '[]')]  -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), 1]);
  end if;

  return sm_sc.fv_sample_x4(i_array, i_period, i_window_len, i_simp_ranges, v_fill_values);

end
$$
language plpgsql stable
parallel safe
cost 100;







-- -- set search_path to sm_sc;
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
-- --     , 3
-- --     , 2
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
-- --     , 3
-- --     , 5
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[array[1, 2, 3, 4, 5, 6], array[7, 8, 9, 10, 11, 12]]
-- --     , 3
-- --     , null
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
-- --     , 3
-- --     , null :: int
-- --     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
-- --   );
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
-- --     , 3
-- --   );
   -- 
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[array[1, 2, 3, 4.6, 5, 6], array[7, 8, 9, 10, 11, 12]] :: float[]
-- --     , 3
-- --     , 3
-- --     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
-- --     , array[array[21, 22], array[23, 24]] :: float[]
-- --   );

-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
-- --     , 3
-- --     , 3
-- --     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
-- --     , array[[[1], [-2]], [[3], [-4]], [[-5], [-6]], [[7], [8]], [[9.7], [10]], [[11], [-12]]]::float[]
-- --   );
   -- 
-- -- select sm_sc.fv_sample_x4
-- --   (
-- --     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
-- --     , 3
-- --     , 3
-- --     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
-- --     -- , array[[[1], [-2]], [[3], [-4]], [[-5], [-6]], [[7], [8]], [[9.7], [10]], [[11], [-12]]]::float[]
-- --   );

-- select sm_sc.fv_sample_x4
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--   , 3
--   , 3
--   , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--   , array
--     [
--       [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--     , [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--     , [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--     , [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--     ]::float[]
--   );

-- select sm_sc.fv_sample_x4
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--   , 3
--   , 3
--   , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--   -- , array
--   --   [
--   --     [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--   --   , [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--   --   , [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--   --   , [[[1], [2]], [[3], [-4]], [[-5], [-6]], [[-7], [-8]], [[9.7], [10]], [[-11], [12]]]
--   --   ]::float[]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_x4_py(float[], int, int, int4range, float);
create or replace function sm_sc.fv_sample_x4_py   -- 3 个参数
(
  i_array          float[]           
, i_period         int                                               -- 采样周期
, i_window_len     int                                               -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
, i_simp_range     int4range                                         -- 越界填充，类似于升采样，周期横向插入 i_fill_values 元素稀释原矩阵
                                                                     -- 如果下界小于窗口上边界，则每组样本上方填充 i_fill_values; 如果上界也小于窗口上边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 如果上界大于窗口下边界，那么下方填充 i_fill_values; 如果下界也大于 窗口下边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
, i_fill_value     float                                             -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns float[]
as
$$
declare
  v_range        int4range   := int4range(1, i_window_len, '[]') * i_simp_range;
  v_cnt_window   int         := ((array_length(i_array, 4) - upper(i_simp_range) + 1) / i_period)  + 1;  -- array_length(i_array, x4)
begin 
  if array_ndims(i_array) = 4
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_py    -- fv_concat_x5
        (
          sm_sc.fv_concat_py   -- fv_concat_x5
          (
            array_fill
            (
              i_fill_value
            , array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), v_cnt_window, least(i_window_len, lower(i_simp_range) - 1)]
            )
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                array_length(i_array, 1)
              , array_length(i_array, 2)
              , array_length(i_array, 3)
              , v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              ]
            , array
              [
                array_length(i_array, 2) * array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 4)
              , i_period
              , 1
              ]
            ))[ : ][ : ][ : ][ : ][lower(v_range) : (upper(v_range) - 1)]
          , 5
          )
        , array_fill
          (
            i_fill_value
          , array[array_length(i_array, 1), array_length(i_array, 2), array_length(i_array, 3), v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1)]
          )
        , 5
        ))[ : ][ : ][ : ][ : ][ : i_window_len]
      , array
        [
          array_length(i_array, 1)
        , array_length(i_array, 2)
        , array_length(i_array, 3)
        , v_cnt_window
          * 
          i_window_len
        ]
      )
    ;
  end if;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,3,5,7]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_x4_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_x4(sm_sc.fv_concat_x4(a_arr[ : ][ : ][ : ][1: 1 + a_no - 1], a_arr[ : ][ : ][ : ][3:3 + a_no - 1]), a_arr[ : ][ : ][ : ][5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,3,6,8]) as a_arr
-- )
-- select 
--   a_no
-- , array_dims(sm_sc.fv_sample_x4_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0))
-- from cte_arr
--   , generate_series(1, 5) tb_a(a_no)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_x_py(float[], int, int, int4range, float);
create or replace function sm_sc.fv_sample_x_py   -- 3 个参数
(
  i_array          float[]           
, i_period         int                                               -- 采样周期
, i_window_len     int                                               -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
, i_simp_range     int4range                                         -- 越界填充，类似于升采样，周期横向插入 i_fill_values 元素稀释原矩阵
                                                                     -- 如果下界小于窗口上边界，则每组样本上方填充 i_fill_values; 如果上界也小于窗口上边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 如果上界大于窗口下边界，那么下方填充 i_fill_values; 如果下界也大于 窗口下边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
, i_fill_value     float                                             -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns float[]
as
$$
declare
  v_range        int4range   := int4range(1, i_window_len, '[]') * i_simp_range;
  v_cnt_window   int         := ((array_length(i_array, 2) - upper(i_simp_range) + 1) / i_period)  + 1;  -- array_length(i_array, x)
begin 
  if array_ndims(i_array) = 2
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x3
        (
          sm_sc.fv_concat_x3
          (
            array_fill
            (
              i_fill_value
            , array[array_length(i_array, 1), v_cnt_window, least(i_window_len, lower(i_simp_range) - 1)]
            )
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                array_length(i_array, 1)
              , v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              ]
            , array
              [
                array_length(i_array, 2)
              , i_period
              , 1
              ]
            ))[ : ][ : ][lower(v_range) : (upper(v_range) - 1)]
          )
        , array_fill
          (
            i_fill_value
          , array[array_length(i_array, 1), v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1)]
          )
        ))[ : ][ : ][ : i_window_len]
      , array
        [
          array_length(i_array, 1)
        , v_cnt_window
          * 
          i_window_len
        ]
      )
    ;
  elsif array_ndims(i_array) = 3
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x3
        (
          sm_sc.fv_concat_x3
          (
            array_fill
            (
              i_fill_value
            , array[array_length(i_array, 1), v_cnt_window, least(i_window_len, lower(i_simp_range) - 1), array_length(i_array, 3)]
            )
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                array_length(i_array, 1)
              , v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              , array_length(i_array, 3)
              ]
            , array
              [
                array_length(i_array, 2) * array_length(i_array, 3)
              , i_period * array_length(i_array, 3)
              , array_length(i_array, 3)
              , 1
              ]
            ))[ : ][ : ][lower(v_range) : (upper(v_range) - 1)][ : ]
          )
        , array_fill
          (
            i_fill_value
          , array[array_length(i_array, 1), v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1), array_length(i_array, 3)]
          )
        ))[ : ][ : ][ : i_window_len][ : ]
      , array
        [
          array_length(i_array, 1)
        , v_cnt_window                                                                  -- -- 
          * 
          i_window_len
        , array_length(i_array, 3)
        ]
      )
    ;
  elsif array_ndims(i_array) = 4
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x3
        (
          sm_sc.fv_concat_x3
          (
            array_fill
            (
              i_fill_value
            , array[array_length(i_array, 1), v_cnt_window, least(i_window_len, lower(i_simp_range) - 1), array_length(i_array, 3), array_length(i_array, 4)]
            )
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                array_length(i_array, 1)
              , v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              , array_length(i_array, 3)
              , array_length(i_array, 4)
              ]
            , array
              [
                array_length(i_array, 2) * array_length(i_array, 3) * array_length(i_array, 4)
              , i_period * array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 4)
              , 1
              ]
            ))[ : ][ : ][lower(v_range) : (upper(v_range) - 1)][ : ][ : ]
          )
        , array_fill
          (
            i_fill_value
          , array[array_length(i_array, 1), v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1), array_length(i_array, 3), array_length(i_array, 4)]
          )
        ))[ : ][ : ][ : i_window_len][ : ][ : ]
      , array
        [
          array_length(i_array, 1)
        , v_cnt_window                                                                  -- -- 
          * 
          i_window_len   
        , array_length(i_array, 3)
        , array_length(i_array, 4)
        ]
      )
    ;
  end if;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,7]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_x_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_x(sm_sc.fv_concat_x(a_arr[ : ][1: 1 + a_no - 1], a_arr[ : ][3:3 + a_no - 1]), a_arr[ : ][5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,7,3]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_x_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_x(sm_sc.fv_concat_x(a_arr[ : ][1: 1 + a_no - 1], a_arr[ : ][3:3 + a_no - 1]), a_arr[ : ][5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,7,3,5]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_x_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_x(sm_sc.fv_concat_x(a_arr[ : ][1: 1 + a_no - 1], a_arr[ : ][3:3 + a_no - 1]), a_arr[ : ][5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,8]) as a_arr
-- )
-- select 
--   a_no
-- , array_dims(sm_sc.fv_sample_x_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0))
-- from cte_arr
--   , generate_series(1, 5) tb_a(a_no)
-- 
-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,8,3]) as a_arr
-- )
-- select 
--   a_no
-- , array_dims(sm_sc.fv_sample_x_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0))
-- from cte_arr
--   , generate_series(1, 5) tb_a(a_no)
-- 
-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[2,8,3,6]) as a_arr
-- )
-- select 
--   a_no
-- , array_dims(sm_sc.fv_sample_x_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0))
-- from cte_arr
--   , generate_series(1, 5) tb_a(a_no)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_y(anyarray, int, int, int4range[], anyarray);
create or replace function sm_sc.fv_sample_y   -- 5 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int               ,                                 -- 1d 采样窗口的长度或 2d 采样的窗口高度。 null 值表示不约束窗口高度
  i_simp_ranges    int4range[]       ,                                 -- 越界填充，类似于升采样，周期横向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口上边界，则每组样本上方填充 i_fill_values; 如果上界也小于窗口上边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口下边界，那么下方填充 i_fill_values; 如果下界也大于 窗口下边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
  i_fill_values    anyarray        -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare
  v_fill_values    i_array%type  := i_fill_values;
begin
  -- 审计维度数量
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_array) not between 1 and 4
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif v_fill_values is not null and i_array is not null and array_ndims(v_fill_values) <> array_ndims(i_array)
    then
      raise exception 'array_ndims of i_fill_values should be the same as i_array''s.  v_fill_values：%', v_fill_values;
    -- 审计填充数值矩阵的宽度
    elsif v_fill_values is not null 
      and 
      (
        array_ndims(v_fill_values) = 2 and array_length(v_fill_values, 2) not in (1, array_length(i_array, 2))
        or array_ndims(v_fill_values) = 3 
          and 
          (
            array_length(v_fill_values, 2) <> array_length(i_array, 2) 
            or array_length(v_fill_values, 3) <> array_length(i_array, 3)
          )
        or array_ndims(v_fill_values) = 4 
          and 
          (
            array_length(v_fill_values, 2) <> array_length(i_array, 2) 
            or array_length(v_fill_values, 3) <> array_length(i_array, 3)
            or array_length(v_fill_values, 4) <> array_length(i_array, 4)
          )
      )
    then
      raise exception 'non-y dim''s width of i_fill_values should the same as i_array''s. ';
    -- 审计采样周期窗口在末尾刚好完整覆盖，不会产生碎片
    elsif array_length(i_array, 1) % i_period > 0
    then
      raise warning 'the slide window with i_period: % and i_window_len: % is not intact at the tail of i_array len: %', i_period, i_window_len, array_length(i_array, 1);
    end if;
  end if;
  
  if v_fill_values is null and array_ndims(i_array) = 1
  then
    v_fill_values := array[nullif(i_array[1], i_array[1])];
  elsif v_fill_values is null and array_ndims(i_array) = 2
  then
    v_fill_values := array_fill(nullif(i_array[1][1], i_array[1][1]), array[1, array_length(i_array, 2)]);
  elsif v_fill_values is null and array_ndims(i_array) = 3
  then
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[1, array_length(i_array, 2), array_length(i_array, 3)]);
  elsif v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[1, array_length(i_array, 2), array_length(i_array, 3), array_length(i_array, 4)]);
  end if;

  i_window_len := coalesce(i_window_len, least((select max(upper(a_range)) - 1 from unnest(i_simp_ranges) tb_a(a_range)), array_length(i_array, 1)));

  -- set search_path to sm_sc;
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) = 1
  then
    return 
    (
      select
        sm_sc.fa_array_concat
        (
          (
            select 
              sm_sc.fa_array_concat(v_fill_values) 
            from 
              generate_series
              (
                lower(a_range * int4range(null::int, 0, '[]')), 
                upper(a_range * int4range(null::int, 0, '[]')) - 1
              ) tb_a_filler_upper(a_upper_idx)
          )
          ||
          sm_sc.fv_bpad
          (
            i_array
            [
              a_idx + lower(a_range * int4range(1, i_window_len + 1, '[)'))
              : 
              a_idx + upper(a_range * int4range(1, i_window_len + 1, '[)')) - 1
            ]
            , v_fill_values
            , upper(a_range * int4range(1, i_window_len + 1, '[)')) - lower(a_range * int4range(1, i_window_len + 1, '[)'))
          )
          ||
          (
            select 
              sm_sc.fa_array_concat(v_fill_values) 
            from 
              generate_series
              (
                lower(a_range * int4range(least(i_window_len + 1, array_length(i_array, 1) - a_idx + 1), null::int, '[]')), 
                upper(a_range * int4range(least(i_window_len + 1, array_length(i_array, 1) - a_idx + 1), null::int, '[]')) - 1
              ) tb_a_filler_upper(a_upper_idx)
          )
          order by a_idx, a_no
        )
      from generate_series(0, array_length(i_array, 1) - 1, i_period) tb_a_idx(a_idx)  -- -- generate_series(0, array_length(i_array, 1) - i_window_len, i_period) tb_a_idx(a_idx)
        , (
            select 
              row_number() over() as a_no
              , a_range 
            from unnest(i_simp_ranges) tb_a_ranges(a_range)
          ) tb_a_range      
    )
    ;
  elsif array_ndims(i_array) between 2 and 4
  then
    return 	
    (
      select
        sm_sc.fa_mx_concat_y
        (
          sm_sc.fv_concat_y
          (
            sm_sc.fv_concat_y
            (
              (
                select 
                  sm_sc.fa_mx_concat_y(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(null::int, 0, '[]')), 
                    upper(a_range * int4range(null::int, 0, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )
              -- -- -- sm_sc.fv_concat_y() -- -- |-||
              
              -- -- sm_sc.fv_bpad -- -- -- 
              -- -- (
              -- --   i_array
              -- --   [
              -- --     a_idx + lower(a_range * int4range(1, i_window_len, '[]'))
              -- --     : 
              -- --     a_idx + upper(a_range * int4range(1, i_window_len, '[]')) - 1
              -- --   ] -- [ : ]
              -- --   , v_fill_values
              -- --   , upper(a_range * int4range(1, i_window_len, '[]')) - lower(a_range * int4range(1, i_window_len, '[]'))
              -- -- )
            , i_array
              [
                a_idx + lower(a_range * int4range(1, i_window_len + 1, '[)'))
                : 
                a_idx + upper(a_range * int4range(1, i_window_len + 1, '[)')) - 1
              ] -- [ : ]
            )
            -- -- -- sm_sc.fv_concat_y() -- -- |-||
          , sm_sc.fv_concat_y
            (
              sm_sc.fv_new
              (
                v_fill_values
                -- -- array[upper(a_range * int4range(1, i_window_len + 1, '[)')) - lower(a_range * int4range(1, i_window_len + 1, '[)')), 1, 1]
              , array[greatest(0, upper(a_range * int4range(1, i_window_len + 1, '[)')) - greatest(lower(a_range * int4range(1, i_window_len + 1, '[)')), array_length(i_array, 1)))]
                || array_fill(1, array[array_ndims(i_array) - 1])
              )
              
              -- -- -- sm_sc.fv_concat_y() -- -- |-||
            , (
                select 
                  sm_sc.fa_mx_concat_y(v_fill_values) 
                from 
                  generate_series
                  (
                    lower(a_range * int4range(least(i_window_len + 1, array_length(i_array, 1) - a_idx + 1), null::int, '[]')), 
                    upper(a_range * int4range(least(i_window_len + 1, array_length(i_array, 1) - a_idx + 1), null::int, '[]')) - 1
                  ) tb_a_filler_upper(a_upper_idx)
              )
            )
          )
          order by a_idx, a_no
        )
      from generate_series(0, array_length(i_array, 1) - 1, i_period) tb_a_idx(a_idx)   -- -- generate_series(0, array_length(i_array, 1) - i_window_len, i_period) tb_a_idx(a_idx)
        , (
            select 
              row_number() over() as a_no
              , a_range 
            from unnest(i_simp_ranges) tb_a_ranges(a_range)
          ) tb_a_range    
    )
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;



-- drop function if exists sm_sc.fv_sample_y(anyarray, int, int, int4range[]);
create or replace function sm_sc.fv_sample_y   -- 2 至 4 个参数
(
  i_array          anyarray          ,
  i_period         int               ,                                 -- 采样周期
  i_window_len     int           default null   ,                      -- 1d 采样窗口的长度或 2d 采样的窗口高度。 null 值表示不约束窗口高度
  i_simp_ranges    int4range[]   default array[int4range(1, 1, '[]')]  -- 越界填充，类似于升采样，周期纵向插入 i_fill_values 元素稀释原矩阵
                                                                       -- 如果下界小于窗口左边界，则每组样本左侧填充 i_fill_values; 如果上界也小于窗口左边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 如果上界大于窗口右边界，那么右侧填充 i_fill_values; 如果下界也大于窗口右边界，那么该 sim_range 和窗口无交集，抽样将全部是填充元素
                                                                       -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
)
returns anyarray
as
$$
declare 
  v_fill_values    i_array%type;
begin
  if v_fill_values is null and array_ndims(i_array) = 1
  then
    v_fill_values := array[nullif(i_array[1], i_array[1])];
  elsif v_fill_values is null and array_ndims(i_array) = 2
  then
    v_fill_values := array_fill(nullif(i_array[1][1], i_array[1][1]), array[1, array_length(i_array, 2)]);
  elsif v_fill_values is null and array_ndims(i_array) = 3
  then
    v_fill_values := array_fill(nullif(i_array[1][1][1], i_array[1][1][1]), array[1, array_length(i_array, 2), array_length(i_array, 3)]);
  elsif v_fill_values is null and array_ndims(i_array) = 4
  then 
    v_fill_values := array_fill(nullif(i_array[1][1][1][1], i_array[1][1][1][1]), array[1, array_length(i_array, 2), array_length(i_array, 3), array_length(i_array, 4)]);
  end if;

  return sm_sc.fv_sample_y(i_array, i_period, i_window_len, i_simp_ranges, v_fill_values);

end
$$
language plpgsql stable
parallel safe
cost 100;






-- -- set search_path to sm_sc;
-- select sm_sc.fv_sample_y
--   (
--     array[array[1, 2], array[3, 4], array[5, 6], array[7, 8], array[9, 10], array[11, 12]]
--     , 3
--     , 2
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- select sm_sc.fv_sample_y
--   (
--     array[array[1, 2], array[3, 4], array[5, 6], array[7, 8], array[9, 10], array[11, 12]]
--     , 3
--     , 5
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- select sm_sc.fv_sample_y
--   (
--     array[array[1, 2], array[3, 4], array[5, 6], array[7, 8], array[9, 10], array[11, 12]]
--     , 3
--     , null :: int
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- select sm_sc.fv_sample_y
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
--     , 3
--     , null :: int
--     , array[int4range(1,2, '[]'), int4range(2,4, '[]')]
--   );
-- select sm_sc.fv_sample_y
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
--     , 3
--   );

-- select sm_sc.fv_sample_y
--   (
--     array[array[1, 2], array[3, 4], array[5, 6], array[7, 8], array[9.7, 10], array[11, 12]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array[array[21, 22], array[23, 24]]::float[]
--   );

-- select sm_sc.fv_sample_y
--   (
--     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array[[[21, 22], [23, 24]],[[31, 32], [33, 34]]]::float[]
--   );

-- select sm_sc.fv_sample_y
--   (
--     array[[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     -- , array[[[21, 22], [23, 24]],[[31, 32], [33, 34]]]::float[]
--   );

-- select sm_sc.fv_sample_y
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     , array[[[[0.1, -0.1], [0.2, -0.2]], [[0.3, -0.3], [0.4, -0.4]], [[0.5, -0.5], [0.6, -0.6]], [[0.7, -0.7], [0.8, -0.8]], [[0.9, -0.9], [0.10, -0.10]], [[0.11, -0.11], [0.12, -0.12]]]]::float[]
--   );

-- select sm_sc.fv_sample_y
--   (
--     array
--     [
--       [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     , [[[1, -1], [2, -2]], [[3, -3], [4, -4]], [[5, -5], [6, -6]], [[7, -7], [8, -8]], [[9.7, -9], [10, -10]], [[11, -11], [12, -12]]]
--     ]::float[]
--     , 3
--     , 3
--     , array[int4range(-1,2, '[]'), int4range(-2,0, '[]'), int4range(3,5, '[]'), int4range(4,5, '[]')]
--     -- , array[[[[0.1, -0.1], [0.2, -0.2]], [[0.3, -0.3], [0.4, -0.4]], [[0.5, -0.5], [0.6, -0.6]], [[0.7, -0.7], [0.8, -0.8]], [[0.9, -0.9], [0.10, -0.10]], [[0.11, -0.11], [0.12, -0.12]]]]::float[]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sample_y_py(float[], int, int, int4range, float);
create or replace function sm_sc.fv_sample_y_py   -- 3 个参数
(
  i_array          float[]           
, i_period         int                                               -- 采样周期
, i_window_len     int                                               -- 1d 采样窗口的长度或 2d 采样的窗口宽度。 null 值表示不约束窗口宽度
, i_simp_range     int4range                                         -- 越界填充，类似于升采样，周期横向插入 i_fill_values 元素稀释原矩阵
                                                                     -- 如果下界小于窗口上边界，则每组样本上方填充 i_fill_values; 如果上界也小于窗口上边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 如果上界大于窗口下边界，那么下方填充 i_fill_values; 如果下界也大于 窗口下边界，那么该 sim_range 和原矩阵序号上下界无交集，抽样将全部是填充元素
                                                                     -- 其中超出上下界的数值代表 i_fill_values 重复填充次数，例如 -1 代表左侧重复填充 1 - (-1) = 2 次
, i_fill_value     float                                             -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns float[]
as
$$
declare
  v_range        int4range   := int4range(1, i_window_len, '[]') * i_simp_range;
  v_cnt_window   int         := ((array_length(i_array, 1) - upper(i_simp_range) + 1) / i_period)  + 1;  -- array_length(i_array, y)
begin 
  if array_ndims(i_array) = 1
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x                                                              
        (
          sm_sc.fv_concat_x                                                             
          (
            array_fill
            (
              i_fill_value
            , array[v_cnt_window, least(i_window_len, lower(i_simp_range) - 1)]
            )                                                                           
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                v_cnt_window    --  采样维度下，窗口滑动次数                            
              , upper(i_simp_range) - 1
              ]
            , array
              [
                i_period
              , 1
              ]
            ))[ : ][lower(v_range) : (upper(v_range) - 1)]                              
          )
        , array_fill
          (
            i_fill_value
          , array[v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1)]
          )                                                                             
        ))[ : ][ : i_window_len]
      , array
        [
          v_cnt_window                                                                  
          * 
          i_window_len                                                                  
        ]
      )
    ;
  elsif array_ndims(i_array) = 2
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x                                                             
        (
          sm_sc.fv_concat_x                                                            
          (
            array_fill
            (
              i_fill_value
            , array[v_cnt_window, least(i_window_len, lower(i_simp_range) - 1), array_length(i_array, 2)]
            )                                 
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              , array_length(i_array, 2)
              ]
            , array
              [
                i_period * array_length(i_array, 2)
              , array_length(i_array, 2)
              , 1
              ]
            ))[ : ][lower(v_range) : (upper(v_range) - 1)][ : ] 
          )
        , array_fill
          (
            i_fill_value
          , array[v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1), array_length(i_array, 2)]
          )                            
        ))[ : ][ : i_window_len][ : ]
      , array
        [
          v_cnt_window
          * 
          i_window_len
        , array_length(i_array, 2)
        ]
      )
    ;
  elsif array_ndims(i_array) = 3
  then 
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x                                                              
        (
          sm_sc.fv_concat_x                                                             
          (
            array_fill
            (
              i_fill_value
            , array[v_cnt_window, least(i_window_len, lower(i_simp_range) - 1), array_length(i_array, 2), array_length(i_array, 3)]
            )                                 
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              , array_length(i_array, 2)
              , array_length(i_array, 3)
              ]
            , array
              [
                i_period * array_length(i_array, 2) * array_length(i_array, 3)
              , array_length(i_array, 2) * array_length(i_array, 3)
              , array_length(i_array, 3)
              , 1
              ]
            ))[ : ][lower(v_range) : (upper(v_range) - 1)][ : ][ : ] 
          )
        , array_fill
          (
            i_fill_value
          , array[v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1), array_length(i_array, 2), array_length(i_array, 3)]
          )                            
        ))[ : ][ : i_window_len][ : ][ : ]
      , array
        [
          v_cnt_window                                                                  -- -- 
          * 
          i_window_len 
        , array_length(i_array, 2)
        , array_length(i_array, 3)
        ]
      )
    ;
  elsif array_ndims(i_array) = 4
  then   
    return 
      sm_sc.fv_opr_reshape_py
      (
        (sm_sc.fv_concat_x                                                    
        (
          sm_sc.fv_concat_x                                                    
          (
            array_fill
            (
              i_fill_value
            , array[v_cnt_window, least(i_window_len, lower(i_simp_range) - 1), array_length(i_array, 2), array_length(i_array, 3), array_length(i_array, 4)]
            )                                 
          , (sm_sc.fv_strided_float_py
            (
              i_array
            , array
              [
                v_cnt_window    --  采样维度下，窗口滑动次数
              , upper(i_simp_range) - 1
              , array_length(i_array, 2)
              , array_length(i_array, 3)
              , array_length(i_array, 4)
              ]
            , array
              [
                i_period * array_length(i_array, 2) * array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 2) * array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 3) * array_length(i_array, 4)
              , array_length(i_array, 4)
              , 1
              ]
            ))[ : ][lower(v_range) : (upper(v_range) - 1)][ : ][ : ][ : ]
          )
        , array_fill
          (
            i_fill_value
          , array[v_cnt_window, greatest(0, i_window_len - upper(i_simp_range) + 1), array_length(i_array, 2), array_length(i_array, 3), array_length(i_array, 4)]
          )          
        ))[ : ][ : i_window_len][ : ][ : ][ : ]
      , array
        [
          v_cnt_window                                                                  -- -- 
          * 
          i_window_len
        , array_length(i_array, 2)
        , array_length(i_array, 3)
        , array_length(i_array, 4)
        ]
      )
    ;
  end if;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7], 2, 3, int4range(1, 3, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7], 2, 2, int4range(1, 3, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7], 2, 5, int4range(1, 3, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7,8], 2, 5, int4range(2, 4, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7,8], 2, 4, int4range(2, 4, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7,8], 2, 3, int4range(2, 4, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7,8], 2, 2, int4range(2, 4, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[1,2,3,4,5,6,7,8], 2, 1, int4range(2, 4, '[]'), 0.0)
-- select sm_sc.fv_sample_y_py(array[[1,-1],[2,-2],[3,-3],[4,-4],[5,-5],[6,-6],[7,-7]], 2, int4range(1, 3, '[]'))

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[7,2]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_y_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_y(sm_sc.fv_concat_y(a_arr[1: 1 + a_no - 1], a_arr[3:3 + a_no - 1]), a_arr[5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[7,2,3]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_y_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_y(sm_sc.fv_concat_y(a_arr[1: 1 + a_no - 1], a_arr[3:3 + a_no - 1]), a_arr[5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- with cte_arr as 
-- (
--    select sm_sc.fv_new_rand(array[7,2,3,5]) as a_arr
-- )
-- select 
--   sm_sc.fv_sample_y_py(a_arr, 2, a_no, int4range(1, 3, '[]'), 0.0)
--   =
--   sm_sc.fv_concat_y(sm_sc.fv_concat_y(a_arr[1: 1 + a_no - 1], a_arr[3:3 + a_no - 1]), a_arr[5:5 + a_no - 1])
-- from cte_arr
--   , generate_series(1, 3) tb_a(a_no)

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_slice_x(anyarray, int4range[]);
create or replace function sm_sc.fv_slice_x
(
  i_arr            anyarray,
  i_slice_range    int4range[]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_length(i_arr, 2) < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for 2d len of i_arr.';
    end if;
  end if;
  
  return 
  (
    select 
      sm_sc.fa_mx_concat_x
      (
        i_arr[ : ][lower(a_range) : upper(a_range) - 1]
      )
    from unnest(i_slice_range) tb_a_range(a_range)
  );
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_slice_x
--   (
--     array[[[[1,2,3,4]],[[11,12,13,14]],[[5,6,7,8]],[[15,16,17,18]]]
--          ,[[[21,22,23,24]],[[31,32,33,34]],[[25,26,27,28]],[[35,36,37,38]]]
--          ,[[[41,42,43,44]],[[61,62,63,64]],[[45,46,47,48]],[[65,66,67,68]]]
--          ,[[[51,52,53,54]],[[71,72,73,74]],[[55,56,57,58]],[[75,76,77,78]]]
--          ]
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_slice_x3(anyarray, int4range[]);
create or replace function sm_sc.fv_slice_x3
(
  i_arr            anyarray,
  i_slice_range    int4range[]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_length(i_arr, 3) < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for 3d len of i_arr.';
    end if;
  end if;
  
  return 
  (
    select 
      sm_sc.fa_mx_concat_x3
      (
        i_arr[ : ][ : ][lower(a_range) : upper(a_range) - 1]
      )
    from unnest(i_slice_range) tb_a_range(a_range)
  );
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_slice_x3
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[5,6,7,8],[15,16,17,18]]]
--          ,[[[21,22,23,24],[31,32,33,34],[25,26,27,28],[35,36,37,38]]]
--          ,[[[41,42,43,44],[61,62,63,64],[45,46,47,48],[65,66,67,68]]]
--          ,[[[51,52,53,54],[71,72,73,74],[55,56,57,58],[75,76,77,78]]]
--          ]
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_slice_x4(anyarray, int4range[]);
create or replace function sm_sc.fv_slice_x4
(
  i_arr            anyarray,
  i_slice_range    int4range[]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_length(i_arr, 4) < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for 4d len of i_arr.';
    end if;
  end if;
  
  return 
  (
    select 
      sm_sc.fa_mx_concat_x4
      (
        i_arr[ : ][ : ][ : ][lower(a_range) : upper(a_range) - 1]
      )
    from unnest(i_slice_range) tb_a_range(a_range)
  );
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_slice_x4
--   (
--     array[[[[1,2,3,4],[11,12,13,14],[5,6,7,8],[15,16,17,18]]]
--          ,[[[21,22,23,24],[31,32,33,34],[25,26,27,28],[35,36,37,38]]]
--          ,[[[41,42,43,44],[61,62,63,64],[45,46,47,48],[65,66,67,68]]]
--          ,[[[51,52,53,54],[71,72,73,74],[55,56,57,58],[75,76,77,78]]]
--          ]
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_slice_y(anyarray, int4range[]);
create or replace function sm_sc.fv_slice_y
(
  i_arr            anyarray,
  i_slice_range    int4range[]
)
returns anyarray
as
$$
-- declare 
begin
  -- set search_path to sm_sc;
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if array_length(i_arr, 1) < (select max(upper(a_range)) - 1 from unnest(i_slice_range) tb_a_range(a_range))
      or 1 > (select min(lower(a_range)) from unnest(i_slice_range) tb_a_range(a_range))
    then
      raise exception 'overflow range for 1d len of i_arr.';
    end if;
  end if;
  
  return 
  (
    select 
      sm_sc.fa_mx_concat_y
      (
        i_arr[lower(a_range) : upper(a_range) - 1]
      )
    from unnest(i_slice_range) tb_a_range(a_range)
  );
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;

-- select sm_sc.fv_slice_y
--   (
--     array[[[[1,2,3,4],[11,12,13,14]],[[5,6,7,8],[15,16,17,18]]]
--          ,[[[21,22,23,24],[31,32,33,34]],[[25,26,27,28],[35,36,37,38]]]
--          ,[[[41,42,43,44],[61,62,63,64]],[[45,46,47,48],[65,66,67,68]]]
--          ,[[[51,52,53,54],[71,72,73,74]],[[55,56,57,58],[75,76,77,78]]]
--          ]
--   , array[int4range(1, 3, '[]'), int4range(2, 3, '[]')]
--   );
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_strided_float_py(float[], int[], int[]);
create or replace function sm_sc.fv_strided_float_py
(
  i_arr            float[]
, i_shape          int[]   -- 如果是要获得 2d-卷积 的滑动窗口驻留痕迹，那么返回卷积块儿矩阵的维数，比 i_arr 的维数多 2
, i_ele_strides    int[]   -- 以二维 i_arr[i_arr_heigh][i_arr_width], 
                           -- 窗口高宽 [i_window_height, i_window_width]
                           -- 步长高宽 = [i_stride_height, i_stride_width]，
                           -- 间隔卷积的间隔(空洞)高宽 = [i_hole_heigh, i_hole_width] 为例子，
                           -- 以 float 元素类型长度为单位：
                           -- i_ele_strides int[4] = 
                           --   array
                           --   [
                           --     i_stride_height * i_arr_width
                           --   , i_stride_width
                           --   , (1 + i_hole_heigh) * i_arr_width
                           --   , 1 + i_hole_width
                           --   ]
)
returns float[]
as
$$
  from numpy.lib.stride_tricks import as_strided
  from numpy import array

  v_len_ele = array(i_arr[0]).dtype.itemsize    # python float64 dtype.itemsize = 8
  return as_strided(array(i_arr), shape = i_shape, strides = v_len_ele * array(i_ele_strides)).tolist()
$$
language plpython3u stable
parallel safe
;

-- select 
--   sm_sc.fv_strided_float_py
--   (
--     array[[1,2,3,4],[5,6,7,8],[9,10,11,12],[13,14,15,16]]  -- sm_sc.fv_new_rand(array[5, 7])
--   , array[2, 2, 3, 3]                                      -- array[3, 5, 3, 3]
--   , array[4, 1, 4, 1]
--   )
-- drop function if exists sm_sc.fv_trim(anyarray, anyelement);
create or replace function sm_sc.fv_trim
(
  i_array               anyarray,
  i_trim_element        anyelement
)
returns anyarray
as
$$
declare -- here
  v_y_cur                 int  ;
  v_x_cur                 int  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- ([][]), ([][][]), ([][][][])
  if array_ndims(i_array) between 2 and 4
  then
    return sm_sc.fv_ltrim(sm_sc.fv_rtrim(sm_sc.fv_atrim(sm_sc.fv_btrim(i_array, i_trim_element), i_trim_element), i_trim_element), i_trim_element);

  -- []
  elsif array_ndims(i_array) = 1
  then
    return sm_sc.fv_ltrim(sm_sc.fv_rtrim(i_array, i_trim_element), i_trim_element);

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_trim(array[[2,0,0,0,0,0], [1,2,0,4,0,0], [0,0,1,0,2,0], [0,1,2,0,0,0], [1,0,1,0,0,3]], 0)
-- select sm_sc.fv_trim(array[0,1,2,3,4,5, 0, 0], 0)
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_upper_tri_mx(anyarray, anyelement);
create or replace function sm_sc.fv_upper_tri_mx
(
  i_array            anyarray      ,
  i_lower_fill_value anyelement        -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare 
  v_lower_fill_value    i_lower_fill_value%type  := i_lower_fill_value;
  v_array_heigh         int                      := array_length(i_array, array_ndims(i_array) - 1);
  v_array_width         int                      := array_length(i_array, array_ndims(i_array));
  v_array_len_y         int                      := array_length(i_array, 1);
  v_array_len_x         int                      := array_length(i_array, 2);
begin
  -- set search_path to sm_sc;
  
  if i_array is null 
  then 
    return null;
  end if;
  
  if v_lower_fill_value is null and array_ndims(i_array) = 1
  then 
    v_lower_fill_value := nullif(i_array[1], i_array[1]);
  elsif v_lower_fill_value is null and array_ndims(i_array) = 2
  then
    v_lower_fill_value := nullif(i_array[1][1], i_array[1][1]);
  elsif v_lower_fill_value is null and array_ndims(i_array) = 3
  then
    v_lower_fill_value := nullif(i_array[1][1][1], i_array[1][1][1]);
  elsif v_lower_fill_value is null and array_ndims(i_array) = 4
  then
    v_lower_fill_value := nullif(i_array[1][1][1][1], i_array[1][1][1][1]);
  end if;

  if array_ndims(i_array) = 2
  then 
    for v_cur in 1 .. case when v_array_width >= v_array_heigh then least(v_array_heigh, v_array_width) - 1 else least(v_array_heigh, v_array_width) end
    loop 
      i_array
        [v_cur + 1 : ] 
        [v_cur : v_cur]
      := 
        array_fill
        (
          v_lower_fill_value
        , array
          [
            v_array_heigh - v_cur
          , 1
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 3
  then 
    for v_cur in 1 .. case when v_array_width >= v_array_heigh then least(v_array_heigh, v_array_width) - 1 else least(v_array_heigh, v_array_width) end
    loop 
      i_array
        [ : ]
        [v_cur + 1 : ] 
        [v_cur : v_cur]
      := 
        array_fill
        (
          v_lower_fill_value
        , array
          [
            v_array_len_y
          , v_array_heigh - v_cur
          , 1
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 4
  then 
    for v_cur in 1 .. case when v_array_width >= v_array_heigh then least(v_array_heigh, v_array_width) - 1 else least(v_array_heigh, v_array_width) end
    loop 
      i_array
        [ : ]
        [ : ]
        [v_cur + 1 : ] 
        [v_cur : v_cur]
      := 
        array_fill
        (
          v_lower_fill_value
        , array
          [
            v_array_len_y
          , v_array_len_x
          , v_array_heigh - v_cur
          , 1
          ]
        )
      ;
    end loop;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
  
  return i_array;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   , 0
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_upper_tri_mx(anyarray);
create or replace function sm_sc.fv_upper_tri_mx
(
  i_array            anyarray
)
returns anyarray
as
$$
-- declare 
begin
  return sm_sc.fv_upper_tri_mx(i_array, null);
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   );

-- set search_path to sm_sc;
-- 相比 fv_upper_tri_mx, fv_upper_tri_mx_ex 的入参对角线也被替换
-- drop function if exists sm_sc.fv_upper_tri_mx_ex(anyarray, anyelement);
create or replace function sm_sc.fv_upper_tri_mx_ex
(
  i_array            anyarray      ,
  i_lower_fill_value anyelement        -- 如果用 default 值，那么参数的伪类型不易支持，那么分为两个入参不同的重载函数
)
returns anyarray
as
$$
declare 
  v_lower_fill_value    i_lower_fill_value%type  := i_lower_fill_value;
  v_array_heigh         int                      := array_length(i_array, array_ndims(i_array) - 1);
  v_array_width         int                      := array_length(i_array, array_ndims(i_array));
  v_array_len_y         int                      := array_length(i_array, 1);
  v_array_len_x         int                      := array_length(i_array, 2);
begin
  -- set search_path to sm_sc;
  
  if i_array is null 
  then 
    return null;
  end if;
  
  if v_lower_fill_value is null and array_ndims(i_array) = 1
  then 
    v_lower_fill_value := nullif(i_array[1], i_array[1]);
  elsif v_lower_fill_value is null and array_ndims(i_array) = 2
  then
    v_lower_fill_value := nullif(i_array[1][1], i_array[1][1]);
  elsif v_lower_fill_value is null and array_ndims(i_array) = 3
  then
    v_lower_fill_value := nullif(i_array[1][1][1], i_array[1][1][1]);
  elsif v_lower_fill_value is null and array_ndims(i_array) = 4
  then
    v_lower_fill_value := nullif(i_array[1][1][1][1], i_array[1][1][1][1]);
  end if;

  if array_ndims(i_array) = 2
  then 
    for v_cur in 1 .. least(v_array_heigh, v_array_width)
    loop 
      i_array
        [v_cur : ] 
        [v_cur : v_cur]
      := 
        array_fill
        (
          v_lower_fill_value
        , array
          [
            v_array_heigh - v_cur + 1
          , 1
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 3
  then 
    for v_cur in 1 .. least(v_array_heigh, v_array_width)
    loop 
      i_array
        [ : ]
        [v_cur : ] 
        [v_cur : v_cur]
      := 
        array_fill
        (
          v_lower_fill_value
        , array
          [
            v_array_len_y
          , v_array_heigh - v_cur + 1
          , 1
          ]
        )
      ;
    end loop;
  elsif array_ndims(i_array) = 4
  then 
    for v_cur in 1 .. least(v_array_heigh, v_array_width)
    loop 
      i_array
        [ : ]
        [ : ]
        [v_cur : ] 
        [v_cur : v_cur]
      := 
        array_fill
        (
          v_lower_fill_value
        , array
          [
            v_array_len_y
          , v_array_len_x
          , v_array_heigh - v_cur + 1
          , 1
          ]
        )
      ;
    end loop;
  else 
    raise exception 'no method for ndims = %.', array_ndims(i_array);
  end if;
  
  return i_array;
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   , 0
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   , 0
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_upper_tri_mx_ex(anyarray);
create or replace function sm_sc.fv_upper_tri_mx_ex
(
  i_array            anyarray
)
returns anyarray
as
$$
-- declare 
begin
  return sm_sc.fv_upper_tri_mx_ex(i_array, null);
end
$$
language plpgsql stable
cost 100;

-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array[[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array[[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array[[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     , [
--         [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       , [[1, 2, 3, 4, 5, 6], [-1, -2, -3, -4, -5, -6], [-1, 2, -3, 4, 5, -6], [1, -2, 3, 4, -5, 6]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     , [
--         [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       , [[1, 2, 3], [-1, -2, -3], [-1, 2, -3], [1, -2, 3]]
--       ]
--     ]
--   );
-- select 
--   sm_sc.fv_upper_tri_mx_ex
--   (
--     array
--     [
--       [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     , [
--         [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       , [[1, 2, 3, 4], [-1, -2, -3, 4], [-1, 2, -3, -4], [1, -2, 3, -4]]
--       ]
--     ]
--   );

-- drop function if exists sm_sc.fv_d_acos(float[][]);
create or replace function sm_sc.fv_d_acos
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     -` (/` ((1.0 :: float-` (i_indepdt_var ^` 2.0 :: float)) ^` 0.5 :: float))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_acos(array[[0.8, 0.6], [0.4, -0.6]])
-- drop function if exists sm_sc.fv_d_acosh(float[][]);
create or replace function sm_sc.fv_d_acosh
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     /` (((i_indepdt_var ^` 2.0 :: float) -` 1.0 :: float) ^` 0.5 :: float)
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_acosh(array[[1.8, 1.6], [1.4, 2.6]])
-- drop function if exists sm_sc.fv_d_acot(float[][]);
create or replace function sm_sc.fv_d_acot
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     -` (/` (1.0 :: float+` (i_indepdt_var ^` 2.0 :: float)))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_acot(array[[0.8, 0.6], [0.4, -0.6]])
-- drop function if exists sm_sc.fv_d_acsc(float[][]);
create or replace function sm_sc.fv_d_acsc
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     -` (/` (i_indepdt_var *` (((i_indepdt_var ^` 2.0 :: float) -` 1.0 :: float) ^` 0.5 :: float)))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_acsc(array[[1.8, 3.6], [2.4, 1.6]])
-- drop function if exists sm_sc.fv_d_asec(float[][]);
create or replace function sm_sc.fv_d_asec
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     /` (i_indepdt_var *` (((i_indepdt_var ^` 2.0 :: float) -` 1.0 :: float) ^` 0.5 :: float))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_asec(array[[1.8, 3.6], [2.4, 1.6]])
-- drop function if exists sm_sc.fv_d_asin(float[][]);
create or replace function sm_sc.fv_d_asin
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     /` ((1.0 :: float-` (i_indepdt_var ^` 2.0 :: float)) ^` 0.5 :: float)
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_asin(array[[0.8, 0.6], [0.4, -0.6]])
-- drop function if exists sm_sc.fv_d_asinh(float[][]);
create or replace function sm_sc.fv_d_asinh
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     /` (((i_indepdt_var ^` 2.0 :: float) +` 1.0 :: float) ^` 0.5 :: float)
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_asinh(array[[0.8, 0.6], [0.4, -0.6]])
-- drop function if exists sm_sc.fv_d_atan(float[][]);
create or replace function sm_sc.fv_d_atan
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     /` (1.0 :: float+` (i_indepdt_var ^` 2.0 :: float))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_atan(array[[0.8, 0.6], [0.4, -0.6]])
-- drop function if exists sm_sc.fv_d_atanh(float[][]);
create or replace function sm_sc.fv_d_atanh
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     /` (1.0 :: float-` (i_indepdt_var ^` 2.0 :: float))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_atanh(array[[1.8, 1.6], [1.4, 2.6]])
-- drop function if exists sm_sc.fv_d_cos(float[][]);
create or replace function sm_sc.fv_d_cos
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
    -` sm_sc.fv_sin(i_indepdt_var)::float[]
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_cos(array[[1.8, 4.6], [1.4, 3.6]])
-- drop function if exists sm_sc.fv_d_cosh(float[][]);
create or replace function sm_sc.fv_d_cosh
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     sm_sc.fv_sinh(i_indepdt_var)::float[]
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_cosh(array[[1.8, 3.6], [2.4, 1.6]])
-- drop function if exists sm_sc.fv_d_cot(float[][]);
create or replace function sm_sc.fv_d_cot
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
    -` (/` (sm_sc.fv_sin(i_indepdt_var)::float[][] ^` 2.0 :: float))
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_cot(array[[1.8, 4.6], [1.4, 3.6]])
-- drop function if exists sm_sc.fv_d_csc(float[][], float[][]);
create or replace function sm_sc.fv_d_csc
(
  i_indepdt_var float[][] , 
  i_depdt_var   float[][] default null
)
returns float[][]
as
$$
declare 
begin
  return
    case 
      when i_depdt_var is not null
        then (-` i_depdt_var) *` sm_sc.fv_cot(i_indepdt_var)::float[][]
      else -` (sm_sc.fv_cot(i_indepdt_var)::float[][] /` sm_sc.fv_sin(i_indepdt_var)::float[][])      -- 优先使用 i_depdt_var     -- -- sm_sc.fv_nullif(... , 0.0 :: float)
    end  
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_csc(array[[0.8, -0.6], [0.4, 0.3]])

-- select 
--   sm_sc.fv_d_csc
--   (
--     array[[0.8, -0.6], [0.4, 0.3]],
--     sm_sc.fv_csc(array[[0.8, -0.6], [0.4, 0.3]])
--   )

-- drop function if exists sm_sc.fv_d_sec(float[][], float[][]);
create or replace function sm_sc.fv_d_sec
(
  i_indepdt_var float[][] , 
  i_depdt_var   float[][] default null
)
returns float[][]
as
$$
declare 
begin
  return
    case 
      when i_depdt_var is not null
        then i_depdt_var *` sm_sc.fv_tan(i_indepdt_var)::float[][]
      else
        sm_sc.fv_tan(i_indepdt_var)::float[][] /` sm_sc.fv_cos(i_indepdt_var)::float[][]     -- 优先使用 i_depdt_var     -- -- sm_sc.fv_nullif(... , 0.0 :: float)
    end     
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_sec(array[[0.8, -0.6], [0.4, 0.3]])

-- select 
--   sm_sc.fv_d_sec
--   (
--     array[[1.8, -4.6], [1.4, 3.6]],
--     sm_sc.fv_sec(array[[0.8, -0.6], [0.4, 0.3]])
--   )

-- drop function if exists sm_sc.fv_d_sin(float[][]);
create or replace function sm_sc.fv_d_sin
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
    sm_sc.fv_cos(i_indepdt_var)::float[]
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_sin(array[[1.8, 4.6], [1.4, 3.6]])
-- drop function if exists sm_sc.fv_d_sinh(float[][]);
create or replace function sm_sc.fv_d_sinh
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
     sm_sc.fv_cosh(i_indepdt_var)::float[]
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_sinh(array[[1.8, 3.6], [2.4, 1.6]])
-- drop function if exists sm_sc.fv_d_tan(float[][]);
create or replace function sm_sc.fv_d_tan
(
  i_indepdt_var float[][]
)
returns float[][]
as
$$
declare 
begin
  return
    /` (sm_sc.fv_cos(i_indepdt_var)::float[][] ^` 2.0 :: float)
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_tan(array[[1.8, 4.6], [1.4, 3.6]])
-- drop function if exists sm_sc.fv_d_tanh(float[][], float[][]);
create or replace function sm_sc.fv_d_tanh
(
  i_indepdt_var float[][],
  i_depdt_var   float[][]  default null
)
returns float[][]
as
$$
declare 
begin
  return
    case 
      when i_depdt_var is not null
        then 1.0 :: float-` (i_depdt_var ^` 2.0 :: float) 
      else
        -- /` (sm_sc.fv_cosh(i_indepdt_var)::float[][] ^` 2.0 :: float)      -- 优先使用 i_depdt_var     -- -- sm_sc.fv_nullif(... , 0.0 :: float)
        1.0 :: float-` (sm_sc.fv_tanh(i_indepdt_var)::float[][] ^` 2.0 :: float) 
    end
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_tanh(array[[1.8, 3.6], [2.4, 1.6]])

-- select 
--   sm_sc.fv_d_tanh
--   (
--     array[[1.8, 3.6], [2.4, 1.6]],
--     sm_sc.fv_tanh(array[[1.8, 3.6], [2.4, 1.6]])
--   )

-- select 
--   sm_sc.fv_d_tanh
--   (
--     null,
--     sm_sc.fv_tanh(array[[1.8, 3.6], [2.4, 1.6]])
--   )
-- drop function if exists sm_sc.fv_acos(double precision[]);
create or replace function sm_sc.fv_acos
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- acos([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := acos(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acos []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := acos(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;
    
  -- acos([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            acos(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- acos([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              acos(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_acos
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acos
--   (
--     array[1.0 :: float/ 2, 1.0 :: float/ 4, 1, -1]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acos
--   (
--     array[[[1.0 :: float/ 2, 1.0 :: float/ 4, 1, -1]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acos
--   (
--     array[[[[1.0 :: float/ 2, 1.0 :: float/ 4, 1, -1]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_acosd(double precision[]);
create or replace function sm_sc.fv_acosd
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- acosd([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := acosd(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acosd []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := acosd(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acosd([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            acosd(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- acosd([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              acosd(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_acosd
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acosd
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acosd
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acosd
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_acosh(double precision[]);
create or replace function sm_sc.fv_acosh
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- acosh([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := acosh(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acosh []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := acosh(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acosh([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            acosh(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- acosh([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              acosh(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_acosh
--   (
--     array[[1, 'Infinity'], [1.5, 2.9]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acosh
--   (
--     array[1, 'Infinity', 1.5, 2.9]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acosh
--   (
--     array[[[1, 'Infinity'], [1.5, 2.9]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acosh
--   (
--     array[[[[1, 'Infinity'], [1.5, 2.9]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_acot(double precision[]);
create or replace function sm_sc.fv_acot
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- acot([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := pi() / 2.0 - atan(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acot []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := pi() / 2.0 - atan(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- acot([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            pi() / 2.0 - atan(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- acot([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              pi() / 2.0 - atan(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_acot
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4], array['-Infinity', 'Infinity']]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acot
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']::double precision[]
--   )
-- ;
-- select sm_sc.fv_acot
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_acot
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_asin(double precision[]);
create or replace function sm_sc.fv_asin
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- asin([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := asin(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- asin []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := asin(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- asin([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            asin(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- asin([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              asin(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_asin
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asin
--   (
--     array[1.0 :: float/ 2, 1.0 :: float/ 4, 1, -1]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asin
--   (
--     array[[[1.0 :: float/ 2, 1.0 :: float/ 4, 1, -1]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asin
--   (
--     array[[[[1.0 :: float/ 2, 1.0 :: float/ 4, 1, -1]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_asind(double precision[]);
create or replace function sm_sc.fv_asind
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- asind([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := asind(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- asind []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := asind(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- asind([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            asind(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- asind([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              asind(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_asind
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asind
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asind
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asind
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]]]]::double precision[]
--   )
-- ;
-- set search_path to sm_sc;
-- support from postgresql v_12
-- drop function if exists sm_sc.fv_asinh(double precision[]);
create or replace function sm_sc.fv_asinh
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;

begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- asinh([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := asinh(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- asinh []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := asinh(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- asinh([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            asinh(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- asinh([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              asinh(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_asinh
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4], array['-Infinity', 'Infinity']]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asinh
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']::double precision[]
--   )
-- ;
-- select sm_sc.fv_asinh
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_asinh
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_atan(double precision[]);
create or replace function sm_sc.fv_atan
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- atan([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := atan(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- atan []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := atan(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- atan([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            atan(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- atan([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              atan(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_atan
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4], array['-Infinity', 'Infinity']]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atan
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']::double precision[]
--   )
-- ;
-- select sm_sc.fv_atan
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atan
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_atand(double precision[]);
create or replace function sm_sc.fv_atand
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- atand([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := atand(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- atand []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := atand(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- atand([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            atand(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- atand([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              atand(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_atand
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4], array['-Infinity', 'Infinity']]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atand
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']::double precision[]
--   )
-- ;
-- select sm_sc.fv_atand
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atand
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4, '-Infinity', 'Infinity']]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_atanh(double precision[]);
create or replace function sm_sc.fv_atanh
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- atanh([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := atanh(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- atanh []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := atanh(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- atanh([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            atanh(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- atanh([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              atanh(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_atanh
--   (
--     array[array[1, -1], array[1.0 :: float/ 2, 1.0 :: float/ 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atanh
--   (
--     array[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atanh
--   (
--     array[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_atanh
--   (
--     array[[[[1, -1, 1.0 :: float/ 2, 1.0 :: float/ 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_cos(double precision[]);
create or replace function sm_sc.fv_cos
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- cos([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := cos(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cos []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := cos(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cos([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            cos(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- cos([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              cos(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_cos
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cos
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cos
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cos
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_cos_py(float[]);
create or replace function sm_sc.fv_cos_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.cos(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_cos_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])
-- drop function if exists sm_sc.fv_cosd(double precision[]);
create or replace function sm_sc.fv_cosd
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- cosd([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := cosd(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cosd []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := cosd(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cosd([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            cosd(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- cosd([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              cosd(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_cosd
--   (
--     array[array[180, -180], array[90, 45]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cosd
--   (
--     array[180, -180, 90, 45]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cosd
--   (
--     array[[[180, -180, 90, 45]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cosd
--   (
--     array[[[[180, -180, 90, 45]]]]::double precision[]
--   )
-- ;
-- set search_path to sm_sc;
-- support from postgresql v_12
-- drop function if exists sm_sc.fv_cosh(double precision[]);
create or replace function sm_sc.fv_cosh
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- cosh([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := cosh(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cosh []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := cosh(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cosh([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            cosh(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- cosh([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              cosh(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_cosh
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cosh
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cosh
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cosh
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_cosh_py(float[]);
create or replace function sm_sc.fv_cosh_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.cosh(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_cosh_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])
-- drop function if exists sm_sc.fv_cot(double precision[]);
create or replace function sm_sc.fv_cot
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- cot([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := cot(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cot []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := cot(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cot([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            cot(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- cot([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              cot(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_cot
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cot
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cot
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cot
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_cotd(double precision[]);
create or replace function sm_sc.fv_cotd
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- cotd([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := cotd(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cotd []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := cotd(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- cotd([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            cotd(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- cotd([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              cotd(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_cotd
--   (
--     array[array[180, -180], array[90, 45]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cotd
--   (
--     array[180, -180, 90, 45]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cotd
--   (
--     array[[[180, -180, 90, 45]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_cotd
--   (
--     array[[[[180, -180, 90, 45]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_csc(double precision[]);
create or replace function sm_sc.fv_csc
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- 1.0 / sin([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := 1.0 / sin(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- 1.0 / sin []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := 1.0 / sin(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- 1.0 / sin([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            1.0 / sin(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- 1.0 / sin([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              1.0 / sin(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_csc
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_csc
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_csc
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_csc
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_degrees(double precision[]);
create or replace function sm_sc.fv_degrees
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- degrees([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := degrees(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- degrees []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := degrees(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- degrees([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            degrees(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- degrees([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              degrees(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_degrees
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_degrees
--   (
--     array[12.3, -12.3, 45.6, -45.6]
--   );
-- select sm_sc.fv_degrees
--   (
--     array[[[12.3, -12.3, 45.6, -45.6]]]
--   );
-- select sm_sc.fv_degrees
--   (
--     array[[[[12.3, -12.3, 45.6, -45.6]]]]
--   );
-- drop function if exists sm_sc.fv_radians(double precision[]);
create or replace function sm_sc.fv_radians
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- radians([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := radians(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- radians []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := radians(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- radians([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            radians(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- radians([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              radians(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_radians
--   (
--     array[array[12.3, -12.3], array[45.6, -45.6]]
--   );
-- select sm_sc.fv_radians
--   (
--     array[12.3, -12.3, 45.6, -45.6]
--   );
-- select sm_sc.fv_radians
--   (
--     array[[[12.3, -12.3, 45.6, -45.6]]]
--   );
-- select sm_sc.fv_radians
--   (
--     array[[[[12.3, -12.3, 45.6, -45.6]]]]
--   );
-- drop function if exists sm_sc.fv_sec(double precision[]);
create or replace function sm_sc.fv_sec
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- 1.0 / cos([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := 1.0 / cos(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- 1.0 / cos []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := 1.0 / cos(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- 1.0 / cos([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            1.0 / cos(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- 1.0 / cos([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              1.0 / cos(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_sec
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sec
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sec
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sec
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_sin(double precision[]);
create or replace function sm_sc.fv_sin
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- sin([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := sin(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sin []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := sin(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sin([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            sin(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- sin([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sin(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_sin
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sin
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sin
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sin
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_sin_py(float[]);
create or replace function sm_sc.fv_sin_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.sin(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_sin_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])
-- drop function if exists sm_sc.fv_sind(double precision[]);
create or replace function sm_sc.fv_sind
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- sind([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := sind(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sind []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := sind(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sind([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            sind(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- sind([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sind(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_sind
--   (
--     array[array[180, -180], array[90, 45]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sind
--   (
--     array[180, -180, 90, 45]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sind
--   (
--     array[[[180, -180, 90, 45]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sind
--   (
--     array[[[180, -180, 90, 45]]]::double precision[]
--   )
-- ;
-- set search_path to sm_sc;
-- support from postgresql v_12
-- drop function if exists sm_sc.fv_sinh(double precision[]);
create or replace function sm_sc.fv_sinh
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- sinh([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := sinh(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sinh []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := sinh(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sinh([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            sinh(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- sinh([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sinh(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_sinh
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sinh
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sinh
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_sinh
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_sinh_py(float[]);
create or replace function sm_sc.fv_sinh_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.sinh(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_sinh_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])
-- drop function if exists sm_sc.fv_tan(double precision[]);
create or replace function sm_sc.fv_tan
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- tan([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := tan(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- tan []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := tan(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- tan([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            tan(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- tan([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              tan(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_tan
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tan
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tan
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tan
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_tan_py(float[]);
create or replace function sm_sc.fv_tan_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.tan(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_tan_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])
-- drop function if exists sm_sc.fv_tand(double precision[]);
create or replace function sm_sc.fv_tand
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;

begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- tand([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := tand(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- tand []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := tand(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- tand([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            tand(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- tand([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              tand(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_tand
--   (
--     array[array[180, -180], array[90, 45]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tand
--   (
--     array[180, -180, 90, -90, 45]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tand
--   (
--     array[[[180, -180, 90, -90, 45]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tand
--   (
--     array[[[[180, -180, 90, -90, 45]]]]::double precision[]
--   )
-- ;
-- set search_path to sm_sc;
-- support from postgresql v_12
-- drop function if exists sm_sc.fv_tanh(double precision[]);
create or replace function sm_sc.fv_tanh
(
  i_array     double precision[]
)
returns double precision[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;

begin
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- tanh([][])
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := tanh(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- tanh []
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := tanh(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- tanh([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            tanh(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- tanh([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              tanh(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;

  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_tanh
--   (
--     array[array[pi(), -pi()], array[pi() / 2, pi() / 4]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tanh
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tanh
--   (
--     array[[[pi(), -pi(), pi() / 2, pi() / 4]]]::double precision[]
--   )
-- ;
-- select sm_sc.fv_tanh
--   (
--     array[[[[pi(), -pi(), pi() / 2, pi() / 4]]]]::double precision[]
--   )
-- ;
-- drop function if exists sm_sc.fv_tanh_py(float[]);
create or replace function sm_sc.fv_tanh_py
(
  i_right    float[]
)
returns float[]
as
$$
  import numpy as np
  return (np.tanh(np.array(i_right))).tolist()
$$
language plpython3u stable
parallel safe
cost 100;

-- select sm_sc.fv_tanh_py(array[-0.4, 1.5, -2.7, 8.2, 0.0])
-- drop function if exists sm_sc.fv_grad_heigh_1st;
create or replace function sm_sc.fv_grad_heigh_1st
(
  i_array        anyarray
)
returns anyarray
as
$$
-- declare

begin
  if array_ndims(i_array) > 4
  then
    raise exception 'no method!';

  elsif array_ndims(i_array) = 1 and array_length(i_array, 1) > 1
  then
    return 
      i_array[2 : ] -` i_array[ : array_length(i_array, 1) - 1];
  
  elsif array_length(i_array, 1) > 1
  then
    return
      i_array[2 : ][ : ] -` i_array[ : array_length(i_array, 1) - 1][ : ];
  
  elsif array_length(i_array, 2) > 1
  then
    return
      i_array[ : ][2 : ][ : ] -` i_array[ : ][ : array_length(i_array, 2) - 1][ : ];
  
  elsif array_length(i_array, 3) > 1
  then
    return
      i_array[ : ][ : ][2 : ][ : ] -` i_array[ : ][ : ][ : array_length(i_array, 3) - 1][ : ];
  
  else 
    return array[] :: i_array%type;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_grad_heigh_1st
--   (
--     array[array[1, 2, 3, 4], array[-1, -2, -3, -4], array[-3, -2, -4, -1]]
--   );

-- select sm_sc.fv_grad_heigh_1st
--   (
--     array[1, 2, 3, 4]
--   );
-- drop function if exists sm_sc.fv_grad_heigh_2nd;
create or replace function sm_sc.fv_grad_heigh_2nd
(
  i_array        anyarray
)
returns anyarray
as
$$
-- declare

begin
  if array_ndims(i_array) > 4
  then
    raise exception 'no method!';

  elsif array_ndims(i_array) = 1 and array_length(i_array, 1) > 2
  then
    return 
      i_array[3 : ] -` i_array[2 : array_length(i_array, 1) - 1] -` i_array[2 : array_length(i_array, 1) - 1] +` i_array[ : array_length(i_array, 1) - 2];
  
  elsif array_length(i_array, 1) > 2
  then
    return
      i_array[3 : ][ : ] -` i_array[2 : array_length(i_array, 1) - 1][ : ] -` i_array[2 : array_length(i_array, 1) - 1][ : ] +` i_array[ : array_length(i_array, 1) - 2][ : ];
  
  elsif array_length(i_array, 2) > 2
  then
    return
      i_array[ : ][3 : ][ : ] -` i_array[ : ][2 : array_length(i_array, 2) - 1][ : ] -` i_array[ : ][2 : array_length(i_array, 2) - 1][ : ] +` i_array[ : ][ : array_length(i_array, 2) - 2][ : ];
  
  elsif array_length(i_array, 3) > 2
  then
    return
      i_array[ : ][ : ][3 : ][ : ] -` i_array[ : ][ : ][2 : array_length(i_array, 3) - 1][ : ] -` i_array[ : ][ : ][2 : array_length(i_array, 3) - 1][ : ] +` i_array[ : ][ : ][ : array_length(i_array, 3) - 2][ : ];
  
  else 
    return array[] :: i_array%type;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_grad_heigh_2nd
--   (
--     array[array[1, 2, 3, 4], array[-1, -2, -3, -4], array[-3, -2, -4, -1]]
--   );

-- select sm_sc.fv_grad_heigh_2nd
--   (
--     array[1, 2, 3, 4]
--   );
-- drop function if exists sm_sc.fv_grad_width_1st;
create or replace function sm_sc.fv_grad_width_1st
(
  i_array        anyarray
)
returns anyarray
as
$$
-- declare

begin
  if array_ndims(i_array) > 4
  then
    raise exception 'no method!';

  elsif array_ndims(i_array) = 1 and array_length(i_array, 1) > 1
  then
    return 
      i_array[2 : ] -` i_array[ : array_length(i_array, 1) - 1];
  
  elsif array_length(i_array, 2) > 1
  then
    return
      i_array[ : ][2 : ] -` i_array[ : ][ : array_length(i_array, 2) - 1];
  
  elsif array_length(i_array, 3) > 1
  then
    return
      i_array[ : ][ : ][2 : ] -` i_array[ : ][ : ][ : array_length(i_array, 3) - 1];
  
  elsif array_length(i_array, 4) > 1
  then
    return
      i_array[ : ][ : ][ : ][2 : ] -` i_array[ : ][ : ][ : ][ : array_length(i_array, 4) - 1];
  
  else 
    return array[] :: i_array%type;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_grad_width_1st
--   (
--     array[array[1, 2, 3, 4], array[-1, -2, -3, -4], array[-3, -2, -4, -1]]
--   );

-- select sm_sc.fv_grad_width_1st
--   (
--     array[1, 2, 3, 4]
--   );
-- drop function if exists sm_sc.fv_grad_width_2nd;
create or replace function sm_sc.fv_grad_width_2nd
(
  i_array        anyarray
)
returns anyarray
as
$$
-- declare

begin
  if array_ndims(i_array) > 4
  then
    raise exception 'no method!';

  elsif array_ndims(i_array) = 1 and array_length(i_array, 1) > 2
  then
    return 
      i_array[3 : ] -` i_array[2 : array_length(i_array, 1) - 1] -` i_array[2 : array_length(i_array, 1) - 1] +` i_array[ : array_length(i_array, 1) - 2];
  
  elsif array_length(i_array, 2) > 2
  then
    return
      i_array[ : ][3 : ] -` i_array[ : ][2 : array_length(i_array, 2) - 1] -` i_array[ : ][2 : array_length(i_array, 2) - 1] +` i_array[ : ][ : array_length(i_array, 2) - 2];
  
  elsif array_length(i_array, 3) > 2
  then
    return
      i_array[ : ][ : ][3 : ] -` i_array[ : ][ : ][2 : array_length(i_array, 3) - 1] -` i_array[ : ][ : ][2 : array_length(i_array, 3) - 1] +` i_array[ : ][ : ][ : array_length(i_array, 3) - 2];
  
  elsif array_length(i_array, 4) > 2
  then
    return
      i_array[ : ][ : ][ : ][3 : ] -` i_array[ : ][ : ][ : ][2 : array_length(i_array, 4) - 1] -` i_array[ : ][ : ][ : ][2 : array_length(i_array, 4) - 1] +` i_array[ : ][ : ][ : ][ : array_length(i_array, 4) - 2];
  
  else 
    return array[] :: i_array%type;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_grad_width_2nd
--   (
--     array[array[1, 2, 3, 4], array[-1, -2, -3, -4], array[-3, -2, -4, -1]]
--   );

-- select sm_sc.fv_grad_width_2nd
--   (
--     array[1, 2, 3, 4]
--   );
-- drop function if exists sm_sc.fv_sgn_fft(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_sgn_fft
(
  i_array           sm_sc.typ_l_complex[]
)
returns sm_sc.typ_l_complex[]
as
$$
declare -- here
-- 如果要返回任意长度，要做循环卷积改造 https://blog.csdn.net/weixin_34128237/article/details/94637989
  v_len_1_ex     int   := case when array_ndims(i_array) = 1 then power(2.0, ceil(log(2.0, array_length(i_array, 1)))) :: int else null end;     -- 时域补零至就近的power(2,n)，频域升采样
  v_w            sm_sc.typ_l_complex     :=   (exp(1)::sm_sc.typ_l_complex) ^ ((0.0 :: float, -2 * pi() / v_len_1_ex)::sm_sc.typ_l_complex);
  v_w_wheel      sm_sc.typ_l_complex     :=   (1.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex;
  v_arr_even     sm_sc.typ_l_complex[]   :=   (select array_agg(coalesce(i_array[a_even_no], (0.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex) order by a_even_no) from generate_series(1, v_len_1_ex, 2) tb_a_even_no(a_even_no));
  v_arr_odd      sm_sc.typ_l_complex[]   :=   (select array_agg(coalesce(i_array[a_odd_no], (0.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex) order by a_odd_no) from generate_series(2, v_len_1_ex, 2) tb_a_odd_no(a_odd_no));
  v_ret          sm_sc.typ_l_complex[]   :=   array_fill(0.0 :: float, array[v_len_1_ex]);
  v_cur          int;

begin
  if array_ndims(i_array) = 1
  then 
    if v_len_1_ex = 1
    then 
      return array[coalesce(i_array[1], (0.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex)];
    end if;
    
    v_arr_even := sm_sc.fv_sgn_fft(v_arr_even);    -- sm_sc.fv_sgn_fft(v_arr_even[1] || (v_arr_even[2 : ] |+~| 1));
    v_arr_odd := sm_sc.fv_sgn_fft(v_arr_odd);    -- sm_sc.fv_sgn_fft(v_arr_odd[1] || (v_arr_odd[2 : ] |+~| 1));
    
    for v_cur in 1 .. (v_len_1_ex / 2)
    loop
      v_ret[v_cur] := v_arr_even[v_cur] + (v_w_wheel * v_arr_odd[v_cur]);
      v_ret[v_cur + v_len_1_ex / 2] := v_arr_even[v_cur] - (v_w_wheel * v_arr_odd[v_cur]);
      v_w_wheel := v_w_wheel * v_w;
    end loop;
    
    return v_ret;    -- v_ret[1] || (v_ret[2 : ] |+~| 1); 
  else
    raise exception 'error dim!';
  end if;
end 
$$
language plpgsql stable
cost 100;

-- select sm_sc.fv_sgn_fft
-- (
-- array
-- [ 
--   1.        ,  0.97952994,  0.91895781,  0.82076344,  0.68896692,  0.52896401,
--   0.34730525,  0.15142778, -0.05064917, -0.25065253, -0.44039415, -0.61210598,
--  -0.75875812, -0.87434662, -0.95413926, -0.99486932, -0.99486932, -0.95413926,
--  -0.87434662, -0.75875812, -0.61210598, -0.44039415, -0.25065253, -0.05064917,
--   0.15142778,  0.34730525,  0.52896401,  0.68896692,  0.82076344,  0.91895781,
--   0.97952994,  1.        
-- ]
-- ):: sm_sc.typ_l_complex[]
-- -- 预期结果：
-- --     [ 1.00000000e+00+0.00000000e+00j  1.61497357e+01+1.59061014e+00j
-- --      -3.53833224e-01-7.03818042e-02j -1.26604779e-01-3.84051399e-02j
-- --      -6.41365736e-02-2.65662386e-02j -3.75538211e-02-2.00729356e-02j
-- --      -2.37109631e-02-1.58431590e-02j -1.55930224e-02-1.27968628e-02j
-- --      -1.04489194e-02-1.04489194e-02j -7.01356383e-03-8.54605226e-03j
-- --      -4.63822579e-03-6.94159545e-03j -2.96316279e-03-5.54368767e-03j
-- --      -1.77736596e-03-4.29094101e-03j -9.52464878e-04-3.13985591e-03j
-- --      -4.09315616e-04-2.05776856e-03j -1.00322243e-04-1.01858882e-03j
-- --       2.22044605e-16+0.00000000e+00j -1.00322243e-04+1.01858882e-03j
-- --      -4.09315616e-04+2.05776856e-03j -9.52464878e-04+3.13985591e-03j
-- --      -1.77736596e-03+4.29094101e-03j -2.96316279e-03+5.54368767e-03j
-- --      -4.63822579e-03+6.94159545e-03j -7.01356383e-03+8.54605226e-03j
-- --      -1.04489194e-02+1.04489194e-02j -1.55930224e-02+1.27968628e-02j
-- --      -2.37109631e-02+1.58431590e-02j -3.75538211e-02+2.00729356e-02j
-- --      -6.41365736e-02+2.65662386e-02j -1.26604779e-01+3.84051399e-02j
-- --      -3.53833224e-01+7.03818042e-02j  1.61497357e+01-1.59061014e+00j]


-- -- # python numpy 的验证    
-- --     import numpy as np
-- --     x = np.linspace(0, 2 * np.pi, 32) #创建一个包含30个点的余弦波信号
-- --     wave = np.cos(x)
-- --     print(wave)
-- --     transformed = np.fft.fft(wave)  #使用fft函数对余弦波信号进行傅里叶变换。
-- --     itransformed = np.fft.ifft(wave)  #使用ifft函数对余弦波信号进行傅里叶逆变换。
-- --     print(np.fft.ifft(transformed))
-- --     print(transformed)
-- --     print(itransformed)
-- drop function if exists sm_sc.fv_sgn_fft2(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_sgn_fft2
(
  i_array          sm_sc.typ_l_complex[]
)
returns sm_sc.typ_l_complex[]
as
$$
-- declare
-- 如果要返回任意长度，要做循环卷积改造 https://blog.csdn.net/weixin_34128237/article/details/94637989

begin
  if array_ndims(i_array) = 2
  then
    return
    (
      with
      cte_fft_x as
      (
      select 
        array_agg(sm_sc.fv_sgn_fft(array(select unnest(i_array[a_no_y:a_no_y][:]))) order by a_no_y) as a_fft_x
      from generate_series(1, array_length(i_array, 1)) tb_a_y(a_no_y)    
      )
      select 
        |^~| array_agg(sm_sc.fv_sgn_fft(array(select unnest(a_fft_x[:][a_no_x:a_no_x]))) order by a_no_x) as a_fft_x
      from generate_series(1, power(2, ceil(log(2, array_length(i_array, 2)))) :: int) tb_a_x(a_no_x), cte_fft_x
    )
    ;
  else
    raise exception 'error dim!';
  end if;
end 
$$
language plpgsql stable
cost 100;

-- select sm_sc.fv_sgn_fft2
-- (
-- array
-- [ 
--   array[1,2,3,4,5,6,7,8,9,10,11,12],
--   array[2,3,4,5,6,7,8,9,10,11,12,13],
--   array[3,4,5,6,7,8,9,10,11,12,13,14],
--   array[4,5,6,7,8,9,10,11,12,13,14,15],
--   array[5,6,7,8,9,10,11,12,13,14,15,16]
-- ]
-- ):: sm_sc.typ_l_complex[]
-- -- 预期结果：
-- -- {{"(510.00000000,0.00000000)","(-213.86790834,42.36236838)","(27.92893270,108.63960980)","(59.57292973,-22.22567143)","(-30.00000030,-29.99999970)","(-19.36477665,40.70326143)","(42.07106810,18.63960880)","(13.65975367,-34.70870036)","(-30.00000000,0.00000000)","(13.65975574,34.70869942)","(42.07106730,-18.63960980)","(-19.36477713,-40.70326077)","(-29.99999970,29.99999970)","(59.57292925,22.22567077)","(27.92893190,-108.63960880)","(-213.86790627,-42.36236744)"}
-- -- ,{"(-64.97056272,246.24978312)","(-9.55192284,-119.58121247)","(-57.87005744,0.41421387)","(3.97292507,27.41999021)","(14.48528122,-14.48528150)","(-20.55154905,-13.86607894)","(-14.41421288,18.07106811)","(13.51325932,8.76414246)","(0.00000000,-14.48528136)","(-20.00442632,4.42688375)","(3.58578624,22.55634885)","(18.75499698,-4.83420387)","(-14.48528122,-14.48528122)","(-17.49008147,30.10871950)","(47.04162984,26.55634885)","(31.35679831,-86.94790847)"}
-- -- ,{"(102.00000000,-24.00000000)","(-36.74624217,12.49981315)","(10.41421364,19.72792194)","(12.41119167,-6.94174005)","(-6.00000006,-5.99999994)","(-2.20477670,7.80883090)","(9.24264068,1.72792174)","(1.93086304,-8.14065241)","(-6.00000000,0.00000000)","(3.53303877,5.74282753)","(7.58578636,-5.72792194)","(-5.54113403,-8.47247351)","(-5.99999994,5.99999994)","(11.41798010,1.94852842)","(0.75735932,-23.72792174)","(-48.80092068,-4.44513403)"}
-- -- ,{"(-31.02943728,42.24978312)","(1.69750642,-25.51010389)","(-11.58578637,-3.92893213)","(-1.38660977,4.29312486)","(2.48528134,-2.48528138)","(-3.80097817,-3.76100745)","(-4.12994221,2.41421365)","(1.32529725,2.16733191)","(0.00000000,-2.48528136)","(-4.42542845,0.09589044)","(-1.04163059,4.55634909)","(2.94295893,0.55254618)","(-2.48528134,-2.48528134)","(-5.06907948,5.57724122)","(6.41421341,8.55634907)","(8.71633327,-9.92469111)"}
-- -- ,{"(102.00000000,0.00000000)","(-42.77358166,8.47247368)","(5.58578654,21.72792196)","(11.91458595,-4.44513429)","(-6.00000006,-5.99999994)","(-3.87295533,8.14065229)","(8.41421362,3.72792176)","(2.73195073,-6.94174008)","(-6.00000000,0.00000000)","(2.73195114,6.94173988)","(8.41421346,-3.72792196)","(-3.87295543,-8.14065215)","(-5.99999994,5.99999994)","(11.91458585,4.44513415)","(5.58578638,-21.72792176)","(-42.77358125,-8.47247348)"}
-- -- ,{"(-31.02943728,-42.24978312)","(8.71633338,9.92469119)","(6.41421344,-8.55634919)","(-5.06907955,-5.57724125)","(-2.48528134,2.48528138)","(2.94295895,-0.55254622)","(-1.04163072,-4.55634911)","(-4.42542848,-0.09589018)","(0.00000000,2.48528136)","(1.32529714,-2.16733199)","(-4.12994224,-2.41421353)","(-3.80097810,3.76100747)","(2.48528134,2.48528134)","(-1.38660979,-4.29312482)","(-11.58578624,3.92893215)","(1.69750645,25.51010363)"}
-- -- ,{"(102.00000000,24.00000000)","(-48.80092115,4.44513419)","(0.75735944,23.72792198)","(11.41798021,-1.94852851)","(-6.00000006,-5.99999994)","(-5.54113396,8.47247366)","(7.58578656,5.72792178)","(3.53303844,-5.74282775)","(-6.00000000,0.00000000)","(1.93086351,8.14065225)","(9.24264056,-1.72792198)","(-2.20477681,-7.80883081)","(-5.99999994,5.99999994)","(12.41119160,6.94173990)","(10.41421344,-19.72792178)","(-36.74624184,-12.49981293)"}
-- -- ,{"(-64.97056272,-246.24978312)","(31.35679884,86.94790929)","(47.04163021,-26.55634935)","(-17.49008187,-30.10871970)","(-14.48528122,14.48528150)","(18.75499723,4.83420361)","(3.58578565,-22.55634913)","(-20.00442669,-4.42688255)","(0.00000000,14.48528136)","(13.51325879,-8.76414328)","(-14.41421325,-18.07106761)","(-20.55154865,13.86607914)","(14.48528122,14.48528122)","(3.97292482,-27.41998994)","(-57.87005685,-0.41421359)","(-9.55192247,119.58121127)"}}


-- drop function if exists sm_sc.fv_sgn_ifft(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_sgn_ifft
(
  i_array           sm_sc.typ_l_complex[]
)
returns sm_sc.typ_l_complex[]
as
$$
declare -- here
-- 如果要返回任意长度，要做循环卷积改造 https://blog.csdn.net/weixin_34128237/article/details/94637989
  v_len_1_ex     int   := case when array_ndims(i_array) = 1 then power(2, ceil(log(2, array_length(i_array, 1)))) :: int else null end;     -- 时域补零至就近的power(2,n)，频域升采样
  v_w            sm_sc.typ_l_complex     :=   (exp(1)::sm_sc.typ_l_complex) ^ ((0.0 :: float, 2 * pi() / v_len_1_ex)::sm_sc.typ_l_complex);
  v_w_wheel      sm_sc.typ_l_complex     :=   (1.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex;
  v_arr_even     sm_sc.typ_l_complex[]   :=   (select array_agg(coalesce(i_array[a_even_no], (0.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex) order by a_even_no) from generate_series(1, v_len_1_ex, 2) tb_a_even_no(a_even_no));
  v_arr_odd      sm_sc.typ_l_complex[]   :=   (select array_agg(coalesce(i_array[a_odd_no], (0.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex) order by a_odd_no) from generate_series(2, v_len_1_ex, 2) tb_a_odd_no(a_odd_no));
  v_ret          sm_sc.typ_l_complex[]   :=   array_fill(0.0 :: float, array[v_len_1_ex]);
  v_cur          int;

begin
  if array_ndims(i_array) = 1
  then 
    if v_len_1_ex = 1
    then 
      return array[coalesce(i_array[1], (0.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex)];
    end if;
    
    v_arr_even := sm_sc.fv_sgn_ifft(v_arr_even);    --  sm_sc.fv_sgn_ifft(v_arr_even[1] || (v_arr_even[2 : ] |+~| 1));
    v_arr_odd := sm_sc.fv_sgn_ifft(v_arr_odd);     -- sm_sc.fv_sgn_ifft(v_arr_odd[1] || (v_arr_odd[2 : ] |+~| 1));
    
    for v_cur in 1 .. (v_len_1_ex / 2)
    loop
      v_ret[v_cur] := (v_arr_even[v_cur] + (v_w_wheel * v_arr_odd[v_cur])) / 2;
      v_ret[v_cur + v_len_1_ex / 2] := (v_arr_even[v_cur] - (v_w_wheel * v_arr_odd[v_cur])) / 2;
      v_w_wheel := v_w_wheel * v_w;
    end loop;
    
    return v_ret;     -- v_ret[1] || (v_ret[2 : ] |+~| 1); 
  else
    raise exception 'error dim!';
  end if;
end 
$$
language plpgsql stable
cost 100;

-- select sm_sc.fv_sgn_ifft
-- (
-- array
-- [ 
--   1.        ,  0.97952994,  0.91895781,  0.82076344,  0.68896692,  0.52896401,
--   0.34730525,  0.15142778, -0.05064917, -0.25065253, -0.44039415, -0.61210598,
--  -0.75875812, -0.87434662, -0.95413926, -0.99486932, -0.99486932, -0.95413926,
--  -0.87434662, -0.75875812, -0.61210598, -0.44039415, -0.25065253, -0.05064917,
--   0.15142778,  0.34730525,  0.52896401,  0.68896692,  0.82076344,  0.91895781,
--   0.97952994,  1.        
-- ]
-- ):: sm_sc.typ_l_complex[]
-- -- 预期结果：
-- --   (0.03125001,0.00000000)
-- --   (0.50467925,0.04970657)
-- --   (-0.01105730,-0.00219944)
-- --   (-0.00395641,-0.00120017)
-- --   (-0.00200427,-0.00083020)
-- --   (-0.00117356,-0.00062728)
-- --   (-0.00074097,-0.00049510)
-- --   (-0.00048728,-0.00039991)
-- --   (-0.00032653,-0.00032653)
-- --   (-0.00021918,-0.00026707)
-- --   (-0.00014495,-0.00021693)
-- --   (-0.00009260,-0.00017324)
-- --   (-0.00005554,-0.00013409)
-- --   (-0.00002977,-0.00009812)
-- --   (-0.00001279,-0.00006431)
-- --   (-0.00000314,-0.00003183)
-- --   (0.00000000,0.00000000)
-- --   (-0.00000314,0.00003183)
-- --   (-0.00001279,0.00006431)
-- --   (-0.00002977,0.00009812)
-- --   (-0.00005554,0.00013409)
-- --   (-0.00009260,0.00017324)
-- --   (-0.00014495,0.00021693)
-- --   (-0.00021917,0.00026707)
-- --   (-0.00032653,0.00032653)
-- --   (-0.00048729,0.00039991)
-- --   (-0.00074097,0.00049510)
-- --   (-0.00117356,0.00062728)
-- --   (-0.00200427,0.00083020)
-- --   (-0.00395641,0.00120017)
-- --   (-0.01105730,0.00219944)
-- --   (0.50467924,-0.04970657)


-- -- # python numpy 的验证    
-- --     import numpy as np
-- --     x = np.linspace(0, 2 * np.pi, 32) #创建一个包含30个点的余弦波信号
-- --     wave = np.cos(x)
-- --     print(wave)
-- --     transformed = np.fft.fft(wave)  #使用fft函数对余弦波信号进行傅里叶变换。
-- --     itransformed = np.fft.ifft(wave)  #使用ifft函数对余弦波信号进行傅里叶逆变换。
-- --     print(np.fft.ifft(transformed))
-- --     print(transformed)
-- --     print(itransformed)
-- drop function if exists sm_sc.fv_sgn_ifft2(sm_sc.typ_l_complex[]);
create or replace function sm_sc.fv_sgn_ifft2
(
  i_array          sm_sc.typ_l_complex[]
)
returns sm_sc.typ_l_complex[]
as
$$
-- declare
-- 如果要返回任意长度，要做循环卷积改造 https://blog.csdn.net/weixin_34128237/article/details/94637989

begin
  if array_ndims(i_array) = 2
  then
    return
    (
      with
      cte_fft_x as
      (
      select 
        array_agg(sm_sc.fv_sgn_ifft(array(select unnest(i_array[a_no_y:a_no_y][:]))) order by a_no_y) as a_fft_x
      from generate_series(1, array_length(i_array, 1)) tb_a_y(a_no_y)    
      )
      select 
        |^~| array_agg(sm_sc.fv_sgn_ifft(array(select unnest(a_fft_x[:][a_no_x:a_no_x]))) order by a_no_x) as a_fft_x
      from generate_series(1, power(2, ceil(log(2, array_length(i_array, 2)))) :: int) tb_a_x(a_no_x), cte_fft_x
    )
    ;
  else
    raise exception 'error dim!';
  end if;
end 
$$
language plpgsql stable
cost 100;

-- select sm_sc.fv_sgn_ifft2
-- (
-- array
-- [ 
--   array[1,2,3,4,5,6,7,8,9,10,11,12],
--   array[2,3,4,5,6,7,8,9,10,11,12,13],
--   array[3,4,5,6,7,8,9,10,11,12,13,14],
--   array[4,5,6,7,8,9,10,11,12,13,14,15],
--   array[5,6,7,8,9,10,11,12,13,14,15,16]
-- ]
-- ):: sm_sc.typ_l_complex[]
-- -- 预期结果：
-- -- {{"(3.98437500,0.00000000)","(-1.67084304,-0.33095601)","(0.21819479,-0.84874696)","(0.46541353,0.17363807)","(-0.23437501,0.23437500)","(-0.15128733,-0.31799424)","(0.32868023,-0.14562195)","(0.10671684,0.27116173)","(-0.23437500,0.00000000)","(0.10671685,-0.27116173)","(0.32868022,0.14562196)","(-0.15128733,0.31799424)","(-0.23437500,-0.23437500)","(0.46541352,-0.17363806)","(0.21819479,0.84874695)","(-1.67084303,0.33095600)"}
-- -- ,{"(-0.50758252,-1.92382643)","(-0.07462441,0.93422823)","(-0.45210983,-0.00323604)","(0.03103848,-0.21421868)","(0.11316626,0.11316628)","(-0.16055899,0.10832875)","(-0.11261105,-0.14118023)","(0.10557234,-0.06846987)","(0.00000000,0.11316626)","(-0.15628459,-0.03458503)","(0.02801396,-0.17622148)","(0.14652342,0.03776722)","(-0.11316626,0.11316626)","(-0.13664126,-0.23522438)","(0.36751274,-0.20747148)","(0.24497499,0.67928054)"}
-- -- ,{"(0.79687500,0.18750000)","(-0.28708002,-0.09765479)","(0.08136105,-0.15412439)","(0.09696244,0.05423235)","(-0.04687500,0.04687500)","(-0.01722482,-0.06100650)","(0.07220813,-0.01349939)","(0.01508487,0.06359885)","(-0.04687500,0.00000000)","(0.02760187,-0.04486584)","(0.05926396,0.04474939)","(-0.04329012,0.06619121)","(-0.04687500,-0.04687500)","(0.08920297,-0.01522288)","(0.00591687,0.18537439)","(-0.38125720,0.03472761)"}
-- -- ,{"(-0.24241748,-0.33007643)","(0.01326177,0.19929769)","(-0.09051396,0.03069479)","(-0.01083289,-0.03354004)","(0.01941626,0.01941627)","(-0.02969515,0.02938287)","(-0.03226518,-0.01886105)","(0.01035389,-0.01693228)","(0.00000000,0.01941626)","(-0.03457366,-0.00074915)","(-0.00813775,-0.03559648)","(0.02299187,-0.00431677)","(-0.01941626,0.01941626)","(-0.03960219,-0.04357220)","(0.05011105,-0.06684648)","(0.06809636,0.07753665)"}
-- -- ,{"(0.79687500,0.00000000)","(-0.33416861,-0.06619120)","(0.04363896,-0.16974939)","(0.09308271,0.03472762)","(-0.04687500,0.04687500)","(-0.03025747,-0.06359885)","(0.06573605,-0.02912439)","(0.02134337,0.05423234)","(-0.04687500,0.00000000)","(0.02134337,-0.05423235)","(0.06573605,0.02912439)","(-0.03025747,0.06359885)","(-0.04687500,-0.04687500)","(0.09308270,-0.03472761)","(0.04363896,0.16974939)","(-0.33416861,0.06619120)"}
-- -- ,{"(-0.24241748,0.33007643)","(0.06809636,-0.07753665)","(0.05011104,0.06684648)","(-0.03960219,0.04357220)","(-0.01941626,-0.01941627)","(0.02299187,0.00431677)","(-0.00813775,0.03559648)","(-0.03457366,0.00074914)","(0.00000000,-0.01941626)","(0.01035388,0.01693228)","(-0.03226518,0.01886104)","(-0.02969515,-0.02938287)","(0.01941626,-0.01941626)","(-0.01083289,0.03354004)","(-0.09051396,-0.03069478)","(0.01326177,-0.19929769)"}
-- -- ,{"(0.79687500,-0.18750000)","(-0.38125720,-0.03472761)","(0.00591687,-0.18537439)","(0.08920298,0.01522288)","(-0.04687500,0.04687500)","(-0.04329011,-0.06619121)","(0.05926396,-0.04474939)","(0.02760187,0.04486584)","(-0.04687500,0.00000000)","(0.01508487,-0.06359885)","(0.07220814,0.01349939)","(-0.01722482,0.06100650)","(-0.04687500,-0.04687500)","(0.09696244,-0.05423235)","(0.08136104,0.15412439)","(-0.28708002,0.09765479)"}
-- -- ,{"(-0.50758252,1.92382643)","(0.24497500,-0.67928055)","(0.36751275,0.20747149)","(-0.13664127,0.23522438)","(-0.11316626,-0.11316628)","(0.14652343,-0.03776722)","(0.02801395,0.17622149)","(-0.15628459,0.03458502)","(0.00000000,-0.11316626)","(0.10557234,0.06846988)","(-0.11261105,0.14118022)","(-0.16055898,-0.10832875)","(0.11316626,-0.11316626)","(0.03103848,0.21421868)","(-0.45210983,0.00323605)","(-0.07462440,-0.93422822)"}}


-- -- https://blog.csdn.net/weixin_38451800/article/details/86514415
-- -- https://zhuanlan.zhihu.com/p/77327349 代码有误：predict_var += v_std**2 改成 predict_var += v_std
-- -- https://zhuanlan.zhihu.com/p/73236905
-- -- 
-- -- Q，R分别对应过程噪声的协方差和观测误差的协方差，这两个量是估计出来的。
-- -- P应该是初始的状态协方差，表示初值与真值的不确定性，
-- drop function if exists sm_sc.fv_sgn_kalman(float[], float, float, float);
create or replace function sm_sc.fv_sgn_kalman
(
  i_wave                    float[],
  i_process_q_covar         float  ,  
  i_measure_r_covar         float  , 
  i_initial_p_covar         float  default null
)
returns float[]    -- 比 i_wave 尾部多一个元素，做为线性预测值（不适合非线性）
as
$$

-- -- declare 
-- --   v_sum_predict_var   float   :=   0.0                     ;
-- --   v_cur_no            int                                           ;
-- --   v_cur_last_ele      float   :=   i_wave[1]               ;
-- --   v_cur_mid_ele       float   :=   v_cur_last_ele          ;
-- --   v_cur_ele           float                                ;
-- --   v_cur_last_p_covar  float   :=   coalesce(i_initial_p_covar, i_measure_r_covar)       ;
-- --   v_cur_mid_p_covar   float                                ;
-- --   v_cur_p_covar       float                                ;
-- --   v_cur_predict_share float                                ;
-- -- 
-- -- begin
-- --   for v_cur_no in 1 .. array_length(i_wave, 1)
-- --   loop
-- --     v_cur_mid_ele := v_cur_last_ele;
-- --     v_cur_mid_p_covar := v_cur_last_p_covar + i_process_q_covar;
-- --     v_cur_predict_share := v_cur_mid_p_covar / (v_cur_mid_p_covar + i_measure_r_covar);
-- --     v_cur_ele := v_cur_mid_ele + (v_cur_predict_share * (i_wave[v_cur_no] - v_cur_mid_ele));
-- --     v_cur_p_covar := (1.0 :: float- v_cur_predict_share) * v_cur_mid_p_covar;
-- --   
-- --     i_wave[v_cur_no] := v_cur_ele;
-- --   
-- --     v_cur_last_ele := v_cur_ele;
-- --     v_cur_last_p_covar  = v_cur_p_covar;
-- --   end loop;
-- -- 
-- --   return i_wave;
-- -- end;

declare 
  v_predict_var       float  :=   coalesce(i_initial_p_covar, i_measure_r_covar)        ;
  v_cur_no            int                             ;
  v_cur_ele           float  :=   i_wave[1]  ;

begin
  for v_cur_no in 2 .. array_length(i_wave, 1)
  loop
	v_predict_var := v_predict_var + i_process_q_covar;
	v_cur_ele := (v_cur_ele * i_measure_r_covar / (i_measure_r_covar + v_predict_var)) + (i_wave[v_cur_no] * v_predict_var / (i_measure_r_covar + v_predict_var));
	v_predict_var := (v_predict_var * i_measure_r_covar) / (v_predict_var + i_measure_r_covar);
	i_wave[v_cur_no] = v_cur_ele;
  end loop;

  -- 多预测一个轨迹点，假定轨迹在最近几个(7)采样点(要求采样足够密集)局部瞬间趋近于线性移动（不适合非线性），线性回归？
  v_cur_no := array_length(i_wave, 1);
  v_cur_ele := 
  (
    select avg(a_ele)
    from
    (
      select ((2 * i_wave[v_cur_no]) - (1 * i_wave[v_cur_no - 1])) as a_ele
      union all
      select ((3 * i_wave[v_cur_no - 1]) - (2 * i_wave[v_cur_no - 2])) as a_ele
      union all
      select ((4 * i_wave[v_cur_no - 2]) - (3 * i_wave[v_cur_no - 3])) as a_ele
      union all
      select ((5 * i_wave[v_cur_no - 3]) - (4 * i_wave[v_cur_no - 4])) as a_ele
      union all
      select ((6 * i_wave[v_cur_no - 4]) - (5 * i_wave[v_cur_no - 5])) as a_ele
      union all
      select ((7 * i_wave[v_cur_no - 5]) - (6 * i_wave[v_cur_no - 6])) as a_ele
      union all
      select ((8 * i_wave[v_cur_no - 6]) - (7 * i_wave[v_cur_no - 7])) as a_ele
    ) t
  );
  return i_wave || v_cur_ele;
end;

$$
language plpgsql stable
parallel safe;

-- -- create extension tablefunc;
-- with 
-- -- 构造精确序列
-- cte_ods as
-- (
--   select 
--     row_number() over() as row_no,
--     a_x,
--     10 * sin(a_x) :: float as ods_ele
--   from generate_series(0.0, pi() :: decimal, (pi() / 100) :: decimal) tb_a(a_x)
-- )
-- ,
-- -- 构造误差序列
-- cte_test as
-- (
--   select
--     row_no,
--     ods_ele + normal_rand(1, 0.0 :: float, 0.5 :: float) :: float as test_ele   -- 假定观测误差标准差 0.5
--   from cte_ods
-- )
-- ,
-- -- 执行卡尔曼滤波
-- cte_predict as
-- (
--   select 
--     sm_sc.fv_sgn_kalman
--     (
--       array_agg(test_ele order by row_no),              
--       0.4 * 0.4                          ,      -- 假定过程误差标准差 0.4
--       0.5 * 0.5                          ,      -- 假定观测误差标准差 0.5
--       0.5 * 0.5
--     ) as predict_arr
--   from cte_test
-- )
-- -- 对比结果
-- select 
--   a_x, a_ods_ele, a_test_ele, a_predict_ele, abs(a_test_ele - a_ods_ele) - abs(a_predict_ele - a_ods_ele)
--   -- sum(abs(a_test_ele - a_ods_ele) - abs(a_predict_ele - a_ods_ele))
-- from unnest
-- (
--   (select array_agg(a_x) from cte_ods),
--   (select array_agg(ods_ele) from cte_ods),
--   (select array_agg(test_ele) from cte_test),
--   (select predict_arr from cte_predict)
-- ) tb_a(a_x, a_ods_ele, a_test_ele, a_predict_ele)










-- drop function if exists sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt(int[], anyarray);
create or replace function sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt
(
  i_indepdt_len    int[],
  i_dloss_ddepdt   anyarray
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len   int[]   :=  (select array_agg(array_length(i_dloss_ddepdt, a_ndim)) from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_ndims(a_ndim));
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_indepdt_len) <> 1
    then 
      raise exception 'unsupport for ndims of i_indepdt_len > 1';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_indepdt_len, 1)
    then 
      raise exception 'unmatch between dims of i_indepdt, i_depdt and i_dloss_ddepdt.';
    elsif 0 <> any(i_indepdt_len %` v_dloss_ddepdt_len)
    then
      raise exception 'unperfect i_indepdt_len for i_dloss_ddepdt at some dims';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
  else
    return sm_sc.fv_new
    (
      i_dloss_ddepdt /` sm_sc.fv_aggr_slice_prod(i_indepdt_len / v_dloss_ddepdt_len) :: float
    , i_indepdt_len / v_dloss_ddepdt_len
    );
  end if;
  
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt
--   (
--     array[6]
--   , sm_sc.fv_new_rand(array[3])
--   )

-- select 
--   sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt
--   (
--     array[6, 8]
--   , sm_sc.fv_new_rand(array[3, 2])
--   )

-- select 
--   sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt
--   (
--     array[6, 8, 8]
--   , sm_sc.fv_new_rand(array[3, 2, 2])
--   )

-- select 
--   sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt
--   (
--     array[6, 8, 8, 6]
--   , sm_sc.fv_new_rand(array[3, 2, 2, 3])
--   )
-- drop function if exists sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt(anyarray, anyarray, anyarray);
create or replace function sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt
(
  i_indepdt        anyarray,
  i_depdt          anyarray,
  i_dloss_ddepdt   anyarray
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_indepdt_len_y         int         := array_length(i_indepdt, 1);
  v_indepdt_len_x         int         := array_length(i_indepdt, 2);
  v_indepdt_len_x3        int         := array_length(i_indepdt, 3);
  v_indepdt_len_x4        int         := array_length(i_indepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_indepdt) <> array_ndims(i_depdt)
      or array_ndims(i_indepdt) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch between dims of i_indepdt, i_depdt and i_dloss_ddepdt.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
        )
        %` 
        (
          select 
            array_agg(array_length(i_depdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_depdt)) tb_a_cur_dim(a_cur_dim)
        )
      )
      or array_dims(i_indepdt) <> array_dims(i_dloss_ddepdt)
    then 
      raise exception 'unperfect i_indepdt''s length for i_depdt and i_dloss_ddepdt at some dims';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_ndims(i_dloss_ddepdt) = 1
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y]);
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
      :=
        i_dloss_ddepdt *` 
        (
          i_depdt ==` 
          i_indepdt
            [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
        ) :: int[] :: float[]
      ;        
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 2
  then  
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
          [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
        :=
          i_dloss_ddepdt *` 
          (
            i_depdt ==` 
            i_indepdt
              [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
              [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
          ) :: int[] :: float[]
        ;        
      end loop;
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x, v_indepdt_len_x3]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_indepdt_len_x3 / v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
            [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
            [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
          :=
            i_dloss_ddepdt *`
            (
              i_depdt ==` 
              i_indepdt
                [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
                [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
                [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
            ) :: int[] :: float[]
                
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x, v_indepdt_len_x3, v_indepdt_len_x4]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_indepdt_len_x3 / v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_indepdt_len_x4 / v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
              [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
              [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
              [(v_cur_x4 - 1) * v_dloss_ddepdt_len_x4 + 1 : v_cur_x4 * v_dloss_ddepdt_len_x4]
            :=
              i_dloss_ddepdt *` 
              (
                i_depdt ==` 
                i_indepdt
                  [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
                  [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
                  [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
                  [(v_cur_x4 - 1) * v_dloss_ddepdt_len_x4 + 1 : v_cur_x4 * v_dloss_ddepdt_len_x4]
                 
              ) :: int[] :: float[]
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_max(a_indepdt, array[3])
--   , sm_sc.fv_new(1.0 :: float, array[3])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_max(a_indepdt, array[3, 2])
--   , sm_sc.fv_new(1.0 :: float, array[3, 2])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_max(a_indepdt, array[3, 2, 2])
--   , sm_sc.fv_new(1.0 :: float, array[3, 2, 2])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8, 6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_max(a_indepdt, array[3, 2, 2, 3])
--   , sm_sc.fv_new(1.0 :: float, array[3, 2, 2, 3])
--   )
-- from cte_rand
-- drop function if exists sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt(anyarray, anyarray, anyarray);
create or replace function sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt
(
  i_indepdt        anyarray,
  i_depdt          anyarray,
  i_dloss_ddepdt   anyarray
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_indepdt_len_y         int         := array_length(i_indepdt, 1);
  v_indepdt_len_x         int         := array_length(i_indepdt, 2);
  v_indepdt_len_x3        int         := array_length(i_indepdt, 3);
  v_indepdt_len_x4        int         := array_length(i_indepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_indepdt) <> array_ndims(i_depdt)
      or array_ndims(i_indepdt) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch between dims of i_indepdt, i_depdt and i_dloss_ddepdt.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
        )
        %` 
        (
          select 
            array_agg(array_length(i_depdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_depdt)) tb_a_cur_dim(a_cur_dim)
        )
      )
      or array_dims(i_indepdt) <> array_dims(i_dloss_ddepdt)
    then 
      raise exception 'unperfect i_indepdt''s length for i_depdt and i_dloss_ddepdt at some dims';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_ndims(i_dloss_ddepdt) = 1
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y]);
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
      :=
        i_dloss_ddepdt *` 
        (
          i_depdt ==` 
          i_indepdt
            [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
        ) :: int[] :: float[]
      ;        
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 2
  then  
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
          [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
        :=
          i_dloss_ddepdt *` 
          (
            i_depdt ==` 
            i_indepdt
              [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
              [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
          ) :: int[] :: float[]
        ;        
      end loop;
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x, v_indepdt_len_x3]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_indepdt_len_x3 / v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
            [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
            [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
          :=
            i_dloss_ddepdt *`
            (
              i_depdt ==` 
              i_indepdt
                [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
                [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
                [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
            ) :: int[] :: float[]
                
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x, v_indepdt_len_x3, v_indepdt_len_x4]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_indepdt_len_x3 / v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_indepdt_len_x4 / v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
              [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
              [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
              [(v_cur_x4 - 1) * v_dloss_ddepdt_len_x4 + 1 : v_cur_x4 * v_dloss_ddepdt_len_x4]
            :=
              i_dloss_ddepdt *` 
              (
                i_depdt ==` 
                i_indepdt
                  [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
                  [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
                  [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
                  [(v_cur_x4 - 1) * v_dloss_ddepdt_len_x4 + 1 : v_cur_x4 * v_dloss_ddepdt_len_x4]
                 
              ) :: int[] :: float[]
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_min(a_indepdt, array[3])
--   , sm_sc.fv_new(1.0 :: float, array[3])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_min(a_indepdt, array[3, 2])
--   , sm_sc.fv_new(1.0 :: float, array[3, 2])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_min(a_indepdt, array[3, 2, 2])
--   , sm_sc.fv_new(1.0 :: float, array[3, 2, 2])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8, 6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_min(a_indepdt, array[3, 2, 2, 3])
--   , sm_sc.fv_new(1.0 :: float, array[3, 2, 2, 3])
--   )
-- from cte_rand
-- drop function if exists sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt(anyarray, anyarray, anyarray);
create or replace function sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt
(
  i_indepdt        anyarray,
  i_depdt          anyarray,
  i_dloss_ddepdt   anyarray
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_indepdt_len_y         int         := array_length(i_indepdt, 1);
  v_indepdt_len_x         int         := array_length(i_indepdt, 2);
  v_indepdt_len_x3        int         := array_length(i_indepdt, 3);
  v_indepdt_len_x4        int         := array_length(i_indepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_indepdt) <> array_ndims(i_depdt)
      or array_ndims(i_indepdt) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch between dims of i_indepdt, i_depdt and i_dloss_ddepdt.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
        )
        %` 
        (
          select 
            array_agg(array_length(i_depdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_depdt)) tb_a_cur_dim(a_cur_dim)
        )
      )
      or array_dims(i_indepdt) <> array_dims(i_dloss_ddepdt)
    then 
      raise exception 'unperfect i_indepdt''s length for i_depdt and i_dloss_ddepdt at some dims';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_ndims(i_dloss_ddepdt) = 1
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y]);
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
      :=
        i_dloss_ddepdt *` 
        i_depdt /` 
        i_indepdt
          [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
      ;        
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 2
  then  
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
          [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
        :=
          i_dloss_ddepdt *` 
          i_depdt /` 
          i_indepdt
            [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
            [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
        ;        
      end loop;
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x, v_indepdt_len_x3]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_indepdt_len_x3 / v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
            [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
            [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
          :=
            i_dloss_ddepdt *` 
            i_depdt /` 
            i_indepdt
              [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
              [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
              [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then    
    v_ret := array_fill(null :: float, array[v_indepdt_len_y, v_indepdt_len_x, v_indepdt_len_x3, v_indepdt_len_x4]);  
    for v_cur_y in 1 .. v_indepdt_len_y / v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_indepdt_len_x / v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_indepdt_len_x3 / v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_indepdt_len_x4 / v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
              [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
              [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
              [(v_cur_x4 - 1) * v_dloss_ddepdt_len_x4 + 1 : v_cur_x4 * v_dloss_ddepdt_len_x4]
            :=
              i_dloss_ddepdt *` 
              i_depdt /` 
              i_indepdt
                [(v_cur_y - 1) * v_dloss_ddepdt_len_y + 1 : v_cur_y * v_dloss_ddepdt_len_y]
                [(v_cur_x - 1) * v_dloss_ddepdt_len_x + 1 : v_cur_x * v_dloss_ddepdt_len_x]
                [(v_cur_x3 - 1) * v_dloss_ddepdt_len_x3 + 1 : v_cur_x3 * v_dloss_ddepdt_len_x3]
                [(v_cur_x4 - 1) * v_dloss_ddepdt_len_x4 + 1 : v_cur_x4 * v_dloss_ddepdt_len_x4]
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_prod(a_indepdt, array[3])
--   , sm_sc.fv_new_rand(array[3])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_prod(a_indepdt, array[3, 2])
--   , sm_sc.fv_new_rand(array[3, 2])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_prod(a_indepdt, array[3, 2, 2])
--   , sm_sc.fv_new_rand(array[3, 2, 2])
--   )
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8, 6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_chunk_prod(a_indepdt, array[3, 2, 2, 3])
--   , sm_sc.fv_new_rand(array[3, 2, 2, 3])
--   )
-- from cte_rand
-- drop function if exists sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt(int[], anyarray);
create or replace function sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt
(
  i_indepdt_len    int[],
  i_dloss_ddepdt   anyarray
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len   int[]   :=  (select array_agg(array_length(i_dloss_ddepdt, a_ndim)) from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_ndims(a_ndim));
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_indepdt_len) <> 1
    then 
      raise exception 'unsupport for ndims of i_indepdt_len > 1';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_indepdt_len, 1)
    then 
      raise exception 'unmatch between dims of i_indepdt, i_depdt and i_dloss_ddepdt.';
    elsif 0 <> any(i_indepdt_len %` v_dloss_ddepdt_len)
    then
      raise exception 'unperfect i_indepdt_len for i_dloss_ddepdt at some dims';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
  else
    return 
      sm_sc.fv_new
      (
        i_dloss_ddepdt
      , i_indepdt_len / v_dloss_ddepdt_len
      );
  end if;
  
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt
--   (
--     array[6]
--   , sm_sc.fv_new_rand(array[3])
--   )

-- select 
--   sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt
--   (
--     array[6, 8]
--   , sm_sc.fv_new_rand(array[3, 2])
--   )

-- select 
--   sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt
--   (
--     array[6, 8, 8]
--   , sm_sc.fv_new_rand(array[3, 2, 2])
--   )

-- select 
--   sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt
--   (
--     array[6, 8, 8, 6]
--   , sm_sc.fv_new_rand(array[3, 2, 2, 3])
--   )
-- drop function if exists sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt(float[], int[]);
create or replace function sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt
(
  -- i_indepdt_len    int[],
  -- i_depdt          float[],
  i_dloss_ddepdt   float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
  -- v_indepdt_len           int[]       := 
  --   (
  --     select 
  --       array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
  --     from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
  --   )
  -- ;
begin
  -- i_cnt_per_grp := 
  --   sm_sc.fv_coalesce
  --   (
  --     v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
  --   , v_indepdt_len
  --   )
  -- ;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_cnt_per_grp, 1) 
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_cnt_per_grp.';
    -- elsif array_dims(i_depdt) <> array_dims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatch between dims of i_depdt and i_dloss_ddepdt.';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y] * i_cnt_per_grp);
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
      :=
        array_fill(1.0 :: float / sm_sc.fv_aggr_slice_prod(i_cnt_per_grp), i_cnt_per_grp) 
        *` 
        i_dloss_ddepdt
          [v_cur_y]
      ;        
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then  
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
        :=
          array_fill(1.0 :: float / sm_sc.fv_aggr_slice_prod(i_cnt_per_grp), i_cnt_per_grp) 
          *` 
          i_dloss_ddepdt
            [v_cur_y]
            [v_cur_x]
        ;        
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
          :=
            array_fill(1.0 :: float / sm_sc.fv_aggr_slice_prod(i_cnt_per_grp), i_cnt_per_grp) 
            *` 
            i_dloss_ddepdt
              [v_cur_y]
              [v_cur_x]
              [v_cur_x3]
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3, v_dloss_ddepdt_len_x4] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
            :=
              array_fill(1.0 :: float / sm_sc.fv_aggr_slice_prod(i_cnt_per_grp), i_cnt_per_grp) 
              *` 
              i_dloss_ddepdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
                [v_cur_x4]
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--   , array[2]
--   ) :: decimal[] ~=` 3

-- select 
--   sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt
--   (
--     array
--     [
--       [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     ]
--   , array[3, 2]
--   ) :: decimal[] ~=` 3

-- select 
--   sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       ]
--     , [
--         [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       ]
--     ]
--   , array[2, 2, 4]
--   ) :: decimal[] ~=` 3

-- select 
--   sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       , [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       ]
--     , [
--         [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       , [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       ]
--     ]
--   , array[2, 2, 4, 3]
--   ) :: decimal[] ~=` 3
-- drop function if exists sm_sc.fv_d_aggr_slice_max_dloss_dindepdt(anyarray, anyarray, anyarray, int[]);
create or replace function sm_sc.fv_d_aggr_slice_max_dloss_dindepdt
(
  i_indepdt        anyarray,
  i_depdt          anyarray,
  i_dloss_ddepdt   anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
    )
  ;
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_cnt_per_grp.';
    elsif array_dims(i_depdt) <> array_dims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch between dims of i_depdt and i_dloss_ddepdt.';
    elsif 
      0 <> any 
      (
        v_indepdt_len
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_indepdt''s length for i_cnt_per_grp at some dims';
    elsif 
      (
        select 
          array_agg(array_length(i_dloss_ddepdt, a_cur_dim) order by a_cur_dim) 
        from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_cur_dim(a_cur_dim)
      )
      <>  
      (
        v_indepdt_len
        / i_cnt_per_grp
      )
    then 
      raise exception 'unmatch length between i_dloss_ddepdt and i_indepdt devided by i_cnt_per_grp';
    end if;
  end if;
  
  if i_depdt is null
  then 
    i_depdt := sm_sc.fv_aggr_slice_max(i_indepdt, i_cnt_per_grp);
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y] * i_cnt_per_grp);
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
      :=
        (
          i_indepdt
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
          ==` 
          i_depdt
            [v_cur_y]
        ) :: int[] :: float[]
        *` 
        i_dloss_ddepdt
          [v_cur_y]
      ;        
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then  
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
        :=
          (
            i_indepdt
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            ==` 
            i_depdt
              [v_cur_y]
              [v_cur_x]
          ) :: int[] :: float[]
          *` 
          i_dloss_ddepdt
            [v_cur_y]
            [v_cur_x]
        ;        
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
          :=
            (
              i_indepdt
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              ==` 
              i_depdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
            ) :: int[] :: float[]
            *` 
            i_dloss_ddepdt
              [v_cur_y]
              [v_cur_x]
              [v_cur_x3]
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3, v_dloss_ddepdt_len_x4] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
            :=
              (
                i_indepdt
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
                ==` 
                i_depdt
                  [v_cur_y]
                  [v_cur_x]
                  [v_cur_x3]
                  [v_cur_x4]
              ) :: int[] :: float[]
              *` 
              i_dloss_ddepdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
                [v_cur_x4]
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_max(a_indepdt, array[3])
--   , sm_sc.fv_new_rand(array[2])
--   , array[3]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_max(a_indepdt, array[3, 2])
--   , sm_sc.fv_new_rand(array[2, 4])
--   , array[3, 2]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_max(a_indepdt, array[3, 2, 2])
--   , sm_sc.fv_new_rand(array[2, 4, 4])
--   , array[3, 2, 2]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8, 6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_max_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_max(a_indepdt, array[3, 2, 2, 3])
--   , sm_sc.fv_new_rand(array[2, 4, 4, 2])
--   , array[3, 2, 2, 3]
--   ) :: decimal[] ~=` 6
-- from cte_rand
-- drop function if exists sm_sc.fv_d_aggr_slice_min_dloss_dindepdt(anyarray, anyarray, anyarray, int[]);
create or replace function sm_sc.fv_d_aggr_slice_min_dloss_dindepdt
(
  i_indepdt        anyarray,
  i_depdt          anyarray,
  i_dloss_ddepdt   anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
    )
  ;
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_cnt_per_grp.';
    elsif array_dims(i_depdt) <> array_dims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch between dims of i_depdt and i_dloss_ddepdt.';
    elsif 
      0 <> any 
      (
        v_indepdt_len
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_indepdt''s length for i_cnt_per_grp at some dims';
    elsif 
      (
        select 
          array_agg(array_length(i_dloss_ddepdt, a_cur_dim) order by a_cur_dim) 
        from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_cur_dim(a_cur_dim)
      )
      <>  
      (
        v_indepdt_len
        / i_cnt_per_grp
      )
    then 
      raise exception 'unmatch length between i_dloss_ddepdt and i_indepdt devided by i_cnt_per_grp';
    end if;
  end if;
  
  if i_depdt is null
  then 
    i_depdt := sm_sc.fv_aggr_slice_min(i_indepdt, i_cnt_per_grp);
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y] * i_cnt_per_grp);
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
      :=
        (
          i_indepdt
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
          ==` 
          i_depdt
            [v_cur_y]
        ) :: int[] :: float[]
        *` 
        i_dloss_ddepdt
          [v_cur_y]
      ;        
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then  
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
        :=
          (
            i_indepdt
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            ==` 
            i_depdt
              [v_cur_y]
              [v_cur_x]
          ) :: int[] :: float[]
          *` 
          i_dloss_ddepdt
            [v_cur_y]
            [v_cur_x]
        ;        
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
          :=
            (
              i_indepdt
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              ==` 
              i_depdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
            ) :: int[] :: float[]
            *` 
            i_dloss_ddepdt
              [v_cur_y]
              [v_cur_x]
              [v_cur_x3]
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3, v_dloss_ddepdt_len_x4] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
            :=
              (
                i_indepdt
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
                ==` 
                i_depdt
                  [v_cur_y]
                  [v_cur_x]
                  [v_cur_x3]
                  [v_cur_x4]
              ) :: int[] :: float[]
              *` 
              i_dloss_ddepdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
                [v_cur_x4]
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_min(a_indepdt, array[3])
--   , sm_sc.fv_new_rand(array[2])
--   , array[3]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_min(a_indepdt, array[3, 2])
--   , sm_sc.fv_new_rand(array[2, 4])
--   , array[3, 2]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_min(a_indepdt, array[3, 2, 2])
--   , sm_sc.fv_new_rand(array[2, 4, 4])
--   , array[3, 2, 2]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8, 6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_min_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_min(a_indepdt, array[3, 2, 2, 3])
--   , sm_sc.fv_new_rand(array[2, 4, 4, 2])
--   , array[3, 2, 2, 3]
--   ) :: decimal[] ~=` 6
-- from cte_rand
-- drop function if exists sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt(anyarray, anyarray, anyarray, int[]);
create or replace function sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt
(
  i_indepdt        anyarray,
  i_depdt          anyarray,
  i_dloss_ddepdt   anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_indepdt, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_indepdt)) tb_a_cur_dim(a_cur_dim)
    )
  ;
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_cnt_per_grp.';
    elsif array_dims(i_depdt) <> array_dims(i_dloss_ddepdt)
    then 
      raise exception 'unmatch between dims of i_depdt and i_dloss_ddepdt.';
    elsif 
      0 <> any 
      (
        v_indepdt_len
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_indepdt''s length for i_cnt_per_grp at some dims';
    elsif 
      (
        select 
          array_agg(array_length(i_dloss_ddepdt, a_cur_dim) order by a_cur_dim) 
        from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_cur_dim(a_cur_dim)
      )
      <>  
      (
        v_indepdt_len
        / i_cnt_per_grp
      )
    then 
      raise exception 'unmatch length between i_dloss_ddepdt and i_indepdt devided by i_cnt_per_grp';
    end if;
  end if;
  
  if i_depdt is null
  then 
    i_depdt := sm_sc.fv_aggr_slice_prod(i_indepdt, i_cnt_per_grp);
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y] * i_cnt_per_grp);
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
      :=
        i_indepdt
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
        *` 
        (
          i_dloss_ddepdt
            [v_cur_y]
          /
          i_depdt
            [v_cur_y]
        )
      ;        
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then  
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
        :=
          i_indepdt
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          *` 
          (
            i_dloss_ddepdt
              [v_cur_y]
              [v_cur_x]
            / 
            i_depdt
              [v_cur_y]
              [v_cur_x]
          )
        ;        
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
          :=
            i_indepdt
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            *` 
            (
              i_dloss_ddepdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
              / 
              i_depdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
            )
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3, v_dloss_ddepdt_len_x4] * i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
            :=
              i_indepdt
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              *` 
              (
                i_dloss_ddepdt
                  [v_cur_y]
                  [v_cur_x]
                  [v_cur_x3]
                  [v_cur_x4]
                /
                i_depdt
                  [v_cur_y]
                  [v_cur_x]
                  [v_cur_x3]
                  [v_cur_x4]
              )
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_prod(a_indepdt, array[3])
--   , sm_sc.fv_new_rand(array[2])
--   , array[3]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_prod(a_indepdt, array[3, 2])
--   , sm_sc.fv_new_rand(array[2, 4])
--   , array[3, 2]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_prod(a_indepdt, array[3, 2, 2])
--   , sm_sc.fv_new_rand(array[2, 4, 4])
--   , array[3, 2, 2]
--   ) :: decimal[] ~=` 6
-- from cte_rand

-- with 
-- cte_rand as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[6, 8, 8, 6]) as a_indepdt
-- )
-- select 
--   sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt
--   (
--     a_indepdt
--   , sm_sc.fv_aggr_slice_prod(a_indepdt, array[3, 2, 2, 3])
--   , sm_sc.fv_new_rand(array[2, 4, 4, 2])
--   , array[3, 2, 2, 3]
--   ) :: decimal[] ~=` 6
-- from cte_rand
-- drop function if exists sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt(int[], float[], int[]);
create or replace function sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt
(
  i_indepdt_len    int[],
  -- i_depdt          float[],
  i_dloss_ddepdt   float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare   
  v_dloss_ddepdt_len_y    int         := array_length(i_dloss_ddepdt, 1);
  v_dloss_ddepdt_len_x    int         := array_length(i_dloss_ddepdt, 2);
  v_dloss_ddepdt_len_x3   int         := array_length(i_dloss_ddepdt, 3);
  v_dloss_ddepdt_len_x4   int         := array_length(i_dloss_ddepdt, 4);
  v_ret                   float[]     ;
  v_cur_y                 int         ;
  v_cur_x                 int         ;
  v_cur_x3                int         ;
  v_cur_x4                int         ;
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      i_indepdt_len[ : array_length(i_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , i_indepdt_len
    )
  ;
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_indepdt_len) <> 1
    then 
      raise exception 'unsupport for ndims of i_indepdt_len > 1';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_indepdt_len, 1)
    then 
      raise exception 'unmatch between dims of i_indepdt, i_depdt and i_dloss_ddepdt.';
    elsif true <> any(i_indepdt_len ==` (i_cnt_per_grp *` array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3, v_dloss_ddepdt_len_x4]))
    then
      raise exception 'unperfect i_indepdt_len, i_cnt_per_grp for i_dloss_ddepdt at some dims';
    elsif array_ndims(i_cnt_per_grp) > 1
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_cnt_per_grp, 1) 
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_cnt_per_grp.';
    -- elsif array_dims(i_depdt) <> array_dims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatch between dims of i_depdt and i_dloss_ddepdt.';
    end if;
  end if;
    
  if i_dloss_ddepdt is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y] *` i_cnt_per_grp);
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      v_ret
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
      := 
        array_fill
        (
          i_dloss_ddepdt
            [v_cur_y]
        , i_cnt_per_grp
        ) 
      ;        
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then  
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x] *` i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        v_ret
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
        :=
          array_fill
          (
            i_dloss_ddepdt
              [v_cur_y]
              [v_cur_x]
          , i_cnt_per_grp
          ) 
        ;        
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3] *` i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          v_ret
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
          :=
            array_fill
            (
              i_dloss_ddepdt
                [v_cur_y]
                [v_cur_x]
                [v_cur_x3]
            , i_cnt_per_grp
            ) 
          ;        
        end loop;
      end loop;
    end loop;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then    
    v_ret := array_fill(null :: float, array[v_dloss_ddepdt_len_y, v_dloss_ddepdt_len_x, v_dloss_ddepdt_len_x3, v_dloss_ddepdt_len_x4] *` i_cnt_per_grp);  
    for v_cur_y in 1 .. v_dloss_ddepdt_len_y
    loop 
      for v_cur_x in 1 .. v_dloss_ddepdt_len_x
      loop 
        for v_cur_x3 in 1 .. v_dloss_ddepdt_len_x3
        loop 
          for v_cur_x4 in 1 .. v_dloss_ddepdt_len_x4
          loop 
            v_ret
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
            :=
              array_fill
              (
                i_dloss_ddepdt
                  [v_cur_y]
                  [v_cur_x]
                  [v_cur_x3]
                  [v_cur_x4]
              , i_cnt_per_grp
              ) 
            ;        
          end loop;
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--   , array[2]
--   ) :: decimal[] ~=` 3

-- select 
--   sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt
--   (
--     array
--     [
--       [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     ]
--   , array[3, 2]
--   ) :: decimal[] ~=` 3

-- select 
--   sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       ]
--     , [
--         [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--       ]
--     ]
--   , array[2, 2, 4]
--   ) :: decimal[] ~=` 3

-- select 
--   sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       , [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       ]
--     , [
--         [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       , [
--           [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         , [2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--         ]
--       ]
--     ]
--   , array[2, 2, 4, 3]
--   ) :: decimal[] ~=` 3
-- drop function if exists sm_sc.fv_aggr_chunk_avg(float[], int[]);
create or replace function sm_sc.fv_aggr_chunk_avg
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
    )
  ;
  
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    return 
    (
      select 
        sm_sc.fa_mx_avg
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_avg
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    return 
    (
      select 
        sm_sc.fa_mx_avg
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    return 
    (
      select 
        sm_sc.fa_mx_avg
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
            [a_cur_x4 : a_cur_x4 + i_cnt_per_grp[4] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
        , generate_series(1, array_length(i_array, 4), i_cnt_per_grp[4]) tb_a_cur_x4(a_cur_x4)
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_chunk_avg
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_chunk_avg
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_chunk_avg
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- drop function if exists sm_sc.fv_aggr_chunk_max(float[], int[]);
create or replace function sm_sc.fv_aggr_chunk_max
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
    )
  ;
  
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    return 
    (
      select 
        sm_sc.fa_mx_max
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_max
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    return 
    (
      select 
        sm_sc.fa_mx_max
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    return 
    (
      select 
        sm_sc.fa_mx_max
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
            [a_cur_x4 : a_cur_x4 + i_cnt_per_grp[4] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
        , generate_series(1, array_length(i_array, 4), i_cnt_per_grp[4]) tb_a_cur_x4(a_cur_x4)
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_chunk_max
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_chunk_max
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_chunk_max
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- drop function if exists sm_sc.fv_aggr_chunk_min(float[], int[]);
create or replace function sm_sc.fv_aggr_chunk_min
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
    )
  ;
  
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    return 
    (
      select 
        sm_sc.fa_mx_min
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_min
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    return 
    (
      select 
        sm_sc.fa_mx_min
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    return 
    (
      select 
        sm_sc.fa_mx_min
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
            [a_cur_x4 : a_cur_x4 + i_cnt_per_grp[4] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
        , generate_series(1, array_length(i_array, 4), i_cnt_per_grp[4]) tb_a_cur_x4(a_cur_x4)
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_chunk_min
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_chunk_min
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_chunk_min
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- drop function if exists sm_sc.fv_aggr_chunk_prod(float[], int[]);
create or replace function sm_sc.fv_aggr_chunk_prod
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
    )
  ;
  
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    return 
    (
      select 
        sm_sc.fa_mx_prod
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_prod
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    return 
    (
      select 
        sm_sc.fa_mx_prod
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    return 
    (
      select 
        sm_sc.fa_mx_prod
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
            [a_cur_x4 : a_cur_x4 + i_cnt_per_grp[4] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
        , generate_series(1, array_length(i_array, 4), i_cnt_per_grp[4]) tb_a_cur_x4(a_cur_x4)
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_chunk_prod
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_chunk_prod
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15]) +` 0.5 :: float
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_chunk_prod
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8]) +` 0.5 :: float
--   , array[2, 3, 3, 4]
--   )
-- drop function if exists sm_sc.fv_aggr_chunk_sum(float[], int[]);
create or replace function sm_sc.fv_aggr_chunk_sum
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_indepdt_len           int[]       := 
    (
      select 
        array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
      from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
    )
  ;
  
begin
  i_cnt_per_grp := 
    sm_sc.fv_coalesce
    (
      v_indepdt_len[ : array_length(v_indepdt_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_indepdt_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
    )
    ;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    return 
    (
      select 
        sm_sc.fa_mx_sum
        (
          i_array
            [a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1]
            [a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]
            [a_cur_x3 : a_cur_x3 + i_cnt_per_grp[3] - 1]
            [a_cur_x4 : a_cur_x4 + i_cnt_per_grp[4] - 1]
        )
      from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
        , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
        , generate_series(1, array_length(i_array, 3), i_cnt_per_grp[3]) tb_a_cur_x3(a_cur_x3)
        , generate_series(1, array_length(i_array, 4), i_cnt_per_grp[4]) tb_a_cur_x4(a_cur_x4)
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_chunk_sum
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_chunk_sum
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_chunk_sum
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_and(bit[]);
create or replace function sm_sc.fv_aggr_slice_and
(
  i_array          bit[]
)
returns bit
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        bit_and(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_and
--   (
--     array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[[[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]]]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[] :: bit[]
--   );


-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_and(bit[], int[]);
create or replace function sm_sc.fv_aggr_slice_and
(
  i_array          bit[],
  i_cnt_per_grp    int[]
)
returns bit[]
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_and
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_and(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_and
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_and
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_and
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_and
--   (
--     array[[B'00101', B'10101', B'11101', B'10101', B'10101', B'10011']
--          ,[B'10101', B'10101', B'11101', B'10101', B'10101', B'10011']
--          ,[B'01101', B'11101', B'10101', B'10001', B'10101', B'10011']
--          ,[B'00101', B'00101', B'11101', B'10101', B'10001', B'10000']
--          ]
--     , array[2, 3]
--   ) 

-- select
--   sm_sc.fv_aggr_slice_and
--   (
--     array
--     [
--       [[B'00101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'10101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'01101', B'11101', B'10101', B'10001', B'10101', B'10011']
--       ,[B'00101', B'00101', B'11101', B'10101', B'10001', B'10000']
--       ]
--     ]
--   , array[1, 2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_and
--   (
--     array
--     [[
--       [[B'00101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'10101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'01101', B'11101', B'10101', B'10001', B'10101', B'10011']
--       ,[B'00101', B'00101', B'11101', B'10101', B'10001', B'10000']
--       ]
--     ]]
--   , array[1, 1, 2, 3]
--   )
-- ----------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_and(boolean[]);
create or replace function sm_sc.fv_aggr_slice_and
(
  i_array          boolean[]
)
returns boolean
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return i_array[0];
  else
    return 
    ( 
      false <> all(i_array)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_and
--   (
--     array[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[true, false, false, true, false, true, false, true]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[[[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]]]
--   );

-- select sm_sc.fv_aggr_slice_and
--   (
--     array[] :: boolean[]
--   );


-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_and(boolean[], int[]);
create or replace function sm_sc.fv_aggr_slice_and
(
  i_array          boolean[],
  i_cnt_per_grp    int[]
)
returns boolean[]
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        / i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_and
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_and(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_and
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_and
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_and
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_and
--   (
--     array[[true, false, false, true, true, false]
--          ,[true, true, false, true, false, false]
--          ,[false, true, false, true, true, true]
--          ,[false, true, true, false, true, true]
--          ,[false, false, true, false, false, true]
--          ,[true, true, false, true, true, false]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_and
--   (
--     array
--     [
--       [[true, false, false, true, true, false]
--       ,[true, true, false, true, false, false]
--       ,[false, true, false, true, true, true]
--       ,[false, true, true, false, true, true]
--       ,[false, false, true, false, false, true]
--       ,[true, true, false, true, true, false]
--       ]
--     ]
--   , array[1, 2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_and
--   (
--     array
--     [[
--       [[true, false, false, true, true, false]
--       ,[true, true, false, true, false, false]
--       ,[false, true, false, true, true, true]
--       ,[false, true, true, false, true, true]
--       ,[false, false, true, false, false, true]
--       ,[true, true, false, true, true, false]
--       ]
--     ]]
--   , array[1, 1, 2, 3]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_avg(anyarray);
create or replace function sm_sc.fv_aggr_slice_avg
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        avg(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_avg
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_avg
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_avg
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_avg
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_avg
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_avg(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_avg
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  v_array_len  int[]         :=
    (
      select 
        array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
      from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
    )
  ;
  
begin
  i_cnt_per_grp :=
    sm_sc.fv_coalesce
    (
      v_array_len[ : array_length(v_array_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_array_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_avg
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_avg(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_avg
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_avg
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_avg
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_avg
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_slice_avg
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_avg
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_coalesce(anyarray);
create or replace function sm_sc.fv_aggr_slice_coalesce
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        sm_sc.fa_coalesce(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_coalesce
--   (
--     array[array[null,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,null,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_coalesce
--   (
--     array[null,null,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_coalesce
--   (
--     array[[[null,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,null,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_coalesce
--   (
--     array[[[[null,null,3,4,25,6],[null,-2,-3,4,35,6]],[[null,2,3,-4,-5,-36],[null,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_coalesce
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_coalesce(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_coalesce
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_coalesce
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_coalesce(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_coalesce
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_coalesce
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_coalesce
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_coalesce
--   (
--     array[[null,2,3,4,5,6]
--          ,[10,20,null,40,50,60]
--          ,[100,200,300,null,500,600]
--          ,[-1,null,-3,-4,-5,-6]
--          ,[-10,-20,null,null,-50,-60]
--          ,[-100,-200,null,-400,-500,-600]
--          ]::float[]
--     , array[2, 3]
--   )

-- select 
--   sm_sc.fv_aggr_slice_coalesce
--   (
--     array
--     [
--       [
--         [null,2,3,4,5,6]
--       , [10,20,null,40,50,60]
--       , [100,200,300,null,500,600]
--       , [-1,null,-3,-4,-5,-6]
--       , [-10,-20,null,null,-50,-60]
--       , [-100,-200,null,-400,-500,-600]
--       ]
--     ]::float[]
--     , array[1, 2, 3]
--   )

-- select 
--   sm_sc.fv_aggr_slice_coalesce
--   (
--     array
--     [
--       [[
--         [null,2,3,4,5,6]
--       , [10,20,null,40,50,60]
--       , [100,200,300,null,500,600]
--       , [-1,null,-3,-4,-5,-6]
--       , [-10,-20,null,null,-50,-60]
--       , [-100,-200,null,-400,-500,-600]
--       ]]
--     ]::float[]
--     , array[1, 1, 2, 3]
--   )
-- -----------------------------
-- -- bit ele_concat
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_concat(anyarray);
create or replace function sm_sc.fv_aggr_slice_concat
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        sm_sc.fa_concat(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_concat
--   (
--     array[[B'010', B'011', B'010', B'011']
--          ,[B'101', B'011', B'010', B'011']
--          ,[B'101', B'011', B'010', B'011']
--          ,[B'101', B'011', B'010', B'011']
--          ,[B'101', B'011', B'010', B'011']
--          ,[B'101', B'011', B'010', B'011']]
--   );

-- select sm_sc.fv_aggr_slice_concat
--   (
--     array[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']
--   );

-- select sm_sc.fv_aggr_slice_concat
--   (
--     array[[[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']],[[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']]]
--   );

-- select sm_sc.fv_aggr_slice_concat
--   (
--     array[[[[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']]]]
--   );

-- select sm_sc.fv_aggr_slice_concat
--   (
--     array[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_concat(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_concat
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_concat
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_concat(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_concat
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_concat
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_concat
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_concat
--   (
--     array[[B'010', B'011', B'010', B'011', B'0101', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01000', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01011', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01000', B'011']
--          ,[B'101', B'011', B'010', B'011', B'0101', B'011']
--          ,[B'101', B'011', B'010', B'011', B'0101111', B'011']]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_concat
--   (
--     array[[[B'010', B'011', B'010', B'011', B'0101', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01000', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01011', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01000', B'011']
--          ,[B'101', B'011', B'010', B'011', B'0101', B'011']
--          ,[B'101', B'011', B'010', B'011', B'0101111', B'011']]]
--   , array[1, 2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_concat
--   (
--     array[[[[B'010', B'011', B'010', B'011', B'0101', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01000', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01011', B'011']
--          ,[B'101', B'011', B'010', B'011', B'01000', B'011']
--          ,[B'101', B'011', B'010', B'011', B'0101', B'011']
--          ,[B'101', B'011', B'010', B'011', B'0101111', B'011']]]]
--   , array[1, 1, 2, 3]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_is_exists_null(anyarray);
create or replace function sm_sc.fv_aggr_slice_is_exists_null
(
  i_array          anyarray,
  i_ele   anyelement   default  null
)
returns boolean
as
$$
declare 
  v_ret      boolean;
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    v_ret   :=   false;
    foreach i_ele in array i_array
    loop 
      if i_ele is null 
      then 
        v_ret = true;
        exit;
      end if;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_is_exists_null(array[4, 6, 9, null])
-- select sm_sc.fv_aggr_slice_is_exists_null(array[[4, 6, 9, null], [5, 7, 3, 3]])
-- select sm_sc.fv_aggr_slice_is_exists_null(array[[[4, 6, 9, null], [5, 7, 3, 3]]])
-- select sm_sc.fv_aggr_slice_is_exists_null(array[[[[4, 6, 9, null], [5, 7, 3, 3]]]])

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_is_exists_null(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_is_exists_null
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns boolean[]
as
$$
declare 
  v_ret    boolean[]      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        null :: boolean, 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_is_exists_null
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_is_exists_null(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        null :: boolean, 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_is_exists_null
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        null :: boolean, 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_is_exists_null
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        null :: boolean, 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_is_exists_null
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_is_exists_null
--   (
--     array[1,2,null,4,5,6, null,20,30,40,null,60, 100,null,300,400,500,null]
--     , array[3]
--   )

-- select 
--   sm_sc.fv_aggr_slice_is_exists_null
--   (
--     array[[1,2,null,4,5,6]
--          ,[null,20,30,40,null,60]
--          ,[100,null,300,400,500,null]
--          ,[-1,null,-3,null,-5,-6]
--          ,[-1,null,-3,null,-5,-6]
--          ,[-10,null,-30,null,null,-60]]
--     , array[2, 3]
--   )

-- select 
--   sm_sc.fv_aggr_slice_is_exists_null
--   (
--     array[[[1,2,null,4,5,6]
--          ,[null,20,30,40,null,60]
--          ,[100,null,300,400,500,null]
--          ,[-1,null,-3,null,-5,-6]
--          ,[-1,null,-3,null,-5,-6]
--          ,[-10,null,-30,null,null,-60]]]
--     , array[1, 2, 3]
--   )

-- select 
--   sm_sc.fv_aggr_slice_is_exists_null
--   (
--     array[[[[1,2,null,4,5,6]
--          ,[null,20,30,40,null,60]
--          ,[100,null,300,400,500,null]
--          ,[-1,null,-3,null,-5,-6]
--          ,[-1,null,-3,null,-5,-6]
--          ,[-10,null,-30,null,null,-60]]]]
--     , array[1, 1, 2, 3]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_max(anyarray);
create or replace function sm_sc.fv_aggr_slice_max
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        max(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_max
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_max
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_max
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_max
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_max
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_max(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_max
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  v_array_len  int[]         :=
    (
      select 
        array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
      from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
    )
  ;
  
begin
  i_cnt_per_grp :=
    sm_sc.fv_coalesce
    (
      v_array_len[ : array_length(v_array_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_array_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_max
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_max(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_max
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_max
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_max
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_max
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_aggr_slice_max
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_max
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_max
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_median(anyarray);
create or replace function sm_sc.fv_aggr_slice_median
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      with 
      cte_ord_no as
      (
        select 
          row_number() over (order by a_val) as a_ord_no,
          a_val
        from unnest(i_array) tb_a(a_val)
      ),
      cte_cnt as 
      (
        select 
          count(a_ord_no) :: float as a_cnt 
        from cte_ord_no
      )
      select avg(a_val) 
      from cte_ord_no, cte_cnt
      where a_ord_no in (floor((a_cnt + 1.0 :: float) / 2.0 :: float)
                           , ceil((a_cnt + 1.0 :: float) / 2.0 :: float)
                          )
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_median
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1.2,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_median
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_median
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_median
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_median
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_median(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_median
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_median
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_median(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_median
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_median
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_median
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_median
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_aggr_slice_median
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_median
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_median
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_min(anyarray);
create or replace function sm_sc.fv_aggr_slice_min
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        min(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_min
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_min
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_min
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_min
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_min
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_min(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_min
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  v_array_len  int[]         :=
    (
      select 
        array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
      from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
    )
  ;
  
begin
  i_cnt_per_grp :=
    sm_sc.fv_coalesce
    (
      v_array_len[ : array_length(v_array_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_array_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_min
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_min(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_min
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_min
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_min
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_min
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_aggr_slice_min
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_min
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_min
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_mode(anyarray);
create or replace function sm_sc.fv_aggr_slice_mode
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        mode() within group (order by a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_mode
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_mode
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_mode
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_mode
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_mode
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_mode(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_mode
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_mode
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_mode(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_mode
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_mode
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_mode
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_mode
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_aggr_slice_mode
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_mode
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_mode
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_or(bit[]);
create or replace function sm_sc.fv_aggr_slice_or
(
  i_array          bit[]
)
returns bit
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        bit_or(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_or
--   (
--     array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[[[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]]]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[] :: bit[]
--   );


-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_or(bit[], int[]);
create or replace function sm_sc.fv_aggr_slice_or
(
  i_array          bit[],
  i_cnt_per_grp    int[]
)
returns bit[]
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_or
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_or(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_or
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_or
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_or
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_or
--   (
--     array[[B'00101', B'10101', B'11101', B'10101', B'10101', B'10011']
--          ,[B'10101', B'10101', B'11101', B'10101', B'10101', B'10011']
--          ,[B'01101', B'11101', B'10101', B'10001', B'10101', B'10011']
--          ,[B'00101', B'00101', B'11101', B'10101', B'10001', B'10000']
--          ]
--     , array[2, 3]
--   ) 

-- select
--   sm_sc.fv_aggr_slice_or
--   (
--     array
--     [
--       [[B'00101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'10101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'01101', B'11101', B'10101', B'10001', B'10101', B'10011']
--       ,[B'00101', B'00101', B'11101', B'10101', B'10001', B'10000']
--       ]
--     ]
--   , array[1, 2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_or
--   (
--     array
--     [[
--       [[B'00101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'10101', B'10101', B'11101', B'10101', B'10101', B'10011']
--       ,[B'01101', B'11101', B'10101', B'10001', B'10101', B'10011']
--       ,[B'00101', B'00101', B'11101', B'10101', B'10001', B'10000']
--       ]
--     ]]
--   , array[1, 1, 2, 3]
--   )
-- ----------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_or(boolean[]);
create or replace function sm_sc.fv_aggr_slice_or
(
  i_array          boolean[]
)
returns boolean
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return i_array[0];
  else
    return 
    ( 
      true = any(i_array)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_or
--   (
--     array[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[true, false, false, true, false, true, false, true]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[[[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]]]
--   );

-- select sm_sc.fv_aggr_slice_or
--   (
--     array[] :: boolean[]
--   );


-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_or(boolean[], int[]);
create or replace function sm_sc.fv_aggr_slice_or
(
  i_array          boolean[],
  i_cnt_per_grp    int[]
)
returns boolean[]
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        / i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_or
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_or(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_or
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_or
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_or
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_aggr_slice_or
--   (
--     array[[true, false, false, true, true, false]
--          ,[true, true, false, true, false, false]
--          ,[false, true, false, true, true, true]
--          ,[false, true, true, false, true, true]
--          ,[false, false, true, false, false, true]
--          ,[true, true, false, true, true, false]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_or
--   (
--     array
--     [
--       [[true, false, false, true, true, false]
--       ,[true, true, false, true, false, false]
--       ,[false, true, false, true, true, true]
--       ,[false, true, true, false, true, true]
--       ,[false, false, true, false, false, true]
--       ,[true, true, false, true, true, false]
--       ]
--     ]
--   , array[1, 2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_or
--   (
--     array
--     [[
--       [[true, false, false, true, true, false]
--       ,[true, true, false, true, false, false]
--       ,[false, true, false, true, true, true]
--       ,[false, true, true, false, true, true]
--       ,[false, false, true, false, false, true]
--       ,[true, true, false, true, true, false]
--       ]
--     ]]
--   , array[1, 1, 2, 3]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_prod(anyarray);
create or replace function sm_sc.fv_aggr_slice_prod
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        sm_sc.fa_prod(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_prod
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_prod
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_prod
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_prod
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_prod
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_prod(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_prod
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  v_array_len  int[]         :=
    (
      select 
        array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
      from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
    )
  ;
  
begin
  i_cnt_per_grp :=
    sm_sc.fv_coalesce
    (
      v_array_len[ : array_length(v_array_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_array_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_prod
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_prod(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_prod
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_prod
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_prod
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_prod
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_prod
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_prod
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_ptp(anyarray);
create or replace function sm_sc.fv_aggr_slice_ptp
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        max(a_ele) - min(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_ptp
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_ptp
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_ptp
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_ptp
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_ptp
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_ptp(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_ptp
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_ptp
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_ptp(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_ptp
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_ptp
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_ptp
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_ptp
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_ptp
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_ptp
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_stddev_pop(anyarray);
create or replace function sm_sc.fv_aggr_slice_stddev_pop
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        stddev_pop(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_stddev_pop
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_pop
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_pop
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_pop
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_pop
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_stddev_pop(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_stddev_pop
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_stddev_pop
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_stddev_pop(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_stddev_pop
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_stddev_pop
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_stddev_pop
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_stddev_pop
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_stddev_pop
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_stddev_pop
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_stddev_samp(anyarray);
create or replace function sm_sc.fv_aggr_slice_stddev_samp
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        stddev_samp(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_stddev_samp
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_samp
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_samp
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_samp
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_stddev_samp
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_stddev_samp(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_stddev_samp
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_stddev_samp
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_stddev_samp(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_stddev_samp
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_stddev_samp
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_stddev_samp
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_stddev_samp
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_stddev_samp
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_stddev_samp
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_sum(anyarray);
create or replace function sm_sc.fv_aggr_slice_sum
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        sum(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_sum
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_sum
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_sum
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_sum
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_sum
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_sum(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_sum
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  v_array_len  int[]         :=
    (
      select 
        array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
      from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
    )
  ;
  
begin
  i_cnt_per_grp :=
    sm_sc.fv_coalesce
    (
      v_array_len[ : array_length(v_array_len, 1) - coalesce(array_length(i_cnt_per_grp, 1), 0)] || i_cnt_per_grp
    , v_array_len
    )
  ;

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_sum
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_sum(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_sum
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_sum
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_sum
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_sum
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   ) :: decimal[] ~=` 6

-- select
--   sm_sc.fv_aggr_slice_sum
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_sum
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_var_pop(anyarray);
create or replace function sm_sc.fv_aggr_slice_var_pop
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        var_pop(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_var_pop
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_pop
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_pop
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_pop
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_pop
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_var_pop(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_var_pop
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_var_pop
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_var_pop(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_var_pop
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_var_pop
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_var_pop
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_var_pop
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_var_pop
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_var_pop
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_aggr_slice_var_samp(anyarray);
create or replace function sm_sc.fv_aggr_slice_var_samp
(
  i_array          anyarray
)
returns anyelement
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) is null
  then
    return null;
  else
    return
    (
      select 
        var_samp(a_ele)
      from unnest(i_array) a_ele  -- tb_a(a_ele)  --  unnest 表内字段别名，对自定义复合类型数组的支持有 bug?，例如：select a_ele from unnest(array[(1.2, 2.3) :: sm_sc.typ_l_complex]) tb_a(a_ele)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_aggr_slice_var_samp
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_samp
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_samp
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_samp
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_aggr_slice_var_samp
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_aggr_slice_var_samp(anyarray, int[]);
create or replace function sm_sc.fv_aggr_slice_var_samp
(
  i_array          anyarray,
  i_cnt_per_grp    int[]
)
returns anyarray
as
$$
declare 
  v_ret    i_array%type      ;
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    v_ret := 
      array_fill
      (
        nullif(i_array[1], i_array[1]), 
        array[array_length(i_array, 1) / i_cnt_per_grp[1]]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      v_ret[v_cur_y] := 
        sm_sc.fv_aggr_slice_var_samp
        (
          i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_aggr_slice_var_samp(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1], i_array[1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        v_ret[v_cur_y][v_cur_x] := 
          sm_sc.fv_aggr_slice_var_samp
          (
            i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1], i_array[1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          v_ret[v_cur_y][v_cur_x][v_cur_x3] := 
            sm_sc.fv_aggr_slice_var_samp
            (
              i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return v_ret;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    v_ret := 
      array_fill
      (
        nullif(i_array[1][1][1][1], i_array[1][1][1][1]), 
        array
        [
          array_length(i_array, 1) / i_cnt_per_grp[1], 
          array_length(i_array, 2) / i_cnt_per_grp[2], 
          array_length(i_array, 3) / i_cnt_per_grp[3], 
          array_length(i_array, 4) / i_cnt_per_grp[4]
        ]
      );
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][v_cur_x4] := 
              sm_sc.fv_aggr_slice_var_samp
              (
                i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select 
--   sm_sc.fv_aggr_slice_var_samp
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_var_samp
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_aggr_slice_var_samp
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- drop function if exists sm_sc.fv_idx_1d_max(anyarray);
create or replace function sm_sc.fv_idx_1d_max
(
  i_arr           anyarray
)
  returns int
as
$$
declare
  v_cur_max_pos      int;
  v_cur              int    :=   1;
  v_len              int;
begin
  if array_ndims(i_arr) = 1
  then
    v_cur_max_pos := 1;
    v_len :=  array_length(i_arr, 1);
    while v_cur < v_len
    loop
      -- 从第二个遍历至最后一个
      v_cur := v_cur + 1;
    
      if i_arr[v_cur] > i_arr[1]
      then
        i_arr[1] := i_arr[v_cur];
        v_cur_max_pos := v_cur;
      end if;
    end loop;

  elsif array_ndims(i_arr) = 2 and array_length(i_arr, 1) = 1 and array_length(i_arr, 2) >= 1
  then
    v_cur_max_pos := 1;
    v_len :=  array_length(i_arr, 2);
    while v_cur < v_len
    loop
      -- 从第二个遍历至最后一个
      v_cur := v_cur + 1;
    
      if i_arr[1][v_cur] > i_arr[1][1]
      then
        i_arr[1][1] := i_arr[1][v_cur];
        v_cur_max_pos := v_cur;
      end if;
    end loop;

  elsif array_ndims(i_arr) = 2 and array_length(i_arr, 2) = 1 and array_length(i_arr, 1) >= 1
  then
    v_cur_max_pos := 1;
    v_len :=  array_length(i_arr, 1);
    while v_cur < v_len
    loop
      -- 从第二个遍历至最后一个
      v_cur := v_cur + 1;
    
      if i_arr[v_cur][1] > i_arr[1][1]
      then
        i_arr[1][1] := i_arr[v_cur][1];
        v_cur_max_pos := v_cur;
      end if;
    end loop;

  elsif array_ndims(i_arr) = 2 and array_length(i_arr, 1) >= 1 and array_length(i_arr, 2) >= 1
  then
    raise exception 'unsupport 2-dims with array_length(i_arr, 1) > 1 and array_length(i_arr, 2) > 1';
  elsif array_ndims(i_arr) > 2
  then
    raise exception 'unsupport n-dims(n > 2)';
  end if;

  return v_cur_max_pos;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_1d_max(array[1.2, 2.3, 5.6, 52.1])
-- select sm_sc.fv_idx_1d_max(array[array[1.2, 2.3, 5.6, 52.1]])
-- select sm_sc.fv_idx_1d_max(array[array[1.2], array[2.3], array[5.6], array[52.1]])

-- drop function if exists sm_sc.fv_idx_1d_min(anyarray);
create or replace function sm_sc.fv_idx_1d_min
(
  i_arr           anyarray
)
  returns int
as
$$
declare
  v_cur_max_pos      int;
  v_cur              int    :=   1;
  v_len              int;
begin
  if array_ndims(i_arr) = 1
  then
    v_cur_max_pos := 1;
    v_len :=  array_length(i_arr, 1);
    while v_cur < v_len
    loop
      -- 从第二个遍历至最后一个
      v_cur := v_cur + 1;
    
      if i_arr[v_cur] < i_arr[1]
      then
        i_arr[1] := i_arr[v_cur];
        v_cur_max_pos := v_cur;
      end if;
    end loop;

  elsif array_ndims(i_arr) = 2 and array_length(i_arr, 1) = 1 and array_length(i_arr, 2) >= 1
  then
    v_cur_max_pos := 1;
    v_len :=  array_length(i_arr, 2);
    while v_cur < v_len
    loop
      -- 从第二个遍历至最后一个
      v_cur := v_cur + 1;
    
      if i_arr[1][v_cur] < i_arr[1][1]
      then
        i_arr[1][1] := i_arr[1][v_cur];
        v_cur_max_pos := v_cur;
      end if;
    end loop;

  elsif array_ndims(i_arr) = 2 and array_length(i_arr, 2) = 1 and array_length(i_arr, 1) >= 1
  then
    v_cur_max_pos := 1;
    v_len :=  array_length(i_arr, 1);
    while v_cur < v_len
    loop
      -- 从第二个遍历至最后一个
      v_cur := v_cur + 1;
    
      if i_arr[v_cur][1] < i_arr[1][1]
      then
        i_arr[1][1] := i_arr[v_cur][1];
        v_cur_max_pos := v_cur;
      end if;
    end loop;

  elsif array_ndims(i_arr) = 2 and array_length(i_arr, 1) >= 1 and array_length(i_arr, 2) >= 1
  then
    raise exception 'unsupport 2-dims with array_length(i_arr, 1) > 1 and array_length(i_arr, 2) > 1';
  elsif array_ndims(i_arr) > 2
  then
    raise exception 'unsupport n-dims(n > 2)';
  end if;

  return v_cur_max_pos;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_1d_min(array[1.2, 2.3, 5.6, 52.1])
-- select sm_sc.fv_idx_1d_min(array[array[1.2, 2.3, 5.6, 52.1]])
-- select sm_sc.fv_idx_1d_min(array[array[1.2], array[2.3], array[5.6], array[52.1]])

-- drop function if exists sm_sc.fv_idx_mx_max(anyarray);
create or replace function sm_sc.fv_idx_mx_max
(
  i_arr           anyarray
)
  returns int[2]
as
$$
declare
  v_cur_max_pos      int[2];
  v_cur_y            int;
  v_cur_x            int;
  v_len_y            int    :=   array_length(i_arr, 1);
  v_len_x            int    :=   array_length(i_arr, 2);
begin
  if array_ndims(i_arr) = 2
  then
    if array_length(i_arr, 1) > 0 and array_length(i_arr, 2) > 0
    then
      v_cur_max_pos := array[1, 1];
    end if;

    v_cur_y := 1;
    while v_cur_y <= v_len_y
    loop 
      v_cur_x := 1;
      while v_cur_x <= v_len_x
      loop 
        if i_arr[v_cur_y][v_cur_x] > i_arr[1][1]
        then
          i_arr[1][1] := i_arr[v_cur_y][v_cur_x];
          v_cur_max_pos := array[v_cur_y, v_cur_x];
        end if;
        v_cur_x := v_cur_x + 1;
      end loop;
      v_cur_y := v_cur_y + 1;
    end loop;
  else
    raise exception 'unsupport n-dims';
  end if;

  return v_cur_max_pos;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_mx_max(array[array[1.2, 2.3, 5.6, 52.1]])
-- select sm_sc.fv_idx_mx_max(array[array[1.2], array[2.3], array[5.6], array[52.1]])

-- drop function if exists sm_sc.fv_idx_mx_min(anyarray);
create or replace function sm_sc.fv_idx_mx_min
(
  i_arr           anyarray
)
  returns int[2]
as
$$
declare
  v_cur_min_pos      int[2];
  v_cur_y            int;
  v_cur_x            int;
  v_len_y            int    :=   array_length(i_arr, 1);
  v_len_x            int    :=   array_length(i_arr, 2);
begin
  if array_ndims(i_arr) = 2
  then
    if array_length(i_arr, 1) > 0 and array_length(i_arr, 2) > 0
    then
      v_cur_min_pos := array[1, 1];
    end if;

    v_cur_y := 1;
    while v_cur_y <= v_len_y
    loop 
      v_cur_x := 1;
      while v_cur_x <= v_len_x
      loop 
        if i_arr[v_cur_y][v_cur_x] < i_arr[1][1]
        then
          i_arr[1][1] := i_arr[v_cur_y][v_cur_x];
          v_cur_min_pos := array[v_cur_y, v_cur_x];
        end if;
        v_cur_x := v_cur_x + 1;
      end loop;
      v_cur_y := v_cur_y + 1;
    end loop;
  else
    raise exception 'unsupport n-dims';
  end if;

  return v_cur_min_pos;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_mx_min(array[array[1.2, 2.3, 5.6, 52.1]])
-- select sm_sc.fv_idx_mx_min(array[array[1.2], array[2.3], array[5.6], array[52.1]])

-- drop function if exists sm_sc.fv_idx_x_max(anyarray);
create or replace function sm_sc.fv_idx_x_max
(
  i_arr           anyarray
)
  returns int[]
as
$$
-- declare
begin
  if array_ndims(i_arr) = 2
  then
    return
    (
      select 
        array[array_agg(sm_sc.fv_idx_1d_max(i_arr[ : ][a_cur_y : a_cur_y]) order by a_cur_y)]
      from generate_series(1, array_length(i_arr, 2)) tb_a_cur_y(a_cur_y)
    );
  else
    raise exception 'unsupport array_ndims.  ';
  end if;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_x_max(array[array[1.2, 2.3, 5.6, 52.1], array[-1.2, 2.3, -5.6, 2.1]])
-- select sm_sc.fv_idx_x_max(array[array[1.2, 2.3], array[2.3, 5.6], array[1.2, 5.6], array[2.3, 52.1]])

-- drop function if exists sm_sc.fv_idx_x_min(anyarray);
create or replace function sm_sc.fv_idx_x_min
(
  i_arr           anyarray
)
  returns int[]
as
$$
-- declare
begin
  if array_ndims(i_arr) = 2
  then
    return
    (
      select 
        array[array_agg(sm_sc.fv_idx_1d_min(i_arr[ : ][a_cur_y : a_cur_y]) order by a_cur_y)]
      from generate_series(1, array_length(i_arr, 2)) tb_a_cur_y(a_cur_y)
    );
  else
    raise exception 'unsupport array_ndims.  ';
  end if;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_x_min(array[array[1.2, 2.3, 5.6, 52.1], array[-1.2, 2.3, -5.6, 2.1]])
-- select sm_sc.fv_idx_x_min(array[array[1.2, 2.3], array[2.3, 5.6], array[1.2, 5.6], array[2.3, 52.1]])

-- drop function if exists sm_sc.fv_idx_y_max(anyarray);
create or replace function sm_sc.fv_idx_y_max
(
  i_arr           anyarray
)
  returns int[]
as
$$
-- declare
begin
  if array_ndims(i_arr) = 2
  then
    return
    (
      select 
        array_agg(array[sm_sc.fv_idx_1d_max(i_arr[a_cur_y : a_cur_y][ : ])] order by a_cur_y)
      from generate_series(1, array_length(i_arr, 1)) tb_a_cur_y(a_cur_y)
    );
  else
    raise exception 'unsupport array_ndims.  ';
  end if;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_y_max(array[array[1.2, 2.3, 5.6, 52.1], array[-1.2, 2.3, -5.6, 2.1]])
-- select sm_sc.fv_idx_y_max(array[array[1.2, 2.3], array[2.3, 5.6], array[1.2, 5.6], array[2.3, 52.1]])

-- drop function if exists sm_sc.fv_idx_y_min(anyarray);
create or replace function sm_sc.fv_idx_y_min
(
  i_arr           anyarray
)
  returns int[]
as
$$
-- declare
begin
  if array_ndims(i_arr) = 2
  then
    return
    (
      select 
        array_agg(array[sm_sc.fv_idx_1d_min(i_arr[a_cur_y : a_cur_y][ : ])] order by a_cur_y)
      from generate_series(1, array_length(i_arr, 1)) tb_a_cur_y(a_cur_y)
    );
  else
    raise exception 'unsupport array_ndims.  ';
  end if;
end
$$
  language plpgsql stable
parallel safe
  cost 100;

-- select sm_sc.fv_idx_y_min(array[array[1.2, 2.3, 5.6, 52.1], array[-1.2, 2.3, -5.6, 2.1]])
-- select sm_sc.fv_idx_y_min(array[array[1.2, 2.3], array[2.3, 5.6], array[1.2, 5.6], array[2.3, 52.1]])

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_ord_by_col(anyarray, int, boolean);
create or replace function sm_sc.fv_ord_by_col
(
  i_array          anyarray, 
  i_ord_col_no     int,
  i_is_desc        boolean   default  null
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) = 2
  then
    if i_is_desc
    then
      return 
      (
        select 
          array_agg
          (
            (select array_agg(a_ele) from unnest( i_array[a_y : a_y][:] ) tb_a(a_ele))
            order by i_array[a_y][i_ord_col_no] desc
          )
        from generate_subscripts(i_array, 1) a_y
      );
    else
      return 
      (
        select 
          array_agg
          (
            (select array_agg(a_ele) from unnest( i_array[a_y : a_y][:] ) tb_a(a_ele))
            order by i_array[a_y][i_ord_col_no]
          )
        from generate_subscripts(i_array, 1) a_y
      );
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_ord_by_col
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]
--     , 4
--     -- , true
--   );

-- select sm_sc.fv_ord_by_col
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]
--     , 4
--     , true
--   );


-- select sm_sc.fv_ord_by_col
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]
--     , 4
--     , false
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_ord_by_row(anyarray, int, boolean);
create or replace function sm_sc.fv_ord_by_row
(
  i_array          anyarray, 
  i_ord_row_no     int,
  i_is_desc        boolean  default null
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) = 2
  then
    if i_is_desc
    then
      return 
      (
        -- -- select  
        -- --   i_array[a_y][a_x]
        -- -- from generate_subscripts(i_array, 1) tb_a_y(a_y)
        -- --   , generate_subscripts(i_array, 2) tb_a_x(a_x)
        -- -- order by a_y, i_array[i_ord_row_no][a_x] desc

        select
          array_agg(a_row order by a_y)
        from generate_subscripts(i_array, 1) tb_a_y(a_y)
          , lateral (select array_agg(i_array[tb_a_y.a_y][a_x] order by i_array[i_ord_row_no][a_x] desc) as a_row from generate_subscripts(i_array, 2) tb_a_x(a_x)) tb_a_x_row
      );
    else
      return 
      (
        -- -- select  
        -- --   i_array[a_y][a_x]
        -- -- from generate_subscripts(i_array, 1) tb_a_y(a_y)
        -- --   , generate_subscripts(i_array, 2) tb_a_x(a_x)
        -- -- order by a_y, i_array[i_ord_row_no][a_x]

        select
          array_agg(a_row order by a_y)
        from generate_subscripts(i_array, 1) tb_a_y(a_y)
          , lateral (select array_agg(i_array[tb_a_y.a_y][a_x] order by i_array[i_ord_row_no][a_x]) as a_row from generate_subscripts(i_array, 2) tb_a_x(a_x)) tb_a_x_row
      );
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_ord_by_row
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]
--     , 4
--     -- , true
--   );

-- select sm_sc.fv_ord_by_row
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]
--     , 4
--     , true
--   );


-- select sm_sc.fv_ord_by_row
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]
--     , 4
--     , false
--   );


-- drop function if exists sm_sc.fv_d_mx_avg(int, int[]);
create or replace function sm_sc.fv_d_mx_avg
(
  i_mx_cnt             int,
  i_depdt_var_len    int[]
)
returns float[][]
as
$$
declare 
begin
  return array_fill(1.0 :: float/ i_mx_cnt, i_depdt_var_len);
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_avg(3, array[2, 3])
-- drop function if exists sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n(float[], int, int, int[]);
create or replace function sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n
(
  i_dloss_ddepdt       float[] ,
  i_range_lower        int     ,
  i_range_upper        int     ,
  i_indepdt_len        int[]   default null
)
returns float[]
as
$$
declare 
begin
  return 
    sm_sc.fv_aggr_chunk_sum
    (
      i_dloss_ddepdt[ : ][ : ][i_range_lower : i_range_upper]
    , i_indepdt_len
    )
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n
--   (
--     array
--     [[
--       [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
--     , [1.1, 2.1, 3.1, 4.1, 5.1, 6.1]
--     , [1.2, 2.2, 3.2, 4.2, 5.2, 6.2]
--     , [1.3, 2.3, 3.3, 4.3, 5.3, 6.3]
--     , [1.4, 2.4, 3.4, 4.4, 5.4, 6.4]
--     ]]
--   , 2
--   , 3
--   )

-- select 
--   sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n
--   (
--     array
--     [[
--       [[1.0], [2.0], [3.0], [4.0], [5.0], [6.0]]
--     , [[1.1], [2.1], [3.1], [4.1], [5.1], [6.1]]
--     , [[1.2], [2.2], [3.2], [4.2], [5.2], [6.2]]
--     , [[1.3], [2.3], [3.3], [4.3], [5.3], [6.3]]
--     , [[1.4], [2.4], [3.4], [4.4], [5.4], [6.4]]
--     ]]
--   , 2
--   , 4
--   )
-- drop function if exists sm_sc.fv_d_mx_concat_x4_dloss_dindepdt_n(float[], int, int, int[]);
create or replace function sm_sc.fv_d_mx_concat_x4_dloss_dindepdt_n
(
  i_dloss_ddepdt       float[] ,
  i_range_lower        int     ,
  i_range_upper        int     ,
  i_indepdt_len        int[]   default null
)
returns float[]
as
$$
declare 
begin
  return 
    sm_sc.fv_aggr_chunk_sum
    (
      i_dloss_ddepdt[ : ][ : ][ : ][i_range_lower : i_range_upper]
    , i_indepdt_len
    )
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_concat_x4_dloss_dindepdt_n
--   (
--     array
--     [[[
--       [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
--     , [1.1, 2.1, 3.1, 4.1, 5.1, 6.1]
--     , [1.2, 2.2, 3.2, 4.2, 5.2, 6.2]
--     , [1.3, 2.3, 3.3, 4.3, 5.3, 6.3]
--     , [1.4, 2.4, 3.4, 4.4, 5.4, 6.4]
--     ]]]
--   , 2
--   , 3
--   )
-- drop function if exists sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n(float[], int, int, int[]);
create or replace function sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n
(
  i_dloss_ddepdt       float[] ,
  i_range_lower        int     ,
  i_range_upper        int     ,
  i_indepdt_len        int[]   default null
)
returns float[]
as
$$
declare 
begin
  return 
    sm_sc.fv_aggr_chunk_sum
    (
      i_dloss_ddepdt[ : ][i_range_lower : i_range_upper]
    , i_indepdt_len
    )
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n
--   (
--     array
--     [
--       [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
--     , [1.1, 2.1, 3.1, 4.1, 5.1, 6.1]
--     , [1.2, 2.2, 3.2, 4.2, 5.2, 6.2]
--     , [1.3, 2.3, 3.3, 4.3, 5.3, 6.3]
--     , [1.4, 2.4, 3.4, 4.4, 5.4, 6.4]
--     ]
--   , 2
--   , 3
--   )

-- select 
--   sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n
--   (
--     array
--     [
--       [[1.0], [2.0], [3.0], [4.0], [5.0], [6.0]]
--     , [[1.1], [2.1], [3.1], [4.1], [5.1], [6.1]]
--     , [[1.2], [2.2], [3.2], [4.2], [5.2], [6.2]]
--     , [[1.3], [2.3], [3.3], [4.3], [5.3], [6.3]]
--     , [[1.4], [2.4], [3.4], [4.4], [5.4], [6.4]]
--     ]
--   , 2
--   , 4
--   )
-- drop function if exists sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n(float[], int, int, int[]);
create or replace function sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n
(
  i_dloss_ddepdt       float[] ,
  i_range_lower        int     ,
  i_range_upper        int     ,
  i_indepdt_len        int[]   default null
)
returns float[]
as
$$
declare 
begin
  return
    sm_sc.fv_aggr_chunk_sum
    (
      i_dloss_ddepdt[i_range_lower : i_range_upper]
    , i_indepdt_len
    )
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n
--   (
--     array
--     [
--       [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
--     , [1.1, 2.1, 3.1, 4.1, 5.1, 6.1]
--     , [1.2, 2.2, 3.2, 4.2, 5.2, 6.2]
--     , [1.3, 2.3, 3.3, 4.3, 5.3, 6.3]
--     , [1.4, 2.4, 3.4, 4.4, 5.4, 6.4]
--     ]
--   , 2
--   , 3
--   )

-- select 
--   sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n
--   (
--     array
--     [
--       [[1.0], [2.0], [3.0], [4.0], [5.0], [6.0]]
--     , [[1.1], [2.1], [3.1], [4.1], [5.1], [6.1]]
--     , [[1.2], [2.2], [3.2], [4.2], [5.2], [6.2]]
--     , [[1.3], [2.3], [3.3], [4.3], [5.3], [6.3]]
--     , [[1.4], [2.4], [3.4], [4.4], [5.4], [6.4]]
--     ]
--   , 2
--   , 4
--   )
-- drop function if exists sm_sc.fv_d_mx_max(float[], float[]);
create or replace function sm_sc.fv_d_mx_max
(
  i_indepdt_var      float[],
  i_depdt_var        float[]
)
returns float[][]
as
$$
declare 
begin
  return (i_depdt_var ==` i_indepdt_var)::int[]::float[];
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_max
--   (
--     array[[1.2, -2.3, 3.3], [1.4, 2.3, 3.8]],
--     array[[-1.2, -2.3, -3.3], [1.3, 2.2, 3.8]]
--   )
-- drop function if exists sm_sc.fv_d_mx_min(float[], float[]);
create or replace function sm_sc.fv_d_mx_min
(
  i_indepdt_var      float[],
  i_depdt_var        float[]
)
returns float[][]
as
$$
declare 
begin
  return (i_depdt_var ==` i_indepdt_var)::int[]::float[];
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_min
--   (
--     array[[1.2, -2.3, 3.3], [1.4, -2.3, 3.8]],
--     array[[1.3, -2.3, 3.8], [1.5, 2.3, 3.9]]
--   )
-- drop function if exists sm_sc.fv_d_mx_prod(float[], float[]);
create or replace function sm_sc.fv_d_mx_prod
(
  i_indepdt_var      float[],
  i_depdt_var        float[]
)
returns float[][]
as
$$
declare 
begin
  return i_depdt_var /` i_indepdt_var;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_prod
--   (
--     array[[1.2, -2.3, 3.3], [1.4, -2.3, 3.8]],
--     array[[1.3, -2.3, 3.8], [1.5, 2.3, 3.9]]
--   )
-- drop function if exists sm_sc.fv_d_mx_sum(int[]);
create or replace function sm_sc.fv_d_mx_sum
(
  i_depdt_var_len    int[]
)
returns float[][]
as
$$
declare 
begin
  return array_fill(1.0 :: float, i_depdt_var_len);
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_mx_sum(array[2, 3])
-- create or replace aggregate sm_sc.fa_mx_sum (anyarray)
drop aggregate if exists sm_sc.fa_mx_sum (anyarray);
create aggregate sm_sc.fa_mx_sum (anyarray)
(
  sfunc = sm_sc.fv_opr_add,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_sum(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_prod (anyarray)
drop aggregate if exists sm_sc.fa_mx_prod (anyarray);
create aggregate sm_sc.fa_mx_prod (anyarray)
(
  sfunc = sm_sc.fv_opr_mul,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_prod(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
-- ) t

-- -- -- -----------------------------------------------------------------------------------
-- -- -- drop type if exists sm_sc.__typ_arr_float_ex cascade;
-- -- create type sm_sc.__typ_arr_float_ex as
-- -- (
-- --   m_arr_mid      float[],
-- --   m_cnt          int
-- -- );

-- ---------------------
-- create or replace function sm_sc.__fv_arr_float_add_ex
drop function if exists sm_sc.__fv_arr_float_add_ex;
create function sm_sc.__fv_arr_float_add_ex
(
  i_left      sm_sc.__typ_arr_float_ex,
  i_right     float[]
)
returns sm_sc.__typ_arr_float_ex
as
$$
-- declare
begin
  i_left.m_arr_mid := i_left.m_arr_mid +` i_right;
  i_left.m_cnt := coalesce(i_left.m_cnt, 0) + 1;
  return i_left;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_arr_float_add_ex(row(array[array[1.2, 1.3], array[2.3, 2.5]], 2), array[array[1.0 :: float, 1.0], array[2.0 :: float, 2.0]])

-- ---------------------
-- create or replace function sm_sc.__fv_mx_avg_final
drop function if exists sm_sc.__fv_mx_avg_final;
create function sm_sc.__fv_mx_avg_final
(
  i_ret_mid      sm_sc.__typ_arr_float_ex
)
returns float[]
as
$$
-- declare
begin
  return i_ret_mid.m_arr_mid /` (i_ret_mid.m_cnt :: float);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_mx_avg_final(row(array[array[1.2, 1.3], array[2.3, 2.5]], 2))

-- ---------------------
-- create or replace aggregate sm_sc.fa_mx_avg (float[])
drop aggregate if exists sm_sc.fa_mx_avg (float[]);
create aggregate sm_sc.fa_mx_avg (float[])
(
  sfunc = sm_sc.__fv_arr_float_add_ex,
  stype = sm_sc.__typ_arr_float_ex,
  initcond = '("{{0}}",0)',
  finalfunc = sm_sc.__fv_mx_avg_final,
  parallel = safe
);

-- select sm_sc.fa_mx_avg(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
-- ) t

-- -- -- -----------------------------------------------------------------------------------
-- -- -- drop type if exists sm_sc.__typ_arr_cpx_ex cascade;
-- -- create type sm_sc.__typ_arr_cpx_ex as
-- -- (
-- --   m_arr_mid      sm_sc.typ_l_complex[],
-- --   m_cnt          int
-- -- );

-- ---------------------
-- create or replace function sm_sc.__fv_arr_cpx_add_ex
drop function if exists sm_sc.__fv_arr_cpx_add_ex;
create function sm_sc.__fv_arr_cpx_add_ex
(
  i_left      sm_sc.__typ_arr_cpx_ex,
  i_right     sm_sc.typ_l_complex[]
)
returns sm_sc.__typ_arr_cpx_ex
as
$$
-- declare
begin
  i_left.m_arr_mid := i_left.m_arr_mid +` i_right;
  i_left.m_cnt := coalesce(i_left.m_cnt, 0) + 1;
  return i_left;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_arr_cpx_add_ex(row(array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]], 2), array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]])

-- ---------------------
-- drop function if exists sm_sc.__fv_mx_avg_final_cpx;
create or replace function sm_sc.__fv_mx_avg_final_cpx
(
  i_ret_mid      sm_sc.__typ_arr_cpx_ex
)
returns sm_sc.typ_l_complex[]
as
$$
-- declare
begin
  return i_ret_mid.m_arr_mid / (i_ret_mid.m_cnt)::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_mx_avg_final_cpx(row(array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]], 2))

-- ---------------------
-- create or replace aggregate sm_sc.fa_mx_avg (sm_sc.typ_l_complex[])
drop aggregate if exists sm_sc.fa_mx_avg (sm_sc.typ_l_complex[]);
create aggregate sm_sc.fa_mx_avg (sm_sc.typ_l_complex[])
(
  sfunc = sm_sc.__fv_arr_cpx_add_ex,
  stype = sm_sc.__typ_arr_cpx_ex,
  initcond = '("{{""(0.0, 0.0)""}}",0)',
  finalfunc = sm_sc.__fv_mx_avg_final_cpx,
  parallel = safe
);

-- select sm_sc.fa_mx_avg(a_val)
-- from 
-- (
--   select array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]] as a_val
--   union all select array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]]
--   union all select array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]]
--   union all select array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]]
--   union all select array[array[(1.2, 1.6)::sm_sc.typ_l_complex, (1.3, 1.9)::sm_sc.typ_l_complex], array[(2.3, -2.1)::sm_sc.typ_l_complex, (2.5, -2.6)::sm_sc.typ_l_complex]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_inner_prod (anyarray)
drop aggregate if exists sm_sc.fa_mx_inner_prod (anyarray);
create aggregate sm_sc.fa_mx_inner_prod (anyarray)
(
  sfunc = sm_sc.fv_opr_mul,
  stype = anyarray,
  initcond = '{}',
  finalfunc = sm_sc.fv_aggr_slice_sum,
  parallel = safe
);

-- select sm_sc.fa_mx_inner_prod(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
--   union all select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_max (anyarray)
drop aggregate if exists sm_sc.fa_mx_max (anyarray);
create aggregate sm_sc.fa_mx_max (anyarray)
(
  sfunc = sm_sc.fv_opr_greatest,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_max(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[11.2, 21.5, 31.8], array[1.2, 1.5, 1.8]]
--   union all select array[array[31.2, 1.5, 21.8], array[61.2, -11.5, 111.8]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_min (anyarray)
drop aggregate if exists sm_sc.fa_mx_min (anyarray);
create aggregate sm_sc.fa_mx_min (anyarray)
(
  sfunc = sm_sc.fv_opr_least,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_min(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[11.2, 21.5, 31.8], array[1.2, 1.5, 1.8]]
--   union all select array[array[31.2, 1.5, 21.8], array[61.2, -11.5, 111.8]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace function sm_sc.__fv_array_concat_1d
drop function if exists sm_sc.__fv_array_concat_1d(anyarray, anyarray);
create function sm_sc.__fv_array_concat_1d
(
  i_left        anyarray,
  l_right       anyarray
)
returns anyarray
as
$$
begin
  if coalesce(array_ndims(i_left), 1) = 1 and coalesce(array_ndims(l_right), 1) = 1
  then
    return i_left || l_right;
  else
    raise exception 'no support n-d array!';
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- create or replace aggregate sm_sc.fa_array_concat (anyarray)
drop aggregate if exists sm_sc.fa_array_concat (anyarray);
create aggregate sm_sc.fa_array_concat (anyarray)
(
  sfunc = sm_sc.__fv_array_concat_1d,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_array_concat(a_val)
-- from 
-- (
--   select array[1.2, 1.5, 1.8] as a_val
--   union all select array[11.2, 21.5, 31.8]
--   union all select array[31.2, 1.5, 21.8]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_concat_y (anyarray)
drop aggregate if exists sm_sc.fa_mx_concat_y (anyarray);
create aggregate sm_sc.fa_mx_concat_y (anyarray)
(
  sfunc = sm_sc.fv_concat_y,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_concat_y(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[11.2, 21.5, 31.8], array[1.2, 1.5, 1.8]]
--   union all select array[array[31.2, 1.5, 21.8], array[61.2, -11.5, 111.8]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_concat_x (anyarray)
drop aggregate if exists sm_sc.fa_mx_concat_x (anyarray);
create aggregate sm_sc.fa_mx_concat_x (anyarray)
(
  sfunc = sm_sc.fv_concat_x,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_concat_x(a_val)
-- from 
-- (
--   select array[array[1.2, 1.5, 1.8], array[11.2, 11.5, 11.8]] as a_val
--   union all select array[array[11.2, 21.5, 31.8], array[1.2, 1.5, 1.8]]
--   union all select array[array[31.2, 1.5, 21.8], array[61.2, -11.5, 111.8]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_concat_x3 (anyarray)
drop aggregate if exists sm_sc.fa_mx_concat_x3 (anyarray);
create aggregate sm_sc.fa_mx_concat_x3 (anyarray)
(
  sfunc = sm_sc.fv_concat_x3,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_concat_x3(a_val)
-- from 
-- (
--   select array[[[1.2, 1.5], [-1.5, 1.8]], [[11.2, -11.5], [11.8, -11.8]]] as a_val union all 
--   select array[[[11.2, -11.2], [21.5, 31.8]], [[-1.2, 1.5], [-1.8, 1.5]]] union all 
--   select array[[[31.2, 1.5], [-21.8, 21.8]], [[61.2, -11.5], [-61.2, 111.8]]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_concat_x4 (anyarray)
drop aggregate if exists sm_sc.fa_mx_concat_x4 (anyarray);
create aggregate sm_sc.fa_mx_concat_x4 (anyarray)
(
  sfunc = sm_sc.fv_concat_x4,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_concat_x4(a_val)
-- from 
-- (
--   select array[[[[1.2, 1.5, 1.8], [11.2, 11.5, 11.8]]]] as a_val union all 
--   select array[[[[11.2, 21.5, 31.8], [1.2, 1.5, 1.8]]]] union all 
--   select array[[[[31.2, 1.5, 21.8], [61.2, -11.5, 111.8]]]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_concat_per_ele (anyarray)
drop aggregate if exists sm_sc.fa_mx_concat_per_ele (anyarray);
create aggregate sm_sc.fa_mx_concat_per_ele (anyarray)
(
  sfunc = sm_sc.fv_opr_concat,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_concat_per_ele(a_val)
-- from 
-- (
--   select array[array[B'110', B'100', B'101'], array[B'101', B'1010', B'100']] as a_val
--   union all select array[array[B'1010', B'110', B'1010'], array[B'010', B'1010', B'110']]
--   union all select array[array[B'1010', B'10', B'1010'], array[B'10', B'101001010', B'00010']]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_or (varbit[])
drop aggregate if exists sm_sc.fa_mx_or (varbit[]);
create aggregate sm_sc.fa_mx_or (varbit[])
(
  sfunc = sm_sc.fv_opr_or,
  stype = varbit[],
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_or(a_val)
-- from 
-- (
--   select array[array[B'1101', B'100', B'101'], array[B'10', B'100101010', B'10010']] as a_val
--   union all select array[array[B'1010', B'110', B'100'], array[B'00', B'100101010', B'10010']]
--   union all select array[array[B'1010', B'101', B'010'], array[B'10', B'101001010', B'00010']]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_and (varbit[])
drop aggregate if exists sm_sc.fa_mx_and (varbit[]);
create aggregate sm_sc.fa_mx_and (varbit[])
(
  sfunc = sm_sc.fv_opr_and,
  stype = varbit[],
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_and(a_val)
-- from 
-- (
--   select array[array[B'110', B'100', B'101'], array[B'101', B'100', B'100']] as a_val
--   union all select array[array[B'101', B'110', B'101'], array[B'010', B'110', B'110']]
--   union all select array[array[B'110', B'110', B'101'], array[B'101', B'100', B'010']]
-- ) t
-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_or (boolean[])
drop aggregate if exists sm_sc.fa_mx_or (boolean[]);
create aggregate sm_sc.fa_mx_or (boolean[])
(
  sfunc = sm_sc.fv_opr_or,
  stype = boolean[],
  initcond = '{f}',
  parallel = safe
);

-- select sm_sc.fa_mx_or(a_val)
-- from 
-- (
--   select array[array[true, false, true], array[true, false, true]] as a_val
--   union all select array[array[true, false, true], array[true, false, true]]
--   union all select array[array[true, false, true], array[true, false, true]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_and (boolean[])
drop aggregate if exists sm_sc.fa_mx_and (boolean[]);
create aggregate sm_sc.fa_mx_and (boolean[])
(
  sfunc = sm_sc.fv_opr_and,
  stype = boolean[],
  initcond = '{t}',
  parallel = safe
);

-- select sm_sc.fa_mx_or(a_val)
-- from 
-- (
--   select array[array[true, false, true], array[true, false, false]] as a_val
--   union all select array[array[true, true, true], array[true, false, true]]
--   union all select array[array[false, false, true], array[false, false, true]]
-- ) t

-- -----------------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_mx_coalesce (anyarray)
drop aggregate if exists sm_sc.fa_mx_coalesce (anyarray);
create aggregate sm_sc.fa_mx_coalesce (anyarray)
(
  sfunc = sm_sc.fv_coalesce,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_mx_coalesce(a_val)
-- from 
-- (
--   select array[array[null, 12, null], array[14, null, 16]] as a_val
--   union all select array[array[null, 22, 23], array[null, null, 26]]
--   union all select array[array[31, 1.5, 1.8], array[11.2, 11.5, 11.8]]
-- ) t
-- elu: i_depdt = case when i_indepdt >= 0.0 then i_indepdt else i_a * (exp(i_indepdt) - 1.0 :: float) end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_elu_asso(float, float, float);
create or replace function sm_sc.fv_d_elu_asso
(
  i_indepdt                float,
  i_asso_value       float,    -- 
  i_depdt                float  default null    -- 可选参数，用于简化复杂度
)
returns float
as
$$
-- declare
begin
  if i_indepdt > 0.0
  then 
    return 1.0;
  else
    return case when i_depdt is not null then i_depdt + i_asso_value else i_asso_value * exp(i_indepdt) end;
  end if;
end
$$
language plpgsql stable;

-- select sm_sc.fv_d_elu_asso(-2.0 :: float, 0.5 :: float, sm_sc.fv_elu(-2.0 :: float, 0.5 :: float))
-- select sm_sc.fv_d_elu_asso(-2.0 :: float, 0.3, sm_sc.fv_elu(-2.0 :: float, 0.3))
-- select sm_sc.fv_d_elu_asso(-2.0 :: float, 0.2)
-- select sm_sc.fv_d_elu_asso(-2.0 :: float, 0.1)
-- select sm_sc.fv_d_elu_asso(0.0 :: float, 0.5 :: float)
-- select sm_sc.fv_d_elu_asso(1.0 :: float, 0.3)
-- select sm_sc.fv_d_elu_asso(2.0 :: float, 0.2, sm_sc.fv_elu(2.0 :: float, 0.2))
-- select sm_sc.fv_d_elu_asso(3.0, 0.1, sm_sc.fv_elu(3.0, 0.1))
-- select sm_sc.fv_d_elu_asso(2.0 :: float, 0.2)
-- select sm_sc.fv_d_elu_asso(3.0, 0.1)
-- gelu: i_depdt = i_depdt = 0.5 * i_indepdt * (1.0 :: float+ tanh(power(2.0 / pi(), 0.5 :: float) * (i_indepdt + (0.044715 * power(i_indepdt, 3)))))
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_gelu(float);
create or replace function sm_sc.fv_d_gelu
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  return 0.5 * tanh(0.0356774 * power(i_indepdt, 3) + (0.797885 * i_indepdt)) + ((0.0535161 * power(i_indepdt, 3) + (0.398942 * i_indepdt)) / power(cosh(0.0356774 * power(i_indepdt, 3) + (0.797885 * i_indepdt)), 2)) + 0.5;
end
$$
language plpgsql stable;


-- select sm_sc.fv_d_gelu(-2.0 :: float)
-- select sm_sc.fv_d_gelu(-1.0 :: float)
-- select sm_sc.fv_d_gelu(-0.5 :: float)
-- select sm_sc.fv_d_gelu(0.0 :: float)
-- select sm_sc.fv_d_gelu(0.5 :: float)
-- select sm_sc.fv_d_gelu(1.0 :: float)
-- select sm_sc.fv_d_gelu(2.0 :: float)
-- leaky_relu: i_depdt = case when i_indepdt >= 0.0 then i_indepdt else i_a * i_indepdt end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_leaky_relu_asso(float, float);
create or replace function sm_sc.fv_d_leaky_relu_asso
(
  i_indepdt                float,
  i_asso_value       float    -- 
)
returns float
as
$$
-- declare
begin
  return case when i_indepdt > 0.0 then 1.0 :: float else i_asso_value end;
end
$$
language plpgsql stable;


-- select sm_sc.fv_d_leaky_relu_asso(-2.0 :: float, 0.1)
-- select sm_sc.fv_d_leaky_relu_asso(0.0 :: float, 0.2)
-- select sm_sc.fv_d_leaky_relu_asso(1.0 :: float, 0.3)
-- relu: i_depdt = case when i_indepdt >= 0.0 then i_indepdt else 0.0 end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_relu(float);
create or replace function sm_sc.fv_d_relu
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  return case when i_indepdt > 0.0 then 1.0 :: float else 0.0 :: float end;
end
$$
language plpgsql stable;


-- select sm_sc.fv_d_relu(-2.0 :: float)
-- select sm_sc.fv_d_relu(0.0 :: float)
-- select sm_sc.fv_d_relu(1.0 :: float)
-- selu: i_depdt = 1.0507009873554804934193349852946 * case when i_indepdt >= 0.0 then i_indepdt else 1.6732632423543772848170429916717 * (exp(i_indepdt) - 1.0 :: float) end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_selu(float, float);
create or replace function sm_sc.fv_d_selu
(
  i_indepdt                float,
  i_depdt                float  default null    -- 可选参数，用于简化复杂度
)
returns float
as
$$
-- declare
begin
  if i_indepdt > 0.0
  then 
    return 1.0507009873554804934193349852946;
  else
    return case when i_depdt is not null then i_depdt + 1.75809934084737685994021752081231934206024580891220830977098282 else 1.75809934084737685994021752081231934206024580891220830977098282 * exp(i_indepdt) end;
  end if;
end
$$
language plpgsql stable;

-- select sm_sc.fv_d_selu(-2.0 :: float, sm_sc.fv_selu(-2.0 :: float))
-- select sm_sc.fv_d_selu(-2.0 :: float)
-- select sm_sc.fv_d_selu(0.0 :: float)
-- select sm_sc.fv_d_selu(1.0 :: float)
-- select sm_sc.fv_d_selu(2.0 :: float, sm_sc.fv_selu(2.0 :: float))
-- select sm_sc.fv_d_selu(3.0, sm_sc.fv_selu(3.0))
-- select sm_sc.fv_d_selu(2.0 :: float)
-- select sm_sc.fv_d_selu(3.0)
-- sigmoid: i_depdt = 1.0 :: float/ (1.0 :: float+ exp(-i_indepdt))
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_sigmoid(float, float);
create or replace function sm_sc.fv_d_sigmoid
(
  i_indepdt                float  ,
  i_depdt                float  default null    -- 可选参数，用于简化复杂度
)
returns float
as
$$
declare -- here
  v_tmp   float;
begin
  if i_depdt is not null
  then
    return i_depdt * (1.0 :: float- i_depdt);
  else
    v_tmp := nullif(exp(i_indepdt / 2), 0.0 :: float);
    return 1.0 :: float/ power(v_tmp + (1 / v_tmp), 2.0 :: float);
  end if;
end
$$
language plpgsql stable;


-- select sm_sc.fv_d_sigmoid(-2.0 :: float)
-- select sm_sc.fv_d_sigmoid(0.0 :: float)
-- select sm_sc.fv_d_sigmoid(1.0 :: float)
-- select sm_sc.fv_d_sigmoid(-2.0 :: float, sm_sc.fv_sigmoid(-2.0 :: float))
-- select sm_sc.fv_d_sigmoid(0.0 :: float, sm_sc.fv_sigmoid(0.0 :: float))
-- select sm_sc.fv_d_sigmoid(1.0 :: float, sm_sc.fv_sigmoid(1.0 :: float))
-- softplus: i_depdt = ln(1 + exp(i_indepdt))

-- drop function if exists sm_sc.fv_d_softplus(float);
create or replace function sm_sc.fv_d_softplus
(
  i_indepdt    float
)
returns float
as
$$
-- declare
begin
  return 1.0 :: float/ (1 + exp(-i_indepdt));
end
$$
language plpgsql stable;

-- select sm_sc.fv_d_softplus(-2.0 :: float)
-- select sm_sc.fv_d_softplus(0.0 :: float)
-- select sm_sc.fv_d_softplus(1.0 :: float)
-- select sm_sc.fv_d_softplus(2.0 :: float)
-- select sm_sc.fv_d_softplus(3.0)
-- swish: i_depdt = i_indepdt * sigmoid(i_indepdt)
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_d_swish(float, float);
create or replace function sm_sc.fv_d_swish
(
  i_indepdt                float,
  i_depdt                float  default null    -- 可选参数，用于简化复杂度
)
returns float
as
$$
declare -- here
  v_sigmoid  float;
begin
  if i_depdt is not null
  then 
    return i_depdt * (1.0 :: float+ i_indepdt - i_depdt) / i_indepdt;
  else
    v_sigmoid := 1.0 :: float/ (1.0 :: float+ exp(-i_indepdt));
    return v_sigmoid * (1.0 :: float+ (i_indepdt * (1 - v_sigmoid)));
  end if;
end
$$
language plpgsql stable;

-- select sm_sc.fv_d_swish(-2.0 :: float, sm_sc.fv_swish(-2.0 :: float))
-- select sm_sc.fv_d_swish(-2.0 :: float)
-- select sm_sc.fv_d_swish(0.0 :: float)
-- select sm_sc.fv_d_swish(1.0 :: float)
-- select sm_sc.fv_d_swish(2.0 :: float, sm_sc.fv_swish(2.0 :: float))
-- select sm_sc.fv_d_swish(3.0, sm_sc.fv_swish(3.0))
-- select sm_sc.fv_d_swish(2.0 :: float)
-- select sm_sc.fv_d_swish(3.0)
-- elu: i_depdt = case when i_indepdt >= 0.0 then i_indepdt else i_a * (exp(i_indepdt) - 1.0 :: float) end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_elu(float, float);
create or replace function sm_sc.fv_elu
(
  i_indepdt                float,
  i_asso_value       float    -- 
)
returns float
as
$$
-- declare
begin
  return case when i_indepdt >= 0.0 then i_indepdt else i_asso_value * (exp(i_indepdt) - 1.0 :: float) end;
end
$$
language plpgsql stable;

-- select sm_sc.fv_elu(-2.0 :: float, 0.5 :: float)
-- select sm_sc.fv_elu(-2.0 :: float, 0.3)
-- select sm_sc.fv_elu(-2.0 :: float, 0.2)
-- select sm_sc.fv_elu(-2.0 :: float, 0.1)
-- select sm_sc.fv_elu(0.0 :: float, 0.5 :: float)
-- select sm_sc.fv_elu(1.0 :: float, 0.3)
-- select sm_sc.fv_elu(2.0 :: float, 0.2)
-- select sm_sc.fv_elu(3.0, 0.1)

-- gelu: i_depdt = 0.5 * i_indepdt * (1.0 :: float+ tanh(power(2.0 / pi(), 0.5 :: float) * (i_indepdt + (0.044715 * power(i_indepdt, 3)))))
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_gelu(float);
create or replace function sm_sc.fv_gelu
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  return 0.5 * i_indepdt * (1.0 :: float+ tanh(0.7978845608028654 * (i_indepdt + (0.044715 * power(i_indepdt, 3)))));
end
$$
language plpgsql stable;


-- select sm_sc.fv_gelu(-2.0 :: float)
-- select sm_sc.fv_gelu(0.0 :: float)
-- select sm_sc.fv_gelu(1.0 :: float)
-- select sm_sc.fv_gelu(2.0 :: float)
-- select sm_sc.fv_gelu(3.0)

-- leaky_relu: i_depdt = case when i_indepdt >= 0.0 then i_indepdt else i_a * i_indepdt end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_leaky_relu(float, float);
create or replace function sm_sc.fv_leaky_relu
(
  i_indepdt                float,
  i_asso_value       float    -- 
)
returns float
as
$$
-- declare
begin
  return case when i_indepdt >= 0.0 then i_indepdt else i_asso_value * i_indepdt end;
end
$$
language plpgsql stable;

-- select sm_sc.fv_leaky_relu(-2.0 :: float, 0.5 :: float)
-- select sm_sc.fv_leaky_relu(-2.0 :: float, 0.3)
-- select sm_sc.fv_leaky_relu(-2.0 :: float, 0.2)
-- select sm_sc.fv_leaky_relu(-2.0 :: float, 0.1)
-- select sm_sc.fv_leaky_relu(0.0 :: float, 0.5 :: float)
-- select sm_sc.fv_leaky_relu(1.0 :: float, 0.3)
-- select sm_sc.fv_leaky_relu(2.0 :: float, 0.2)
-- select sm_sc.fv_leaky_relu(3.0, 0.1)

-- relu: i_depdt = case when i_indepdt >= 0.0 then i_indepdt else 0.0 end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_relu(float);
create or replace function sm_sc.fv_relu
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  return greatest(i_indepdt, 0.0 :: float);
end
$$
language plpgsql stable;


-- select sm_sc.fv_relu(-2.0 :: float)
-- select sm_sc.fv_relu(0.0 :: float)
-- select sm_sc.fv_relu(1.0 :: float)

-- selu: i_depdt = 1.0507009873554804934193349852946 * case when i_indepdt >= 0.0 then i_indepdt else 1.6732632423543772848170429916717 * (exp(i_indepdt) - 1.0 :: float) end
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_selu(float);
create or replace function sm_sc.fv_selu
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  return 1.0507009873554804934193349852946 * case when i_indepdt >= 0.0 then i_indepdt else 1.6732632423543772848170429916717 * (exp(i_indepdt) - 1.0 :: float) end;
end
$$
language plpgsql stable;

-- select sm_sc.fv_selu(-4.0)
-- select sm_sc.fv_selu(-3.0)
-- select sm_sc.fv_selu(-2.0 :: float)
-- select sm_sc.fv_selu(-1.0 :: float)
-- select sm_sc.fv_selu(0.0 :: float)
-- select sm_sc.fv_selu(1.0 :: float)
-- select sm_sc.fv_selu(2.0 :: float)
-- select sm_sc.fv_selu(3.0)

-- sigmoid: i_depdt = 1.0 :: float/ (1.0 :: float+ exp(-i_indepdt))
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_sigmoid(float);
create or replace function sm_sc.fv_sigmoid
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  if i_indepdt >= 0 
  then
    return 
      -- 幂运算下溢元素，用 -inf 替代
      1.0 :: float
      / 
      (
        1.0 :: float
        + 
        exp
        (
          case 
            when i_indepdt > 7.45e2 
              then '-inf' 
            else -i_indepdt 
          end
        )
      )
    ;
  else 
    -- 上溢，直接返回
    if i_indepdt < -7.09e2 :: float 
    then 
      return 0.0 :: float;
    else 
      return 
        -- 幂运算下溢元素，用 -inf 替代
        1.0 :: float
        / 
        (
          1.0 :: float
          + 
          exp
          (
            -i_indepdt
          )
        )
      ;
    end if;
  end if;
end
$$
language plpgsql stable;


-- select sm_sc.fv_sigmoid(-2.0 :: float)
-- select sm_sc.fv_sigmoid(0.0 :: float)
-- select sm_sc.fv_sigmoid(1.0 :: float)

-- drop function if exists sm_sc.fv_softplus(float);
create or replace function sm_sc.fv_softplus
(
  i_indepdt    float
)
returns float
as
$$
-- declare
begin
  return ln(1 + exp(i_indepdt));
end
$$
language plpgsql stable;

-- select sm_sc.fv_softplus(-2.0 :: float)
-- select sm_sc.fv_softplus(0.0 :: float)
-- select sm_sc.fv_softplus(1.0 :: float)
-- select sm_sc.fv_softplus(2.0 :: float)
-- select sm_sc.fv_softplus(3.0)
-- swish: i_depdt = i_indepdt * sigmoid(i_indepdt)
-- https://baijiahao.baidu.com/s?id=1653421414340022957
-- https://blog.csdn.net/qq_29831163/article/details/89887655
-- drop function if exists sm_sc.fv_swish(float);
create or replace function sm_sc.fv_swish
(
  i_indepdt                float
)
returns float
as
$$
-- declare
begin
  return i_indepdt / (1.0 :: float+ exp(-i_indepdt));
end
$$
language plpgsql stable;


-- select sm_sc.fv_swish(-2.0 :: float)
-- select sm_sc.fv_swish(0.0 :: float)
-- select sm_sc.fv_swish(1.0 :: float)


-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_d_redistr_softmax_1d_dloss_dindepdt(float[], float[], float[]);
create or replace function sm_sc.__fv_d_redistr_softmax_1d_dloss_dindepdt
(
  i_depdt                  float[]                 ,                     -- softmax 输出
  i_dloss_ddepdt           float[]                 ,                     -- 此入参传入 dloss/dindepdt, 用于 softmax 直接求取 dloss/ddepdt
  i_indepdt                float[]                                       -- softmax 算子的输入，来自上一层算子的输出
)
returns float[]     -- 输出列序与 i_indepdt 枚举值 one_hot 序一致
as
$$
declare 
  v_y                 float[];
  v_dloss_dindepdt          float[];

begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度
    if array_ndims(i_indepdt) <> 2 or array_length(i_indepdt, 1) <> 1 and array_length(i_indepdt, 2) <> 1
      or array_ndims(i_depdt) <> 2 or array_length(i_depdt, 1) <> 1 and array_length(i_depdt, 2) <> 1
      or array_dims(i_dloss_ddepdt) <> array_dims(i_indepdt) and array_dims(i_dloss_ddepdt) <> array_dims(i_depdt)
    then 
      raise exception 'array_ndims and (array_length(, 1) or array_length(, 2)) should be 2 and 1';
    end if;
  end if;
  
  if i_depdt is null 
  then 
    i_depdt := sm_sc.fv_redistr_softmax(i_indepdt);
  end if;

  -- if array_length(i_depdt, 2) = 1
  -- then
  --   v_y := |^~| i_depdt;
  -- else
  --   v_y := i_depdt;
  --   i_depdt := |^~| i_depdt;
  -- end if;
  -- 
  -- -- -- -- i_depdt |**| v_y 是对称矩阵，开销可优化减半
  -- -- to avoid 'ERROR: array size exceeds the maximum allowed (134217727 = 128 * 1024 * 1024 -1)'
  -- v_dloss_dindepdt := (sm_sc.fv_eye(0.0 :: float, 2, variadic sm_sc.fv_mx_ele_2d_2_1d(v_y)) -` (i_depdt |**| v_y))  *` i_dloss_ddepdt; -- ~=` 8
  -- 
  -- if array_length(i_dloss_ddepdt, 2) = 1
  -- then
  --   return |^~| (sm_sc.fv_aggr_slice_sum(v_dloss_dindepdt, array[array_length(v_dloss_dindepdt, 1), 1]));
  -- else
  --   return |^~| (sm_sc.fv_aggr_slice_sum(v_dloss_dindepdt, array[1, array_length(v_dloss_dindepdt, 2)]));
  -- end if;
  
  -- 零概率的预测值将导致 softmax 求导失效，所以用一个极小概率对零值替换截断
  i_depdt := i_depdt @>` (exp(-2.0e2) :: float);
  
  return 
    i_depdt *` (i_dloss_ddepdt -` (|@+| (i_depdt *` i_dloss_ddepdt)))
  ;

end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_d_redistr_softmax_1d_dloss_dindepdt
--   (
--     sm_sc.fv_redistr_softmax(array[array[1, 2, 3, 4, 5]] :: float[]),
--     -` array[array[0.0 :: float, 0.0 :: float, 0.0 :: float, 0.0 :: float, 1.0]] /` sm_sc.fv_redistr_softmax(array[array[1, 2, 3, 4, 5]] :: float[]),
--     array[array[1, 2, 3, 4, 5]]
--   );

-- select sm_sc.__fv_d_redistr_softmax_1d_dloss_dindepdt
--   (
--     sm_sc.fv_redistr_softmax(array[array[1], array[2], array[3], array[4], array[5]] :: float[]),
--     -` array[array[0.0 :: float], array[0.0 :: float], array[0.0 :: float], array[0.0 :: float], array[1.0 :: float]] /` sm_sc.fv_redistr_softmax(array[array[1], array[2], array[3], array[4], array[5]] :: float[]),
--     array[array[1], array[2], array[3], array[4], array[5]]
--   );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_redistr_softmax_dloss_dindepdt(float[], float[], float[], int[]);
create or replace function sm_sc.fv_d_redistr_softmax_dloss_dindepdt
(
  i_depdt                  float[]                                      -- softmax 输出
, i_dloss_ddepdt           float[]                                      -- 此入参传入 dloss/dindepdt, 用于 softmax 直接求取 dloss/ddepdt
, i_indepdt                float[]      default null                    -- softmax 算子的输入，来自上一层算子的输出
, i_cnt_per_grp            int[]        default null
)
returns float[]     -- 输出列序与 i_indepdt 枚举值 one_hot 序一致
as
$$
declare 
  v_len               int;
  v_stddev_samp       float;
begin
  if i_cnt_per_grp is null
  then 
    i_cnt_per_grp := 
      (
        select 
          array_agg(array_length(i_dloss_ddepdt, a_dim_no) order by a_dim_no)
        from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a(a_dim_no)
      )
    ;
  elsif sm_sc.fv_aggr_slice_is_exists_null(i_cnt_per_grp)
  then 
    i_cnt_per_grp :=
      sm_sc.fv_coalesce
      (
        i_cnt_per_grp
      , (
          select 
            array_agg(array_length(i_dloss_ddepdt, a_dim_no) order by a_dim_no)
          from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a(a_dim_no)
        )
      )
    ;
  end if;
  
  -- 审计维度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_dloss_ddepdt) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_dloss_ddepdt and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_dloss_ddepdt, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_dloss_ddepdt''s length for i_cnt_per_grp at some dims';
    end if;

    if array_ndims(i_depdt) > 4 or array_ndims(i_indepdt) > 4 or array_ndims(i_dloss_ddepdt) > 4
      or array_dims(i_dloss_ddepdt) <> array_dims(i_depdt) or array_dims(i_dloss_ddepdt) <> array_dims(i_indepdt)
      -- -- or array_length(i_dloss_ddepdt, 1) <> array_length(i_depdt, 1)
      -- -- or array_length(i_dloss_ddepdt, 2) <> array_length(i_depdt, 2)
      -- -- or array_length(i_dloss_ddepdt, 3) <> array_length(i_depdt, 3)
      -- -- or array_length(i_dloss_ddepdt, 4) <> array_length(i_depdt, 4)
      -- -- or array_length(i_dloss_ddepdt, 1) <> array_length(i_indepdt, 1)
      -- -- or array_length(i_dloss_ddepdt, 2) <> array_length(i_indepdt, 2)
      -- -- or array_length(i_dloss_ddepdt, 3) <> array_length(i_indepdt, 3)
      -- -- or array_length(i_dloss_ddepdt, 4) <> array_length(i_indepdt, 4)
    then 
      raise exception 'unmatch ndims or length between i_depdt, i_indepdt and i_dloss_ddepdt.';
    end if;
  end if;

  if i_depdt is null
  then
    i_depdt := sm_sc.fv_redistr_softmax(i_indepdt, i_cnt_per_grp);
  end if;

  return 
    i_depdt 
    *` 
    (
      i_dloss_ddepdt 
      -` 
      sm_sc.fv_repeat_axis_py
      (
        (
          i_depdt 
          *` 
          i_dloss_ddepdt
        ) 
        |@+| 
        i_cnt_per_grp
      , (select array_agg(a_no order by a_no) from generate_series(1, array_ndims(i_dloss_ddepdt)) tb_a(a_no))
      , i_cnt_per_grp
      )
    )
  ; -- ~=` 8
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_redistr_softmax_dloss_dindepdt
--   (
--     sm_sc.fv_redistr_softmax(array[array[1, 5], array[2, 4], array[3, 3], array[4, 2], array[5, 1]]),
--     (-` array[array[0.0 :: float, 1.0], array[0.0 :: float, 0.0], array[0.0 :: float, 0.0], array[0.0 :: float, 0.0], array[1.0 :: float, 0.0]] /` sm_sc.fv_redistr_softmax(array[array[1, 5], array[2, 4], array[3, 3], array[4, 2], array[5, 1]])),
--     array[array[1, 5], array[2, 4], array[3, 3], array[4, 2], array[5, 1]]
--   );

-- select sm_sc.fv_d_redistr_softmax_dloss_dindepdt
--   (
--     sm_sc.fv_redistr_softmax(array[array[1, 5], array[2, 4], array[3, 3], array[4, 2], array[5, 1]]),
--     (-` array[array[0.0 :: float, 1.0], array[0.0 :: float, 0.0], array[0.0 :: float, 0.0], array[0.0 :: float, 0.0], array[1.0 :: float, 0.0]] /` sm_sc.fv_redistr_softmax(array[array[1, 5], array[2, 4], array[3, 3], array[4, 2], array[5, 1]])),
--     array[array[1, 5], array[2, 4], array[3, 3], array[4, 2], array[5, 1]]
--   );

-- with 
-- cte_arr as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[2, 3, 4]) as a_arr
-- )
-- select 
--   sm_sc.fv_d_redistr_softmax_dloss_dindepdt
--   (
--     sm_sc.fv_redistr_softmax(a_arr)
--   , array[[[0,0,0,0],[0,0,0,0],[0,0,0,0]],[[1,0,0,0],[0,0,0,0],[0,0,0,0]]]
--   , a_arr
--   ) 
-- from cte_arr

-- with 
-- cte_arr as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[2, 3, 4, 2]) as a_arr
-- )
-- select 
--   sm_sc.fv_d_redistr_softmax_dloss_dindepdt
--   (
--     sm_sc.fv_redistr_softmax(a_arr)
--   , array[[[[0,0],[0,0],[0,0],[0,0]],[[0,0],[0,0],[0,0],[0,0]],[[0,0],[0,0],[0,0],[0,0]]],[[[1,0],[0,0],[0,0],[0,0]],[[0,0],[0,0],[0,0],[0,0]],[[0,0],[0,0],[0,0],[0,0]]]]
--   , a_arr
--   ) 
-- from cte_arr

-- do
-- $$
-- declare 
--   v_dloss_ddepdt   float[]  :=   sm_sc.fv_new(0.0, array[4, 6, 9]);
-- begin
--   v_dloss_ddepdt[1][1][1]   :=   1.0;
--   raise notice '%',
--   (
--     with 
--     cte_arr as 
--     (
--       select 
--         sm_sc.fv_new_rand(array[4, 6, 9]) as a_arr
--     )
--     select 
--       sm_sc.fv_d_redistr_softmax_dloss_dindepdt
--       (
--         sm_sc.fv_redistr_softmax(a_arr)
--       , v_dloss_ddepdt
--       , a_arr
--       , array[2, 3, 3]
--       ) 
--     from cte_arr
--   );
-- end
-- $$
-- language plpgsql

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_redistr_zscore(float[], float[], int[]);
create or replace function sm_sc.fv_d_redistr_zscore
(
  i_depdt_var                  float[]                                     -- zscore 输出
-- , i_dloss_ddepdt               float[]                                      -- 此入参传入 dloss/dindepdt, 用于 zscore 直接求取 dloss/ddepdt
, i_indepdt_var                float[]                                     -- zscore 算子的输入，来自上一层算子的输出
, i_cnt_per_grp                int[]            default null
)
returns float[]     -- 输出列序与 i_indepdt_var 枚举值 one_hot 序一致
as
$$
declare 
  v_len               float;
  v_stddev_samp       float[];

begin
  if i_cnt_per_grp is null
  then 
    i_cnt_per_grp := 
      (
        select 
          array_agg(array_length(i_depdt_var, a_dim_no) order by a_dim_no)
        from generate_series(1, array_ndims(i_depdt_var)) tb_a(a_dim_no)
      )
    ;
  elsif sm_sc.fv_aggr_slice_is_exists_null(i_cnt_per_grp)
  then 
    i_cnt_per_grp :=
      sm_sc.fv_coalesce
      (
        i_cnt_per_grp
      , (
          select 
            array_agg(array_length(i_depdt_var, a_dim_no) order by a_dim_no)
          from generate_series(1, array_ndims(i_depdt_var)) tb_a(a_dim_no)
        )
      )
    ;
  end if;
  
  -- 审计维度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_depdt_var) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_depdt_var and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_depdt_var, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_depdt_var)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_depdt_var''s length for i_cnt_per_grp at some dims';
    end if;

    if array_ndims(i_depdt_var) > 4 or array_ndims(i_indepdt) > 4 or array_ndims(i_depdt_var) > 4
      or array_dims(i_depdt_var) <> array_dims(i_indepdt)
      -- -- or array_length(i_depdt_var, 1) <> array_length(i_indepdt, 1)
      -- -- or array_length(i_depdt_var, 2) <> array_length(i_indepdt, 2)
      -- -- or array_length(i_depdt_var, 3) <> array_length(i_indepdt, 3)
      -- -- or array_length(i_depdt_var, 4) <> array_length(i_indepdt, 4)
    then 
      raise exception 'unmatch ndims or length between i_depdt_var, i_indepdt and i_depdt_var.';
    end if;
  end if;

  if i_depdt_var is null
  then
    i_depdt_var := sm_sc.fv_redistr_zscore(i_indepdt_var, i_cnt_per_grp);
  end if;
  
  v_stddev_samp :=
    sm_sc.fv_repeat_axis_py
    (
      sm_sc.fv_aggr_slice_stddev_samp(i_indepdt_var, i_cnt_per_grp)
    , (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no))
    , i_cnt_per_grp
    )
  ;
  v_len := |@*| i_cnt_per_grp;

  return 
    -- (
    (
      (
        (- 1.0 :: float /` (v_len *` v_stddev_samp)) 
        *` 
        (
          i_depdt_var 
          *` 
          sm_sc.fv_repeat_axis_py
          (
            i_depdt_var |@+| i_cnt_per_grp
          , (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no))
          , i_cnt_per_grp
          )
        )
      ) 
      -` (1.0 :: float /` (v_stddev_samp *` v_len))
    )
    +` (i_depdt_var +` ((v_len - 1.0 :: float) /` v_stddev_samp))
    -- )  
    -- *` i_dloss_ddepdt
  ; -- ~=` 8

end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_redistr_zscore
--   (
--     sm_sc.fv_redistr_zscore(array[[1, 2, 3, 4, 5]]),
--     array[[1, 2, 3, 4, 5]]
--   );

-- select sm_sc.fv_d_redistr_zscore
--     sm_sc.fv_d_redistr_zscore(array[array[1], array[2], array[3], array[4], array[5]]),
--   (
--     -- -` array[array[0.0 :: float], array[0.0 :: float], array[0.0 :: float], array[0.0 :: float], array[1.0 :: float]] /` sm_sc.fv_standlize_mx_zscore(array[array[1], array[2], array[3], array[4], array[5]]),
--     array[array[1], array[2], array[3], array[4], array[5]]
--   );

-- with 
-- cte_arr as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[2, 3, 4]) as a_arr
-- )
-- select 
--   sm_sc.fv_d_redistr_zscore(sm_sc.fv_redistr_zscore(a_arr), a_arr) 
-- from cte_arr

-- with 
-- cte_arr as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[2, 3, 4, 3]) as a_arr
-- )
-- select 
--   sm_sc.fv_d_redistr_zscore(sm_sc.fv_redistr_zscore(a_arr), a_arr) 
-- from cte_arr

-- with 
-- cte_arr as 
-- (
--   select 
--     sm_sc.fv_new_rand(array[4, 6, 9]) as a_arr
-- )
-- select 
--   sm_sc.fv_d_redistr_zscore
--   (
--     sm_sc.fv_redistr_zscore(a_arr)
--   , a_arr
--   , array[2, 3, 3]
--   ) 
-- from cte_arr
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_redistr_0_1(float[]);
create or replace function sm_sc.fv_redistr_0_1
(
  i_array          float[]
)
returns float[]
as
$$
declare -- here
  v_max     float   :=   sm_sc.fv_aggr_slice_max(i_array) ;
  v_min     float   :=   sm_sc.fv_aggr_slice_min(i_array) ;
  v_ptp     float   :=   v_max - v_min;      
begin
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) <= 4
  then
    if abs(v_ptp) < 1e-256
    then 
      if v_max = 0.0
      then 
        return i_array;
      else 
        return array_fill(1.0 :: float, (select array_agg(array_length(i_array, a_dim) order by a_dim) from generate_series(1, array_ndims(i_array)) tb_a_dim(a_dim)));
      end if;
    else
      return (i_array -` v_min) /` v_ptp;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_0_1
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_redistr_0_1
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_redistr_0_1
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_redistr_0_1
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_redistr_0_1
--   (
--     array[]::float[]
--   );

-- select sm_sc.fv_redistr_0_1
--   (
--     array[0.0, 0.0, 0.0]::float[]
--   );

-- select sm_sc.fv_redistr_0_1
--   (
--     array[0.5, 0.5, 0.5]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_redistr_0_1(float[], int[]);
create or replace function sm_sc.fv_redistr_0_1
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      i_array
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
      := 
        sm_sc.fv_redistr_0_1
        (
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_redistr_0_1(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]] 
        := 
          sm_sc.fv_redistr_0_1
          (
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]] 
          := 
            sm_sc.fv_redistr_0_1
            (
              i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]] 
            := 
              sm_sc.fv_redistr_0_1
              (
                i_array
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_redistr_0_1
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9] :: float[]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_0_1
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_0_1
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_0_1
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_redistr_centralize(float[]);
create or replace function sm_sc.fv_redistr_centralize
(
  i_array          float[]
)
returns float[]
as
$$
-- declare -- here
begin
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) <= 4
  then
    return i_array -` sm_sc.fv_aggr_slice_avg(i_array);
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_centralize
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_redistr_centralize
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_redistr_centralize
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_redistr_centralize
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_redistr_centralize
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_redistr_centralize(float[], int[]);
create or replace function sm_sc.fv_redistr_centralize
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      i_array
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
      := 
        sm_sc.fv_redistr_centralize
        (
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_redistr_centralize(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]] 
        := 
          sm_sc.fv_redistr_centralize
          (
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]] 
          := 
            sm_sc.fv_redistr_centralize
            (
              i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]] 
            := 
              sm_sc.fv_redistr_centralize
              (
                i_array
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_redistr_centralize
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9] :: float[]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_centralize
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_centralize
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_centralize
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_redistr_l1l2ln(float[], int);
create or replace function sm_sc.fv_redistr_l1l2ln
(
  i_array          float[]    ,
  i_n              int
)
returns float[]
as
$$
declare -- here
  v_norm_l1l2ln     float   :=   sm_sc.fv_aggr_slice_sum((@|`  i_array) ^` i_n :: float) ^ (1.0 :: float/ i_n :: float);
begin
  if array_ndims(i_array) is null
  then 
    return array[] :: float[];
  elsif array_ndims(i_array) <= 4
  then
    if abs(v_norm_l1l2ln) < 1e-256
    then 
      return i_array;
    else 
      return i_array /` v_norm_l1l2ln ;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_l1l2ln
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   , 3
--   );

-- select sm_sc.fv_redistr_l1l2ln
--   (
--     array[1,2,3,4,5,6]::float[]
--   , 3
--   );

-- select sm_sc.fv_redistr_l1l2ln
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   , 3
--   );

-- select sm_sc.fv_redistr_l1l2ln
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   , 3
--   );

-- select sm_sc.fv_redistr_l1l2ln
--   (
--     array[]::float[]
--   , 3
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_redistr_l1l2ln(float[], int, int[]);
create or replace function sm_sc.fv_redistr_l1l2ln
(
  i_array          float[]    ,
  i_n              int         ,
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      i_array
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
      := 
        sm_sc.fv_redistr_l1l2ln
        (
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        , i_n
        );
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_redistr_l1l2ln(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]] 
        := 
          sm_sc.fv_redistr_l1l2ln
          (
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          , i_n
          );
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]] 
          := 
            sm_sc.fv_redistr_l1l2ln
            (
              i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            , i_n
            );
        end loop;
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]] 
            := 
              sm_sc.fv_redistr_l1l2ln
              (
                i_array
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              , i_n
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_redistr_l1l2ln
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9] :: float[]
--   , 3
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_l1l2ln
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--   , 3
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_l1l2ln
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , 3
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_l1l2ln
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , 3
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_redistr_log(float[]);
create or replace function sm_sc.fv_redistr_log
(
  i_array          float[]
)
returns float[]
as
$$
declare -- here
  v_ln_max     float   :=   ln(sm_sc.fv_aggr_slice_max(i_array));       
begin
  if array_ndims(i_array) is null
  then 
    return array[] :: float[];
  elsif array_ndims(i_array) <= 4
  then
    if abs(v_ln_max) < 1e-256
    then 
      return i_array;
    else
      return (^!` i_array) /` v_ln_max ;
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_log
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_redistr_log
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_redistr_log
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_redistr_log
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_redistr_log
--   (
--     array[]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_redistr_log(float[], int[]);
create or replace function sm_sc.fv_redistr_log
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      i_array
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
      := 
        sm_sc.fv_redistr_log
        (
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_redistr_log(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]] 
        := 
          sm_sc.fv_redistr_log
          (
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]] 
          := 
            sm_sc.fv_redistr_log
            (
              i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]] 
            := 
              sm_sc.fv_redistr_log
              (
                i_array
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_redistr_log
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, 1.9] :: float[]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_log
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, 1.9]
--          ,[3.25, 6.4, 6.6, 6.9, 2.65, 4.6]
--          ,[2.3, 5.1, 8.2, 2.56, 3.33, 1.9]
--          ,[3.25, 6.4, 6.6, 6.9, 2.65, 4.6]
--          ] :: float[]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_log
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_log
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_redistr_ne1_1(float[]);
create or replace function sm_sc.fv_redistr_ne1_1
(
  i_array          float[]
)
returns float[]
as
$$
declare -- here
  v_max     float   :=   sm_sc.fv_aggr_slice_max(i_array) ;
  v_min     float   :=   sm_sc.fv_aggr_slice_min(i_array) ;
  v_ptp     float   :=   v_max - v_min;      
  v_2_median  float   :=   v_max + v_min;
begin
  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) <= 4
  then
    if abs(v_ptp) < 1e-256
    then 
      return array_fill(0.0 :: float, (select array_agg(array_length(i_array, a_dim) order by a_dim) from generate_series(1, array_ndims(i_array)) tb_a_dim(a_dim)));
    else 
      return (i_array -` v_2_median) /` (v_ptp / 2.0);
    end if;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_ne1_1
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_redistr_ne1_1
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_redistr_ne1_1
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_redistr_ne1_1
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_redistr_ne1_1
--   (
--     array[]::float[]
--   );

-- select sm_sc.fv_redistr_ne1_1
--   (
--     array[1.0, 1.0, 1.0]::float[]
--   );

-- ------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_redistr_ne1_1(float[], int[]);
create or replace function sm_sc.fv_redistr_ne1_1
(
  i_array          float[],
  i_cnt_per_grp    int[]
)
returns float[]
as
$$
declare 
  v_cur_y  int               ;
  v_cur_x  int               ;
  v_cur_x3 int               ;
  v_cur_x4 int               ;
  
begin
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;
  
  if i_array is null
  then 
    return null;
    
  elsif array_length(i_cnt_per_grp, 1) = 1
  then 
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      i_array
        [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]] 
      := 
        sm_sc.fv_redistr_ne1_1
        (
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
        );
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 2
  then
    -- -- return 
    -- -- (
    -- --   with
    -- --   cte_slice_x as 
    -- --   (
    -- --     select 
    -- --       a_cur_y,
    -- --       array_agg
    -- --       (
    -- --         sm_sc.fv_redistr_ne1_1(i_array[a_cur_y : a_cur_y + i_cnt_per_grp[1] - 1][a_cur_x : a_cur_x + i_cnt_per_grp[2] - 1]) 
    -- --         order by a_cur_x
    -- --       ) as a_slice_x
    -- --     from generate_series(1, array_length(i_array, 1), i_cnt_per_grp[1]) tb_a_cur_y(a_cur_y)
    -- --       , generate_series(1, array_length(i_array, 2), i_cnt_per_grp[2]) tb_a_cur_x(a_cur_x)
    -- --     group by a_cur_y
    -- --   )
    -- --   select 
    -- --     array_agg(a_slice_x order by a_cur_y)
    -- --   from cte_slice_x
    -- -- )
    -- -- ;
    
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        i_array
          [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
          [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]] 
        := 
          sm_sc.fv_redistr_ne1_1
          (
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
          );
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 3
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          i_array
            [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
            [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
            [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]] 
          := 
            sm_sc.fv_redistr_ne1_1
            (
              i_array
                [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
            );
        end loop;
      end loop;
    end loop;
    return i_array;
    
  elsif array_length(i_cnt_per_grp, 1) = 4
  then
    for v_cur_y in 1 .. array_length(i_array, 1) / i_cnt_per_grp[1]
    loop 
      for v_cur_x in 1 .. array_length(i_array, 2) / i_cnt_per_grp[2]
      loop 
        for v_cur_x3 in 1 .. array_length(i_array, 3) / i_cnt_per_grp[3]
        loop 
          for v_cur_x4 in 1 .. array_length(i_array, 4) / i_cnt_per_grp[4]
          loop 
            i_array
              [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
              [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
              [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
              [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]] 
            := 
              sm_sc.fv_redistr_ne1_1
              (
                i_array
                  [(v_cur_y - 1) * i_cnt_per_grp[1] + 1 : v_cur_y * i_cnt_per_grp[1]]
                  [(v_cur_x - 1) * i_cnt_per_grp[2] + 1 : v_cur_x * i_cnt_per_grp[2]]
                  [(v_cur_x3 - 1) * i_cnt_per_grp[3] + 1 : v_cur_x3 * i_cnt_per_grp[3]]
                  [(v_cur_x4 - 1) * i_cnt_per_grp[4] + 1 : v_cur_x4 * i_cnt_per_grp[4]]
              );
          end loop;
        end loop;
      end loop;
    end loop;
    return i_array;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_redistr_ne1_1
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9] :: float[]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_ne1_1
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_ne1_1
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_ne1_1
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
drop function if exists sm_sc.fv_redistr_softmax(float[], int[]);
create or replace function sm_sc.fv_redistr_softmax
(
  i_array          float[] ,
  i_cnt_per_grp    int[]      default null
)
returns float[]
as
$$
-- declare
begin
  if i_cnt_per_grp is null
  then 
    i_cnt_per_grp := 
      (
        select 
          array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
        from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
      )
    ;
  elsif sm_sc.fv_aggr_slice_is_exists_null(i_cnt_per_grp)
  then 
    i_cnt_per_grp :=
      sm_sc.fv_coalesce
      (
        i_cnt_per_grp
      , (
          select 
            array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
          from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
        )
      )
    ;
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;

  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) <= 4
  then
    i_array :=    i_array -` sm_sc.fv_repeat_axis_py(i_array |@>| i_cnt_per_grp, (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no)), i_cnt_per_grp);  -- 避免上溢
    
-- -- debug 
-- raise notice 'i_array: %;', i_array;
    
    i_array :=         
      -- 幂运算下溢元素，用 -inf 替代，等价于 sum(0)。   
      i_array 
      +` 
      sm_sc.fv_ele_replace
      (
        (i_array <` -1.0e2 :: float) :: int[] :: float[]      -- 此处exp(-7.45e2)刚好不溢出，但进一步运算可能溢出
      , array[1.0 :: float]
      , '-inf' :: float
      )
    ;
    i_array := ^` i_array;
  
-- -- debug 
-- raise notice 'sm_sc.fv_aggr_slice_prod(i_cnt_per_grp) :: decimal: %;  cardinality(i_array) :: decimal: %;  i_cnt_per_grp: %;  i_array: %;  sm_sc.fv_aggr_slice_sum(i_array, i_cnt_per_grp): %'
-- , sm_sc.fv_aggr_slice_prod(i_cnt_per_grp) :: decimal
-- , cardinality(i_array) :: decimal
-- , i_cnt_per_grp
-- , i_array
-- , sm_sc.fv_aggr_slice_sum(i_array, i_cnt_per_grp)
-- ;
  
    return 
      sm_sc.fv_ele_replace
      (
        -- https://baijiahao.baidu.com/s?id=1772381174338322854&wfr=spider&for=pc
        -- 对 softmax_1 改进，避免小数量分类失真。
        i_array 
        /` 
        (
          sm_sc.fv_repeat_axis_py
          (
            sm_sc.fv_aggr_slice_sum(i_array, i_cnt_per_grp)
          , (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no))
          , i_cnt_per_grp
          )
        )
      , array['NaN' :: float]
      , 0.0 :: float
      );
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_softmax
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_redistr_softmax
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_redistr_softmax
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_redistr_softmax
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_redistr_softmax
--   (
--     array[]::float[]
--   );

-- select sm_sc.fv_redistr_softmax
--   (
--     array[0.9, -0.3, -0.6]::float[]
--   );

-- select 
--   sm_sc.fv_redistr_softmax
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9] :: float[]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_softmax
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_softmax
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_softmax
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
drop function if exists sm_sc.fv_redistr_softmax_ex(float[], int[]);
create or replace function sm_sc.fv_redistr_softmax_ex
(
  i_array          float[] ,
  i_cnt_per_grp    int[]      default null
)
returns float[]
as
$$
-- declare
begin
  if i_cnt_per_grp is null
  then 
    i_cnt_per_grp := 
      (
        select 
          array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
        from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
      )
    ;
  elsif sm_sc.fv_aggr_slice_is_exists_null(i_cnt_per_grp)
  then 
    i_cnt_per_grp :=
      sm_sc.fv_coalesce
      (
        i_cnt_per_grp
      , (
          select 
            array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
          from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
        )
      )
    ;
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;

  if array_ndims(i_array) is null
  then 
    return i_array;
  elsif array_ndims(i_array) <= 4
  then
    i_array :=    i_array -` sm_sc.fv_repeat_axis_py(i_array |@>| i_cnt_per_grp, (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no)), i_cnt_per_grp);  -- 避免上溢
    
    i_array :=         
      -- 幂运算下溢元素，用 -inf 替代，等价于 sum(0)。   
      i_array 
      +` 
      sm_sc.fv_ele_replace
      (
        (i_array <` -1.0e2 :: float) :: int[] :: float[]      -- 此处exp(-7.45e2)刚好不溢出，但进一步运算可能溢出
      , array[1.0 :: float]
      , '-inf' :: float
      )
    ;
    i_array := ^` i_array;
  
-- -- debug 
-- raise notice 'sm_sc.fv_aggr_slice_prod(i_cnt_per_grp) :: decimal: %;  cardinality(i_array) :: decimal: %;  i_cnt_per_grp: %;  i_array: %;  sm_sc.fv_aggr_slice_sum(i_array, i_cnt_per_grp): %'
-- , sm_sc.fv_aggr_slice_prod(i_cnt_per_grp) :: decimal
-- , cardinality(i_array) :: decimal
-- , i_cnt_per_grp
-- , i_array
-- , sm_sc.fv_aggr_slice_sum(i_array, i_cnt_per_grp)
-- ;
  
    return 
      sm_sc.fv_ele_replace
      (
        -- https://baijiahao.baidu.com/s?id=1772381174338322854&wfr=spider&for=pc
        -- 对 softmax_1 改进，避免小数量分类失真。
        i_array 
        /` 
        (
          sm_sc.fv_repeat_axis_py
          (
            (log(2.0, sm_sc.fv_aggr_slice_prod(i_cnt_per_grp) :: decimal) / 128.0) :: float 
            +` 
            sm_sc.fv_aggr_slice_sum(i_array, i_cnt_per_grp)
          , (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no))
          , i_cnt_per_grp
          )
        )
      , array['NaN' :: float]
      , 0.0 :: float
      );
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_redistr_zscore(float[], int[]);
create or replace function sm_sc.fv_redistr_zscore
(
  i_array          float[]
, i_cnt_per_grp    int[]  default null
)
returns float[]
as
$$
-- declare -- here
begin
  if i_cnt_per_grp is null
  then 
    i_cnt_per_grp := 
      (
        select 
          array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
        from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
      )
    ;
  elsif sm_sc.fv_aggr_slice_is_exists_null(i_cnt_per_grp)
  then 
    i_cnt_per_grp :=
      sm_sc.fv_coalesce
      (
        i_cnt_per_grp
      , (
          select 
            array_agg(array_length(i_array, a_dim_no) order by a_dim_no)
          from generate_series(1, array_ndims(i_array)) tb_a(a_dim_no)
        )
      )
    ;
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_cnt_per_grp) > 1 
    then 
      raise exception 'unsupport ndims of i_cnt_per_grp > 1.';
    elsif array_ndims(i_array) <> array_length(i_cnt_per_grp, 1)
    then 
      raise exception 'unmatch between ndims of i_array and length of i_cnt_per_grp.';
    elsif 
      0 <> any 
      (
        (
          select 
            array_agg(array_length(i_array, a_cur_dim) order by a_cur_dim) 
          from generate_series(1, array_ndims(i_array)) tb_a_cur_dim(a_cur_dim)
        )
        %` i_cnt_per_grp
      )
    then 
      raise exception 'unperfect i_array''s length for i_cnt_per_grp at some dims';
    end if;
  end if;

  if array_ndims(i_array) is null
  then 
    return array[] :: float[];
  elsif array_ndims(i_array) <= 4
  then
    return 
      (
        i_array 
        -` 
        sm_sc.fv_repeat_axis_py
        (
          sm_sc.fv_aggr_slice_avg(i_array, i_cnt_per_grp)
        , (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no))
        , i_cnt_per_grp
        )
      ) 
      /` 
      sm_sc.fv_repeat_axis_py
      (
        sm_sc.fv_aggr_slice_stddev_samp(i_array, i_cnt_per_grp)
      , (select array_agg(a_no) from generate_series(1, array_length(i_cnt_per_grp, 1)) tb_a(a_no))
      , i_cnt_per_grp
      )
    ;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_redistr_zscore
--   (
--     array[array[1,2,3,4,5,6]
--         , array[10,20,30,40,50,60]
--         , array[100,200,300,400,500,600]
--         , array[-1,-2,-3,-4,-5,-6]
--         , array[-10,-20,-30,-40,-50,-60]
--         , array[-100,-200,-300,-400,-500,-600]
--          ]::float[]
--   );

-- select sm_sc.fv_redistr_zscore
--   (
--     array[1,2,3,4,5,6]::float[]
--   );

-- select sm_sc.fv_redistr_zscore
--   (
--     array[[[1,2,3,4,25,6],[-1,-2,-3,4,5,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,5,6]]]::float[]
--   );

-- select sm_sc.fv_redistr_zscore
--   (
--     array[[[[1,2,3,4,25,6],[-1,-2,-3,4,35,6]],[[1,2,3,-4,-5,-36],[1,12,3,14,25,6]]],[[[1,12,3,4,25,6],[-1,-42,-3,4,5,6]],[[1,2,13,-4,-5,-36],[1,12,3,14,5,6]]]]::float[]
--   );

-- select sm_sc.fv_redistr_zscore
--   (
--     array[]::float[]
--   );

-- select 
--   sm_sc.fv_redistr_zscore
--   (
--     array[2.3, 5.1, 8.2, 2.56, 3.33, -1.9] :: float[]
--     , array[2]
--   )

-- select 
--   sm_sc.fv_redistr_zscore
--   (
--     array[[2.3, 5.1, 8.2, 2.56, 3.33, -1.9]
--          ,[3.25, 6.4, 6.6, 6.9, -2.65, -4.6]
--          ,[-2.3, 5.1, -8.2, 2.56, -3.33, -1.9]
--          ,[3.25, -6.4, -6.6, 6.9, -2.65, -4.6]
--          ] :: float[]
--     , array[2, 3]
--   )

-- select
--   sm_sc.fv_redistr_zscore
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15])
--   , array[2, 3, 3]
--   )

-- select
--   sm_sc.fv_redistr_zscore
--   (
--     sm_sc.fv_new_rand(array[6, 9, 15, 8])
--   , array[2, 3, 3, 4]
--   )
-- 计算图的设计
-- https://www.zhihu.com/question/66200879/answer/317086336
-- https://blog.csdn.net/hohaizx/article/details/82313143
-- -- 图中一个顶点对另一个顶点的导数等于该顶点到另一个顶点的路径上所有导数的乘积
-- -- 当存在从一个顶点到另一个顶点的多条路径时，那么这个顶点对另一个顶点的导数等于所有路径上导数的和
-- --------------------------------------------------------------------------------------------------------------------------------
-- 标量 scalar 运算，而非张量矩阵运算的求导
-- 返回微分数值

-- 张量运算的求导，需构建雅可比矩阵
-- https://blog.csdn.net/bitcarmanlee/article/details/78819025

-- drop function if exists sm_sc.ft_gradient(varchar(64)[], jsonb);
create or replace function sm_sc.ft_gradient
(
  i_indepdt_vars_names           varchar(64)[],
  -- i_indepdt_var_name           varchar(64),
  i_opr_chain                  jsonb   -- 假定入参值已全部写入 out_value
)                              
returns table                  
(                              
  o_indepdt_vars_name            varchar(64),
  o_grad                       float
)
as
$$
declare -- here
  v_sess_id               bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)       := replace(gen_random_uuid()::char(36), '-', '')::char(32);
  v_cur_out_params        varchar(64)[];
  v_cur_opr_var_params    varchar(64)[]  := i_indepdt_vars_names;
  v_cur_indepdt_var_name    varchar(64);
  v_o_grads               float[];
  v_cur_no                int  := 1;

-- debug
  v_debug_1   text;
  v_debug_2   text;


begin
  -- 计算图反序列化至缓存表
  insert into sm_sc._vt_fn_grad__chain
  (
    sess_id              ,
    out_param             ,
    param_loc             ,
    in_param              ,
    in_value              ,
    out_opr
  )
  select
    v_sess_id                     ,
    o_out_param    as out_param    ,
    o_param_loc    as param_loc    ,
    o_in_param     as in_param     ,
    o_in_value     as in_value     ,
    o_out_opr      as out_opr  
  from sm_sc.ft_computational_graph_deserialize(i_opr_chain)   -- (pgv_get('vars', 'a_opr_chain_03', NULL::jsonb))
  ;

  insert into sm_sc._vt_fn_grad__forward
  (
    sess_id              ,
    out_param             ,
    in_params             ,
    in_values             ,
    out_opr
  )
  -- 化作函数式形式，准备执行函数式计算
  select 
    v_sess_id,
    out_param,
    array_agg(in_param order by param_loc) as in_params,
    array_agg(in_value order by param_loc) as in_values,
    out_opr
  from sm_sc._vt_fn_grad__chain
  where sess_id = v_sess_id
  group by out_param, out_opr
  ;

  -- 前向运算结果
  while exists (select  from sm_sc._vt_fn_grad__forward where sess_id = v_sess_id and out_param = '' and calcu_val is null) and v_cur_no <= 100
  loop

    -- 找到本轮入参已经齐备、又尚未做前向计算的计算关系，本轮函数式数学计算
    with
    cte_forward as
    (
      update sm_sc._vt_fn_grad__forward
      set calcu_val = sm_sc.fv_algebra_execute(out_opr, in_values)
      where sess_id = v_sess_id
        and array_position(in_values, null) is null
        and calcu_val is null
      returning out_param
    )
    select array_agg(distinct out_param) :: varchar(64)[] into v_cur_out_params from cte_forward
    ;

    -- 本轮计算结果代入下一轮 in_param, 更新至计算链寄存表
    with 
    cte_upd_in_param as
    (
      update sm_sc._vt_fn_grad__chain tar
      set in_value = sour.calcu_val
      from sm_sc._vt_fn_grad__forward sour
      where sour.out_param = tar.in_param
        and tar.sess_id = v_sess_id
        and sour.sess_id = v_sess_id
        and tar.in_param = any(v_cur_out_params)
      returning tar.out_param
    )
    select array_agg(distinct out_param) into v_cur_out_params from cte_upd_in_param
    ;

    -- 本轮计算结果代入下一轮函数式
    with 
    cte_in_values as
    (
      select 
        out_param,
        array_agg(in_value order by param_loc) as in_values
      from sm_sc._vt_fn_grad__chain
      where sess_id = v_sess_id
        and out_param = any(v_cur_out_params)
      group by sess_id, out_param
    )
    update sm_sc._vt_fn_grad__forward tar
    set in_values = sour.in_values
    from cte_in_values sour
    where tar.sess_id = v_sess_id
      and sour.out_param = tar.out_param
    ;

    -- update sm_sc._vt_fn_grad__forward tar
    -- set in_values = sm_sc.fv_pos_replaces(tar.in_values, array_positions(tar.in_params, sour.out_param), sour.calcu_val)  -- -- 无法更新多行多个 sour.calcu_val 到 in_values，_vt_fn_grad__forward 单表更新不可行
    -- from sm_sc._vt_fn_grad__forward sour
    -- where sour.out_param = any(tar.in_params)
    --   and sour.out_param = any(v_cur_out_params)
    --   and tar.sess_id = v_sess_id
    --   and sour.sess_id = v_sess_id
    -- ;
    v_cur_no := v_cur_no + 1;
  end loop;

  -- 更新协参
  update sm_sc._vt_fn_grad__chain tar
  set co_vals = nullif(sour.in_values[ : tar.param_loc - 1] || sour.in_values[tar.param_loc + 1 : ], array[]::float[])
  from sm_sc._vt_fn_grad__forward sour
  where sour.out_param = tar.out_param
    and tar.sess_id = v_sess_id
    and sour.sess_id = v_sess_id
  ;

  v_cur_no := 1;

  -- 神经元内前向梯度
  -- -- 求每步(边)原子运算梯度
  while v_cur_opr_var_params is not null and v_cur_no <= 100
  loop
    with cte_front as
    (
      update sm_sc._vt_fn_grad__chain
      set grad_val = 
            sm_sc.fv_gradient_opr
            (
              in_value, 
              out_opr, 
              param_loc, 
              calcu_val, 
              co_vals
            )
      where sess_id = v_sess_id
        and in_param = any(v_cur_opr_var_params)
        and grad_val is null
      returning out_param
    )
    select array_agg(distinct out_param) :: varchar(64)[] into v_cur_opr_var_params from cte_front
    ;
    v_cur_no := v_cur_no + 1;
  end loop;


  foreach v_cur_indepdt_var_name in array i_indepdt_vars_names
  loop 
    with recursive
    cte_front as
    (
      select 
        out_param,
        in_param,
        param_loc,
        grad_val as chain_grad_val
      from sm_sc._vt_fn_grad__chain
      where sess_id = v_sess_id
        and in_param = v_cur_indepdt_var_name
      union all
      select 
        tb_a_incre.out_param,
        tb_a_incre.in_param,
        tb_a_incre.param_loc,
        (tb_a_main.chain_grad_val * tb_a_incre.grad_val)::float as chain_grad_val
      from cte_front tb_a_main
      inner join sm_sc._vt_fn_grad__chain tb_a_incre
      on tb_a_incre.in_param = tb_a_main.out_param
      where tb_a_incre.sess_id = v_sess_id
    )
    select 
      array_append(v_o_grads, sum(chain_grad_val)) into v_o_grads
    from cte_front
	where out_param = ''
    ;
  end loop;

  delete from sm_sc._vt_fn_grad__forward where sess_id = v_sess_id;
  delete from sm_sc._vt_fn_grad__chain where sess_id = v_sess_id;

  return query
    select 
      depdt_vars_name, grad
    from unnest(i_indepdt_vars_names, v_o_grads) tb_a_ret(depdt_vars_name, grad);

end
$$
language plpgsql volatile
cost 100;



-- -- select pgv_set('vars', 'a_opr_chain_03', 
-- -- '
-- -- {
-- --   "out_param": "v_z_in",
-- --   "opr": "sm_sc.fv_opr_add",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in",
-- --           "out_value": 1.5
-- --         },
-- --         {
-- --           "out_param": "v_y_in",
-- --           "out_value": 1.5
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_exp",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "opr": "sm_sc.fv_opr_mul",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x_in",
-- --               "out_value": 1.5
-- --             },
-- --             {
-- --               "out_param": "v_y_in",
-- --               "out_value": 1.5
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_pow",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in",
-- --           "out_value": 1.5
-- --         },
-- --         {
-- --           "out_value": 2
-- --         }
-- --       ]
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select * from sm_sc.ft_gradient(array['v_x_in'], pgv_get('vars', 'a_opr_chain_03', NULL::jsonb))
-- select * from sm_sc.ft_gradient(array['v_x_in', 'v_y_in'], pgv_get('vars', 'a_opr_chain_03', NULL::jsonb))

-- 标量 scalar 运算，而非张量矩阵运算的求导
-- 张量运算的求导，需构建雅可比矩阵
-- https://blog.csdn.net/bitcarmanlee/article/details/78819025

-- 计算图的设计
-- https://www.zhihu.com/question/66200879/answer/317086336
-- https://blog.csdn.net/hohaizx/article/details/82313143
--    1. 图中一个顶点对另一个顶点的导数等于该顶点到另一个顶点的路径上所有导数的乘积；
--    2. 当存在从一个顶点到另一个顶点的多条路径时，那么这个顶点对另一个顶点的导数等于所有路径上导数的和；
--    3. 计算图的三套结构
--      a. 原始运算结构: 
--           y = fn(x), 对应本程序参数引用名：i_depdt_var_name = fn(i_indepdt_var_name)
--      b. 原子运算求导结构
--           d_forword/d_back = d_fn(v_back)
--      c. 链式求导结构
--           d_forword/d_x = d_fn_ex(x)
-- drop function if exists sm_sc.ft_gradient_graph(varchar(64), anynonarray, varchar(64));
create or replace function sm_sc.ft_gradient_graph
(
  i_indepdt_var_name             varchar(64),
  -- i_depdt_var_name               varchar(64),
  i_opr_chart                    anynonarray,    -- 仅支持 text (代数式) 或 json (计算关系 chart ) 类型,
  i_grad_name                    varchar(64)      default null        -- 最终微分结果变量名
)
returns -- sm_sc.fv_computational_graph_serialize   return jsonb
  -- 约定：如果 i_grad_name is null, 那么返回结果有且只有唯一一条记录 where o_out_param is null，表示计算图的根结果
  table
  (
    o_out_param_x              varchar(64)      ,
    o_in_param_x               varchar(64)      ,
    o_in_value_x               float   ,
    o_param_loc_x              int              ,
    o_out_opr_x                varchar(64)
  )
as
$$
-- declare
begin
  return query
    with recursive
    -- 01. jsonb 配置反序列化为表记录
    --   原始运算计算图 关系
    cte_cfg_chain as
    (
      select
        o_out_param  ::varchar(64)     as out_param  ,
        o_in_param   ::varchar(64)     as in_param     ,
        o_in_value   ::float  as in_value     ,
        o_param_loc  ::int             as param_loc      ,
        o_out_opr    ::varchar(64)     as out_opr  
      from sm_sc.ft_computational_graph_deserialize(i_opr_chart)   -- (pgv_get('vars', 'a_opr_chain_03', NULL::jsonb))
    ),
    -- 02. 配置表去重为原始计算关系表
    --   原始运算计算图 关系
    cte_cfg_graph as
    (
      select
        out_param                        as    out_param                ,
        max(in_param) :: varchar(64)     as    in_param                 ,
        max(in_value) :: float  as    in_value                 ,
        param_loc                        as    param_loc                ,
        max(out_opr)  :: varchar(64)     as    out_opr
      from cte_cfg_chain
      group by out_param, param_loc
    ),
    -- 03. 前向传播，从有向无环图，过滤出目标自变量-因变量的路径
    --   原始运算计算图 关系 子集
    -- recursive
    cte_cfg_graph_forward as
    (
      select 
        tb_a_init.out_param             as out_param          ,
        tb_a_init.in_param              as in_param           ,
        tb_a_init.in_value              as in_value           ,
        tb_a_init.param_loc             as param_loc          ,
        tb_a_init.out_opr               as out_opr            ,
        -- 以下字段，控制递归
        1                                 as recur_dept         ,
        array[tb_a_init.out_param::varchar(64)]::varchar(64)[]
                                          as cycle_path_check   ,
        false                             as is_cycle  
      from cte_cfg_graph tb_a_init
      where in_param = i_indepdt_var_name    -- 'v_x_in'
      union    -- 前向传播路径去重
      select 
        tb_a_incre.out_param            as out_param          ,
        tb_a_incre.in_param             as in_param           ,
        tb_a_incre.in_value             as in_value           ,
        tb_a_incre.param_loc            as param_loc          ,
        tb_a_incre.out_opr              as out_opr            ,
        -- 以下字段，控制递归
        tb_a_cur.recur_dept + 1         as recur_dept         ,
        (tb_a_cur.cycle_path_check || (array[tb_a_incre.in_param::varchar(64)]::varchar(64)[]))::varchar(64)[]
                                          as   cycle_path_check,
        tb_a_incre.in_param = any(cycle_path_check)
                                          as is_cycle
      from cte_cfg_graph_forward tb_a_cur
      inner join cte_cfg_graph tb_a_incre
        on tb_a_incre.in_param = tb_a_cur.out_param
          and tb_a_incre.in_param <> ''   -- i_depdt_var_name        -- 'v_z_in'
      -- where tb_a_cur.recur_dept <= 100    -- for safe
      --  and not tb_a_cur.is_cycle
    ),
    -- 04. 协参准备, sm_sc.ft_gradient_opr_graph 要用到, group 等操作避免重复求导
    --   原始运算计算图 节点 子集
    cte_opr_gragh as
    (
      select 
        tb_a_depdt_var.out_param          as     out_param,
        max(tb_a_depdt_var.in_param)::varchar(64)      
                                            as     in_param,
        max(tb_a_depdt_var.out_opr)::varchar(64)
                                            as     out_opr,
        tb_a_depdt_var.param_loc          as     param_loc,
        array_agg(tb_a_co_var.in_param) filter(where tb_a_co_var.in_param is not null) :: varchar(64)[]  --  order by tb_a_co_var.param_loc)  -- 双目运算符的协参只有一个，多目运算符（加法、乘法）的协参可能有多个，对排序不敏感
                                            as     co_params,
        array_agg(tb_a_co_var.in_value) filter(where tb_a_co_var.in_value is not null) :: varchar(64)[]  --  order by tb_a_co_var.param_loc)  -- 双目运算符的协参只有一个，多目运算符（加法、乘法）的协参可能有多个，对排序不敏感
                                            as     co_values
      from cte_cfg_graph tb_a_depdt_var
      inner join cte_cfg_graph_forward tb_a_delt_depdt_var
        on tb_a_depdt_var.out_param = tb_a_delt_depdt_var.out_param
          and tb_a_depdt_var.param_loc = tb_a_delt_depdt_var.param_loc
          and tb_a_depdt_var.out_opr = tb_a_delt_depdt_var.out_opr
          and tb_a_depdt_var.in_param = tb_a_delt_depdt_var.in_param
      left join cte_cfg_graph tb_a_co_var
        on tb_a_co_var.out_param = tb_a_depdt_var.out_param
          and tb_a_co_var.param_loc <> tb_a_depdt_var.param_loc
      where tb_a_depdt_var.in_param is not null    -- 常量的微分无用，过滤掉
      group by 
        tb_a_depdt_var.out_param,
        tb_a_depdt_var.param_loc
    ),
    -- 05. 对原子运算求导
    --   原子运算求导计算图 关系
    cte_gradient as
    (
      select 
        tb_a_opr.out_param,
        tb_a_opr.in_param,
        tb_a_opr.out_opr,
        tb_a_opr.param_loc,
        tb_a_opr_grad.o_out_param  :: varchar(64)      as o_out_param,
        tb_a_opr_grad.o_in_param   :: varchar(64)      as o_in_param ,
        tb_a_opr_grad.o_in_value   :: float   as o_in_value ,
        tb_a_opr_grad.o_param_loc  :: int              as o_param_loc,
        tb_a_opr_grad.o_out_opr    :: varchar(64)      as o_out_opr  
      from cte_opr_gragh tb_a_opr
        ,sm_sc.ft_gradient_opr_graph
          (
            tb_a_opr.in_param::varchar(64), 
            tb_a_opr.out_opr::varchar(64), 
            tb_a_opr.param_loc::int, 
            tb_a_opr.co_params::varchar(64)[], 
            tb_a_opr.co_values::float[],
            md5('d_' || tb_a_opr.out_param || '_d_' || tb_a_opr.in_param || '_loc_' || (tb_a_opr.param_loc::int))
          ) tb_a_opr_grad
    ),
    -- 06. 计算关系查询总表，包含原始计算关系、导数计算关系的所有中间引用、自变量入参引用、因变量出参引用和目标导数引用
    --   原始运算计算图、原子运算求导计算图汇总 关系
    cte_opr_dic as
    (
      select 
        out_param     ,
        in_param      ,
        in_value      ,
        param_loc     ,
        out_opr
      from cte_cfg_graph
      union
      select 
        o_out_param   as   out_param     ,
        o_in_param    as   in_param      ,
        o_in_value    as   in_value      ,
        o_param_loc   as   param_loc     ,
        o_out_opr     as   out_opr
      from cte_gradient
    ),
    -- 07. 原子运算导数对象列表
    --   原子运算求导计算图汇总 节点
    cte_gradient_graph as
    (
      select 
        out_param    ,
        in_param     ,
        out_opr      ,
        param_loc    ,
        o_out_param  ,
        array_agg(o_in_param) filter(where o_in_param is not null)  --  order by o_param_loc)  -- 双目运算符的协参只有一个，多目运算符（加法、乘法）的协参可能有多个，对排序不敏感
          as     o_co_params,
        array_agg(o_in_value) filter(where o_in_value is not null)  --  order by o_param_loc)  -- 双目运算符的协参只有一个，多目运算符（加法、乘法）的协参可能有多个，对排序不敏感
          as     o_co_values
      from cte_gradient
      where o_out_param = md5('d_' || out_param || '_d_' || in_param || '_loc_' || (param_loc::int))
      group by 
        out_param    ,
        in_param     ,
        out_opr      ,
        param_loc    ,
        o_out_param
    ),
    
    -- 08. 反向传播，链式求导
    --   链式求导计算图 反向乘法链
    -- recursive
    cte_gradient_graph_back as
    (
      select 
        ''::varchar(64)                   as out_param          ,
        ''::varchar(64)                   as in_param           ,                  -- i_depdt_var_name  -- 'v_z_in'
        ''  ::varchar(64)                 as out_opr            ,
        1 :: int                          as param_loc          ,
        array[]::varchar(64)[]            as o_clain_mul        ,
        -- 以下字段，控制递归
        1                                 as recur_dept         ,
        array[]::varchar(64)[]            as cycle_path_check   ,
        false                             as is_cycle  
      -- -- -- where exists (select  from cte_cfg_graph_forward where in_param = i_indepdt_var_name)
      union all    -- 反向传播路径不去重
      select 
        tb_a_cur.in_param               as out_param          ,
        tb_a_incre.in_param             as in_param           ,
        tb_a_incre.out_opr              as out_opr            ,
        tb_a_incre.param_loc            as param_loc          ,
        (tb_a_cur.o_clain_mul || tb_a_incre.o_out_param)::varchar(64)[]
                                          as o_clain_mul        ,
        -- 以下字段，控制递归
        tb_a_cur.recur_dept + 1         as recur_dept         ,
        (tb_a_cur.cycle_path_check || (array[tb_a_incre.in_param::varchar(64)]::varchar(64)[]))::varchar(64)[]
                                          as   cycle_path_check,
        tb_a_incre.in_param = any(cycle_path_check)
                                          as is_cycle
      from cte_gradient_graph_back tb_a_cur
      inner join cte_gradient_graph tb_a_incre
        on tb_a_incre.out_param = tb_a_cur.in_param
      -- where tb_a_cur.recur_dept <= 100    -- for safe
      --  and not tb_a_cur.is_cycle
      --   and tb_a_cur.in_param <> i_indepdt_var_name             -- 'v_x_in'
    ),
    -- 09. 反向传播，分支合计操作
    --   链式求导计算图 关系 根节点：d_indepdt_var_d_depdt_var
    cte_gradient_graph_sum as
    (
      select 
        ''::varchar(64)                   as out_param          ,
        md5('d__d_' || i_indepdt_var_name || '_1_{}')::varchar(64)     -- 'v_z_in'     -- 'v_x_in'
                                          as in_param           ,
        null::float              as in_value           ,
        1   ::int                         as param_loc          ,
        ''  ::varchar(64)                 as out_opr
      where i_grad_name is null
        and exists (select  from cte_cfg_graph_forward where in_param = i_indepdt_var_name)   -- 'v_x_in'
      union all
      select 
        coalesce(i_grad_name, md5('d__d_' || i_indepdt_var_name || '_1_{}')::varchar(64))     -- 'v_z_in'     -- 'v_x_in'
                                          as out_param          ,
        md5(out_param || in_param || (param_loc::text) || (o_clain_mul::text))::varchar(64)
                                          as in_param           ,
        null::float              as in_value           ,
        row_number() over()::int          as param_loc          ,
        'sm_sc.fv_opr_add' as out_opr
      from cte_gradient_graph_back
      where in_param = i_indepdt_var_name                               -- 'v_x_in' 
      union all
      select 
        md5(out_param || in_param || (param_loc::text) || (o_clain_mul::text))::varchar(64)
                                          as out_param          ,
        a_clain_in_param::varchar(64)
                                          as in_param           ,
        null::float              as in_value           ,
        row_number() over(partition by o_clain_mul)::int
                                          as param_loc          ,
        'sm_sc.fv_opr_mul' as out_opr
      from cte_gradient_graph_back, unnest(o_clain_mul) tb_a_clain(a_clain_in_param)
      where in_param = i_indepdt_var_name                              -- 'v_x_in' 
    ),
    -- 10. 补齐求导计算图协变量，组合成求导计算全图、算数运算化简
    --   链式求导计算图 关系
    -- recursive
    cte_gradient_graph_ful as
    (
      select 
        out_param                                               ,
        in_param                                                ,
        in_value                                                ,
        param_loc                                               ,
        out_opr                                                 ,
        -- 以下字段，控制递归
        1                                 as recur_dept         ,
        array[in_param]::varchar(64)[]    as cycle_path_check   ,
        false                             as is_cycle  
      from cte_gradient_graph_sum
      union  -- 需要去重
      select 
        tb_a_incre.out_param                                  ,
        tb_a_incre.in_param                                   ,
        tb_a_incre.in_value                                   ,
        tb_a_incre.param_loc                                  ,
        tb_a_incre.out_opr                                    ,
        -- 以下字段，控制递归
        tb_a_cur.recur_dept + 1         as recur_dept         ,
        (tb_a_cur.cycle_path_check || (array[tb_a_incre.in_param::varchar(64)]))::varchar(64)[]
                                          as   cycle_path_check,
        tb_a_incre.in_param = any(cycle_path_check)
                                          as is_cycle
      from cte_gradient_graph_ful tb_a_cur
      inner join cte_opr_dic tb_a_incre
      on tb_a_incre.out_param = tb_a_cur.in_param
      where not tb_a_cur.is_cycle
      --   and tb_a_cur.recur_dept <= 100    -- for safe
      --  and not tb_a_cur.is_cycle
    )
    select
      out_param            as o_out_param  ,
      in_param             as o_in_param   ,
      in_value             as o_in_value   ,
      param_loc            as o_param_loc  ,
      out_opr              as o_out_opr
    from cte_gradient_graph_ful
    order by out_param, param_loc
  ;
end
$$
language plpgsql volatile
cost 100;


-- select * from sm_sc.ft_gradient_graph('v_x1_in', 'w1 * v_x1_in + w2 * v_x2 + w3 * v_x3'::text)
-- select * from sm_sc.ft_gradient_graph('v_x_in', '(v_x_in * v_y) + exp(v_x_in * v_y) + power(v_x_in, 2)'::text)


-- -- select pgv_set('vars', 'a_opr_chain', 
-- -- '
-- -- {
-- --   "out_param": "v_y_in",
-- --   "opr": "sm_sc.fv_opr_add",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "out_param": "w0_in"
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "w1_in"
-- --         },
-- --         {
-- --           "out_param": "v_x1_in"
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "w2_in"
-- --         },
-- --         {
-- --           "opr": "sm_sc.fv_opr_pow",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x1_in"
-- --             },
-- --             {
-- --               "out_value": 2
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "w3_in"
-- --         },
-- --         {
-- --           "opr": "sm_sc.fv_opr_pow",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x1_in"
-- --             },
-- --             {
-- --               "out_value": 3
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select * from sm_sc.ft_gradient_graph('v_x1_in', pgv_get('vars', 'a_opr_chain', NULL::jsonb))

-- -----------------------------------------------------------------------------------------------------

-- -- select pgv_set('vars', 'a_opr_chain', 
-- -- '
-- -- {
-- --   "out_param": "v_z_in",
-- --   "opr": "sm_sc.fv_opr_add",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in"
-- --         },
-- --         {
-- --           "out_param": "v_y_in"
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_exp",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "opr": "sm_sc.fv_opr_mul",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x_in"
-- --             },
-- --             {
-- --               "out_param": "v_y_in"
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_pow",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in"
-- --         },
-- --         {
-- --           "out_value": 2
-- --         }
-- --       ]
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select * from sm_sc.ft_gradient_graph('v_x_in', pgv_get('vars', 'a_opr_chain', NULL::jsonb))


-- 计算图原子运算求导
-- drop function if exists sm_sc.ft_gradient_opr_graph(varchar(64), varchar(64), int, varchar(64), float, varchar(64));
create or replace function sm_sc.ft_gradient_opr_graph
( 
  i_depdt_var_name               varchar(64)                     ,    -- 自变量名
  i_fn                           varchar(64)                     ,    -- i_fn = '' and i_co_value is not null, 则恒为常数；对于 y = x, 则 i_fn 设为 ''
  i_depdt_var_loc                int              default 1      ,    -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 sub, div, pow, log 四种运算操作敏感；其他运算操作用不到 i_depdt_var_loc
  i_co_name                      varchar(64)      default null   ,    -- fn 配套的另一个入参名，该配套入参位置与 i_depdt_var_loc 对立；如果该另一入参是常数，则 i_co_name 为空，i_co_value 填值
  i_co_value                     float   default null   ,    -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立；如果该另一入参是变量，则 i_co_value 为空，i_co_name 填值
    -- 对于常量引用，请装入 i_co_name, 数字协参可以装入 i_co_value, 也可以作为字符串装入 i_co_name
    -- 预留 i_co_value 的目的为了未来可以扩展后续步骤的数学运算
  i_grad_name                    varchar(64)      default null        -- 最终微分结果变量名
  -- 约定：当 i_fn = '' and i_co_value is null, 表示函数: y = x;  
  --       当 i_fn = '' and i_co_value is not null, 表示函数: y = C (C 为 常数);  
)
returns 
  -- 约定：如果 i_grad_name is null, 那么返回结果有且只有唯一一条记录 where o_out_param is null，表示计算图的根结果
  table
  (
    o_out_param                varchar(64)      ,
    o_in_param                 varchar(64)      ,
    o_in_value                 float   ,
    o_param_loc                int              ,
    o_out_opr                  varchar(64)        
  )
as
$$
declare -- here
  v_sql_code  text;
  v_sql_ret   float;
  v_tmp       float;
begin

  if i_fn = '' and (i_co_value is not null or i_co_name is not null)
  then 
    return query
      select 
        i_grad_name   
                             ::varchar(64)    as      o_out_param        ,
        null                 ::varchar(64)    as      o_in_param         ,
        0.0                  ::float as      o_in_value         ,
        1                    ::int            as      o_param_loc          ,
        ''                   ::varchar(64)    as      o_out_opr   
      ;
  elsif i_fn = 'sm_sc.fv_opr_add'
    or i_fn = '' and i_co_value is null and i_co_name is null
  then 
    return query
      select 
        i_grad_name   
                             ::varchar(64)    as      o_out_param        ,
        null                 ::varchar(64)    as      o_in_param         ,
        1.0 :: float                 ::float as      o_in_value         ,
        1                    ::int            as      o_param_loc          ,
        ''                   ::varchar(64)    as      o_out_opr   
      ;
  elsif i_fn = 'sm_sc.fv_opr_mul'
  then 
    return query
      select 
        i_grad_name  
                             ::varchar(64)    as      o_out_param        ,
        i_co_name            ::varchar(64)    as      o_in_param         ,
        i_co_value           ::float as      o_in_value         ,
        1                    ::int            as      o_param_loc          ,
        ''                   ::varchar(64)    as      o_out_opr   
      ;
  elsif i_fn = 'sm_sc.fv_opr_sub'
  then 
    return query
      select 
        i_grad_name   
                             ::varchar(64)    as      o_out_param        ,
        null                 ::varchar(64)    as      o_in_param         ,
        case i_depdt_var_loc 
          when 1 
            then 1.0
          when 2 
            then -1.0
        end                  ::float as      o_in_value         ,
        1                    ::int            as      o_param_loc          ,
        ''                   ::varchar(64)    as      o_out_opr   
      ;
  elsif i_fn = 'sm_sc.fv_opr_div'
  then 
    if i_depdt_var_loc = 1
    then 
      return query
        select 
          coalesce(i_grad_name, md5('1.0 :: float/ ' || coalesce(i_co_name, i_co_value::text)::text))   
                               ::varchar(64)    as      o_out_param        ,
          null                 ::varchar(64)    as      o_in_param         ,
          1.0 :: float                 ::float as      o_in_value         ,
          1                    ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_div' ::varchar(64)    as      o_out_opr   
        union all
        select 
          coalesce(i_grad_name, md5('1.0 :: float/ ' || coalesce(i_co_name, i_co_value::text)::text))     
                                    ::varchar(64)    as      o_out_param        ,
          i_co_name                 ::varchar(64)    as      o_in_param         ,
          nullif(i_co_value, 0.0 :: float)   ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_div'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          null                                      as      o_out_param        ,
          md5('1.0 :: float/ ' || coalesce(i_co_name, i_co_value::text)::text)   
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
      ;
    elsif i_depdt_var_loc = 2
    then 
      return query
        select 
          md5('power(' || i_depdt_var_name || ', -2)')
                                    ::varchar(64)    as      o_out_param      ,
          i_depdt_var_name          ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('power(' || i_depdt_var_name || ', -2)')
                                    ::varchar(64)    as      o_out_param      ,
          null                      ::varchar(64)    as      o_in_param         ,
          -2.0                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'      ::varchar(64)    as      o_out_opr    
        union all
        select  
          md5(coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', -2)')
                                    ::varchar(64)    as      o_out_param      ,
          i_co_name                 ::varchar(64)    as      o_in_param         ,
          i_co_value                ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'      ::varchar(64)    as      o_out_opr   
        union all
        select  
          md5(coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', -2)')
                                    ::varchar(64)    as      o_out_param      ,
          md5('power(' || i_depdt_var_name || ', -2)')
                                    ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'      ::varchar(64)    as      o_out_opr   
        union all
        select  
          coalesce(i_grad_name, md5('-' || coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', -2)'))   
                               ::varchar(64)    as      o_out_param        ,
          null                      ::varchar(64)    as      o_in_param         ,
          0.0                       ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_sub'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          coalesce(i_grad_name, md5('-' || coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', -2)'))   
                               ::varchar(64)    as      o_out_param        ,
          md5(coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', -2)')
                                    ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_sub'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          null                                      as      o_out_param        ,
          md5('-' || coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', -2)') 
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
      ;
    end if;
  elsif i_fn = 'sm_sc.fv_opr_pow'
  then 
    if i_depdt_var_loc = 1
    then 
      return query
        select 
          md5(coalesce(i_co_name, i_co_value::text) || ' - 1')
                                    ::varchar(64)    as      o_out_param      ,
          i_co_name                 ::varchar(64)    as      o_in_param         ,
          i_co_value                ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_sub'      ::varchar(64)    as      o_out_opr   
        union all  
        select 
          md5(coalesce(i_co_name, i_co_value::text) || ' - 1')
                                    ::varchar(64)    as      o_out_param      ,
          null                      ::varchar(64)    as      o_in_param         ,
          1.0 :: float                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_sub'      ::varchar(64)    as      o_out_opr 
        union all
        select 
          md5('power(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ' - 1)')
                                    ::varchar(64)    as      o_out_param      ,
          i_depdt_var_name          ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('power(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ' - 1)')
                                    ::varchar(64)    as      o_out_param      ,
          md5(coalesce(i_co_name, i_co_value::text) || ' - 1')
                                    ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          coalesce(i_grad_name, md5(coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ' - 1)'))   
                                    ::varchar(64)    as      o_out_param        ,
          i_co_name                 ::varchar(64)    as      o_in_param         ,
          i_co_value                ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          coalesce(i_grad_name, md5(coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ' - 1)'))   
                                    ::varchar(64)    as      o_out_param        ,
          md5('power(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ' - 1)')
                                    ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          null                                      as      o_out_param        ,
          md5(coalesce(i_co_name, i_co_value::text) || ' * power(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ' - 1)') 
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
      ;
    elsif i_depdt_var_loc = 2
    then
      return query
        select 
          md5('power(' || coalesce(i_co_name, i_co_value::text) || ', ' || i_depdt_var_name || ')')
                                  ::varchar(64)    as      o_out_param      ,
          i_co_name               ::varchar(64)    as      o_in_param         ,
          i_co_value              ::float as      o_in_value         ,
          1                       ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'    ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('power(' || coalesce(i_co_name, i_co_value::text) || ', ' || i_depdt_var_name || ')')
                                  ::varchar(64)    as      o_out_param      ,
          i_depdt_var_name        ::varchar(64)    as      o_in_param         ,
          null                    ::float as      o_in_value         ,
          2                       ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'    ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                  ::varchar(64)    as      o_out_param      ,
          i_co_name               ::varchar(64)    as      o_in_param         ,
          i_co_value              ::float as      o_in_value         ,
          1                       ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_ln'     ::varchar(64)    as      o_out_opr   
        union all
        select 
          coalesce(i_grad_name, md5('power(' || coalesce(i_co_name, i_co_value::text) || ', ' || i_depdt_var_name || ') * ln(' || coalesce(i_co_name, i_co_value::text) || ')'))   
                                    ::varchar(64)    as      o_out_param        ,
          md5('power(' || coalesce(i_co_name, i_co_value::text) || ', ' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          1                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'      ::varchar(64)    as      o_out_opr      
        union all
        select 
          coalesce(i_grad_name, md5('power(' || coalesce(i_co_name, i_co_value::text) || ', ' || i_depdt_var_name || ') * ln(' || coalesce(i_co_name, i_co_value::text) || ')'))     
                                    ::varchar(64)    as      o_out_param        ,
          md5('ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                    ::varchar(64)    as      o_in_param         ,
          null                      ::float as      o_in_value         ,
          2                         ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'      ::varchar(64)    as      o_out_opr   
        union all
        select 
          null                                      as      o_out_param        ,
          md5('power(' || coalesce(i_co_name, i_co_value::text) || ', ' || i_depdt_var_name || ') * ln(' || coalesce(i_co_name, i_co_value::text) || ')')   
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
      ;
    end if;
  elsif i_fn = 'sm_sc.fv_opr_exp'
  then 
    return query
      select 
        i_grad_name   
                                    ::varchar(64)    as      o_out_param        ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_exp'        ::varchar(64)    as      o_out_opr   
    ;
  elsif i_fn = 'sm_sc.fv_opr_log'
  then 
    if i_depdt_var_loc = 1
    then
      return query
        select 
          md5('log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_log'        ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          i_co_name                   ::varchar(64)    as      o_in_param         ,
          i_co_value                  ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_log'        ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2)')
                                      ::varchar(64)    as      o_out_param      ,
          md5('log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          md5('power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2)')
                                      ::varchar(64)    as      o_out_param      ,
          null                        ::varchar(64)    as      o_in_param         ,
          2.0                         ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('-power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2)')
                                      ::varchar(64)    as      o_out_param      ,
          null                        ::varchar(64)    as      o_in_param         ,
          0.0                         ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
        union all
        select 
          md5('-power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2)')
                                      ::varchar(64)    as      o_out_param      ,
          md5('power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2)')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5('ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          i_co_name                   ::varchar(64)    as      o_in_param         ,
          i_co_value                  ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_ln'         ::varchar(64)    as      o_out_opr      
        union all
        select 
          md5(i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'        ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5(i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          md5('ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          coalesce(i_grad_name, md5('-power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2) / (' || i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || '))'))   
                                      ::varchar(64)    as      o_out_param        ,
          md5('-power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2)')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          coalesce(i_grad_name, md5('-power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2) / (' || i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || '))'))   
                                      ::varchar(64)    as      o_out_param        ,
          md5(i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          null                                      as      o_out_param        ,
          md5('-power(log(' || i_depdt_var_name || ', ' || coalesce(i_co_name, i_co_value::text) || '), 2) / (' || i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || '))')  
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
      ;
    elsif i_depdt_var_loc = 2
    then 
      return query
        select 
          md5('ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          i_co_name                   ::varchar(64)    as      o_in_param         ,
          i_co_value                  ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_ln'         ::varchar(64)    as      o_out_opr      
        union all
        select 
          md5(i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'        ::varchar(64)    as      o_out_opr   
        union all
        select 
          md5(i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_out_param      ,
          md5('ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_mul'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          coalesce(i_grad_name, md5('1.0 :: float/ (' || i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || '))'))   
                                      ::varchar(64)    as      o_out_param        ,
          null                        ::varchar(64)    as      o_in_param         ,
          1.0 :: float                        ::float as      o_in_value         ,
          1                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          coalesce(i_grad_name, md5('1.0 :: float/ (' || i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || '))'))   
                                      ::varchar(64)    as      o_out_param        ,
          md5(i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || ')')
                                      ::varchar(64)    as      o_in_param         ,
          null                        ::float as      o_in_value         ,
          2                           ::int            as      o_param_loc          ,
          'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
        union all
        select 
          null                                      as      o_out_param        ,
          md5('1.0 :: float/ (' || i_depdt_var_name || ' * ln(' || coalesce(i_co_name, i_co_value::text) || '))')
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
      ;
    end if;
  elsif i_fn = 'sm_sc.fv_opr_ln'
  then 
    return query
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ ' || i_depdt_var_name ::text))   
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ ' || i_depdt_var_name ::text)) 
                                    ::varchar(64)    as      o_out_param        ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                      as      o_out_param        ,
        md5('1.0 :: float/ ' || i_depdt_var_name ::text) 
                                                  as      o_in_param         ,
        null                                      as      o_in_value         ,
        1                                         as      o_param_loc          ,
        ''                                        as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_sin'
  then 
    return query
      select 
        i_grad_name   
                                    ::varchar(64)    as      o_out_param        ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_cos'            ::varchar(64)    as      o_out_opr    
    ;
  elsif i_fn = 'sm_sc.fv_cos'
  then 
    return query
      select 
        md5('sin(' || i_depdt_var_name || ')')  
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_sin'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-sin(' || i_depdt_var_name || ')'))   
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.0                         ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-sin(' || i_depdt_var_name || ')'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('sin(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        null                                      as      o_out_param        ,
        md5('-sin(' || i_depdt_var_name || ')')
                                                  as      o_in_param         ,
        null                                      as      o_in_value         ,
        1                                         as      o_param_loc          ,
        ''                                        as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_tan'
  then 
    return query
      select 
        md5('cos(' || i_depdt_var_name || ')')  
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_cos'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(cos(' || i_depdt_var_name || '), 2)')  
                                    ::varchar(64)    as      o_out_param      ,
        md5('cos(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr       
      union all
      select 
        md5('power(cos(' || i_depdt_var_name || '), 2)')  
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(cos(' || i_depdt_var_name || '), 2)'))   
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(cos(' || i_depdt_var_name || '), 2)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('power(cos(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        null                                      as      o_out_param        ,
        md5('1.0 :: float/ power(cos(' || i_depdt_var_name || '), 2)')
                                                  as      o_in_param         ,
        null                                      as      o_in_value         ,
        1                                         as      o_param_loc          ,
        ''                                        as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_cot'
  then 
    return query
      select 
        md5('sin(' || i_depdt_var_name || ')')  
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_sin'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(sin(' || i_depdt_var_name || '), 2)')  
                                    ::varchar(64)    as      o_out_param      ,
        md5('sin(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr       
      union all
      select 
        md5('power(sin(' || i_depdt_var_name || '), 2)')  
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float/ power(sin(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float/ power(sin(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(sin(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ power(sin(' || i_depdt_var_name || '), 2)'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.0                         ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ power(sin(' || i_depdt_var_name || '), 2)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('1.0 :: float/ power(sin(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        null                                      as      o_out_param        ,
        md5('-1.0 :: float/ power(sin(' || i_depdt_var_name || '), 2)')
                                                  as      o_in_param         ,
        null                                      as      o_in_value         ,
        1                                         as      o_param_loc          ,
        ''                                        as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_sec'
  then 
    return query
      select 
        md5('tan(' || i_depdt_var_name || ')') 
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_tan'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('cos(' || i_depdt_var_name || ')') 
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_cos'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('tan(' || i_depdt_var_name || ') / cos(' || i_depdt_var_name || ')'))  
                                    ::varchar(64)    as      o_out_param        ,
        md5('tan(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('tan(' || i_depdt_var_name || ') / cos(' || i_depdt_var_name || ')')) 
                                    ::varchar(64)    as      o_out_param        ,
        md5('cos(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        null                                         as      o_out_param        ,
        md5('tan(' || i_depdt_var_name || ') / cos(' || i_depdt_var_name || ')') 
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_csc'
  then 
    return query
      select 
        md5('cot(' || i_depdt_var_name || ')') 
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_cot'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('sin(' || i_depdt_var_name || ')') 
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_sin'            ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('cot(' || i_depdt_var_name || ') / sin(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_out_param      ,
        md5('cot(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('cot(' || i_depdt_var_name || ') / sin(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_out_param      ,
        md5('sin(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        coalesce(i_grad_name, md5('-cot(' || i_depdt_var_name || ') / sin(' || i_depdt_var_name || ')'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.0                         ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-cot(' || i_depdt_var_name || ') / sin(' || i_depdt_var_name || ')'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('cot(' || i_depdt_var_name || ') / sin(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        null                                         as      o_out_param        ,
        md5('-cot(' || i_depdt_var_name || ') / sin(' || i_depdt_var_name || ')')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_asin'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select
        md5('power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.5                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_acos'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select
        md5('power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.5                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select 
        md5('1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.0                         ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('-1.0 :: float/ power(1.0 :: float- power(' || i_depdt_var_name || ', 2), 0.5 :: float)')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_atan'
  then  
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float+ power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_add'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        md5('1.0 :: float+ power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_add'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('1.0 :: float+ power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_acot'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float+ power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_add'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        md5('1.0 :: float+ power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_add'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))')
                                    ::varchar(64)    as      o_out_param      ,
        md5('1.0 :: float+ power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))'))  
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.0                         ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))')) 
                                    ::varchar(64)    as      o_out_param        ,
        md5('1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('-1.0 :: float/ (1.0 :: float+ power(' || i_depdt_var_name || ', 2))') 
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_asec'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr      
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr       
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr
      union all
      select
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.5                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select
        md5(i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5(i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))'))   
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))'))    
                                    ::varchar(64)    as      o_out_param        ,
        md5(i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))')  
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_acsc'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr      
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr       
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr
      union all
      select
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.5                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select
        md5(i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5(i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select 
        md5('1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))')
                                    ::varchar(64)    as      o_out_param      ,
        md5(i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.0                         ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('-1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('-1.0 :: float/ (' || i_depdt_var_name || ' * power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float))')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_sinh'
  then 
    return query
      select 
        i_grad_name   
                                    ::varchar(64)    as      o_out_param        ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_cosh'           ::varchar(64)    as      o_out_opr    
    ;
  elsif i_fn = 'sm_sc.fv_cosh'
  then 
    return query
      select 
        i_grad_name   
                                    ::varchar(64)    as      o_out_param        ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_sinh'           ::varchar(64)    as      o_out_opr    
    ;
  elsif i_fn = 'sm_sc.fv_tanh'
  then 
    return query
      select 
        md5('cosh(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_cosh'           ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(cosh(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('cosh(' || i_depdt_var_name || ')')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(cosh(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(cosh(' || i_depdt_var_name || '), 2)'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(cosh(' || i_depdt_var_name || '), 2)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('power(cosh(' || i_depdt_var_name || '), 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ power(cosh(' || i_depdt_var_name || '), 2)')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_asinh'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) + 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_add'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) + 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_add'        ::varchar(64)    as      o_out_opr     
      union all
      select
        md5('power(power(' || i_depdt_var_name || ', 2) + 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2) + 1.0')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(power(' || i_depdt_var_name || ', 2) + 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.5                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(power(' || i_depdt_var_name || ', 2) + 1.0 :: float, 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(power(' || i_depdt_var_name || ', 2) + 1.0 :: float, 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('power(power(' || i_depdt_var_name || ', 2) + 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ power(power(' || i_depdt_var_name || ', 2) + 1.0 :: float, 0.5 :: float)')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_acosh'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2) - 1.0')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        0.5                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr   
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)'))
                                    ::varchar(64)    as      o_out_param        ,
        md5('power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ power(power(' || i_depdt_var_name || ', 2) - 1.0 :: float, 0.5 :: float)')
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  elsif i_fn = 'sm_sc.fv_atanh'
  then 
    return query
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        i_depdt_var_name            ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        2.0                         ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_pow'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_out_param      ,
        md5('power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_sub'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ (1.0 :: float- power(' || i_depdt_var_name || ', 2))'))   
                                    ::varchar(64)    as      o_out_param        ,
        null                        ::varchar(64)    as      o_in_param         ,
        1.0 :: float                        ::float as      o_in_value         ,
        1                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr    
      union all
      select 
        coalesce(i_grad_name, md5('1.0 :: float/ (1.0 :: float- power(' || i_depdt_var_name || ', 2))'))  
                                    ::varchar(64)    as      o_out_param        ,
        md5('1.0 :: float- power(' || i_depdt_var_name || ', 2)')
                                    ::varchar(64)    as      o_in_param         ,
        null                        ::float as      o_in_value         ,
        2                           ::int            as      o_param_loc          ,
        'sm_sc.fv_opr_div'        ::varchar(64)    as      o_out_opr     
      union all
      select 
        null                                         as      o_out_param        ,
        md5('1.0 :: float/ (1.0 :: float- power(' || i_depdt_var_name || ', 2))') 
                                                     as      o_in_param         ,
        null                                         as      o_in_value         ,
        1                                            as      o_param_loc          ,
        ''                                           as      o_out_opr   
      where i_grad_name is null
    ;
  else 
    raise exception 'unknown function: % !', i_fn;
  end if;

end
$$
language plpgsql stable
cost 100;


-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   ''
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   ''                      ,
-- --   1                       ,
-- --   'C'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   ''                      ,
-- --   1                       ,
-- --   null                    ,
-- --   12
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'    ,
-- --   1
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'    ,
-- --   2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   2                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   'a'                     ,
-- --   null::float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_sub'    ,
-- --   1
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_sub'    ,
-- --   2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_div'    ,
-- --   1                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_div'    ,
-- --   1                       ,
-- --   'a'                     ,
-- --   null :: float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_div'    ,
-- --   2                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_div'    ,
-- --   2                       ,
-- --   'a'                     ,
-- --   null :: float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_pow'    ,
-- --   1                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_pow'    ,
-- --   1                       ,
-- --   'a'                     ,
-- --   null :: float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_pow'    ,
-- --   2                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_pow'    ,
-- --   2                       ,
-- --   'a'                     ,
-- --   null :: float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_exp' 
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_log'    ,
-- --   1                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_log'    ,
-- --   1                       ,
-- --   'a'                     ,
-- --   null :: float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_log'    ,
-- --   2                       ,
-- --   null                    ,
-- --   1.2
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_log'    ,
-- --   2                       ,
-- --   'a'                     ,
-- --   null :: float
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_ln'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_sin'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_cos'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_tan'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_cot'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_sec'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_csc'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_asin'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_acos'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_atan'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_acot'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_asec'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_acsc'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_sinh'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_cosh'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_tanh'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_asinh'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_acosh'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_atanh'
-- -- );

-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   ''      ,
-- --   null      ,
-- --   null      ,
-- --   null ::float     ,
-- --   'dydx_xxx'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_atanh'      ,
-- --   null                    ,
-- --   null                    ,
-- --   null     :: float       ,
-- --   'dydx_xxx'
-- -- );


-- ---------------------------------------------------------------------------------------
-- 计算图原子运算求导
-- 为了支持多目运算符
-- drop function if exists sm_sc.ft_gradient_opr_graph(varchar(64), varchar(64), int, varchar(64)[], float[], varchar(64));
create or replace function sm_sc.ft_gradient_opr_graph
( 
  i_depdt_var_name               varchar(64)                     ,    -- 自变量名
  i_fn                           varchar(64)                     ,    -- i_fn = '' and i_co_value is not null, 则恒为常数；对于 y = x, 则 i_fn 设为 ''
  i_depdt_var_loc                int                             ,    -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 sub, div, pow, log 四种运算操作敏感；其他运算操作用不到 i_depdt_var_loc
  i_co_names                     varchar(64)[]                   ,    -- fn 配套的另一个入参名，该配套入参位置与 i_depdt_var_loc 对立；如果该另一入参是常数，则 i_co_name 为空，i_co_value 填值
  i_co_values                    float[] default null   ,    -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立；如果该另一入参是变量，则 i_co_value 为空，i_co_name 填值
  i_grad_name                    varchar(64)      default null        -- 最终微分结果变量名
  -- 约定：当 i_fn = '' and i_co_value is null, 表示函数: y = x;  
  --       当 i_fn = '' and i_co_value is not null, 表示函数: y = C (C 为 常数);  
)
returns 
  table
  (
    o_out_param                varchar(64)      ,
    o_in_param                 varchar(64)      ,
    o_in_value                 float   ,
    o_param_loc                int              ,
    o_out_opr                  varchar(64)        
  )
as
$$
-- declare
begin
  if coalesce(array_length(i_co_names, 1), 0) + coalesce(array_length(i_co_values, 1), 0) < 2
  then 
    return query
      select 
        tb_a.o_out_param    ,
        tb_a.o_in_param     ,
        tb_a.o_in_value     ,
        tb_a.o_param_loc    ,
        tb_a.o_out_opr  
      from sm_sc.ft_gradient_opr_graph(i_depdt_var_name, i_fn, i_depdt_var_loc, i_co_names[1], i_co_values[1], i_grad_name) tb_a
    ;
  else
    if i_fn = 'sm_sc.fv_opr_add'
    then
      return query
        select 
          i_grad_name          ::varchar(64)        as      o_out_param      ,
          null                 ::varchar(64)        as      o_in_param         ,
          1.0 :: float                 ::float     as      o_in_value         ,
          1                    ::int                as      o_param_loc          ,
          ''                   ::varchar(64)        as      o_out_opr   
      ;                                           
    elsif i_fn = 'sm_sc.fv_opr_mul'             
    then     
      return query                                     
        select
          coalesce(i_grad_name, md5('prod(' || array_to_string(i_co_names::text[] || i_co_values::text[], ', ')::text || ')'))
                                  ::varchar(64)        as      o_out_param      ,
          i_co_names[a_name_no]   ::varchar(64)        as      o_in_param       ,
          null                    ::float     as      o_in_value       ,
          row_number() over()     ::int                as      o_param_loc      ,
          case 
            when coalesce(array_length(i_co_names, 1), 0) > 1 
                or i_co_values is not null 
              then i_fn 
            else '' 
          end                     ::varchar(64)        as      o_out_opr   
        from generate_series(1, array_length(i_co_names, 1))  tb_a_name(a_name_no) 
        -- where coalesce(array_length(i_co_names, 1), 0) > 0     
        union all                                   
        -- -- select                                      
        -- --   md5('prod(' || array_to_string(i_co_names::text[] || i_co_values::text[], ', ')::text || ')')
        -- --                        ::varchar(64)        as      o_out_param      ,
        -- --   null                 ::varchar(64)        as      o_in_param         ,
        -- --   sm_sc.fv_aggr_slice_prod(i_co_values)   as      o_in_value         ,
        -- --   1                    ::int                as      o_param_loc          ,
        -- --   case 
        -- --     when coalesce(array_length(i_co_names, 1), 0) > 0 
        -- --       then i_fn 
        -- --     else '' 
        -- --   end                 ::varchar(64)        as      o_out_opr   
        -- -- where coalesce(array_length(i_co_values, 1), 0) > 0
        select                                      
          coalesce(i_grad_name, md5('prod(' || array_to_string(i_co_names::text[] || i_co_values::text[], ', ')::text || ')'))
                                    ::varchar(64)         as      o_out_param      ,
          null                      ::varchar(64)         as      o_in_param         ,
          i_co_values[a_value_no]   ::float      as      o_in_value         ,
          array_length(i_co_names, 1) + row_number() over()   ::int                 as      o_param_loc          ,
          case 
            when coalesce(array_length(i_co_values, 1), 0) > 0 
                or i_co_names is not null
              then i_fn 
            else '' 
          end                 ::varchar(64)        as      o_out_opr   
        from generate_series(1, array_length(i_co_values, 1))  tb_a_value(a_value_no) 
        -- where coalesce(array_length(i_co_values, 1), 0) > 0
        union all
        select 
          null                                      as      o_out_param        ,
          md5('prod(' || array_to_string(i_co_names::text[] || i_co_values::text[], ', ')::text || ')')
                                                    as      o_in_param         ,
          null                                      as      o_in_value         ,
          1                                         as      o_param_loc          ,
          ''                                        as      o_out_opr   
        where i_grad_name is null
        ;
    else
      raise exception 'unknown function: % for such co_params!', i_fn;
    end if;
  end if;
end
$$
language plpgsql volatile
cost 100;


-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'    ,
-- --   2                       ,
-- --   null                    ,
-- --   array[1.2]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'    ,
-- --   1                       ,
-- --   array['a']                     ,
-- --   null
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'    ,
-- --   1                       ,
-- --   array['a', 'b']                     ,
-- --   array[1.2, 1.1]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   2                       ,
-- --   null                    ,
-- --   array[1.2]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   2                       ,
-- --   null                    ,
-- --   array[1.2, 1.1]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a']                     ,
-- --   null
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a', 'b']                     ,
-- --   null
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a']                     ,
-- --   array[1.2]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a']                     ,
-- --   array[1.2, 1.1]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a', 'b']                     ,
-- --   array[1.2]
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a', 'b']                     ,
-- --   array[1.2, 1.1]
-- -- );


-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_mul'    ,
-- --   1                       ,
-- --   array['a', 'b']         ,
-- --   array[1.2, 1.1]         ,
-- --   'dydx_balabala'
-- -- );
-- -- select * from sm_sc.ft_gradient_opr_graph
-- -- (
-- --   'x'                     ,
-- --   'sm_sc.fv_opr_add'    ,
-- --   1                       ,
-- --   array['a', 'b']         ,
-- --   array[1.2, 1.1]         ,
-- --   'dydx_balabala'
-- -- );
-- 计算图的设计
-- https://www.zhihu.com/question/66200879/answer/317086336
-- https://blog.csdn.net/hohaizx/article/details/82313143
-- -- 图中一个顶点对另一个顶点的导数等于该顶点到另一个顶点的路径上所有导数的乘积
-- -- 当存在从一个顶点到另一个顶点的多条路径时，那么这个顶点对另一个顶点的导数等于所有路径上导数的和
-- --------------------------------------------------------------------------------------------------------------------------------
-- 代数式的求导：
-- 返回推导出来的导数代数式
-- 计算图入参为 text (代数式) 或 json (计算关系 chart )
-- drop function if exists sm_sc.fv_gradient(varchar(64), anynonarray);
create or replace function sm_sc.fv_gradient
(
  i_indepdt_var_name               varchar(64),
  -- i_indepdt_var_name             varchar(64),
  i_opr_chart                    anynonarray     -- 仅支持 text (代数式) 或 json (计算关系 chart ) 类型,
)
returns text
as
$$
declare -- here
  v_root_out_param  varchar(64);
  v_ret             text;
  v_sess_id         bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)   := replace(gen_random_uuid()::char(36), '-', '')::char(32);

begin

  -- 用临时表并不是优雅的实现方式，期待未来版本 cte 的迭代部分支持分组聚合
  -- 01. 计算图关系列表
  insert into sm_sc._vt_fn_grad__graph
  (
    sess_id      ,
    o_out_param   ,
    o_in_param    ,
    o_in_value    ,
    o_param_loc   ,
    o_out_opr     ,
    is_decimal     
  )
  select
    v_sess_id,
    o_out_param_x        ::    varchar(64)       as o_out_param   ,
    o_in_param_x         ::    varchar(64)       as o_in_param    ,
    o_in_value_x         ::    varchar(64)       as o_in_value    ,
    o_param_loc_x        ::    int               as o_param_loc   ,
    o_out_opr_x          ::    varchar(64)       as o_out_opr     ,
    (o_in_value_x is not null)                   as is_decimal      -- 用于运算化简
  from sm_sc.ft_gradient_graph(i_indepdt_var_name, i_opr_chart)
  ;

  select o_in_param into v_root_out_param from sm_sc._vt_fn_grad__graph where o_out_param = '' and sess_id = v_sess_id limit 1;

  -- 02. 初始化插入计算图入参起点，包括常参、变参
  insert into sm_sc._vt_fn_grad__algebra
  (
    sess_id               ,
    o_out_param            ,
    o_out_algebra          ,
    is_decimal   
  )
  -- 常数代数
  select
    v_sess_id,
    o_out_param     :: varchar(64)     as   o_out_param     ,
    max(o_in_value) :: text            as   o_out_algebra   ,
    true                               as   is_decimal      -- 用于运算化简
  from sm_sc._vt_fn_grad__graph t
  where o_out_param <> ''
    and o_in_param is null
    and o_in_value is not null
    and sess_id = v_sess_id
  group by sess_id, o_out_param
  having count(*) = 1
    and max(o_out_opr) is null 
    and max(o_param_loc) = 1
  union all
  -- 入参代数
  select
    v_sess_id,
    tb_main.o_in_param   :: varchar(64)   as   o_out_param     ,
    tb_main.o_in_param   :: text          as   o_out_algebra   ,
    false                                 as   is_decimal      -- 用于运算化简
  from sm_sc._vt_fn_grad__graph tb_main
  left join sm_sc._vt_fn_grad__graph tb_a_comp
  on tb_main.o_in_param = tb_a_comp.o_out_param
    and tb_a_comp.sess_id = v_sess_id
  where tb_a_comp.o_out_param is null
    and tb_main.o_in_param is not null
    and tb_main.sess_id = v_sess_id
  group by tb_main.sess_id, tb_main.o_in_param
  ; 

  -- select * from sm_sc._vt_fn_grad__graph where sess_id = v_sess_id
  -- select * from sm_sc._vt_fn_grad__algebra where sess_id = v_sess_id

  if exists (select  from sm_sc._vt_fn_grad__algebra where sess_id = v_sess_id)
  then
    while not exists(select  from sm_sc._vt_fn_grad__algebra where o_out_param = v_root_out_param and sess_id = v_sess_id)
    loop
      insert into sm_sc._vt_fn_grad__algebra
      (
        sess_id      ,
        o_out_param   ,
        o_out_algebra ,
        is_decimal
      )
      with 
      cte_algebra as
      (
        select 
          tb_a_front_point.o_out_param      ,
          case
            when  
                -- 化简单一入参直接代入的场景一，包括直传单目非负常量、直传单目变量
                count(*) = 1 
                and (tb_a_front_point.o_out_opr = '' or tb_a_front_point.o_out_opr in ('sm_sc.fv_opr_add', 'sm_sc.fv_opr_mul'))
              then
                max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value))
            when  
                -- 化简为单一入参直接代入的场景三，包括减零、除以一、幂一
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_sub'
                  and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 2) || '%'
                or
                tb_a_front_point.o_out_opr in ('sm_sc.fv_opr_div', 'sm_sc.fv_opr_pow') 
                  and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 2) || '%'
              then
                max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value))
        
            when 
                -- 化简为单一入参直接代入的场景二，包括 n 目中加 n-1 个零、 n 目中乘 n-1 个一的场景
                tb_a_front_point.o_out_opr in ('sm_sc.fv_opr_add', 'sm_sc.fv_opr_mul')
                and
                count(*) 
                  filter 
                  (
                    where tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_add' and '0.0000000000000000000' like (coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value) || '%')
                      or tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_mul' and '1.0000000000000000000' like (coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value) || '%')
                  ) 
                >= count(*) - 1
              then
                case tb_a_front_point.o_out_opr
                  when 'sm_sc.fv_opr_add'
                    then
                      coalesce
                      (
                        max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) 
                          filter 
                          (
                            where '0.0000000000000000000' not like (coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value) || '%')
                          )
                        , '0.0'
                      )
                  when 'sm_sc.fv_opr_mul'
                    then
                      coalesce
                      (
                        max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) 
                          filter 
                          (
                            where '1.0000000000000000000' not like (coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value) || '%')
                          )
                        , '1.0'
                      )
                end
        
            -- 化简场景四，包括一幂得一、对数一得零、幂零得一、零幂得零，其中幂函数包括 pow, exp, 对数函数包括 log, ln
            when
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_pow'
                  and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_pow'
                  and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 2) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_exp'
                  and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
              then
                '1.0'
            when
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_pow'
                  and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_log'
                  and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 2) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_ln'
                  and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
              then
                '0.0'
        
            -- 运算符表达式，包括四则运算、幂指运算、对数运算
            when tb_a_front_point.o_out_opr in ('sm_sc.fv_opr_add', 'sm_sc.fv_opr_sub', 'sm_sc.fv_opr_mul', 'sm_sc.fv_opr_div', 'sm_sc.fv_opr_pow'/*, 'sm_sc.fv_opr_log'*/)
              then 
                '(' 
                || string_agg
                   (
                     -- 化简多目加零、多目乘一场景
                     case 
                       when 
                           -- tb_a_front_point.o_in_value::float = 0.0 and tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_add'
                           -- or tb_a_front_point.o_in_value::float = 1.0 :: floatand tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_mul'
                           tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_add' and '0.0000000000000000000' like (coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value) || '%')
                           or tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_mul' and '1.0000000000000000000' like (coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value) || '%')
                         then null :: text
                       else coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)
                     end
                     , case tb_a_front_point.o_out_opr 
                         when 'sm_sc.fv_opr_add' then ' + ' 
                         when 'sm_sc.fv_opr_sub' then ' - ' 
                         when 'sm_sc.fv_opr_mul' then ' * ' 
                         when 'sm_sc.fv_opr_div' then ' / ' 
                         when 'sm_sc.fv_opr_pow' then ' ^ ' 
                         /*when 'sm_sc.fv_opr_log' then ' ^! '*/  
                       end
                     order by tb_a_front_point.o_param_loc
                   ) 
                || ')'
        
            -- 函数式表达
            else
              case tb_a_front_point.o_out_opr
                when 'sm_sc.fv_opr_add'   then 'add'        
                when 'sm_sc.fv_opr_sub'   then 'sub'        
                when 'sm_sc.fv_opr_mul'   then 'mul'        
                when 'sm_sc.fv_opr_div'   then 'div'        
                when 'sm_sc.fv_opr_pow'   then 'pow'        
                when 'sm_sc.fv_opr_pow'   then 'power'      
                when 'sm_sc.fv_opr_exp'   then 'exp'        
                when 'sm_sc.fv_opr_log'   then 'log'        
                when 'sm_sc.fv_opr_ln'    then 'ln'         
                when 'sm_sc.fv_sin'       then 'sin'        
                when 'sm_sc.fv_cos'       then 'cos'        
                when 'sm_sc.fv_tan'       then 'tan'        
                when 'sm_sc.fv_cot'       then 'cot'        
                when 'sm_sc.fv_sec'       then 'sec'        
                when 'sm_sc.fv_csc'       then 'csc'        
                when 'sm_sc.fv_asin'      then 'asin'       
                when 'sm_sc.fv_acos'      then 'acos'       
                when 'sm_sc.fv_atan'      then 'atan'       
                when 'sm_sc.fv_acot'      then 'acot'       
                when 'sm_sc.fv_asec'      then 'asec'       
                when 'sm_sc.fv_acsc'      then 'acsc'       
                when 'sm_sc.fv_sinh'      then 'sinh'       
                when 'sm_sc.fv_cosh'      then 'cosh'       
                when 'sm_sc.fv_tanh'      then 'tanh'       
                when 'sm_sc.fv_asinh'     then 'asinh'      
                when 'sm_sc.fv_acosh'     then 'acosh'      
                when 'sm_sc.fv_atanh'     then 'atanh'      
                else coalesce(tb_a_front_point.o_out_opr, '')
              end
              || '(' 
              || string_agg
                 (
                   coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)
                   , ', ' 
                   order by tb_a_front_point.o_param_loc
                 ) 
              || ')'
          end 
            as o_out_algebra,
          case
            -- 化简场景四，包括一幂得一、对数一得零、幂零得一、零幂得零，此时 is_decimal 亦要更新为 true
            when
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_pow'
                  and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_pow'
                  and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 2) || '%'
                -- or
                -- tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_exp'
                --   and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_pow'
                  and '0.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
                or
                tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_log'
                  and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 2) || '%'
                -- or
                -- tb_a_front_point.o_out_opr = 'sm_sc.fv_opr_ln'
                --   and '1.0000000000000000000' like max(coalesce(tb_a_back_point.o_out_algebra, tb_a_front_point.o_in_value)) filter (where tb_a_front_point.o_param_loc = 1) || '%'
              then
                true
            else
              (false <> all(array_agg(coalesce(tb_a_back_point.is_decimal, tb_a_front_point.is_decimal))))
          end
            as is_decimal
        from sm_sc._vt_fn_grad__graph tb_a_front_point
        left join sm_sc._vt_fn_grad__algebra tb_a_back_point
        on tb_a_front_point.o_in_param = tb_a_back_point.o_out_param
          and tb_a_back_point.sess_id = v_sess_id
        where tb_a_front_point.o_out_param <> ''
          -- 避免重复代入
          and tb_a_front_point.o_out_param not in (select o_out_param from sm_sc._vt_fn_grad__algebra where sess_id = v_sess_id)
          and tb_a_front_point.sess_id = v_sess_id
        group by tb_a_front_point.o_out_param, tb_a_front_point.o_out_opr, tb_a_front_point.sess_id
        having count(*) filter (where tb_a_back_point.o_out_param is null and tb_a_front_point.o_in_value is null and tb_a_back_point.sess_id is null) = 0
      )
      select
        v_sess_id, 
        o_out_param,
        case when is_decimal then sm_sc.fv_algebra_execute(o_out_algebra)::text else o_out_algebra end as o_out_algebra,
        is_decimal
      from cte_algebra
      ;
    end loop;
  end if;

  select 
    o_out_algebra into v_ret
  from sm_sc._vt_fn_grad__algebra
  where o_out_param = v_root_out_param
    and sess_id = v_sess_id
  limit 1;
  
  -- 清理释放表变量
  delete from sm_sc._vt_fn_grad__graph where sess_id = v_sess_id;
  delete from sm_sc._vt_fn_grad__algebra where sess_id = v_sess_id;

  return coalesce(v_ret, '0');

end
$$
language plpgsql volatile
cost 100;

-- -- 代数式的求导：
-- -- 返回推导出来的导数代数式
-- -- 计算图入参为 代数式
-- select sm_sc.fv_gradient('v_x1_in', 'w1 * v_x1_in + w2 * v_x2 + w3 * v_x3'::text)
-- select sm_sc.fv_gradient('v_x_in', '(v_x_in * v_y) + exp(v_x_in * v_y) + power(v_x_in, 2)'::text)

-- -- 计算图为 json 配置
-- -- select pgv_set('vars', 'a_opr_chain_03', 
-- -- '
-- -- {
-- --   "out_param": "v_z_in",
-- --   "opr": "sm_sc.fv_opr_add",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in"
-- --         },
-- --         {
-- --           "out_param": "v_y_in"
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_exp",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "opr": "sm_sc.fv_opr_mul",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x_in"
-- --             },
-- --             {
-- --               "out_param": "v_y_in"
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_pow",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in"
-- --         },
-- --         {
-- --           "out_value": 2
-- --         }
-- --       ]
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select sm_sc.fv_gradient('v_x_in'::text, pgv_get('vars', 'a_opr_chain_03', NULL::jsonb));

-- -- 参考 https://baike.baidu.com/item/%E9%9B%85%E5%8F%AF%E6%AF%94%E7%9F%A9%E9%98%B5/10753754
-- drop function if exists sm_sc.fv_gradient_jacobi(varchar(64)[], text[]);
create or replace function sm_sc.fv_gradient_jacobi
(
  i_indepdt_var_names               varchar(64)[],
  i_depdt_var_opr_charts            text[]
)
returns text[]
as
$$
begin
  return 
  (
    with
    cte_arr_indepdt as
    (
      select 
        a_idx_depdt,
        array_agg(sm_sc.fv_gradient(i_indepdt_var_names[a_idx_indepdt], i_depdt_var_opr_charts[a_idx_depdt]) order by a_idx_indepdt) as a_arr_indepdt
      from generate_series(1, array_length(i_depdt_var_opr_charts, 1)) tb_a_idx_depdt(a_idx_depdt)
        , generate_series(1, array_length(i_indepdt_var_names, 1)) tb_a_idx_indepdt(a_idx_indepdt)
      group by a_idx_depdt
    )
    select  
      array_agg(a_arr_indepdt order by a_idx_depdt)
    from cte_arr_indepdt
  );
end
$$
language plpgsql volatile
cost 100;

-- select sm_sc.fv_gradient_jacobi(array['v_x1_in'], array['w1 * v_x1_in + w2 * v_x2 + w3 * v_x3'])
-- select sm_sc.fv_gradient_jacobi(array['r', 'a', 'b'], array['r*cos(a)*sin(b)', 'r*sin(a)*sin(b)', 'r*cos(b)'])
-- 本函数以标量 scalar为入参/出参，数组版本参考：sm_sc.fv_lambda_arr_ddepdt_dindepdt
-- 
-- drop function if exists sm_sc.fv_gradient_opr(float, varchar(64), int, float, float);
create or replace function sm_sc.fv_gradient_opr
(
  i_indepdt_var              float                  ,
  i_fn                       varchar(64)                     ,    -- i_fn = '' and i_co_value is not null, 则恒为常数；对于 y = x, 则 i_fn 设为 ''
  i_indepdt_var_loc          int              default 1      ,    -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 sub, div, pow, log 四种运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
  i_depdt_var                float   default null   ,    -- 非必要入参
  i_co_value                 float   default null        -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
  -- 约定：当 i_fn = '' and i_co_value is null, 表示函数: y = x;  
  --       当 i_fn = '' and i_co_value is not null, 表示函数: y = C (C 为 常数);  
)
returns float
as
$$
declare -- here
  v_sql_code  text;
  v_sql_ret   float;
  v_tmp       float;
begin

  if i_fn is null
  then 
    return null :: float;

  elsif i_fn = '' and i_co_value is not null
  then 
    return 0.0;
  -- -- -- elsif i_fn = 'sm_sc.fv_opr_const'
  -- -- -- then 
  -- -- --   return 0.0;

  -- if 判断覆盖到的 fn, 不需要查表，性能稍好
  elsif i_fn = 'sm_sc.fv_opr_add' 
    or i_fn = '' and i_co_value is null
  then 
    return 1.0;
  elsif i_fn = 'sm_sc.fv_opr_mul'
  then 
    return i_co_value;
  elsif i_fn = 'sm_sc.fv_opr_sub'
  then 
    return
      case i_indepdt_var_loc 
        when 1 
          then 1.0
        when 2 
          then -1.0
      end;
  elsif i_fn = 'sm_sc.fv_opr_div'
  then 
    return
      case i_indepdt_var_loc 
        when 1 
          then 1.0 :: float/ nullif(i_co_value, 0.0 :: float)
        when 2 
          then - i_co_value / nullif(power(i_indepdt_var, 2), 0.0 :: float)
      end;
  elsif i_fn = 'sm_sc.fv_opr_pow'
  then 
    return
      case i_indepdt_var_loc 
        when 1 
          then -- 优先使用 i_depdt_var 减小算法复杂度 -- -- i_co_value * power(i_indepdt_var, i_co_value - 1)
            case 
              when i_depdt_var is not null and i_indepdt_var <> 0.0
                then i_co_value * i_depdt_var / nullif(i_indepdt_var, 0.0 :: float)
              else i_co_value * power(i_indepdt_var, i_co_value - 1)
            end
        when 2 
          then -- 优先使用 i_depdt_var 减小算法复杂度 -- -- power(i_co_value, i_indepdt_var) * ln(i_co_value)
            case
              when i_depdt_var is not null
                then i_depdt_var * ln(i_co_value)
              else power(i_co_value, i_indepdt_var) * ln(i_co_value)
            end
      end;
  elsif i_fn = 'sm_sc.fv_opr_exp'
  then 
    return 
      case 
        when i_depdt_var is not null
          then i_depdt_var
        else exp(i_indepdt_var)
      end;
  elsif i_fn = 'sm_sc.fv_opr_log'
  then 
    return
      case i_indepdt_var_loc 
        when 1 
          then -- 优先使用 i_depdt_var 减小算法复杂度 -- -- -1.0 :: float/ (power(log(i_co_value, i_indepdt_var), 2) * i_indepdt_var * ln(i_co_value))
            case
              when i_depdt_var is not null
                then power(i_depdt_var, 2) / nullif(i_indepdt_var * ln(i_co_value), 0.0 :: float)
              else -power(log(i_indepdt_var :: decimal, i_co_value :: decimal), 2) / nullif(i_indepdt_var * ln(i_co_value :: decimal), 0.0 :: float)
            end
        when 2 
          then 1.0 :: float/ nullif(i_indepdt_var * ln(i_co_value), 0.0 :: float)
      end;
  elsif i_fn = 'sm_sc.fv_opr_ln'
  then 
    return 1.0 :: float/ i_indepdt_var;
  elsif i_fn = 'sm_sc.fv_sin'
  then 
    return cos(i_indepdt_var);
  elsif i_fn = 'sm_sc.fv_cos'
  then 
    return -sin(i_indepdt_var);
  elsif i_fn = 'sm_sc.fv_tan'
  then 
    return 1.0 :: float/ nullif(power(cos(i_indepdt_var)::float, 2), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_cot'
  then 
    return -1.0 :: float/ nullif(power(sin(i_indepdt_var)::float, 2), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_sec'
  then 
    return -- 优先使用 i_depdt_var 减小算法复杂度 -- -- tan(i_indepdt_var) / cos(i_indepdt_var);
      case
        when i_depdt_var is not null
          then i_depdt_var * tan(i_indepdt_var)
        else tan(i_indepdt_var) / nullif(cos(i_indepdt_var), 0.0 :: float)
      end;
  elsif i_fn = 'sm_sc.fv_csc'
  then 
    return -- 优先使用 i_depdt_var 减小算法复杂度 -- -- -cot(i_indepdt_var) / sin(i_indepdt_var);
      case
        when i_depdt_var is not null
          then -i_depdt_var * cot(i_indepdt_var)
        else -cot(i_indepdt_var) / nullif(sin(i_indepdt_var), 0.0 :: float)
      end;
  elsif i_fn = 'sm_sc.fv_asin'
  then 
    return 1.0 :: float/ nullif(power(1.0 :: float- power(i_indepdt_var, 2), 0.5 :: float), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_acos'
  then 
    return -1.0 :: float/ nullif(power(1.0 :: float- power(i_indepdt_var, 2), 0.5 :: float), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_atan'
  then 
    return 1.0 :: float/ nullif(1.0 :: float+ power(i_indepdt_var, 2), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_acot'
  then 
    return -1.0 :: float/ nullif(1.0 :: float+ power(i_indepdt_var, 2), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_asec'
  then 
    return 1.0 :: float/ nullif(i_indepdt_var * power(power(i_indepdt_var, 2) - 1.0 :: float, 0.5 :: float), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_acsc'
  then 
    return -1.0 :: float/ nullif(i_indepdt_var * power(power(i_indepdt_var, 2) - 1.0 :: float, 0.5 :: float), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_sinh'
  then 
    return cosh(i_indepdt_var);
  elsif i_fn = 'sm_sc.fv_cosh'
  then 
    return sinh(i_indepdt_var);
  elsif i_fn = 'sm_sc.fv_tanh'
  then 
    return -- 优先使用 i_depdt_var 减小算法复杂度 -- 1.0 :: float/ power(cosh(i_indepdt_var)::float, 2) 
      case 
        when i_depdt_var is not null 
          then 1.0 :: float- power(i_depdt_var, 2) 
        else 1.0 :: float/ nullif(power(cosh(i_indepdt_var)::float, 2), 0.0 :: float) 
      end;
  -- -- elsif i_fn = 'sm_sc.fv_coth'
  -- -- then 
  -- --   return
  -- -- elsif i_fn = 'sm_sc.fv_sech'
  -- -- then 
  -- --   return
  -- -- elsif i_fn = 'sm_sc.fv_csch'
  -- -- then 
  -- --   return
  elsif i_fn = 'sm_sc.fv_asinh'
  then 
    return 1.0 :: float/ nullif(power(power(i_indepdt_var, 2) + 1.0 :: float, 0.5 :: float), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_acosh'
  then 
    return 1.0 :: float/ nullif(power(power(i_indepdt_var, 2) - 1.0 :: float, 0.5 :: float), 0.0 :: float);
  elsif i_fn = 'sm_sc.fv_atanh'
  then 
    return 1.0 :: float/ nullif(1.0 :: float- power(i_indepdt_var, 2), 0.0 :: float);
  -- -- elsif i_fn = 'sm_sc.fv_acoth'
  -- -- then 
  -- --   return
  -- -- elsif i_fn = 'sm_sc.fv_asech'
  -- -- then 
  -- --   return
  -- -- elsif i_fn = 'sm_sc.fv_acsch'
  -- -- then 
  -- --   return
  elsif i_fn = 'sm_sc.fv_sigmoid'
  then 
    if i_depdt_var is not null  -- 优先使用 i_depdt_var 减小算法复杂度 -- 
    then 
      return i_depdt_var * (1.0 :: float- i_depdt_var);
    else
      v_tmp := nullif(exp(i_indepdt_var / 2), 0.0 :: float);
      return 1.0 :: float/ nullif(power(v_tmp + (1 / v_tmp), 2.0 :: float), 0.0 :: float);
    end if;
  -- 用户自定义函数/导数函数注册在 enum 表中，计算时查表，再动态 sql, 性能不如硬编码映射
  -- 自定义函数/导数函数只单目和双目运算（包括左目、右目），三目运算要预先分解定义为多个双目运算
  -- -- elsif exists (select from sm_sc.tb_dic_enum where enum_group = i_fn and enum_order = i_indepdt_var_loc)
  elsif exists 
  (
    select 
    from sm_sc.tb_dic_enum 
    where enum_name = 'node_fn_type'
      and enum_value = i_fn 
      -- and enum_order = i_indepdt_var_loc
    limit 1
  )
  then 
    select 
      'select ' || enum_value || '($1' || case when i_co_value is null then '' else ', $2' end || ')'
    into v_sql_code
    from sm_sc.tb_dic_enum 
    where enum_value = i_fn
    limit 1
    ;
-- raise notice 'debug: v_sql_code: %', v_sql_code;
    execute v_sql_code into v_sql_ret using case when i_indepdt_var_loc = 1 then i_indepdt_var else i_co_value end, case when i_indepdt_var_loc = 1 then i_co_value else i_indepdt_var end;

    return v_sql_ret;
  else 
    raise exception 'unknown function!';
  end if;

end
$$
language plpgsql volatile
cost 100;

-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   null :: varchar(64)           ,
-- --   1               ,
-- --   null            ,
-- --   null :: float
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   ''              ,
-- --   1               ,
-- --   null            ,
-- --   null :: float
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   ''              ,
-- --   1               ,
-- --   null            ,
-- --   12
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_opr_add'        -- ,
-- --   -- 0               ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_opr_mul'        ,
-- --   0               ,
-- --   null            ,
-- --   2.3
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_opr_sub'        ,
-- --   1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_opr_sub'        ,
-- --   2               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_opr_div'        ,
-- --   1               ,
-- --   null            ,
-- --   1.5
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_opr_div'        ,
-- --   2               ,
-- --   null            ,
-- --   1.5
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_pow'        ,
-- --   1               ,
-- --   power(2.0 :: float, 3.0) :: float ,
-- --   3.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_pow'        ,
-- --   1               ,
-- --   null            ,
-- --   3.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_pow'        ,
-- --   2               ,
-- --   power(3.0, 2.0 :: float) :: float,
-- --   3.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_pow'        ,
-- --   2               ,
-- --   null            ,
-- --   3.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_exp'        -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_exp'        ,
-- --   1               ,
-- --   exp(2.0 :: float)::float  -- ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_log'        ,
-- --   1               ,
-- --   log(2.0, 8.0)::float   ,
-- --   8.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_opr_log'        ,
-- --   1               ,
-- --   null            ,
-- --   8.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   8.0             ,
-- --   'sm_sc.fv_opr_log'        ,
-- --   2               ,
-- --   null            ,
-- --   2.0
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   5.0             ,
-- --   'sm_sc.fv_opr_ln'         -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   pi()::float / 2.0      ,
-- --   'sm_sc.fv_sin'        ,
-- --   1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   pi()::float / 2.0      ,
-- --   'sm_sc.fv_cos'        ,
-- --   1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -pi()::float / 3.0     ,
-- --   'sm_sc.fv_tan'        -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -pi()::float / 3.0     ,
-- --   'sm_sc.fv_cot'        -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -pi():: float / 6.0      ,
-- --   'sm_sc.fv_sec'        -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -pi():: float / 6.0      ,
-- --   'sm_sc.fv_sec'           ,
-- --   1                  ,
-- --   1.0 :: float/ cos(-pi() / 6.0) :: float  -- ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -pi() :: float / 3.0     ,
-- --   'sm_sc.fv_csc'        -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -pi() :: float / 3.0     ,
-- --   'sm_sc.fv_csc'        ,
-- --   1               ,
-- --   1.0 :: float/ sin(-pi() / 3.0) :: float  -- ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -0.5            ,
-- --   'sm_sc.fv_asin'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -0.5            ,
-- --   'sm_sc.fv_acos'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_atan'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -0.5            ,
-- --   'sm_sc.fv_acot'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.5             ,
-- --   'sm_sc.fv_asec'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   -1.5            ,
-- --   'sm_sc.fv_acsc'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_sinh'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_cosh'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_tanh'       -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   1.0 :: float            ,
-- --   'sm_sc.fv_tanh'       ,
-- --   1               ,
-- --   tanh(1.0 :: float)::float  -- ,
-- --   -- null
-- -- );
-- -- -- select sm_sc.fv_gradient_opr
-- -- -- (
-- -- --   1.0 :: float            ,
-- -- --   'sm_sc.fv_coth'       -- ,
-- -- --   -- 1               -- ,
-- -- --   -- null            ,
-- -- --   -- null
-- -- -- );
-- -- -- select sm_sc.fv_gradient_opr
-- -- -- (
-- -- --   1.0 :: float            ,
-- -- --   'sm_sc.fv_sech'       -- ,
-- -- --   -- 1               -- ,
-- -- --   -- null            ,
-- -- --   -- null
-- -- -- );
-- -- -- select sm_sc.fv_gradient_opr
-- -- -- (
-- -- --   1.0 :: float            ,
-- -- --   'sm_sc.fv_csch'       -- ,
-- -- --   -- 1               -- ,
-- -- --   -- null            ,
-- -- --   -- null
-- -- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   3.5             ,
-- --   'sm_sc.fv_asinh'      -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   3.5             ,
-- --   'sm_sc.fv_acosh'      -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   3.5             ,
-- --   'sm_sc.fv_atanh'      -- ,
-- --   -- 1               -- ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- -- select sm_sc.fv_gradient_opr
-- -- -- (
-- -- --   1.0 :: float            ,
-- -- --   'sm_sc.fv_acoth'      -- ,
-- -- --   -- 1               -- ,
-- -- --   -- null            ,
-- -- --   -- null
-- -- -- );
-- -- -- select sm_sc.fv_gradient_opr
-- -- -- (
-- -- --   1.0 :: float            ,
-- -- --   'sm_sc.fv_asech'      -- ,
-- -- --   -- 1               -- ,
-- -- --   -- null            ,
-- -- --   -- null
-- -- -- );
-- -- -- select sm_sc.fv_gradient_opr
-- -- -- (
-- -- --   1.0 :: float            ,
-- -- --   'sm_sc.fv_acsch'      ,
-- -- --   1               -- ,
-- -- --   -- null            ,
-- -- --   -- null
-- -- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_sigmoid'    -- ,
-- --   -- 1               ,
-- --   -- null            ,
-- --   -- null
-- -- );
-- -- select sm_sc.fv_gradient_opr
-- -- (
-- --   2.0             ,
-- --   'sm_sc.fv_sigmoid'    ,
-- --   1               ,
-- --   1.0 :: float/ (1.0 :: float+ exp(-2.0 :: float))::float -- ,
-- --   -- null
-- -- );

-- select sm_sc.fv_gradient_opr
-- (
--   2.0             ,
--   'sm_sc.fv_softplus'    -- ,
--   -- 1               ,
--   -- null ,
--   -- null
-- );
-- select sm_sc.fv_gradient_opr
-- (
--   2.0             ,
--   'sm_sc.fv_leaky_relu'    ,
--   1               ,
--   null ,
--   0.3
-- );

-- ---------------------------------------------------------------------------
-- 标量 scalar
-- 支持多目运算符
-- drop function if exists sm_sc.fv_gradient_opr(float, varchar(64), int, float, float[]);
create or replace function sm_sc.fv_gradient_opr
(
  i_indepdt_var              float                  ,
  i_fn                     varchar(64)                     ,    -- i_fn = '' and i_co_value is not null, 则恒为常数；对于 y = x, 则 i_fn 设为 ''
  i_indepdt_var_loc          int                             ,    -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 sub, div, pow, log 四种运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
  i_depdt_var            float                  ,    -- 非必要入参
  i_co_values              float[]                     -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
  -- 约定：当 i_fn = '' and i_co_value is null, 表示函数: y = x;  
  --       当 i_fn = '' and i_co_value is not null, 表示函数: y = C (C 为 常数);  
)
returns float
as
$$
-- declare
begin
  if coalesce(array_length(i_co_values, 1), 0) < 2
  then 
    return sm_sc.fv_gradient_opr(i_indepdt_var, i_fn, i_indepdt_var_loc, i_depdt_var, i_co_values[1])
    ;
  else
    if i_fn = 'sm_sc.fv_opr_add'
    then
      return 1.0;                                           
    elsif i_fn = 'sm_sc.fv_opr_mul'             
    then                                          
      return 
        case 
          when array_length(i_co_values, 1) > 3 and i_indepdt_var <> 0.0 and i_depdt_var is not null
            then i_depdt_var / nullif(i_indepdt_var, 0.0 :: float)
          else sm_sc.fv_aggr_slice_prod(i_co_values)
        end
      ;
    else
      raise exception 'unknown function: % for such co_params!', i_fn;
    end if;
  end if;
end
$$
language plpgsql stable
cost 100;


-- select sm_sc.fv_gradient_opr
-- (
--   1.0 :: float            ,
--   'sm_sc.fv_opr_add'        ,
--   0               ,
--   null            ,
--   array[1.5, 2.5]
-- );

-- select sm_sc.fv_gradient_opr
-- (
--   1.0 :: float            ,
--   'sm_sc.fv_opr_mul'        ,
--   0               ,
--   null            ,
--   array[1.5, 2.5]
-- );

-- select sm_sc.fv_gradient_opr
-- (
--   2.0             ,
--   'sm_sc.fv_opr_div'    ,
--   1               ,
--   null ,
--   array[1.6]
-- );
-- drop function if exists sm_sc.fv_d_activate_boxcox(float[], float);
create or replace function sm_sc.fv_d_activate_boxcox
(
  i_indepdt            float[]        ,
  i_lambda           float    -- 
)
returns float[]
as
$$
-- declare -- here
begin
  -- 审计：不支持负值
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if true = any(i_indepdt <=` 0.0 :: float)
    then 
      raise exception 'unsupport negative value in i_indepdt.';
    end if;
  end if;
  
  if array_length(i_indepdt, array_ndims(i_indepdt)) is null
  then 
    return i_indepdt;
  elsif array_ndims(i_indepdt) <= 4
  then 
    if i_lambda = 0
    then 
      return 1.0 :: float /` i_indepdt;
    else 
      return i_indepdt ^` (i_lambda :: float - 1.0);
    end if;
  else
    raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_boxcox(array[[1.0 :: float, 2.0], [3.0, 4.0]], 0.2)
-- select sm_sc.fv_d_activate_boxcox(array[[[1.0 :: float, 2.0], [3.0, 4.0]]], 0.2)
-- select sm_sc.fv_d_activate_boxcox(array[[[[1.0 :: float, 2.0], [3.0, 4.0]]]], 0.2)
-- select sm_sc.fv_d_activate_boxcox(array[1.5, 2.5, 3.5], 0.2)
-- select sm_sc.fv_d_activate_boxcox(array[]::float[], 0.2)
-- select sm_sc.fv_d_activate_boxcox(array[array[], array []]::float[], 0.3)
-- drop function if exists sm_sc.fv_d_activate_elu(float[], float);
create or replace function sm_sc.fv_d_activate_elu
(
  i_indepdt            float[]        ,
  i_asso_value       float    -- 
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_indepdt, array_ndims(i_indepdt)) is null
  then 
    return i_indepdt;
  end if;

  -- sm_sc.fv_d_elu([][], )
  if array_ndims(i_indepdt) =  2
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_indepdt, 2)
      loop
        i_indepdt[v_y_cur][v_x_cur] := sm_sc.fv_d_elu_asso(i_indepdt[v_y_cur][v_x_cur], i_asso_value);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_elu [],
  elsif array_ndims(i_indepdt) = 1
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop
      i_indepdt[v_y_cur] := sm_sc.fv_d_elu_asso(i_indepdt[v_y_cur], i_asso_value);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_elu_asso([][][])
  elsif array_ndims(i_indepdt) = 3
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          i_indepdt[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_d_elu_asso(i_indepdt[v_y_cur][v_x_cur][v_x3_cur], i_asso_value)
          ;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  -- sm_sc.fv_d_elu_asso([][][][])
  elsif array_ndims(i_indepdt) = 4
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_indepdt, 4)
          loop
            i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_d_elu_asso(i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_asso_value)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  else
    raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_elu(array[[1.0 :: float, -2.0], [3.0, 4.0]], 0.3)
-- select sm_sc.fv_d_activate_elu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]], 0.3)
-- select sm_sc.fv_d_activate_elu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]], 0.3)
-- select sm_sc.fv_d_activate_elu(array[1.5, -2.5, 3.5], 0.3)
-- select sm_sc.fv_d_activate_elu(array[]::float[], 0.3)
-- select sm_sc.fv_d_activate_elu(array[array[], array []]::float[], 0.3)
-- drop function if exists sm_sc.fv_d_activate_gelu(float[]);
create or replace function sm_sc.fv_d_activate_gelu
(
  i_indepdt     float[]
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_indepdt, array_ndims(i_indepdt)) is null
  then 
    return i_indepdt;
  end if;

  -- sm_sc.fv_d_gelu([][], )
  if array_ndims(i_indepdt) =  2
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_indepdt, 2)
      loop
        i_indepdt[v_y_cur][v_x_cur] := sm_sc.fv_d_gelu(i_indepdt[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_gelu [],
  elsif array_ndims(i_indepdt) = 1
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop
      i_indepdt[v_y_cur] := sm_sc.fv_d_gelu(i_indepdt[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_gelu([][][])
  elsif array_ndims(i_indepdt) = 3
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          i_indepdt[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_d_gelu(i_indepdt[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  -- sm_sc.fv_d_gelu([][][][])
  elsif array_ndims(i_indepdt) = 4
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_indepdt, 4)
          loop
            i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_d_gelu(i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  else
    raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_gelu(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_d_activate_gelu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_d_activate_gelu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_d_activate_gelu(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_d_activate_gelu(array[]::float[])
-- select sm_sc.fv_d_activate_gelu(array[array[], array []]::float[])
-- drop function if exists sm_sc.fv_d_activate_leaky_relu(float[], float);
create or replace function sm_sc.fv_d_activate_leaky_relu
(
  i_indepdt            float[]        ,
  i_asso_value       float    -- 
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_indepdt, array_ndims(i_indepdt)) is null
  then 
    return i_indepdt;
  end if;

  -- sm_sc.fv_d_leaky_relu([][], )
  if array_ndims(i_indepdt) =  2
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_indepdt, 2)
      loop
        i_indepdt[v_y_cur][v_x_cur] := sm_sc.fv_d_leaky_relu_asso(i_indepdt[v_y_cur][v_x_cur], i_asso_value);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_leaky_relu [],
  elsif array_ndims(i_indepdt) = 1
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop
      i_indepdt[v_y_cur] := sm_sc.fv_d_leaky_relu_asso(i_indepdt[v_y_cur], i_asso_value);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_leaky_relu_asso([][][])
  elsif array_ndims(i_indepdt) = 3
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          i_indepdt[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_d_leaky_relu_asso(i_indepdt[v_y_cur][v_x_cur][v_x3_cur], i_asso_value)
          ;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  -- sm_sc.fv_d_leaky_relu_asso([][][][])
  elsif array_ndims(i_indepdt) = 4
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_indepdt, 4)
          loop
            i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_d_leaky_relu_asso(i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_asso_value)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  else
    raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_leaky_relu(array[[1.0 :: float, -2.0], [3.0, 4.0]], 0.2)
-- select sm_sc.fv_d_activate_leaky_relu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]], 0.2)
-- select sm_sc.fv_d_activate_leaky_relu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]], 0.2)
-- select sm_sc.fv_d_activate_leaky_relu(array[1.5, -2.5, 3.5], 0.2)
-- select sm_sc.fv_d_activate_leaky_relu(array[]::float[], 0.2)
-- select sm_sc.fv_d_activate_leaky_relu(array[array[], array []]::float[], 0.3)
-- drop function if exists sm_sc.fv_d_activate_relu(float[]);
create or replace function sm_sc.fv_d_activate_relu
(
  i_indepdt     float[]
)
returns float[]
as
$$
-- -- declare -- here
-- --   v_x_cur   int  := 1  ;
-- --   v_y_cur   int  := 1  ;
begin
-- --   -- log(null :: float[][], float) = null :: float[][]
-- --   if array_length(i_indepdt, array_ndims(i_indepdt)) is null
-- --   then 
-- --     return i_indepdt;
-- --   end if;
-- -- 
-- --   -- sm_sc.fv_d_relu([][], )
-- --   if array_ndims(i_indepdt) =  2
-- --   then
-- --     while v_y_cur <= array_length(i_indepdt, 1)
-- --     loop 
-- --       v_x_cur := 1  ;
-- --       while v_x_cur <= array_length(i_indepdt, 2)
-- --       loop
-- --         i_indepdt[v_y_cur][v_x_cur] := sm_sc.fv_d_relu(i_indepdt[v_y_cur][v_x_cur]);
-- --         v_x_cur := v_x_cur + 1;
-- --       end loop;
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_indepdt;
-- -- 
-- --   -- sm_sc.fv_d_relu [],
-- --   elsif array_ndims(i_indepdt) = 1
-- --   then
-- --     while v_y_cur <= array_length(i_indepdt, 1)
-- --     loop
-- --       i_indepdt[v_y_cur] := sm_sc.fv_d_relu(i_indepdt[v_y_cur]);
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_indepdt;
-- -- 
-- --   else
-- --     raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
-- --   end if;

  return (i_indepdt >` 0.0 :: float) :: int[] :: float[];
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_relu(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_d_activate_relu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_d_activate_relu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_d_activate_relu(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_d_activate_relu(array[]::float[])
-- select sm_sc.fv_d_activate_relu(array[[], []]::float[])
-- drop function if exists sm_sc.fv_d_activate_selu(float[]);
create or replace function sm_sc.fv_d_activate_selu
(
  i_indepdt     float[]
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_indepdt, array_ndims(i_indepdt)) is null
  then 
    return i_indepdt;
  end if;

  -- sm_sc.fv_d_selu([][], )
  if array_ndims(i_indepdt) =  2
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_indepdt, 2)
      loop
        i_indepdt[v_y_cur][v_x_cur] := sm_sc.fv_d_selu(i_indepdt[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_selu [],
  elsif array_ndims(i_indepdt) = 1
  then
    while v_y_cur <= array_length(i_indepdt, 1)
    loop
      i_indepdt[v_y_cur] := sm_sc.fv_d_selu(i_indepdt[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_indepdt;

  -- sm_sc.fv_d_selu([][][])
  elsif array_ndims(i_indepdt) = 3
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          i_indepdt[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_d_selu(i_indepdt[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  -- sm_sc.fv_d_selu([][][][])
  elsif array_ndims(i_indepdt) = 4
  then
    for v_y_cur in 1 .. array_length(i_indepdt, 1)
    loop
      for v_x_cur in 1 .. array_length(i_indepdt, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_indepdt, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_indepdt, 4)
          loop
            i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_d_selu(i_indepdt[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_indepdt;
    
  else
    raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_selu(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_d_activate_selu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_d_activate_selu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_d_activate_selu(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_d_activate_selu(array[]::float[])
-- select sm_sc.fv_d_activate_selu(array[array[], array []]::float[])
-- drop function if exists sm_sc.fv_d_activate_sigmoid(float[], float[]);
create or replace function sm_sc.fv_d_activate_sigmoid
(
  i_indepdt         float[], 
  i_depdt         float[]   default null
)
returns float[]
as
$$
declare -- here
  v_tmp   float[];
begin
  -- 审计维度
  if array_length(i_indepdt, array_ndims(i_indepdt)) is null and array_length(i_depdt, array_ndims(i_depdt)) is null
  then 
    return null;
  end if;
  
  if i_depdt is null
  then 
    i_depdt := sm_sc.fv_activate_sigmoid(i_indepdt);
  end if;
  
  return i_depdt *` (1.0 :: float-` i_depdt);

  -- -- -- 以下虽然算法复杂度较优，但不方便处理溢出
  -- -- if i_depdt is not null
  -- -- then
  -- --   return i_depdt *` (1.0 :: float-` i_depdt);
  -- -- else
  -- --   -- -- -- -- -- 截断处理溢出更不合理
  -- --   -- -- -- -- v_tmp := sm_sc.fv_nullif(^` ((((-` i_indepdt) +` sm_sc.fv_ele_replace(((-` i_indepdt) <` -1.49e3 :: float) :: int[] :: float[], array[1.0 :: float], '-inf' :: float))) /` 2.0 :: float), 0.0 :: float);
  -- --   v_tmp := sm_sc.fv_nullif(^` (i_indepdt /` 2.0 :: float), 0.0 :: float);
  -- --   return 1.0 :: float /` ((v_tmp +` (1.0 :: float /` v_tmp)) ^` 2.0 :: float);
  -- -- end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_sigmoid(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_d_activate_sigmoid(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_d_activate_sigmoid(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_d_activate_sigmoid(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_d_activate_sigmoid(array[]::float[])
-- select sm_sc.fv_d_activate_sigmoid(array[array[], array []]::float[])
-- drop function if exists sm_sc.fv_d_activate_softplus(float[]);
create or replace function sm_sc.fv_d_activate_softplus
(
  i_indepdt     float[]
)
returns float[]
as
$$
-- -- declare -- here
-- --   v_x_cur   int  := 1  ;
-- --   v_y_cur   int  := 1  ;
begin
  -- -- -- log(null :: float[][], float) = null :: float[][]
  -- -- if array_length(i_indepdt, array_ndims(i_indepdt)) is null
  -- -- then 
  -- --   return i_indepdt;
  -- -- end if;
  -- -- 
  -- -- -- sm_sc.fv_d_softplus([][], )
  -- -- if array_ndims(i_indepdt) =  2
  -- -- then
  -- --   while v_y_cur <= array_length(i_indepdt, 1)
  -- --   loop 
  -- --     v_x_cur := 1  ;
  -- --     while v_x_cur <= array_length(i_indepdt, 2)
  -- --     loop
  -- --       i_indepdt[v_y_cur][v_x_cur] := sm_sc.fv_d_softplus(i_indepdt[v_y_cur][v_x_cur]);
  -- --       v_x_cur := v_x_cur + 1;
  -- --     end loop;
  -- --     v_y_cur := v_y_cur + 1;
  -- --   end loop;
  -- --   return i_indepdt;
  -- -- 
  -- -- -- sm_sc.fv_d_softplus [],
  -- -- elsif array_ndims(i_indepdt) = 1
  -- -- then
  -- --   while v_y_cur <= array_length(i_indepdt, 1)
  -- --   loop
  -- --     i_indepdt[v_y_cur] := sm_sc.fv_d_softplus(i_indepdt[v_y_cur]);
  -- --     v_y_cur := v_y_cur + 1;
  -- --   end loop;
  -- --   return i_indepdt;
  -- -- 
  -- -- else
  -- --   raise exception 'no method for such length!  Dim: %;', array_dims(i_indepdt);
  -- -- end if;
  
  return 1.0 :: float /` (1.0 :: float +` (^` (-` i_indepdt)));
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_softplus(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_d_activate_softplus(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_d_activate_softplus(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_d_activate_softplus(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_d_activate_softplus(array[]::float[])
-- select sm_sc.fv_d_activate_softplus(array[[],  []]::float[])
-- drop function if exists sm_sc.fv_d_activate_swish(float[], float[]);
create or replace function sm_sc.fv_d_activate_swish
(
  -- i_array     float[]
  i_indepdt         float[], 
  i_depdt         float[]   default null
)
returns float[]
as
$$
declare -- here
  -- -- v_x_cur   int  := 1  ;
  -- -- v_y_cur   int  := 1  ;
  v_sigmoid  float[];
begin
  -- -- -- log(null :: float[][], float) = null :: float[][]
  -- -- if array_length(i_array, array_ndims(i_array)) is null
  -- -- then 
  -- --   return i_array;
  -- -- end if;
  -- -- 
  -- -- -- sm_sc.fv_d_swish([][], )
  -- -- if array_ndims(i_array) =  2
  -- -- then
  -- --   while v_y_cur <= array_length(i_array, 1)
  -- --   loop 
  -- --     v_x_cur := 1  ;
  -- --     while v_x_cur <= array_length(i_array, 2)
  -- --     loop
  -- --       i_array[v_y_cur][v_x_cur] := sm_sc.fv_d_swish(i_array[v_y_cur][v_x_cur]);
  -- --       v_x_cur := v_x_cur + 1;
  -- --     end loop;
  -- --     v_y_cur := v_y_cur + 1;
  -- --   end loop;
  -- --   return i_array;
  -- -- 
  -- -- -- sm_sc.fv_d_swish [],
  -- -- elsif array_ndims(i_array) = 1
  -- -- then
  -- --   while v_y_cur <= array_length(i_array, 1)
  -- --   loop
  -- --     i_array[v_y_cur] := sm_sc.fv_d_swish(i_array[v_y_cur]);
  -- --     v_y_cur := v_y_cur + 1;
  -- --   end loop;
  -- --   return i_array;
  -- -- 
  -- -- else
  -- --   raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  -- -- end if;
  
  if i_depdt is not null
  then 
    return i_depdt *` (1.0 :: float +` i_indepdt -` i_depdt) /` i_indepdt;
  else
    v_sigmoid := 1.0 :: float /` (1.0 :: float +` (^` (-` i_indepdt)));
    return v_sigmoid *` (1.0 :: float +` (i_indepdt *` (1.0 :: float -` v_sigmoid)));
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_d_activate_swish(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_d_activate_swish(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_d_activate_swish(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_d_activate_swish(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_d_activate_swish(array[]::float[])
-- select sm_sc.fv_d_activate_swish(array[[],  []]::float[])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_activate_boxcox(float[], float);
create or replace function sm_sc.fv_activate_boxcox
(
  i_array     float[],
  i_lambda    float
)
returns float[]
as
$$
-- declare 

begin
  -- 审计：不支持负值
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if true = any(i_array <=` 0.0 :: float)
    then 
      raise exception 'unsupport negative value in i_array.';
    end if;
  end if;
  
  -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  if array_ndims(i_array) is null
  then 
    return array[] :: float[];
  elsif i_lambda = 0.0
  then
    return ^!` i_array;
  else
    return ((i_array ^` i_lambda) -` 1.0 :: float) /` i_lambda;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_activate_boxcox
--   (
--     array[[12.5, 9.3], [5.6, 32.4]]::float[]
--     , 2
--   )
-- ;
-- select sm_sc.fv_activate_boxcox
--   (
--     array[[[12.5, 9.3], [5.6, 32.4]]]::float[]
--     , 2
--   )
-- ;
-- select sm_sc.fv_activate_boxcox
--   (
--     array[[[[12.5, 9.3], [5.6, 32.4]]]]::float[]
--     , 2
--   )
-- ;
-- select sm_sc.fv_activate_boxcox
--   (
--     array[10.3, 1.9, 88.6, 1001.6]::float[]
--     , 0.0
--   )
-- ;
-- drop function if exists sm_sc.fv_activate_elu(float[], float);
create or replace function sm_sc.fv_activate_elu
(
  i_array       float[]        ,
  i_asso_value  float    -- 
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- sm_sc.fv_elu([][], )
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := sm_sc.fv_elu(i_array[v_y_cur][v_x_cur], i_asso_value);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sm_sc.fv_elu [],
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := sm_sc.fv_elu(i_array[v_y_cur], i_asso_value);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;


  -- sm_sc.fv_elu([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_elu(i_array[v_y_cur][v_x_cur][v_x3_cur], i_asso_value)
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- sm_sc.fv_elu([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_elu(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_asso_value)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_activate_elu(array[[1.0 :: float, -2.0], [3.0, 4.0]], 0.3)
-- select sm_sc.fv_activate_elu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]], 0.3)
-- select sm_sc.fv_activate_elu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]], 0.3)
-- select sm_sc.fv_activate_elu(array[1.5, -2.5, 3.5], 0.2)
-- select sm_sc.fv_activate_elu(array[]::float[], 0.2)
-- select sm_sc.fv_activate_elu(array[array[], array []]::float[], 0.2)
-- drop function if exists sm_sc.fv_activate_gelu(float[]);
create or replace function sm_sc.fv_activate_gelu
(
  i_array     float[]
)
returns float[]
as
$$
-- -- declare -- here
-- --   v_x_cur   int  := 1  ;
-- --   v_y_cur   int  := 1  ;
begin
-- --   -- log(null :: float[][], float) = null :: float[][]
-- --   if array_length(i_array, array_ndims(i_array)) is null
-- --   then 
-- --     return i_array;
-- --   end if;
-- -- 
-- --   -- sm_sc.fv_gelu([][], )
-- --   if array_ndims(i_array) =  2
-- --   then
-- --     while v_y_cur <= array_length(i_array, 1)
-- --     loop 
-- --       v_x_cur := 1  ;
-- --       while v_x_cur <= array_length(i_array, 2)
-- --       loop
-- --         i_array[v_y_cur][v_x_cur] := sm_sc.fv_gelu(i_array[v_y_cur][v_x_cur]);
-- --         v_x_cur := v_x_cur + 1;
-- --       end loop;
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_array;
-- -- 
-- --   -- sm_sc.fv_gelu [],
-- --   elsif array_ndims(i_array) = 1
-- --   then
-- --     while v_y_cur <= array_length(i_array, 1)
-- --     loop
-- --       i_array[v_y_cur] := sm_sc.fv_gelu(i_array[v_y_cur]);
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_array;
-- -- 
-- --   else
-- --     raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
-- --   end if;

  return 0.5 :: float *` i_array *` (1.0 :: float +` sm_sc.fv_tanh(0.7978845608028654 :: float *` (i_array +` (0.044715 :: float *` (i_array ^` 3.0 :: float)))));
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_activate_gelu(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_activate_gelu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_activate_gelu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_activate_gelu(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_activate_gelu(array[]::float[])
-- select sm_sc.fv_activate_gelu(array[array[], array []]::float[])
-- drop function if exists sm_sc.fv_activate_leaky_relu(float[], float);
create or replace function sm_sc.fv_activate_leaky_relu
(
  i_array     float[]        ,
  i_asso_value  float    -- 
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  if i_asso_value is null 
  then 
    i_asso_value := 0.01;
  end if;
  
  -- leaky_relu(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- sm_sc.fv_leaky_relu([][], )
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := sm_sc.fv_leaky_relu(i_array[v_y_cur][v_x_cur], i_asso_value);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sm_sc.fv_leaky_relu [],
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := sm_sc.fv_leaky_relu(i_array[v_y_cur], i_asso_value);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;


  -- sm_sc.fv_leaky_relu([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_leaky_relu(i_array[v_y_cur][v_x_cur][v_x3_cur], i_asso_value)
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- sm_sc.fv_leaky_relu([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_leaky_relu(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur], i_asso_value)
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_activate_leaky_relu(array[[1.0 :: float, -2.0], [3.0, 4.0]], 0.3)
-- select sm_sc.fv_activate_leaky_relu(array[[[1.0 :: float, -2.0], [3.0, 4.0]]], 0.3)
-- select sm_sc.fv_activate_leaky_relu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]], 0.3)
-- select sm_sc.fv_activate_leaky_relu(array[1.5, -2.5, 3.5], 0.2)
-- select sm_sc.fv_activate_leaky_relu(array[]::float[], 0.2)
-- select sm_sc.fv_activate_leaky_relu(array[array[], array []]::float[], 0.2)
-- drop function if exists sm_sc.fv_activate_relu(float[]);
create or replace function sm_sc.fv_activate_relu  
(
  i_array     float[]
)
returns float[]
as
$$
-- declare 

begin
  return i_array @>` (0.0 :: float);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_activate_relu(array[[12.3, -25.1], [-2.56, 3.25]]) ~=` 6
-- select sm_sc.fv_activate_relu(array[[[12.3, -25.1], [-2.56, 3.25]]]) ~=` 6
-- select sm_sc.fv_activate_relu(array[[[[12.3, -25.1], [-2.56, 3.25]]]]) ~=` 6
-- select sm_sc.fv_activate_relu(array[12.3, 25.1, -28.33]) ~=` 6
-- select sm_sc.fv_activate_relu(array[]::float[]) ~=` 6
-- select sm_sc.fv_activate_relu(array[array[], array []]::float[]) ~=` 6
-- drop function if exists sm_sc.fv_activate_selu(float[]);
create or replace function sm_sc.fv_activate_selu
(
  i_array     float[]
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- selu(null :: float[][], float) = null :: float[][]
  if array_length(i_array, array_ndims(i_array)) is null
  then 
    return i_array;
  end if;

  -- sm_sc.fv_selu([][], )
  if array_ndims(i_array) =  2
  then
    while v_y_cur <= array_length(i_array, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_array, 2)
      loop
        i_array[v_y_cur][v_x_cur] := sm_sc.fv_selu(i_array[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sm_sc.fv_selu [],
  elsif array_ndims(i_array) = 1
  then
    while v_y_cur <= array_length(i_array, 1)
    loop
      i_array[v_y_cur] := sm_sc.fv_selu(i_array[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_array;

  -- sm_sc.fv_selu([][][])
  elsif array_ndims(i_array) = 3
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          i_array[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_selu(i_array[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_array;
    
  -- sm_sc.fv_selu([][][][])
  elsif array_ndims(i_array) = 4
  then
    for v_y_cur in 1 .. array_length(i_array, 1)
    loop
      for v_x_cur in 1 .. array_length(i_array, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_array, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_array, 4)
          loop
            i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_selu(i_array[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_array;
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_activate_selu(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_activate_selu(array[[[1.0 :: float, -2.0], [3.0, 4.0]],[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_activate_selu(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_activate_selu(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_activate_selu(array[]::float[])
-- select sm_sc.fv_activate_selu(array[array[], array []]::float[])
-- drop function if exists sm_sc.fv_activate_sigmoid(anyarray);
create or replace function sm_sc.fv_activate_sigmoid
(
  i_right     anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- exp([][])
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        i_right[v_y_cur][v_x_cur] := sm_sc.fv_sigmoid(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- exp []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      i_right[v_y_cur] := sm_sc.fv_sigmoid(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return i_right;

  -- exp([][][])
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_sigmoid(i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- exp([][][][])
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_sigmoid(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur])
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_activate_sigmoid(array[array[12.3, 25.1], array[2.56, '-inf' :: float]])
-- select sm_sc.fv_activate_sigmoid(array[12.3, 'inf' :: float, 28.33, '-inf' :: float])
-- select sm_sc.fv_activate_sigmoid(array[]::float[])
-- select sm_sc.fv_activate_sigmoid(array[array[], array []]::float[])
-- select 
--   sm_sc.fv_activate_sigmoid
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.fv_activate_sigmoid
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )
-- drop function if exists sm_sc.fv_activate_softplus(float[]);
create or replace function sm_sc.fv_activate_softplus
(
  i_array     float[]
)
returns float[]
as
$$
-- -- declare -- here
-- --   v_x_cur   int  := 1  ;
-- --   v_y_cur   int  := 1  ;
begin
-- --   -- log(null :: float[][], float) = null :: float[][]
-- --   if array_length(i_array, array_ndims(i_array)) is null
-- --   then 
-- --     return i_array;
-- --   end if;
-- -- 
-- --   -- sm_sc.fv_softplus([][], )
-- --   if array_ndims(i_array) =  2
-- --   then
-- --     while v_y_cur <= array_length(i_array, 1)
-- --     loop 
-- --       v_x_cur := 1  ;
-- --       while v_x_cur <= array_length(i_array, 2)
-- --       loop
-- --         i_array[v_y_cur][v_x_cur] := sm_sc.fv_softplus(i_array[v_y_cur][v_x_cur]);
-- --         v_x_cur := v_x_cur + 1;
-- --       end loop;
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_array;
-- -- 
-- --   -- sm_sc.fv_softplus [],
-- --   elsif array_ndims(i_array) = 1
-- --   then
-- --     while v_y_cur <= array_length(i_array, 1)
-- --     loop
-- --       i_array[v_y_cur] := sm_sc.fv_softplus(i_array[v_y_cur]);
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_array;
-- -- 
-- --   else
-- --     raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
-- --   end if;

  return ^!` (1.0 :: float +` (^` i_array));
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_activate_softplus(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_activate_softplus(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_activate_softplus(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_activate_softplus(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_activate_softplus(array[]::float[])
-- select sm_sc.fv_activate_softplus(array[array[], array []]::float[])
-- drop function if exists sm_sc.fv_activate_swish(float[]);
create or replace function sm_sc.fv_activate_swish
(
  i_array     float[]
)
returns float[]
as
$$
-- -- declare -- here
-- --   v_x_cur   int  := 1  ;
-- --   v_y_cur   int  := 1  ;
begin
-- --   -- log(null :: float[][], float) = null :: float[][]
-- --   if array_length(i_array, array_ndims(i_array)) is null
-- --   then 
-- --     return i_array;
-- --   end if;
-- -- 
-- --   -- sm_sc.fv_swish([][], )
-- --   if array_ndims(i_array) =  2
-- --   then
-- --     while v_y_cur <= array_length(i_array, 1)
-- --     loop 
-- --       v_x_cur := 1  ;
-- --       while v_x_cur <= array_length(i_array, 2)
-- --       loop
-- --         i_array[v_y_cur][v_x_cur] := sm_sc.fv_swish(i_array[v_y_cur][v_x_cur]);
-- --         v_x_cur := v_x_cur + 1;
-- --       end loop;
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_array;
-- -- 
-- --   -- sm_sc.fv_swish [],
-- --   elsif array_ndims(i_array) = 1
-- --   then
-- --     while v_y_cur <= array_length(i_array, 1)
-- --     loop
-- --       i_array[v_y_cur] := sm_sc.fv_swish(i_array[v_y_cur]);
-- --       v_y_cur := v_y_cur + 1;
-- --     end loop;
-- --     return i_array;
-- -- 
-- --   else
-- --     raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
-- --   end if;

  return i_array /` (1.0 :: float +` (^` (-` i_array)));
end
$$
language plpgsql stable
cost 100;
-- select sm_sc.fv_activate_swish(array[[1.0 :: float, -2.0], [3.0, 4.0]])
-- select sm_sc.fv_activate_swish(array[[[1.0 :: float, -2.0], [3.0, 4.0]]])
-- select sm_sc.fv_activate_swish(array[[[[1.0 :: float, -2.0], [3.0, 4.0]]]])
-- select sm_sc.fv_activate_swish(array[1.5, -2.5, 3.5])
-- select sm_sc.fv_activate_swish(array[]::float[])
-- select sm_sc.fv_activate_swish(array[array[], array []]::float[])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_activate_tanh(float[]);
create or replace function sm_sc.fv_activate_tanh
(
  i_array     float[]
)
returns float[]
as
$$
-- declare 

begin
  -- -- -- fn(null :: double precision[][], double precision) = null :: double precision[][]
  -- -- if array_ndims(i_array) is null
  -- -- then 
  -- --   return array[] :: float[];
  -- -- end if;
  -- -- 
  -- -- if version() > 'PostgreSQL 12'
  -- -- then
  -- --   return sm_sc.fv_tanh(i_array);
  -- -- else
  -- --   -- tanh([][]) pg11及以下版本本支持 tanh 函数，需要自然指数运算
  -- --   i_array := ^` (2.0  :: float *` i_array);
  -- --   return (i_array -` 1.0 :: float) /` (i_array +` 1.0 :: float);
  -- -- end if;
  
  return sm_sc.fv_tanh(i_array);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_activate_tanh
--   (
--     array[[pi(), -pi()], [pi() / 2, pi() / 4]]::float[]
--   )
-- ;
-- select sm_sc.fv_activate_tanh
--   (
--     array[[[pi(), -pi()], [pi() / 2, pi() / 4]]]::float[]
--   )
-- ;
-- select sm_sc.fv_activate_tanh
--   (
--     array[[[[pi(), -pi()], [pi() / 2, pi() / 4]]]]::float[]
--   )
-- ;
-- select sm_sc.fv_activate_tanh
--   (
--     array[pi(), -pi(), pi() / 2, pi() / 4]::float[]
--   )
-- ;
-- drop function if exists sm_sc.ft_computational_graph_deserialize(jsonb);
create or replace function sm_sc.ft_computational_graph_deserialize
(
  i_opr_chain     jsonb
)
returns    
  table
  (
    o_out_param              varchar(64)      ,
    o_in_param               varchar(64)      ,
    o_in_value               float    ,
    o_param_loc              int              ,
    o_out_opr                varchar(64)
  )
as
$$
-- declare
begin
  return query
    with
    recursive cte_a_back_oprs as
    (
      select
        ''::varchar(64) as out_param,
        trim((select jsonb_path_query(i_opr_chain, '$.out_param')::varchar(64) limit 1), '"')::varchar(64) as in_param,
        case 
          -- 常量在 json 内取值
          when jsonb_path_exists(i_opr_chain, '$.out_value') 
            then (select jsonb_path_query(i_opr_chain, '$.out_value') limit 1)::float 
          else null::float 
        end
        as in_value,
        1::int as param_loc,
        ''::varchar(64) as out_opr,
        trim((select jsonb_path_query(i_opr_chain, '$.opr')::varchar(64) limit 1), '"')::varchar(64) as back_opr,
        jsonb_path_query_array(i_opr_chain, '$.in_params[*]') as back_params
      union all
      select 
        tb_a_forward.in_param as out_param,
        case 
          -- 参数 在 json 内取 in_name
          when jsonb_path_exists(in_param_ex, '$.out_value') or jsonb_path_exists(in_param_ex, '$.out_param')
            then trim((select jsonb_path_query(in_param_ex, '$.out_param') limit 1) ::varchar(64), '"')::varchar(64) 
          -- 中间结果 生成临时 in_name
          else md5((select jsonb_path_query(in_param_ex, '$.opr')::text limit 1) || '||' || (select jsonb_path_query(in_param_ex, '$.in_params[*]')::text limit 1))::varchar(64)
          end 
        as in_param,
        case 
          when jsonb_path_exists(in_param_ex, '$.out_value') 
            then (select jsonb_path_query(in_param_ex, '$.out_value') limit 1) 
          else null 
        end ::float 
        as in_value,
        row_number() over(partition by tb_a_forward.in_param)::int as param_loc,
        tb_a_forward.back_opr as out_opr,
        trim((select jsonb_path_query(in_param_ex, '$.opr')::varchar(64) limit 1), '"')::varchar(64) as back_opr,
        jsonb_path_query_array(in_param_ex, '$.in_params[*]') as back_params
      from cte_a_back_oprs tb_a_forward, jsonb_path_query(back_params, '$[*]') tb_a_in_params(in_param_ex)
    )
    select
      out_param                    :: varchar(64)                                   ,
      max(in_param)                :: varchar(64)                 as  in_param      ,
      max(in_value)                :: float               as  in_value      ,
      param_loc                    :: int                                           ,
      coalesce(max(out_opr), '')   :: varchar(64)                 as out_opr
    from cte_a_back_oprs
    group by out_param, param_loc;
end
$$
language plpgsql volatile
cost 100;



-- -- select pgv_set('vars', 'a_opr_chain_01', 
-- -- '
-- -- {
-- --   "out_param": "v_y_in",
-- --   "opr": "sm_sc.fv_opr_add",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "out_param": "w0_in"
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "w1_in"
-- --         },
-- --         {
-- --           "out_param": "v_x1_in"
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "w2_in"
-- --         },
-- --         {
-- --           "opr": "sm_sc.fv_opr_pow",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x1_in"
-- --             },
-- --             {
-- --               "out_value": 2
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "w3_in"
-- --         },
-- --         {
-- --           "opr": "sm_sc.fv_opr_pow",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x1_in"
-- --             },
-- --             {
-- --               "out_value": 3
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select * from sm_sc.ft_computational_graph_deserialize(pgv_get('vars', 'a_opr_chain_01', NULL::jsonb))

-- -----------------------------------------------------------------------------------------------
-- -- select pgv_set('vars', 'a_opr_chain', 
-- -- '
-- -- {
-- --   "out_param": "v_z_in",
-- --   "opr": "sm_sc.fv_opr_add",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "opr": "sm_sc.fv_opr_mul",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "out_param": "v_x_in"
-- --         },
-- --         {
-- --           "out_param": "v_y_in"
-- --         }
-- --       ]
-- --     },
-- --     {
-- --       "opr": "sm_sc.fv_opr_exp",
-- --       "in_params": 
-- --       [
-- --         {
-- --           "opr": "sm_sc.fv_opr_mul",
-- --           "in_params": 
-- --           [
-- --             {
-- --               "out_param": "v_x_in"
-- --             },
-- --             {
-- --               "out_param": "v_y_in"
-- --             }
-- --           ]
-- --         }
-- --       ]
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select
--   o_out_param                                         ,
--   max(o_in_param)    as    o_in_param                 ,
--   max(o_in_value)    as    o_in_value                 ,
--   o_param_loc                                             ,
--   max(o_out_opr)   as    o_out_opr
-- from sm_sc.ft_computational_graph_deserialize(pgv_get('vars', 'a_opr_chain', NULL::jsonb))
-- group by o_out_param, o_param_loc


-- -----------------------------------------------------------------------------------------------
-- -- select pgv_set('vars', 'a_opr_chain_6', 
-- -- '
-- -- {
-- --   "out_param": "v_y_in",
-- --   "opr": "sm_sc.fv_opr_pow",
-- --   "in_params": 
-- --   [
-- --     {
-- --       "out_param": "v_x_in"
-- --     },
-- --     {
-- --       "out_param": "v_x_in"
-- --     }
-- --   ]
-- -- }
-- -- '::jsonb);

-- select
--   o_out_param                                         ,
--   max(o_in_param)    as    o_in_param                 ,
--   max(o_in_value)    as    o_in_value                 ,
--   o_param_loc                                             ,
--   max(o_out_opr)   as    o_out_opr
-- from sm_sc.ft_computational_graph_deserialize(pgv_get('vars', 'a_opr_chain_6', NULL::jsonb))
-- group by o_out_param, o_param_loc


-- ----------------------------------------------------------------------------------------------------------------------------------------------
-- -- 清理多余括号
-- drop function if exists sm_sc.fv_clean_bracket(text);
create or replace function sm_sc.fv_clean_bracket
(
  i_opr_algebra text
)
returns text
as
$$
declare -- here
  v_rgx_bracket_var         text    :=    '(?<![a-zA-Z0-9_\.])\(([^\(\)]+?)\)'   ;
  v_rgx_bracket2_fn_vars    text    :=    '\((\([^\(\)]+?\))\)'                  ;
begin
  -- 替换以单变量为内容的括号，其中单入参函数的文法，做否定预查，不需要保留括号
  while i_opr_algebra ~ v_rgx_bracket_var
  loop
    select regexp_replace(i_opr_algebra, v_rgx_bracket_var, '\1', 'g') into i_opr_algebra;
  end loop;

  -- 替换以函数变量为内容的括号，保留一副括号
  while i_opr_algebra ~ v_rgx_bracket2_fn_vars
  loop
    select regexp_replace(i_opr_algebra, v_rgx_bracket2_fn_vars, '\1', 'g') into i_opr_algebra;
  end loop;

  return i_opr_algebra;
end
$$
language plpgsql stable
cost 100;

-- select sm_sc.fv_clean_bracket('fm(((v_a,v_b)))+((v_c))')



-- ----------------------------------------------------------------------------------------------------------------------------------------------
-- drop function if exists sm_sc.ft_computational_graph_deserialize(text);
create or replace function sm_sc.ft_computational_graph_deserialize
(
  i_opr_algebra     text
)
returns    
  table
  (
    o_out_param              varchar(64)      ,
    o_in_param               varchar(64)      ,
    o_in_value               float    ,
    o_param_loc              int              ,
    o_out_opr                varchar(64)
  )
as
$$
declare -- here
  -- 清理空白字符、首轮多余括号
  v_opr_algebra              text     :=   sm_sc.fv_clean_bracket(regexp_replace(i_opr_algebra, '\s', '', 'g'));
  v_reg_arr                  text[];
  v_cache_last_opr_algebra   text;

  -- 变量实体。
  v_rgx_var                  text     :=    '((?<![a-zA-Z0-9_\)])\-[0-9]+(\.?[0-9]*)?(?![a-zA-Z0-9_\(])|(?<![a-zA-Z0-9_\)])[0-9]+(\.?[0-9]*)?(?![a-zA-Z0-9_\(])|(?<![a-zA-Z0-9_\)])[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?(?![a-zA-Z0-9_\(]))';  -- 数字常量与变量引用名区分，用于后续分别装配value/param；负常数，做后向预查，保证六则运算符优先级高于负号
  -- 第零优先级函数名                       
  v_rgx_fn                   text     :=    '((?<![a-zA-Z0-9_])[a-zA-Z0-9_]([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?(?=\())';
  -- 第一优先级运算符      单目 ^ ^!               
  v_rgx_opr_l1               text     :=    '(?<=[a-zA-Z0-9_\.\)])(\^\!?)(?![a-zA-Z0-9_])';
  -- 第二优先级运算符      双目 ^ ^!                  
  v_rgx_opr_l2               text     :=    '(?<=[a-zA-Z0-9_\.\)])(\^\!?)(?=[a-zA-Z0-9_])'; 
  -- -- -- 第三优先级运算符    双目  *  /                                   
  -- --   v_rgx_opr_l3               text     :=    '(?<=[a-zA-Z0-9_\.\)])(\*\/)(?=[a-zA-Z0-9_])';      
  -- 第四优先级运算符      单目 -                                           
  v_rgx_opr_l4               text     :=    '(?<![a-zA-Z0-9_\.\)])(\-)(?=[a-zA-Z0-9_])';    
  -- -- -- 第五优先级运算符    双目  +  -                                        
  -- --  v_rgx_opr_l5               text     :=    '(?<=[a-zA-Z0-9_\.\)])(\+\-)(?=[a-zA-Z0-9_])'; 
  -- -- -- 第六优先级等式运算符   双目   =                                    
  -- --  v_rgx_opr_l6               text     :=    '(?<=[a-zA-Z0-9_\.\)])(\=)(?=[a-zA-Z0-9_])';     

                                             
-- 第零优先级代数式，函数式                       
declare v_obj_rgx_fn_x             text     :=    '(' || v_rgx_fn || '(\(' || v_rgx_var || '(,' || v_rgx_var || ')*' || '\))' || ')';         
-- 第一优先级运算符代数式，自然常数幂指运算
declare v_obj_rgx_algebra_l1       text     :=    '(' || v_rgx_var || v_rgx_opr_l1 || ')';
-- 第二优先级运算符代数式，幂指运算
declare v_obj_rgx_algebra_l2       text     :=    '(' || v_rgx_var || v_rgx_opr_l2 || v_rgx_var || ')';   
-- 第三优先级运算符代数式，乘除
declare v_obj_rgx_algebra_l3       text     :=    '(' || v_rgx_var || '(\/)' || v_rgx_var || '|' || v_rgx_var || '(((\*)' || v_rgx_var || ')+))'; 
-- 第四优先级运算符代数式，负运算
declare v_obj_rgx_algebra_l4       text     :=    '(' || v_rgx_opr_l4 || v_rgx_var || ')';
-- 第五优先级运算符代数式，加减
declare v_obj_rgx_algebra_l5       text     :=    '(' || v_rgx_var || '(\-)' || v_rgx_var || '|' || v_rgx_var || '(((\+)' || v_rgx_var || ')+))';  
-- 第六优先级等式表达式，等式
declare v_obj_rgx_equation         text     :=    '(' || v_rgx_var || '=' || v_rgx_var || ')';

declare v_loop_no_01               int      :=    1;
declare v_loop_no_02               int      :=    1;
declare v_sess_id                  bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32) :=    replace(gen_random_uuid()::char(36), '-', '')::char(32);
   
begin

  -- -- -- 缓存计算关系
  -- -- -- drop table if exists sm_sc._vt_fn_compu_graph_deseri__graph;
  -- -- create temp table sm_sc._vt_fn_compu_graph_deseri__graph
  -- -- (
  -- --   out_param            varchar(64)      ,
  -- --   in_param             varchar(64)      ,
  -- --   in_value             float    ,
  -- --   param_loc            int              ,
  -- --   out_opr              varchar(64)      ,
  -- --   unique (out_param, param_loc) 
  -- -- );

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 00 -- v_opr_algebra: %', v_opr_algebra;

  -- 代入化简，直至不在化简结果不再变化，或直至最简
  while (v_opr_algebra <> v_cache_last_opr_algebra or v_cache_last_opr_algebra is null)
    and (
          v_opr_algebra !~ ('^' || v_rgx_var || '$')
          or v_opr_algebra !~ ('^' || v_obj_rgx_equation || '$')
        )
    and v_loop_no_01 <= 100
  loop 
    v_loop_no_01   :=  v_loop_no_01  +  1;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 01 -- v_opr_algebra: %', v_opr_algebra;

    v_cache_last_opr_algebra := v_opr_algebra;
    -- 第零优先级优先识别函数式
    v_loop_no_02   :=  1;
    while v_opr_algebra ~ v_obj_rgx_fn_x
      and v_loop_no_02 <= 100
    loop
      v_loop_no_02   :=  v_loop_no_02  +  1;
      -- 逐一发现
      select regexp_matches(v_opr_algebra, v_obj_rgx_fn_x) into v_reg_arr;
    
      -- 解析
      if not exists (select from sm_sc._vt_fn_compu_graph_deseri__graph where out_param = md5(v_reg_arr[1]) and sess_id = v_sess_id)
      then
        insert into sm_sc._vt_fn_compu_graph_deseri__graph
        (
          sess_id          ,
          out_param         ,
          in_param          ,
          in_value          ,
          param_loc         ,
          out_opr   
        )
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when a_in_param[1] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and a_in_param[1] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then a_in_param[1] end
                                       as   in_param,
          case when a_in_param[1] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then a_in_param[1]::float end
                                       as   in_value,
          row_number() over()          as   param_loc,
          case v_reg_arr[2]
            when 'add'        then 'sm_sc.fv_opr_add'
            when 'sub'        then 'sm_sc.fv_opr_sub'
            when 'mul'        then 'sm_sc.fv_opr_mul'
            when 'div'        then 'sm_sc.fv_opr_div'
            when 'pow'        then 'sm_sc.fv_opr_pow'
            when 'power'      then 'sm_sc.fv_opr_pow'
            when 'exp'        then 'sm_sc.fv_opr_exp'
            when 'log'        then 'sm_sc.fv_opr_log'
            when 'ln'         then 'sm_sc.fv_opr_ln'
            when 'sin'        then 'sm_sc.fv_sin'
            when 'cos'        then 'sm_sc.fv_cos'
            when 'tan'        then 'sm_sc.fv_tan'
            when 'cot'        then 'sm_sc.fv_cot'
            when 'sec'        then 'sm_sc.fv_sec'
            when 'csc'        then 'sm_sc.fv_csc'
            when 'asin'       then 'sm_sc.fv_asin'
            when 'acos'       then 'sm_sc.fv_acos'
            when 'atan'       then 'sm_sc.fv_atan'
            when 'acot'       then 'sm_sc.fv_acot'
            when 'asec'       then 'sm_sc.fv_asec'
            when 'acsc'       then 'sm_sc.fv_acsc'
            when 'sinh'       then 'sm_sc.fv_sinh'
            when 'cosh'       then 'sm_sc.fv_cosh'
            when 'tanh'       then 'sm_sc.fv_tanh'
            when 'asinh'      then 'sm_sc.fv_asinh'
            when 'acosh'      then 'sm_sc.fv_acosh'
            when 'atanh'      then 'sm_sc.fv_atanh'
            else v_reg_arr[2]
          end
                                       as out_opr
        from regexp_matches(v_reg_arr[4], v_rgx_var, 'g') as tb_a_in_params(a_in_param)
        ;
      end if;
    
      -- 即以识别处理的表达式替换为中间变量，准备下次识别
      select 
        sm_sc.fv_clean_bracket
        (
          regexp_replace
          (
            v_opr_algebra
--            , v_reg_arr[1]
            , regexp_replace
              (
                v_reg_arr[1]
                , '([\+\-\*\/\.\^\!\(\)\,\:\<\>\[\]\\])'
                , '\\\1'
                , 'g'
              )
            , md5(v_reg_arr[1])
            , 'g'
          )
        ) 
        into v_opr_algebra;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 011 -- v_opr_algebra: %', v_opr_algebra;
    end loop;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 02 -- v_opr_algebra: %', v_opr_algebra;
    
    -- 第一优先识别代数式
    v_loop_no_02   :=  1;
    while v_opr_algebra ~ v_obj_rgx_algebra_l1
      and v_loop_no_02 <= 100
    loop
      v_loop_no_02   :=  v_loop_no_02  +  1;
      -- 逐一发现
      select regexp_matches(v_opr_algebra, v_obj_rgx_algebra_l1) into v_reg_arr;
    
      -- 解析
      if not exists (select from sm_sc._vt_fn_compu_graph_deseri__graph where out_param = md5(v_reg_arr[1]) and sess_id = v_sess_id)
      then
        insert into sm_sc._vt_fn_compu_graph_deseri__graph
        (
          sess_id          ,
          out_param         ,
          in_param          ,
          in_value          ,
          param_loc         ,
          out_opr   
        )
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when v_reg_arr[2] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_reg_arr[2] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[2] end
                                       as   in_param,
          case when v_reg_arr[2] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[2]::float end
                                       as   in_value,
          1                            as   param_loc,
          case v_reg_arr[6]
            when '^'        then 'sm_sc.fv_opr_exp'
            when '^!'       then 'sm_sc.fv_opr_ln'
          end
                                       as out_opr
        ;
      end if;

      -- 即以识别处理的表达式替换为中间变量，准备下次识别
      select 
        sm_sc.fv_clean_bracket
        (
          regexp_replace
          (
            v_opr_algebra
--            , v_reg_arr[1]
            , regexp_replace
              (
                v_reg_arr[1]
                , '([\+\-\*\/\.\^\!\(\)\,\:\<\>\[\]\\])'
                , '\\\1'
                , 'g'
              )
            , md5(v_reg_arr[1])
            , 'g'
          )
        ) 
        into v_opr_algebra;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 021 -- v_opr_algebra: %', v_opr_algebra;
    end loop;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 03 -- v_opr_algebra: %', v_opr_algebra;
    -- 第二优先识别代数式
    v_loop_no_02   :=  1;
    while v_opr_algebra ~ v_obj_rgx_algebra_l2
      and v_loop_no_02 <= 100
    loop
      v_loop_no_02   :=  v_loop_no_02  +  1;
      -- 逐一发现
      select regexp_matches(v_opr_algebra, v_obj_rgx_algebra_l2) into v_reg_arr;
    
      -- 解析
      if not exists (select from sm_sc._vt_fn_compu_graph_deseri__graph where out_param = md5(v_reg_arr[1]) and sess_id = v_sess_id)
      then
        insert into sm_sc._vt_fn_compu_graph_deseri__graph
        (
          sess_id          ,
          out_param         ,
          in_param          ,
          in_value          ,
          param_loc         ,
          out_opr   
        )
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when v_reg_arr[2] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_reg_arr[2] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[2] end
                                       as   in_param,
          case when v_reg_arr[2] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[2]::float end
                                       as   in_value,
          1                            as   param_loc,
          case v_reg_arr[6]
            when '^'        then 'sm_sc.fv_opr_pow'
            when '^!'       then 'sm_sc.fv_opr_log'
          end
                                       as out_opr
        union all
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when v_reg_arr[7] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_reg_arr[7] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[7] end
                                       as   in_param,
          case when v_reg_arr[7] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[7]::float end
                                       as   in_value,
          2                            as   param_loc,
          case v_reg_arr[6]
            when '^'        then 'sm_sc.fv_opr_pow'
            when '^!'       then 'sm_sc.fv_opr_log'
          end
                                       as out_opr
        ;
      end if;
    
      -- 即以识别处理的表达式替换为中间变量，准备下次识别
      select 
        sm_sc.fv_clean_bracket
        (
          regexp_replace
          (
            v_opr_algebra
--            , v_reg_arr[1]
            , regexp_replace
              (
                v_reg_arr[1]
                , '([\+\-\*\/\.\^\!\(\)\,\:\<\>\[\]\\])'
                , '\\\1'
                , 'g'
              )
            , md5(v_reg_arr[1])
            , 'g'
          )
        ) 
        into v_opr_algebra;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 031 -- v_opr_algebra: %', v_opr_algebra;
    end loop;
 
-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 04 -- v_opr_algebra: %', v_opr_algebra;   
    -- 第三优先识别代数式
    v_loop_no_02   :=  1;
    while v_opr_algebra ~ v_obj_rgx_algebra_l3
      and v_loop_no_02 <= 100
    loop
      v_loop_no_02   :=  v_loop_no_02  +  1;
      -- 逐一发现
      select regexp_matches(v_opr_algebra, v_obj_rgx_algebra_l3) into v_reg_arr;
    
      -- 解析
      if not exists (select from sm_sc._vt_fn_compu_graph_deseri__graph where out_param = md5(v_reg_arr[1]) and sess_id = v_sess_id)
      then
        insert into sm_sc._vt_fn_compu_graph_deseri__graph
        (
          sess_id          ,
          out_param         ,
          in_param          ,
          in_value          ,
          param_loc         ,
          out_opr   
        )
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when coalesce(v_reg_arr[2], v_reg_arr[11]) ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and coalesce(v_reg_arr[2], v_reg_arr[11]) !~ '^\-?[0-9]+(\.?[0-9]*)?$' then coalesce(v_reg_arr[2], v_reg_arr[11]) end
                                       as   in_param,
          case when coalesce(v_reg_arr[2], v_reg_arr[11]) ~ '^\-?[0-9]+(\.?[0-9]*)?$' then coalesce(v_reg_arr[2], v_reg_arr[11])::float end
                                       as   in_value,
          1                            as   param_loc,
          case coalesce(v_reg_arr[6], v_reg_arr[17])
            when '*'        then 'sm_sc.fv_opr_mul'
            when '/'        then 'sm_sc.fv_opr_div'
          end
                                       as out_opr
        union all
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when v_reg_arr[7] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_reg_arr[7] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[7] end
                                       as   in_param,
          case when v_reg_arr[7] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[7]::float end
                                       as   in_value,
          2                            as   param_loc,
          'sm_sc.fv_opr_div'         as   out_opr
        where v_reg_arr[7] is not null
        union all
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when a_in_param[1] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and a_in_param[1] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then a_in_param[1] end
                                       as   in_param,
          case when a_in_param[1] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then a_in_param[1]::float end
                                       as   in_value,
          row_number() over () + 1     as   param_loc,
          'sm_sc.fv_opr_mul'         as   out_opr
        from regexp_matches(v_reg_arr[15], v_rgx_var, 'g')   tb_a(a_in_param)
        where v_reg_arr[15] is not null
        ;
      end if;
    
      -- 即以识别处理的表达式替换为中间变量，准备下次识别
      select 
        sm_sc.fv_clean_bracket
        (
          regexp_replace
          (
            v_opr_algebra
--            , v_reg_arr[1]
            , regexp_replace
              (
                v_reg_arr[1]
                , '([\+\-\*\/\.\^\!\(\)\,\:\<\>\[\]\\])'
                , '\\\1'
                , 'g'
              )
            , md5(v_reg_arr[1])
            , 'g'
          )
        ) 
        into v_opr_algebra;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 041 -- v_reg_arr[1]: %', v_reg_arr[1];
-- -- -- -- raise notice 'debug -- 041 -- v_opr_algebra: %', v_opr_algebra;
    end loop;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 05 -- v_opr_algebra: %', v_opr_algebra;
    -- 第四优先识别代数式
    v_loop_no_02   :=  1;
    while v_opr_algebra ~ v_obj_rgx_algebra_l4
      and v_loop_no_02 <= 100
    loop
      v_loop_no_02   :=  v_loop_no_02  +  1;
      -- 逐一发现
      select regexp_matches(v_opr_algebra, v_obj_rgx_algebra_l4) into v_reg_arr;

      -- 解析
      if not exists (select from sm_sc._vt_fn_compu_graph_deseri__graph where out_param = md5(v_reg_arr[1]) and sess_id = v_sess_id)
      then
        insert into sm_sc._vt_fn_compu_graph_deseri__graph
        (
          sess_id          ,
          out_param         ,
          in_param          ,
          in_value          ,
          param_loc         ,
          out_opr   
        )
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param  ,
          null                         as   in_param   ,
          0.0                          as   in_value   ,
          1                            as   param_loc  ,
          'sm_sc.fv_opr_sub'         as   out_opr
        union all
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param  ,
          v_reg_arr[3]                 as   in_param   ,
          null                         as   in_value   ,
          2                            as   param_loc  ,
          'sm_sc.fv_opr_sub'         as   out_opr
        ;
      end if;

      -- 即以识别处理的表达式替换为中间变量，准备下次识别
      select 
        sm_sc.fv_clean_bracket
        (
          regexp_replace
          (
            v_opr_algebra
--            , v_reg_arr[1]
            , regexp_replace
              (
                v_reg_arr[1]
                , '([\+\-\*\/\.\^\!\(\)\,\:\<\>\[\]\\])'
                , '\\\1'
                , 'g'
              )
            , md5(v_reg_arr[1])
            , 'g'
          )
        ) 
        into v_opr_algebra;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 051 -- v_opr_algebra: %', v_opr_algebra;
    end loop; 

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 06 -- v_opr_algebra: %', v_opr_algebra;
    -- 第五优先识别代数式
    v_loop_no_02   :=  1;
    while v_opr_algebra ~ v_obj_rgx_algebra_l5
      and v_loop_no_02 <= 100
    loop
      v_loop_no_02   :=  v_loop_no_02  +  1;
      -- 逐一发现
      select regexp_matches(v_opr_algebra, v_obj_rgx_algebra_l5) into v_reg_arr;
    
      -- 解析
      if not exists (select from sm_sc._vt_fn_compu_graph_deseri__graph where out_param = md5(v_reg_arr[1]) and sess_id = v_sess_id)
      then
        insert into sm_sc._vt_fn_compu_graph_deseri__graph
        (
          sess_id          ,
          out_param         ,
          in_param          ,
          in_value          ,
          param_loc         ,
          out_opr   
        )
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when coalesce(v_reg_arr[2], v_reg_arr[11]) ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and coalesce(v_reg_arr[2], v_reg_arr[11]) !~ '^\-?[0-9]+(\.?[0-9]*)?$' then coalesce(v_reg_arr[2], v_reg_arr[11]) end
                                       as   in_param,
          case when coalesce(v_reg_arr[2], v_reg_arr[11]) ~ '^\-?[0-9]+(\.?[0-9]*)?$' then coalesce(v_reg_arr[2], v_reg_arr[11])::float end
                                       as   in_value,
          1                            as   param_loc,
          case coalesce(v_reg_arr[6], v_reg_arr[17])
            when '+'        then 'sm_sc.fv_opr_add'
            when '-'        then 'sm_sc.fv_opr_sub'
          end
                                       as   out_opr
        union all
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when v_reg_arr[7] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_reg_arr[7] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[7] end
                                       as   in_param,
          case when v_reg_arr[7] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[7]::float end
                                       as   in_value,
          2                            as   param_loc,
          'sm_sc.fv_opr_sub'         as   out_opr
        where v_reg_arr[7] is not null
        union all
        select 
          v_sess_id,
          md5(v_reg_arr[1])            as   out_param,
          case when a_in_param[1] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and a_in_param[1] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then a_in_param[1] end
                                       as   in_param,
          case when a_in_param[1] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then a_in_param[1]::float end
                                       as   in_value,
          row_number() over () + 1     as   param_loc,
          'sm_sc.fv_opr_add'         as   out_opr
        from regexp_matches(v_reg_arr[15], v_rgx_var, 'g')   tb_a(a_in_param)
        where v_reg_arr[15] is not null
        ;
      end if;
    
      -- 即以识别处理的表达式替换为中间变量，准备下次识别
      select 
        sm_sc.fv_clean_bracket
        (
          regexp_replace
          (
            v_opr_algebra
--            , v_reg_arr[1]
            , regexp_replace
              (
                v_reg_arr[1]
                , '([\+\-\*\/\.\^\!\(\)\,\:\<\>\[\]\\])'
                , '\\\1'
                , 'g'
              )
            , md5(v_reg_arr[1])
            , 'g'
          )
        ) 
        into v_opr_algebra;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 061 -- v_reg_arr[1]: %', v_reg_arr[1];
-- -- -- -- raise notice 'debug -- 061 -- v_opr_algebra: %', v_opr_algebra;
    end loop;
    
  end loop;

-- -- -- -- -- debug
-- -- -- -- raise notice 'debug -- 07 -- v_opr_algebra: %', v_opr_algebra;
  -- 第六优先识别等式左侧的因变量关联关系
  if v_opr_algebra ~ ('^' || v_obj_rgx_equation || '$')
  then
    select regexp_matches(v_opr_algebra, v_obj_rgx_equation) into v_reg_arr;
    -- 解析
    insert into sm_sc._vt_fn_compu_graph_deseri__graph
    (
      sess_id          ,
      out_param         ,
      in_param          ,
      in_value          ,
      param_loc         ,
      out_opr   
    )
    select 
      v_sess_id,
      case when v_reg_arr[6] like '-%' then md5(v_reg_arr[6]) else v_reg_arr[2]::varchar(64) end  
                                   as   out_param,
      case when v_reg_arr[6] ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_reg_arr[6] !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[6] end
                                   as   in_param,
      case when v_reg_arr[6] ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_reg_arr[6]::float end
                                   as   in_value,
      1                            as   param_loc,
      ''  ::varchar(64)            as   out_opr
    ;    
  -- 最简中间变量而没有等式的情况
  elsif v_opr_algebra ~ ('^' || v_rgx_var || '$')
  then
    insert into sm_sc._vt_fn_compu_graph_deseri__graph
    (
      sess_id          ,
      out_param         ,
      in_param          ,
      in_value          ,
      param_loc         ,
      out_opr   
    )
    select 
      v_sess_id,
      ''::varchar(64)              as   out_param,
      case when v_opr_algebra ~ '^[a-zA-Z0-9_]+([a-zA-Z0-9_\.]*[a-zA-Z0-9_])?$' and v_opr_algebra !~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_opr_algebra end
                                   as   in_param,
      case when v_opr_algebra ~ '^\-?[0-9]+(\.?[0-9]*)?$' then v_opr_algebra::float end
                                   as   in_value,
      1                            as   param_loc,
      ''  ::varchar(64)            as   out_opr
    ;
  -- 无法化简为最简，解析失败
  else
    raise exception 'unknown algebra!';
  end if;

  return query
    select 
      out_param    ,
      in_param     ,
      in_value     ,
      param_loc    ,
      out_opr
    from sm_sc._vt_fn_compu_graph_deseri__graph
    where sess_id = v_sess_id
  ;

  -- -- drop table if exists sm_sc._vt_fn_compu_graph_deseri__graph;
  delete from sm_sc._vt_fn_compu_graph_deseri__graph
  where sess_id = v_sess_id;
end
$$
language plpgsql volatile
cost 100;

-- -- set force_parallel_mode = 'off';
-- select * from sm_sc.ft_computational_graph_deserialize('v_y_in = w1 * v_x1_in + w2 * v_x2 + w3 * v_x3')
-- select * from sm_sc.ft_computational_graph_deserialize('v_z_in = (v_x_in * v_y) + exp(v_x_in * v_y) + power(v_x_in, 2)')
-- -- 卷积的矩阵乘法实现：
-- --   https://www.cnblogs.com/shine-lee/p/10775831.html
-- --   https://blog.csdn.net/qq_40263477/article/details/104979609
-- -- 卷积的 fft 实现
-- --   https://blog.csdn.net/qq_37527608/article/details/120922348
-- -- 卷积与矩阵乘法算法调优：
-- --  https://blog.csdn.net/lizhengx/article/details/83246833
-- -- 卷积求导
-- --  https://blog.csdn.net/wangjianyu0115/article/details/62222301

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex(float[][], float[][], int[2], int[4]);
create or replace function sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
(
  -- i_array_len          int[2]                                               ,  -- 原矩阵(卷积第一目参数)高宽，
  i_dloss_ddepdt           float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵
  i_window             float[]                                     ,  -- 卷积核窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]          -- 上下左右补齐行数/列数
)
returns float[][]
as
$$
declare
  -- -- v_dloss_dindepdt               float[];     --   intact 之后的新矩阵
  -- -- v_len_y                  int             :=   coalesce(i_padding[1], 0) + i_array_len[1] + coalesce(i_padding[2], 0);     --   新矩阵高
  -- -- v_len_x                  int             :=   coalesce(i_padding[3], 0) + i_array_len[2] + coalesce(i_padding[4], 0);     --   新矩阵宽
  -- -- v_cur_y                  int             ;     --   新矩阵高游标
  -- -- v_cur_x                  int             ;     --   新矩阵宽游标
  v_windows_cnt_reciproca  float  :=   1.0 :: float/ (array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) * array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)));     -- 开窗数量的倒数
  -- -- v_window_len_y           int             :=   array_length(i_window, array_ndims(i_window) - 1);
  -- -- v_window_len_x           int             :=   array_length(i_window, array_ndims(i_window));
  v_return                 float[]  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度
    if array_ndims(i_dloss_ddepdt) > 4
    then 
      raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) - array_ndims(i_window) = 2
  then 
    i_window = array[[i_window]];
  elsif array_ndims(i_dloss_ddepdt) - array_ndims(i_window) = 1
  then 
    i_window = array[i_window];
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    v_return :=  
      sm_sc.fv_conv_2d_im2col_py
      (
        array_fill(0.0 :: float, array[array_length(i_window, 1) - 1, 1])  -- 上方填零扩展
        |-||
        (
          sm_sc.fv_sample_y
          (
            array_fill(0.0 :: float, array[1, array_length(i_window, 2) - 1])  -- 左侧填零扩展
            ||||
            (
              sm_sc.fv_sample_x
              (
                i_dloss_ddepdt
                , 1
                , 1     
                , array[int4range(1, i_stride[2], '[]')] -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
                , array[array[0.0 :: float]]
              )
            )[ : ][ : (array_length(i_dloss_ddepdt, 2) - 1) * i_stride[2] + 1]     -- 结尾稀释截断，相当于仅在原矩阵间隙周期插值
            ||||
            array_fill(0.0 :: float, array[1, array_length(i_window, 2) - 1])  -- 右侧填零扩展
            , 1
            , 1     
            , array[int4range(1, i_stride[1], '[]')]  -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
            , array[array[0.0 :: float]]
          )
        )[ : (array_length(i_dloss_ddepdt, 1) - 1) * i_stride[1] + 1][ : ]
        |-||
        array_fill(0.0 :: float, array[array_length(i_window, 1) - 1, 1])   -- 下方填零扩展
        , |~~| i_window     -- i_window_back
        -- -- , 0.0         -- i_bias_back
        -- -- , array[1, 1] -- i_stride_back
      )
    ;
    
    return 
    (
      v_return[1 + i_padding[1] : array_length(v_return, 1) - i_padding[2]][1 + i_padding[3] : array_length(v_return, 2) - i_padding[4]]
      *` v_windows_cnt_reciproca
    ) :: float[]
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_return :=  
      sm_sc.fv_conv_2d_im2col_py
      (
        array_fill(0.0 :: float, array[1, array_length(i_window, 2) - 1, 1])  -- 上方填零扩展
        |-||
        (
          sm_sc.fv_sample_x
          (
            array_fill(0.0 :: float, array[1, 1, array_length(i_window, 3) - 1])  -- 左侧填零扩展
            ||||
            (
              sm_sc.fv_sample_x3
              (
                i_dloss_ddepdt
                , 1
                , 1     
                , array[int4range(1, i_stride[2], '[]')] -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
                , array[[[0.0 :: float]]]
              )
            )[ : ][ : (array_length(i_dloss_ddepdt, 3) - 1) * i_stride[2] + 1]     -- 结尾稀释截断，相当于仅在原矩阵间隙周期插值
            ||||
            array_fill(0.0 :: float, array[1, 1, array_length(i_window, 3) - 1])  -- 右侧填零扩展
            , 1
            , 1     
            , array[int4range(1, i_stride[1], '[]')]  -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
            , array[[[0.0 :: float]]]
          )
        )[ : ][ : (array_length(i_dloss_ddepdt, 2) - 1) * i_stride[1] + 1][ : ]
        |-||
        array_fill(0.0 :: float, array[1, array_length(i_window, 2) - 1, 1])   -- 下方填零扩展
        , |~~| i_window     -- i_window_back
        -- -- , 0.0         -- i_bias_back
        -- -- , array[1, 1] -- i_stride_back
      )
    ;
    
    return 
    (
      v_return[ : ][1 + i_padding[1] : array_length(v_return, 2) - i_padding[2]][1 + i_padding[3] : array_length(v_return, 3) - i_padding[4]]
      *` v_windows_cnt_reciproca
    ) :: float[]
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_return :=  
      sm_sc.fv_conv_2d_im2col_py
      (
        array_fill(0.0 :: float, array[1, 1, array_length(i_window, 3) - 1, 1])  -- 上方填零扩展
        |-||
        (
          sm_sc.fv_sample_x3
          (
            array_fill(0.0 :: float, array[1, 1, 1, array_length(i_window, 4) - 1])  -- 左侧填零扩展
            ||||
            (
              sm_sc.fv_sample_x4
              (
                i_dloss_ddepdt
                , 1
                , 1     
                , array[int4range(1, i_stride[2], '[]')] -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
                , array[[[[0.0 :: float]]]]
              )
            )[ : ][ : (array_length(i_dloss_ddepdt, 4) - 1) * i_stride[2] + 1]     -- 结尾稀释截断，相当于仅在原矩阵间隙周期插值
            ||||
            array_fill(0.0 :: float, array[1, 1, 1, array_length(i_window, 4) - 1])  -- 右侧填零扩展
            , 1
            , 1     
            , array[int4range(1, i_stride[1], '[]')]  -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
            , array[[[[0.0 :: float]]]]
          )
        )[ : ][ : ][ : (array_length(i_dloss_ddepdt, 3) - 1) * i_stride[1] + 1][ : ]
        |-||
        array_fill(0.0 :: float, array[1, 1, array_length(i_window, 3) - 1, 1])   -- 下方填零扩展
        , |~~| i_window     -- i_window_back
        -- -- , 0.0         -- i_bias_back
        -- -- , array[1, 1] -- i_stride_back
      )
    ;
    
    return 
    (
      v_return[ : ][ : ][1 + i_padding[1] : array_length(v_return, 3) - i_padding[2]][1 + i_padding[3] : array_length(v_return, 4) - i_padding[4]]
      *` v_windows_cnt_reciproca
    ) :: float[]
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
--   (
--    -- array[5, 7],
--    array[[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--    , array[[0.5 :: float, 1.1, 0.5], [1.1, 2.1, 1.1], [0.5 :: float, 1.1, 0.5]]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
--   (
--    -- array[5, 6],
--    array[[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--    , array[[0.5 :: float, 1.1, 0.5], [1.1, 2.1, 1.1], [0.5 :: float, 1.1, 0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
--   (
--    -- array[2, 5, 6],
--    array
--    [
--         [[1.1, 1.2, 1.1]
--         ,[2.1, 3.1, 4.1]
--         ,[-1.1, 0.1, 3.1]
--         ],
--         [[1.1, 1.3, 1.1]
--         ,[1.7, 1.1, 2.1]
--         ,[-1.1, 1.6, 1.1]
--         ]
--    ]
--    , array[[0.5 :: float, 1.1, 0.5], [1.1, 2.1, 1.1], [0.5 :: float, 1.1, 0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
--   (
--    -- array[2, 2, 5, 6],
--    array
--    [
--      [
--         [[1.1, 1.2, 1.1]
--         ,[2.1, 3.1, 4.1]
--         ,[-1.1, 0.1, 3.1]
--         ],
--         [[1.1, 1.3, 1.1]
--         ,[1.7, 1.1, 2.1]
--         ,[-1.1, 1.6, 1.1]
--         ]
--      ],
--      [
--         [[-1.1, 1.2, 1.1]
--         ,[2.1, -3.1, 4.1]
--         ,[-1.1, 0.1, -3.1]
--         ],
--         [[1.1, -1.3, 1.1]
--         ,[-1.7, 1.1, 2.1]
--         ,[1.1, -1.6, 1.1]
--         ]
--      ]
--    ]
--    , array[[0.5 :: float, 1.1, 0.5], [1.1, 2.1, 1.1], [0.5 :: float, 1.1, 0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
--   (
--    -- array[2, 2, 5, 6],
--    array
--    [
--      [
--         [[1.1, 1.2, 1.1]
--         ,[2.1, 3.1, 4.1]
--         ,[-1.1, 0.1, 3.1]
--         ],
--         [[1.1, 1.3, 1.1]
--         ,[1.7, 1.1, 2.1]
--         ,[-1.1, 1.6, 1.1]
--         ]
--      ],
--      [
--         [[-1.1, 1.2, 1.1]
--         ,[2.1, -3.1, 4.1]
--         ,[-1.1, 0.1, -3.1]
--         ],
--         [[1.1, -1.3, 1.1]
--         ,[-1.7, 1.1, 2.1]
--         ,[1.1, -1.6, 1.1]
--         ]
--      ]
--    ]
--    , array
--      [
--        [
--          [[-0.5, 1.1, 0.5], [-1.1, 2.1, 1.1], [-0.5, 1.1, 0.5]]
--        , [[0.5, 1.1, -0.5], [1.1, 2.1, -1.1], [0.5, 1.1, -0.5]]
--        ]
--       , [
--          [[0.5, -1.1, 0.5], [1.1, -2.1, 1.1], [0.5, -1.1, 0.5]]
--        , [[0.5, -1.1, 0.5], [-1.1, 2.1, -1.1], [0.5, -1.1, 0.5]]
--        ]
--      ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   ) :: decimal[] ~=` 3;
-- -- -- -- 卷积的矩阵乘法实现：
-- -- -- --   https://www.cnblogs.com/shine-lee/p/10775831.html
-- -- -- --   https://blog.csdn.net/qq_40263477/article/details/104979609
-- -- 卷积的 fft 实现
-- --   https://blog.csdn.net/qq_37527608/article/details/120922348
-- -- 卷积与矩阵乘法算法调优：
-- --      -- --  https://blog.csdn.net/lizhengx/article/details/83246833
-- --      -- -- 卷积求导
-- --      -- --  https://blog.csdn.net/wangjianyu0115/article/details/62222301
-- --      
-- --      -- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_2d_dloss_dindepdt_2(float[], float[], int[], int[2], int[4], float);
create or replace function sm_sc.fv_d_conv_2d_dloss_dindepdt_2
(
  i_array              float[]                                     ,  -- 原矩阵(卷积第一目参数)
  i_dloss_ddepdt       float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵
  i_window_len         int[]                                               ,  -- 卷积核窗口矩阵规格，如果是三维、四维，那么长度为三维或四维
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float      default  0.0                        -- 补齐填充元素值
)
returns float[]
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_dloss_ddepdt_len_heigh int   :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) ;
  v_dloss_ddepdt_len_width int   :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
  v_window_len             int[2] :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
  v_ret                    float[];
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4) or array_ndims(i_dloss_ddepdt) not in (2, 3, 4) or array_ndims(i_dloss_ddepdt) <> array_ndims(i_array)
    then 
      raise exception 'unsupport ndims of i_array and i_dloss_ddepdt.';
    elsif array_ndims(i_dloss_ddepdt) = 3 and array_length(i_dloss_ddepdt, 1) <> array_length(i_array, 1)
      or (array_ndims(i_dloss_ddepdt) = 4 and (array_length(i_dloss_ddepdt, 1) <> array_length(i_array, 1) or array_length(i_dloss_ddepdt, 2) <> array_length(i_array, 2)))
    then 
      raise exception 'unmatch length at 1d of 3d or 1d / 2d of 4d';
    elsif (v_array_len_heigh_ex - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (v_array_len_width_ex - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    elsif v_dloss_ddepdt_len_heigh <> (v_array_len_heigh_ex - v_window_len[1]) / i_stride[1] + 1
      or v_dloss_ddepdt_len_width <> (v_array_len_width_ex - v_window_len[2]) / i_stride[2] + 1
    then
      raise exception 'unmatch length between y and dloss/dy.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
      |~~|
      (
        select 
          sm_sc.fa_mx_sum
          (
            i_array[col_a_y : col_a_y + v_window_len[1] - 1][col_a_x : col_a_x + v_window_len[2] - 1] 
              *` i_dloss_ddepdt[(col_a_y - 1) / i_stride[1] + 1][(col_a_x - 1) / i_stride[2] + 1]
          )
        from generate_series(1, v_array_len_heigh_ex - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
      );
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
    (
      select 
        array_agg -- sm_sc.fa_mx_sum 
        (
          sm_sc.fv_d_conv_2d_dloss_dindepdt_2
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , sm_sc.fv_mx_slice_3d_2_2d
            (
              i_dloss_ddepdt[a_cur_y : a_cur_y]
            , 1
            )
          , v_window_len     
          , i_stride           
          , i_padding       
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_dloss_ddepdt, 1)) tb_a_cur_y(a_cur_y)
    );
    if array_length(i_window_len, 1) = 2
    then 
      v_ret := 
        sm_sc.fv_mx_slice_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum(v_ret, array[array_length(v_ret, 1), 1, 1])
        , 1
        );
    end if;
    return v_ret;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
    (
      with 
      cte_agg_x as 
      (
        select 
          a_cur_y, 
          array_agg    -- sm_sc.fa_mx_sum 
          (
            sm_sc.fv_d_conv_2d_dloss_dindepdt_2
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , sm_sc.fv_mx_slice_4d_2_2d
              (
                i_dloss_ddepdt[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , v_window_len     
            , i_stride           
            , i_padding       
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_dloss_ddepdt, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_dloss_ddepdt, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    if array_length(i_window_len, 1) = 2
    then 
      v_ret := 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum(v_ret, array[array_length(v_ret, 1), array_length(v_ret, 2), 1, 1])
        , array[1, 2]
        , array[1, 1]
        );
    end if;
    return v_ret;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ] :: float[]
--    , array[[1.1, 1.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[3, 3]
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ] :: float[]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2
--   (
--     array
--       [
--         [
--           [1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [-1,2,-3,4,5,6]
--         , [10,-20,30,40,50,-60]
--         , [100,200,-300,400,500,600]
--         , [-1,2,-3,-4,-5,-6]
--         , [-10,-20,30,-40,50,-60]
--         ]
--       ]
--    , array[[[1.1, 1.1, -1.1], [1.1, -2.1, 1.1], [-1.1, 1.1, 1.1]],[[-1.1, 1.1, 1.1], [1.1, -2.1, 1.1], [1.1, 1.1, -1.1]]]
--    , array[3, 3]     --  array[2, 3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2
--   (
--    array
--    [
--      [
--        [
--          [1,2,3,4,5,6]
--        , [10,20,30,40,50,60]
--        , [100,200,300,400,500,600]
--        , [-1,-2,-3,-4,-5,-6]
--        , [-10,-20,-30,-40,-50,-60]
--        ]
--      , [
--          [-1,2,-3,4,5,6]
--        , [10,-20,30,40,50,-60]
--        , [100,200,-300,400,500,600]
--        , [-1,2,-3,-4,-5,-6]
--        , [-10,-20,30,-40,50,-60]
--        ]
--      ]
--    , [
--        [
--          [1,2,3,-4,5,6]
--        , [10,20,30,40,50,60]
--        , [100,-200,300,400,500,600]
--        , [-1,2,-3,-4,5,-6]
--        , [10,20,-30,-40,-50,60]
--        ]
--      , [
--          [1,2,-3,-4,5,6]
--        , [10,-20,-30,40,50,60]
--        , [100,200,-300,-400,-500,600]
--        , [-1,2,-3,-4,5,-6]
--        , [-10,-20,30,-40,50,-60]
--        ]
--      ]
--    ]  
--    , array
--      [
--        [[[1.1, 1.1, -1.1], [-1.1, -2.1, -1.1], [-1.1, 1.1, 1.1]]
--        ,[[-1.1, 1.1, 1.1], [1.1, -2.1, 1.1], [1.1, 1.1, -1.1]]]
--      , [[[1.1, 1.1, -1.1], [1.1, -2.1, 1.1], [-1.1, 1.1, 1.1]]
--        ,[[-1.1, 1.1, 1.1], [-1.1, -2.1, -1.1], [1.1, 1.1, -1.1]]]
--      ]
--    , array[3, 3]    -- array[2, 2, 3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   ) :: decimal[] ~=` 3;
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py(float[], float[], int[], int[2], int[4], float);
create or replace function sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
(
  i_array              float[]                                                -- 原矩阵(卷积第一目参数)
, i_dloss_ddepdt       float[]                                                -- 即已求出的损失函数对 y 的导数矩阵
, i_window_len         int[]                                                  -- 卷积核窗口矩阵规格，如果是三维、四维，那么长度为三维或四维
, i_stride             int[2]              default  array[1, 1]               -- 纵向与横向步长
, i_padding            int[4]              default  array[0, 0, 0, 0]         -- 上下左右补齐行数/列数
, i_padding_value      float      default  0.0                                -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_dloss_ddepdt    = np.float64(i_dloss_ddepdt)
  v_background      = np.float64(i_array)
  v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :
    return \
    ( \
    np.sum( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_dloss_ddepdt.shape[-2], v_dloss_ddepdt.shape[-1], v_window_len[-2], v_window_len[-1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .reshape(v_dloss_ddepdt.shape[-2] * v_dloss_ddepdt.shape[-1], v_window_len[-2], v_window_len[-1]) \
    * v_dloss_ddepdt.reshape(v_dloss_ddepdt.shape[-2] * v_dloss_ddepdt.shape[-1], 1, 1) \
    , axis=0) \
    )[::-1, ::-1].tolist()
  
  elif v_background.ndim == 3 :
    v_return = \
    np.sum( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_dloss_ddepdt.shape[-2], v_dloss_ddepdt.shape[-1], v_window_len[-2], v_window_len[-1]) \
    , strides=(v_ele_len * v_background.shape[1] * v_background.shape[2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .reshape(v_background.shape[0], v_dloss_ddepdt.shape[-2] * v_dloss_ddepdt.shape[-1], v_window_len[-2], v_window_len[-1]) \
    * v_dloss_ddepdt.reshape(v_dloss_ddepdt.shape[0], v_dloss_ddepdt.shape[-2] * v_dloss_ddepdt.shape[-1], 1, 1) \
    , axis=0)
    
    if v_window_len.size == 2 :
      return np.sum(v_return, axis=0)[::-1, ::-1].tolist()
    else : # v_window_len.size == 3
      return v_return[::, ::-1, ::-1].tolist()

  
  elif v_background.ndim == 4 :
    v_return = \
    np.sum( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], v_dloss_ddepdt.shape[-2], v_dloss_ddepdt.shape[-1], v_window_len[-2], v_window_len[-1]) \
    , strides=(v_ele_len * v_background.shape[1] * v_background.shape[2] * v_background.shape[3], v_ele_len * v_background.shape[2] * v_background.shape[3], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .reshape(v_background.shape[0], v_background.shape[1], v_dloss_ddepdt.shape[-2] * v_dloss_ddepdt.shape[-1], v_window_len[-2], v_window_len[-1]) \
    * v_dloss_ddepdt.reshape(v_dloss_ddepdt.shape[0], v_dloss_ddepdt.shape[1], v_dloss_ddepdt.shape[-2] * v_dloss_ddepdt.shape[-1], 1, 1) \
    , axis=0)
    
    if v_window_len.size == 2 :
      return np.sum(np.sum(v_return, axis=0), axis=0)[::-1, ::-1].tolist()
    elif v_window_len.size == 3 :
      return np.sum(v_return, axis=0)[::, ::-1, ::-1].tolist()
    else :   # v_window_len.size == 4
      return v_return[::, ::, ::-1, ::-1].tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ] :: float[]
--    , array[[1.1, 1.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[3, 3]
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ] :: float[]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
--   (
--     array
--       [
--         [
--           [1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [-1,2,-3,4,5,6]
--         , [10,-20,30,40,50,-60]
--         , [100,200,-300,400,500,600]
--         , [-1,2,-3,-4,-5,-6]
--         , [-10,-20,30,-40,50,-60]
--         ]
--       ]
--    , array[[[1.1, 1.1, -1.1], [1.1, -2.1, 1.1], [-1.1, 1.1, 1.1]],[[-1.1, 1.1, 1.1], [1.1, -2.1, 1.1], [1.1, 1.1, -1.1]]]
--    , array[3, 3]     --  array[2, 3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
--   (
--    array
--    [
--      [
--        [
--          [1,2,3,4,5,6]
--        , [10,20,30,40,50,60]
--        , [100,200,300,400,500,600]
--        , [-1,-2,-3,-4,-5,-6]
--        , [-10,-20,-30,-40,-50,-60]
--        ]
--      , [
--          [-1,2,-3,4,5,6]
--        , [10,-20,30,40,50,-60]
--        , [100,200,-300,400,500,600]
--        , [-1,2,-3,-4,-5,-6]
--        , [-10,-20,30,-40,50,-60]
--        ]
--      ]
--    , [
--        [
--          [1,2,3,-4,5,6]
--        , [10,20,30,40,50,60]
--        , [100,-200,300,400,500,600]
--        , [-1,2,-3,-4,5,-6]
--        , [10,20,-30,-40,-50,60]
--        ]
--      , [
--          [1,2,-3,-4,5,6]
--        , [10,-20,-30,40,50,60]
--        , [100,200,-300,-400,-500,600]
--        , [-1,2,-3,-4,5,-6]
--        , [-10,-20,30,-40,50,-60]
--        ]
--      ]
--    ]  
--    , array
--      [
--        [[[1.1, 1.1, -1.1], [-1.1, -2.1, -1.1], [-1.1, 1.1, 1.1]]
--        ,[[-1.1, 1.1, 1.1], [1.1, -2.1, 1.1], [1.1, 1.1, -1.1]]]
--      , [[[1.1, 1.1, -1.1], [1.1, -2.1, 1.1], [-1.1, 1.1, 1.1]]
--        ,[[-1.1, 1.1, 1.1], [-1.1, -2.1, -1.1], [1.1, 1.1, -1.1]]]
--      ]
--    , array[3, 3]    -- array[2, 2, 3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   ) :: decimal[] ~=` 3;
-- -- 对偏置卷积的偏移量求导
-- drop function if exists sm_sc.fv_d_conv_2d_dloss_dindepdt_3(float[], int[]);
create or replace function sm_sc.fv_d_conv_2d_dloss_dindepdt_3
(
  i_dloss_ddepdt           float[]           ,                          -- 即已求出的损失函数对 y 的导数矩阵
  i_indepdt_var_len        int[]   default array[1, 1]
)
returns float[]
as
$$
-- declare 
begin
  if array_length(i_indepdt_var_len, 1) = 2
  then 
    return array[array[sm_sc.fv_aggr_slice_sum(i_dloss_ddepdt)]];
    
  elsif array_length(i_indepdt_var_len, 1) = 3
  then 
    return sm_sc.fv_aggr_slice_sum(i_dloss_ddepdt, array[1, array_length(i_dloss_ddepdt, 2), array_length(i_dloss_ddepdt, 3)]);
    
  elsif array_length(i_indepdt_var_len, 1) = 4
  then 
    return sm_sc.fv_aggr_slice_sum(i_dloss_ddepdt, array[1, 1, array_length(i_dloss_ddepdt, 3), array_length(i_dloss_ddepdt, 4)]);
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_3
--   (
--     array
--     [
--       [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--     , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--     , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--     , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--     , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--     ] :: float[]
--   )

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_3
--   (
--     array
--     [
--       [
--         [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--       , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--       , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--       , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--       , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--       ]
--     , [
--         [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--       , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--       , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--       , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--       , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--       ]
--     ]
--   , array[2, 1, 1]
--   )

-- select sm_sc.fv_d_conv_2d_dloss_dindepdt_3
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     ]
--   , array[2, 2, 1, 1]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1(int, float[], float[], int, int[2], int[4]);
create or replace function sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1
(
  -- -- i_array_grp_x_x_len      int                                                  ,  -- i_array_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本。本参数是 i_array_grp_x 的宽度
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_dloss_ddepdt_1d_grp        float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵 的 降维后样本集合
  i_window_flat            float[1][]                                  ,  -- 卷积核窗口矩阵的扁平化
  i_window_len_x           int                                                  ,  -- 卷积核窗口的宽度
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]          -- 上下左右补齐行数/列数
)
returns float[][]
as
$$
declare 
  v_dloss_ddepdt_array_len_2   int              :=  (i_padding[3] + i_1d_2_2d_cnt_per_grp + i_padding[4] - i_window_len_x) / i_stride[2] + 1;
  -- -- v_array_y_len           int              :=  i_array_grp_x_x_len / i_1d_2_2d_cnt_per_grp;
  v_window                 float[] :=  sm_sc.fv_mx_ele_1d_2_2d(i_window_flat[1 : 1][ : array_length(i_window_flat, 2) - (array_length(i_window_flat, 2) % i_window_len_x)], i_window_len_x);
begin
  -- 审计二维长度
  if array_ndims(i_dloss_ddepdt_1d_grp) <> 2
  then 
    raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt_1d_grp);
  -- -- elsif i_array_grp_x_x_len % i_1d_2_2d_cnt_per_grp <> 0
  -- -- then 
  -- --   raise exception 'imperfect window at grp_x_cnt.';
  else
    return 
    (
      select 
        sm_sc.fa_mx_concat_y
        (
          array 
          [
            sm_sc.fv_mx_ele_2d_2_1d
            (
              sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex    -- -- sm_sc.fv_d_conv_2d_dloss_dindepdt_1
              (         
                -- -- array[v_array_y_len, i_1d_2_2d_cnt_per_grp],
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_dloss_ddepdt_1d_grp[a_cur_y : a_cur_y][ : ]
                  , v_dloss_ddepdt_array_len_2
                )                                        , 
                v_window                                 ,
                i_stride                                 ,
                i_padding   
              )
            )
          ]
          order by a_cur_y
        )
      from generate_series(1, array_length(i_dloss_ddepdt_1d_grp, 1)) tb_a_cur_y(a_cur_y)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select 
--   sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1
--   (
--     -- -- 5*6,
--     6,
--     array
--     [
--       [
--         1.1, 1.1, 1.1
--       , 1.1, 1.1, 1.1
--       , 1.1, 1.1, 1.1
--       ],
--       [
--         2.1, 3.1, 0.1
--       , 1.1, -1.1, -0.1
--       , -1.1, 1.1, -1.1
--       ],
--       [
--         0.1, -2.1, 1.1
--       , -1.1, 3.1, 2.1
--       , 1.1, -1.1, 1.1
--       ]
--     ],
--     array
--     [
--       [
--         0.5, 1.1, 0.5
--       , 1.1, 2.1, 1.1
--       , 0.5, 1.1, 0.5
--       ]
--     ]
--     , 3
--     , array[2, 2]
--     , array[1, 1, 1, 0]
--   )
--   :: decimal[] ~=` 3

-- select sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1
--   (
--     -- -- 5*5,
--     5,
--     array
--     [
--       [
--         1.1, 1.1, 1.1
--       , 1.1, 1.1, 1.1
--       , 1.1, 1.1, 1.1
--       ],
--       [
--         2.1, 3.1, 0.1
--       , 1.1, -1.1, -0.1
--       , -1.1, 1.1, -1.1
--       ],
--       [
--         0.1, -2.1, 1.1
--       , -1.1, 3.1, 2.1
--       , 1.1, -1.1, 1.1
--       ]
--     ]
--    , array[[0.5, 1.1, 0.5, 1.1, 2.1, 1.1, 0.5, 1.1, 0.5]]
--    , 3
--   )
--   :: decimal[] ~=` 3
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2(float[], int, float[], int[2], boolean, int[2], int[4], float);
create or replace function sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2
(
  i_array_grp_x            float[]                                     ,  -- i_array_grp_x_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_dloss_ddepdt_1d_grp        float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵 的 降维后样本集合
  i_window_len             int[2]                                               ,  -- 卷积核窗口矩阵高宽
  i_window_bias_label      boolean             default  false                   ,  -- 卷积核是否附加偏置
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value          float      default  0.0                        -- 补齐填充元素值
)
returns float[][]
as
$$
declare 
  v_dloss_ddepdt_array_len_2   int :=  (i_padding[3] + i_1d_2_2d_cnt_per_grp + i_padding[4] - i_window_len[2]) / i_stride[2] + 1;
  v_return                 float[]   ;
begin
  -- 审计二维长度
  if array_ndims(i_array_grp_x) <> 2
  then 
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array_grp_x);
  elsif array_length(i_array_grp_x, 2) % i_1d_2_2d_cnt_per_grp <> 0
  then 
    raise exception 'imperfect window at grp_x_cnt.';
  else
    v_return :=
      array
      [
        sm_sc.fv_mx_ele_2d_2_1d
        (
          (
            select 
              sm_sc.fa_mx_sum
              (
                sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py   -- -- sm_sc.fv_d_conv_2d_dloss_dindepdt_2_ex
                (
                  sm_sc.fv_mx_ele_1d_2_2d
                  (
                    i_array_grp_x[a_cur_y : a_cur_y][ : ]
                    , i_1d_2_2d_cnt_per_grp
                  )                                        , 
                  sm_sc.fv_mx_ele_1d_2_2d
                  (
                    i_dloss_ddepdt_1d_grp[a_cur_y : a_cur_y][ : ]
                    , v_dloss_ddepdt_array_len_2
                  )                                        , 
                  i_window_len                             ,
                  i_stride                                 ,
                  i_padding                                ,
                  i_padding_value       
                )
              )
            from generate_series(1, array_length(i_array_grp_x, 1)) tb_a_cur_y(a_cur_y)
          )
        )
      ];
    
    if i_window_bias_label
    then 
      v_return := v_return |||| array[array[sm_sc.fv_aggr_slice_sum(i_dloss_ddepdt_1d_grp)]];
    end if;
    
    return v_return;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--       ]
--     ]
--     , 7,
--     array
--     [
--       array[1.1, 1.1, 1.1
--        ,1.1, 1.1, 1.1
--        ],
--       array[2.1, 3.1, 0.1
--        ,1.1, -1.1, -0.1
--        ],
--       array[0.1, -2.1, 1.1
--        ,-1.1, 3.1, 2.1
--        ]
--     ]
--    , array[3, 3]
--    , true    -- false
--    , array[2, 2]
--   )
--   :: decimal[] ~=` 3

-- select sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--       ]
--     ]
--     , 6,
--    array
--    [
--      array[1.1, 1.1, 1.1
--       ,1.1, 1.1, 1.1
--       ,1.1, 1.1, 1.1
--       ],
--      array[2.1, 3.1, 0.1
--       ,1.1, -1.1, -0.1
--       ,0.1, 2.1, -0.1
--       ],
--      array[0.1, -2.1, 1.1
--       ,-1.1, 3.1, 2.1
--       ,-1.1, -1.1, 0.1
--       ]
--    ]
--    , array[3, 3]
--    , true    -- false
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_add_dloss_dindepdt_1
-- (
--   -- anyarray, 
--   int[]
-- -- , anyarray
-- , int[2]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_add_dloss_dindepdt_1
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
begin
  -- if i_background_len is null
  -- then 
  --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    -- elsif -- -- i_background is not null and 
    --   array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    -- elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_2d
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_length(v_window_len, 1) = 3 and v_window_len[1] <> array_length(i_dloss_ddepdt, 1)
      or array_length(v_window_len, 1) = 4 and (v_window_len[1] <> array_length(i_dloss_ddepdt, 1) or v_window_len[2] <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_add_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_add_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_add_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--       [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]                
--     , [1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--     , [2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--     , [3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--     , [4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--     , [5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--     ] 
--   , [ 
--       [-1.0, 2.0, -3.0, 4.0, -5.0, 6.0, -7.0, 8.0, -9.0]                
--     , [-1.1, 1.2, -1.3, 1.4, -1.5, 1.6, -1.7, 1.8, -1.9]                
--     , [-2.1, 2.2, -2.3, 2.4, -2.5, 2.6, -2.7, 2.8, -2.9]                       
--     , [-3.1, 3.2, -3.3, 3.4, -3.5, 3.6, -3.7, 3.8, -3.9]                
--     , [-4.1, 4.2, -4.3, 4.4, -4.5, 4.6, -4.7, 4.8, -4.9]           
--     , [-5.1, 5.2, -5.3, 5.4, -5.5, 5.6, -5.7, 5.8, -5.9]
--     ]
--   ] :: float[],                                                -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                         -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_add_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[2, 2, 6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]        -- array[2, 2, 3, 3]
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[2, 2, 6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--       [
--         [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]                
--       , [1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--       , [2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--       , [3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--       , [4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--       , [5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--       ] 
--     , [ 
--         [-1.0, 2.0, -3.0, 4.0, -5.0, 6.0, -7.0, 8.0, -9.0]                
--       , [-1.1, 1.2, -1.3, 1.4, -1.5, 1.6, -1.7, 1.8, -1.9]                
--       , [-2.1, 2.2, -2.3, 2.4, -2.5, 2.6, -2.7, 2.8, -2.9]                       
--       , [-3.1, 3.2, -3.3, 3.4, -3.5, 3.6, -3.7, 3.8, -3.9]                
--       , [-4.1, 4.2, -4.3, 4.4, -4.5, 4.6, -4.7, 4.8, -4.9]           
--       , [-5.1, 5.2, -5.3, 5.4, -5.5, 5.6, -5.7, 5.8, -5.9]
--       ]
--     ]
--   , [
--       [
--         [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]                
--       , [1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--       , [2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--       , [3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--       , [4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--       , [5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--       ] 
--     , [ 
--         [-1.0, 2.0, -3.0, 4.0, -5.0, 6.0, -7.0, 8.0, -9.0]                
--       , [-1.1, 1.2, -1.3, 1.4, -1.5, 1.6, -1.7, 1.8, -1.9]                
--       , [-2.1, 2.2, -2.3, 2.4, -2.5, 2.6, -2.7, 2.8, -2.9]                       
--       , [-3.1, 3.2, -3.3, 3.4, -3.5, 3.6, -3.7, 3.8, -3.9]                
--       , [-4.1, 4.2, -4.3, 4.4, -4.5, 4.6, -4.7, 4.8, -4.9]           
--       , [-5.1, 5.2, -5.3, 5.4, -5.5, 5.6, -5.7, 5.8, -5.9]
--       ]
--     ]
--   ] :: float[],                                                -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_add_dloss_dindepdt_2
-- (
--   -- anyarray
--   -- , 
--   int[]
--   -- , anyarray
--   , int[]
--   -- , anyarray
--   -- , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_add_dloss_dindepdt_2
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]        :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if i_background_len is null
    -- then 
    --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    -- 
    -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      -- or array_ndims(i_background) not between 2 and 4 
      -- or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif i_window is not null and array_ndims(i_window) <> 2
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    -- els
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
    
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return    
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
        return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_add_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- i_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_add_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- i_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_add_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,    -- array[2, 3, 3]          -- i_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array   
--   [           
--     [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ]
--   , [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ]
--   ]
--   :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_add_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,    -- array[2, 2, 3, 3]   -- i_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array   
--   [
--     [           
--       [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     , [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     ]
--   , [           
--       [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     , [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     ]
--   ]
--   :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
-- (
--   -- anyarray
--   -- , 
--   int[]
--   , anyarray
--   -- , int[]
--   -- , anyarray
--   -- , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
(
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            -- int[2]    :=   array[array_length(i_window, 1), array_length(i_window, 2)];
                          -- alias for i_window_len;
                             int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif i_window is not null and array_ndims(i_window) <> 2
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
    
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          -- 矩阵点算求导
          *` 
          sm_sc.fv_d_div_1
          (
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
          ) 
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            -- 矩阵点算求导
            *` 
            sm_sc.fv_d_div_1
            (
              i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            ) 
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            -- 矩阵点算求导
            *` 
            sm_sc.fv_d_div_1
            (
              i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            ) 
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
-- (
--   -- array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[1, -1, -1, 1],                                      -- i_padding               int[4]    
--   2.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
-- (
--   -- array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
-- (
--   -- array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]      --  array[2, 3, 3]
--   array              
--   [
--     [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]]
--   , [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   ]:: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
-- (
--   anyarray
--   -- , int[2]
--   , anyarray
--   -- , int[]
--   -- , anyarray
--   -- , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
(
  i_window                anyarray           ,
  -- i_window_len            int[2]             ,
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
begin
  -- if v_background_len is null
  -- then 
  --   v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
  -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
      i_padding_value
    );

  if array_ndims(i_dloss_ddepdt) = 2
  then
      
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` sm_sc.fv_d_div_2
          (
            i_background[(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            i_window
          )
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_div_2
          (
            i_background
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                  ) 
              else i_window 
            end
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_div_2
          (
            i_background
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                  ) 
              else i_window 
            end
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
-- (
--   array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                           -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
-- (
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
-- (
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray    --   array[[[1,4,-7],[-2,5,8],[3,-6,9]],[[-1,-4,7],[2,-5,-8],[-3,6,-9]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [
--     [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]]
--   , [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]]
--   ] 
--   :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ] 
--   , [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ] 
--   ]
--   :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
-- (
--   anyarray
-- -- , int[]
-- , anyarray
-- -- , int[]
-- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
(
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_depdt_var             anyarray           ,  -- 尽量传 i_depdt_var, 节省开销
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
  v_depdt_var_len            int[2]    :=   array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
                          -- alias for i_depdt_var_len;
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if v_depdt_var_len is null
    -- then 
    --   v_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null or v_depdt_var_len is null or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, v_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    elsif v_depdt_var_len is not null and v_depdt_var_len <> v_dloss_ddepdt_len_ex
    then 
      raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          sm_sc.fv_d_log_1
          (
            i_window,
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
            i_depdt_var[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                       [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
          )   
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_log_1
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[i_window]
                    , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            , i_depdt_var
                [ : ]
                [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
            )   
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_log_1
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[[i_window]]
                    , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            , i_depdt_var
                [ : ]
                [ : ]
                [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
            )   
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   sm_sc.fv_conv_de_log                                         -- i_depdt_var             anyarray 
--   (
--     array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.5  :: float                  -- i_padding_value         anyelement
--   ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray   -- array[[[1.3,4,7],[2,5,8],[3,6,9]], [[1.3,4,7],[2,5,8],[3,6,9]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array         
--   [     
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array      
--   [                 
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]  
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
-- (
--   anyarray
-- -- , int[]
-- , anyarray
-- -- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
(
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                       -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                       -- alias for i_window_len;
begin
  -- if v_background_len is null
  -- then 
  --   v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
  -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_dloss_ddepdt) = 2
  then
      
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_log_2
          (
            i_window,
            i_background[(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]]
          )
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_log_2
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                  ) 
              else i_window 
            end,
            i_background
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_log_2
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                  ) 
              else i_window 
            end,
            i_background
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len        int[2]    
--   -- sm_sc.fv_conv_de_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len        int[2]    
--   -- sm_sc.fv_conv_de_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]],[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len        int[2]    
--   -- sm_sc.fv_conv_de_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array    
--   [          
--     [
--      [1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ] 
--   , [
--      [-1.2, 2, -3, 4, -5, 6, -7, 8, -9]                
--     ,[-11, 12, -13, 14, -15, 16, -17, 18, -19]                
--     ,[-21, 22, -23, 24, -25, 26, -27, 28, -29]                       
--     ,[-31, 32, -33, 34, -35, 36, -37, 38, -39]                
--     ,[-41, 42, -43, 44, -45, 46, -47, 48, -49]           
--     ,[-51, 52, -53, 54, -55, 56, -57, 58, -59]
--     ] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray      --  array[[[[1.3,4,7],[2,5,8],[3,6,9]],[[1.3,4,7],[2,5,8],[3,6,9]]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]],[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]]]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len        int[2]    
--   -- sm_sc.fv_conv_de_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array    
--   [[          
--     [
--      [1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ] 
--   , [
--      [-1.2, 2, -3, 4, -5, 6, -7, 8, -9]                
--     ,[-11, 12, -13, 14, -15, 16, -17, 18, -19]                
--     ,[-21, 22, -23, 24, -25, 26, -27, 28, -29]                       
--     ,[-31, 32, -33, 34, -35, 36, -37, 38, -39]                
--     ,[-41, 42, -43, 44, -45, 46, -47, 48, -49]           
--     ,[-51, 52, -53, 54, -55, 56, -57, 58, -59]
--     ] 
--   ]]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
-- (anyarray
-- -- , int[2]
-- , anyarray
-- -- , int[]
-- , anyarray
-- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
(
  i_window                anyarray           ,
  -- i_window_len            int[2]             ,
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_depdt_var             anyarray           ,  -- 尽量传 i_depdt_var, 节省开销
  i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
  v_depdt_var_len            int[2]    :=   array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
                          -- alias for i_depdt_var_len;
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if v_depdt_var_len is null
    -- then 
    --   v_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null or v_depdt_var_len is null or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, v_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    elsif v_depdt_var_len is not null and v_depdt_var_len <> v_dloss_ddepdt_len_ex
    then 
      raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          sm_sc.fv_d_pow_1
          (
            i_window,
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
            i_depdt_var[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                       [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
          )   
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_pow_1
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[i_window]
                    , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
              i_depdt_var
                [ : ]
                [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
            )   
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_pow_1
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[[i_window]]
                    , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
              i_depdt_var
                [ : ]
                [ : ]
                [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
            )   
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
-- (
--   array[[1.2,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   sm_sc.fv_conv_de_pow                                         -- i_depdt_var             anyarray 
--   (
--     array[[1.2,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.5  :: float                  -- i_padding_value         anyelement
--   ), 
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
-- (
--   array[[1.2,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
-- (
--   array[[1.2,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray    -- array[[[1.2,4,7],[2,5,8],[3,6,9]],[[1.2,4,7],[2,5,8],[3,6,9]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]        -- array[2, 6, 9]
--   array              
--   [
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.5  :: float                                             -- i_padding_value         anyelement
-- );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
-- (
--   anyarray
--   -- , int[2]
--   , anyarray
--   , int[]
--   , anyarray
--   , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
(
  i_window                anyarray           ,  -- 当 i_depdt_var 不为空，可以不传 i_window
  -- i_window_len            int[2]             ,
  i_background            anyarray           ,
  i_background_len        int[]             ,
  i_depdt_var             anyarray           ,  -- 尽量传 i_depdt_var, 节省开销
  i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len     -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                       -- alias for i_window_len;
  v_depdt_var_len      -- int[2]    :=   array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
                          alias for i_depdt_var_len;
begin
  if v_background_len is null
  then 
    v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  if v_depdt_var_len is null
  then 
    v_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
  end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
  -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null or v_depdt_var_len is null or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, v_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    elsif v_depdt_var_len is not null and v_depdt_var_len <> v_dloss_ddepdt_len
    then 
      raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if i_depdt_var is null 
  then 
    i_background := 
      sm_sc.fv_augmented
      (
        i_background, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
        i_padding_value
      );
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt          
          sm_sc.fv_d_pow_2
          ( 
            case 
              when i_depdt_var is null 
                then i_background[(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                                 [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]]
            end,
            i_window,
            i_depdt_var[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                       [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;  
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_pow_2
          (
            case 
              when i_depdt_var is null 
                then i_background
                       [ : ]
                       [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                       [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]]
            end,
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                  ) 
              else i_window 
            end,
            i_depdt_var
              [ : ]
              [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
              [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_pow_2
          (
            case 
              when i_depdt_var is null 
                then i_background
                       [ : ]
                       [ : ]
                       [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                       [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]]
            end,
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                  ) 
              else i_window 
            end,
            i_depdt_var
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
              [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]    
--   sm_sc.fv_conv_de_pow                                         -- i_depdt_var             anyarray 
--   (
--     array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.5  :: float                  -- i_padding_value         anyelement
--   ), 
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[],  -- array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]    
--   sm_sc.fv_conv_de_pow                                         -- i_depdt_var             anyarray 
--   (
--     array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.5  :: float                  -- i_padding_value         anyelement
--   ), 
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [[[1.2, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1, 9.1]                
--   ,[1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--   ,[2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--   ,[3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--   ,[4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--   ,[5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[[1.2, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1, 9.1]                
--   ,[1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--   ,[2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--   ,[3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--   ,[4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--   ,[5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
-- (
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                 -- i_window                anyarray   -- array[[[1.3,4,7],[2,5,8],[3,6,9]],[[1.3,4,7],[2,5,8],[3,6,9]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   array              
--   [
--     [
--      [1.2, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1, 9.1]                
--     ,[1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--     ,[2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--     ,[3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--     ,[4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--     ,[5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--     ]
--   , [
--      [1.2, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1, 9.1]                
--     ,[1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--     ,[2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--     ,[3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--     ,[4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--     ,[5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--     ]
--   ] :: float[],     -- i_background            anyarray  
--   array[2, 6, 9]                   ,                           -- v_background_len        int[2]
--   null :: float[]               ,                           -- i_depdt_var             anyarray 
--   array[2, 6, 9]                   ,                           -- i_depdt_var_len         int[2]
--   array              
--   [
--     [
--      [1.2, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1, 9.1]                
--     ,[1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--     ,[2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--     ,[3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--     ,[4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--     ,[5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--     ]
--   , [
--      [1.2, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1, 9.1]                
--     ,[1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--     ,[2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--     ,[3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--     ,[4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--     ,[5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--     ]
--   ] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
-- (
-- --   anyarray
-- -- , 
--   int[]
-- , anyarray
-- -- , int[]
-- , int
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
(
  -- i_window_ex             anyarray           ,
  i_window_ex_len         int[]             ,
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_window_len_width      int                                                  ,  -- 滑动窗口高宽规格
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_ex_len         -- int[2]        :=   array[array_length(i_window_ex, array_ndims(i_window_ex) - 1), array_length(i_window_ex, array_ndims(i_window_ex))];
                          -- alias for i_window_ex_len;
                             int[2]        :=   i_window_ex_len[array_length(i_window_ex_len, 1) - 1 : array_length(i_window_ex_len, 1)];
  v_background_len_ex        int[2]        :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]        :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]        :=   array[v_window_ex_len[2], i_window_len_width];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif array_ndims(i_window_ex) <> 2
    -- then 
    --   raise exception 'no method for such i_window_ex length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_ex), v_window_ex_len[1], v_window_ex_len[2];
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif v_window_len[1] <> v_window_ex_len[2]
    then 
      raise exception 'imperfect window at 2d and window_x at 1d.';
    elsif i_dloss_ddepdt is null 
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) <> ((coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) * v_window_ex_len[1]
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) <> ((coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) * v_window_len[2]
    then 
      raise exception 'imperfect i_dloss_ddepdt.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        ( 
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_ex_len[1] + 1 : a_cur_y * v_window_ex_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          |**|
          (
            |^~|
            sm_sc.fv_d_prod_mx_1
            (
              i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                          [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
          -- 矩阵点算求导
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          ( 
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_ex_len[1] + 1 : a_cur_y * v_window_ex_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            |**|
            (
              |^~|
              sm_sc.fv_d_prod_mx_1
              (
                i_background
                  [ : ]
                  [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                  [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
              )
            )
            -- 矩阵点算求导
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
      
    if array_length(i_window_ex_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          ( 
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_ex_len[1] + 1 : a_cur_y * v_window_ex_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            |**|
            (
              |^~|
              sm_sc.fv_d_prod_mx_1
              (
                i_background
                  [ : ]
                  [ : ]
                  [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                  [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
              )
            )
            -- 矩阵点算求导
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
     
    if array_length(i_window_ex_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
-- (
--   -- array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],                           -- i_window_ex                anyarray  
--   array[4, 3]                   ,                           -- v_window_ex_len            int[2]    
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]     
--   3                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                       
--   ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--   ,[71, 72, 73, 74, 75, 76, 77, 78, 79]            
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
-- (   
--   -- array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   array[4, 3]                   ,                           -- v_window_ex_len            int[2]    
--   array              
--   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]      
--   3                   ,                           -- v_window_len            int[2] 
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                         
--   ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--   ,[71, 72, 73, 74, 75, 76, 77, 78, 79]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
-- (   
--   -- array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   array[4, 3]                   ,                           -- v_window_ex_len            int[2]      -- array[2, 4, 3]
--   array    
--   [          
--     [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]]
--   , [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   ] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]      
--   3                   ,                           -- v_window_len            int[2] 
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array     
--   [                 
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                         
--     ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--     ,[71, 72, 73, 74, 75, 76, 77, 78, 79]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]   
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                         
--     ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--     ,[71, 72, 73, 74, 75, 76, 77, 78, 79]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
-- (
--   anyarray
-- -- , int[]
-- -- , anyarray
-- , int[]
-- , int
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
(
  i_window_ex             anyarray           ,
  -- i_window_ex_len         int[]             ,
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  i_window_len_width      int                                                  ,  -- 滑动窗口高宽规格的宽
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare  
  v_window_ex_len         int[2]        :=   array[array_length(i_window_ex, array_ndims(i_window_ex) - 1), array_length(i_window_ex, array_ndims(i_window_ex))];
                       -- alias for i_window_ex_len;
  v_background_len     -- int[2]        :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          int[2]        :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]        :=   array[v_window_ex_len[2], i_window_len_width];
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
  -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      -- or array_ndims(i_background) not between 2 and 4 
      -- or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif array_ndims(i_window_ex) <> 2 and array_ndims(i_window_ex) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window_ex length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_ex), v_window_ex_len[1], v_window_ex_len[2];
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif v_window_len[1] <> v_window_ex_len[2]  -- v_window_len[1] <> v_window_ex_len[2]
    then 
      raise exception 'imperfect window at 2d and window_x at 1d.';
    elsif i_dloss_ddepdt is null 
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) <> ((coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) * v_window_ex_len[1]
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) <> ((coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) * v_window_len[2]
    then 
      raise exception 'imperfect i_dloss_ddepdt.';
    elsif array_ndims(i_window_ex) = 3 and array_length(i_window_ex, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window_ex) = 4 and (array_length(i_window_ex, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window_ex, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window_ex and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_ex_len[2] + 1 : a_cur_width * v_window_ex_len[2]] -- 损失函数导数的独立窗口分组
          |**| sm_sc.fv_d_prod_mx_2(i_window_ex)   -- 独立窗口矩阵点算求导 ddepdt_dindepdt
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_ex_len[2] + 1 : a_cur_width * v_window_ex_len[2]] -- 损失函数导数的独立窗口分组
          -- 此处矩阵乘法依赖于向第三维的广播扩展
          |**| -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window_ex) = 2 
              then 
                sm_sc.fv_new
                (
                  array[sm_sc.fv_d_prod_mx_2(i_window_ex)]
                , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                ) 
            else sm_sc.fv_d_prod_mx_2(i_window_ex) 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_ex_len[2] + 1 : a_cur_width * v_window_ex_len[2]] -- 损失函数导数的独立窗口分组
          -- 此处矩阵乘法依赖于向第三维的广播扩展
          |**|   -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window_ex) = 2 
              then 
                sm_sc.fv_new
                (
                  array[[sm_sc.fv_d_prod_mx_2(i_window_ex)]]
                , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                ) 
            else sm_sc.fv_d_prod_mx_2(i_window_ex) 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
-- (  
--   array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],                           -- i_window_ex                anyarray  
--   -- array[4, 3]                   ,                           -- v_window_ex_len            int[2]    
--   -- array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]     
--   3                             ,                           -- v_window_len            int
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                       
--   ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--   ,[71, 72, 73, 74, 75, 76, 77, 78, 79]            
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
-- (
--   array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   -- array[4, 3]                   ,                           -- v_window_ex_len            int[2]    
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]      
--   3                               ,                           -- v_window_len            int
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                         
--   ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--   ,[71, 72, 73, 74, 75, 76, 77, 78, 79]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
-- (
--   array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   -- array[4, 3]                   ,                           -- v_window_ex_len            int[2]    
--   -- array              
--   -- [[
--   --  [-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]
--   -- ]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]      
--   3                             ,                           -- v_window_len            int
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--    [
--     [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--    ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--    ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--    ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--    ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                         
--    ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--    ,[71, 72, 73, 74, 75, 76, 77, 78, 79]           
--    ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--    ]
--   ,[
--     [-1, -2, -3, -4, -5, -6, -7, -8, -9]                
--    ,[-11, -12, -13, -14, -15, -16, -17, -18, -19]                
--    ,[-21, -22, -23, -24, -25, -26, -27, -28, -29]                       
--    ,[-31, -32, -33, -34, -35, -36, -37, -38, -39]                
--    ,[-41, -42, -43, -44, -45, -46, -47, -48, -49]                         
--    ,[-61, -62, -63, -64, -65, -66, -67, -68, -69]                
--    ,[-71, -72, -73, -74, -75, -76, -77, -78, -79]           
--    ,[-51, -52, -53, -54, -55, -56, -57, -58, -59]
--    ]
--   ] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
-- (
--   array[[1,4,7],[6,2,5],[8,7,3],[6,9,8]] :: float[],          -- i_window_ex                anyarray    -- array[[[[1,4,7],[6,2,5],[8,7,3],[6,9,8]],[[1,4,7],[6,2,5],[8,7,3],[6,9,8]]],[[[1,4,7],[6,2,5],[8,7,3],[6,9,8]],[[1,4,7],[6,2,5],[8,7,3],[6,9,8]]]]
--   -- array[4, 3]                   ,                           -- v_window_ex_len            int[2]    
--   -- array              
--   -- [[[
--   --  [-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]
--   -- ]]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]      -- array[2, 2, 6, 9]
--   3                             ,                           -- v_window_len            int
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[8, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--      [
--       [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--      ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--      ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--      ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--      ,[41, 42, 43, 44, 45, 46, 47, 48, 49]                         
--      ,[61, 62, 63, 64, 65, 66, 67, 68, 69]                
--      ,[71, 72, 73, 74, 75, 76, 77, 78, 79]           
--      ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--      ]
--     ,[
--       [-1, -2, -3, -4, -5, -6, -7, -8, -9]                
--      ,[-11, -12, -13, -14, -15, -16, -17, -18, -19]                
--      ,[-21, -22, -23, -24, -25, -26, -27, -28, -29]                       
--      ,[-31, -32, -33, -34, -35, -36, -37, -38, -39]                
--      ,[-41, -42, -43, -44, -45, -46, -47, -48, -49]                         
--      ,[-61, -62, -63, -64, -65, -66, -67, -68, -69]                
--      ,[-71, -72, -73, -74, -75, -76, -77, -78, -79]           
--      ,[-51, -52, -53, -54, -55, -56, -57, -58, -59]
--      ]
--     ]
--   , [
--      [
--       [-1, 2, -3, 4, -5, 6, -7, 8, -9]                
--      ,[-11, 12, -13, 14, -15, 16, -17, 18, -19]                
--      ,[-21, 22, -23, 24, -25, 26, -27, 28, -29]                       
--      ,[-31, 32, -33, 34, -35, 36, -37, 38, -39]                
--      ,[-41, 42, -43, 44, -45, 46, -47, 48, -49]                         
--      ,[-61, 62, -63, 64, -65, 66, -67, 68, -69]                
--      ,[-71, 72, -73, 74, -75, 76, -77, 78, -79]           
--      ,[-51, 52, -53, 54, -55, 56, -57, 58, -59]
--      ]
--     ,[
--       [1, -2, 3, -4, 5, -6, 7, -8, 9]                
--      ,[11, -12, 13, -14, 15, -16, 17, -18, 19]                
--      ,[21, -22, 23, -24, 25, -26, 27, -28, 29]                       
--      ,[31, -32, 33, -34, 35, -36, 37, -38, 39]                
--      ,[41, -42, 43, -44, 45, -46, 47, -48, 49]                         
--      ,[61, -62, 63, -64, 65, -66, 67, -68, 69]                
--      ,[71, -72, 73, -74, 75, -76, 77, -78, 79]           
--      ,[51, -52, 53, -54, 55, -56, 57, -58, 59]
--      ]
--     ] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
-- (
-- --   anyarray
-- , int[]
-- -- , anyarray
-- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
(
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]        :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if i_background_len is null
    -- then 
    --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    -- 
    -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      -- or array_ndims(i_background) not between 2 and 4 
      -- or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif i_window is not null and array_ndims(i_window) <> 2
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    -- els
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
     
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
     
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
-- (   
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]   
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]  
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]     -- array[2, 3, 3]
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]     -- array[2, 6, 9]
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array               
--   [
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
-- (
-- --   anyarray
-- , int[]
-- -- , anyarray
-- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
(
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len     -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
begin
  -- if i_background_len is null
  -- then 
  --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
  -- end if;
  -- 
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
  -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      -- or array_ndims(i_background) not between 2 and 4 
      -- or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- if i_background is not null and array_ndims(i_background) <> 2
    -- then 
    --   raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    -- elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    -- els
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_length(i_window_len, 1) = 3 and i_window_len[1] <> array_length(i_dloss_ddepdt, 1)
      or array_length(i_window_len, 1) = 4 and (i_window_len[1] <> array_length(i_dloss_ddepdt, 1) or i_window_len[2] <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          -` i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                           [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          -` i_dloss_ddepdt
               [ : ]
               [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
               [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          -` i_dloss_ddepdt
               [ : ]
               [ : ]
               [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
               [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]        -- array[2, 3, 3]
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]        -- array[2, 6, 9]
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ] 
--   , [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_div_dloss_dindepdt_1
-- (
--   -- anyarray
--   -- , 
--     int[]
--   , anyarray
--   -- , int[]
--   -- , anyarray
--   -- , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_div_dloss_dindepdt_1
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
begin
  -- if v_background_len is null
  -- then 
  --   v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    -- elsif -- -- i_background is not null and 
    --   array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    elsif i_window is not null and array_ndims(i_window) not between 2 and 4
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_2d
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` sm_sc.fv_d_div_1(i_window)   -- 独立窗口矩阵点算求导 ddepdt_dindepdt
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *`   -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window) = 2 
              then 
                sm_sc.fv_new
                (
                  array[sm_sc.fv_d_div_1(i_window)]
                , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                ) 
            else sm_sc.fv_d_div_1(i_window) 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window) = 2 
              then 
                sm_sc.fv_new
                (
                  array[[sm_sc.fv_d_div_1(i_window)]]
                , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                ) 
            else sm_sc.fv_d_div_1(i_window) 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_div_dloss_dindepdt_1
-- (
--   -- array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                           -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_div_dloss_dindepdt_1
-- (
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_div_dloss_dindepdt_1
-- (
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_div_dloss_dindepdt_1
-- (
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]         --  array[1, 2, 6, 9]
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],             -- i_window                anyarray       --  array[[[[1,4,-7],[-2,5,8],[3,-6,9]],[[1,4,-7],[-2,5,8],[3,-6,9]]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_div_dloss_dindepdt_2
-- (
--   anyarray
--   -- , int[]
--   , anyarray
--   -- , int[]
--   -- , anyarray
--   -- , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   , anyelement
-- );
create or replace function sm_sc.fv_d_conv_div_dloss_dindepdt_2
(
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          -- 矩阵点算求导
          *` 
          sm_sc.fv_d_div_2
          (
            i_window,
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
          )
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            -- 矩阵点算求导
            *` 
            sm_sc.fv_d_div_2
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[i_window]
                    , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            -- 矩阵点算求导
            *` 
            sm_sc.fv_d_div_2
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[[i_window]]
                    , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_div_dloss_dindepdt_2
-- (
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                           -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_div_dloss_dindepdt_2
-- (
--   array              
--   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_div_dloss_dindepdt_2
-- (
--   array          
--   [          
--     [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]]
--   , [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   ] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  -- array[[[1,4,-7],[-2,5,8],[3,-6,9]],[[1,4,-7],[-2,5,8],[3,-6,9]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array                    
--   [
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_log_dloss_dindepdt_1
-- (
--   anyarray
-- , int[]
-- , anyarray
-- , int[]
-- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_log_dloss_dindepdt_1
(
  i_background            anyarray           ,
  i_background_len        int[]             ,
  i_window                anyarray           ,
  i_window_len            int[]             ,
  i_depdt_var             anyarray           ,  -- 尽量传 i_depdt_var, 节省开销
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            -- int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
                             int[2]    :=   case 
                                              when i_window is not null 
                                                then array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))] 
                                              else i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)]
                                            end;
  v_depdt_var_len            int[2]    :=   array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
                          -- alias for i_depdt_var_len;
begin
  -- if v_background_len is null
  -- then 
  --   v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if v_depdt_var_len is null
  -- then 
  --   v_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    elsif -- -- i_background is not null and 
      array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null or v_depdt_var_len is null or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, v_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    elsif v_depdt_var_len is not null and v_depdt_var_len <> v_dloss_ddepdt_len_2d
    then 
      raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_log_1
          (
            i_background[(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            i_window,
            i_depdt_var[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                       [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_log_1
          (
            i_background
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                  ) 
              else i_window 
            end,
            i_depdt_var
              [ : ]
              [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
              [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_log_1
          (
            i_background
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                  ) 
              else i_window 
            end,
            i_depdt_var
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
              [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_log_dloss_dindepdt_1
-- (
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   sm_sc.fv_conv_log                                         -- i_depdt_var             anyarray 
--   (
--     array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.5  :: float                  -- i_padding_value         anyelement
--   ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_log_dloss_dindepdt_1
-- (
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_log_dloss_dindepdt_1
-- (
--   array              
--   [[
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_log_dloss_dindepdt_1
-- (
--   array              
--   [[
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                        -- i_window                anyarray     -- array[[[[1.3,4,7],[2,5,8],[3,6,9]], [[1.3,4,7],[2,5,8],[3,6,9]]]]
--   array[3, 3]                   ,                           -- v_window_len            int[2]                 -- array[1, 2, 3, 3]
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_log_dloss_dindepdt_2
-- (
--   anyarray
-- -- , int[]
-- , anyarray
-- -- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_log_dloss_dindepdt_2
(
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
    -- end if;
    
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    elsif i_window is not null and array_ndims(i_window) <> 2
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          sm_sc.fv_d_log_2
          (
            i_window,
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
          )
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_log_2
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[i_window]
                    , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_log_2
            (
              case 
                when array_ndims(i_window) = 2 
                  then 
                    sm_sc.fv_new
                    (
                      array[[i_window]]
                    , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                    ) 
                else i_window 
              end
            , i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_ndims(i_window) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_log_dloss_dindepdt_2
-- (
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- sm_sc.fv_conv_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_log_dloss_dindepdt_2
-- (
--   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- sm_sc.fv_conv_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_log_dloss_dindepdt_2
-- (
--   array[[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]],[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray    -- array[[[1.3,4,7],[2,5,8],[3,6,9]],[[1.3,4,7],[2,5,8],[3,6,9]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- sm_sc.fv_conv_log                                         -- i_depdt_var             anyarray 
--   -- (
--   --   array[[1.2,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--   --   array[[1.3,4,7],[2,5,8],[3,6,9]] :: float[],
--   --   array[1, 1]        ,           -- i_stride                int[2]    
--   --   array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--   --   0.5  :: float                  -- i_padding_value         anyelement
--   -- ), 
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array   
--   [                   
--     [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1.2, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.5  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_mul_dloss_dindepdt_1
-- (
-- --   anyarray,
--   int[]
-- , anyarray
-- -- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_mul_dloss_dindepdt_1
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
begin
  -- if v_background_len is null
  -- then 
  --   v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
  -- end if;
  
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    -- elsif -- -- i_background is not null and 
    --   array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_2d
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` sm_sc.fv_d_mul(i_window)   -- 独立窗口矩阵点算求导 ddepdt_dindepdt
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window) = 2 
              then 
                sm_sc.fv_new
                (
                  array[i_window]
                , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                ) 
            else i_window 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window) = 2 
              then 
                sm_sc.fv_new
                (
                  array[[i_window]]
                , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                ) 
            else i_window 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_mul_dloss_dindepdt_1
-- (
--   -- array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                           -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_mul_dloss_dindepdt_1
-- (
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_mul_dloss_dindepdt_1
-- (
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]       --   array[1, 2, 6, 9]
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],              -- i_window                anyarray   -- array[[[[1,4,-7],[-2,5,8],[3,-6,9]],[[-1,-4,7],[2,-5,-8],[-3,6,-9]]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_mul_dloss_dindepdt_2
-- (
--   anyarray
-- -- , int[]
-- -- , anyarray
-- , int[2]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_mul_dloss_dindepdt_2
(
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  -- i_window                anyarray           ,
  i_window_len            int[2]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            -- int[2]    :=   array[array_length(i_window, 1), array_length(i_window, 2)];
                          -- alias for i_window_len;
                             int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif i_window is not null and array_ndims(i_window) <> 2
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null 
      -- or i_depdt_var_len is null 
      or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          sm_sc.fv_d_mul
          (
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
          )
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_mul
            (
              i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_mul
            (
              i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_mul_dloss_dindepdt_2
-- (
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   -- array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_mul_dloss_dindepdt_2
-- (
--   array              
--   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   -- array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_mul_dloss_dindepdt_2
-- (
--   array   
--   [                   
--     [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]]    
--   , [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   ]:: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   -- array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,   -- array[2, 3, 3]          -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array  
--   [                   
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]      
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_pow_dloss_dindepdt_1
-- (anyarray
-- -- , int[]
-- , anyarray
-- -- , int[]
-- , anyarray
-- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_pow_dloss_dindepdt_1
(
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_window                anyarray           ,
  -- i_window_len            int[]             ,
  i_depdt_var             anyarray           ,  -- 尽量传 i_depdt_var, 节省开销
  i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
  v_depdt_var_len         int[2]    :=   array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
                          -- alias for i_depdt_var_len;
begin
  -- if v_background_len is null
  -- then 
  --   v_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if v_depdt_var_len is null
  -- then 
  --   v_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
  -- end if;
  
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    elsif -- -- i_background is not null and 
      array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len is null or v_window_len is null or v_depdt_var_len is null or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len, v_window_len, v_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    elsif v_depdt_var_len is not null and v_depdt_var_len <> v_dloss_ddepdt_len_2d
    then 
      raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_pow_1
          (
            i_background[(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            i_window,
            i_depdt_var[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                       [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_pow_1
          (
            i_background
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                  ) 
              else i_window 
            end,
            i_depdt_var
              [ : ]
              [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
              [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          sm_sc.fv_d_pow_1
          (
            i_background
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * i_stride[1] + 1 : (a_cur_heigh - 1) * i_stride[1] + v_window_len[1]]
              [(a_cur_width - 1) * i_stride[2] + 1 : (a_cur_width - 1) * i_stride[2] + v_window_len[2]],
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                  ) 
              else i_window 
            end,
            i_depdt_var
              [ : ]
              [ : ]
              [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
              [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]]
          )   
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_1
-- (
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   sm_sc.fv_conv_pow                                         -- i_depdt_var             anyarray 
--   (
--     array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[[1,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.0  :: float                  -- i_padding_value         anyelement
--   ), 
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_1
-- (
--   array              
--   [[-1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_1
-- (
--   array              
--   [[
--     [[-1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[-1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],              -- i_window                anyarray    -- array[[[[1,4,-7],[-2,5,8],[3,-6,9]],[[1,4,-7],[-2,5,8],[3,-6,9]]]]
--   -- array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_pow_dloss_dindepdt_2
-- (
--   anyarray
--   -- , int[]
--   , anyarray
--   , int[]
--   , anyarray
--   , int[]
--   , anyarray
--   , int[2]
--   , int[4]
--   , anyelement
-- );
create or replace function sm_sc.fv_d_conv_pow_dloss_dindepdt_2
(
  i_background            anyarray           ,
  -- i_background_len        int[]             ,
  i_window                anyarray           ,  -- 当 i_depdt_var 不为空，可以不传 i_window
  i_window_len            int[]             ,
  i_depdt_var             anyarray           ,  -- 尽量传 i_depdt_var, 节省开销
  i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len_ex        int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            -- int[2]    :=   array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))];
                          -- alias for i_window_len;
                             int[2]    :=   case 
                                              when i_window is not null 
                                                then array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
                                              else i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)] 
                                            end;
  v_depdt_var_len         -- int[2]    :=   array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
                          -- alias for i_depdt_var_len;
                             int[2]    :=   i_depdt_var_len[array_length(i_depdt_var_len, 1) - 1 : array_length(i_depdt_var_len, 1)];
  v_ret                      i_dloss_ddepdt%type;
begin
  if i_window is not null
  then 
    i_window_len := (select array_agg(array_length(i_window, a_dim) order by a_dim) from generate_series(1, array_ndims(i_window)) tb_a_dim(a_dim));
  end if;  
  
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if v_background_len_ex is null
    -- then 
    --   v_background_len_ex := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    if v_window_len is null
    then 
      v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    end if;
    if v_depdt_var_len is null
    then 
      v_depdt_var_len := array[array_length(i_depdt_var, array_ndims(i_depdt_var) - 1), array_length(i_depdt_var, array_ndims(i_depdt_var))];
    end if;
    
    -- 审计二维长度
    if i_background is not null and array_ndims(i_background) <> 2
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_ex[1], v_background_len_ex[2];
    elsif i_window is not null and array_ndims(i_window) <> 2
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif v_background_len_ex is null or v_window_len is null or v_depdt_var_len is null or i_dloss_ddepdt is null 
    then 
      raise exception 'v_background_len_ex, v_window_len, v_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    elsif v_depdt_var_len is not null and v_depdt_var_len <> v_dloss_ddepdt_len_ex
    then 
      raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2 
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          *` 
          sm_sc.fv_d_pow_2
          (
            case when i_depdt_var is null then i_window end,
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
            i_depdt_var[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                       [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
          )
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_pow_2
            (
              case 
                when i_depdt_var is null 
                  then 
                    case 
                      when array_ndims(i_window) = 2 
                        then 
                          sm_sc.fv_new
                          (
                            array[i_window]
                          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                          ) 
                      else i_window 
                    end
              end,
              i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
              i_depdt_var
                [ : ]
                [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            *` 
            sm_sc.fv_d_pow_2
            (
              case 
                when i_depdt_var is null 
                  then 
                    case 
                      when array_ndims(i_window) = 2 
                        then 
                          sm_sc.fv_new
                          (
                            array[[i_window]]
                          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                          ) 
                      else i_window 
                    end
              end,
              i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]],
              i_depdt_var
                [ : ]
                [ : ]
                [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]]
            )
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_2
-- (
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   sm_sc.fv_conv_pow                                         -- i_depdt_var             anyarray 
--   (
--     array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[[1,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.0  :: float                  -- i_padding_value         anyelement
--   ), 
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_2
-- (
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]    
--   null, -- array[[1,4,7],[2,5,8],[3,6,9]] :: float[],                -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   sm_sc.fv_conv_pow                                         -- i_depdt_var             anyarray 
--   (
--     array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],
--     array[[1,4,7],[2,5,8],[3,6,9]] :: float[],
--     array[1, 1]        ,           -- i_stride                int[2]    
--     array[0, 0, 0, 0]  ,           -- i_padding               int[4]    
--     0.0  :: float                  -- i_padding_value         anyelement
--   ), 
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_2
-- (
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]] :: float[],                        -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_pow_dloss_dindepdt_2
-- (
--   array                  
--   [
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]    
--   array[[1,4,-7],[-2,5,8],[3,-6,9]], :: float[],                        -- i_window                anyarray   -- array[[[1,4,-7],[-2,5,8],[3,-6,9]], [[1,4,-7],[-2,5,8],[3,-6,9]]] 
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   null :: float[]               ,                           -- i_depdt_var             anyarray  
--   array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array    
--   [          
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]]
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
-- (
-- -- anyarray
-- -- , 
-- int[2]
-- , int
-- , anyarray
-- -- , int[2]
-- -- , anyarray
-- -- , int[2]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  i_window_len_heigh      int              ,  -- 滑动窗口高宽规格的高
  i_window_ex             anyarray           ,
  -- i_window_ex_len         int[2]             ,  -- 窗口自变量高宽规格，该窗口自变量高宽与背景矩阵滑动窗口高宽 i_window_len 不一致，为矩阵相乘关系
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_ex_len         int[2]    :=   array[array_length(i_window_ex, array_ndims(i_window_ex) - 1), array_length(i_window_ex, array_ndims(i_window_ex))];
                          -- alias for i_window_ex_len;
  v_background_len        -- int[2]        :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len            int[2]    :=   array[i_window_len_heigh, array_length(i_window_ex, 1)];
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    -- elsif -- -- i_background is not null and 
    --   array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    elsif array_ndims(i_window_ex) <> 2 and array_ndims(i_window_ex) <> array_ndims(i_dloss_ddepdt)
    then 
      raise exception 'no method for such i_window_ex length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_ex), v_window_ex_len[1], v_window_ex_len[2];
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif v_window_len[2] <> v_window_ex_len[1]
    then 
      raise exception 'imperfect window at 2d and window_x at 1d.';
    elsif i_dloss_ddepdt is null 
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) <> ((coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) * v_window_len[1]
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) <> ((coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) * v_window_ex_len[2]
    then 
      raise exception 'imperfect i_dloss_ddepdt.';
    elsif array_ndims(i_window_ex) = 3 and array_length(i_window_ex, 1) <> array_length(i_dloss_ddepdt, 1)
      or array_ndims(i_window_ex) = 4 and (array_length(i_window_ex, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window_ex, 2) <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window_ex and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_ex_len[2] + 1 : a_cur_width * v_window_ex_len[2]] -- 损失函数导数的独立窗口分组
          |**| (|^~| sm_sc.fv_d_prod_mx_1(i_window_ex))   -- 独立窗口矩阵点算求导 ddepdt_dindepdt
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_ex_len[2] + 1 : a_cur_width * v_window_ex_len[2]] -- 损失函数导数的独立窗口分组
          -- 此处矩阵乘法依赖于向第三维的广播扩展
          |**| -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window_ex) = 2 
              then 
                sm_sc.fv_new
                (
                  array[|^~| sm_sc.fv_d_prod_mx_1(i_window_ex)]
                , array[array_length(i_dloss_ddepdt, 1), 1, 1]
                ) 
            else |^~| sm_sc.fv_d_prod_mx_1(i_window_ex) 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_ex_len[2] + 1 : a_cur_width * v_window_ex_len[2]] -- 损失函数导数的独立窗口分组
          -- 此处矩阵乘法依赖于向第三维的广播扩展
          |**| -- 独立窗口矩阵点算求导 ddepdt_dindepdt
          case 
            when array_ndims(i_window_ex) = 2 
              then 
                sm_sc.fv_new
                (
                  array[[|^~| sm_sc.fv_d_prod_mx_1(i_window_ex)]]
                , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
                ) 
            else |^~| sm_sc.fv_d_prod_mx_1(i_window_ex) 
          end
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
-- (
--   -- array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   array[4, 5]                   ,                           -- v_background_len        int[2]     
--   3                             ,                           -- v_window_len            int[2]    
--   array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],                           -- i_window_ex                anyarray  
--   -- array[3, 4]                   ,                           -- v_window_ex_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
-- (
--   -- array              
--   -- [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   -- ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   -- ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   -- ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   -- ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   -- ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]      
--   3                             ,                           -- v_window_len            int[2]    
--   array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   -- array[3, 4]                   ,                           -- v_window_ex_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]                                       -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
-- (
--   -- array  
--   -- [[            
--   --   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   --   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   --   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   --   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   --   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   --   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   -- ]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]      
--   3                             ,                           -- v_window_len            int[2]    
--   array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   -- array[3, 4]                   ,                           -- v_window_ex_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]                                       -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
-- (
--   -- array  
--   -- [[            
--   --   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   --   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   --   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   --   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   --   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   --   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   --  ,[[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   --   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   --   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   --   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   --   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   --   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   -- ]] :: float[],     -- i_background            anyarray  
--   array[6, 9]                   ,                           -- v_background_len        int[2]      array[1, 2, 6, 9]
--   3                             ,                           -- v_window_len            int[2]    
--   array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],             -- i_window_ex                anyarray  -- array[[[[1,4,7,6],[2,5,8,7],[3,6,9,8]],[[1,4,7,6],[2,5,8,7],[3,6,9,8]]]]
--   -- array[3, 4]                   ,                           -- v_window_ex_len            int[2]    --   array[1, 2, 3, 4]     
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] 
--    ,[[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] 
--   ]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]                                       -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
-- (
--   anyarray
-- -- , int[]
-- , int
-- -- , anyarray
-- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
(
  i_background            anyarray           ,
  -- i_background_len        int[]              ,
  i_window_len_heigh      int                ,  -- 滑动窗口高宽规格的高
  -- i_window_ex             anyarray           ,
  i_window_ex_len         int[]             ,  -- 窗口自变量高宽规格，该窗口自变量高宽与背景矩阵滑动窗口高宽 i_window_len 不一致，为矩阵相乘关系
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare  
  v_window_ex_len         -- int[2]        :=   array[array_length(i_window_ex, array_ndims(i_window_ex) - 1), array_length(i_window_ex, array_ndims(i_window_ex))];
                          -- alias for i_window_ex_len;
                             int[2]        :=   i_window_ex_len[array_length(i_window_ex_len, 1) - 1 : array_length(i_window_ex_len, 1)];
  v_background_len_ex        int[2]        :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                          -- int[2]        :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]        :=   array[i_window_len_heigh, v_window_ex_len[1]];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif array_ndims(i_window_ex) <> 2
    -- then 
    --   raise exception 'no method for such i_window_ex length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_ex), v_window_ex_len[1], v_window_ex_len[2];
    elsif (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif v_window_len[2] <> v_window_ex_len[1]
    then 
      raise exception 'imperfect window at 2d and window_x at 1d.';
    elsif i_dloss_ddepdt is null 
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) <> ((coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) * v_window_len[1]
      or array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) <> ((coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) * v_window_ex_len[2]
    then 
      raise exception 'imperfect i_dloss_ddepdt.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_ex[1] + i_padding[2], v_background_len_ex[2] + i_padding[4]], 
      i_padding_value
    );
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      select 
        sm_sc.fa_mx_sum
        ( 
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_ex_len[1] + 1 : a_cur_y * v_window_ex_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          |**|
          sm_sc.fv_d_prod_mx_2
          (
            i_background[(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                        [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
          )
          -- 矩阵点算求导
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          ( 
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_ex_len[1] + 1 : a_cur_y * v_window_ex_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            |**|
            sm_sc.fv_d_prod_mx_2
            (
              i_background
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
            -- 矩阵点算求导
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_ex_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          sm_sc.fa_mx_sum
          ( 
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_ex_len[1] + 1 : a_cur_y * v_window_ex_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
            |**|
            sm_sc.fv_d_prod_mx_2
            (
              i_background
                [ : ]
                [ : ]
                [(a_cur_y - 1) * i_stride[1] + 1 : (a_cur_y - 1) * i_stride[1] + v_window_len[1]]
                [(a_cur_x - 1) * i_stride[2] + 1 : (a_cur_x - 1) * i_stride[2] + v_window_len[2]]
            )
            -- 矩阵点算求导
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len_ex[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len_ex[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_ex_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
-- (
--   array[[1,2,3,4,5],[3,4,5,6,7],[2,4,6,8,5],[7,5,3,4,8]] :: float[],   -- i_background            anyarray  
--   -- array[4, 5]                   ,                           -- v_background_len_ex        int[2]     
--   3                                ,                           -- v_window_ex_len            int 
--   -- array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],                           -- i_window_ex                anyarray  
--   array[3, 4]                   ,                           -- v_window_ex_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
-- (
--   array              
--   [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--   ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--   ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] :: float[],     -- i_background            anyarray  
--   -- array[6, 9]                   ,                           -- v_background_len_ex        int[2]     
--   3                   ,                                      -- v_window_ex_len            int
--   -- array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],                        -- i_window_ex                anyarray  
--   array[3, 4]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
-- (
--   array  
--   [                      
--     [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   , [[-1, 2, 3, 4, 5, -6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, -24, 25, 26, -27, 28, 29]                       
--     ,[31, -32, -33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, -43, 44, 45, 46, 47, 48, -49]           
--     ,[51, -52, 53, 54, -55, 56, 57, -58, 59]] 
--   ] :: float[],     -- i_background            anyarray  
--   -- array[2, 6, 9]                   ,                           -- v_background_len_ex        int[2]     
--   3                             ,                           -- v_window_ex_len            int    
--   -- array[[1,4,7,6],[2,5,8,7],[3,6,9,8]] :: float[],       -- i_window_ex                anyarray  -- array[[[1,4,7,6],[2,5,8,7],[3,6,9,8]],[[1,4,7,6],[2,5,8,7],[3,6,9,8]]] :: float[]
--   array[3, 4]                   ,                           -- v_window_len            int[2]       -- array[2, 3, 4]
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 12]                   ,                           -- i_depdt_var_len         int[2]    -- array[2, 6, 12]
--   array       
--   [                  
--     [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]]  
--   , [[1, 2, 3, 4, 5, 6, 7, 8, 9, -1, -2, -3]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19, -11, -12, -13]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29, -21, -22, -23]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39, -31, -32, -33]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49, -41, -42, -43]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59, -51, -52, -53]] 
--   ]:: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  ,                                      -- i_padding               int[4]    
--   0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_sub_dloss_dindepdt_1
-- (
-- --   anyarray
-- --  , 
--   int[]
-- -- , anyarray
-- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_sub_dloss_dindepdt_1
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
begin
  -- if i_background_len is null
  -- then 
  --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    -- elsif -- -- i_background is not null and 
    --   array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    -- elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_2d
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_length(i_window_len, 1) = 3 and i_window_len[1] <> array_length(i_dloss_ddepdt, 1)
      or array_length(i_window_len, 1) = 4 and (i_window_len[1] <> array_length(i_dloss_ddepdt, 1) or i_window_len[2] <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_1
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[2, 1, 6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]      -- array[2, 1, 3, 3]
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--       [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--     ]
--   , [
--       [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] 
--     ]
--   ] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_conv_sub_dloss_dindepdt_2
-- (
-- --   anyarray
-- , int[]
-- -- , anyarray
-- , int[]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_conv_sub_dloss_dindepdt_2
(
  -- i_background            anyarray           ,
  i_background_len        int[]              ,
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]              ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_ex      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
  v_ret                      i_dloss_ddepdt%type;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- if i_background_len is null
    -- then 
    --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
    -- end if;
    -- if v_window_len is null
    -- then 
    --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
    -- end if;
    -- if i_depdt_var_len is null
    -- then 
    --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
    -- end if;
    -- 
    -- -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
      or array_ndims(i_background) not between 2 and 4 
      or array_ndims(i_dloss_ddepdt) <> array_ndims(i_background)
    then 
      raise exception 'no method for such ndims!';
    -- elsif i_window is not null and array_ndims(i_window) <> 2
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    -- els
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_ex[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_ex[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_ex
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then 
    return     
    (    
      select 
        -`
        sm_sc.fa_mx_sum
        (
          i_dloss_ddepdt[(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
                        [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
        )
      from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
        , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
    )
    ;
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_ret := 
      (    
        select 
          -`
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_ele_3d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), 1, 1]
          )
        , array[1, 2]
        , 2
        )
      ;
    else 
      return v_ret;
    end if;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_ret := 
      (    
        select 
          -`
          sm_sc.fa_mx_sum
          (
            i_dloss_ddepdt
              [ : ]
              [ : ]
              [(a_cur_y - 1) * v_window_len[1] + 1 : a_cur_y * v_window_len[1]]
              [(a_cur_x - 1) * v_window_len[2] + 1 : a_cur_x * v_window_len[2]] -- 损失函数导数的独立窗口分组
          )
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_y(a_cur_y)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_x(a_cur_x)
      );
    
    if array_length(i_window_len, 1) = 2
    then
      return 
        sm_sc.fv_mx_slice_4d_2_2d
        (
          sm_sc.fv_aggr_slice_sum
          (
            v_ret
          , array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1]
          )
        , array[1, 2]
        , array[1, 1]
        )
      ;
    else 
      return v_ret;
    end if;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,    -- array[2, 3, 3]          -- i_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array   
--   [           
--     [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ]
--   , [
--      [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--     ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--     ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--     ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--     ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--     ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--     ]
--   ]
--   :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_conv_sub_dloss_dindepdt_2
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,    -- array[2, 2, 3, 3]   -- i_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array   
--   [
--     [           
--       [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     , [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     ]
--   , [           
--       [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     , [
--        [1, 2, 3, 4, 5, 6, 7, 8, 9]                
--       ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--       ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--       ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--       ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--       ,[51, 52, 53, 54, 55, 56, 57, 58, 59]
--       ]
--     ]
--   ]
--   :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );
-- -- 参考：https://blog.csdn.net/LoseInVain/article/details/98451913

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_avg_dloss_dindepdt(int[2], float[], int[2], int[4]);
create or replace function sm_sc.fv_d_pool_avg_dloss_dindepdt
(
  -- -- i_background_len     int[]                                               ,  -- 背景二维平面的高宽大小
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_dloss_ddepdt       float[]                                              ,  -- 即已求出的损失函数对 y 的导数矩阵
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]          -- 上下左右补齐行数/列数
)
returns float[][]
as
$$
declare
  v_dloss_dindepdt         float[][]       ;    --   intact 之后的新矩阵
  v_dloss_ddepdt_len_heigh int             :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) ;
  v_dloss_ddepdt_len_width int             :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) ;
  -- -- v_background_len         -- int[2]       :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- --                          int[2]          :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_y                      int             :=   (v_dloss_ddepdt_len_heigh - 1) * i_stride[1] + i_window_len[1];  -- --  coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0);     --   新矩阵高
  v_x                      int             :=   (v_dloss_ddepdt_len_width - 1) * i_stride[2] + i_window_len[2];  -- --  coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0);     --   新矩阵宽
  v_cur_y                  int             ;     --   新矩阵高游标
  v_cur_x                  int             ;     --   新矩阵宽游标
  v_windows_cnt_reciproca  float           :=   1.0 :: float/ (v_dloss_ddepdt_len_heigh * v_dloss_ddepdt_len_width * i_window_len[1] * i_window_len[2]);     -- 开窗数量的倒数
begin

  -- -- -- 审计
  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then
  -- --   if v_dloss_ddepdt_len_heigh <> (v_y - i_window_len[1]) / i_stride[1] + 1
  -- --     or v_dloss_ddepdt_len_width <> (v_x - i_window_len[2]) / i_stride[2] + 1
  -- --   then
  -- --     raise exception 'unmatch length between y and dloss/dy.';
  -- --   end if;
  -- -- end if;

  if array_ndims(i_dloss_ddepdt) = 2
  then 
    v_dloss_dindepdt := sm_sc.fv_new(0.0 :: float, array[v_y, v_x]);
    
    v_cur_y := v_y;
    while v_cur_y >= 1
    loop 
      v_cur_x := v_x;
      while v_cur_x >= 1
      loop 
        v_dloss_dindepdt[v_cur_y][v_cur_x] := 
          coalesce 
          (
            (
              select
                sum
                (    
                  v_windows_cnt_reciproca * i_dloss_ddepdt[a_cur_y_dloss_ddepdt][a_cur_x_dloss_ddepdt]     --   dloss_dindepdt 当个元素对应的 dloss_ddepdt 元素                 
                ) 
              from 
                -- 以 y 方向为例推导
                -- 对于 i_indepdt 的第 v_cur_y 个元素，寻找其在 i_dloss_ddepdt 对应的  a_cur_y_dloss_ddepdt 
                -- a_cur_y_dloss_ddepdt 这个滑窗之前存在 (a_cur_y_dloss_ddepdt - 1) 个 i_stride[1]，且 v_cur_y 落入该窗口
                -- 即: (a_cur_y_dloss_ddepdt - 1) * i_stride[1] + 1 <= v_cur_y <= (a_cur_y_dloss_ddepdt - 1) * i_stride[1] + i_window_len
                --   => v_cur_y - i_window_len <= (a_cur_y_dloss_ddepdt - 1) * i_stride[1] <= v_cur_y - 1
                --   => (v_cur_y - i_window_len) / i_stride[1] + 1 <= a_cur_y_dloss_ddepdt <= (v_cur_y - 1) / i_stride[1] + 1
                generate_series(greatest(ceil((v_cur_y :: decimal - i_window_len[1]) / i_stride[1]) + 1, 1), least((v_cur_y - 1) / i_stride[1] + 1, v_y), 1) tb_a_cur_y(a_cur_y_dloss_ddepdt),  
                generate_series(greatest(ceil((v_cur_x :: decimal - i_window_len[2]) / i_stride[2]) + 1, 1), least((v_cur_x - 1) / i_stride[2] + 1, v_x), 1) tb_a_cur_x(a_cur_x_dloss_ddepdt)
            )
            , 0.0
          )
          ;
        v_cur_x := v_cur_x - 1;
      end loop;
      v_cur_y := v_cur_y - 1;
    end loop;
    
    return -- 还原为排除 intact 的子矩阵
      v_dloss_dindepdt[1 + i_padding[1] : v_y - i_padding[2]][1 + i_padding[3] : v_x - i_padding[4]]
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_d_pool_avg_dloss_dindepdt
          (
            -- --   i_background_len,
            i_window_len 
          , sm_sc.fv_mx_slice_3d_2_2d
            (
              i_dloss_ddepdt[a_cur_y : a_cur_y]
            , 1
            )    
          , i_stride       
          , i_padding    
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_dloss_ddepdt, 1)) tb_a_cur_y(a_cur_y)
    );
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_d_pool_avg_dloss_dindepdt
            (
              -- --   i_background_len,
              i_window_len  
            , sm_sc.fv_mx_slice_4d_2_2d
              (
                i_dloss_ddepdt[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )   
            , i_stride       
            , i_padding    
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_dloss_ddepdt, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_dloss_ddepdt, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_avg_dloss_dindepdt
--   (
--      -- -- array[5, 7],
--      array[3, 3]
--    , array[[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_pool_avg_dloss_dindepdt
--   (
--      -- -- array[5, 6],
--      array[3, 3]
--    , array[[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );

-- select sm_sc.fv_d_pool_avg_dloss_dindepdt
--   (
--      -- -- array[5, 6]
--      array[3, 3]
--    , array
--       [
--         [[1.1, 1.1, 1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--       , [[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[-1.1, 1.1, -1.1]
--          ]
--       ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );

-- select sm_sc.fv_d_pool_avg_dloss_dindepdt
--   (
--     -- array[5, 6],
--     array[3, 3]
--   , array
--     [
--       [
--         [[1.1, 1.1, 1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--       , [[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[-1.1, 1.1, -1.1]
--          ]
--       ]
--     , [
--         [[1.1, 1.1, -1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[-1.1, 1.1, 1.1]
--          ]
--       , [[-1.1, -1.1, 1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[-1.1, 1.1, -1.1]
--          ]
--       ]
--     ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );
-- -- 卷积的矩阵乘法实现：
-- --   https://www.cnblogs.com/shine-lee/p/10775831.html
-- --   https://blog.csdn.net/qq_40263477/article/details/104979609
-- -- 卷积的 fft 实现
-- --   https://blog.csdn.net/qq_37527608/article/details/120922348
-- -- 卷积与矩阵乘法算法调优：
-- --  https://blog.csdn.net/lizhengx/article/details/83246833
-- -- 卷积求导
-- --  https://blog.csdn.net/wangjianyu0115/article/details/62222301

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_avg_dloss_dindepdt_ex(int[2], float[][], int[2], int[4]);
create or replace function sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
(
  -- i_array_len          int[2]                                               ,  -- 原矩阵(卷积第一目参数)高宽，
  i_window_len         int[2]                                     ,  -- 卷积核窗口
  i_dloss_ddepdt           float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]          -- 上下左右补齐行数/列数
)
returns float[][]
as
$$
declare
  -- -- v_dloss_dindepdt               float[];     --   intact 之后的新矩阵
  -- -- v_len_y                  int             :=   coalesce(i_padding[1], 0) + i_array_len[1] + coalesce(i_padding[2], 0);     --   新矩阵高
  -- -- v_len_x                  int             :=   coalesce(i_padding[3], 0) + i_array_len[2] + coalesce(i_padding[4], 0);     --   新矩阵宽
  -- -- v_cur_y                  int             ;     --   新矩阵高游标
  -- -- v_cur_x                  int             ;     --   新矩阵宽游标
  v_windows_cnt_reciproca  float  :=   
    1.0 :: float/ (array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) * array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) * i_window_len[1] * i_window_len[2]);     -- 开窗数量的倒数
  -- -- v_window_len_y           int             :=   array_length(i_window, array_ndims(i_window) - 1);
  -- -- v_window_len_x           int             :=   array_length(i_window, array_ndims(i_window));
  v_return                 float[]  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度
    if array_ndims(i_dloss_ddepdt) > 4
    then 
      raise exception 'no method for such length!  Dims: %;', array_dims(i_dloss_ddepdt);
    -- -- elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1)
    -- --   or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_dloss_ddepdt, 1) or array_length(i_window, 2) <> array_length(i_dloss_ddepdt, 2))
    -- -- then 
    -- --   raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  -- -- if array_ndims(i_dloss_ddepdt) - array_ndims(i_window) = 2
  -- -- then 
  -- --   i_window = array[[i_window]];
  -- -- elsif array_ndims(i_dloss_ddepdt) - array_ndims(i_window) = 1
  -- -- then 
  -- --   i_window = array[i_window];
  -- -- end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    v_return :=  
      sm_sc.fv_conv_2d_im2col_py
      (
        array_fill(0.0 :: float, array[i_window_len[1] - 1, 1])  -- 上方填零扩展
        |-||
        (
          sm_sc.fv_sample_y
          (
            array_fill(0.0 :: float, array[1, i_window_len[2] - 1])  -- 左侧填零扩展
            ||||
            (
              sm_sc.fv_sample_x
              (
                i_dloss_ddepdt
                , 1
                , 1     
                , array[int4range(1, i_stride[2], '[]')] -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
                , array[array[0.0 :: float]]
              )
            )[ : ][ : (array_length(i_dloss_ddepdt, 2) - 1) * i_stride[2] + 1]     -- 结尾稀释截断，相当于仅在原矩阵间隙周期插值
            ||||
            array_fill(0.0 :: float, array[1, i_window_len[2] - 1])  -- 右侧填零扩展
            , 1
            , 1     
            , array[int4range(1, i_stride[1], '[]')]  -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
            , array[array[0.0 :: float]]
          )
        )[ : (array_length(i_dloss_ddepdt, 1) - 1) * i_stride[1] + 1][ : ]
        |-||
        array_fill(0.0 :: float, array[i_window_len[1] - 1, 1])   -- 下方填零扩展
      , array_fill(1.0 :: float, i_window_len)     -- i_window_back
      -- -- , 0.0         -- i_bias_back
      -- -- , array[1, 1] -- i_stride_back
      )
    ;

    return 
    (
      v_return[1 + i_padding[1] : array_length(v_return, 1) - i_padding[2]][1 + i_padding[3] : array_length(v_return, 2) - i_padding[4]]
      *` v_windows_cnt_reciproca
    ) :: float[]
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    v_return :=  
      sm_sc.fv_conv_2d_im2col_py
      (
        array_fill(0.0 :: float, array[1, i_window_len[1] - 1, 1])  -- 上方填零扩展
        |-||
        (
          sm_sc.fv_sample_x
          (
            array_fill(0.0 :: float, array[1, 1, i_window_len[2] - 1])  -- 左侧填零扩展
            ||||
            (
              sm_sc.fv_sample_x3
              (
                i_dloss_ddepdt
                , 1
                , 1     
                , array[int4range(1, i_stride[2], '[]')] -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
                , array[[[0.0 :: float]]]
              )
            )[ : ][ : ][ : (array_length(i_dloss_ddepdt, 3) - 1) * i_stride[2] + 1]     -- 结尾稀释截断，相当于仅在原矩阵间隙周期插值
            ||||
            array_fill(0.0 :: float, array[1, 1, i_window_len[2] - 1])  -- 右侧填零扩展
            , 1
            , 1     
            , array[int4range(1, i_stride[1], '[]')]  -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
            , array[[[0.0 :: float]]]
          )
        )[ : ][ : (array_length(i_dloss_ddepdt, 2) - 1) * i_stride[1] + 1][ : ]
        |-||
        array_fill(0.0 :: float, array[1, i_window_len[1] - 1, 1])   -- 下方填零扩展
      , array_fill(1.0 :: float, i_window_len)     -- i_window_back
      -- -- , 0.0         -- i_bias_back
      -- -- , array[1, 1] -- i_stride_back
      )
    ;

    return 
    (
      v_return[ : ][1 + i_padding[1] : array_length(v_return, 2) - i_padding[2]][1 + i_padding[3] : array_length(v_return, 3) - i_padding[4]]
      *` v_windows_cnt_reciproca
    ) :: float[]
    ;
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    v_return :=  
      sm_sc.fv_conv_2d_im2col_py
      (
        array_fill(0.0 :: float, array[1, 1, i_window_len[1] - 1, 1])  -- 上方填零扩展
        |-||
        (
          sm_sc.fv_sample_x3
          (
            array_fill(0.0 :: float, array[1, 1, 1, i_window_len[2] - 1])  -- 左侧填零扩展
            ||||
            (
              sm_sc.fv_sample_x4
              (
                i_dloss_ddepdt
                , 1
                , 1     
                , array[int4range(1, i_stride[2], '[]')] -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
                , array[[[[0.0 :: float]]]]
              )
            )[ : ][ : ][ : ][ : (array_length(i_dloss_ddepdt, 4) - 1) * i_stride[2] + 1]     -- 结尾稀释截断，相当于仅在原矩阵间隙周期插值
            ||||
            array_fill(0.0 :: float, array[1, 1, 1, i_window_len[2] - 1])  -- 右侧填零扩展
            , 1
            , 1     
            , array[int4range(1, i_stride[1], '[]')]  -- 前向 i_stride - 1 当作求导入参的稀释插入宽度
            , array[[[[0.0 :: float]]]]
          )
        )[ : ][ : ][ : (array_length(i_dloss_ddepdt, 3) - 1) * i_stride[1] + 1][ : ]
        |-||
        array_fill(0.0 :: float, array[1, 1, i_window_len[1] - 1, 1])   -- 下方填零扩展
      , array_fill(1.0 :: float, i_window_len)     -- i_window_back
      -- -- , 0.0         -- i_bias_back
      -- -- , array[1, 1] -- i_stride_back
      )
    ;
    
    return 
    (
      v_return[ : ][ : ][1 + i_padding[1] : array_length(v_return, 3) - i_padding[2]][1 + i_padding[3] : array_length(v_return, 4) - i_padding[4]]
      *` v_windows_cnt_reciproca
    ) :: float[]
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
--   (
--      -- -- array[5, 7],
--      array[3, 3]
--    , array[[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
--   (
--      -- -- array[5, 7],
--      array[3, 3]
--    , array[[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );

-- select sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
--   (
--      -- -- array[5, 7]
--      array[3, 3]
--    , array
--       [
--         [[1.1, 1.1, 1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--       , [[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[-1.1, 1.1, -1.1]
--          ]
--       ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );

-- select sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
--   (
--     -- array[5, 7],
--     array[3, 3]
--   , array
--     [
--       [
--         [[1.1, 1.1, 1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--          ]
--       , [[1.1, -1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[-1.1, 1.1, -1.1]
--          ]
--       ]
--     , [
--         [[1.1, 1.1, -1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[-1.1, 1.1, 1.1]
--          ]
--       , [[-1.1, -1.1, 1.1]
--         ,[1.1, -1.1, 1.1]
--         ,[-1.1, 1.1, -1.1]
--          ]
--       ]
--     ]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt(int, float[], int[2], int[2], int[4]);
create or replace function sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt
(
  -- -- i_indepdt_grp_x_x_len          int                                                  ,  -- i_indepdt_1d_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本，本参数为 i_indepdt_grp_x 的宽度
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_dloss_ddepdt_1d_grp        float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵 的 降维后样本集合
  i_window_len             int[2]                                               ,  -- 卷积核窗口矩阵高宽
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]          -- 上下左右补齐行数/列数
)
returns float[][]
as
$$
declare 
  v_dloss_ddepdt_array_len_2   int :=  (i_padding[3] + i_1d_2_2d_cnt_per_grp + i_padding[4] - i_window_len[2]) / i_stride[2] + 1;
  -- -- v_array_y_len            int :=  i_indepdt_grp_x_x_len / i_1d_2_2d_cnt_per_grp;
begin
  -- 审计二维长度
  if array_ndims(i_dloss_ddepdt_1d_grp) <> 2
  then 
    raise exception 'no method for such length, i_dloss_ddepdt_1d_grp!  Dims: %;', array_ndims(i_dloss_ddepdt_1d_grp);
  -- -- elsif i_indepdt_grp_x_x_len % i_1d_2_2d_cnt_per_grp <> 0
  -- -- then
  -- --   raise exception 'imperfect window at grp_x_cnt.';
  else
    return 
    (
      select 
        sm_sc.fa_mx_concat_y
        (
          array 
          [
            sm_sc.fv_mx_ele_2d_2_1d
            (
              sm_sc.fv_d_pool_avg_dloss_dindepdt
              (
                -- -- array[v_array_y_len, i_1d_2_2d_cnt_per_grp]   , 
                i_window_len                                  ,
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_dloss_ddepdt_1d_grp[a_cur_y : a_cur_y][ : ]
                  , v_dloss_ddepdt_array_len_2
                )                                             , 
                i_stride                                      ,
                i_padding      
              )
            )
          ]
          order by a_cur_y
        )
      from generate_series(1, array_length(i_dloss_ddepdt_1d_grp, 1)) tb_a_cur_y(a_cur_y)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt
--   (
--     -- 5*7,
--     7,
--     array
--     [
--       array[1.1, 1.1, 1.1
--        ,1.1, 1.1, 1.1
--        ],
--       array[2.1, 3.1, 0.1
--        ,1.1, -1.1, -0.1
--        ],
--       array[0.1, -2.1, 1.1
--        ,-1.1, 3.1, 2.1
--        ]
--     ]
--    , array[3, 3]
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt
--   (
--     -- 5*6
--     6,
--    array
--    [
--      array[1.1, 1.1, 1.1
--       ,1.1, 1.1, 1.1
--       ,1.1, 1.1, 1.1
--       ],
--      array[2.1, 3.1, 0.1
--       ,1.1, -1.1, -0.1
--       ,0.1, 2.1, -0.1
--       ],
--      array[0.1, -2.1, 1.1
--       ,-1.1, 3.1, 2.1
--       ,-1.1, -1.1, 0.1
--       ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--   );
-- -- 参考：https://blog.csdn.net/LoseInVain/article/details/98451913

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_max_dloss_dindepdt(float[], int[2], float[], float[], int[2], int[4], float);
create or replace function sm_sc.fv_d_pool_max_dloss_dindepdt
(
  i_indepdt                  float[]                                   ,  -- 原矩阵
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_dloss_ddepdt       float[]                                   ,  -- 即已求出的损失函数对 y 的导数矩阵
  i_depdt                  float[][]  default  null                    ,  -- 即已求出的算子结果矩阵
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float      default  '-inf'       -- 补齐填充元素值
)
returns float[][]
as
$$
declare
  v_dloss_dindepdt               float[][] ;       --   intact 之后的新矩阵
  v_indepdt                      float[][] ;
  v_y                      int             :=   coalesce(i_padding[1], 0) + array_length(i_indepdt, 1) + coalesce(i_padding[2], 0);     --   新矩阵高
  v_x                      int             :=   coalesce(i_padding[3], 0) + array_length(i_indepdt, 2) + coalesce(i_padding[4], 0);     --   新矩阵宽
  v_cur_y                  int             ;     --   新矩阵高游标
  v_cur_x                  int             ;     --   新矩阵宽游标
  v_dloss_ddepdt_len_heigh     int    :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) ;
  v_dloss_ddepdt_len_width     int    :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) ;
  v_windows_cnt_reciproca  float  :=   1.0 :: float/ (v_dloss_ddepdt_len_heigh * v_dloss_ddepdt_len_width);     -- 开窗数量的倒数
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(v_dloss_dindepdt) <> array_ndims(i_indepdt)
    then 
      raise exception 'unmatch ndims between v_dloss_dindepdt and i_indepdt';
    elsif array_ndims(v_dloss_dindepdt) = 3 and v_dloss_ddepdt_len_heigh <> array_length(i_indepdt, 1)
      or array_ndims(v_dloss_dindepdt) = 4 and (v_dloss_ddepdt_len_heigh <> array_length(i_indepdt, 1) or v_dloss_ddepdt_len_width <> array_length(i_indepdt, 2))
    then 
      raise exception 'unmatch length at 1d of 3d or 1d / 2d of 4d';
    elsif array_dims(i_dloss_ddepdt) <> array_dims(i_depdt)
      or v_dloss_ddepdt_len_heigh <> (v_y - i_window_len[1]) / i_stride[1] + 1
      or v_dloss_ddepdt_len_width <> (v_x - i_window_len[2]) / i_stride[2] + 1
    then
      raise exception 'unmatch length between y and dloss/dy.';
    end if;
  end if;

  if array_ndims(i_dloss_ddepdt) = 2
  then 
    v_indepdt := 
      sm_sc.fv_augmented
      (
        i_indepdt, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[array_length(i_indepdt, 1) + i_padding[2], array_length(i_indepdt, 2) + i_padding[4]], 
        i_padding_value
      );
  
    if i_depdt is null
    then
      i_depdt := sm_sc.fv_pool_max(i_indepdt, i_window_len, i_stride, i_padding, i_padding_value);
    end if;
    
    v_dloss_dindepdt := sm_sc.fv_new(0.0 :: float, array[v_y, v_x]);
    
    v_cur_y := v_y;
    while v_cur_y >= 1
    loop 
      v_cur_x := v_x;
      while v_cur_x >= 1
      loop 
        v_dloss_dindepdt[v_cur_y][v_cur_x] := 
          coalesce 
          (
            (
              select
                sum
                (
                  case when i_depdt[a_cur_y_dloss_ddepdt][a_cur_x_dloss_ddepdt] = v_indepdt[v_cur_y][v_cur_x] 
                    then v_windows_cnt_reciproca else 0.0 end                   --  dloss_dindepdt 当个元素是否是当个窗口的最大值      
                           * i_dloss_ddepdt[a_cur_y_dloss_ddepdt][a_cur_x_dloss_ddepdt]     --   dloss_dindepdt 当个元素对应的 dloss_ddepdt 元素                 
                ) 
              from 
                -- 以 y 方向为例推导
                -- 对于 v_indepdt 的第 v_cur_y 个元素，寻找其在 i_dloss_ddepdt 对应的  a_cur_y_dloss_ddepdt 
                -- a_cur_y_dloss_ddepdt 这个滑窗之前存在 (a_cur_y_dloss_ddepdt - 1) 个 i_stride[1]，且 v_cur_y 落入该窗口
                -- 即: (a_cur_y_dloss_ddepdt - 1) * i_stride[1] + 1 <= v_cur_y <= (a_cur_y_dloss_ddepdt - 1) * i_stride[1] + i_window_len
                --   => v_cur_y - i_window_len <= (a_cur_y_dloss_ddepdt - 1) * i_stride[1] <= v_cur_y - 1
                --   => (v_cur_y - i_window_len) / i_stride[1] + 1 <= a_cur_y_dloss_ddepdt <= (v_cur_y - 1) / i_stride[1] + 1
                generate_series(greatest(ceil((v_cur_y :: decimal - i_window_len[1]) / i_stride[1]) + 1, 1), least((v_cur_y - 1) / i_stride[1] + 1, v_y), 1) tb_a_cur_y(a_cur_y_dloss_ddepdt),  
                generate_series(greatest(ceil((v_cur_x :: decimal - i_window_len[2]) / i_stride[2]) + 1, 1), least((v_cur_x - 1) / i_stride[2] + 1, v_x), 1) tb_a_cur_x(a_cur_x_dloss_ddepdt)
            )
            , 0.0
          )
          ;
        v_cur_x := v_cur_x - 1;
      end loop;
      v_cur_y := v_cur_y - 1;
    end loop;
    
    return -- 还原为排除 intact 的子矩阵
      v_dloss_dindepdt[1 + i_padding[1] : v_y - i_padding[2]][1 + i_padding[3] : v_x - i_padding[4]]
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_d_pool_max_dloss_dindepdt
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_indepdt[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len  
          , sm_sc.fv_mx_slice_3d_2_2d
            (
              i_dloss_ddepdt[a_cur_y : a_cur_y]
            , 1
            )
          , sm_sc.fv_mx_slice_3d_2_2d
            (
              i_depdt[a_cur_y : a_cur_y]
            , 1
            )
          , i_stride       
          , i_padding  
          , i_padding_value  
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_dloss_ddepdt, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_dloss_ddepdt) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_d_pool_max_dloss_dindepdt
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_indepdt[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len 
            , sm_sc.fv_mx_slice_4d_2_2d
              (
                i_dloss_ddepdt[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , sm_sc.fv_mx_slice_4d_2_2d
              (
                i_depdt[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_stride       
            , i_padding  
            , i_padding_value 
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_dloss_ddepdt, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_dloss_ddepdt, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
  
  end if;
end
$$
language plpgsql stable
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_max_dloss_dindepdt
--   (
--     array[array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0]
--         , array[10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0]
--         , array[100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0]
--         , array[-1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , array[-10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0]
--          ]
--    , array[3, 3]
--    , array[array[1.1, 1.1, 1.1]
--         ,array[1.1, 1.1, 1.1]
--          ]
--    , null
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_pool_max_dloss_dindepdt
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ]::float[][]
--    , array[3, 3]
--    , array[array[1.1, 1.1, 1.1]
--         ,array[1.1, 1.1, 1.1]
--         ,array[1.1, 1.1, 1.1]
--          ]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_d_pool_max_dloss_dindepdt
--   (
--     array
--       [
--         [
--           [1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [1,2,3,-4,-5,6]
--         , [10,20,30,40,-50,60]
--         , [100,200,-300,400,500,600]
--         , [-1,2,-3,-4,-5,6]
--         , [10,20,-30,40,-50,-60]
--         ]
--       ]
--    , array[3, 3]
--    , array
--       [
--         [
--          [1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ]
--       , [
--          [1.1, -1.1, 1.1]
--         ,[1.1, 1.1, -1.1]
--         ,[-1.1, 1.1, 1.1]
--         ]
--       ]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_d_pool_max_dloss_dindepdt
--   (
--     array
--     [
--       [
--         [
--           [1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [1,2,3,-4,-5,6]
--         , [10,20,30,40,-50,60]
--         , [100,200,-300,400,500,600]
--         , [-1,2,-3,-4,-5,6]
--         , [10,20,-30,40,-50,-60]
--         ]
--       ]
--     , [
--         [
--           [-1,2,-3,4,5,6]
--         , [10,20,-30,40,-50,60]
--         , [100,-200,-300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [1,-2,3,4,-5,6]
--         , [10,20,30,40,-50,60]
--         , [100,-200,-300,400,-500,-600]
--         , [-1,2,-3,-4,-5,6]
--         , [10,20,30,40,-50,-60]
--         ]
--       ]
--     ]
--    , array[3, 3]
--    , array
--      [
--        [
--          [
--           [1.1, 1.1, 1.1]
--          ,[1.1, 1.1, 1.1]
--          ,[1.1, 1.1, 1.1]
--          ]
--        , [
--           [1.1, -1.1, 1.1]
--          ,[1.1, 1.1, -1.1]
--          ,[-1.1, 1.1, 1.1]
--          ]
--        ]
--      , [
--          [
--           [1.1, -1.1, 1.1]
--          ,[1.1, -1.1, 1.1]
--          ,[1.1, -1.1, 1.1]
--          ]
--        , [
--           [1.1, 1.1, 1.1]
--          ,[-1.1, -1.1, -1.1]
--          ,[-1.1, 1.1, 1.1]
--          ]
--        ]
--      ]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );
-- -- 参考：https://blog.csdn.net/LoseInVain/article/details/98451913

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_max_dloss_dindepdt_ex(float[], int[2], float[], float[], int[2], int[4], float);
create or replace function sm_sc.fv_d_pool_max_dloss_dindepdt_ex
(
  i_indepdt                  float[]                                   ,  -- 原矩阵
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_dloss_ddepdt       float[]                                   ,  -- 即已求出的损失函数对 y 的导数矩阵
  i_depdt                  float[][]  default  null                    ,  -- 即已求出的算子结果矩阵
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float      default  '-inf'       -- 补齐填充元素值
)
returns float[][]
as
$$
declare
  -- v_indepdt                                float[]   ;
  v_strided_dloss_ddepdt                   float[]   ;     --   intact 之后的新矩阵
  v_strided_dloss_ddepdt_width_destrided   float[]   ;
  v_strided_dloss_ddepdt_heigh_destrided   float[]   ;
  v_indepdt_len_heigh                      int       :=   coalesce(i_padding[1], 0) + array_length(i_indepdt, array_ndims(i_indepdt) - 1) + coalesce(i_padding[2], 0);     --   新矩阵高
  v_indepdt_len_width                      int       :=   coalesce(i_padding[3], 0) + array_length(i_indepdt, array_ndims(i_indepdt)) + coalesce(i_padding[4], 0);     --   新矩阵宽
  v_dloss_ddepdt_len_heigh                 int       :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1) ;
  v_dloss_ddepdt_len_width                 int       :=   array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt)) ;
  v_windows_cnt_reciproca  float  :=   1.0 :: float/ (v_dloss_ddepdt_len_heigh * v_dloss_ddepdt_len_width);     -- 开窗数量的倒数
  v_cnt_overlap_width                      int       ;
  v_cnt_overlap_heigh                      int       ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_dloss_ddepdt) <> array_ndims(i_indepdt)
    then 
      raise exception 'unmatch ndims between i_dloss_ddepdt and i_indepdt';
    elsif array_dims(i_dloss_ddepdt) <> array_dims(i_depdt)
      or v_dloss_ddepdt_len_heigh <> (v_y - i_window_len[1]) / i_stride[1] + 1
      or v_dloss_ddepdt_len_width <> (v_x - i_window_len[2]) / i_stride[2] + 1
    then
      raise exception 'unmatch length between y and dloss/dy.';
    end if;
  end if;

  if i_depdt is null
  then
    i_depdt := sm_sc.fv_pool_max(i_indepdt, i_window_len, i_stride, i_padding, i_padding_value);
  end if;
  
  -- 对于窗口长度大于步长的场景，将相邻窗口以错位分层的方式分组，再对分组做矩阵颗粒度合计聚合。如下是一维长度为 13、窗口宽度为 5、步长为 2 的错位分层分组示意图:
  -- 
  -- 以 pool_none 算子操作为例: 
  -- 假定:
  --   i_indepdt = array[1,2,3,4,5,6,7,8,9,10,11,12,13];
  --   i_dloss_ddepdt = array[-1,-2,-3,-4,-5]
  -- 
  -- 采样，得到各窗口轨迹:
  --   i_indepdt_window = 
  --     array
  --     [
  --       [ 1, 2, 3, 4, 5], [ 7, 8, 9,10,11]
  --     ,       [ 3, 4, 5, 6, 7], [ 9,10,11,12,13]
  --     ,             [ 5, 6, 7, 8, 9]
  --     ];
  -- 
  -- 窗口导数值映射至窗口内每个元素位置，按照窗口轨迹，将重叠窗口排版至不同分组
  --   i_dloss_ddepdt_repeat = 
  --     array 
  --     [
  --     , -1, -1, -1, -1, -1    , -4, -4, -4, -4, -4
  --             , -2, -2, -2, -2, -2    , -5, -5, -5, -5, -5
  --                     , -3, -3, -3, -3, -3
  --     ];
  -- 
  -- 不同分组矩阵粒度聚合合计，便能合计出某个元素被多个窗口划过重叠部分的每个导数分量
  --   i_dloss_dindepdt = 
  --     array 
  --     [ -1, -1, -3, -3, -6, -5, -9, -7, -12,-9, -9, -5, -5];
  -- 


  v_strided_dloss_ddepdt :=
    sm_sc.fv_repeat_axis_py
    (
      sm_sc.fv_repeat_axis_py
      (
        i_dloss_ddepdt *` v_windows_cnt_reciproca
      , -1
      , i_window_len[2]
      )
    , -2
    , i_window_len[1]
    )
    *`
    ( -- 寻找到各个滑动区域的 max 位置，置 1
      (
        sm_sc.fv_pool_none_py
        (
          i_indepdt
        , i_window_len
        , i_stride
        , i_padding
        , i_padding_value
        ) 
        ==` 
        sm_sc.fv_repeat_axis_py
        (
          sm_sc.fv_repeat_axis_py
          (
            i_depdt
          , -1
          , i_window_len[2]
          )
        , -2
        , i_window_len[1]
        )          
      )
      :: int[] :: float[]
    )
  ;

  if array_ndims(i_dloss_ddepdt) = 2
  then     
    -- -1 维度
    if i_window_len[2] <= i_stride[2]                                                                  
    then                                                                                                
      v_strided_dloss_ddepdt_width_destrided :=
        sm_sc.fv_sample_x_py    -- fv_sample_-1  
        (
          v_strided_dloss_ddepdt
        , i_stride[2]                              -- i_window_len
        , i_stride[2]                              -- i_period
        , int4range(1, i_window_len[2], '[]')      -- i_simp_range
        , 0.0 :: float
        )   -- )[ : ][ : v_indepdt_len_width + i_padding[3] + i_padding[4] - (i_stride[2] - i_window_len[2])]
      ;
    else  -- i_window_len[2] > i_stride[2]
      v_cnt_overlap_width := (i_window_len[2] - 1) / i_stride[2] + 1;
      v_strided_dloss_ddepdt_width_destrided := 
        (
          with 
          cte_slice_sample as 
          (
            select 
              a_no
            , sm_sc.fv_lpad    -- sm_sc.fv_apad
              (
                sm_sc.fv_sample_x_py   -- fv_sample_-1
                (
                  sm_sc.fv_concat_x      -- fv_concat_y
                  (
                    v_strided_dloss_ddepdt
                      [ : ]
                      [i_window_len[2] * (a_no - 1) + 1 : ]    -- array_length(v_strided_dloss_ddepdt, -1)
                  , array_fill
                    (
                      0.0 :: float
                    , array
                      [
                        array_length(v_strided_dloss_ddepdt, 1)
                      , (v_cnt_overlap_width - a_no) * i_window_len[2]   -- v_cnt_overlap_heigh
                      ]
                    )
                  )
                , v_cnt_overlap_width * i_window_len[2]
                , i_stride[2] * v_cnt_overlap_width -- i_window_len[2]
                , int4range(1, i_window_len[2], '[]')
                , 0.0 :: float
                )
              , array[[0.0]] :: float[]
              , (a_no - 1) * i_stride[2]
              ) 
              as a_slice_sample
            from generate_series(1, v_cnt_overlap_width) tb_a(a_no)
          ),
          cte_slice_max_len as 
          (
            select 
              max(array_length(a_slice_sample, 2)) as a_slice_max_len   -- array_length(a_slice_sample, -1)
            from cte_slice_sample
          )
          select 
            (sm_sc.fa_mx_sum
            (
              sm_sc.fv_rpad     -- sm_sc.fv_bpad
              (
                a_slice_sample
              , array[[0.0]] :: float[]
              , a_slice_max_len - array_length(a_slice_sample, 2)     -- array_length(a_slice_sample, -1)
              )
            ))[ : ][ : v_indepdt_len_width]   -- -1 切片
          from cte_slice_sample, cte_slice_max_len
        );
    end if;
    
    -- -2 维度 
    if i_window_len[1] <= i_stride[1]
    then 
      v_strided_dloss_ddepdt_heigh_destrided :=
        sm_sc.fv_sample_y_py    -- fv_sample_-2  
        (
          v_strided_dloss_ddepdt_width_destrided
        , i_stride[1]                              -- i_window_len
        , i_stride[1]                              -- i_period
        , int4range(1, i_window_len[1], '[]')      -- i_simp_range
        , 0.0 :: float
        )    -- )[ : v_indepdt_len_heigh + i_padding[1] + i_padding[2] - (i_stride[1] - i_window_len[1])][ : ]
      ;
    else  -- i_window_len[1] > i_stride[1]
      v_cnt_overlap_heigh := (i_window_len[1] - 1) / i_stride[1] + 1;
      v_strided_dloss_ddepdt_heigh_destrided := 
        (
          with 
          cte_slice_sample as 
          (
            select 
              a_no
            , sm_sc.fv_apad    -- sm_sc.fv_lpad
              (
                sm_sc.fv_sample_y_py   -- fv_sample_-2
                (
                  sm_sc.fv_concat_y   -- -- -- fv_concat_x
                  (
                    v_strided_dloss_ddepdt_width_destrided
                      [i_window_len[1] * (a_no - 1) + 1 : ]
                      [ : ]    -- array_length(v_strided_dloss_ddepdt_width_destrided, -2)
                  , array_fill
                    (
                      0.0 :: float
                    , array
                      [
                        (v_cnt_overlap_heigh - a_no) * i_window_len[1]  -- -- -- v_cnt_overlap_width
                      , array_length(v_strided_dloss_ddepdt_width_destrided, 2)
                      ]
                    )
                  )
                , v_cnt_overlap_heigh * i_window_len[1]
                , i_stride[1] * v_cnt_overlap_heigh -- i_window_len[1]
                , int4range(1, i_window_len[1], '[]')
                , 0.0 :: float
                )                
              , array[[0.0]] :: float[]
              , (a_no - 1) * i_stride[1]
              ) 
              as a_slice_sample
            from generate_series(1, v_cnt_overlap_heigh) tb_a(a_no)
          ),
          cte_slice_max_len as 
          (
            select 
              max(array_length(a_slice_sample, 1)) as a_slice_max_len   -- array_length(a_slice_sample, -2)
            from cte_slice_sample
          )
          select 
            (sm_sc.fa_mx_sum
            (
              sm_sc.fv_bpad     -- sm_sc.fv_rpad
              (
                a_slice_sample
              , array[[0.0]] :: float[]
              , a_slice_max_len - array_length(a_slice_sample, 1)     -- array_length(a_slice_sample, -2)
              )
            ))[ : v_indepdt_len_heigh][ : ]   -- -2 切片
          from cte_slice_sample, cte_slice_max_len
        );
    end if;

    return 
      v_strided_dloss_ddepdt_heigh_destrided    -- 去除 pad
        [i_padding[1] + 1 : i_padding[1] + array_length(i_indepdt, 1)]         -- array_length(i_indepdt, -2)
        [i_padding[3] + 1 : i_padding[3] + array_length(i_indepdt, 2)]         -- array_length(i_indepdt, -1)
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then    
    -- -1 维度
    if i_window_len[2] <= i_stride[2]                                                                  
    then                                                                                                
      v_strided_dloss_ddepdt_width_destrided := 
        sm_sc.fv_sample_x3_py     -- fv_sample_-1 
        (
          v_strided_dloss_ddepdt
        , i_stride[2]                              -- i_window_len
        , i_stride[2]                              -- i_period
        , int4range(1, i_window_len[2], '[]')      -- i_simp_range
        , 0.0 :: float
        );  
    else  -- i_window_len[2] > i_stride[2]
      v_cnt_overlap_width := (i_window_len[2] - 1) / i_stride[2] + 1;
      v_strided_dloss_ddepdt_width_destrided := 
        (
          with 
          cte_slice_sample as 
          (
            select 
              a_no
            , sm_sc.fv_lpad    -- sm_sc.fv_apad
              (
                sm_sc.fv_sample_x3_py   -- fv_sample_-1
                (
                  sm_sc.fv_concat_x3      -- fv_concat_y
                  (
                    v_strided_dloss_ddepdt
                      [ : ]
                      [ : ]
                      [i_window_len[2] * (a_no - 1) + 1 : ]    -- array_length(v_strided_dloss_ddepdt, -1)
                  , array_fill
                    (
                      0.0 :: float
                    , array
                      [
                        array_length(v_strided_dloss_ddepdt, 1)
                      , array_length(v_strided_dloss_ddepdt, 2)
                      , (v_cnt_overlap_width - a_no) * i_window_len[2]   -- v_cnt_overlap_heigh
                      ]
                    )
                  )
                , v_cnt_overlap_width * i_window_len[2]
                , i_stride[2] * v_cnt_overlap_width -- i_window_len[2]
                , int4range(1, i_window_len[2], '[]')
                , 0.0 :: float
                )
              , array[[[0.0]]] :: float[]
              , (a_no - 1) * i_stride[2]
              ) 
              as a_slice_sample
            from generate_series(1, v_cnt_overlap_width) tb_a(a_no)
          ),
          cte_slice_max_len as 
          (
            select 
              max(array_length(a_slice_sample, 3)) as a_slice_max_len   -- array_length(a_slice_sample, -1)
            from cte_slice_sample
          )
          select 
            (sm_sc.fa_mx_sum
            (
              sm_sc.fv_rpad     -- sm_sc.fv_bpad
              (
                a_slice_sample
              , array[[[0.0]]] :: float[]
              , a_slice_max_len - array_length(a_slice_sample, 3)     -- array_length(a_slice_sample, -1)
              )
            ))[ : ][ : ][ : v_indepdt_len_width]   -- -1 切片
          from cte_slice_sample, cte_slice_max_len
        );
    end if;
    
    -- -2 维度 
    if i_window_len[1] <= i_stride[1]
    then 
      v_strided_dloss_ddepdt_heigh_destrided :=
        sm_sc.fv_sample_x_py   -- fv_sample_-2  
        (
          v_strided_dloss_ddepdt_width_destrided
        , i_stride[1]                              -- i_window_len
        , i_stride[1]                              -- i_period
        , int4range(1, i_window_len[1], '[]')      -- i_simp_range
        , 0.0 :: float
        ); 
    else  -- i_window_len[1] > i_stride[1]
      v_cnt_overlap_heigh := (i_window_len[1] - 1) / i_stride[1] + 1;
      v_strided_dloss_ddepdt_heigh_destrided := 
        (
          with 
          cte_slice_sample as 
          (
            select 
              a_no
            , sm_sc.fv_apad    -- sm_sc.fv_lpad
              (
                sm_sc.fv_sample_x_py   -- fv_sample_-2
                (
                  sm_sc.fv_concat_x   -- -- -- fv_concat_x
                  (
                    v_strided_dloss_ddepdt_width_destrided
                      [ : ]
                      [i_window_len[1] * (a_no - 1) + 1 : ]
                      [ : ]    -- array_length(v_strided_dloss_ddepdt_width_destrided, -2)
                  , array_fill
                    (
                      0.0 :: float
                    , array
                      [
                        array_length(v_strided_dloss_ddepdt_width_destrided, 1)
                      , (v_cnt_overlap_heigh - a_no) * i_window_len[1]  -- -- -- v_cnt_overlap_width
                      , array_length(v_strided_dloss_ddepdt_width_destrided, 3)
                      ]
                    )
                  )
                , v_cnt_overlap_heigh * i_window_len[1]
                , i_stride[1] * v_cnt_overlap_heigh -- i_window_len[1]
                , int4range(1, i_window_len[1], '[]')
                , 0.0 :: float
                )                
              , array[[[0.0]]] :: float[]
              , (a_no - 1) * i_stride[1]
              ) 
              as a_slice_sample
            from generate_series(1, v_cnt_overlap_heigh) tb_a(a_no)
          ),
          cte_slice_max_len as 
          (
            select 
              max(array_length(a_slice_sample, 2)) as a_slice_max_len   -- array_length(a_slice_sample, -2)
            from cte_slice_sample
          )
          select 
            (sm_sc.fa_mx_sum
            (
              sm_sc.fv_bpad     -- sm_sc.fv_rpad
              (
                a_slice_sample
              , array[[[0.0]]] :: float[]
              , a_slice_max_len - array_length(a_slice_sample, 2)     -- array_length(a_slice_sample, -2)
              )
            ))[ : ][ : v_indepdt_len_heigh][ : ]   -- -2 切片
          from cte_slice_sample, cte_slice_max_len
        );
    end if;

    return 
      v_strided_dloss_ddepdt_heigh_destrided    -- 去除 pad
        [ : ]
        [i_padding[1] + 1 : i_padding[1] + array_length(i_indepdt, 2)]         -- array_length(i_indepdt, -2)
        [i_padding[3] + 1 : i_padding[3] + array_length(i_indepdt, 3)]         -- array_length(i_indepdt, -1)
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then    
    -- -1 维度
    if i_window_len[2] <= i_stride[2]                                                                  
    then                                                                                                
      v_strided_dloss_ddepdt_width_destrided := 
        sm_sc.fv_sample_x4_py     -- fv_sample_-1 
        (
          v_strided_dloss_ddepdt
        , i_stride[2]                              -- i_window_len
        , i_stride[2]                              -- i_period
        , int4range(1, i_window_len[2], '[]')      -- i_simp_range
        , 0.0 :: float
        );  
    else  -- i_window_len[2] > i_stride[2]
      v_cnt_overlap_width := (i_window_len[2] - 1) / i_stride[2] + 1;
      v_strided_dloss_ddepdt_width_destrided := 
        (
          with 
          cte_slice_sample as 
          (
            select 
              a_no
            , sm_sc.fv_lpad    -- sm_sc.fv_apad
              (
                sm_sc.fv_sample_x4_py   -- fv_sample_-1
                (
                  sm_sc.fv_concat_x4      -- fv_concat_y
                  (
                    v_strided_dloss_ddepdt
                      [ : ]
                      [ : ]
                      [ : ]
                      [i_window_len[2] * (a_no - 1) + 1 : ]    -- array_length(v_strided_dloss_ddepdt, -1)
                  , array_fill
                    (
                      0.0 :: float
                    , array
                      [
                        array_length(v_strided_dloss_ddepdt, 1)
                      , array_length(v_strided_dloss_ddepdt, 2)
                      , array_length(v_strided_dloss_ddepdt, 3)
                      , (v_cnt_overlap_width - a_no) * i_window_len[2]   -- v_cnt_overlap_heigh
                      ]
                    )
                  )
                , v_cnt_overlap_width * i_window_len[2]
                , i_stride[2] * v_cnt_overlap_width -- i_window_len[2]
                , int4range(1, i_window_len[2], '[]')
                , 0.0 :: float
                )
              , array[[[[0.0]]]] :: float[]
              , (a_no - 1) * i_stride[2]
              ) 
              as a_slice_sample
            from generate_series(1, v_cnt_overlap_width) tb_a(a_no)
          ),
          cte_slice_max_len as 
          (
            select 
              max(array_length(a_slice_sample, 4)) as a_slice_max_len   -- array_length(a_slice_sample, -1)
            from cte_slice_sample
          )
          select 
            (sm_sc.fa_mx_sum
            (
              sm_sc.fv_rpad     -- sm_sc.fv_bpad
              (
                a_slice_sample
              , array[[[[0.0]]]] :: float[]
              , a_slice_max_len - array_length(a_slice_sample, 4)     -- array_length(a_slice_sample, -1)
              )
            ))[ : ][ : ][ : ][ : v_indepdt_len_width]   -- -1 切片
          from cte_slice_sample, cte_slice_max_len
        );
    end if;
    
    -- -2 维度 
    if i_window_len[1] <= i_stride[1]
    then 
      v_strided_dloss_ddepdt_heigh_destrided :=
        sm_sc.fv_sample_x3_py   -- fv_sample_-2  
        (
          v_strided_dloss_ddepdt_width_destrided
        , i_stride[1]                              -- i_window_len
        , i_stride[1]                              -- i_period
        , int4range(1, i_window_len[1], '[]')      -- i_simp_range
        , 0.0 :: float
        ); 
    else  -- i_window_len[1] > i_stride[1]
      v_cnt_overlap_heigh := (i_window_len[1] - 1) / i_stride[1] + 1;
      v_strided_dloss_ddepdt_heigh_destrided := 
        (
          with 
          cte_slice_sample as 
          (
            select 
              a_no
            , sm_sc.fv_apad    -- sm_sc.fv_lpad
              (
                sm_sc.fv_sample_x3_py   -- fv_sample_-2
                (
                  sm_sc.fv_concat_x3   -- -- -- fv_concat_x
                  (
                    v_strided_dloss_ddepdt_width_destrided
                      [ : ]
                      [ : ]
                      [i_window_len[1] * (a_no - 1) + 1 : ]
                      [ : ]    -- array_length(v_strided_dloss_ddepdt_width_destrided, -2)
                  , array_fill
                    (
                      0.0 :: float
                    , array
                      [
                        array_length(v_strided_dloss_ddepdt_width_destrided, 1)
                      , array_length(v_strided_dloss_ddepdt_width_destrided, 2)
                      , (v_cnt_overlap_heigh - a_no) * i_window_len[1]  -- -- -- v_cnt_overlap_width
                      , array_length(v_strided_dloss_ddepdt_width_destrided, 4)
                      ]
                    )
                  )
                , v_cnt_overlap_heigh * i_window_len[1]
                , i_stride[1] * v_cnt_overlap_heigh -- i_window_len[1]
                , int4range(1, i_window_len[1], '[]')
                , 0.0 :: float
                )                
              , array[[[[0.0]]]] :: float[]
              , (a_no - 1) * i_stride[1]
              ) 
              as a_slice_sample
            from generate_series(1, v_cnt_overlap_heigh) tb_a(a_no)
          ),
          cte_slice_max_len as 
          (
            select 
              max(array_length(a_slice_sample, 3)) as a_slice_max_len   -- array_length(a_slice_sample, -2)
            from cte_slice_sample
          )
          select 
            (sm_sc.fa_mx_sum
            (
              sm_sc.fv_bpad     -- sm_sc.fv_rpad
              (
                a_slice_sample
              , array[[[[0.0]]]] :: float[]
              , a_slice_max_len - array_length(a_slice_sample, 3)     -- array_length(a_slice_sample, -2)
              )
            ))[ : ][ : ][ : v_indepdt_len_heigh][ : ]   -- -2 切片
          from cte_slice_sample, cte_slice_max_len
        );
    end if;

    return 
      v_strided_dloss_ddepdt_heigh_destrided    -- 去除 pad
        [ : ]
        [ : ]
        [i_padding[1] + 1 : i_padding[1] + array_length(i_indepdt, 3)]         -- array_length(i_indepdt, -2)
        [i_padding[3] + 1 : i_padding[3] + array_length(i_indepdt, 4)]         -- array_length(i_indepdt, -1)
    ;
  
  end if;
end
$$
language plpgsql stable
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_max_dloss_dindepdt_ex
--   (
--     array[array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0]
--         , array[10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0]
--         , array[100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0]
--         , array[-1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , array[-10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0]
--          ]
--    , array[3, 3]
--    , array[array[1.1, 1.1, 1.1]
--         ,array[1.1, 1.1, 1.1]
--          ]
--    , null
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_pool_max_dloss_dindepdt_ex
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ]::float[][]
--    , array[3, 3]
--    , array[array[1.1, 1.1, 1.1]
--         ,array[1.1, 1.1, 1.1]
--         ,array[1.1, 1.1, 1.1]
--          ]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_d_pool_max_dloss_dindepdt_ex
--   (
--     array
--       [
--         [
--           [1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [1,2,3,-4,-5,6]
--         , [10,20,30,40,-50,60]
--         , [100,200,-300,400,500,600]
--         , [-1,2,-3,-4,-5,6]
--         , [10,20,-30,40,-50,-60]
--         ]
--       ]
--    , array[3, 3]
--    , array
--       [
--         [
--          [1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ,[1.1, 1.1, 1.1]
--         ]
--       , [
--          [1.1, -1.1, 1.1]
--         ,[1.1, 1.1, -1.1]
--         ,[-1.1, 1.1, 1.1]
--         ]
--       ]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_d_pool_max_dloss_dindepdt_ex
--   (
--     array
--     [
--       [
--         [
--           [1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [1,2,3,-4,-5,6]
--         , [10,20,30,40,-50,60]
--         , [100,200,-300,400,500,600]
--         , [-1,2,-3,-4,-5,6]
--         , [10,20,-30,40,-50,-60]
--         ]
--       ]
--     , [
--         [
--           [-1,2,-3,4,5,6]
--         , [10,20,-30,40,-50,60]
--         , [100,-200,-300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--         ]
--       , [
--           [1,-2,3,4,-5,6]
--         , [10,20,30,40,-50,60]
--         , [100,-200,-300,400,-500,-600]
--         , [-1,2,-3,-4,-5,6]
--         , [10,20,30,40,-50,-60]
--         ]
--       ]
--     ]
--    , array[3, 3]
--    , array
--      [
--        [
--          [
--           [1.1, 1.1, 1.1]
--          ,[1.1, 1.1, 1.1]
--          ,[1.1, 1.1, 1.1]
--          ]
--        , [
--           [1.1, -1.1, 1.1]
--          ,[1.1, 1.1, -1.1]
--          ,[-1.1, 1.1, 1.1]
--          ]
--        ]
--      , [
--          [
--           [1.1, -1.1, 1.1]
--          ,[1.1, -1.1, 1.1]
--          ,[1.1, -1.1, 1.1]
--          ]
--        , [
--           [1.1, 1.1, 1.1]
--          ,[-1.1, -1.1, -1.1]
--          ,[-1.1, 1.1, 1.1]
--          ]
--        ]
--      ]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt(float[], int, float[], int[2], float[], int[2], int[4], float);
create or replace function sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt
(
  i_indepdt_grp_x                float[]                                     ,  -- i_indepdt_grp_x_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_dloss_ddepdt_1d_grp        float[]                                     ,  -- 即已求出的损失函数对 y 的导数矩阵 的 降维后样本集合
  i_window_len             int[2]                                               ,  -- 卷积核窗口矩阵高宽
  i_depdt_1d_grp               float[]    default  null                    ,  -- 即已求出的算子结果矩阵
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value          float      default  '-inf'       -- 补齐填充元素值
)
returns float[][]
as
$$
declare 
  v_dloss_ddepdt_array_len_2   int :=  (i_padding[3] + i_1d_2_2d_cnt_per_grp + i_padding[4] - i_window_len[2]) / i_stride[2] + 1;
begin
  -- 审计二维长度
  if array_ndims(i_indepdt_grp_x) <> 2
  then 
    raise exception 'no method for such length!  Dims: %;', array_dims(i_indepdt_grp_x);
  elsif array_length(i_indepdt_grp_x, 2) % i_1d_2_2d_cnt_per_grp <> 0
  then 
    raise exception 'imperfect window at grp_x_cnt.';
  else
    return 
    (
      select 
        sm_sc.fa_mx_concat_y
        (
          array 
          [
            sm_sc.fv_mx_ele_2d_2_1d
            (
              sm_sc.fv_d_pool_max_dloss_dindepdt
              (
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_indepdt_grp_x[a_cur_y : a_cur_y][ : ]
                  , i_1d_2_2d_cnt_per_grp
                )                                        , 
                i_window_len                             ,
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_dloss_ddepdt_1d_grp[a_cur_y : a_cur_y][ : ]
                  , v_dloss_ddepdt_array_len_2
                )                                        , 
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_depdt_1d_grp[a_cur_y : a_cur_y][ : ]
                  , v_dloss_ddepdt_array_len_2
                )                                        ,
                i_stride                                 ,
                i_padding                                ,
                i_padding_value          
              )
            )
          ]
          order by a_cur_y
        )
      from generate_series(1, array_length(i_indepdt_grp_x, 1)) tb_a_cur_y(a_cur_y)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--       ]
--     ]
--     , 7,
--     array
--     [
--       array[1.1, 1.1, 1.1
--        ,1.1, 1.1, 1.1
--        ],
--       array[2.1, 3.1, 0.1
--        ,1.1, -1.1, -0.1
--        ],
--       array[0.1, -2.1, 1.1
--        ,-1.1, 3.1, 2.1
--        ]
--     ]
--    , array[3, 3]
--    , null
--    , array[2, 2]
--   );

-- select sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--       ]
--     ]
--     , 6,
--    array
--    [
--      array[1.1, 1.1, 1.1
--       ,1.1, 1.1, 1.1
--       ,1.1, 1.1, 1.1
--       ],
--      array[2.1, 3.1, 0.1
--       ,1.1, -1.1, -0.1
--       ,0.1, 2.1, -0.1
--       ],
--      array[0.1, -2.1, 1.1
--       ,-1.1, 3.1, 2.1
--       ,-1.1, -1.1, 0.1
--       ]
--    ]
--    , array[3, 3]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_d_pool_none_dloss_dindepdt
-- (
--   -- anyarray, 
--   int[]
-- -- , anyarray
-- , int[2]
-- -- , anyarray
-- -- , int[]
-- , anyarray
-- , int[2]
-- , int[4]
-- -- , anyelement
-- );
create or replace function sm_sc.fv_d_pool_none_dloss_dindepdt
(
  -- i_background            anyarray           ,
  i_background_len        int[]             ,
  -- i_window                anyarray           ,
  i_window_len            int[]             ,
  -- i_depdt_var             anyarray           ,
  -- i_depdt_var_len         int[]             ,
  i_dloss_ddepdt          anyarray           ,
  i_stride                int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding               int[4]              default  array[0, 0, 0, 0]       -- ,  -- 上下左右补齐行数/列数
  -- i_padding_value         anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_dloss_ddepdt_len_2d      int[2]    :=   array[array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt) - 1), array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))];
  v_background_len        -- int[2]    :=   array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
                             int[2]    :=   i_background_len[array_length(i_background_len, 1) - 1 : array_length(i_background_len, 1)];
  v_window_len               int[2]    :=   i_window_len[array_length(i_window_len, 1) - 1 : array_length(i_window_len, 1)];
begin
  -- if i_background_len is null
  -- then 
  --   i_background_len := array[array_length(i_background, array_ndims(i_background) - 1), array_length(i_background, array_ndims(i_background))];
  -- end if;
  -- if v_window_len is null
  -- then 
  --   v_window_len := array[array_length(i_window, 1), array_length(i_window, 2)];
  -- end if;
  -- if i_depdt_var_len is null
  -- then 
  --   i_depdt_var_len := array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)];
  -- end if;

  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_dloss_ddepdt) not between 2 and 4
    then 
      raise exception 'unsupport ndims of i_dloss_ddepdt';
    -- elsif -- -- i_background is not null and 
    --   array_ndims(i_background) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'unmatched ndims between i_background and i_dloss_ddepdt';
    -- elsif i_window is not null and array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_dloss_ddepdt)
    -- then 
    --   raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    -- elsif i_background_len is null or v_window_len is null or i_depdt_var_len is null or i_dloss_ddepdt is null 
    -- then 
    --   raise exception 'i_background_len, v_window_len, i_depdt_var_len, i_dloss_ddepdt should be not null.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1 <> v_dloss_ddepdt_len_2d[1] / v_window_len[1]
    then                                                                                                                                         
      raise exception 'imperfect dloss_ddepdt at 1d.';                                                                                           
    elsif (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1 <> v_dloss_ddepdt_len_2d[2] / v_window_len[2]
    then 
      raise exception 'imperfect dloss_ddepdt at 2d.';
    -- elsif i_depdt_var_len is not null and i_depdt_var_len <> v_dloss_ddepdt_len_2d
    -- then 
    --   raise exception 'unmatched sizes of dloss_ddepdt and depdt_var_len.';
    elsif array_length(v_window_len, 1) = 3 and v_window_len[1] <> array_length(i_dloss_ddepdt, 1)
      or array_length(v_window_len, 1) = 4 and (v_window_len[1] <> array_length(i_dloss_ddepdt, 1) or v_window_len[2] <> array_length(i_dloss_ddepdt, 2))
    then 
      raise exception 'unmatch length between i_window and i_dloss_ddepdt at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_dloss_ddepdt) = 2
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt[(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
                        [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
              [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
              [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , 0.0
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          array_agg(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  array_agg(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 3
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  elsif array_ndims(i_dloss_ddepdt) = 4
  then
    return     
    (    
      with
      -- dloss_ddepdt 的各个独立窗口
      cte_slice_dloss_dindepdt_ex as
      (
        select 
          a_cur_heigh,
          a_cur_width,
          i_dloss_ddepdt
            [ : ]
            [ : ]
            [(a_cur_heigh - 1) * v_window_len[1] + 1 : a_cur_heigh * v_window_len[1]]
            [(a_cur_width - 1) * v_window_len[2] + 1 : a_cur_width * v_window_len[2]] -- 损失函数导数的独立窗口分组
            as a_slice_dloss_dindepdt_ex
        from generate_series(1, (coalesce(i_padding[1], 0) + v_background_len[1] + coalesce(i_padding[2], 0) - v_window_len[1]) / i_stride[1] + 1) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, (coalesce(i_padding[3], 0) + v_background_len[2] + coalesce(i_padding[4], 0) - v_window_len[2]) / i_stride[2] + 1) tb_a_cur_width(a_cur_width)
      ),
      -- 
      cte_ele_dloss_dindepdt as
      (
        select 
          tb_a_cur_heigh.a_cur_heigh,
          tb_a_cur_width.a_cur_width,
          coalesce
          (
            sm_sc.fa_mx_sum
            (
              tb_a_slice.a_slice_dloss_dindepdt_ex
                [ : ]
                [ : ]
                [tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1]) : tb_a_cur_heigh.a_cur_heigh - ((tb_a_slice.a_cur_heigh - 1) * i_stride[1])]
                [tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2]) : tb_a_cur_width.a_cur_width - ((tb_a_slice.a_cur_width - 1) * i_stride[2])]
            )
            , array_fill(0.0, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2), 1, 1])
          ) as a_dloss_dindepdt_ele
        from generate_series(1, v_background_len[1]) tb_a_cur_heigh(a_cur_heigh)
          , generate_series(1, v_background_len[2]) tb_a_cur_width(a_cur_width)
          , cte_slice_dloss_dindepdt_ex tb_a_slice
        where tb_a_cur_heigh.a_cur_heigh between (tb_a_slice.a_cur_heigh - 1) * i_stride[1] + 1 and tb_a_slice.a_cur_heigh * i_stride[1] + v_window_len[1]
          and tb_a_cur_width.a_cur_width between (tb_a_slice.a_cur_width - 1) * i_stride[2] + 1 and tb_a_slice.a_cur_width * i_stride[2] + v_window_len[2]
        group by tb_a_cur_heigh.a_cur_heigh, tb_a_cur_width.a_cur_width
      ),
      cte_y_slice_dloss_dindepdt as 
      (
        select 
          a_cur_heigh,
          sm_sc.fa_mx_concat_x4(a_dloss_dindepdt_ele order by a_cur_width) as a_dloss_dindepdt_y_slice
        from cte_ele_dloss_dindepdt
        group by a_cur_heigh
      )
      select 
        sm_sc.fv_augmented
		(
		  sm_sc.fa_mx_concat_x3(a_dloss_dindepdt_y_slice order by a_cur_heigh), 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len[1] + i_padding[2], v_background_len[2] + i_padding[4]], 
		  0.0 :: float
		)
      from cte_y_slice_dloss_dindepdt
    )
    ;
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_d_pool_none_dloss_dindepdt
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[4, 5]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[1, 1]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_pool_none_dloss_dindepdt
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [[1, 2, 3, 4, 5, 6, 7, 8, 9]                
--   ,[11, 12, 13, 14, 15, 16, 17, 18, 19]                
--   ,[21, 22, 23, 24, 25, 26, 27, 28, 29]                       
--   ,[31, 32, 33, 34, 35, 36, 37, 38, 39]                
--   ,[41, 42, 43, 44, 45, 46, 47, 48, 49]           
--   ,[51, 52, 53, 54, 55, 56, 57, 58, 59]] :: float[],        -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_pool_none_dloss_dindepdt
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]    
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--       [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]                
--     , [1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--     , [2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--     , [3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--     , [4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--     , [5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--     ] 
--   , [ 
--       [-1.0, 2.0, -3.0, 4.0, -5.0, 6.0, -7.0, 8.0, -9.0]                
--     , [-1.1, 1.2, -1.3, 1.4, -1.5, 1.6, -1.7, 1.8, -1.9]                
--     , [-2.1, 2.2, -2.3, 2.4, -2.5, 2.6, -2.7, 2.8, -2.9]                       
--     , [-3.1, 3.2, -3.3, 3.4, -3.5, 3.6, -3.7, 3.8, -3.9]                
--     , [-4.1, 4.2, -4.3, 4.4, -4.5, 4.6, -4.7, 4.8, -4.9]           
--     , [-5.1, 5.2, -5.3, 5.4, -5.5, 5.6, -5.7, 5.8, -5.9]
--     ]
--   ] :: float[],                                                -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                         -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );

-- select sm_sc.fv_d_pool_none_dloss_dindepdt
-- (
--   -- null :: float[]               ,                           -- i_background            anyarray  
--   array[2, 2, 6, 9]                   ,                           -- i_background_len        int[2]    
--   -- null :: float[]               ,                           -- i_window                anyarray  
--   array[3, 3]                   ,                           -- v_window_len            int[2]        -- array[2, 2, 3, 3]
--   -- null :: float[]               ,                           -- i_depdt_var             anyarray  
--   -- array[2, 2, 6, 9]                   ,                           -- i_depdt_var_len         int[2]    
--   array              
--   [
--     [
--       [
--         [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]                
--       , [1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--       , [2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--       , [3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--       , [4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--       , [5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--       ] 
--     , [ 
--         [-1.0, 2.0, -3.0, 4.0, -5.0, 6.0, -7.0, 8.0, -9.0]                
--       , [-1.1, 1.2, -1.3, 1.4, -1.5, 1.6, -1.7, 1.8, -1.9]                
--       , [-2.1, 2.2, -2.3, 2.4, -2.5, 2.6, -2.7, 2.8, -2.9]                       
--       , [-3.1, 3.2, -3.3, 3.4, -3.5, 3.6, -3.7, 3.8, -3.9]                
--       , [-4.1, 4.2, -4.3, 4.4, -4.5, 4.6, -4.7, 4.8, -4.9]           
--       , [-5.1, 5.2, -5.3, 5.4, -5.5, 5.6, -5.7, 5.8, -5.9]
--       ]
--     ]
--   , [
--       [
--         [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]                
--       , [1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9]                
--       , [2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9]                       
--       , [3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9]                
--       , [4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9]           
--       , [5.1, 5.2, 5.3, 5.4, 5.5, 5.6, 5.7, 5.8, 5.9]
--       ] 
--     , [ 
--         [-1.0, 2.0, -3.0, 4.0, -5.0, 6.0, -7.0, 8.0, -9.0]                
--       , [-1.1, 1.2, -1.3, 1.4, -1.5, 1.6, -1.7, 1.8, -1.9]                
--       , [-2.1, 2.2, -2.3, 2.4, -2.5, 2.6, -2.7, 2.8, -2.9]                       
--       , [-3.1, 3.2, -3.3, 3.4, -3.5, 3.6, -3.7, 3.8, -3.9]                
--       , [-4.1, 4.2, -4.3, 4.4, -4.5, 4.6, -4.7, 4.8, -4.9]           
--       , [-5.1, 5.2, -5.3, 5.4, -5.5, 5.6, -5.7, 5.8, -5.9]
--       ]
--     ]
--   ] :: float[],                                                -- i_dloss_ddepdt          anyarray  
--   array[3, 3]        ,                                      -- i_stride                int[2]    
--   array[0, 0, 0, 0]  -- ,                                      -- i_padding               int[4]    
--   -- 0.0  :: float                                             -- i_padding_value         anyelement
-- );



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_2d_grp_x(float[], int, float[], int, int[2], int[4], float);
create or replace function sm_sc.fv_conv_2d_grp_x
(
  i_array_grp_x            float[]                                     ,  -- i_array_grp_x_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_window_flat            float[1][]                                  ,  -- 卷积核窗口矩阵的扁平化
  i_window_len_x           int                                                  ,  -- 卷积核窗口的宽度
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value          float      default  0.0                        -- 补齐填充元素值
)
returns float[][]
as
$$
declare
  v_window        float[]    ;
  v_windos_bias   float      ;
begin
  -- 审计二维长度
  if array_ndims(i_array_grp_x) <> 2
  then 
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array_grp_x);
  elsif array_length(i_array_grp_x, 2) % i_1d_2_2d_cnt_per_grp <> 0
  then 
    raise exception 'imperfect window at grp_x_cnt.';
  elsif array_length(i_window_flat, 2) % i_window_len_x > 1
  then 
    raise exception 'len_x of window_flat %% window_len_x should be 0 or 1.';
  else
    if array_length(i_window_flat, 2) % i_window_len_x = 1
    then
      v_window := sm_sc.fv_mx_ele_1d_2_2d(i_window_flat[1 : 1][ : array_length(i_window_flat, 2) - (array_length(i_window_flat, 2) % i_window_len_x)], i_window_len_x);
      v_windos_bias := i_window_flat[1][array_length(i_window_flat, 2)];
    elsif array_length(i_window_flat, 2) % i_window_len_x = 0
    then 
      v_window := sm_sc.fv_mx_ele_1d_2_2d(i_window_flat, i_window_len_x);
      v_windos_bias := null;
    end if;
    
    return 
    (
      select 
        sm_sc.fa_mx_concat_y
        (
          array 
          [
            sm_sc.fv_mx_ele_2d_2_1d
            (
              sm_sc.fv_conv_2d_im2col_py
              (
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_array_grp_x[a_cur_y : a_cur_y][ : ]
                  , i_1d_2_2d_cnt_per_grp
                )                                        , 
                v_window                                 , -- -- -- 
                nullif(array[v_windos_bias], array[null::float])    ,
                i_stride                                 ,
                i_padding                                ,
                i_padding_value
              )
            )
          ]
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array_grp_x, 1)) tb_a_cur_y(a_cur_y)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_2d_grp_x
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--       ]
--     ]
--     , 7
--     , array[array[0, 1, 0, 1, 1, 1, 0, 1, 0]]    -- -- array[array[0, 1, 0], array[1, 1, 1], array[0, 1, 0]]
--     -- , array[array[0, 1, 0, 1, 1, 1, 0, 1, 0, 0.5]]   -- 带偏移量
--     , 3
--     , array[2, 2]
--   );

-- select sm_sc.fv_conv_2d_grp_x
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--       ]
--     ]
--     , 6
--     , array[array[1.1, 1.1, 1.1, 1.1, 2.1, 1.1, 1.1, 1.1, 1.1]]     -- -- array[array[1.1, 1.1, 1.1], array[1.1, 2.1, 1.1], array[1.1, 1.1, 1.1]]
--     -- , array[array[1.1, 1.1, 1.1, 1.1, 2.1, 1.1, 1.1, 1.1, 1.1, 0.5]]          -- 带偏移量
--     , 3
--     , array[2, 2]
--     , array[1, 1, 1, 0]
--     , 0
--   );
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i http://pypi.douban.com/simple/ --trusted-host pypi.douban.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- drop function if exists sm_sc.fv_conv_2d_im2col_py(float[], float[], float[], int[2], int[4], float);
create or replace function sm_sc.fv_conv_2d_im2col_py
(
  i_background         float[]                                      -- 背景矩阵，可以为三维四维，背景以最高两个维度当作滑动高宽
, i_window             float[]                                      -- 卷积核窗口矩阵，限二维窗口
, i_window_bias        float[]    default  null                     -- 卷积核的偏移量。限定与 i_window 的维数一致，且窗口高宽以外的维度长度与 i_background, i_window 一致，窗口高宽的维度长度都是1
, i_stride             int[2]     default  array[1, 1]              -- 纵向与横向步长
, i_padding            int[4]     default  array[0, 0, 0, 0]        -- 上下左右补齐行数/列数
, i_padding_value      float      default  0.0                      -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  # from scipy import signal
  
  v_background      = np.float64(i_background)     
  v_window          = np.float64(i_window)         
  # v_window_bias     = np.float64(i_window_bias)  
  # v_stride          = np.int8(i_stride)          
  # v_padding         = np.int8(i_padding)         
  # v_padding_value   = np.float64(i_padding_value)
  
  # mirror heigh & mirror width
  if v_window.ndim == 2 :
    v_window = v_window[::-1, ::-1]
  elif v_window.ndim == 3 :
    v_window = v_window[::, ::-1, ::-1]
  elif v_window.ndim == 4 :
    v_window = v_window[::, ::, ::-1, ::-1]
  
  
  # brodcast window dims align
  if v_window.ndim == v_background.ndim - 1 :
    v_window = np.repeat(v_window[np.newaxis, : ], v_background.shape[0], axis = 0)
  elif v_window.ndim == v_background.ndim - 2 :
    v_window = np.repeat( \
    np.repeat(v_window[np.newaxis , np.newaxis, :], v_background.shape[1], axis = 1) \
    , v_background.shape[0], axis = 0)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize
  v_return_heigh = (v_background.shape[v_background.ndim - 2] - v_window.shape[v_background.ndim - 2]) // i_stride[0] + 1
  v_return_width = (v_background.shape[v_background.ndim - 1] - v_window.shape[v_background.ndim - 1]) // i_stride[1] + 1
  
  # im2col, matmul
  if v_background.ndim == 2 :
    v_return = np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_return_heigh, v_return_width, v_window.shape[0], v_window.shape[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .reshape(v_return_heigh * v_return_width, v_window.shape[0] * v_window.shape[1])
  
    v_return = np.matmul(np.float64(v_return), np.float64(v_window).reshape(v_window.shape[0] * v_window.shape[1], 1)) \
    .reshape(v_return_heigh, v_return_width)
  
  elif v_background.ndim == 3 :
    v_return = np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_return_heigh, v_return_width, v_window.shape[1], v_window.shape[2]) \
    , strides=(v_ele_len * v_background.shape[1] * v_background.shape[2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .reshape(v_background.shape[0], v_return_heigh * v_return_width, v_window.shape[1] * v_window.shape[2])
    
    v_return = np.matmul(np.float64(v_return), np.float64(v_window).reshape(v_window.shape[0], v_window.shape[1] * v_window.shape[2], 1)) \
    .reshape(v_return.shape[0], v_return_heigh, v_return_width)
  
  elif v_background.ndim == 4 :
    v_return = np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], v_return_heigh, v_return_width, v_window.shape[2], v_window.shape[3]) \
    , strides=(v_ele_len * v_background.shape[1] * v_background.shape[2] * v_background.shape[3], v_ele_len * v_background.shape[2] * v_background.shape[3], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .reshape(v_background.shape[0], v_background.shape[1], v_return_heigh * v_return_width, v_window.shape[2] * v_window.shape[3])
    
    v_return = np.matmul(np.float64(v_return), np.float64(v_window).reshape(v_window.shape[0], v_window.shape[1], v_window.shape[2] * v_window.shape[3], 1)) \
    .reshape(v_return.shape[0], v_return.shape[1], v_return_heigh, v_return_width)
  
  if i_window_bias :
    return (v_return + i_window_bias).tolist()
  else :
    return v_return.tolist()
$$
language plpython3u stable
parallel safe
;

-- select sm_sc.fv_conv_2d_im2col_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ] :: float[]
--    , array[[0, 1, 0], [1, 1, 1], [0, 1, 0]]
--    , array[[0.5]]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_im2col_py
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_im2col_py
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[[0.0]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_im2col_py
--   (
--     array
--     [
--      [[1,2,3,4,5,6]
--      ,[10,20,30,40,50,60]
--      ,[100,200,300,400,500,600]
--      ,[-1,-2,-3,-4,-5,-6]
--      ,[-10,-20,-30,-40,-50,-60]
--      ]
--     ,[[1,2,3,4,5,6]
--      ,[100,200,300,400,500,600]
--      ,[10,20,30,40,50,60]
--      ,[-10,-20,-30,-40,-50,-60]
--      ,[-1,-2,-3,-4,-5,-6]
--      ]
--     ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[[0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_im2col_py
--   (
--     array
--     [
--      [
--       [[1,2,3,4,5,6]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ]
--      ,[[1,2,3,4,5,6]
--       ,[100,200,300,400,500,600]
--       ,[10,20,30,40,50,60]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[-1,-2,-3,-4,-5,-6]
--       ]
--      ],
--      [
--       [[10,20,30,40,50,60]
--       ,[1,2,3,4,5,6]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[100,200,300,400,500,600]
--       ]
--      ,[[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[1,2,3,4,5,6]
--       ]
--      ]
--     ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[[0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_im2col_py
--   (
--     array
--     [
--      [
--       [[1,2,3,4,5,6]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ]
--      ,[[1,2,3,4,5,6]
--       ,[100,200,300,400,500,600]
--       ,[10,20,30,40,50,60]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[-1,-2,-3,-4,-5,-6]
--       ]
--      ],
--      [
--       [[10,20,30,40,50,60]
--       ,[1,2,3,4,5,6]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[100,200,300,400,500,600]
--       ]
--      ,[[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[1,2,3,4,5,6]
--       ]
--      ]
--     ]
--    , array
--     [
--      [
--       [[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--      ,[[1.1, 1.1, 1.1], [1.1, 3.1, 1.1], [1.1, 1.1, 1.1]]
--      ],
--      [
--       [[1.1, 1.1, 1.1], [1.1, 2.5, 1.1], [1.1, 1.1, 1.1]]
--      ,[[1.1, 1.1, 1.1], [1.1, 0.1, 1.1], [1.1, 1.1, 1.1]]
--      ]
--     ]
--    , array[[[[0.5]],[[1.0]]],[[[-1.0]],[[0.9]]]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;
-- -- 卷积的矩阵乘法实现：
-- --   https://www.cnblogs.com/shine-lee/p/10775831.html
-- --   https://blog.csdn.net/qq_40263477/article/details/104979609
-- -- 卷积的 fft 实现
-- --   https://blog.csdn.net/qq_37527608/article/details/120922348
-- -- 卷积与矩阵乘法算法调优：
-- --  https://blog.csdn.net/lizhengx/article/details/83246833
-- -- 卷积求导
-- --  https://blog.csdn.net/wangjianyu0115/article/details/62222301
-- -- 带偏置的卷积
-- --  https://blog.csdn.net/weixin_40519315/article/details/105115657

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_2d_v_im2col(float[], float[], float[], int[2], int[4], float);
create or replace function sm_sc.fv_conv_2d_v_im2col
(
  i_background         float[]                                     ,  -- 背景矩阵，可以为三维四维，背景以最高两个维度当作滑动高宽
  i_window             float[]                                     ,  -- 卷积核窗口矩阵，限二维窗口
  i_window_bias        float[]    default  null                    ,  -- 卷积核的偏移量。限定与 i_window 的维数一致，且窗口高宽以外的维度长度与 i_background, i_window 一致，窗口高宽的维度长度都是1
  i_stride             int[2]     default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]     default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float      default  0.0                        -- 补齐填充元素值
)
returns float[]
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))      ;
  v_return                float[]   ;   
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_window) <> 2 
      and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window ndims match such i_background!';
    elsif array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background ndims!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_ndims(i_window_bias) <> array_ndims(i_window)
      or array_length(i_window_bias, array_ndims(i_window_bias) - 1) <> 1
      or array_length(i_window_bias, array_ndims(i_window_bias)) <> 1
      or array_ndims(i_window_bias) = 3 and array_length(i_window, 1) <> array_length(i_window_bias, 1)
      or array_ndims(i_window_bias) = 4 and (array_length(i_window, 1) <> array_length(i_window_bias, 1) or array_length(i_window, 2) <> array_length(i_window_bias, 2))
    then 
      raise exception 'unmatch ndims or length for such i_window_bias';
    end if;
  end if;
    
  if array_ndims(i_background) = 2
  then 
    if 0 <> any(i_padding)
    then 
      i_background := 
        sm_sc.fv_augmented
        (
          i_background, 
          array[-i_padding[1] + 1, -i_padding[3] + 1], 
          array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
          i_padding_value
        );
    end if;
    v_background_len_heigh :=    array_length(i_background, array_ndims(i_background) - 1)  ;
    v_background_len_width :=    array_length(i_background, array_ndims(i_background))      ;
  
    v_return :=    
    (
      -- -- https://blog.csdn.net/qq_40263477/article/details/104979609
      -- -- https://www.cnblogs.com/shine-lee/p/10775831.html
      -- -- https://zhuanlan.zhihu.com/p/74567600
      -- -- https://baike.baidu.com/item/%E5%A8%81%E8%AF%BA%E6%A0%BC%E6%8B%89%E5%BE%B7%E5%BF%AB%E9%80%9F%E5%82%85%E9%87%8C%E5%8F%B6%E5%8F%98%E6%8D%A2/22800076?fr=ge_ala
      -- -- https://zhuanlan.zhihu.com/p/409050916
      -- -- https://zhuanlan.zhihu.com/p/102351953
      with 
      cte_im2col as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_mx_ele_2d_2_1d(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1]) order by col_a_x) as a_im2col
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select 
        sm_sc.fv_mx_ele_1d_2_2d
        (
          sm_sc.fv_mx_ele_2d_2_1d
          (
            sm_sc.fa_mx_concat_y(a_im2col order by col_a_y) 
            |**| (|^~| array[sm_sc.fv_mx_ele_2d_2_1d(|~~| i_window)])   -- 卷积核要翻转 180
          )
          ,
          (v_background_len_width - v_window_len_width + i_stride[2]) / i_stride[2]
        ) 
      from cte_im2col
    )
    ;

    if i_window_bias is not null
    then 
      v_return := v_return +` i_window_bias;
    end if;
    
    return v_return;
  
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_conv_2d_v_im2col
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_background[a_cur_y : a_cur_y]
            , 1
            )
          , case 
              when array_ndims(i_window) = 2 
                then i_window 
              else      
                sm_sc.fv_mx_slice_3d_2_2d
                (
                  i_window[a_cur_y : a_cur_y]
                , 1
                )
            end
          , case 
              when array_ndims(i_window_bias) = 2 
                then i_window_bias 
              else      
                sm_sc.fv_mx_slice_3d_2_2d
                (
                  i_window_bias[a_cur_y : a_cur_y]
                , 1
                )
            end
          , i_stride       
          , i_padding      
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_background, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_conv_2d_v_im2col
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_background[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , case 
                when array_ndims(i_window) = 2 
                  then i_window 
                else      
                  sm_sc.fv_mx_slice_4d_2_2d
                  (
                    i_window[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
                  , array[1, 2]
                  , array[1, 1]
                  )
              end      
            , case 
                when array_ndims(i_window_bias) = 2 
                  then i_window_bias 
                else      
                  sm_sc.fv_mx_slice_4d_2_2d
                  (
                    i_window_bias[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
                  , array[1, 2]
                  , array[1, 1]
                  )
              end     
            , i_stride       
            , i_padding      
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_background, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_background, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_2d_v_im2col
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ] :: float[]
--    , array[[0, 1, 0], [1, 1, 1], [0, 1, 0]]
--    , array[[0.5]]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_v_im2col
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , null
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_v_im2col
--   (
--     array[[1,2,3,4,5,6]
--         , [10,20,30,40,50,60]
--         , [100,200,300,400,500,600]
--         , [-1,-2,-3,-4,-5,-6]
--         , [-10,-20,-30,-40,-50,-60]
--          ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[[0.0]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_v_im2col
--   (
--     array
--     [
--      [[1,2,3,4,5,6]
--      ,[10,20,30,40,50,60]
--      ,[100,200,300,400,500,600]
--      ,[-1,-2,-3,-4,-5,-6]
--      ,[-10,-20,-30,-40,-50,-60]
--      ]
--     ,[[1,2,3,4,5,6]
--      ,[100,200,300,400,500,600]
--      ,[10,20,30,40,50,60]
--      ,[-10,-20,-30,-40,-50,-60]
--      ,[-1,-2,-3,-4,-5,-6]
--      ]
--     ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[[0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_v_im2col
--   (
--     array
--     [
--      [
--       [[1,2,3,4,5,6]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ]
--      ,[[1,2,3,4,5,6]
--       ,[100,200,300,400,500,600]
--       ,[10,20,30,40,50,60]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[-1,-2,-3,-4,-5,-6]
--       ]
--      ],
--      [
--       [[10,20,30,40,50,60]
--       ,[1,2,3,4,5,6]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[100,200,300,400,500,600]
--       ]
--      ,[[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[1,2,3,4,5,6]
--       ]
--      ]
--     ]
--    , array[[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--    , array[[0.5]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_2d_v_im2col
--   (
--     array
--     [
--      [
--       [[1,2,3,4,5,6]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ]
--      ,[[1,2,3,4,5,6]
--       ,[100,200,300,400,500,600]
--       ,[10,20,30,40,50,60]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[-1,-2,-3,-4,-5,-6]
--       ]
--      ],
--      [
--       [[10,20,30,40,50,60]
--       ,[1,2,3,4,5,6]
--       ,[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[100,200,300,400,500,600]
--       ]
--      ,[[-1,-2,-3,-4,-5,-6]
--       ,[-10,-20,-30,-40,-50,-60]
--       ,[10,20,30,40,50,60]
--       ,[100,200,300,400,500,600]
--       ,[1,2,3,4,5,6]
--       ]
--      ]
--     ]
--    , array
--     [
--      [
--       [[1.1, 1.1, 1.1], [1.1, 2.1, 1.1], [1.1, 1.1, 1.1]]
--      ,[[1.1, 1.1, 1.1], [1.1, 3.1, 1.1], [1.1, 1.1, 1.1]]
--      ],
--      [
--       [[1.1, 1.1, 1.1], [1.1, 2.5, 1.1], [1.1, 1.1, 1.1]]
--      ,[[1.1, 1.1, 1.1], [1.1, 0.1, 1.1], [1.1, 1.1, 1.1]]
--      ]
--     ]
--    , array[[[[0.5]],[[1.0]]],[[[-1.0]],[[0.9]]]]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0.0
--   ) :: decimal[] ~=` 3;
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_add(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_add
(
  i_background              anyarray                                             ,
  i_window             anyarray                                             ,  -- 窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
    
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] +` i_window order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            +` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[i_window]
                     , array[array_length(i_background, 1), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            +` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[[i_window]]
                     , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_add
--   (
--     array
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--   , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array[2, 2]
--   );

-- select sm_sc.fv_conv_add
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,-40.0,-50.0,-60.0,70.0]
--         ]
--       ]
--   , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array[2, 2]
--   );

-- select sm_sc.fv_conv_add
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array[2, 2]
--   );

-- select sm_sc.fv_conv_add
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array[2, 2]
--   );




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_de_div(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_de_div
(
  i_window             anyarray                                             ,  -- 窗口
  i_background              anyarray                                             ,
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_window /` i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_background, 1), 1, 1]
                  ) 
              else i_window 
            end
            /` i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                  ) 
              else i_window 
            end
            /` i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_de_div
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_div
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_div
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_div
--   (
--     array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_de_log(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_de_log
(
  i_window             anyarray                                             ,  -- 窗口
  i_background              anyarray                                             ,
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_window ^!` i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_background, 1), 1, 1]
                  ) 
              else i_window 
            end
            ^!` i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                  ) 
              else i_window 
            end
            ^!` i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_de_log
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_log
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_log
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_log
--   (
--     array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_de_pow(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_de_pow
(
  i_window             anyarray                                             ,  -- 窗口
  i_background              anyarray                                             ,
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_window ^` i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_background, 1), 1, 1]
                  )
              else i_window 
            end
            ^` i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                  ) 
              else i_window 
            end
            ^` i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_de_pow
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_pow
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_pow
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_pow
--   (
--     array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_de_prod_mx(anyarray, anyarray, int, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_de_prod_mx
(
  i_window_ex          anyarray                                             ,  -- 窗口自变量，该窗口自变量高宽与背景矩阵滑动窗口高宽不一致，为矩阵相乘关系
  i_background         anyarray                                             ,
  i_window_len_width   int                                                  ,  -- 滑动窗口高宽规格
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
  v_window_len            int[2] :=    array[array_length(i_window_ex, array_ndims(i_window_ex)), i_window_len_width];
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window_ex) <> 2 and array_ndims(i_window_ex) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window_ex length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_ex), v_window_len[1], v_window_len[2];
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    -- elsif v_window_len[1] <> array_length(i_window_ex, 2)
    -- then 
    --   raise exception 'imperfect window at 1d and window_x at 2d.';
    elsif array_ndims(i_window_ex) = 3 and array_length(i_window_ex, 1) <> array_length(i_background, 1)
      or array_ndims(i_window_ex) = 4 and (array_length(i_window_ex, 1) <> array_length(i_background, 1) or array_length(i_window_ex, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window_ex and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
    
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_window_ex |**| i_background[col_a_y : col_a_y + v_window_len[1] - 1][col_a_x : col_a_x + v_window_len[2] - 1] order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            case 
              when array_ndims(i_window_ex) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window_ex]
                  , array[array_length(i_background, 1), 1, 1]
                  ) 
              else i_window_ex
            end
            |**| i_background[ : ][col_a_x : col_a_x + v_window_len[1] - 1][col_a_x3 : col_a_x3 + v_window_len[2] - 1] 
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            case 
              when array_ndims(i_window_ex) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[array[i_window_ex]]
                  , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                  ) 
              else i_window_ex
            end
            |**| i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len[1] - 1][col_a_x4 : col_a_x4 + v_window_len[2] - 1] 
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_de_prod_mx
--   (
--     array[[1.0, 2.0], [-3.0, -1.0], [-3.0, 2.0]]
--   , array
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--   , 2
--   );

-- select sm_sc.fv_conv_de_prod_mx
--   (
--     array[[1.0, 2.0], [-3.0, -1.0], [-3.0, 2.0]]
--   , array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,-40.0,-50.0,-60.0,70.0]
--         ]
--       ]
--   , 2
--   );

-- select sm_sc.fv_conv_de_prod_mx
--   (
--     array[[1.0, 2.0], [-3.0, -1.0], [-3.0, 2.0]]
--   , array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   , 2
--   );

-- select sm_sc.fv_conv_de_prod_mx
--   (
--     array
--     [
--       [
--         [[1.5, 2.0], [3.0, 1.4], [2.0, 3.0]]
--       , [[1.5, 2.0], [3.0, 1.4], [2.0, 3.0]]
--       ]
--     , [
--         [[1.5, 2.0], [3.0, 1.4], [2.0, 3.0]]
--       , [[1.5, 2.0], [3.0, 1.4], [2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   , 2
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_de_sub(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_de_sub
(
  i_window             anyarray                                             ,  -- 窗口
  i_background              anyarray                                             ,
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_window -` i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[i_window]
                  , array[array_length(i_background, 1), 1, 1]
                  ) 
              else i_window 
            end
            -` i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            case 
              when array_ndims(i_window) = 2 
                then 
                  sm_sc.fv_new
                  (
                    array[[i_window]]
                  , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                  ) 
              else i_window 
            end
            -` i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_de_sub
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_sub
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_sub
--   (
--     array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_de_sub
--   (
--     array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_div(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_div
(
  i_background              anyarray                                             ,
  i_window             anyarray                                             ,  -- 窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] /` i_window order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            /` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[i_window]
                     , array[array_length(i_background, 1), 1, 1]
                     ) 
                 else i_window 
               end
             order by col_a_x3
           ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            /` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[[i_window]]
                     , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_div
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_div
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_div
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_div
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_log(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_log
(
  i_background              anyarray                                             ,
  i_window             anyarray                                             ,  -- 窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] ^!` i_window order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            ^!` case 
                  when array_ndims(i_window) = 2 
                    then 
                      sm_sc.fv_new
                      (
                        array[i_window]
                      , array[array_length(i_background, 1), 1, 1]
                      ) 
                  else i_window 
                end
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            ^!` case 
                  when array_ndims(i_window) = 2 
                    then 
                      sm_sc.fv_new
                      (
                        array[[i_window]]
                      , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                      ) 
                 else i_window 
               end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_log
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_log
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_log
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_log
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_mul(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_mul
(
  i_background              anyarray                                             ,
  i_window             anyarray                                             ,  -- 窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] *` i_window order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            *` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[i_window] 
                     , array[array_length(i_background, 1), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            *` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[[i_window]]
                     , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_mul
--   (
--     array
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_mul
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_mul
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_mul
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_pow(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_pow
(
  i_background              anyarray                                             ,
  i_window             anyarray                                             ,  -- 窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] ^` i_window order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            ^` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[i_window]
                     , array[array_length(i_background, 1), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            ^` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[[i_window]]
                     , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_pow
--   (
--     array
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_pow
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_pow
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_pow
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_prod_mx(anyarray, anyarray, int, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_prod_mx
(
  i_background         anyarray                                             ,
  i_window_ex          anyarray                                             ,  -- 窗口自变量，该窗口自变量高宽与背景矩阵滑动窗口高宽不一致，为矩阵相乘关系
  i_window_len_heigh   int                                                  ,  -- 滑动窗口高宽规格
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
  v_window_len            int[2] :=    array[i_window_len_heigh, array_length(i_window_ex, array_ndims(i_window_ex) - 1)];
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window_ex) <> 2 and array_ndims(i_window_ex) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window_ex length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_ex), v_window_len[1], v_window_len[2];
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    -- elsif v_window_len[2] <> array_length(i_window_ex, 1)
    -- then 
    --   raise exception 'imperfect window at 2d and window_x at 1d.';
    elsif array_ndims(i_window_ex) = 3 and array_length(i_window_ex, 1) <> array_length(i_background, 1)
      or array_ndims(i_window_ex) = 4 and (array_length(i_window_ex, 1) <> array_length(i_background, 1) or array_length(i_window_ex, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window_ex and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
    
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len[1] - 1][col_a_x : col_a_x + v_window_len[2] - 1] |**| i_window_ex order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len[1] - 1][col_a_x3 : col_a_x3 + v_window_len[2] - 1] 
            |**| case 
                   when array_ndims(i_window_ex) = 2 
                     then 
                       sm_sc.fv_new
                       (
                         array[i_window_ex]
                       , array[array_length(i_background, 1), 1, 1]
                       ) 
                   else i_window_ex
                 end
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len[1] - 1][col_a_x4 : col_a_x4 + v_window_len[2] - 1] 
            |**| case 
                   when array_ndims(i_window_ex) = 2 
                     then 
                       sm_sc.fv_new
                       (
                         array[array[i_window_ex]]
                       , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                       ) 
                   else i_window_ex
                 end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len[1] + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len[2] + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_prod_mx
--   (
--     array
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--   , array[[1.0, 2.0, -3.0], [-1.0, -3.0, 2.0]]
--   , 2
--   );

-- select sm_sc.fv_conv_prod_mx
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,-40.0,-50.0,-60.0,70.0]
--         ]
--       ]
--   , array[[1.0, 2.0, -3.0], [-1.0, -3.0, 2.0]]
--   , 2
--   );

-- select sm_sc.fv_conv_prod_mx
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   , array[[1.0, 2.0, -3.0], [-1.0, -3.0, 2.0]]
--   , 2
--   );

-- select sm_sc.fv_conv_prod_mx
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0]]
--       ]
--     ]
--   , 2
--   );




-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_conv_sub(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_conv_sub
(
  i_background              anyarray                                             ,
  i_window             anyarray                                             ,  -- 窗口
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    array_length(i_window, array_ndims(i_window) - 1)  ;
  v_window_len_width      int    :=    array_length(i_window, array_ndims(i_window))  ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window) <> 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
      
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] -` i_window order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            -` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[i_window]
                     , array[array_length(i_background, 1), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1] 
            -` case 
                 when array_ndims(i_window) = 2 
                   then 
                     sm_sc.fv_new
                     (
                       array[[i_window]]
                     , array[array_length(i_background, 1), array_length(i_background, 2), 1, 1]
                     ) 
                 else i_window 
               end
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_conv_sub
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_sub
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_sub
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_conv_sub
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array
--     [
--       [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       , [[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--       ]
--     ]
--   , array[2, 2]
--   ) :: decimal[] ~=` 3;
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_2d_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_2d_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          array_agg(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] |`| i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_2d_im2col_py
      (
        i_background 
      , i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_2d_stride_1
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_2d_stride_1
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_2d_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[-1.5, 2.0, -3.0], [-1.4, 2.0, -3.0], [-3.0, 2.0, -1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [-3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, -2.0, 3.0], [1.4, -2.0, 3.0], [3.0, -2.0, 1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [3.0, 2.0, -1.7]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_2d_stride_1
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_2d_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_2d_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          array_agg(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] |`| i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_2d_im2col_py
      (
        i_background
      , i_window
      , null
      , array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_2d_stride_window
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_2d_stride_window
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_2d_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[-1.5, 2.0, -3.0], [-1.4, 2.0, -3.0], [-3.0, 2.0, -1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [-3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, -2.0, 3.0], [1.4, -2.0, 3.0], [3.0, -2.0, 1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [3.0, 2.0, -1.7]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_2d_stride_window
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_add_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_add_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!';
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] +` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_add
      (
        i_background     , 
        i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_add_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );
-- select sm_sc.fv_opr_conv_add_stride_1
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );
-- select sm_sc.fv_opr_conv_add_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   );
-- select sm_sc.fv_opr_conv_add_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_add_stride_1
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_add_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_add_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!';
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] +` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_add
      (
        i_background     , 
        i_window,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_add_stride_window
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );
-- select sm_sc.fv_opr_conv_add_stride_window
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );
-- select sm_sc.fv_opr_conv_add_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   );
-- select sm_sc.fv_opr_conv_add_stride_window
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_add_stride_window
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_div_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_div_stride_1
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window /` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_div
      (
        i_window     ,
        i_background      
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_div_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_div_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_div_stride_1
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_div_stride_1
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_div_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_div_stride_window
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window /` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_div
      (
        i_window     ,
        i_background      ,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_div_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_div_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_div_stride_window
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_div_stride_window
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_log_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_log_stride_1
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window ^!` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_log
      (
        i_window     ,
        i_background      
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_log_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_log_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_log_stride_1
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_log_stride_1
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_log_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_log_stride_window
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window ^!` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_log
      (
        i_window     ,
        i_background      ,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_log_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_log_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_log_stride_window
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_log_stride_window
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_pow_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_pow_stride_1
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window ^` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_pow
      (
        i_window     ,
        i_background      
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_pow_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [1.0, 2.0, 3.0], [3.0, 2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,-3.0,4.0,5.0,-6.0,7.0,8.0,9.0]
--         , [1.2,2.2,-3.3,4.1,5.6,-6.2,7.4,8.4,9.4]
--         , [1.3,2.3,-3.3,4.3,5.3,-6.3,7.3,8.3,9.3]
--         , [1.4,2.4,-3.4,4.4,5.4,-6.4,7.4,8.4,9.4]
--         , [1.5,2.5,-3.5,4.5,5.5,-6.5,7.5,8.5,9.5]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_pow_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [1.0, 2.0, 3.0], [3.0, 2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,-4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,-4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,-4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,-4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,-4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,-6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,-6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,-6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,-6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,-6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_pow_stride_1
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,-8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,-8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,-8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,-8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,-8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,-3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,-3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,-3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,-3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,-3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_pow_stride_1
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_pow_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_pow_stride_window
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window ^` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_pow
      (
        i_window     ,
        i_background      ,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_pow_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [1.0, 2.0, 3.0], [3.0, 2.0, 1.0]]
--   , array
--     [
--       [1.1,2.0,3.0,4.0,-5.0,6.0,7.0,8.0,9.0]
--     , [1.2,2.2,3.3,4.1,-5.6,6.2,7.4,8.4,9.4]
--     , [1.3,2.3,3.3,4.3,-5.3,6.3,7.3,8.3,9.3]
--     , [1.4,2.4,3.4,4.4,-5.4,6.4,7.4,8.4,9.4]
--     , [1.5,2.5,3.5,4.5,-5.5,6.5,7.5,8.5,9.5]
--     , [1.5,2.5,3.5,4.5,-5.5,6.5,7.5,8.5,9.5]
--     ]
--   );

-- select sm_sc.fv_opr_conv_de_pow_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_pow_stride_window
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_pow_stride_window
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_sub_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_sub_stride_1
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window -` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_sub
      (
        i_window     ,
        i_background      
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_sub_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_sub_stride_1
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_sub_stride_1
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_sub_stride_1
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_de_sub_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_de_sub_stride_window
(                                         
  i_window             anyarray      ,    -- 窗口
  i_background         anyarray    
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_window -` i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_de_sub
      (
        i_window     ,
        i_background      ,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_de_sub_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         ]
--   );

-- select sm_sc.fv_opr_conv_de_sub_stride_window
--   (
--     array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   , array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_sub_stride_window
--   (
--     array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   , array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5]
--         ]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_de_sub_stride_window
--   (
--     array[1.5, 2, 3]
--   , array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_div_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_div_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] /` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_div
      (
        i_background     , 
        i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_div_stride_1
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_div_stride_1
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_div_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_div_stride_1
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_div_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_div_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] /` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_div
      (
        i_background     , 
        i_window,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_div_stride_window
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_div_stride_window
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_div_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_div_stride_window
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_log_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_log_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] ^!` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_log
      (
        i_background     , 
        i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_log_stride_1
--   (
--     array
--     [
--       [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--     , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--     , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--     , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--     , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--     ]
--    , array[[1.0, 2.0, 3.0], [1.0, 2.0, 3.0], [3.0, 2.0, 1.0]]
--   )

-- select sm_sc.fv_opr_conv_log_stride_1
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_log_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_log_stride_1
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_log_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_log_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] ^!` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_log
      (
        i_background     , 
        i_window,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_log_stride_window
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_log_stride_window
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_log_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_log_stride_window
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_mul_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_mul_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] *` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_mul
      (
        i_background     , 
        i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_mul_stride_1
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_mul_stride_1
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_mul_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[-1.5, 2.0, -3.0], [-1.4, 2.0, -3.0], [-3.0, 2.0, -1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [-3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, -2.0, 3.0], [1.4, -2.0, 3.0], [3.0, -2.0, 1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [3.0, 2.0, -1.7]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_mul_stride_1
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_mul_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_mul_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] *` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_mul
      (
        i_background     , 
        i_window,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_mul_stride_window
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_mul_stride_window
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_mul_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2]
--         , [1.3,2.3,3.3,4.3,5.3,6.3]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[-1.5, 2.0, -3.0], [-1.4, 2.0, -3.0], [-3.0, 2.0, -1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [-3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, -2.0, 3.0], [1.4, -2.0, 3.0], [3.0, -2.0, 1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [3.0, 2.0, -1.7]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_mul_stride_window
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_pow_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_pow_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] ^` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_pow
      (
        i_background     , 
        i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_pow_stride_1
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_pow_stride_1
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_pow_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[-1.5, 2.0, -3.0], [-1.4, 2.0, -3.0], [-3.0, 2.0, -1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [-3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, -2.0, 3.0], [1.4, -2.0, 3.0], [3.0, -2.0, 1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [3.0, 2.0, -1.7]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_pow_stride_1
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_pow_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_pow_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] ^` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_pow
      (
        i_background     , 
        i_window,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_pow_stride_window
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_pow_stride_window
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_pow_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[-1.5, 2.0, -3.0], [-1.4, 2.0, -3.0], [-3.0, 2.0, -1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [-3.0, 2.0, 1.7]]
--       ]
--     , [
--         [[1.5, -2.0, 3.0], [1.4, -2.0, 3.0], [3.0, -2.0, 1.7]]
--       , [[-1.5, 2.0, 3.0], [1.4, -2.0, 3.0], [3.0, 2.0, -1.7]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_pow_stride_window
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_sub_stride_1(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_sub_stride_1
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] -` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_sub
      (
        i_background     , 
        i_window
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_sub_stride_1
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_sub_stride_1
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_sub_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         , [1.4,2.4,3.4,4.4,5.4,6.4,7.4,8.4,9.4]
--         , [1.5,2.5,3.5,4.5,5.5,6.5,7.5,8.5,9.5]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_sub_stride_1
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conv_sub_stride_window(anyarray, anyarray);
create or replace function sm_sc.fv_opr_conv_sub_stride_window
(
  i_background              anyarray                                             ,
  i_window             anyarray           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4
    then 
      raise exception 'no method for such i_background length!  Dims: %;', array_dims(i_background);
    elsif array_ndims(i_window) > 2 and array_ndims(i_window) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window);
    elsif array_ndims(i_window) = 3 and array_length(i_window, 1) <> array_length(i_background, 1)
      or array_ndims(i_window) = 4 and (array_length(i_window, 1) <> array_length(i_background, 1) or array_length(i_window, 2) <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    elsif array_length(i_background, array_ndims(i_background)) % array_length(i_window, array_ndims(i_window)) > 0 
      or array_length(i_background, array_ndims(i_background) - 1) % array_length(i_window, array_ndims(i_window) - 1) > 0 -- and array_ndims(i_window) > 1
    then 
      raise exception 'unperfect i_background ''s length for i_background''s.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_ndims(i_window) = 1
  then 
    if array_length(i_background, 1) < array_length(i_window, 1)
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat(i_background[col_a_y : col_a_y + array_length(i_window, 1) - 1] -` i_window order by col_a_y)
        from generate_series(1, array_length(i_background, 1) - array_length(i_window, 1) + 1, array_length(i_window, 1)) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < array_length(i_window, 1)
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < array_length(i_window, 2)
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_conv_sub
      (
        i_background     , 
        i_window,
        array[array_length(i_window, array_ndims(i_window) - 1), array_length(i_window, array_ndims(i_window))]
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conv_sub_stride_window
--   (
--     array
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--    , array[[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [3.0, -2.0, 1.0]]
--   );

-- select sm_sc.fv_opr_conv_sub_stride_window
--   (
--     array
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--   , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_sub_stride_window
--   (
--     array
--     [
--       [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--     , [
--         [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       , [
--           [1.1,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]
--         , [1.2,2.2,3.3,4.1,5.6,6.2,7.4,8.4,9.4]
--         , [1.3,2.3,3.3,4.3,5.3,6.3,7.3,8.3,9.3]
--         ]
--       ]
--     ]
--   , array
--     [
--       [
--         [[0.15, 0.2, 0.3], [1.4, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       , [[1.4, 0.2, 0.3], [0.15, 0.2, 0.3], [0.3, 0.2, 0.17]]
--       ]
--     , [
--         [[1.5, 2.0, 3.0], [3.0, 2.0, 1.7], [1.4, 2.0, 3.0]]
--       , [[3.0, 2.0, 1.7], [1.4, 2.0, 3.0], [1.5, 2.0, 3.0]]
--       ]
--     ]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_opr_conv_sub_stride_window
--   (
--     array[1.3, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[1.5, 2, 3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_pool_avg_stride_1(anyarray, int[]);
create or replace function sm_sc.fv_opr_pool_avg_stride_1
(
  i_background              anyarray                                             ,
  i_window_len              int[]           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4 or array_ndims(i_window_len) > 1
    then 
      raise exception 'no method for such i_background or i_window_len ndims!';
    elsif array_length(i_window_len, 1) > 2 and array_length(i_window_len, 1) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window_len);
    elsif array_length(i_window_len, 1) = 3 and i_window_len[1] <> array_length(i_background, 1)
      or array_length(i_window_len, 1) = 4 and (i_window_len[1] <> array_length(i_background, 1) or i_window_len[2] <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_length(i_window_len, 1) = 1
  then 
    if array_length(i_background, 1) < i_window_len[1]
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          array_agg
          (
            sm_sc.fv_aggr_slice_avg
            (
              i_background[col_a_y : col_a_y + i_window_len[1] - 1] 
            )
            order by col_a_y
          )
        from generate_series(1, array_length(i_background, 1) - i_window_len[1] + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < i_window_len[1]
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < i_window_len[2]
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_pool_avg
      (
        i_background     , 
        i_window_len
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_pool_avg_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_avg_stride_1
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--    , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_avg_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     ]
--   , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_avg_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[3, 3]
--   );

-- select sm_sc.fv_opr_pool_avg_stride_1
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_pool_max_stride_1(anyarray, int[]);
create or replace function sm_sc.fv_opr_pool_max_stride_1
(
  i_background              anyarray                                             ,
  i_window_len              int[]           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4 or array_ndims(i_window_len) > 1
    then 
      raise exception 'no method for such i_background or i_window_len ndims!';
    elsif array_length(i_window_len, 1) > 2 and array_length(i_window_len, 1) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window_len);
    elsif array_length(i_window_len, 1) = 3 and i_window_len[1] <> array_length(i_background, 1)
      or array_length(i_window_len, 1) = 4 and (i_window_len[1] <> array_length(i_background, 1) or i_window_len[2] <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_length(i_window_len, 1) = 1
  then 
    if array_length(i_background, 1) < i_window_len[1]
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          array_agg
          (
            sm_sc.fv_aggr_slice_max
            (
              i_background[col_a_y : col_a_y + i_window_len[1] - 1] 
            )
            order by col_a_y
          )
        from generate_series(1, array_length(i_background, 1) - i_window_len[1] + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < i_window_len[1]
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < i_window_len[2]
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_pool_max
      (
        i_background     , 
        i_window_len
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_pool_max_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_max_stride_1
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--    , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_max_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     ]
--   , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_max_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[3, 3]
--   );

-- select sm_sc.fv_opr_pool_max_stride_1
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_pool_none_stride_1(anyarray, int[]);
create or replace function sm_sc.fv_opr_pool_none_stride_1
(
  i_background              anyarray                                             ,
  i_window_len              int[]           -- 窗口
)
returns anyarray
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) > 4 or array_ndims(i_window_len) > 1
    then 
      raise exception 'no method for such i_background or i_window_len ndims!';
    elsif array_length(i_window_len, 1) > 2 and array_length(i_window_len, 1) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %;', array_dims(i_window_len);
    elsif array_length(i_window_len, 1) = 3 and i_window_len[1] <> array_length(i_background, 1)
      or array_length(i_window_len, 1) = 4 and (i_window_len[1] <> array_length(i_background, 1) or i_window_len[2] <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  if array_ndims(i_background) = 1 and array_length(i_window_len, 1) = 1
  then 
    if array_length(i_background, 1) < i_window_len[1]
    then 
      raise exception 'imperfect window at 1d.';
    else 
      return 
      (
        select 
          sm_sc.fa_array_concat
          (
            i_background[col_a_y : col_a_y + i_window_len[1] - 1] 
            order by col_a_y
          )
        from generate_series(1, array_length(i_background, 1) - i_window_len[1] + 1) tb_a_y(col_a_y)
      )
      ;
    end if;
  end if;
  
  if array_length(i_background, array_ndims(i_background) - 1) < i_window_len[1]
  then 
    raise exception 'imperfect window at 1d.';
  elsif array_length(i_background, array_ndims(i_background)) < i_window_len[2]
  then 
    raise exception 'imperfect window at 2d.';
  else
    return 
      sm_sc.fv_pool_none
      (
        i_background     , 
        i_window_len
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_pool_none_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_none_stride_1
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--    , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_none_stride_1
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--       ]
--     ]
--   , array[3, 3]
--   );
-- select sm_sc.fv_opr_pool_none_stride_1
--   (
--     array
--        [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0, 8.0, 9.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0, 8.0, 9.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0, 8.0, 9.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0, 8.0, 9.0]
--         ]
--    , array[3, 3]
--   );

-- select sm_sc.fv_opr_pool_none_stride_1
--   (
--     array[1, 2, 3, 4, 5, 6, 7, 8, 9]
--     , array[3]
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_avg(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_avg
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_avg(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_avg
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_avg
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_avg
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_avg
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_avg
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_avg_2d_grp_x(float[], int, int[2], int[2], int[4], float);
create or replace function sm_sc.fv_pool_avg_2d_grp_x
(
  i_array_grp_x            float[]                                     ,  -- i_array_grp_x_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_window_len             int[2]                                               ,  -- 池化窗口高宽大小
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value          float      default  0.0                        -- 补齐填充元素值
)
returns float[][]
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array_grp_x) <> 2
  then 
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array_grp_x);
  elsif array_length(i_array_grp_x, 2) % i_1d_2_2d_cnt_per_grp <> 0
  then 
    raise exception 'imperfect window at grp_x_cnt.';
  else
    return 
    (
      select 
        sm_sc.fa_mx_concat_y
        (
          array 
          [
            sm_sc.fv_mx_ele_2d_2_1d
            (
              sm_sc.fv_pool_avg_py
              (
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_array_grp_x[a_cur_y : a_cur_y][ : ]
                  , i_1d_2_2d_cnt_per_grp
                )                                        , 
                i_window_len                             ,
                i_stride                                 ,
                i_padding                                ,
                i_padding_value       
              )
            )
          ]
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array_grp_x, 1)) tb_a_cur_y(a_cur_y)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_avg_2d_grp_x
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--       ]
--     ]
--     , 7
--     , array[3, 3]
--     , array[2, 2]
--   );

-- select sm_sc.fv_pool_avg_2d_grp_x
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--       ]
--     ]
--     , 6
--     , array[3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_avg_py(float[], int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_avg_py
(
  i_array              float[]                                              ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float               default  '0.0'     -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_background      = np.float64(i_array)
  # v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :    
    return \
    np.mean( \
    np.mean( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
  
  elif v_background.ndim == 3 :
    return \
    np.mean( \
    np.mean( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()

  
  elif v_background.ndim == 4 :
    return \
    np.mean( \
    np.mean( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2] * v_background.shape[-3], v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_avg_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_avg_py
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_avg_py
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_coalesce(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_coalesce
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_coalesce(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_coalesce
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_coalesce
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_coalesce
--   (
--     array[[1,2,null,4,5,6,null]
--         , [null,20,30,40,null,60,60]
--         , [100,null,300,400,500,null,null]
--         , [-1,null,-3,null,-5,-6,-7]
--         , [-10,null,-30,null,null,-60,null]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   );

-- select sm_sc.fv_pool_coalesce
--   (
--     array[[1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );

-- select sm_sc.fv_pool_coalesce
--   (
--     array
--       [
--         [
--           [1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--         ]
--       , [
--           [1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   );

-- select sm_sc.fv_pool_coalesce
--   (
--    array
--    [
--      [
--        [
--          [1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--        ]
--      , [
--          [1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--        ]
--      ]
--    , [
--        [
--          [1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--        ]
--      , [
--          [1,2,null,4,5,6]
--         , [null,20,30,40,null,60]
--         , [100,null,300,400,500,null]
--         , [-1,null,-3,null,-5,-6]
--         , [-10,null,-30,null,null,-60]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   );



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_max(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_max
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '-99999999999999999'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_max(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_max
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_max
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_max
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_max
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_max
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_max_2d_grp_x(float[], int, int[2], int[2], int[4], float);
create or replace function sm_sc.fv_pool_max_2d_grp_x
(
  i_array_grp_x            float[]                                     ,  -- i_array_grp_x_grp_x[n:n][:] 是 第 n 个 2d 降维为 1d 样本
  i_1d_2_2d_cnt_per_grp    int                                                  ,  -- 升维的每维组元素个数，通常二维图像的的宽度
  i_window_len             int[2]                                               ,  -- 池化窗口高宽大小
  i_stride                 int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding                int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value          float      default  0.0                        -- 补齐填充元素值
)
returns float[][]
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array_grp_x) <> 2
  then 
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array_grp_x);
  elsif array_length(i_array_grp_x, 2) % i_1d_2_2d_cnt_per_grp <> 0
  then 
    raise exception 'imperfect window at grp_x_cnt.';
  else
    return 
    (
      select 
        sm_sc.fa_mx_concat_y
        (
          array 
          [
            sm_sc.fv_mx_ele_2d_2_1d
            (
              sm_sc.fv_pool_max_py
              (
                sm_sc.fv_mx_ele_1d_2_2d
                (
                  i_array_grp_x[a_cur_y : a_cur_y][ : ]
                  , i_1d_2_2d_cnt_per_grp
                )                                        , 
                i_window_len                             ,
                i_stride                                 ,
                i_padding                                ,
                i_padding_value       
              )
            )
          ]
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array_grp_x, 1)) tb_a_cur_y(a_cur_y)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_max_2d_grp_x
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0,7.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0,-7.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0 :: float,70.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0 :: float,-70.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0 :: float,700.0
--       ]
--     ]
--     , 7
--     , array[3, 3]
--     , array[2, 2]
--   );

-- select sm_sc.fv_pool_max_2d_grp_x
--   (
--     array
--     [
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--       ],
--       array[1.0 :: float,2.0 :: float,3.0,4.0,5.0,6.0
--         , -1.0 :: float,-2.0 :: float,-3.0,-4.0,-5.0,-6.0
--         , 10.0 :: float,20.0 :: float,30.0 :: float,40.0 :: float,50.0 :: float,60.0
--         , -10.0 :: float,-20.0 :: float,-30.0 :: float,-40.0 :: float,-50.0 :: float,-60.0
--         , 100.0 :: float,200.0 :: float,300.0 :: float,400.0 :: float,500.0 :: float,600.0
--       ]
--     ]
--     , 6
--     , array[3, 3]
--    , array[2, 2]
--    , array[1, 1, 1, 0]
--    , 0
--   );
-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_max_py(float[], int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_max_py
(
  i_array              float[]                                              ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float               default  '-inf'     -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_background      = np.float64(i_array)
  # v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :    
    return \
    np.max( \
    np.max( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
  
  elif v_background.ndim == 3 :
    return \
    np.max( \
    np.max( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()

  
  elif v_background.ndim == 4 :
    return \
    np.max( \
    np.max( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2] * v_background.shape[-3], v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_max_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_max_py
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_max_py
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_median(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_median
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_median(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_median
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_median
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_median
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_median
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_median
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_min(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_min
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '99999999999999999'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_min(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_min
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_min
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_min
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_min
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_min
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_min_py(float[], int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_min_py
(
  i_array              float[]                                              ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float               default  'inf'     -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_background      = np.float64(i_array)
  # v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :    
    return \
    np.min( \
    np.min( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
  
  elif v_background.ndim == 3 :
    return \
    np.min( \
    np.min( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()

  
  elif v_background.ndim == 4 :
    return \
    np.min( \
    np.min( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2] * v_background.shape[-3], v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_min_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_min_py
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_min_py
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_mode(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_mode
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_mode(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_mode
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_mode
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_mode
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_mode
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_mode
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_none(anyarray, anyarray, int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_none
(
  i_background              anyarray                                             ,
  i_window_len            int[]                                                  ,
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_window_len_heigh      int    :=    i_window_len[array_length(i_window_len, 1) - 1]        ;
  v_window_len_width      int    :=    i_window_len[array_length(i_window_len, 1)]            ;
  v_background_len_heigh  int    :=    array_length(i_background, array_ndims(i_background) - 1)  ;
  v_background_len_width  int    :=    array_length(i_background, array_ndims(i_background))  ;
  v_background_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_background_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_background) not between 2 and 4
    then 
      raise exception 'no method for such i_background length!  Dims: %; len_1: %; len_2: %;', array_dims(i_background), v_background_len_heigh, v_background_len_width;
    elsif array_ndims(i_window_len) <> 1 or array_length(i_window_len, 1) <> 2 and array_length(i_window_len, 1) <> array_ndims(i_background)
    then 
      raise exception 'no method for such i_window length!  Dims: %; len_1: %; len_2: %;', array_dims(i_window_len), v_window_len_heigh, v_window_len_width;
    elsif (coalesce(i_padding[1], 0) + v_background_len_heigh + coalesce(i_padding[2], 0) - v_window_len_heigh) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at 1d.';
    elsif (coalesce(i_padding[3], 0) + v_background_len_width + coalesce(i_padding[4], 0) - v_window_len_width) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at 2d.';
    elsif array_length(i_window_len, 1) = 3 and i_window_len[1] <> array_length(i_background, 1)
      or array_length(i_window_len, 1) = 4 and (i_window_len[1] <> array_length(i_background, 1) or i_window_len[2] <> array_length(i_background, 2))
    then 
      raise exception 'unmatch length between i_window and i_background at 3d or 4d.';
    end if;
  end if;
  
  i_background := 
    sm_sc.fv_augmented
    (
      i_background, 
      array[-i_padding[1] + 1, -i_padding[3] + 1], 
      array[v_background_len_heigh + i_padding[2], v_background_len_width + i_padding[4]], 
      i_padding_value
    );
    
  if array_ndims(i_background) = 2
  then 
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          sm_sc.fa_mx_concat_x(i_background[col_a_y : col_a_y + v_window_len_heigh - 1][col_a_x : col_a_x + v_window_len_width - 1] order by col_a_x) as a_ret_y
        from generate_series(1, v_background_len_heigh_ex - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_background_len_width_ex - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select sm_sc.fa_mx_concat_y(a_ret_y order by col_a_y) from cte_ret_y
    );
  elsif array_ndims(i_background) = 3
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x,
          sm_sc.fa_mx_concat_x3
          (
            i_background[ : ][col_a_x : col_a_x + v_window_len_heigh - 1][col_a_x3 : col_a_x3 + v_window_len_width - 1] 
            order by col_a_x3
          ) as a_ret_x
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x(col_a_x)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x3(col_a_x3)
        group by col_a_x
      )
      select sm_sc.fa_mx_concat_x(a_ret_x order by col_a_x) from cte_ret_x
    );
  elsif array_ndims(i_background) = 4
  then 
    return 
    (
      with 
      cte_ret_x as
      (
        select 
          col_a_x3,
          sm_sc.fa_mx_concat_x4
          (
            i_background[ : ][ : ][col_a_x3 : col_a_x3 + v_window_len_heigh - 1][col_a_x4 : col_a_x4 + v_window_len_width - 1]
            order by col_a_x4
          ) as a_ret_x2
        from generate_series(1, v_background_len_heigh - v_window_len_heigh + i_stride[1], i_stride[1]) tb_a_x3(col_a_x3)
          , generate_series(1, v_background_len_width - v_window_len_width + i_stride[2], i_stride[2]) tb_a_x4(col_a_x4)
        group by col_a_x3
      )
      select sm_sc.fa_mx_concat_x3(a_ret_x2 order by col_a_x3) from cte_ret_x
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_none
--   (
--     array
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--   , array[3, 3]
--   , array[2, 2]
--   );

-- select sm_sc.fv_pool_none
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,-40.0,-50.0,-60.0,70.0]
--         ]
--       ]
--   , array[3, 3]
--   , array[2, 2]
--   );

-- select sm_sc.fv_pool_none
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   , array[3, 3]
--   , array[2, 2]
--   );

-- select sm_sc.fv_pool_none
--   (
--     array
--     [
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,-7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [-100.0,200.0,300.0,400.0,-500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,-20.0,30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--     , [
--         [
--           [1.0,-2.0,3.0,-4.0,-5.0,6.0,7.0]
--         , [-10.0,20.0,30.0,40.0,-50.0,60.0,-70.0]
--         , [100.0,200.0,-300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,3.0,-4.0,5.0,-6.0,-7.0]
--         , [-10.0,20.0,-30.0,40.0,-50.0,-60.0,70.0]
--         ]
--       , [
--           [1.0,2.0,-3.0,4.0,5.0,6.0,-7.0]
--         , [-10.0,20.0,30.0,40.0,50.0,60.0,-70.0]
--         , [100.0,200.0,300.0,-400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,4.0,-5.0,-6.0,-7.0]
--         , [10.0,-20.0,-30.0,40.0,50.0,60.0,70.0]
--         ]
--       ]
--     ]
--   -- , array[[1.5, 2.0, 3.0], [1.4, 2.0, 3.0], [3.0, 2.0, 1.7]]
--  ,  array[2, 2, 3, 3]
--   , array[2, 2]
--   );




-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_none_py(float[], int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_none_py
(
  i_array              float[]                                              ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float               default  '0.0'     -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_background      = np.float64(i_array)
  # v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :    
    return \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .transpose(0,2,1,3) \
    .reshape(((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1) * i_window_len[0], ((v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1) * i_window_len[1]) \
    .tolist()
  
  elif v_background.ndim == 3 :
    return \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .transpose(0,1,3,2,4) \
    .reshape(v_background.shape[0], ((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1) * i_window_len[0], ((v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1) * i_window_len[1]) \
    .tolist()
  
  elif v_background.ndim == 4 :
    return \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2] * v_background.shape[-3], v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    .transpose(0,1,2,4,3,5) \
    .reshape(v_background.shape[0], v_background.shape[1], ((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1) * i_window_len[0], ((v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1) * i_window_len[1]) \
    .tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_none_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_none_py
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_none_py
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_prod(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_prod
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '1.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_prod(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_prod
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_prod
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_prod
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_prod
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_prod
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_prod_py(float[], int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_prod_py
(
  i_array              float[]                                              ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float               default  '1.0'     -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_background      = np.float64(i_array)
  # v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :    
    return \
    np.prod( \
    np.prod( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
  
  elif v_background.ndim == 3 :
    return \
    np.prod( \
    np.prod( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()

  
  elif v_background.ndim == 4 :
    return \
    np.prod( \
    np.prod( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2] * v_background.shape[-3], v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_prod_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_prod_py
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_prod_py
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_ptp(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_ptp
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  null                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_ptp(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_ptp
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_ptp
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_ptp
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_ptp
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_ptp
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_sum(anyarray, int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_sum
(
  i_array              anyarray                                             ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      anyelement          default  '0.0'                        -- 补齐填充元素值
)
returns anyarray
as
$$
declare 
  v_array_len_heigh        int   :=   array_length(i_array, array_ndims(i_array) - 1) ;
  v_array_len_width        int   :=   array_length(i_array, array_ndims(i_array))     ;
  v_array_len_heigh_ex     int   :=   coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0);     --   新背景矩阵高
  v_array_len_width_ex     int   :=   coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0);     --   新背景矩阵宽
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计二维长度
    if array_ndims(i_array) not in (2, 3, 4)
    then 
      raise exception 'unsupport ndims of i_array.';
    elsif (coalesce(i_padding[1], 0) + v_array_len_heigh + coalesce(i_padding[2], 0) - i_window_len[1]) % i_stride[1] <> 0
    then 
      raise exception 'imperfect window at heigh.';
    elsif (coalesce(i_padding[3], 0) + v_array_len_width + coalesce(i_padding[4], 0) - i_window_len[2]) % i_stride[2] <> 0
    then 
      raise exception 'imperfect window at width.';
    end if;
  end if;
  
  if array_ndims(i_array) = 2
  then
    i_array := 
      sm_sc.fv_augmented
      (
        i_array, 
        array[-i_padding[1] + 1, -i_padding[3] + 1], 
        array[v_array_len_heigh + i_padding[2], v_array_len_width + i_padding[4]], 
        i_padding_value
      );
    return 
    (
      with 
      cte_ret_y as
      (
        select 
          col_a_y,
          array_agg(sm_sc.fv_aggr_slice_sum(i_array[col_a_y : col_a_y + i_window_len[1] - 1][col_a_x : col_a_x + i_window_len[2] - 1]) order by col_a_x) as ret_y
        from generate_series(1, v_array_len_heigh_ex - i_window_len[1] + i_stride[1], i_stride[1]) tb_a_y(col_a_y)
          , generate_series(1, v_array_len_width_ex - i_window_len[2] + i_stride[2], i_stride[2]) tb_a_x(col_a_x)
        group by col_a_y
      )
      select array_agg(ret_y order by col_a_y) from cte_ret_y
    );
  
  elsif array_ndims(i_array) = 3
  then 
    return 
    (
      select 
        array_agg 
        (
          sm_sc.fv_pool_sum
          (
            sm_sc.fv_mx_slice_3d_2_2d
            (
              i_array[a_cur_y : a_cur_y]
            , 1
            )
          , i_window_len       
          , i_stride             
          , i_padding          
          , i_padding_value
          )
          order by a_cur_y
        )
      from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
    );
  
  elsif array_ndims(i_array) = 4
  then 
    return 
    (
      with 
      cte_agg_x as
      (
        select 
          a_cur_y,
          array_agg 
          (
            sm_sc.fv_pool_sum
            (
              sm_sc.fv_mx_slice_4d_2_2d
              (
                i_array[a_cur_y : a_cur_y][a_cur_x : a_cur_x][ : ][ : ]
              , array[1, 2]
              , array[1, 1]
              )
            , i_window_len       
            , i_stride             
            , i_padding          
            , i_padding_value
            )
            order by a_cur_x
          ) as a_agg_x
        from generate_series(1, array_length(i_array, 1)) tb_a_cur_y(a_cur_y)
          , generate_series(1, array_length(i_array, 2)) tb_a_cur_x(a_cur_x)
        group by a_cur_y
      )
      select 
        array_agg(a_agg_x order by a_cur_y)
      from cte_agg_x
    );
    
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_sum
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_sum
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_sum
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- py 版本的矩阵乘法
-- -- 需要安装 plpython3u
--   dnf -y install postgresql13-plpython3.x86_64
-- --   dnf -y install postgresql13-pltcl.x86_64
--   pip3 install numpy --timeout=100 -i https://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com

-- -- 需要安装扩展
--   create extension plpython3u;
-- --   create extension pltclu;  -- or pltcl

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_pool_sum_py(float[], int[2], int[2], int[4], anyelement);
create or replace function sm_sc.fv_pool_sum_py
(
  i_array              float[]                                              ,
  i_window_len         int[2]                                               ,  -- 池化窗口高宽大小
  i_stride             int[2]              default  array[1, 1]             ,  -- 纵向与横向步长
  i_padding            int[4]              default  array[0, 0, 0, 0]       ,  -- 上下左右补齐行数/列数
  i_padding_value      float               default  '0.0'     -- 补齐填充元素值
)
returns float[]
as
$$
  import numpy as np
  
  v_background      = np.float64(i_array)
  # v_window_len      = np.int8(i_window_len)
  
  # padding
  if not np.array_equal(i_padding, np.array([0, 0, 0, 0])) :
    if v_background.ndim == 2 :
      v_background = np.pad(v_background \
      , ((i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 3 :
      v_background = np.pad(v_background \
      , ((0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
    elif v_background.ndim == 4 :
      v_background = np.pad(v_background \
      , ((0, 0), (0, 0), (i_padding[0], i_padding[1]), (i_padding[2], i_padding[3])) \
      , 'constant' \
      , constant_values = i_padding_value)
  
  v_ele_len = v_background.dtype.itemsize

  # im2col, matmul
  if v_background.ndim == 2 :    
    return \
    np.sum( \
    np.sum( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=((v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
  
  elif v_background.ndim == 3 :
    return \
    np.sum( \
    np.sum( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()

  
  elif v_background.ndim == 4 :
    return \
    np.sum( \
    np.sum( \
    np.lib.stride_tricks.as_strided(v_background \
    , shape=(v_background.shape[0], v_background.shape[1], (v_background.shape[-2] - i_window_len[0]) // i_stride[0] + 1, (v_background.shape[-1] - i_window_len[1]) // i_stride[1] + 1, i_window_len[0], i_window_len[1]) \
    , strides=(v_ele_len * v_background.shape[-1] * v_background.shape[-2] * v_background.shape[-3], v_ele_len * v_background.shape[-1] * v_background.shape[-2], v_ele_len * i_stride[0] * v_background.shape[-1], v_ele_len * i_stride[1], v_ele_len * v_background.shape[-1], v_ele_len) \
    ) \
    , axis=-1 \
    ) \
    , axis=-1 \
    ).tolist()
      
$$
language plpython3u stable
parallel safe
;

-- -- set search_path to sm_sc;
-- select sm_sc.fv_pool_sum_py
--   (
--     array[[1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--          ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_sum_py
--   (
--     array
--       [
--         [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       , [
--           [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--         , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--         , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--         , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--         , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--         ]
--       ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;

-- select sm_sc.fv_pool_sum_py
--   (
--    array
--    [
--      [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    , [
--        [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      , [
--          [1.0,2.0,3.0,4.0,5.0,6.0,7.0]
--        , [10.0,20.0,30.0,40.0,50.0,60.0,70.0]
--        , [100.0,200.0,300.0,400.0,500.0,600.0,700.0]
--        , [-1.0,-2.0,-3.0,-4.0,-5.0,-6.0,-7.0]
--        , [-10.0,-20.0,-30.0,-40.0,-50.0,-60.0,-70.0]
--        ]
--      ]
--    ]
--    , array[3, 3]
--    , array[2, 2]
--   ) :: decimal[] ~=` 3;



-- -- 动态执行代数式/函数式的字符串，返回计算数值结果，用于推导，不适用于高性能计算
-- drop function if exists sm_sc.fv_algebra_execute(text);
create or replace function sm_sc.fv_algebra_execute
(
  i_algebra_sql     text
)
returns float
as
$$
declare -- here
  v_ret    float;
begin
  execute ('select (' || i_algebra_sql || '):: float') into v_ret;
  return v_ret;
end
$$
language plpgsql stable
cost 100;

-- select sm_sc.fv_algebra_execute('2.5 + 3.6')
-- select sm_sc.fv_algebra_execute('(2.5 + 3.6)')

-- ---------------------------------------------------------------------------------------------
-- 函数式方式的数学计算
-- drop function if exists sm_sc.fv_algebra_execute(varchar(64), float[]);
create or replace function sm_sc.fv_algebra_execute
(
  i_fn           varchar(64)   ,
  i_fn_params    float[]
)
returns float
as
$$
begin
  -- -- return
  -- --   case i_fn
  -- --     when 'sm_sc.fv_opr_add'    then sm_sc.fv_aggr_slice_sum(i_fn_params)         :: float
  -- --     when 'sm_sc.fv_opr_sub'    then i_fn_params[1] - i_fn_params[2]                :: float
  -- --     when 'sm_sc.fv_opr_mul'    then sm_sc.fv_aggr_slice_prod(i_fn_params)        :: float
  -- --     when 'sm_sc.fv_opr_div'    then i_fn_params[1] / nullif(i_fn_params[2], 0.0 :: float)   :: float
  -- --     when 'sm_sc.fv_opr_pow'    then power(i_fn_params[1], i_fn_params[2])          :: float
  -- --     when 'sm_sc.fv_opr_exp'    then exp(i_fn_params[1])                            :: float
  -- --     when 'sm_sc.fv_opr_log'    then log(i_fn_params[1] :: decimal, i_fn_params[2] :: decimal)    :: float
  -- --     when 'sm_sc.fv_opr_ln'     then ln(i_fn_params[1])                             :: float
  -- --     when 'sm_sc.fv_sin'        then sin(i_fn_params[1])                            :: float
  -- --     when 'sm_sc.fv_cos'        then cos(i_fn_params[1])                            :: float
  -- --     when 'sm_sc.fv_tan'        then tan(i_fn_params[1])                            :: float
  -- --     when 'sm_sc.fv_cot'        then cot(i_fn_params[1])                            :: float
  -- --     when 'sm_sc.fv_sec'        then 1.0 :: float/ nullif(cos(i_fn_params[1]), 0.0 :: float)         :: float
  -- --     when 'sm_sc.fv_csc'        then 1.0 :: float/ nullif(sin(i_fn_params[1]), 0.0 :: float)         :: float
  -- --     when 'sm_sc.fv_asin'       then asin(i_fn_params[1])                           :: float
  -- --     when 'sm_sc.fv_acos'       then acos(i_fn_params[1])                           :: float
  -- --     when 'sm_sc.fv_atan'       then atan(i_fn_params[1])                           :: float
  -- --     when 'sm_sc.fv_acot'       then atan(1.0 :: float/ nullif(i_fn_params[1], 0.0 :: float))        :: float
  -- --     when 'sm_sc.fv_asec'       then acos(1.0 :: float/ nullif(i_fn_params[1], 0.0 :: float))        :: float
  -- --     when 'sm_sc.fv_acsc'       then asin(1.0 :: float/ nullif(i_fn_params[1], 0.0 :: float))        :: float
  -- --     when 'sm_sc.fv_sinh'       then sinh(i_fn_params[1])                           :: float
  -- --     when 'sm_sc.fv_cosh'       then cosh(i_fn_params[1])                           :: float
  -- --     when 'sm_sc.fv_tanh'       then tanh(i_fn_params[1])                           :: float
  -- --     when 'sm_sc.fv_asinh'      then asinh(i_fn_params[1])                          :: float
  -- --     when 'sm_sc.fv_acosh'      then acosh(i_fn_params[1])                          :: float
  -- --     when 'sm_sc.fv_atanh'      then atanh(i_fn_params[1])                          :: float
  -- --     else i_fn_params[1]
  -- --   end
  -- -- ;
  if i_fn = 'sm_sc.fv_opr_add'
  then 
    return sm_sc.fv_aggr_slice_sum(i_fn_params);
  elsif i_fn = 'sm_sc.fv_opr_sub'
  then 
    return i_fn_params[1] - i_fn_params[2];
  elsif i_fn = 'sm_sc.fv_opr_mul'
  then 
    return sm_sc.fv_aggr_slice_prod(i_fn_params);
  elsif i_fn = 'sm_sc.fv_opr_div'
  then 
    return i_fn_params[1] / nullif(i_fn_params[2], 0.0);
  elsif i_fn = 'sm_sc.fv_opr_pow'
  then 
    return power(i_fn_params[1], i_fn_params[2]);
  elsif i_fn = 'sm_sc.fv_opr_exp'
  then 
    return exp(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_opr_log'
  then 
    return log(i_fn_params[1] :: decimal, i_fn_params[2] :: decimal);
  elsif i_fn = 'sm_sc.fv_opr_ln'
  then 
    return ln(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_sin'
  then 
    return sin(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_cos'
  then 
    return cos(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_tan'
  then 
    return tan(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_cot'
  then 
    return cot(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_sec'
  then 
    return 1.0 / nullif(cos(i_fn_params[1]), 0.0);
  elsif i_fn = 'sm_sc.fv_csc'
  then 
    return 1.0 / nullif(sin(i_fn_params[1]), 0.0);
  elsif i_fn = 'sm_sc.fv_asin'
  then 
    return asin(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_acos'
  then 
    return acos(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_atan'
  then 
    return atan(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_acot'
  then 
    return atan(1.0 / nullif(i_fn_params[1], 0.0));
  elsif i_fn = 'sm_sc.fv_asec'
  then 
    return acos(1.0 / nullif(i_fn_params[1], 0.0));
  elsif i_fn = 'sm_sc.fv_acsc'
  then 
    return asin(1.0 / nullif(i_fn_params[1], 0.0));
  elsif i_fn = 'sm_sc.fv_sinh'
  then 
    return sinh(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_cosh'
  then 
    return cosh(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_tanh'
  then 
    return tanh(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_asinh'
  then 
    return asinh(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_acosh'
  then 
    return acosh(i_fn_params[1]);
  elsif i_fn = 'sm_sc.fv_atanh'
  then 
    return atanh(i_fn_params[1]);
  else 
    return i_fn_params[1];
  end if;
end
$$
language plpgsql stable
cost 100;


-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_add'   , array[1.5, 2.0 :: float, 3.4])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_sub'   , array[1.5, 2.0])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_mul'   , array[1.5, 2.0 :: float, 3.4])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_div'   , array[1.5, 2.0])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_pow'   , array[1.5, 2.0])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_exp'   , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_log'   , array[1.5, 2.0])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_opr_ln'    , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_sin'       , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_cos'       , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_tan'       , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_cot'       , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_sec'       , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_csc'       , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_asin'      , array[0.5 :: float])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_acos'      , array[0.5 :: float])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_atan'      , array[0.5 :: float])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_acot'      , array[0.5 :: float])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_asec'      , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_acsc'      , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_sinh'      , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_cosh'      , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_tanh'      , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_asinh'     , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_acosh'     , array[1.5])
-- select sm_sc.fv_algebra_execute('sm_sc.fv_atanh'     , array[0.5 :: float])
-- drop function if exists sm_sc.fv_lambda_arr(bigint, varchar(64), float[], float[][], float[], float[][]);
create or replace function sm_sc.fv_lambda_arr
(
  i_node_no               bigint,
  i_lambda                varchar(64)                      ,
  i_param_1               float[]                 ,
  i_param_2               float[]   default null  ,      -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立
  i_input_arr_asso        float[]   default null  ,   
  i_param_3               float[]   default null
)
returns float[][]
as
$$
declare 
  -- -- v_lambda varchar(64)    :=    coalesce((select 'v...' from sm_sc.tb_dic... where col...'k...' = i_lambda limit 1), i_lambda)
 _v_debug         float[][];
 
 -- -- debug 
 -- v_begin_clock    timestamp     := clock_timestamp();
begin
  -- 动态 sql 会影响性能，所以还是用 case 判断
  
-- -- debug 
-- raise notice 'lambda % begin:: i_lambda: %; len_input: %; len_co_val: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(i_param_1, 1), array_length(i_param_1, 2)], array[array_length(i_param_2, 1), array_length(i_param_2, 2)], to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_param_1 is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(i_param_1) is true
    then 
      raise exception 'there is null value in i_param_1!  i_node_no: %.  ', i_node_no;
    elsif 'NaN' :: float = any(i_param_1)
      or 'NaN' :: float = any(i_param_2)
      or i_param_1 is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(i_param_1) is true
    then 
      raise exception 'there is NaN value in i_param_1 or i_param_2!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  if i_lambda like '07_%'
  then 
    if i_lambda = '07_aggr_slice_sum'
    then 
      _v_debug := 
        sm_sc.fv_aggr_slice_sum(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_prod'
    then
      _v_debug := 
        sm_sc.fv_aggr_slice_prod(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_avg'
    then
      _v_debug := 
        sm_sc.fv_aggr_slice_avg(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_max'
    then
      _v_debug := 
        sm_sc.fv_aggr_slice_max(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_min'
    then
      _v_debug := 
        sm_sc.fv_aggr_slice_min(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_sum'
    then 
      _v_debug := 
        sm_sc.fv_aggr_chunk_sum(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_prod'
    then
      _v_debug := 
        sm_sc.fv_aggr_chunk_prod(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_avg'
    then
      _v_debug := 
        sm_sc.fv_aggr_chunk_avg(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_max'
    then
      _v_debug := 
        sm_sc.fv_aggr_chunk_max(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_min'
    then
      _v_debug := 
        sm_sc.fv_aggr_chunk_min(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    end if;
    
  -- -- elsif i_lambda like '06_%'
  -- -- then 
  -- --   if i_lambda = '06_aggr_mx_sum'
  -- --   then 
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_prod'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_avg'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_max'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_min'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_y'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_x'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_x3'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_x4'
  -- --   then
  -- --     _v_debug := 
  -- --       
  -- --     ;
  -- --   end if;
    
  elsif i_lambda like '05_%'
  then 
    if i_lambda = '05_pool_max_2d_grp_x'
    then 
      _v_debug := 
        sm_sc.fv_pool_max_2d_grp_x
        (
          i_param_1                                                           ,   
          i_input_arr_asso[1] :: int                                              ,   -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg_2d_grp_x'
    then
      _v_debug := 
        sm_sc.fv_pool_avg_2d_grp_x
        (
          i_param_1                                                           ,   
          i_input_arr_asso[1] :: int                                              ,   -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_max'
    then
      _v_debug := 
        sm_sc.fv_pool_max_py
        (
          i_param_1                                                           ,   
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg'
    then
      _v_debug := 
        sm_sc.fv_pool_avg_py
        (
          i_param_1                                                           ,   
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_none'
    then
      _v_debug := 
        sm_sc.fv_pool_none_py
        (
          i_param_1                                                           ,   
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_2d_grp_x'
    then
      _v_debug := 
        sm_sc.fv_conv_2d_grp_x
        (
          i_param_1   :: float[]                                     ,   
          i_input_arr_asso[1] :: int                                              ,   -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_param_2           :: float[]                                     ,   
          i_input_arr_asso[3] :: int                                              ,   -- 规约：存放 i_window_len_x 
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_2d'
    then
      _v_debug := 
        sm_sc.fv_conv_2d_im2col_py
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          i_param_3                                              ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_add'
    then
      _v_debug := 
        sm_sc.fv_conv_add
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_sub'
    then
      _v_debug := 
        sm_sc.fv_conv_sub
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_mul'
    then
      _v_debug := 
        sm_sc.fv_conv_mul
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_div'
    then
      _v_debug := 
        sm_sc.fv_conv_div
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_pow'
    then
      _v_debug := 
        sm_sc.fv_conv_pow
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_log'
    then
      _v_debug := 
        sm_sc.fv_conv_log
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_prod_mx'
    then
      _v_debug := 
        sm_sc.fv_conv_prod_mx
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          i_input_arr_asso[2] :: int                                        ,   -- 规约：存放 i_window_len_heigh
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_sub'
    then
      _v_debug := 
        sm_sc.fv_conv_de_sub
        (
          i_param_1   :: float[]                                     ,   -- 窗口
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_div'
    then
      _v_debug := 
        sm_sc.fv_conv_de_div
        (
          i_param_1   :: float[]                                     ,   -- 窗口
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_pow'
    then
      _v_debug := 
        sm_sc.fv_conv_de_pow
        (
          i_param_1   :: float[]                                     ,   -- 窗口
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_log'
    then
      _v_debug := 
        sm_sc.fv_conv_de_log
        (
          i_param_1   :: float[]                                     ,   -- 窗口
          i_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_prod_mx'
    then
      _v_debug := 
        sm_sc.fv_conv_de_prod_mx
        (
          i_param_1   :: float[]                                     ,   
          i_param_2           :: float[]                                     ,   
          i_input_arr_asso[3] :: int                                        ,   -- 规约：存放 i_window_len_width
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    end if;
    
  elsif i_lambda like '04_%'
  then 
    if i_lambda = '04_new'
    then 
      _v_debug := 
        sm_sc.fv_new(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '04_repeat_axis'
    then
      _v_debug := 
        sm_sc.fv_repeat_axis_py(i_param_1, sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][ : ]) :: int[], sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][ : ]) :: int[])
      ;
    elsif i_lambda = '04_apad'
    then
      _v_debug := 
        sm_sc.fv_apad(i_param_1, i_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_bpad'
    then
      _v_debug := 
        sm_sc.fv_bpad(i_param_1, i_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_lpad'
    then
      _v_debug := 
        sm_sc.fv_lpad(i_param_1, i_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_rpad'
    then
      _v_debug := 
        sm_sc.fv_lpad(i_param_1, i_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_transpose'
    then
      _v_debug := 
        sm_sc.fv_opr_transpose_py(i_param_1)
      ;
    elsif i_lambda = '04_transpose_i'
    then
      _v_debug := 
        sm_sc.fv_opr_transpose_i_py(i_param_1)
      ;
    elsif i_lambda = '04_chunk_transpose'
    then
      _v_debug := 
        sm_sc.fv_chunk_transpose(i_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_transpose_nd'
    then
      _v_debug := 
        sm_sc.fv_opr_transpose_nd_py(i_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_90'
    then
      _v_debug := 
        sm_sc.fv_opr_turn_90_py(i_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_180'
    then
      _v_debug := 
        sm_sc.fv_opr_turn_180_py(i_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_mirror'
    then
      _v_debug := 
        sm_sc.fv_opr_mirror_py(i_param_1, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_ele_3d_2_2d'
    then
      _v_debug := 
        sm_sc.fv_mx_ele_3d_2_2d_py
        (
          i_param_1
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        )
      ;
    elsif i_lambda = '04_mx_ele_2d_2_3d'
    then
      _v_debug := 
        sm_sc.fv_mx_ele_2d_2_3d_py
        (
          i_param_1
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int
        , i_input_arr_asso[4] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_4d_2_3d'
    then
      _v_debug := 
        sm_sc.fv_mx_ele_4d_2_3d_py
        (
          i_param_1
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        )
      ;
    elsif i_lambda = '04_mx_ele_3d_2_4d'
    then
      _v_debug := 
        sm_sc.fv_mx_ele_3d_2_4d_py
        (
          i_param_1
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int
        , i_input_arr_asso[4] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_flatten_2dims'
    then
      _v_debug := 
        sm_sc.fv_mx_ele_flatten_2dims_py
        (
          i_param_1
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        )
      ;
    elsif i_lambda = '04_mx_slice_3d_2_2d'
    then
      _v_debug := 
        sm_sc.fv_mx_slice_3d_2_2d_py(i_param_1, i_input_arr_asso[1] :: int, i_input_arr_asso[2] :: int)
      ;
    elsif i_lambda = '04_mx_slice_4d_2_2d'
    then
      _v_debug := 
        sm_sc.fv_mx_slice_4d_2_2d_py(i_param_1, sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][1 : 2]) :: int[], sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][1 : 2]) :: int[])
      ;
    elsif i_lambda = '04_mx_slice_4d_2_3d'
    then
      _v_debug := 
        sm_sc.fv_mx_slice_4d_2_3d_py(i_param_1, i_input_arr_asso[1] :: int, i_input_arr_asso[2] :: int)
      ;
    elsif i_lambda = '04_mx_ascend_dim'
    then
      _v_debug := 
        sm_sc.fv_mx_ascend_dim(i_param_1, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_descend_dim'
    then
      _v_debug := 
        sm_sc.fv_mx_descend_dim_py(i_param_1, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_rand_pick_y'
    then
      _v_debug := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_y_pick(i_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_rand_pick_x'
    then
      _v_debug := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_x_pick(i_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_rand_pick_x3'
    then
      _v_debug := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_x3_pick(i_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_rand_pick_x4'
    then
      _v_debug := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_x4_pick(i_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_chunk'
    then
      _v_debug := 
        sm_sc.fv_chunk
        (
          i_param_1
        , i_input_arr_asso :: int[]
        )
      ;
    elsif i_lambda = '04_slice_y'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_slice_y
          (
            i_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_slice_x
          (
            i_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x3'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_slice_x3
          (
            i_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x4'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_slice_x4
          (
            i_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_y'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_sample_y    -- fv_sample_y_py
          (
            i_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(i_param_2, case array_ndims(i_param_1) when 1 then array[0.0] when 2 then array[[0.0]] when 3 then array[[[0.0]]] when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_sample_x    -- fv_sample_x_py
          (
            i_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(i_param_2, case array_ndims(i_param_1) when 2 then array[[0.0]] when 3 then array[[[0.0]]] when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x3'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_sample_x3    -- fv_sample_x3_py
          (
            i_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(i_param_2, case array_ndims(i_param_1) when 3 then array[[[0.0]]] when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x4'
    then
      _v_debug := 
      (
        select 
          sm_sc.fv_sample_x4     -- fv_sample_x4_py
          (
            i_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(i_param_2, case array_ndims(i_param_1) when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_lower_tri_mx'
    then
      _v_debug := sm_sc.fv_lower_tri_mx(i_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_upper_tri_mx'
    then
      _v_debug := sm_sc.fv_upper_tri_mx(i_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_lower_tri_mx_ex'
    then
      _v_debug := sm_sc.fv_lower_tri_mx_ex(i_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_upper_tri_mx_ex'
    then
      _v_debug := sm_sc.fv_upper_tri_mx_ex(i_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_lmask'
    then
      _v_debug := sm_sc.fv_lmask(i_param_1, i_param_2 :: int[], i_input_arr_asso[1]);
    elsif i_lambda = '04_rmask'
    then
      _v_debug := sm_sc.fv_rmask(i_param_1, i_param_2 :: int[], i_input_arr_asso[1]);
    elsif i_lambda = '04_amask'
    then
      _v_debug := sm_sc.fv_amask(i_param_1, i_param_2 :: int[], i_input_arr_asso[1]);
    elsif i_lambda = '04_bmask'
    then
      _v_debug := sm_sc.fv_bmask(i_param_1, i_param_2 :: int[], i_input_arr_asso[1]);
    end if;
    
  elsif i_lambda like '03_%'
  then 
    if i_lambda = '03_sigmoid'
    then 
      _v_debug := 
        sm_sc.fv_activate_sigmoid(i_param_1)
      ;
    elsif i_lambda = '03_relu'
    then
      _v_debug := 
        sm_sc.fv_activate_relu(i_param_1)
      ;
    elsif i_lambda = '03_leaky_relu'
    then
      _v_debug := 
        sm_sc.fv_activate_leaky_relu(i_param_1, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_elu'
    then
      _v_debug := 
        sm_sc.fv_activate_elu(i_param_1, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_selu'
    then
      _v_debug := 
        sm_sc.fv_activate_selu(i_param_1)
      ;
    elsif i_lambda = '03_gelu'
    then
      _v_debug := 
        sm_sc.fv_activate_gelu(i_param_1)
      ;
    elsif i_lambda = '03_swish'
    then
      _v_debug := 
        sm_sc.fv_activate_swish(i_param_1)
      ;
    elsif i_lambda = '03_softplus'
    then
      _v_debug := 
        sm_sc.fv_activate_softplus(i_param_1)
      ;
    elsif i_lambda = '03_boxcox'
    then
      _v_debug := 
        sm_sc.fv_activate_boxcox(i_param_1, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_softmax'
    then
      _v_debug := 
        sm_sc.fv_redistr_softmax(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '03_softmax_ex'
    then
      _v_debug := 
        sm_sc.fv_redistr_softmax_ex(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '03_zscore'
    then
      _v_debug := 
        sm_sc.fv_redistr_zscore(i_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    end if;
    
  elsif i_lambda like '02_%'
  then 
    if i_lambda = '02_sin'
    then
      _v_debug := 
        sm_sc.fv_sin_py(i_param_1)
      ;
    elsif i_lambda = '02_cos'
    then
      _v_debug := 
        sm_sc.fv_cos_py(i_param_1)
      ;
    elsif i_lambda = '02_tan'
    then
      _v_debug := 
        sm_sc.fv_tan_py(i_param_1)
      ;
    elsif i_lambda = '02_cot'
    then
      _v_debug := 
        sm_sc.fv_cot(i_param_1)
      ;
    elsif i_lambda = '02_sec'
    then
      _v_debug := 
        sm_sc.fv_sec(i_param_1)
      ;
    elsif i_lambda = '02_csc'
    then
      _v_debug := 
        sm_sc.fv_csc(i_param_1)
      ;
    elsif i_lambda = '02_asin'
    then
      _v_debug := 
        sm_sc.fv_asin(i_param_1)
      ;
    elsif i_lambda = '02_acos'
    then
      _v_debug := 
        sm_sc.fv_acos(i_param_1)
      ;
    elsif i_lambda = '02_atan'
    then
      _v_debug := 
        sm_sc.fv_atan(i_param_1)
      ;
    elsif i_lambda = '02_acot'
    then
      _v_debug := 
        sm_sc.fv_acot(i_param_1)
      ;
    elsif i_lambda = '02_sinh'
    then
      _v_debug := 
        sm_sc.fv_sinh_py(i_param_1)
      ;
    elsif i_lambda = '02_cosh'
    then
      _v_debug := 
        sm_sc.fv_cosh_py(i_param_1)
      ;
    elsif i_lambda = '02_tanh'
    then
      _v_debug := 
        sm_sc.fv_tanh_py(i_param_1)
      ;
    -- -- elsif i_lambda = '02_sech'
    -- -- then
    -- --   _v_debug := 
    -- --     sm_sc.fv_sech(i_param_1)
    -- --   ;
    -- -- elsif i_lambda = '02_csch'
    -- -- then
    -- --   _v_debug := 
    -- --     sm_sc.fv_csch(i_param_1)
    -- --   ;
    elsif i_lambda = '02_asinh'
    then
      _v_debug := 
        sm_sc.fv_asinh(i_param_1)
      ;
    elsif i_lambda = '02_acosh'
    then
      _v_debug := 
        sm_sc.fv_acosh(i_param_1)
      ;
    elsif i_lambda = '02_atanh'
    then
      _v_debug := 
        sm_sc.fv_atanh(i_param_1)
      ;
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda = '01_add'
    then 
      _v_debug := 
        sm_sc.fv_opr_add_py(i_param_1, i_param_2)
      ;
    elsif i_lambda = '01_mul'
    then
      _v_debug := 
        sm_sc.fv_opr_mul_py(i_param_1, i_param_2)
      ;
    elsif i_lambda = '01_sub'
    then
      _v_debug := 
        sm_sc.fv_opr_sub_py(i_param_1, i_param_2)
      ;
    elsif i_lambda = '01_0sub'
    then
      _v_debug := 
        sm_sc.fv_opr_sub_py(i_param_1)
      ;
    elsif i_lambda = '01_div'
    then
      _v_debug := 
        sm_sc.fv_opr_div_py(i_param_1, i_param_2)
      ;
    elsif i_lambda = '01_1div'
    then
      _v_debug := 
        sm_sc.fv_opr_div_py(i_param_1)
      ;
    elsif i_lambda = '01_pow'
    then
      _v_debug := 
        sm_sc.fv_opr_pow_py(i_param_1, i_param_2)
      ;
    elsif i_lambda = '01_log'
    then
      _v_debug := 
        sm_sc.fv_opr_log_py(i_param_1, i_param_2)
      ;
    elsif i_lambda = '01_exp'
    then
      _v_debug := 
        sm_sc.fv_opr_exp_py(i_param_1)
      ;
    elsif i_lambda = '01_ln'
    then
      _v_debug := 
        sm_sc.fv_opr_ln_py(i_param_1)
      ;
    elsif i_lambda = '01_abs'
    then
      _v_debug := 
        sm_sc.fv_opr_abs(i_param_1)
      ;
    elsif i_lambda = '01_prod_mx'
    then
      _v_debug := 
        -- sm_sc.fv_opr_prod_mx_py(i_param_1, i_param_2)
        i_param_1 |**| i_param_2
      ;
    elsif i_lambda = '01_chunk_prod_mx'
    then
      _v_debug := 
        sm_sc.fv_chunk_prod_mx(i_param_1, i_param_2,  i_input_arr_asso[1 : 3] :: int[])
      ;
    end if;
    
  elsif i_lambda like '00_%'
  then 
  
    if i_lambda = '00_none'
    then
      _v_debug := 
        sm_sc.fv_nn_none(i_param_1)
      ;

    -- -- elsif i_lambda = '00_const'
    -- -- then 
    -- --   return 
    -- --     -- (select node_depdt_vals from sm_sc.tb_nn_node where node_no = i_node_no and work_no = i_work_no)
    -- --   ;

    -- -- -- 出于减少数据拷贝、内存交换、资源开销的优化目的，nn input 节点的数据准备不在 lambda 执行。
    -- -- elsif i_lambda = '00_full_dataset'
    -- -- then 
    -- --   return 
    -- --   (
    -- --     select 
    -- --       array_agg(i_indepdt order by ord_no)
    -- --     from sm_sc.tb_nn_train_input_buff 
    -- --     where work_no = (select work_no from sm_sc.tb_nn_node where node_no = i_node_no)
    -- --   );
    -- -- 
    -- -- elsif i_lambda = '00_buff_slice_rand_pick'
    -- -- then 
    -- --   return 
    -- --     -- sm_sc.ft_nn_buff_slice_rand_pick()
    -- --   ;
    
    end if;
    
  elsif i_lambda like '81_%'
  then 
    return 
      sm_sc.ufv_lambda_arr
      (
        i_node_no       
      , i_lambda        
      , i_param_1       
      , i_param_2       
      , i_input_arr_asso
      , i_param_3       
      )
    ;
    
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if _v_debug is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(_v_debug) is true
    then 
      raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
    elsif 'NaN' :: float = any(_v_debug)
    then 
      raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  return _v_debug;

-- -- debug 
-- raise notice 'lambda % end:: i_lambda: %; time: %;', i_node_no, i_lambda, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return _v_debug;


-- -- debug 
-- raise notice 'i_node_no: %; i_lambda: %; time: %;', i_node_no, i_lambda, v_begin_clock - clock_timestamp();

  exception when others then
    raise exception 
    ' fn: sm_sc.fv_lambda_arr
      i_node_no: %
      i_lambda: %
      len of i_param_1: %
      len of i_param_2: %
      i_input_arr_asso: %
      sqlerrm: %
    '
    , i_node_no
    , i_lambda
    , array_dims(i_param_1)
    , array_dims(i_param_2)
    , i_input_arr_asso
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_lambda_arr
--   (
--     100001
--     , '03_elu'
--     , array[[1.23, 3.34],[-7.2, -0.25]]
--     , null
--     , array[0.1]
--   );






-- 本函数以数组为入参，标量版本参考 sm_sc.fv_gradient_opr
-- 对于一些原函数,  本求导函数无法求取 ddepdt/dindepdt，参看 sm_sc.fv_lambda_arr_dloss_dindepdt 求取 dloss/dindepdt
-- 本函数求取因变量对自变量的导数，没有对自变量广播求逆。
-- drop function if exists sm_sc.fv_lambda_arr_ddepdt_dindepdt(bigint, varchar(64), float[], int, float[], float[], float[]);
create or replace function sm_sc.fv_lambda_arr_ddepdt_dindepdt
(
  i_node_no               bigint,
  i_lambda                varchar(64)              ,
  i_indepdt_var           float[]                  ,
  i_indepdt_var_loc       int       default 1      ,    -- 单目、双目为 x 参数实际位置，无目/常量y为0，并目/聚合为 n；该参数对 prod_mx, sub, div, pow, log 等运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
  i_co_value              float[]   default null   ,    -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
  i_input_arr_asso        float[]   default null   ,    
  i_depdt_var             float[]   default null   ,    -- 前向的因变量，非必要入参
  -- i_dloss_ddepdt          float[] default null   ,       -- 此入参传入 dloss/ddepdt, 用于 反向传播阶段 求取 dloss/dindepdt
  i_indepdt_var_len       int[2]    default null        -- 自变量的高宽规格，agg_concat_x, agg_concat_y, agg_sum, agg_avg, slice_x, slice_y, add, sub conv_add/conv_sub/conv_de_add/conv_de_sub 等算子可用到该参数优化入参，以避免传参 i_indepdt_var，从而降低堆区开销
)
returns float[][]
as
$$
declare 
  _v_debug           float[]    ;
  v_indepdt_var_len  int[]      := (select array_agg(array_length(i_indepdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(i_indepdt_var)) tb_a_ndim(a_ndim));
  v_depdt_var_len    int[]      := (select array_agg(array_length(i_depdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(i_depdt_var)) tb_a_ndim(a_ndim));
 
 -- -- debug 
 -- v_begin_clock    timestamp     := clock_timestamp();
begin
-- -- debug
-- raise notice '
-- sm_sc.fv_lambda_arr_ddepdt_dindepdt debug :
--   i_lambda                : %; 
--   i_indepdt_var           : %; 
--   i_indepdt_var_loc       : %; 
--   i_co_value              : %; 
--   i_input_arr_asso   : %; 
--   i_depdt_var           : %; 
-- ', $1, $2, $3, $4, $5, $6
-- ;

-- -- debug 
-- raise notice 'lambda_delta % begin:: i_lambda: %; len_input: %; len_co_val: %; indepdt_var_loc: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(i_indepdt_var, 1), array_length(i_indepdt_var, 2)], array[array_length(i_co_value, 1), array_length(i_co_value, 2)], i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 'NaN' :: float = any(i_indepdt_var)
      or 'NaN' :: float = any(i_co_value)
      or 'NaN' :: float = any(i_depdt_var)
    then 
      raise exception 'there is NaN value in i_indepdt_var, i_co_value or i_depdt_var!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
    
  if i_lambda like '06_%'
  then 
    if i_lambda = '06_aggr_mx_sum'
    then 
      return   -- _v_debug := 
        sm_sc.fv_d_mx_sum(v_depdt_var_len)
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '06_aggr_mx_prod'
    then
      return   -- _v_debug :=  
        sm_sc.fv_d_mx_prod(i_indepdt_var, i_depdt_var)
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '06_aggr_mx_avg'
    then
      return   -- _v_debug :=  
        sm_sc.fv_d_mx_avg(i_input_arr_asso[1] :: int, v_depdt_var_len)
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '06_aggr_mx_max'
    then
      return 
        sm_sc.fv_d_mx_max(i_indepdt_var, i_depdt_var)
      ;
    elsif i_lambda = '06_aggr_mx_min'
    then
      return 
        sm_sc.fv_d_mx_min(i_indepdt_var, i_depdt_var)
      ;

    end if;
    
  elsif i_lambda like '03_%'
  then 
    if i_lambda = '03_sigmoid'
    then 
      return 
        sm_sc.fv_d_activate_sigmoid(i_indepdt_var, i_depdt_var)
      ;
    elsif i_lambda = '03_relu'
    then
      return 
        sm_sc.fv_d_activate_relu(i_indepdt_var)
      ;
    elsif i_lambda = '03_leaky_relu'
    then
      return 
        sm_sc.fv_d_activate_leaky_relu(i_indepdt_var, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_elu'
    then
      return 
        sm_sc.fv_d_activate_elu(i_indepdt_var, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_selu'
    then
      return 
        sm_sc.fv_d_activate_selu(i_indepdt_var)
      ;
    elsif i_lambda = '03_gelu'
    then
      return 
        sm_sc.fv_d_activate_gelu(i_indepdt_var)
      ;
    elsif i_lambda = '03_swish'
    then
      return 
        sm_sc.fv_d_activate_swish(i_indepdt_var)
      ;
    elsif i_lambda = '03_softplus'
    then
      return 
        sm_sc.fv_d_activate_softplus(i_indepdt_var)
      ;
    elsif i_lambda = '03_boxcox'
    then
      return 
        sm_sc.fv_d_activate_boxcox(i_indepdt_var, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_zscore'
    then
      return 
        sm_sc.fv_d_redistr_zscore(i_depdt_var, i_indepdt_var, i_input_arr_asso[1 : 4] :: int[])
      ;
    end if;
    
  elsif i_lambda like '02_%'
  then 
    if i_lambda = '02_sin'
    then
      return 
        sm_sc.fv_d_sin(i_indepdt_var)
      ;
    elsif i_lambda = '02_cos'
    then
      return 
        sm_sc.fv_d_cos(i_indepdt_var)
      ;
    elsif i_lambda = '02_tan'
    then
      return 
        sm_sc.fv_d_tan(i_indepdt_var)
      ;
    elsif i_lambda = '02_cot'
    then
      return 
        sm_sc.fv_d_cot(i_indepdt_var)
      ;
    elsif i_lambda = '02_sec'
    then
      return 
        sm_sc.fv_d_sec(i_indepdt_var, i_depdt_var)
      ;
    elsif i_lambda = '02_csc'
    then
      return 
        sm_sc.fv_d_csc(i_indepdt_var, i_depdt_var)
      ;
    elsif i_lambda = '02_asin'
    then
      return 
        sm_sc.fv_d_asin(i_indepdt_var)
      ;
    elsif i_lambda = '02_acos'
    then
      return 
        sm_sc.fv_d_acos(i_indepdt_var)
      ;
    elsif i_lambda = '02_atan'
    then
      return 
        sm_sc.fv_d_atan(i_indepdt_var)
      ;
    elsif i_lambda = '02_acot'
    then
      return 
        sm_sc.fv_d_acot(i_indepdt_var)
      ;
    elsif i_lambda = '02_sinh'
    then
      return 
        sm_sc.fv_d_sinh(i_indepdt_var)
      ;
    elsif i_lambda = '02_cosh'
    then
      return 
        sm_sc.fv_d_cosh(i_indepdt_var)
      ;
    elsif i_lambda = '02_tanh'
    then
      return 
        sm_sc.fv_d_tanh(i_indepdt_var, i_depdt_var)
      ;
    -- -- elsif i_lambda = '02_sech'
    -- -- then
    -- --   return 
    -- --     sm_sc.fv_d_sech(i_indepdt_var)
    -- --   ;
    -- -- elsif i_lambda = '02_csch'
    -- -- then
    -- --   return 
    -- --     sm_sc.fv_d_csch(i_indepdt_var)
    -- --   ;
    elsif i_lambda = '02_asinh'
    then
      return 
        sm_sc.fv_d_asinh(i_indepdt_var)
      ;
    elsif i_lambda = '02_acosh'
    then
      return 
        sm_sc.fv_d_acosh(i_indepdt_var)
      ;
    elsif i_lambda = '02_atanh'
    then
      return 
        sm_sc.fv_d_atanh(i_indepdt_var)
      ;
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda = '01_add'
    then 
      return   -- _v_debug := 
        sm_sc.fv_d_add(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_mul'
    then
      return   -- _v_debug := 
        sm_sc.fv_d_mul(i_co_value)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_sub'
    then
      if i_indepdt_var_loc = 1
      then 
        return   -- _v_debug := 
          sm_sc.fv_d_sub_1(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return   -- _v_debug := 
          sm_sc.fv_d_sub_2(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_0sub'
    then 
      return   -- _v_debug := 
        sm_sc.fv_d_sub_2(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_div'
    then
      if i_indepdt_var_loc = 1
      then 
        return   -- _v_debug := 
          sm_sc.fv_d_div_1(i_co_value)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return   -- _v_debug := 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_div_2_un_de_broadcast(i_indepdt_var, i_co_value)  -- 保留 ddepdt_dindepdt 广播
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_1div'
    then 
      return   -- _v_debug := 
        sm_sc.fv_d_div_2_un_de_broadcast(i_indepdt_var)
      ;
      
    elsif i_lambda = '01_pow'
    then
      if i_indepdt_var_loc = 1
      then 
        return   -- _v_debug := 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_pow_1_un_de_broadcast(i_indepdt_var, i_co_value, i_depdt_var)  -- 保留 ddepdt_dindepdt 广播
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return   -- _v_debug := 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_pow_2_un_de_broadcast(i_indepdt_var, i_co_value, i_depdt_var)  -- 保留 ddepdt_dindepdt 广播
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_log'
    then
      if i_indepdt_var_loc = 1
      then 
        return   -- _v_debug := 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_log_1_un_de_broadcast(i_indepdt_var, i_co_value, i_depdt_var)  -- 保留 ddepdt_dindepdt 广播
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return   -- _v_debug := 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_log_2_un_de_broadcast(i_indepdt_var, i_co_value)  -- 保留 ddepdt_dindepdt 广播
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(i_indepdt_var) < cardinality(i_co_value)
      -- --   or cardinality(i_indepdt_var) < cardinality(i_depdt_var)
      -- -- then 
      -- --   return sm_sc.fv_aggr_chunk_sum(_v_debug, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_exp'
    then
      return 
        sm_sc.fv_d_exp(i_indepdt_var, i_depdt_var)
      ;
    elsif i_lambda = '01_ln'
    then
      return 
        sm_sc.fv_d_ln(i_indepdt_var)
      ;
      
    elsif i_lambda = '01_abs'
    then 
      return
        sm_sc.fv_d_abs(i_indepdt_var)
      ;

    end if;
    
  elsif i_lambda like '81_%'
  then 
    return 
      sm_sc.ufv_lambda_arr_ddepdt_dindepdt
      (
        i_node_no        
      , i_lambda         
      , i_indepdt_var    
      , i_indepdt_var_loc
      , i_co_value       
      , i_input_arr_asso 
      , i_depdt_var      
      , i_indepdt_var_len
      )
    ;
    
  end if;  

  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then 
  -- --   if _v_debug is null
  -- --     -- or sm_sc.fv_aggr_slice_is_exists_null(_v_debug) is true
  -- --   then 
  -- --     raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
  -- --   elsif 'NaN' :: float = any(_v_debug)
  -- --   then 
  -- --     raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
  -- --   end if;
  -- -- end if;
  -- -- 
  -- -- return _v_debug;

-- -- debug 
-- raise notice 'lambda_delta % end:: i_lambda: %; indepdt_var_loc: %; time: %;', i_node_no, i_lambda, i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return _v_debug;

-- -- debug 
-- raise notice 'dddi:i_back_node_no: %; i_fore_node_no: %; loc: %; i_lambda: %; time: %;', i_back_node_no, i_fore_node_no, i_indepdt_var_loc, i_lambda, v_begin_clock - clock_timestamp();

exception when others then
  raise exception 
  ' fn: sm_sc.fv_lambda_arr_ddepdt_dindepdt
    i_node_no: %;
    i_lambda: %;
    len of i_indepdt_var: %;
    i_indepdt_var_loc: %;
    len of i_co_value: %;
    i_input_arr_asso: %;
    len of i_depdt_var: %;
    v_indepdt_var_len: %;
    sqlerrm: %;
  '
  , i_node_no
  , i_lambda             
  , array_dims(i_indepdt_var)          
  , i_indepdt_var_loc      
  , array_dims(i_co_value)
  , i_input_arr_asso
  , array_dims(i_depdt_var)
  , v_indepdt_var_len      
  , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;







-- 本函数以数组为入参，标量版本参考 sm_sc.fv_gradient_opr
-- 对于一些原函数,  本求导函数无法求取 ddepdt/dindepdt，参看 sm_sc.fv_lambda_arr_dloss_dindepdt 求取 dloss/dindepdt
-- 本函数求取因变量对自变量的导数，没有对自变量广播求逆。
-- drop function if exists sm_sc.fv_lambda_arr_ddepdt_dindepdt_p(char(6), bigint, bigint, bigint, varchar(64), varchar(64), int, varchar(64), float[], varchar(64), int[2]);
create or replace function sm_sc.fv_lambda_arr_ddepdt_dindepdt_p
(
  i_model_code_6          char(6)
, i_work_no               bigint
, i_fore_node_no          bigint
, i_back_node_no          bigint
, i_lambda                varchar(64)                 
, i_indepdt_var_p         varchar(64)                 
, i_indepdt_var_loc       int           default 1         -- 单目、双目为 x 参数实际位置，无目/常量y为0，并目/聚合为 n；该参数对 prod_mx, sub, div, pow, log 等运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
, i_co_value_p            varchar(64)   default null      -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
, i_input_arr_asso        float[]       default null      
, i_depdt_var_p           varchar(64)   default null      -- 前向的因变量，非必要入参
, i_indepdt_var_len       int[2]        default null      -- 自变量的高宽规格，agg_concat_x, agg_concat_y, agg_sum, agg_avg, slice_x, slice_y, add, sub conv_add/conv_sub/conv_de_add/conv_de_sub 等算子可用到该参数优化入参，以避免传参 v_indepdt_var，从而降低堆区开销
)
returns varchar(64)
as
$$
declare 
  v_indepdt_var      float[]    := sm_sc.__fv_get_kv(i_indepdt_var_p);
  v_co_value         float[]    := sm_sc.__fv_get_kv(i_co_value_p);
  v_depdt_var        float[]    := sm_sc.__fv_get_kv(i_depdt_var_p);
  v_indepdt_var_len  int[]      := (select array_agg(array_length(v_indepdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(v_indepdt_var)) tb_a_ndim(a_ndim));
  v_depdt_var_len    int[]      := (select array_agg(array_length(v_depdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(v_depdt_var)) tb_a_ndim(a_ndim));
  v_ret              float[]    ;
 
 -- debug 
 v_begin_clock    timestamp     := clock_timestamp();
begin
-- -- debug
-- raise notice '
-- sm_sc.fv_lambda_arr_ddepdt_dindepdt_p debug :
--   i_lambda                : %; 
--   v_indepdt_var           : %; 
--   i_indepdt_var_loc       : %; 
--   v_co_value              : %; 
--   i_input_arr_asso   : %; 
--   v_depdt_var           : %; 
-- ', $1, $2, $3, $4, $5, $6
-- ;

-- -- debug 
-- raise notice 'lambda_delta % begin:: i_lambda: %; len_input: %; len_co_val: %; indepdt_var_loc: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(v_indepdt_var, 1), array_length(v_indepdt_var, 2)], array[array_length(v_co_value, 1), array_length(v_co_value, 2)], i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 'NaN' :: float = any(v_indepdt_var)
      or 'NaN' :: float = any(v_co_value)
      or 'NaN' :: float = any(v_depdt_var)
    then 
      raise exception 'there is NaN value in v_indepdt_var, v_co_value or v_depdt_var!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
    
  if i_lambda like '06_%'
  then 
    if i_lambda = '06_aggr_mx_sum'
    then 
      v_ret :=   -- return 
        sm_sc.fv_d_mx_sum(v_depdt_var_len)
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '06_aggr_mx_prod'
    then
      v_ret :=   -- return  
        sm_sc.fv_d_mx_prod(v_indepdt_var, v_depdt_var)
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '06_aggr_mx_avg'
    then
      v_ret :=   -- return  
        sm_sc.fv_d_mx_avg(i_input_arr_asso[1] :: int, v_depdt_var_len)
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '06_aggr_mx_max'
    then
      v_ret := 
        sm_sc.fv_d_mx_max(v_indepdt_var, v_depdt_var)
      ;
    elsif i_lambda = '06_aggr_mx_min'
    then
      v_ret := 
        sm_sc.fv_d_mx_min(v_indepdt_var, v_depdt_var)
      ;

    end if;
    
  elsif i_lambda like '03_%'
  then 
    if i_lambda = '03_sigmoid'
    then 
      v_ret := 
        sm_sc.fv_d_activate_sigmoid(v_indepdt_var, v_depdt_var)
      ;
    elsif i_lambda = '03_relu'
    then
      v_ret := 
        sm_sc.fv_d_activate_relu(v_indepdt_var)
      ;
    elsif i_lambda = '03_leaky_relu'
    then
      v_ret := 
        sm_sc.fv_d_activate_leaky_relu(v_indepdt_var, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_elu'
    then
      v_ret := 
        sm_sc.fv_d_activate_elu(v_indepdt_var, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_selu'
    then
      v_ret := 
        sm_sc.fv_d_activate_selu(v_indepdt_var)
      ;
    elsif i_lambda = '03_gelu'
    then
      v_ret := 
        sm_sc.fv_d_activate_gelu(v_indepdt_var)
      ;
    elsif i_lambda = '03_swish'
    then
      v_ret := 
        sm_sc.fv_d_activate_swish(v_indepdt_var)
      ;
    elsif i_lambda = '03_softplus'
    then
      v_ret := 
        sm_sc.fv_d_activate_softplus(v_indepdt_var)
      ;
    elsif i_lambda = '03_boxcox'
    then
      v_ret := 
        sm_sc.fv_d_activate_boxcox(v_indepdt_var, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_zscore'
    then
      v_ret := 
        sm_sc.fv_d_redistr_zscore(v_depdt_var, v_indepdt_var, i_input_arr_asso[1 : 4] :: int[])
      ;
    end if;
    
  elsif i_lambda like '02_%'
  then 
    if i_lambda = '02_sin'
    then
      v_ret := 
        sm_sc.fv_d_sin(v_indepdt_var)
      ;
    elsif i_lambda = '02_cos'
    then
      v_ret := 
        sm_sc.fv_d_cos(v_indepdt_var)
      ;
    elsif i_lambda = '02_tan'
    then
      v_ret := 
        sm_sc.fv_d_tan(v_indepdt_var)
      ;
    elsif i_lambda = '02_cot'
    then
      v_ret := 
        sm_sc.fv_d_cot(v_indepdt_var)
      ;
    elsif i_lambda = '02_sec'
    then
      v_ret := 
        sm_sc.fv_d_sec(v_indepdt_var, v_depdt_var)
      ;
    elsif i_lambda = '02_csc'
    then
      v_ret := 
        sm_sc.fv_d_csc(v_indepdt_var, v_depdt_var)
      ;
    elsif i_lambda = '02_asin'
    then
      v_ret := 
        sm_sc.fv_d_asin(v_indepdt_var)
      ;
    elsif i_lambda = '02_acos'
    then
      v_ret := 
        sm_sc.fv_d_acos(v_indepdt_var)
      ;
    elsif i_lambda = '02_atan'
    then
      v_ret := 
        sm_sc.fv_d_atan(v_indepdt_var)
      ;
    elsif i_lambda = '02_acot'
    then
      v_ret := 
        sm_sc.fv_d_acot(v_indepdt_var)
      ;
    elsif i_lambda = '02_sinh'
    then
      v_ret := 
        sm_sc.fv_d_sinh(v_indepdt_var)
      ;
    elsif i_lambda = '02_cosh'
    then
      v_ret := 
        sm_sc.fv_d_cosh(v_indepdt_var)
      ;
    elsif i_lambda = '02_tanh'
    then
      v_ret := 
        sm_sc.fv_d_tanh(v_indepdt_var, v_depdt_var)
      ;
    -- -- elsif i_lambda = '02_sech'
    -- -- then
    -- --   v_ret := 
    -- --     sm_sc.fv_d_sech(v_indepdt_var)
    -- --   ;
    -- -- elsif i_lambda = '02_csch'
    -- -- then
    -- --   v_ret := 
    -- --     sm_sc.fv_d_csch(v_indepdt_var)
    -- --   ;
    elsif i_lambda = '02_asinh'
    then
      v_ret := 
        sm_sc.fv_d_asinh(v_indepdt_var)
      ;
    elsif i_lambda = '02_acosh'
    then
      v_ret := 
        sm_sc.fv_d_acosh(v_indepdt_var)
      ;
    elsif i_lambda = '02_atanh'
    then
      v_ret := 
        sm_sc.fv_d_atanh(v_indepdt_var)
      ;
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda = '01_add'
    then 
      v_ret :=   -- return 
        sm_sc.fv_d_add(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, i_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_mul'
    then
      v_ret :=   -- return 
        sm_sc.fv_d_mul(v_co_value)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, v_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_sub'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret :=   -- return 
          sm_sc.fv_d_sub_1(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret :=   -- return 
          sm_sc.fv_d_sub_2(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, i_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_0sub'
    then 
      v_ret :=   -- return 
        sm_sc.fv_d_sub_2(v_depdt_var_len)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
      ;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, i_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_div'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret :=   -- return 
          sm_sc.fv_d_div_1(v_co_value)  -- 保留 ddepdt_dindepdt 广播，不传 i_indepdt_var_len
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret :=   -- return 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_div_2_un_de_broadcast(v_indepdt_var, v_co_value)  -- 保留 ddepdt_dindepdt 广播
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, i_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_1div'
    then 
      v_ret :=   -- return 
        sm_sc.fv_d_div_2_un_de_broadcast(v_indepdt_var)
      ;
      
    elsif i_lambda = '01_pow'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret :=   -- return 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_pow_1_un_de_broadcast(v_indepdt_var, v_co_value, v_depdt_var)  -- 保留 ddepdt_dindepdt 广播
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret :=   -- return 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_pow_2_un_de_broadcast(v_indepdt_var, v_co_value, v_depdt_var)  -- 保留 ddepdt_dindepdt 广播
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, i_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_log'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret :=   -- return 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_log_1_un_de_broadcast(v_indepdt_var, v_co_value, v_depdt_var)  -- 保留 ddepdt_dindepdt 广播
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret :=   -- return 
          -- -- 不采用 sm_sc.fv_d_div_2，由 dloss_dindepdt 环节统一处理广播逆运算
          sm_sc.fv_d_log_2_un_de_broadcast(v_indepdt_var, v_co_value)  -- 保留 ddepdt_dindepdt 广播
        ;
      end if;
      -- -- -- 广播情况的导数倍数处理
      -- -- if cardinality(v_indepdt_var) < cardinality(v_co_value)
      -- --   or cardinality(v_indepdt_var) < cardinality(v_depdt_var)
      -- -- then 
      -- --   v_ret := sm_sc.fv_aggr_chunk_sum(v_ret, i_indepdt_var_len);
      -- -- end if;
      
    elsif i_lambda = '01_exp'
    then
      v_ret := 
        sm_sc.fv_d_exp(v_indepdt_var, v_depdt_var)
      ;
    elsif i_lambda = '01_ln'
    then
      v_ret := 
        sm_sc.fv_d_ln(v_indepdt_var)
      ;
      
    elsif i_lambda = '01_abs'
    then 
      v_ret :=
        sm_sc.fv_d_abs(v_indepdt_var)
      ;

    end if;
    
  elsif i_lambda like '81_%'
  then 
    v_ret := 
      sm_sc.ufv_lambda_arr_ddepdt_dindepdt
      (
        i_node_no        
      , i_lambda         
      , v_indepdt_var    
      , i_indepdt_var_loc
      , v_co_value       
      , i_input_arr_asso 
      , v_depdt_var      
      , i_indepdt_var_len
      )
    ;
    
  end if;  

  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then 
  -- --   if v_ret is null
  -- --     -- or sm_sc.fv_aggr_slice_is_exists_null(v_ret) is true
  -- --   then 
  -- --     raise exception 'there is null value in v_ret :=ing!  i_node_no: %.  ', i_node_no;
  -- --   elsif 'NaN' :: float = any(v_ret)
  -- --   then 
  -- --     raise exception 'there is NaN value in v_ret :=ing!  i_node_no: %.  ', i_node_no;
  -- --   end if;
  -- -- end if;
  -- -- 
  -- -- return v_ret;

-- -- debug 
-- raise notice 'lambda_delta % end:: i_lambda: %; indepdt_var_loc: %; time: %;', i_node_no, i_lambda, i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

-- -- debug 
-- raise notice 'dddi:i_back_node_no: %; i_fore_node_no: %; loc: %; i_lambda: %; time: %;', i_back_node_no, i_fore_node_no, i_indepdt_var_loc, i_lambda, v_begin_clock - clock_timestamp();

  return 
    sm_sc.__fv_set_kv
    (
      v_ret
    , i_model_code_6 || 
      '_' || i_work_no :: varchar   ||
      '_' || i_fore_node_no :: varchar ||
      '_' || i_back_node_no :: varchar ||
      '_dddi' || -- 'd_depdt_d_indepdt'
      '__' || i_indepdt_var_loc :: varchar
    )
  ;

exception when others then
  raise exception 
  ' fn: sm_sc.fv_lambda_arr_ddepdt_dindepdt_p
    i_node_no: %;
    i_lambda: %;
    len of v_indepdt_var: %;
    i_indepdt_var_loc: %;
    len of v_co_value: %;
    i_input_arr_asso: %;
    len of v_depdt_var: %;
    i_indepdt_var_len: %;
    sqlerrm: %;
  '
  , i_node_no
  , i_lambda             
  , array_dims(v_indepdt_var)          
  , i_indepdt_var_loc      
  , array_dims(v_co_value)
  , i_input_arr_asso
  , array_dims(v_depdt_var)
  , i_indepdt_var_len      
  , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;







-- 本函数以数组为入参，标量版本参考 sm_sc.fv_gradient_opr
-- 对于 conv_2d, pool_max, pool_avg, softmax, zscore 本函数求取 dloss/dindepdt, 而不是求取 ddepdt/dindepdt
-- drop function if exists sm_sc.fv_lambda_arr_dloss_dindepdt(bigint, varchar(64), float[], int, float[], float[], float[], int, float[]);
create or replace function sm_sc.fv_lambda_arr_dloss_dindepdt
(
  i_node_no               bigint,
  i_lambda                varchar(64)                       ,
  i_indepdt_var           float[]                  ,
  i_indepdt_var_loc       int     default 1        ,    -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 prod_mx, sub, div, pow, log, softmax 等运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
  i_co_value              float[] default null     ,    -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
  i_input_arr_asso        float[] default null     ,    
  i_depdt_var             float[] default null     ,    -- 前向的因变量
  i_dloss_ddepdt          float[] default null     ,    -- 此入参传入 dloss/ddepdt, 用于 反向传播阶段 求取 dloss/dindepdt
  i_indepdt_var_len       int[]   default null     ,    -- 自变量的高宽规格，agg_concat_x, agg_concat_y, agg_sum, agg_avg, slice_x, slice_y, add, sub 会用到改参数，以避免传参 i_indepdt_var，从而降低堆区开销
  i_ddepdt_dindepdt       float[] default null          -- 因变量对自变量导数
)
returns float[][]
as
$$
declare 
  -- -- _v_debug           float[][];
  v_indepdt_var_len  int[]      := coalesce(i_indepdt_var_len, (select array_agg(array_length(i_indepdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(i_indepdt_var)) tb_a_ndim(a_ndim)));
 
 -- -- debug 
 -- v_begin_clock    timestamp     := clock_timestamp();
begin
-- -- debug
-- raise notice '
-- sm_sc.fv_lambda_arr_dloss_dindepdt debug :
--   i_lambda                : %; 
--   i_indepdt_var             : %; 
--   i_indepdt_var_loc         : %; 
--   i_co_value              : %; 
--   i_input_arr_asso   : %; 
--   i_depdt_var           : %; 
-- ', $1, $2, $3, $4, $5, $6
-- ;

  -- 动态 sql 会影响性能，所以还是用 case 判断
  
-- -- debug 
-- raise notice 'lambda_dloss_dindepdt % begin:: i_lambda: %; len_dloss_dindepdt: %; indepdt_var_loc: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2)], i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 'NaN' :: float = any(i_indepdt_var)
      or 'NaN' :: float = any(i_co_value)
      or 'NaN' :: float = any(i_depdt_var)
      or 'NaN' :: float = any(i_dloss_ddepdt)
    then 
      raise exception 'there is NaN value in i_indepdt_var, i_co_value, i_depdt_var or i_dloss_ddepdt!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  if i_lambda like '07_%'
  then 
    if i_lambda = '07_aggr_slice_sum'
    then 
      return 
        sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt(v_indepdt_var_len, i_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_prod'
    then
      return 
        sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt(i_indepdt_var, i_depdt_var, i_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_avg'
    then
      return 
        sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_max'
    then
      return 
        sm_sc.fv_d_aggr_slice_max_dloss_dindepdt(i_indepdt_var, i_depdt_var, i_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_min'
    then
      return 
        sm_sc.fv_d_aggr_slice_min_dloss_dindepdt(i_indepdt_var, i_depdt_var, i_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_sum'
    then 
      return 
        sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt(i_input_arr_asso[1 : 4] :: int[], i_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_prod'
    then
      return 
        sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt(i_indepdt_var, i_depdt_var, i_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_avg'
    then
      return 
        sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt(i_input_arr_asso[1 : 4] :: int[], i_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_max'
    then
      return 
        sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt(i_indepdt_var, i_depdt_var, i_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_min'
    then
      return 
        sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt(i_indepdt_var, i_depdt_var, i_dloss_ddepdt)
      ;
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '06_%'
  then 
    if i_lambda = '06_aggr_mx_concat_y'
    then
      return 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n
          (
            i_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            v_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    elsif i_lambda = '06_aggr_mx_concat_x'
    then
      return 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n
          (
            i_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            v_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    elsif i_lambda = '06_aggr_mx_concat_x3'
    then
      return 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n
          (
            i_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            v_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    elsif i_lambda = '06_aggr_mx_concat_x4'
    then
      return 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_x4_dloss_dindepdt_n
          (
            i_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            v_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;

  elsif i_lambda like '05_%'
  then 
    if i_lambda = '05_pool_max_2d_grp_x'
    then 
      return 
        sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt
        (
          i_indepdt_var                              ,                                       -- i_array_grp_x
          i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_dloss_ddepdt                         ,
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          i_depdt_var                            ,                                       -- i_depdt_1d_grp
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
          coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg_2d_grp_x'
    then
      return 
        sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt
        (
          -- -- i_input_arr_asso[11] :: int         ,                                       -- 规约：存放 i_array_grp_x_len
          i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_dloss_ddepdt                         ,
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
        )
      ;
    elsif i_lambda = '05_pool_max'
    then
      return 
        sm_sc.fv_d_pool_max_dloss_dindepdt
        (
          i_indepdt_var                              ,                                       -- i_array
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          i_dloss_ddepdt                         ,
          i_depdt_var                            ,                                       -- i_depdt_1d_grp
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
          coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg'
    then
      return 
        sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
        (
          -- -- v_indepdt_var_len                 ,                                         -- 规约：自变量规格
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          i_dloss_ddepdt                         ,
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
        )
      ;
    elsif i_lambda = '05_pool_none'
    then
      return 
        sm_sc.fv_d_pool_none_dloss_dindepdt
        (
          v_indepdt_var_len                 ,                                         -- 规约：自变量规格
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          i_dloss_ddepdt                         ,
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
        )
      ;
    elsif i_lambda = '05_conv_2d_grp_x'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1
          (
            i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
            i_dloss_ddepdt                      ,
            i_co_value                          ,                                       -- i_window
            i_input_arr_asso[3] :: int          ,                                       -- 规约：存放 i_window_len_x
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2
          (
            i_co_value                               ,                                  -- i_array_grp_x
            i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
            i_dloss_ddepdt                         ,
            i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
            i_input_arr_asso[12] :: int :: boolean                                  ,   -- 规约：存放 i_window_bias_label
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_2d'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
          (
            i_dloss_ddepdt,
            i_co_value,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
          (
            i_co_value,
            i_dloss_ddepdt,
            i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 3
      then 
        return 
          sm_sc.fv_d_conv_2d_dloss_dindepdt_3(i_dloss_ddepdt, v_indepdt_var_len)
        ;      
      end if;
    elsif i_lambda = '05_conv_add'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_add_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_add_dloss_dindepdt_2
          (
        --  i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_sub'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_sub_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_sub_dloss_dindepdt_2
          (
        --  i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_mul'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_mul_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_mul_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_div'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_div_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_div_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_indepdt_var,
        --  v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_pow'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_pow_dloss_dindepdt_1
          (
            i_indepdt_var,
        --  v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_depdt_var,
            array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_pow_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_indepdt_var,
            v_indepdt_var_len,
            i_depdt_var,
            array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_log'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_log_dloss_dindepdt_1
          (
            i_indepdt_var,
            v_indepdt_var_len,
            i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_log_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_indepdt_var,
        --  v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
            i_input_arr_asso[2] :: int,                                                 -- 规约：存放 i_window_len_heigh
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_input_arr_asso[2] :: int,                                                 -- 规约：存放 i_window_len_heigh
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_sub'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
          (
        --  i_co_value,
            array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_indepdt_var,
            v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_div'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_indepdt_var,
        --  v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_pow'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
          (
            i_indepdt_var,
        --  v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_depdt_var,
            array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_indepdt_var,
            v_indepdt_var_len,
            i_depdt_var,
            array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_log'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
          (
            i_indepdt_var,
        --  v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_indepdt_var,
        --  v_indepdt_var_len,
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
          (
        --  i_indepdt_var,
            v_indepdt_var_len,
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
            i_input_arr_asso[3] :: int,                                                 -- 规约：存放 i_window_len_width
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
          (
            i_co_value,
        --  array[array_length(i_co_value, 1), array_length(i_co_value, 2)],
        --  i_indepdt_var,
            v_indepdt_var_len,
            i_input_arr_asso[3] :: int,                                                 -- 规约：存放 i_window_len_width
        --  i_depdt_var,
        --  array[array_length(i_depdt_var, 1), array_length(i_depdt_var, 2)],
            i_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;
  
  elsif i_lambda like '04_%'
  then 
    if i_lambda = '04_new'
    then 
      return 
        sm_sc.fv_d_new_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '04_repeat_axis'
    then 
      return 
        sm_sc.fv_d_repeat_axis_dloss_dindepdt(i_dloss_ddepdt, sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][ : ]) :: int[], sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][ : ]) :: int[])
      ;
    elsif i_lambda = '04_apad'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_apad_dloss_dindepdt_1(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1) - 1])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_apad_dloss_dindepdt_2(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1) - 1], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_bpad'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_bpad_dloss_dindepdt_1(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1) - 1])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_bpad_dloss_dindepdt_2(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1) - 1], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_lpad'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_lpad_dloss_dindepdt_1(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1)])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_lpad_dloss_dindepdt_2(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1)], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_rpad'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_rpad_dloss_dindepdt_1(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1)])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_rpad_dloss_dindepdt_2(i_dloss_ddepdt, v_indepdt_var_len[array_length(v_indepdt_var_len, 1)], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_transpose'
    then
      return 
        sm_sc.fv_d_transpose_dloss_dindepdt(i_dloss_ddepdt)
      ;
    elsif i_lambda = '04_transpose_i'
    then
      return 
        sm_sc.fv_d_transpose_i_dloss_dindepdt(i_dloss_ddepdt)
      ;
    elsif i_lambda = '04_chunk_transpose'
    then
      return 
        sm_sc.fv_d_chunk_transpose_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_transpose_nd'
    then
      return 
        sm_sc.fv_d_transpose_nd_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_90'
    then
      return 
        sm_sc.fv_d_turn_90_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_180'
    then
      return 
        sm_sc.fv_d_turn_180_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_mirror'
    then
      return 
        sm_sc.fv_d_mirror_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_ele_3d_2_2d'
    then
      return 
        sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        , v_indepdt_var_len
        )
      ;
    elsif i_lambda = '04_mx_ele_2d_2_3d'
    then
      return 
        sm_sc.fv_d_mx_ele_2d_2_3d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int
        , i_input_arr_asso[4] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_4d_2_3d'
    then
      return 
        sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        , v_indepdt_var_len
        )
      ;
    elsif i_lambda = '04_mx_ele_3d_2_4d'
    then
      return 
        sm_sc.fv_d_mx_ele_3d_2_4d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_flatten_2dims'
    then
      return 
        sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt
        (
          i_dloss_ddepdt
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        , v_indepdt_var_len
        )
      ;
    elsif i_lambda = '04_mx_slice_3d_2_2d'
    then
      return 
        sm_sc.fv_d_mx_slice_3d_2_2d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , v_indepdt_var_len
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        )
      ;
    elsif i_lambda = '04_mx_slice_4d_2_2d'
    then
      return 
        sm_sc.fv_d_mx_slice_4d_2_2d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , v_indepdt_var_len
        , sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][ : ] :: int[])
        , sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][ : ] :: int[])
        )
      ;
    elsif i_lambda = '04_mx_slice_4d_2_3d'
    then
      return 
        sm_sc.fv_d_mx_slice_4d_2_3d_dloss_dindepdt
        (
          i_dloss_ddepdt
        , v_indepdt_var_len
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        )
      ;
    elsif i_lambda = '04_mx_ascend_dim'
    then
      return 
        sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_descend_dim'
    then
      return 
        sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(i_dloss_ddepdt, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_chunk'
    then
      return 
        sm_sc.fv_d_chunk_dloss_dindepdt(i_dloss_ddepdt, v_indepdt_var_len, sm_sc.fv_mx_descend_dim(i_input_arr_asso[1][ : ]) :: int[])
      ;
    elsif i_lambda = '04_slice_y'
    then
      return 
      (
        select 
          sm_sc.fv_d_slice_y_dloss_dindepdt
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[1]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x'
    then
      return 
      (
        select 
          sm_sc.fv_d_slice_x_dloss_dindepdt
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[2]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x3'
    then
      return 
      (
        select 
          sm_sc.fv_d_slice_x3_dloss_dindepdt
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[3]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x4'
    then
      return 
      (
        select 
          sm_sc.fv_d_slice_x4_dloss_dindepdt
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[4]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_y'
    then
      return 
      (
        select 
          sm_sc.fv_d_sample_y_dloss_dindepdt_1
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[1]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x'
    then
      return 
      (
        select 
          sm_sc.fv_d_sample_x_dloss_dindepdt_1
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[2]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x3'
    then
      return 
      (
        select 
          sm_sc.fv_d_sample_x3_dloss_dindepdt_1
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[3]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x4'
    then
      return 
      (
        select 
          sm_sc.fv_d_sample_x4_dloss_dindepdt_1
          (
            i_dloss_ddepdt
          , v_indepdt_var_len[4]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_lower_tri_mx'
    then
      return sm_sc.fv_d_lower_tri_mx_dloss_dindepdt(i_dloss_ddepdt);
    elsif i_lambda = '04_upper_tri_mx'
    then
      return sm_sc.fv_d_upper_tri_mx_dloss_dindepdt(i_dloss_ddepdt);
    elsif i_lambda = '04_lower_tri_mx_ex'
    then
      return sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt(i_dloss_ddepdt);
    elsif i_lambda = '04_upper_tri_mx_ex'
    then
      return sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt(i_dloss_ddepdt);
    elsif i_lambda = '04_lmask'
    then
      return sm_sc.fv_d_lmask_dloss_dindepdt_1(i_dloss_ddepdt, i_co_value :: int[]);
    elsif i_lambda = '04_rmask'
    then
      return sm_sc.fv_d_rmask_dloss_dindepdt_1(i_dloss_ddepdt, i_co_value :: int[]);
    elsif i_lambda = '04_amask'
    then
      return sm_sc.fv_d_amask_dloss_dindepdt_1(i_dloss_ddepdt, i_co_value :: int[]);
    elsif i_lambda = '04_bmask'
    then
      return sm_sc.fv_d_bmask_dloss_dindepdt_1(i_dloss_ddepdt, i_co_value :: int[]);
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '03_%'
  then 
    if i_lambda = '03_softmax'
    then
      return 
        sm_sc.fv_d_redistr_softmax_dloss_dindepdt(i_depdt_var, i_dloss_ddepdt, i_indepdt_var, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '03_softmax_ex'
    then
      return 
        sm_sc.fv_d_redistr_softmax_dloss_dindepdt(i_depdt_var, i_dloss_ddepdt, i_indepdt_var, i_input_arr_asso[1 : 4] :: int[])
      ;
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda = '01_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_prod_mx_dloss_dindepdt_1(i_dloss_ddepdt, i_co_value, v_indepdt_var_len)
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_prod_mx_dloss_dindepdt_2(i_dloss_ddepdt, i_co_value, v_indepdt_var_len)
        ;
      end if;
    elsif i_lambda = '01_chunk_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        return 
          sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1(i_dloss_ddepdt, i_co_value, v_indepdt_var_len, i_input_arr_asso[1 : 3] :: int[])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        return 
          sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2(i_dloss_ddepdt, i_co_value, v_indepdt_var_len, i_input_arr_asso[1 : 3] :: int[])
        ;
      end if;
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '00_%'
  then 
    if i_lambda = '00_none'
    then
      return 
        sm_sc.fv_d_nn_none_dloss_dindepdt(i_dloss_ddepdt)
      ;
    else 
      return i_dloss_ddepdt *` i_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '81_%'
  then 
    return 
      sm_sc.ufv_lambda_arr_dloss_dindepdt
      (
        i_node_no        
      , i_lambda         
      , i_indepdt_var    
      , i_indepdt_var_loc
      , i_co_value       
      , i_input_arr_asso 
      , i_depdt_var      
      , i_dloss_ddepdt   
      , v_indepdt_var_len
      , i_ddepdt_dindepdt
      )
    ;
    
  else 
    return i_dloss_ddepdt *` i_ddepdt_dindepdt;
  end if;  

  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then 
  -- --   if _v_debug is null
  -- --     -- or sm_sc.fv_aggr_slice_is_exists_null(_v_debug) is true
  -- --   then 
  -- --     raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
  -- --   elsif 'NaN' :: float = any(_v_debug)
  -- --   then 
  -- --     raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
  -- --   end if;
  -- -- end if;
  -- -- 
  -- -- return _v_debug;
  
-- -- debug 
-- raise notice 'lambda_dloss_dindepdt % end:: i_lambda: %; indepdt_var_loc: %; time:%;', i_node_no, i_lambda, i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return _v_debug;

-- -- debug 
-- raise notice 'dldi: i_back_node_no: %; i_fore_node_no: %; loc: %; i_lambda: %; time: %;', i_back_node_no, i_fore_node_no, i_indepdt_var_loc, i_lambda, v_begin_clock - clock_timestamp();

  exception when others then
    raise exception 
    ' i_node_no: %
      fn: sm_sc.fv_lambda_arr_dloss_dindepdt
      i_lambda: %
      len of i_indepdt_var: %
      i_indepdt_var_loc: %
      len of i_co_value: %
      i_input_arr_asso: %
      len of i_depdt_var: %
      len of i_dloss_ddepdt: %
      sqlerrm: %
    '
    , i_node_no
    , i_lambda             
    , v_indepdt_var_len
    , i_indepdt_var_loc      
    , array_dims(i_co_value)
    , i_input_arr_asso
    , array_dims(i_depdt_var)
    , array_dims(i_dloss_ddepdt)
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;



-- select 
--   sm_sc.fv_lambda_arr_dloss_dindepdt
--   (
--     100001
--     , 'softmax_x'
--     , array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]]
--     , 2
--     , null
--     , null
--     , sm_sc.fv_standlize_x_softmax(array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]])
--     , (-` array[[0.0 :: float, 0.0 :: float, 1.0], [0.0 :: float, 0.0 :: float, 1.0]]) /` sm_sc.fv_standlize_x_softmax(array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]])
--   );




-- 本函数以数组为入参，标量版本参考 sm_sc.fv_gradient_opr
-- 对于 conv_2d, pool_max, pool_avg, softmax, zscore 本函数求取 dloss/dindepdt, 而不是求取 ddepdt/dindepdt
-- drop function if exists sm_sc.fv_lambda_arr_dloss_dindepdt_p(char(6), bigint, bigint, bigint, varchar(64), varchar(64), int, varchar(64), float[], varchar(64), int, varchar(64));
create or replace function sm_sc.fv_lambda_arr_dloss_dindepdt_p
(
  i_model_code_6            char(6)
, i_work_no                 bigint
, i_fore_node_no            bigint
, i_back_node_no            bigint
, i_lambda                  varchar(64)                       
, i_indepdt_var_p           varchar(64)                  
, i_indepdt_var_loc         int         default 1            -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 prod_mx, sub, div, pow, log, softmax 等运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
, i_co_value_p              varchar(64) default null         -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
, i_input_arr_asso          float[]     default null         
, i_depdt_var_p             varchar(64) default null         -- 前向的因变量
, i_dloss_ddepdt_p          varchar(64) default null         -- 此入参传入 dloss/ddepdt, 用于 反向传播阶段 求取 dloss/dindepdt
, i_indepdt_var_len         int[]       default null         -- 自变量的高宽规格，agg_concat_x, agg_concat_y, agg_sum, agg_avg, slice_x, slice_y, add, sub 会用到改参数，以避免传参 v_indepdt_var，从而降低堆区开销
, i_ddepdt_dindepdt_p       varchar(64) default null         -- 因变量对自变量导数
)
returns varchar(64)
as
$$
declare 
  v_indepdt_var      float[]    := sm_sc.__fv_get_kv(i_indepdt_var_p);
  v_co_value         float[]    := sm_sc.__fv_get_kv(i_co_value_p);
  v_depdt_var        float[]    := sm_sc.__fv_get_kv(i_depdt_var_p);
  v_dloss_ddepdt     float[]    := sm_sc.__fv_get_kv(i_dloss_ddepdt_p);
  v_ddepdt_dindepdt  float[]    := sm_sc.__fv_get_kv(i_ddepdt_dindepdt_p);
  v_ret              float[];
  i_indepdt_var_len  int[]      := coalesce(i_indepdt_var_len, (select array_agg(array_length(v_indepdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(v_indepdt_var)) tb_a_ndim(a_ndim)));
 
 -- debug 
 v_begin_clock    timestamp     := clock_timestamp();
begin
-- -- debug
-- raise notice '
-- sm_sc.fv_lambda_arr_dloss_dindepdt_p debug :
--   i_lambda                : %; 
--   v_indepdt_var             : %; 
--   i_indepdt_var_loc         : %; 
--   v_co_value              : %; 
--   i_input_arr_asso   : %; 
--   v_depdt_var           : %; 
-- ', $1, $2, $3, $4, $5, $6
-- ;

  -- 动态 sql 会影响性能，所以还是用 case 判断
  
-- -- debug 
-- raise notice 'lambda_dloss_dindepdt % begin:: i_lambda: %; len_dloss_dindepdt: %; indepdt_var_loc: %; time: %;'
-- , i_fore_node_no, i_lambda, array[array_length(v_dloss_ddepdt, 1), array_length(v_dloss_ddepdt, 2)], i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 'NaN' :: float = any(v_indepdt_var)
      or 'NaN' :: float = any(v_co_value)
      or 'NaN' :: float = any(v_depdt_var)
      or 'NaN' :: float = any(v_dloss_ddepdt)
    then 
      raise exception 'there is NaN value in v_indepdt_var, v_co_value, v_depdt_var or v_dloss_ddepdt!  i_fore_node_no: %.  ', i_fore_node_no;
    end if;
  end if;
  
  if i_lambda like '07_%'
  then 
    if i_lambda = '07_aggr_slice_sum'
    then 
      v_ret := 
        sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt(i_indepdt_var_len, v_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_prod'
    then
      v_ret := 
        sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt(v_indepdt_var, v_depdt_var, v_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_avg'
    then
      v_ret := 
        sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_max'
    then
      v_ret := 
        sm_sc.fv_d_aggr_slice_max_dloss_dindepdt(v_indepdt_var, v_depdt_var, v_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_min'
    then
      v_ret := 
        sm_sc.fv_d_aggr_slice_min_dloss_dindepdt(v_indepdt_var, v_depdt_var, v_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_sum'
    then 
      v_ret := 
        sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt(i_input_arr_asso[1 : 4] :: int[], v_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_prod'
    then
      v_ret := 
        sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt(v_indepdt_var, v_depdt_var, v_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_avg'
    then
      v_ret := 
        sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt(i_input_arr_asso[1 : 4] :: int[], v_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_max'
    then
      v_ret := 
        sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt(v_indepdt_var, v_depdt_var, v_dloss_ddepdt)
      ;
    elsif i_lambda = '07_aggr_chunk_min'
    then
      v_ret := 
        sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt(v_indepdt_var, v_depdt_var, v_dloss_ddepdt)
      ;
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '06_%'
  then 
    if i_lambda = '06_aggr_mx_concat_y'
    then
      v_ret := 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n
          (
            v_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            i_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    elsif i_lambda = '06_aggr_mx_concat_x'
    then
      v_ret := 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n
          (
            v_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            i_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    elsif i_lambda = '06_aggr_mx_concat_x3'
    then
      v_ret := 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n
          (
            v_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            i_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    elsif i_lambda = '06_aggr_mx_concat_x4'
    then
      v_ret := 
      (
        with recursive 
        cte_idx as 
        (
          select 
            1 :: int as a_cur,
            0 :: int as a_accum_len
          union all 
          select 
            a_cur + 1 as a_cur, 
            a_accum_len + i_input_arr_asso[a_cur] :: int as a_accum_len
          from cte_idx
          where a_cur <= i_indepdt_var_loc - 1  -- array_length(i_input_arr_asso, 1)
        )
        select 
          sm_sc.fv_d_mx_concat_x4_dloss_dindepdt_n
          (
            v_dloss_ddepdt, 
            a_accum_len + 1, 
            a_accum_len + i_input_arr_asso[i_indepdt_var_loc] :: int,
            i_indepdt_var_len
          )
        from cte_idx
        where a_cur = i_indepdt_var_loc
      )
      ;
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;

  elsif i_lambda like '05_%'
  then 
    if i_lambda = '05_pool_max_2d_grp_x'
    then 
      v_ret := 
        sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt
        (
          v_indepdt_var                              ,                                       -- i_array_grp_x
          i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
          v_dloss_ddepdt                         ,
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          v_depdt_var                            ,                                       -- i_depdt_1d_grp
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
          coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg_2d_grp_x'
    then
      v_ret := 
        sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt
        (
          -- -- i_input_arr_asso[11] :: int         ,                                       -- 规约：存放 i_array_grp_x_len
          i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
          v_dloss_ddepdt                         ,
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
        )
      ;
    elsif i_lambda = '05_pool_max'
    then
      v_ret := 
        sm_sc.fv_d_pool_max_dloss_dindepdt_ex
        (
          v_indepdt_var                              ,                                       -- i_array
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          v_dloss_ddepdt                         ,
          v_depdt_var                            ,                                       -- i_depdt_1d_grp
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
          coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg'
    then
      v_ret := 
        sm_sc.fv_d_pool_avg_dloss_dindepdt_ex
        (
          -- i_indepdt_var_len                 ,                                         -- 规约：自变量规格
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          v_dloss_ddepdt                         ,
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
        )
      ;
    elsif i_lambda = '05_pool_none'
    then
      v_ret := 
        sm_sc.fv_d_pool_none_dloss_dindepdt
        (
          i_indepdt_var_len                 ,                                         -- 规约：自变量规格
          i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
          v_dloss_ddepdt                         ,
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
        )
      ;
    elsif i_lambda = '05_conv_2d_grp_x'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1
          (
            i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
            v_dloss_ddepdt                      ,
            v_co_value                          ,                                       -- i_window
            i_input_arr_asso[3] :: int          ,                                       -- 规约：存放 i_window_len_x
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2
          (
            v_co_value                               ,                                  -- i_array_grp_x
            i_input_arr_asso[1] :: int          ,                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
            v_dloss_ddepdt                         ,
            i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
            i_input_arr_asso[12] :: int :: boolean                                  ,   -- 规约：存放 i_window_bias_label
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_2d'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex
          (
            v_dloss_ddepdt,
            v_co_value,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])         -- 规约：存放 i_padding
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py
          (
            v_co_value,
            v_dloss_ddepdt,
            i_indepdt_var_len  ,  -- i_input_arr_asso[2 : 3] :: int[]    ,                                       -- 规约：存放 i_window_len
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 3
      then 
        v_ret := 
          sm_sc.fv_d_conv_2d_dloss_dindepdt_3(v_dloss_ddepdt, i_indepdt_var_len)
        ;      
      end if;
    elsif i_lambda = '05_conv_add'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_add_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_add_dloss_dindepdt_2
          (
        --  v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_sub'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_sub_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_sub_dloss_dindepdt_2
          (
        --  v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_mul'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_mul_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_mul_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_div'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_div_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_div_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_indepdt_var,
        --  i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_pow'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_pow_dloss_dindepdt_1
          (
            v_indepdt_var,
        --  i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_depdt_var,
            array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_pow_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_indepdt_var,
            i_indepdt_var_len,
            v_depdt_var,
            array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_log'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_log_dloss_dindepdt_1
          (
            v_indepdt_var,
            i_indepdt_var_len,
            v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_log_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_indepdt_var,
        --  i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
            i_input_arr_asso[2] :: int,                                                 -- 规约：存放 i_window_len_heigh
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            i_input_arr_asso[2] :: int,                                                 -- 规约：存放 i_window_len_heigh
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_sub'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2
          (
        --  v_co_value,
            array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_indepdt_var,
            i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_div'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_div_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_div_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_indepdt_var,
        --  i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_pow'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1
          (
            v_indepdt_var,
        --  i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_depdt_var,
            array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_indepdt_var,
            i_indepdt_var_len,
            v_depdt_var,
            array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_log'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_log_dloss_dindepdt_1
          (
            v_indepdt_var,
        --  i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_log_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            v_indepdt_var,
        --  i_indepdt_var_len,
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    elsif i_lambda = '05_conv_de_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1
          (
        --  v_indepdt_var,
            i_indepdt_var_len,
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
            i_input_arr_asso[3] :: int,                                                 -- 规约：存放 i_window_len_width
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding
            coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2
          (
            v_co_value,
        --  array[array_length(v_co_value, 1), array_length(v_co_value, 2)],
        --  v_indepdt_var,
            i_indepdt_var_len,
            i_input_arr_asso[3] :: int,                                                 -- 规约：存放 i_window_len_width
        --  v_depdt_var,
        --  array[array_length(v_depdt_var, 1), array_length(v_depdt_var, 2)],
            v_dloss_ddepdt,
            coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride
            coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])  -- ,   -- 规约：存放 i_padding
        --  coalesce(i_input_arr_asso[10] :: float          ,0.0              )         -- 规约：存放 i_padding_value
          )
        ;      
      end if;
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;
  
  elsif i_lambda like '04_%'
  then 
    if i_lambda = '04_new'
    then 
      v_ret := 
        sm_sc.fv_d_new_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '04_repeat_axis'
    then 
      v_ret := 
        sm_sc.fv_d_repeat_axis_dloss_dindepdt(v_dloss_ddepdt, sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][ : ]) :: int[], sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][ : ]) :: int[])
      ;
    elsif i_lambda = '04_apad'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_apad_dloss_dindepdt_1(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1) - 1])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_apad_dloss_dindepdt_2(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1) - 1], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_bpad'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_bpad_dloss_dindepdt_1(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1) - 1])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_bpad_dloss_dindepdt_2(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1) - 1], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_lpad'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_lpad_dloss_dindepdt_1(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1)])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_lpad_dloss_dindepdt_2(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1)], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_rpad'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_rpad_dloss_dindepdt_1(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1)])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_rpad_dloss_dindepdt_2(v_dloss_ddepdt, i_indepdt_var_len[array_length(i_indepdt_var_len, 1)], i_input_arr_asso[1] :: int)
        ;      
      end if;
    elsif i_lambda = '04_transpose'
    then
      v_ret := 
        sm_sc.fv_d_transpose_dloss_dindepdt(v_dloss_ddepdt)
      ;
    elsif i_lambda = '04_transpose_i'
    then
      v_ret := 
        sm_sc.fv_d_transpose_i_dloss_dindepdt(v_dloss_ddepdt)
      ;
    elsif i_lambda = '04_chunk_transpose'
    then
      v_ret := 
        sm_sc.fv_d_chunk_transpose_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_transpose_nd'
    then
      v_ret := 
        sm_sc.fv_d_transpose_nd_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_90'
    then
      v_ret := 
        sm_sc.fv_d_turn_90_dloss_dindepdt_py(v_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_180'
    then
      v_ret := 
        sm_sc.fv_d_turn_180_dloss_dindepdt_py(v_dloss_ddepdt, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_mirror'
    then
      v_ret := 
        sm_sc.fv_d_mirror_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_ele_3d_2_2d'
    then
      v_ret := 
        sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        , i_indepdt_var_len
        )
      ;
    elsif i_lambda = '04_mx_ele_2d_2_3d'
    then
      v_ret := 
        sm_sc.fv_d_mx_ele_2d_2_3d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int
        , i_input_arr_asso[4] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_4d_2_3d'
    then
      v_ret := 
        sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        , i_indepdt_var_len
        )
      ;
    elsif i_lambda = '04_mx_ele_3d_2_4d'
    then
      v_ret := 
        sm_sc.fv_d_mx_ele_3d_2_4d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_flatten_2dims'
    then
      v_ret := 
        sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt_py
        (
          v_dloss_ddepdt
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        , i_indepdt_var_len
        )
      ;
    elsif i_lambda = '04_mx_slice_3d_2_2d'
    then
      v_ret := 
        sm_sc.fv_d_mx_slice_3d_2_2d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_indepdt_var_len
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        )
      ;
    elsif i_lambda = '04_mx_slice_4d_2_2d'
    then
      v_ret := 
        sm_sc.fv_d_mx_slice_4d_2_2d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_indepdt_var_len
        , sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][ : ] :: int[])
        , sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][ : ] :: int[])
        )
      ;
    elsif i_lambda = '04_mx_slice_4d_2_3d'
    then
      v_ret := 
        sm_sc.fv_d_mx_slice_4d_2_3d_dloss_dindepdt
        (
          v_dloss_ddepdt
        , i_indepdt_var_len
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        )
      ;
    elsif i_lambda = '04_mx_ascend_dim'
    then
      v_ret := 
        sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_descend_dim'
    then
      v_ret := 
        sm_sc.fv_d_mx_descend_dim_dloss_dindepdt(v_dloss_ddepdt, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_chunk'
    then
      v_ret := 
        sm_sc.fv_d_chunk_dloss_dindepdt(v_dloss_ddepdt, i_indepdt_var_len, sm_sc.fv_mx_descend_dim(i_input_arr_asso[1][ : ]) :: int[])
      ;
    elsif i_lambda = '04_slice_y'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_slice_y_dloss_dindepdt
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[1]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_slice_x_dloss_dindepdt
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[2]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x3'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_slice_x3_dloss_dindepdt
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[3]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x4'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_slice_x4_dloss_dindepdt
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[4]
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_y'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_sample_y_dloss_dindepdt_1
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[1]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_sample_x_dloss_dindepdt_1
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[2]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x3'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_sample_x3_dloss_dindepdt_1
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[3]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x4'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_d_sample_x4_dloss_dindepdt_1
          (
            v_dloss_ddepdt
          , i_indepdt_var_len[4]
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_lower_tri_mx'
    then
      v_ret := sm_sc.fv_d_lower_tri_mx_dloss_dindepdt(v_dloss_ddepdt);
    elsif i_lambda = '04_upper_tri_mx'
    then
      v_ret := sm_sc.fv_d_upper_tri_mx_dloss_dindepdt(v_dloss_ddepdt);
    elsif i_lambda = '04_lower_tri_mx_ex'
    then
      v_ret := sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt(v_dloss_ddepdt);
    elsif i_lambda = '04_upper_tri_mx_ex'
    then
      v_ret := sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt(v_dloss_ddepdt);
    elsif i_lambda = '04_lmask'
    then
      v_ret := sm_sc.fv_d_lmask_dloss_dindepdt_1(v_dloss_ddepdt, v_co_value :: int[]);
    elsif i_lambda = '04_rmask'
    then
      v_ret := sm_sc.fv_d_rmask_dloss_dindepdt_1(v_dloss_ddepdt, v_co_value :: int[]);
    elsif i_lambda = '04_amask'
    then
      v_ret := sm_sc.fv_d_amask_dloss_dindepdt_1(v_dloss_ddepdt, v_co_value :: int[]);
    elsif i_lambda = '04_bmask'
    then
      v_ret := sm_sc.fv_d_bmask_dloss_dindepdt_1(v_dloss_ddepdt, v_co_value :: int[]);
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '03_%'
  then 
    if i_lambda = '03_softmax'
    then
      v_ret := 
        sm_sc.fv_d_redistr_softmax_dloss_dindepdt(v_depdt_var, v_dloss_ddepdt, v_indepdt_var, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '03_softmax_ex'
    then
      v_ret := 
        sm_sc.fv_d_redistr_softmax_dloss_dindepdt(v_depdt_var, v_dloss_ddepdt, v_indepdt_var, i_input_arr_asso[1 : 4] :: int[])
      ;
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda = '01_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_prod_mx_dloss_dindepdt_1(v_dloss_ddepdt, v_co_value, i_indepdt_var_len)
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_prod_mx_dloss_dindepdt_2(v_dloss_ddepdt, v_co_value, i_indepdt_var_len)
        ;
      end if;
    elsif i_lambda = '01_chunk_prod_mx'
    then
      if i_indepdt_var_loc = 1
      then 
        v_ret := 
          sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1(v_dloss_ddepdt, v_co_value, i_indepdt_var_len, i_input_arr_asso[1 : 3] :: int[])
        ;
      elsif i_indepdt_var_loc = 2
      then 
        v_ret := 
          sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2(v_dloss_ddepdt, v_co_value, i_indepdt_var_len, i_input_arr_asso[1 : 3] :: int[])
        ;
      end if;
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '00_%'
  then 
    if i_lambda = '00_none'
    then
      v_ret := 
        sm_sc.fv_d_nn_none_dloss_dindepdt(v_dloss_ddepdt)
      ;
    else 
      v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
    end if;
    
  elsif i_lambda like '81_%'
  then 
    v_ret := 
      sm_sc.ufv_lambda_arr_dloss_dindepdt
      (
        i_fore_node_no        
      , i_lambda         
      , v_indepdt_var    
      , i_indepdt_var_loc
      , v_co_value       
      , i_input_arr_asso 
      , v_depdt_var      
      , v_dloss_ddepdt   
      , i_indepdt_var_len
      , v_ddepdt_dindepdt
      )
    ;
    
  else 
    v_ret := v_dloss_ddepdt *` v_ddepdt_dindepdt;
  end if;  

  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then 
  -- --   if v_ret is null
  -- --     -- or sm_sc.fv_aggr_slice_is_exists_null(v_ret) is true
  -- --   then 
  -- --     raise exception 'there is null value in v_ret :=ing!  i_fore_node_no: %.  ', i_fore_node_no;
  -- --   elsif 'NaN' :: float = any(v_ret)
  -- --   then 
  -- --     raise exception 'there is NaN value in v_ret :=ing!  i_fore_node_no: %.  ', i_fore_node_no;
  -- --   end if;
  -- -- end if;
  -- -- 
  -- -- v_ret := v_ret;
  
-- -- debug 
-- raise notice 'lambda_dloss_dindepdt % end:: i_lambda: %; indepdt_var_loc: %; time:%;', i_fore_node_no, i_lambda, i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

-- -- debug 
-- raise notice 'dldi: i_back_node_no: %; i_fore_node_no: %; loc: %; i_lambda: %; time: %;', i_back_node_no, i_fore_node_no, i_indepdt_var_loc, i_lambda, v_begin_clock - clock_timestamp();

  return 
    sm_sc.__fv_set_kv
    (
      v_ret
    , i_model_code_6 || 
      '_' || i_work_no :: varchar   ||
      '_' || i_fore_node_no :: varchar ||
      '_' || i_back_node_no :: varchar ||
      '_dldi'  -- 'd_loss_d_indepdt'
      '__' || i_indepdt_var_loc :: varchar
    )
  ;

  exception when others then
    raise exception 
    ' i_fore_node_no: %
      fn: sm_sc.fv_lambda_arr_dloss_dindepdt_p
      i_lambda: %
      len of v_indepdt_var: %
      i_indepdt_var_loc: %
      len of v_co_value: %
      i_input_arr_asso: %
      len of v_depdt_var: %
      len of v_dloss_ddepdt: %
      sqlerrm: %
    '
    , i_fore_node_no
    , i_lambda             
    , i_indepdt_var_len
    , i_indepdt_var_loc      
    , array_dims(v_co_value)
    , i_input_arr_asso
    , array_dims(v_depdt_var)
    , array_dims(v_dloss_ddepdt)
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;



-- select 
--   sm_sc.fv_lambda_arr_dloss_dindepdt_p
--   (
--     100001
--     , 'softmax_x'
--     , array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]]
--     , 2
--     , null
--     , null
--     , sm_sc.fv_standlize_x_softmax(array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]])
--     , (-` array[[0.0 :: float, 0.0 :: float, 1.0], [0.0 :: float, 0.0 :: float, 1.0]]) /` sm_sc.fv_standlize_x_softmax(array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]])
--   );




-- drop function if exists sm_sc.fv_lambda_arr_len(bigint, varchar(64), int[], int[], float[], int[]);
create or replace function sm_sc.fv_lambda_arr_len
(
  i_node_no                   bigint                          ,
  i_lambda                    varchar(64)                     ,
  i_input_p1_len              int[]                           ,
  i_input_p2_len              int[]             default null  ,      -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立
  i_input_arr_asso            float[]           default null  ,    
  i_input_p3_len              int[]             default null  
)
returns int[]
as
$$
declare 
  v_arr_asso       int[]     := i_input_arr_asso ;
  v_p1_ndims       int       := array_length(i_input_p1_len, 1);
  v_p2_ndims       int       := array_length(i_input_p2_len, 1);
  v_p3_ndims       int       := array_length(i_input_p3_len, 1);
  v_p1_len_heigh   int       := i_input_p1_len[v_p1_ndims - 1];
  v_p1_len_width   int       := i_input_p1_len[v_p1_ndims];
  v_p2_len_heigh   int       := i_input_p2_len[v_p2_ndims - 1];
  v_p2_len_width   int       := i_input_p2_len[v_p2_ndims];
  v_p3_len_heigh   int       := i_input_p3_len[v_p3_ndims - 1];
  v_p3_len_width   int       := i_input_p3_len[v_p3_ndims];
  v_n_ndims_len    int[]     := 
    -- case 
    --   when v_p1_ndims = 4 then i_input_p1_len[1 : 2] 
    --   when v_p1_ndims = 3 then i_input_p1_len[1 : 1]
    --   when v_p2_ndims = 4 then i_input_p2_len[1 : 2]
    --   when v_p2_ndims = 3 then i_input_p2_len[1 : 1]
    --   when v_p3_ndims = 4 then i_input_p3_len[1 : 2]
    --   when v_p3_ndims = 3 then i_input_p3_len[1 : 1]
    --   else null
    -- end
    (
      select 
        sm_sc.fa_mx_max(a_n_ndims_len)
      from 
      (
        select 
          (sm_sc.fv_lpad(i_input_p1_len, array[1], greatest(v_p1_ndims, v_p2_ndims, v_p3_ndims) - v_p1_ndims))
            [ : greatest(v_p1_ndims, v_p2_ndims, v_p3_ndims) - 2]
          as a_n_ndims_len
        union all 
        select 
          (sm_sc.fv_lpad(i_input_p2_len, array[1], greatest(v_p1_ndims, v_p2_ndims, v_p3_ndims) - v_p2_ndims))
            [ : greatest(v_p1_ndims, v_p2_ndims, v_p3_ndims) - 2]
        union all 
        select 
          (sm_sc.fv_lpad(i_input_p3_len, array[1], greatest(v_p1_ndims, v_p2_ndims, v_p3_ndims) - v_p3_ndims))
            [ : greatest(v_p1_ndims, v_p2_ndims, v_p3_ndims) - 2]
      ) tb_a
    )
    ;
  
begin
  -- 审计各参数各维度长度是否匹配
  if i_lambda like '00_%'
  then 
    if i_lambda = '00_buff_slice_rand_pick'
    then 
      return 
        sm_sc.fv_aggr_slice_sum(v_arr_asso[3 : 3][ : ])
      ;

    elsif i_lambda = '00_const'
    then 
      return 
      (
        select 
          array_agg(array_length(i_input_arr_asso, a_ndim) order by a_ndim) 
        from generate_series(1, array_ndims(i_input_arr_asso)) tb_a_ndim(a_ndim)
      )
      ;

    elsif i_lambda = '00_none'
    then 
      return 
        i_input_p1_len
      ;

    elsif i_lambda = '00_full_dataset'
    then 
      return 
        (
          select 
            count(*) 
          from sm_sc.tb_nn_train_input_buff 
          where work_no = (select work_no from sm_sc.tb_nn_node where node_no = i_node_no)
        )
      ;

    elsif i_lambda = '00_buff_slice_rand_pick'
    then 
      return 
        sm_sc.fv_aggr_slice_sum(i_input_arr_asso[3 : 3][ : ])
      ;
  
    else
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
      then 
        return 
          i_input_p1_len;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda in ('01_add', '01_mul', '01_sub', '01_div', '01_mod', '01_pow', '01_log')
    then
      if not exists 
             (
               select  
               from unnest(sm_sc.__fv_mirror_y(i_input_p1_len), sm_sc.__fv_mirror_y(i_input_p2_len)) tb_a_len(a_len_left, a_len_right)
               where a_len_left <> a_len_right
                 and a_len_left <> 1
                 and a_len_right <> 1
             )
      then 
        return 
          (
            select 
              sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
            from unnest(sm_sc.__fv_mirror_y(i_input_p1_len), sm_sc.__fv_mirror_y(i_input_p2_len)) tb_a_len(a_len_left, a_len_right)
          )
        ;
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
      
    elsif i_lambda = '01_prod_mx'
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0 
        and v_p1_len_width > 0 and coalesce(v_p2_len_width, v_arr_asso[3]) > 0
        and v_p1_len_width = coalesce(v_p2_len_heigh, v_arr_asso[2]) 
      then 
        return 
          -- v_arr_asso[1] :: int                                                   -- 规约：存放 array_length(i_indepdt, 1)
          -- v_arr_asso[2] :: int                                                   -- 规约：存放 array_length(i_indepdt, 2), 也即 array_length(i_w, 1)
          -- v_arr_asso[3] :: int                                                   -- 规约：存放 array_length(i_w, 2)
          v_n_ndims_len ||
          array[v_p1_len_heigh, coalesce(v_p2_len_width, v_arr_asso[3])];
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
      
    elsif i_lambda = '01_chunk_prod_mx'
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0 and v_p2_len_heigh > 0 and v_p2_len_width > 0
        and v_p1_len_heigh % v_arr_asso[1] :: int = 0
        and v_p1_len_width % v_arr_asso[2] :: int = 0
        and v_p2_len_heigh % v_arr_asso[2] :: int = 0
        and v_p2_len_width % v_arr_asso[3] :: int = 0
        and v_p1_len_heigh / v_arr_asso[1] = v_p2_len_heigh / v_arr_asso[2]
        and v_p1_len_width / v_arr_asso[2] = v_p2_len_width / v_arr_asso[3]
      then 
        return 
          -- v_arr_asso[1] :: int                                                   -- 规约：存放 矩阵乘法 chunk 入参一高度规格                    
          -- v_arr_asso[2] :: int                                                   -- 规约：存放 矩阵乘法 chunk 入参一宽度规格，也即入参二高度规格
          -- v_arr_asso[3] :: int                                                   -- 规约：存放 矩阵乘法 chunk 入参二宽度规格                    
          v_n_ndims_len ||
          array[v_p1_len_heigh, v_p2_len_width];
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;

    else
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
      then 
        return 
          i_input_p1_len;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    end if;

  elsif i_lambda like '04_%'
  then 
    if i_lambda = '04_new'
    then
      if array_length(i_input_p1_len, 1) = array_length(v_arr_asso, 1)
      then 
        return 
          i_input_p1_len * v_arr_asso
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
     
    elsif i_lambda = '04_repeat_axis'
    then
      return 
        i_input_p1_len
        *
        (
          select 
            array_agg(coalesce(a_repeat, 1) order by a_dim_no)
          from generate_series(1, v_p1_ndims) tb_a_dim_no(a_dim_no)
          left join unnest(v_arr_asso[1 : 1][ : ], v_arr_asso[2 : 2][ : ]) tb_a_axis_no(a_axis_no, a_repeat)
            on tb_a_axis_no.a_axis_no = tb_a_dim_no.a_dim_no
        )
      ;
      
    elsif i_lambda = '04_apad'
    then
      if (array_length(i_input_p2_len, 1) = array_length(i_input_p1_len, 1)
          or array_length(i_input_p2_len, 1) <= array_length(i_input_p1_len, 1) and array_length(i_input_p2_len, 1) <= 2
         )
        and v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh >= 0 and v_p2_len_width >= 0
        and v_p1_len_width = v_p2_len_width
        and (v_n_ndims_len = i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] 
             or i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] is null
            )
      then 
        return 
          v_n_ndims_len ||
          array
          [
            (v_p2_len_heigh * v_arr_asso[1]) + v_p1_len_heigh
          , v_p1_len_width
          ]
        ;
      else 
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_bpad'
    then
      if (array_length(i_input_p2_len, 1) = array_length(i_input_p1_len, 1)
          or array_length(i_input_p2_len, 1) <= array_length(i_input_p1_len, 1) and array_length(i_input_p2_len, 1) <= 2
         )
        and v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh >= 0 and v_p2_len_width >= 0
        and v_p1_len_width = v_p2_len_width
        and (v_n_ndims_len = i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] 
             or i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] is null
            )
      then 
        return 
          v_n_ndims_len ||
          array
          [
            v_p1_len_heigh + (v_p2_len_heigh * v_arr_asso[1])
          , v_p1_len_width
          ]
        ;
      else 
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_lpad'
    then
      if (array_length(i_input_p2_len, 1) = array_length(i_input_p1_len, 1)
          or array_length(i_input_p2_len, 1) <= array_length(i_input_p1_len, 1) and array_length(i_input_p2_len, 1) <= 2
         )
        and v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh >= 0 and v_p2_len_width >= 0
        and v_p1_len_heigh = v_p2_len_heigh
        and (v_n_ndims_len = i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] 
             or i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] is null
            )
      then 
        return 
          v_n_ndims_len ||
          array
          [
            v_p1_len_heigh
          , (v_p2_len_width * v_arr_asso[1]) + v_p1_len_width
          ]
        ;
      else 
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_rpad'
    then
      if (array_length(i_input_p2_len, 1) = array_length(i_input_p1_len, 1)
          or array_length(i_input_p2_len, 1) <= array_length(i_input_p1_len, 1) and array_length(i_input_p2_len, 1) <= 2
         )
        and v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh >= 0 and v_p2_len_width >= 0
        and v_p1_len_heigh = v_p2_len_heigh
        and (v_n_ndims_len = i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] 
             or i_input_p2_len[ : array_length(i_input_p2_len, 1) - 2] is null
            )
      then 
        return 
          v_n_ndims_len ||
          array
          [
            v_p1_len_heigh
          , v_p1_len_width + (v_p2_len_width * v_arr_asso[1])
          ]
        ;
      else 
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda in ('04_transpose', '04_transpose_i', '04_chunk_transpose')
    then
      return 
        v_n_ndims_len ||
        array
        [
          v_p1_len_width
        , v_p1_len_heigh
        ]
      ;
      
    elsif i_lambda = '04_transpose_nd'
    then
      if array_length(i_input_p1_len, 1) = 2
      then 
        return 
          array
          [
            i_input_p1_len[2]
          , i_input_p1_len[1]
          ]
        ;
      elsif array_length(i_input_p1_len, 1) = 3 
      then 
        if v_arr_asso in (array[1, 2], array[2, 1])
        then 
          return 
            array
            [
              i_input_p1_len[2]
            , i_input_p1_len[1]
            , i_input_p1_len[3]
            ]
          ;
        elsif v_arr_asso in (array[1, 3], array[3, 1])
        then 
          return 
            array
            [
              i_input_p1_len[3]
            , i_input_p1_len[2]
            , i_input_p1_len[1]
            ]
          ;
        elsif v_arr_asso in (array[3, 2], array[2, 3])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[3]
            , i_input_p1_len[2]
            ]
          ;
        end if;
      elsif array_length(i_input_p1_len, 1) = 4 
      then 
        if v_arr_asso in (array[1, 2], array[2, 1])
        then 
          return 
            array
            [
              i_input_p1_len[2]
            , i_input_p1_len[1]
            , i_input_p1_len[3]
            , i_input_p1_len[4]
            ]
          ;
        elsif v_arr_asso in (array[1, 3], array[3, 1])
        then 
          return 
            array
            [
              i_input_p1_len[3]
            , i_input_p1_len[2]
            , i_input_p1_len[1]
            , i_input_p1_len[4]
            ]
          ;
        elsif v_arr_asso in (array[3, 2], array[2, 3])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[3]
            , i_input_p1_len[2]
            , i_input_p1_len[4]
            ]
          ;
        elsif v_arr_asso in (array[1, 4], array[4, 1])
        then 
          return 
            array
            [
              i_input_p1_len[4]
            , i_input_p1_len[2]
            , i_input_p1_len[3]
            , i_input_p1_len[1]
            ]
          ;
        elsif v_arr_asso in (array[2, 4], array[4, 2])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[4]
            , i_input_p1_len[3]
            , i_input_p1_len[2]
            ]
          ;
        elsif v_arr_asso in (array[3, 4], array[4, 3])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[2]
            , i_input_p1_len[4]
            , i_input_p1_len[3]
            ]
          ;
        end if;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_turn_90'
    then
      if array_length(i_input_p1_len, 1) = 2 
      then 
        return 
          array
          [
            i_input_p1_len[2]
          , i_input_p1_len[1]
          ]
        ;
      elsif array_length(i_input_p1_len, 1) = 3 
      then 
        if v_arr_asso in (array[1, 2], array[2, 1])
        then 
          return 
            array
            [
              i_input_p1_len[2]
            , i_input_p1_len[1]
            , i_input_p1_len[3]
            ]
          ;
        elsif v_arr_asso in (array[1, 3], array[3, 1])
        then 
          return 
            array
            [
              i_input_p1_len[3]
            , i_input_p1_len[2]
            , i_input_p1_len[1]
            ]
          ;
        elsif v_arr_asso in (array[3, 2], array[2, 3])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[3]
            , i_input_p1_len[2]
            ]
          ;
        end if;
      elsif array_length(i_input_p1_len, 1) = 4 
      then 
        if v_arr_asso in (array[1, 2], array[2, 1])
        then 
          return 
            array
            [
              i_input_p1_len[2]
            , i_input_p1_len[1]
            , i_input_p1_len[3]
            , i_input_p1_len[4]
            ]
          ;
        elsif v_arr_asso in (array[1, 3], array[3, 1])
        then 
          return 
            array
            [
              i_input_p1_len[3]
            , i_input_p1_len[2]
            , i_input_p1_len[1]
            , i_input_p1_len[4]
            ]
          ;
        elsif v_arr_asso in (array[3, 2], array[2, 3])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[3]
            , i_input_p1_len[2]
            , i_input_p1_len[4]
            ]
          ;
        elsif v_arr_asso in (array[1, 4], array[4, 1])
        then 
          return 
            array
            [
              i_input_p1_len[4]
            , i_input_p1_len[2]
            , i_input_p1_len[3]
            , i_input_p1_len[1]
            ]
          ;
        elsif v_arr_asso in (array[2, 4], array[4, 2])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[4]
            , i_input_p1_len[3]
            , i_input_p1_len[2]
            ]
          ;
        elsif v_arr_asso in (array[3, 4], array[4, 3])
        then 
          return 
            array
            [
              i_input_p1_len[1]
            , i_input_p1_len[2]
            , i_input_p1_len[4]
            , i_input_p1_len[3]
            ]
          ;
        end if;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_ele_3d_2_2d'
    then
      if array_length(i_input_p1_len, 1) = 3
      then 
        i_input_p1_len[v_arr_asso[2]] 
          := i_input_p1_len[v_arr_asso[1]] * i_input_p1_len[v_arr_asso[2]]
        ;
        return 
          i_input_p1_len[ : v_arr_asso[1] - 1] || i_input_p1_len[v_arr_asso[1] + 1 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_ele_2d_2_3d'
    then
      if array_length(i_input_p1_len, 1) = 2
      then 
        if v_arr_asso[4] :: int :: boolean -- i_if_dim_pin_ele_on_from
        then 
          v_arr_asso[4] := i_input_p1_len[v_arr_asso[2]];   -- 借 v_arr_asso[4] 当临时变量寄存 i_input_p1_len[v_arr_asso[2]]
          i_input_p1_len[v_arr_asso[2]] := v_arr_asso[1];
          return 
            i_input_p1_len[ : v_arr_asso[3] - 1] || v_arr_asso[4] / v_arr_asso[1] || i_input_p1_len[ : v_arr_asso[3] + 1]
          ;
        else -- not i_if_dim_pin_ele_on_from
          v_arr_asso[4] := i_input_p1_len[v_arr_asso[3]];   -- 借 v_arr_asso[4] 当临时变量寄存 i_input_p1_len[v_arr_asso[3]]
          i_input_p1_len[v_arr_asso[3]] := v_arr_asso[1];
          return 
            i_input_p1_len[ : v_arr_asso[2] - 1] || v_arr_asso[4] / v_arr_asso[1] || i_input_p1_len[ : v_arr_asso[2] + 1]
          ;
        end if;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_ele_4d_2_3d'
    then
      if array_length(i_input_p1_len, 1) = 4
      then 
        i_input_p1_len[v_arr_asso[2]] 
          := i_input_p1_len[v_arr_asso[1]] * i_input_p1_len[v_arr_asso[2]]
        ;
        return 
          i_input_p1_len[ : v_arr_asso[1] - 1] || i_input_p1_len[v_arr_asso[1] + 1 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_ele_3d_2_4d'
    then
      if array_length(i_input_p1_len, 1) = 3
      then 
        if v_arr_asso[4] :: int :: boolean -- i_if_dim_pin_ele_on_from
        then 
          v_arr_asso[4] := i_input_p1_len[v_arr_asso[2]];   -- 借 v_arr_asso[4] 当临时变量寄存 i_input_p1_len[v_arr_asso[2]]
          i_input_p1_len[v_arr_asso[2]] := v_arr_asso[1];
          return 
            i_input_p1_len[ : v_arr_asso[3] - 1] || v_arr_asso[4] / v_arr_asso[1] || i_input_p1_len[ : v_arr_asso[3] + 1]
          ;
        else -- not i_if_dim_pin_ele_on_from
          v_arr_asso[4] := i_input_p1_len[v_arr_asso[3]];   -- 借 v_arr_asso[4] 当临时变量寄存 i_input_p1_len[v_arr_asso[3]]
          i_input_p1_len[v_arr_asso[3]] := v_arr_asso[1];
          return 
            i_input_p1_len[ : v_arr_asso[2] - 1] || v_arr_asso[4] / v_arr_asso[1] || i_input_p1_len[ : v_arr_asso[2] + 1]
          ;
        end if;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_ele_flatten_2dims'
    then
      if array_length(i_input_p1_len, 1) between 2 and 4
      then 
        i_input_p1_len[v_arr_asso[2]] := i_input_p1_len[v_arr_asso[1]] * i_input_p1_len[v_arr_asso[2]];
        i_input_p1_len[v_arr_asso[1]] := 1;
        return 
          i_input_p1_len
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda in ('04_mx_slice_3d_2_2d', '04_mx_slice_4d_2_3d')
    then
      if array_length(i_input_p1_len, 1) in (3, 4)
      then 
        return 
          sm_sc.fv_pos_replaces(i_input_p1_len, v_arr_asso[1 : 1], 1)
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_slice_4d_2_2d'
    then
      if array_length(i_input_p1_len, 1) = 4
      then 
        return 
          sm_sc.fv_pos_replaces(i_input_p1_len, sm_sc.fv_mx_ele_2d_2_1d(v_arr_asso[1 : 1][ : ]), 1)
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_ascend_dim'
    then
      if array_length(i_input_p1_len, 1) + v_arr_asso[1] <= 4
      then 
        return 
          array_fill(1, v_arr_asso[1 : 1]) || i_input_p1_len
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_mx_descend_dim'
    then
      if array_length(i_input_p1_len, 1) <= 4
      then 
        return 
          i_input_p1_len[1 + v_arr_asso[1] : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_rand_pick_y'
    then
      if array_length(i_input_p1_len, 1) >= 1
        and i_input_p1_len[1] >= v_arr_asso[1]
      then 
        return 
          -- 规约：v_arr_asso[1] 存放所设置的 取样数量
          v_arr_asso[1 : 1] || i_input_p1_len[2 : ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    elsif i_lambda = '04_rand_pick_x'
    then
      if array_length(i_input_p1_len, 1) >= 2
        and i_input_p1_len[2] >= v_arr_asso[1]
      then 
        return 
          -- 规约：v_arr_asso[1] 存放所设置的 取样数量
          i_input_p1_len[1 : 1] || v_arr_asso[1 : 1] || i_input_p1_len[3 : ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    elsif i_lambda = '04_rand_pick_x3'
    then
      if array_length(i_input_p1_len, 1) >= 3
        and i_input_p1_len[3] >= v_arr_asso[1]
      then 
        return 
          -- 规约：v_arr_asso[1] 存放所设置的 取样数量
          i_input_p1_len[1 : 2] || v_arr_asso[1 : 1] || i_input_p1_len[4 : ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    elsif i_lambda = '04_rand_pick_x4'
    then
      if array_length(i_input_p1_len, 1) >= 4
        and i_input_p1_len[4] >= v_arr_asso[1]
      then 
        return 
          -- 规约：v_arr_asso[1] 存放所设置的 取样数量
          i_input_p1_len[ : 3] || v_arr_asso[1 : 1];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_chunk'
    then
      if false = all(v_arr_asso[1][ : ] <=` 0)
      or false = all(v_arr_asso[1][ : ] >` v_arr_asso[2][ : ])
      or false = all(v_arr_asso[2][ : ] >` array[i_input_p1_len])
      then 
        return 
          -- 规约：v_arr_asso[1][1 : ] 存放所设置的切块儿位置下界
          --       v_arr_asso[2][1 : ] 存放所设置的切块儿位置上界
          sm_sc.fv_mx_descend_dim(v_arr_asso[2 : 2][1 : ] -` v_arr_asso[1 : 1][1 : ]) +` 1;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_slice_y'
    then
      if array_length(i_input_p1_len, 1) >= 1
        and i_input_p1_len[1] > any(v_arr_asso)
      then 
        return 
          -- 规约：v_arr_asso[1 : 2] 存放所设置的切片位置上下界
          array
          [
            (
              select 
                case 
                  when array_length(v_arr_asso, 1) = 2
                    then sum(v_arr_asso[2][a_range_cur] - v_arr_asso[1][a_range_cur] + 1) 
                  else sum(v_arr_asso[1][a_range_cur])
                end
              from generate_series(1, array_length(v_arr_asso, 2)) tb_a_range_cur(a_range_cur)
            )
          ] || 
          i_input_p1_len[2 : ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_slice_x'
    then
      if array_length(i_input_p1_len, 1) >= 2
        and i_input_p1_len[2] > any(v_arr_asso)
      then 
        return 
          -- 规约：v_arr_asso[1 : 2] 存放所设置的切片位置上下界
          i_input_p1_len[1 : 1] || 
          array
          [
            (
              select 
                case 
                  when array_length(v_arr_asso, 1) = 2
                    then sum(v_arr_asso[2][a_range_cur] - v_arr_asso[1][a_range_cur] + 1) 
                  else sum(v_arr_asso[1][a_range_cur])
                end
              from generate_series(1, array_length(v_arr_asso, 2)) tb_a_range_cur(a_range_cur)
            )
          ] || 
          i_input_p1_len[3 : ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_slice_x3'
    then
      if array_length(i_input_p1_len, 1) >= 3
        and i_input_p1_len[3] > any(v_arr_asso)
      then 
        return 
          -- 规约：v_arr_asso[1 : 2] 存放所设置的切片位置上下界
          i_input_p1_len[1 : 2] || 
          array
          [
            (
              select 
                case 
                  when array_length(v_arr_asso, 1) = 2
                    then sum(v_arr_asso[2][a_range_cur] - v_arr_asso[1][a_range_cur] + 1) 
                  else sum(v_arr_asso[1][a_range_cur])
                end
              from generate_series(1, array_length(v_arr_asso, 2)) tb_a_range_cur(a_range_cur)
            )
          ] || 
          i_input_p1_len[4 : ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_slice_x4'
    then
      if array_length(i_input_p1_len, 1) >= 4
        and i_input_p1_len[4] > any(v_arr_asso)
      then 
        return 
          -- 规约：v_arr_asso[1 : 2] 存放所设置的切片位置上下界
          i_input_p1_len[1 : 3] || 
          array
          [
            (
              select 
                case 
                  when array_length(v_arr_asso, 1) = 2
                    then sum(v_arr_asso[2][a_range_cur] - v_arr_asso[1][a_range_cur] + 1) 
                  else sum(v_arr_asso[1][a_range_cur])
                end
              from generate_series(1, array_length(v_arr_asso, 2)) tb_a_range_cur(a_range_cur)
            )
          ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_sample_y'
    then
      if array_length(i_input_p1_len, 1) >= 1
        and i_input_p1_len[1] % v_arr_asso[1][1] = 0
      then 
        return
          (
            with 
            cte_multirange as 
            (
              select 
                int4range(1, v_arr_asso[2][1], '[]') * int4range(a_range_lower, a_range_upper + 1, '[)')
                as a_range
              from unnest(v_arr_asso[3 : 3], v_arr_asso[4 : 4]) tb_a(a_range_lower, a_range_upper)
            )
            select 
              sum 
              (
                i_input_p1_len[1] 
                / v_arr_asso[1][1]
                * (upper(a_range) - lower(a_range))
              )
            from cte_multirange
          )
          || i_input_p1_len[2 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_sample_x'
    then
      if array_length(i_input_p1_len, 1) >= 2
        and i_input_p1_len[2] % v_arr_asso[1][1] = 0
      then 
        return 
          i_input_p1_len[1 : 1] ||   
          (
            with 
            cte_multirange as 
            (
              select 
                int4range(1, v_arr_asso[2][1], '[]') * int4range(a_range_lower, a_range_upper + 1, '[)')
                as a_range
              from unnest(v_arr_asso[3 : 3], v_arr_asso[4 : 4]) tb_a(a_range_lower, a_range_upper)
            )
            select 
              sum 
              (
                i_input_p1_len[2] 
                / v_arr_asso[1][1]
                * (upper(a_range) - lower(a_range))
              )
            from cte_multirange
          )
          || i_input_p1_len[3 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_sample_x3'
    then
      if array_length(i_input_p1_len, 1) >= 3
        and i_input_p1_len[3] % v_arr_asso[1][1] = 0
      then 
        return 
          i_input_p1_len[ : 2] ||   
          (
            with 
            cte_multirange as 
            (
              select 
                int4range(1, v_arr_asso[2][1], '[]') * int4range(a_range_lower, a_range_upper + 1, '[)')
                as a_range
              from unnest(v_arr_asso[3 : 3], v_arr_asso[4 : 4]) tb_a(a_range_lower, a_range_upper)
            )
            select 
              sum 
              (
                i_input_p1_len[3] 
                / v_arr_asso[1][1]
                * (upper(a_range) - lower(a_range))
              )
            from cte_multirange
          )
          || i_input_p1_len[4 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '04_sample_x4'
    then
      if array_length(i_input_p1_len, 1) >= 4
        and i_input_p1_len[4] % v_arr_asso[1][1] = 0
      then 
        return 
          i_input_p1_len[ : 3] ||   
          (
            with 
            cte_multirange as 
            (
              select 
                int4range(1, v_arr_asso[2][1], '[]') * int4range(a_range_lower, a_range_upper + 1, '[)')
                as a_range
              from unnest(v_arr_asso[3 : 3], v_arr_asso[4 : 4]) tb_a(a_range_lower, a_range_upper)
            )
            select 
              sum 
              (
                i_input_p1_len[4] 
                / v_arr_asso[1][1]
                * (upper(a_range) - lower(a_range))
              )
            from cte_multirange
          )
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;

    else
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
      then 
        return 
          i_input_p1_len;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    end if;

  elsif i_lambda like '05_%'
  then 
    if i_lambda in ('05_pool_max_2d_grp_x', '05_pool_avg_2d_grp_x')
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p1_len_width % (v_arr_asso[1]) = 0
        and (v_arr_asso[1] + coalesce(v_arr_asso[6], 0) + coalesce(v_arr_asso[7], 0) - v_arr_asso[2]) % coalesce(v_arr_asso[4], 1) = 0
        and (v_p1_len_width / v_arr_asso[1] + coalesce(v_arr_asso[8], 0) + coalesce(v_arr_asso[9], 0) - v_arr_asso[3]) % coalesce(v_arr_asso[5], 1) = 0
      then 
        return 
          -- v_arr_asso[1]                                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
          -- v_arr_asso[2 : 3]                                           -- 规约：存放 i_window_len
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
          -- coalesce(v_arr_asso[10] :: float ,0.0              )       -- 规约：存放 i_padding_value
          v_n_ndims_len ||
          array
          [
            v_p1_len_heigh,
            ((v_p1_len_width / v_arr_asso[1] + coalesce(v_arr_asso[8], 0) + coalesce(v_arr_asso[9], 0) - v_arr_asso[3]) / coalesce(v_arr_asso[5], 1) + 1)   
            * ((v_arr_asso[1] + coalesce(v_arr_asso[6], 0) + coalesce(v_arr_asso[7], 0) - v_arr_asso[2]) / coalesce(v_arr_asso[4], 1) + 1)         
          ];
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda in ('05_pool_max', '05_pool_avg', '05_pool_none')
    then
      -- coalesce(v_arr_asso[2 : 3]        ,array[1, 1]      )       -- 规约：存放 i_window_len
      -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
      -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding   
      -- coalesce(v_arr_asso[10] :: float ,0.0              )        -- 规约：存放 i_padding_value   
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - coalesce(v_arr_asso[2], 1)) % coalesce(v_arr_asso[4], 1) = 0
        and (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - coalesce(v_arr_asso[3], 1)) % coalesce(v_arr_asso[5], 1) = 0
      then 
        if i_lambda in ('05_pool_max', '05_pool_avg')
        then 
          return 
            v_n_ndims_len ||
            array
            [
              (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - coalesce(v_arr_asso[2], 1)) / coalesce(v_arr_asso[4], 1) + 1,
              (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - coalesce(v_arr_asso[3], 1)) / coalesce(v_arr_asso[5], 1) + 1
            ]
          ;
        elsif i_lambda = '05_pool_none'
        then 
          return 
            v_n_ndims_len ||
            array
            [
              ((coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - coalesce(v_arr_asso[2], 1)) / coalesce(v_arr_asso[4], 1) + 1) * coalesce(v_arr_asso[2], 1),
              ((coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - coalesce(v_arr_asso[3], 1)) / coalesce(v_arr_asso[5], 1) + 1) * coalesce(v_arr_asso[3], 1)
            ]
          ;
        end if;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
          
    elsif i_lambda = '05_conv_2d_grp_x'
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh = 1 and v_p2_len_width > 0
        and v_arr_asso[2] * v_arr_asso[3] in (v_p2_len_width - 1, v_p2_len_width)
        and v_p1_len_width % (v_arr_asso[1]) = 0
        and (v_arr_asso[1] + coalesce(v_arr_asso[6], 0) + coalesce(v_arr_asso[7], 0) - v_arr_asso[2]) % coalesce(v_arr_asso[4], 1) = 0
        and (v_p1_len_width / v_arr_asso[1] + coalesce(v_arr_asso[8], 0) + coalesce(v_arr_asso[9], 0) - v_arr_asso[3]) % coalesce(v_arr_asso[5], 1) = 0
      then 
        return
          -- v_arr_asso[1]                                               -- 规约：存放 i_1d_2_2d_cnt_per_grp
          -- v_arr_asso[2 : 3]                                           -- 规约：存放 i_window_len
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
          -- coalesce(v_arr_asso[10] :: float ,0.0              )        -- 规约：存放 i_padding_value
          v_n_ndims_len ||
          array
          [
            v_p1_len_heigh,
            ((v_p1_len_width / v_arr_asso[1] + coalesce(v_arr_asso[8], 0) + coalesce(v_arr_asso[9], 0) - coalesce(v_arr_asso[3], v_p2_len_width / v_arr_asso[2])) / coalesce(v_arr_asso[5], 1) + 1)      
            * ((v_arr_asso[1] + coalesce(v_arr_asso[6], 0) + coalesce(v_arr_asso[7], 0) - coalesce(v_arr_asso[2], v_p2_len_heigh / v_arr_asso[3])) / coalesce(v_arr_asso[4], 1) + 1)     
          ];
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '05_conv_2d'
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh > 0 and v_p2_len_width > 0
        and (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p2_len_heigh) % coalesce(v_arr_asso[4], 1) = 0
        and (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_width) % coalesce(v_arr_asso[5], 1) = 0
        and (i_input_p3_len is null or v_p3_ndims = v_p2_ndims and v_p3_len_heigh = 1 and v_p3_len_width = 1)
        and (
              i_input_p3_len is null 
             or v_p3_ndims = 2 
             or v_p3_ndims = 3 and i_input_p3_len[1] = i_input_p2_len[1] 
             or v_p3_ndims = 4 and i_input_p3_len[1] = i_input_p2_len[1] and i_input_p3_len[2] = i_input_p2_len[2]
            )
      then 
        return
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
          -- coalesce(v_arr_asso[10] :: float ,0.0              )        -- 规约：存放 i_padding_value
          v_n_ndims_len ||
          array
          [
            (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p2_len_heigh) / coalesce(v_arr_asso[4], 1) + 1,
            (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_width) / coalesce(v_arr_asso[5], 1) + 1     
          ];
      else
        raise exception 'unmatch p1, p2, p3.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_p3_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, i_input_p3_len, v_arr_asso;
      end if;
    
    elsif i_lambda in ('05_conv_add', '05_conv_sub', '05_conv_mul', '05_conv_div', '05_conv_pow', '05_conv_log')
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh > 0 and v_p2_len_width > 0
        and (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p2_len_heigh) % coalesce(v_arr_asso[4], 1) = 0
        and (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_width) % coalesce(v_arr_asso[5], 1) = 0
      then 
        return
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
          -- coalesce(v_arr_asso[10] :: float ,0.0              )        -- 规约：存放 i_padding_value
          v_n_ndims_len ||
          array
          [
            ((coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p2_len_heigh) / coalesce(v_arr_asso[4], 1) + 1) * v_p2_len_heigh,
            ((coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_width) / coalesce(v_arr_asso[5], 1) + 1) * v_p2_len_width
          ];
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda in ('05_conv_de_sub', '05_conv_de_div', '05_conv_de_pow', '05_conv_de_log')
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh > 0 and v_p2_len_width > 0
        and (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p2_len_heigh) % coalesce(v_arr_asso[4], 1) = 0
        and (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_width) % coalesce(v_arr_asso[5], 1) = 0
      then 
        return
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
          -- coalesce(v_arr_asso[10] :: float ,0.0              )        -- 规约：存放 i_padding_value
          v_n_ndims_len ||
          array
          [
            ((coalesce(v_arr_asso[6], 0) + v_p2_len_heigh + coalesce(v_arr_asso[7], 0) - v_p1_len_heigh) / coalesce(v_arr_asso[4], 1) + 1) * v_p1_len_heigh,
            ((coalesce(v_arr_asso[8], 0) + v_p2_len_width + coalesce(v_arr_asso[9], 0) - v_p1_len_width) / coalesce(v_arr_asso[5], 1) + 1) * v_p1_len_width
          ];
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p2_len: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p2_len, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '05_conv_prod_mx'
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh > 0 and v_p2_len_width > 0
        and (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_arr_asso[2]) % coalesce(v_arr_asso[4], 1) = 0
        and (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_heigh) % coalesce(v_arr_asso[5], 1) = 0
      then 
        return
          -- coalesce(v_arr_asso[2 : 2]        ,array[1, 1]      )       -- 规约：存放 i_window_len_heigh
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding     
          v_n_ndims_len ||
          array
          [
            ((coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_arr_asso[2]) / coalesce(v_arr_asso[4], 1) + 1) * v_arr_asso[2],
            ((coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_p2_len_width) / coalesce(v_arr_asso[5], 1) + 1) * v_p2_len_width
          ]
        ;
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '05_conv_de_prod_mx'
    then
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
        and v_p2_len_heigh > 0 and v_p2_len_width > 0
        and (coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p1_len_width) % coalesce(v_arr_asso[4], 1) = 0
        and (coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_arr_asso[3]) % coalesce(v_arr_asso[5], 1) = 0
      then 
        return
          -- coalesce(v_arr_asso[3 : 3]        ,array[1, 1]      )       -- 规约：存放 i_window_len_width
          -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
          -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding     
          v_n_ndims_len ||
          array
          [
            ((coalesce(v_arr_asso[6], 0) + v_p1_len_heigh + coalesce(v_arr_asso[7], 0) - v_p1_len_width) / coalesce(v_arr_asso[4], 1) + 1) * v_p1_len_width,
            ((coalesce(v_arr_asso[8], 0) + v_p1_len_width + coalesce(v_arr_asso[9], 0) - v_arr_asso[3]) / coalesce(v_arr_asso[5], 1) + 1) * v_arr_asso[3]
          ]
        ;
      else
        raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p2_len: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p2_len, i_input_p1_len, v_arr_asso;
      end if;

    else
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
      then 
        return 
          i_input_p1_len;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
      
    end if;

  elsif i_lambda like '06_%'
  then 
    if i_lambda = '06_aggr_mx_concat_y'
    then
      if array_length(i_input_p1_len, 1) >= 1 -- or array_length(i_input_p2_len, 1) >= 1
      then 
        return 
          sm_sc.fv_aggr_slice_sum(v_arr_asso[1 : ]) || i_input_p1_len[2 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '06_aggr_mx_concat_x'
    then
      if array_length(i_input_p1_len, 2) >= 1 or array_length(i_input_p1_len, 1) >= 1
      then 
        return 
          i_input_p1_len[1 : 1] || sm_sc.fv_aggr_slice_sum(v_arr_asso[1 : ]) || i_input_p1_len[3 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '06_aggr_mx_concat_x3'
    then
      if array_length(i_input_p1_len, 3) >= 1 or array_length(i_input_p1_len, 1) >= 1
      then 
        return 
          i_input_p1_len[ : 2] || sm_sc.fv_aggr_slice_sum(v_arr_asso[1 : ]) || i_input_p1_len[4 : ]
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda = '06_aggr_mx_concat_x4'
    then
      if array_length(i_input_p1_len, 4) >= 1 or array_length(i_input_p1_len, 1) >= 1
      then 
        return 
          i_input_p1_len[ : 3] || sm_sc.fv_aggr_slice_sum(v_arr_asso[1 : ])
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    else
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
      then 
        return 
          i_input_p1_len;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    end if;

  elsif i_lambda like '07_%'
  then 
    if i_lambda in ('07_aggr_slice_sum', '07_aggr_slice_avg', '07_aggr_slice_max', '07_aggr_slice_min')
    then
      v_arr_asso := 
        sm_sc.fv_coalesce
        (
          i_input_p1_len[ : array_length(i_input_p1_len, 1) - coalesce(array_length(v_arr_asso, 1), 0)] || v_arr_asso
        , i_input_p1_len
        )
      ;
      
      if array_length(i_input_p1_len, 1) >= array_length(v_arr_asso, 1)
      then 
        return 
          i_input_p1_len / v_arr_asso
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    
    elsif i_lambda in ('07_aggr_chunk_sum', '07_aggr_chunk_avg', '07_aggr_chunk_max', '07_aggr_chunk_min')
    then
      v_arr_asso := 
        sm_sc.fv_coalesce
        (
          -- sm_sc.fv_lpad
          -- (
          --   v_arr_asso
          -- , array[null :: int]
          -- , array_length(i_input_p1_len, 1) - array_length(v_arr_asso, 1)
          -- )
          i_input_p1_len[ : array_length(i_input_p1_len, 1) - coalesce(array_length(v_arr_asso, 1), 0)] || v_arr_asso
        , i_input_p1_len
        )
      ;
      
      if array_length(i_input_p1_len, 1) >= array_length(v_arr_asso, 1)
      then 
        return 
          v_arr_asso
        ;
      else 
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;

    else
      if v_p1_len_heigh > 0 and v_p1_len_width > 0
      then 
        return 
          i_input_p1_len;
      else
        raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
      end if;
    end if;
    
  elsif i_lambda like '81_%'
  then 
    return 
      sm_sc.ufv_lambda_arr_len
      (
        i_node_no       
      , i_lambda        
      , i_input_p1_len  
      , i_input_p2_len  
      , i_input_arr_asso
      , i_input_p3_len  
      )
    ;
      
  else 
    if v_p1_len_heigh > 0 and v_p1_len_width > 0
    then 
      return 
        i_input_p1_len;
    else
      raise exception 'unmatch p1.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, v_arr_asso;
    end if;

  end if;

  exception when others then
    raise exception 
    ' fn: sm_sc.fv_lambda_arr_len
      i_node_no: %
      i_lambda: %
      i_input_p1_len: %
      i_input_p2_len: %
      v_arr_asso: %
      sqlerrm: %
    '
    , i_node_no
    , i_lambda
    , i_input_p1_len
    , i_input_p2_len
    , v_arr_asso
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;



-- select 
--   sm_sc.fv_lambda_arr_len
--   (
--     null
--     '05_conv_2d_grp_x',                       -- i_lambda              
--     array[70, 784],                           -- i_input_p1_len
--     array[1, 25],                             -- i_input_p2_len        
--     array[28, 5, 5, 1, 1, 2, 2, 2, 2, 0]      -- v_arr_asso 
--   )

-- -- select 
-- --         array
-- --         [
-- --           v_p1_len_heigh,
-- --           (v_p1_len_width / v_arr_asso[1] + coalesce(v_arr_asso[8], 0) + coalesce(v_arr_asso[9], 0) - coalesce(v_arr_asso[3], v_p2_len_width)) / coalesce(v_arr_asso[5], 1)       
-- --           * (v_arr_asso[1] + coalesce(v_arr_asso[6], 0) + coalesce(v_arr_asso[7], 0) - coalesce(v_arr_asso[2], v_p2_len_heigh)) / coalesce(v_arr_asso[4], 1)     
-- --         ]
-- -- 
-- --         -- v_arr_asso[1]                                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
-- --         -- v_arr_asso[2 : 3]                                           -- 规约：存放 i_window_len
-- --         -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
-- --         -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
-- --         -- coalesce(v_arr_asso[10] :: float ,0.0              )       -- 规约：存放 i_padding_value
-- drop function if exists sm_sc.fv_lambda_arr_p(char(6), bigint, bigint, bigint, varchar(64), varchar(64), varchar(64), float[], varchar(64));
create or replace function sm_sc.fv_lambda_arr_p
(
  i_model_code_6          char(6)
, i_work_no               bigint
, i_node_no               bigint
, i_sess_id               bigint
, i_lambda                varchar(64)                 
, i_param_1_p             varchar(64)                 
, i_param_2_p             varchar(64)   default null        -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立
, i_input_arr_asso        float[]       default null     
, i_param_3_p             varchar(64)   default null
)
returns varchar(64)
as
$$
declare 
  -- -- v_lambda varchar(64)    :=    coalesce((select 'v...' from sm_sc.tb_dic... where col...'k...' = i_lambda limit 1), i_lambda)
  v_param_1               float[] := sm_sc.__fv_get_kv(i_param_1_p);
  v_param_2               float[] := sm_sc.__fv_get_kv(i_param_2_p);
  v_param_3               float[] := sm_sc.__fv_get_kv(i_param_3_p);
  v_ret                   float[];
 
 -- debug 
 v_begin_clock    timestamp     := clock_timestamp();
begin
  -- 动态 sql 会影响性能，所以还是用 case 判断
  
-- -- debug 
-- raise notice 'lambda % begin:: i_lambda: %; len_input: %; len_co_val: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(v_param_1, 1), array_length(v_param_1, 2)], array[array_length(v_param_2, 1), array_length(v_param_2, 2)], to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if v_param_1 is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(v_param_1) is true
    then 
      raise exception 'there is null value in v_param_1!  i_node_no: %.  ', i_node_no;
    elsif 'NaN' :: float = any(v_param_1)
      or 'NaN' :: float = any(v_param_2)
      or v_param_1 is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(v_param_1) is true
    then 
      raise exception 'there is NaN value in v_param_1 or v_param_2!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  if i_lambda like '07_%'
  then 
    if i_lambda = '07_aggr_slice_sum'
    then 
      v_ret := 
        sm_sc.fv_aggr_slice_sum(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_prod'
    then
      v_ret := 
        sm_sc.fv_aggr_slice_prod(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_avg'
    then
      v_ret := 
        sm_sc.fv_aggr_slice_avg(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_max'
    then
      v_ret := 
        sm_sc.fv_aggr_slice_max(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_slice_min'
    then
      v_ret := 
        sm_sc.fv_aggr_slice_min(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_sum'
    then 
      v_ret := 
        sm_sc.fv_aggr_chunk_sum(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_prod'
    then
      v_ret := 
        sm_sc.fv_aggr_chunk_prod(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_avg'
    then
      v_ret := 
        sm_sc.fv_aggr_chunk_avg(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_max'
    then
      v_ret := 
        sm_sc.fv_aggr_chunk_max(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '07_aggr_chunk_min'
    then
      v_ret := 
        sm_sc.fv_aggr_chunk_min(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    end if;
    
  -- -- elsif i_lambda like '06_%'
  -- -- then 
  -- --   if i_lambda = '06_aggr_mx_sum'
  -- --   then 
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_prod'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_avg'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_max'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_min'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_y'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_x'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_x3'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   elsif i_lambda = '06_aggr_mx_concat_x4'
  -- --   then
  -- --     v_ret := 
  -- --       
  -- --     ;
  -- --   end if;
    
  elsif i_lambda like '05_%'
  then 
    if i_lambda = '05_pool_max_2d_grp_x'
    then 
      v_ret := 
        sm_sc.fv_pool_max_2d_grp_x
        (
          v_param_1                                                           ,   
          i_input_arr_asso[1] :: int                                              ,   -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg_2d_grp_x'
    then
      v_ret := 
        sm_sc.fv_pool_avg_2d_grp_x
        (
          v_param_1                                                           ,   
          i_input_arr_asso[1] :: int                                              ,   -- 规约：存放 i_1d_2_2d_cnt_per_grp
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_max'
    then
      v_ret := 
        sm_sc.fv_pool_max_py
        (
          v_param_1                                                           ,   
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_avg'
    then
      v_ret := 
        sm_sc.fv_pool_avg_py
        (
          v_param_1                                                           ,   
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_pool_none'
    then
      v_ret := 
        sm_sc.fv_pool_none_py
        (
          v_param_1                                                           ,   
          i_input_arr_asso[2 : 3] :: int[]                                        ,   -- 规约：存放 i_window_len
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_2d_grp_x'
    then
      v_ret := 
        sm_sc.fv_conv_2d_grp_x
        (
          v_param_1   :: float[]                                     ,   
          i_input_arr_asso[1] :: int                                              ,   -- 规约：存放 i_1d_2_2d_cnt_per_grp
          v_param_2           :: float[]                                     ,   
          i_input_arr_asso[3] :: int                                              ,   -- 规约：存放 i_window_len_x 
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_2d'
    then
      v_ret := 
        sm_sc.fv_conv_2d_im2col_py
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          v_param_3                                              ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_add'
    then
      v_ret := 
        sm_sc.fv_conv_add
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_sub'
    then
      v_ret := 
        sm_sc.fv_conv_sub
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_mul'
    then
      v_ret := 
        sm_sc.fv_conv_mul
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_div'
    then
      v_ret := 
        sm_sc.fv_conv_div
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_pow'
    then
      v_ret := 
        sm_sc.fv_conv_pow
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_log'
    then
      v_ret := 
        sm_sc.fv_conv_log
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_prod_mx'
    then
      v_ret := 
        sm_sc.fv_conv_prod_mx
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          i_input_arr_asso[2] :: int                                        ,   -- 规约：存放 i_window_len_heigh
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_sub'
    then
      v_ret := 
        sm_sc.fv_conv_de_sub
        (
          v_param_1   :: float[]                                     ,   -- 窗口
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_div'
    then
      v_ret := 
        sm_sc.fv_conv_de_div
        (
          v_param_1   :: float[]                                     ,   -- 窗口
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_pow'
    then
      v_ret := 
        sm_sc.fv_conv_de_pow
        (
          v_param_1   :: float[]                                     ,   -- 窗口
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_log'
    then
      v_ret := 
        sm_sc.fv_conv_de_log
        (
          v_param_1   :: float[]                                     ,   -- 窗口
          v_param_2           :: float[]                                     ,   
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    elsif i_lambda = '05_conv_de_prod_mx'
    then
      v_ret := 
        sm_sc.fv_conv_de_prod_mx
        (
          v_param_1   :: float[]                                     ,   
          v_param_2           :: float[]                                     ,   
          i_input_arr_asso[3] :: int                                        ,   -- 规约：存放 i_window_len_width
          coalesce(i_input_arr_asso[4 : 5] :: int[]       ,array[1, 1]      )     ,   -- 规约：存放 i_stride       
          coalesce(i_input_arr_asso[6 : 9] :: int[]       ,array[0, 0, 0, 0])     ,   -- 规约：存放 i_padding      
          coalesce(i_input_arr_asso[10] :: float ,0.0              )         -- 规约：存放 i_padding_value
        )
      ;
    end if;
    
  elsif i_lambda like '04_%'
  then 
    if i_lambda = '04_new'
    then 
      v_ret := 
        sm_sc.fv_new(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '04_repeat_axis'
    then
      v_ret := 
        sm_sc.fv_repeat_axis_py(v_param_1, sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][ : ]) :: int[], sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][ : ]) :: int[])
      ;
    elsif i_lambda = '04_apad'
    then
      v_ret := 
        sm_sc.fv_apad(v_param_1, v_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_bpad'
    then
      v_ret := 
        sm_sc.fv_bpad(v_param_1, v_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_lpad'
    then
      v_ret := 
        sm_sc.fv_lpad(v_param_1, v_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_rpad'
    then
      v_ret := 
        sm_sc.fv_lpad(v_param_1, v_param_2, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_transpose'
    then
      v_ret := 
        sm_sc.fv_opr_transpose_py(v_param_1)
      ;
    elsif i_lambda = '04_transpose_i'
    then
      v_ret := 
        sm_sc.fv_opr_transpose_i_py(v_param_1)
      ;
    elsif i_lambda = '04_chunk_transpose'
    then
      v_ret := 
        sm_sc.fv_chunk_transpose(v_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_transpose_nd'
    then
      v_ret := 
        sm_sc.fv_opr_transpose_nd_py(v_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_90'
    then
      v_ret := 
        sm_sc.fv_opr_turn_90_py(v_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_turn_180'
    then
      v_ret := 
        sm_sc.fv_opr_turn_180_py(v_param_1, i_input_arr_asso[1 : 2] :: int[])
      ;
    elsif i_lambda = '04_mirror'
    then
      v_ret := 
        sm_sc.fv_opr_mirror_py(v_param_1, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_ele_3d_2_2d'
    then
      v_ret := 
        sm_sc.fv_mx_ele_3d_2_2d_py
        (
          v_param_1
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        )
      ;
    elsif i_lambda = '04_mx_ele_2d_2_3d'
    then
      v_ret := 
        sm_sc.fv_mx_ele_2d_2_3d_py
        (
          v_param_1
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int
        , i_input_arr_asso[4] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_4d_2_3d'
    then
      v_ret := 
        sm_sc.fv_mx_ele_4d_2_3d_py
        (
          v_param_1
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        )
      ;
    elsif i_lambda = '04_mx_ele_3d_2_4d'
    then
      v_ret := 
        sm_sc.fv_mx_ele_3d_2_4d_py
        (
          v_param_1
        , i_input_arr_asso[1] :: int
        , i_input_arr_asso[2] :: int
        , i_input_arr_asso[3] :: int
        , i_input_arr_asso[4] :: int :: boolean
        )
      ;
    elsif i_lambda = '04_mx_ele_flatten_2dims'
    then
      v_ret := 
        sm_sc.fv_mx_ele_flatten_2dims_py
        (
          v_param_1
        , i_input_arr_asso[1 : 2] :: int[]
        , i_input_arr_asso[3] :: int
        )
      ;
    elsif i_lambda = '04_mx_slice_3d_2_2d'
    then
      v_ret := 
        sm_sc.fv_mx_slice_3d_2_2d_py(v_param_1, i_input_arr_asso[1] :: int, i_input_arr_asso[2] :: int)
      ;
    elsif i_lambda = '04_mx_slice_4d_2_2d'
    then
      v_ret := 
        sm_sc.fv_mx_slice_4d_2_2d_py(v_param_1, sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[1 : 1][1 : 2]) :: int[], sm_sc.fv_mx_ele_2d_2_1d(i_input_arr_asso[2 : 2][1 : 2]) :: int[])
      ;
    elsif i_lambda = '04_mx_slice_4d_2_3d'
    then
      v_ret := 
        sm_sc.fv_mx_slice_4d_2_3d_py(v_param_1, i_input_arr_asso[1] :: int, i_input_arr_asso[2] :: int)
      ;
    elsif i_lambda = '04_mx_ascend_dim'
    then
      v_ret := 
        sm_sc.fv_mx_ascend_dim(v_param_1, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_mx_descend_dim'
    then
      v_ret := 
        sm_sc.fv_mx_descend_dim_py(v_param_1, i_input_arr_asso[1] :: int)
      ;
    elsif i_lambda = '04_rand_pick_y'
    then
      v_ret := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_y_pick(v_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_rand_pick_x'
    then
      v_ret := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_x_pick(v_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_rand_pick_x3'
    then
      v_ret := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_x3_pick(v_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_rand_pick_x4'
    then
      v_ret := 
      (
        select 
          o_slices
        from sm_sc.ft_rand_slice_x4_pick(v_param_1, i_input_arr_asso[1] :: int)
      )
      ;
    elsif i_lambda = '04_chunk'
    then
      v_ret := 
        sm_sc.fv_chunk
        (
          v_param_1
        , i_input_arr_asso :: int[]
        )
      ;
    elsif i_lambda = '04_slice_y'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_slice_y
          (
            v_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_slice_x
          (
            v_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x3'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_slice_x3
          (
            v_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_slice_x4'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_slice_x4
          (
            v_param_1
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          )
        from unnest(i_input_arr_asso[1 : 1][ : ], i_input_arr_asso[2 : 2][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_y'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_sample_y    -- fv_sample_y_py
          (
            v_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(v_param_2, case array_ndims(v_param_1) when 1 then array[0.0] when 2 then array[[0.0]] when 3 then array[[[0.0]]] when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_sample_x    -- fv_sample_x_py
          (
            v_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(v_param_2, case array_ndims(v_param_1) when 2 then array[[0.0]] when 3 then array[[[0.0]]] when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x3'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_sample_x3   -- fv_sample_x3_py
          (
            v_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(v_param_2, case array_ndims(v_param_1) when 3 then array[[[0.0]]] when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_sample_x4'
    then
      v_ret := 
      (
        select 
          sm_sc.fv_sample_x4      -- fv_sample_x4_py
          (
            v_param_1
          , i_input_arr_asso[1][1] :: int
          , i_input_arr_asso[2][1] :: int
          , array_agg(int4range(a_range_lower :: int, a_range_upper :: int + 1, '[)'))
          , coalesce(v_param_2, case array_ndims(v_param_1) when 4 then array[[[[0.0]]]] end)
          )
        from unnest(i_input_arr_asso[3 : 3][ : ], i_input_arr_asso[4 : 4][ : ]) tb_a_range(a_range_lower, a_range_upper)
      )
      ;
    elsif i_lambda = '04_lower_tri_mx'
    then
      v_ret := sm_sc.fv_lower_tri_mx(v_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_upper_tri_mx'
    then
      v_ret := sm_sc.fv_upper_tri_mx(v_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_lower_tri_mx_ex'
    then
      v_ret := sm_sc.fv_lower_tri_mx_ex(v_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_upper_tri_mx_ex'
    then
      v_ret := sm_sc.fv_upper_tri_mx_ex(v_param_1, i_input_arr_asso[1]);
    elsif i_lambda = '04_lmask'
    then
      v_ret := sm_sc.fv_lmask(v_param_1, v_param_2 :: int[], i_input_arr_asso[1]);
    elsif i_lambda = '04_rmask'
    then
      v_ret := sm_sc.fv_rmask(v_param_1, v_param_2 :: int[], i_input_arr_asso[1]);
    elsif i_lambda = '04_amask'
    then
      v_ret := sm_sc.fv_amask(v_param_1, v_param_2 :: int[], i_input_arr_asso[1]);
    elsif i_lambda = '04_bmask'
    then
      v_ret := sm_sc.fv_bmask(v_param_1, v_param_2 :: int[], i_input_arr_asso[1]);
    end if;
    
  elsif i_lambda like '03_%'
  then 
    if i_lambda = '03_sigmoid'
    then 
      v_ret := 
        sm_sc.fv_activate_sigmoid(v_param_1)
      ;
    elsif i_lambda = '03_relu'
    then
      v_ret := 
        sm_sc.fv_activate_relu(v_param_1)
      ;
    elsif i_lambda = '03_leaky_relu'
    then
      v_ret := 
        sm_sc.fv_activate_leaky_relu(v_param_1, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_elu'
    then
      v_ret := 
        sm_sc.fv_activate_elu(v_param_1, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_selu'
    then
      v_ret := 
        sm_sc.fv_activate_selu(v_param_1)
      ;
    elsif i_lambda = '03_gelu'
    then
      v_ret := 
        sm_sc.fv_activate_gelu(v_param_1)
      ;
    elsif i_lambda = '03_swish'
    then
      v_ret := 
        sm_sc.fv_activate_swish(v_param_1)
      ;
    elsif i_lambda = '03_softplus'
    then
      v_ret := 
        sm_sc.fv_activate_softplus(v_param_1)
      ;
    elsif i_lambda = '03_boxcox'
    then
      v_ret := 
        sm_sc.fv_activate_boxcox(v_param_1, i_input_arr_asso[1])
      ;
    elsif i_lambda = '03_softmax'
    then
      v_ret := 
        sm_sc.fv_redistr_softmax(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '03_softmax_ex'
    then
      v_ret := 
        sm_sc.fv_redistr_softmax_ex(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    elsif i_lambda = '03_zscore'
    then
      v_ret := 
        sm_sc.fv_redistr_zscore(v_param_1, i_input_arr_asso[1 : 4] :: int[])
      ;
    end if;
    
  elsif i_lambda like '02_%'
  then 
    if i_lambda = '02_sin'
    then
      v_ret := 
        sm_sc.fv_sin(v_param_1)
      ;
    elsif i_lambda = '02_cos'
    then
      v_ret := 
        sm_sc.fv_cos(v_param_1)
      ;
    elsif i_lambda = '02_tan'
    then
      v_ret := 
        sm_sc.fv_tan(v_param_1)
      ;
    elsif i_lambda = '02_cot'
    then
      v_ret := 
        sm_sc.fv_cot(v_param_1)
      ;
    elsif i_lambda = '02_sec'
    then
      v_ret := 
        sm_sc.fv_sec(v_param_1)
      ;
    elsif i_lambda = '02_csc'
    then
      v_ret := 
        sm_sc.fv_csc(v_param_1)
      ;
    elsif i_lambda = '02_asin'
    then
      v_ret := 
        sm_sc.fv_asin(v_param_1)
      ;
    elsif i_lambda = '02_acos'
    then
      v_ret := 
        sm_sc.fv_acos(v_param_1)
      ;
    elsif i_lambda = '02_atan'
    then
      v_ret := 
        sm_sc.fv_atan(v_param_1)
      ;
    elsif i_lambda = '02_acot'
    then
      v_ret := 
        sm_sc.fv_acot(v_param_1)
      ;
    elsif i_lambda = '02_sinh'
    then
      v_ret := 
        sm_sc.fv_sinh(v_param_1)
      ;
    elsif i_lambda = '02_cosh'
    then
      v_ret := 
        sm_sc.fv_cosh(v_param_1)
      ;
    elsif i_lambda = '02_tanh'
    then
      v_ret := 
        sm_sc.fv_tanh(v_param_1)
      ;
    -- -- elsif i_lambda = '02_sech'
    -- -- then
    -- --   v_ret := 
    -- --     sm_sc.fv_sech(v_param_1)
    -- --   ;
    -- -- elsif i_lambda = '02_csch'
    -- -- then
    -- --   v_ret := 
    -- --     sm_sc.fv_csch(v_param_1)
    -- --   ;
    elsif i_lambda = '02_asinh'
    then
      v_ret := 
        sm_sc.fv_asinh(v_param_1)
      ;
    elsif i_lambda = '02_acosh'
    then
      v_ret := 
        sm_sc.fv_acosh(v_param_1)
      ;
    elsif i_lambda = '02_atanh'
    then
      v_ret := 
        sm_sc.fv_atanh(v_param_1)
      ;
    end if;
    
  elsif i_lambda like '01_%'
  then 
    if i_lambda = '01_add'
    then 
      v_ret := 
        sm_sc.fv_opr_add_py(v_param_1, v_param_2)
      ;
    elsif i_lambda = '01_mul'
    then
      v_ret := 
        sm_sc.fv_opr_mul_py(v_param_1, v_param_2)
      ;
    elsif i_lambda = '01_sub'
    then
      v_ret := 
        sm_sc.fv_opr_sub_py(v_param_1, v_param_2)
      ;
    elsif i_lambda = '01_0sub'
    then
      v_ret := 
        sm_sc.fv_opr_sub_py(v_param_1)
      ;
    elsif i_lambda = '01_div'
    then
      v_ret := 
        sm_sc.fv_opr_div_py(v_param_1, v_param_2)
      ;
    elsif i_lambda = '01_1div'
    then
      v_ret := 
        sm_sc.fv_opr_div_py(v_param_1)
      ;
    elsif i_lambda = '01_pow'
    then
      v_ret := 
        sm_sc.fv_opr_pow_py(v_param_1, v_param_2)
      ;
    elsif i_lambda = '01_log'
    then
      v_ret := 
        sm_sc.fv_opr_log_py(v_param_1, v_param_2)
      ;
    elsif i_lambda = '01_exp'
    then
      v_ret := 
        sm_sc.fv_opr_exp_py(v_param_1)
      ;
    elsif i_lambda = '01_ln'
    then
      v_ret := 
        sm_sc.fv_opr_ln_py(v_param_1)
      ;
    elsif i_lambda = '01_abs'
    then
      v_ret := 
        sm_sc.fv_opr_abs(v_param_1)
      ;
    elsif i_lambda = '01_prod_mx'
    then
      v_ret := 
        -- sm_sc.fv_opr_prod_mx_py(v_param_1, v_param_2)
        v_param_1 |**| v_param_2
      ;
    elsif i_lambda = '01_chunk_prod_mx'
    then
      v_ret := 
        sm_sc.fv_chunk_prod_mx(v_param_1, v_param_2,  i_input_arr_asso[1 : 3] :: int[])
      ;
    end if;
    
  elsif i_lambda like '00_%'
  then 
  
    if i_lambda = '00_none'
    then
      v_ret := 
        sm_sc.fv_nn_none(v_param_1)
      ;

    -- -- elsif i_lambda = '00_const'
    -- -- then 
    -- --   return 
    -- --     -- (select node_depdt_vals from sm_sc.tb_nn_node where node_no = i_node_no and work_no = i_work_no)
    -- --   ;

    -- -- -- 出于减少数据拷贝、内存交换、资源开销的优化目的，nn input 节点的数据准备不在 lambda 执行。
    -- -- elsif i_lambda = '00_full_dataset'
    -- -- then 
    -- --   return 
    -- --   (
    -- --     select 
    -- --       array_agg(i_indepdt order by ord_no)
    -- --     from sm_sc.tb_nn_train_input_buff 
    -- --     where work_no = (select work_no from sm_sc.tb_nn_node where node_no = i_node_no)
    -- --   );
    -- -- 
    -- -- elsif i_lambda = '00_buff_slice_rand_pick'
    -- -- then 
    -- --   return 
    -- --     -- sm_sc.ft_nn_buff_slice_rand_pick()
    -- --   ;
    
    end if;
    
  elsif i_lambda like '81_%'
  then 
    return 
      sm_sc.__fv_set_kv
      (
        sm_sc.ufv_lambda_arr
        (
          i_node_no       
        , i_lambda        
        , v_param_1       
        , v_param_2       
        , i_input_arr_asso
        , v_param_3       
        )
      )
    ;
    
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if v_ret is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(v_ret) is true
    then 
      raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
    elsif 'NaN' :: float = any(v_ret)
    then 
      raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
    end if;
  end if;

-- -- debug 
-- raise notice 'independent_var:i_node_no: %; i_lambda: %; time: %;', i_node_no, i_lambda, v_begin_clock - clock_timestamp();
  
  return 
    sm_sc.__fv_set_kv
    (
      v_ret
    , i_model_code_6 || 
      '_' || i_work_no :: varchar ||
      '_' || i_node_no :: varchar ||
      '__d__' || -- 'depdt'
      coalesce(i_sess_id :: varchar, '')
    )
  ;

-- -- debug 
-- raise notice 'd:lambda % end:: i_lambda: %; time: %;', i_node_no, i_lambda, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return v_ret;

  exception when others then
    raise exception 
    ' fn: sm_sc.fv_lambda_arr_p
      i_node_no: %
      i_lambda: %
      len of v_param_1: %
      len of v_param_2: %
      i_input_arr_asso: %
      sqlerrm: %
    '
    , i_node_no
    , i_lambda
    , array_dims(v_param_1)
    , array_dims(v_param_2)
    , i_input_arr_asso
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_lambda_arr_p
--   (
--     100001
--     , '03_elu'
--     , array[[1.23, 3.34],[-7.2, -0.25]]
--     , null
--     , array[0.1]
--   );






-- drop function if exists sm_sc.fv_lambda_dloss_dz(varchar(32), float[], float[]);
create or replace function sm_sc.fv_lambda_dloss_dz
(
  i_loss_fn_type    varchar(32)                     -- 损失函数类型说明，参看字典表 enum_name = 'loss_fn_type'
, i_nn_depdt        float[]                         -- 训练输出预测值
, i_true            float[]                         -- 训练集真实值
, i_node_type       varchar(64)                     -- output_01, output_02, output_03, output_04, ...
)
returns float[]
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_nn_depdt) > 4 
      or array_ndims(i_true) > 4 
      or array_length(i_nn_depdt, 1) <> array_length(i_true, 1) 
      or array_length(i_nn_depdt, 2) <> array_length(i_true, 2)
      or array_length(i_nn_depdt, 3) <> array_length(i_true, 3)
      or array_length(i_nn_depdt, 4) <> array_length(i_true, 4)
    then
      raise exception 'no method for such length!  Z_Dims: %; True_Dims: %;', array_dims(i_nn_depdt), array_dims(i_true);
    end if;
  end if;

  if i_loss_fn_type = '101'
  then 
    return 
      sm_sc.fv_dloss_dz_least_square(i_nn_depdt, i_true)
    ;
  elsif i_loss_fn_type = '201'
  then 
    return 
      sm_sc.fv_dloss_dz_cross_entropy(i_nn_depdt, i_true)
    ;
  elsif i_loss_fn_type = '202'
  then 
    return 
      sm_sc.fv_dloss_dz_cross_entropy(i_nn_depdt, i_true) /` array_length(i_nn_depdt, 2) :: float
    ;
  elsif i_loss_fn_type = '203'
  then 
    if i_node_type = 'output_01'
    then 
      return 
        sm_sc.fv_dloss_dz_cross_entropy_true_onehot_idx(i_nn_depdt, i_true)
      ;
    else 
      return null; 
    end if;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_lambda_dloss_dz('101', array[array[1, 2, 3], array[2, 3, 4]], array[array[1.1, 2.1, 3.2], array[2.2, 3.3, 4.2]])
-- 参考 https://zhuanlan.zhihu.com/p/35709485

-- drop function if exists sm_sc.fv_lambda_loss(varchar(32), float[], float[], float[], float[], float[], float[], float[], float[]);
create or replace function sm_sc.fv_lambda_loss
(
  i_loss_fn_type    varchar(32)                                       -- 损失函数类型说明，参看字典表 enum_name = 'loss_fn_type'
, i_nn_depdt_01     float[]                                           -- 训练输出预测值 01
, i_true_01         float[]                                           -- 训练集真实值 01
, i_nn_depdt_02     float[]          default null                     -- 训练输出预测值 01
, i_true_02         float[]          default null                     -- 训练集真实值 01
, i_nn_depdt_03     float[]          default null                     -- 训练输出预测值 01
, i_true_03         float[]          default null                     -- 训练集真实值 01
, i_nn_depdt_04     float[]          default null                     -- 训练输出预测值 01
, i_true_04         float[]          default null                     -- 训练集真实值 01
)
returns float
as
$$
-- declare
  
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_nn_depdt_01) > 4 
      or array_ndims(i_true_01) > 4 
      or array_dims(i_nn_depdt_01, 1) <> array_dims(i_true_01, 1) 
    then
      raise exception 'no method for such length!  Y_Dims: %; Z_Dims: %;', array_dims(i_nn_depdt_01), array_dims(i_true_01);
    end if;
  end if;
  
  if i_loss_fn_type = '101'
  then 
    return 
      sm_sc.fv_loss_least_square(i_nn_depdt_01, i_true_01)
    ;
  elsif i_loss_fn_type = '201'
  then 
    return 
      sm_sc.fv_loss_cross_entropy(i_nn_depdt_01, i_true_01)
    ;
  elsif i_loss_fn_type = '202'
  then 
    return 
      sm_sc.fv_loss_cross_entropy(i_nn_depdt_01, i_true_01) / (array_length(i_nn_depdt_01, 2)) :: float
    ;
  elsif i_loss_fn_type = '203'
  then 
    return 
      sm_sc.fv_loss_cross_entropy_true_onehot_idx
      (
        i_nn_depdt_01
      , i_true_01
      -- -- , i_nn_depdt_02
      -- , i_true_02
      )
    ;
  else 
    raise exception 'unsupport loss fn type.';
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_lambda_loss('201', array[[1.0, 2, 3], [2, 3, 4]], array[[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]])
-- select sm_sc.fv_lambda_loss('201', array[[[1.0, 2, 3], [2, 3, 4]], [[1, 2, 3], [2, 3, 4]]], array[[[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]], [[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]]])
-- drop function if exists sm_sc.fv_d_nn_none_dloss_dindepdt(anyarray);
create or replace function sm_sc.fv_d_nn_none_dloss_dindepdt
(
  i_dloss_ddepdt    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  return i_dloss_ddepdt;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.fv_d_nn_none_dloss_dindepdt
--   (
--     array[[1, 2, 3], [4, 5, 6]]
--   )
-- -- 用于从规约的 buff 表中分组随机获得单次训练的小批量
-- -- 规约输出
-- drop function if exists sm_sc.ft_nn_buff_slice_rand(bigint, int4range[], int[]);
create or replace function sm_sc.ft_nn_buff_slice_rand
(
  i_work_no            bigint             ,
  i_slice_ranges       int4range[]        ,     
  i_rand_pick_cnts     int[]    
)
-- -- 规约：buff 表为 sm_sc.tb_nn_train_input_buff
returns table
(
  o_ord_no               int,
  o_slice_rand_pick      float[] 
)
as
$$
declare 
  v_ord_nos     int[]  := 
    (
      with
      cte_pick_ord_no as 
      (
        select 
          row_number() over() as a_range_no,
          -- 允许采样有小概率重复，对训练影响可控，避免 fv_rand_1d_ele_pick 的递归调用对内存的较大开销
          -- -- sm_sc.fv_rand_1d_ele_pick
          sm_sc.fv_rand_1d_ele
          (
            upper(a_range) - lower(a_range), i_rand_pick_cnts[row_number() over()]
          ) :: int[] 
          +` (lower(a_range) - 1) as a_range_eles
        from unnest(i_slice_ranges) tb_a(a_range)
      )
      select 
        sm_sc.fa_array_concat(a_range_eles order by a_range_no)
      from cte_pick_ord_no
    )
  ;
begin

  return query
    select 
      tb_a_buff.ord_no as o_ord_no,
      tb_a_buff.i_indepdt    as o_slice_rand_pick
    from generate_series(1, array_length(v_ord_nos, 1)) tb_a(a_ord_no)
    left join sm_sc.tb_nn_train_input_buff tb_a_buff
      on tb_a_buff.ord_no = v_ord_nos[a_ord_no]
        and tb_a_buff.work_no = i_work_no
  ;

end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select *
-- from
--   sm_sc.ft_nn_buff_slice_rand
--   (
--     2022032401,
--     array
--     [
--       int4range(1, 5923, '[]'), 
--       int4range(5924, 12665, '[]'),
--       int4range(12666, 18623, '[]'),
--       int4range(18624, 24754, '[]'),
--       int4range(24755, 30596, '[]'),
--       int4range(30597, 36017, '[]'),
--       int4range(36018, 41935, '[]'),
--       int4range(41936, 48200, '[]'),
--       int4range(48201, 54052, '[]'),
--       int4range(54053, 60001, '[]')
--     ],
--     array_fill(7, array[10])
--   ) tb_a

-- -- 用于从规约的 buff 表中分组随机获得单次训练的小批量
-- -- 规约输出
-- drop function if exists sm_sc.ft_nn_buff_slice_rand_pick(bigint, int4range[], int[]);
create or replace function sm_sc.ft_nn_buff_slice_rand_pick
(
  i_work_no            bigint             ,
  i_slice_ranges       int4range[]        ,     
  i_rand_pick_cnts     int[]
)
-- -- 规约：buff 表为 sm_sc.tb_nn_train_input_buff
returns table
(
  o_ord_no                    int
, o_rand_pick_indepdt_01      float[] 
, o_rand_pick_indepdt_02      float[] 
, o_rand_pick_indepdt_03      float[] 
, o_rand_pick_indepdt_04      float[] 
)
as
$$
declare 
  v_ord_nos     int[]  := 
    (
      with
      cte_pick_ord_no as 
      (
        select 
          row_number() over() as a_range_no,
          -- 允许采样有小概率重复，对训练影响可控，避免 fv_rand_1d_ele_pick 的递归调用对内存的较大开销
          -- -- sm_sc.fv_rand_1d_ele
          sm_sc.fv_rand_1d_ele_pick
          (
            upper(a_range) - lower(a_range), i_rand_pick_cnts[row_number() over()]
          ) :: int[] 
          +` (lower(a_range) - 1) as a_range_eles
        from unnest(i_slice_ranges) tb_a(a_range)
      )
      select 
        sm_sc.fa_array_concat(a_range_eles order by a_range_no)
      from cte_pick_ord_no
    )
  ;
begin

  return query
    select 
      tb_a_buff.ord_no as o_ord_no
    , i_indepdt_01 as o_rand_pick_indepdt_01
    , i_indepdt_02 as o_rand_pick_indepdt_02
    , i_indepdt_03 as o_rand_pick_indepdt_03
    , i_indepdt_04 as o_rand_pick_indepdt_04
    from generate_series(1, array_length(v_ord_nos, 1)) tb_a(a_ord_no)
    left join sm_sc.tb_nn_train_input_buff tb_a_buff
      on tb_a_buff.ord_no = v_ord_nos[a_ord_no]
        and tb_a_buff.work_no = i_work_no
  ;

end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select *
-- from
--   sm_sc.ft_nn_buff_slice_rand_pick
--   (
--     2022032401,
--     array
--     [
--       int4range(1, 5923, '[]'), 
--       int4range(5924, 12665, '[]'),
--       int4range(12666, 18623, '[]'),
--       int4range(18624, 24754, '[]'),
--       int4range(24755, 30596, '[]'),
--       int4range(30597, 36017, '[]'),
--       int4range(36018, 41935, '[]'),
--       int4range(41936, 48200, '[]'),
--       int4range(48201, 54052, '[]'),
--       int4range(54053, 60001, '[]')
--     ],
--     array_fill(7, array[10])
--   ) tb_a

-- drop function if exists sm_sc.ft_nn_in_out(bigint, bigint, float[], float[], float[], float[], int[]);
create or replace function sm_sc.ft_nn_in_out
(
  i_work_no            bigint
, i_sess_id            bigint
, i_indepdt_01 float[]                      -- 传入的自变量比训练集的记录大一个维度，用于一次请求传入多条预测数据
, i_indepdt_02 float[] default null
, i_indepdt_03 float[] default null
, i_indepdt_04 float[] default null
, i_depdt_nos  int[]   default array[1]         -- 指定要返回的因变量序号
)
returns table 
(
  o_depdt_01   float[]
, o_depdt_02   float[]
, o_depdt_03   float[]
, o_depdt_04   float[]
)
as
$$
declare 
  v_cur_nn_depth     int           :=   
    coalesce 
    (
      -- (select nn_depth_no from sm_sc.__vt_nn_node where work_no = i_work_no and node_type = 'prod_input' limit 1),
      (
        select 
          min(nn_depth_no)
        from sm_sc.__vt_nn_node 
        where work_no = i_work_no 
          and 
          (
             node_type = 'input_01' and i_indepdt_01 is not null
          or node_type = 'input_02' and i_indepdt_02 is not null
          or node_type = 'input_03' and i_indepdt_03 is not null
          or node_type = 'input_04' and i_indepdt_04 is not null
          )
        limit 1
      ),
      0 -- -- (select nn_depth_no from sm_sc.__vt_nn_node where work_no = i_work_no and node_type = 'input' limit 1)
    )
  ;
  v_nn_depth         int;
  v_return           float[];
  v_output_node_type varchar(64)[] :=   
    (  
      select 
        array_agg
        (
          case a_depdt_no 
            when 1 then 'output_01' 
            when 2 then 'output_02' 
            when 3 then 'output_03' 
            when 4 then 'output_04' 
          end
        )
      from unnest(i_depdt_nos) tb_a(a_depdt_no)
    )
  ;
  v_output_01        float[];
  v_output_02        float[];
  v_output_03        float[];
  v_output_04        float[];
begin
  -- -- -- 审计 session 结构
  -- -- create temp table if not exists sm_sc.__vt_tmp_nn_node
  -- -- (
  -- --   sess_id                 varchar(32),
  -- --   work_no                 bigint,           
  -- --   node_no                 bigint   ,           
  -- --   node_type               varchar(64)  ,    
  -- --   node_fn_type            varchar(64)  ,    
  -- --   node_fn_asso_value float[], 
  -- --   nn_depth_no             int  ,  
  -- --   node_depdt_vals              float[],
  -- --   primary key (sess_id, work_no, node_no)
  -- -- )
  -- -- ;
  
  -- -- 创建 session -- 需改造为并发访问通道和访问队列，而不用每次都 insert 一份模型
  -- if i_sess_id is null 
  -- then 
  --   i_sess_id  := lower(sm_sc.fv_get_global_seq());
  --   insert into sm_sc.__vt_tmp_nn_node
  --   (
  --     sess_id                        ,
  --     work_no                        ,
  --     node_no                        ,
  --     node_type                      ,
  --     node_fn_type                   ,
  --     node_fn_asso_value        ,
  --     nn_depth_no                    ,
  --     node_depdt_vals             
  --   )
  --   select 
  --     i_sess_id                    ,
  --     work_no                         ,
  --     node_no                         ,
  --     node_type                       ,
  --     coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  ,
  --     case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         ,
  --     nn_depth_no                     ,
  --     case 
  --       -- -- when 'prod_input' 
  --       -- --   then i_in
  --       when node_type = 'input_01' 
  --         then i_indepdt_01
  --       when node_type = 'input_02' 
  --         then i_indepdt_02
  --       when node_type = 'input_03' 
  --         then i_indepdt_03
  --       when node_type = 'input_04' 
  --         then i_indepdt_04
  --       -- when 'offset'
  --       --   then array[array[1]]
  --       when node_type = 'weight' or node_fn_type = '00_const'
  --         then node_depdt_vals
  --       else null
  --     end             
  --   from sm_sc.__vt_nn_node
  --   where work_no = i_work_no
  --   ;
  --   -- commit;
  -- end if;
  
  -- 审计 sess_id 是否有效
  if not exists (select  from sm_sc.__vt_nn_sess where sess_status = '1' and work_no = i_work_no and sess_id = i_sess_id)
    and i_sess_id <> 0  -- i_sess_id = 0 用于测试，无需注册，不支持并发
  then 
    raise exception 'not exists active sess_id: %, for work_no: %.  ', i_sess_id, i_work_no;
  end if;
  
  update sm_sc.__vt_tmp_nn_node
  set 
    work_no = i_work_no
  , node_depdt_vals = 
      case node_type
        when 'input_01' then i_indepdt_01
        when 'input_02' then i_indepdt_02
        when 'input_03' then i_indepdt_03
        when 'input_04' then i_indepdt_04
      end
  where sess_id = i_sess_id
    and node_type like 'input_%'
  ;  
  
  -- 前向传播
  -- -- v_nn_depth := (select max(nn_depth_no) from sm_sc.__vt_tmp_nn_node where sess_id = i_sess_id and work_no = i_work_no);
  v_nn_depth := (select max(nn_depth_no) from sm_sc.__vt_tmp_nn_node where sess_id = i_sess_id and work_no = i_work_no and node_type = any(v_output_node_type));
  while v_cur_nn_depth <= v_nn_depth
  loop 
    with     
    -- 参数准备，单目、双目（三目）算子
    cte_indepdt as
    (
      select 
        tb_a_fore.node_no
      , tb_a_fore.node_fn_type
      , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 1 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_1st
      , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 2 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_2nd
      , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 3 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_3rd
      from sm_sc.__vt_tmp_nn_node tb_a_fore
      inner join sm_sc.__vt_nn_path tb_a_path
        on tb_a_path.work_no = i_work_no   -- 2021082501
          and tb_a_path.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.__vt_tmp_nn_node tb_a_back
        on tb_a_back.sess_id = i_sess_id
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_path.back_node_no
      -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
      inner join sm_sc.tb_dic_enum tb_a_dic
        on tb_a_dic.enum_name = 'node_fn_type'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_dic.enum_group in ('1_p', '2_p', '3_p')
      where tb_a_fore.sess_id = i_sess_id
        and tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    )
    update sm_sc.__vt_tmp_nn_node tb_a_tar_depdt
    set 
      node_depdt_vals = 
        sm_sc.fv_lambda_arr
        (
          tb_a_indepdt_fore.node_no
        , tb_a_tar_depdt.node_fn_type
        , tb_a_indepdt_fore.a_bi_opr_input_1st
        , tb_a_indepdt_fore.a_bi_opr_input_2nd
        , tb_a_tar_depdt.node_fn_asso_value
        , tb_a_indepdt_fore.a_bi_opr_input_3rd
        )
    from cte_indepdt tb_a_indepdt_fore
    where tb_a_tar_depdt.sess_id = i_sess_id
      and tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
      and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
    ;      
    
    with 
    -- 参数准备，聚合算子
    cte_indepdt as
    (
      -- 聚合后的入参归并结果当作单目运算唯一参数
      -- 另一个同类型的限制是，一个CASE无法阻止其所包含的聚集表达式 的计算...取而代之的是，可以使用 一个WHERE或FILTER子句来首先阻止有问题的输入行到达 一个聚集函数。
      -- 采用 filter 过滤掉不必要的聚合输入，达到减少聚合计算以及入参宽高一致性保证等安全问题
      select 
        tb_a_fore.node_no
      , tb_a_fore.node_fn_type
      , case 
          when tb_a_fore.node_fn_type = '06_aggr_mx_sum'
            then sm_sc.fa_mx_sum(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_sum')
          when tb_a_fore.node_fn_type = '06_aggr_mx_prod'
            then sm_sc.fa_mx_prod(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_prod')
          when tb_a_fore.node_fn_type = '06_aggr_mx_avg'
            then sm_sc.fa_mx_avg(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')
          when tb_a_fore.node_fn_type = '06_aggr_mx_max'
            then sm_sc.fa_mx_max(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_max')
          when tb_a_fore.node_fn_type = '06_aggr_mx_min'
            then sm_sc.fa_mx_min(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_min')
          when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
            then sm_sc.fa_mx_concat_y(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
          when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x'
            then sm_sc.fa_mx_concat_x(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
          when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3'
            then sm_sc.fa_mx_concat_x3(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
          when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4'
            then sm_sc.fa_mx_concat_x4(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
        end
        as a_opr_depdt
      from sm_sc.__vt_tmp_nn_node tb_a_fore
      inner join sm_sc.__vt_nn_path tb_a_path
        on tb_a_path.work_no = i_work_no   -- 2021082501
          and tb_a_path.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.__vt_tmp_nn_node tb_a_back
        on tb_a_back.sess_id = i_sess_id
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_path.back_node_no
      -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
      inner join sm_sc.tb_dic_enum tb_a_dic
        on tb_a_dic.enum_name = 'node_fn_type'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_dic.enum_group = 'n_p'
      where tb_a_fore.sess_id = i_sess_id
        and tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    )
    update sm_sc.__vt_tmp_nn_node tb_a_tar_depdt
    set 
      node_depdt_vals = tb_a_indepdt_fore.a_opr_depdt
    from cte_indepdt tb_a_indepdt_fore
    where tb_a_tar_depdt.sess_id = i_sess_id
      and tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
      and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
    ;
    -- commit;
    v_cur_nn_depth := v_cur_nn_depth + 1;
  end loop;
  
-- -- debug
-- raise notice 'row_cnt: %', (select count(*) from sm_sc.__vt_tmp_nn_node where sess_id = i_sess_id and work_no = i_work_no);

  -- 准备输出结构。当前先逐个输出结构赋值，期待 pg unpivot
  select 
    node_depdt_vals
  into 
    v_output_01
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_01'
  ;
  select 
    node_depdt_vals
  into 
    v_output_02
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_02'
  ;
  select 
    node_depdt_vals
  into 
    v_output_03
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_03'
  ;
  select 
    node_depdt_vals
  into 
    v_output_04
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_04'
  ;
  
  -- -- 清理 session 数据
  -- delete from sm_sc.__vt_tmp_nn_node 
  -- where sess_id = i_sess_id
  --   -- -- and work_no = i_work_no
  -- ;
  
  -- 返回结果
  return query
    select 
      v_output_01
    , v_output_02
    , v_output_03
    , v_output_04
  ;
  
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- -- set min_parallel_table_scan_size = 8;
-- -- set min_parallel_index_scan_size = 16;
-- -- set force_parallel_mode = 'off';
-- -- set max_parallel_workers_per_gather = 1;
-- -- set parallel_setup_cost = 10000;
-- -- set parallel_tuple_cost = 10000.0;

-- -- 不支持开并行，务必关闭并行后执行 sm_sc.ft_nn_in_out
-- select 
--   o_depdt_01
-- from
--   sm_sc.ft_nn_in_out
--   (
--     -54321   --2021112502, 
--   , array[[0.0, 0.0]] :: float[]
--       +` sm_sc.fv_new_randn(0.0 :: float, 0.1, array[1, 2])
--   ) tb_a
-- drop function if exists sm_sc.ft_nn_in_out_p(bigint, bigint, float[], float[], float[], float[], int[]);
create or replace function sm_sc.ft_nn_in_out_p
(
  i_work_no            bigint
, i_sess_id            bigint
, i_indepdt_01 float[]                      -- 传入的自变量比训练集的记录大一个维度，用于一次请求传入多条预测数据
, i_indepdt_02 float[] default null
, i_indepdt_03 float[] default null
, i_indepdt_04 float[] default null
, i_depdt_nos  int[]   default array[1]         -- 指定要返回的因变量序号
)
returns table 
(
  o_depdt_01   float[]
, o_depdt_02   float[]
, o_depdt_03   float[]
, o_depdt_04   float[]
)
as
$$
declare 
  -- v_model_code_6     char(6)       :=   left(md5(random()::text), 6);
  v_cur_nn_depth     int           :=   
    coalesce 
    (
      -- (select nn_depth_no from sm_sc.__vt_nn_node where work_no = i_work_no and node_type = 'prod_input' limit 1),
      (
        select 
          min(nn_depth_no)
        from sm_sc.__vt_nn_node 
        where work_no = i_work_no 
          and 
          (
             node_type = 'input_01' and i_indepdt_01 is not null
          or node_type = 'input_02' and i_indepdt_02 is not null
          or node_type = 'input_03' and i_indepdt_03 is not null
          or node_type = 'input_04' and i_indepdt_04 is not null
          )
        limit 1
      ),
      0 -- -- (select nn_depth_no from sm_sc.__vt_nn_node where work_no = i_work_no and node_type = 'input' limit 1)
    )
  ;
  v_nn_depth         int;
  v_return           float[];
  v_output_node_type varchar(64)[] :=   
    (  
      select 
        array_agg
        (
          case a_depdt_no 
            when 1 then 'output_01' 
            when 2 then 'output_02' 
            when 3 then 'output_03' 
            when 4 then 'output_04' 
          end
        )
      from unnest(i_depdt_nos) tb_a(a_depdt_no)
    )
  ;
  v_output_01        float[];
  v_output_02        float[];
  v_output_03        float[];
  v_output_04        float[];
  v_null             varchar(64)[];
begin
  
  -- 审计 sess_id 是否有效
  if not exists (select  from sm_sc.__vt_nn_sess where sess_status = '1' and work_no = i_work_no and sess_id = i_sess_id)
    and i_sess_id <> 0   -- i_sess_id = 0 用于测试，无需注册，不支持并发
  then 
    raise exception 'not exists active sess_id: %, for work_no: %.  ', i_sess_id, i_work_no;
  end if;
  
  update sm_sc.__vt_tmp_nn_node
  set 
    work_no = i_work_no
  -- , p_node_depdt = 
  --     case node_type
  --       when 'input_01' then i_indepdt_01
  --       when 'input_02' then i_indepdt_02
  --       when 'input_03' then i_indepdt_03
  --       when 'input_04' then i_indepdt_04
  --     end
  , p_node_depdt = 
      sm_sc.__fv_set_kv
      (
        case node_type
          when 'input_01' then i_indepdt_01
          when 'input_02' then i_indepdt_02
          when 'input_03' then i_indepdt_03
          when 'input_04' then i_indepdt_04
        end
      , -- v_model_code_6 || 
        '_' || work_no :: varchar  || 
        '_' || node_no :: varchar  || 
        '__d' ||  -- 'depdt'
        '_'  || substr(node_type, 7) || -- input_no
        '_' || i_sess_id :: varchar
      )
  where sess_id = i_sess_id
    and node_type like 'input_%'
  ;  
  
  -- 前向传播
  -- -- v_nn_depth := (select max(nn_depth_no) from sm_sc.__vt_tmp_nn_node where sess_id = i_sess_id and work_no = i_work_no);
  v_nn_depth := (select max(nn_depth_no) from sm_sc.__vt_tmp_nn_node where sess_id = i_sess_id and work_no = i_work_no and node_type = any(v_output_node_type));
  while v_cur_nn_depth <= v_nn_depth
  loop 
    with     
    -- 参数准备，单目、双目（三目）算子
    cte_indepdt as
    (
      select 
        tb_a_fore.node_no
      , tb_a_fore.node_fn_type
      , sm_sc.fa_coalesce(case when tb_a_path.path_ord_no = 1 then tb_a_back.p_node_depdt end) as a_bi_opr_input_1st_p
      , sm_sc.fa_coalesce(case when tb_a_path.path_ord_no = 2 then tb_a_back.p_node_depdt end) as a_bi_opr_input_2nd_p
      , sm_sc.fa_coalesce(case when tb_a_path.path_ord_no = 3 then tb_a_back.p_node_depdt end) as a_bi_opr_input_3rd_p
      from sm_sc.__vt_tmp_nn_node tb_a_fore
      inner join sm_sc.__vt_nn_path tb_a_path
        on tb_a_path.work_no = i_work_no   -- 2021082501
          and tb_a_path.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.__vt_tmp_nn_node tb_a_back
        on tb_a_back.sess_id = i_sess_id
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_path.back_node_no
      -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
      inner join sm_sc.tb_dic_enum tb_a_dic
        on tb_a_dic.enum_name = 'node_fn_type'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_dic.enum_group in ('1_p', '2_p', '3_p')
      where tb_a_fore.sess_id = i_sess_id
        and tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    )
    update sm_sc.__vt_tmp_nn_node tb_a_tar_depdt
    set 
      p_node_depdt = 
        sm_sc.fv_lambda_arr_p
        (
          ''
        , i_work_no
        , tb_a_indepdt_fore.node_no
        , i_sess_id
        , tb_a_tar_depdt.node_fn_type
        , tb_a_indepdt_fore.a_bi_opr_input_1st_p
        , tb_a_indepdt_fore.a_bi_opr_input_2nd_p
        , tb_a_tar_depdt.node_fn_asso_value
        , tb_a_indepdt_fore.a_bi_opr_input_3rd_p
        )
    from cte_indepdt tb_a_indepdt_fore
    where tb_a_tar_depdt.sess_id = i_sess_id
      and tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
      and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
    ;      
    
    with 
    -- 参数准备，聚合算子
    cte_indepdt as
    (
      -- 聚合后的入参归并结果当作单目运算唯一参数
      -- 另一个同类型的限制是，一个CASE无法阻止其所包含的聚集表达式 的计算...取而代之的是，可以使用 一个WHERE或FILTER子句来首先阻止有问题的输入行到达 一个聚集函数。
      -- 采用 filter 过滤掉不必要的聚合输入，达到减少聚合计算以及入参宽高一致性保证等安全问题
      select 
        tb_a_fore.node_no
      , tb_a_fore.node_fn_type
      , sm_sc.__fv_set_kv
        (
          case 
            when tb_a_fore.node_fn_type = '06_aggr_mx_sum'
              then sm_sc.fa_mx_sum(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_sum')
            when tb_a_fore.node_fn_type = '06_aggr_mx_prod'
              then sm_sc.fa_mx_prod(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_prod')
            when tb_a_fore.node_fn_type = '06_aggr_mx_avg'
              then sm_sc.fa_mx_avg(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')
            when tb_a_fore.node_fn_type = '06_aggr_mx_max'
              then sm_sc.fa_mx_max(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_max')
            when tb_a_fore.node_fn_type = '06_aggr_mx_min'
              then sm_sc.fa_mx_min(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_min')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
              then sm_sc.fa_mx_concat_y(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x'
              then sm_sc.fa_mx_concat_x(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3'
              then sm_sc.fa_mx_concat_x3(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4'
              then sm_sc.fa_mx_concat_x4(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
          end
        , -- v_model_code_6 || 
          '_' || i_work_no :: varchar   ||
          '_' || tb_a_fore.node_no :: varchar ||
          '__d_aggr_'
        )
        as a_opr_depdt_p
      from sm_sc.__vt_tmp_nn_node tb_a_fore
      inner join sm_sc.__vt_nn_path tb_a_path
        on tb_a_path.work_no = i_work_no   -- 2021082501
          and tb_a_path.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.__vt_tmp_nn_node tb_a_back
        on tb_a_back.sess_id = i_sess_id
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_path.back_node_no
      -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
      inner join sm_sc.tb_dic_enum tb_a_dic
        on tb_a_dic.enum_name = 'node_fn_type'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_dic.enum_group = 'n_p'
      where tb_a_fore.sess_id = i_sess_id
        and tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    )
    update sm_sc.__vt_tmp_nn_node tb_a_tar_depdt
    set 
      p_node_depdt = tb_a_indepdt_fore.a_opr_depdt_p
    from cte_indepdt tb_a_indepdt_fore
    where tb_a_tar_depdt.sess_id = i_sess_id
      and tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
      and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
    ;
    -- commit;
    
    -- 清理当前层前向传播后的无用张量 kv
    -- 清理 depdt
    with 
    cte_clear_kv_list as 
    (
      select
        tb_a_back_node.node_no
      , max(tb_a_back_node.p_node_depdt) as p_node_depdt
      from sm_sc.__vt_tmp_nn_node tb_a_main_fore_node
      inner join sm_sc.__vt_nn_path tb_a_main_path
        on tb_a_main_path.fore_node_no = tb_a_main_fore_node.node_no 
          and tb_a_main_path.work_no = i_work_no
      inner join sm_sc.__vt_tmp_nn_node tb_a_back_node
        on tb_a_back_node.node_no = tb_a_main_path.back_node_no
          and tb_a_back_node.work_no = i_work_no
      inner join sm_sc.tb_dic_enum tb_a_back_dic 
        on tb_a_back_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_back_dic.enum_key = tb_a_back_node.node_fn_type
          and substr(tb_a_back_dic.enum_value, 5, 2) = '00'  -- 第一/二目求导不需要因变量
      inner join sm_sc.__vt_nn_path tb_a_path
        on tb_a_path.work_no = i_work_no
          and tb_a_path.back_node_no = tb_a_back_node.node_no
      inner join sm_sc.__vt_tmp_nn_node tb_a_fore_node
        on tb_a_fore_node.work_no = i_work_no
          and tb_a_fore_node.node_no = tb_a_path.fore_node_no
      where tb_a_main_fore_node.work_no = i_work_no
        and tb_a_main_fore_node.nn_depth_no = v_cur_nn_depth
      group by tb_a_back_node.node_no 
      having max(tb_a_fore_node.nn_depth_no) <= v_cur_nn_depth
    ),
    cte_upd_node_k_2_null as 
    (
      update sm_sc.__vt_tmp_nn_node tb_a_back_node
      set 
        p_node_depdt        = null
      from cte_clear_kv_list tb_a_key_list
      where tb_a_back_node.node_no = tb_a_key_list.node_no
        and tb_a_back_node.work_no = i_work_no
    )
    select  --  perform 暂不支持 cte，所以采用 select  into null 暂时代替。
      -- 同一 dml/dql 中，要尽量避免多次调用 sm_sc.__fv_delete_kv，所以将待清理 arr_key 集中在一处 __fv_delete_kv，放在最外层，
      sm_sc.__fv_delete_kv
      (
        (select array_agg(p_node_depdt) from cte_clear_kv_list)
      ) into v_null
    ;
      
-- -- debug
-- raise notice 'debug 002.9. v_null: %', v_null;
    v_null := null;
    
    v_cur_nn_depth := v_cur_nn_depth + 1;

  end loop;
  
-- -- debug
-- raise notice 'row_cnt: %', (select count(*) from sm_sc.__vt_tmp_nn_node where sess_id = i_sess_id and work_no = i_work_no);

  -- 准备输出结构。当前先逐个输出结构赋值，期待 pg unpivot
  select 
    sm_sc.__fv_get_kv(p_node_depdt)
  into 
    v_output_01
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_01'
  ;
  select 
    sm_sc.__fv_get_kv(p_node_depdt)
  into 
    v_output_02
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_02'
  ;
  select 
    sm_sc.__fv_get_kv(p_node_depdt)
  into 
    v_output_03
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_03'
  ;
  select 
    sm_sc.__fv_get_kv(p_node_depdt)
  into 
    v_output_04
  from sm_sc.__vt_tmp_nn_node
  where sess_id = i_sess_id
    and work_no = i_work_no
    and node_type = 'output_04'
  ;
  
  -- -- 清理 session 数据
  -- delete from sm_sc.__vt_tmp_nn_node 
  -- where sess_id = i_sess_id
  --   -- -- and work_no = i_work_no
  -- ;
  
  -- 清理 kv 张量: depdt
  perform 
    sm_sc.__fv_delete_kv
    (
      array_agg(p_node_depdt)
    )
  from sm_sc.__vt_tmp_nn_node 
  where work_no = i_work_no
    and node_type <> 'weight'
    and node_fn_type <> '00_const'
    and sess_id = i_sess_id
  ;
  update sm_sc.__vt_tmp_nn_node
  set 
    p_node_depdt        = null
  where work_no = i_work_no
    and node_type <> 'weight'
    and node_fn_type <> '00_const'
    and sess_id = i_sess_id
  ;  

  -- 返回结果
  return query
    select 
      v_output_01
    , v_output_02
    , v_output_03
    , v_output_04
  ;
  
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- -- set min_parallel_table_scan_size = 8;
-- -- set min_parallel_index_scan_size = 16;
-- -- set force_parallel_mode = 'off';
-- -- set max_parallel_workers_per_gather = 1;
-- -- set parallel_setup_cost = 10000;
-- -- set parallel_tuple_cost = 10000.0;

-- -- 不支持开并行，务必关闭并行后执行 sm_sc.ft_nn_in_out_p
-- select 
--   o_depdt_01
-- from
--   sm_sc.ft_nn_in_out_p
--   (
--     -54321   --2021112502, 
--   , array[[0.0, 0.0]] :: float[]
--       +` sm_sc.fv_new_randn(0.0 :: float, 0.1, array[1, 2])
--   ) tb_a
-- drop function if exists sm_sc.ft_nn_node2node_path(bigint, bigint, bigint);
create or replace function sm_sc.ft_nn_node2node_path
(
  i_work_no         bigint    , 
  i_begin_node_no   bigint    , 
  i_end_node_no     bigint      
)
returns table
(
  o_back_node_no    bigint    ,
  o_fore_node_no    bigint    ,
  o_path_ord_no     int       ,
  o_node_type       varchar(16)
)
as 
$$
declare
  v_begin_depth     int        :=  (select nn_depth_no from sm_sc.tb_nn_node where work_no = i_work_no and node_no = i_begin_node_no);
  v_end_depth       int        :=  (select nn_depth_no from sm_sc.tb_nn_node where work_no = i_work_no and node_no = i_end_node_no);
  
begin

  if exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and nn_depth_no is null and node_type <> 'input')
  then
    raise exception 'NN depth exception!  There are nodes without nn_depth_no value.';
  end if;
  
  if v_begin_depth >= v_end_depth
  then
    raise exception 'NN depth exception!  nn_depth_no of i_begin_node_no must be <= i_end_node_no''s.';
  end if;
  
  return query
    with recursive 
    cte_aggr_fore as 
    (
      select 
        tb_a_base.back_node_no,
        tb_a_base.fore_node_no,
        tb_a_base.path_ord_no
      from sm_sc.tb_nn_path tb_a_base
      where back_node_no = i_begin_node_no         -- 103010025
        and tb_a_base.work_no = i_work_no     -- 2022030501
      union
      select 
        tb_a_incre.back_node_no,
    	tb_a_incre.fore_node_no,
        tb_a_incre.path_ord_no
      from cte_aggr_fore tb_a_base
      inner join sm_sc.tb_nn_path tb_a_incre
        on tb_a_incre.work_no = i_work_no     -- 2022030501
        and tb_a_incre.back_node_no = tb_a_base.fore_node_no
      inner join sm_sc.tb_nn_node tb_a_node_incre
        on tb_a_node_incre.work_no = i_work_no     -- 2022030501
        and tb_a_node_incre.node_no = tb_a_incre.fore_node_no
        and tb_a_node_incre.nn_depth_no <= v_end_depth      -- 60
    ),
    cte_aggr_back as 
    (
      select 
        tb_a_base.back_node_no,
        tb_a_base.fore_node_no,
        tb_a_base.path_ord_no
      from sm_sc.tb_nn_path tb_a_base
      inner join cte_aggr_fore tb_a_fore
        on tb_a_fore.back_node_no = tb_a_base.back_node_no
        and tb_a_fore.fore_node_no = tb_a_base.fore_node_no
      where tb_a_base.fore_node_no = i_end_node_no         -- 106020001
        and tb_a_base.work_no = i_work_no     -- 2022030501
      union
      select 
        tb_a_decre.back_node_no,
    	tb_a_decre.fore_node_no,
        tb_a_decre.path_ord_no
      from cte_aggr_back tb_a_base
      -- 有向无环图的局部两节点之间遍历：两节点深度之间的 前向扩散 与 反向扩散 的交集就是局部遍历
      inner join sm_sc.tb_nn_path tb_a_decre
        on tb_a_decre.work_no = i_work_no     -- 2022030501
        and tb_a_decre.fore_node_no = tb_a_base.back_node_no
      inner join cte_aggr_fore tb_a_fore
        on tb_a_fore.back_node_no = tb_a_decre.back_node_no
        and tb_a_fore.fore_node_no = tb_a_decre.fore_node_no
      inner join sm_sc.tb_nn_node tb_a_node_decre
        on tb_a_node_decre.work_no = i_work_no     -- 2022030501
        and tb_a_node_decre.node_no = tb_a_decre.back_node_no
        and tb_a_node_decre.nn_depth_no >= v_begin_depth      -- 1
    )
    select 
      back_node_no, 
      fore_node_no,
      path_ord_no ,
      'main' :: varchar(16) as node_type
    from cte_aggr_back
    union 
    select 
      tb_a_asso.back_node_no, 
      tb_a_asso.fore_node_no,
      tb_a_asso.path_ord_no ,
      'leaf' :: varchar(16) as node_type
    from cte_aggr_back tb_a_back_tail
    inner join sm_sc.tb_nn_path tb_a_asso
      on tb_a_asso.fore_node_no = tb_a_back_tail.back_node_no
      and tb_a_asso.work_no = i_work_no     -- 2022030501
    left join cte_aggr_back tb_a_main
      on tb_a_main.fore_node_no = tb_a_asso.fore_node_no
      and tb_a_main.back_node_no = tb_a_asso.back_node_no
    where tb_a_main.back_node_no is null
      and tb_a_asso.fore_node_no <> i_begin_node_no         -- 103010025
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;


-- -- select * from sm_sc.tb_nn_path
-- -- select * from sm_sc.tb_nn_node where node_no in (103010025, 106020001)
-- select o_back_node_no, o_fore_node_no, o_path_ord_no, o_node_type from sm_sc.ft_nn_node2node_path(2022030501, 103010025, 106020001)
-- drop function if exists sm_sc.fv_idx_samp_by_samp(int4multirange, int4multirange);
create or replace function sm_sc.fv_idx_samp_by_samp
(
  i_sour_idxes         int4multirange     ,   -- 采样前原始序号集合
  i_fn_idxes           int4multirange         -- 本次采样到的顺序号集合
)
returns     int4multirange                    -- 返回采样后的原始序号集合
as 
$$
-- declare 
begin    
  -- 当 $2 是 null 时，返回 $1
  if i_fn_idxes is null 
  then 
    return i_sour_idxes;
  else 
    return 
    (
      with 
      cte_sour_unnest as 
      (
        select 
          row_number() over() as idx_y_no,
          (upper(abs_idx_range) - 1) - lower(abs_idx_range) + 1 as idx_range_len,
          abs_idx_range
        from unnest(i_sour_idxes) tb_abs_idx_range(abs_idx_range)
        where abs_idx_range is not null
      ),
      cte_fn_unnest as 
      (
        select 
          tb_a_main.idx_y_no,
          coalesce(sum(tb_a_above.idx_range_len), 0) as above_idx_range_len,
          tb_a_main.abs_idx_range,
          int4range
          (
            (coalesce(sum(tb_a_above.idx_range_len), 0) + 1) :: int,
            (coalesce(sum(tb_a_above.idx_range_len), 0) + max(tb_a_main.idx_range_len)) :: int,
            '[]'
          ) as a_fn_idx_range
        from cte_sour_unnest tb_a_main
        left join cte_sour_unnest tb_a_above
          on tb_a_above.idx_y_no < tb_a_main.idx_y_no
        group by tb_a_main.idx_y_no, tb_a_main.abs_idx_range
      ),
      cte_rela_range as 
      (
        select 
          tb_a_sour.idx_y_no,
          tb_a_sour.abs_idx_range,
          tb_a_sour.above_idx_range_len,
          tb_a_sour.a_fn_idx_range * tb_a_fn_idx.a_fn_idx as rela_idx_range
        from unnest(i_fn_idxes) tb_a_fn_idx(a_fn_idx)
        inner join cte_fn_unnest tb_a_sour
          on tb_a_sour.a_fn_idx_range && tb_a_fn_idx.a_fn_idx  
      )        
      select 
        sm_sc.fa_range_or 
        (
          int4multirange
          (
            int4range 
            (
              (lower(abs_idx_range) + lower(rela_idx_range) - above_idx_range_len - 1) :: int,
              (lower(abs_idx_range) + (upper(rela_idx_range) - 1) - above_idx_range_len - 1) :: int,
              '[]'
            )
          )
        )
      from cte_rela_range
    )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- -- 当 $2 是 null 时，返回 $1
-- select 
--   sm_sc.fv_idx_samp_by_samp
--   (
--     int4multirange
--     (
--       int4range(1, 3, '[]'),
--       int4range(8, 12, '[]'),
--       int4range(13, 15, '[]')
--     ),
--     null
--   )

-- -- 当 $1 是单值单区间，返回单值多区间（通常来自 random）
-- select 
--   sm_sc.fv_idx_samp_by_samp
--   (
--     int4multirange
--     (
--       int4range(1, 15, '[]')
--     ),
--     int4multirange
--     (
--       int4range(3, 3, '[]'),
--       int4range(6, 6, '[]'),
--       int4range(8, 8, '[]'),
--       int4range(9, 9, '[]')  --,
--       -- int4range(19, 19, '[]')
--     )
--   )

-- -- 当 $1, $2 是普通多值或多区间时，返回多区间（通常来自 random）
-- select 
--   sm_sc.fv_idx_samp_by_samp
--   (
--     int4multirange      -- 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, 15
--     (
--       int4range(1, 3, '[]'),
--       int4range(8, 12, '[]'),
--       int4range(13, 15, '[]')
--     ),
--     int4multirange      -- 1, 2, 3, (8, 9, 10), 11, (12, 13, 14, 15)
--     (
--       int4range(4, 6, '[]'),
--       int4range(8, 12, '[]') -- ,
--       -- int4range(13, 15, '[]')
--     )
--   )


-- ---------------------------------------------------------------------------------
-- drop function if exists sm_sc.fv_idx_samp_by_samp(int4multirange[], int4multirange);
create or replace function sm_sc.fv_idx_samp_by_samp
(
  i_sour_idxes_arr     int4multirange[]     ,   -- 采样前原始序号集合
  i_fn_idxes           int4multirange         -- 本次采样到的顺序号集合
)
returns     int4multirange[]                    -- 返回采样后的原始序号集合
as 
$$
-- declare 
begin    
  -- 当 $2 是 null 时，返回 $1
  if i_fn_idxes is null 
  then 
    return i_sour_idxes_arr;
  else 
    return 
    (
      with 
      cte_sour_unnest as 
      (
        select 
          row_number() over() as a_idx_y_no,
          -- (upper(a_sour_idxes) - 1) - lower(a_sour_idxes) + 1 as idx_range_len,
          sm_sc.fv_multirange_len(a_sour_idxes) as idx_multirange_len,
          a_sour_idxes
        from unnest(i_sour_idxes_arr) tb_abs_idx_range(a_sour_idxes)
        where a_sour_idxes is not null
      ),
      cte_fn_unnest as 
      (
        select 
          tb_a_main.a_idx_y_no,
          coalesce(sum(tb_a_above.idx_multirange_len), 0) as above_idx_range_len,
          tb_a_main.a_sour_idxes,
          int4range
          (
            (coalesce(sum(tb_a_above.idx_multirange_len), 0) + 1) :: int,
            (coalesce(sum(tb_a_above.idx_multirange_len), 0) + max(tb_a_main.idx_multirange_len)) :: int,
            '[]'
          ) as a_fn_idx_range
        from cte_sour_unnest tb_a_main
        left join cte_sour_unnest tb_a_above
          on tb_a_above.a_idx_y_no < tb_a_main.a_idx_y_no
        group by tb_a_main.a_idx_y_no, tb_a_main.a_sour_idxes
      ),
      cte_rela_range as 
      (
        select 
          tb_a_sour.a_idx_y_no,
          tb_a_sour.a_sour_idxes,
          tb_a_sour.above_idx_range_len,
          tb_a_sour.a_fn_idx_range,
          tb_a_sour.a_fn_idx_range * tb_a_fn_idx.a_fn_idx as rela_idx_range
        from unnest(i_fn_idxes) tb_a_fn_idx(a_fn_idx)
        inner join cte_fn_unnest tb_a_sour
          on tb_a_sour.a_fn_idx_range && tb_a_fn_idx.a_fn_idx  
      )        
      select 
        array_agg 
        (
          sm_sc.fv_idx_samp_by_samp
          (
            a_sour_idxes,
            sm_sc.fv_multirange_move(multirange(rela_idx_range), - above_idx_range_len :: int)
          )
          order by a_idx_y_no
        )
      from cte_rela_range
      where lower(rela_idx_range) < upper(rela_idx_range)
    );
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_idx_samp_by_samp
--   (
--     array    -- 1, 2, 3, 8, 9, 10, 11, 13, 14, 15, 4, 5, 6, 8, 9, 10, 11, 12, 15, 16, 17, 18, 19
--     [
--       int4multirange
--       (
--         int4range(1, 3, '[]'),
--         int4range(8, 11, '[]'),
--         int4range(13, 15, '[]')
--       ),
--       int4multirange
--       (
--         int4range(4, 6, '[]'),
--         int4range(8, 12, '[]') -- ,
--         -- int4range(15, 19, '[]')
--       )
--     ]
--   , int4multirange     -- (1, 2), 3, (8, 9, 10), 11, (13, 14, 15, 4), 5, 6, 8, 9, (10, 11, 12), 15, 16, 17, 18, 19
--     (
--       int4range(1, 2, '[]'),
--       int4range(4, 6, '[]'),
--       int4range(8, 11, '[]'),
--       int4range(16, 18, '[]')
--     )
--   )
-- drop function if exists sm_sc.fv_nn_node2node_val(bigint, bigint, bigint);
create or replace function sm_sc.fv_nn_node2node_val
(
  i_work_no            bigint,
  i_begin_node_no       bigint,
  i_end_node_no       bigint
)
returns float[]
as 
$$
declare
  v_sess_id            bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)   :=  replace(gen_random_uuid()::char(36), '-', '')::char(32);
  v_cur_depth_no       int;
  v_begin_depth_no     int        :=  (select nn_depth_no from sm_sc.tb_nn_node where work_no = i_work_no and node_no = i_begin_node_no);
  v_end_depth_no       int        :=  (select nn_depth_no from sm_sc.tb_nn_node where work_no = i_work_no and node_no = i_end_node_no);
  v_ret                float[];
  
begin
  insert into sm_sc.__vt_tmp_node_node2node
  (
    sess_id      ,
    work_no      ,
    node_no      ,
    node_type
  )
  select 
    v_sess_id                          ,
    i_work_no                          ,
    fore_node_no                       ,
    max(node_type) as node_type
  from sm_sc.ft_nn_node2node_path(i_work_no, i_begin_node_no, i_end_node_no) tb_a(back_node_no, fore_node_no, path_ord_no, node_type)
  -- where node_type = 'nn_main'
  group by fore_node_no
  ;

  for v_cur_depth_no in v_begin_depth_no .. v_end_depth_no
  loop
    with 
    cte_aggr_o as 
    (
      select 
        tb_a_node_tar.node_no,
        max(tb_a_node_fore_dic.node_fn_type) as node_fn_type,
        case
          -- when max(tb_a_node_fore_dic.node_fn_type) in ('rand_pick_y', 'rand_pick_x', 'new')
          --   then sm_sc.fa_mx_coalesce((tb_a_node_back_dic.node_o).m_vals)
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_concat_y'
            then sm_sc.fa_mx_concat_y((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_concat_y')
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_concat_x'
            then sm_sc.fa_mx_concat_x((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_concat_x')
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_sum'
            then sm_sc.fa_mx_sum((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_sum')
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_avg'
            then sm_sc.fa_mx_avg((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_avg')
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_max'
            then sm_sc.fa_mx_max((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_max')
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_min'
            then sm_sc.fa_mx_min((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_min')
          when max(tb_a_node_fore_dic.node_fn_type) = 'agg_prod'
            then sm_sc.fa_mx_prod((tb_a_node_back_dic.node_o).m_vals order by tb_a_path.path_ord_no)
              filter(where tb_a_node_fore_dic.node_fn_type = 'agg_prod')
          else sm_sc.fa_mx_coalesce(case when tb_a_path.path_ord_no = 1 then (tb_a_node_back_dic.node_o).m_vals else null end order by tb_a_path.path_ord_no)
              filter(where tb_a_path.path_ord_no = 1)
        end as a_back_o_s_1st,
        sm_sc.fa_mx_coalesce(case when tb_a_path.path_ord_no = 2 then (tb_a_node_back_dic.node_o).m_vals else null end order by tb_a_path.path_ord_no) 
              filter(where tb_a_path.path_ord_no = 2)as a_back_o_s_2nd,
        sm_sc.fa_mx_coalesce(tb_a_node_fore_dic.node_fn_asso_value) as node_fn_asso_value
      from sm_sc.__vt_tmp_node_node2node tb_a_node_tar
      inner join sm_sc.tb_nn_node tb_a_node_fore_dic
        on tb_a_node_tar.node_no = tb_a_node_fore_dic.node_no
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.fore_node_no = tb_a_node_fore_dic.node_no
      inner join sm_sc.tb_nn_node tb_a_node_back_dic
        on tb_a_node_back_dic.node_no = tb_a_path.back_node_no
      where tb_a_node_tar.node_type = 'nn_main'
        and tb_a_node_fore_dic.work_no = i_work_no
        and tb_a_node_tar.work_no = i_work_no
        and tb_a_path.work_no = i_work_no
        and tb_a_node_tar.sess_id = v_sess_id
        and tb_a_node_fore_dic.nn_depth_no = v_cur_depth_no
      group by tb_a_node_tar.node_no, tb_a_node_tar.sess_id
    )
    update sm_sc.__vt_tmp_node_node2node tb_a_node_tar
    set 
      node_depdt_vals = 
        case 
          when node_fn_type like 'agg_%'
            then tb_a_sour.a_back_o_s_1st
          else          
            sm_sc.fv_lambda_arr
            (
              tb_a_node_tar.node_no, 
              tb_a_sour.node_fn_type, 
              tb_a_sour.a_back_o_s_1st, 
              tb_a_sour.a_back_o_s_2nd,
              tb_a_sour.node_fn_asso_value
            )
        end
    from cte_aggr_o tb_a_sour
    where tb_a_node_tar.node_type = 'nn_main'
      and tb_a_node_tar.node_no = tb_a_sour.node_no
      and tb_a_node_tar.sess_id = v_sess_id
    ;
  end loop;
  
  select 
    node_depdt_vals into v_ret 
  from sm_sc.__vt_tmp_node_node2node
  where sess_id = v_sess_id
    and node_no = i_end_node_no
  ;  
  delete from sm_sc.__vt_tmp_node_node2node
  where sess_id = v_sess_id
  ;  
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select sm_sc.fv_nn_node2node_val(2022030501, 103010025, 106020001)
-- drop function if exists sm_sc.fv_nn_node_pick_depdt_idx(varchar(64), int4multirange, float[]);
create or replace function sm_sc.fv_nn_node_pick_depdt_idx
(
  i_node_fn_type           varchar(64),
  i_indepdt_idx            int4multirange[],
  i_node_fn_asso_value     float[]
)
returns     int4multirange[]                    -- 返回采样后的原始序号集合
as 
$$
-- declare 
begin  
  -- 参看 readme 文档，算子说明的“训练集输入集合是否变化”
  if i_node_fn_type = '04_new'
  then 
    return sm_sc.fv_new(i_indepdt_idx, i_node_fn_asso_value[1 : ] :: int[]);
  elsif i_node_fn_type = '04_slice_y' 
  then 
    return 
    (
      select 
        array_agg
        (
          sm_sc.fv_idx_samp_by_samp
          (
            i_indepdt_idx, 
            int4multirange(int4range(i_node_fn_asso_value[1][tb_a_cur_range_no] :: int, coalesce(i_node_fn_asso_value[2][tb_a_cur_range_no], i_node_fn_asso_value[1]) :: int + 1, '[)'))
          )
          order by tb_a_cur_range_no
        )
      from generate_series(1, array_length(i_node_fn_asso_value, 2)) tb_a_cur_range_no(tb_a_cur_range_no)
    );
  elsif i_node_fn_type = '04_sample_y' 
  then 
    return 
    (
      select
        sm_sc.fa_concat_y
        (
          sm_sc.fv_sample_y
          (
            (select a_ele from unnest(i_indepdt_idx) tb_a_range(a_range), generate_series(lower(a_range), upper(a_range) - 1) tb_a_ele(a_ele))
          , i_node_fn_asso_value[1][1]
          , i_node_fn_asso_value[2][1]
          , int4range(i_node_fn_asso_value[3][tb_a_cur_range_no], i_node_fn_asso_value[4][tb_a_cur_range_no] + 1, '[)')
          )
          order by tb_a_cur_range_no
        )
      from generate_series(1, array_length(i_node_fn_asso_value, 2)) tb_a_cur_range_no(tb_a_cur_range_no)
    );
  -- -- elsif i_node_fn_type = '04_rand_pick_y'    -- 不支持反向传播
  -- -- then 
  -- --   return
  -- --     sm_sc.fv_idx_samp_by_samp
  -- --     (
  -- --       i_indepdt_idx, 
  -- --       (
  -- --         select array_agg(int4range(a_ele :: int, a_ele :: int, '[]')) 
  -- --         from unnest(tb_a_indepdt_fore.bi_opr_input_1st[ : ][array_length(tb_a_indepdt_fore.bi_opr_input_1st, 2) : ]) tb_a_ele(a_ele)   -- 规约在 fv_lambda_arr 中， rand_pick_y, rand_pick_x 算子的最后一列/行，为 rand 到的切片序号
  -- --       )
  -- --     );
  -- -- elsif i_node_fn_type = '06_aggr_mx_concat_y'   -- 不支持反向传播
  -- -- then 
  -- --   return sm_sc.fa_range_or(i_indepdt_idx);
  else 
    return i_indepdt_idx;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_nn_node_pick_depdt_idx
--   (
--     '04_slice_y'
--   , array[int4multirange(int4range(2, 5, '[]')), int4multirange(int4range(7, 11, '[]'))]
--   , array[[3.0], [6.0]]
--   )
-- drop function if exists sm_sc.fv_nn_none(anyarray);
create or replace function sm_sc.fv_nn_none
(
  i_indepdt    anyarray
)
returns anyarray
as
$$
-- declare 
begin
  return i_indepdt;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select sm_sc.fv_nn_none(array[1.0])
-- drop function if exists sm_sc.fv_nn_weight_len(varchar(64), int, float[], int[]);
create or replace function sm_sc.fv_nn_weight_len
(
  i_node_fn_type             varchar(64)      ,
  i_path_ord_no              int              ,
  i_node_fn_asso_value       float[]          ,
  i_node_depdt_val_len       int[]       default null
)
returns int[]
as
$$
-- declare 
begin
  return 
    case 
      when i_node_fn_type = '05_conv_2d_grp_x' and i_path_ord_no = 2
        then 
          -- 卷积核扁平化
          case 
            when i_node_fn_asso_value[12] :: int :: boolean is true
              -- 配套偏移量
              then array[1, i_node_fn_asso_value[2] :: int * i_node_fn_asso_value[3] :: int + 1]
            else array[1, i_node_fn_asso_value[2] :: int * i_node_fn_asso_value[3] :: int]
          end    
      when i_node_fn_type = '01_prod_mx' and i_path_ord_no = 2
        -- then i_node_fn_asso_value[4] || (i_node_fn_asso_value[5] || i_node_fn_asso_value[2 : 3])    -- 参看字典表 enum_name = 'node_fn_asso_value' and enum_group = '01_prod_mx'
        then i_node_depdt_val_len[ : array_length(i_node_depdt_val_len, 1) - 2] || i_node_fn_asso_value[2 : 3]
      when i_node_fn_type = '05_conv_2d' and i_path_ord_no = 2
        -- then i_node_fn_asso_value[11] || (i_node_fn_asso_value[12] || i_node_fn_asso_value[2 : 3])    -- 参看字典表 enum_name = 'node_fn_asso_value' and enum_group = '05_conv_2d'
        then i_node_depdt_val_len[ : array_length(i_node_depdt_val_len, 1) - 2] || i_node_fn_asso_value[2 : 3]
      when i_node_fn_type = '05_conv_2d' and i_path_ord_no = 3
        then i_node_depdt_val_len[ : array_length(i_node_depdt_val_len, 1) - 2] || array[1, 1]
      -- else array[[1.0]]
    end
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;


-- select 
--   sm_sc.fv_nn_weight_len
--   (
--     '01_prod_mx'
--   , 2
--   , array[2.0, 3.0, 5.0]
--   , array[8, 10, 2, 5]
--   )
-- drop function if exists sm_sc.ft_bpe(text[], int, int, int, int, intboolean);
create or replace function sm_sc.ft_bpe
(
  i_seq_code                             text[]                  -- 原始输入序列数组
, i_threshold_loop_cnt                   int                     -- 约束：迭代合并 token 的次数做为循环结束的阈值
, i_threshold_token_list_cnt             int       default 0     -- 约束：token list 的字典规模做为循环结束的阈值，缺省 0 代表不约束循环结束阈值
, i_threshold_seq_token_len              int       default 0     -- 期望：编码后，每个 seq 包含 token 数量的最大值做为循环结束的阈值，缺省 0 代表不约束循环结束阈值
, i_threshold_avg_token_len_4_seq        int       default 0     -- 初步评估：对于某一个 seq 来说，当包含的 token 数已经小于 i_threshold_seq_token_len，那么判断：其包含的 token 的平均长度超过该阈值，则不再参与 bpe。缺省 0 代表不做此判断
, i_is_match_begin_end                   boolean   default false -- 是否区分匹配 seq 的起始和终止。如果区分，则要对 seq 中的两个逃逸字符 '^$' 做特殊处理
)
returns table 
(
  o_ret_seq_code              text[]                  -- 输出序列数组。元素顺序与原始输入一一对应
, o_ret_token_list            text[]                  -- token 清单，字典表
)
as
$$
declare 
  v_sess_id                   bigint        :=  lower(sm_sc.fv_get_global_seq());
  v_token_ex                  text          ;    -- 每次循环寄存被发现的频率最高的 2gram，带有一个空格
  v_token_ex_freq             int           ;    -- 每次循环寄存被发现的频率最高的 2gram 的频率次数(该频次仅用于单次审查，可能不包含已经满足阈值的成熟 seq 的 token 片段)
  v_seq_code_no_arr           int[]         ;    -- 每次循环寄存被 token 合并更新的 seq_no 清单
  v_ret_seq_token             text[]        ;    -- 返回结构一
  v_ret_token_list            text[]        ;    -- 返回结构二
  
  v_token_merge_cnt_cur       int           := 1;
  -- v_token_ex_freq_ex          int           ;    -- 被发现的频率最高的 2gram 的频率次数(实际次数，包含包含已经满足阈值的成熟 seq 的 token 片段)
begin 
  -- 审计三个逃逸字符"^ $"(起始、空格、结尾)
  if (|@|`| (i_seq_code ~` case when i_is_match_begin_end then '[\^ \$]' :: text else ' ' end))
  then 
    raise exception 'can''t support these 3 char "^ $" in sequence_code.  ';
  end if;

  -- 原始 seq 导入 buff 表
  insert into sm_sc.__vt_tmp_bpe_seq(sess_id, seq_code, seq_code_no)
  select 
    v_sess_id
  , case 
      when i_is_match_begin_end 
        then '^' || i_seq_code[a_cur] || '$' 
      else i_seq_code[a_cur] 
    end
  , a_cur
  from generate_series(1, array_length(i_seq_code, 1)) tb_a_cur(a_cur)
  ;
  
  -- 初始化其他信息：空格为间隔符，切分 seq；初始化序列被分成 token 的数量
  update sm_sc.__vt_tmp_bpe_seq
  set 
    seq_token     = rtrim(regexp_replace(seq_code, '(.)', '\1 ', 'g'))
  , seq_split_cnt = length(seq_code)
  ;
  
  -- 初始化 token list
  insert into sm_sc.__vt_tmp_bpe_token_list(sess_id, token, token_freq)
  select 
    v_sess_id
  , a_token
  , count(a_token) -- as token_freq
  from sm_sc.__vt_tmp_bpe_seq, regexp_split_to_table(seq_token, ' ') tb_a(a_token)
  group by a_token
  ;
  
  -- 初始化 2gram
  update sm_sc.__vt_tmp_bpe_seq
  set 
    seq_token_2gram = 
      (
        select 
          array_agg(a_split_arr_token[a_cur] || ' ' || a_split_arr_token[a_cur + 1] order by a_cur)
        from regexp_split_to_array(seq_token, ' ') as a_split_arr_token
          , generate_series(1, array_length(a_split_arr_token, 1) - 1) tb_a_cur(a_cur)
      )
  where sess_id = v_sess_id
  ;
  
  -- 开始迭代更新
  --   每次都全量统计，不是很优雅的实现，
  --   如果增量统计实现，过滤代价也不小。
  --   如果 seq 和 token 规模特别大，也可以考虑分组 bpe 和 token 采样计算出现频率等策略
  while 
    exists
    (
      select  from sm_sc.__vt_tmp_bpe_seq
      where sess_id = v_sess_id 
        and seq_split_cnt > greatest(2, i_threshold_seq_token_len)
    )
    and (v_token_merge_cnt_cur <= i_threshold_loop_cnt or i_threshold_loop_cnt = 0)
    and 
      (
        (select count(token) from sm_sc.__vt_tmp_bpe_token_list where token_freq > 0) < i_threshold_token_list_cnt 
        or i_threshold_token_list_cnt = 0
      )
  loop         
    -- 找到频率最高，长度较长的 2gram 加入 token 清单
    with
    -- -- cte_filter_seq as 
    -- -- (
    -- --   select 
    -- --     seq_token_2gram
    -- --   from sm_sc.__vt_tmp_bpe_seq
    -- --   where sess_id = v_sess_id
    -- --     and seq_token_2gram is not null
    -- --     and 
    -- --       (
    -- --         i_threshold_seq_token_len = 0 
    -- --         or i_threshold_avg_token_len_4_seq = 0
    -- --         -- 参与 bpe 的条件：要么包含 token 还有很多；要么这个 seq 里，各个 token 的包含的原始子 token 平均还很少
    -- --         or seq_split_cnt > i_threshold_seq_token_len    
    -- --         or length(seq_code) / (greatest(seq_split_cnt, 2) - 1) <= i_threshold_avg_token_len_4_seq   -- 减一：还能经得起一次合并，仍然在阈值范围内
    -- --       )
    -- -- ), 
    -- -- cte_low_aggr_2gram as 
    -- -- (
    -- --   select 
    -- --     a_token_ex
    -- --   , count(a_token_ex) as a_token_ex_freq
    -- --   from cte_filter_seq, unnest(seq_token_2gram) tb_a(a_token_ex)
    -- --   group by a_token_ex
    -- -- )
    cte_low_aggr_2gram as 
    (
      select 
        a_token_ex
      , count(a_token_ex) as a_token_ex_freq
      , max(seq_split_cnt) as a_max_seq_split_cnt
      , min(least(seq_token_2gram[1], seq_token_2gram[2])) as a_min_sub_token_len
      from sm_sc.__vt_tmp_bpe_seq, unnest(seq_token_2gram) tb_a(a_token_ex)
      where sess_id = v_sess_id
        and seq_token_2gram is not null
        and 
          (
            i_threshold_seq_token_len = 0 
            or i_threshold_avg_token_len_4_seq = 0
            -- 参与 bpe 的条件：要么包含 token 还有很多；要么这个 seq 里，各个 token 的包含的原始子 token 平均还很少
            or seq_split_cnt > i_threshold_seq_token_len    
            or length(seq_code) / (greatest(seq_split_cnt, 2) - 1) <= i_threshold_avg_token_len_4_seq   -- 减一：还能经得起一次合并，仍然在阈值范围内
          )
      group by a_token_ex
    )
    select 
      a_token_ex          
    , a_token_ex_freq   
    into 
      v_token_ex
    , v_token_ex_freq
    from cte_low_aggr_2gram
    order by     -- tf-idf 为大原则，尽量满足阈值为策略
      a_token_ex_freq desc
    , greatest(a_max_seq_split_cnt, i_threshold_seq_token_len) desc
    , length(a_token_ex) desc
    , a_min_sub_token_len desc
    limit 1
    ;

    -- v_token_ex_freq_ex := -- v_token_ex_freq
    --   (
    --     select 
    --       count(a_token)
    --     from sm_sc.__vt_tmp_bpe_seq tb_a_seq, unnest(tb_a_seq.seq_token_2gram) tb_a_token(a_token)
    --     where v_token_ex = any(tb_a_seq.seq_token_2gram)  -- 可以 gin + tsvector 优化
    --       and a_token = v_token_ex
    --   )
    -- ;

-- -- -- debug
-- -- raise notice 'v_token_merge_cnt_cur: %; v_token_ex: %; v_token_ex_freq: %;', v_token_merge_cnt_cur, v_token_ex, v_token_ex_freq;   -- v_token_ex_freq_ex

    exit when v_token_ex is null;
    
    insert into sm_sc.__vt_tmp_bpe_token_list as tb_a_tar
    (
      sess_id
    , token
    , token_freq
    )
    values 
    (
      v_sess_id
    , replace(v_token_ex, ' ', '')
    , v_token_ex_freq        -- v_token_ex_freq_ex
    )
    on conflict(sess_id, token) do update set
      token_freq = tb_a_tar.token_freq + excluded.token_freq
    ;
    
    -- 被并入新 token 的 sub-token 的次数要从 sub-token 总次数中减计
    update sm_sc.__vt_tmp_bpe_token_list
    -- 如果只“更新满足阈值的记录”，那么 - v_token_ex_freq，否则 - v_token_ex_freq_ex
    set token_freq = token_freq - v_token_ex_freq          -- token_freq - v_token_ex_freq_ex
    where token in (split_part(v_token_ex, ' ', 1), split_part(v_token_ex, ' ', 2))
      and sess_id = v_sess_id
    ;
    
    -- 更新含有新 token 的 seq_token
    with
    cte_upd_seq_token as 
    (
      update sm_sc.__vt_tmp_bpe_seq
      set 
        seq_token = 
          regexp_replace
          (
            seq_token
          , ('(?<=^| )' || regexp_replace(v_token_ex, '([\^\$])', '\\\1', 'g') || '(?= |$)')
          , replace(v_token_ex, ' ', '')
          , 'g'
          )                                                                                                 -- replace(seq_token, v_token_ex, replace(v_token_ex, ' ', ''))
      where sess_id = v_sess_id
        and v_token_ex = any(seq_token_2gram)   -- and seq_token ~ ('(?<=^| )' || regexp_replace(v_token_ex, '([\^\$])', '\\\1', 'g') || '(?= |$)')    -- seq_token like ('%' || v_token_ex || '%')
        -- 下面过滤条件注释掉，表示不只是“更新满足阈值的记录”
        -- and 
        --   (
        --     i_threshold_seq_token_len = 0 
        --     or i_threshold_avg_token_len_4_seq = 0
        --     -- 参与 bpe 的条件：要么包含 token 还有很多；要么这个 seq 里，各个 token 的包含的原始子 token 平均还很少
        --     or seq_split_cnt > i_threshold_seq_token_len    
        --     or length(seq_code) / (greatest(seq_split_cnt, 2) - 1) <= i_threshold_avg_token_len_4_seq   -- 减一：还能经得起一次合并，仍然在阈值范围内
        --   )
      returning seq_code_no
    )
    select 
      array_agg(seq_code_no) into v_seq_code_no_arr
    from cte_upd_seq_token
    ;
    
    -- 再更新序列被分成 token 的数量，用于判断循环终止
    update sm_sc.__vt_tmp_bpe_seq tb_a_tar
    set 
      seq_split_cnt = length(seq_token) - length(replace(seq_token, ' ', '')) + 1
    , seq_token_2gram = 
        (
          select 
            array_agg(a_split_arr_token[a_cur] || ' ' || a_split_arr_token[a_cur + 1] order by a_cur)
          from regexp_split_to_array(seq_token, ' ') as a_split_arr_token
            , generate_series(1, array_length(a_split_arr_token, 1) - 1) tb_a_cur(a_cur)
        )
    from unnest(v_seq_code_no_arr) tb_a_sour(seq_code_no)
    where tb_a_tar.sess_id = v_sess_id
      and tb_a_tar.seq_code_no = tb_a_sour.seq_code_no
    ;
    
    -- 循环次数计数，用于判断循环终止
    v_token_merge_cnt_cur := v_token_merge_cnt_cur + 1;
  end loop;
  
  -- 收尾返回
  v_ret_seq_token := 
    (
      select 
        array_agg(seq_token order by seq_code_no) 
      from sm_sc.__vt_tmp_bpe_seq 
      where sess_id = v_sess_id 
    )
  ;
  v_ret_token_list := 
    (
      select 
        array_agg(token) 
      from sm_sc.__vt_tmp_bpe_token_list 
      where sess_id = v_sess_id 
        and token_freq > 0
        and token not in ('^', '$')
    )
  ;
  delete from sm_sc.__vt_tmp_bpe_seq where sess_id = v_sess_id;
  delete from sm_sc.__vt_tmp_bpe_token_list where sess_id = v_sess_id;
  
  return query
    select v_ret_seq_token, v_ret_token_list
  ;
end 
$$
language plpgsql volatile 
parallel unsafe
;

-- with 
-- cte_seq_code as
-- (
--   select 
--     cn_chr
--   , row_number(order by cn_chr) over() as a_no
--   , cn_write_ord_no 
--   from sm_sc.tb_lang_cn_chr_dic 
--   where cn_write_ord_no is not null
--   -- order by cn_chr 
--   -- limit 100
-- ),
-- cte_bpe_arr as 
-- (
--   select 
--     o_ret_seq_code  
--   , o_ret_token_list
--   from 
--     sm_sc.ft_bpe
--     (
--       (
--         select array_agg(cn_write_ord_no) 
--         from cte_seq_code
--       )
--     , 5000   -- 循环次数
--     , 2048   -- 字典规模                                                           -- 字典里多少类偏旁
--     , 4      -- bpe 后，seq 长度，包含 token 个数                                  -- 一个汉字(seq)包含多少个偏旁结构
--     , 3      -- bpe 后，一个 seq 中，各个 token 包含原始 sub-token 的个数均值      -- 偏旁笔划数
--     )
-- )
-- select 
--   cn_chr
-- , o_ret_seq_code[a_no]
-- from cte_bpe_arr, cte_seq_code


-- -- -- delete from sm_sc.__vt_tmp_bpe_seq;
-- -- -- delete from sm_sc.__vt_tmp_bpe_token_list;
-- -- 
-- -- select seq_split_cnt, count(seq_split_cnt)
-- --   -- * 
-- -- from sm_sc.__vt_tmp_bpe_seq 
-- -- 
-- -- group by seq_split_cnt
-- -- limit 50
-- -- 
-- -- select * from sm_sc.__vt_tmp_bpe_token_list 
-- -- -- where token_freq > 0
-- -- limit 50

-- create table __vt_tmp_bpe_seq_1101 as select * from sm_sc.__vt_tmp_bpe_seq;
-- create table __vt_tmp_bpe_token_list_1101 as select * from sm_sc.__vt_tmp_bpe_token_list;
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ft_onehot_dic(anyarray, float, float);
create or replace function sm_sc.ft_onehot_dic
(
  i_array              anyarray
, i_upper              float    default    1.0 :: float
, i_lower              float    default    0.0
)
returns table 
(
  o_ele     anyelement
, o_onehot  float[]    
)
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) = 2 and array_length(i_array, 2) = 1
  then 
    i_array := sm_sc.fv_mx_ele_2d_2_1d(i_array);
  elsif array_ndims(i_array) <> 1
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
  
  return query
    with 
    cte_ele as
    (
      select 
        a_no
      , rank() over(order by i_array[a_no]) :: int as a_no_rank
      from generate_series(1, array_length(i_array, 1)) tb_a(a_no)
    ),
    cte_dist_ele as
    (
      select 
        a_no_rank 
      , row_number() over(order by a_no_rank) :: int as a_no_x
      , i_array[min(a_no)]    as a_ele
      from cte_ele
      group by a_no_rank
    ), 
    cte_dist_cnt as 
    (
      select 
        count(a_no_x) :: int as a_dist_cnt
      from cte_dist_ele
    )
    select 
      a_ele
    , array_fill(i_lower, array[a_no_x - 1]) || i_upper || array_fill(i_lower, array[a_dist_cnt - a_no_x])
    from cte_dist_ele, cte_dist_cnt
    order by a_no_rank
  ;

end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select * 
-- from 
--   sm_sc.ft_onehot_dic
--   (
--     array[1,2,3,1,4,3,5,6]
--   );

-- select  * 
-- from 
--   sm_sc.ft_onehot_dic
--   (
--     array[array[1]
--         , array[10]
--         , array[100]
--         , array[-1]
--         , array[10]
--         , array[-10]
--          ]
--   );

-- select  * 
-- from 
--   sm_sc.ft_onehot_dic
--   (
--     array[1,2,3,4,5,6]
--     ,2.0
--   );

-- select  * 
-- from 
--   sm_sc.ft_onehot_dic
--   (
--     array[1,2,3,4,5,6]
--     , 1.0
--     , -1.0
--   );

-- select  * 
-- from 
--   sm_sc.ft_onehot_dic
--   (
--     array[array[1]
--         , array[10]
--         , array[100]
--         , array[-1]
--         , array[-10]
--          ]
--     , 1.0
--     , -1.0
--   );

-- -- 参考
-- --   https://blog.csdn.net/qq_36653505/article/details/81701181

-- -- create sequence if not exists huffman_seq start 1000000000;

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_huffman(float[]);
create or replace function sm_sc.fv_huffman
(
  i_weights     float[]
)
returns varbit[]
as
$$
declare 
  v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);
  v_ret       varbit[];
begin

  -- -- create temp table if not exists sm_sc.__vt_tmp_huffman
  -- -- (
  -- --   sess_id         char(32)                                          ,    -- 回话id
  -- --   node_no         bigint       default nextval('huffman_seq')       ,    -- 自然序列编号
  -- --   is_compared     boolean      default false                        ,    -- 是否已经比较入树
  -- --   node_weight     float                                    ,    -- 权值
  -- --   is_org          boolean                                           ,    -- 是否是原始 input 的节点
  -- --   tree_code       varbit                                            ,    -- 单次比较后入树结果，左子树为0，右子树为1，
  -- --   father_node_no  bigint                                               ,    -- 父节点
  -- --   primary key (sess_id, node_no)
  -- -- )
  -- -- ;
  -- -- 
  -- -- create index if not exists __idx_huffman_node_weight
  -- --   on sm_sc.__vt_tmp_huffman(sess_id, is_compared, node_weight);  
  -- -- create index if not exists __idx_huffman_father
  -- --   on sm_sc.__vt_tmp_huffman(sess_id, father_node_no);
  
  -- i_weights 当做初始 node
  insert into sm_sc.__vt_tmp_huffman
  (
    sess_id             ,
    node_no             ,
    is_compared         ,
    node_weight         ,
    is_org
  )
  select 
    v_sess_id                  as sess_id          ,
    row_number() over()        as node_no          ,
    false                      as is_compared      ,
    a_node_weight              as node_weight      ,
    true                       as is_org
  from unnest(i_weights) tb_a(a_node_weight)
  ;
  
  while exists (select from sm_sc.__vt_tmp_huffman where sess_id = v_sess_id and is_compared = false having count(*) > 1)
  loop
    with
    -- 根据 weight 排序，查找 weight 最小的两个 
    cte_min_2 as
    (
      select 
        node_no                                             ,
        (~ (row_number() over() - 1) :: bit) :: varbit    as tree_code ,
        node_weight
      from sm_sc.__vt_tmp_huffman
      where sess_id = v_sess_id
        and is_compared is false
      order by node_weight 
      limit 2
    ),
    -- 插入新的父节点记录
    cte_insert as
    (
      insert into sm_sc.__vt_tmp_huffman
      (
        sess_id,
        is_compared,
        node_weight,
        is_org
      )
      select 
        v_sess_id        as  sess_id       ,
        false            as  is_compared   ,
        sum(node_weight) as  node_weight   ,
        false            as  is_org
      from cte_min_2
      returning node_no
    )
    -- 更新入树标记和父节点路径关系
    update sm_sc.__vt_tmp_huffman tb_a_sour
    set 
      is_compared = true,
      tree_code = tb_a_min.tree_code,
      father_node_no = tb_a_insert.node_no
    from cte_min_2 tb_a_min, cte_insert tb_a_insert
    where tb_a_min.node_no = tb_a_sour.node_no
      and tb_a_sour.sess_id = v_sess_id
    ;
  end loop;

  -- 从顶点，迭代树的深度，拼接 huffman 编码，最终输出
  with recursive
  cte_nodes as
  (
    select 
      node_no,
      '' :: varbit as huffman_tree_code,
      is_org
    from sm_sc.__vt_tmp_huffman
    where sess_id = v_sess_id
      and is_compared is false 
    union all 
    select 
      tb_a_incre.node_no,
      tb_a_main.huffman_tree_code || tb_a_incre.tree_code as huffman_tree_code,
      tb_a_incre.is_org
    from cte_nodes tb_a_main
    inner join sm_sc.__vt_tmp_huffman tb_a_incre
      on tb_a_incre.sess_id = v_sess_id
        and tb_a_incre.father_node_no = tb_a_main.node_no
  )
  select 
    array_agg(huffman_tree_code order by node_no) into v_ret
  from cte_nodes
  where is_org is true
  ;
  
  delete from sm_sc.__vt_tmp_huffman;
  return v_ret;

end
$$
language plpgsql volatile 
parallel unsafe
cost 100;

-- -- set force_parallel_mode = 'off';
-- select sm_sc.fv_huffman(array[1, 3, 8, 2, 9, 13, 19])

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_onehot(anyarray, float, float);
create or replace function sm_sc.fv_onehot
(
  i_array              anyarray,
  i_upper               float    default    1.0 :: float  ,
  i_lower                float    default    0.0
)
returns float[]     -- 输出列序与 i_array 元素原位置一致
as
$$
-- declare 
begin
  -- 审计二维长度
  if array_ndims(i_array) = 2 and array_length(i_array, 2) = 1
  then 
    i_array := sm_sc.fv_mx_ele_2d_2_1d(i_array);
  elsif array_ndims(i_array) <> 1
  then
    raise exception 'no method for such length!  Dims: %;', array_dims(i_array);
  end if;
    
  return 
  (
    with 
    cte_ele as
    (
      select 
        a_no
      , rank() over(order by i_array[a_no]) :: int as a_no_rank
      from generate_series(1, array_length(i_array, 1)) tb_a(a_no)
    ),
    cte_dist_ele as
    (
      select 
        a_no_rank 
      , array_agg(a_no) as a_nos_per_enum
      from cte_ele
      group by a_no_rank
    ),
    cte_onehot as
    (
      select 
        a_no
      , array_agg(case when cte_ele.a_no = any(cte_dist_ele.a_nos_per_enum) then i_upper else i_lower end order by cte_dist_ele.a_no_rank) as a_onehot
      from cte_ele, cte_dist_ele
      group by a_no
    )
    select array_agg(a_onehot order by a_no) from cte_onehot
  );
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_onehot
--   (
--     array[1,2,3,1,4,3,5,6]
--   );

-- select sm_sc.fv_onehot
--   (
--     array[array[1]
--         , array[10]
--         , array[100]
--         , array[-1]
--         , array[10]
--         , array[-10]
--          ]
--   );

-- select sm_sc.fv_onehot
--   (
--     array[1,2,3,4,5,6]
--     ,2.0
--   );

-- select sm_sc.fv_onehot
--   (
--     array[1,2,3,4,5,6]
--     , 1.0
--     , -1.0
--   );

-- select sm_sc.fv_onehot
--   (
--     array[array[1]
--         , array[10]
--         , array[100]
--         , array[-1]
--         , array[-10]
--          ]
--     , 1.0
--     , -1.0
--   );

-- drop function if exists sm_sc.fv_dloss_dz_cross_entropy(, float[], float[]);
create or replace function sm_sc.fv_dloss_dz_cross_entropy
(
  i_z               float[]          ,      -- 训练输出预测值
  i_true            float[]                 -- 训练集真实值
)
returns float[]
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_z) > 4 
      or array_ndims(i_true) > 4 
      or array_dims(i_z) <> array_dims(i_true) 
    then
      raise exception 'no method for such length!  array_dims(i_z): %; array_dims(i_true): %;', array_dims(i_z), array_dims(i_true);
    end if;
  end if;

  return 
    -` i_true /` sm_sc.fv_ele_replace((i_z *` array_length(i_z, 1) :: float) :: float[], array[0.0 :: float], 1.0e-128 :: float)   -- esp: 1.0e-308 :: float -- 1.0 / 1.0e308 :: float  此处308刚好不溢出，但进一步运算可能溢出
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_dloss_dz_cross_entropy(array[array[1, 2, 3], array[2, 3, 4]], array[array[1.1, 2.1, 3.2], array[2.2, 3.3, 4.2]])
-- drop function if exists sm_sc.fv_dloss_dz_cross_entropy_true_onehot_idx(, float[], float[]);
create or replace function sm_sc.fv_dloss_dz_cross_entropy_true_onehot_idx
(
  i_z               float[]          ,      -- 训练输出预测值
  i_true_onehot_idx float[]                 -- 训练集真实值
)
returns float[]
as
$$
declare 
  v_len_z           int[]     :=   (select array_agg(array_length(i_z, a_no) order by a_no) from generate_series(1, array_ndims(i_z)) tb_a(a_no));
  v_true_onehot_idx int[]     :=   (select array_agg(array_length(i_true_onehot_idx, a_no) order by a_no) from generate_series(1, array_ndims(i_true_onehot_idx)) tb_a(a_no));
  v_ret             float[]   :=   array_fill(0.0, v_len_z);
  v_cur_y           int       ;
  v_cur_x           int       ;
  v_cur_x3          int       ;
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_z) > 4 
      or array_ndims(i_true_onehot_idx) > 4 
      or array_length(i_true_onehot_idx, 1) <> 1
      or v_len_z[ : array_ndims(i_z) - 1] <> v_true_onehot_idx[ : array_ndims(i_true_onehot_idx) - 1]
    then
      raise exception 'no method for such length!  Y_Dims: %; Z_Dims: %;', array_dims(i_z), array_dims(i_true_onehot_idx);
    end if;
  end if;

  -- -- return 
  -- --   -- -` i_true /` sm_sc.fv_ele_replace((i_z *` array_length(i_z, 1) :: float) :: float[], array[0.0 :: float], 1.0e-128 :: float)   -- esp: 1.0e-308 :: float -- 1.0 / 1.0e308 :: float  此处308刚好不溢出，但进一步运算可能溢出
  -- -- ;
  
  if array_ndims(i_z) = 1
  then 
    v_ret[i_true_onehot_idx[1]] :=  
      -1.0 / coalesce(nullif(i_z[i_true_onehot_idx[1]], 0.0), 1.0e-128 :: float)
    ;
  elsif array_ndims(i_z) = 2
  then 
    for v_cur_y in 1 .. array_length(i_true_onehot_idx, 1)
    loop
      v_ret[v_cur_y][i_true_onehot_idx[v_cur_y][1]] :=  
        -1.0 / coalesce(nullif(i_z[v_cur_y][i_true_onehot_idx[v_cur_y][1]], 0.0), 1.0e-128 :: float)
      ;
    end loop;
  elsif array_ndims(i_z) = 3
  then 
    for v_cur_y in 1 .. array_length(i_true_onehot_idx, 1)
    loop
      for v_cur_x in 1 .. array_length(i_true_onehot_idx, 2)
      loop
          v_ret[v_cur_y][v_cur_x][i_true_onehot_idx[v_cur_y][v_cur_x][1]] :=  
            -1.0 / coalesce(nullif(i_z[v_cur_y][v_cur_x][i_true_onehot_idx[v_cur_y][v_cur_x][1]], 0.0), 1.0e-128 :: float)
          ;
      end loop;
    end loop;
  elsif array_ndims(i_z) = 4
  then 
    for v_cur_y in 1 .. array_length(i_true_onehot_idx, 1)
    loop
      for v_cur_x in 1 .. array_length(i_true_onehot_idx, 2)
      loop
        for v_cur_x3 in 1 .. array_length(i_true_onehot_idx, 3)
        loop
            v_ret[v_cur_y][v_cur_x][v_cur_x3][i_true_onehot_idx[v_cur_y][v_cur_x][v_cur_x3][1]] :=  
              -1.0 / coalesce(nullif(i_z[v_cur_y][v_cur_x][v_cur_x3][i_true_onehot_idx[v_cur_y][v_cur_x][v_cur_x3][1]], 0.0), 1.0e-128 :: float)
            ;
        end loop;
      end loop;
    end loop;
  end if;
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_dloss_dz_cross_entropy_true_onehot_idx(array[[0.1, 0.2, 0.7], [0.2, 0.3, 0.5]], array[[3], [2]])
-- drop function if exists sm_sc.fv_dloss_dz_least_square(, float[], float[]);
create or replace function sm_sc.fv_dloss_dz_least_square
(
  i_z               float[]          ,      -- 训练输出预测值
  i_true            float[]                 -- 训练集真实值
)
returns float[]
as
$$
-- declare 
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_z) > 4 
      or array_ndims(i_true) > 4 
      or array_length(i_z, 1) <> array_length(i_true, 1) 
      or array_length(i_z, 2) <> array_length(i_true, 2)
      or array_length(i_z, 3) <> array_length(i_true, 3)
      or array_length(i_z, 4) <> array_length(i_true, 4)
    then
      raise exception 'no method for such length!  Z_Dims: %; True_Dims: %;', array_dims(i_z), array_dims(i_true);
    end if;
  end if;

  if sm_sc.fv_aggr_slice_max(i_true) is null
  then 
    return 
      i_z /` sqrt(array_length(i_z, 1) :: float)    -- 适用于 i_true 已经在 nn 中定义并减掉了的场景
    ;
  else
    return 
      (i_z -` i_true) /` sqrt(array_length(i_z, 1) :: float)
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_dloss_dz_least_square(array[array[1, 2, 3], array[2, 3, 4]], array[array[1.1, 2.1, 3.2], array[2.2, 3.3, 4.2]])
-- 参考 https://zhuanlan.zhihu.com/p/35709485

-- drop function if exists sm_sc.fv_loss_cross_entropy(float[], float[]);
create or replace function sm_sc.fv_loss_cross_entropy
(
  i_z               float[]          ,      -- 训练输出预测值
  i_true            float[]                 -- 训练集真实值
)
returns float
as
$$
-- declare
  
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_z) > 4 
      or array_ndims(i_true) > 4 
      or array_dims(i_z, 1) <> array_dims(i_true, 1) 
    then
      raise exception 'no method for such length!  Y_Dims: %; Z_Dims: %;', array_dims(i_z), array_dims(i_true);
    end if;
  end if;
 
  -- log(0.0)处理参考: https://blog.csdn.net/wekings/article/details/123578464
  return 
    - sm_sc.fv_aggr_slice_sum(i_true *` (^!` sm_sc.fv_ele_replace(i_z, array[0.0 :: float], 1.0e-128 :: float))) / array_length(i_z, 1) :: float  -- esp: 1e-323 :: float 此处323刚好不溢出，但进一步运算可能溢出
  ;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_loss_cross_entropy(array[[1.0, 2, 3], [2, 3, 4]], array[[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]])
-- select sm_sc.fv_loss_cross_entropy(array[[[1.0, 2, 3], [2, 3, 4]], [[1, 2, 3], [2, 3, 4]]], array[[[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]], [[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]]])
-- 参考 https://zhuanlan.zhihu.com/p/35709485

-- drop function if exists sm_sc.fv_loss_cross_entropy_true_onehot_idx(float[], float[]);
create or replace function sm_sc.fv_loss_cross_entropy_true_onehot_idx
(
  i_nn_depdt_01                 float[]                 -- 训练输出预测值
, i_true_01_onehot_idx          float[]                 -- 训练集真实值
-- --, i_nn_depdt_02                 float[]                 -- 训练输出预测值
-- , i_true_02                     float[]                 -- 训练集真实值
)
returns float
as
$$
declare
  v_len_z           int[]     :=   (select array_agg(array_length(i_nn_depdt_01, a_no) order by a_no) from generate_series(1, array_ndims(i_nn_depdt_01)) tb_a(a_no));
  v_true_onehot_idx int[]     :=   (select array_agg(array_length(i_true_01_onehot_idx, a_no) order by a_no) from generate_series(1, array_ndims(i_true_01_onehot_idx)) tb_a(a_no));
  v_ret             float[]   :=   array_fill(0.0, v_len_z);
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_nn_depdt_01) > 4 
      or array_ndims(i_true_01_onehot_idx) > 4 
      or array_length(i_true_01_onehot_idx, 1) <> 1
      or v_len_z[ : array_ndims(i_nn_depdt_01) - 1] <> v_true_onehot_idx[ : array_ndims(i_true_01_onehot_idx) - 1]
    then
      raise exception 'no method for such length!  Y_Dims: %; Z_Dims: %;', array_dims(i_nn_depdt_01), array_dims(i_true_01_onehot_idx);
    end if;
  end if;
 
  -- -- return 
  -- --   -- esp: 1e-323 :: float 此处323刚好不溢出，但进一步运算可能溢出
  -- --   - sm_sc.fv_aggr_slice_sum(i_true *` (^!` sm_sc.fv_ele_replace(i_nn_depdt_01, array[0.0 :: float], 1.0e-8 :: float))) / array_length(i_nn_depdt_01, 1) :: float
  -- -- ;

  -- log(0.0)处理参考: https://blog.csdn.net/wekings/article/details/123578464
  if array_ndims(i_nn_depdt_01) = 1
  then 
    return 
      - ln
        (
          -- coalesce(nullif(i_nn_depdt_01[i_true_01_onehot_idx[1]], 0.0), 1.0e-8 :: float)
          least(greatest(i_nn_depdt_01[i_true_01_onehot_idx[1]], 0.0001 / (array_length(i_nn_depdt_01, 1) - 1)), 0.9999)
        ) -- / array_length(i_nn_depdt_01, 1) :: float
    ;
  elsif array_ndims(i_nn_depdt_01) = 2
  then 
    return 
      - 
      (
        select 
          sum
          (
            ln
            (
              -- coalesce(nullif(i_nn_depdt_01[a_y_no][i_true_01_onehot_idx[a_y_no][1]], 0.0), 1.0e-8 :: float)
              least(greatest(i_nn_depdt_01[a_y_no][i_true_01_onehot_idx[a_y_no][1]], 0.0001 / (array_length(i_nn_depdt_01, 2) - 1)), 0.9999)
            )
          )
        from generate_series(1, array_length(i_true_01_onehot_idx, 1)) tb_a_y_no(a_y_no)
      ) 
      / array_length(i_true_01_onehot_idx, 1) :: float
    ;
  elsif array_ndims(i_nn_depdt_01) = 3
  then 
    return 
      - 
      (
        select 
          sum
          (
            ln
            (
              -- coalesce(nullif(i_nn_depdt_01[a_y_no][a_x_no][i_true_01_onehot_idx[a_y_no][a_x_no][1]], 0.0), 1.0e-8 :: float)
              least(greatest(i_nn_depdt_01[a_y_no][a_x_no][i_true_01_onehot_idx[a_y_no][a_x_no][1]], 0.0001 / (array_length(i_nn_depdt_01, 3) - 1)), 0.9999)
            )
          )
        from generate_series(1, array_length(i_true_01_onehot_idx, 1)) tb_a_y_no(a_y_no)
          , generate_series(1, array_length(i_true_01_onehot_idx, 2)) tb_a_x_no(a_x_no)
      ) 
      / array_length(i_true_01_onehot_idx, 1) :: float
    ;
  elsif array_ndims(i_nn_depdt_01) = 4
  then 
    return 
      - 
      (
        select 
          sum
          (
            ln
            (
              -- coalesce(nullif(i_nn_depdt_01[a_y_no][a_x_no][a_x3_no][i_true_01_onehot_idx[a_y_no][a_x_no][a_x3_no][1]], 0.0), 1.0e-8 :: float)
              least(greatest(i_nn_depdt_01[a_y_no][a_x_no][a_x3_no][i_true_01_onehot_idx[a_y_no][a_x_no][a_x3_no][1]], 0.0001 / (array_length(i_nn_depdt_01, 4) - 1)), 0.9999)
            )
          )
        from generate_series(1, array_length(i_true_01_onehot_idx, 1)) tb_a_y_no(a_y_no)
          , generate_series(1, array_length(i_true_01_onehot_idx, 2)) tb_a_x_no(a_x_no)
          , generate_series(1, array_length(i_true_01_onehot_idx, 3)) tb_a_x3_no(a_x3_no)
      ) 
      / array_length(i_true_01_onehot_idx, 1) :: float
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_loss_cross_entropy_true_onehot_idx(array[[0.1, 0.2, 0.7], [0.2, 0.3, 0.5]], array[[3.0], [3.0]])
-- select sm_sc.fv_loss_cross_entropy_true_onehot_idx(array[[[0.1, 0.2, 0.7], [0.1, 0.2, 0.7]], [[0.1, 0.2, 0.7], [0.1, 0.2, 0.7]]], array[[[3.0], [2.0]], [[1.0], [3.0]]])
-- drop function if exists sm_sc.fv_loss_least_square(float[], float[]);
create or replace function sm_sc.fv_loss_least_square
(
  i_z               float[]          ,      -- 训练输出预测值
  i_true            float[]                 -- 训练集真实值
)
returns float
as
$$
-- declare
  
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    -- 审计维度长度约束与对齐
    if array_ndims(i_z) > 4 
      or array_ndims(i_true) > 4 
      or array_dims(i_z, 1) <> array_dims(i_true, 1) 
    then
      raise exception 'no method for such length!  Y_Dims: %; Z_Dims: %;', array_dims(i_z), array_dims(i_true);
    end if;
  end if;
  
  if i_true is null or sm_sc.fv_aggr_slice_max(i_true) is null
  then 
    return 
      (sm_sc.fv_aggr_slice_sum(i_z ^` 2.0 :: float) / array_length(i_z, 1) :: float) ^ 0.5 :: float
    ;
  else
    return 
      (sm_sc.fv_aggr_slice_sum((i_z -` i_true) ^` 2.0 :: float) / array_length(i_z, 1) :: float) ^ 0.5 :: float
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_loss_least_square(array[[1.0, 2, 3], [2, 3, 4]], array[[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]])
-- select sm_sc.fv_loss_least_square(array[[[1.0, 2, 3], [2, 3, 4]], [[1, 2, 3], [2, 3, 4]]], array[[[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]], [[1.1, 2.1, 3.2], [2.2, 3.3, 4.2]]])
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_cos(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_cos
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
declare 
  v_tmp   sm_sc.typ_l_complex  := exp((0.0, 1.0) :: sm_sc.typ_l_complex * i_right);
begin  -- i_right.m_re, i_right.m_im
  return row(cos(i_right.m_re) * ((exp(i_right.m_im) + exp(-i_right.m_im)) / 2.0 :: float), sin(i_right.m_re) * ((exp(-i_right.m_im) - exp(i_right.m_im)) / 2.0 :: float)) :: sm_sc.typ_l_complex;
  -- return (v_tmp + (1.0 / v_tmp)) / (2.0, 0.0);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_cos
--   (
--     (-5.6, 5.6)
--   );
-- select sm_sc.fv_cos
--   (
--     100.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_add(sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_add
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(i_left.m_re + i_right.m_re, i_left.m_im + i_right.m_im)::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_add
--   (
--     (12.3, -12.3),
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_add
--   (
--     100.0 :: float,
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_add
--   (
--     (-45.6, -45.6),
--     100.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_ceil(sm_sc.typ_l_complex, int);
create or replace function sm_sc.fv_opr_ceil
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
declare -- here
  v_balan   float  := 5.0 * power(0.1, i_right + 1) :: float  ;
begin
  return 
    (
      case 
        when i_left.m_re = round(i_left.m_re :: decimal, i_right) 
          then i_left.m_re
        else round((i_left.m_re + v_balan) :: decimal, i_right) 
      end,
      case 
        when i_left.m_im = round(i_left.m_im :: decimal, i_right) 
          then i_left.m_im
        else round((i_left.m_im + v_balan) :: decimal, i_right) 
      end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_ceil
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_ceil(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_ceil
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (ceil(i_right.m_re), ceil(i_right.m_im))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_ceil
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conjugate(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_conjugate
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(i_right.m_re, -i_right.m_im) :: sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conjugate
--   (
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_conjugate
--   (
--     100.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_conjugate_45(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_conjugate_45
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (i_right.m_im, i_right.m_re);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_conjugate_45
--   (
--     (45.6, -45.6)
--   );
-- select sm_sc.fv_opr_conjugate_45
--   (
--     100.0
--   );
-- select sm_sc.fv_opr_conjugate_45
--   (
--     (0, -16.0)
--   );
-- select sm_sc.fv_opr_conjugate_45
--   (
--     0.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_div(sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_div
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return i_left * (~ i_right) * (1.0 :: float/ (i_right.m_re ^ 2 + i_right.m_im ^ 2));
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_div
--   (
--     (12.3, -12.3),
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_div
--   (
--     100.0 :: float,
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_div
--   (
--     (-45.6, -45.6),
--     100.0
--   );

-- ------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_div(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_div
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return sm_sc.fv_opr_div(1.0 :: float, i_right);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_div
--   (
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_div
--   (
--     100.0
--   );
-- -- select sm_sc.fv_opr_div
-- --   (
-- --     0.0
-- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_exp(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_exp
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    exp(i_right.m_re) * (cos(i_right.m_im), sin(i_right.m_im))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_exp
--   (
--     (45.6, -45.6)
--   );
-- select sm_sc.fv_opr_exp
--   (
--     (0, pi()::float)
--   );
-- select sm_sc.fv_opr_exp
--   (
--     (0, -pi()::float/2)
--   );
-- select sm_sc.fv_opr_exp
--   (
--     pi()::float
--   );
-- select sm_sc.fv_opr_exp
--   (
--     (0, -pi()::float)
--   );
-- -- select sm_sc.fv_opr_exp
-- --   (
-- --     0.0
-- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_floor(sm_sc.typ_l_complex, int);
create or replace function sm_sc.fv_opr_floor
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
declare -- here
  v_balan   float  := 5.0 * power(0.1, i_right + 1) :: float  ;
begin
  return 
    (
      case 
        when i_left.m_re = round(i_left.m_re :: decimal, i_right) 
          then i_left.m_re
        else round((i_left.m_re - v_balan) :: decimal, i_right) 
      end,
      case 
        when i_left.m_im = round(i_left.m_im :: decimal, i_right) 
          then i_left.m_im
        else round((i_left.m_im - v_balan) :: decimal, i_right) 
      end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_floor
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_floor(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_floor
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (floor(i_right.m_re), floor(i_right.m_im))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_floor
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_imaginary(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_imaginary
(
  i_right    sm_sc.typ_l_complex
)
returns float
as
$$
-- declare 
begin
  return i_right.m_im;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_imaginary
--   (
--     (45.6, -45.6)
--   );
-- select sm_sc.fv_opr_imaginary
--   (
--     100.0
--   );
-- select sm_sc.fv_opr_imaginary
--   (
--     (0, -16.0)
--   );
-- select sm_sc.fv_opr_imaginary
--   (
--     0.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_ln(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_ln
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    row
    (
      ln(sm_sc.fv_opr_norm(i_right)),
      case 
        when i_right.m_re = 0.0 and i_right.m_im > 0.0 
          then atan('Infinity') ::double precision
        when i_right.m_re = 0.0 and i_right.m_im < 0.0 
          then atan('-Infinity') ::double precision
        when i_right.m_re = 0.0 and i_right.m_im = 0.0 
          then null -- ln(0)
        else
          -- atan(i_right.m_im::double precision / i_right.m_re::double precision)   --   + k * pi()   -- 可以获得多个结果，其中k为整数
          -- 简化在 ( -pi(), pi() ] 半开半闭区间，绝对值最小的取值
          atan(i_right.m_im::double precision / i_right.m_re::double precision) 
          + case 
              when i_right.m_re < 0.0 and i_right.m_im >= 0 then pi()
              when i_right.m_re < 0.0 and i_right.m_im < 0 then -pi()
              else 0.0
            end
       end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_ln
--   (
--     (45.6, -45.6)
--   );
-- select sm_sc.fv_opr_ln
--   (
--     100.0
--   );
-- select sm_sc.fv_opr_ln
--   (
--     (0, -16.0)
--   );
-- -- select sm_sc.fv_opr_ln
-- --   (
-- --     0.0
-- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_log(sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_log
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return sm_sc.fv_opr_ln(i_right) / sm_sc.fv_opr_ln(i_left);
end
$$
language plpgsql stable
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_log
--   (
--     (12.3, -12.3),
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_log
--   (
--     100.0 :: float,
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_log
--   (
--     (-45.6, -45.6),
--     100.0
--   );


-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_mul(sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_mul
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(
              i_left.m_re * i_right.m_re - i_left.m_im * i_right.m_im,
              i_left.m_re * i_right.m_im + i_left.m_im * i_right.m_re
            )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_mul
--   (
--     (12.3, -12.3),
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_mul
--   (
--     100.0 :: float,
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_mul
--   (
--     (-45.6, -45.6),
--     100.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_norm(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_norm
(
  i_right    sm_sc.typ_l_complex
)
returns float
as
$$
-- declare 
begin
  return sqrt(i_right.m_re ^ 2 + i_right.m_im ^ 2);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_norm
--   (
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_norm
--   (
--     100.0
--   );

-- drop function if exists sm_sc.fv_opr_norm(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_norm
(
  i_right     sm_sc.typ_l_complex[]
)
returns float[]
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     float[]   := case array_ndims(i_right) when 2 then array_fill(null :: float, array[array_length(i_right, 1), array_length(i_right, 2)]) when 1 then array_fill(null :: float, array[array_length(i_right, 1)]) else null end;

begin
  -- log(null :: float[][], float) = null :: float[][]
  if array_length(i_right, array_ndims(i_right)) is null
  then 
    return i_right;
  end if;

  -- norm([][])
  if array_ndims(i_right) =  2
  then
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.fv_opr_norm(i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- abs []
  elsif array_ndims(i_right) = 1
  then
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := sm_sc.fv_opr_norm(i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- abs [][][]
  elsif array_ndims(i_right) = 3
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          i_right[v_y_cur][v_x_cur][v_x3_cur] = 
            sm_sc.fv_opr_norm(i_right[v_y_cur][v_x_cur][v_x3_cur])
          ;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  -- abs [][][][]
  elsif array_ndims(i_right) = 4
  then
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] = 
              sm_sc.fv_opr_norm(i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]) 
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return i_right;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.fv_opr_norm
--   (
--     -` array[[row(32.5, -1.5) :: sm_sc.typ_l_complex, row(1.26, -1.5) :: sm_sc.typ_l_complex, row(33.6, -1.5) :: sm_sc.typ_l_complex]
--            , [row(-9.1, -1.5) :: sm_sc.typ_l_complex, row(8.6, -1.5) :: sm_sc.typ_l_complex, row(4.69, -1.5) :: sm_sc.typ_l_complex]]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_pow(sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_pow
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  if i_left = 0.0 :: sm_sc.typ_l_complex
  then 
    return 0.0 :: sm_sc.typ_l_complex;
  else
    return sm_sc.fv_opr_exp(sm_sc.fv_opr_ln(i_left) * i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_pow
--   (
--     (2.3, -2.3),
--     (-5.6, -5.6)
--   );
-- select sm_sc.fv_opr_pow
--   (
--     10.0 :: float,
--     (-4.6, -4.6)
--   );
-- select sm_sc.fv_opr_pow
--   (
--     (-4.6, -4.6),
--     10.0
--   );
-- select (-1.0 :: float, 0.0 :: float)::sm_sc.typ_l_complex ^ pi()::float::sm_sc.typ_l_complex

-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_real(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_real
(
  i_right    sm_sc.typ_l_complex
)
returns float
as
$$
-- declare 
begin
  return i_right.m_re;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_real
--   (
--     (45.6, -45.6)
--   );
-- select sm_sc.fv_opr_real
--   (
--     100.0
--   );
-- select sm_sc.fv_opr_real
--   (
--     (0, -16.0)
--   );
-- select sm_sc.fv_opr_real
--   (
--     0.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_round(sm_sc.typ_l_complex, int);
create or replace function sm_sc.fv_opr_round
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (round(i_left.m_re, i_right), round(i_left.m_im, i_right))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_round
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_round(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_round
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (round(i_right.m_re), round(i_right.m_im))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_round
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_sub(sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_sub
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(i_left.m_re - i_right.m_re, i_left.m_im - i_right.m_im)::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_sub
--   (
--     (12.3, -12.3),
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_sub
--   (
--     100.0 :: float,
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_sub
--   (
--     (-45.6, -45.6),
--     100.0
--   );

-- ------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_sub(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_sub
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(- i_right.m_re, - i_right.m_im)::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_sub
--   (
--     (-45.6, -45.6)
--   );
-- select sm_sc.fv_opr_sub
--   (
--     100.0
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_trunc(sm_sc.typ_l_complex, int);
create or replace function sm_sc.fv_opr_trunc
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    (trunc(i_left.m_re :: decimal, i_right), trunc(i_left.m_im :: decimal, i_right))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_trunc
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_opr_trunc(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_opr_trunc
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (trunc(i_right.m_re :: decimal), trunc(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_opr_trunc
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.fv_sin(sm_sc.typ_l_complex);
create or replace function sm_sc.fv_sin
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
declare 
  -- v_tmp   sm_sc.typ_l_complex  := exp((0.0, 1.0) :: sm_sc.typ_l_complex * i_right);
begin  -- i_right.m_re, i_right.m_im
  return row(sin(i_right.m_re) * ((exp(i_right.m_im) + exp(-i_right.m_im)) / 2.0 :: float), cos(i_right.m_re) * ((exp(i_right.m_im) - exp(-i_right.m_im)) / 2.0 :: float)) :: sm_sc.typ_l_complex;
  -- return (v_tmp - (1.0 / v_tmp)) / (0.0, 2.0);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.fv_sin
--   (
--     (-5.6, 5.6)
--   );
-- select sm_sc.fv_sin
--   (
--     100.0
--   );
-- set search_path to sm_sc;
-- -- sm_sc.fv_opr_norm 的别名
-- drop function if exists public.abs(sm_sc.typ_l_complex);
create or replace function public.abs
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(abs(i_right.m_re), abs(i_right.m_im )) :: sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.abs
--   (
--     (-45.6, -45.6)::sm_sc.typ_l_complex
--   );
-- select public.abs
--   (
--     100.0::sm_sc.typ_l_complex
--   );

-- -- -- drop function if exists abs(sm_sc.typ_l_complex);
-- -- create or replace function abs
-- -- (
-- --   i_right    sm_sc.typ_l_complex
-- -- )
-- -- returns float
-- -- as
-- -- $$
-- -- -- declare 
-- -- begin
-- --   return sqrt(i_right.m_re ^ 2 + i_right.m_im ^ 2);
-- -- end
-- -- $$
-- -- language plpgsql stable
-- -- parallel safe
-- -- cost 100;
-- set search_path to sm_sc;
-- drop function if exists public.ceil(sm_sc.typ_l_complex, int);
create or replace function public.ceil
(
  i_left     sm_sc.typ_l_complex    ,
  i_num_digits    int
)
returns sm_sc.typ_l_complex
as
$$
declare -- here
  v_balan   decimal  := 5.0 * power(0.1, i_num_digits + 1)  ;
begin
  return 
    (

      case 
        when i_left.m_re = round(i_left.m_re :: decimal, i_num_digits) 
          then i_left.m_re
        else round(i_left.m_re :: decimal + v_balan, i_num_digits) 
      end,
      case 
        when i_left.m_im = round(i_left.m_im :: decimal, i_num_digits) 
          then i_left.m_im
        else round(i_left.m_im :: decimal + v_balan, i_num_digits) 
      end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.ceil
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists public.ceil(sm_sc.typ_l_complex);
create or replace function public.ceil
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (ceil(i_right.m_re :: decimal), ceil(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.ceil
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- drop function if exists public.exp(sm_sc.typ_l_complex);
create or replace function public.exp
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    exp(i_right.m_re) * (cos(i_right.m_im), sin(i_right.m_im))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.exp
--   (
--     (45.6, -45.6)
--   );
-- select public.exp
--   (
--     (0, pi()::float)
--   );
-- select public.exp
--   (
--     (0, -pi()::float/2)
--   );
-- select public.exp
--   (
--     pi()::float
--   );
-- select public.exp
--   (
--     (0, -pi()::float)
--   );
-- -- select public.exp
-- --   (
-- --     0.0
-- --   );
-- set search_path to sm_sc;
-- drop function if exists public.floor(sm_sc.typ_l_complex, int);
create or replace function public.floor
(
  i_left     sm_sc.typ_l_complex    ,
  i_num_digits    int
)
returns sm_sc.typ_l_complex
as
$$
declare -- here
  v_balan   decimal  := 5.0 * power(0.1, i_num_digits + 1)  ;
begin
  return 
    (
      case 
        when i_left.m_re = round(i_left.m_re :: decimal, i_num_digits) 
          then i_left.m_re
        else round(i_left.m_re :: decimal - v_balan, i_num_digits) 
      end,
      case 
        when i_left.m_im = round(i_left.m_im :: decimal, i_num_digits) 
          then i_left.m_im
        else round(i_left.m_im :: decimal - v_balan, i_num_digits) 
      end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.floor
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists public.floor(sm_sc.typ_l_complex);
create or replace function public.floor
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (floor(i_right.m_re :: decimal), floor(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.floor
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- -- sm_sc.fv_opr_round 的别名
-- drop function if exists public.round(sm_sc.typ_l_complex, int);
create or replace function public.round
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (round(i_left.m_re :: decimal, i_right), round(i_left.m_im :: decimal, i_right))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.round
--   (
--     (12.35464645, -12.30325015)::sm_sc.typ_l_complex,
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists public.round(sm_sc.typ_l_complex);
create or replace function public.round
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (round(i_right.m_re :: decimal), round(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.round
--   (
--     (-45.6, -45.6)::sm_sc.typ_l_complex
--   );
-- set search_path to sm_sc;
-- drop function if exists public.trunc(sm_sc.typ_l_complex, int);
create or replace function public.trunc
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    (trunc(i_left.m_re :: decimal, i_right), trunc(i_left.m_im :: decimal, i_right))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.trunc
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists public.trunc(sm_sc.typ_l_complex);
create or replace function public.trunc
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (trunc(i_right.m_re :: decimal), trunc(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select public.trunc
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- -- sm_sc.fv_opr_norm 的别名
-- drop function if exists sm_sc.abs(sm_sc.typ_l_complex);
create or replace function sm_sc.abs
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return row(abs(i_right.m_re), abs(i_right.m_im )) :: sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.abs
--   (
--     (-45.6, -45.6)::sm_sc.typ_l_complex
--   );
-- select sm_sc.abs
--   (
--     100.0::sm_sc.typ_l_complex
--   );

-- -- -- drop function if exists abs(sm_sc.typ_l_complex);
-- -- create or replace function abs
-- -- (
-- --   i_right    sm_sc.typ_l_complex
-- -- )
-- -- returns float
-- -- as
-- -- $$
-- -- -- declare 
-- -- begin
-- --   return sqrt(i_right.m_re ^ 2 + i_right.m_im ^ 2);
-- -- end
-- -- $$
-- -- language plpgsql stable
-- -- parallel safe
-- -- cost 100;
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ceil(sm_sc.typ_l_complex, int);
create or replace function sm_sc.ceil
(
  i_left     sm_sc.typ_l_complex    ,
  i_num_digits    int
)
returns sm_sc.typ_l_complex
as
$$
declare -- here
  v_balan   decimal  := 5.0 * power(0.1, i_num_digits + 1)  ;
begin
  return 
    (

      case 
        when i_left.m_re = round(i_left.m_re :: decimal, i_num_digits) 
          then i_left.m_re
        else round(i_left.m_re :: decimal + v_balan, i_num_digits) 
      end,
      case 
        when i_left.m_im = round(i_left.m_im :: decimal, i_num_digits) 
          then i_left.m_im
        else round(i_left.m_im :: decimal + v_balan, i_num_digits) 
      end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.ceil
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ceil(sm_sc.typ_l_complex);
create or replace function sm_sc.ceil
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (ceil(i_right.m_re :: decimal), ceil(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.ceil
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.exp(sm_sc.typ_l_complex);
create or replace function sm_sc.exp
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    exp(i_right.m_re) * (cos(i_right.m_im), sin(i_right.m_im))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.exp
--   (
--     (45.6, -45.6)
--   );
-- select sm_sc.exp
--   (
--     (0, pi()::float)
--   );
-- select sm_sc.exp
--   (
--     (0, -pi()::float/2)
--   );
-- select sm_sc.exp
--   (
--     pi()::float
--   );
-- select sm_sc.exp
--   (
--     (0, -pi()::float)
--   );
-- -- select sm_sc.exp
-- --   (
-- --     0.0
-- --   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.floor(sm_sc.typ_l_complex, int);
create or replace function sm_sc.floor
(
  i_left     sm_sc.typ_l_complex    ,
  i_num_digits    int
)
returns sm_sc.typ_l_complex
as
$$
declare -- here
  v_balan   decimal  := 5.0 * power(0.1, i_num_digits + 1)  ;
begin
  return 
    (
      case 
        when i_left.m_re = round(i_left.m_re :: decimal, i_num_digits) 
          then i_left.m_re
        else round(i_left.m_re :: decimal - v_balan, i_num_digits) 
      end,
      case 
        when i_left.m_im = round(i_left.m_im :: decimal, i_num_digits) 
          then i_left.m_im
        else round(i_left.m_im :: decimal - v_balan, i_num_digits) 
      end
    )::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.floor
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.floor(sm_sc.typ_l_complex);
create or replace function sm_sc.floor
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (floor(i_right.m_re :: decimal), floor(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.floor
--   (
--     (-45.6, -45.6)
--   );
-- set search_path to sm_sc;
-- -- sm_sc.fv_opr_round 的别名
-- drop function if exists sm_sc.round(sm_sc.typ_l_complex, int);
create or replace function sm_sc.round
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (round(i_left.m_re :: decimal, i_right), round(i_left.m_im :: decimal, i_right))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.round
--   (
--     (12.35464645, -12.30325015)::sm_sc.typ_l_complex,
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.round(sm_sc.typ_l_complex);
create or replace function sm_sc.round
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (round(i_right.m_re :: decimal), round(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.round
--   (
--     (-45.6, -45.6)::sm_sc.typ_l_complex
--   );
-- set search_path to sm_sc;
-- drop function if exists sm_sc.trunc(sm_sc.typ_l_complex, int);
create or replace function sm_sc.trunc
(
  i_left     sm_sc.typ_l_complex    ,
  i_right    int
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return 
    (trunc(i_left.m_re :: decimal, i_right), trunc(i_left.m_im :: decimal, i_right))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.trunc
--   (
--     (12.35464645, -12.30325015),
--     2
--   );

-- -------------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.trunc(sm_sc.typ_l_complex);
create or replace function sm_sc.trunc
(
  i_right    sm_sc.typ_l_complex
)
returns sm_sc.typ_l_complex
as
$$
-- declare 
begin
  return (trunc(i_right.m_re :: decimal), trunc(i_right.m_im :: decimal))::sm_sc.typ_l_complex;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- -- set search_path to sm_sc;
-- select sm_sc.trunc
--   (
--     (-45.6, -45.6)
--   );
-- create or replace aggregate sum (sm_sc.typ_l_complex)
drop aggregate if exists sum (sm_sc.typ_l_complex);
create aggregate sum (sm_sc.typ_l_complex)
(
  sfunc = sm_sc.fv_opr_add,
  stype = sm_sc.typ_l_complex,
  initcond = '(0.0,0.0)',
  parallel = safe
);

-- select sum(a_val)
-- from 
-- (
--   select (32.5 :: float, 54.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (-32.5 :: float, -24.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (-42.5 :: float, 14.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (12.5 :: float, -54.2 :: float)::sm_sc.typ_l_complex as a_val
-- ) tb_a

-- create or replace aggregate sum (sm_sc.typ_l_complex)
drop aggregate if exists sm_sc.fa_l_complex_sum(sm_sc.typ_l_complex);
create aggregate sm_sc.fa_l_complex_sum (sm_sc.typ_l_complex)
(
  sfunc = sm_sc.fv_opr_add,
  stype = sm_sc.typ_l_complex,
  initcond = '(0.0,0.0)',
  parallel = safe
);

-- select sm_sc.fa_l_complex_sum(a_val)
-- from 
-- (
--   select (32.5 :: float, 54.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (-32.5 :: float, -24.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (-42.5 :: float, 14.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (12.5 :: float, -54.2 :: float)::sm_sc.typ_l_complex as a_val
-- ) tb_a

-- -- -- -----------------------------------------------------------------------------------
-- -- -- drop type if exists sm_sc.__typ_l_complex_ex cascade;
-- -- create type sm_sc.__typ_l_complex_ex as
-- -- (
-- --   m_arr_mid      sm_sc.typ_l_complex,
-- --   m_cnt          int
-- -- );

-- ---------------------
-- create or replace function sm_sc.fv_l_complex_add_ex
drop function if exists sm_sc.fv_l_complex_add_ex;
create function sm_sc.fv_l_complex_add_ex
(
  i_left      sm_sc.__typ_l_complex_ex,
  i_right     sm_sc.typ_l_complex
)
returns sm_sc.__typ_l_complex_ex
as
$$
-- declare
begin
  i_left.m_arr_mid := i_left.m_arr_mid + i_right;
  i_left.m_cnt := coalesce(i_left.m_cnt, 0) + 1;
  return i_left;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_l_complex_add_ex(row((12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex, 2), (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex)

-- ---------------------
-- create or replace function sm_sc.__fv_mx_avg_final
drop function if exists sm_sc.__fv_mx_avg_final(sm_sc.__typ_l_complex_ex);
create or replace function sm_sc.__fv_mx_avg_final
(
  i_ret_mid      sm_sc.__typ_l_complex_ex
)
returns sm_sc.typ_l_complex
as
$$
-- declare
begin
  return i_ret_mid.m_arr_mid / (i_ret_mid.m_cnt :: float);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_mx_avg_final(row((12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex, 2))

-- ---------------------
-- create or replace aggregate sm_sc.fa_mx_avg (sm_sc.typ_l_complex)
drop aggregate if exists sm_sc.fa_mx_avg (sm_sc.typ_l_complex);
create aggregate sm_sc.fa_mx_avg (sm_sc.typ_l_complex)
(
  sfunc = sm_sc.fv_l_complex_add_ex,
  stype = sm_sc.__typ_l_complex_ex,
  initcond = '("(0.0, 0.0)",0)',
  finalfunc = sm_sc.__fv_mx_avg_final,
  parallel = safe
);

-- select sm_sc.fa_mx_avg(a_val)
-- from 
-- (
--   select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
-- ) t

-- ---------------------
-- create or replace aggregate avg (sm_sc.typ_l_complex)
drop aggregate if exists avg (sm_sc.typ_l_complex);
create aggregate avg (sm_sc.typ_l_complex)
(
  sfunc = sm_sc.fv_l_complex_add_ex,
  stype = sm_sc.__typ_l_complex_ex,
  initcond = '("(0.0, 0.0)",0)',
  finalfunc = sm_sc.__fv_mx_avg_final,
  parallel = safe
);

-- select avg(a_val)
-- from 
-- (
--   select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex as a_val
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
--   union all select (12.5 :: float, 24.2 :: float)::sm_sc.typ_l_complex
-- ) t
drop function if exists sm_sc.__fv_delete_kv(varchar(64)[]);
create or replace function sm_sc.__fv_delete_kv
(
  i_arr_keys      varchar(64)[]
)
returns varchar(64)[]
as
$$
declare
  v_key  varchar(64)                    ;
  v_ret  varchar(64)[]  default  array[] :: varchar(64)[];
begin
  if array_ndims(i_arr_keys) = 1
  then 
    foreach v_key in array i_arr_keys
    loop 
      delete from sm_sc.__vt_array_kv
      where arr_key = v_key
      returning arr_key into v_key
      ;
      if v_key is not null 
      then 
        v_ret :=  v_ret || v_key;
      end if
      ;
    end loop;
  end if;
  return v_ret;
end
$$
language plpgsql volatile
parallel safe
cost 1000
;

-- select sm_sc.__fv_delete_kv(array[sm_sc.__fv_set_kv(array[3.1,4.6], 'test_001_val'), 'test_002_val'])
drop function if exists sm_sc.__fv_get_kv(varchar(64), boolean);
create or replace function sm_sc.__fv_get_kv
(
  i_arr_key      varchar(64)
, i_if_delete    boolean     default   false
)
returns float[]
as
$$
declare
  v_ret    float[];
begin
  if i_if_delete 
  then 
    delete from sm_sc.__vt_array_kv where arr_key = i_arr_key
    returning arr_val into v_ret
    ;
    return v_ret;
  else 
    return 
      (select arr_val from sm_sc.__vt_array_kv where arr_key = i_arr_key)
    ;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 1000
;

-- select sm_sc.__fv_get_kv(sm_sc.__fv_set_kv(sm_sc.fv_new_rand(array[2, 3])))
drop function if exists sm_sc.__fv_get_kvs(varchar(64)[], boolean);
create or replace function sm_sc.__fv_get_kvs
(
  i_arr_keys      varchar(64)[]
, i_if_delete    boolean     default   false
)
returns sm_sc.__typ_array_ex[]
as
$$
declare
  v_ret  sm_sc.__typ_array_ex[];
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_arr_keys) <> 1
    then
      raise exception 'unsupport array_ndims. ';
    end if;
  end if;

  if i_if_delete 
  then 
    with 
    cte_key as 
    (
      select  
        a_no
      , i_arr_keys[a_no] as a_key
      from generate_series(1, array_length(i_arr_keys, 1)) tb_a(a_no)
    ),
    cte_del as 
    (
      delete from sm_sc.__vt_array_kv tb_a_kv
      using cte_key tb_a_key
      where tb_a_kv.arr_key = tb_a_key.a_key
      returning tb_a_key.a_no, tb_a_kv.arr_val
    )
    select 
      array_agg(tb_a_kv.arr_val :: sm_sc.__typ_array_ex order by tb_a.a_no) 
    into v_ret
    from generate_series(1, array_length(i_arr_keys, 1)) tb_a(a_no)
    left join cte_del tb_a_kv
      on tb_a_kv.a_no = tb_a.a_no
    ;
    return v_ret;
  else 
    return 
    (
      select 
        array_agg(tb_a_kv.arr_val :: sm_sc.__typ_array_ex order by tb_a.a_no)
      from generate_series(1, array_length(i_arr_keys, 1)) tb_a(a_no)
      left join sm_sc.__vt_array_kv tb_a_kv
        on arr_key = i_arr_keys[a_no]
    )
    ;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 1000
;

-- select sm_sc.__fv_get_kvs(array[sm_sc.__fv_set_kv(sm_sc.fv_new_rand(array[2, 3]))])
-- select sm_sc.__fv_get_kvs(array[sm_sc.__fv_set_kv(sm_sc.fv_new_rand(array[2, 3])), 'no_such_key'])
-- select sm_sc.__fv_get_kvs(array[sm_sc.__fv_set_kv(sm_sc.fv_new_rand(array[2, 3]))], true)
-- select sm_sc.__fv_get_kvs(array[sm_sc.__fv_set_kv(sm_sc.fv_new_rand(array[2, 3])), 'no_such_key'], true)
drop function if exists sm_sc.__fv_set_kv(float[], varchar(64));
create or replace function sm_sc.__fv_set_kv
(
  i_arr          float[]
, i_arr_key      varchar(64)    default null
)
returns varchar(64)
as
$$
-- declare
  -- v_key  varchar(64);
begin
  if i_arr is not null or i_arr_key is not null
  then 
    insert into sm_sc.__vt_array_kv
    (
      arr_key
    , arr_val
    )
    values 
    (
      coalesce(i_arr_key, gen_random_uuid()::varchar(64))
    , i_arr
    )
    on conflict (arr_key) do
    update set 
      arr_val           = EXCLUDED.arr_val
    , last_update_date  = now()
    returning arr_key into i_arr_key
    ;
    return i_arr_key;
  else 
    return null;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 1000
;

-- select sm_sc.__fv_set_kv(array[1.0, 2.3])
-- select sm_sc.__fv_set_kv(sm_sc.fv_new_rand(array[2, 3]))
drop function if exists sm_sc.__fv_set_kvs(float[], varchar(64)[]);
create or replace function sm_sc.__fv_set_kvs
(
  i_arrs          sm_sc.__typ_array_ex[]
, i_arr_keys      varchar(64)[]    default null
)
returns varchar(64)[]
as
$$
-- declare
  -- v_key  varchar(64);
begin
  -- 审计
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_arrs) <> 1 or array_ndims(i_arr_keys) <> 1
    then
      raise exception 'unsupport array_ndims. ';
    elsif array_length(i_arrs, 1) <> array_length(i_arr_keys, 1)
    then 
      raise exception 'unmatched array_length. ';
    end if;
  end if;

  if sm_sc.fv_aggr_slice_coalesce(i_arrs) is not null or sm_sc.fv_aggr_slice_coalesce(i_arr_keys) is not null
  then 
    with 
    cte_unnest as 
    (
      select
        a_no
      , coalesce(i_arr_keys[a_no], gen_random_uuid()::varchar(64)) as a_key
      , i_arrs[a_no].m_array as a_value
      from generate_series(1, array_length(i_arrs, 1)) tb_a(a_no)
      where i_arr_keys[a_no] is not null or i_arrs[a_no].m_array is not null
    ),
    cte_upd as 
    (
      insert into sm_sc.__vt_array_kv
      (
        arr_key
      , arr_val
      )
      select
        coalesce(i_arr_keys[a_no], gen_random_uuid()::varchar(64))
      , i_arrs[a_no].m_array
      from cte_unnest
      on conflict (arr_key) do
      update set 
        arr_val           = EXCLUDED.arr_val
      , last_update_date  = now()
    )
    select 
      array_agg(a_key order by a_no)
    into i_arr_keys
    from cte_unnest
    ;
    return i_arr_keys;
  else 
    return null;
  end if;
end
$$
language plpgsql volatile
parallel safe
cost 1000
;

-- select sm_sc.__fv_set_kvs(array[array[1.0, 2.3] :: float[] :: sm_sc.__typ_array_ex, array[-1.0, 1.3, 9.4] :: float[] :: sm_sc.__typ_array_ex])
-- select sm_sc.__fv_set_kvs(array[array[1.0, 2.3] :: float[] :: sm_sc.__typ_array_ex, array[-1.0, 1.3, 9.4] :: float[] :: sm_sc.__typ_array_ex, null])
-- select sm_sc.__fv_set_kvs(array[sm_sc.fv_new_rand(array[2, 3]) :: sm_sc.__typ_array_ex, array[-1.0, 1.3, 9.4] :: float[] :: sm_sc.__typ_array_ex], array['test_key_1', null])
-- select sm_sc.__fv_set_kvs(array[sm_sc.fv_new_rand(array[2, 3]) :: sm_sc.__typ_array_ex, array[-1.0, 1.3, 9.4] :: float[] :: sm_sc.__typ_array_ex, null], array['test_key_1', null, null])

-- drop function if exists sm_sc.fv_coalesce_ex(anyelement, anyelement);
create or replace function sm_sc.fv_coalesce_ex
(
  i_left  anyelement,
  i_right anyelement
)
returns anyelement
as
$$
begin
  return coalesce(i_left, i_right);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.fv_coalesce_ex(null::text, 'abc')
-- -----------------------------

-- create or replace aggregate sm_sc.fa_coalesce (anyelement)
drop aggregate if exists sm_sc.fa_coalesce(anyelement);
create aggregate sm_sc.fa_coalesce (anyelement)
(
  sfunc = sm_sc.fv_coalesce_ex,
  stype = anyelement
);
-- select sm_sc.fa_coalesce(a_val)
-- from (select null as a_val union all select 'abc' union all select 'cde') tb_a(a_val)
-- -----------------------------
-- set search_path to sm_sc;

-- drop function if exists sm_sc.__fv_concat_ex(anyelement, anyelement);
create or replace function sm_sc.__fv_concat_ex
(
  i_left       anyelement    ,
  i_right      anyelement
)
returns anyelement
as
$$
-- declare
begin
  return i_left || i_right;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_concat_ex(B'1010', B'0101')

-- -----------------------------
-- create or replace aggregate sm_sc.fa_concat (anyelement)
drop aggregate if exists sm_sc.fa_concat(anyelement);
create aggregate sm_sc.fa_concat (anyelement)
(
  sfunc = sm_sc.__fv_concat_ex,
  stype = anyelement,
  initcond = ''
);

-- select sm_sc.fa_concat(a_val)
-- from 
-- (
--   select B'1010' as a_val
--   union all select B'1110'
--   union all select B'1011'
--   union all select B'0010'
--   union all select B'1000'
-- ) t

-- select sm_sc.fa_concat(a_val order by a_val)
-- from 
-- (
--   select B'1010' as a_val
--   union all select B'1110'
--   union all select B'1011'
--   union all select B'0010'
--   union all select B'1000'
-- ) t

-- select sm_sc.fa_concat(a_val)
-- from 
-- (
--   select 'abc' as a_val
--   union all select 'faera'
--   union all select '4564'
--   union all select 'kuk7'
--   union all select '66yy'
-- ) t

-- select sm_sc.fa_concat(a_val order by a_val)
-- from 
-- (
--   select 'abc' as a_val
--   union all select 'faera'
--   union all select '4564'
--   union all select 'kuk7'
--   union all select '66yy'
-- ) t

-- create or replace aggregate sm_sc.fa_concat (jsonb)
drop aggregate if exists sm_sc.fa_concat(jsonb);
create aggregate sm_sc.fa_concat (jsonb)
(
  sfunc = sm_sc.__fv_concat_ex,
  stype = jsonb,
  initcond = '[]'
);

-- select sm_sc.fa_concat(a_val order by a_val)
-- from 
-- (
--   select '[{"node_no" : 101, "node_type" : "weight", "node_fn_type" : "00_const", "node_desc" : "w_q"}]'::jsonb  as a_val
--   union all select '[{"node_no" : 105, "node_type" : null, "node_fn_type" : "01_prod_mx", "node_desc" : "|**| w_k"}]'::jsonb
--   union all select '[{"node_no" : 107, "node_type" : null, "node_fn_type" : "04_transpose", "node_desc" : "transpose(k)"}]'::jsonb
-- ) t
-- drop function if exists sm_sc.__fv_prod_ex(anyelement, anyelement);
create or replace function sm_sc.__fv_prod_ex
(
  i_left       anyelement    ,
  i_right      anyelement
)
returns anyelement
as
$$
-- declare
begin
  return i_left * i_right;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_prod_ex(1.1, 1.2)

-- -----------------------------
-- create or replace aggregate sm_sc.fa_prod (anyelement)
drop aggregate if exists sm_sc.fa_prod(anyelement);
create aggregate sm_sc.fa_prod (anyelement)
(
  sfunc = sm_sc.__fv_prod_ex,
  stype = anyelement,
  initcond = 1
);

-- select sm_sc.fa_prod(a_val)
-- from 
-- (
--   select 1.1 as a_val
--   union all select 1.2
--   union all select 1.3
--   union all select 1.4
--   union all select 1.5
-- ) t

-- -----------------------------
-- drop function if exists sm_sc.__fv_range_and_ex(anymultirange, anymultirange);
create or replace function sm_sc.__fv_range_and_ex
(
  i_left       anymultirange    ,
  i_right      anymultirange
)
returns anymultirange
as
$$
-- declare
begin
  return coalesce(i_left, '{(,)}') * coalesce(i_right, '{(,)}');
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_and_ex(int8range(1, 2, '[]') :: int8multirange, int8range(5, 6, '[]') :: int8multirange)
-- select sm_sc.__fv_range_and_ex(int8range(1, 2, '[]') :: int8multirange, null)

-- drop function if exists sm_sc.__fv_range_and_ex(anyrange, anyrange);
create or replace function sm_sc.__fv_range_and_ex
(
  i_left       anyrange    ,
  i_right      anyrange
)
returns anyrange
as
$$
-- declare
begin
  return coalesce(i_left, '[,]') * coalesce(i_right, '[,]');
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_and_ex(int8range(1, 2, '[]'), int8range(5, 6, '[]'))
-- select sm_sc.__fv_range_and_ex(int8range(1, 2, '[]'), '[,]' :: int8range)

-- -----------------------------
-- create or replace aggregate sm_sc.fa_range_and (anymultirange)
drop aggregate if exists sm_sc.fa_range_and(anymultirange);
create aggregate sm_sc.fa_range_and (anymultirange)
(
  sfunc = sm_sc.__fv_range_and_ex,
  stype = anymultirange,
  initcond = '{(,)}',
  parallel = safe
);

-- select sm_sc.fa_range_and(a_val)
-- from 
-- (
--   select int8range(1, 10, '[]') :: int8multirange as a_val
--   union all select int8range(5, 9, '[]') :: int8multirange
--   union all select int8range(3, 8, '[]') :: int8multirange
--   union all select int8range(4, 7, '[]') :: int8multirange
--   union all select int8range(2, 9, '[]') :: int8multirange
--   union all select null
-- ) t

-- create or replace aggregate sm_sc.fa_range_and (anyrange)
drop aggregate if exists sm_sc.fa_range_and(anyrange);
create aggregate sm_sc.fa_range_and (anyrange)
(
  sfunc = sm_sc.__fv_range_and_ex,
  stype = anyrange,
  initcond = '(,)',
  parallel = safe
);

-- select sm_sc.fa_range_and(a_val)
-- from 
-- (
--   select int8range(1, 10, '[]') as a_val
--   union all select int8range(5, 9, '[]')
--   union all select int8range(3, 8, '[]')
--   union all select int8range(4, 7, '[]')
--   union all select int8range(2, 9, '[]')
--   union all select null
-- ) t
-- ---------------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_range_arr_and(anyarray, anyarray);
create or replace function sm_sc.__fv_range_arr_and
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt   int[]  ;
  v_ret         i_left % type := case when cardinality(i_left) >= cardinality(i_right) then i_left when cardinality(i_left) < cardinality(i_right) then i_right end;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  -- v_ret := array_fill(null, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] + []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] + array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return coalesce(i_left, i_right);
  
  -- [] + []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := sm_sc.__fv_range_and_ex(i_left[v_y_cur], i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][] + [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.__fv_range_and_ex(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- [][][] + [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            sm_sc.__fv_range_and_ex
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
            , i_right[v_y_cur][v_x_cur][v_x3_cur]
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] + [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              sm_sc.__fv_range_and_ex
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.__fv_range_arr_and
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.__fv_range_arr_and
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.__fv_range_arr_and
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.__fv_range_arr_and
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.__fv_range_arr_and(array[1], array[1,2,3]);
-- select sm_sc.__fv_range_arr_and(array[1], array[array[1,2,3]]);
-- select sm_sc.__fv_range_arr_and(array[array[1]], array[1,2,3]);
-- select sm_sc.__fv_range_arr_and(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.__fv_range_arr_and
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.__fv_range_arr_and
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.__fv_range_arr_and(anyarray, anyelement);
create or replace function sm_sc.__fv_range_arr_and
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     i_left % type  ;
  v_null    i_right % type := null;
begin
  -- (null :: float[][] <  null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return v_ret;

  -- ([][] + float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.__fv_range_and_ex(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] + float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := sm_sc.__fv_range_and_ex(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] + float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] =  
            sm_sc.__fv_range_and_ex
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
            , i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] + float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] =  
              sm_sc.__fv_range_and_ex
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_arr_and(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.__fv_range_arr_and(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.__fv_range_arr_and(array[]::float[], 8.8::float)
-- select sm_sc.__fv_range_arr_and(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.__fv_range_arr_and
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.__fv_range_arr_and
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.__fv_range_arr_and(anyelement, anyarray);
create or replace function sm_sc.__fv_range_arr_and
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     i_right % type  ;
  v_null    i_left % type := null;
begin
  -- (null :: float[][] <= null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- (float + [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.__fv_range_and_ex(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float + [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := sm_sc.__fv_range_and_ex(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float + [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            sm_sc.__fv_range_and_ex
            (
              i_left 
            , i_right[v_y_cur][v_x_cur][v_x3_cur] 
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float + [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              sm_sc.__fv_range_and_ex
              (
              i_left
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_arr_and(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.__fv_range_arr_and(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.__fv_range_arr_and(8.8::float, array[]::float[] )
-- select sm_sc.__fv_range_arr_and(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.__fv_range_arr_and
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.__fv_range_arr_and
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- ---------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_range_and (anyarray)
drop aggregate if exists sm_sc.fa_range_and(anyarray);
create aggregate sm_sc.fa_range_and (anyarray)
(
  sfunc = sm_sc.__fv_range_arr_and,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_range_and(a_val)
-- from 
-- (
--   select array[int8range(1, 20, '[]') :: int8multirange, int8range(-20, -1, '[]') :: int8multirange] as a_val
--   union all select array[int8range(5, 60, '[]') :: int8multirange, int8range(-60, -5, '[]') :: int8multirange]
--   union all select array[int8range(3, 70, '[]') :: int8multirange, int8range(-70, -3, '[]') :: int8multirange]
--   union all select array[int8range(7, 10, '[]') :: int8multirange, int8range(-10, -7, '[]') :: int8multirange]
--   union all select array[int8range(5, 10, '[]') :: int8multirange, int8range(-10, -5, '[]') :: int8multirange]
--   union all select null
-- ) t

-- drop function if exists sm_sc.__fv_range_or_ex(anymultirange, anymultirange);
create or replace function sm_sc.__fv_range_or_ex
(
  i_left       anymultirange    ,
  i_right      anymultirange
)
returns anymultirange
as
$$
-- declare
begin
  return coalesce(i_left, i_right) + coalesce(i_right, i_left);
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_or_ex(int8range(1, 2, '[]') :: int8multirange, int8range(5, 6, '[]') :: int8multirange)
-- select sm_sc.__fv_range_or_ex(int8range(1, 2, '[]') :: int8multirange, null)

-- drop function if exists sm_sc.__fv_range_or_ex(anyrange, anyrange);
create or replace function sm_sc.__fv_range_or_ex
(
  i_left       anyrange    ,
  i_right      anyrange
)
returns anymultirange
as
$$
-- declare
begin
  return multirange(coalesce(i_left, i_right)) + multirange(coalesce(i_right, i_left));
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_or_ex(int8range(1, 2, '[]'), int8range(5, 6, '[]'))
-- select sm_sc.__fv_range_or_ex(int8range(1, 2, '[]'), null)

-- -----------------------------
-- create or replace aggregate sm_sc.fa_range_or (anymultirange)
drop aggregate if exists sm_sc.fa_range_or(anymultirange);
create aggregate sm_sc.fa_range_or (anymultirange)
(
  sfunc = sm_sc.__fv_range_or_ex,
  stype = anymultirange,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_range_or(a_val)
-- from 
-- (
--   select int8range(1, 2, '[]') :: int8multirange as a_val
--   union all select int8range(5, 6, '[]') :: int8multirange
--   union all select int8range(3, 7, '[]') :: int8multirange
--   union all select int8range(9, 13, '[]') :: int8multirange
--   union all select int8range(11, 17, '[]') :: int8multirange
--   union all select null
-- ) t
-- ----------------------------------------------------------------------------
-- set search_path to sm_sc;
-- drop function if exists sm_sc.__fv_range_arr_or(anyarray, anyarray);
create or replace function sm_sc.__fv_range_arr_or
(
  i_left     anyarray    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_y_cur       int    := 1  ;
  v_x_cur       int    := 1  ;
  v_x3_cur      int    := 1  ;
  v_x4_cur      int    := 1  ;
  v_len_left    int[]  := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
  v_len_right   int[]  := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
  v_len_depdt   int[]  ;
  v_ret         i_left % type := case when cardinality(i_left) >= cardinality(i_right) then i_left when cardinality(i_left) < cardinality(i_right) then i_right end;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if exists 
       (
         select  
         from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
         where a_len_left <> a_len_right
           and a_len_left <> 1
           and a_len_right <> 1
       )
    then
      raise exception 'no method!';
    end if;
  end if;
  
  v_len_depdt :=
    (
      select 
        sm_sc.__fv_mirror_y(array_agg(greatest(a_len_left, a_len_right)))
      from unnest(sm_sc.__fv_mirror_y(v_len_left), sm_sc.__fv_mirror_y(v_len_right)) tb_a_len(a_len_left, a_len_right)
    )
  ;
  -- v_ret := array_fill(null, v_len_depdt);
  
  -- 整理入参一
  if v_len_left <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 3
    then 
      i_left := array[[[i_left]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 2
    then 
      i_left := array[[i_left]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_left, 1) = 1
    then 
      i_left := array[i_left];
    end if;
    
    v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_left <> v_len_depdt
    then 
      i_left := sm_sc.fv_new(i_left, v_len_depdt / v_len_left);
      v_len_left := (select array_agg(array_length(i_left, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_left)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;
  
  -- 整理入参二
  if v_len_right <> v_len_depdt
  then 
    -- 对齐维度
    if array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 3
    then 
      i_right := array[[[i_right]]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 2
    then 
      i_right := array[[i_right]];
    elsif array_length(v_len_depdt, 1) - array_length(v_len_right, 1) = 1
    then 
      i_right := array[i_right];
    end if;
    
    v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    
    -- 对齐维长
    if v_len_right <> v_len_depdt
    then 
      i_right := sm_sc.fv_new(i_right, v_len_depdt / v_len_right);
      v_len_right := (select array_agg(array_length(i_right, a_cur_dim) order by a_cur_dim) from generate_series(1, array_ndims(i_right)) tb_a_cur_dim(a_cur_dim));
    end if;
  end if;

  -- array[] + []
  if v_len_left is null and i_left is not null -- = 0
  then 
    return i_right;
  -- [] + array[]
  elsif v_len_right is null and i_right is not null -- = 0
  then 
    return i_left;
  
  elsif i_left is null or i_right is null
  then
    return coalesce(i_left, i_right);
  
  -- [] + []
  elsif array_length(v_len_left, 1) = 1
  then 
    while v_y_cur <= v_len_left[1]
    loop
      v_ret[v_y_cur] := sm_sc.__fv_range_or_ex(i_left[v_y_cur], i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- [][] + [][]
  elsif array_length(v_len_left, 1) = 2
  then
    while v_y_cur <= v_len_left[1]
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= v_len_left[2]
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.__fv_range_or_ex(i_left[v_y_cur][v_x_cur], i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;
    
  -- [][][] + [][][]
  elsif array_length(v_len_left, 1) = 3
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            sm_sc.__fv_range_or_ex
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
            , i_right[v_y_cur][v_x_cur][v_x3_cur]
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- [][][][] + [][][][]
  elsif array_length(v_len_left, 1) = 4
  then
    for v_y_cur in 1 .. v_len_left[1]
    loop
      for v_x_cur in 1 .. v_len_left[2]
      loop
        for v_x3_cur in 1 .. v_len_left[3]
        loop
          for v_x4_cur in 1 .. v_len_left[4]
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              sm_sc.__fv_range_or_ex
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur]
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;

  -- 审计二维长度
  else
    raise exception 'no method for such length!  v_len_left: %; v_len_right: %;', v_len_left, v_len_right;
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.__fv_range_arr_or
--   ( 
--     <>` (sm_sc.fv_new_rand(array[2, 3, 1, 5]) -` 0.5 :: float)
--   , <>` (sm_sc.fv_new_rand(array[   1, 4, 5]) -` 0.5 :: float)
--   )
-- -- set search_path to sm_sc;
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[1.3, 52.3], array[5.6, 45.6]]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[32.5], array[9.1]],
--     array[array[12.3, 2.3], array[45.6, 25.6]]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[12.3, 32.3], array[45.6, 15.6]],
--     array[array[12.5], array[19.1]]
--   );
-- -- select sm_sc.__fv_range_arr_or
-- --   (
-- --     array[3.5, 9.1],
-- --     array[array[2.3, 12.3], array[45.6, 4.6]]
-- --   );
-- -- select sm_sc.__fv_range_arr_or
-- --   (
-- --     array[array[12.3, 2.3], array[45.6, 5.6]],
-- --     array[12.5, 19.1]
-- --   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[3.5, 9.1]],
--     array[array[2.3, 12.3], array[45.6, 4.6]]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[12.3, 2.3], array[45.6, 5.6]],
--     array[array[12.5, 19.1]]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[], array []]::float[],
--     array[]::float[]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[array[], array []]::float[],
--     array[array[], array []]::float[]
--   );
-- select sm_sc.__fv_range_arr_or
--   (
--     array[12.3, 2.3, 45.6],
--     array[12.5, 19.1, 5.6]
--   );
-- select sm_sc.__fv_range_arr_or(array[1], array[1,2,3]);
-- select sm_sc.__fv_range_arr_or(array[1], array[array[1,2,3]]);
-- select sm_sc.__fv_range_arr_or(array[array[1]], array[1,2,3]);
-- select sm_sc.__fv_range_arr_or(array[array[1]], array[array[1,2,3]]);
-- set session pg4ml._v_is_debug_check = '1';
-- set session pg4ml._v_is_debug_check = '0';
-- select 
--   sm_sc.__fv_range_arr_or
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , array[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]
--   )
-- select 
--   sm_sc.__fv_range_arr_or
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , array[[[[1.2,2.3,3.4],[0.3,0.4,0.7]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]],[[[1.6,2.7,3.4],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[1.4,2.2,0.8]]]]
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.__fv_range_arr_or(anyarray, anyelement);
create or replace function sm_sc.__fv_range_arr_or
(
  i_left     anyarray    ,
  i_right    anyelement
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     i_left % type  ;
  v_null    i_right % type := null;
begin
  -- (null :: float[][] <  null :: float[][])
  if array_length(i_left, array_ndims(i_left)) is null and i_right is not null
  then 
    return v_ret;
  -- -- elsif i_right is null
  -- -- then
  -- --   return v_ret;

  -- ([][] + float)
  elsif array_ndims(i_left) =  2
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1), array_length(i_left, 2)]);
    while v_y_cur <= array_length(i_left, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_left, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.__fv_range_or_ex(i_left[v_y_cur][v_x_cur], i_right);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([] + float)
  elsif array_ndims(i_left) =  1
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1)]);
    while v_y_cur <= array_length(i_left, 1)
    loop
      v_ret[v_y_cur] := sm_sc.__fv_range_or_ex(i_left[v_y_cur], i_right);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- ([][][] + float)
  elsif array_ndims(i_left) =  3
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] =  
            sm_sc.__fv_range_or_ex
            (
              i_left[v_y_cur][v_x_cur][v_x3_cur] 
            , i_right
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- ([][][][] + float)
  elsif array_ndims(i_left) = 4
  then
    v_ret := array_fill(v_null, array[array_length(i_left, 1), array_length(i_left, 2), array_length(i_left, 3), array_length(i_left, 4)]);
    for v_y_cur in 1 .. array_length(i_left, 1)
    loop
      for v_x_cur in 1 .. array_length(i_left, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_left, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_left, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] =  
              sm_sc.__fv_range_or_ex
              (
                i_left[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              , i_right
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_left);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_arr_or(array[array[12.3, 25.1], array[2.56, 3.25]], 8.8)
-- select sm_sc.__fv_range_arr_or(array[12.3, 25.1, 2.56, 3.25], 8.8)
-- select sm_sc.__fv_range_arr_or(array[]::float[], 8.8::float)
-- select sm_sc.__fv_range_arr_or(array[array[], array []]::float[], 8.8::float)
-- select 
--   sm_sc.__fv_range_arr_or
--   (
--     array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   , 2.2
--   )
-- select 
--   sm_sc.__fv_range_arr_or
--   (
--     array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   , 2.2
--   )

-- -----------------------------------------------------------------------
-- drop function if exists sm_sc.__fv_range_arr_or(anyelement, anyarray);
create or replace function sm_sc.__fv_range_arr_or
(
  i_left     anyelement    ,
  i_right    anyarray
)
returns anyarray
as
$$
declare -- here
  v_x_cur   int  := 1  ;
  v_y_cur   int  := 1  ;
  v_x3_cur      int  := 1  ;
  v_x4_cur      int  := 1  ;
  v_ret     i_right % type  ;
  v_null    i_left % type := null;
begin
  -- (null :: float[][] <= null :: float[][])
  if array_length(i_right, array_ndims(i_right)) is null and i_left is not null
  then 
    return v_ret;
  -- -- elsif i_left is null
  -- -- then
  -- --   return null::anyarray;

  -- (float + [][])
  elsif array_ndims(i_right) =  2
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1), array_length(i_right, 2)]);
    while v_y_cur <= array_length(i_right, 1)
    loop 
      v_x_cur := 1  ;
      while v_x_cur <= array_length(i_right, 2)
      loop
        v_ret[v_y_cur][v_x_cur] := sm_sc.__fv_range_or_ex(i_left, i_right[v_y_cur][v_x_cur]);
        v_x_cur := v_x_cur + 1;
      end loop;
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float + [])
  elsif array_ndims(i_right) =  1
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1)]);
    while v_y_cur <= array_length(i_right, 1)
    loop
      v_ret[v_y_cur] := sm_sc.__fv_range_or_ex(i_left, i_right[v_y_cur]);
      v_y_cur := v_y_cur + 1;
    end loop;
    return v_ret;

  -- (float + [][][])
  elsif array_ndims(i_right) = 3
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          v_ret[v_y_cur][v_x_cur][v_x3_cur] := 
            sm_sc.__fv_range_or_ex
            (
              i_left 
            , i_right[v_y_cur][v_x_cur][v_x3_cur] 
            )
          ;
        end loop;    
      end loop;
    end loop;
    return v_ret;
    
  -- (float + [][][][])
  elsif array_ndims(i_right) = 4
  then
    v_ret := array_fill(v_null, array[array_length(i_right, 1), array_length(i_right, 2), array_length(i_right, 3), array_length(i_right, 4)]);
    for v_y_cur in 1 .. array_length(i_right, 1)
    loop
      for v_x_cur in 1 .. array_length(i_right, 2)
      loop
        for v_x3_cur in 1 .. array_length(i_right, 3)
        loop
          for v_x4_cur in 1 .. array_length(i_right, 4)
          loop
            v_ret[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] := 
              sm_sc.__fv_range_or_ex
              (
              i_left
              , i_right[v_y_cur][v_x_cur][v_x3_cur][v_x4_cur] 
              )
            ;
          end loop;
        end loop;    
      end loop;
    end loop;
    return v_ret;
   
  else
    raise exception 'no method for such length!  Dims: %;', array_dims(i_right);
  end if;
end
$$
language plpgsql stable
parallel safe
cost 100;
-- select sm_sc.__fv_range_arr_or(8.8, array[array[12.3, 25.1], array[2.56, 3.25]])
-- select sm_sc.__fv_range_arr_or(8.8, array[12.3, 25.1, 2.56, 3.25])
-- select sm_sc.__fv_range_arr_or(8.8::float, array[]::float[] )
-- select sm_sc.__fv_range_arr_or(8.8::float, array[array[], array []]::float[])
-- select 
--   sm_sc.__fv_range_arr_or
--   (
--     2.2
--   , array[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]]
--   )
-- select 
--   sm_sc.__fv_range_arr_or
--   (
--     2.2
--   , array[[[[1,2,3],[1.2,2.3,3.4]],[[0.5,0.7,0.8],[0.3,0.4,0.7]]],[[[1.6,2.7,3.4],[1.4,2.2,0.8]],[[-0.5,1.7,0.8],[2.3,-0.4,-2.7]]]]
--   )

-- -------------------------------------------------------------------------
-- create or replace aggregate sm_sc.fa_range_or (anyarray)
drop aggregate if exists sm_sc.fa_range_or(anyarray);
create aggregate sm_sc.fa_range_or (anyarray)
(
  sfunc = sm_sc.__fv_range_arr_or,
  stype = anyarray,
  initcond = '{}',
  parallel = safe
);

-- select sm_sc.fa_range_or(a_val)
-- from 
-- (
--   select array[int8range(1, 2, '[]') :: int8multirange, int8range(-2, -1, '[]') :: int8multirange] as a_val
--   union all select array[int8range(5, 6, '[]') :: int8multirange, int8range(-6, -5, '[]') :: int8multirange]
--   union all select array[int8range(3, 7, '[]') :: int8multirange, int8range(-7, -3, '[]') :: int8multirange]
--   union all select array[int8range(9, 13, '[]') :: int8multirange, int8range(-13, -9, '[]') :: int8multirange]
--   union all select array[int8range(11, 17, '[]') :: int8multirange, int8range(-17, -11, '[]') :: int8multirange]
--   union all select null
-- ) t

drop function if exists sm_sc.ft_regexp_matches(text, text, text);
create or replace function sm_sc.ft_regexp_matches
(
  in i_text text, 
  in i_reg  text, 
  in i_opt  text default ''
)
  -- returns timestamp 
  returns table
    (o_matches_text text[])
as
$$
-- declare here
begin
  -- 最终输出
  return query
    select regexp_matches(i_text, i_reg, i_opt) as o_matches_text
  ; 

  -- exception, then null
  exception
    when others 
    then 
      return query select null;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.ft_regexp_matches('aaaaabbbaaa', 'a.a')
-- select sm_sc.ft_regexp_matches('aaaaabbbaaa', 'A.A', 'gi')
-- drop function if exists sm_sc.fv_enum_rand(varchar(64), int);
create or replace function sm_sc.fv_enum_rand
(
  i_enum_type_name    varchar(64),
  i_cnt               int
)
returns name[] -- pg_enum%enumlabel
as
$$
declare -- here
  v_enum_type_name    varchar(64)  := (select coalesce(a_match[1], i_enum_type_name) from regexp_matches(i_enum_type_name, '(?<=\.)[^\.]+') tb_a(a_match) limit 1);
  v_enum_type_scheme  varchar(64)  := (select coalesce(a_match[1], 'public') from regexp_matches(i_enum_type_name, '[^\.]+(?=\.)') tb_a(a_match) limit 1);
  v_enum_dic          name[]       :=  
  (
    select 
      array_agg(tb_a_enum.enumlabel)
    from pg_type tb_a_ty
    inner join pg_enum tb_a_enum
      on tb_a_enum.enumtypid = tb_a_ty.oid
	inner join pg_namespace tb_a_ns
	  on tb_a_ns.oid = tb_a_ty.typnamespace
    where tb_a_ty.typname = v_enum_type_name
	  and tb_a_ns.nspname = v_enum_type_scheme
  );
  v_dic_len    int      :=  array_length(v_enum_dic, 1);

begin
  return 
  (
    select 
      array_agg(v_enum_dic[round(random() * v_dic_len + 0.5 :: float)::int])
    from generate_series(1, i_cnt)
  )
  ;
end
$$
language plpgsql volatile
parallel safe
cost 100;

-- create type sm_sc.typ_enum_day as enum('SUN','MON','TUE','WED','THU','FRI','SAT');
-- -- -- select sm_sc.fv_enum_rand('sm_sc.typ_enum_day', 3);
-- select sm_sc.fv_enum_rand('typ_enum_day', 3);
-- drop function if exists sm_sc.fv_enum_rand_pick(varchar(64), int);
create or replace function sm_sc.fv_enum_rand_pick
(
  i_enum_type_name    varchar(64),
  i_cnt               int
)
returns name[] -- pg_enum%enumlabel
as
$$
declare -- here
  v_enum_type_name    varchar(64)  := (select coalesce(a_match[1], i_enum_type_name) from regexp_matches(i_enum_type_name, '(?<=\.)[^\.]+') tb_a(a_match) limit 1);
  v_enum_type_scheme  varchar(64)  := (select coalesce(a_match[1], 'public') from regexp_matches(i_enum_type_name, '[^\.]+(?=\.)') tb_a(a_match) limit 1);
  v_enum_dic    name[]  :=  
  (
    select 
      array_agg(tb_a_enum.enumlabel)
    from pg_type tb_a_ty
    inner join pg_enum tb_a_enum
      on tb_a_enum.enumtypid = tb_a_ty.oid
	inner join pg_namespace tb_a_ns
	  on tb_a_ns.oid = tb_a_ty.typnamespace
    where tb_a_ty.typname = v_enum_type_name
	  and tb_a_ns.nspname = v_enum_type_scheme
  );
  v_dic_len    int      :=  array_length(v_enum_dic, 1);

begin
  if i_cnt > v_dic_len
  then
    raise exception 'pick_cnt must be less than all_cnt.';
  else
    return   
    (
      select 
        array_agg(v_enum_dic[a_cur])
      from unnest(sm_sc.fv_rand_1d_ele_pick(v_dic_len, i_cnt)) tb_a(a_cur)
    ) 
  ;
  end if;
end
$$
language plpgsql volatile
parallel unsafe
cost 100;

-- create type sm_sc.typ_enum_day as enum('SUN','MON','TUE','WED','THU','FRI','SAT');
-- select sm_sc.fv_enum_rand_pick('sm_sc.typ_enum_day', 3);
-- -- -- 避免 parallel 调用，也避免获得重复 seq 值，
-- -- -- 将返回单值函数，合并至返回 range 函数。保证单例调用，保证 parallel unsafe 生效
-- -- drop function if exists sm_sc.fv_get_global_seq();
-- -- create or replace function sm_sc.fv_get_global_seq()
-- -- returns bigint
-- -- as
-- -- $$
-- -- -- declare
-- -- begin
-- --   return nextval('sm_sc.__seq_global' :: regclass);
-- -- end
-- -- $$
-- -- language plpgsql volatile
-- -- parallel unsafe
-- -- cost 100
-- -- ;
-- ----------------------------------------------------
drop function if exists sm_sc.fv_get_global_seq(bigint);
create or replace function sm_sc.fv_get_global_seq
(
  i_cnt      bigint   default  1
)
returns int8range
as
$$
-- 不采用 pg sequence 特性，而采用自定义 seq 表。
-- pg sequence 的 currval() 只支持 session 范围内的，获得 seq 当前值，导致各个 session 无法获得一致的全局 seq 当前值。
-- -- -- declare
-- -- begin
-- --   return int8range(currval('sm_sc.__seq_global'), setval('sm_sc.__seq_global', currval('sm_sc.__seq_global') + i_cnt) - 1, '[]');

declare
  v_cur_val      bigint   ;
begin
  perform  from sm_sc.__vt_global_seq where seq_no = 1 for update;
  -- perform pg_sleep(15);
  update sm_sc.__vt_global_seq
  set cur_val = cur_val + i_cnt
  where seq_no = 1
  returning cur_val into v_cur_val
  ;
  return int8range(v_cur_val - i_cnt + 1, v_cur_val, '[]');
end
$$
language plpgsql volatile
parallel unsafe
cost 100
;

-- select sm_sc.fv_get_global_seq()
-- union all 
-- select sm_sc.fv_get_global_seq(3)
-- drop function if exists sm_sc.fv_multirange_len(anymultirange);
create or replace function sm_sc.fv_multirange_len
(
  i_multirange     anymultirange
)
returns anyelement
as
$$
--declare
begin
  return 
  (
    select 
      sum(upper(a_range) - lower(a_range))
    from unnest(i_multirange) tb_a_range(a_range)
  )
  ;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_multirange_len(int8multirange(int8range(1, 3, '[]'), int8range(9, 10, '[]')))
-- select sm_sc.fv_multirange_len(nummultirange(numrange(1.6, 3.9, '[]'), numrange(9.1, 10.8, '[]')))
-- drop function if exists sm_sc.fv_multirange_move(anymultirange, anyelement);
create or replace function sm_sc.fv_multirange_move
(
  i_multirange     anymultirange,
  i_move           anyelement
)
returns anymultirange
as
$$
--declare
begin
  return 
  (
    select 
      sm_sc.fa_range_or(multirange(sm_sc.fv_range_move(a_range, i_move)))
    from unnest(i_multirange) tb_a_range(a_range)
  )
  ;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_multirange_move(int8multirange(int8range(1, 3, '[]'), int8range(9, 10, '[]')), 10 :: bigint)
-- select sm_sc.fv_multirange_move(nummultirange(numrange(1.6, 3.9, '[]'), numrange(9.1, 10.8, '[]')), 12.1)
drop function if exists sm_sc.fv_o_distance(float[2][], int);
create or replace function sm_sc.fv_o_distance
(
  i_points_arr               float[2][]     ,
  i_dimension_cnt            int    default null
)
returns float
as
$$
begin
  return
  (
    select
      power(sum(power(i_points_arr[1][a_dimension_no] - i_points_arr[2][a_dimension_no], 2)), 0.5 :: float)
    from generate_series(1, coalesce(i_dimension_cnt, array_length(i_points_arr, 2))) tb_a_dimension_no(a_dimension_no)
  )
  ;
end;
$$
  language plpgsql volatile
  cost 100;
-- select sm_sc.fv_o_distance(array[array[22.2, 77.7], array[55.5, 33.3]], 2)
-- select sm_sc.fv_o_distance(array[array[22.2, 77.7], array[55.5, 33.3]])

-- --------------------------------
drop function if exists sm_sc.fv_o_distance(float[], float[], int);
create or replace function sm_sc.fv_o_distance
(
  i_host_points_arr               float[]     ,
  i_guest_points_arr               float[]     ,
  i_dimension_cnt            int   default null
)
returns float
as
$$
begin
  return
  (
    select
      power(sum(power(i_host_points_arr[a_dimension_no] - i_guest_points_arr[a_dimension_no], 2)), 0.5 :: float)
    from generate_series(1, coalesce(i_dimension_cnt, least(array_length(i_host_points_arr, 1), array_length(i_guest_points_arr, 1)))) tb_a_dimension_no(a_dimension_no)
  )
  ;
end;
$$
  language plpgsql volatile
  cost 100;
-- select sm_sc.fv_o_distance(array[22.2, 77.7], array[55.5, 33.3], 2)
-- select sm_sc.fv_o_distance(array[22.2, 77.7], array[55.5, 33.3])
-- drop function if exists sm_sc.fv_range_move(anyrange, anyelement);
create or replace function sm_sc.fv_range_move
(
  i_range          anyrange,
  i_move           anyelement
)
returns anyrange
as
$$
-- declare
--   v_range    text   :=   i_range;
begin
  return 
    case when lower_inc(i_range) then '[' else '(' end ||       -- substr(v_range, 1, 1) || 
    lower(i_range) + i_move || 
    ',' || 
    upper(i_range) + i_move || 
    case when upper_inc(i_range) then ']' else ')' end
  ;
end 
$$
language plpgsql stable
parallel safe
cost 100;

-- select sm_sc.fv_range_move(int8range(1, 3, '[]'), 6 :: bigint)
-- select sm_sc.fv_range_move(numrange(1.6, 3.9, '[]'), -2.9)
drop function if exists sm_sc.fv_regexp_matches_strs_once(text, text, text);
create or replace function sm_sc.fv_regexp_matches_strs_once
(
  in i_text text, 
  in i_reg  text, 
  in i_opt  text default ''
)
  -- returns timestamp 
  returns text[]
as
$$
-- declare here
begin
  -- 最终输出
  if i_opt like '%g%'
  then
    raise exception 'this fn not support globel option';
  else
    return (select regexp_matches(i_text, i_reg, i_opt) limit 1);
  end if
  ; 

  -- exception, then null
  exception
    when others 
    then 
      return null;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_regexp_matches_strs_once('aaaaabbbaaa', 'a.a')
-- select sm_sc.fv_regexp_matches_strs_once('aaaaabbbaaa', 'A.A', 'i')
-- -- https://www.zhihu.com/question/35199474

-- -- ① 来自不同特征值的特征向量线性无关。
-- -- ② n 重特征值最多有 n-r(λiE-A) 个特征值。特别的，一重特征值有且仅有一个线性无关的特征向量。
-- -- n重根如果有n个线性无关的特征向量,则也可对角化

-- -- 方阵特征值对应的基础解系包含的向量个数不大于特征根的重数。比如说一个二重特征根它对应的特征向量基础解系的个数可能为2也可能为1。有的课本把特征根的重数叫做特征值的代数重数，把特征值对应特征向量的基础解系个数叫做特征值的几何重数，于是刚才所描述的原理也可以这么叙述：方阵特征根的几何重数不大于代数重数。那么题主可能要问，特征根几何重数等于代数重数的方阵，和特征根几何重数小于代数重数的方阵有什么区别呢？区别在于，如果一个方阵的所有特征根的几何重数均等于其代数重数，那么这个方阵是可以相似对角化的，如果这个方阵中只要有一个（或以上）的特征值的几何重数小于其代数重数，那么这个方阵就是无法相似对角化的。

-- -- 假设可对角化，基础解系相关x数量判定，
-- -- https://zhidao.baidu.com/question/1367688294526474059.html

-- -- define return eigen_values + eigen_arrays struct
-- drop type if exists cls_mx_eigen_return;
-- create type cls_mx_eigen_return as
-- (
--   mx_eigen_values    float[]      ,   -- 特征值，按大小排序
--   mx_eigen_arrays    float[][]        -- 特征向量，与mx_eigen_values一一对应；转置后便是特征分解的左边矩阵；再取逆后便是右边矩阵
-- );
-- 
-- -- --------------------------------

-- drop function sm_sc.ft_mx_evd(float[][]);
create or replace function sm_sc.ft_mx_evd
(
  in i_matrix float[][]
)
  returns table
  (
    o_eigen_value    float         ,
    o_eigen_array    float[]
  )
as
$$
-- declare here
declare 
  v_len                        int                   := array_length(i_matrix, 1);   -- 入参矩阵阶数
  v_eigen_values               float[];                                      -- 出参所有特征值
  v_eigen_values_distinct      float[];                                      -- 出参去重特征值
  v_len_distinct               int;                                                  -- 去重特征值个数（循环求解特征向量次数）
  v_eigen_arrays               float[][];                                    -- 出参所有特征向量
  v_eigen_array                float[];                                      -- 临时变量，满秩特征方程的特解(特征向量)
 
  v_loop_cur                   int                   := 1;                           -- 循环自增控制变量
  v_determinant_mx             float[][];                                    -- 行列式矩阵
  v_determinant_mx_sim         float[][];                                    -- 行列式矩阵最简行阶梯矩阵
  v_related_xs                 int[];                                                -- 相关列元编号清单
  v_unrelated_xs               int[];                                                -- 无关列元编号清单
  v_related_xs_len             int;                                                  -- 相关列元个数（当次特征值对应特征向量个数）
  v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);

begin
  -- -- drop table if exists sm_sc.__vt_tmp_eigen_arrays;
  -- -- create temp table sm_sc.__vt_tmp_eigen_arrays
  -- -- (
  -- --   sess_id        char(32)    ,
  -- --   eigen_value    float     ,
  -- --   eigen_array    float[]
  -- -- ) with (parallel_workers = 4);

  v_eigen_values = sm_sc.fv_mx_evd_value(i_matrix);

  -- 重复特征值，只循环计算一次，可能会算出对应多个特征向量。
  -- 基于8位精度特征值，以5位精度归零足够小的特征值，发现相同的零特征值归为一组，使代数重数与几何重数更准确
  v_eigen_values_distinct = (select array_agg(distinct round(eigen_value, 5) order by round(eigen_value, 5) desc) from unnest(v_eigen_values) eigen_value);
  v_len_distinct = array_length(v_eigen_values_distinct, 1);
  
  -- 代入每个特征值，求特征向量
  while v_loop_cur <= v_len_distinct
  loop
    -- 代入特征值，生成行列式矩阵
    v_determinant_mx = 
      sm_sc.fv_eye_unit
      (
        v_len
        , v_eigen_values_distinct[v_loop_cur]
      )
      -` i_matrix
    ;

    v_determinant_mx_sim = 
      sm_sc.fv_mx_rows_step_simple
      (
        v_determinant_mx
      );

    -- 查找非独立相关x编号清单（v_len - 行列式矩阵的秩）
    -- 非独立相关x的个数上限为当次特征值重复值出现次数，按上三角数值大小倒序选取；上三角数值约束绝对值远大于0；
    -- 期望非独立相关x的个数就是当次特征值重复值出现次数，否则几何重数小于代数重数，该矩阵不是可对角化矩阵；
    -- 如果存在上三角数值绝对值远大于0，却未入选非独立相关x，会是误差引起
    v_related_xs = 
      (
        with 
        cte_related_xs as
        (
          select 
            v_cur_x,
            max(abs(v_determinant_mx_sim[v_cur_y][v_cur_x])) as max_x   -- 同一个相关column x可能出现在多行，以绝对值最大的行值作为入选相关x依据
          from generate_series(1, v_len) v_cur_y
            , generate_series(v_cur_y + 1, v_len) v_cur_x
          where abs(v_determinant_mx_sim[v_cur_y][v_cur_x]) >= 1e-128 :: float
          group by v_cur_x
        ),
        cte_related_xs_grp as
        (
          select 
            v_cur_x
          from cte_related_xs
          order by max_x desc
          -- 未出现重复，则v_related_xs为空
          -- 权衡取舍后，绝对值小于0.00001的真实特征值，都将近似为0
          limit coalesce(nullif((select count(cur) from generate_series(1, v_len) cur where v_eigen_values_distinct[v_loop_cur] = round(v_eigen_values[cur], 5) :: float), 1), 0)  
        )
        select 
          array_agg(distinct v_cur_x order by v_cur_x)
        from cte_related_xs_grp
      );

    -- 生成独立无关x编号清单
    v_unrelated_xs = 
      (
        select 
          array_agg(v_related_cur order by v_related_cur) 
        from generate_series(1, v_len) v_related_cur
        left join unnest(v_related_xs) v_unrelated
        on v_unrelated = v_related_cur
        where v_unrelated is null
      );

    -- 求解基础解系
    v_related_xs_len = array_length(v_related_xs, 1);
    if v_related_xs_len is null
    then
      -- 满秩，求解非零独立x，除以L2范数正交化
      v_eigen_array = 
        sm_sc.fv_mx_ele_2d_2_1d
        (
          |^~|
          (
            sm_sc.fv_mx_rows_step_simple
            (
              v_determinant_mx |||| array_fill(1.0 :: float, array[v_len])  -- 方程右侧全是等值，得出的求解和基础解系同比例，为啥呢？
            )
          )[:][v_len + 1 : v_len + 1]
        )
      ;

      insert into sm_sc.__vt_tmp_eigen_arrays(sess_id, eigen_value, eigen_array)
      select 
        v_sess_id,
        v_eigen_values_distinct[v_loop_cur],
        v_eigen_array /` sm_sc.fv_arr_norm_2(v_eigen_array)
       ;

    else
      -- 欠秩，且假设可对角化
      -- 构造基础解系方程组
      with 
      cte_basic_solute 
      (
        eigen_array_grp    ,
        pos                ,
        array_meta_val
      )
      as
      (
        select 
          -- -- -- -- 相关x的序号清单
          -- -- -- ,
          -- 用相关列做自由变量，求取基础解系中无关x的值
          related_x_cur, array_meta_related.*
        from 
          generate_series(1, v_related_xs_len) related_x_cur,
          unnest
          (
            v_unrelated_xs,
            ( -- 相关列移动到方程等号右边，变号
              -` 
              sm_sc.fv_mx_ele_2d_2_1d 
              ( -- 找到所有相关x列
                |^~| v_determinant_mx_sim[1 : v_len - v_related_xs_len][v_related_xs[related_x_cur] : v_related_xs[related_x_cur]]
              )
            )
          ) array_meta_related
        union all
        -- 自然基底1作为相关x值，然后和无关x值按照x_no顺序合并
        select 
          -- 当次cur的自然基底向量
          related_x_cur as eigen_array_grp
          , v_unrelated_xs[related_x_cur] as pos
          , array_meta_unrelated as array_meta_val
        from generate_series(1, v_related_xs_len) related_x_cur
          , unnest(sm_sc.fv_mx_ele_2d_2_1d((sm_sc.fv_eye_unit(v_related_xs_len, 1.0 :: float))[related_x_cur:related_x_cur][:])) as array_meta_unrelated
      )
      insert into sm_sc.__vt_tmp_eigen_arrays(sess_id, eigen_value, eigen_array)
      select 
        v_sess_id,
        eigen_value, 
        eigen_array_x /` sm_sc.fv_arr_norm_2(eigen_array_x)  as eigen_array   -- 除以L2范数正交化
      from 
      (
        select 
          v_eigen_values_distinct[v_loop_cur]     as eigen_value   ,
          array_agg(array_meta_val order by pos)      as eigen_array_x
        from cte_basic_solute
        group by eigen_array_grp
      )  t_tmp
      ;

    end if;
    

    v_loop_cur = v_loop_cur + 1;
  end loop;

  return query
    select 
      eigen_value    as o_eigen_value    ,
      eigen_array    as o_eigen_array
    from sm_sc.__vt_tmp_eigen_arrays
    where sess_id = v_sess_id
  ;

  delete from sm_sc.__vt_tmp_eigen_arrays where sess_id = v_sess_id;
end
$$
  language plpgsql volatile
  parallel unsafe
  cost 100;

-- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[1,2],[3,4]])
-- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[1,2,3],[0,8,9],[4,5,6]])
-- -- https://jingyan.baidu.com/article/27fa7326afb4c146f8271ff3.html
-- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[-1,1,0],[-4,3,0],[1,0,2]])  -- 特征值相同例子，无法对角化
-- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[-2,1,1],[0,2,0],[-4,1,3]])  -- 特征值相同例子，可对角化
-- -- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[1,2,3,4],[0,8,9,10],[4,5,6,7],[5,7,2,10]])  -- 本算法不支持求解复数特征值
-- -- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[0,1,2],[1,1,4],[2,-1,0]])  -- 本算法不支持求解复数特征值
-- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[
--   [1.0000,2.0000,3.0000,4.0000,5.0000,6.0000,7.0000],  
--   [2.0000,3.0000,4.0000,5.0000,6.0000,7.0000,8.0000],  
--   [3.0000,4.0000,5.0000,6.0000,7.0000,8.0000,9.0000],  
--   [4.0000,5.0000,6.0000,7.0000,8.0000,9.0000,10.000],   
--   [5.0000,6.0000,7.0000,8.0000,9.0000,10.000,11.000],   
--   [6.0000,7.0000,8.0000,9.0000,10.000,11.000,12.000],   
--   [7.0000,8.0000,9.0000,10.000,11.000,12.000,13.000]
--   ])


-- select o_eigen_value, o_eigen_array from sm_sc.ft_mx_evd(array[[2,3],[1,7]])
-- truncate table t_tmp_debug_info
-- select * from t_tmp_debug_info




-- -- https://blog.csdn.net/Supper_Shenchao/article/details/121785115

-- drop function sm_sc.ft_mx_lu(float[][]);
create or replace function sm_sc.ft_mx_lu
(
  in i_mx float[][]
)
--   returns float[2][][]  
-- -- 出参解释：
-- -- [1][:][:]: L矩阵
-- -- [2][:][:]: U矩阵
returns TABLE
(
  o_mx_l    float[][],
  o_mx_u    float[][]
)

as
$$
-- declare here
declare 
  v_matrix_len  int                   :=  array_length(i_mx, 1);
  v_mx_l        float[][]    :=  sm_sc.fv_eye_unit(v_matrix_len);       -- 下三角矩阵
  v_mx_u        float[][]    :=  array_fill(0.0 :: float, array[v_matrix_len, v_matrix_len]);    -- 上三角矩阵
  
  v_cur_1       int;
  v_cur_2       int;

begin

  if v_matrix_len <> array_length(i_mx, 2)
  then 
    raise exception 'array_length(i_mx, 2) should be equal to array_length(i_mx, 1)';
  -- -- -- elsif
  -- -- --   -- -- -- 判断 顺序主子式 是否为零
  -- -- -- then 
  -- -- --   
  else
    -- 初始化 上三角矩阵第一行，（期待，可以优雅的对数组变量的切片整体赋值）
    for v_cur_1 in 1 .. v_matrix_len
    loop 
      v_mx_u[1][v_cur_1] = i_mx[1][v_cur_1];
    end loop;

    for v_cur_1 in 2 .. v_matrix_len
    loop 
    
      -- 更新 v_mx_l 这一列
      for v_cur_2 in v_cur_1 .. v_matrix_len
      -- 根据矩阵乘法，i_mx[v_cur_2][v_cur_1] = |@+| (v_mx_l[v_cur_2 : v_cur_2][ : ] *` v_mx_u[ : ][v_cur_1 : v_cur_1])
      -- 又，v_mx_u 上三角矩阵，所以 v_mx_u[v_cur_1 + 1 : ][v_cur_1 : v_cur_1] 的元素都是 0.0；
      -- 推出：i_mx[v_cur_2][v_cur_1] = |@+| (v_mx_l[v_cur_2 : v_cur_2][ : v_cur_1] *` v_mx_u[ : v_cur_1][v_cur_1 : v_cur_1])
      --   即：i_mx[v_cur_2][v_cur_1 - 1] = |@+| (v_mx_l[v_cur_2 : v_cur_2][ : v_cur_1 - 2] *` v_mx_u[ : v_cur_1 - 2][v_cur_1 - 1 : v_cur_1 - 1]) 
      --                                       + (v_mx_l[v_cur_2][v_cur_1 - 1] * v_mx_u[v_cur_1 - 1][v_cur_1 - 1])
      loop 
        v_mx_l[v_cur_2][v_cur_1 - 1] := (i_mx[v_cur_2][v_cur_1 - 1] - coalesce((|@+| (sm_sc.fv_mx_ele_2d_2_1d(v_mx_l[v_cur_2 : v_cur_2][ : v_cur_1 - 2]) *` sm_sc.fv_mx_ele_2d_2_1d(v_mx_u[ : v_cur_1 - 2][v_cur_1 - 1 : v_cur_1 - 1]))), 0.0 :: float)) / nullif(v_mx_u[v_cur_1 - 1][v_cur_1 - 1], 0.0 :: float);
      end loop;


      -- 更新 v_mx_u 这一行
      for v_cur_2 in v_cur_1 .. v_matrix_len
      -- 根据矩阵乘法，i_mx[v_cur_1][v_cur_2] = |@+| (v_mx_l[v_cur_1 : v_cur_1][ : ] *` v_mx_u[ : ][v_cur_2 : v_cur_2])
      -- 又，v_mx_l 下三角矩阵，所以 v_mx_l[v_cur_1 : v_cur_1][v_cur_1 + 1 : ] 的元素都是 0.0；v_mx_l[v_cur_1][v_cur_1] == 1
      -- 推出：i_mx[v_cur_1][v_cur_2] = |@+| (v_mx_l[v_cur_1 : v_cur_1][ : v_cur_1] *` v_mx_u[ : v_cur_1][v_cur_2 : v_cur_2])
      --   即：i_mx[v_cur_1][v_cur_2] = |@+| (v_mx_l[v_cur_1 : v_cur_1][ : v_cur_1 - 1] *` v_mx_u[ : v_cur_1 - 1][v_cur_2 : v_cur_2]) 
      --                                   + (v_mx_l[v_cur_1][v_cur_1] * v_mx_u[v_cur_1][v_cur_2])
      -- 也即：i_mx[v_cur_1][v_cur_2] = |@+| (v_mx_l[v_cur_1 : v_cur_1][ : v_cur_1 - 1] *` v_mx_u[ : v_cur_1 - 1][v_cur_2 : v_cur_2]) 
      --                                   + v_mx_u[v_cur_1][v_cur_2])
      loop 
        v_mx_u[v_cur_1][v_cur_2] := i_mx[v_cur_1][v_cur_2] - coalesce(|@+| (sm_sc.fv_mx_ele_2d_2_1d(v_mx_l[v_cur_1 : v_cur_1][ : v_cur_1 - 1]) *` sm_sc.fv_mx_ele_2d_2_1d(v_mx_u[ : v_cur_1 - 1][v_cur_2 : v_cur_2])), 0.0 :: float);
      end loop;

    end loop
    ;
  end if;
  
  return query
    select v_mx_l, v_mx_u
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select o_mx_l, o_mx_u from sm_sc.ft_mx_lu(array[[1,2],[3,4]])
-- select o_mx_l, o_mx_u from sm_sc.ft_mx_lu(array[[1,2,3],[0,8,9],[4,5,6]])
-- select o_mx_l, o_mx_u from sm_sc.ft_mx_lu(array[[2,1,-1,3],[4,3,-2,11],[-4,1,5,8],[6,2,12,3]])
-- select o_mx_l |**| o_mx_u from sm_sc.ft_mx_lu(array[[2,1,-1,3],[4,3,-2,11],[-4,1,5,8],[6,2,12,3]])
-- select o_mx_l, o_mx_u from sm_sc.ft_mx_lu(array[[0,1,2],[1,1,4],[2,-1,0]])

-- select o_mx_l, o_mx_u from sm_sc.ft_mx_lu(array[[2,3],[1,7]])
-- -- householder
-- -- https://wenku.baidu.com/view/c2e34678168884868762d6f9.html
-- -- https://wenku.baidu.com/view/63a32e72f242336c1eb95e89.html

-- -- define return Q+R struct
-- drop type if exists cls_mx_qr_return;
-- create type cls_mx_qr_return as
-- (
--   mx_q    float[][]      ,
--   mx_r    float[][]
-- );
-- 
-- -- --------------------------------

-- drop function sm_sc.ft_mx_qr(float[][]);
create or replace function sm_sc.ft_mx_qr
(
  in i_matrix float[][]
)
--   returns float[2][][]  
-- -- 出参解释：
-- -- [1][:][:]: q矩阵
-- -- [2][:][:]: r矩阵
returns TABLE
(
  o_mx_q    float[][],
  o_mx_r    float[][]
)

as
$$
-- declare here
declare 
  v_loop_cnt_01           int                   := 0;
  v_loop_cnt_02           int;
  v_loop_matrix           float[][]     := i_matrix;
 
  v_dims_len                int                   :=   array_length(i_matrix, 1) ;                 -- 方阵维数
  v_col_array             float[]; -- 当次循环下，待变换一列的下子列
  v_col_down_part_norm    float;   -- 当次循环下，待变换一列的下子列的模
  v_array_e               float[];  -- 当次循环下，自然基底向量
  v_array_w               float[];  -- 当次循环下，欧米伽法向量
  v_array_w_norm          float;    -- 当次循环下，欧米伽法向量的模
  v_mx_w                  float[][];  -- 当次循环下，欧米伽矩阵
  v_mx_w_prepend          float[][];  -- 当次循环下，欧米伽矩阵的左上扩展
  v_mx_q                  float[][]    := sm_sc.fv_eye_unit(v_dims_len);  -- q矩阵
  v_mx_r                  float[][]    := i_matrix;  -- r矩阵

begin


  if v_dims_len = array_length(i_matrix, 2)
    -- -- -- and /*待处理欠秩判断？？*/
  then
    while v_loop_cnt_01 < v_dims_len - 1
    loop
      -- 准备本次迭代入参子矩阵
      v_loop_cnt_01 = v_loop_cnt_01 + 1;
      v_loop_matrix = v_mx_r[v_loop_cnt_01 : ][v_loop_cnt_01 : ];

      -- 下子列
      v_col_array = sm_sc.fv_mx_ele_2d_2_1d(|^~| v_loop_matrix[1:][1:1]);

      -- 下子列的模
      v_col_down_part_norm = sm_sc.fv_arr_norm_2(v_col_array);  
      if v_col_down_part_norm < 1e-128 :: float
      then 
        continue;
      end if;

      -- 自然基底向量
      v_array_e = array_fill(0.0 :: float, array[v_dims_len + 1 - v_loop_cnt_01]);
      v_array_e[1] = 1.0;

      -- 欧米伽法向量
      v_array_w = v_col_array -` (v_col_down_part_norm *` v_array_e);   -- 下一步才除以模
      v_array_w_norm = sm_sc.fv_arr_norm_2(v_array_w);
      if v_array_w_norm < 1e-128 :: float
      then 
        continue;
      end if;

      v_array_w = v_array_w /` v_array_w_norm;

      -- 欧米伽矩阵
      v_mx_w = sm_sc.fv_eye_unit(v_dims_len + 1 - v_loop_cnt_01) -` (2.0 :: float *` ((|^~| array[v_array_w]) |**| array[v_array_w]));

      -- 欧米伽矩阵的左上扩展
      v_mx_w_prepend = sm_sc.fv_area_replace(sm_sc.fv_eye_unit(v_dims_len), array[v_loop_cnt_01, v_loop_cnt_01], v_mx_w);

      -- 迭代Q矩阵
      v_mx_q = v_mx_q |**| v_mx_w_prepend;

      -- 迭代R矩阵
      v_mx_r = v_mx_w_prepend |**| v_mx_r;

    end loop;

    -- return array[v_mx_q, v_mx_r]
    return query 
      select v_mx_q, v_mx_r
    ;

  else
    return query 
      select 
        null :: float[][], 
        null :: float[][];
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select o_mx_q, o_mx_r from sm_sc.ft_mx_qr(array[[1,2],[3,4]])
-- select o_mx_q, o_mx_r from sm_sc.ft_mx_qr(array[[1,2,3],[0,8,9],[4,5,6]])
-- select o_mx_q, o_mx_r from sm_sc.ft_mx_qr(array[[1,2,3,4],[0,8,9,10],[4,5,6,7],[5,7,2,10]])
-- select o_mx_q, o_mx_r from sm_sc.ft_mx_qr(array[[0,1,2],[1,1,4],[2,-1,0]])

-- select o_mx_q, o_mx_r from sm_sc.ft_mx_qr(array[[2,3],[1,7]])
-- truncate table t_tmp_debug_info
-- select * from t_tmp_debug_info
-- drop function sm_sc.ft_mx_svd(float[][]);
create or replace function sm_sc.ft_mx_svd
(
  in i_matrix float[][]
)
  returns table
  (
    o_singular_values     float[]           ,
    o_singular_matrix_u    float[][]         ,
    o_singular_matrix_v    float[][]
  )
as
$$
-- declare here
declare 
  v_len_u int := array_length(i_matrix, 1);
  v_len_v int := array_length(i_matrix, 2);

begin

  return query
    with 
    cte_singular_u as
    (
      select
        array_agg(sqrt(o_eigen_value) order by o_eigen_value desc) as o_eigen_values_sqrt,
        array_agg(o_eigen_array order by o_eigen_value desc) as o_eigen_arrays
      from sm_sc.ft_mx_evd(i_matrix |**| (|^~| i_matrix))
    ),
    cte_singular_v as
    (
      select
        -- array_agg(o_eigen_value order by o_eigen_value desc) as o_eigen_values,
        array_agg(o_eigen_array order by o_eigen_value desc) as o_eigen_arrays
      from sm_sc.ft_mx_evd((|^~| i_matrix) |**| i_matrix)
    )
    select 
      t_u.o_eigen_values_sqrt[ 1 : least(v_len_u, v_len_v) ]    as o_singular_values      ,
      |^~| t_u.o_eigen_arrays              as o_singular_matrix_u    ,
      |^~| t_v.o_eigen_arrays              as o_singular_matrix_v
    from cte_singular_u t_u
      , cte_singular_v t_v
  ;

end
$$
  language plpgsql volatile
  cost 100;

-- select o_singular_values, o_singular_matrix_u, o_singular_matrix_v from sm_sc.ft_mx_svd(array[
--     [1,1,1],
--     [1,2,3],
--     [1,10,100],
--     [1,0,0]
--   ])




-- drop function sm_sc.fv_arr_norm_1(float[]);
create or replace function sm_sc.fv_arr_norm_1
(
  in i_array float[]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      sum(abs(i_array[v_cur]))
    from generate_series(1, array_length(i_array, 1)) v_cur
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_arr_norm_1(array[1,2])
-- select sm_sc.fv_arr_norm_1(array[1,2,3])
-- drop function sm_sc.fv_arr_norm_2(float[]);
create or replace function sm_sc.fv_arr_norm_2
(
  in i_array float[]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      sqrt(sum(i_array[v_cur] * i_array[v_cur]))
    from generate_series(1, array_length(i_array, 1)) v_cur
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_arr_norm_2(array[1,2])
-- select sm_sc.fv_arr_norm_2(array[1,2,3])
-- drop function sm_sc.fv_arr_norm_nega_inf(float[]);
create or replace function sm_sc.fv_arr_norm_nega_inf
(
  in i_array float[]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      min(abs(i_array[v_cur]))
    from generate_series(1, array_length(i_array, 1)) v_cur
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_arr_norm_nega_inf(array[1,2])
-- select sm_sc.fv_arr_norm_nega_inf(array[1,2,3])
-- drop function sm_sc.fv_arr_norm_p(float[], int);
create or replace function sm_sc.fv_arr_norm_p
(
  in i_array float[],
  in i_p     int
)
  returns float
as
$$
-- declare here

begin
  if i_p > 0
  then
    return
    (
      select 
        power(sum(power(i_array[v_cur], i_p)), 1.0 :: float/ i_p)
      from generate_series(1, array_length(i_array, 1)) v_cur
    )
    ;
  else
    return null;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_arr_norm_p(array[1,2], 2)
-- select sm_sc.fv_arr_norm_p(array[1,2,3], 3)
-- drop function sm_sc.fv_arr_norm_posi_inf(float[]);
create or replace function sm_sc.fv_arr_norm_posi_inf
(
  in i_array float[]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      max(abs(i_array[v_cur]))
    from generate_series(1, array_length(i_array, 1)) v_cur
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_arr_norm_posi_inf(array[1,2])
-- select sm_sc.fv_arr_norm_posi_inf(array[1,2,3])
-- drop function sm_sc.fv_mx_col_mix(float[][], int, int, int);
create or replace function sm_sc.fv_mx_col_mix
(
  in i_original_matrix   float[][]   ,
  in i_target_column_no  int                  ,
  in i_target_row_no     int                  ,
  in i_refer_column_no   int
)
  returns float[][]
as
$$
-- declare here
declare v_cur       int := 1;
declare v_times     float;

begin

  if i_original_matrix[i_target_row_no][i_target_column_no] = 0.0
  then 
    return i_original_matrix;

  elseif i_original_matrix[i_target_row_no][i_refer_column_no] = 0.0
  then
    return null::float[][];

  else
    v_times = i_original_matrix[i_target_row_no][i_target_column_no] / i_original_matrix[i_target_row_no][i_refer_column_no];
    while v_cur <= array_length(i_original_matrix, 1)
    loop
      i_original_matrix[v_cur][i_target_column_no] = i_original_matrix[v_cur][i_target_column_no] - i_original_matrix[v_cur][i_refer_column_no] * v_times;
      v_cur = v_cur + 1;
    end loop;
    i_original_matrix[i_target_row_no][i_target_column_no] = 0.0;
    return i_original_matrix;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_col_mix(array[[1.9, 34.5, 0.55, 45.7, 400.5],[45.9, 4.6, 34.5, 0.55, 45.7],[3.2, 7.7, 1.9, 34.5, 0.55],[4.7, 9.0, 4.6, 34.5, 0.55]], 2, 3, 4)

-- drop function sm_sc.fv_mx_determinant(float[][]);
create or replace function sm_sc.fv_mx_determinant
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here
declare v_cur_loop_1       int         :=   1;
declare v_cur_loop_2       int;
declare v_y_len            int         :=   array_length(i_matrix, 1);
declare v_x_len            int         :=   array_length(i_matrix, 2);
declare v_row_no           int;
declare v_compare_val      float;
declare v_cur_part_array   float[];
declare v_is_negetive      boolean     :=  true;
declare v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);

begin
  -- -- -- pg的多维数组服务操纵切片，转化为一维数组表
  -- -- drop table if exists sm_sc.__vt_tmp_matrix;
  -- -- create temp table sm_sc.__vt_tmp_matrix
  -- -- (
  -- --   sess_id                char(32)        ,  
  -- --   -- org_row_no       int            ,    -- 原行号
  -- --   new_row_no       int            ,    -- 新行号
  -- --   array_x          float[]      -- ,    -- 行向量
  -- --   -- transform_desc   text           ,    -- 初等变换轨迹描述
  -- --   -- primary key(sess_id, org_row_no)
  -- -- ) with (parallel_workers = 4);
  -- -- 
  -- -- create index idx_tmp_matrix on sm_sc.__vt_tmp_matrix (sess_id, new_row_no);

  insert into sm_sc.__vt_tmp_matrix
  (
    sess_id,
    -- org_row_no  ,
    new_row_no  ,
    array_x
  )
  select 
    v_sess_id,
    -- v_cur_loop_1_y,
    v_cur_loop_1_y,
    array_agg(i_matrix[v_cur_loop_1_y][v_cur_loop_1_x] order by v_cur_loop_1_x)
  from generate_series(1, v_y_len) v_cur_loop_1_y
    , generate_series(1, v_x_len) v_cur_loop_1_x
  group by v_cur_loop_1_y
  ;

  -- 沿主对角线，逐行化简为上梯形
  while v_cur_loop_1 < v_y_len
  loop 
    -- 判断后续行，该列元是否都为0。存在非零则进入该行该列元判断；不存在非零则跳过该列；
    if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id)
    then 
      -- 如果当前行当前列元为零，则需换行
      if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and array_x[v_cur_loop_1] = 0.0 and sess_id = v_sess_id)
      then
        v_row_no = (select new_row_no from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id limit 1);

        update sm_sc.__vt_tmp_matrix
        set new_row_no = case new_row_no 
                           when v_row_no then v_cur_loop_1 
                           else v_row_no
                         end
        where new_row_no in (v_row_no, v_cur_loop_1)
          and sess_id = v_sess_id
		;

        -- update sm_sc.__vt_tmp_matrix
        -- set array_x[v_cur_loop_1 : ] = -` array_x[v_cur_loop_1 : ]
        -- where new_row_no = v_cur_loop_1
        --   and sess_id = v_sess_id
        -- ;

        v_is_negetive = not v_is_negetive;   -- 正负换号
      end if;

      -- 由方阵左上角，后续行逐行消元，成为0左下三角
      v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      update sm_sc.__vt_tmp_matrix
      set array_x[v_cur_loop_1 : ] 
            = array[0.0 :: float] ||
              (
                array_x[v_cur_loop_1 + 1 : ]
                -` 
                (
                  (array_x[v_cur_loop_1] / v_compare_val)
                  *` v_cur_part_array
                )
              )
      where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0
        and sess_id = v_sess_id
      ;
   
    elseif exists (select  from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and array_x[v_cur_loop_1] = 0.0 and sess_id = v_sess_id)
    then 
      v_is_negetive = null;
      exit;  -- 终止
    end if;

    v_cur_loop_1 = v_cur_loop_1 + 1;
  end loop;

  -- 算乘积
  v_compare_val = 1.0;
  v_cur_loop_1 = 1;
  if v_is_negetive is not null
  then
    while v_cur_loop_1 <= least(v_y_len, v_x_len)
    loop
      v_compare_val = v_compare_val * (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      v_cur_loop_1 = v_cur_loop_1 + 1;
    end loop;
  end if;

  return
    case 
      when v_is_negetive is null then 0.0
      -- else v_compare_val
      when v_is_negetive is true then 1.0 :: float* v_compare_val
      when v_is_negetive is false then -1.0 :: float* v_compare_val
      else null
    end
  ;

  delete from sm_sc.__vt_tmp_matrix where sess_id = v_sess_id;
end
$$
  language plpgsql volatile
  parallel unsafe
  cost 100;

-- select sm_sc.fv_mx_determinant(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_determinant(array[[1,2,3],[0,8,9],[4,5,6]])
-- select sm_sc.fv_mx_determinant(array[[1,2,3,4],[0,8,9,10],[4,5,6,7],[8,2,6,4]])
-- -- householder
-- -- https://wenku.baidu.com/view/c2e34678168884868762d6f9.html
-- -- https://wenku.baidu.com/view/63a32e72f242336c1eb95e89.html

-- -- 本算法不支持求解复数特征值。通常准对称实矩阵，特征值为实数
-- drop function sm_sc.fv_mx_evd_value(float[][]);
create or replace function sm_sc.fv_mx_evd_value
(
  in i_matrix float[][]
)
  returns float[]
as
$$
-- declare here
declare 
  v_loop_cnt_01      int                   := 1;
  v_len              int                   := array_length(i_matrix, 1);

begin
  if v_len = array_length(i_matrix, 2)
  then
    while v_loop_cnt_01 <= 1000  -- 如果1000次循环内仍然未收敛，则终止
      and exists (select from generate_series(1, v_len) v_cur_y, generate_series(1, v_cur_y - 1) v_cur_x where abs(i_matrix[v_cur_y][v_cur_x]) > 1e-128 :: float)
    loop
      i_matrix = (select o_mx_r |**| o_mx_q from sm_sc.ft_mx_qr(i_matrix));
      v_loop_cnt_01 = v_loop_cnt_01 + 1;
    end loop;

    return 
      (select array_agg(i_matrix[v_cur][v_cur] order by i_matrix[v_cur][v_cur] desc) from generate_series(1, v_len) v_cur)
    ;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_evd_value(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_evd_value(array[[1,2,3],[0,8,9],[4,5,6]])
-- -- select sm_sc.fv_mx_evd_value(array[[1,2,3,4],[0,8,9,10],[4,5,6,7],[5,7,2,10]])  -- 本算法不支持求解复数特征值
-- select sm_sc.fv_mx_evd_value(array[[0,1,2],[1,1,4],[2,-1,0]])
-- select sm_sc.fv_mx_evd_value(array[
--   [1.0000,2.0000,3.0000,4.0000,5.0000,6.0000,7.0000],  
--   [2.0000,3.0000,4.0000,5.0000,6.0000,7.0000,8.0000],  
--   [3.0000,4.0000,5.0000,6.0000,7.0000,8.0000,9.0000],  
--   [4.0000,5.0000,6.0000,7.0000,8.0000,9.0000,10.000],   
--   [5.0000,6.0000,7.0000,8.0000,9.0000,10.000,11.000],   
--   [6.0000,7.0000,8.0000,9.0000,10.000,11.000,12.000],   
--   [7.0000,8.0000,9.0000,10.000,11.000,12.000,13.000]
--   ])


-- select sm_sc.fv_mx_evd_value(array[[2,3],[1,7]])
-- truncate table t_tmp_debug_info
-- select * from t_tmp_debug_info




-- -- -- ----------------------------------------------------------------------------

-- for debug data
-- -- with 
-- -- cte_loop_1 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr(array[[5,-3,2],[6,-4,4],[4,-4,5]])
-- -- ),
-- -- cte_loop_2 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_1))
-- -- ),
-- -- cte_loop_3 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_2))
-- -- ),
-- -- cte_loop_4 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_3))
-- -- ),
-- -- cte_loop_5 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_4))
-- -- ),
-- -- cte_loop_6 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_5))
-- -- ),
-- -- cte_loop_7 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_6))
-- -- ),
-- -- cte_loop_8 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_7))
-- -- ),
-- -- cte_loop_9 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_8))
-- -- ),
-- -- cte_loop_10 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_9))
-- -- ),
-- -- cte_loop_11 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_10))
-- -- ),
-- -- cte_loop_12 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_11))
-- -- ),
-- -- cte_loop_13 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_12))
-- -- ),
-- -- cte_loop_14 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_13))
-- -- ),
-- -- cte_loop_15 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_14))
-- -- ),
-- -- cte_loop_16 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_15))
-- -- ),
-- -- cte_loop_17 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_16))
-- -- ),
-- -- cte_loop_18 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_17))
-- -- ),
-- -- cte_loop_19 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_18))
-- -- ),
-- -- cte_loop_20 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_19))
-- -- ),
-- -- cte_loop_21 as
-- -- (
-- --   select o_mx_q, o_mx_r from sm_sc.ft_mx_qr((select o_mx_r |**| o_mx_q from cte_loop_20))
-- -- )
-- -- select o_mx_r |**| o_mx_q from cte_loop_21
-- drop function sm_sc.fv_mx_inversion(float[][]);
create or replace function sm_sc.fv_mx_inversion
(
  in i_matrix float[][]
)
  returns float[][]
as
$$
-- declare here
declare v_cur_loop_1       int         :=   1;
declare v_y_len            int         :=   array_length(i_matrix, 1);
declare v_x_len            int         :=   array_length(i_matrix, 2);
declare v_row_no           int;
declare v_compare_val      float;
declare v_cur_part_array   float[];
declare v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);

begin

  if v_y_len = v_x_len
  then
    -- -- -- pg的多维数组服务操纵切片，转化为一维数组表
    -- -- drop table if exists sm_sc.__vt_tmp_matrix;
    -- -- create temp table sm_sc.__vt_tmp_matrix
    -- -- (
    -- --   sess_id                char(32)        ,  
    -- --   -- org_row_no       int            ,    -- 原行号
    -- --   new_row_no       int            ,    -- 新行号
    -- --   array_x          float[]      -- ,    -- 行向量
    -- --   -- transform_desc   text           ,    -- 初等变换轨迹描述
    -- --   -- primary key(sess_id, org_row_no)
    -- -- ) with (parallel_workers = 4);
    -- -- 
    -- -- create index idx_tmp_matrix on sm_sc.__vt_tmp_matrix (sess_id, new_row_no);
    
    insert into sm_sc.__vt_tmp_matrix
    (
      sess_id,
      -- org_row_no  ,
      new_row_no  ,
      array_x
    )
    select 
      v_sess_id,
      -- v_cur_loop_1_y,
      v_cur_loop_1_y,
      array_agg
      (
        -- 横向拼接单位矩阵
        case 
          when v_cur_loop_1_x <= v_x_len 
            then i_matrix[v_cur_loop_1_y][v_cur_loop_1_x] 
          when v_cur_loop_1_x - v_x_len = v_cur_loop_1_y
            then 1.0
          else 0.0
        end
        order by v_cur_loop_1_x
      )
    from generate_series(1, v_y_len) v_cur_loop_1_y
      , generate_series(1, v_x_len * 2) v_cur_loop_1_x
    group by v_cur_loop_1_y
    ;
    
    -- 沿主对角线，逐行化简为上梯形
    while v_cur_loop_1 < v_y_len
    loop 
      -- 判断后续行，该列元是否都为0。存在非零则进入该行该列元判断；不存在非零则跳过该列；
      if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id)
      then 
        -- 如果当前行当前列元为零，则需换行
        if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and array_x[v_cur_loop_1] = 0.0 and sess_id = v_sess_id)
        then
          v_row_no = (select new_row_no from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id limit 1);
    
          update sm_sc.__vt_tmp_matrix
          set new_row_no = case new_row_no 
                             when v_row_no then v_cur_loop_1 
                             else v_row_no
                           end
          where new_row_no in (v_row_no, v_cur_loop_1)
            and sess_id = v_sess_id
	  	;
    
          -- -- 换行变号
          -- update sm_sc.__vt_tmp_matrix
          -- set array_x[v_cur_loop_1 : ] = -` array_x[v_cur_loop_1 : ]
          -- where new_row_no = v_cur_loop_1
          --   and sess_id = v_sess_id
          -- ;
        end if;
    
        -- 由方阵左上角，后续行逐行消元，成为0左下三角
        v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
        v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
        update sm_sc.__vt_tmp_matrix
        set array_x[v_cur_loop_1 : ] 
              = array[0.0 :: float] ||
                (
                  array_x[v_cur_loop_1 + 1 : ]
                  -` 
                  (
                    (array_x[v_cur_loop_1] / v_compare_val)
                    *` v_cur_part_array
                  )
                )
        where new_row_no > v_cur_loop_1 
          and array_x[v_cur_loop_1] <> 0.0
          and sess_id = v_sess_id
        ;
    
      end if;
    
      v_cur_loop_1 = v_cur_loop_1 + 1;
    end loop;
    
    -- 由方阵右下角，下至上前续行逐行消元，成为0左下三角
    v_cur_loop_1 = least(v_y_len, v_x_len);
    while v_cur_loop_1 > 1
    loop
      v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      if v_compare_val <> 0.0
      then
        v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
        update sm_sc.__vt_tmp_matrix
        set array_x[v_cur_loop_1 : ] 
              = array[0.0 :: float] ||
                (
                  array_x[v_cur_loop_1 + 1 : ]
                  -` 
                  (
                    (array_x[v_cur_loop_1] / v_compare_val)
                    *` v_cur_part_array
                  )
                )
        where new_row_no < v_cur_loop_1
          and array_x[v_cur_loop_1] <> 0.0
          and sess_id = v_sess_id
        ;
      end if;
      v_cur_loop_1 = v_cur_loop_1 - 1;
    end loop;

    -- 判断是否满秩
    if not exists (select  from sm_sc.__vt_tmp_matrix where array_x[new_row_no] = 0.0 and sess_id = v_sess_id)
    then
      -- 主对角线值归一
      update sm_sc.__vt_tmp_matrix
      set array_x[new_row_no : ] 
            = array[1.0 :: float] || (array_x[new_row_no + 1 : ] /` array_x[new_row_no])
      where new_row_no <= least(v_y_len, v_x_len)
        and sess_id = v_sess_id
        -- and array_x[new_row_no] <> 0.0
      ;
      
      return
      (
        select 
          array_agg
          (
            array_x[v_x_len + 1 : ]  -- 原来拼接的单位矩阵变换后就是逆矩阵
            order by array_x[least(new_row_no, v_x_len)] desc, new_row_no  -- 全零行，下移，此处往下，主对角线可能出现拐弯，相比“两行位置对调”，行序变化较大
          ) 
        from sm_sc.__vt_tmp_matrix
        where sess_id = v_sess_id
      )
      ;
    else 
      return null::float[][];
    end if;
  else
    return null::float[][];
  end if;

  delete from sm_sc.__vt_tmp_matrix where sess_id = v_sess_id;
end
$$
  language plpgsql volatile
  parallel unsafe
  cost 100;

-- select sm_sc.fv_mx_inversion(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_inversion(array[[1,2,3],[0,8,9],[4,5,6]])
-- select sm_sc.fv_mx_inversion(array[[1,2,3,4],[0,8,9,10],[4,5,6,7]])
-- select sm_sc.fv_mx_inversion(array[[0,1,2],[1,1,4],[2,-1,0]])
-- drop function sm_sc.fv_mx_is_symmetry(float[][]);
create or replace function sm_sc.fv_mx_is_symmetry
(
  in i_matrix float[][]
)
  returns boolean
as
$$
-- declare here
begin
  if array_length(i_matrix, 1) <> array_length(i_matrix, 2)
  then
    return false;
  elseif array_length(i_matrix, 1) = array_length(i_matrix, 2)
    and exists 
        (
          select from 
            generate_series(1, array_length(i_matrix, 1)) cur_y
            , generate_series(1, cur_y - 1) cur_x
          where i_matrix[cur_y][cur_x] <> i_matrix[cur_x][cur_y]
        )
  then
    return false;
  else 
    return true;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_is_symmetry(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_is_symmetry(array[[1,3,5,7],[2,4,6,8]])
-- select sm_sc.fv_mx_is_symmetry(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]])
-- -- 幂法/改进幂法/反幂法
-- -- https://wenku.baidu.com/view/c52758e31fb91a37f111f18583d049649b660ea5.html  第九章矩阵特征值与特征向量计算方法

-- drop function sm_sc.fv_mx_max_abs_evd_value(float[][]);
create or replace function sm_sc.fv_mx_max_abs_evd_value
(
  in i_matrix float[][]
)
   returns float
as
$$
-- declare here
declare 
  v_loop_cnt_01             int                   :=   0;
  v_dims_len                  int                   :=   array_length(i_matrix, 1) ;                 -- 方阵维数
								    
  v_cur_array_x             float[]       :=   array_fill(2.0 :: float, array[v_dims_len]); 
  v_cur_eigenvalue          float;
  v_last_cur_eigenvalue     float;
-- declare v_loop_fasten_factor      float         :=   0.0;  -- -- 迭代加速因子  -- 机制未知，待研究验证
-- declare v_loop_fasten_factor_2nd  float         :=   0.0;  -- -- 迭代加速初始因子

begin


  if v_dims_len = array_length(i_matrix, 2)
    -- and /*待处理欠秩、除数为零*/
  then
    while v_loop_cnt_01 < 100
      and (v_last_cur_eigenvalue is null or abs(v_cur_eigenvalue - v_last_cur_eigenvalue) >= 0.001) or v_loop_cnt_01 < 10
    loop 
      v_loop_cnt_01 = v_loop_cnt_01 + 1;
      v_last_cur_eigenvalue = v_cur_eigenvalue;
      -- if v_loop_cnt_01 = 2                                 -- --
      -- then                                                   -- --
      --   v_loop_fasten_factor_2nd = v_cur_eigenvalue;     -- --
      -- end if;                                                -- --


      -- v_cur_eigenvalue = (select max(v_cur_array_x[cur]) + v_loop_fasten_factor from generate_series(1, v_dims_len) cur);  -- --
      v_cur_eigenvalue = (select v_cur_array_x[cur] from generate_series(1, v_dims_len) cur order by abs(v_cur_array_x[cur]) limit 1);
      if abs(v_cur_eigenvalue) <= 0.0001
      then
        exit;
      end if;

      -- v_loop_fasten_factor = (v_cur_eigenvalue + v_loop_fasten_factor_2nd) / 2.0; -- --       
      v_cur_array_x = sm_sc.fv_mx_ele_2d_2_1d(|^~| (i_matrix |**| (|^~| (array[v_cur_array_x] /` v_cur_eigenvalue))));
      -- v_cur_array_x = sm_sc.fv_mx_ele_2d_2_1d(|^~| ((i_matrix -` sm_sc.fv_eye_unit(v_dims_len, v_loop_fasten_factor)) |**| (|^~| (v_cur_array_x /` v_cur_eigenvalue))));  -- --
    end loop;

    -- return case when abs(v_cur_eigenvalue - v_last_cur_eigenvalue) >= 0.001 then null else v_cur_eigenvalue + v_loop_fasten_factor end;  -- --
    return case when abs(v_cur_eigenvalue - v_last_cur_eigenvalue) >= 0.001 then null else v_cur_eigenvalue end;

  else
    return null;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_max_abs_evd_value(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_max_abs_evd_value(array[[1,2,3],[0,8,9],[4,5,6]])
-- select sm_sc.fv_mx_max_abs_evd_value(array[[1,2,3,4],[0,8,9,10],[4,5,6,7],[5,7,2,10]])
-- select sm_sc.fv_mx_max_abs_evd_value(array[[0,1,2],[1,1,4],[2,-1,0]])

-- select sm_sc.fv_mx_max_abs_evd_value(array[[2,3],[1,7]])
-- select sm_sc.fv_mx_max_abs_evd_value(array[[-2.0000,1.0000],[1.5000,-0.5000]])
-- truncate table t_tmp_debug_info
-- select * from t_tmp_debug_info
-- -- 幂法/改进幂法/反幂法
-- -- https://wenku.baidu.com/view/c52758e31fb91a37f111f18583d049649b660ea5.html  第九章矩阵特征值与特征向量计算方法

-- drop function sm_sc.fv_mx_min_abs_evd_value(float[][]);
create or replace function sm_sc.fv_mx_min_abs_evd_value
(
  in i_matrix float[][]
)
   returns float
as
$$
-- declare here

begin
  return 1.0 :: float/ sm_sc.fv_mx_max_abs_evd_value(sm_sc.fv_mx_inversion(i_matrix));
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_min_abs_evd_value(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_min_abs_evd_value(array[[1,2,3],[0,8,9],[4,5,6]])
-- select sm_sc.fv_mx_min_abs_evd_value(array[[1,2,3,4],[0,8,9,10],[4,5,6,7],[5,7,2,10]])
-- select sm_sc.fv_mx_min_abs_evd_value(array[[0,1,2],[1,1,4],[2,-1,0]])

-- select sm_sc.fv_mx_min_abs_evd_value(array[[2,3],[1,7]])
-- truncate table t_tmp_debug_info
-- select * from t_tmp_debug_info
-- -- https://zhuanlan.zhihu.com/p/35897775
-- drop function sm_sc.fv_mx_norm_col(float[][]);
create or replace function sm_sc.fv_mx_norm_col
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      max(sum_col_abs)
    from 
    (
      select 
        sum(abs(i_matrix[v_cur_y][v_cur_x])) as sum_col_abs
      from generate_series(1, array_length(i_matrix, 1)) v_cur_y
        , generate_series(1, array_length(i_matrix, 2)) v_cur_x
      group by v_cur_x
    ) v_sum_col_abs
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_norm_col(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_col(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_norm_eigenvalue(float[][]);
create or replace function sm_sc.fv_mx_norm_eigenvalue
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      sqrt(max((sm_sc.fv_mx_evd_value((|^~| i_matrix) |**| i_matrix))[v_cur_x]))
    from generate_series(1, array_length(i_matrix, 2)) v_cur_x
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- -- -- 尚未调试 等待 sm_sc.fv_mx_evd_value 完成
-- select sm_sc.fv_mx_norm_eigenvalue(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_eigenvalue(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_norm_inf(float[][]);
create or replace function sm_sc.fv_mx_norm_inf
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      max(sum_row_abs)
    from 
    (
      select 
        sum(abs(i_matrix[v_cur_y][v_cur_x])) as sum_row_abs
      from generate_series(1, array_length(i_matrix, 1)) v_cur_y
        , generate_series(1, array_length(i_matrix, 2)) v_cur_x
      group by v_cur_y
    ) v_sum_row_abs
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_norm_inf(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_inf(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_norm_l0(float[][]);
create or replace function sm_sc.fv_mx_norm_l0
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      count(*)
    from generate_series(1, array_length(i_matrix, 1)) v_cur_y
      , generate_series(1, array_length(i_matrix, 2)) v_cur_x
    where i_matrix[v_cur_y][v_cur_x] <> 0
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_norm_l0(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_l0(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_norm_l1(float[][]);
create or replace function sm_sc.fv_mx_norm_l1
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      sum(abs(i_matrix[v_cur_y][v_cur_x]))
    from generate_series(1, array_length(i_matrix, 1)) v_cur_y
      , generate_series(1, array_length(i_matrix, 2)) v_cur_x
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_norm_l1(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_l1(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_norm_l21(float[][]);
create or replace function sm_sc.fv_mx_norm_l21
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      sum(col_l2_f)
    from
    (
      select 
        sqrt(sum(i_matrix[v_cur_y][v_cur_x] * i_matrix[v_cur_y][v_cur_x])) as col_l2_f
      from generate_series(1, array_length(i_matrix, 1)) v_cur_y
        , generate_series(1, array_length(i_matrix, 2)) v_cur_x
      group by v_cur_x
    ) v_col_l2_f
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_norm_l21(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_l21(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_norm_l2_f(float[][]);
create or replace function sm_sc.fv_mx_norm_l2_f
(
  in i_matrix float[][]
)
  returns float
as
$$
-- declare here

begin
  return
  (
    select 
      sqrt(sum(i_matrix[v_cur_y][v_cur_x] * i_matrix[v_cur_y][v_cur_x]))
    from generate_series(1, array_length(i_matrix, 1)) v_cur_y
      , generate_series(1, array_length(i_matrix, 2)) v_cur_x
  )
  ;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_norm_l2_f(array[[1,2],[0,7],[5,0],[3,4]])
-- select sm_sc.fv_mx_norm_l2_f(array[[1,-2,0,7],[5,0,3,4],[0,0,0,0],[0,-2,0,-6]])
-- drop function sm_sc.fv_mx_rank(float[][]);
create or replace function sm_sc.fv_mx_rank
(
  in i_matrix float[][]
)
  returns int
as
$$
-- declare here
declare v_cur_loop_1       int         :=   1;
declare v_cur_loop_2       int;
declare v_y_len            int         :=   array_length(i_matrix, 1);
declare v_x_len            int         :=   array_length(i_matrix, 2);
declare v_row_no           int;
declare v_compare_val      float;
declare v_cur_part_array   float[];
declare v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);

begin
  -- -- -- pg的多维数组服务操纵切片，转化为一维数组表
  -- -- drop table if exists sm_sc.__vt_tmp_matrix;
  -- -- create temp table sm_sc.__vt_tmp_matrix
  -- -- (
  -- --   sess_id                char(32)        ,  
  -- --   -- org_row_no       int            ,    -- 原行号
  -- --   new_row_no       int            ,    -- 新行号
  -- --   array_x          float[]      -- ,    -- 行向量
  -- --   -- transform_desc   text           ,    -- 初等变换轨迹描述
  -- --   -- primary key(sess_id, org_row_no)
  -- -- ) with (parallel_workers = 4);
  -- -- 
  -- -- create index idx_tmp_matrix on sm_sc.__vt_tmp_matrix (sess_id, new_row_no);

  insert into sm_sc.__vt_tmp_matrix
  (
    sess_id,
    -- org_row_no  ,
    new_row_no  ,
    array_x
  )
  select 
    v_sess_id,
    -- v_cur_loop_1_y,
    v_cur_loop_1_y,
    array_agg(i_matrix[v_cur_loop_1_y][v_cur_loop_1_x] order by v_cur_loop_1_x)
  from generate_series(1, v_y_len) v_cur_loop_1_y
    , generate_series(1, v_x_len) v_cur_loop_1_x
  group by v_cur_loop_1_y
  ;

  -- 沿主对角线，逐行化简为上梯形
  while v_cur_loop_1 < v_y_len
  loop 
    -- 判断后续行，该列元是否都为0。存在非零则进入该行该列元判断；不存在非零则跳过该列；
    if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id)
    then 
      -- 如果当前行当前列元为零，则需换行
      if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and array_x[v_cur_loop_1] = 0.0 and sess_id = v_sess_id)
      then
        v_row_no = (select new_row_no from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id limit 1);

        update sm_sc.__vt_tmp_matrix
        set new_row_no = case new_row_no 
                           when v_row_no then v_cur_loop_1 
                           else v_row_no
                         end
        where new_row_no in (v_row_no, v_cur_loop_1)
          and sess_id = v_sess_id
		;

        -- -- 换行变号
        -- update sm_sc.__vt_tmp_matrix
        -- set array_x[v_cur_loop_1 : ] = -` array_x[v_cur_loop_1 : ]
        -- where new_row_no = v_cur_loop_1
        --   and sess_id = v_sess_id
        -- ;
      end if;

      -- 由方阵左上角，后续行逐行消元，成为0左下三角
      v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      update sm_sc.__vt_tmp_matrix
      set array_x[v_cur_loop_1 : ] 
            = array[0.0 :: float] ||
              (
                array_x[v_cur_loop_1 + 1 : ]
                -` 
                (
                  (array_x[v_cur_loop_1] / v_compare_val)
                  *` v_cur_part_array
                )
              )
      where new_row_no > v_cur_loop_1 
        and array_x[v_cur_loop_1] <> 0.0
        and sess_id = v_sess_id
      ;

    end if;

    v_cur_loop_1 = v_cur_loop_1 + 1;
  end loop;

  return
  (
    select 
      count(*)
    from generate_series(1, greatest(v_y_len, v_x_len)) diagonal_cur
    inner join sm_sc.__vt_tmp_matrix mx
    on mx.new_row_no = diagonal_cur
    where mx.array_x[diagonal_cur] <> 0
      and mx.sess_id = v_sess_id
  )
  ;

  delete from sm_sc.__vt_tmp_matrix where sess_id = v_sess_id;
end
$$
  language plpgsql volatile
  parallel unsafe
  cost 100;

-- select sm_sc.fv_mx_rank(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_rank(array[[1,2,3],[0,8,9],[4,5,6]])
-- drop function sm_sc.fv_mx_reorder_col(float[][], int[]);
create or replace function sm_sc.fv_mx_reorder_col
(
  in i_matrix       float[][],
  in i_new_order    int[]     -- etc.: (array[null,2,4,5,3]) mean old_4->new_3; old_5->new_4; old_3->new_5
)
  returns float[][]
as
$$
-- declare here
begin
  if array_length(i_new_order, 1) > array_length(i_matrix, 2)
    or
    exists 
    (
      select 
      from (select cur_old_pos_tar from unnest(i_new_order) cur_old_pos_tar where cur_old_pos_tar is not null) t1
      full join (select cur_new_pos_tar from generate_series(1, array_length(i_new_order, 1)) cur_new_pos_tar where i_new_order[cur_new_pos_tar] is not null) t2
      on t1.cur_old_pos_tar = t2.cur_new_pos_tar
      where i_new_order[t2.cur_new_pos_tar] is null
        or t1.cur_old_pos_tar is null
    )
  then 
    return null;
  else
    return
    (
      select 
        array_agg(array_new_x order by new_y_ord)
      from
      (
        select 
          v_cur_y as new_y_ord,
          array_agg(i_matrix[v_cur_y][coalesce(i_new_order[v_cur_new_x], v_cur_new_x)] order by v_cur_new_x) as array_new_x
        from generate_series(1, array_length(i_matrix, 1)) v_cur_y
          , generate_series(1, array_length(i_matrix, 2)) v_cur_new_x
        group by v_cur_y
      ) t_tmp_array_y
    )
    ;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_reorder_col(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]], array[null,3,4,2])
-- select sm_sc.fv_mx_reorder_col(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]], array[null,3,4,2,6,5])
-- select sm_sc.fv_mx_reorder_col(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]], array[null,4,null,2])
-- drop function sm_sc.fv_mx_reorder_row(float[][], int[]);
create or replace function sm_sc.fv_mx_reorder_row
(
  in i_matrix       float[][],
  in i_new_order    int[]     -- etc.: (array[null,2,4,5,3]) mean old_4->new_3; old_5->new_4; old_3->new_5
)
  returns float[][]
as
$$
-- declare here
begin
  if array_length(i_new_order, 1) > array_length(i_matrix, 1)
    or
    exists 
    (
      select 
      from (select cur_old_pos_tar from unnest(i_new_order) cur_old_pos_tar where cur_old_pos_tar is not null) t1
      full join (select cur_new_pos_tar from generate_series(1, array_length(i_new_order, 1)) cur_new_pos_tar where i_new_order[cur_new_pos_tar] is not null) t2
      on t1.cur_old_pos_tar = t2.cur_new_pos_tar
      where i_new_order[t2.cur_new_pos_tar] is null
        or t1.cur_old_pos_tar is null
    )
  then 
    return null;
  else
    return
    (
      select 
        array_agg(array_new_x order by new_y_ord)
      from
      (
        select 
          v_cur_new_y as new_y_ord,
          array_agg(i_matrix[coalesce(i_new_order[v_cur_new_y], v_cur_new_y)][v_cur_x] order by v_cur_x) as array_new_x
        from generate_series(1, array_length(i_matrix, 1)) v_cur_new_y
          , generate_series(1, array_length(i_matrix, 2)) v_cur_x
        group by v_cur_new_y
      ) t_tmp_array_new_y
    )
    ;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_reorder_row(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]], array[null,3,4,2])
-- select sm_sc.fv_mx_reorder_row(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]], array[null,3,4,2,6,5])
-- select sm_sc.fv_mx_reorder_row(array[[1,3,5,7],[3,4,6,8],[5,6,6,8],[7,8,8,8]], array[null,4,null,2])
-- drop function sm_sc.fv_mx_row_mix(float[][], int, int, int);
create or replace function sm_sc.fv_mx_row_mix
(
  in i_original_matrix   float[][]   ,
  in i_target_row_no     int                  ,
  in i_target_column_no  int                  ,
  in i_refer_row_no      int
)
  returns float[][]
as
$$
-- declare here
declare v_cur       int := 1;
declare v_times     float;

begin

  if i_original_matrix[i_target_row_no][i_target_column_no] = 0.0
  then 
    return i_original_matrix;

  elseif i_original_matrix[i_refer_row_no][i_target_column_no] = 0.0
  then
    return null::float[][];

  else
    v_times = i_original_matrix[i_target_row_no][i_target_column_no] / i_original_matrix[i_refer_row_no][i_target_column_no];
    while v_cur <= array_length(i_original_matrix, 2)
    loop
      i_original_matrix[i_target_row_no][v_cur] = i_original_matrix[i_target_row_no][v_cur] - i_original_matrix[i_refer_row_no][v_cur] * v_times;
      v_cur = v_cur + 1;
    end loop;
    i_original_matrix[i_target_row_no][i_target_column_no] = 0.0;
    return i_original_matrix;
  end if;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_row_mix(array[[1.9, 34.5, 0.55, 45.7, 400.5],[45.9, 4.6, 34.5, 0.55, 45.7],[3.2, 7.7, 1.9, 34.5, 0.55],[4.7, 9.0, 4.6, 34.5, 0.55]], 2, 3, 4)

-- drop function sm_sc.fv_mx_row_step(float[][]);
create or replace function sm_sc.fv_mx_row_step
(
  in i_matrix float[][]
)
  returns float[][]
as
$$
-- declare here
declare 
  v_cur_loop_1       int         :=   1;
  v_y_len            int         :=   array_length(i_matrix, 1);
  v_x_len            int         :=   array_length(i_matrix, 2);
  v_row_no           int;
  v_compare_val      float;
  v_cur_part_array   float[];
  v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);

begin
  -- -- -- pg的多维数组服务操纵切片，转化为一维数组表
  -- -- drop table if exists sm_sc.__vt_tmp_matrix;
  -- -- create temp table sm_sc.__vt_tmp_matrix
  -- -- (
  -- --   sess_id                char(32)        ,  
  -- --   -- org_row_no       int            ,    -- 原行号
  -- --   new_row_no       int            ,    -- 新行号
  -- --   array_x          float[]      -- ,    -- 行向量
  -- --   -- transform_desc   text           ,    -- 初等变换轨迹描述
  -- --   -- primary key(sess_id, org_row_no)
  -- -- ) with (parallel_workers = 4);
  -- -- 
  -- -- create index idx_tmp_matrix on sm_sc.__vt_tmp_matrix (sess_id, new_row_no);

  insert into sm_sc.__vt_tmp_matrix
  (
    sess_id,
    -- org_row_no  ,
    new_row_no  ,
    array_x
  )
  select 
    v_sess_id,
    -- v_cur_loop_1_y,
    v_cur_loop_1_y,
    array_agg(i_matrix[v_cur_loop_1_y][v_cur_loop_1_x] order by v_cur_loop_1_x)
  from generate_series(1, v_y_len) v_cur_loop_1_y
    , generate_series(1, v_x_len) v_cur_loop_1_x
  group by v_cur_loop_1_y
  ;

  -- 沿主对角线，逐行化简为上梯形
  while v_cur_loop_1 < v_y_len
  loop 
    -- 判断后续行，该列元是否都为0。存在非零则进入该行该列元判断；不存在非零则跳过该列；
    if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id)
    then 
      -- 如果当前行当前列元为零，则需换行
      if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and array_x[v_cur_loop_1] = 0.0 and sess_id = v_sess_id)
      then
        v_row_no = (select new_row_no from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id limit 1);

        update sm_sc.__vt_tmp_matrix
        set new_row_no = case new_row_no 
                           when v_row_no then v_cur_loop_1 
                           else v_row_no
                         end
        where new_row_no in (v_row_no, v_cur_loop_1)
          and sess_id = v_sess_id
		;

        -- -- 换行变号
        -- update sm_sc.__vt_tmp_matrix
        -- set array_x[v_cur_loop_1 : ] = -` array_x[v_cur_loop_1 : ]
        -- where new_row_no = v_cur_loop_1
        --   and sess_id = v_sess_id
        -- ;
      end if;

      -- 由方阵左上角，后续行逐行消元，成为0左下三角
      v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      update sm_sc.__vt_tmp_matrix
      set array_x[v_cur_loop_1 : ] 
            = array[0.0 :: float] ||
              (
                array_x[v_cur_loop_1 + 1 : ]
                -` 
                (
                  (array_x[v_cur_loop_1] / v_compare_val)
                  *` v_cur_part_array
                )
              )
      where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0
        and sess_id = v_sess_id
      ;

    end if;

    v_cur_loop_1 = v_cur_loop_1 + 1;
  end loop;

  return
    (select array_agg(array_x order by new_row_no) from sm_sc.__vt_tmp_matrix where sess_id = v_sess_id)
  ;

  delete from sm_sc.__vt_tmp_matrix where sess_id = v_sess_id;
end
$$
  language plpgsql volatile
  parallel unsafe
  cost 100;

-- select sm_sc.fv_mx_row_step(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_row_step(array[[1,2,3],[0,8,9],[4,5,6]])
-- drop function sm_sc.fv_mx_rows_step_simple(float[][]);
create or replace function sm_sc.fv_mx_rows_step_simple
(
  in i_matrix float[][]
)
  returns float[][]
as
$$
-- declare here
declare v_cur_loop_1       int         :=   1;
declare v_y_len            int         :=   array_length(i_matrix, 1);
declare v_x_len            int         :=   array_length(i_matrix, 2);
declare v_row_no           int;
declare v_compare_val      float;
declare v_cur_part_array   float[];
declare v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)     :=   replace(gen_random_uuid()::text, '-', '')::char(32);

begin
  -- -- pg的多维数组服务操纵切片，转化为一维数组表
  -- drop table if exists sm_sc.__vt_tmp_matrix;
  -- create temp table sm_sc.__vt_tmp_matrix
  -- (
  --   sess_id                char(32)        ,  
  --   -- org_row_no       int            ,    -- 原行号
  --   new_row_no       int            ,    -- 新行号
  --   array_x          float[]      -- ,    -- 行向量
  --   -- transform_desc   text           ,    -- 初等变换轨迹描述
  --   -- primary key(sess_id, org_row_no)
  -- ) with (parallel_workers = 4);
  -- 
  -- create index idx_tmp_matrix on sm_sc.__vt_tmp_matrix (sess_id, new_row_no);

  insert into sm_sc.__vt_tmp_matrix
  (
    sess_id,
    -- org_row_no  ,
    new_row_no  ,
    array_x
  )
  select 
    v_sess_id,
    -- v_cur_loop_1_y,
    v_cur_loop_1_y,
    array_agg(i_matrix[v_cur_loop_1_y][v_cur_loop_1_x] order by v_cur_loop_1_x)
  from generate_series(1, v_y_len) v_cur_loop_1_y
    , generate_series(1, v_x_len) v_cur_loop_1_x
  group by v_cur_loop_1_y
  ;

  -- 沿主对角线，逐行化简为上梯形
  while v_cur_loop_1 < v_y_len
  loop 
    -- 判断后续行，该列元是否都为0。存在非零则进入该行该列元判断；不存在非零则跳过该列；
    if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id)
    then 
      -- 如果当前行当前列元为零，则需换行
      if exists (select  from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and array_x[v_cur_loop_1] = 0.0 and sess_id = v_sess_id)
      then
        v_row_no = (select new_row_no from sm_sc.__vt_tmp_matrix where new_row_no > v_cur_loop_1 and array_x[v_cur_loop_1] <> 0.0 and sess_id = v_sess_id limit 1);

        update sm_sc.__vt_tmp_matrix
        set new_row_no = case new_row_no 
                           when v_row_no then v_cur_loop_1 
                           else v_row_no
                         end
        where new_row_no in (v_row_no, v_cur_loop_1)
          and sess_id = v_sess_id
		;

        -- -- 换行变号
        -- update sm_sc.__vt_tmp_matrix
        -- set array_x[v_cur_loop_1 : ] = -` array_x[v_cur_loop_1 : ]
        -- where new_row_no = v_cur_loop_1
        --   and sess_id = v_sess_id
        -- ;
      end if;

      -- 由方阵左上角，后续行逐行消元，成为0左下三角
      v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      update sm_sc.__vt_tmp_matrix
      set array_x[v_cur_loop_1 : ] 
            = array[0.0 :: float] ||
              (
                array_x[v_cur_loop_1 + 1 : ]
                -` 
                   (
                     (array_x[v_cur_loop_1] / v_compare_val)
                     *` v_cur_part_array
                   )
              )
      where new_row_no > v_cur_loop_1 
        and array_x[v_cur_loop_1] <> 0.0
        and sess_id = v_sess_id
      ;

    end if;

    v_cur_loop_1 = v_cur_loop_1 + 1;
  end loop;

  -- 由方阵右下角，下至上前续行逐行消元，成为0左下三角
  v_cur_loop_1 = least(v_y_len, v_x_len);
  while v_cur_loop_1 > 1
  loop
    v_compare_val = (select array_x[v_cur_loop_1] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
    if v_compare_val <> 0.0
    then
      v_cur_part_array = (select array_x[v_cur_loop_1 + 1 : ] from sm_sc.__vt_tmp_matrix where new_row_no = v_cur_loop_1 and sess_id = v_sess_id);
      update sm_sc.__vt_tmp_matrix
      set array_x[v_cur_loop_1 : ] 
            = array[0.0 :: float] ||
              (
                array_x[v_cur_loop_1 + 1 : ]
                -` 
                (
                  (array_x[v_cur_loop_1] / v_compare_val)
                  *` v_cur_part_array
                )
              )
      where new_row_no < v_cur_loop_1
        and array_x[v_cur_loop_1] <> 0.0
        and sess_id = v_sess_id
      ;
    end if;
    v_cur_loop_1 = v_cur_loop_1 - 1;
  end loop;

  -- 主对角线值归一，如有必要先判断是否满秩（比如求逆矩阵，需要判断是否满秩）
  update sm_sc.__vt_tmp_matrix
  set array_x[new_row_no : ] 
        = array[1.0 :: float] ||
          (
            (1.0 :: float/ array_x[new_row_no])
            *` array_x[new_row_no + 1 : ]
          )
  where new_row_no <= least(v_y_len, v_x_len)
    and array_x[new_row_no] <> 0.0
    and sess_id = v_sess_id
  ;

  return
  (
    select 
      array_agg
      (
        array_x 
        order by array_x[least(new_row_no, v_x_len)] desc, new_row_no  -- 全零行，下移，此处往下，主对角线可能出现拐弯，相比“两行位置对调”，行序变化较大
      ) 
    from sm_sc.__vt_tmp_matrix
    where sess_id = v_sess_id
  )
  ;

  delete from sm_sc.__vt_tmp_matrix where sess_id = v_sess_id;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_rows_step_simple(array[[1,2],[3,4]])
-- select sm_sc.fv_mx_rows_step_simple(array[[1,2,3],[0,8,9],[4,5,6]])
-- select sm_sc.fv_mx_rows_step_simple(array[[1,2,3,4],[0,8,9,10],[4,5,6,7]])
-- select sm_sc.fv_mx_rows_step_simple(array[[0,1,2,1,0,0],[1,1,4,0,1,0],[2,-1,0,0,0,1]])
-- drop function sm_sc.fv_mx_times_col(float[][], int, float);
create or replace function sm_sc.fv_mx_times_col
(
  in i_original_matrix   float[][]   ,
  in i_column_no            int                  ,
  in i_times             float
)
  returns float[][]
as
$$
-- declare here
declare v_cur int := 1;

begin
  while v_cur <= array_length(i_original_matrix, 1)
  loop
    i_original_matrix[v_cur][i_column_no] = i_original_matrix[v_cur][i_column_no] * i_times;
    v_cur = v_cur + 1;
  end loop;

  return i_original_matrix;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_times_col(array[[1.9, 34.5, 0.55, 45.7, 400.5],[45.9, 4.6, 34.5, 0.55, 45.7],[3.2, 7.7, 1.9, 34.5, 0.55],[4.7, 9.0, 4.6, 34.5, 0.55]], 2, 3.3)

-- drop function sm_sc.fv_mx_times_row(float[][], int, float);
create or replace function sm_sc.fv_mx_times_row
(
  in i_original_matrix   float[][]   ,
  in i_row_no            int                  ,
  in i_times             float
)
  returns float[][]
as
$$
-- declare here
declare v_cur int := 1;

begin
  while v_cur <= array_length(i_original_matrix, 2)
  loop
    i_original_matrix[i_row_no][v_cur] = i_original_matrix[i_row_no][v_cur] * i_times;
    v_cur = v_cur + 1;
  end loop;

  return i_original_matrix;
end
$$
  language plpgsql volatile
  cost 100;

-- select sm_sc.fv_mx_times_row(array[[1.9, 34.5, 0.55, 45.7, 400.5],[45.9, 4.6, 34.5, 0.55, 45.7],[3.2, 7.7, 1.9, 34.5, 0.55],[4.7, 9.0, 4.6, 34.5, 0.55]], 2, 3.3)

-- drop function if exists sm_sc.ufv_prod_mx_based_org_chunk(float[], float[], int, float[]);
create or replace function sm_sc.ufv_prod_mx_based_org_chunk
(
  i_left                            float[]
, i_right                           float[]
, i_l_w_r_h_chunk_pos               int
, i_chunk_above_left_above_left     float[]  default null
)
returns float[]
as
$$
declare
  v_h   int   :=  array_length(i_chunk_above_left_above_left, 1);
  v_w   int   :=  array_length(i_chunk_above_left_above_left, 2);
begin
  if i_chunk_above_left_above_left is null
  then
    return
      i_left |**| i_right
    ;
  else
    return 
      (
        (
          i_chunk_above_left_above_left 
          +` 
          (i_left[:v_h][i_l_w_r_h_chunk_pos+1:] |**| i_right[i_l_w_r_h_chunk_pos+1:][:v_w])
        )                                                -- a. * .l = al    -- (i_left[:v_h][:] |**| i_right[:][:v_w])
        ||||                                                                                
        (i_left[:v_h][:] |**| i_right[:][v_w+1:])        -- a. * .r = ar
      )
      |-||
      (
        (i_left[v_h+1:][:] |**| i_right[:][:v_w])        -- b. * .l = bl
        ||||
        (i_left[v_h+1:][:] |**| i_right[:][v_w+1:])      -- b. * .r = br
      )
    ;
  end if;
end
$$
language plpgsql stable
parallel safe
;
-- with 
-- cte_input as 
-- (
--   select 
--     sm_sc.fv_new_randn(0.0, 1.1, array[5,8]) as a_left
--   , sm_sc.fv_new_randn(0.0, 1.1, array[8,7]) as a_right
--   , 6 as a_l_w_r_h_chunk_pos
-- )
-- select 
--   (
--     sm_sc.ufv_prod_mx_based_org_chunk
--     (
--       a_left
--     , a_right
--     , a_l_w_r_h_chunk_pos
--     ) :: decimal[] ~=` 6
--   )
--   ==`
--   (
--     sm_sc.ufv_prod_mx_based_org_chunk
--     (
--       a_left
--     , a_right
--     , a_l_w_r_h_chunk_pos
--     , a_left[ : 4][ : a_l_w_r_h_chunk_pos] |**| a_right[ : a_l_w_r_h_chunk_pos][ : 5]
--     ) :: decimal[] ~=` 6
--   )
-- from cte_input
-- drop function if exists sm_sc.ufv_prod_mx_slice(float[], float[]);
create or replace function sm_sc.ufv_prod_mx_slice
(
  i_left                            float[]
, i_right                           float[]
)
returns float[]
as
$$
declare
  v_w_h   int   :=  least(array_length(i_left, array_ndims(i_left)), array_length(i_right, array_ndims(i_right) - 1));
begin
  if array_ndims(i_left) = 2
  then 
    if array_ndims(i_right) = 2
    then 
      return i_left[ : ][ : v_w_h] |**| i_right[ : v_w_h][ : ];
    elsif array_ndims(i_right) = 3
    then 
      return i_left[ : ][ : v_w_h] |**| i_right[ : ][ : v_w_h][ : ];
    elsif array_ndims(i_right) = 4
    then 
      return i_left[ : ][ : v_w_h] |**| i_right[ : ][ : ][ : v_w_h][ : ];
    end if;
  elsif array_ndims(i_left) = 3
  then 
    if array_ndims(i_right) = 2
    then 
      return i_left[ : ][ : ][ : v_w_h] |**| i_right[ : v_w_h][ : ];
    elsif array_ndims(i_right) = 3
    then 
      return i_left[ : ][ : ][ : v_w_h] |**| i_right[ : ][ : v_w_h][ : ];
    elsif array_ndims(i_right) = 4
    then 
      return i_left[ : ][ : ][ : v_w_h] |**| i_right[ : ][ : ][ : v_w_h][ : ];
    end if;
  elsif array_ndims(i_left) = 4
  then 
    if array_ndims(i_right) = 2
    then 
      return i_left[ : ][ : ][ : ][ : v_w_h] |**| i_right[ : v_w_h][ : ];
    elsif array_ndims(i_right) = 3
    then 
      return i_left[ : ][ : ][ : ][ : v_w_h] |**| i_right[ : ][ : v_w_h][ : ];
    elsif array_ndims(i_right) = 4
    then 
      return i_left[ : ][ : ][ : ][ : v_w_h] |**| i_right[ : ][ : ][ : v_w_h][ : ];
    end if;
  
  end if;
end
$$
language plpgsql stable
parallel safe
;

-- select 
--   (
--     sm_sc.ufv_prod_mx_slice
--     (
--       sm_sc.fv_new_randn(0.0, 1.1, array[5,3])
--     , sm_sc.fv_new_randn(0.0, 1.1, array[8,7])
--     ) :: decimal[] ~=` 6
--   )

-- select 
--   (
--     sm_sc.ufv_prod_mx_slice
--     (
--       sm_sc.fv_new_randn(0.0, 1.1, array[5,8])
--     , sm_sc.fv_new_randn(0.0, 1.1, array[4,7])
--     ) :: decimal[] ~=` 6
--   )

-- select 
--   (
--     sm_sc.ufv_prod_mx_slice
--     (
--       sm_sc.fv_new_randn(0.0, 1.1, array[3,5,8])
--     , sm_sc.fv_new_randn(0.0, 1.1, array[4,7])
--     ) :: decimal[] ~=` 6
--   )

-- select 
--   (
--     sm_sc.ufv_prod_mx_slice
--     (
--       sm_sc.fv_new_randn(0.0, 1.1, array[5,8])
--     , sm_sc.fv_new_randn(0.0, 1.1, array[5,4,7])
--     ) :: decimal[] ~=` 6
--   )

-- select 
--   (
--     sm_sc.ufv_prod_mx_slice
--     (
--       sm_sc.fv_new_randn(0.0, 1.1, array[3,5,8])
--     , sm_sc.fv_new_randn(0.0, 1.1, array[6,3,4,7])
--     ) :: decimal[] ~=` 6
--   )

-- select 
--   (
--     sm_sc.ufv_prod_mx_slice
--     (
--       sm_sc.fv_new_randn(0.0, 1.1, array[5,8])
--     , sm_sc.fv_new_randn(0.0, 1.1, array[6,5,4,7])
--     ) :: decimal[] ~=` 6
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1(float[], int[], int[]);
create or replace function sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
(
  i_dloss_ddepdt          float[]
, i_indepdt_len           int[]
, i_row_idx               int[]
)
returns float[]
as
$$
declare -- here
  v_indepdt_len_col       int    := i_indepdt_len[array_length(i_indepdt_len, 1) - 1];
  v_len_indepdt           int[]  
    :=  
      (select array_agg(array_length(i_dloss_ddepdt, a_dim) order by a_dim) from generate_series(1, array_ndims(i_dloss_ddepdt) - 2) tb_a_dim(a_dim))
      || 
      v_indepdt_len_col
      ||
      array_length(i_dloss_ddepdt, array_ndims(i_dloss_ddepdt))
  ;
  v_len_row_idx           int[]
    :=
      (select array_agg(array_length(i_row_idx, a_dim) order by a_dim) from generate_series(1, array_ndims(i_row_idx)) tb_a_dim(a_dim))
  ;
  v_dloss_dindepdt        float[];
  v_cur_y                 int    ;
  v_cur_x                 int    ;
  v_cur_x3                int    ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_length(v_len_indepdt, 1) not between 2 and 4 or array_ndims(i_row_idx) not between 2 and 4
    then
      raise exception 'no method for such ndims. ';
      
    elsif array_ndims(i_dloss_ddepdt) < array_ndims(i_row_idx)
    then 
      raise exception 'no method! i_col_arr ndims should >= i_row_idx. ';
      
    elsif array_length(i_row_idx, array_ndims(i_row_idx) - 1) <> 1
    then
      raise exception 'heigh of i_row_idx should be 1. ';
      
    elsif 
      (v_len_indepdt[array_length(v_len_indepdt, 1) - array_ndims(i_row_idx) + 1 : array_length(v_len_indepdt, 1) - 2] || v_indepdt_len_col || v_len_indepdt[array_length(v_len_indepdt, 1)])
      <>
      v_len_row_idx[ : array_ndims(i_row_idx)]
    then 
      raise exception 'unmatch array_length between indepdt and row_idx';
      
    elsif 
      sm_sc.fv_aggr_slice_and
      (
        i_row_idx <=` v_len_indepdt[array_length(v_len_indepdt, 1) - 1]
      )
    then 
      raise exception 'overflow length row_idx for indepdt''s heigh. ';
    end if;
  end if;

  -- 初始化自变量
  v_dloss_dindepdt := array_fill(0.0 :: float, v_len_indepdt);
  if array_length(v_len_row_idx, 1) = 2
  then 
    if array_length(v_len_indepdt, 1) = 2
    then
      for v_cur_x in 1 .. v_len_indepdt[2]
      loop 
        v_dloss_dindepdt[i_row_idx[1][v_cur_x]][v_cur_x] := i_dloss_ddepdt[1][v_cur_x];
      end loop;
    
    elsif array_length(v_len_indepdt, 1) = 3
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x3 in 1 .. v_len_indepdt[3]
        loop 
          v_dloss_dindepdt[v_cur_y][i_row_idx[1][v_cur_x3]][v_cur_x3] := i_dloss_ddepdt[v_cur_y][1][v_cur_x3];
        end loop;
      end loop;
      
    elsif array_length(v_len_indepdt, 1) = 4
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x in 1 .. v_len_indepdt[2]
        loop 
          for v_cur_x4 in 1 .. v_len_indepdt[4]
          loop 
            v_dloss_dindepdt[v_cur_y][v_cur_x][i_row_idx[1][v_cur_x4]][v_cur_x4] := i_dloss_ddepdt[v_cur_y][v_cur_x][1][v_cur_x4];
          end loop;
        end loop;
      end loop;
      
    end if;
  
  elsif array_length(v_len_row_idx, 1) = 3
  then 
    if array_length(v_len_indepdt, 1) = 3
    then 
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x3 in 1 .. v_len_indepdt[3]
        loop 
          v_dloss_dindepdt[v_cur_y][i_row_idx[v_cur_y][1][v_cur_x3]][v_cur_x3] := i_dloss_ddepdt[v_cur_y][1][v_cur_x3];
        end loop;
      end loop;
    
    elsif array_length(v_len_indepdt, 1) = 4
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x in 1 .. v_len_indepdt[2]
        loop 
          for v_cur_x4 in 1 .. v_len_indepdt[4]
          loop 
            v_dloss_dindepdt[v_cur_y][v_cur_x][i_row_idx[v_cur_x][1][v_cur_x4]][v_cur_x4] := i_dloss_ddepdt[v_cur_y][v_cur_x][1][v_cur_x4];
          end loop;
        end loop;
      end loop;
    end if;
    
  elsif array_length(v_len_row_idx, 1) = 4
  then 
    for v_cur_y in 1 .. v_len_indepdt[1]
    loop 
      for v_cur_x in 1 .. v_len_indepdt[2]
      loop 
        for v_cur_x4 in 1 .. v_len_indepdt[4]
        loop 
          v_dloss_dindepdt[v_cur_y][v_cur_x][i_row_idx[v_cur_y][v_cur_x][1][v_cur_x4]][v_cur_x4] := i_dloss_ddepdt[v_cur_y][v_cur_x][1][v_cur_x4];
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_dloss_dindepdt;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[1, 5]) 
--   , array[3, 5]
--   , ((sm_sc.fv_new_rand(array[1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 1, 5]) 
--   , array[2, 3, 5]
--   , ((sm_sc.fv_new_rand(array[1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 2, 1, 5]) 
--   , array[2, 2, 3, 5]
--   , ((sm_sc.fv_new_rand(array[1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 1, 5]) 
--   , array[2, 3, 5]
--   , ((sm_sc.fv_new_rand(array[2, 1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , array[2, 3, 3, 5]
--   , ((sm_sc.fv_new_rand(array[   3, 1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1, 5]) 
--   , array[2, 3, 3, 5]
--   , ((sm_sc.fv_new_rand(array[2, 3, 1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ufv_query_by_width_row_idx_from_col(float[], int[]);
create or replace function sm_sc.ufv_query_by_width_row_idx_from_col
(
  i_col_arr    float[]
, i_row_idx    int[]
)
returns float[]
as
$$
declare -- here
  v_len_col_arr      int[]         := (select array_agg(array_length(i_col_arr, a_dim) order by a_dim) from generate_series(1, array_ndims(i_col_arr)) tb_a_dim(a_dim));
  v_len_row_idx      int[]         := (select array_agg(array_length(i_row_idx, a_dim) order by a_dim) from generate_series(1, array_ndims(i_row_idx)) tb_a_dim(a_dim));
  v_ret              i_col_arr%type;
  v_cur_y            int           ;
  v_cur_x            int           ;
  v_cur_x3           int           ;
  -- v_cur_x4           int           ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_col_arr) not between 2 and 4 or array_ndims(i_row_idx) not between 2 and 4
    then
      raise exception 'no method for such ndims. ';
      
    elsif array_ndims(i_col_arr) < array_ndims(i_row_idx)
    then 
      raise exception 'no method! i_col_arr ndims should >= i_row_idx. ';
      
    elsif v_len_row_idx[array_ndims(i_row_idx) - 1] <> 1
    then 
      raise exception 'width of i_row_idx should be 1. ';
      
    elsif 
      (
        v_len_col_arr[array_ndims(i_col_arr) - array_ndims(i_row_idx) + 1 : array_ndims(i_col_arr) - 2]
        ||
        v_len_col_arr[array_ndims(i_col_arr)]
      )
      <>
      (
        v_len_row_idx[ : array_ndims(i_row_idx) - 2]
        ||
        v_len_row_idx[array_ndims(i_row_idx)]
      )
    then 
      raise exception 'unmatch array_length between col_arr and row_idx';
      
    elsif 
      sm_sc.fv_aggr_slice_and
      (
        i_row_idx <=` v_len_col_arr[array_ndims(i_col_arr) - 1]
      )
    then 
      raise exception 'overflow length row_idx for col_arr''s heigh. ';
      
    end if;
  end if;

  if array_length(v_len_row_idx, 1) = 2
  then 
    if array_length(v_len_col_arr, 1) = 2
    then 
      v_ret := array_fill(0.0 :: float, v_len_row_idx);
      for v_cur_x in 1 .. v_len_col_arr[2]
      loop 
        v_ret[1][v_cur_x] := i_col_arr[i_row_idx[1][v_cur_x]][v_cur_x];
      end loop;
      -- -- return 
      -- -- (
      -- --   select 
      -- --     sm_sc.fa_mx_concat_y
      -- --     (
      -- --       i_col_arr[a_row_idx][i_row_idx[a_row_idx]]
      -- --       order by a_row_idx
      -- --     )
      -- --   from generate_series(1, v_len_col_arr[1]) tb_a_row_idx(a_row_idx)
      -- -- )
      -- -- ;
      
    elsif array_length(v_len_col_arr, 1) = 3
    then
      v_ret := array_fill(0.0 :: float, v_len_col_arr[1] || v_len_row_idx);
      for v_cur_y in 1 .. v_len_col_arr[1]
      loop 
        for v_cur_x3 in 1 .. v_len_col_arr[3]
        loop 
          v_ret[v_cur_y][1][v_cur_x3] := i_col_arr[v_cur_y][i_row_idx[1][v_cur_x3]][v_cur_x3];
        end loop;
      end loop;
    
    elsif array_length(v_len_col_arr, 1) = 4
    then
      v_ret := array_fill(0.0 :: float, v_len_col_arr[1 : 2] || v_len_row_idx);
      for v_cur_y in 1 .. v_len_col_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_col_arr[2]
        loop 
          for v_cur_x4 in 1 .. v_len_col_arr[4]
          loop 
            v_ret[v_cur_y][v_cur_x][1][v_cur_x4] := i_col_arr[v_cur_y][v_cur_x][i_row_idx[1][v_cur_x4]][v_cur_x4];
          end loop;
        end loop;
      end loop;
    end if;
  
  elsif array_length(v_len_row_idx, 1) = 3
  then 
    if array_length(v_len_col_arr, 1) = 3
    then
      v_ret := array_fill(0.0 :: float, v_len_row_idx);
      for v_cur_y in 1 .. v_len_col_arr[1]
      loop 
        for v_cur_x3 in 1 .. v_len_col_arr[3]
        loop 
          v_ret[v_cur_y][1][v_cur_x3] := i_col_arr[v_cur_y][i_row_idx[v_cur_y][1][v_cur_x3]][v_cur_x3];
        end loop;
      end loop;
    
    elsif array_length(v_len_col_arr, 1) = 4
    then
      v_ret := array_fill(0.0 :: float, v_len_col_arr[1] || v_len_row_idx);
      for v_cur_y in 1 .. v_len_col_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_col_arr[2]
        loop 
          for v_cur_x4 in 1 .. v_len_col_arr[4]
          loop 
            v_ret[v_cur_y][v_cur_x][1][v_cur_x4] := i_col_arr[v_cur_y][v_cur_x][i_row_idx[v_cur_x][1][v_cur_x4]][v_cur_x4];
          end loop;
        end loop;
      end loop;
    end if; 
    
  elsif array_length(v_len_row_idx, 1) = 4
  then 
    if array_length(v_len_col_arr, 1) = 4
    then
      v_ret := array_fill(0.0 :: float, v_len_row_idx);
      for v_cur_y in 1 .. v_len_col_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_col_arr[2]
        loop 
          for v_cur_x4 in 1 .. v_len_col_arr[4]
          loop 
            v_ret[v_cur_y][v_cur_x][1][v_cur_x4] := i_col_arr[v_cur_y][v_cur_x][i_row_idx[v_cur_y][v_cur_x][1][v_cur_x4]][v_cur_x4];
          end loop;
        end loop;
      end loop;
    end if; 
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.ufv_query_by_width_row_idx_from_col
--   ( 
--     sm_sc.fv_new_rand(array[3, 5]) 
--   , ((sm_sc.fv_new_rand(array[1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_query_by_width_row_idx_from_col
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_query_by_width_row_idx_from_col
--   ( 
--     sm_sc.fv_new_rand(array[2, 2, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_query_by_width_row_idx_from_col
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[2, 1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_query_by_width_row_idx_from_col
--   ( 
--     sm_sc.fv_new_rand(array[2, 2, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[   2, 1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_query_by_width_row_idx_from_col
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[2, 3, 1, 5]) *` 3.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1(float[], int[], int[]);
create or replace function sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
(
  i_dloss_ddepdt          float[]
, i_indepdt_len           int[]
, i_col_idx               int[]
)
returns float[]
as
$$
declare -- here
  v_len_indepdt           int[]  
    :=  
      (select array_agg(array_length(i_dloss_ddepdt, a_dim) order by a_dim) from generate_series(1, array_ndims(i_dloss_ddepdt) - 1) tb_a_dim(a_dim))
      || 
      i_indepdt_len[array_length(i_indepdt_len, 1)]
  ;
  v_len_col_idx           int[]
    :=
      (select array_agg(array_length(i_col_idx, a_dim) order by a_dim) from generate_series(1, array_ndims(i_col_idx)) tb_a_dim(a_dim))
  ;
  v_dloss_dindepdt        float[];
  v_cur_y                 int    ;
  v_cur_x                 int    ;
  v_cur_x3                int    ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_length(v_len_indepdt, 1) not between 2 and 4 or array_ndims(i_col_idx) not between 2 and 4
    then
      raise exception 'no method for such ndims. ';
      
    elsif array_ndims(i_dloss_ddepdt) < array_ndims(i_col_idx)
    then 
      raise exception 'no method! i_col_arr ndims should >= i_row_idx. ';
      
    elsif array_length(i_col_idx, array_ndims(i_col_idx)) <> 1
    then
      raise exception 'width of i_col_idx should be 1. ';
      
    elsif 
      v_len_indepdt[array_length(v_len_indepdt, 1) - array_ndims(i_col_idx) + 1 : array_length(v_len_indepdt, 1) - 1]
      <>
      v_len_col_idx[ : array_ndims(i_col_idx) - 1]
    then 
      raise exception 'unmatch array_length between indepdt and col_idx';
      
    elsif 
      sm_sc.fv_aggr_slice_and
      (
        i_col_idx <=` v_len_indepdt[array_length(v_len_indepdt, 1)]
      )
    then 
      raise exception 'overflow length col_idx for indepdt''s width. ';
    end if;
  end if;

  -- 初始化自变量
  v_dloss_dindepdt := array_fill(0.0 :: float, v_len_indepdt);
  if array_length(v_len_col_idx, 1) = 2
  then 
    if array_length(v_len_indepdt, 1) = 2
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        v_dloss_dindepdt[v_cur_y][i_col_idx[v_cur_y][1]] := i_dloss_ddepdt[v_cur_y][1];
      end loop;
    
    elsif array_length(v_len_indepdt, 1) = 3
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x in 1 .. v_len_indepdt[2]
        loop 
          v_dloss_dindepdt[v_cur_y][v_cur_x][i_col_idx[v_cur_x][1]] := i_dloss_ddepdt[v_cur_y][v_cur_x][1];
        end loop;
      end loop;
      
    elsif array_length(v_len_indepdt, 1) = 4
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x in 1 .. v_len_indepdt[2]
        loop 
          for v_cur_x3 in 1 .. v_len_indepdt[3]
          loop 
            v_dloss_dindepdt[v_cur_y][v_cur_x][v_cur_x3][i_col_idx[v_cur_x3][1]] := i_dloss_ddepdt[v_cur_y][v_cur_x][v_cur_x3][1];
          end loop;
        end loop;
      end loop;
      
    end if;
  
  elsif array_length(v_len_col_idx, 1) = 3
  then 
    if array_length(v_len_indepdt, 1) = 3
    then 
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x in 1 .. v_len_indepdt[2]
        loop 
          v_dloss_dindepdt[v_cur_y][v_cur_x][i_col_idx[v_cur_y][v_cur_x][1]] := i_dloss_ddepdt[v_cur_y][v_cur_x][1];
        end loop;
      end loop;
    
    elsif array_length(v_len_indepdt, 1) = 4
    then
      for v_cur_y in 1 .. v_len_indepdt[1]
      loop 
        for v_cur_x in 1 .. v_len_indepdt[2]
        loop 
          for v_cur_x3 in 1 .. v_len_indepdt[3]
          loop 
            v_dloss_dindepdt[v_cur_y][v_cur_x][v_cur_x3][i_col_idx[v_cur_x][v_cur_x3][1]] := i_dloss_ddepdt[v_cur_y][v_cur_x][v_cur_x3][1];
          end loop;
        end loop;
      end loop;
    end if;
    
  elsif array_length(v_len_col_idx, 1) = 4
  then 
    for v_cur_y in 1 .. v_len_indepdt[1]
    loop 
      for v_cur_x in 1 .. v_len_indepdt[2]
      loop 
        for v_cur_x3 in 1 .. v_len_indepdt[3]
        loop 
          v_dloss_dindepdt[v_cur_y][v_cur_x][v_cur_x3][i_col_idx[v_cur_y][v_cur_x][v_cur_x3][1]] := i_dloss_ddepdt[v_cur_y][v_cur_x][v_cur_x3][1];
        end loop;
      end loop;
    end loop;
    
  end if;
  
  return v_dloss_dindepdt;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[3, 1]) 
--   , array[3, 5]
--   , ((sm_sc.fv_new_rand(array[3, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 1]) 
--   , array[2, 3, 5]
--   , ((sm_sc.fv_new_rand(array[3, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 2, 3, 1]) 
--   , array[2, 2, 3, 5]
--   , ((sm_sc.fv_new_rand(array[3, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[3, 2, 1]) 
--   , array[3, 2, 5]
--   , ((sm_sc.fv_new_rand(array[3, 2, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 2, 1]) 
--   , array[2, 3, 2, 5]
--   , ((sm_sc.fv_new_rand(array[   3, 2, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 2, 1]) 
--   , array[2, 3, 2, 5]
--   , ((sm_sc.fv_new_rand(array[2, 3, 2, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- set search_path to sm_sc;
-- drop function if exists sm_sc.ufv_query_by_heigh_col_idx_from_row(float[], int[]);
create or replace function sm_sc.ufv_query_by_heigh_col_idx_from_row
(
  i_row_arr    float[]
, i_col_idx    int[]
)
returns float[]
as
$$
declare -- here
  v_len_row_arr      int[]         := (select array_agg(array_length(i_row_arr, a_dim) order by a_dim) from generate_series(1, array_ndims(i_row_arr)) tb_a_dim(a_dim));
  v_len_col_idx      int[]         := (select array_agg(array_length(i_col_idx, a_dim) order by a_dim) from generate_series(1, array_ndims(i_col_idx)) tb_a_dim(a_dim));
  v_ret              i_row_arr%type;
  v_cur_y            int           ;
  v_cur_x            int           ;
  v_cur_x3           int           ;
  -- v_cur_x4           int           ;
begin
  -- set search_path to sm_sc;
  -- 审计维度、长度
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then
    if array_ndims(i_row_arr) not between 2 and 4 or array_ndims(i_col_idx) not between 2 and 4
    then
      raise exception 'no method for such ndims. ';
      
    elsif array_ndims(i_row_arr) < array_ndims(i_col_idx)
    then 
      raise exception 'no method! i_row_arr ndims should >= i_col_idx. ';
      
    elsif v_len_col_idx[array_ndims(i_col_idx)] <> 1
    then 
      raise exception 'width of i_col_idx should be 1. ';
      
    elsif 
      v_len_row_arr[array_ndims(i_row_arr) - array_ndims(i_col_idx) + 1 : array_ndims(i_row_arr) - 1]
      <>
      v_len_col_idx[ : array_ndims(i_col_idx) - 1]
    then 
      raise exception 'unmatch array_length between row_arr and col_idx';
      
    elsif 
      sm_sc.fv_aggr_slice_and
      (
        i_col_idx <=` v_len_row_arr[array_ndims(i_row_arr)]
      )
    then 
      raise exception 'overflow length col_idx for row_arr''s width. ';
      
    end if;
  end if;

  if array_length(v_len_col_idx, 1) = 2
  then 
    if array_length(v_len_row_arr, 1) = 2
    then 
      v_ret := array_fill(0.0 :: float, v_len_col_idx);
      for v_cur_y in 1 .. v_len_row_arr[1]
      loop 
        v_ret[v_cur_y][1] := i_row_arr[v_cur_y][i_col_idx[v_cur_y][1]];
      end loop;
      -- -- return 
      -- -- (
      -- --   select 
      -- --     sm_sc.fa_mx_concat_y
      -- --     (
      -- --       i_row_arr[a_row_idx][i_col_idx[a_row_idx]]
      -- --       order by a_row_idx
      -- --     )
      -- --   from generate_series(1, v_len_row_arr[1]) tb_a_row_idx(a_row_idx)
      -- -- )
      -- -- ;
      
    elsif array_length(v_len_row_arr, 1) = 3
    then
      v_ret := array_fill(0.0 :: float, v_len_row_arr[1] || v_len_col_idx);
      for v_cur_y in 1 .. v_len_row_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_row_arr[2]
        loop 
          v_ret[v_cur_y][v_cur_x][1] := i_row_arr[v_cur_y][v_cur_x][i_col_idx[v_cur_x][1]];
        end loop;
      end loop;
    
    elsif array_length(v_len_row_arr, 1) = 4
    then
      v_ret := array_fill(0.0 :: float, v_len_row_arr[1 : 2] || v_len_col_idx);
      for v_cur_y in 1 .. v_len_row_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_row_arr[2]
        loop 
          for v_cur_x3 in 1 .. v_len_row_arr[3]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][1] := i_row_arr[v_cur_y][v_cur_x][v_cur_x3][i_col_idx[v_cur_x3][1]];
          end loop;
        end loop;
      end loop;
    end if;
  
  elsif array_length(v_len_col_idx, 1) = 3
  then 
    if array_length(v_len_row_arr, 1) = 3
    then
      v_ret := array_fill(0.0 :: float, v_len_col_idx);
      for v_cur_y in 1 .. v_len_row_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_row_arr[2]
        loop 
          v_ret[v_cur_y][v_cur_x][1] := i_row_arr[v_cur_y][v_cur_x][i_col_idx[v_cur_y][v_cur_x][1]];
        end loop;
      end loop;
    
    elsif array_length(v_len_row_arr, 1) = 4
    then
      v_ret := array_fill(0.0 :: float, v_len_row_arr[1] || v_len_col_idx);
      for v_cur_y in 1 .. v_len_row_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_row_arr[2]
        loop 
          for v_cur_x3 in 1 .. v_len_row_arr[3]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][1] := i_row_arr[v_cur_y][v_cur_x][v_cur_x3][i_col_idx[v_cur_x][v_cur_x3][1]];
          end loop;
        end loop;
      end loop;
    end if; 
    
  elsif array_length(v_len_col_idx, 1) = 4
  then 
    if array_length(v_len_row_arr, 1) = 4
    then
      v_ret := array_fill(0.0 :: float, v_len_col_idx);
      for v_cur_y in 1 .. v_len_row_arr[1]
      loop 
        for v_cur_x in 1 .. v_len_row_arr[2]
        loop 
          for v_cur_x3 in 1 .. v_len_row_arr[3]
          loop 
            v_ret[v_cur_y][v_cur_x][v_cur_x3][1] := i_row_arr[v_cur_y][v_cur_x][v_cur_x3][i_col_idx[v_cur_y][v_cur_x][v_cur_x3][1]];
          end loop;
        end loop;
      end loop;
    end if; 
    
  end if;
  
  return v_ret;
end
$$
language plpgsql stable
parallel safe
cost 100;

-- select 
--   sm_sc.ufv_query_by_heigh_col_idx_from_row
--   ( 
--     sm_sc.fv_new_rand(array[3, 5]) 
--   , ((sm_sc.fv_new_rand(array[3, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_query_by_heigh_col_idx_from_row
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[3, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_query_by_heigh_col_idx_from_row
--   ( 
--     sm_sc.fv_new_rand(array[2, 2, 3, 5]) 
--   , ((sm_sc.fv_new_rand(array[3, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_query_by_heigh_col_idx_from_row
--   ( 
--     sm_sc.fv_new_rand(array[3, 2, 5]) 
--   , ((sm_sc.fv_new_rand(array[3, 2, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- select 
--   sm_sc.ufv_query_by_heigh_col_idx_from_row
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 2, 5]) 
--   , ((sm_sc.fv_new_rand(array[   3, 2, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )

-- select 
--   sm_sc.ufv_query_by_heigh_col_idx_from_row
--   ( 
--     sm_sc.fv_new_rand(array[2, 3, 2, 5]) 
--   , ((sm_sc.fv_new_rand(array[2, 3, 2, 1]) *` 5.0 :: float) :: decimal[] +` 0.5 ~=` 0) :: int[]
--   )
-- drop function if exists sm_sc.ufv_lambda_arr(bigint, varchar(64), float[], float[][], float[], float[][]);
create or replace function sm_sc.ufv_lambda_arr
(
  i_node_no               bigint,
  i_lambda                varchar(64)                      ,
  i_param_1               float[]                 ,
  i_param_2               float[]   default null  ,      -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立
  i_input_arr_asso        float[]   default null  ,   
  i_param_3               float[]   default null
)
returns float[][]
as
$$
declare 
  -- -- v_lambda varchar(64)    :=    coalesce((select 'v...' from sm_sc.tb_dic... where col...'k...' = i_lambda limit 1), i_lambda)
 _v_debug         float[][];
begin
  -- 动态 sql 会影响性能，所以还是用 case 判断
  
-- -- debug 
-- raise notice 'lambda % begin:: i_lambda: %; len_input: %; len_co_val: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(i_param_1, 1), array_length(i_param_1, 2)], array[array_length(i_param_2, 1), array_length(i_param_2, 2)], to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if i_param_1 is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(i_param_1) is true
    then 
      raise exception 'there is null value in i_param_1!  i_node_no: %.  ', i_node_no;
    elsif 'NaN' :: float = any(i_param_1)
      or 'NaN' :: float = any(i_param_2)
      or i_param_1 is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(i_param_1) is true
    then 
      raise exception 'there is NaN value in i_param_1 or i_param_2!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  if i_lambda = '81_query_from_col'
  then 
    return 
      sm_sc.ufv_query_by_width_row_idx_from_col
      (
        i_param_1        
      , i_param_2 :: int[]
      )
    ;
  elsif i_lambda = '81_query_from_row'
  then 
    return 
      sm_sc.ufv_query_by_heigh_col_idx_from_row
      (
        i_param_1        
      , i_param_2 :: int[]
      )
    ;
  elsif i_lambda = '81_prod_mx_slice_no_backward'
  then 
    return 
      sm_sc.ufv_prod_mx_slice
      (
        i_param_1        
      , i_param_2
      )
    ;
  elsif i_lambda = '81_prod_mx_chunk'
  then 
    return 
      sm_sc.ufv_prod_mx_based_org_chunk
      (
        i_param_1        
      , i_param_2        
      , i_input_arr_asso[1]
      , i_param_3
      )
    ;
    
  else 
    raise exception 'no defination for this lambda %', i_lambda;
  end if;
  
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if _v_debug is null
      -- or sm_sc.fv_aggr_slice_is_exists_null(_v_debug) is true
    then 
      raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
    elsif 'NaN' :: float = any(_v_debug)
    then 
      raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  return _v_debug;

-- -- debug 
-- raise notice 'lambda % end:: i_lambda: %; time: %;', i_node_no, i_lambda, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return _v_debug;

  exception when others then
    raise exception 
    ' fn: sm_sc.ufv_lambda_arr
      i_node_no: %
      i_lambda: %
      len of i_param_1: %
      len of i_param_2: %
      i_input_arr_asso: %
      sqlerrm: %
    '
    , i_node_no
    , i_lambda
    , array_dims(i_param_1)
    , array_dims(i_param_2)
    , i_input_arr_asso
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;

-- select 
--   sm_sc.ufv_lambda_arr
--   (
--     100001
--     , '81_prod_mx_slice_no_backward'
--     , array[[1.23, 3.34],[-7.2, -0.25]]
--     , array[[1.23, 3.34],[-7.2, -0.25],[-7.2, -0.25]]
--   );

-- 本函数以数组为入参，标量版本参考 sm_sc.fv_gradient_opr
-- 对于一些原函数,  本求导函数无法求取 ddepdt/dindepdt，参看 sm_sc.fv_lambda_arr_dloss_dindepdt 求取 dloss/dindepdt
-- 本函数求取因变量对自变量的导数，没有对自变量广播求逆。
-- drop function if exists sm_sc.ufv_lambda_arr_ddepdt_dindepdt(bigint, varchar(64), float[], int, float[], float[], float[]);
create or replace function sm_sc.ufv_lambda_arr_ddepdt_dindepdt
(
  i_node_no               bigint,
  i_lambda                varchar(64)              ,
  i_indepdt_var           float[]                  ,
  i_indepdt_var_loc       int       default 1      ,    -- 单目、双目为 x 参数实际位置，无目/常量y为0，并目/聚合为 n；该参数对 prod_mx, sub, div, pow, log 等运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
  i_co_value              float[]   default null   ,    -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
  i_input_arr_asso        float[]   default null   ,    
  i_depdt_var             float[]   default null   ,    -- 前向的因变量，非必要入参
  -- i_dloss_ddepdt          float[] default null   ,       -- 此入参传入 dloss/ddepdt, 用于 反向传播阶段 求取 dloss/dindepdt
  i_indepdt_var_len       int[2]    default null        -- 自变量的高宽规格，agg_concat_x, agg_concat_y, agg_sum, agg_avg, slice_x, slice_y, add, sub conv_add/conv_sub/conv_de_add/conv_de_sub 等算子可用到该参数优化入参，以避免传参 i_indepdt_var，从而降低堆区开销
)
returns float[][]
as
$$
declare 
  _v_debug           float[]    ;
  -- v_indepdt_var_len  int[]      := (select array_agg(array_length(i_indepdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(i_indepdt_var)) tb_a_ndim(a_ndim));
  -- v_depdt_var_len    int[]      := (select array_agg(array_length(i_depdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(i_depdt_var)) tb_a_ndim(a_ndim));
begin
-- -- debug
-- raise notice '
-- sm_sc.ufv_lambda_arr_ddepdt_dindepdt debug :
--   i_lambda                : %; 
--   i_indepdt_var           : %; 
--   i_indepdt_var_loc       : %; 
--   i_co_value              : %; 
--   i_input_arr_asso   : %; 
--   i_depdt_var           : %; 
-- ', $1, $2, $3, $4, $5, $6
-- ;

-- -- debug 
-- raise notice 'lambda_delta % begin:: i_lambda: %; len_input: %; len_co_val: %; indepdt_var_loc: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(i_indepdt_var, 1), array_length(i_indepdt_var, 2)], array[array_length(i_co_value, 1), array_length(i_co_value, 2)], i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 'NaN' :: float = any(i_indepdt_var)
      or 'NaN' :: float = any(i_co_value)
      or 'NaN' :: float = any(i_depdt_var)
    then 
      raise exception 'there is NaN value in i_indepdt_var, i_co_value or i_depdt_var!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
    
  -- -- -- if i_lambda = '81_...'
  -- -- -- then 
  -- -- --   return 
  -- -- --     sm_sc.ufv_...
  -- -- --     (
  -- -- --       ...
  -- -- --     , ...
  -- -- --     , ...
  -- -- --     , ...
  -- -- --     , ...
  -- -- --     , ...
  -- -- --     , ...
  -- -- --     , ...
  -- -- --     )
  -- -- --   ;
  -- -- -- else 
  -- -- --   raise exception 'no defination for this lambda %', i_lambda;
  -- -- -- end if;  

  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then 
  -- --   if _v_debug is null
  -- --     -- or sm_sc.fv_aggr_slice_is_exists_null(_v_debug) is true
  -- --   then 
  -- --     raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
  -- --   elsif 'NaN' :: float = any(_v_debug)
  -- --   then 
  -- --     raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
  -- --   end if;
  -- -- end if;
  -- -- 
  -- -- return _v_debug;

-- -- debug 
-- raise notice 'lambda_delta % end:: i_lambda: %; indepdt_var_loc: %; time: %;', i_node_no, i_lambda, i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return _v_debug;

exception when others then
  raise exception 
  ' fn: sm_sc.ufv_lambda_arr_ddepdt_dindepdt
    i_node_no: %;
    i_lambda: %;
    len of i_indepdt_var: %;
    i_indepdt_var_loc: %;
    len of i_co_value: %;
    i_input_arr_asso: %;
    len of i_depdt_var: %;
    v_indepdt_var_len: %;
    sqlerrm: %;
  '
  , i_node_no
  , i_lambda             
  , array_dims(i_indepdt_var)          
  , i_indepdt_var_loc      
  , array_dims(i_co_value)
  , i_input_arr_asso
  , array_dims(i_depdt_var)
  , v_indepdt_var_len      
  , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;







-- 本函数以数组为入参，标量版本参考 sm_sc.fv_gradient_opr
-- 对于 conv_2d, pool_max, pool_avg, softmax, zscore 本函数求取 dloss/dindepdt, 而不是求取 ddepdt/dindepdt
-- drop function if exists sm_sc.ufv_lambda_arr_dloss_dindepdt(bigint, varchar(64), float[], int, float[], float[], float[], int, float[]);
create or replace function sm_sc.ufv_lambda_arr_dloss_dindepdt
(
  i_node_no               bigint,
  i_lambda                varchar(64)                       ,
  i_indepdt_var           float[]                  ,
  i_indepdt_var_loc       int     default 1        ,    -- 单目、双目为 x 参数实际位置，并目(无目/常量y)为0；该参数对 prod_mx, sub, div, pow, log, softmax 等运算操作敏感；其他运算操作用不到 i_indepdt_var_loc 
  i_co_value              float[] default null     ,    -- fn 配套的另一个入参值，该配套入参位置与 i_indepdt_var_loc 对立
  i_input_arr_asso        float[] default null     ,    
  i_depdt_var             float[] default null     ,    -- 前向的因变量
  i_dloss_ddepdt          float[] default null     ,    -- 此入参传入 dloss/ddepdt, 用于 反向传播阶段 求取 dloss/dindepdt
  i_indepdt_var_len       int[]   default null     ,    -- 自变量的高宽规格，agg_concat_x, agg_concat_y, agg_sum, agg_avg, slice_x, slice_y, add, sub 会用到改参数，以避免传参 i_indepdt_var，从而降低堆区开销
  i_ddepdt_dindepdt       float[] default null          -- 因变量对自变量导数
)
returns float[][]
as
$$
declare 
  -- -- _v_debug           float[][];
  v_indepdt_var_len  int[]      := coalesce(i_indepdt_var_len, (select array_agg(array_length(i_indepdt_var, a_ndim) order by a_ndim) from generate_series(1, array_ndims(i_indepdt_var)) tb_a_ndim(a_ndim)));
begin
-- -- debug
-- raise notice '
-- sm_sc.ufv_lambda_arr_dloss_dindepdt debug :
--   i_lambda                : %; 
--   i_indepdt_var             : %; 
--   i_indepdt_var_loc         : %; 
--   i_co_value              : %; 
--   i_input_arr_asso   : %; 
--   i_depdt_var           : %; 
-- ', $1, $2, $3, $4, $5, $6
-- ;

  -- 动态 sql 会影响性能，所以还是用 case 判断
  
-- -- debug 
-- raise notice 'lambda_dloss_dindepdt % begin:: i_lambda: %; len_dloss_dindepdt: %; indepdt_var_loc: %; time: %;'
-- , i_node_no, i_lambda, array[array_length(i_dloss_ddepdt, 1), array_length(i_dloss_ddepdt, 2)], i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');

  if current_setting('pg4ml._v_is_debug_check', true) = '1'
  then 
    if 'NaN' :: float = any(i_indepdt_var)
      or 'NaN' :: float = any(i_co_value)
      or 'NaN' :: float = any(i_depdt_var)
      or 'NaN' :: float = any(i_dloss_ddepdt)
    then 
      raise exception 'there is NaN value in i_indepdt_var, i_co_value, i_depdt_var or i_dloss_ddepdt!  i_node_no: %.  ', i_node_no;
    end if;
  end if;
  
  if i_lambda = '81_query_from_col'
  then 
    return 
      sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1
      (
        i_dloss_ddepdt        
      , i_indepdt_var_len         
      , i_co_value :: int[]
      )
    ;
  elsif i_lambda = '81_query_from_row'
  then 
    return 
      sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1
      (
        i_dloss_ddepdt        
      , i_indepdt_var_len         
      , i_co_value :: int[]
      )
    ;
    
  else 
    raise exception 'no defination for this lambda %', i_lambda;
  end if;  

  -- -- if current_setting('pg4ml._v_is_debug_check', true) = '1'
  -- -- then 
  -- --   if _v_debug is null
  -- --     -- or sm_sc.fv_aggr_slice_is_exists_null(_v_debug) is true
  -- --   then 
  -- --     raise exception 'there is null value in returning!  i_node_no: %.  ', i_node_no;
  -- --   elsif 'NaN' :: float = any(_v_debug)
  -- --   then 
  -- --     raise exception 'there is NaN value in returning!  i_node_no: %.  ', i_node_no;
  -- --   end if;
  -- -- end if;
  -- -- 
  -- -- return _v_debug;
  
-- -- debug 
-- raise notice 'lambda_dloss_dindepdt % end:: i_lambda: %; indepdt_var_loc: %; time:%;', i_node_no, i_lambda, i_indepdt_var_loc, to_char(clock_timestamp(), 'YYYYMMDD HH24:MI:SS.MS');
--   return _v_debug;

  exception when others then
    raise exception 
    ' i_node_no: %
      fn: sm_sc.ufv_lambda_arr_dloss_dindepdt
      i_lambda: %
      len of i_indepdt_var: %
      i_indepdt_var_loc: %
      len of i_co_value: %
      i_input_arr_asso: %
      len of i_depdt_var: %
      len of i_dloss_ddepdt: %
      sqlerrm: %
    '
    , i_node_no
    , i_lambda             
    , array_dims(i_indepdt_var)
    , i_indepdt_var_loc      
    , array_dims(i_co_value)
    , i_input_arr_asso
    , array_dims(i_depdt_var)
    , array_dims(i_dloss_ddepdt)
    , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;



-- select 
--   sm_sc.ufv_lambda_arr_dloss_dindepdt
--   (
--     100001
--     , 'softmax_x'
--     , array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]]
--     , 2
--     , null
--     , null
--     , sm_sc.fv_standlize_x_softmax(array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]])
--     , (-` array[[0.0 :: float, 0.0 :: float, 1.0], [0.0 :: float, 0.0 :: float, 1.0]]) /` sm_sc.fv_standlize_x_softmax(array[[1.23, 3.34, 0.96],[2.2, 0.25, 7.7]])
--   );




-- drop function if exists sm_sc.ufv_lambda_arr_len(bigint, varchar(64), int[], int[], float[], int[]);
create or replace function sm_sc.ufv_lambda_arr_len
(
  i_node_no                   bigint                          ,
  i_lambda                    varchar(64)                     ,
  i_input_p1_len              int[]                           ,
  i_input_p2_len              int[]             default null  ,      -- fn 配套的另一个入参值，该配套入参位置与 i_depdt_var_loc 对立
  i_input_arr_asso            float[]           default null  ,    
  i_input_p3_len              int[]             default null  
)
returns int[]
as
$$
declare 
  v_arr_asso       int[]     := i_input_arr_asso ;
  v_p1_ndims       int       := array_length(i_input_p1_len, 1);
  v_p2_ndims       int       := array_length(i_input_p2_len, 1);
  v_p3_ndims       int       := array_length(i_input_p3_len, 1);
  v_p1_len_heigh   int       := i_input_p1_len[v_p1_ndims - 1];
  v_p1_len_width   int       := i_input_p1_len[v_p1_ndims];
  v_p2_len_heigh   int       := i_input_p2_len[v_p2_ndims - 1];
  v_p2_len_width   int       := i_input_p2_len[v_p2_ndims];
  v_p3_len_heigh   int       := i_input_p3_len[v_p3_ndims - 1];
  v_p3_len_width   int       := i_input_p3_len[v_p3_ndims];
  v_n_ndims_len    int[]     := 
    case 
      when v_p1_ndims = 4 then i_input_p1_len[1 : 2] 
      when v_p1_ndims = 3 then i_input_p1_len[1 : 1]
      when v_p2_ndims = 4 then i_input_p2_len[1 : 2]
      when v_p2_ndims = 3 then i_input_p2_len[1 : 1]
      when v_p3_ndims = 4 then i_input_p3_len[1 : 2]
      when v_p3_ndims = 3 then i_input_p3_len[1 : 1]
      else null
    end;
  
begin
  -- 审计各参数各维度长度是否匹配
  if i_lambda = '81_query_from_col'
  then 
    return 
      i_input_p1_len[ : array_length(i_input_p1_len, 1) - 2] || array[1] || i_input_p1_len[array_length(i_input_p1_len, 1)]
    ;
  elsif i_lambda = '81_query_from_row'
  then 
    return 
      i_input_p1_len[ : array_length(i_input_p1_len, 1) - 1] || array[1]
    ;
  elsif i_lambda = '81_prod_mx_slice_no_backward'
  then 
    if v_p1_len_heigh > 0 and v_p1_len_width > 0 
      and v_p1_len_width > 0 and v_p2_len_width > 0
      -- and v_p1_len_width = coalesce(v_p2_len_heigh, v_arr_asso[2]) 
    then 
      return 
        v_n_ndims_len ||
        array[v_p1_len_heigh, v_p2_len_width];
    else
      raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
    end if;
    
  elsif i_lambda = '81_prod_mx_chunk'
  then 
    if v_p1_len_heigh > 0 and v_p1_len_width > 0 
      and v_p1_len_width > 0 and v_p2_len_width > 0
      and v_p1_len_heigh >= v_p3_len_heigh
      and v_p2_len_width >= v_p3_len_width
      and v_p3_ndims = greatest(v_p1_ndims, v_p2_ndims)
      and v_p1_len_width = v_p2_len_heigh
      and v_arr_asso[1] <= v_p1_len_width
    then 
      return 
        -- v_arr_asso[1] :: int                                                   -- 规约：存放 array_length(i_indepdt, 1)
        v_n_ndims_len ||
        array[v_p1_len_heigh, v_p2_len_width];
    else
      raise exception 'unmatch p1, p2.  i_node_no: %; i_lambda: %; i_input_p1_len: %; i_input_p2_len: %; i_input_arr_asso: %;', i_node_no, i_lambda, i_input_p1_len, i_input_p2_len, v_arr_asso;
    end if;
    
  else 
    raise exception 'no defination for this lambda %', i_lambda;
  end if;  

-- exception when others then
--   raise exception 
--   ' fn: sm_sc.ufv_lambda_arr_len
--     i_node_no: %
--     i_lambda: %
--     i_input_p1_len: %
--     i_input_p2_len: %
--     v_arr_asso: %
--     sqlerrm: %
--   '
--   , i_node_no
--   , i_lambda
--   , i_input_p1_len
--   , i_input_p2_len
--   , v_arr_asso
--   , sqlerrm;

end
$$
language plpgsql volatile
parallel safe
cost 100;



-- select 
--   sm_sc.ufv_lambda_arr_len
--   (
--     'conv_2d',                                -- i_lambda              
--     array[70, 784],                           -- i_input_p1_len
--     array[1, 25],                              -- i_input_p2_len        
--     array[28, 5, 5, 1, 1, 2, 2, 2, 2, 0]      -- v_arr_asso 
--   )

-- -- select 
-- --         array
-- --         [
-- --           v_p1_len_heigh,
-- --           (v_p1_len_width / v_arr_asso[1] + coalesce(v_arr_asso[8], 0) + coalesce(v_arr_asso[9], 0) - coalesce(v_arr_asso[3], v_p2_len_width)) / coalesce(v_arr_asso[5], 1)       
-- --           * (v_arr_asso[1] + coalesce(v_arr_asso[6], 0) + coalesce(v_arr_asso[7], 0) - coalesce(v_arr_asso[2], v_p2_len_heigh)) / coalesce(v_arr_asso[4], 1)     
-- --         ]
-- -- 
-- --         -- v_arr_asso[1]                                                       -- 规约：存放 i_1d_2_2d_cnt_per_grp
-- --         -- v_arr_asso[2 : 3]                                           -- 规约：存放 i_window_len
-- --         -- coalesce(v_arr_asso[4 : 5]        ,array[1, 1]      )       -- 规约：存放 i_stride       
-- --         -- coalesce(v_arr_asso[6 : 9]        ,array[0, 0, 0, 0])       -- 规约：存放 i_padding      
-- --         -- coalesce(v_arr_asso[10] :: float ,0.0              )       -- 规约：存放 i_padding_value
drop function if exists sm_sc.ufv_upd_att_ffn_chunk(bigint, bigint, int, int);
create or replace function sm_sc.ufv_upd_att_ffn_chunk
(
  
  i_work_no                            bigint
, i_combine_no                         bigint
, i_w_ff1_heigh_ff2_width              int    -- 通常对应 attention_qkv 的 i_v_width 
                                              -- 或多头的 i_v_width 的 concat
                                              -- 或 i_token_embedding_len
, i_w_ff1_width_ff2_heigh              int
)
returns void
as 
$$
-- declare 
begin
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[1] = i_w_ff1_heigh_ff2_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 3
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][1] = i_w_ff1_heigh_ff2_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 9
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2] = i_w_ff1_heigh_ff2_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 4
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][2] = i_w_ff1_heigh_ff2_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 10
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[3] = i_w_ff1_heigh_ff2_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 2
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[1] = i_w_ff1_width_ff2_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 6
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][1] = i_w_ff1_width_ff2_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 10
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2] = i_w_ff1_width_ff2_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (3, 7)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][2] = i_w_ff1_width_ff2_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 9
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[3] = i_w_ff1_width_ff2_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 4
  ;
end
$$
language plpgsql stable
parallel safe
;

-- select 
--   sm_sc.ufv_upd_att_ffn_chunk
--   (
--     -9                                -- i_work_no   
--   , -99                               -- i_combine_no
--   , 64                                -- i_w_ff1_heigh_ff2_width  
--   , 64                                -- i_w_ff1_width_ff2_heigh   
--   )

drop function if exists sm_sc.ufv_upd_att_qkv_chunk(bigint, bigint, int, int, int, int);
create or replace function sm_sc.ufv_upd_att_qkv_chunk
(
  i_work_no                               bigint
, i_combine_no                            bigint
, i_seq_len                               int
, i_qk_width                              int
, i_kv_heigh                              int
, i_v_width                               int
)
returns void
as 
$$
-- declare 
begin
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[1] = i_seq_len
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (4, 5, 6, 8, 9, 14)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][1] = i_seq_len
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 18
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2] = i_qk_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (1, 2, 8)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][2] = i_qk_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (15, 16)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[3] = i_qk_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (4, 5)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2] = i_kv_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (9, 14)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][2] = i_kv_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 18
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[3] = i_kv_heigh
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (8, 13)
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2] = i_v_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 3
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[2][2] = i_v_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no = 17
  ;
  update sm_sc.__vt_tmp_nn_node
  set node_fn_asso_value[3] = i_v_width
  where work_no = i_work_no
    and node_grp_no = i_combine_no
    and node_grp_ord_no in (6, 14)
  ;
end
$$
language plpgsql stable
parallel safe
;

-- select 
--   sm_sc.ufv_upd_att_qkv_chunk
--   (
--     -9                                -- i_work_no   
--   , -99                               -- i_combine_no
--   , 40                                -- i_seq_len   
--   , 64                                -- i_qk_width  
--   , 64                                -- i_kv_heigh  
--   , 64                                -- i_v_width   
--   )

drop function if exists sm_sc.ufv_combine_att_ffn(int, int, float, int);
create or replace function sm_sc.ufv_combine_att_ffn
(
  i_w_ff1_heigh_ff2_width              int    -- 通常对应 attention_qkv 的 i_v_width 
                                              -- 或多头的 i_v_width 的 concat
                                              -- 或 i_token_embedding_len
, i_w_ff1_width_ff2_heigh              int
, i_leak_relu_alpha                    float    default   0.01
, i_ndims                              int      default   3
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select                                           -- need token_embedding input as p1
      1 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+` token_embedding as res'                                      as node_desc       
    union all                                                          
    select                                                             
      2 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_zscore'                                                      as node_fn_type
    , sm_sc.fv_lpad(array[1, i_w_ff1_heigh_ff2_width], array[1], i_ndims - 2)                             
                                                                       as node_fn_asso_value
    , 'layer_normalize'                                                as node_desc
    union all                                                          
    select                                                             
      3 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_w_ff1_heigh_ff2_width, i_w_ff1_width_ff2_heigh]          as node_fn_asso_value
    , 'w1 of ffn'                                                      as node_desc
    union all                                                          
    select                                                             
      4 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[null, i_w_ff1_heigh_ff2_width, i_w_ff1_width_ff2_heigh]    as node_fn_asso_value
    , '|**| w1 of ffn'                                                 as node_desc
    union all                                                          
    select                                                             
      5 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_leaky_relu'                                                  as node_fn_type
    , array[i_leak_relu_alpha]                                         as node_fn_asso_value
    , 'leaky_relu of ffn'                                              as node_desc
    union all                                                          
    select                                                             
      6 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_w_ff1_width_ff2_heigh, i_w_ff1_heigh_ff2_width]          as node_fn_asso_value
    , 'w2 of ffn'                                                      as node_desc
    union all                                                          
    select                                                             
      7 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[null, i_w_ff1_width_ff2_heigh, i_w_ff1_heigh_ff2_width]    as node_fn_asso_value
    , '|**| w2 of ffn'                                                 as node_desc
    union all                                                          
    select                                                             -- this is output of single transformer encoder or decoder
      8 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+` attention as res'                                            as node_desc       
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 2               -- layer_normalize
      and tb_a_back_node.a_tmp_no = 1               -- +` token_embedding as res
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4               -- |**| w1 of ffn
      and tb_a_back_node.a_tmp_no = 2               -- layer_normalize
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4               -- |**| w1 of ffn
      and tb_a_back_node.a_tmp_no = 3               -- w1 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 5               -- leaky_relu of ffn
      and tb_a_back_node.a_tmp_no = 4               -- |**| w1 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 7               -- |**| w2 of ffn
      and tb_a_back_node.a_tmp_no = 5               -- leaky_relu of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 7               -- |**| w2 of ffn
      and tb_a_back_node.a_tmp_no = 6               -- w2 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8               -- +` attention as res
      and tb_a_back_node.a_tmp_no = 2               -- layer_normalize
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8               -- +` attention as res
      and tb_a_back_node.a_tmp_no = 7               -- |**| w2 of ffn
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 1
    union all
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 2                                              as path_ord_no
    , 2                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 1
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 8
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_ffn
--   (
--     40
--   , 64
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_ffn 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 82,
--             "node_desc": "+` token_embedding as res",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 83,
--             "node_desc": "layer_normalize",
--             "node_type": null,
--             "node_fn_type": "03_zscore",
--             "node_fn_asso_value": "{1,1,40}"
--           },
--           {
--             "node_no": 84,
--             "node_desc": "w1 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,64}"
--           },
--           {
--             "node_no": 85,
--             "node_desc": "|**| w1 of ffn",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{NULL,40,64}"
--           },
--           {
--             "node_no": 86,
--             "node_desc": "leaky_relu of ffn",
--             "node_type": null,
--             "node_fn_type": "03_leaky_relu",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 87,
--             "node_desc": "w2 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{64,40}"
--           },
--           {
--             "node_no": 88,
--             "node_desc": "|**| w2 of ffn",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{NULL,64,40}"
--           },
--           {
--             "node_no": 89,
--             "node_desc": "+` attention as res",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 1,
--             "back_node_no": 82,
--             "fore_node_no": 83
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 83,
--             "fore_node_no": 85
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 84,
--             "fore_node_no": 85
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 85,
--             "fore_node_no": 86
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 86,
--             "fore_node_no": 88
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 87,
--             "fore_node_no": 88
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 83,
--             "fore_node_no": 89
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 88,
--             "fore_node_no": 89
--           }
--         ],
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 82,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 2,
--             "fore_node_no": 82
--             "combine_in_no": 2
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 89
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes

drop function if exists sm_sc.ufv_combine_att_ffn_chunk(int, int, float, int);
create or replace function sm_sc.ufv_combine_att_ffn_chunk
(
  i_w_ff1_heigh_ff2_width              int    -- 通常对应 attention_qkv 的 i_v_width 
                                              -- 或多头的 i_v_width 的 concat
                                              -- 或 i_token_embedding_len
, i_w_ff1_width_ff2_heigh              int
, i_leak_relu_alpha                    float    default   0.01
, i_ndims                              int      default   3
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select                                           -- need token_embedding input as p1
      1 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+` token_embedding as res'                                      as node_desc       
    union all                                                          
    select                                                             
      2 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_zscore'                                                      as node_fn_type
    , sm_sc.fv_lpad(array[1, i_w_ff1_heigh_ff2_width], array[1], i_ndims - 2)                             
                                                                       as node_fn_asso_value
    , 'layer_normalize'                                                as node_desc
    union all                                                          
    select                                                             
      3 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_w_ff1_heigh_ff2_width, i_w_ff1_width_ff2_heigh]          as node_fn_asso_value
    , 'w1 of ffn'                                                      as node_desc
    union all                                                          
    select                                                             
      4 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[null, i_w_ff1_heigh_ff2_width, i_w_ff1_width_ff2_heigh]    as node_fn_asso_value
    , '|**| w1 of ffn'                                                 as node_desc
    union all                                                          
    select                                                             
      5 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_leaky_relu'                                                  as node_fn_type
    , array[i_leak_relu_alpha]                                         as node_fn_asso_value
    , 'leaky_relu of ffn'                                              as node_desc
    union all                                                          
    select                                                             
      6 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_w_ff1_width_ff2_heigh, i_w_ff1_heigh_ff2_width]          as node_fn_asso_value
    , 'w2 of ffn'                                                      as node_desc
    union all                                                          
    select                                                             
      7 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[null, i_w_ff1_width_ff2_heigh, i_w_ff1_heigh_ff2_width]    as node_fn_asso_value
    , '|**| w2 of ffn'                                                 as node_desc
    union all                                                          
    select                                                             -- this is output of single transformer encoder or decoder
      8 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+` attention as res'                                            as node_desc       
    union all                                                          
    select                                                             
      9 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_chunk'                                                       as node_fn_type
    , array[[1, 1],[i_w_ff1_heigh_ff2_width, i_w_ff1_width_ff2_heigh]] as node_fn_asso_value
    , 'chunk w1 of ffn'                                                as node_desc
    union all                                                          
    select                                                             
      10 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_chunk'                                                       as node_fn_type
    , array[[1, 1],[i_w_ff1_width_ff2_heigh, i_w_ff1_heigh_ff2_width]] as node_fn_asso_value
    , 'chunk w2 of ffn'                                                as node_desc
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 2               -- layer_normalize
      and tb_a_back_node.a_tmp_no = 1               -- +` token_embedding as res
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4               -- |**| w1 of ffn
      and tb_a_back_node.a_tmp_no = 2               -- layer_normalize
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 9               -- chunk w1 of ffn
      and tb_a_back_node.a_tmp_no = 3               -- w1 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4               -- |**| w1 of ffn
      and tb_a_back_node.a_tmp_no = 9               -- chunk w1 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 5               -- leaky_relu of ffn
      and tb_a_back_node.a_tmp_no = 4               -- |**| w1 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 7               -- |**| w2 of ffn
      and tb_a_back_node.a_tmp_no = 5               -- leaky_relu of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 10              -- chunk w2 of ffn
      and tb_a_back_node.a_tmp_no = 6               -- w2 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 7               -- |**| w2 of ffn
      and tb_a_back_node.a_tmp_no = 10              -- chunk w2 of ffn
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8               -- +` attention as res
      and tb_a_back_node.a_tmp_no = 2               -- layer_normalize
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8               -- +` attention as res
      and tb_a_back_node.a_tmp_no = 7               -- |**| w2 of ffn
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 1
    union all
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 2                                              as path_ord_no
    , 2                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 1
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 8
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'combine_node_ord_no' , a_tmp_no
              , 'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_ffn_chunk
--   (
--     40
--   , 64
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_ffn_chunk 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 46,
--             "node_desc": "+` token_embedding as res",
--             "node_fn_type": "01_add",
--             "combine_node_ord_no": 1
--           },
--           {
--             "node_no": 47,
--             "node_desc": "layer_normalize",
--             "node_fn_type": "03_zscore",
--             "combine_node_ord_no": 2,
--             "node_fn_asso_value": "{1,1,40}"
--           },
--           {
--             "node_no": 48,
--             "node_desc": "w1 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 3,
--             "node_fn_asso_value": "{40,64}"
--           },
--           {
--             "node_no": 49,
--             "node_desc": "|**| w1 of ffn",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 4,
--             "node_fn_asso_value": "{NULL,40,64}"
--           },
--           {
--             "node_no": 50,
--             "node_desc": "leaky_relu of ffn",
--             "node_fn_type": "03_leaky_relu",
--             "combine_node_ord_no": 5,
--             "node_fn_asso_value": "{0.01}"
--           },
--           {
--             "node_no": 51,
--             "node_desc": "w2 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 6,
--             "node_fn_asso_value": "{64,40}"
--           },
--           {
--             "node_no": 52,
--             "node_desc": "|**| w2 of ffn",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 7,
--             "node_fn_asso_value": "{NULL,64,40}"
--           },
--           {
--             "node_no": 53,
--             "node_desc": "+` attention as res",
--             "node_fn_type": "01_add",
--             "combine_node_ord_no": 8
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 1,
--             "back_node_no": 46,
--             "fore_node_no": 47
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 47,
--             "fore_node_no": 49
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 48,
--             "fore_node_no": 49
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 49,
--             "fore_node_no": 50
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 50,
--             "fore_node_no": 52
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 51,
--             "fore_node_no": 52
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 47,
--             "fore_node_no": 53
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 52,
--             "fore_node_no": 53
--           }
--         ],
--         "combine_no": 54,
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 46,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 2,
--             "fore_node_no": 46,
--             "combine_in_no": 2
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 53
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_combine_no as
-- -- (
-- --   select                                
-- --     a_j_combine_no  :: int          as combine_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.combine_no') tb_a_combine_no(a_j_combine_no)
-- -- ) -- select combine_no from cte_combine_no
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'combine_node_ord_no' )  :: int          as combine_node_ord_no
-- --   , (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select combine_node_ord_no, node_no, node_type, node_fn_type, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes

drop function if exists sm_sc.ufv_combine_att_qkv(int, int, int, int, int, boolean);
create or replace function sm_sc.ufv_combine_att_qkv
(
  i_seq_len                               int
, i_token_embedding_len                   int
, i_qk_width                              int
, i_kv_heigh                              int
, i_v_width                               int
, i_is_mask_arr                           boolean
-- , i_ngram                                 int        default 1
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select 
      1 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_qk_width] :: float[]              as node_fn_asso_value
    , 'w_q'                                                            as node_desc
    union all                                                          
    select                                                             
      2 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_qk_width]                         as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_qk_width]
    , 'w_k'                                                            as node_desc
    union all                                                          
    select                                                             
      3 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_v_width]                          as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_v_width]
    , 'w_v'                                                            as node_desc
    union all                                                          
    select                                                             -- need token_embedding input as p1
      4 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_q'                                                       as node_desc       
    union all                                                          
    select                                                             -- need token_embedding input as p1
      5 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_k'                                                       as node_desc       
    union all                                                          
    select                                                             -- need token_embedding input as p1
      6 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_v_width]               as node_fn_asso_value
    , '|**| w_v'                                                       as node_desc       
    union all                                                          
    select                                                             
      7 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_transpose'                                                   as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'transpose(k)'                                                   as node_desc
    union all                                                          
    select                                                             
      8 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_qk_width, i_kv_heigh]                         as node_fn_asso_value
    , 'q |**| transpose(k)'                                            as node_desc
    union all                                                          
    -- 不采用下三角矩阵算子，而采用元素加 masked 矩阵的方式；后者可以灵活的调整 masked 策略
    select                                                             
      9 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_seq_len, i_kv_heigh]                                     as node_fn_asso_value
    , 'masked_arr'                                                     as node_desc
    where i_is_mask_arr
    union all                                                          
    select                                                             
      10 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+ sequence_mask'                                                as node_desc
    where i_is_mask_arr
    union all                                                          
    select                                                             
      11 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[1]                                                         as node_fn_asso_value   -- array[1, i_seq_len, i_kv_heigh]
    , 'const(qk_width ^ 0.5)'                                          as node_desc
    union all                                                          
    select                                                             
      12 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_div'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '/` (qk_width ^ 0.5)'                                            as node_desc
    union all                                                          
    select                                                             
      13 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_softmax'                                                     as node_fn_type
    , array[1, 1, i_kv_heigh]                                          as node_fn_asso_value
    , 'softmax'                                                        as node_desc
    union all                                                          
    select                                                             
      14 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_kv_heigh, i_v_width]                          as node_fn_asso_value
    , '|**| v'                                                         as node_desc
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4                -- |**| w_q
      and tb_a_back_node.a_tmp_no = 1                -- w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 5                -- |**| w_k
      and tb_a_back_node.a_tmp_no = 2                -- w_k
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 6                -- |**| w_v
      and tb_a_back_node.a_tmp_no = 3                -- w_v
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 7                -- transpose(k)
      and tb_a_back_node.a_tmp_no = 5                -- |**| w_k
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8                -- q |**| transpose(k)
      and tb_a_back_node.a_tmp_no = 4                -- |**| w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8                -- q |**| transpose(k)
      and tb_a_back_node.a_tmp_no = 7                -- transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 10               -- + sequence_mask
      and tb_a_back_node.a_tmp_no = 8                -- q |**| transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 10               -- + sequence_mask
      and tb_a_back_node.a_tmp_no = 9                -- masked_arr
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 12               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no = 10               -- + sequence_mask
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 12               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no = 11               -- const(qk_width ^ 0.5)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 13               -- softmax
      and tb_a_back_node.a_tmp_no = 12               -- /` (qk_width ^ 0.5)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 14               -- |**| v
      and tb_a_back_node.a_tmp_no = 13               -- softmax
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 14               -- |**| v
      and tb_a_back_node.a_tmp_no = 6                -- |**| w_v
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 4
    union all
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 2                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 5
    union all
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 3                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 6
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 14 -- 23
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_qkv
--   (
--     40                                -- i_seq_len            
--   , 64                                -- i_token_embedding_len
--   , 64                                -- i_qk_width           
--   , 64                                -- i_kv_heigh           
--   , 64                                -- i_v_width            
--   , true                                  -- i_is_mask_arr      
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_qkv 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 68,
--             "node_desc": "w_q",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 69,
--             "node_desc": "w_k",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 70,
--             "node_desc": "w_v",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 71,
--             "node_desc": "|**| w_q",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 72,
--             "node_desc": "|**| w_k",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 73,
--             "node_desc": "|**| w_v",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 74,
--             "node_desc": "transpose(k)",
--             "node_type": null,
--             "node_fn_type": "04_transpose",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 75,
--             "node_desc": "q |**| transpose(k)",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 76,
--             "node_desc": "masked_arr",
--             "node_type": null,
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,64}"
--           },
--           {
--             "node_no": 77,
--             "node_desc": "+ sequence_mask",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 78,
--             "node_desc": "const(qk_width ^ 0.5)",
--             "node_type": null,
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1,40,64}"
--           },
--           {
--             "node_no": 79,
--             "node_desc": "/` (qk_width ^ 0.5)",
--             "node_type": null,
--             "node_fn_type": "01_div",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 80,
--             "node_desc": "softmax",
--             "node_type": null,
--             "node_fn_type": "03_softmax",
--             "node_fn_asso_value": "{1,1,64}"
--           },
--           {
--             "node_no": 81,
--             "node_desc": "|**| v",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 2,
--             "back_node_no": 68,
--             "fore_node_no": 71
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 69,
--             "fore_node_no": 72
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 70,
--             "fore_node_no": 73
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 72,
--             "fore_node_no": 74
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 71,
--             "fore_node_no": 75
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 74,
--             "fore_node_no": 75
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 75,
--             "fore_node_no": 77
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 76,
--             "fore_node_no": 77
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 77,
--             "fore_node_no": 79
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 78,
--             "fore_node_no": 79
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 79,
--             "fore_node_no": 80
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 80,
--             "fore_node_no": 81
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 73,
--             "fore_node_no": 81
--           }
--         ],
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 71,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 72,
--             "combine_in_no": 2
--           },
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 73,
--             "combine_in_no": 3
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 81
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_fn_asso_value, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes
drop function if exists sm_sc.ufv_combine_att_qkv_chunk(int, int, int, int, int, boolean);
create or replace function sm_sc.ufv_combine_att_qkv_chunk
(
  i_seq_len                               int
, i_token_embedding_len                   int
, i_qk_width                              int
, i_kv_heigh                              int
, i_v_width                               int
, i_is_mask_arr                           boolean
-- , i_ngram                                 int        default 1
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select 
      1 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_qk_width] :: float[]              as node_fn_asso_value
    , 'w_q'                                                            as node_desc
    union all                                                          
    select                                                             
      2 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_qk_width]                         as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_qk_width]
    , 'w_k'                                                            as node_desc
    union all                                                          
    select                                                             
      3 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_v_width]                          as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_v_width]
    , 'w_v'                                                            as node_desc
    union all                                                          
    select                                                             -- need token_embedding input as p1
      4 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_q'                                                       as node_desc       
    union all                                                          
    select                                                             -- need token_embedding input as p1
      5 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_k'                                                       as node_desc       
    union all                                                          
    select                                                             -- need token_embedding input as p1
      6 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_v_width]               as node_fn_asso_value
    , '|**| w_v'                                                       as node_desc       
    union all                                                          
    select                                                             
      7 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_transpose'                                                   as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'transpose(k)'                                                   as node_desc
    union all                                                          
    select                                                             
      8 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_qk_width, i_kv_heigh]                         as node_fn_asso_value
    , 'q |**| transpose(k)'                                            as node_desc
    union all                                                          
    -- 不采用下三角矩阵算子，而采用元素加 masked 矩阵的方式；后者可以灵活的调整 masked 策略
    select                                                             
      9 :: int                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_seq_len, i_kv_heigh]                                     as node_fn_asso_value
    , 'masked_arr'                                                     as node_desc
    where i_is_mask_arr
    union all                                                          
    select                                                             
      10 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+ sequence_mask'                                                as node_desc
    where i_is_mask_arr
    union all                                                          
    select                                                             
      11 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[1]                                                         as node_fn_asso_value   -- array[1, i_seq_len, i_kv_heigh]
    , 'const(qk_width ^ 0.5)'                                          as node_desc
    union all                                                          
    select                                                             
      12 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_div'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '/` (qk_width ^ 0.5)'                                            as node_desc
    union all                                                          
    select                                                             
      13 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_softmax'                                                     as node_fn_type
    , array[1, 1, i_kv_heigh]                                          as node_fn_asso_value
    , 'softmax'                                                        as node_desc
    union all                                                          
    select                                                             
      14 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_kv_heigh, i_v_width]                          as node_fn_asso_value
    , '|**| v'                                                         as node_desc
    union all                                                          
    select                                                             
      15 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_chunk'                                                       as node_fn_type
    , array[[1, 1],[i_token_embedding_len, i_qk_width]]                as node_fn_asso_value
    , 'w_q_chunk'                                                      as node_desc
    union all                                                          
    select                                                             
      16 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_chunk'                                                       as node_fn_type
    , array[[1, 1],[i_token_embedding_len, i_qk_width]]                as node_fn_asso_value
    , 'w_k_chunk'                                                      as node_desc
    union all                                                          
    select                                                             
      17 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_chunk'                                                       as node_fn_type
    , array[[1, 1],[i_token_embedding_len, i_v_width]]                 as node_fn_asso_value
    , 'w_v_chunk'                                                      as node_desc
    union all                                                          
    select                                                             
      18 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_chunk'                                                       as node_fn_type
    , array[[1, 1],[i_seq_len, i_kv_heigh]]                            as node_fn_asso_value
    , 'decode_mask_chunk'                                              as node_desc
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 15               -- chunk w_q
      and tb_a_back_node.a_tmp_no = 1                -- w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4                -- |**| w_q
      and tb_a_back_node.a_tmp_no = 15               -- chunk w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 16               -- chunk w_k
      and tb_a_back_node.a_tmp_no = 2                -- w_k
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 5                -- |**| w_k
      and tb_a_back_node.a_tmp_no = 16               -- chunk w_k
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 17               -- chunk w_v
      and tb_a_back_node.a_tmp_no = 3                -- w_v
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 6                -- |**| w_v
      and tb_a_back_node.a_tmp_no = 17               -- chunk w_v
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 7                -- transpose(k)
      and tb_a_back_node.a_tmp_no = 5                -- |**| w_k
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8                -- q |**| transpose(k)
      and tb_a_back_node.a_tmp_no = 4                -- |**| w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 8                -- q |**| transpose(k)
      and tb_a_back_node.a_tmp_no = 7                -- transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 10               -- + sequence_mask
      and tb_a_back_node.a_tmp_no = 8                -- q |**| transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 18               -- chunk masked_arr
      and tb_a_back_node.a_tmp_no = 9                -- masked_arr
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 10               -- + sequence_mask
      and tb_a_back_node.a_tmp_no = 18               -- chunk masked_arr
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 12               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no = 10               -- + sequence_mask
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 12               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no = 11               -- const(qk_width ^ 0.5)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 13               -- softmax
      and tb_a_back_node.a_tmp_no = 12               -- /` (qk_width ^ 0.5)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 14               -- |**| v
      and tb_a_back_node.a_tmp_no = 13               -- softmax
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 14               -- |**| v
      and tb_a_back_node.a_tmp_no = 6                -- |**| w_v
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 4
    union all
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 2                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 5
    union all
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 3                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 6
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 14 -- 23
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'combine_node_ord_no' , a_tmp_no
              , 'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_qkv_chunk
--   (
--     40                                -- i_seq_len            
--   , 64                                -- i_token_embedding_len
--   , 64                                -- i_qk_width           
--   , 64                                -- i_kv_heigh           
--   , 64                                -- i_v_width            
--   , true                                  -- i_is_mask_arr      
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_qkv_chunk 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 23,
--             "node_desc": "w_q",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 1,
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 24,
--             "node_desc": "w_k",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 2,
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 25,
--             "node_desc": "w_v",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 3,
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 26,
--             "node_desc": "|**| w_q",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 4,
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 27,
--             "node_desc": "|**| w_k",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 5,
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 28,
--             "node_desc": "|**| w_v",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 6,
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 29,
--             "node_desc": "transpose(k)",
--             "node_fn_type": "04_transpose",
--             "combine_node_ord_no": 7
--           },
--           {
--             "node_no": 30,
--             "node_desc": "q |**| transpose(k)",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 8,
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 31,
--             "node_desc": "masked_arr",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 9,
--             "node_fn_asso_value": "{40,64}"
--           },
--           {
--             "node_no": 32,
--             "node_desc": "+ sequence_mask",
--             "node_fn_type": "01_add",
--             "combine_node_ord_no": 10
--           },
--           {
--             "node_no": 33,
--             "node_desc": "const(qk_width ^ 0.5)",
--             "node_fn_type": "00_const",
--             "combine_node_ord_no": 11,
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 34,
--             "node_desc": "/` (qk_width ^ 0.5)",
--             "node_fn_type": "01_div",
--             "combine_node_ord_no": 12
--           },
--           {
--             "node_no": 35,
--             "node_desc": "softmax",
--             "node_fn_type": "03_softmax",
--             "combine_node_ord_no": 13,
--             "node_fn_asso_value": "{1,1,64}"
--           },
--           {
--             "node_no": 36,
--             "node_desc": "|**| v",
--             "node_fn_type": "01_prod_mx",
--             "combine_node_ord_no": 14,
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 37,
--             "node_desc": "w_q_chunk",
--             "node_fn_type": "04_chunk",
--             "combine_node_ord_no": 15,
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 38,
--             "node_desc": "w_k_chunk",
--             "node_fn_type": "04_chunk",
--             "combine_node_ord_no": 16,
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 39,
--             "node_desc": "w_v_chunk",
--             "node_fn_type": "04_chunk",
--             "combine_node_ord_no": 17,
--             "node_fn_asso_value": "{64,64}"
--           },
--           {
--             "node_no": 40,
--             "node_desc": "decode_mask_chunk",
--             "node_fn_type": "04_chunk",
--             "combine_node_ord_no": 18,
--             "node_fn_asso_value": "{40,64}"
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 2,
--             "back_node_no": 37,
--             "fore_node_no": 26
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 38,
--             "fore_node_no": 27
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 39,
--             "fore_node_no": 28
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 27,
--             "fore_node_no": 29
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 26,
--             "fore_node_no": 30
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 29,
--             "fore_node_no": 30
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 30,
--             "fore_node_no": 32
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 40,
--             "fore_node_no": 32
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 32,
--             "fore_node_no": 34
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 33,
--             "fore_node_no": 34
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 34,
--             "fore_node_no": 35
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 35,
--             "fore_node_no": 36
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 28,
--             "fore_node_no": 36
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 23,
--             "fore_node_no": 37
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 24,
--             "fore_node_no": 38
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 25,
--             "fore_node_no": 39
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 31,
--             "fore_node_no": 40
--           }
--         ],
--         "combine_no": 41,
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 26,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 27,
--             "combine_in_no": 2
--           },
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 28,
--             "combine_in_no": 3
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 36
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_combine_no as
-- -- (
-- --   select                                
-- --     a_j_combine_no  :: int          as combine_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.combine_no') tb_a_combine_no(a_j_combine_no)
-- -- ) -- select combine_no from cte_combine_no
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'combine_node_ord_no' )  :: int          as combine_node_ord_no
-- --   , (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select combine_node_ord_no, node_no, node_type, node_fn_type, node_fn_asso_value, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes
drop function if exists sm_sc.ufv_combine_att_qkv_ngram(int, int, int, int, boolean, int);
create or replace function sm_sc.ufv_combine_att_qkv_ngram
(
  i_seq_len                               int
, i_token_embedding_len                   int
, i_qk_width                              int
-- , i_kv_heigh                              int
, i_v_width                               int
, i_is_mask_arr                           boolean
, i_gram_n                                int
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select 
      000 + 1                                                          as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_none'                                                        as node_fn_type
    , null                                     :: float[]              as node_fn_asso_value
    , 'qkv_input'                                                      as node_desc
    union all                          
    select 
      100 + 1                                                          as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_qk_width] :: float[]              as node_fn_asso_value
    , 'w_q'                                                            as node_desc
    union all                                                          
    select                                                             
      200 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_qk_width]                         as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_qk_width]
    , 'w_k_' || a_no || 'gram'                                         as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      300 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_token_embedding_len, i_v_width]                          as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_v_width]
    , 'w_v_' || a_no || 'gram'                                         as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             -- need token_embedding input as p1
      400 + 1                                                          as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_q'                                                       as node_desc       
    union all                                                          
    select                                                             -- need token_embedding input as p1
      500 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_k_' || a_no || 'gram'                                    as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)  
    union all                                                          
    select                                                             -- need token_embedding input as p1
      600 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_v_width]               as node_fn_asso_value
    , '|**| w_v_' || a_no || 'gram'                                    as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      700 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_transpose'                                                   as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'transpose(k)_' || a_no || 'gram'                                as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      800 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_qk_width, i_seq_len - a_no + 1]               as node_fn_asso_value
    , 'q |**| transpose(k)_' || a_no || 'gram'                         as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    -- 不采用下三角矩阵算子，而采用元素加 masked 矩阵的方式；后者可以灵活的调整 masked 策略
    select                                                             
      900 + a_no                                                       as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_seq_len, i_seq_len - a_no + 1]                           as node_fn_asso_value
    , 'masked_arr_' || a_no || 'gram'                                  as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    where i_is_mask_arr
    union all                                                          
    select                                                             
      1000 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+ sequence_mask_' || a_no || 'gram'                             as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    where i_is_mask_arr
    union all                                                          
    select                                                             
      1100 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[1]                                                         as node_fn_asso_value   -- array[1, i_seq_len, i_seq_len - a_no + 1]
    , 'const(qk_width ^ 0.5)_' || a_no || 'gram'                       as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      1200 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_div'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '/` (qk_width ^ 0.5)_' || a_no || 'gram'                         as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      1300 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_softmax'                                                     as node_fn_type
    , array[1, 1, i_seq_len - a_no + 1]                                as node_fn_asso_value
    , 'softmax_' || a_no || 'gram'                                     as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      1400 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_seq_len - a_no + 1, i_v_width]                as node_fn_asso_value
    , '|**| v_' || a_no || 'gram'                                      as node_desc
    from generate_series(1, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      1700 + 1                                                         as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_sum'                                                 as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'qkv_ngram mx_sum'                                               as node_desc
    union all                                                          
    select                                                             
      1800 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '05_pool_none'                                                   as node_fn_type
    , array[null, a_no, i_token_embedding_len, 1, 1, 0, 0, 0, 0, 0]    as node_fn_asso_value
    , 'ngram 05_pool_none'                                             as node_desc
    from generate_series(2, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      1900 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[i_seq_len - a_no + 1, a_no]                                as node_fn_asso_value
    , 'ngram 01_chunk_prod_mx weight 1st'                              as node_desc
    from generate_series(2, i_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      2000 + a_no                                                      as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_chunk_prod_mx'                                               as node_fn_type
    , array[1, a_no, i_token_embedding_len]                            as node_fn_asso_value
    , 'ngram 01_chunk_prod_mx'                                         as node_desc
    from generate_series(2, i_gram_n) as tb_a(a_no)
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 400 + 1                -- |**| w_q
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 18                -- ngram 05_pool_none
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 20               -- ngram 01_chunk_prod_mx
      and tb_a_back_node.a_tmp_no / 100 = 18               -- ngram 05_pool_none
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 20               -- ngram 01_chunk_prod_mx
      and tb_a_back_node.a_tmp_no / 100 = 19               -- ngram 01_chunk_prod_mx weight 1st
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 5                -- |**| w_k_1gram
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
      and tb_a_fore_node.a_tmp_no % 100 = 1
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 5                -- |**| w_k_ngram n >= 2
      and tb_a_back_node.a_tmp_no / 100 = 20               -- ngram 01_chunk_prod_mx
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 6                -- |**| w_v_1gram
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
      and tb_a_fore_node.a_tmp_no % 100 = 1
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 6                -- |**| w_v_ngram n > 2
      and tb_a_back_node.a_tmp_no / 100 = 20                -- ngram aggr_mx_concat_x3
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
      -- and tb_a_fore_node.a_tmp_no % 100 > 1
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 400 + 1                -- |**| w_q
      and tb_a_back_node.a_tmp_no = 100 + 1                -- w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 5                -- |**| w_k_ngram
      and tb_a_back_node.a_tmp_no / 100 = 2                -- w_k_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100 % 10
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 6                -- |**| w_v_ngram
      and tb_a_back_node.a_tmp_no / 100 = 3                -- w_v_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100 % 10
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 7                -- transpose(k)_ngram
      and tb_a_back_node.a_tmp_no / 100 = 5                -- |**| w_k_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 8                -- q |**| transpose(k)_ngram
      and tb_a_back_node.a_tmp_no = 400 + 1               -- |**| w_q_ngram
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 8                -- q |**| transpose(k)_ngram
      and tb_a_back_node.a_tmp_no / 100 = 7                -- transpose(k)_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 10               -- + sequence_mask_ngram
      and tb_a_back_node.a_tmp_no / 100 = 8                -- q |**| transpose(k)_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 10               -- + sequence_mask_ngram
      and tb_a_back_node.a_tmp_no / 100 = 9                -- masked_arr_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 12               -- /` -- -- -- (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no / 100 = 10               -- + sequence_mask_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 12               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no / 100 = 11               -- const(qk_width ^ 0.5)_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 13               -- softmax_ngram
      and tb_a_back_node.a_tmp_no / 100 = 12               -- /` (qk_width ^ 0.5)
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 14               -- |**| v_ngram
      and tb_a_back_node.a_tmp_no / 100 = 13               -- softmax_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 14               -- |**| v_ngram
      and tb_a_back_node.a_tmp_no / 100 = 6                -- |**| w_v_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , tb_a_back_node.a_tmp_no % 100                  as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1700 + 1               -- qkv_ngram mx_sum
      and tb_a_back_node.a_tmp_no / 100 = 14                -- |**| v_ngram
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 000 + 1
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 1700 + 1
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_qkv_ngram
--   (
--     40                                -- i_seq_len            
--   , 64                                -- i_token_embedding_len
--   , 64                                -- i_qk_width                  
--   , 64                                -- i_v_width            
--   , true                              -- i_is_mask_arr 
--   , 3     
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_qkv_ngram 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 44050,
--             "node_desc": "qkv_input",
--             "node_fn_type": "00_none"
--           },
--           {
--             "node_no": 44051,
--             "node_desc": "w_q",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44052,
--             "node_desc": "w_k_1gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44053,
--             "node_desc": "w_k_2gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44054,
--             "node_desc": "w_k_3gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44055,
--             "node_desc": "w_v_1gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44056,
--             "node_desc": "w_v_2gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44057,
--             "node_desc": "w_v_3gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{32,64}"
--           },
--           {
--             "node_no": 44058,
--             "node_desc": "|**| w_q",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44059,
--             "node_desc": "|**| w_k_1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44060,
--             "node_desc": "|**| w_k_2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44061,
--             "node_desc": "|**| w_k_3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44062,
--             "node_desc": "|**| w_v_1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44063,
--             "node_desc": "|**| w_v_2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44064,
--             "node_desc": "|**| w_v_3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,64}"
--           },
--           {
--             "node_no": 44065,
--             "node_desc": "transpose(k)_1gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 44066,
--             "node_desc": "transpose(k)_2gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 44067,
--             "node_desc": "transpose(k)_3gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 44068,
--             "node_desc": "q |**| transpose(k)_1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,40}"
--           },
--           {
--             "node_no": 44069,
--             "node_desc": "q |**| transpose(k)_2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,39}"
--           },
--           {
--             "node_no": 44070,
--             "node_desc": "q |**| transpose(k)_3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,38}"
--           },
--           {
--             "node_no": 44071,
--             "node_desc": "masked_arr_1gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,40}"
--           },
--           {
--             "node_no": 44072,
--             "node_desc": "masked_arr_2gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,39}"
--           },
--           {
--             "node_no": 44073,
--             "node_desc": "masked_arr_3gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,38}"
--           },
--           {
--             "node_no": 44074,
--             "node_desc": "+ sequence_mask_1gram",
--             "node_fn_type": "01_add"
--           },
--           {
--             "node_no": 44075,
--             "node_desc": "+ sequence_mask_2gram",
--             "node_fn_type": "01_add"
--           },
--           {
--             "node_no": 44076,
--             "node_desc": "+ sequence_mask_3gram",
--             "node_fn_type": "01_add"
--           },
--           {
--             "node_no": 44077,
--             "node_desc": "const(qk_width ^ 0.5)_1gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 44078,
--             "node_desc": "const(qk_width ^ 0.5)_2gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 44079,
--             "node_desc": "const(qk_width ^ 0.5)_3gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 44080,
--             "node_desc": "/` (qk_width ^ 0.5)_1gram",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 44081,
--             "node_desc": "/` (qk_width ^ 0.5)_2gram",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 44082,
--             "node_desc": "/` (qk_width ^ 0.5)_3gram",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 44083,
--             "node_desc": "softmax_1gram",
--             "node_fn_type": "03_softmax",
--             "node_fn_asso_value": "{1,1,40}"
--           },
--           {
--             "node_no": 44084,
--             "node_desc": "softmax_2gram",
--             "node_fn_type": "03_softmax",
--             "node_fn_asso_value": "{1,1,39}"
--           },
--           {
--             "node_no": 44085,
--             "node_desc": "softmax_3gram",
--             "node_fn_type": "03_softmax",
--             "node_fn_asso_value": "{1,1,38}"
--           },
--           {
--             "node_no": 44086,
--             "node_desc": "|**| v_1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,40,64}"
--           },
--           {
--             "node_no": 44087,
--             "node_desc": "|**| v_2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,39,64}"
--           },
--           {
--             "node_no": 44088,
--             "node_desc": "|**| v_3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,38,64}"
--           },
--           {
--             "node_no": 44089,
--             "node_desc": "qkv_ngram mx_sum",
--             "node_fn_type": "06_aggr_mx_sum"
--           },
--           {
--             "node_no": 44090,
--             "node_desc": "ngram 05_pool_none",
--             "node_fn_type": "05_pool_none",
--             "node_fn_asso_value": "{NULL,2,64,1,1,0,0,0,0,0}"
--           },
--           {
--             "node_no": 44091,
--             "node_desc": "ngram 05_pool_none",
--             "node_fn_type": "05_pool_none",
--             "node_fn_asso_value": "{NULL,3,64,1,1,0,0,0,0,0}"
--           },
--           {
--             "node_no": 44092,
--             "node_desc": "ngram 01_chunk_prod_mx weight 1st",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{39,2}"
--           },
--           {
--             "node_no": 44093,
--             "node_desc": "ngram 01_chunk_prod_mx weight 1st",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{38,3}"
--           },
--           {
--             "node_no": 44094,
--             "node_desc": "ngram 01_chunk_prod_mx",
--             "node_fn_type": "01_chunk_prod_mx",
--             "node_fn_asso_value": "{1,2,64}"
--           },
--           {
--             "node_no": 44095,
--             "node_desc": "ngram 01_chunk_prod_mx",
--             "node_fn_type": "01_chunk_prod_mx",
--             "node_fn_asso_value": "{1,3,64}"
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44050,
--             "fore_node_no": 44058
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44051,
--             "fore_node_no": 44058
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44050,
--             "fore_node_no": 44059
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44052,
--             "fore_node_no": 44059
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44094,
--             "fore_node_no": 44060
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44053,
--             "fore_node_no": 44060
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44095,
--             "fore_node_no": 44061
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44054,
--             "fore_node_no": 44061
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44050,
--             "fore_node_no": 44062
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44055,
--             "fore_node_no": 44062
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44094,
--             "fore_node_no": 44063
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44056,
--             "fore_node_no": 44063
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44095,
--             "fore_node_no": 44064
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44057,
--             "fore_node_no": 44064
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44059,
--             "fore_node_no": 44065
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44060,
--             "fore_node_no": 44066
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44061,
--             "fore_node_no": 44067
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44058,
--             "fore_node_no": 44068
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44065,
--             "fore_node_no": 44068
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44058,
--             "fore_node_no": 44069
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44066,
--             "fore_node_no": 44069
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44058,
--             "fore_node_no": 44070
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44067,
--             "fore_node_no": 44070
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44068,
--             "fore_node_no": 44074
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44071,
--             "fore_node_no": 44074
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44069,
--             "fore_node_no": 44075
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44072,
--             "fore_node_no": 44075
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44070,
--             "fore_node_no": 44076
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44073,
--             "fore_node_no": 44076
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44074,
--             "fore_node_no": 44080
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44077,
--             "fore_node_no": 44080
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44075,
--             "fore_node_no": 44081
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44078,
--             "fore_node_no": 44081
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44076,
--             "fore_node_no": 44082
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44079,
--             "fore_node_no": 44082
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44080,
--             "fore_node_no": 44083
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44081,
--             "fore_node_no": 44084
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44082,
--             "fore_node_no": 44085
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44083,
--             "fore_node_no": 44086
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44062,
--             "fore_node_no": 44086
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44084,
--             "fore_node_no": 44087
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44063,
--             "fore_node_no": 44087
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44085,
--             "fore_node_no": 44088
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44064,
--             "fore_node_no": 44088
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44086,
--             "fore_node_no": 44089
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44087,
--             "fore_node_no": 44089
--           },
--           {
--             "path_ord_no": 3,
--             "back_node_no": 44088,
--             "fore_node_no": 44089
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44050,
--             "fore_node_no": 44090
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44050,
--             "fore_node_no": 44091
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44092,
--             "fore_node_no": 44094
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44090,
--             "fore_node_no": 44094
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 44093,
--             "fore_node_no": 44095
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 44091,
--             "fore_node_no": 44095
--           }
--         ],
--         "combine_no": 44096,
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 44050,
--             "combine_in_no": 1
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 44089
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_fn_asso_value, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes
drop function if exists sm_sc.ufv_combine_att_seg4d_symm_ngram_pos_1head(int, int, int, int, int, int[], int);
create or replace function sm_sc.ufv_combine_att_seg4d_symm_ngram_pos_1head
(
  i_seq_len                               int
, i_seg_len                               int
, i_token_embedding_len                   int                 -- 设置：word_embedding 长度，对称性要求为偶数
, i_qk_width                              int
-- , i_kv_heigh                              int
, i_v_width                               int
, i_ngram                                 int[]
, i_seg_embedding_len                     int     default null
-- 多头涉及到 concat 节点是否必要，在 nn 构建时规划，而不在本 ufv_combine 实现。
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
  v_gram_n             int     :=   array_length(i_ngram, 1)     ;
  v_seg_cnt            int     :=   i_seq_len / i_seg_len        ;  
begin
  if true = any(i_ngram >=` 100)
  then 
    raise exception 'unsupport n >= 100 in i_ngram[]';
  end if;

  if i_seg_embedding_len is null 
  then 
    i_seg_embedding_len := i_token_embedding_len * i_seg_len;
  end if;

  -- 准备 node
  with
  cte_nodes as 
  (
    select 
      000 + 1                                                          as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_zscore'                                                      as node_fn_type
    , array[1, 1, 1, i_token_embedding_len]      :: float[]            as node_fn_asso_value
    , 'qkv_input'                                                      as node_desc
    union all                          
    select 
      100 + 1                                                          as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[v_seg_cnt, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , 'w_q'                                                            as node_desc
    union all                                                          
    select                                                             
      200 + i_ngram[a_no]                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[v_seg_cnt, i_token_embedding_len, i_qk_width]              as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_qk_width]
    , 'w_k_' || a_no || 'th_gram'                                      as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      300 + i_ngram[a_no]                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[v_seg_cnt, i_token_embedding_len, i_v_width]               as node_fn_asso_value       -- array[i_token_embedding_len * i_ngram, i_v_width]
    , 'w_v_' || a_no || 'th_gram'                                      as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             -- need token_embedding input as p1
      400 + 1                                                          as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_q'                                                       as node_desc       
    union all                                                          
    select                                                             -- need token_embedding input as p1
      500 + i_ngram[a_no]                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_qk_width]              as node_fn_asso_value
    , '|**| w_k_' || a_no || 'gram'                                    as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)  
    union all                                                          
    select                                                             -- need token_embedding input as p1
      600 + i_ngram[a_no]                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_seq_len, i_token_embedding_len, i_v_width]               as node_fn_asso_value
    , '|**| w_v_' || a_no || 'gram'                                    as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all                                                          
    select                                                             
      700                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_transpose'                                                   as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'transpose(k)                   '                                as node_desc
    union all                                                          
    select                                                             
      800                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        i_seq_len
      , i_qk_width
      , (i_seg_len * array_length(i_ngram, 1)) - (|@+| i_ngram) + array_length(i_ngram, 1) + v_seg_cnt
      ]                                                                as node_fn_asso_value
    , 'q |**| transpose(k)'                                            as node_desc
    
    union all
    select 
      83400 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_none'                                                        as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'embedding_seg none ' || a_no || 'gram'                     as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      83500 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_transpose'                                                   as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'embedding_seg transpose ' || a_no || 'gram'                     as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      83600 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array
      [
        v_seg_cnt
      , i_token_embedding_len
      , i_token_embedding_len
      ]                                                                as node_fn_asso_value
    , 'embedding_seg prod_mx weight k ' || a_no || 'gram'              as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      83700 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        i_token_embedding_len
      , i_token_embedding_len
      , (i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]
      ]                                                                as node_fn_asso_value
    , 'embedding_seg prod_mx k ' || a_no || 'gram'                     as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      83800 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array
      [
        v_seg_cnt
      , 1
      , (i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]
      ]                                                                as node_fn_asso_value
    , 'embedding_seg prod_mx weight q ' || a_no || 'gram'              as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      83900 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        1
      , (i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]
      , i_token_embedding_len
      ]                                                                as node_fn_asso_value
    , 'embedding_seg prod_mx q ' || a_no || 'gram'                     as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      84000 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        1
      , i_token_embedding_len
      , (i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]
      ]                                                                as node_fn_asso_value
    , 'embedding_seg prod_mx qk ' || a_no || 'gram'                    as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      84100 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[1]                                                         as node_fn_asso_value
    , 'ngram_seg_len ^ 0.5 ' || a_no || 'gram'                         as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    union all
    select 
      84200 + i_ngram[a_no]                                            as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_div'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '/ ngram_seg_len ^ 0.5 ' || a_no || 'gram'                       as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    
    union all
    select 
      851                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_concat_x4'                                           as node_fn_type
    , (
        select 
          array_agg
          (
            (i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no] 
            order by a_no
          ) 
        from generate_series(1, v_gram_n) tb_a(a_no)
      ) 
       || array[1]                                                     as node_fn_asso_value
    , 'embedding_seg concat_x4 position encode'                        as node_desc
    union all
    select                                                             
      852                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[1, v_seg_cnt, 1, 1]                                        as node_fn_asso_value
    , 'seg position encode'                                            as node_desc
    union all
    select 
      853                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_transpose_nd'                                                as node_fn_type
    , array[2, 3]                                                      as node_fn_asso_value
    , 'embedding_seg flatten transpose'                                as node_desc
    union all
    select 
      854                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array
      [
        -- (
        --   select sum((i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]) 
        --   from generate_series(1, v_gram_n) tb_a(a_no)
        -- ) + 1
        i_seg_embedding_len
      , i_qk_width
      ] :: float[]                                                     as node_fn_asso_value
    , 'w_ks_seg'                                                       as node_desc
    union all
    select 
      855                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array
      [
        -- (
        --   select sum((i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]) 
        --   from generate_series(1, v_gram_n) tb_a(a_no)
        -- ) + 1
        i_seg_embedding_len
      , i_v_width
      ] :: float[]                                                     as node_fn_asso_value
    , 'w_vs_seg'                                                       as node_desc
    union all                                                          
    select                                                             
      856                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        v_seg_cnt
      -- -- , (v_gram_n * i_token_embedding_len) + 1
      -- , (
      --     select sum((i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]) 
      --     from generate_series(1, v_gram_n) tb_a(a_no)
      --   ) + 1
      , i_seg_embedding_len
      , i_qk_width
      ]                                                                as node_fn_asso_value
    , 'embedding_seg |**| w_ks_seg'                                      as node_desc
    union all                                                          
    select                                                             
      857                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        v_seg_cnt
      -- -- , (v_gram_n * i_token_embedding_len) + 1
      -- , (
      --     select sum((i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]) 
      --     from generate_series(1, v_gram_n) tb_a(a_no)
      --   ) + 1
      , i_seg_embedding_len
      , i_v_width
      ]                                                                as node_fn_asso_value
    , 'embedding_seg |**| w_vs_seg'                                      as node_desc
    union all                                                          
    select                                                             
      858                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_concat_x3'                                           as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'k aggr_mx_concat_x3'                                            as node_desc
    union all                                                          
    select                                                             
      859                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_concat_x3'                                           as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'v aggr_mx_concat_x3'                                            as node_desc
    
    union all                                                          
    -- 不采用下三角矩阵算子，而采用元素加 masked 矩阵的方式；后者可以灵活的调整 masked 策略
    select                                                             
      900                                                              as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array
      [
        1
      , v_seg_cnt
      , i_seg_len
      , (i_seg_len * array_length(i_ngram, 1)) 
        - (|@+| i_ngram) 
        + array_length(i_ngram, 1) 
        + v_seg_cnt
      ]                                                                as node_fn_asso_value
    , 'masked_arr'                                                     as node_desc
    union all                                                          
    select                                                             
      1000                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '+ sequence_mask'                                                as node_desc
    union all                                                          
    select                                                             
      1100                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , array[1]                                                         as node_fn_asso_value   -- array[1, i_seq_len, i_seq_len - a_no + 1]
    , 'const(qk_width ^ 0.5)'                                          as node_desc
    union all                                                          
    select                                                             
      1200                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_div'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , '/` (qk_width ^ 0.5)'                                            as node_desc
    union all                                                          
    select                                                             
      1300                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_softmax'                                                     as node_fn_type
    , array
      [
        1
      , 1
      , 1
      , (i_seg_len * array_length(i_ngram, 1)) 
        - (|@+| i_ngram) 
        + array_length(i_ngram, 1) 
        + v_seg_cnt
      ]                                                                as node_fn_asso_value
    , 'softmax'                                                        as node_desc
    union all                                                          
    select                                                             
      1400                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        i_seg_len
      , (i_seg_len * array_length(i_ngram, 1))
        - (|@+| i_ngram) 
        + array_length(i_ngram, 1)
        + v_seg_cnt
      , i_v_width
      ]                                                                as node_fn_asso_value
    , '|**| v'                                                         as node_desc
    
    -- ngram embedding
    union all                                                          
    select                                                             
      1800 + i_ngram[a_no]                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '05_pool_none'                                                   as node_fn_type
    , array[null, i_ngram[a_no], i_token_embedding_len, 1, 1, 0, 0, 0, 0, 0]    as node_fn_asso_value
    , 'ngram 05_pool_none'                                             as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    where i_ngram[a_no] > 1
    union all                                                          
    select                                                             
      1900 + i_ngram[a_no]                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array[v_seg_cnt, i_seg_len - i_ngram[a_no] + 1, i_ngram[a_no]]   as node_fn_asso_value
    , 'ngram 01_chunk_prod_mx weight 1st'                              as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    where i_ngram[a_no] > 1
    union all                                                          
    select                                                             
      2000 + i_ngram[a_no]                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_chunk_prod_mx'                                               as node_fn_type
    , array[1, i_ngram[a_no], i_token_embedding_len]                   as node_fn_asso_value
    , 'ngram 01_chunk_prod_mx'                                         as node_desc
    from generate_series(1, v_gram_n) as tb_a(a_no)
    where i_ngram[a_no] > 1
    
    union all                                                          
    select                                                             
      2100                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array
      [
        1
      , (
          select sum((i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]) 
          from generate_series(1, v_gram_n) tb_a(a_no)
        ) + 1
      , i_seg_embedding_len
      ]                                                                as node_fn_asso_value
    , 'seg v 01_prod_mx'                                               as node_desc
    union all
    select 
      2200                                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , 'weight'                                                         as node_type
    , '00_const'                                                       as node_fn_type
    , array
      [
        v_seg_cnt
      , (
          select sum((i_seg_len - i_ngram[a_no] + 1) * i_ngram[a_no]) 
          from generate_series(1, v_gram_n) tb_a(a_no)
        ) + 1
      , i_seg_embedding_len
      ] :: float[]                                                     as node_fn_asso_value
    , 'w_seg_v'                                                        as node_desc
  ),
  -- 准备 path
  cte_paths as 
  (
    -- embedding_seg
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 834              -- embedding_seg none
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
      and tb_a_fore_node.a_tmp_no % 100 = 1
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 834              -- embedding_seg none
      and tb_a_back_node.a_tmp_no / 100 = 18               -- ngram 05_pool_none
      and tb_a_fore_node.a_tmp_no % 100 > 1
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
      
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 835              -- embedding_seg transpose
      and tb_a_back_node.a_tmp_no / 100 = 834              -- embedding_seg none
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 837              -- embedding_seg prod_mx k
      and tb_a_back_node.a_tmp_no / 100 = 836              -- embedding_seg prod_mx weight k
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 837              -- embedding_seg prod_mx k
      and tb_a_back_node.a_tmp_no / 100 = 835              -- embedding_seg transpose
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 839              -- embedding_seg prod_mx q
      and tb_a_back_node.a_tmp_no / 100 = 838              -- embedding_seg prod_mx weight q
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 839              -- embedding_seg prod_mx q
      and tb_a_back_node.a_tmp_no / 100 = 834              -- embedding_seg none
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 840              -- embedding_seg prod_mx qk
      and tb_a_back_node.a_tmp_no / 100 = 839              -- embedding_seg prod_mx q
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 840              -- embedding_seg prod_mx qk
      and tb_a_back_node.a_tmp_no / 100 = 837              -- embedding_seg prod_mx k
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 842              -- / i_token_embedding_len ^ 0.5
      and tb_a_back_node.a_tmp_no / 100 = 840              -- embedding_seg prod_mx qk
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 842              -- / i_token_embedding_len ^ 0.5
      and tb_a_back_node.a_tmp_no / 100 = 841              -- i_token_embedding_len ^ 0.5
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
      
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , tb_a_back_node.a_tmp_no % 100                  as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 851                    -- embedding_seg concat_x4
      and tb_a_back_node.a_tmp_no / 100 = 842              -- embedding_seg prod_mx ngram
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 853                    -- embedding_seg flatten transpose
      and tb_a_back_node.a_tmp_no = 2100                   -- seg v 01_prod_mx
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 2100                   -- seg v 01_prod_mx
      and tb_a_back_node.a_tmp_no = 2200                   -- w_seg_v
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 2100                   -- seg v 01_prod_mx
      and tb_a_back_node.a_tmp_no = 851                    -- embedding_seg concat_x4
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , v_gram_n + 1                                   as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 851                    -- embedding_seg concat_x4
      and tb_a_back_node.a_tmp_no = 852                    -- seg position encode

    -- seg ngram embedding
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 18                -- ngram 05_pool_none
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 20               -- ngram 01_chunk_prod_mx
      and tb_a_back_node.a_tmp_no / 100 = 18               -- ngram 05_pool_none
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 20               -- ngram 01_chunk_prod_mx
      and tb_a_back_node.a_tmp_no / 100 = 19               -- ngram 01_chunk_prod_mx weight 1st
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100    
    
    union all
    -- q embedding |**| weight
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 400 + 1                -- |**| w_q  path 1
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
    union all
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 400 + 1                -- |**| w_q  path 2
      and tb_a_back_node.a_tmp_no = 100 + 1                -- w_q
    union all 
    -- k token embedding |**| weight   gram 1
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 5                -- |**| w_k_1gram
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
      and i_ngram[tb_a_fore_node.a_tmp_no % 100] = 1  -- -- and tb_a_fore_node.a_tmp_no % 100 = 1
    union all 
    -- k token embedding |**| weight   gram 1  path n > 1
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 5                -- |**| w_k_ngram n >= 2
      and tb_a_back_node.a_tmp_no / 100 = 20                -- ngram 01_chunk_prod_mx
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
      -- and tb_a_fore_node.a_tmp_no % 100 > 1
    union all 
    -- v token embedding |**| weight   gram 1
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 6                -- |**| w_v_1gram
      and tb_a_back_node.a_tmp_no = 000 + 1                -- qkv_input
      and i_ngram[tb_a_fore_node.a_tmp_no % 100] = 1  -- -- and tb_a_fore_node.a_tmp_no % 100 = 1
    union all 
    -- v token embedding |**| weight   gram 1  path n > 1
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 6                -- |**| w_v_ngram n >= 2
      and tb_a_back_node.a_tmp_no / 100 = 20                -- ngram embedding_kv_
      and tb_a_back_node.a_tmp_no % 100 = tb_a_fore_node.a_tmp_no % 100
      -- and tb_a_fore_node.a_tmp_no % 100 > 1
    union all
    -- k token embedding |**| weight   path 2
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 5                -- |**| w_k_ngram
      and tb_a_back_node.a_tmp_no / 100 = 2                -- w_k_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100 % 10
    union all 
    -- v token embedding |**| weight   path 2
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no / 100 = 6                -- |**| w_v_ngram
      and tb_a_back_node.a_tmp_no / 100 = 3                -- w_v_ngram
      and tb_a_fore_node.a_tmp_no % 100 = tb_a_back_node.a_tmp_no % 100 % 10
    union all 
    
    -- k seg embedding |**| weight    gram 1
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 856                -- embedding_seg |**| w_ks_seg
      and tb_a_back_node.a_tmp_no = 853                -- embedding_seg concat_x4 position encode
    union all
    -- k seg embedding |**| weight    gram 2
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 856                -- embedding_seg |**| w_ks_seg
      and tb_a_back_node.a_tmp_no = 854                -- w_ks_seg
    union all
    
    -- v seg embedding |**| weight    gram 1
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 857                -- embedding_seg |**| w_vs_seg
      and tb_a_back_node.a_tmp_no = 853                -- embedding_seg concat_x4 position encode
    union all
    
    -- v seg embedding |**| weight    gram 2
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 857                -- embedding_seg |**| w_vs_seg
      and tb_a_back_node.a_tmp_no = 855                -- w_vs_seg
      
    union all
    -- k part of token embedding
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , tb_a_back_node.a_tmp_no % 100                  as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 858                -- k aggr_mx_concat_x3
      and tb_a_back_node.a_tmp_no / 100 = 5            -- |**| w_k_
    union all
    -- k part of concat_x3 seg embedding
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , v_gram_n + 1                                   as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 858                -- k aggr_mx_concat_x3
      and tb_a_back_node.a_tmp_no = 856                -- embedding_seg |**| w_ks_seg
      
    union all
    -- v part of token embedding
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , tb_a_back_node.a_tmp_no % 100                  as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 859                -- k aggr_mx_concat_x3
      and tb_a_back_node.a_tmp_no / 100 = 6            -- |**| w_v_
    union all
    -- v part of concat_x3 seg embedding
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , v_gram_n + 1                                   as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 859                -- v aggr_mx_concat_x3
      and tb_a_back_node.a_tmp_no = 857                -- embedding_seg |**| w_vs_seg
    
    -- transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 700                -- transpose(k)
      and tb_a_back_node.a_tmp_no = 858                -- k aggr_mx_concat_x3
    
    -- q |**| transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 800                -- q |**| transpose(k)
      and tb_a_back_node.a_tmp_no = 400 + 1            -- |**| w_q
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 800                -- q |**| transpose(k)
      and tb_a_back_node.a_tmp_no = 700                -- transpose(k)
    
    -- + sequence_mask
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1000               -- + sequence_mask
      and tb_a_back_node.a_tmp_no = 800                -- q |**| transpose(k)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1000               -- + sequence_mask
      and tb_a_back_node.a_tmp_no = 900                -- masked_arr
    
    -- /` (qk_width ^ 0.5)
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1200               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no = 1000               --+ sequence_mask
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1200               -- /` (qk_width ^ 0.5)
      and tb_a_back_node.a_tmp_no = 1100               -- const(qk_width ^ 0.5)
    
    -- softmax
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1300               -- softmax
      and tb_a_back_node.a_tmp_no = 1200               -- /` (qk_width ^ 0.5)
    
    -- |**| v
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1400               -- |**| v
      and tb_a_back_node.a_tmp_no = 1300               -- softmax
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 1400               -- |**| v
      and tb_a_back_node.a_tmp_no = 859                -- v aggr_mx_concat_x3
    
    

    -- v aggr_mx_concat_x3
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- need token_embedding input as p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 000 + 1
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 1400
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_seg4d_symm_ngram_pos_1head
--   (
--     40                                -- i_seq_len      
--   , 10                                -- i_seg_len         
--   , 24                                -- i_token_embedding_len
--   , 64                                -- i_qk_width           
-- --   , 64                                -- i_kv_heigh           
--   , 64                                -- i_v_width              
--   , array[1, 2, 3, 4]                 -- i_ngram                  
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_seg4d_symm_ngram_pos_1head 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 3240,
--             "node_desc": "qkv_input",
--             "node_fn_type": "03_zscore",
--             "node_fn_asso_value": "{1,1,1,24}"
--           },
--           {
--             "node_no": 3241,
--             "node_desc": "w_q",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3242,
--             "node_desc": "w_k_1th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3243,
--             "node_desc": "w_k_2th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3244,
--             "node_desc": "w_k_3th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3245,
--             "node_desc": "w_k_4th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3246,
--             "node_desc": "w_v_1th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3247,
--             "node_desc": "w_v_2th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3248,
--             "node_desc": "w_v_3th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3249,
--             "node_desc": "w_v_4th_gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,64}"
--           },
--           {
--             "node_no": 3250,
--             "node_desc": "|**| w_q",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3251,
--             "node_desc": "|**| w_k_1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3252,
--             "node_desc": "|**| w_k_2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3253,
--             "node_desc": "|**| w_k_3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3254,
--             "node_desc": "|**| w_k_4gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3255,
--             "node_desc": "|**| w_v_1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3256,
--             "node_desc": "|**| w_v_2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3257,
--             "node_desc": "|**| w_v_3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3258,
--             "node_desc": "|**| w_v_4gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,24,64}"
--           },
--           {
--             "node_no": 3259,
--             "node_desc": "transpose(k)                   ",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 3260,
--             "node_desc": "q |**| transpose(k)",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,38}"
--           },
--           {
--             "node_no": 3297,
--             "node_desc": "embedding_seg concat_x4 position encode",
--             "node_fn_type": "06_aggr_mx_concat_x4",
--             "node_fn_asso_value": "{10,18,24,28,1}"
--           },
--           {
--             "node_no": 3298,
--             "node_desc": "seg position encode",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1,4,1,1}"
--           },
--           {
--             "node_no": 3299,
--             "node_desc": "embedding_seg flatten transpose",
--             "node_fn_type": "04_transpose_nd",
--             "node_fn_asso_value": "{2,3}"
--           },
--           {
--             "node_no": 3300,
--             "node_desc": "w_ks_seg",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{96,64}"
--           },
--           {
--             "node_no": 3301,
--             "node_desc": "w_vs_seg",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{96,64}"
--           },
--           {
--             "node_no": 3302,
--             "node_desc": "embedding_seg |**| w_ks_seg",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{4,96,64}"
--           },
--           {
--             "node_no": 3303,
--             "node_desc": "embedding_seg |**| w_vs_seg",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{4,96,64}"
--           },
--           {
--             "node_no": 3304,
--             "node_desc": "k aggr_mx_concat_x3",
--             "node_fn_type": "06_aggr_mx_concat_x3"
--           },
--           {
--             "node_no": 3305,
--             "node_desc": "v aggr_mx_concat_x3",
--             "node_fn_type": "06_aggr_mx_concat_x3"
--           },
--           {
--             "node_no": 3306,
--             "node_desc": "masked_arr",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1,4,10,38}"
--           },
--           {
--             "node_no": 3307,
--             "node_desc": "+ sequence_mask",
--             "node_fn_type": "01_add"
--           },
--           {
--             "node_no": 3308,
--             "node_desc": "const(qk_width ^ 0.5)",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 3309,
--             "node_desc": "/` (qk_width ^ 0.5)",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 3310,
--             "node_desc": "softmax",
--             "node_fn_type": "03_softmax",
--             "node_fn_asso_value": "{1,1,1,38}"
--           },
--           {
--             "node_no": 3311,
--             "node_desc": "|**| v",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{10,38,64}"
--           },
--           {
--             "node_no": 3312,
--             "node_desc": "ngram 05_pool_none",
--             "node_fn_type": "05_pool_none",
--             "node_fn_asso_value": "{NULL,2,24,1,1,0,0,0,0,0}"
--           },
--           {
--             "node_no": 3313,
--             "node_desc": "ngram 05_pool_none",
--             "node_fn_type": "05_pool_none",
--             "node_fn_asso_value": "{NULL,3,24,1,1,0,0,0,0,0}"
--           },
--           {
--             "node_no": 3314,
--             "node_desc": "ngram 05_pool_none",
--             "node_fn_type": "05_pool_none",
--             "node_fn_asso_value": "{NULL,4,24,1,1,0,0,0,0,0}"
--           },
--           {
--             "node_no": 3315,
--             "node_desc": "ngram 01_chunk_prod_mx weight 1st",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,9,2}"
--           },
--           {
--             "node_no": 3316,
--             "node_desc": "ngram 01_chunk_prod_mx weight 1st",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,8,3}"
--           },
--           {
--             "node_no": 3317,
--             "node_desc": "ngram 01_chunk_prod_mx weight 1st",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,7,4}"
--           },
--           {
--             "node_no": 3318,
--             "node_desc": "ngram 01_chunk_prod_mx",
--             "node_fn_type": "01_chunk_prod_mx",
--             "node_fn_asso_value": "{1,2,24}"
--           },
--           {
--             "node_no": 3319,
--             "node_desc": "ngram 01_chunk_prod_mx",
--             "node_fn_type": "01_chunk_prod_mx",
--             "node_fn_asso_value": "{1,3,24}"
--           },
--           {
--             "node_no": 3320,
--             "node_desc": "ngram 01_chunk_prod_mx",
--             "node_fn_type": "01_chunk_prod_mx",
--             "node_fn_asso_value": "{1,4,24}"
--           },
--           {
--             "node_no": 3321,
--             "node_desc": "seg v 01_prod_mx",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,81,96}"
--           },
--           {
--             "node_no": 3322,
--             "node_desc": "w_seg_vs",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,81,96}"
--           },
--           {
--             "node_no": 3261,
--             "node_desc": "embedding_seg none 1gram",
--             "node_fn_type": "00_none"
--           },
--           {
--             "node_no": 3262,
--             "node_desc": "embedding_seg none 2gram",
--             "node_fn_type": "00_none"
--           },
--           {
--             "node_no": 3263,
--             "node_desc": "embedding_seg none 3gram",
--             "node_fn_type": "00_none"
--           },
--           {
--             "node_no": 3264,
--             "node_desc": "embedding_seg none 4gram",
--             "node_fn_type": "00_none"
--           },
--           {
--             "node_no": 3265,
--             "node_desc": "embedding_seg transpose 1gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 3266,
--             "node_desc": "embedding_seg transpose 2gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 3267,
--             "node_desc": "embedding_seg transpose 3gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 3268,
--             "node_desc": "embedding_seg transpose 4gram",
--             "node_fn_type": "04_transpose"
--           },
--           {
--             "node_no": 3269,
--             "node_desc": "embedding_seg prod_mx weight k 1gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,24}"
--           },
--           {
--             "node_no": 3270,
--             "node_desc": "embedding_seg prod_mx weight k 2gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,24}"
--           },
--           {
--             "node_no": 3271,
--             "node_desc": "embedding_seg prod_mx weight k 3gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,24}"
--           },
--           {
--             "node_no": 3272,
--             "node_desc": "embedding_seg prod_mx weight k 4gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,24,24}"
--           },
--           {
--             "node_no": 3273,
--             "node_desc": "embedding_seg prod_mx k 1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{24,24,10}"
--           },
--           {
--             "node_no": 3274,
--             "node_desc": "embedding_seg prod_mx k 2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{24,24,18}"
--           },
--           {
--             "node_no": 3275,
--             "node_desc": "embedding_seg prod_mx k 3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{24,24,24}"
--           },
--           {
--             "node_no": 3276,
--             "node_desc": "embedding_seg prod_mx k 4gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{24,24,28}"
--           },
--           {
--             "node_no": 3277,
--             "node_desc": "embedding_seg prod_mx weight q 1gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,1,10}"
--           },
--           {
--             "node_no": 3278,
--             "node_desc": "embedding_seg prod_mx weight q 2gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,1,18}"
--           },
--           {
--             "node_no": 3279,
--             "node_desc": "embedding_seg prod_mx weight q 3gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,1,24}"
--           },
--           {
--             "node_no": 3280,
--             "node_desc": "embedding_seg prod_mx weight q 4gram",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{4,1,28}"
--           },
--           {
--             "node_no": 3281,
--             "node_desc": "embedding_seg prod_mx q 1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,10,24}"
--           },
--           {
--             "node_no": 3282,
--             "node_desc": "embedding_seg prod_mx q 2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,18,24}"
--           },
--           {
--             "node_no": 3283,
--             "node_desc": "embedding_seg prod_mx q 3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,24,24}"
--           },
--           {
--             "node_no": 3284,
--             "node_desc": "embedding_seg prod_mx q 4gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,28,24}"
--           },
--           {
--             "node_no": 3285,
--             "node_desc": "embedding_seg prod_mx qk 1gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,24,10}"
--           },
--           {
--             "node_no": 3286,
--             "node_desc": "embedding_seg prod_mx qk 2gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,24,18}"
--           },
--           {
--             "node_no": 3287,
--             "node_desc": "embedding_seg prod_mx qk 3gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,24,24}"
--           },
--           {
--             "node_no": 3288,
--             "node_desc": "embedding_seg prod_mx qk 4gram",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{1,24,28}"
--           },
--           {
--             "node_no": 3289,
--             "node_desc": "ngram_seg_len ^ 0.5 1gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 3290,
--             "node_desc": "ngram_seg_len ^ 0.5 2gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 3291,
--             "node_desc": "ngram_seg_len ^ 0.5 3gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 3292,
--             "node_desc": "ngram_seg_len ^ 0.5 4gram",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1}"
--           },
--           {
--             "node_no": 3293,
--             "node_desc": "/ ngram_seg_len ^ 0.5 1gram",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 3294,
--             "node_desc": "/ ngram_seg_len ^ 0.5 2gram",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 3295,
--             "node_desc": "/ ngram_seg_len ^ 0.5 3gram",
--             "node_fn_type": "01_div"
--           },
--           {
--             "node_no": 3296,
--             "node_desc": "/ ngram_seg_len ^ 0.5 4gram",
--             "node_fn_type": "01_div"
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3250
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3241,
--             "fore_node_no": 3250
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3251
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3242,
--             "fore_node_no": 3251
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3318,
--             "fore_node_no": 3252
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3243,
--             "fore_node_no": 3252
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3319,
--             "fore_node_no": 3253
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3244,
--             "fore_node_no": 3253
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3320,
--             "fore_node_no": 3254
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3245,
--             "fore_node_no": 3254
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3255
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3246,
--             "fore_node_no": 3255
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3318,
--             "fore_node_no": 3256
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3247,
--             "fore_node_no": 3256
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3319,
--             "fore_node_no": 3257
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3248,
--             "fore_node_no": 3257
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3320,
--             "fore_node_no": 3258
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3249,
--             "fore_node_no": 3258
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3304,
--             "fore_node_no": 3259
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3250,
--             "fore_node_no": 3260
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3259,
--             "fore_node_no": 3260
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3261
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3312,
--             "fore_node_no": 3262
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3313,
--             "fore_node_no": 3263
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3314,
--             "fore_node_no": 3264
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3261,
--             "fore_node_no": 3265
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3262,
--             "fore_node_no": 3266
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3263,
--             "fore_node_no": 3267
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3264,
--             "fore_node_no": 3268
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3269,
--             "fore_node_no": 3273
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3265,
--             "fore_node_no": 3273
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3270,
--             "fore_node_no": 3274
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3266,
--             "fore_node_no": 3274
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3271,
--             "fore_node_no": 3275
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3267,
--             "fore_node_no": 3275
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3272,
--             "fore_node_no": 3276
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3268,
--             "fore_node_no": 3276
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3277,
--             "fore_node_no": 3281
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3261,
--             "fore_node_no": 3281
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3278,
--             "fore_node_no": 3282
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3262,
--             "fore_node_no": 3282
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3279,
--             "fore_node_no": 3283
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3263,
--             "fore_node_no": 3283
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3280,
--             "fore_node_no": 3284
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3264,
--             "fore_node_no": 3284
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3281,
--             "fore_node_no": 3285
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3273,
--             "fore_node_no": 3285
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3282,
--             "fore_node_no": 3286
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3274,
--             "fore_node_no": 3286
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3283,
--             "fore_node_no": 3287
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3275,
--             "fore_node_no": 3287
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3284,
--             "fore_node_no": 3288
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3276,
--             "fore_node_no": 3288
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3285,
--             "fore_node_no": 3293
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3289,
--             "fore_node_no": 3293
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3286,
--             "fore_node_no": 3294
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3290,
--             "fore_node_no": 3294
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3287,
--             "fore_node_no": 3295
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3291,
--             "fore_node_no": 3295
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3288,
--             "fore_node_no": 3296
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3292,
--             "fore_node_no": 3296
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3293,
--             "fore_node_no": 3297
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3294,
--             "fore_node_no": 3297
--           },
--           {
--             "path_ord_no": 3,
--             "back_node_no": 3295,
--             "fore_node_no": 3297
--           },
--           {
--             "path_ord_no": 4,
--             "back_node_no": 3296,
--             "fore_node_no": 3297
--           },
--           {
--             "path_ord_no": 5,
--             "back_node_no": 3298,
--             "fore_node_no": 3297
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3321,
--             "fore_node_no": 3299
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3299,
--             "fore_node_no": 3302
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3300,
--             "fore_node_no": 3302
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3299,
--             "fore_node_no": 3303
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3301,
--             "fore_node_no": 3303
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3251,
--             "fore_node_no": 3304
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3252,
--             "fore_node_no": 3304
--           },
--           {
--             "path_ord_no": 3,
--             "back_node_no": 3253,
--             "fore_node_no": 3304
--           },
--           {
--             "path_ord_no": 4,
--             "back_node_no": 3254,
--             "fore_node_no": 3304
--           },
--           {
--             "path_ord_no": 5,
--             "back_node_no": 3302,
--             "fore_node_no": 3304
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3255,
--             "fore_node_no": 3305
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3256,
--             "fore_node_no": 3305
--           },
--           {
--             "path_ord_no": 3,
--             "back_node_no": 3257,
--             "fore_node_no": 3305
--           },
--           {
--             "path_ord_no": 4,
--             "back_node_no": 3258,
--             "fore_node_no": 3305
--           },
--           {
--             "path_ord_no": 5,
--             "back_node_no": 3303,
--             "fore_node_no": 3305
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3260,
--             "fore_node_no": 3307
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3306,
--             "fore_node_no": 3307
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3307,
--             "fore_node_no": 3309
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3308,
--             "fore_node_no": 3309
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3309,
--             "fore_node_no": 3310
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3310,
--             "fore_node_no": 3311
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3305,
--             "fore_node_no": 3311
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3312
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3313
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3240,
--             "fore_node_no": 3314
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3315,
--             "fore_node_no": 3318
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3312,
--             "fore_node_no": 3318
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3316,
--             "fore_node_no": 3319
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3313,
--             "fore_node_no": 3319
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3317,
--             "fore_node_no": 3320
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3314,
--             "fore_node_no": 3320
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 3297,
--             "fore_node_no": 3321
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 3322,
--             "fore_node_no": 3321
--           }
--         ],
--         "combine_no": 3323,
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 3240,
--             "combine_in_no": 1
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 3311
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_fn_asso_value, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes
drop function if exists sm_sc.ufv_combine_att_tar_fn(int, int, int, int);
create or replace function sm_sc.ufv_combine_att_tar_fn
(
  i_z_heigh              int                       -- 通常对应 i_seq_len
, i_z_width              int                       -- 通常对应 i_w_ff1_width_ff2_heigh 和 i_token_embedding_len
, i_token_dic_len        int                       -- 词表中 token 的总数量
, i_array_ndims          int    default 3          -- 张量维度数，3 或 4
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select                                           -- need token_embedding input as p1
      10 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_const'                                                       as node_fn_type
    , case i_array_ndims 
        when 3
          then array[1, i_z_width, i_token_dic_len] 
        when 4
          then array[1, 1, i_z_width, i_token_dic_len] 
      end                                                              as node_fn_asso_value
    , 'tar_fn token_dic_embedding'                                     as node_desc       
    union all                                                          
    select                                                             
      20 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_prod_mx'                                                     as node_fn_type
    , array[i_z_heigh, i_z_width, i_token_dic_len]                     as node_fn_asso_value
    , 'tar_fn prod_mx'                                                 as node_desc
    union all                                                          
    select                                                             
      30 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '01_add'                                                         as node_fn_type
    , null                                                             as node_fn_asso_value
    , 'tar_fn add padding'                                             as node_desc
    union all                                                          
    select                                                             
      40 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '03_softmax'                                                     as node_fn_type
    , case i_array_ndims
        when 3
          then array[1, 1, i_token_dic_len]                          
        when 4
          then array[1, 1, 1, i_token_dic_len]                                   
      end                                                              as node_fn_asso_value
    , 'tar_fn softmax'                                                 as node_desc   
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 2                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 20               -- tar_fn prod_mx
      and tb_a_back_node.a_tmp_no = 10               -- tar_fn token_dic_embedding
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 30               -- tar_fn add padding
      and tb_a_back_node.a_tmp_no = 20               -- tar_fn prod_mx
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 40               -- tar_fn softmax
      and tb_a_back_node.a_tmp_no = 30               -- tar_fn add padding
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select                                           -- trs decoder as input p1
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 20
    union all
    select                                           -- trs decoder as input p1
      node_no                                        as fore_node_no
    , 2                                              as path_ord_no
    , 2                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 30
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = 40
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_att_tar_fn
--   (
--     40
--   , 64
--   , 25000
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_att_tar_fn 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 4916,
--             "node_desc": "token_dic_embedding",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{1,64,25000}"
--           },
--           {
--             "node_no": 4917,
--             "node_desc": "attention |**| (|^~| token_dic_embedding)",
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{40,64,25000}"
--           },
--           {
--             "node_no": 4918,
--             "node_desc": "seq token probability on each dic embedding",
--             "node_fn_type": "03_softmax",
--             "node_fn_asso_value": "{1,1,25000}"
--           },
--           {
--             "node_no": 4919,
--             "node_desc": "get seq token probability on true token by idx",
--             "node_fn_type": "81_query_from_row"
--           },
--           {
--             "node_no": 4920,
--             "node_desc": "^!` (u[1] ... u[i])",
--             "node_fn_type": "01_ln"
--           },
--           {
--             "node_no": 4921,
--             "node_desc": "sum(^!` (u[1] ... u[i]))",
--             "node_fn_type": "07_aggr_slice_sum"
--           },
--           {
--             "node_no": 4922,
--             "node_desc": "-` sum(^!` (u[1] ... u[i]))",
--             "node_fn_type": "01_0sub"
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 2,
--             "back_node_no": 4916,
--             "fore_node_no": 4917
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 4917,
--             "fore_node_no": 4918
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 4918,
--             "fore_node_no": 4919
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 4919,
--             "fore_node_no": 4920
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 4920,
--             "fore_node_no": 4921
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 4921,
--             "fore_node_no": 4922
--           }
--         ],
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 4917,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 2,
--             "fore_node_no": 4919,
--             "combine_in_no": 2
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 4922
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes

drop function if exists sm_sc.ufv_combine_ngram_4_att_kv(int, int, int);
create or replace function sm_sc.ufv_combine_ngram_4_att_kv
(
  i_seq_len                             int
, i_token_embedding_len                 int
, i_ngram_n                             int
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select                                           -- need token_embedding input as p1
      1 :: int                                                        as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '00_none'                                                        as node_fn_type
    , array
      [
        null :: float
      , i_ngram_n
      , i_token_embedding_len
      , 1
      , i_token_embedding_len
      , 0, 0, 0, 0
      , null
      ]                                                                as node_fn_asso_value
    , 'none'                                                           as node_desc       
    union all                                                          
    select                                                             
      a_ngram_no + 1 :: int                                           as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_sample_x'                                                    as node_fn_type
    , array
      [
        [i_ngram_n]
      , [null :: int]
      , [a_ngram_no]
      , [a_ngram_no]
      ]                                                                as node_fn_asso_value
    , 'sample_x'                                                       as node_desc
    from generate_series(1, i_ngram_n) tb_a_ngram_no(a_ngram_no)
    union all                                                          
    select                                                             
      i_ngram_n + 2 :: int                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_concat_x3'                                           as node_fn_type
    , array_fill
      (
        i_token_embedding_len
      , array[i_ngram_n]
      )                                                                as node_fn_asso_value
    , 'concat_x3'                                                      as node_desc
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from generate_series(1, i_ngram_n) tb_a_ngram_no(a_ngram_no)
      , cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = a_ngram_no + 1  -- sample_x
      and tb_a_back_node.a_tmp_no = 1               -- pool_none
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , a_ngram_no                                     as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from generate_series(1, i_ngram_n) tb_a_ngram_no(a_ngram_no)
      , cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = i_ngram_n + 2   -- concat_x3
      and tb_a_back_node.a_tmp_no = a_ngram_no + 1  -- sample_x
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 1
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = i_ngram_n + 2
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_ngram_4_att_kv
--   (
--     40
--   , 64
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_ngram_4_att_kv 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 82,
--             "node_desc": "+` token_embedding as res",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 83,
--             "node_desc": "layer_normalize",
--             "node_type": null,
--             "node_fn_type": "03_zscore",
--             "node_fn_asso_value": "{1,1,40}"
--           },
--           {
--             "node_no": 84,
--             "node_desc": "w1 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,64}"
--           },
--           {
--             "node_no": 85,
--             "node_desc": "|**| w1 of ffn",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{NULL,40,64}"
--           },
--           {
--             "node_no": 86,
--             "node_desc": "leaky_relu of ffn",
--             "node_type": null,
--             "node_fn_type": "03_leaky_relu",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 87,
--             "node_desc": "w2 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{64,40}"
--           },
--           {
--             "node_no": 88,
--             "node_desc": "|**| w2 of ffn",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{NULL,64,40}"
--           },
--           {
--             "node_no": 89,
--             "node_desc": "+` attention as res",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 1,
--             "back_node_no": 82,
--             "fore_node_no": 83
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 83,
--             "fore_node_no": 85
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 84,
--             "fore_node_no": 85
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 85,
--             "fore_node_no": 86
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 86,
--             "fore_node_no": 88
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 87,
--             "fore_node_no": 88
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 83,
--             "fore_node_no": 89
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 88,
--             "fore_node_no": 89
--           }
--         ],
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 82,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 2,
--             "fore_node_no": 82,
--             "combine_in_no": 2
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 89
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes

drop function if exists sm_sc.ufv_combine_ngram_4_att_kv_bi_seq(int, int, int);
create or replace function sm_sc.ufv_combine_ngram_4_att_kv_bi_seq
(
  i_seq_len                             int     -- 要求为偶数，前后缀长度相同
, i_token_embedding_len                 int
, i_ngram_n                             int
)
returns jsonb
-- 复合算子的构建依赖于 sm_sc.__vt_global_seq 做为全局序列的稳定性
as 
$$
declare 
  v_j_nodes            jsonb   ;
  v_j_paths            jsonb   ;
  v_j_input_paths      jsonb   ;
  v_j_output_nodes     jsonb   ;
begin
  -- 准备 node
  with
  cte_nodes as 
  (
    select                                           -- need token_embedding input as p1
      1                                                                as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_slice_x'                                                     as node_fn_type
    , array
      [
        1
      , i_seq_len / 2
      ]                                                                as node_fn_asso_value
    , 'slice_x'                                                        as node_desc       
    union all                                                          
    select                                           -- need token_embedding input as p1
      2                                                                as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_slice_x'                                                     as node_fn_type
    , array
      [
        i_seq_len / 2 + 1
      , i_seq_len
      ]                                                                as node_fn_asso_value
    , 'slice_x'                                                        as node_desc       
    union all                                                          
    select                                           -- need token_embedding input as p1
      3                                                                as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '05_pool_none'                                                   as node_fn_type
    , array
      [
        null :: float
      , i_ngram_n
      , i_token_embedding_len
      , 1
      , i_token_embedding_len
      , 0, 0, 0, 0
      , null
      ]                                                                as node_fn_asso_value
    , 'pool_none'                                                      as node_desc       
    union all                                                          
    select                                           -- need token_embedding input as p1
      4                                                                as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '05_pool_none'                                                   as node_fn_type
    , array
      [
        null :: float
      , i_ngram_n
      , i_token_embedding_len
      , 1
      , i_token_embedding_len
      , 0, 0, 0, 0
      , null
      ]                                                                as node_fn_asso_value
    , 'pool_none'                                                      as node_desc     
    union all                                                          
    select                                                             
      5                                                                as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_concat_x'                                            as node_fn_type
    , array_fill 
      (
        (i_seq_len / 2 - i_ngram_n + 1) * i_ngram_n
      , array[2]
      )                                                                as node_fn_asso_value
    , 'concat_x'                                                       as node_desc  
    union all                                                          
    select                                                             
      a_ngram_no + 5 :: int                                           as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '04_sample_x'                                                    as node_fn_type
    , array
      [
        [i_ngram_n]
      , [null :: int]
      , [a_ngram_no]
      , [a_ngram_no]
      ]                                                                as node_fn_asso_value
    , 'sample_x'                                                       as node_desc
    from generate_series(1, i_ngram_n) tb_a_ngram_no(a_ngram_no)
    union all                                                          
    select                                                             
      i_ngram_n + 6 :: int                                             as a_tmp_no
    , lower(sm_sc.fv_get_global_seq())                                 as node_no
    , null                                                             as node_type
    , '06_aggr_mx_concat_x3'                                           as node_fn_type
    , array_fill
      (
        i_token_embedding_len
      , array[i_ngram_n]
      )                                                                as node_fn_asso_value
    , 'concat_x3'                                                      as node_desc
  ),
  -- 准备 path
  cte_paths as 
  (
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 3               -- pool_none
      and tb_a_back_node.a_tmp_no = 1               -- slice_x
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 4               -- pool_none
      and tb_a_back_node.a_tmp_no = 2               -- slice_x
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 5               -- concat_x
      and tb_a_back_node.a_tmp_no = 3               -- pool_none
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = 5               -- concat_x
      and tb_a_back_node.a_tmp_no = 4               -- pool_none
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , 1                                              as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from generate_series(1, i_ngram_n) tb_a_ngram_no(a_ngram_no)
      , cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = a_ngram_no + 5  -- sample_x
      and tb_a_back_node.a_tmp_no = 5               -- pool_none
    union all 
    select 
      tb_a_fore_node.node_no                         as fore_node_no   
    , a_ngram_no                                     as path_ord_no
    , tb_a_back_node.node_no                         as back_node_no     
    from generate_series(1, i_ngram_n) tb_a_ngram_no(a_ngram_no)
      , cte_nodes tb_a_fore_node
      , cte_nodes tb_a_back_node
    where tb_a_fore_node.a_tmp_no = i_ngram_n + 6   -- concat_x3
      and tb_a_back_node.a_tmp_no = a_ngram_no + 5  -- sample_x
  ),
  -- 准备输入路径
  cte_input_paths as 
  (
    select
      node_no                                        as fore_node_no
    , 1                                              as path_ord_no
    , 1                                              as combine_in_no
    from cte_nodes
    where a_tmp_no = 1
  ),
  -- 准备输出节点
  cte_output_nodes as 
  (
    select 
      node_no
    , 1                                              as combine_out_no
    from cte_nodes
    where a_tmp_no = i_ngram_n + 6
  )
  -- 装入四个 json
  select 
    (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_strip_nulls
            (
              json_build_object
              (
                'node_no'             , node_no
              , 'node_type'           , node_type
              , 'node_fn_type'        , node_fn_type
              , 'node_fn_asso_value'  , node_fn_asso_value :: text
              , 'node_desc'           , node_desc
              )
            )
          ) :: jsonb
        order by a_tmp_no
        )
      from cte_nodes
    ) as a_j_nodes
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'back_node_no'        , back_node_no
            )
          ) :: jsonb
        order by fore_node_no, path_ord_no
        )
      from cte_paths
    ) as a_j_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'fore_node_no'        , fore_node_no
            , 'path_ord_no'         , path_ord_no
            , 'combine_in_no'       , combine_in_no
            )
          ) :: jsonb
        order by fore_node_no
        )
      from cte_input_paths
    ) as a_j_input_paths
  , (
      select 
        sm_sc.fa_concat
        (
          json_build_array
          (
            json_build_object
            (
              'output_node_no'      , node_no
            , 'combine_out_no'      , combine_out_no
            )
          ) :: jsonb
        order by node_no
        )
      from cte_output_nodes
    ) as a_j_output_nodes
  into 
    v_j_nodes
  , v_j_paths
  , v_j_input_paths
  , v_j_output_nodes
  ;

  return 
    jsonb_build_object
    (
      'combine_no'            , lower(sm_sc.fv_get_global_seq())
    , 'nodes'                 , v_j_nodes 
    , 'paths'                 , v_j_paths
    , 'input_paths'           , v_j_input_paths
    , 'output_nodes'          , v_j_output_nodes
    )
  ;
end
$$
language plpgsql volatile
parallel safe
;

-- select 
--   sm_sc.ufv_combine_ngram_4_att_kv_bi_seq
--   (
--     40
--   , 64
--   )

-- with 
-- cte_j_trs as 
-- (
--   select
--     -- 以下 jsonb 由调用 sm_sc.ufv_combine_ngram_4_att_kv_bi_seq 获得
--     '
--       {
--         "nodes": [
--           {
--             "node_no": 82,
--             "node_desc": "+` token_embedding as res",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 83,
--             "node_desc": "layer_normalize",
--             "node_type": null,
--             "node_fn_type": "03_zscore",
--             "node_fn_asso_value": "{1,1,40}"
--           },
--           {
--             "node_no": 84,
--             "node_desc": "w1 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{40,64}"
--           },
--           {
--             "node_no": 85,
--             "node_desc": "|**| w1 of ffn",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{NULL,40,64}"
--           },
--           {
--             "node_no": 86,
--             "node_desc": "leaky_relu of ffn",
--             "node_type": null,
--             "node_fn_type": "03_leaky_relu",
--             "node_fn_asso_value": null
--           },
--           {
--             "node_no": 87,
--             "node_desc": "w2 of ffn",
--             "node_type": "weight",
--             "node_fn_type": "00_const",
--             "node_fn_asso_value": "{64,40}"
--           },
--           {
--             "node_no": 88,
--             "node_desc": "|**| w2 of ffn",
--             "node_type": null,
--             "node_fn_type": "01_prod_mx",
--             "node_fn_asso_value": "{NULL,64,40}"
--           },
--           {
--             "node_no": 89,
--             "node_desc": "+` attention as res",
--             "node_type": null,
--             "node_fn_type": "01_add",
--             "node_fn_asso_value": null
--           }
--         ],
--         "paths": [
--           {
--             "path_ord_no": 1,
--             "back_node_no": 82,
--             "fore_node_no": 83
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 83,
--             "fore_node_no": 85
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 84,
--             "fore_node_no": 85
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 85,
--             "fore_node_no": 86
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 86,
--             "fore_node_no": 88
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 87,
--             "fore_node_no": 88
--           },
--           {
--             "path_ord_no": 1,
--             "back_node_no": 83,
--             "fore_node_no": 89
--           },
--           {
--             "path_ord_no": 2,
--             "back_node_no": 88,
--             "fore_node_no": 89
--           }
--         ],
--         "input_paths": [
--           {
--             "path_ord_no": 1,
--             "fore_node_no": 82,
--             "combine_in_no": 1
--           },
--           {
--             "path_ord_no": 2,
--             "fore_node_no": 82,
--             "combine_in_no": 2
--           }
--         ],
--         "output_nodes": [
--           {
--             "combine_out_no": 1,
--             "output_node_no": 89
--           }
--         ]
--       }
--     ' :: jsonb
--     as a_j_trs
-- ),
-- -- cte_nodes as
-- -- (
-- --   select                                
-- --     (a_j_node         ->> 'node_no'             )  :: int          as node_no
-- --   , (a_j_node         ->> 'node_type'           )                  as node_type
-- --   , (a_j_node         ->> 'node_fn_type'        )                  as node_fn_type
-- --   , (a_j_node         ->> 'node_fn_asso_value'  )  :: float[]      as node_fn_asso_value
-- --   , (a_j_node         ->> 'node_desc'           )                  as node_desc
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.nodes[*]') tb_a_nodes(a_j_node)
-- -- ) -- select node_no, node_type, node_fn_type, node_desc from cte_nodes
-- -- cte_paths as
-- -- (
-- --   select 
-- --     (a_j_path         ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_path         ->> 'back_node_no'        )  :: int          as back_node_no
-- --   , (a_j_path         ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.paths[*]') tb_a_path(a_j_path)
-- -- ) -- select path_ord_no, back_node_no, fore_node_no from cte_paths
-- -- cte_input_paths as
-- -- (
-- --   select 
-- --     (a_j_input_paths  ->> 'path_ord_no'         )  :: int          as path_ord_no
-- --   , (a_j_input_paths  ->> 'fore_node_no'        )  :: int          as fore_node_no
-- --   , (a_j_input_paths  ->> 'combine_in_no'       )  :: int          as combine_in_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.input_paths[*]') tb_a_input_paths(a_j_input_paths)
-- -- ) -- select path_ord_no, fore_node_no, combine_in_no from cte_input_paths
-- -- cte_output_nodes as
-- -- (
-- --   select 
-- --     (a_j_output_nodes ->> 'output_node_no'      )  :: int          as output_node_no
-- --   , (a_j_output_nodes ->> 'combine_out_no'      )  :: int          as combine_out_no
-- --   from cte_j_trs, jsonb_path_query(a_j_trs, '$.output_nodes[*]') tb_a_output_nodes(a_j_output_nodes)
-- -- ) -- select output_node_no, combine_out_no from cte_output_nodes
-- 参考 https://blog.csdn.net/zhq9695/article/details/82805097

-- 初始学习率：1e-4
-- https://www.zhihu.com/question/387050717

drop procedure if exists sm_sc.prc_linear_regression;
create or replace procedure sm_sc.prc_linear_regression
(
  i_work_no                          bigint                                   ,                                                                                                -- 训练任务编号，用于任务暂停和接续、数据回显
  -- i_indepdt                                float[][]                       ,      -- [v_len_x1_y1][v_len_x2_w1]                                                         -- 乘法左矩阵，其中自行填写 w0 = 1.0 规约为该入参首列
  -- i_depdt                                float[][]                       ,      -- [v_len_x1_y1][v_len_w2_y2]                                                         -- 乘法结果
  i_learn_rate                       float    default    0.0001        ,                                                                                                -- 学习率
  i_limit_train_times                int               default    10000       ,                                                                                                -- 最大训练次数，然后未完成中止
  -- i_grad_descent_batch_cnt           int               default    10000       ,   -- 建议大一些，远远超过 v_i_indepdt, v_i_depdt 的各维度长度，尽量防止因小批量梯度下降引起的鞍点训练终止 -- 小批量梯度下降，每批记录数
  i_loss_delta_least_stop_threshold  float    default    0.0001                                                                                                       -- 触发收敛的损失函数梯度阈值
)
as
$$
declare -- here
-- -- 数学原理：已知矩阵 v_i_indepdt, v_i_depdt, 且 v_i_indepdt |**| v_ret_w == (v_i_depdt), 求 v_ret_w
  v_i_indepdt                       float[][]:= (select array_agg(i_indepdt order by ord_no) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no);
  v_i_depdt                       float[][]:= (select array_agg(i_depdt order by ord_no) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no);
  v_len_x1_y1                 int               := array_length(v_i_indepdt, 1);
  v_len_x1_y1_samp            int               := least(greatest(floor(v_len_x1_y1 / 10), 10000), v_len_x1_y1, 10000);       -- least(i_grad_descent_batch_cnt, v_len_x1_y1); -- 小批量梯度下降，每批记录数 
  v_len_x2_w1                 int               := array_length(v_i_indepdt, 2);
  v_len_w2_y2                 int               := array_length(v_i_depdt, 2);
  v_ret_w                     float[];           -- [v_len_x2_w1][v_len_w2_y2] 
  v_x_rand_samp               float[];           -- [v_len_x1_y1_samp][v_len_x2_w1];
  v_y_rand_samp               float[];           -- [v_len_x1_y1][v_len_w2_y2]
  v_cur_loss_delta            float[];           -- [v_len_x2_w1][v_len_w2_y2]
  -- -- v_cur_loss_delta_last       float[];           -- [v_len_x2_w1][v_len_w2_y2]
  v_cur_no                    int               := 1;
  v_cur_loss_delta_l1_sum     float[];           -- [v_len_x2_w1][v_len_w2_y2]   
  v_cur_loss_delta_l2_sum     float[];           -- [v_len_x2_w1][v_len_w2_y2]   
  v_beta_l1                   float    := 0.9;
  v_beta_l2                   float    := 0.999;
  v_grad_l1                   float;             -- 寄存梯度本次变化的一阶矩，优化重复计算


begin
  set search_path to public;
  -- 审计 array_length(v_i_depdt, 1) == v_len_x1_y1
  if array_length(v_i_depdt, 1) <> v_len_x1_y1
  then
    raise exception 'error length of v_i_depdt';
  end if;

  -- 新建任务或者提取既有任务寄存信息
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then
    select 
      -- work_no,
      coalesce(learn_cnt + 1, v_cur_no),
      -- loss_delta,
      coalesce(ret_w, sm_sc.fv_new_randn(0.0, 0.5, array[v_len_x2_w1, v_len_w2_y2]))
    into  
      v_cur_no,
      v_ret_w
    from sm_sc.tb_classify_task 
    where work_no = i_work_no
    ;
  else
    insert into sm_sc.tb_classify_task
    (
      work_no, 
      learn_cnt, 
      -- loss_delta,
      ret_w
    )
    select 
      i_work_no, 
      0,
      -- sm_sc.fv_aggr_slice_sum(sqart(v_cur_loss_delta ^` 2)), 
      coalesce(ret_w, sm_sc.fv_new_randn(0.0, 0.5, array[v_len_x2_w1, v_len_w2_y2]))
    ;
  end if;

raise notice 'step 00, @|`  v_cur_loss_delta: %;', @|`  v_cur_loss_delta ~=` 4;

  while 
    (
      v_cur_loss_delta is null
      or v_grad_l1 >= i_loss_delta_least_stop_threshold
    )
    and v_cur_no <= i_limit_train_times
  loop

raise notice 'step 0, loop: %;', v_cur_no;
    -- -- v_cur_loss_delta_last = v_cur_loss_delta;

raise notice 'step 1 begin: v_x_rand_samp: %', v_x_rand_samp ~=` 4;
raise notice 'step 1 begin: v_y_rand_samp: %', v_y_rand_samp ~=` 4;

    -- 小批量随机梯度下降
    -- -- 小批量取样，降低复杂度和计算量
    select 
      array_agg(array(select unnest(v_i_indepdt[a_y_no : a_y_no][:]))),
      array_agg(array(select unnest(v_i_depdt[a_y_no : a_y_no][:])))
    into 
      v_x_rand_samp, 
      v_y_rand_samp 
    from (select a_y_no from generate_series(1, v_len_x1_y1) tb_a(a_y_no) order by random() limit v_len_x1_y1_samp) tb_a_ex
    ;

raise notice 'step 1 end: v_x_rand_samp: %', v_x_rand_samp ~=` 4;
raise notice 'step 1 end: v_y_rand_samp: %', v_y_rand_samp ~=` 4;
raise notice 'step 2 begin: v_cur_loss_delta: %', v_cur_loss_delta ~=` 4;

    -- -- 计算梯度
    v_cur_loss_delta := 
    (
      with 
      cte_cur_loss_delta_x1_y1 as
      (
        select 
          a_cur_w2_y2,
          array_agg
          (
            array(select unnest
            (
              sm_sc.fv_aggr_slice_sum
              (
                v_x_rand_samp[a_cur_x1_y1 : a_cur_x1_y1][:] 
                |**| v_ret_w[:][a_cur_w2_y2 : a_cur_w2_y2]
              , array[1, 1]
              )
              -` v_y_rand_samp[a_cur_x1_y1 : a_cur_x1_y1][a_cur_w2_y2 : a_cur_w2_y2]
            ))
            order by a_cur_x1_y1
          ) 
          *` v_x_rand_samp
            as a_cur_loss_delta_x1_y1
        from generate_series(1, v_len_x1_y1_samp) tb_a_x1_y1(a_cur_x1_y1)
          , generate_series(1, v_len_w2_y2) tb_a_w2_y2(a_cur_w2_y2)
        group by a_cur_w2_y2
      ),
      cte_cur_cost_w2_y2 as
      (
        select 
          a_cur_w2_y2,
          array_agg
          (
            array
            [
              sm_sc.fv_aggr_slice_sum(a_cur_loss_delta_x1_y1[1 : v_len_x1_y1_samp][a_cur_x2_w1 : a_cur_x2_w1]) 
            ]
            order by a_cur_x2_w1
          ) as a_cur_loss_delta_w2_y2
        from cte_cur_loss_delta_x1_y1
          , generate_series(1, v_len_x2_w1) tb_a_x2_w1(a_cur_x2_w1)
        group by a_cur_w2_y2
      )
      select sm_sc.fa_mx_concat_x(a_cur_loss_delta_w2_y2 order by a_cur_w2_y2) / v_len_x1_y1_samp :: float from cte_cur_cost_w2_y2
    );

raise notice 'step 2 end: v_cur_loss_delta: %', v_cur_loss_delta ~=` 4;
raise notice 'step 3 begin: v_ret_w: %', v_ret_w ~=` 4;

    v_grad_l1 = sm_sc.fv_aggr_slice_sum(@|`  v_cur_loss_delta);

    -- -- -- -- 梯度加入惯性（混入历史梯度成分），协助摆脱局部最优
    -- -- -- 下面已经实现了Adam，本质是带动量的RMSProp
    -- -- v_cur_loss_delta := coalesce((v_cur_loss_delta +` (0.618 *` v_cur_loss_delta_last)) /` 1.618, v_cur_loss_delta);  -- 裴波那契等比衰减系数

    -- -- 梯度下降优化 https://blog.csdn.net/jiaoyangwm/article/details/81457623
    -- -- 自适应学习率
    -- 一阶矩等比衰减系数 0.9
    v_cur_loss_delta_l1_sum = (v_beta_l1 *` coalesce(v_cur_loss_delta_l1_sum, v_cur_loss_delta)) +` ((1.0 - v_beta_l1) *` v_cur_loss_delta);
    -- 二阶矩等比衰减系数 0.999
    v_cur_loss_delta_l2_sum = (v_beta_l2 *` coalesce(v_cur_loss_delta_l2_sum, (v_cur_loss_delta ^` 2.0))) +` ((1.0 - v_beta_l2) *` (v_cur_loss_delta ^` 2.0));

    -- -- -- 梯度下降一次，迭代一次 v_ret_w   -- 原始的梯度下降算法
    -- v_ret_w := v_ret_w -` (i_learn_rate *` v_cur_loss_delta);
    -- -- -- 梯度下降一次，迭代一次 v_ret_w   -- adam 梯度下降算法，以及一阶矩、二阶矩无偏估计
    v_ret_w := 
      v_ret_w -` 
      (
        (
          v_cur_loss_delta_l1_sum *` 
          (i_learn_rate / (1.0 - (v_beta_l1 ^ v_cur_no)))
        ) /` 
        (
          sm_sc.fv_ele_replace
          (
            (
              v_cur_loss_delta_l2_sum /` (1.0 - (v_beta_l2 ^ v_cur_no))
            ) ^` 0.5
            , array[0.0]
            , 0.00000001       -- -- -- 全局 eps = 0.00000001
          )
        )
      );


raise notice 'step 3 end: v_ret_w: %', v_ret_w ~=` 4;
    -- 寄存至 sm_sc.tb_classify_task
    insert into sm_sc.tb_classify_task
    (
      work_no, 
      learn_cnt , 
      loss_delta,
      ret_w
    )
    select 
      i_work_no, 
      v_cur_no, 
      v_grad_l1, 
      v_ret_w
    on conflict(work_no) do 
    update set
      learn_cnt = v_cur_no,
      loss_delta = v_grad_l1,
      ret_w = v_ret_w
    ;

    -- 循环游标 + 1
    v_cur_no := v_cur_no + 1;
  end loop;

  if v_grad_l1 >= i_learn_rate / 0.00001    -- v_grad_l1 >= i_loss_delta_least_stop_threshold
  then
    raise notice 'uncompleted!';
  end if;
end
$$
language plpgsql;

-- -- create extension pg_variables;

-- select pgv_set('vars', 'this_work_no_02', nextval('seq_learn_work')::bigint);
-- -- truncate table sm_sc.tb_classify_task
-- insert into sm_sc.tb_classify_task(work_no, learn_cnt) 
-- select 
--   pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--   0;

-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   work_no           ,
--   ord_no            ,
--   i_depdt               ,
--   i_indepdt    
-- )
-- select 
--   pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--   a_idx,
--   sm_sc.fv_mx_ele_2d_2_1d
--   ((|^~| array[array[2.0, 4.0, 6.0]])[a_idx : a_idx][ : ]),
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       array
--       [ 
--         [1.0, 1.0]               ,
--         [1.0, 2.0]            ,
--         [1.0, 3.0]
--       ]
--     )[a_idx : a_idx][ : ]
--   )
-- from generate_series(1, 3) tb_a(a_idx);

-- call sm_sc.prc_linear_regression
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   0.05      ,
--   2500       -- ,
--   -- 20      ,
--   -- 0.01
-- );
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- -- 期望输出： |^~| array[array[0.0, 2.0]]

-- -----------------------------------------
-- select pgv_set('vars', 'this_work_no_03', nextval('seq_learn_work')::bigint);
-- -- truncate table sm_sc.tb_classify_task
-- insert into sm_sc.tb_classify_task(work_no, learn_cnt) 
-- select 
--   pgv_get('vars', 'this_work_no_03', NULL::bigint), 
--   2500;

-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   work_no             ,
--   ord_no              ,
--   i_depdt                 ,
--   i_indepdt    
-- )
-- select 
--   pgv_get('vars', 'this_work_no_03', NULL::bigint),
--   a_idx, 
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (|^~| (
--             array[[2.0 + 1.0, 4.0 + 1.0, 6.0 + 1.0, 8.0 + 1.0, 10.0 + 1.0, 12.0 + 1.0]
--                  ,[3.0 - 1.0, 6.0 - 1.0, 9.0 - 1.0, 12.0 - 1.0, 15.0 - 1.0, 18.0 - 1.0]] 
--             +` fv_new_randn(0, 0.01, array[2, 6])
--           )
--     )[a_idx : a_idx][ : ]
--   ), 
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       array
--       [ 
--         [1.0, 1.0]               ,
--         [1.0, 2.0]            ,
--         [1.0, 3.0]            ,
--         [1.0, 4.0]            ,
--         [1.0, 5.0]            ,
--         [1.0, 6.0]
--       ] +` fv_new_randn(0, 0.01, array[6, 1])
--     )[a_idx : a_idx][ : ]
--   )
-- from generate_series(1, 6) tb_a(a_idx);

-- call sm_sc.prc_linear_regression
-- (
--   pgv_get('vars', 'this_work_no_03', NULL::bigint)   ,
--   0.05      ,
--   2500       -- ,
--   -- 20      ,
--   -- 0.01
-- );
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_03', NULL::bigint)
-- -- 期望输出： |^~| array[array[1.0, 2.0], array[-1.0, 3.0]]

-- -----------------------------------------
-- select pgv_set('vars', 'this_work_no_01', nextval('seq_learn_work')::bigint);
-- insert into sm_sc.tb_classify_task(work_no, learn_cnt) 
-- select 
--   pgv_get('vars', 'this_work_no_01', NULL::bigint), 
--   2500;

-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   work_no          ,
--   ord_no           ,
--   i_depdt              ,
--   i_indepdt    
-- )
-- select 
--   pgv_get('vars', 'this_work_no_01', NULL::bigint), 
--   a_idx, 
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       array
--       [
--         array[22.1000 ,    12.2000     ],
--         array[-10.0000,     -7.6000 ],  
--         array[46.7000 ,    27.8000     ],
--         array[35.4000 ,    19.8000     ],
--         array[32.5000 ,    19.0000     ]
--       ]
--     )[a_idx : a_idx][ : ] 
--   ),
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       array
--       [ 
--         array[1.0, 1.1, 2.1, 0.1],
--         array[1.0, 1.2, 1.2, -3.2],
--         array[1.0, 0.7, 2.3, 3.3],
--         array[1.0, 1.4, 3.4, 0.4],
--         array[1.0, 0.5, 2.5, 1.5]
--       ]
--     )[a_idx : a_idx][ : ]
--   )
-- from generate_series(1, 6) tb_a(a_idx);

-- call sm_sc.prc_linear_regression
-- (
--   pgv_get('vars', 'this_work_no_01', NULL::bigint)   ,
--   0.05      ,
--   2500       -- ,
--   -- 20      ,
--   -- 0.01
-- );
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_01', NULL::bigint)
-- -- 期望输出： 
-- --   array
-- --   [
-- --     array[0.0, 0.0],
-- --     array[6.0, 3.0],
-- --     array[7.0, 4.0],
-- --     array[8.0, 5.0]
-- --   ]

-- -----------------------------------------
-- -- 以下用例量级相差悬殊，未做标准化，梯度溢出
-- select pgv_set('vars', 'this_work_no_01', nextval('seq_learn_work')::bigint);
-- insert into sm_sc.tb_classify_task(work_no, learn_cnt) 
-- select 
--   pgv_get('vars', 'this_work_no_01', NULL::bigint), 
--   2500;

-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   work_no             ,
--   ord_no              ,
--   i_depdt                 ,
--   i_indepdt    
-- )
-- select 
--   pgv_get('vars', 'this_work_no_01', NULL::bigint), 
--   a_idx, 
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       |^~| array[array[3222.83, 32203.73, 322012.73, 3220102.73, 32201002.73]]
--     )[a_idx : a_idx][ : ] 
--   ),
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     array
--     [ 
--       [1.1, 20.1, 300.1]               ,
--       [10.1, 200.1, 3000.1]            ,
--       [100.1, 2000.1, 30000.1]         ,
--       [1000.1, 20000.1, 300000.1]      ,
--       [10000.1, 200000.1, 3000000.1]
--     ]
--   )[a_idx : a_idx][ : ]
-- from generate_series(1, 6) tb_a(a_idx);

-- call sm_sc.prc_linear_regression
-- (
--   pgv_get('vars', 'this_work_no_01', NULL::bigint)   ,
--   0.05      ,
--   2500       -- ,
--   -- 20      ,
--   -- 0.01
-- );
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_01', NULL::bigint)
-- -- 期望输出： |^~| array[array[8.1], array[9.1], array[10.1]]


-- 参考 https://blog.csdn.net/zhq9695/article/details/82814197
--      https://blog.csdn.net/weixin_39797176/article/details/99488766   该文中有误，似然函数和梯度取负，才收敛

-- 初始学习率：1e-4
-- https://www.zhihu.com/question/387050717

drop procedure if exists sm_sc.prc_logistic_regression;
create or replace procedure sm_sc.prc_logistic_regression
(
  i_work_no                          bigint                                   ,                                                                                                -- 训练任务编号，用于任务暂停和接续、数据回显
  -- i_indepdt                                float[][]                       ,      -- [v_len_x1_y1][v_len_x2_w1]                                                         -- 对象的已知参数矩阵
  -- i_depdt                                float[][]                       ,      -- [v_len_x1_y1][v_len_w2_y2]                                                         -- 分类结果，要求类似onehot编码形式，0.0 <=` v_i_depdt <=` 1.0
  i_learn_rate                       float    default    0.0001        ,                                                                                                -- 学习率
  i_limit_train_times                int               default    10000       ,                                                                                                -- 最大训练次数，然后未完成中止
  -- i_grad_descent_batch_cnt           int               default    10000       ,   -- 建议大一些，远远超过 v_i_indepdt, v_i_depdt 的各维度长度，尽量防止因小批量梯度下降引起的鞍点训练终止 -- 小批量梯度下降，每批记录数
  i_loss_delta_least_stop_threshold  float    default    0.0001                                                                                                       -- 触发收敛的损失函数梯度阈值
)
as
$$
declare -- here
-- -- 数学原理：已知矩阵 v_i_indepdt, v_i_depdt, 且 sigmoid(v_i_indepdt |**| v_ret_w) == (v_i_depdt), 求 v_ret_w
  v_i_indepdt                       float[][]:= (select array_agg(i_indepdt order by ord_no) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no);
  v_i_depdt                       float[][]:= (select array_agg(i_depdt order by ord_no) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no);
  v_len_x1_y1                 int               := array_length(v_i_indepdt, 1); 
  v_len_x2_w1                 int               := array_length(v_i_indepdt, 2) + 1;
  v_x                         float[][]:= sm_sc.fv_new(1.0, array[v_len_x1_y1, 1]) |||| v_i_indepdt;
  v_len_x1_y1_samp            int               := least(greatest(floor(v_len_x1_y1 / 10), 10000), v_len_x1_y1, 10000);       -- least(i_grad_descent_batch_cnt, v_len_x1_y1); -- 小批量梯度下降，每批记录数
  v_len_w2_y2                 int               := array_length(v_i_depdt, 2);
  v_ret_w                     float[];           -- [v_len_x2_w1][v_len_w2_y2]   
  v_x_rand_samp               float[];           -- [v_len_x1_y1_samp][v_len_x2_w1];
  v_i_depdt_rand_samp             float[];           -- [v_len_x1_y1][v_len_w2_y2]
  v_cur_loss_delta            float[];           -- [v_len_x2_w1][v_len_w2_y2]
  -- -- v_cur_loss_delta_last       float[];           -- [v_len_x2_w1][v_len_w2_y2]
  v_cur_no                    int               := 1;
  v_cur_loss_delta_l1_sum     float[];            -- [v_len_x2_w1][v_len_w2_y2]
  v_cur_loss_delta_l2_sum     float[];            -- [v_len_x2_w1][v_len_w2_y2] 
  v_beta_l1                   float    := 0.9;
  v_beta_l2                   float    := 0.999;
  v_grad_l1                   float;             -- 寄存梯度本次变化的一阶矩，优化重复计算


begin
  set search_path to public;
  -- 审计 array_length(v_i_depdt, 1) == v_len_x1_y1
  if array_length(v_i_depdt, 1) <> v_len_x1_y1
  then
    raise exception 'error length of v_i_depdt';
  end if;

  -- 新建任务或者提取既有任务寄存信息
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then
    select 
      -- work_no,
      coalesce(learn_cnt + 1, v_cur_no),
      -- loss_delta,
      coalesce(ret_w, sm_sc.fv_new_randn(0.0, 0.5, array[v_len_x2_w1, v_len_w2_y2]))
    into  
      v_cur_no,
      v_ret_w
    from sm_sc.tb_classify_task 
    where work_no = i_work_no
    ;
  else
    insert into sm_sc.tb_classify_task
    (
      work_no, 
      learn_cnt, 
      -- loss_delta,
      ret_w
    )
    select 
      i_work_no, 
      0,
      -- sm_sc.fv_aggr_slice_sum(sqart(v_cur_loss_delta ^` 2)), 
      coalesce(ret_w, sm_sc.fv_new_randn(0.0, 0.5, array[v_len_x2_w1, v_len_w2_y2]))
    ;
  end if;

raise notice 'step 00, @|`  v_cur_loss_delta: %;', @|`  v_cur_loss_delta ~=` 4;

  while 
    (
      v_cur_loss_delta is null
      or v_grad_l1 >= i_loss_delta_least_stop_threshold
    )
    and v_cur_no <= i_limit_train_times
  loop

raise notice 'step 0, loop: %;', v_cur_no;
    -- -- v_cur_loss_delta_last = v_cur_loss_delta;

raise notice 'step 1 begin: v_x_rand_samp: %', v_x_rand_samp ~=` 4;
raise notice 'step 1 begin: v_i_depdt_rand_samp: %', v_i_depdt_rand_samp ~=` 4;

    -- 小批量随机梯度下降
    -- -- 小批量取样，降低复杂度和计算量
    select 
      array_agg(array(select unnest(v_x[a_y_no : a_y_no][:]))),
      array_agg(array(select unnest(v_i_depdt[a_y_no : a_y_no][:])))
    into 
      v_x_rand_samp, 
      v_i_depdt_rand_samp 
    from (select a_y_no from generate_series(1, v_len_x1_y1) tb_a(a_y_no) order by random() limit v_len_x1_y1_samp) tb_a_ex
    ;

raise notice 'step 1 end: v_x_rand_samp: %', v_x_rand_samp ~=` 4;
raise notice 'step 1 end: v_i_depdt_rand_samp: %', v_i_depdt_rand_samp ~=` 4;
raise notice 'step 2 begin: v_cur_loss_delta: %', v_cur_loss_delta ~=` 4;

    -- -- 计算梯度
    v_cur_loss_delta := 
    (
      with 
      cte_cur_loss_delta_x1_y1 as
      (
        select 
          a_cur_w2_y2,
          array_agg
          (
            array(select unnest
            (
              sm_sc.fv_activate_sigmoid
              (
                sm_sc.fv_aggr_slice_sum
                (
                  v_x_rand_samp[a_cur_x1_y1 : a_cur_x1_y1][:] 
                  |**| v_ret_w[:][a_cur_w2_y2 : a_cur_w2_y2]
                , array[1, 1]
                )
              )
              -` v_i_depdt_rand_samp[a_cur_x1_y1 : a_cur_x1_y1][a_cur_w2_y2 : a_cur_w2_y2]
            ))
            order by a_cur_x1_y1
          ) 
          *` v_x_rand_samp
            as a_cur_loss_delta_x1_y1
        from generate_series(1, v_len_x1_y1_samp) tb_a_x1_y1(a_cur_x1_y1)
          , generate_series(1, v_len_w2_y2) tb_a_w2_y2(a_cur_w2_y2)
        group by a_cur_w2_y2
      ),
      cte_cur_cost_w2_y2 as
      (
        select 
          a_cur_w2_y2,
          array_agg
          (
            array
            [
              sm_sc.fv_aggr_slice_sum(a_cur_loss_delta_x1_y1[1 : v_len_x1_y1_samp][a_cur_x2_w1 : a_cur_x2_w1]) 
            ]
            order by a_cur_x2_w1
          ) as a_cur_loss_delta_w2_y2
        from cte_cur_loss_delta_x1_y1
          , generate_series(1, v_len_x2_w1) tb_a_x2_w1(a_cur_x2_w1)
        group by a_cur_w2_y2
      )
      select sm_sc.fa_mx_concat_x(a_cur_loss_delta_w2_y2 order by a_cur_w2_y2) / v_len_x1_y1_samp :: float from cte_cur_cost_w2_y2
    );

raise notice 'step 2 end: v_cur_loss_delta: %', v_cur_loss_delta ~=` 4;
raise notice 'step 3 begin: v_ret_w: %', v_ret_w ~=` 4;

    v_grad_l1 = sm_sc.fv_aggr_slice_sum(@|`  v_cur_loss_delta);

    -- -- -- -- 梯度加入惯性（混入历史梯度成分），协助摆脱局部最优
    -- -- -- 下面已经实现了Adam，本质是带动量的RMSProp
    -- -- v_cur_loss_delta := coalesce((v_cur_loss_delta +` (0.618 *` v_cur_loss_delta_last)) /` 1.618, v_cur_loss_delta);  -- 裴波那契等比衰减系数

    -- -- 梯度下降优化 https://blog.csdn.net/jiaoyangwm/article/details/81457623
    -- -- 自适应学习率 Adam
    -- 一阶矩等比衰减系数 0.9
    v_cur_loss_delta_l1_sum = (v_beta_l1 *` coalesce(v_cur_loss_delta_l1_sum, v_cur_loss_delta)) +` ((1.0 - v_beta_l1) *` v_cur_loss_delta);
    -- 二阶矩等比衰减系数 0.999
    v_cur_loss_delta_l2_sum = (v_beta_l2 *` coalesce(v_cur_loss_delta_l2_sum, (v_cur_loss_delta ^` 2.0))) +` ((1.0 - v_beta_l2) *` (v_cur_loss_delta ^` 2.0));

    -- -- -- 梯度下降一次，迭代一次 v_ret_w   -- 原始的梯度下降算法
    -- v_ret_w := v_ret_w -` (i_learn_rate *` v_cur_loss_delta);
    -- -- -- 梯度下降一次，迭代一次 v_ret_w   -- adam 梯度下降算法，以及一阶矩、二阶矩无偏估计
    v_ret_w := 
      v_ret_w -` 
      (
        (
          v_cur_loss_delta_l1_sum *` 
          (i_learn_rate / (1.0 - (v_beta_l1 ^ v_cur_no)))
        ) /` 
        (
          sm_sc.fv_ele_replace
          (
            (
              v_cur_loss_delta_l2_sum /` (1.0 - (v_beta_l2 ^ v_cur_no))
            ) ^` 0.5
            , array[0.0]
            , 0.00000001        -- -- -- 全局 eps = 0.00000001
          )
        )
      );


raise notice 'step 3 end: v_ret_w: %', v_ret_w ~=` 4;
    -- 寄存至 sm_sc.tb_classify_task
    insert into sm_sc.tb_classify_task
    (
      work_no, 
      learn_cnt , 
      loss_delta,
      ret_w
    )
    select 
      i_work_no, 
      v_cur_no, 
      v_grad_l1, 
      v_ret_w
    on conflict(work_no) do 
    update set
      learn_cnt = v_cur_no,
      loss_delta = v_grad_l1,
      ret_w = v_ret_w
    ;

    -- 循环游标 + 1
    v_cur_no := v_cur_no + 1;
  end loop;

  if v_grad_l1 >= i_learn_rate / 0.00001    -- v_grad_l1 >= i_loss_delta_least_stop_threshold
  then
    raise notice 'uncompleted!';
  end if;
end
$$
language plpgsql;

-- -- create extension pg_variables;

-- -- 二分类
-- select pgv_set('vars', 'this_work_no_02', nextval('seq_learn_work')::bigint);
-- -- truncate table sm_sc.tb_classify_task
-- insert into sm_sc.tb_classify_task(work_no, learn_cnt) 
-- select 
--   pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--   0;
-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   work_no     ,
--   ord_no      ,
--   i_depdt         ,
--   i_indepdt    
-- )
-- select 
--   pgv_get('vars', 'this_work_no_02', NULL::bigint),
--   a_idx, 
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (sm_sc.fn_onehot(|^~| array[array['a', 'a', 'a', 'a', 'a', 'b', 'b', 'b', 'b', 'b']]))[a_idx : a_idx][ : ]
--   ),
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       array
--       [ 
--        [2.0, 0.0]            ,
--        [2.5, 0.5]            ,
--        [3.0, 1.0]            ,
--        [3.5, 1.5]            ,
--        [4.0, 2.0]            ,
--        [0.0, 2.0]            ,
--        [0.5, 2.5]            ,
--        [1.0, 3.0]            ,
--        [1.5, 3.5]            ,
--        [2.0, 4.0]            
--       ]
--     )[a_idx : a_idx][ : ]
--   )
-- from generate_series(1, 10) tb_a(a_idx);

-- call sm_sc.prc_logistic_regression
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   0.5      ,
--   1200       -- ,
--   -- 20      ,
--   -- 0.01
-- );
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- -- 期望输出： array[[-0.9243,-0.9243],[4.2689,-3.7785],[-3.7785,4.2689]]
-- -- 验证：select sm_sc.fv_activate_sigmoid(array[array[1.0, 3.1, 0.9]] |**| array[[-0.9243,-0.9243],[4.2689,-3.7785],[-3.7785,4.2689]])
-- --       其中 3.1, 0.9 为新对象属性入参，出参分别为类别 'a', 'b' 的概率

-- -----------------------------------------
-- -- 多分类
-- select pgv_set('vars', 'this_work_no_02', nextval('seq_learn_work')::bigint);
-- -- truncate table sm_sc.tb_classify_task
-- insert into sm_sc.tb_classify_task(work_no, learn_cnt) 
-- select 
--   pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--   0;

-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   work_no    ,
--   ord_no     ,
--   i_depdt        ,
--   i_indepdt    
-- )
-- select 
--   pgv_get('vars', 'this_work_no_02', NULL::bigint),
--   a_idx, 
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (sm_sc.fv_onehot(|^~| array[array['a', 'a', 'a', 'c', 'c', 'c', 'b', 'b', 'b', 'd', 'd', 'd']]))[a_idx : a_idx][ : ]
--   ),
--   sm_sc.fv_mx_ele_2d_2_1d
--   (
--     (
--       array
--       [ 
--         [1.0, 1.0]            ,
--         [1.0, 1.0]            ,
--         [1.0, 1.0]            ,
--         [-1.0, 1.0]            ,
--         [-1.0, 1.0]            ,
--         [-1.0, 1.0]            ,
--         [1.0, -1.0]            ,
--         [1.0, -1.0]            ,
--         [1.0, -1.0]            ,
--         [-1.0, -1.0]            ,
--         [-1.0, -1.0]            ,
--         [-1.0, -1.0]            
--       ] +` sm_sc.fv_new_randn(0.0, 0.1, array[12, 2])
--     )[a_idx : a_idx][ : ]
--   )
-- from generate_series(1, 12) tb_a(a_idx);

-- call sm_sc.prc_logistic_regression
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   0.05      ,
--   1500       -- ,
--   -- 20      ,
--   -- 0.01
-- );
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- -- 期望输出： array[[-8.1805,-6.7478,-7.8229,-6.4841],[8.3520,7.6351,-8.0533,-6.5274],[7.3270,-7.2030,7.7274,-6.0817]] 的正负一致
-- -- 验证：select sm_sc.fv_activate_sigmoid(array[array[1.0, 0.9, -1.1]] |**| array[[-8.1805,-6.7478,-7.8229,-6.4841],[8.3520,7.6351,-8.0533,-6.5274],[7.3270,-7.2030,7.7274,-6.0817]])
-- --       其中 0.9, -1.1 为新对象属性入参，出参分别为类别 'a', 'b', 'c', 'd' 的概率，本例 'b' (y[2]) 概率较大

drop procedure if exists sm_sc.prc_nn_prepare(bigint, int, int[], int4range[]);
create or replace procedure sm_sc.prc_nn_prepare
(
  i_work_no                          bigint                      -- 训练任务编号
, i_limit_train_times                int                         -- 规划的训练总次数
, i_batch_amt_per_range              int[]       default null    -- 每次训练在每个训练集区间选取小批量的数量。
                                                                 -- 要么 array_length(i_batch_amt_per_range, 1) = array_length(i_batch_range, 1), 要么 array_length(i_batch_amt_per_range, 1) = 1
                                                                 -- 如果 array_length(i_batch_amt_per_range, 1) = 1，那么所有区间选取数量相同，当次训练选取训练集数据总数量:  i_batch_amt * array_length(i_batch_range, 1)
                                                                 -- 如果 i_batch_amt_per_range, i_batch_range 都是 null，那么继承 00_buff_slice_rand_pick 既有的配置。
, i_batch_range                      int4range[] default null    -- 训练集选取区间策略
)
as
$$
declare -- here
  v_cur_learn_cnt          bigint               := (select learn_cnt from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号，当 v_cur_learn_cnt = 0 认作为新建的训练任务。
  v_input_asso_value       int[]   ;
  v_cur_node_nos           bigint[];
  v_cur_node_nos_last      bigint[];
  v_input_nodes            bigint[]             := (select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and (node_type in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_fn_type = '00_const') limit 1);
  v_cur_nn_depth           int                  := 0;   -- 深度层数游标
  v_max_nn_depth           int;

begin
  set search_path to public;
  
  -- ------------------------------- 配置小批量随机训练集数量 -----------------------------------------------
  -- 刚开始训练的准备阶段，则必须制定随机批量数量。后续接续训练，则可以空下来
  if v_cur_learn_cnt = 0 
    and i_batch_amt_per_range is null
  then 
    raise exception 'should configure i_batch_amt_per_range($3) at the beginning of new training task';
  -- 审计 i_batch_amt_per_range，i_batch_range 两个配置维长要对齐。
  elsif array_length(i_batch_amt_per_range, 1) <> array_length(i_batch_range, 1)
    and array_length(i_batch_amt_per_range, 1) <> 1
  then 
    raise exception 'should array_length(i_batch_amt_per_range, 1) = array_length(i_batch_range, 1) or array_length(i_batch_amt_per_range, 1) = 1';
  -- 审计随机批量配置是否超出训练集序号范围。
  elsif exists 
    (
      select  
      from unnest(i_batch_range) tb_a_range(a_range) 
      where 
        not
        (
          a_range 
          <@ 
          (select int4range(min(ord_no), max(ord_no),'[]') from sm_sc.tb_nn_train_input_buff where work_no = i_work_no)
        )
    )
  then 
    raise exception 'outof range of train dataset in sm_sc.tb_nn_train_input_buff.ord_no where work_no = i_work_no';
  -- i_batch_amt_per_range 不为空，则配置到 00_buff_slice_rand_pick 算子。
  elsif array_length(i_batch_amt_per_range, 1) = 1 and i_batch_range is not null
  then 
    i_batch_amt_per_range := sm_sc.fv_new(i_batch_amt_per_range, array[coalesce(array_length(i_batch_range, 1), 1)]);
  end if;
    
  if i_batch_range is null 
  then 
    update sm_sc.tb_nn_node tb_node
    set 
      node_fn_asso_value = 
        case
          when i_batch_amt_per_range is not null 
            then (select array[array[min(ord_no)], array[max(ord_no)], i_batch_amt_per_range] from sm_sc.tb_nn_train_input_buff where work_no = i_work_no)
          else 
            tb_node.node_fn_asso_value
        end
    where node_fn_type = '00_buff_slice_rand_pick'
      and node_type = 'input_01'
      and work_no = i_work_no
    returning node_fn_asso_value :: int[] into v_input_asso_value
    ;
    commit;
  else
    update sm_sc.tb_nn_node tb_node
    set 
      node_fn_asso_value = 
        array
        [
          a_lowers,
          a_uppers,
          i_batch_amt_per_range
        ]
    from 
    (
      select
        array_agg(lower(i_batch_range[a_range_no]) order by a_range_no) as a_lowers
      , array_agg(upper(i_batch_range[a_range_no]) - 1 order by a_range_no) as a_uppers
      from generate_series(1, array_length(i_batch_range, 1)) tb_a_range_no(a_range_no)
    ) tb_a_ranges(a_lowers, a_uppers)
    where node_fn_type = '00_buff_slice_rand_pick'
      and node_type = 'input_01'
      and work_no = i_work_no
    returning node_fn_asso_value :: int[] into v_input_asso_value
    ;
    commit;
  end if;
  
-- -- -- debug 
-- -- raise notice 'input_01 v_input_asso_value: %', array_dims(v_input_asso_value);
  
  -- ------------------------------- 审计部分 -----------------------------------------------
  -- 审计 训练任务是否存在
  if not exists(select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then
    raise exception 'no such work NO. = %!', i_work_no;

  -- 审计 输入节点是否在有效范围内
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') group by node_type having count(*) > 1)
  then
    raise exception 'more than 1 input node, or expect one-hot i_indepdt_*!';
    
  -- -- -- 审计 测试与验证的输入节点是否只有一个
  -- -- elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'prod_input' having count(*) > 1)
  -- -- then
  -- --   raise exception 'more than 1 prod_input node, or expect one-hot i_indepdt_*!';
  -- -- 
  -- 审计 输出节点是否在有效范围内
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('output_01', 'output_02', 'output_03', 'output_04') group by node_type having count(*) > 1)
  then
    raise exception 'more than 1 output node, or expect one-hot i_depdt_*!';
    
  -- 审计 训练集同一输入节点每条数据的规格同一
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      group by work_no 
      having (count(distinct array_dims(i_indepdt_01)) > 1
              or count(distinct array_dims(i_indepdt_02)) > 1
              or count(distinct array_dims(i_indepdt_03)) > 1
              or count(distinct array_dims(i_indepdt_04)) > 1
             )
    )
  then
    raise exception 'there should not be different array_dims(i_indepdt_*).';
    
  -- 审计 训练集同一输出节点每条数据的规格同一
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      group by work_no 
      having count(distinct array_dims(i_depdt_01)) > 1
        or count(distinct array_dims(i_depdt_02)) > 1
        or count(distinct array_dims(i_depdt_03)) > 1
        or count(distinct array_dims(i_depdt_04)) > 1
    )
  then
    raise exception 'there should not be different arrray_length(i_depdt_*, 2).';
    
  -- 审计 训练集不存在 null 之元素
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_01)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_02)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_03)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_04)
    )
  then 
    raise exception 'there are some null element in input i_indepdt_*.';
    
  -- 审计 训练集判断结果不存在 null 之元素
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_01)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_02)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_03)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_04)
    )
  then 
    raise warning 'there are some null element in input i_depdt_*.';

  -- 审计 小批量随机的训练集记录数目是否符合要求
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' and node_fn_type = '00_buff_slice_rand_pick')
    -- and v_input_cnt < sm_sc.fv_aggr_slice_max(v_input_asso_value[2 : 2][ : ])
    and exists (select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no group by work_no having count(*) < sm_sc.fv_aggr_slice_max(v_input_asso_value[2 : 2][ : ]))
  then 
    raise exception 'count of training data should not be less than slices'' uppers of input node.';
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' and node_fn_type = '00_buff_slice_rand_pick')
    and true = any(v_input_asso_value[3 : 3][ : ] >` (v_input_asso_value[2 : 2][ : ] -` v_input_asso_value[1 : 1][ : ] +` 1))
  then 
    raise exception 'there should be not any 00_buff_slice_rand_pick more than its slice range.';
    
  -- 00_const 节点的算子超参数审计工作
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_fn_type = '00_const' and node_fn_asso_value is null)
  then 
    raise exception 'there should be not any null node_fn_asso_value at 00_const.';
    
  -- 非 weight 的 00_const 节点的 node_depdt_vals 初始化审计工作
  elsif exists 
    (
      select  
      from sm_sc.tb_nn_node 
      where work_no = i_work_no 
        and node_fn_type = '00_const' 
        and node_type is distinct from 'weight'
        and 
        (
          node_depdt_vals is null
          -- -- -- 以下注释代码的数组规格审查，阻碍了点算的广播特性
          -- -- or array_length(node_depdt_vals, 1) <> (node_fn_asso_value[1] :: int) 
          -- -- or array_length(node_depdt_vals, 2) <> (node_fn_asso_value[2] :: int)
          -- -- or array_length(node_depdt_vals, 3) <> (node_fn_asso_value[3] :: int)
          -- -- or array_length(node_depdt_vals, 4) <> (node_fn_asso_value[4] :: int)
        )
    )
  then 
    raise exception 'there should be not any null node_depdt_vals or unmatch length configured by node_fn_asso_value at 00_const.';

  -- 审计 算子入参数量
  --     审计双目
  elsif exists 
  (
    select 
    from sm_sc.tb_dic_enum tb_a_enum
    inner join sm_sc.tb_nn_node tb_a_node 
      on tb_a_node.node_fn_type = tb_a_enum.enum_key
        and tb_a_node.work_no = i_work_no
    inner join sm_sc.tb_nn_path tb_a_path 
      on tb_a_path.fore_node_no = tb_a_node.node_no
        and tb_a_path.work_no = i_work_no
    where tb_a_enum.enum_name = 'node_fn_type'
      and tb_a_enum.enum_group = '2_p'
    group by tb_a_path.fore_node_no
    having count(tb_a_path.path_ord_no) <> 2
  )
  then
    raise exception 'too many or too few pathes point to an fn with 2_p.';
  
  --     审计单目
  elsif exists
  (
    select 
    from sm_sc.tb_dic_enum tb_a_enum
    inner join sm_sc.tb_nn_node tb_a_node 
      on tb_a_node.node_fn_type = tb_a_enum.enum_key
        and tb_a_node.work_no = i_work_no
    inner join sm_sc.tb_nn_path tb_a_path 
      on tb_a_path.fore_node_no = tb_a_node.node_no
        and tb_a_path.work_no = i_work_no
    where tb_a_enum.enum_name = 'node_fn_type'
      and tb_a_enum.enum_group = '1_p'
    group by tb_a_path.fore_node_no
    having count(tb_a_path.path_ord_no) > 1
  )
  then 
    raise exception 'too many pathes point to an fn with 1_p.';

  -- 审计 是否存在孤岛网络
  elsif exists 
  (
    with recursive
    cte_main_nodes as
    (
      select 
        node_no
      from sm_sc.tb_nn_node
      where work_no = i_work_no
        and node_type in ('output_01', 'output_02', 'output_03', 'output_04')
      union
      select 
        tb_a_incre.node_no
      from cte_main_nodes tb_a_main
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.fore_node_no = tb_a_main.node_no
      inner join sm_sc.tb_nn_node tb_a_incre
        on tb_a_incre.node_no = tb_a_path.back_node_no
          and tb_a_incre.work_no = i_work_no
    )
    select 
    from cte_main_nodes
    having count(node_no) < (select count(node_no) from sm_sc.tb_nn_node where work_no = i_work_no)
  )
  then
    raise exception 'there should not be alone nodes.';

  -- 审计通过，新的任务的初始化工作(learn_cnt = 0)
  elsif v_cur_learn_cnt = 0
  then
    
-- -- -- debug
-- raise notice 'L298. begin: v_cur_node_nos: %;', v_cur_node_nos;
  
    update sm_sc.tb_nn_node tb_a_tar
    set 
      -- -- node_depdt_len   = 
      -- --   array 
      -- --   [
      -- --     case tb_a_tar.node_fn_type 
      -- --       when '00_full_dataset' 
      -- --         then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
      -- --       when '00_buff_slice_rand_pick' 
      -- --         then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
      -- --     end,
      -- --     v_input_len_x
      -- --   ]
      -- -- ,                        -- -- array[array_length(tb_a_sour.i_indepdt_01, 1), array_length(tb_a_sour.i_indepdt_01, 2)],
      is_fore_node = 
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then false 
          when '00_buff_slice_rand_pick' 
            then true 
        end
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type = 'input_01'
    ;
    commit;
    
    update sm_sc.tb_nn_node tb_a_tar
    set 
      is_fore_node = (select is_fore_node from sm_sc.tb_nn_node where work_no = i_work_no and tb_a_tar.node_type = 'input_01')
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
    ;
    commit;
    
    -- 00_const 节点的准备工作
    update sm_sc.tb_nn_node
    set 
      is_fore_node = false
    , is_back_node = false
    where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
    ;
    commit;
    
-- -- -- debug
-- raise notice 'L226. begin: node_depdt_len of input: %;', (select node_depdt_len from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01');
  
    -- 标记前向传播节点
    -- 缺省 true. 从神经网络的初始起点（包括）开始审计，至 01_prod_mx, 05_conv_2d_grp_x, rand 类算子为止（不包括），沿途皆为 false    
    -- 更新 is_fore_node
    v_cur_node_nos := 
      (
        select array_agg(node_no) from sm_sc.tb_nn_node 
        where work_no = i_work_no 
          and 
          (
            node_type in ('input_01', 'input_02', 'input_03', 'input_04')
            or node_fn_type = '00_const' and node_type <> 'weight'
          )
      );     -- -- v_input_nodes
    while array_length(v_cur_node_nos, 1) > 0
    loop 
      -- pg 暂不支持 recursive cte 使用 aggr, 先只能用循环
      with
      cte_fore_nodes as 
      (
        select 
          tb_a_fore.node_no,
          max(tb_a_fore.node_fn_type) as node_fn_type
        from sm_sc.tb_nn_path tb_a_fore_path
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
            and tb_a_fore.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_path tb_a_back_path
          on tb_a_back_path.fore_node_no = tb_a_fore.node_no
            and tb_a_back_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.node_no = tb_a_back_path.back_node_no
            and tb_a_back.work_no = i_work_no   -- 2021082501
        where tb_a_fore_path.back_node_no = any(v_cur_node_nos)
          and tb_a_fore.node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
          and tb_a_fore_path.work_no = i_work_no   -- 2021082501
        group by tb_a_fore.node_no
        having count(distinct tb_a_back.node_no) =
                 count(distinct tb_a_back.node_no) 
                   filter 
                   (
                     where tb_a_back.is_fore_node is false
                   )
      ),
      cte_upd as
      (
        update sm_sc.tb_nn_node tb_a_tar
        set 
          is_fore_node = false
        from cte_fore_nodes tb_a_sour
        where tb_a_sour.node_no = tb_a_tar.node_no
          and tb_a_tar.work_no = i_work_no   -- 2021082501
      )
      select 
        array_agg(distinct node_no) into v_cur_node_nos 
      from cte_fore_nodes
      where node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
      ;
      commit;
  
-- -- -- debug
-- raise notice 'L345.: v_cur_node_nos: %;', v_cur_node_nos;
  
    end loop;
    
    -- 标记反向传播节点
    -- 缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true

    update sm_sc.tb_nn_node
    set 
      is_back_node = true
    where work_no = i_work_no   -- 2021082501
      -- and node_no = any(v_init_nodes)
      and node_type = 'weight'
    ;
    commit;

    with recursive
    cte_back_nodes as
    (
      select 
        tb_a_main.node_no
      from sm_sc.tb_nn_node tb_a_main
      where tb_a_main.work_no = i_work_no   -- 2021082501
        and tb_a_main.is_back_node is true
      union
      select
        tb_a_path.fore_node_no
      from cte_back_nodes tb_a_back
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.back_node_no = tb_a_back.node_no 
      where tb_a_path.work_no = i_work_no   -- 2021082501
    )
    update sm_sc.tb_nn_node tb_a_tar
    set
      is_back_node = true
    from cte_back_nodes tb_a_sour
    where tb_a_sour.node_no = tb_a_tar.node_no
      and tb_a_tar.work_no = i_work_no   -- 2021082501
      -- and tb_a_tar.node_type <> 'output_01'
    ;
    commit;   
  end if;
  
  -- 每次启动训练，都重新计算 node_depdt_len，用于适配动态调整随机小批量的每个类别抽样数量
  update sm_sc.tb_nn_node tb_a_tar
  set 
    node_depdt_len   = 
      array 
      [
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
          when '00_buff_slice_rand_pick' 
            then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
        end
      ]
      || 
      (
        select 
          array_agg(array_length(i_indepdt_01, a_cur) order by a_cur)
        from (select i_indepdt_01 from sm_sc.tb_nn_train_input_buff where work_no = i_work_no limit 1) tb_a_buff(i_indepdt_01)
          , generate_series(1, array_ndims(tb_a_buff.i_indepdt_01)) tb_a_cur(a_cur)
      )
  where tb_a_tar.work_no = i_work_no   -- 2021082501
    and tb_a_tar.node_type = 'input_01'
  ;
  commit;
  
  -- 初始化其他模态输入节点的规格
  update sm_sc.tb_nn_node tb_a_tar
  set 
    node_depdt_len = 
      array 
      [
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
          when '00_buff_slice_rand_pick' 
            then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
        end
      ]
      || 
      (
        select 
          array_agg(array_length(i_indepdt_ex, a_cur) order by a_cur)
        from 
          (select 
             case tb_a_tar.node_type 
               when 'input_02'
                 then i_indepdt_02
               when 'input_03'
                 then i_indepdt_03
               when 'input_04'
                 then i_indepdt_04
             end                    
             as i_indepdt_ex 
           from sm_sc.tb_nn_train_input_buff where work_no = i_work_no limit 1
          ) tb_a_buff(i_indepdt_ex)
        , generate_series(1, array_ndims(tb_a_buff.i_indepdt_ex)) tb_a_cur(a_cur)
      )
  where tb_a_tar.work_no = i_work_no   -- 2021082501
    and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
  ;
  commit;

  -- 对齐训练次数；训练过程中，输入、偏移量系数节点不参与传播
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = case when node_type in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_fn_type = '00_const' then i_limit_train_times else v_cur_learn_cnt end,
    learn_cnt_back = case when node_type in ('input_01', 'input_02', 'input_03', 'input_04') or (node_fn_type = '00_const' and node_type is distinct from 'weight') then i_limit_train_times else v_cur_learn_cnt end
  where work_no = i_work_no   -- 2021082501
  ;
  commit;

-- -- -- debug
-- raise notice 'L102. begin: v_cur_learn_cnt: %', v_cur_learn_cnt;

  -- 传播规划
  -- 先做个 learn_cnt_.. 的负标记
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = - 1,
    learn_cnt_back = - 1
  where work_no = i_work_no
    and ((node_type not in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_type is null) and node_fn_type <> '00_const')
  ;
  commit;
  -- 前向打标
  v_cur_node_nos := v_input_nodes;
  v_cur_nn_depth := 0;

-- -- -- debug
-- raise notice 'L224. begin: v_cur_node_nos: %; v_input_nodes: %; v_cur_nn_depth: %;', v_cur_node_nos, v_input_nodes, v_cur_nn_depth;

  update sm_sc.tb_nn_node 
  set nn_depth_no = 0 
  where work_no = i_work_no 
    and (node_type in ('input_01', 'input_02', 'input_03', 'input_04') or (node_fn_type = '00_const' and node_type is distinct from 'weight'));
  commit;

  while 
    exists 
    (
      select         
      from sm_sc.tb_nn_node tb_a_fore
      where tb_a_fore.work_no = i_work_no    -- 2021082501
        and tb_a_fore.learn_cnt_fore < v_cur_learn_cnt
      limit 1
    )
  loop
    v_cur_nn_depth := v_cur_nn_depth + 1;
    v_cur_node_nos_last := v_cur_node_nos;
    with 
    -- 由 learn_cnt_fore 约束，找到与上次前向传播有关的，可前向计算的节点
    cte_y as
    (
      select 
        tb_a_fore.node_no
      -- , tb_a_fore.node_fn_type
      from sm_sc.tb_nn_path tb_a_fore_path
      inner join sm_sc.tb_nn_node tb_a_fore
        on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
          and tb_a_fore.work_no = i_work_no   -- 2021082501
      inner join sm_sc.tb_nn_path tb_a_back_path
        on tb_a_back_path.fore_node_no = tb_a_fore.node_no
          and tb_a_back_path.work_no = i_work_no   -- 2021082501
      inner join sm_sc.tb_nn_node tb_a_back
        on tb_a_back.node_no = tb_a_back_path.back_node_no
          and tb_a_back.work_no = i_work_no   -- 2021082501
      where tb_a_fore_path.work_no = i_work_no   -- 2021082501
        and tb_a_fore.learn_cnt_fore < v_cur_learn_cnt 
        and tb_a_fore_path.back_node_no = any(v_cur_node_nos || v_input_nodes) 
      group by tb_a_fore.node_no -- , tb_a_fore.node_fn_type
      having count(*) filter (where tb_a_back.learn_cnt_fore > tb_a_fore.learn_cnt_fore) = count(*)
    ),
    cte_upd_fore_label as
    (
      update sm_sc.tb_nn_node tb_a_tar
      set 
        learn_cnt_fore = v_cur_learn_cnt,
        nn_depth_no = v_cur_nn_depth
      from cte_y tb_a_y
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_y.node_no = tb_a_tar.node_no
      returning tb_a_tar.node_no
    )
    select array_agg(node_no) into v_cur_node_nos from cte_upd_fore_label
    ;
    commit;

-- -- debug
-- raise notice 'L256. begin: nn_depth updated nodes = %: %', v_cur_node_nos, v_cur_nn_depth;
    if v_cur_node_nos is not distinct from v_cur_node_nos_last
    then 
      raise exception 'v_cur_node_nos is the same as v_cur_node_nos_last: %.', coalesce(v_cur_node_nos_last :: text, 'null');
    end if;

  end loop;

  v_max_nn_depth := v_cur_nn_depth;

  -- 修改 weight 节点的深度， = min(下层 01_prod_mx 的深度) - 1，保证反向传播及时传递到 weight 节点
  with
  cte_nn_depth as
  (
    select 
      tb_a_back.node_no,
      min(tb_a_fore.nn_depth_no) - 1 as nn_depth_no
    from sm_sc.tb_nn_node tb_a_back
    inner join sm_sc.tb_nn_path tb_a_path
      on tb_a_back.node_no = tb_a_path.back_node_no
        and tb_a_path.work_no = i_work_no
    inner join sm_sc.tb_nn_node tb_a_fore
      on tb_a_path.fore_node_no = tb_a_fore.node_no
        and tb_a_fore.work_no = i_work_no
    where tb_a_back.work_no = i_work_no
      and tb_a_back.node_type = 'weight'
    group by tb_a_back.node_no
  )
  update sm_sc.tb_nn_node tb_a_back
  set nn_depth_no = tb_a.nn_depth_no
  from cte_nn_depth tb_a
  where tb_a_back.work_no = i_work_no
    and tb_a.node_no = tb_a_back.node_no
    -- and tb_a_back.node_type = 'weight'
  ;
  commit;

  -- 传播规划完毕，恢复 learn_cnt 为真实标记
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = v_cur_learn_cnt,
    learn_cnt_back = v_cur_learn_cnt
  where work_no = i_work_no   -- 2021082501
    and (node_type not in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') and node_fn_type <> '00_const')
  ;
  commit;

  
  -- ---------- 填充缺省协参部分。遍历 nn_depth，每次遍历，首先更新非 weight 节点的 node_fn_asso_value，再更新 node_depdt_len，结束 nn_depth 遍历，最后更新 weight 的 node_depdt_vals  ----------------------

  -- ------------------------------- 准备数据 -----------------------------------------------
-- -- debug
raise notice 'prepare data begin!!!!!!'; 

  -- ------------------------------- 审计数据部分 -----------------------------------------------
-- -- debug
raise notice 'check data len begin!!!!!!';

  -- 前向通烟更新 node_depdt_len

  update sm_sc.tb_nn_node
  set node_depdt_len = node_fn_asso_value :: int[]
  where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
  ;
  commit;

    
  v_cur_nn_depth = 1;
  while v_cur_nn_depth <= v_max_nn_depth
  loop

    -- np aggr 情况，化为 1p
    with
    cte_concat_dims_change as 
    (
      select 
        tb_a_fore.node_no
      , (
          array 
          [
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[1], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
              else max(coalesce(tb_a_back_pn.node_depdt_len[1], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_y')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[2], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
              else max(coalesce(tb_a_back_pn.node_depdt_len[2], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[3], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
              else max(coalesce(tb_a_back_pn.node_depdt_len[3], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x3')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[4], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
              else max(coalesce(tb_a_back_pn.node_depdt_len[4], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x4')
            end
          ] 
        )[1 : max(array_length(tb_a_back_pn.node_depdt_len, 1))]
          as a_pn_len
      -- 若干类型算子粒度的超参数在此自动配置填充，而不需要手动配置。参看字典表对各类算子 node_fn_asso_value 的解释
      , case tb_a_fore.node_fn_type
          when '06_aggr_mx_avg'
            then array[count(tb_a_back_pn.node_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')]
          when '06_aggr_mx_concat_y'
            then array_agg(tb_a_back_pn.node_depdt_len[1] order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
          when '06_aggr_mx_concat_x'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[2], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
          when '06_aggr_mx_concat_x3'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[3], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
          when '06_aggr_mx_concat_x4'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[4], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
        end as a_node_fn_asso_value
      from sm_sc.tb_nn_node tb_a_fore
      inner join sm_sc.tb_nn_path tb_a_path_pn
        on tb_a_path_pn.work_no = i_work_no   -- 2021082501
          and tb_a_path_pn.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.tb_nn_node tb_a_back_pn
        on tb_a_back_pn.work_no = i_work_no   -- 2021082501
          and tb_a_back_pn.node_no = tb_a_path_pn.back_node_no
      where tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
        and tb_a_fore.node_fn_type in ('06_aggr_mx_avg', '06_aggr_mx_concat_y', '06_aggr_mx_concat_x', '06_aggr_mx_concat_x3', '06_aggr_mx_concat_x4')
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    ),
    cte_p1_p2 as 
    (
      select 
        tb_a_fore.node_no,
        coalesce(tb_a_back_pn.a_pn_len, tb_a_back_p1.node_depdt_len) as a_p1_len,
        case when tb_a_fore.node_fn_type not like '06_aggr_mx_%' then tb_a_back_p2.node_depdt_len end as a_p2_len,
        tb_a_back_pn.a_node_fn_asso_value
      from sm_sc.tb_nn_node tb_a_fore
      inner join sm_sc.tb_nn_path tb_a_path_p1
        on tb_a_path_p1.work_no = i_work_no   -- 2021082501
          and tb_a_path_p1.path_ord_no = 1 -- -- 如果与 前向节点的 path_ord_no 冲突，使用 00_none 算子隔离。
          and tb_a_path_p1.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.tb_nn_node tb_a_back_p1
        on tb_a_back_p1.work_no = i_work_no   -- 2021082501
          and tb_a_back_p1.node_no = tb_a_path_p1.back_node_no
      left join sm_sc.tb_nn_path tb_a_path_p2
        on tb_a_path_p2.work_no = i_work_no   -- 2021082501
          and tb_a_path_p2.path_ord_no = 2 -- -- 如果与 前向节点的 path_ord_no 冲突，使用 00_none 算子隔离。
          and tb_a_path_p2.fore_node_no = tb_a_fore.node_no
      left join sm_sc.tb_nn_node tb_a_back_p2
        on tb_a_back_p2.work_no = i_work_no   -- 2021082501
          and tb_a_back_p2.node_no = tb_a_path_p2.back_node_no
      left join cte_concat_dims_change tb_a_back_pn
        on tb_a_back_pn.node_no = tb_a_fore.node_no
      where tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
    )
    update sm_sc.tb_nn_node tb_a_fore
    set 
      node_fn_asso_value = 
        coalesce(tb_a_fore.node_fn_asso_value, tb_a_p1_p2.a_node_fn_asso_value) 
    , node_depdt_len = 
        sm_sc.fv_lambda_arr_len
        (
          tb_a_fore.node_no,
          tb_a_fore.node_fn_type, 
          tb_a_p1_p2.a_p1_len :: int[], 
          tb_a_p1_p2.a_p2_len :: int[],
          coalesce(tb_a_fore.node_fn_asso_value, tb_a_p1_p2.a_node_fn_asso_value)
        )
    from cte_p1_p2 tb_a_p1_p2
    where tb_a_fore.work_no = i_work_no   -- 2021082501
      and tb_a_fore.node_no = tb_a_p1_p2.node_no
    ;
    commit;

    v_cur_nn_depth := v_cur_nn_depth + 1;
  end loop;

-- ---------------------------------------------------------------
  -- weight 的高宽，由前向算子的高宽反推得出
  update sm_sc.tb_nn_node tb_a_back 
  set 
    node_fn_asso_value = 
      sm_sc.fv_nn_weight_len
      (
        tb_a_fore.node_fn_type
      , tb_a_path.path_ord_no
      , tb_a_fore.node_fn_asso_value
      )
  from sm_sc.tb_nn_path tb_a_path, sm_sc.tb_nn_node tb_a_fore 
  where tb_a_back.work_no = i_work_no   -- 2021082501
    and tb_a_path.work_no = i_work_no   -- 2021082501
    and tb_a_fore.work_no = i_work_no   -- 2021082501
    and tb_a_path.back_node_no = tb_a_back.node_no
    and tb_a_fore.node_no = tb_a_path.fore_node_no
    and tb_a_back.node_type = 'weight'     
    and tb_a_back.node_fn_asso_value is null                   -- weight 节点的 tb_a_back.node_fn_type is null
    -- and tb_a_path.path_ord_no = 2
  ;
  commit;

-- ------------------------------------------------------------------------------
  -- 首轮训练之前，以 node_depdt_len 为依据，确定 weight 的矩阵规格
  if v_cur_learn_cnt = 0
  then    
    -- 初始化 w 权值，为避免鞍点，采用随机
    update sm_sc.tb_nn_node 
    set 
      node_depdt_vals = 
        sm_sc.fv_new_randn
        (
          0.0 :: float 
          
        -- , 1.0 / 0.6745  :: float     -- 假定，权重（斜率）是无偏的，大于 0.0 和小于 0.0 的概率相同，都是 50%
        --                              -- 参看标准正态分布表
        --                              -- 误差概率分布：当统计值的误差介于±0.6745(标准误差)范围时，概率为 50%；误差介于±2范围时，概率为95%；误差介于±3范围时，概率为99.7%；
        --                              -- 也可以自行修改该标准差，使用权重参数的 xaviar 初始化, He 初始化 等方法
        , 1.0 
          / 
          power
          (
            greatest
            (
              node_fn_asso_value[array_length(node_fn_asso_value, 1) - 1]
            , node_fn_asso_value[array_length(node_fn_asso_value, 1)]
            )
          , 0.5
          )
        , node_fn_asso_value :: int[]
        )
    -- , node_depdt_len = node_fn_asso_value
    where work_no = i_work_no   -- 2021082501
      and node_type = 'weight'          -- weight 节点的 node_fn_type = '' 或者 is null
      and node_fn_type = '00_const'
    ;
    commit;
  end if;

-- ------------------------------------------------------------------------------
  -- 审查 nn 节点算子类型的超参数是否完善配置
  v_cur_node_nos := 
    (
      with 
      cte_fn_asso_value_range
      as 
      (
        select 
          enum_group
        , sm_sc.fa_range_or(int4range(lower(enum_range) :: int, upper(enum_range) :: int, '[]') :: int4multirange) as a_fn_asso_value_range
        from sm_sc.tb_dic_enum
        where enum_name = 'node_fn_asso_value'
        group by enum_group
      )
      select 
        array_agg(distinct tb_a_node.node_no)
      from cte_fn_asso_value_range tb_a_dic
      inner join sm_sc.tb_nn_node tb_a_node 
        on tb_a_node.node_fn_type = tb_a_dic.enum_group
      where tb_a_node.work_no = i_work_no
        and (not array_length(tb_a_node.node_fn_asso_value, 1) <@ tb_a_dic.a_fn_asso_value_range)
            or tb_a_node.node_fn_asso_value is null
    )
  ;

  delete from sm_sc.__vt_nn_node where work_no = i_work_no;
  insert into sm_sc.__vt_nn_node
  (
    work_no                      ,
    node_no                  ,        
    node_type                    ,
    node_fn_type                 ,
    node_fn_asso_value      ,
    nn_depth_no                  ,
    node_depdt_vals                  
  )
  select 
    work_no                      ,
    node_no                  ,        
    node_type                    ,
    node_fn_type                 ,
    node_fn_asso_value      ,
    nn_depth_no                  ,
    case when node_type = 'weight' or node_fn_type = '00_const' then node_depdt_vals end as node_depdt_vals                 
  from sm_sc.tb_nn_node
  where work_no = i_work_no
  ;    
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  insert into sm_sc.__vt_nn_path
  (
    work_no     
  , fore_node_no
  , path_ord_no 
  , back_node_no
  )
  select 
    work_no     
  , fore_node_no
  , path_ord_no 
  , back_node_no
  from sm_sc.tb_nn_path
  where work_no = i_work_no
  ;
  commit;

  if array_length(v_cur_node_nos, 1) > 0 
  then 
    raise warning 'these node_fn_asso_value is null, please check nodes: %.', v_cur_node_nos;
  else 
    raise notice 'prepare well done!!!!!!';
  end if;
  
  update sm_sc.tb_nn_node 
  set dropout_ratio = 0.0
  where work_no = i_work_no
    and dropout_ratio is null
  ;
  
  update sm_sc.tb_nn_node 
  set dropout_rescale = 1.0
  where work_no = i_work_no
    and dropout_rescale is null
  ;
  
  update sm_sc.tb_nn_node 
  set dropout_depdt_val = 0.0
  where work_no = i_work_no
    and dropout_depdt_val is null
  ;
  
  update sm_sc.tb_nn_node 
  set is_dropout = false
  where work_no = i_work_no
    and is_dropout is null
  ;
  
end
$$
language plpgsql;




-- -- 执行准备和检查
-- call sm_sc.prc_nn_prepare
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   2500      -- ,
--   -- 200
-- );
drop procedure if exists sm_sc.prc_nn_prepare_p(bigint, int, int[], int4range[]);
create or replace procedure sm_sc.prc_nn_prepare_p
(
  i_work_no                          bigint                      -- 训练任务编号
, i_limit_train_times                int                         -- 规划的训练总次数
, i_batch_amt_per_range              int[]       default null    -- 每次训练在每个训练集区间选取小批量的数量。
                                                                 -- 要么 array_length(i_batch_amt_per_range, 1) = array_length(i_batch_range, 1), 要么 array_length(i_batch_amt_per_range, 1) = 1
                                                                 -- 如果 array_length(i_batch_amt_per_range, 1) = 1，那么所有区间选取数量相同，当次训练选取训练集数据总数量:  i_batch_amt * array_length(i_batch_range, 1)
                                                                 -- 如果 i_batch_amt_per_range, i_batch_range 都是 null，那么继承 00_buff_slice_rand_pick 既有的配置。
, i_batch_range                      int4range[] default null    -- 训练集选取区间策略
)
as
$$
declare -- here
  v_model_code_6           char(6)              := (select model_code_6 from sm_sc.tb_classify_task where work_no = i_work_no);
  v_cur_learn_cnt          bigint               := (select learn_cnt from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号，当 v_cur_learn_cnt = 0 认作为新建的训练任务。
  v_input_asso_value       int[]   ;
  v_cur_node_nos           bigint[];
  v_cur_node_nos_last      bigint[];
  v_input_nodes            bigint[]             := (select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and (node_type in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_fn_type = '00_const') limit 1);
  v_cur_nn_depth           int                  := 0;   -- 深度层数游标
  v_max_nn_depth           int;

begin
  set search_path to public;
  
  -- ------------------------------- 配置小批量随机训练集数量 -----------------------------------------------
  -- 刚开始训练的准备阶段，则必须制定随机批量数量。后续接续训练，则可以空下来
  if v_cur_learn_cnt = 0 
    and i_batch_amt_per_range is null
  then 
    raise exception 'should configure i_batch_amt_per_range($3) at the beginning of new training task';
  -- 审计 i_batch_amt_per_range，i_batch_range 两个配置维长要对齐。
  elsif array_length(i_batch_amt_per_range, 1) <> array_length(i_batch_range, 1)
    and array_length(i_batch_amt_per_range, 1) <> 1
  then 
    raise exception 'should array_length(i_batch_amt_per_range, 1) = array_length(i_batch_range, 1) or array_length(i_batch_amt_per_range, 1) = 1';
  -- 审计随机批量配置是否超出训练集序号范围。
  elsif exists 
    (
      select  
      from unnest(i_batch_range) tb_a_range(a_range) 
      where 
        not
        (
          a_range 
          <@ 
          (select int4range(min(ord_no), max(ord_no),'[]') from sm_sc.tb_nn_train_input_buff where work_no = i_work_no)
        )
    )
  then 
    raise exception 'outof range of train dataset in sm_sc.tb_nn_train_input_buff.ord_no where work_no = i_work_no';
  -- i_batch_amt_per_range 不为空，则配置到 00_buff_slice_rand_pick 算子。
  elsif array_length(i_batch_amt_per_range, 1) = 1 and i_batch_range is not null
  then 
    i_batch_amt_per_range := sm_sc.fv_new(i_batch_amt_per_range, array[coalesce(array_length(i_batch_range, 1), 1)]);
  end if;
    
  if i_batch_range is null 
  then 
    update sm_sc.tb_nn_node tb_node
    set 
      node_fn_asso_value = 
        case
          when i_batch_amt_per_range is not null 
            then (select array[array[min(ord_no)], array[max(ord_no)], i_batch_amt_per_range] from sm_sc.tb_nn_train_input_buff where work_no = i_work_no)
          else 
            tb_node.node_fn_asso_value
        end
    where node_fn_type = '00_buff_slice_rand_pick'
      and node_type = 'input_01'
      and work_no = i_work_no
    returning node_fn_asso_value :: int[] into v_input_asso_value
    ;
    commit;
  else
    update sm_sc.tb_nn_node tb_node
    set 
      node_fn_asso_value = 
        array
        [
          a_lowers,
          a_uppers,
          i_batch_amt_per_range
        ]
    from 
    (
      select
        array_agg(lower(i_batch_range[a_range_no]) order by a_range_no) as a_lowers
      , array_agg(upper(i_batch_range[a_range_no]) - 1 order by a_range_no) as a_uppers
      from generate_series(1, array_length(i_batch_range, 1)) tb_a_range_no(a_range_no)
    ) tb_a_ranges(a_lowers, a_uppers)
    where node_fn_type = '00_buff_slice_rand_pick'
      and node_type = 'input_01'
      and work_no = i_work_no
    returning node_fn_asso_value :: int[] into v_input_asso_value
    ;
    commit;
  end if;
  
-- -- -- debug 
-- -- raise notice 'input_01 v_input_asso_value: %', array_dims(v_input_asso_value);
  
  -- ------------------------------- 审计部分 -----------------------------------------------
  -- 审计 训练任务是否存在
  if not exists(select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then
    raise exception 'no such work NO. = %!', i_work_no;

  -- 审计 输入节点是否在有效范围内
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') group by node_type having count(*) > 1)
  then
    raise exception 'more than 1 input node, or expect one-hot i_indepdt_*!';
    
  -- -- -- 审计 测试与验证的输入节点是否只有一个
  -- -- elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'prod_input' having count(*) > 1)
  -- -- then
  -- --   raise exception 'more than 1 prod_input node, or expect one-hot i_indepdt_*!';
  -- -- 
  -- 审计 输出节点是否在有效范围内
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('output_01', 'output_02', 'output_03', 'output_04') group by node_type having count(*) > 1)
  then
    raise exception 'more than 1 output node, or expect one-hot i_depdt_*!';
    
  -- 审计 训练集同一输入节点每条数据的规格同一
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      group by work_no 
      having (count(distinct array_dims(i_indepdt_01)) > 1
              or count(distinct array_dims(i_indepdt_02)) > 1
              or count(distinct array_dims(i_indepdt_03)) > 1
              or count(distinct array_dims(i_indepdt_04)) > 1
             )
    )
  then
    raise exception 'there should not be different array_dims(i_indepdt_*).';
    
  -- 审计 训练集同一输出节点每条数据的规格同一
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      group by work_no 
      having count(distinct array_dims(i_depdt_01)) > 1
        or count(distinct array_dims(i_depdt_02)) > 1
        or count(distinct array_dims(i_depdt_03)) > 1
        or count(distinct array_dims(i_depdt_04)) > 1
    )
  then
    raise exception 'there should not be different arrray_length(i_depdt_*, 2).';
    
  -- 审计 训练集不存在 null 之元素
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_01)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_02)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_03)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_04)
    )
  then 
    raise exception 'there are some null element in input i_indepdt_*.';
    
  -- 审计 训练集判断结果不存在 null 之元素
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_01)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_02)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_03)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_04)
    )
  then 
    raise warning 'there are some null element in input i_depdt_*.';

  -- 审计 小批量随机的训练集记录数目是否符合要求
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' and node_fn_type = '00_buff_slice_rand_pick')
    -- and v_input_cnt < sm_sc.fv_aggr_slice_max(v_input_asso_value[2 : 2][ : ])
    and exists (select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no group by work_no having count(*) < sm_sc.fv_aggr_slice_max(v_input_asso_value[2 : 2][ : ]))
  then 
    raise exception 'count of training data should not be less than slices'' uppers of input node.';
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' and node_fn_type = '00_buff_slice_rand_pick')
    and true = any(v_input_asso_value[3 : 3][ : ] >` (v_input_asso_value[2 : 2][ : ] -` v_input_asso_value[1 : 1][ : ] +` 1))
  then 
    raise exception 'there should be not any 00_buff_slice_rand_pick more than its slice range.';
    
  -- 00_const 节点的算子超参数审计工作
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_fn_type = '00_const' and node_fn_asso_value is null)
  then 
    raise exception 'there should be not any null node_fn_asso_value at 00_const.';
    
  -- 非 weight 的 00_const 节点的 p_node_depdt 初始化审计工作
  elsif exists 
    (
      select  
      from sm_sc.tb_nn_node 
      where work_no = i_work_no 
        and node_fn_type = '00_const' 
        and node_type is distinct from 'weight'
        and 
        (
          p_node_depdt is null
          -- -- -- 以下注释代码的数组规格审查，阻碍了点算的广播特性
          -- -- or array_length(node_depdt_vals, 1) <> (node_fn_asso_value[1] :: int) 
          -- -- or array_length(node_depdt_vals, 2) <> (node_fn_asso_value[2] :: int)
          -- -- or array_length(node_depdt_vals, 3) <> (node_fn_asso_value[3] :: int)
          -- -- or array_length(node_depdt_vals, 4) <> (node_fn_asso_value[4] :: int)
        )
    )
  then 
    raise exception 'there should be not any null p_node_depdt or unmatch length configured by node_fn_asso_value at 00_const.';

  -- 审计 算子入参数量
  --     审计双目
  elsif exists 
  (
    select 
    from sm_sc.tb_dic_enum tb_a_enum
    inner join sm_sc.tb_nn_node tb_a_node 
      on tb_a_node.node_fn_type = tb_a_enum.enum_key
        and tb_a_node.work_no = i_work_no
    inner join sm_sc.tb_nn_path tb_a_path 
      on tb_a_path.fore_node_no = tb_a_node.node_no
        and tb_a_path.work_no = i_work_no
    where tb_a_enum.enum_name = 'node_fn_type'
      and tb_a_enum.enum_group = '2_p'
    group by tb_a_path.fore_node_no
    having count(tb_a_path.path_ord_no) <> 2
  )
  then
    raise exception 'too many or too few pathes point to an fn with 2_p.';
  
  --     审计单目
  elsif exists
  (
    select 
    from sm_sc.tb_dic_enum tb_a_enum
    inner join sm_sc.tb_nn_node tb_a_node 
      on tb_a_node.node_fn_type = tb_a_enum.enum_key
        and tb_a_node.work_no = i_work_no
    inner join sm_sc.tb_nn_path tb_a_path 
      on tb_a_path.fore_node_no = tb_a_node.node_no
        and tb_a_path.work_no = i_work_no
    where tb_a_enum.enum_name = 'node_fn_type'
      and tb_a_enum.enum_group = '1_p'
    group by tb_a_path.fore_node_no
    having count(tb_a_path.path_ord_no) > 1
  )
  then 
    raise exception 'too many pathes point to an fn with 1_p.';

  -- 审计 是否存在孤岛网络
  elsif exists 
  (
    with recursive
    cte_main_nodes as
    (
      select 
        node_no
      from sm_sc.tb_nn_node
      where work_no = i_work_no
        and node_type in ('output_01', 'output_02', 'output_03', 'output_04')
      union
      select 
        tb_a_incre.node_no
      from cte_main_nodes tb_a_main
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.fore_node_no = tb_a_main.node_no
      inner join sm_sc.tb_nn_node tb_a_incre
        on tb_a_incre.node_no = tb_a_path.back_node_no
          and tb_a_incre.work_no = i_work_no
    )
    select 
    from cte_main_nodes
    having count(node_no) < (select count(node_no) from sm_sc.tb_nn_node where work_no = i_work_no)
  )
  then
    raise exception 'there should not be alone nodes.';

  -- 审计通过，新的任务的初始化工作(learn_cnt = 0)
  elsif v_cur_learn_cnt = 0
  then
    
-- -- -- debug
-- raise notice 'L298. begin: v_cur_node_nos: %;', v_cur_node_nos;
  
    update sm_sc.tb_nn_node tb_a_tar
    set 
      -- -- node_depdt_len   = 
      -- --   array 
      -- --   [
      -- --     case tb_a_tar.node_fn_type 
      -- --       when '00_full_dataset' 
      -- --         then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
      -- --       when '00_buff_slice_rand_pick' 
      -- --         then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
      -- --     end,
      -- --     v_input_len_x
      -- --   ]
      -- -- ,                        -- -- array[array_length(tb_a_sour.i_indepdt_01, 1), array_length(tb_a_sour.i_indepdt_01, 2)],
      is_fore_node = 
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then false 
          when '00_buff_slice_rand_pick' 
            then true 
        end
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type = 'input_01'
    ;
    commit;
    
    update sm_sc.tb_nn_node tb_a_tar
    set 
      is_fore_node = (select is_fore_node from sm_sc.tb_nn_node where work_no = i_work_no and tb_a_tar.node_type = 'input_01')
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
    ;
    commit;
    
    -- 00_const 节点的准备工作
    update sm_sc.tb_nn_node
    set 
      is_fore_node = false
    , is_back_node = false
    where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
    ;
    commit;
    
-- -- -- debug
-- raise notice 'L226. begin: node_depdt_len of input: %;', (select node_depdt_len from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01');
  
    -- 标记前向传播节点
    -- 缺省 true. 从神经网络的初始起点（包括）开始审计，至 01_prod_mx, 05_conv_2d_grp_x, rand 类算子为止（不包括），沿途皆为 false    
    -- 更新 is_fore_node
    v_cur_node_nos := 
      (
        select array_agg(node_no) from sm_sc.tb_nn_node 
        where work_no = i_work_no 
          and 
          (
            node_type in ('input_01', 'input_02', 'input_03', 'input_04')
            or node_fn_type = '00_const' and node_type <> 'weight'
          )
      );     -- -- v_input_nodes
    while array_length(v_cur_node_nos, 1) > 0
    loop 
      -- pg 暂不支持 recursive cte 使用 aggr, 先只能用循环
      with
      cte_fore_nodes as 
      (
        select 
          tb_a_fore.node_no,
          max(tb_a_fore.node_fn_type) as node_fn_type
        from sm_sc.tb_nn_path tb_a_fore_path
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
            and tb_a_fore.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_path tb_a_back_path
          on tb_a_back_path.fore_node_no = tb_a_fore.node_no
            and tb_a_back_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.node_no = tb_a_back_path.back_node_no
            and tb_a_back.work_no = i_work_no   -- 2021082501
        where tb_a_fore_path.back_node_no = any(v_cur_node_nos)
          and tb_a_fore.node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
          and tb_a_fore_path.work_no = i_work_no   -- 2021082501
        group by tb_a_fore.node_no
        having count(distinct tb_a_back.node_no) =
                 count(distinct tb_a_back.node_no) 
                   filter 
                   (
                     where tb_a_back.is_fore_node is false
                   )
      ),
      cte_upd as
      (
        update sm_sc.tb_nn_node tb_a_tar
        set 
          is_fore_node = false
        from cte_fore_nodes tb_a_sour
        where tb_a_sour.node_no = tb_a_tar.node_no
          and tb_a_tar.work_no = i_work_no   -- 2021082501
      )
      select 
        array_agg(distinct node_no) into v_cur_node_nos 
      from cte_fore_nodes
      where node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
      ;
      commit;
  
-- -- -- debug
-- raise notice 'L345.: v_cur_node_nos: %;', v_cur_node_nos;
  
    end loop;
    
    -- 标记反向传播节点
    -- 缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true

    update sm_sc.tb_nn_node
    set 
      is_back_node = true
    where work_no = i_work_no   -- 2021082501
      -- and node_no = any(v_init_nodes)
      and node_type = 'weight'
    ;
    commit;

    with recursive
    cte_back_nodes as
    (
      select 
        tb_a_main.node_no
      from sm_sc.tb_nn_node tb_a_main
      where tb_a_main.work_no = i_work_no   -- 2021082501
        and tb_a_main.is_back_node is true
      union
      select
        tb_a_path.fore_node_no
      from cte_back_nodes tb_a_back
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.back_node_no = tb_a_back.node_no 
      where tb_a_path.work_no = i_work_no   -- 2021082501
    )
    update sm_sc.tb_nn_node tb_a_tar
    set
      is_back_node = true
    from cte_back_nodes tb_a_sour
    where tb_a_sour.node_no = tb_a_tar.node_no
      and tb_a_tar.work_no = i_work_no   -- 2021082501
      -- and tb_a_tar.node_type <> 'output_01'
    ;
    commit;   
  end if;
  
  -- 每次启动训练，都重新计算 node_depdt_len，用于适配动态调整随机小批量的每个类别抽样数量
  update sm_sc.tb_nn_node tb_a_tar
  set 
    node_depdt_len   = 
      array 
      [
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
          when '00_buff_slice_rand_pick' 
            then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
        end
      ]
      || 
      (
        select 
          array_agg(array_length(i_indepdt_01, a_cur) order by a_cur)
        from (select i_indepdt_01 from sm_sc.tb_nn_train_input_buff where work_no = i_work_no limit 1) tb_a_buff(i_indepdt_01)
          , generate_series(1, array_ndims(tb_a_buff.i_indepdt_01)) tb_a_cur(a_cur)
      )
  where tb_a_tar.work_no = i_work_no   -- 2021082501
    and tb_a_tar.node_type = 'input_01'
  ;
  commit;
  
  -- 初始化其他模态输入节点的规格
  update sm_sc.tb_nn_node tb_a_tar
  set 
    node_depdt_len = 
      array 
      [
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
          when '00_buff_slice_rand_pick' 
            then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
        end
      ]
      || 
      (
        select 
          array_agg(array_length(i_indepdt_ex, a_cur) order by a_cur)
        from 
          (select 
             case tb_a_tar.node_type 
               when 'input_02'
                 then i_indepdt_02
               when 'input_03'
                 then i_indepdt_03
               when 'input_04'
                 then i_indepdt_04
             end                    
             as i_indepdt_ex 
           from sm_sc.tb_nn_train_input_buff where work_no = i_work_no limit 1
          ) tb_a_buff(i_indepdt_ex)
        , generate_series(1, array_ndims(tb_a_buff.i_indepdt_ex)) tb_a_cur(a_cur)
      )
  where tb_a_tar.work_no = i_work_no   -- 2021082501
    and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
  ;
  commit;

  -- 对齐训练次数；训练过程中，输入、偏移量系数节点不参与传播
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = case when node_type in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_fn_type = '00_const' then i_limit_train_times else v_cur_learn_cnt end,
    learn_cnt_back = case when node_type in ('input_01', 'input_02', 'input_03', 'input_04') or (node_fn_type = '00_const' and node_type is distinct from 'weight') then i_limit_train_times else v_cur_learn_cnt end
  where work_no = i_work_no   -- 2021082501
  ;
  commit;

-- -- -- debug
-- raise notice 'L102. begin: v_cur_learn_cnt: %', v_cur_learn_cnt;

  -- 传播规划
  -- 先做个 learn_cnt_.. 的负标记
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = - 1,
    learn_cnt_back = - 1
  where work_no = i_work_no
    and ((node_type not in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_type is null) and node_fn_type <> '00_const')
  ;
  commit;
  -- 前向打标
  v_cur_node_nos := v_input_nodes;
  v_cur_nn_depth := 0;

-- -- -- debug
-- raise notice 'L224. begin: v_cur_node_nos: %; v_input_nodes: %; v_cur_nn_depth: %;', v_cur_node_nos, v_input_nodes, v_cur_nn_depth;

  update sm_sc.tb_nn_node 
  set nn_depth_no = 0 
  where work_no = i_work_no 
    and (node_type in ('input_01', 'input_02', 'input_03', 'input_04') or (node_fn_type = '00_const' and node_type is distinct from 'weight'));
  commit;

  while 
    exists 
    (
      select         
      from sm_sc.tb_nn_node tb_a_fore
      where tb_a_fore.work_no = i_work_no    -- 2021082501
        and tb_a_fore.learn_cnt_fore < v_cur_learn_cnt
      limit 1
    )
  loop
    v_cur_nn_depth := v_cur_nn_depth + 1;
    v_cur_node_nos_last := v_cur_node_nos;
    with 
    -- 由 learn_cnt_fore 约束，找到与上次前向传播有关的，可前向计算的节点
    cte_y as
    (
      select 
        tb_a_fore.node_no
      -- , tb_a_fore.node_fn_type
      from sm_sc.tb_nn_path tb_a_fore_path
      inner join sm_sc.tb_nn_node tb_a_fore
        on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
          and tb_a_fore.work_no = i_work_no   -- 2021082501
      inner join sm_sc.tb_nn_path tb_a_back_path
        on tb_a_back_path.fore_node_no = tb_a_fore.node_no
          and tb_a_back_path.work_no = i_work_no   -- 2021082501
      inner join sm_sc.tb_nn_node tb_a_back
        on tb_a_back.node_no = tb_a_back_path.back_node_no
          and tb_a_back.work_no = i_work_no   -- 2021082501
      where tb_a_fore_path.work_no = i_work_no   -- 2021082501
        and tb_a_fore.learn_cnt_fore < v_cur_learn_cnt 
        and tb_a_fore_path.back_node_no = any(v_cur_node_nos || v_input_nodes) 
      group by tb_a_fore.node_no -- , tb_a_fore.node_fn_type
      having count(*) filter (where tb_a_back.learn_cnt_fore > tb_a_fore.learn_cnt_fore) = count(*)
    ),
    cte_upd_fore_label as
    (
      update sm_sc.tb_nn_node tb_a_tar
      set 
        learn_cnt_fore = v_cur_learn_cnt,
        nn_depth_no = v_cur_nn_depth
      from cte_y tb_a_y
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_y.node_no = tb_a_tar.node_no
      returning tb_a_tar.node_no
    )
    select array_agg(node_no) into v_cur_node_nos from cte_upd_fore_label
    ;
    commit;

-- -- debug
-- raise notice 'L256. begin: nn_depth updated nodes = %: %', v_cur_node_nos, v_cur_nn_depth;
    if v_cur_node_nos is not distinct from v_cur_node_nos_last
    then 
      raise exception 'v_cur_node_nos is the same as v_cur_node_nos_last: %.', coalesce(v_cur_node_nos_last :: text, 'null');
    end if;

  end loop;

  v_max_nn_depth := v_cur_nn_depth;

  -- 修改 weight 节点的深度， = min(下层 01_prod_mx 的深度) - 1，保证反向传播及时传递到 weight 节点
  with
  cte_nn_depth as
  (
    select 
      tb_a_back.node_no,
      min(tb_a_fore.nn_depth_no) - 1 as nn_depth_no
    from sm_sc.tb_nn_node tb_a_back
    inner join sm_sc.tb_nn_path tb_a_path
      on tb_a_back.node_no = tb_a_path.back_node_no
        and tb_a_path.work_no = i_work_no
    inner join sm_sc.tb_nn_node tb_a_fore
      on tb_a_path.fore_node_no = tb_a_fore.node_no
        and tb_a_fore.work_no = i_work_no
    where tb_a_back.work_no = i_work_no
      and tb_a_back.node_type = 'weight'
    group by tb_a_back.node_no
  )
  update sm_sc.tb_nn_node tb_a_back
  set nn_depth_no = tb_a.nn_depth_no
  from cte_nn_depth tb_a
  where tb_a_back.work_no = i_work_no
    and tb_a.node_no = tb_a_back.node_no
    -- and tb_a_back.node_type = 'weight'
  ;
  commit;

  -- 传播规划完毕，恢复 learn_cnt 为真实标记
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = v_cur_learn_cnt,
    learn_cnt_back = v_cur_learn_cnt
  where work_no = i_work_no   -- 2021082501
    and (node_type not in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') and node_fn_type <> '00_const')
  ;
  commit;

  
  -- ---------- 填充缺省协参部分。遍历 nn_depth，每次遍历，首先更新非 weight 节点的 node_fn_asso_value，再更新 node_depdt_len，结束 nn_depth 遍历，最后更新 weight 的 node_depdt_vals  ----------------------

  -- ------------------------------- 准备数据 -----------------------------------------------
-- -- debug
raise notice 'prepare data begin!!!!!!'; 

  -- ------------------------------- 审计数据部分 -----------------------------------------------
-- -- debug
raise notice 'check data len begin!!!!!!';

  -- 前向通烟更新 node_depdt_len

  update sm_sc.tb_nn_node
  set node_depdt_len = node_fn_asso_value :: int[]
  where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
  ;
  commit;

    
  v_cur_nn_depth = 1;
  while v_cur_nn_depth <= v_max_nn_depth
  loop

    -- np aggr 情况，化为 1p
    with
    cte_concat_dims_change as 
    (
      select 
        tb_a_fore.node_no
      , (
          array 
          [
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[1], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
              else max(coalesce(tb_a_back_pn.node_depdt_len[1], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_y')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[2], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
              else max(coalesce(tb_a_back_pn.node_depdt_len[2], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[3], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
              else max(coalesce(tb_a_back_pn.node_depdt_len[3], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x3')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[4], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
              else max(coalesce(tb_a_back_pn.node_depdt_len[4], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x4')
            end
          ] 
        )[1 : max(array_length(tb_a_back_pn.node_depdt_len, 1))]
          as a_pn_len
      -- 若干类型算子粒度的超参数在此自动配置填充，而不需要手动配置。参看字典表对各类算子 node_fn_asso_value 的解释
      , case tb_a_fore.node_fn_type
          when '06_aggr_mx_avg'
            then array[count(tb_a_back_pn.node_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')]
          when '06_aggr_mx_concat_y'
            then array_agg(tb_a_back_pn.node_depdt_len[1] order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
          when '06_aggr_mx_concat_x'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[2], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
          when '06_aggr_mx_concat_x3'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[3], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
          when '06_aggr_mx_concat_x4'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[4], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
        end as a_node_fn_asso_value
      from sm_sc.tb_nn_node tb_a_fore
      inner join sm_sc.tb_nn_path tb_a_path_pn
        on tb_a_path_pn.work_no = i_work_no   -- 2021082501
          and tb_a_path_pn.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.tb_nn_node tb_a_back_pn
        on tb_a_back_pn.work_no = i_work_no   -- 2021082501
          and tb_a_back_pn.node_no = tb_a_path_pn.back_node_no
      where tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
        and tb_a_fore.node_fn_type in ('06_aggr_mx_avg', '06_aggr_mx_concat_y', '06_aggr_mx_concat_x', '06_aggr_mx_concat_x3', '06_aggr_mx_concat_x4')
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    ),
    cte_p1_p2 as 
    (
      select 
        tb_a_fore.node_no,
        coalesce(tb_a_back_pn.a_pn_len, tb_a_back_p1.node_depdt_len) as a_p1_len,
        case when tb_a_fore.node_fn_type not like '06_aggr_mx_%' then tb_a_back_p2.node_depdt_len end as a_p2_len,
        tb_a_back_pn.a_node_fn_asso_value
      from sm_sc.tb_nn_node tb_a_fore
      inner join sm_sc.tb_nn_path tb_a_path_p1
        on tb_a_path_p1.work_no = i_work_no   -- 2021082501
          and tb_a_path_p1.path_ord_no = 1 -- -- 如果与 前向节点的 path_ord_no 冲突，使用 00_none 算子隔离。
          and tb_a_path_p1.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.tb_nn_node tb_a_back_p1
        on tb_a_back_p1.work_no = i_work_no   -- 2021082501
          and tb_a_back_p1.node_no = tb_a_path_p1.back_node_no
      left join sm_sc.tb_nn_path tb_a_path_p2
        on tb_a_path_p2.work_no = i_work_no   -- 2021082501
          and tb_a_path_p2.path_ord_no = 2 -- -- 如果与 前向节点的 path_ord_no 冲突，使用 00_none 算子隔离。
          and tb_a_path_p2.fore_node_no = tb_a_fore.node_no
      left join sm_sc.tb_nn_node tb_a_back_p2
        on tb_a_back_p2.work_no = i_work_no   -- 2021082501
          and tb_a_back_p2.node_no = tb_a_path_p2.back_node_no
      left join cte_concat_dims_change tb_a_back_pn
        on tb_a_back_pn.node_no = tb_a_fore.node_no
      where tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
    )
    update sm_sc.tb_nn_node tb_a_fore
    set 
      node_fn_asso_value = 
        coalesce(tb_a_fore.node_fn_asso_value, tb_a_p1_p2.a_node_fn_asso_value) 
    , node_depdt_len = 
        sm_sc.fv_lambda_arr_len
        (
          tb_a_fore.node_no,
          tb_a_fore.node_fn_type, 
          tb_a_p1_p2.a_p1_len :: int[], 
          tb_a_p1_p2.a_p2_len :: int[],
          coalesce(tb_a_fore.node_fn_asso_value, tb_a_p1_p2.a_node_fn_asso_value)
        )
    from cte_p1_p2 tb_a_p1_p2
    where tb_a_fore.work_no = i_work_no   -- 2021082501
      and tb_a_fore.node_no = tb_a_p1_p2.node_no
    ;
    commit;

    v_cur_nn_depth := v_cur_nn_depth + 1;
  end loop;

-- ---------------------------------------------------------------
  -- weight 的高宽，由前向算子的高宽反推得出
  update sm_sc.tb_nn_node tb_a_back 
  set 
    node_fn_asso_value = 
      sm_sc.fv_nn_weight_len
      (
        tb_a_fore.node_fn_type
      , tb_a_path.path_ord_no
      , tb_a_fore.node_fn_asso_value
      )
  from sm_sc.tb_nn_path tb_a_path, sm_sc.tb_nn_node tb_a_fore 
  where tb_a_back.work_no = i_work_no   -- 2021082501
    and tb_a_path.work_no = i_work_no   -- 2021082501
    and tb_a_fore.work_no = i_work_no   -- 2021082501
    and tb_a_path.back_node_no = tb_a_back.node_no
    and tb_a_fore.node_no = tb_a_path.fore_node_no
    and tb_a_back.node_type = 'weight'     
    and tb_a_back.node_fn_asso_value is null                   -- weight 节点的 tb_a_back.node_fn_type is null
    -- and tb_a_path.path_ord_no = 2
  ;
  commit;

-- ------------------------------------------------------------------------------
  -- 首轮训练之前，以 node_depdt_len 为依据，确定 weight 的矩阵规格
  if v_cur_learn_cnt = 0
  then    
    -- 初始化 w 权值，为避免鞍点，采用随机
    update sm_sc.tb_nn_node 
    set 
      p_node_depdt = 
        sm_sc.__fv_set_kv
        (
          sm_sc.fv_new_randn
          (
            0.0 :: float 
            
          -- , 1.0 / 0.6745  :: float     -- 假定，权重（斜率）是无偏的，大于 0.0 和小于 0.0 的概率相同，都是 50%
          --                              -- 参看标准正态分布表
          --                              -- 误差概率分布：当统计值的误差介于±0.6745(标准误差)范围时，概率为 50%；误差介于±2范围时，概率为95%；误差介于±3范围时，概率为99.7%；
          --                              -- 也可以自行修改该标准差，使用权重参数的 xaviar 初始化, He 初始化 等方法
          , 1.0 
            / 
            power
            (
              greatest
              (
                node_fn_asso_value[array_length(node_fn_asso_value, 1) - 1]
              , node_fn_asso_value[array_length(node_fn_asso_value, 1)]
              )
            , 0.5
            )
          , node_fn_asso_value :: int[]
          )
        , v_model_code_6 || 
          '_' || work_no :: varchar   ||
          '_' || node_no :: varchar ||
          '__d__'   -- 'depdt'
        )
    -- , node_depdt_len = node_fn_asso_value
    where work_no = i_work_no   -- 2021082501
      and node_type = 'weight'          -- weight 节点的 node_fn_type = '' 或者 is null
      and node_fn_type = '00_const'
    ;
    commit;
  end if;

-- ------------------------------------------------------------------------------
  -- 审查 nn 节点算子类型的超参数是否完善配置
  v_cur_node_nos := 
    (
      with 
      cte_fn_asso_value_range
      as 
      (
        select 
          enum_group
        , sm_sc.fa_range_or(int4range(lower(enum_range) :: int, upper(enum_range) :: int, '[]') :: int4multirange) as a_fn_asso_value_range
        from sm_sc.tb_dic_enum
        where enum_name = 'node_fn_asso_value'
        group by enum_group
      )
      select 
        array_agg(distinct tb_a_node.node_no)
      from cte_fn_asso_value_range tb_a_dic
      inner join sm_sc.tb_nn_node tb_a_node 
        on tb_a_node.node_fn_type = tb_a_dic.enum_group
      where tb_a_node.work_no = i_work_no
        and (not array_length(tb_a_node.node_fn_asso_value, 1) <@ tb_a_dic.a_fn_asso_value_range)
            or tb_a_node.node_fn_asso_value is null
    )
  ;

  delete from sm_sc.__vt_nn_node where work_no = i_work_no;
  insert into sm_sc.__vt_nn_node
  (
    work_no                      ,
    node_no                  ,        
    node_type                    ,
    node_fn_type                 ,
    node_fn_asso_value      ,
    nn_depth_no                  ,
    p_node_depdt                  
  )
  select 
    work_no                      ,
    node_no                  ,        
    node_type                    ,
    node_fn_type                 ,
    node_fn_asso_value      ,
    nn_depth_no                  ,
    case when node_type = 'weight' or node_fn_type = '00_const' then p_node_depdt end as p_node_depdt                 
  from sm_sc.tb_nn_node
  where work_no = i_work_no
  ;    
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  insert into sm_sc.__vt_nn_path
  (
    work_no     
  , fore_node_no
  , path_ord_no 
  , back_node_no
  )
  select 
    work_no     
  , fore_node_no
  , path_ord_no 
  , back_node_no
  from sm_sc.tb_nn_path
  where work_no = i_work_no
  ;
  commit;

  if array_length(v_cur_node_nos, 1) > 0 
  then 
    raise warning 'these node_fn_asso_value is null, please check nodes: %.', v_cur_node_nos;
  else 
    raise notice 'prepare well done!!!!!!';
  end if;
  
  update sm_sc.tb_nn_node 
  set dropout_ratio = 0.0
  where work_no = i_work_no
    and dropout_ratio is null
  ;
  
  update sm_sc.tb_nn_node 
  set dropout_rescale = 1.0
  where work_no = i_work_no
    and dropout_rescale is null
  ;
  
  update sm_sc.tb_nn_node 
  set dropout_depdt_val = 0.0
  where work_no = i_work_no
    and dropout_depdt_val is null
  ;
  
  update sm_sc.tb_nn_node 
  set is_dropout = false
  where work_no = i_work_no
    and is_dropout is null
  ;
  
end
$$
language plpgsql;




-- -- 执行准备和检查
-- call sm_sc.prc_nn_prepare_p
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   2500      -- ,
--   -- 200
-- );
-- 参看设计文档 《反向传播链式求导步骤编排》

-- 初始学习率：1e-4
-- https://www.zhihu.com/question/387050717

drop procedure if exists sm_sc.prc_nn_train(bigint, float, float, boolean, float, float);
create or replace procedure sm_sc.prc_nn_train
(
  i_work_no                          bigint                             -- 训练任务编号
-- , i_loss_fn_type                     varchar(32)[]                      -- 损失函数选取类型。1: 最小二乘法; 2: 交叉熵
, i_learn_rate                       float    default    0.0003         -- 学习率
, i_loss_delta_least_stop_threshold  float    default    0.0001         -- 触发收敛的损失函数梯度阈值
, i_is_adam                          boolean  default    false          -- 是否开启 adam 优化
, i_l1                               float    default    null           -- L1 正则化率，缺省不做 L1 正则化
, i_l2                               float    default    null           -- L2 正则化率，缺省不做 L2 正则化
)
as
$$
declare -- here
  v_learn_cnt_init          int               := (select learn_cnt from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号初始值
  v_cur_learn_cnt           int               := v_learn_cnt_init;   -- 训练次数序号
  v_depdt_01                float[]  ; -- -- 训练集本轮采样之多模态 01
  v_depdt_02                float[]  ; -- -- 训练集本轮采样之多模态 02
  v_depdt_03                float[]  ; -- -- 训练集本轮采样之多模态 03
  v_depdt_04                float[]  ; -- -- 训练集本轮采样之多模态 04
  v_cur_node_nos            bigint[];  -- for debug
  v_cur_loss                float;     -- 损失函数值
  v_input_nodes             bigint[]          := 
    (
      select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') -- limit 1
    );
  v_cur_nn_depth            int               := 0;   -- 深度层数游标
  v_nn_depth                int;                      -- 深度层数
  v_limit_train_times       int               := (select learn_cnt_fore from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' limit 1);
  v_beta_l1                 float    := 0.9;      -- Adam 梯度下降算法一阶矩等比衰减系数
  v_beta_l2                 float    := 0.999;    -- Adam 梯度下降算法二阶矩等比衰减系数
  
  -- 求导时，需要因变量的算子类型集合
  v_arr_fn_depdt_delta      varchar(64)[]   :=   (select array_agg(enum_key) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method' and (substr(enum_value, 5, 1) = '1' or substr(enum_value, 6, 1) = '1'));
  v_loss_fn_type            varchar(32)     := (select loss_fn_type from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号初始值

  v_is_input_02             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_02');
  v_is_input_03             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_03');
  v_is_input_04             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_04');
  
  v_is_output_01            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_01')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_01 is not null))
  ;
  v_is_output_02            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_02')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_02 is not null))
  ;
  v_is_output_03            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_03')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_03 is not null))
  ;
  v_is_output_04            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_04')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_04 is not null))
  ;
begin
  set search_path to public;
  
  -- -- -- 强制开启并行
  -- -- -- select * from pg_settings where name ~ 'paral' or name in ('max_worker_processes') limit 100
  -- -- -- show max_worker_processes
  set min_parallel_table_scan_size = 0;
  set min_parallel_index_scan_size = 0;
  set force_parallel_mode = 'off';
  set max_parallel_workers_per_gather = 64;
  set parallel_setup_cost = 0;
  set parallel_tuple_cost = 0.0;

  -- 清理旧有训练结果
  delete from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  commit;
  
  -- -- -- -- 每次训练前，调整抽样数量等策略
  -- -- -- call sm_sc.prc_nn_prepare_per_train(v_cur_learn_cnt, v_cur_loss, ...);

  -- 审计 训练是否已经满足损失函数最低阈值
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no and loss_delta <= i_loss_delta_least_stop_threshold)
  then
    raise notice 'The loss_delta is adequately less than i_loss_delta_least_stop_threshold already!';
    return;
  end if;

  v_nn_depth := (select max(nn_depth_no) from sm_sc.tb_nn_node where work_no = i_work_no);

  raise notice 'Training begin; at %', date_trunc('milliseconds', now());

  -- 开始训练
  while 
  (
    i_loss_delta_least_stop_threshold <= v_cur_loss
    or v_cur_loss is null
  )
    and v_cur_learn_cnt < v_limit_train_times
  loop

-- -- -- debug
-- raise notice 'debug 001. step begin: v_cur_learn_cnt: %; at %;', v_cur_learn_cnt, date_trunc('milliseconds', now());

    -- 触发 dropout 机制，根据 dropout 概率，确定 dropout 节点清单
    update sm_sc.tb_nn_node
    set is_dropout = (dropout_ratio > random())
    where work_no = i_work_no
      and dropout_ratio > 0.0
    ;
    commit;
    
    -- 填充 dropout 节点的因变量
    update sm_sc.tb_nn_node
    set node_depdt_vals = array_fill(dropout_depdt_val, node_depdt_len)
    where work_no = i_work_no
      and is_dropout
    ;

    if exists
       (
         select  from sm_sc.tb_nn_node 
         where work_no = i_work_no 
           and node_fn_type = '00_buff_slice_rand_pick' 
           and node_type = 'input_01'
           limit 1
       )
    then
      -- 如果对训练集采用小批量随机采样，那么每轮初始化 input 值，以支持不同轮次更换训练集
      with
      cte_slice_rand_pick_agg as 
      (
        -- 要确保各模态的小批量随机采样序号集合一致。
        -- 规约：以 input_01 模态的 00_buff_slice_rand_pick 算子超参数配置为基准。而忽略其他模态的配置。
        select  
          array_agg
          (
            int4multirange
            (
              int4range
              (
                o_ord_no :: int
              , o_ord_no :: int + 1
              , '[)'
              )
            )
          )                                                                as a_pick_depdt_idx
        , array_agg(tb_a_idx.o_rand_pick_indepdt_01 order by o_ord_no)     as a_rand_pick_indepdt_01
        , case when v_is_input_02 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_02, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_02
        , case when v_is_input_03 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_03, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_03
        , case when v_is_input_04 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_04, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_04
        from sm_sc.tb_nn_node tb_a_node,
          sm_sc.ft_nn_buff_slice_rand_pick
          (
            -- 参看 00_buff_slice_rand_pick 的 node_fn_asso_value 字典表
            i_work_no,
            (
              select 
                array_agg(int4range(tb_a_node.node_fn_asso_value[1][a_ord] :: int, tb_a_node.node_fn_asso_value[2][a_ord] :: int + 1, '[)') order by a_ord)
              from generate_series(1, array_length(tb_a_node.node_fn_asso_value, 2)) tb_a(a_ord)
            )
          , sm_sc.fv_mx_ele_2d_2_1d(tb_a_node.node_fn_asso_value[3 : 3][ : ] :: int[])
          ) tb_a_idx(o_ord_no, o_rand_pick_indepdt_01, o_rand_pick_indepdt_02, o_rand_pick_indepdt_03, o_rand_pick_indepdt_04)
        where tb_a_node.work_no = i_work_no
          and tb_a_node.node_type = 'input_01'
          and tb_a_node.node_fn_type = '00_buff_slice_rand_pick'
      )
      update sm_sc.tb_nn_node tb_a_tar
      set 
        node_depdt_vals = 
          case 
            when tb_a_tar.node_type = 'input_01'
              then a_rand_pick_indepdt_01
            when tb_a_tar.node_type = 'input_02'
              then a_rand_pick_indepdt_02
            when tb_a_tar.node_type = 'input_03'
              then a_rand_pick_indepdt_03
            when tb_a_tar.node_type = 'input_04'
              then a_rand_pick_indepdt_04
          end
      , pick_depdt_idx = a_pick_depdt_idx
      from cte_slice_rand_pick_agg tb_a_sour
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_tar.node_fn_type = '00_buff_slice_rand_pick' 
        and tb_a_tar.node_type in ('input_01', 'input_02', 'input_03', 'input_04')
      ;
      commit;
    elsif exists(select  from sm_sc.tb_nn_node where work_no = i_work_no and node_fn_type = '00_full_dataset' and node_type = 'input_01')
    then 
      -- 如果不做小批量随机采样，那么使用训练集全集
      with
      cte_slice_rand_pick as 
      (
        select 
          array_agg(i_indepdt_01)        as a_x_vals_01,
          array_agg(i_indepdt_02)        as a_x_vals_02,
          array_agg(i_indepdt_03)        as a_x_vals_03,
          array_agg(i_indepdt_04)        as a_x_vals_04,
          count(*) :: int       as a_x_cnt
        from sm_sc.tb_nn_train_input_buff
        where work_no = i_work_no
      )
      update sm_sc.tb_nn_node tb_a_tar
      set 
        node_depdt_vals = case tb_a_tar.node_type when 'input_01' then a_x_vals_01 when 'input_02' then a_x_vals_02 when 'input_03' then a_x_vals_03 when 'input_04' then a_x_vals_04 end  
      , pick_depdt_idx = array[int4multirange(int4range(1, a_x_cnt + 1, '[)'))]
      from cte_slice_rand_pick tb_a_sour
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_tar.node_type in ('input_01', 'input_02', 'input_03', 'input_04')
      ;
      commit;
    end if;

    v_cur_nn_depth := 0;

    -- 前向传播

-- -- for debug
-- v_cur_node_nos := 
--   (
--     select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') -- limit 1
--   );
      
    while v_cur_nn_depth < v_nn_depth
    loop
      v_cur_nn_depth := v_cur_nn_depth + 1;

-- -- -- debug
-- raise notice 'debug 002. step fore begin: v_cur_learn_cnt: %; v_cur_nn_depth: %; v_learn_cnt_init: %; at %;', v_cur_learn_cnt, v_cur_nn_depth, v_learn_cnt_init, date_trunc('milliseconds', now());

      -- 按照 nn_depth_no，逐层前向传播。初始节点只前向传播一次
      with 
      -- 参数准备，单目、双目（三目）算子
      cte_indepdt as
      (
        select 
          tb_a_fore.node_no
        , tb_a_fore.node_fn_type
        , sm_sc.fa_range_or(tb_a_back.pick_depdt_idx) as a_pick_depdt_idx
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 1 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_1st
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 2 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_2nd
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 3 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_3rd
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 1 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_1st
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 2 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_2nd
     -- , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 3 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_3rd
        , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 1 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_1st_back
        , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 2 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_2nd_back
     -- , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 3 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_3rd_back
        from sm_sc.tb_nn_node tb_a_fore
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.work_no = i_work_no   -- 2021082501
            and tb_a_path.fore_node_no = tb_a_fore.node_no
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.work_no = i_work_no   -- 2021082501
            and tb_a_back.node_no = tb_a_path.back_node_no
        -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
        inner join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type'
            and tb_a_dic.enum_key = tb_a_fore.node_fn_type
            and tb_a_dic.enum_group in ('1_p', '2_p', '3_p')
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.nn_depth_no = v_cur_nn_depth
          and (tb_a_fore.is_fore_node or v_cur_learn_cnt = v_learn_cnt_init)
          and tb_a_fore.is_dropout is not true
        group by tb_a_fore.node_no, tb_a_fore.node_fn_type
      )
      ,
      -- 前向传播
      cte_upd_fore_node_indepdt as 
      (
        update sm_sc.tb_nn_node tb_a_tar_depdt
        set 
          pick_depdt_idx =
            sm_sc.fv_nn_node_pick_depdt_idx
            (
              tb_a_indepdt_fore.node_fn_type
            , tb_a_indepdt_fore.a_pick_depdt_idx
            , tb_a_tar_depdt.node_fn_asso_value
            )
        , node_depdt_vals = 
            sm_sc.fv_lambda_arr
            (
              tb_a_indepdt_fore.node_no
            , tb_a_tar_depdt.node_fn_type
            , tb_a_indepdt_fore.a_bi_opr_input_1st
            , tb_a_indepdt_fore.a_bi_opr_input_2nd
            , tb_a_tar_depdt.node_fn_asso_value
            , tb_a_indepdt_fore.a_bi_opr_input_3rd
            )  
            -- dropout 缩放
            *` tb_a_tar_depdt.dropout_rescale
        , learn_cnt_fore = v_cur_learn_cnt + 1  -- 强制对齐训练次数，避免路径跟踪混乱   -- tb_a_tar_depdt.learn_cnt_fore + 1
        from cte_indepdt tb_a_indepdt_fore
        where tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
        returning 
          tb_a_tar_depdt.node_no as a_cur_node_no
          -- 寄存、传递若干算子的因变量，对 ddepdt_dindepdt 是必要的。
        , case when tb_a_tar_depdt.node_fn_type = any(v_arr_fn_depdt_delta) then tb_a_tar_depdt.node_depdt_vals end as a_node_depdt_vals
      )
--       ,
      -- 前向传播同步求导 ddepdt_dindepdt

-- -- for debug
-- cte_upd_path as 
-- (

        update sm_sc.tb_nn_path tb_a_tar_path
        set 
          ddepdt_dindepdt =
            sm_sc.fv_lambda_arr_ddepdt_dindepdt
            (
              tb_a_indepdt_fore.node_no
            , tb_a_depdt.node_fn_type
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 1, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_1st 
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 2, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_2nd
             -- when tb_a_tar_path.path_ord_no = 3 
             --   then tb_a_indepdt_fore.a_bi_opr_input_3rd
              end 
            , tb_a_tar_path.path_ord_no
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 3, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_2nd 
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 4, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_1st
              end 
            , tb_a_depdt.node_fn_asso_value 
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 5, 1) = '1'
                  or tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 6, 1) = '1'
                  then tb_a_depdt_val.a_node_depdt_vals
              end 
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 7, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_len_1st
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 8, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_len_2nd
             -- when tb_a_tar_path.path_ord_no = 3 
             --   then tb_a_indepdt_fore.a_bi_opr_input_len_3rd
              end 
            )
        from cte_indepdt tb_a_indepdt_fore
          , cte_upd_fore_node_indepdt tb_a_depdt_val
          , sm_sc.tb_nn_node tb_a_depdt
          , sm_sc.tb_dic_enum tb_a_dic
        where tb_a_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_depdt.node_no = tb_a_indepdt_fore.node_no
          and tb_a_tar_path.work_no = i_work_no   -- 2021082501
          and tb_a_tar_path.fore_node_no = tb_a_indepdt_fore.node_no
          -- 求导时机 = '前向传播同步'
          and tb_a_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_dic.enum_group = '0'
          and tb_a_dic.enum_key = tb_a_depdt.node_fn_type
          and tb_a_depdt_val.a_cur_node_no = tb_a_tar_path.fore_node_no
          and (tb_a_indepdt_fore.a_is_bi_opr_input_1st_back and tb_a_tar_path.path_ord_no = 1
               or tb_a_indepdt_fore.a_is_bi_opr_input_2nd_back and tb_a_tar_path.path_ord_no = 2
            -- or tb_a_indepdt_fore.a_is_bi_opr_input_3rd_back and tb_a_tar_path.path_ord_no = 3
              )
              
-- -- for debug
-- )
-- select 
--   array_agg(a_cur_node_no) -- || v_cur_node_nos
--     into v_cur_node_nos
-- from cte_upd_fore_node_indepdt

      ;      
      commit;
      -- --------------------------------------------------
      with 
      -- 参数准备，聚合算子
      cte_indepdt as
      (
        -- 聚合后的入参归并结果当作单目运算唯一参数
        -- 另一个同类型的限制是，一个CASE无法阻止其所包含的聚集表达式 的计算...取而代之的是，可以使用 一个WHERE或FILTER子句来首先阻止有问题的输入行到达 一个聚集函数。
        -- 采用 filter 过滤掉不必要的聚合输入，达到减少聚合计算以及入参宽高一致性保证等安全问题
        select 
          tb_a_fore.node_no
        , tb_a_fore.node_fn_type
        , case 
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
              then sm_sc.fa_mx_concat_y(tb_a_back.pick_depdt_idx order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
             -- 假定：如果 node_fn_type <>  '06_aggr_mx_concat_y'，那么来路的 pick_depdt_idx 要么为空，要么统一一致
            else sm_sc.fa_range_or(tb_a_back.pick_depdt_idx) filter(where tb_a_fore.node_fn_type <>  '06_aggr_mx_concat_y')
          end
          as a_pick_depdt_idx
        , case 
            when tb_a_fore.node_fn_type = '06_aggr_mx_sum'
              then sm_sc.fa_mx_sum(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_sum')
            when tb_a_fore.node_fn_type = '06_aggr_mx_prod'
              then sm_sc.fa_mx_prod(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_prod')
            when tb_a_fore.node_fn_type = '06_aggr_mx_avg'
              then sm_sc.fa_mx_avg(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')
            when tb_a_fore.node_fn_type = '06_aggr_mx_max'
              then sm_sc.fa_mx_max(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_max')
            when tb_a_fore.node_fn_type = '06_aggr_mx_min'
              then sm_sc.fa_mx_min(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_min')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
              then sm_sc.fa_mx_concat_y(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x'
              then sm_sc.fa_mx_concat_x(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3'
              then sm_sc.fa_mx_concat_x3(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4'
              then sm_sc.fa_mx_concat_x4(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
          end
          as a_opr_depdt
        , sm_sc.fa_mx_max(tb_a_back.node_depdt_len) as a_opr_depdt_len
        , (sm_sc.fa_mx_or(array[tb_a_back.is_back_node]))[1] as a_is_opr_input_back
        from sm_sc.tb_nn_node tb_a_fore
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.work_no = i_work_no   -- 2021082501
            and tb_a_path.fore_node_no = tb_a_fore.node_no
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.work_no = i_work_no   -- 2021082501
            and tb_a_back.node_no = tb_a_path.back_node_no
        -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
        inner join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type'
            and tb_a_dic.enum_key = tb_a_fore.node_fn_type
            and tb_a_dic.enum_group = 'n_p'
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.nn_depth_no = v_cur_nn_depth
          and (tb_a_fore.is_fore_node or v_cur_learn_cnt = v_learn_cnt_init)
        group by tb_a_fore.node_no, tb_a_fore.node_fn_type
      )
      ,
      -- 前向传播
      cte_upd_fore_node_indepdt as 
      (
        update sm_sc.tb_nn_node tb_a_tar_depdt
        set 
          pick_depdt_idx     = tb_a_indepdt_fore.a_pick_depdt_idx
        , node_depdt_vals    = tb_a_indepdt_fore.a_opr_depdt
        , learn_cnt_fore     = v_cur_learn_cnt + 1  -- 强制对齐训练次数，避免路径跟踪混乱   -- tb_a_tar_depdt.learn_cnt_fore + 1
        from cte_indepdt tb_a_indepdt_fore
        where tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
        returning 
          tb_a_tar_depdt.node_no as a_cur_node_no
      )
--       ,
      -- 前向传播同步求导 ddepdt_dindepdt
      
-- -- for debug
-- cte_upd_path as 
-- (

        update sm_sc.tb_nn_path tb_a_tar_path
        set 
          ddepdt_dindepdt =
            case tb_a_fore.node_fn_type
              when '06_aggr_mx_sum'
                then sm_sc.fv_d_mx_sum(tb_a_indepdt_fore.a_opr_depdt_len)
              when '06_aggr_mx_prod'
                then sm_sc.fv_d_mx_prod(tb_a_back.node_depdt_vals, tb_a_fore.node_depdt_vals)
              when '06_aggr_mx_avg'
                then sm_sc.fv_d_mx_avg(tb_a_fore.node_fn_asso_value[1] :: int, tb_a_indepdt_fore.a_opr_depdt_len)
              when '06_aggr_mx_max'
                then sm_sc.fv_d_mx_max(tb_a_back.node_depdt_vals, tb_a_fore.node_depdt_vals)
              when '06_aggr_mx_min'
                then sm_sc.fv_d_mx_min(tb_a_back.node_depdt_vals, tb_a_fore.node_depdt_vals)
            end
        from cte_indepdt tb_a_indepdt_fore
          , sm_sc.tb_nn_node tb_a_fore
          , sm_sc.tb_nn_node tb_a_back
          , sm_sc.tb_dic_enum tb_a_dic
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.node_no = tb_a_indepdt_fore.node_no
          and tb_a_tar_path.work_no = i_work_no   -- 2021082501
          and tb_a_tar_path.fore_node_no = tb_a_indepdt_fore.node_no
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_tar_path.back_node_no
          -- 求导时机 = '前向传播同步'
          and tb_a_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_dic.enum_group = '0'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_indepdt_fore.a_is_opr_input_back
          
-- -- for debug
-- )
-- select 
--   array_agg(a_cur_node_no) || v_cur_node_nos
--     into v_cur_node_nos
-- from cte_upd_fore_node_indepdt

      ;      
      commit;

-- -- -- debug
-- raise notice 'debug 003. step fore end: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

    end loop;
-- -----------------------------------------------------------------------
    -- 对 i_depdt_01 小批量抽样，获得本轮 v_depdt
    if v_is_output_01
    then 
      select 
        array_agg(tb_a_buff.i_depdt_01 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_01
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_01'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;
    
-- -- debug
-- raise notice 'debug 003.1. v_depdt_01: %', array_dims(v_depdt_01);
   
    if v_is_output_02
    then 
       select 
         array_agg(tb_a_buff.i_depdt_02 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
       into 
         v_depdt_02
       from sm_sc.tb_nn_node tb_a_sour
       inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
         on tb_a_sour.node_type = 'output_02'
       inner join sm_sc.tb_nn_train_input_buff tb_a_buff
         on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
       where tb_a_sour.work_no = i_work_no
         and tb_a_buff.work_no = i_work_no
       ;
    end if;
    
    if v_is_output_03
    then
      select 
        array_agg(tb_a_buff.i_depdt_03 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_03
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_03'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;
    
    if v_is_output_04
    then 
      select 
        array_agg(tb_a_buff.i_depdt_04 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_04
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_04'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;

    -- 损失函数
    v_cur_loss = 
      sm_sc.fv_lambda_loss
      (
        v_loss_fn_type :: varchar(32)
      , (select node_depdt_vals from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_01')
      , v_depdt_01
      , (select node_depdt_vals from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_02')
      , v_depdt_02
      , (select node_depdt_vals from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_03')
      , v_depdt_03
      , (select node_depdt_vals from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_04')
      , v_depdt_04
      ) 
    ;

-- -- -- debug
-- raise notice 'debug 004. step fore end: len of v_depdt: %; v_cur_loss: %; at %;', array_dims(v_depdt_01), v_cur_loss, date_trunc('milliseconds', now());

-- -----------------------------------------------------------------------
    -- 反向传播
    -- 反向传播起点：全局损失函数对输出 z 求导(均方差、最小二乘法)
    -- 规约：node_type in ('output_01', 'output_02', 'output_03', 'output_04') 各只有一个
    update sm_sc.tb_nn_node t_a_tar
    set 
      node_dloss_ddepdt = 
        sm_sc.fv_lambda_dloss_dz
        (
          v_loss_fn_type
        , t_a_tar.node_depdt_vals
        , case node_type when 'output_01' then v_depdt_01 when 'output_02' then v_depdt_02 when 'output_03' then v_depdt_03 when 'output_04' then v_depdt_04 end
        , node_type
        )
    , learn_cnt_back = v_cur_learn_cnt + 1 -- 强制对齐训练次数，避免路径计算混乱    -- t_tar.learn_cnt_back + 1
    where node_type in ('output_01', 'output_02', 'output_03', 'output_04')
      and work_no = i_work_no   -- 2021082501
    ;
    commit;

-- -- -- debug
-- raise notice 'debug 005. step fore end: v_cur_learn_cnt: %; at %;', v_cur_learn_cnt, date_trunc('milliseconds', now());

    v_cur_nn_depth := v_nn_depth;
    
-- -- for debug
-- v_cur_node_nos := (select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('output_01', 'output_02', 'output_03', 'output_04') limit 1);

    while 
      exists 
      (
        select  from sm_sc.tb_nn_node tb_a_back
        where tb_a_back.work_no = i_work_no    -- 2021082501
          and tb_a_back.learn_cnt_back <= v_cur_learn_cnt
          and tb_a_back.is_back_node is true
        limit 1
      )
      and v_cur_nn_depth > 0
    loop

-- -- -- debug
-- raise notice 'debug 006. step back begin: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

      with 
      cte_dloss_dindepdt as
      (
        select 
          tb_a_back.node_no,
          sm_sc.fa_mx_sum
          (
            sm_sc.fv_lambda_arr_dloss_dindepdt
            (
              tb_a_fore.node_no
            , tb_a_fore.node_fn_type
            -- 以下入参是否传入，由 字典表 enum_name = 'node_fn_type_delta_method' 控制
            -- 对于聚合算子的所有自变量，无论顺序，统一以 path_ord_no = 1 配置该控制
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 1, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 2, 1) = '1'
                  then tb_a_back.node_depdt_vals 
              end
            , tb_a_path.path_ord_no 
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 3, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 4, 1) = '1'
                  then tb_a_back_co.node_depdt_vals
              end
            , tb_a_fore.node_fn_asso_value
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 5, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 6, 1) = '1'
                  then tb_a_fore.node_depdt_vals
              end
            , tb_a_fore.node_dloss_ddepdt
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 7, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 8, 1) = '1'
                  or tb_a_path.path_ord_no = 3 -- 当前仅 05_conv_2d 支持第三目运算符，且求导需要自变量规格
                  then tb_a_back.node_depdt_len
              end
            , tb_a_path.ddepdt_dindepdt
            )
          ) as a_dloss_dindepdt
        from sm_sc.tb_nn_node tb_a_back
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.back_node_no = tb_a_back.node_no
      	  and tb_a_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_path.fore_node_no
          and tb_a_fore.work_no = i_work_no   -- 2021082501
        left join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type_delta_method'
          -- and tb_a_dic.enum_group = '1'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
        left join sm_sc.tb_nn_path tb_a_path_co
          on tb_a_path_co.fore_node_no = tb_a_fore.node_no
      	  and tb_a_path_co.work_no = i_work_no   -- 2021082501
          and (tb_a_path.path_ord_no = 1 and tb_a_path_co.path_ord_no = 2
               or tb_a_path.path_ord_no = 2 and tb_a_path_co.path_ord_no = 1)
        left join sm_sc.tb_nn_node tb_a_back_co
          on tb_a_back_co.node_no = tb_a_path_co.back_node_no
          and tb_a_back_co.work_no = i_work_no   -- 2021082501
        where tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.nn_depth_no = v_cur_nn_depth - 1
          and tb_a_back.is_back_node
        group by tb_a_back.node_no
      )
      
-- -- for debug v_cur_node_nos
-- ,
-- cte_upd_node as
-- (

        update sm_sc.tb_nn_node t_tar
        set 
          node_dloss_ddepdt = t_sour.a_dloss_dindepdt,
          learn_cnt_back = v_cur_learn_cnt + 1 -- 强制对齐训练次数，避免路径计算混乱    -- t_tar.learn_cnt_back + 1
        from cte_dloss_dindepdt t_sour
        where t_sour.node_no = t_tar.node_no

-- -- for debug v_cur_node_nos
--   returning t_tar.node_no as a_cur_node_no
-- )
-- select 
--   array_agg(a_cur_node_no) into v_cur_node_nos
-- from cte_upd_node

      ;
      commit;
      
      -- de_broadcast, dloss_ddepdt 的广播逆运算
      update sm_sc.tb_nn_node t_tar
      set 
        node_dloss_ddepdt = sm_sc.fv_aggr_chunk_sum(node_dloss_ddepdt, node_depdt_len)
      where work_no = i_work_no   -- 2021082501
        and nn_depth_no = v_cur_nn_depth - 1
        and cardinality(node_dloss_ddepdt) / cardinality(node_depdt_vals) > 1
        and cardinality(node_dloss_ddepdt) % cardinality(node_depdt_vals) = 0
      ; 
      commit;

-- -- -- debug
-- raise notice 'debug 007. step back end: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at: %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

      v_cur_nn_depth := v_cur_nn_depth - 1;

    end loop;
    
    -- L1 正则化
    if i_l1 is not null 
    then  
      update sm_sc.tb_nn_node tar
      set 
        node_depdt_vals = tar.node_depdt_vals -` i_l1
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
    end if;
    
    -- L2 正则化
    if i_l2 is not null 
    then  
      update sm_sc.tb_nn_node tar
      set 
        node_depdt_vals = tar.node_depdt_vals *` (1.0 - i_l2)
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
    end if;

    -- 更新 w 参数，一次训练完成
    if not i_is_adam or i_is_adam is null
    then 
      update sm_sc.tb_nn_node tar
      set 
        -- node_depdt_vals = tar.node_depdt_vals -` (i_learn_rate *` tar.node_dloss_ddepdt)
        node_depdt_vals = 
          tar.node_depdt_vals 
          -` 
          (
            i_learn_rate 
            *` 
            -- 梯度截断，绝对值不超过 1000
            (
              100.0 :: float 
              *` 
              sm_sc.fv_tanh
              (
                tar.node_dloss_ddepdt 
                /` 
                100.0 :: float
              )
            )
            -- -- 如下梯度截断和差异更平缓一些
            -- (
            --   100.0 :: float 
            --   *` 
            --   (
            --     sm_sc.fv_tanh
            --     (
            --       (
            --         (@|` tar.node_dloss_ddepdt)
            --         /` 
            --         100.0 :: float
            --       )
            --       ^` 
            --       0.5 :: float
            --     )
            --     ^`
            --     2.0 :: float
            --   )
            --   *` 
            --   ((<>` tar.node_dloss_ddepdt) :: float[])
            -- )
          )
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;

    else
      -- 采用 Adam 梯度下降算法
      update sm_sc.tb_nn_node tar
      set 
        -- node_depdt_vals = tar.node_depdt_vals -` (i_learn_rate *` tar.node_dloss_ddepdt)
        cost_delta_l1 = (v_beta_l1 *` coalesce(case when array_length(cost_delta_l1, 1) = array_length(tar.node_dloss_ddepdt, 1) then cost_delta_l1 else null end, tar.node_dloss_ddepdt)) +` ((1.0 :: float - v_beta_l1) *` tar.node_dloss_ddepdt),
        cost_delta_l2 = (v_beta_l2 *` coalesce(case when array_length(cost_delta_l2, 1) = array_length(tar.node_dloss_ddepdt, 1) then cost_delta_l2 else null end, (tar.node_dloss_ddepdt ^` 2.0 :: float))) +` ((1.0 :: float - v_beta_l2) *` (tar.node_dloss_ddepdt ^` 2.0 :: float))
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
      
      update sm_sc.tb_nn_node tar
      set 
        node_depdt_vals =
          tar.node_depdt_vals -`
          (
            (
              cost_delta_l1 *`
              (i_learn_rate / (1.0 :: float - (v_beta_l1 ^ (least(v_cur_learn_cnt, 7070) + 1))))      -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
            ) /`
            (
              sm_sc.fv_ele_replace
              (
                (
                  cost_delta_l2 /` (1.0 :: float - (v_beta_l2 ^ (least(v_cur_learn_cnt, 7070) + 1)))  -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
                ) ^` 0.5 :: float
                , array[0.0 :: float]
                , 1e-128 :: float   -- 1e-128 :: float        
              )
            )
          )
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
    end if;

    v_cur_learn_cnt := v_cur_learn_cnt + 1;

    -- 寄存至 sm_sc.tb_classify_task
    if v_cur_learn_cnt % 1 = 0     -- 或者 100 次， 1000 次， 10000次。。。
      or v_cur_learn_cnt >= v_limit_train_times
      or v_cur_loss < i_loss_delta_least_stop_threshold
    then 
      insert into sm_sc.tb_classify_task
      (
        work_no
      , learn_cnt
      , loss_delta
      )
      select 
        i_work_no
      , v_cur_learn_cnt
      , v_cur_loss
      on conflict(work_no) do 
      update set
        learn_cnt = v_cur_learn_cnt
      , loss_delta = v_cur_loss
      ;

      insert into sm_sc.__vt_nn_node
      (
        work_no                      ,
        node_no                  ,        
        node_type                    ,
        node_fn_type                 ,
        node_fn_asso_value      ,
        nn_depth_no                  ,
        node_depdt_vals                  
      )
      select 
        tb_a_sour.work_no                      ,
        tb_a_sour.node_no                  ,        
        tb_a_sour.node_type                    ,
        tb_a_sour.node_fn_type                 ,
        tb_a_sour.node_fn_asso_value      ,
        tb_a_sour.nn_depth_no                  ,
        case when tb_a_sour.node_type = 'weight' or tb_a_sour.node_fn_type = '00_const' then tb_a_sour.node_depdt_vals end as node_depdt_vals                 
      from sm_sc.tb_nn_node tb_a_sour
      where tb_a_sour.work_no = i_work_no
      on conflict(work_no, node_no) do 
      update set
        node_type                 = EXCLUDED.node_type                   ,
        node_fn_type              = EXCLUDED.node_fn_type                ,
        node_fn_asso_value        = EXCLUDED.node_fn_asso_value          ,
        nn_depth_no               = EXCLUDED.nn_depth_no                 ,
        node_depdt_vals                 = case when EXCLUDED.node_type = 'weight' or EXCLUDED.node_fn_type = '00_const' then EXCLUDED.node_depdt_vals end
      ;    
      commit;
      
      -- -- 谨慎开启 vacuum。如果可能有其他训练任务同时进行，那么会被 vacuum 阻塞
      -- vacuum full sm_sc.tb_nn_node;
      -- vacuum full sm_sc.tb_nn_path;
      -- vacuum full sm_sc.__vt_nn_node;

-- debug
raise notice 'Training No. % end; v_cur_loss: %; at %;', v_cur_learn_cnt, v_cur_loss, date_trunc('milliseconds', now());

    end if;
  end loop;

  raise notice 'Training task end report: v_cur_learn_cnt: %; v_cur_loss: %; at %;', v_cur_learn_cnt, v_cur_loss, date_trunc('milliseconds', now());

  -- 更新 0 号 sess 部署。规约: 0 号 session 为每轮训练后，默认部署。
  delete from sm_sc.__vt_tmp_nn_node
  where work_no = i_work_no
    and sess_id = 0
  ;

  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id              
  , work_no              
  , node_no              
  , node_type            
  , node_fn_type         
  , node_fn_asso_value   
  , nn_depth_no          
  , node_depdt_vals      
  )
  select 
    0                    
  , work_no              
  , node_no              
  , node_type            
  , coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  
  , case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         
  , nn_depth_no                     
  , case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then node_depdt_vals
      else null
    end             
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  
end
$$
language plpgsql;

-- -- 用例 1, 二分类
-- -- -- 执行准备和检查
-- -- call sm_sc.prc_nn_prepare
-- -- (
-- --   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
-- --   2500      -- ,
-- --   -- 200
-- -- );


-- -- 开始训练
-- call sm_sc.prc_nn_train
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   1        ,
--   0.9      ,
--   0.1
-- );

-- -- -- 观察输出
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- select node_no, node_fn_type, learn_cnt_fore, learn_cnt_back, tb_a.node_depdt_vals, (tb_a.reg_w).m_vals  from sm_sc.tb_nn_node tb_a where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- select 
--   sm_sc.ft_nn_in_out
--   (
--     pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--     array[array[0.0, 0.0], array[0.0, 1.0], array[1.0, 0.0], array[1.0, 1.0]] 
--       +` sm_sc.fv_new_randn(0.0, 0.1, array[4, 2])
--   )

-- -- -- 期望用例举例：
-- -- -- select sigmoid(w31 * sigmoid(w11 + w12 * x1 + w13 * x2) + w32 * sigmoid(w21 + w22 * x1 + w23 * x2))
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 0 + 20.0 * 0) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 0 + 20.0 * 0)) union all   -- 期望输出，0
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 0 + 20.0 * 1) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 0 + 20.0 * 1)) union all   -- 期望输出，1
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 1 + 20.0 * 0) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 1 + 20.0 * 0)) union all   -- 期望输出，1
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 1 + 20.0 * 1) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 1 + 20.0 * 1))             -- 期望输出，0

-- -- -----------------------------------------------------------------------------
-- -- 用例 2, 多分类
-- -- -- 执行准备和检查
-- -- call sm_sc.prc_nn_prepare
-- -- (
-- --   2021121701   ,
-- --   5000      -- ,
-- --   -- 200
-- -- );
-- 
-- 
-- -- 开始训练
-- call sm_sc.prc_nn_train
-- (
--   2021121701   ,
--   2        ,
--   0.8      ,
--   0.1
-- );
-- 
-- -- -- 观察输出
-- select 
--   sm_sc.ft_nn_in_out
--   (
--     2021121701, 
--     array[array[0.0, 0.0, 0.0], array[0.0, 1.0, 0.0], array[1.0, 0.0, 0.0], array[1.0, 1.0, 0.0]
--           , array[0.0, 0.0, 1.0], array[0.0, 1.0, 1.0], array[1.0, 0.0, 1.0], array[1.0, 1.0, 1.0]] 
--       +` sm_sc.fv_new_randn(0.0, 0.1, array[8, 3])
--   )

-- 参看设计文档 《反向传播链式求导步骤编排》

-- 初始学习率：1e-4
-- https://www.zhihu.com/question/387050717

drop procedure if exists sm_sc.prc_nn_train_p(bigint, float, float, boolean, float, float, boolean);
create or replace procedure sm_sc.prc_nn_train_p
(
  i_work_no                          bigint                             -- 训练任务编号
-- , i_loss_fn_type                     varchar(32)[]                      -- 损失函数选取类型。1: 最小二乘法; 2: 交叉熵
, i_learn_rate                       float    default    0.0003         -- 学习率
, i_loss_delta_least_stop_threshold  float    default    0.0001         -- 触发收敛的损失函数梯度阈值
, i_is_adam                          boolean  default    false          -- 是否开启 adam 优化
, i_l1                               float    default    null           -- L1 正则化率，缺省不做 L1 正则化
, i_l2                               float    default    null           -- L2 正则化率，缺省不做 L2 正则化
, i_tensor_log                       boolean  default    false          -- 是否记录当轮次传播的中间张量
)
as
$$
declare -- here
  v_learn_cnt_init          int               := (select learn_cnt from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号初始值
  v_cur_learn_cnt           int               := v_learn_cnt_init;   -- 训练次数序号
  v_depdt_01                float[]  ; -- -- 训练集本轮采样之多模态 01
  v_depdt_02                float[]  ; -- -- 训练集本轮采样之多模态 02
  v_depdt_03                float[]  ; -- -- 训练集本轮采样之多模态 03
  v_depdt_04                float[]  ; -- -- 训练集本轮采样之多模态 04
  v_cur_node_nos            bigint[];  -- for debug
  v_cur_loss                float;     -- 损失函数值
  v_input_nodes             bigint[]          := 
    (
      select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') -- limit 1
    );
  v_cur_nn_depth            int               := 0;   -- 深度层数游标
  v_nn_depth                int;                      -- 深度层数
  v_limit_train_times       int               := (select learn_cnt_fore from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' limit 1);
  v_beta_l1                 float    := 0.9;      -- Adam 梯度下降算法一阶矩等比衰减系数
  v_beta_l2                 float    := 0.999;    -- Adam 梯度下降算法二阶矩等比衰减系数
  
  -- 求导时，需要因变量的算子类型集合
  v_arr_fn_depdt_delta      varchar(64)[]   :=   (select array_agg(enum_key) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method' and (substr(enum_value, 5, 1) = '1' or substr(enum_value, 6, 1) = '1'));
  v_loss_fn_type            varchar(32)     := (select loss_fn_type from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号初始值

  v_is_input_02             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_02');
  v_is_input_03             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_03');
  v_is_input_04             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_04');
  
  v_is_output_01            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_01')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_01 is not null))
  ;
  v_is_output_02            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_02')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_02 is not null))
  ;
  v_is_output_03            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_03')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_03 is not null))
  ;
  v_is_output_04            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_04')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_04 is not null))
  ;
  v_model_code_6            char(6)         :=
    (select model_code_6 from sm_sc.tb_classify_task where work_no = i_work_no)
  ;
  v_null                    varchar(64)[];
begin
  set search_path to public;
  
  -- -- 强行关闭并行
  -- set max_parallel_workers = 0;
  
  -- -- -- -- 强制开启并行
  -- -- -- -- select * from pg_settings where name ~ 'paral' or name in ('max_worker_processes') limit 100
  -- -- -- -- show max_worker_processes
  -- set min_parallel_table_scan_size = 0;
  -- set min_parallel_index_scan_size = 0;
  -- set force_parallel_mode = 'off';
  -- set max_parallel_workers_per_gather = 64;
  -- set parallel_setup_cost = 0;
  -- set parallel_tuple_cost = 0.0;

  -- 清理旧有训练结果
  delete from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  commit;
  
  -- -- -- -- 每次训练前，调整抽样数量等策略
  -- -- -- call sm_sc.prc_nn_prepare_per_train(v_cur_learn_cnt, v_cur_loss, ...);

  -- 审计 训练是否已经满足损失函数最低阈值
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no and loss_delta <= i_loss_delta_least_stop_threshold)
  then
    raise notice 'The loss_delta is adequately less than i_loss_delta_least_stop_threshold already!';
    return;
  end if;

  v_nn_depth := (select max(nn_depth_no) from sm_sc.tb_nn_node where work_no = i_work_no);

  raise notice 'Training begin; at %', date_trunc('milliseconds', now());

  -- 开始训练
  while 
  (
    i_loss_delta_least_stop_threshold <= v_cur_loss
    or v_cur_loss is null
  )
    and v_cur_learn_cnt < v_limit_train_times
  loop

-- -- -- debug
-- raise notice 'debug 001. step begin: v_cur_learn_cnt: %; at %;', v_cur_learn_cnt, date_trunc('milliseconds', now());

    -- 触发 dropout 机制，根据 dropout 概率，确定 dropout 节点清单
    update sm_sc.tb_nn_node
    set is_dropout = (dropout_ratio > random())
    where work_no = i_work_no
      and dropout_ratio > 0.0
    ;
    commit;
    
    -- 填充 dropout 节点的因变量
    update sm_sc.tb_nn_node
    set 
      p_node_depdt = 
        sm_sc.__fv_set_kv
        (
          array_fill(dropout_depdt_val, node_depdt_len)
        , v_model_code_6 || 
          '_' || work_no :: varchar ||
          '_' || node_no :: varchar ||
          '__d__'  -- 'depdt'
        )
    where work_no = i_work_no
      and is_dropout
    ;

    if exists
       (
         select  from sm_sc.tb_nn_node 
         where work_no = i_work_no 
           and node_fn_type = '00_buff_slice_rand_pick' 
           and node_type = 'input_01'
           limit 1
       )
    then
      -- 如果对训练集采用小批量随机采样，那么每轮初始化 input 值，以支持不同轮次更换训练集
      with
      cte_slice_rand_pick_agg as 
      (
        -- 要确保各模态的小批量随机采样序号集合一致。
        -- 规约：以 input_01 模态的 00_buff_slice_rand_pick 算子超参数配置为基准。而忽略其他模态的配置。
        select  
          array_agg
          (
            int4multirange
            (
              int4range
              (
                o_ord_no :: int
              , o_ord_no :: int + 1
              , '[)'
              )
            )
          )                                                                as a_pick_depdt_idx
        , array_agg(tb_a_idx.o_rand_pick_indepdt_01 order by o_ord_no)     as a_rand_pick_indepdt_01
        , case when v_is_input_02 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_02, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_02
        , case when v_is_input_03 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_03, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_03
        , case when v_is_input_04 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_04, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_04
        from sm_sc.tb_nn_node tb_a_node,
          sm_sc.ft_nn_buff_slice_rand_pick
          (
            -- 参看 00_buff_slice_rand_pick 的 node_fn_asso_value 字典表
            i_work_no,
            (
              select 
                array_agg(int4range(tb_a_node.node_fn_asso_value[1][a_ord] :: int, tb_a_node.node_fn_asso_value[2][a_ord] :: int + 1, '[)') order by a_ord)
              from generate_series(1, array_length(tb_a_node.node_fn_asso_value, 2)) tb_a(a_ord)
            )
          , sm_sc.fv_mx_ele_2d_2_1d(tb_a_node.node_fn_asso_value[3 : 3][ : ]) :: int[]
          ) tb_a_idx(o_ord_no, o_rand_pick_indepdt_01, o_rand_pick_indepdt_02, o_rand_pick_indepdt_03, o_rand_pick_indepdt_04)
        where tb_a_node.work_no = i_work_no
          and tb_a_node.node_type = 'input_01'
          and tb_a_node.node_fn_type = '00_buff_slice_rand_pick'
      )
      update sm_sc.tb_nn_node tb_a_tar
      set 
        p_node_depdt = 
          sm_sc.__fv_set_kv
          (
            case 
              when tb_a_tar.node_type = 'input_01'
                then a_rand_pick_indepdt_01
              when tb_a_tar.node_type = 'input_02'
                then a_rand_pick_indepdt_02
              when tb_a_tar.node_type = 'input_03'
                then a_rand_pick_indepdt_03
              when tb_a_tar.node_type = 'input_04'
                then a_rand_pick_indepdt_04
            end
          , v_model_code_6 || 
            '_' || tb_a_tar.work_no :: varchar ||
            '_' || tb_a_tar.node_no :: varchar ||
            '__d' ||  -- 'depdt'
            '_' || ltrim(substr(tb_a_tar.node_type, 7), '0') ||
            '_'
          )
      , pick_depdt_idx = a_pick_depdt_idx
      from cte_slice_rand_pick_agg tb_a_sour
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_tar.node_fn_type = '00_buff_slice_rand_pick' 
        and tb_a_tar.node_type in ('input_01', 'input_02', 'input_03', 'input_04')
      ;
      commit;
    elsif exists(select  from sm_sc.tb_nn_node where work_no = i_work_no and node_fn_type = '00_full_dataset' and node_type = 'input_01')
    then 
      -- 如果不做小批量随机采样，那么使用训练集全集
      with
      cte_slice_rand_pick as 
      (
        select 
          array_agg(i_indepdt_01)        as a_x_vals_01,
          array_agg(i_indepdt_02)        as a_x_vals_02,
          array_agg(i_indepdt_03)        as a_x_vals_03,
          array_agg(i_indepdt_04)        as a_x_vals_04,
          count(*) :: int       as a_x_cnt
        from sm_sc.tb_nn_train_input_buff
        where work_no = i_work_no
      )
      update sm_sc.tb_nn_node tb_a_tar
      set 
        p_node_depdt = 
          sm_sc.__fv_set_kv
          (
            case tb_a_tar.node_type 
              when 'input_01' 
                then a_x_vals_01 
              when 'input_02' 
                then a_x_vals_02 
              when 'input_03' 
                then a_x_vals_03 
              when 'input_04' 
                then a_x_vals_04 
            end  
          , v_model_code_6 || 
            '_' || tb_a_tar.work_no :: varchar ||
            '_' || tb_a_tar.node_no :: varchar ||
            '__d__'  -- 'depdt'
          )
      , pick_depdt_idx = array[int4multirange(int4range(1, a_x_cnt + 1, '[)'))]
      from cte_slice_rand_pick tb_a_sour
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_tar.node_type in ('input_01', 'input_02', 'input_03', 'input_04')
      ;
      commit;
    end if;

    v_cur_nn_depth := 0;

    -- 前向传播

-- -- for debug
-- v_cur_node_nos := 
--   (
--     select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') -- limit 1
--   );
      
    while v_cur_nn_depth < v_nn_depth
    loop
      v_cur_nn_depth := v_cur_nn_depth + 1;

-- -- -- debug
-- raise notice 'debug 002. step fore begin: v_cur_learn_cnt: %; v_cur_nn_depth: %; v_learn_cnt_init: %; at %;', v_cur_learn_cnt, v_cur_nn_depth, v_learn_cnt_init, date_trunc('milliseconds', now());

      -- 按照 nn_depth_no，逐层前向传播。初始节点只前向传播一次
      with 
      -- 参数准备，单目、双目（三目）算子
      cte_indepdt as
      (
        select 
          tb_a_fore.node_no
        , tb_a_fore.node_fn_type
        , sm_sc.fa_range_or(tb_a_back.pick_depdt_idx) as a_pick_depdt_idx
        , sm_sc.fa_coalesce(case when tb_a_path.path_ord_no = 1 then tb_a_back.p_node_depdt end) as a_bi_opr_input_1st_p
        , sm_sc.fa_coalesce(case when tb_a_path.path_ord_no = 2 then tb_a_back.p_node_depdt end) as a_bi_opr_input_2nd_p
        , sm_sc.fa_coalesce(case when tb_a_path.path_ord_no = 3 then tb_a_back.p_node_depdt end) as a_bi_opr_input_3rd_p
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 1 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_1st
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 2 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_2nd
     -- , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 3 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_3rd
        , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 1 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_1st_back
        , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 2 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_2nd_back
     -- , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 3 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_3rd_back
        from sm_sc.tb_nn_node tb_a_fore
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.work_no = i_work_no   -- 2021082501
            and tb_a_path.fore_node_no = tb_a_fore.node_no
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.work_no = i_work_no   -- 2021082501
            and tb_a_back.node_no = tb_a_path.back_node_no
        -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
        inner join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type'
            and tb_a_dic.enum_key = tb_a_fore.node_fn_type
            and tb_a_dic.enum_group in ('1_p', '2_p', '3_p')
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.nn_depth_no = v_cur_nn_depth
          and (tb_a_fore.is_fore_node or v_cur_learn_cnt = v_learn_cnt_init)
          and tb_a_fore.is_dropout is not true
        group by tb_a_fore.node_no, tb_a_fore.node_fn_type
      )
      ,
      -- 前向传播
      cte_upd_fore_node_indepdt as 
      (
        update sm_sc.tb_nn_node tb_a_tar_depdt
        set 
          pick_depdt_idx =
            sm_sc.fv_nn_node_pick_depdt_idx
            (
              tb_a_indepdt_fore.node_fn_type
            , tb_a_indepdt_fore.a_pick_depdt_idx
            , tb_a_tar_depdt.node_fn_asso_value
            )
        , p_node_depdt = 
            sm_sc.__fv_set_kv 
            (
              sm_sc.__fv_get_kv 
              (
                sm_sc.fv_lambda_arr_p
                (
                  v_model_code_6
                , i_work_no
                , tb_a_indepdt_fore.node_no
                , null -- i_sess_id
                , tb_a_tar_depdt.node_fn_type
                , tb_a_indepdt_fore.a_bi_opr_input_1st_p
                , tb_a_indepdt_fore.a_bi_opr_input_2nd_p
                , tb_a_tar_depdt.node_fn_asso_value
                , tb_a_indepdt_fore.a_bi_opr_input_3rd_p
                )  
              )  
              -- dropout 缩放
              *` tb_a_tar_depdt.dropout_rescale
            , v_model_code_6 || 
              '_' || i_work_no :: varchar ||
              '_' || tb_a_indepdt_fore.node_no :: varchar ||
              '__d__'  -- 'depdt'
            )
        , learn_cnt_fore = v_cur_learn_cnt + 1  -- 强制对齐训练次数，避免路径跟踪混乱   -- tb_a_tar_depdt.learn_cnt_fore + 1
        from cte_indepdt tb_a_indepdt_fore
        where tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
        returning 
          tb_a_tar_depdt.node_no as a_cur_node_no
          -- 寄存、传递若干算子的因变量，对 p_ddepdt_dindepdt 是必要的。
        , case when tb_a_tar_depdt.node_fn_type = any(v_arr_fn_depdt_delta) then tb_a_tar_depdt.p_node_depdt end as a_p_node_depdt
      )
--       ,
      -- 前向传播同步求导 p_ddepdt_dindepdt

-- -- for debug
-- cte_upd_path as 
-- (

        update sm_sc.tb_nn_path tb_a_tar_path
        set 
          p_ddepdt_dindepdt =
            sm_sc.fv_lambda_arr_ddepdt_dindepdt_p
            (
              v_model_code_6
            , i_work_no
            , tb_a_indepdt_fore.node_no
            , tb_a_tar_path.back_node_no
            , tb_a_depdt.node_fn_type
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 1, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_1st_p 
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 2, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_2nd_p
             -- when tb_a_tar_path.path_ord_no = 3 
             --   then tb_a_indepdt_fore.a_bi_opr_input_3rd_p
              end 
            , tb_a_tar_path.path_ord_no
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 3, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_2nd_p 
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 4, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_1st_p
              end 
            , tb_a_depdt.node_fn_asso_value 
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 5, 1) = '1'
                  or tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 6, 1) = '1'
                  then tb_a_depdt_val.a_p_node_depdt
              end 
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 7, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_len_1st
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 8, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_len_2nd
             -- when tb_a_tar_path.path_ord_no = 3 
             --   then tb_a_indepdt_fore.a_bi_opr_input_len_3rd
              end 
            )
        from cte_indepdt tb_a_indepdt_fore
          , cte_upd_fore_node_indepdt tb_a_depdt_val
          , sm_sc.tb_nn_node tb_a_depdt
          , sm_sc.tb_dic_enum tb_a_dic
        where tb_a_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_depdt.node_no = tb_a_indepdt_fore.node_no
          and tb_a_tar_path.work_no = i_work_no   -- 2021082501
          and tb_a_tar_path.fore_node_no = tb_a_indepdt_fore.node_no
          -- 求导时机 = '前向传播同步'
          and tb_a_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_dic.enum_group = '0'
          and tb_a_dic.enum_key = tb_a_depdt.node_fn_type
          and tb_a_depdt_val.a_cur_node_no = tb_a_tar_path.fore_node_no
          and (tb_a_indepdt_fore.a_is_bi_opr_input_1st_back and tb_a_tar_path.path_ord_no = 1
               or tb_a_indepdt_fore.a_is_bi_opr_input_2nd_back and tb_a_tar_path.path_ord_no = 2
            -- or tb_a_indepdt_fore.a_is_bi_opr_input_3rd_back and tb_a_tar_path.path_ord_no = 3
              )
              
-- -- for debug
-- )
-- select 
--   array_agg(a_cur_node_no) -- || v_cur_node_nos
--     into v_cur_node_nos
-- from cte_upd_fore_node_indepdt

      ;      
      commit;
      -- --------------------------------------------------
      with 
      -- 参数准备，聚合算子
      cte_indepdt as
      (
        -- 聚合后的入参归并结果当作单目运算唯一参数
        -- 另一个同类型的限制是，一个CASE无法阻止其所包含的聚集表达式 的计算...取而代之的是，可以使用 一个WHERE或FILTER子句来首先阻止有问题的输入行到达 一个聚集函数。
        -- 采用 filter 过滤掉不必要的聚合输入，达到减少聚合计算以及入参宽高一致性保证等安全问题
        select 
          tb_a_fore.node_no
        , tb_a_fore.node_fn_type
        , case 
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
              then sm_sc.fa_mx_concat_y(tb_a_back.pick_depdt_idx order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
             -- 假定：如果 node_fn_type <>  '06_aggr_mx_concat_y'，那么来路的 pick_depdt_idx 要么为空，要么统一一致
            else sm_sc.fa_range_or(tb_a_back.pick_depdt_idx) filter(where tb_a_fore.node_fn_type <>  '06_aggr_mx_concat_y')
          end
          as a_pick_depdt_idx
        , sm_sc.__fv_set_kv
          (
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_sum'
                then sm_sc.fa_mx_sum(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_sum')
              when tb_a_fore.node_fn_type = '06_aggr_mx_prod'
                then sm_sc.fa_mx_prod(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_prod')
              when tb_a_fore.node_fn_type = '06_aggr_mx_avg'
                then sm_sc.fa_mx_avg(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')
              when tb_a_fore.node_fn_type = '06_aggr_mx_max'
                then sm_sc.fa_mx_max(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_max')
              when tb_a_fore.node_fn_type = '06_aggr_mx_min'
                then sm_sc.fa_mx_min(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_min')
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
                then sm_sc.fa_mx_concat_y(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x'
                then sm_sc.fa_mx_concat_x(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3'
                then sm_sc.fa_mx_concat_x3(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4'
                then sm_sc.fa_mx_concat_x4(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt) order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
            end
          , v_model_code_6 || 
            '_' || i_work_no :: varchar   ||
            '_' || tb_a_fore.node_no :: varchar ||
            '__d_aggr_'
          )
          as a_opr_depdt_p
        , sm_sc.fa_mx_max(tb_a_back.node_depdt_len) as a_opr_depdt_len
        , (sm_sc.fa_mx_or(array[tb_a_back.is_back_node]))[1] as a_is_opr_input_back
        from sm_sc.tb_nn_node tb_a_fore
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.work_no = i_work_no   -- 2021082501
            and tb_a_path.fore_node_no = tb_a_fore.node_no
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.work_no = i_work_no   -- 2021082501
            and tb_a_back.node_no = tb_a_path.back_node_no
        -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
        inner join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type'
            and tb_a_dic.enum_key = tb_a_fore.node_fn_type
            and tb_a_dic.enum_group = 'n_p'
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.nn_depth_no = v_cur_nn_depth
          and (tb_a_fore.is_fore_node or v_cur_learn_cnt = v_learn_cnt_init)
        group by tb_a_fore.node_no, tb_a_fore.node_fn_type
      )
      ,
      -- 前向传播
      cte_upd_fore_node_indepdt as 
      (
        update sm_sc.tb_nn_node tb_a_tar_depdt
        set 
          pick_depdt_idx     = tb_a_indepdt_fore.a_pick_depdt_idx
        , p_node_depdt    = tb_a_indepdt_fore.a_opr_depdt_p
        , learn_cnt_fore     = v_cur_learn_cnt + 1  -- 强制对齐训练次数，避免路径跟踪混乱   -- tb_a_tar_depdt.learn_cnt_fore + 1
        from cte_indepdt tb_a_indepdt_fore
        where tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
        returning 
          tb_a_tar_depdt.node_no as a_cur_node_no
      )
--       ,
      -- 前向传播同步求导 p_ddepdt_dindepdt
      
-- -- for debug
-- cte_upd_path as 
-- (

        update sm_sc.tb_nn_path tb_a_tar_path
        set 
          p_ddepdt_dindepdt =
            case tb_a_fore.node_fn_type
              when '06_aggr_mx_sum'
                then 
                  sm_sc.__fv_set_kv
                  (
                    sm_sc.fv_d_mx_sum(tb_a_indepdt_fore.a_opr_depdt_len)
                  , v_model_code_6 || 
                    '_' || tb_a_tar_path.work_no :: varchar        ||
                    '_' || tb_a_tar_path.fore_node_no :: varchar ||
                    '_' || tb_a_tar_path.back_node_no :: varchar || 
                    '_dddi__'  -- 'd_depdt_d_indepdt'
                  )
              when '06_aggr_mx_prod'
                then 
                  sm_sc.__fv_set_kv
                  (
                    sm_sc.fv_d_mx_prod(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt), sm_sc.__fv_get_kv(tb_a_fore.p_node_depdt))
                  , v_model_code_6 || 
                    '_' || tb_a_tar_path.work_no :: varchar        ||
                    '_' || tb_a_tar_path.fore_node_no :: varchar || 
                    '_' || tb_a_tar_path.back_node_no :: varchar || 
                    '_dddi__'  -- 'd_depdt_d_indepdt'
                  )
              when '06_aggr_mx_avg'
                then 
                  sm_sc.__fv_set_kv
                  (
                    sm_sc.fv_d_mx_avg(tb_a_fore.node_fn_asso_value[1] :: int, tb_a_indepdt_fore.a_opr_depdt_len)
                  , v_model_code_6 || 
                    '_' || tb_a_tar_path.work_no :: varchar      ||
                    '_' || tb_a_tar_path.fore_node_no :: varchar || 
                    '_' || tb_a_tar_path.back_node_no :: varchar || 
                    '_dddi__'  -- 'd_depdt_d_indepdt'
                  )
              when '06_aggr_mx_max'
                then 
                  sm_sc.__fv_set_kv
                  (
                    sm_sc.fv_d_mx_max(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt), sm_sc.__fv_get_kv(tb_a_fore.p_node_depdt))
                  , v_model_code_6 || 
                    '_' || tb_a_tar_path.work_no :: varchar        ||
                    '_' || tb_a_tar_path.fore_node_no :: varchar || 
                    '_' || tb_a_tar_path.back_node_no :: varchar || 
                    '_dddi__'  -- 'd_depdt_d_indepdt'
                  )
              when '06_aggr_mx_min'
                then 
                  sm_sc.__fv_set_kv
                  (
                    sm_sc.fv_d_mx_min(sm_sc.__fv_get_kv(tb_a_back.p_node_depdt), sm_sc.__fv_get_kv(tb_a_fore.p_node_depdt))
                  , v_model_code_6 || 
                    '_' || tb_a_tar_path.work_no :: varchar        ||
                    '_' || tb_a_tar_path.fore_node_no :: varchar ||
                    '_' || tb_a_tar_path.back_node_no :: varchar ||
                    '_dddi__'  -- 'd_depdt_d_indepdt'
                  )
            end
        from cte_indepdt tb_a_indepdt_fore
          , sm_sc.tb_nn_node tb_a_fore
          , sm_sc.tb_nn_node tb_a_back
          , sm_sc.tb_dic_enum tb_a_dic
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.node_no = tb_a_indepdt_fore.node_no
          and tb_a_tar_path.work_no = i_work_no   -- 2021082501
          and tb_a_tar_path.fore_node_no = tb_a_indepdt_fore.node_no
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_tar_path.back_node_no
          -- 求导时机 = '前向传播同步'
          and tb_a_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_dic.enum_group = '0'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_indepdt_fore.a_is_opr_input_back
          
-- -- for debug
-- )
-- select 
--   array_agg(a_cur_node_no) || v_cur_node_nos
--     into v_cur_node_nos
-- from cte_upd_fore_node_indepdt

      ;      
      commit;

      if not i_tensor_log
      then
        -- 清理当前层前向传播后的无用张量 kv
        -- 清理 depdt
        with 
        cte_clear_kv_list as 
        (
          select
            tb_a_back_node.node_no
          , max(tb_a_back_node.p_node_depdt) as p_node_depdt
          from sm_sc.tb_nn_node tb_a_main_fore_node
          inner join sm_sc.tb_nn_path tb_a_main_path
            on tb_a_main_path.fore_node_no = tb_a_main_fore_node.node_no 
              and tb_a_main_path.work_no = i_work_no
          inner join sm_sc.tb_nn_node tb_a_back_node
            on tb_a_back_node.node_no = tb_a_main_path.back_node_no
              and tb_a_back_node.work_no = i_work_no
          inner join sm_sc.tb_dic_enum tb_a_back_dic 
            on tb_a_back_dic.enum_name = 'node_fn_type_delta_method'
              and tb_a_back_dic.enum_key = tb_a_back_node.node_fn_type
              and substr(tb_a_back_dic.enum_value, 5, 2) = '00'  -- 第一/二目求导不需要因变量
          inner join sm_sc.tb_nn_path tb_a_path
            on tb_a_path.work_no = i_work_no
              and tb_a_path.back_node_no = tb_a_back_node.node_no
          inner join sm_sc.tb_nn_node tb_a_fore_node
            on tb_a_fore_node.work_no = i_work_no
              and tb_a_fore_node.node_no = tb_a_path.fore_node_no
          left join sm_sc.tb_dic_enum tb_a_fore_dic 
            on tb_a_fore_dic.enum_name = 'node_fn_type_delta_method'
              and tb_a_fore_dic.enum_key = tb_a_fore_node.node_fn_type
          left join sm_sc.tb_nn_path tb_a_asso_path
            on tb_a_asso_path.work_no = i_work_no 
              and tb_a_asso_path.fore_node_no = tb_a_fore_node.node_no
              and tb_a_fore_node.is_back_node
          left join sm_sc.tb_nn_node tb_a_asso_back_node 
            on tb_a_asso_back_node.work_no = i_work_no
              and tb_a_asso_back_node.node_no = tb_a_asso_path.back_node_no
              and tb_a_asso_path.path_ord_no <> tb_a_path.path_ord_no
              and tb_a_asso_back_node.is_back_node
          where tb_a_main_fore_node.work_no = i_work_no
            and tb_a_main_fore_node.nn_depth_no = v_cur_nn_depth
          group by tb_a_back_node.node_no 
          having max(tb_a_fore_node.nn_depth_no) <= v_cur_nn_depth
            and   -- 不是反向传播求导所需要的自变量，才可以被清理。否则，反向传播求导会用到，也不能被清理。
              max
              (
                -- fore_node 第一目求导是否用到该 back 变量
                case 
                  when tb_a_path.path_ord_no = 1 
                    then substr(tb_a_fore_dic.enum_value, 1, 1) 
                  when tb_a_path.path_ord_no = 2 and tb_a_asso_back_node.node_no is not null
                    then substr(tb_a_fore_dic.enum_value, 3, 1) 
                  when tb_a_path.path_ord_no = 3 
                    then '0'
                  else '0'
                end
                ||
                -- fore_node 第二目求导是否用到该 back 变量
                case 
                  when tb_a_path.path_ord_no = 2 
                    then substr(tb_a_fore_dic.enum_value, 2, 1) 
                  when tb_a_path.path_ord_no = 1 and tb_a_asso_back_node.node_no is not null
                    then substr(tb_a_fore_dic.enum_value, 4, 1) 
                  when tb_a_path.path_ord_no = 3 
                    then '0'
                  else '0'
                end
              )
              = '00'
        ),
        cte_upd_node_k_2_null as 
        (
          update sm_sc.tb_nn_node tb_a_back_node
          set 
            p_node_depdt        = null
          from cte_clear_kv_list tb_a_key_list
          where tb_a_back_node.node_no = tb_a_key_list.node_no
            and tb_a_back_node.work_no = i_work_no
        )
        select  --  perform 暂不支持 cte，所以采用 select  into null 暂时代替。
          -- 同一 dml/dql 中，要尽量避免多次调用 sm_sc.__fv_delete_kv，所以将待清理 arr_key 集中在一处 __fv_delete_kv，放在最外层，
          sm_sc.__fv_delete_kv
          (
            (select array_agg(p_node_depdt) from cte_clear_kv_list)
          ) into v_null
        ;
      
-- -- debug
-- raise notice 'debug 002.9. v_null: %', v_null;
        v_null := null;
      end if;
-- -- -- debug
-- raise notice 'debug 003. step fore end: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

    end loop;
-- -----------------------------------------------------------------------
    -- 对 i_depdt_01 小批量抽样，获得本轮 v_depdt
    if v_is_output_01
    then 
      select 
        array_agg(tb_a_buff.i_depdt_01 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_01
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_01'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;
    
-- -- debug
-- raise notice 'debug 003.1. v_depdt_01: %', array_dims(v_depdt_01);
   
    if v_is_output_02
    then 
       select 
         array_agg(tb_a_buff.i_depdt_02 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
       into 
         v_depdt_02
       from sm_sc.tb_nn_node tb_a_sour
       inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
         on tb_a_sour.node_type = 'output_02'
       inner join sm_sc.tb_nn_train_input_buff tb_a_buff
         on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
       where tb_a_sour.work_no = i_work_no
         and tb_a_buff.work_no = i_work_no
       ;
    end if;
    
    if v_is_output_03
    then
      select 
        array_agg(tb_a_buff.i_depdt_03 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_03
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_03'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;
    
    if v_is_output_04
    then 
      select 
        array_agg(tb_a_buff.i_depdt_04 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_04
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_04'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;

    -- 损失函数
    v_cur_loss = 
      sm_sc.fv_lambda_loss
      (
        v_loss_fn_type :: varchar(32)
      , (select sm_sc.__fv_get_kv(p_node_depdt) from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_01')
      , v_depdt_01
      , (select sm_sc.__fv_get_kv(p_node_depdt) from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_02')
      , v_depdt_02
      , (select sm_sc.__fv_get_kv(p_node_depdt) from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_03')
      , v_depdt_03
      , (select sm_sc.__fv_get_kv(p_node_depdt) from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_04')
      , v_depdt_04
      ) 
    ;

-- -- -- debug
-- raise notice 'debug 004. step fore end: len of v_depdt: %; v_cur_loss: %; at %;', array_dims(v_depdt_01), v_cur_loss, date_trunc('milliseconds', now());

-- -----------------------------------------------------------------------
    -- 反向传播
    -- 反向传播起点：全局损失函数对输出 z 求导(均方差、最小二乘法)
    -- 规约：node_type in ('output_01', 'output_02', 'output_03', 'output_04') 各只有一个
    update sm_sc.tb_nn_node t_a_tar
    set 
      p_node_dloss_ddepdt = 
        sm_sc.__fv_set_kv 
        (
          sm_sc.fv_lambda_dloss_dz
          (
            v_loss_fn_type
          , sm_sc.__fv_get_kv(t_a_tar.p_node_depdt)
          , case node_type when 'output_01' then v_depdt_01 when 'output_02' then v_depdt_02 when 'output_03' then v_depdt_03 when 'output_04' then v_depdt_04 end
          , node_type
          )
        , v_model_code_6 || 
          '_' || i_work_no :: varchar   ||
          '__' || node_no :: varchar ||     -- back_node_no
          '_dldi__'   -- 'd_loss_d_indepdt'
        )
    , learn_cnt_back = v_cur_learn_cnt + 1 -- 强制对齐训练次数，避免路径计算混乱    -- t_tar.learn_cnt_back + 1
    where node_type in ('output_01', 'output_02', 'output_03', 'output_04')
      and work_no = i_work_no   -- 2021082501
    ;
    commit;

-- -- -- debug
-- raise notice 'debug 005. step fore end: v_cur_learn_cnt: %; at %;', v_cur_learn_cnt, date_trunc('milliseconds', now());

    v_cur_nn_depth := v_nn_depth;
    
-- -- for debug
-- v_cur_node_nos := (select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('output_01', 'output_02', 'output_03', 'output_04') limit 1);

    while 
      exists 
      (
        select  from sm_sc.tb_nn_node tb_a_back
        where tb_a_back.work_no = i_work_no    -- 2021082501
          and tb_a_back.learn_cnt_back <= v_cur_learn_cnt
          and tb_a_back.is_back_node is true
        limit 1
      )
      and v_cur_nn_depth > 0
    loop

-- -- -- debug
-- raise notice 'debug 006. step back begin: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

      with 
      cte_dloss_dindepdt as
      (
        select 
          tb_a_back.node_no,
          sm_sc.fa_mx_sum
          (
            sm_sc.__fv_get_kv
            (
              sm_sc.fv_lambda_arr_dloss_dindepdt_p
              (
                v_model_code_6
              , i_work_no
              , tb_a_fore.node_no
              , tb_a_path.back_node_no
              , tb_a_fore.node_fn_type
              -- 以下入参是否传入，由 字典表 enum_name = 'node_fn_type_delta_method' 控制
              -- 对于聚合算子的所有自变量，无论顺序，统一以 path_ord_no = 1 配置该控制
              , case 
                  when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                       and substr(tb_a_dic.enum_value, 1, 1) = '1'
                    or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 2, 1) = '1'
                    then tb_a_back.p_node_depdt 
                end
              , tb_a_path.path_ord_no 
              , case 
                  when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                       and substr(tb_a_dic.enum_value, 3, 1) = '1'
                    or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 4, 1) = '1'
                    then tb_a_back_co.p_node_depdt
                end
              , tb_a_fore.node_fn_asso_value
              , case 
                  when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                       and substr(tb_a_dic.enum_value, 5, 1) = '1'
                    or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 6, 1) = '1'
                    then tb_a_fore.p_node_depdt
                end
              , tb_a_fore.p_node_dloss_ddepdt
              , case 
                  when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                       and substr(tb_a_dic.enum_value, 7, 1) = '1'
                    or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 8, 1) = '1'
                    or tb_a_path.path_ord_no = 3 -- 当前仅 05_conv_2d 支持第三目运算符，且求导需要自变量规格
                    then tb_a_back.node_depdt_len
                end
              , tb_a_path.p_ddepdt_dindepdt
              )
            , true
            )
          ) as a_dloss_dindepdt
        from sm_sc.tb_nn_node tb_a_back
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.back_node_no = tb_a_back.node_no
      	  and tb_a_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_path.fore_node_no
          and tb_a_fore.work_no = i_work_no   -- 2021082501
        left join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type_delta_method'
          -- and tb_a_dic.enum_group = '1'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
        left join sm_sc.tb_nn_path tb_a_path_co
          on tb_a_path_co.fore_node_no = tb_a_fore.node_no
      	  and tb_a_path_co.work_no = i_work_no   -- 2021082501
          and (tb_a_path.path_ord_no = 1 and tb_a_path_co.path_ord_no = 2
               or tb_a_path.path_ord_no = 2 and tb_a_path_co.path_ord_no = 1)
        left join sm_sc.tb_nn_node tb_a_back_co
          on tb_a_back_co.node_no = tb_a_path_co.back_node_no
          and tb_a_back_co.work_no = i_work_no   -- 2021082501
        where tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.nn_depth_no = v_cur_nn_depth - 1
          and tb_a_back.is_back_node
        group by tb_a_back.node_no
      )
      
-- -- for debug v_cur_node_nos
-- ,
-- cte_upd_node as
-- (

      update sm_sc.tb_nn_node t_tar
      set 
        p_node_dloss_ddepdt = 
          sm_sc.__fv_set_kv 
          (
            t_sour.a_dloss_dindepdt
          , v_model_code_6 || 
            '_' || i_work_no :: varchar   ||
            '_' || t_tar.node_no :: varchar ||
            '__dldi__'  -- 'd_loss_d_indepdt'
          )
      , learn_cnt_back = v_cur_learn_cnt + 1 -- 强制对齐训练次数，避免路径计算混乱    -- t_tar.learn_cnt_back + 1
      from cte_dloss_dindepdt t_sour
      where t_sour.node_no = t_tar.node_no

-- -- for debug v_cur_node_nos
--   returning t_tar.node_no as a_cur_node_no
-- )
-- select 
--   array_agg(a_cur_node_no) into v_cur_node_nos
-- from cte_upd_node

      ;
      commit;
      
      -- de_broadcast, dloss_ddepdt 的广播逆运算
      update sm_sc.tb_nn_node t_tar
      set 
        p_node_dloss_ddepdt = 
          sm_sc.__fv_set_kv
          (
            sm_sc.fv_aggr_chunk_sum(sm_sc.__fv_get_kv(p_node_dloss_ddepdt), node_depdt_len)
          , v_model_code_6 || 
            '_' || t_tar.work_no :: varchar ||
            '_' || t_tar.node_no :: varchar ||
            '__dldi__'  -- 'd_loss_d_indepdt'
          )
      where work_no = i_work_no   -- 2021082501
        and nn_depth_no = v_cur_nn_depth - 1
        and cardinality(sm_sc.__fv_get_kv(p_node_dloss_ddepdt)) / cardinality(sm_sc.__fv_get_kv(p_node_depdt)) > 1
        and cardinality(sm_sc.__fv_get_kv(p_node_dloss_ddepdt)) % cardinality(sm_sc.__fv_get_kv(p_node_depdt)) = 0
      ; 
      commit;

-- -- -- debug
-- raise notice 'debug 007. step back end: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at: %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

      if not i_tensor_log
      then
        -- 清理当前层反向传播后的无用张量 kv
        -- 清理 dloss_ddepdt, depdt
        with 
        -- 情形一: 不被比 v_cur_nn_depth 更小 depth 的节点，当求导协参(另一个自变量)
        cte_clear_kv_list_covalue as 
        (
          select
            tb_a_main_back_node.node_no
          , max(tb_a_main_back_node.p_node_depdt) as p_node_depdt
          , array_agg(distinct tb_a_path.p_ddepdt_dindepdt) filter(where tb_a_path.p_ddepdt_dindepdt is not null) as p_ddepdt_dindepdt_s
          from sm_sc.tb_nn_node tb_a_main_back_node
          inner join sm_sc.tb_nn_path tb_a_main_path
            on tb_a_main_path.back_node_no = tb_a_main_back_node.node_no
              and tb_a_main_path.work_no = i_work_no
          inner join sm_sc.tb_nn_node tb_a_fore_node
            on tb_a_fore_node.node_no = tb_a_main_path.fore_node_no
              and tb_a_fore_node.work_no = i_work_no
          inner join sm_sc.tb_dic_enum tb_a_dic 
            on tb_a_dic.enum_name = 'node_fn_type_delta_method'
              and tb_a_dic.enum_key = tb_a_fore_node.node_fn_type
          inner join sm_sc.tb_nn_path tb_a_path
            on tb_a_path.work_no = i_work_no
            and tb_a_path.fore_node_no = tb_a_fore_node.node_no
          left join sm_sc.tb_nn_node tb_a_back_node
            on tb_a_back_node.work_no = i_work_no
            and tb_a_back_node.node_no = tb_a_path.back_node_no
            and tb_a_back_node.is_back_node
            -- 双目求导需要协参，目位置分别判断
            and 
            (
              tb_a_main_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 4, 1) = '1' 
              or 
              tb_a_main_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 3, 1) = '1'
            )             
          where tb_a_main_back_node.work_no = i_work_no
            and tb_a_main_back_node.nn_depth_no = v_cur_nn_depth
            -- and tb_a_main_back_node.p_node_dloss_ddepdt is not null
            and tb_a_main_back_node.node_type <> 'weight'
            and tb_a_main_back_node.node_fn_type <> '00_const'
          group by tb_a_main_back_node.node_no 
          having min(tb_a_back_node.nn_depth_no) >= v_cur_nn_depth
            or count(tb_a_back_node.nn_depth_no) = 0
        ),
        cte_upd_node_k_2_null as 
        (
          update sm_sc.tb_nn_node tb_a_fore_node
          set 
            p_node_depdt        = null
          from cte_clear_kv_list_covalue tb_a_key_list
          where tb_a_fore_node.node_no = tb_a_key_list.node_no
            and tb_a_fore_node.work_no = i_work_no
        )
        select   --  perform 暂不支持 cte，所以采用 select  into null 暂时代替。
          sm_sc.__fv_delete_kv
          (
            (
              select 
                array_agg(p_node_depdt) 
              from cte_clear_kv_list_covalue
            )
          ) into v_null
        ;
        commit;
        
        -- 情形二: 不被比 v_cur_nn_depth - 1 更小 depth 的节点，当做因变量
        with
        cte_clear_kv_list_dloss as 
        (
          select 
            tb_a_fore_node.node_no
          , max(tb_a_fore_node.p_node_dloss_ddepdt) as p_node_dloss_ddepdt
          , array_agg(distinct tb_a_path.p_ddepdt_dindepdt) filter(where tb_a_path.p_ddepdt_dindepdt is not null) as p_ddepdt_dindepdt_s
          from sm_sc.tb_nn_node tb_a_main_back_node
          inner join sm_sc.tb_nn_path tb_a_main_path
            on tb_a_main_path.back_node_no = tb_a_main_back_node.node_no
              and tb_a_main_path.work_no = i_work_no
          inner join sm_sc.tb_nn_node tb_a_fore_node
            on tb_a_fore_node.node_no = tb_a_main_path.fore_node_no
              and tb_a_fore_node.work_no = i_work_no
          inner join sm_sc.tb_nn_path tb_a_path
            on tb_a_path.work_no = i_work_no
            and tb_a_path.fore_node_no = tb_a_fore_node.node_no
          left join sm_sc.tb_nn_node tb_a_back_node
            on tb_a_back_node.work_no = i_work_no
            and tb_a_back_node.node_no = tb_a_path.back_node_no
            and tb_a_back_node.is_back_node
          where tb_a_main_back_node.work_no = i_work_no
            and tb_a_main_back_node.nn_depth_no = v_cur_nn_depth - 1
            and tb_a_fore_node.p_node_dloss_ddepdt is not null
            and tb_a_fore_node.node_type <> 'weight'
            and tb_a_fore_node.node_fn_type <> '00_const'
          group by tb_a_fore_node.node_no
          having min(tb_a_back_node.nn_depth_no) >= v_cur_nn_depth - 1
            or count(tb_a_back_node.nn_depth_no) = 0
        ),
        cte_upd_node_k_2_null as 
        (
          update sm_sc.tb_nn_node tb_a_fore_node
          set 
            p_node_dloss_ddepdt = null
          from cte_clear_kv_list_dloss tb_a_key_list
          where tb_a_fore_node.node_no = tb_a_key_list.node_no
            and tb_a_fore_node.work_no = i_work_no
        ),
        cte_upd_path_k_2_null as 
        (
          update sm_sc.tb_nn_path tb_a_fore_path
          set 
            p_ddepdt_dindepdt = null
          from (select node_no, a_p_ddepdt_dindepdt from cte_clear_kv_list_dloss, unnest(p_ddepdt_dindepdt_s) a_p_ddepdt_dindepdt) tb_a_key_list
          where tb_a_fore_path.fore_node_no = tb_a_key_list.node_no
            and tb_a_fore_path.p_ddepdt_dindepdt = tb_a_key_list.a_p_ddepdt_dindepdt
            and tb_a_fore_path.work_no = i_work_no
        )
        select   --  perform 暂不支持 cte，所以采用 select  into null 暂时代替。
          sm_sc.__fv_delete_kv
          (
            (
              select 
                array_agg(p_node_dloss_ddepdt) 
                || 
                sm_sc.fa_array_concat(p_ddepdt_dindepdt_s) 
              from cte_clear_kv_list_dloss
            )
          ) into v_null
        ;
        
        commit;
-- -- debug
-- raise notice 'debug 007.1. v_null: %', v_null;
        v_null := null;
      end if;
      
      v_cur_nn_depth := v_cur_nn_depth - 1;

    end loop;
    
    -- L1 正则化
    if i_l1 is not null 
    then  
      -- update sm_sc.tb_nn_node tar
      -- set 
      --   p_node_depdt = tar.p_node_depdt -` i_l1
      -- where work_no = i_work_no   -- 2021082501
      --   and node_type = 'weight'
      -- ;
      -- commit;
      
      perform
        sm_sc.__fv_set_kvs
        (
          array_agg 
          (
            (
              sm_sc.__fv_get_kv(tar.p_node_depdt) -` i_l1
            ) :: sm_sc.__typ_array_ex
            order by node_no
          )
        , array_agg(p_node_depdt order by node_no)
        )
      from sm_sc.tb_nn_node tar
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
        and p_node_depdt is not null
      ;
      commit;
    end if;
    
    -- L2 正则化
    if i_l2 is not null 
    then  
      -- update sm_sc.tb_nn_node tar
      -- set 
      --   p_node_depdt = tar.p_node_depdt *` (1.0 - i_l2)
      -- where work_no = i_work_no   -- 2021082501
      --   and node_type = 'weight'
      -- ;
      -- commit;
      
      perform
        sm_sc.__fv_set_kvs
        (
          array_agg 
          (
            (
              sm_sc.__fv_get_kv(tar.p_node_depdt) *` (1.0 - i_l2)
            ) :: sm_sc.__typ_array_ex
            order by node_no
          )
        , array_agg(p_node_depdt order by node_no)
        )
      from sm_sc.tb_nn_node tar
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
        and p_node_depdt is not null
      ;
      commit;
    end if;

    -- 更新 w 参数，一次训练完成
    if not i_is_adam or i_is_adam is null
    then 
      -- update sm_sc.tb_nn_node tar
      -- set 
      --   -- p_node_depdt = tar.p_node_depdt -` (i_learn_rate *` tar.p_node_dloss_ddepdt)
      --   p_node_depdt = 
      --     tar.p_node_depdt 
      --     -` 
      --     (
      --       i_learn_rate 
      --       *` 
      --       -- 梯度截断，绝对值不超过 1000
      --       (
      --         100.0 :: float 
      --         *` 
      --         sm_sc.fv_tanh
      --         (
      --           tar.p_node_dloss_ddepdt 
      --           /` 
      --           100.0 :: float
      --         )
      --       )
      --       -- -- 如下梯度截断和差异更平缓一些
      --       -- (
      --       --   100.0 :: float 
      --       --   *` 
      --       --   (
      --       --     sm_sc.fv_tanh
      --       --     (
      --       --       (
      --       --         (@|` tar.p_node_dloss_ddepdt)
      --       --         /` 
      --       --         100.0 :: float
      --       --       )
      --       --       ^` 
      --       --       0.5 :: float
      --       --     )
      --       --     ^`
      --       --     2.0 :: float
      --       --   )
      --       --   *` 
      --       --   ((<>` tar.p_node_dloss_ddepdt) :: float[])
      --       -- )
      --     )
      -- where work_no = i_work_no   -- 2021082501
      --   and node_type = 'weight'
      -- ;
      -- commit;
      
      perform
        sm_sc.__fv_set_kvs
        (
          array_agg 
          (
            (
              sm_sc.__fv_get_kv(p_node_depdt)
              -` 
              (
                i_learn_rate 
                *` 
                -- 梯度截断，绝对值不超过 1000
                (
                  100.0 :: float 
                  *` 
                  sm_sc.fv_tanh
                  (
                    sm_sc.__fv_get_kv(p_node_dloss_ddepdt)
                    /` 
                    100.0 :: float
                  )
                )
              )
            ) :: sm_sc.__typ_array_ex
          order by node_no
          )
        , array_agg(p_node_depdt order by node_no)
        )
      from sm_sc.tb_nn_node
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
        and p_node_dloss_ddepdt is not null
      ;

    else
      -- 采用 Adam 梯度下降算法
      update sm_sc.tb_nn_node tar
      set 
        -- p_node_depdt = tar.p_node_depdt -` (i_learn_rate *` tar.p_node_dloss_ddepdt)
        cost_delta_l1 = (v_beta_l1 *` coalesce(case when array_length(cost_delta_l1, 1) = array_length(sm_sc.__fv_get_kv(tar.p_node_dloss_ddepdt), 1) then cost_delta_l1 else null end, sm_sc.__fv_get_kv(tar.p_node_dloss_ddepdt))) +` ((1.0 :: float - v_beta_l1) *` sm_sc.__fv_get_kv(tar.p_node_dloss_ddepdt)),
        cost_delta_l2 = (v_beta_l2 *` coalesce(case when array_length(cost_delta_l2, 1) = array_length(sm_sc.__fv_get_kv(tar.p_node_dloss_ddepdt), 1) then cost_delta_l2 else null end, (sm_sc.__fv_get_kv(tar.p_node_dloss_ddepdt) ^` 2.0 :: float))) +` ((1.0 :: float - v_beta_l2) *` (sm_sc.__fv_get_kv(tar.p_node_dloss_ddepdt) ^` 2.0 :: float))
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
      
      -- update sm_sc.tb_nn_node tar
      -- set 
      --   p_node_depdt =
      --     tar.p_node_depdt 
      --     -`
      --     (
      --       (
      --         cost_delta_l1 *`
      --         (i_learn_rate / (1.0 :: float - (v_beta_l1 ^ (least(v_cur_learn_cnt, 7070) + 1))))      -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
      --       ) /`
      --       (
      --         sm_sc.fv_ele_replace
      --         (
      --           (
      --             cost_delta_l2 /` (1.0 :: float - (v_beta_l2 ^ (least(v_cur_learn_cnt, 7070) + 1)))  -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
      --           ) ^` 0.5 :: float
      --           , array[0.0 :: float]
      --           , 1e-128 :: float   -- 1e-128 :: float        
      --         )
      --       )
      --     )
      -- where work_no = i_work_no   -- 2021082501
      --   and node_type = 'weight'
      -- ;
      -- commit;
      
      perform
        sm_sc.__fv_set_kvs
        (
          array_agg
          (
            (
              sm_sc.__fv_get_kv(p_node_depdt)
              -` 
              (
                i_learn_rate 
                -`
                (
                  (
                    cost_delta_l1 *`
                    (i_learn_rate / (1.0 :: float - (v_beta_l1 ^ (least(v_cur_learn_cnt, 7070) + 1))))      -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
                  ) /`
                  (
                    sm_sc.fv_ele_replace
                    (
                      (
                        cost_delta_l2 /` (1.0 :: float - (v_beta_l2 ^ (least(v_cur_learn_cnt, 7070) + 1)))  -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
                      ) ^` 0.5 :: float
                      , array[0.0 :: float]
                      , 1e-128 :: float   -- 1e-128 :: float        
                    )
                  )
                )
              )
            ) :: sm_sc.__typ_array_ex
            order by node_no
          )
        , array_agg(p_node_depdt order by node_no)
        )
      from sm_sc.tb_nn_node
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
        and p_node_depdt is not null
      ;
    end if;

    v_cur_learn_cnt := v_cur_learn_cnt + 1;

    -- 寄存至 sm_sc.tb_classify_task
    if v_cur_learn_cnt % 1 = 0     -- 或者 100 次， 1000 次， 10000次。。。
      or v_cur_learn_cnt >= v_limit_train_times
      or v_cur_loss < i_loss_delta_least_stop_threshold
    then 
      insert into sm_sc.tb_classify_task
      (
        work_no
      , learn_cnt
      , loss_delta
      )
      select 
        i_work_no
      , v_cur_learn_cnt
      , v_cur_loss
      on conflict(work_no) do 
      update set
        learn_cnt = v_cur_learn_cnt
      , loss_delta = v_cur_loss
      ;
    
      insert into sm_sc.__vt_nn_node
      (
        work_no                      ,
        node_no                      ,        
        node_type                    ,
        node_fn_type                 ,
        node_fn_asso_value           ,
        nn_depth_no                  ,
        node_depdt_vals              ,
        p_node_depdt                  
      )
      select 
        tb_a_sour.work_no                      ,
        tb_a_sour.node_no                      ,        
        tb_a_sour.node_type                    ,
        tb_a_sour.node_fn_type                 ,
        tb_a_sour.node_fn_asso_value           ,
        tb_a_sour.nn_depth_no                  ,
        sm_sc.__fv_get_kv(tb_a_sour.p_node_depdt),
        case 
          when tb_a_sour.node_type = 'weight' 
            or tb_a_sour.node_fn_type = '00_const' 
          then 
            sm_sc.__fv_set_kv
            (
              sm_sc.__fv_get_kv(tb_a_sour.p_node_depdt)
            , -- v_model_code_6 || 
              '_' || i_work_no :: varchar   ||
              '_' || tb_a_sour.node_no :: varchar ||
              '____'  -- 'depdt'
            )
        end as p_node_depdt                 
      from sm_sc.tb_nn_node tb_a_sour
      where tb_a_sour.work_no = i_work_no
      on conflict(work_no, node_no) do 
      update set
        node_type                 = EXCLUDED.node_type                    ,
        node_fn_type              = EXCLUDED.node_fn_type                 ,
        node_fn_asso_value        = EXCLUDED.node_fn_asso_value           ,
        nn_depth_no               = EXCLUDED.nn_depth_no                  ,
        node_depdt_vals           = sm_sc.__fv_get_kv(EXCLUDED.p_node_depdt),
        p_node_depdt              
          = case 
              when EXCLUDED.node_type = 'weight' or EXCLUDED.node_fn_type = '00_const' 
              then
                sm_sc.__fv_set_kv
                (
                  sm_sc.__fv_get_kv(EXCLUDED.p_node_depdt)
                , -- v_model_code_6 || 
                  '_' || i_work_no :: varchar   ||
                  '_' || EXCLUDED.node_no :: varchar ||
                  '____'  -- 'depdt'
                ) 
            end
      ;    
      commit;

      -- 以下 upsert 是为了尽量支持训练时也能验证 sess_id = 0 的推理，
      -- 训练全部结束后也会彻底 delete_insert 一份。
      insert into sm_sc.__vt_tmp_nn_node
      (
        sess_id              
      , work_no              
      , node_no              
      , node_type            
      , node_fn_type         
      , node_fn_asso_value   
      , nn_depth_no          
      , p_node_depdt      
      )
      select 
        0                    
      , work_no              
      , node_no              
      , node_type            
      , coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  
      , case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         
      , nn_depth_no                     
      , case 
          -- -- -- when 'prod_input' 
          -- -- --   then i_in
          -- when node_type = 'input_01' 
          --   then i_indepdt_01
          -- when node_type = 'input_02' 
          --   then i_indepdt_02
          -- when node_type = 'input_03' 
          --   then i_indepdt_03
          -- when node_type = 'input_04' 
          --   then i_indepdt_04
          -- -- when 'offset'
          -- --   then array[array[1]]
          when node_type = 'weight' or node_fn_type = '00_const'
            then p_node_depdt -- || '__0'
          else null
        end             
      from sm_sc.__vt_nn_node
      where work_no = i_work_no
      on conflict(sess_id, work_no, node_no) do 
      update set
        node_type                 = EXCLUDED.node_type                    ,
        node_fn_type              = EXCLUDED.node_fn_type                 ,
        node_fn_asso_value        = EXCLUDED.node_fn_asso_value           ,
        nn_depth_no               = EXCLUDED.nn_depth_no                  ,
        p_node_depdt              = EXCLUDED.p_node_depdt
      ;    
      commit;

    end if;
    
    if not i_tensor_log
    then 
      -- 清理 kv 张量: dloss_ddepdt, depdt, ddepdt_dindepdt
      perform 
        sm_sc.__fv_delete_kv
        (
          array_agg(p_node_dloss_ddepdt) || array_agg(p_node_depdt)
        )
      from sm_sc.tb_nn_node 
      where work_no = i_work_no
        and node_type is distinct from 'weight' 
        and node_fn_type is distinct from '00_const'
      ;
      update sm_sc.tb_nn_node
      set 
        p_node_depdt        = null
      , p_node_dloss_ddepdt = null
      where work_no = i_work_no
        and node_type is distinct from 'weight' 
        and node_fn_type is distinct from '00_const'
      ;
      
      perform 
        sm_sc.__fv_delete_kv
        (
          array_agg(p_ddepdt_dindepdt)
        )
      from sm_sc.tb_nn_path 
      where work_no = i_work_no
      ;
      update sm_sc.tb_nn_path 
      set p_ddepdt_dindepdt = null
      where work_no = i_work_no
      ;
      
      -- -- 清理本轮训练该模型有关的所有非 weight 张量 kv
      -- -- 该清理是不必要的，而且也跳过了 get/set 接口，调用了 __vt_array_kv，入侵了 kv 存储，改写会很繁琐
      -- perform 
      --   sm_sc.__fv_delete_kv
      --   (
      --     array_agg(arr_key)
      --   )
      -- from sm_sc.__vt_array_kv tb_a_kv
      -- left join sm_sc.tb_nn_node tb_a_node
      --   on tb_a_node.p_node_depdt = tb_a_kv.arr_key 
      --     and tb_a_node.work_no = i_work_no
      -- where arr_key like (v_model_code_6 || '_' || i_work_no :: varchar || '_%')
      --   and tb_a_node.node_type is distinct from 'weight' 
      --   and tb_a_node.node_fn_type is distinct from '00_const'
      -- ;
    end if;
    
    raise notice 'Training task end report: v_cur_learn_cnt: %; v_cur_loss: %; at %;', v_cur_learn_cnt, v_cur_loss, date_trunc('milliseconds', now());

  end loop;
      
-- debug
raise notice 'Training No. % end; v_cur_loss: %; at %;', v_cur_learn_cnt, v_cur_loss, date_trunc('milliseconds', now());
  
  -- 更新 0 号 sess 部署。规约: 0 号 session 为每轮训练后，默认部署。
  delete from sm_sc.__vt_tmp_nn_node
  where work_no = i_work_no
    and sess_id = 0
  ;
  
  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id              
  , work_no              
  , node_no              
  , node_type            
  , node_fn_type         
  , node_fn_asso_value   
  , nn_depth_no          
  , p_node_depdt      
  )
  select 
    0                    
  , work_no              
  , node_no              
  , node_type            
  , coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  
  , case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         
  , nn_depth_no                     
  , case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then p_node_depdt -- || '__0'
      else null
    end             
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  -- on conflict(work_no, node_no) do 
  -- update set
  --   node_type                 = EXCLUDED.node_type                    ,
  --   node_fn_type              = EXCLUDED.node_fn_type                 ,
  --   node_fn_asso_value        = EXCLUDED.node_fn_asso_value           ,
  --   nn_depth_no               = EXCLUDED.nn_depth_no                  ,
  --   p_node_depdt              = EXCLUDED.p_node_depdt
  ;    
  commit;
  
end
$$
language plpgsql;

-- -- 用例 1, 二分类
-- -- -- 执行准备和检查
-- -- call sm_sc.prc_nn_prepare
-- -- (
-- --   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
-- --   2500      -- ,
-- --   -- 200
-- -- );


-- -- 开始训练
-- call sm_sc.prc_nn_train_p
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   1        ,
--   0.9      ,
--   0.1
-- );

-- -- -- 观察输出
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- select node_no, node_fn_type, learn_cnt_fore, learn_cnt_back, tb_a.p_node_depdt, (tb_a.reg_w).m_vals  from sm_sc.tb_nn_node tb_a where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- select 
--   sm_sc.ft_nn_in_out
--   (
--     pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--     array[array[0.0, 0.0], array[0.0, 1.0], array[1.0, 0.0], array[1.0, 1.0]] 
--       +` sm_sc.fv_new_randn(0.0, 0.1, array[4, 2])
--   )

-- -- -- 期望用例举例：
-- -- -- select sigmoid(w31 * sigmoid(w11 + w12 * x1 + w13 * x2) + w32 * sigmoid(w21 + w22 * x1 + w23 * x2))
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 0 + 20.0 * 0) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 0 + 20.0 * 0)) union all   -- 期望输出，0
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 0 + 20.0 * 1) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 0 + 20.0 * 1)) union all   -- 期望输出，1
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 1 + 20.0 * 0) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 1 + 20.0 * 0)) union all   -- 期望输出，1
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 1 + 20.0 * 1) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 1 + 20.0 * 1))             -- 期望输出，0

-- -- -----------------------------------------------------------------------------
-- -- 用例 2, 多分类
-- -- -- 执行准备和检查
-- -- call sm_sc.prc_nn_prepare
-- -- (
-- --   2021121701   ,
-- --   5000      -- ,
-- --   -- 200
-- -- );
-- 
-- 
-- -- 开始训练
-- call sm_sc.prc_nn_train_p
-- (
--   2021121701   ,
--   2        ,
--   0.8      ,
--   0.1
-- );
-- 
-- -- -- 观察输出
-- select 
--   sm_sc.ft_nn_in_out
--   (
--     2021121701, 
--     array[array[0.0, 0.0, 0.0], array[0.0, 1.0, 0.0], array[1.0, 0.0, 0.0], array[1.0, 1.0, 0.0]
--           , array[0.0, 0.0, 1.0], array[0.0, 1.0, 1.0], array[1.0, 0.0, 1.0], array[1.0, 1.0, 1.0]] 
--       +` sm_sc.fv_new_randn(0.0, 0.1, array[8, 3])
--   )

drop procedure if exists sm_sc.prc_nn_add_sess;
create or replace procedure sm_sc.prc_nn_add_sess
(
  i_work_no                          bigint            ,                                                                                                -- 训练任务编号
  i_sess_cnt                         int               default    1                                                                                                      -- 最大训练次数，然后未完成中止
)
as
$$
declare -- here
  v_sess_range        int8range       := sm_sc.fv_get_global_seq(i_sess_cnt)    ;
  

begin
  set search_path to public;
  
  -- -- -- 规约: 0 号 session 为每轮训练后，默认部署。
  -- -- if 0 :: bigint <@ i_sess_multirange
  -- -- then 
  -- --   raise exception 'can''t deploy 0 as sess id.  0 is default sess id for train task at pg4ml.  ';
  -- -- end if;
  
  insert into sm_sc.__vt_nn_sess
  (
    work_no
  , sess_id
  , sess_status
  )
  select 
    i_work_no
  , a_sess_no
  , '0'              -- 参看字典表  nn_sess_status。0: 空闲; 1 占用;
  from generate_series(lower(v_sess_range), upper(v_sess_range) - 1) tb_a(a_sess_no)
  ;
  
  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id                        ,
    work_no                        ,
    node_no                        ,
    node_type                      ,
    node_fn_type                   ,
    node_fn_asso_value        ,
    nn_depth_no                    ,
    node_depdt_vals             
  )
  select 
    a_sess_no                    ,
    work_no                         ,
    node_no                         ,
    node_type                       ,
    coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  ,
    case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         ,
    nn_depth_no                     ,
    case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then node_depdt_vals
      else null
    end             
  from sm_sc.__vt_nn_node
    , generate_series(lower(v_sess_range), upper(v_sess_range) - 1) tb_a(a_sess_no)
  where work_no = i_work_no
  ;
    
end
$$
language plpgsql;

-- call sm_sc.prc_nn_add_sess
-- (
--   -76543
-- , 3
-- )
drop procedure if exists sm_sc.prc_nn_add_sess_p;
create or replace procedure sm_sc.prc_nn_add_sess_p
(
  i_work_no                          bigint            ,                                                                                                -- 训练任务编号
  i_sess_cnt                         int               default    1                                                                                                      -- 最大训练次数，然后未完成中止
)
as
$$
declare -- here
  v_sess_range        int8range       := sm_sc.fv_get_global_seq(i_sess_cnt)    ;
  

begin
  set search_path to public;
  
  -- -- -- 规约: 0 号 session 为每轮训练后，默认部署。
  -- -- if 0 :: bigint <@ i_sess_multirange
  -- -- then 
  -- --   raise exception 'can''t deploy 0 as sess id.  0 is default sess id for train task at pg4ml.  ';
  -- -- end if;
  
  insert into sm_sc.__vt_nn_sess
  (
    work_no
  , sess_id
  , sess_status
  )
  select 
    i_work_no
  , a_sess_no
  , '0'              -- 参看字典表  nn_sess_status。0: 空闲; 1 占用;
  from generate_series(lower(v_sess_range), upper(v_sess_range) - 1) tb_a(a_sess_no)
  ;
  
  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id                        ,
    work_no                        ,
    node_no                        ,
    node_type                      ,
    node_fn_type                   ,
    node_fn_asso_value        ,
    nn_depth_no                    ,
    p_node_depdt             
  )
  select 
    a_sess_no                    ,
    work_no                         ,
    node_no                         ,
    node_type                       ,
    coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  ,
    case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         ,
    nn_depth_no                     ,
    case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then p_node_depdt
      else null
    end             
  from sm_sc.__vt_nn_node
    , generate_series(lower(v_sess_range), upper(v_sess_range) - 1) tb_a(a_sess_no)
  where work_no = i_work_no
  ;
    
end
$$
language plpgsql;

-- call sm_sc.prc_nn_add_sess_p
-- (
--   -76543
-- , 3
-- )
drop procedure if exists sm_sc.prc_nn_del_sess;
create or replace procedure sm_sc.prc_nn_del_sess
(
  i_work_no                          bigint            ,        -- 训练任务编号
  i_sess_multirange                  int8multirange
)
as
$$
-- declare -- here
  
begin
  set search_path to public;
  
  delete from sm_sc.__vt_nn_sess
  where work_no = i_work_no
    and sess_id <@ i_sess_multirange
  ;
  
  delete from sm_sc.__vt_tmp_nn_node
  where work_no = i_work_no
    and sess_id <@ i_sess_multirange
  ;
    
end
$$
language plpgsql;


drop procedure if exists sm_sc.prc_nn_del_sess;
create or replace procedure sm_sc.prc_nn_del_sess
(
  i_work_no                          bigint            ,        -- 训练任务编号
  i_sess_multirange                  int8multirange
)
as
$$
-- declare -- here
  
begin
  set search_path to public;
  
  delete from sm_sc.__vt_nn_sess
  where work_no = i_work_no
    and sess_id <@ i_sess_multirange
  ;
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_node_depdt)) 
  from sm_sc.__vt_tmp_nn_node 
  where work_no = i_work_no
    and sess_id <@ i_sess_multirang
  ;
  
  delete from sm_sc.__vt_tmp_nn_node
  where work_no = i_work_no
    and sess_id <@ i_sess_multirange
  ;
    
end
$$
language plpgsql;


drop procedure if exists sm_sc.prc_nn_release_sess;
create or replace procedure sm_sc.prc_nn_release_sess
(
  i_work_no      bigint
, i_sess_id      bigint
)
as
$$
-- declare -- here  

begin
  -- set search_path to public;
  update sm_sc.__vt_nn_sess
  set sess_status = '0'
  where work_no = i_work_no
    and sess_id = i_sess_id
  ;
end
$$
language plpgsql;

-- do
-- $$
-- declare 
--   v_sess_id  bigint   :=   907;
-- begin
--   call sm_sc.prc_nn_release_sess
--   (
--     -76543
--   , v_sess_id
--   );
--   raise notice 'release sess_id: %', v_sess_id;
-- end
-- $$
-- language plpgsql
drop procedure if exists sm_sc.prc_nn_subscribe_sess;
create or replace procedure sm_sc.prc_nn_subscribe_sess
(
     in i_work_no      bigint
, inout o_sess_id      bigint
)
as
$$
-- declare -- here  

begin
  -- set search_path to public;
  select  
    sess_id into o_sess_id
  from sm_sc.__vt_nn_sess 
  where work_no = i_work_no
    and sess_status = '0'
  limit 1 
  for update
  ;
  
  if o_sess_id is null
  then   
    raise exception 'sess id are all in use.  ';
  else
    update sm_sc.__vt_nn_sess
    set sess_status = '1'
    where work_no = i_work_no
      and sess_id = o_sess_id
    ;
  end if;
end
$$
language plpgsql;

-- do
-- $$
-- declare 
--   v_sess_id  bigint;
-- begin
--   call sm_sc.prc_nn_subscribe_sess
--   (
--     -76543
--   , v_sess_id
--   );
--   raise notice 'subscribe sess_id: %', v_sess_id;
-- end
-- $$
-- language plpgsql
drop procedure if exists sm_sc.prc_add_pretrain_from_infer(bigint, varchar(32));
create or replace procedure sm_sc.prc_add_pretrain_from_infer
(
  i_work_no           bigint
, i_loss_fn_type      varchar(32)
) as 
$$
declare
  v_cur_node_nos           bigint[];
begin
  delete from sm_sc.tb_classify_task where work_no = i_work_no;
  delete from sm_sc.tb_nn_node where work_no = i_work_no;
  delete from sm_sc.tb_nn_path where work_no = i_work_no;
  -- delete from sm_sc.tb_nn_train_input_buff where work_no = i_work_no;
  
  insert into sm_sc.tb_classify_task
  (
    work_no     
  , learn_cnt   
  , loss_fn_type
  )
  select 
    i_work_no     
  , 1   
  , i_loss_fn_type
  ;
  
  insert into sm_sc.tb_nn_node
  (
    work_no           
  , node_no           
  , node_type         
  , node_fn_type      
  , node_fn_asso_value
  , nn_depth_no       
  -- , learn_cnt_fore    
  -- , learn_cnt_back    
  -- , is_fore_node      
  -- , is_back_node  
  -- , dropout_ratio     
  -- , dropout_rescale   
  -- , dropout_depdt_val 
  -- , is_dropout        
  -- , node_depdt_len    
  -- , pick_depdt_idx    
                      
  , node_depdt_vals   
  -- , node_dloss_ddepdt 
  -- , cost_delta_l1     
  -- , cost_delta_l2     
  -- , node_grp_no       
  -- , node_grp_ord_no   
  -- , node_desc   
  )
  select 
    work_no           
  , node_no           
  , node_type         
  , node_fn_type      
  , node_fn_asso_value
  , nn_depth_no       
  , node_depdt_vals  
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  
  insert into sm_sc.tb_nn_path
  (
    work_no
  , fore_node_no
  , path_ord_no 
  , back_node_no
  )
  select 
    work_no
  , fore_node_no
  , path_ord_no 
  , back_node_no
  from sm_sc.__vt_nn_path
  where work_no = i_work_no
  ;
  
  commit;
  
    update sm_sc.tb_nn_node tb_a_tar
    set 
      -- -- node_depdt_len   = 
      -- --   array 
      -- --   [
      -- --     case tb_a_tar.node_fn_type 
      -- --       when '00_full_dataset' 
      -- --         then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
      -- --       when '00_buff_slice_rand_pick' 
      -- --         then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
      -- --     end,
      -- --     v_input_len_x
      -- --   ]
      -- -- ,                        -- -- array[array_length(tb_a_sour.i_indepdt_01, 1), array_length(tb_a_sour.i_indepdt_01, 2)],
      is_fore_node = 
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then false 
          when '00_buff_slice_rand_pick' 
            then true 
        end
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type = 'input_01'
    ;
    commit;
    
    update sm_sc.tb_nn_node tb_a_tar
    set 
      is_fore_node = (select is_fore_node from sm_sc.tb_nn_node where work_no = i_work_no and tb_a_tar.node_type = 'input_01')
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
    ;
    commit;
    
    -- 00_const 节点的准备工作
    update sm_sc.tb_nn_node
    set 
      is_fore_node = false
    , is_back_node = false
    where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
    ;
    commit;
    
-- -- -- debug
-- raise notice 'L226. begin: node_depdt_len of input: %;', (select node_depdt_len from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01');
  
    -- 标记前向传播节点
    -- 缺省 true. 从神经网络的初始起点（包括）开始审计，至 01_prod_mx, 05_conv_2d_grp_x, rand 类算子为止（不包括），沿途皆为 false    
    -- 更新 is_fore_node
    v_cur_node_nos := 
      (
        select array_agg(node_no) from sm_sc.tb_nn_node 
        where work_no = i_work_no 
          and 
          (
            node_type in ('input_01', 'input_02', 'input_03', 'input_04')
            or node_fn_type = '00_const' and node_type <> 'weight'
          )
      );     -- -- v_input_nodes
    while array_length(v_cur_node_nos, 1) > 0
    loop 
      -- pg 暂不支持 recursive cte 使用 aggr, 先只能用循环
      with
      cte_fore_nodes as 
      (
        select 
          tb_a_fore.node_no,
          max(tb_a_fore.node_fn_type) as node_fn_type
        from sm_sc.tb_nn_path tb_a_fore_path
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
            and tb_a_fore.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_path tb_a_back_path
          on tb_a_back_path.fore_node_no = tb_a_fore.node_no
            and tb_a_back_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.node_no = tb_a_back_path.back_node_no
            and tb_a_back.work_no = i_work_no   -- 2021082501
        where tb_a_fore_path.back_node_no = any(v_cur_node_nos)
          and tb_a_fore.node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
          and tb_a_fore_path.work_no = i_work_no   -- 2021082501
        group by tb_a_fore.node_no
        having count(distinct tb_a_back.node_no) =
                 count(distinct tb_a_back.node_no) 
                   filter 
                   (
                     where tb_a_back.is_fore_node is false
                   )
      ),
      cte_upd as
      (
        update sm_sc.tb_nn_node tb_a_tar
        set 
          is_fore_node = false
        from cte_fore_nodes tb_a_sour
        where tb_a_sour.node_no = tb_a_tar.node_no
          and tb_a_tar.work_no = i_work_no   -- 2021082501
      )
      select 
        array_agg(distinct node_no) into v_cur_node_nos 
      from cte_fore_nodes
      where node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
      ;
      commit;
  
-- -- -- debug
-- raise notice 'L345.: v_cur_node_nos: %;', v_cur_node_nos;
  
    end loop;
    
    -- 标记反向传播节点
    -- 缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true

    update sm_sc.tb_nn_node
    set 
      is_back_node = true
    where work_no = i_work_no   -- 2021082501
      -- and node_no = any(v_init_nodes)
      and node_type = 'weight'
    ;
    commit;

    with recursive
    cte_back_nodes as
    (
      select 
        tb_a_main.node_no
      from sm_sc.tb_nn_node tb_a_main
      where tb_a_main.work_no = i_work_no   -- 2021082501
        and tb_a_main.is_back_node is true
      union
      select
        tb_a_path.fore_node_no
      from cte_back_nodes tb_a_back
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.back_node_no = tb_a_back.node_no 
      where tb_a_path.work_no = i_work_no   -- 2021082501
    )
    update sm_sc.tb_nn_node tb_a_tar
    set
      is_back_node = true
    from cte_back_nodes tb_a_sour
    where tb_a_sour.node_no = tb_a_tar.node_no
      and tb_a_tar.work_no = i_work_no   -- 2021082501
      -- and tb_a_tar.node_type <> 'output_01'
    ;
    commit;   

end
$$
language plpgsql;

-- call 
-- sm_sc.prc_add_pretrain_from_infer
-- (
--   -000000002
-- , '201'
-- );
drop procedure if exists sm_sc.prc_add_pretrain_from_infer_p(bigint, varchar(32), char(6));
create or replace procedure sm_sc.prc_add_pretrain_from_infer_p
(
  i_work_no           bigint
, i_loss_fn_type      varchar(32)
, i_model_code_6      char(6)    default left(md5(random()::text), 6)  -- 如果没有，那么随机生成
) as 
$$
declare
  v_cur_node_nos           bigint[];
begin
  -- model_code_6 优先使用指定 code；


  delete from sm_sc.tb_classify_task where work_no = i_work_no;
  delete from sm_sc.tb_nn_node where work_no = i_work_no;
  delete from sm_sc.tb_nn_path where work_no = i_work_no;
  -- delete from sm_sc.tb_nn_train_input_buff where work_no = i_work_no;
  
  insert into sm_sc.tb_classify_task
  (
    work_no     
  , model_code_6
  , learn_cnt   
  , loss_fn_type
  )
  select 
    i_work_no  
  , i_model_code_6
  , 1   
  , i_loss_fn_type
  ;
  
  insert into sm_sc.tb_nn_node
  (
    work_no           
  , node_no           
  , node_type         
  , node_fn_type      
  , node_fn_asso_value
  , nn_depth_no       
  -- , learn_cnt_fore    
  -- , learn_cnt_back    
  -- , is_fore_node      
  -- , is_back_node  
  -- , dropout_ratio     
  -- , dropout_rescale   
  -- , dropout_depdt_val 
  -- , is_dropout        
  -- , node_depdt_len    
  -- , pick_depdt_idx    
                      
  -- , node_depdt_vals   
  , p_node_depdt
  -- , node_dloss_ddepdt 
  -- , cost_delta_l1     
  -- , cost_delta_l2     
  -- , node_grp_no       
  -- , node_grp_ord_no   
  -- , node_desc   
  )
  select 
    work_no           
  , node_no           
  , node_type         
  , node_fn_type      
  , node_fn_asso_value
  , nn_depth_no       
  -- , node_depdt_vals  
  , case 
      when node_depdt_vals is not null 
        then
        sm_sc.__fv_set_kv
        (
          node_depdt_vals
        , i_model_code_6 ||
          '_' || i_work_no :: varchar   ||
          '_' || node_no :: varchar ||
          '__d__'  -- 'depdt'
        )
      else null
    end
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  
  insert into sm_sc.tb_nn_path
  (
    work_no
  , fore_node_no
  , path_ord_no 
  , back_node_no
  )
  select 
    work_no
  , fore_node_no
  , path_ord_no 
  , back_node_no
  from sm_sc.__vt_nn_path
  where work_no = i_work_no
  ;
  
  commit;
  
    update sm_sc.tb_nn_node tb_a_tar
    set 
      -- -- node_depdt_len   = 
      -- --   array 
      -- --   [
      -- --     case tb_a_tar.node_fn_type 
      -- --       when '00_full_dataset' 
      -- --         then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
      -- --       when '00_buff_slice_rand_pick' 
      -- --         then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
      -- --     end,
      -- --     v_input_len_x
      -- --   ]
      -- -- ,                        -- -- array[array_length(tb_a_sour.i_indepdt_01, 1), array_length(tb_a_sour.i_indepdt_01, 2)],
      is_fore_node = 
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then false 
          when '00_buff_slice_rand_pick' 
            then true 
        end
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type = 'input_01'
    ;
    commit;
    
    update sm_sc.tb_nn_node tb_a_tar
    set 
      is_fore_node = (select is_fore_node from sm_sc.tb_nn_node where work_no = i_work_no and tb_a_tar.node_type = 'input_01')
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
    ;
    commit;
    
    -- 00_const 节点的准备工作
    update sm_sc.tb_nn_node
    set 
      is_fore_node = false
    , is_back_node = false
    where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
    ;
    commit;
    
-- -- -- debug
-- raise notice 'L226. begin: node_depdt_len of input: %;', (select node_depdt_len from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01');
  
    -- 标记前向传播节点
    -- 缺省 true. 从神经网络的初始起点（包括）开始审计，至 01_prod_mx, 05_conv_2d_grp_x, rand 类算子为止（不包括），沿途皆为 false    
    -- 更新 is_fore_node
    v_cur_node_nos := 
      (
        select array_agg(node_no) from sm_sc.tb_nn_node 
        where work_no = i_work_no 
          and 
          (
            node_type in ('input_01', 'input_02', 'input_03', 'input_04')
            or node_fn_type = '00_const' and node_type <> 'weight'
          )
      );     -- -- v_input_nodes
    while array_length(v_cur_node_nos, 1) > 0
    loop 
      -- pg 暂不支持 recursive cte 使用 aggr, 先只能用循环
      with
      cte_fore_nodes as 
      (
        select 
          tb_a_fore.node_no,
          max(tb_a_fore.node_fn_type) as node_fn_type
        from sm_sc.tb_nn_path tb_a_fore_path
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
            and tb_a_fore.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_path tb_a_back_path
          on tb_a_back_path.fore_node_no = tb_a_fore.node_no
            and tb_a_back_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.node_no = tb_a_back_path.back_node_no
            and tb_a_back.work_no = i_work_no   -- 2021082501
        where tb_a_fore_path.back_node_no = any(v_cur_node_nos)
          and tb_a_fore.node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
          and tb_a_fore_path.work_no = i_work_no   -- 2021082501
        group by tb_a_fore.node_no
        having count(distinct tb_a_back.node_no) =
                 count(distinct tb_a_back.node_no) 
                   filter 
                   (
                     where tb_a_back.is_fore_node is false
                   )
      ),
      cte_upd as
      (
        update sm_sc.tb_nn_node tb_a_tar
        set 
          is_fore_node = false
        from cte_fore_nodes tb_a_sour
        where tb_a_sour.node_no = tb_a_tar.node_no
          and tb_a_tar.work_no = i_work_no   -- 2021082501
      )
      select 
        array_agg(distinct node_no) into v_cur_node_nos 
      from cte_fore_nodes
      where node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
      ;
      commit;
  
-- -- -- debug
-- raise notice 'L345.: v_cur_node_nos: %;', v_cur_node_nos;
  
    end loop;
    
    -- 标记反向传播节点
    -- 缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true

    update sm_sc.tb_nn_node
    set 
      is_back_node = true
    where work_no = i_work_no   -- 2021082501
      -- and node_no = any(v_init_nodes)
      and node_type = 'weight'
    ;
    commit;

    with recursive
    cte_back_nodes as
    (
      select 
        tb_a_main.node_no
      from sm_sc.tb_nn_node tb_a_main
      where tb_a_main.work_no = i_work_no   -- 2021082501
        and tb_a_main.is_back_node is true
      union
      select
        tb_a_path.fore_node_no
      from cte_back_nodes tb_a_back
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.back_node_no = tb_a_back.node_no 
      where tb_a_path.work_no = i_work_no   -- 2021082501
    )
    update sm_sc.tb_nn_node tb_a_tar
    set
      is_back_node = true
    from cte_back_nodes tb_a_sour
    where tb_a_sour.node_no = tb_a_tar.node_no
      and tb_a_tar.work_no = i_work_no   -- 2021082501
      -- and tb_a_tar.node_type <> 'output_01'
    ;
    commit;   

end
$$
language plpgsql;

-- call 
-- sm_sc.prc_add_pretrain_from_infer_p
-- (
--   -000000002
-- , '201'
-- );
drop procedure if exists sm_sc.prc_del_infer_model(bigint);
create or replace procedure sm_sc.prc_del_infer_model
(
  i_work_no           bigint
) as 
$$
declare
begin
  delete from sm_sc.__vt_nn_node where work_no = i_work_no;
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  delete from sm_sc.__vt_tmp_nn_node where work_no = i_work_no;
  commit;
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_del_infer_model
-- (
--   -000000002
-- );
drop procedure if exists sm_sc.prc_del_infer_model_p(bigint);
create or replace procedure sm_sc.prc_del_infer_model_p
(
  i_work_no           bigint
) as 
$$
declare
begin
  delete from sm_sc.__vt_tmp_nn_node 
  where work_no = i_work_no
  ;
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_node_depdt)) 
  from sm_sc.__vt_nn_node 
  where work_no = i_work_no
  ;
  delete from sm_sc.__vt_nn_node 
  where work_no = i_work_no
  ; 
    
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_node_depdt)) 
  from sm_sc.__vt_tmp_nn_node 
  where work_no = i_work_no
  ;
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_del_infer_model_p
-- (
--   -000000002
-- );
drop procedure if exists sm_sc.prc_del_pretrain_model(bigint);
create or replace procedure sm_sc.prc_del_pretrain_model
(
  i_work_no           bigint
) as 
$$
declare
begin
  delete from sm_sc.tb_classify_task where work_no = i_work_no;
  delete from sm_sc.tb_nn_node where work_no = i_work_no;
  delete from sm_sc.tb_nn_path where work_no = i_work_no;
  delete from sm_sc.tb_nn_train_input_buff where work_no = i_work_no;
  commit;
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_del_pretrain_model
-- (
--   -000000002
-- );
drop procedure if exists sm_sc.prc_del_pretrain_model_p(bigint);
create or replace procedure sm_sc.prc_del_pretrain_model_p
(
  i_work_no           bigint
) as 
$$
declare
begin
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_node_depdt)) 
  , sm_sc.__fv_delete_kv(array_agg(p_node_dloss_ddepdt)) 
  from sm_sc.tb_nn_node 
  where work_no = i_work_no
  ;
  delete from sm_sc.tb_nn_node where work_no = i_work_no;
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_ddepdt_dindepdt)) 
  from sm_sc.tb_nn_path 
  where work_no = i_work_no
  ;
  delete from sm_sc.tb_nn_path where work_no = i_work_no;
  
  delete from sm_sc.tb_nn_train_input_buff where work_no = i_work_no;
  delete from sm_sc.tb_classify_task where work_no = i_work_no;
  commit;
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_del_pretrain_model_p
-- (
--   -000000002
-- );
drop procedure if exists sm_sc.prc_export_infer_model(bigint, text, text);
create or replace procedure sm_sc.prc_export_infer_model
(
  i_work_no           bigint
-- , i_model_folder_path text        -- etc. /var/lib/pgsql/
, i_model_node_file   text                  
, i_model_path_file   text                  
) as 
$$
declare
begin  
  if exists (select  from sm_sc.__vt_nn_node where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no           
            , node_no           
            , node_type         
            , node_fn_type      
            , node_fn_asso_value
            , nn_depth_no       
            , node_depdt_vals  
            from sm_sc.__vt_nn_node
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_node_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.__vt_nn_node.';
  end if;
  
  if exists (select  from sm_sc.__vt_nn_path where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no
            , fore_node_no
            , path_ord_no
            , back_node_no
            from sm_sc.__vt_nn_path 
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_path_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.__vt_nn_path.';
  end if;  
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_export_infer_model
-- (
--   -2
-- , '/var/lib/pgsql/sm_sc.tb_nn_node_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_path_000000002.gz'  
-- )
drop procedure if exists sm_sc.prc_export_infer_model_p(bigint, text, text);
create or replace procedure sm_sc.prc_export_infer_model_p
(
  i_work_no           bigint
-- , i_model_folder_path text        -- etc. /var/lib/pgsql/
, i_model_node_file   text                  
, i_model_path_file   text                  
) as 
$$
declare
begin  
  if exists (select  from sm_sc.__vt_nn_node where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no           
            , node_no           
            , node_type         
            , node_fn_type      
            , node_fn_asso_value
            , nn_depth_no       
            , node_depdt_vals  
            , p_node_depdt
            from sm_sc.__vt_nn_node
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_node_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.__vt_nn_node.';
  end if;
  
  if exists (select  from sm_sc.__vt_nn_path where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no
            , fore_node_no
            , path_ord_no
            , back_node_no
            from sm_sc.__vt_nn_path 
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_path_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.__vt_nn_path.';
  end if;  
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_export_infer_model_p
-- (
--   -2
-- , '/var/lib/pgsql/sm_sc.tb_nn_node_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_path_000000002.gz'  
-- )
drop procedure if exists sm_sc.prc_export_pretrain_model(bigint, text, text, text, text);
create or replace procedure sm_sc.prc_export_pretrain_model
(
  i_work_no           bigint
-- , i_model_folder_path text
-- , i_if_train_dateset  boolean
, i_model_work_file   text                  
, i_model_node_file   text                  
, i_model_path_file   text                  
, i_train_data_file   text default null     
) as 
$$
declare
begin
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no
            , learn_cnt
            , loss_fn_type
            , loss_delta
            , null as ret_w
            from sm_sc.tb_classify_task 
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_work_file || ''' 
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.tb_classify_task.';
  end if;
  
  if exists (select  from sm_sc.tb_nn_node where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no           
            , node_no           
            , node_type         
            , node_fn_type      
            , node_fn_asso_value
            , nn_depth_no       
            , learn_cnt_fore    
            , learn_cnt_back    
            , is_fore_node      
            , is_back_node    
            , dropout_ratio     
            , dropout_rescale   
            , dropout_depdt_val 
            , is_dropout        
            , node_depdt_len    
            , null as pick_depdt_idx  
            , case when node_type = ''weight'' or node_fn_type = ''00_const'' then node_depdt_vals end as node_depdt_vals 
            , null as node_dloss_ddepdt 
            , null as cost_delta_l1     
            , null as cost_delta_l2     
            , node_grp_no       
            , node_grp_ord_no   
            , node_desc         
            from sm_sc.tb_nn_node
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_node_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.tb_nn_node.';
  end if;
  
  if exists (select  from sm_sc.tb_nn_path where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no
            , fore_node_no
            , path_ord_no
            , back_node_no
            , null as ddepdt_dindepdt
            from sm_sc.tb_nn_path 
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_path_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.tb_nn_path.';
  end if;
  if i_train_data_file is not null
  then 
    if exists (select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no)
    then 
      execute 
        '
          copy               
            (
              select 
                work_no      
              , ord_no       
              , i_depdt_01   
              , i_depdt_02   
              , i_depdt_03   
              , i_depdt_04   
              , i_indepdt_01 
              , i_indepdt_02 
              , i_indepdt_03 
              , i_indepdt_04 
              , i_dataset_dtl
              from sm_sc.tb_nn_train_input_buff 
              where work_no = ' || i_work_no || '
            )
          to program         ''gzip > ' || i_train_data_file || '''
          delimiter          ''|'' 
          encoding           ''UTF8'' 
          csv
          header
        '
      ;
    else 
      raise exception 'not exists i_work_no from sm_sc.tb_nn_train_input_buff.';
    end if;
  end if;
  
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_export_pretrain_model
-- (
--   -2
-- , '/var/lib/pgsql/sm_sc.tb_classify_task_000000002.gz'
-- , '/var/lib/pgsql/sm_sc.tb_nn_node_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_path_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_train_input_buff_000000002.gz'  
-- )
drop procedure if exists sm_sc.prc_export_pretrain_model_p(bigint, text, text, text, text, text);
create or replace procedure sm_sc.prc_export_pretrain_model_p
(
  i_work_no             bigint
-- , i_model_folder_path   text
-- , i_if_train_dateset    boolean
, i_model_work_file     text                  
, i_model_node_file     text                  
, i_model_path_file     text    
, i_model_array_kv_file text                  
, i_train_data_file     text default null     
) as 
$$
declare
begin
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no
            , model_code_6
            , learn_cnt
            , loss_fn_type
            , loss_delta
            , null as ret_w
            from sm_sc.tb_classify_task 
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_work_file || ''' 
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.tb_classify_task.';
  end if;
  
  if exists (select  from sm_sc.tb_nn_node where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no           
            , node_no           
            , node_type         
            , node_fn_type      
            , node_fn_asso_value
            , nn_depth_no       
            , learn_cnt_fore    
            , learn_cnt_back    
            , is_fore_node      
            , is_back_node    
            , dropout_ratio     
            , dropout_rescale   
            , dropout_depdt_val 
            , is_dropout        
            , node_depdt_len    
            , null as pick_depdt_idx  
            , case when node_type = ''weight'' or node_fn_type = ''00_const'' then p_node_depdt end as p_node_depdt 
            , null as p_node_dloss_ddepdt 
            , null as cost_delta_l1     
            , null as cost_delta_l2     
            , node_grp_no       
            , node_grp_ord_no   
            , node_desc         
            from sm_sc.tb_nn_node
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_node_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
    
    execute 
      '
        copy
          (
            select 
              p_node_depdt
            , sm_sc.__fv_get_kv(p_node_depdt) as node_depdt_vals
            from sm_sc.tb_nn_node
            where work_no = ' || i_work_no || '
              and p_node_depdt is not null
              and node_type = ''weight'' or node_fn_type = ''00_const''
          )
        to program         ''gzip > ' || i_model_array_kv_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.tb_nn_node.';
  end if;
  
  if exists (select  from sm_sc.tb_nn_path where work_no = i_work_no)
  then 
    execute 
      '
        copy               
          (
            select 
              work_no
            , fore_node_no
            , path_ord_no
            , back_node_no
            , null as p_ddepdt_dindepdt
            from sm_sc.tb_nn_path 
            where work_no = ' || i_work_no || '
          )
        to program         ''gzip > ' || i_model_path_file || '''
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  else 
    raise exception 'not exists i_work_no from sm_sc.tb_nn_path.';
  end if;
  if i_train_data_file is not null
  then 
    if exists (select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no)
    then 
      execute 
        '
          copy               
            (
              select 
                work_no      
              , ord_no       
              , i_depdt_01   
              , i_depdt_02   
              , i_depdt_03   
              , i_depdt_04   
              , i_indepdt_01 
              , i_indepdt_02 
              , i_indepdt_03 
              , i_indepdt_04 
              , i_dataset_dtl
              from sm_sc.tb_nn_train_input_buff 
              where work_no = ' || i_work_no || '
            )
          to program         ''gzip > ' || i_train_data_file || '''
          delimiter          ''|'' 
          encoding           ''UTF8'' 
          csv
          header
        '
      ;
    else 
      raise exception 'not exists i_work_no from sm_sc.tb_nn_train_input_buff.';
    end if;
  end if;
  
end
$$
language plpgsql;

-- call 
-- sm_sc.prc_export_pretrain_model_p
-- (
--   -2
-- , '/var/lib/pgsql/sm_sc.tb_classify_task_000000002.gz'
-- , '/var/lib/pgsql/sm_sc.tb_nn_node_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_path_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_train_input_buff_000000002.gz'  
-- )
drop procedure if exists sm_sc.prc_import_infer_model(bigint, text, text);
create or replace procedure sm_sc.prc_import_infer_model
(
  i_work_no           bigint
, i_model_node_file   text
, i_model_path_file   text
) as 
$$
declare
begin
  delete from sm_sc.__vt_nn_node where work_no = i_work_no;
  commit;
  execute 
    '
      copy               sm_sc.__vt_nn_node 
      from program       ''gzip -d < ' || i_model_node_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  commit;
  execute 
    '
      copy               sm_sc.__vt_nn_path 
      from program       ''gzip -d < ' || i_model_path_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  insert into sm_sc.__vt_nn_sess
  (
    work_no
  , sess_id
  , sess_status
  )
  select 
    i_work_no
  , 0
  , '1'              -- 参看字典表  nn_sess_status。0: 空闲; 1 占用;
  on conflict (work_no, sess_id) do nothing
  ;
  
  delete from sm_sc.__vt_tmp_nn_node where work_no = i_work_no and sess_id = 0;
  -- 缺省添加 sess_id = 0 ，用户演示验证
  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id                        ,
    work_no                        ,
    node_no                        ,
    node_type                      ,
    node_fn_type                   ,
    node_fn_asso_value        ,
    nn_depth_no                    ,
    node_depdt_vals             
  )
  select 
    0                               ,
    work_no                         ,
    node_no                         ,
    node_type                       ,
    coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  ,
    case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         ,
    nn_depth_no                     ,
    case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then node_depdt_vals
      else null
    end             
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
end
$$
language plpgsql;

-- call
-- sm_sc.prc_import_infer_model
-- (
--   -000000002
-- , '/var/lib/pgsql/sm_sc.__vt_nn_node_000000002.gz'
-- , '/var/lib/pgsql/sm_sc.__vt_nn_path_000000002.gz'

-- )
drop procedure if exists sm_sc.prc_import_infer_model_p(bigint, text, text);
create or replace procedure sm_sc.prc_import_infer_model_p
(
  i_work_no           bigint
, i_model_node_file   text
, i_model_path_file   text
) as 
$$
declare
begin
  delete from sm_sc.__vt_nn_node where work_no = i_work_no;
  commit;
  execute 
    '
      copy               
        sm_sc.__vt_nn_node 
        (
          work_no           
        , node_no           
        , node_type         
        , node_fn_type      
        , node_fn_asso_value
        , nn_depth_no       
        , node_depdt_vals   
        , p_node_depdt      
        )
      from program       ''gzip -d < ' || i_model_node_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  commit;
  execute 
    '
      copy               
        sm_sc.__vt_nn_path 
        (
          work_no     
        , fore_node_no
        , path_ord_no 
        , back_node_no
        )
      from program       ''gzip -d < ' || i_model_path_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_node_depdt)) 
  from sm_sc.__vt_nn_node 
  where work_no = i_work_no 
    and p_node_depdt is not null
  ;
  
  perform 
    sm_sc.__fv_set_kv(node_depdt_vals, p_node_depdt) 
  from sm_sc.__vt_nn_node 
  where work_no = i_work_no 
    and p_node_depdt is not null
  ;
  
  insert into sm_sc.__vt_nn_sess
  (
    work_no
  , sess_id
  , sess_status
  )
  select 
    i_work_no
  , 0
  , '1'              -- 参看字典表  nn_sess_status。0: 空闲; 1 占用;
  on conflict (work_no, sess_id) do nothing
  ;
  
  delete from sm_sc.__vt_tmp_nn_node where work_no = i_work_no and sess_id = 0;
  -- 缺省添加 sess_id = 0 ，用户演示验证
  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id                        ,
    work_no                        ,
    node_no                        ,
    node_type                      ,
    node_fn_type                   ,
    node_fn_asso_value        ,
    nn_depth_no                    ,
    p_node_depdt             
  )
  select 
    0                               ,
    work_no                         ,
    node_no                         ,
    node_type                       ,
    coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  ,
    case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         ,
    nn_depth_no                     ,
    case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then p_node_depdt
      else null
    end             
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
end
$$
language plpgsql;

-- call
-- sm_sc.prc_import_infer_model_p
-- (
--   -000000002
-- , '/var/lib/pgsql/sm_sc.__vt_nn_node_000000002.gz'
-- , '/var/lib/pgsql/sm_sc.__vt_nn_path_000000002.gz'

-- )
drop procedure if exists sm_sc.prc_import_pretrain_model(bigint, text, text, text, text);
create or replace procedure sm_sc.prc_import_pretrain_model
(
  i_work_no           bigint  ,
  i_model_work_file   text                  
, i_model_node_file   text                  
, i_model_path_file   text                  
, i_train_data_file   text default null     
) as 
$$
declare
begin
  delete from sm_sc.tb_classify_task where work_no = i_work_no;
  commit;
  execute 
    '
      copy               sm_sc.tb_classify_task 
      from program       ''gzip -d < ' || i_model_work_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  delete from sm_sc.tb_nn_node where work_no = i_work_no;
  commit;
  execute 
    '
      copy               sm_sc.tb_nn_node 
      from program       ''gzip -d < ' || i_model_node_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  delete from sm_sc.tb_nn_path where work_no = i_work_no;
  commit;
  execute 
    '
      copy               sm_sc.tb_nn_path 
      from program       ''gzip -d < ' || i_model_path_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  if i_train_data_file is not null 
  then 
    delete from sm_sc.tb_nn_train_input_buff where work_no = i_work_no;
    commit;
    execute 
      '
        copy               sm_sc.tb_nn_train_input_buff 
        from program       ''gzip -d < ' || i_train_data_file || ''' 
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  end if;
end
$$
language plpgsql;

-- call
-- sm_sc.prc_import_pretrain_model
-- (
--   -000000002
-- , '/var/lib/pgsql/sm_sc.tb_classify_task_000000002.gz'
-- , '/var/lib/pgsql/sm_sc.tb_nn_node_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_path_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_train_input_buff_000000002.gz'  
-- )
drop procedure if exists sm_sc.prc_import_pretrain_model_p(bigint, text, text, text, text, text);
create or replace procedure sm_sc.prc_import_pretrain_model_p
(
  i_work_no             bigint
, i_model_work_file     text                  
, i_model_node_file     text                  
, i_model_path_file     text   
, i_model_array_kv_file text               
, i_train_data_file     text default null     
) as 
$$
declare
begin
  delete from sm_sc.tb_classify_task where work_no = i_work_no;
  commit;
  execute 
    '
      copy               
        sm_sc.tb_classify_task 
        (
          work_no     
        , model_code_6
        , learn_cnt   
        , loss_fn_type
        , loss_delta 
        , ret_w
        )
      from program       ''gzip -d < ' || i_model_work_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  delete from sm_sc.tb_nn_node where work_no = i_work_no;
  commit;
  execute 
    '
      copy               
        sm_sc.tb_nn_node 
        (
          work_no             
        , node_no             
        , node_type           
        , node_fn_type        
        , node_fn_asso_value  
        , nn_depth_no         
        , learn_cnt_fore      
        , learn_cnt_back      
        , is_fore_node        
        , is_back_node      
        , dropout_ratio       
        , dropout_rescale     
        , dropout_depdt_val   
        , is_dropout          
        , node_depdt_len      
        , pick_depdt_idx    
        , p_node_depdt        
        , p_node_dloss_ddepdt 
        , cost_delta_l1       
        , cost_delta_l2       
        , node_grp_no         
        , node_grp_ord_no     
        , node_desc     
        )
      from program       ''gzip -d < ' || i_model_node_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  perform 
    sm_sc.__fv_delete_kv(array_agg(p_node_depdt)) 
  from sm_sc.tb_nn_node 
  where work_no = i_work_no 
    and p_node_depdt is not null
  ;
  
  delete from sm_sc.__vt_tmp_array_kv tb_a_kv
  using sm_sc.tb_nn_node tb_a_node
  where tb_a_node.p_node_depdt = tb_a_kv.arr_key
    and tb_a_node.work_no = i_work_no
    -- and tb_a_node.p_node_depdt is not null
  ;
  commit;
  
  execute 
    '
      copy               
        sm_sc.__vt_tmp_array_kv 
        (
          arr_key
        , arr_val
        )
      from program       ''gzip -d < ' || i_model_array_kv_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  perform
    sm_sc.__fv_set_kv(tb_a_kv.arr_val, tb_a_node.p_node_depdt) 
  from sm_sc.tb_nn_node tb_a_node
  inner join sm_sc.__vt_tmp_array_kv tb_a_kv
    on tb_a_node.p_node_depdt = tb_a_kv.arr_key
  where tb_a_node.work_no = i_work_no
    -- and tb_a_node.p_node_depdt is not null
  ;  
  commit;
  
  delete from sm_sc.__vt_tmp_array_kv tb_a_kv
  using sm_sc.tb_nn_node tb_a_node
  where tb_a_node.p_node_depdt = tb_a_kv.arr_key
    and tb_a_node.work_no = i_work_no
    -- and tb_a_node.p_node_depdt is not null
  ;
  commit;
  
  delete from sm_sc.tb_nn_path where work_no = i_work_no;
  commit;
  execute 
    '
      copy               
        sm_sc.tb_nn_path 
        (
          work_no     
        , fore_node_no
        , path_ord_no 
        , back_node_no 
        , p_ddepdt_dindepdt
        )
      from program       ''gzip -d < ' || i_model_path_file || ''' 
      delimiter          ''|'' 
      encoding           ''UTF8'' 
      csv
      header
    '
  ;
  
  if i_train_data_file is not null 
  then 
    delete from sm_sc.tb_nn_train_input_buff where work_no = i_work_no;
    commit;
    execute 
      '
        copy               
          sm_sc.tb_nn_train_input_buff 
          (
            work_no      
          , ord_no       
          , i_depdt_01   
          , i_depdt_02   
          , i_depdt_03   
          , i_depdt_04   
          , i_indepdt_01 
          , i_indepdt_02 
          , i_indepdt_03 
          , i_indepdt_04 
          , i_dataset_dtl
          )
        from program       ''gzip -d < ' || i_train_data_file || ''' 
        delimiter          ''|'' 
        encoding           ''UTF8'' 
        csv
        header
      '
    ;
  end if;
end
$$
language plpgsql;

-- call
-- sm_sc.prc_import_pretrain_model_p
-- (
--   -000000002
-- , '/var/lib/pgsql/sm_sc.tb_classify_task_000000002.gz'
-- , '/var/lib/pgsql/sm_sc.tb_nn_node_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_path_000000002.gz'  
-- , '/var/lib/pgsql/sm_sc.tb_nn_train_input_buff_000000002.gz'  
-- )
drop procedure if exists sm_sc.prc_nn_add_sess;
create or replace procedure sm_sc.prc_nn_add_sess
(
  i_work_no                          bigint            ,                                                                                                -- 训练任务编号
  i_sess_cnt                         int               default    1                                                                                                      -- 最大训练次数，然后未完成中止
)
as
$$
declare -- here
  v_sess_range        int8range       := sm_sc.fv_get_global_seq(i_sess_cnt)    ;
  

begin
  set search_path to public;
  
  -- -- -- 规约: 0 号 session 为每轮训练后，默认部署。
  -- -- if 0 :: bigint <@ i_sess_multirange
  -- -- then 
  -- --   raise exception 'can''t deploy 0 as sess id.  0 is default sess id for train task at pg4ml.  ';
  -- -- end if;
  
  insert into sm_sc.__vt_nn_sess
  (
    work_no
  , sess_id
  , sess_status
  )
  select 
    i_work_no
  , a_sess_no
  , '0'              -- 参看字典表  nn_sess_status。0: 空闲; 1 占用;
  from generate_series(lower(v_sess_range), upper(v_sess_range) - 1) tb_a(a_sess_no)
  ;
  
  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id                        ,
    work_no                        ,
    node_no                        ,
    node_type                      ,
    node_fn_type                   ,
    node_fn_asso_value        ,
    nn_depth_no                    ,
    node_depdt_vals             
  )
  select 
    a_sess_no                    ,
    work_no                         ,
    node_no                         ,
    node_type                       ,
    coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  ,
    case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         ,
    nn_depth_no                     ,
    case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then node_depdt_vals
      else null
    end             
  from sm_sc.__vt_nn_node
    , generate_series(lower(v_sess_range), upper(v_sess_range) - 1) tb_a(a_sess_no)
  where work_no = i_work_no
  ;
    
end
$$
language plpgsql;

-- call sm_sc.prc_nn_add_sess
-- (
--   -76543
-- , 3
-- )
drop procedure if exists sm_sc.prc_nn_del_sess;
create or replace procedure sm_sc.prc_nn_del_sess
(
  i_work_no                          bigint            ,        -- 训练任务编号
  i_sess_multirange                  int8multirange
)
as
$$
-- declare -- here
  
begin
  set search_path to public;
  
  delete from sm_sc.__vt_nn_sess
  where work_no = i_work_no
    and sess_id <@ i_sess_multirange
  ;
  
  delete from sm_sc.__vt_tmp_nn_node
  where work_no = i_work_no
    and sess_id <@ i_sess_multirange
  ;
    
end
$$
language plpgsql;


drop procedure if exists sm_sc.prc_nn_prepare;
create or replace procedure sm_sc.prc_nn_prepare
(
  i_work_no                          bigint                                   ,                                                                                                -- 训练任务编号
  i_limit_train_times                int               default    5                                                                                                        -- 最大训练次数，然后未完成中止
)
as
$$
declare -- here
  v_cur_learn_cnt          bigint               := (select learn_cnt from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号
  v_input_asso_value       int[]                := (select node_fn_asso_value :: int[] from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' limit 1);
  v_cur_node_nos           bigint[];
  v_cur_node_nos_last      bigint[];
  v_input_nodes            bigint[]             := (select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and (node_type in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_fn_type = '00_const') limit 1);
  v_cur_nn_depth           int                  := 0;   -- 深度层数游标
  v_max_nn_depth           int;

begin
  set search_path to public;

  -- ------------------------------- 审计部分 -----------------------------------------------
  -- 审计 训练任务是否存在
  if not exists(select  from sm_sc.tb_classify_task where work_no = i_work_no)
  then
    raise exception 'no such work NO. = %!', i_work_no;

  -- 审计 输入节点是否在有效范围内
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') group by node_type having count(*) > 1)
  then
    raise exception 'more than 1 input node, or expect one-hot i_indepdt_*!';
    
  -- -- -- 审计 测试与验证的输入节点是否只有一个
  -- -- elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'prod_input' having count(*) > 1)
  -- -- then
  -- --   raise exception 'more than 1 prod_input node, or expect one-hot i_indepdt_*!';
  -- -- 
  -- 审计 输出节点是否在有效范围内
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('output_01', 'output_02', 'output_03', 'output_04') group by node_type having count(*) > 1)
  then
    raise exception 'more than 1 output node, or expect one-hot i_depdt_*!';
    
  -- 审计 训练集同一输入节点每条数据的规格同一
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      group by work_no 
      having (count(distinct array_dims(i_indepdt_01)) > 1
              or count(distinct array_dims(i_indepdt_02)) > 1
              or count(distinct array_dims(i_indepdt_03)) > 1
              or count(distinct array_dims(i_indepdt_04)) > 1
             )
    )
  then
    raise exception 'there should not be different array_dims(i_indepdt_*).';
    
  -- 审计 训练集同一输出节点每条数据的规格同一
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      group by work_no 
      having count(distinct array_dims(i_depdt_01)) > 1
        or count(distinct array_dims(i_depdt_02)) > 1
        or count(distinct array_dims(i_depdt_03)) > 1
        or count(distinct array_dims(i_depdt_04)) > 1
    )
  then
    raise exception 'there should not be different arrray_length(i_depdt_*, 2).';
    
  -- 审计 训练集不存在 null 之元素
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_01)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_02)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_03)
      and sm_sc.fv_aggr_slice_is_exists_null(i_indepdt_04)
    )
  then 
    raise exception 'there are some null element in input i_indepdt_*.';
    
  -- 审计 训练集判断结果不存在 null 之元素
  elsif exists 
    (
      select  from sm_sc.tb_nn_train_input_buff 
      where work_no = i_work_no 
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_01)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_02)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_03)
        and sm_sc.fv_aggr_slice_is_exists_null(i_depdt_04)
    )
  then 
    raise warning 'there are some null element in input i_depdt_*.';

  -- 审计 小批量随机的训练集记录数目是否符合要求
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' and node_fn_type = '00_buff_slice_rand_pick')
    -- and v_input_cnt < sm_sc.fv_aggr_slice_max(v_input_asso_value[2 : 2][ : ])
    and exists (select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no group by work_no having count(*) < sm_sc.fv_aggr_slice_max(v_input_asso_value[2 : 2][ : ]))
  then 
    raise exception 'count of training data should not be less than slices'' uppers of input node.';
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' and node_fn_type = '00_buff_slice_rand_pick')
    and true = any(v_input_asso_value[3 : 3][ : ] >` (v_input_asso_value[2 : 2][ : ] -` v_input_asso_value[1 : 1][ : ] +` 1))
  then 
    raise exception 'there should be not any 00_buff_slice_rand_pick more than its slice range.';
    
  -- 00_const 节点的算子超参数审计工作
  elsif exists (select  from sm_sc.tb_nn_node where work_no = i_work_no and node_fn_type = '00_const' and node_fn_asso_value is null)
  then 
    raise exception 'there should be not any null node_fn_asso_value at 00_const.';
    
  -- 非 weight 的 00_const 节点的 node_depdt_vals 初始化审计工作
  elsif exists 
    (
      select  
      from sm_sc.tb_nn_node 
      where work_no = i_work_no 
        and node_fn_type = '00_const' 
        and node_type is distinct from 'weight'
        and 
        (
          node_depdt_vals is null
          -- -- -- 一下注释代码的数组规格审查，阻碍了点算的广播特性
          -- -- or array_length(node_depdt_vals, 1) <> (node_fn_asso_value[1] :: int) 
          -- -- or array_length(node_depdt_vals, 2) <> (node_fn_asso_value[2] :: int)
          -- -- or array_length(node_depdt_vals, 3) <> (node_fn_asso_value[3] :: int)
          -- -- or array_length(node_depdt_vals, 4) <> (node_fn_asso_value[4] :: int)
        )
    )
  then 
    raise exception 'there should be not any null node_depdt_vals or unmatch length configured by node_fn_asso_value at 00_const.';

  -- 审计 算子入参数量
  --     审计双目
  elsif exists 
  (
    select 
    from sm_sc.tb_dic_enum tb_a_enum
    inner join sm_sc.tb_nn_node tb_a_node 
      on tb_a_node.node_fn_type = tb_a_enum.enum_key
        and tb_a_node.work_no = i_work_no
    inner join sm_sc.tb_nn_path tb_a_path 
      on tb_a_path.fore_node_no = tb_a_node.node_no
        and tb_a_path.work_no = i_work_no
    where tb_a_enum.enum_name = 'node_fn_type'
      and tb_a_enum.enum_group = '2_p'
    group by tb_a_path.fore_node_no
    having count(tb_a_path.path_ord_no) <> 2
  )
  then
    raise exception 'too many or too few pathes point to an fn with 2_p.';
  
  --     审计单目
  elsif exists
  (
    select 
    from sm_sc.tb_dic_enum tb_a_enum
    inner join sm_sc.tb_nn_node tb_a_node 
      on tb_a_node.node_fn_type = tb_a_enum.enum_key
        and tb_a_node.work_no = i_work_no
    inner join sm_sc.tb_nn_path tb_a_path 
      on tb_a_path.fore_node_no = tb_a_node.node_no
        and tb_a_path.work_no = i_work_no
    where tb_a_enum.enum_name = 'node_fn_type'
      and tb_a_enum.enum_group = '1_p'
    group by tb_a_path.fore_node_no
    having count(tb_a_path.path_ord_no) > 1
  )
  then 
    raise exception 'too many pathes point to an fn with 1_p.';

  -- 审计 是否存在孤岛网络
  elsif exists 
  (
    with recursive
    cte_main_nodes as
    (
      select 
        node_no
      from sm_sc.tb_nn_node
      where work_no = i_work_no
        and node_type in ('output_01', 'output_02', 'output_03', 'output_04')
      union
      select 
        tb_a_incre.node_no
      from cte_main_nodes tb_a_main
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.fore_node_no = tb_a_main.node_no
      inner join sm_sc.tb_nn_node tb_a_incre
        on tb_a_incre.node_no = tb_a_path.back_node_no
          and tb_a_incre.work_no = i_work_no
    )
    select 
    from cte_main_nodes
    having count(node_no) < (select count(node_no) from sm_sc.tb_nn_node where work_no = i_work_no)
  )
  then
    raise exception 'there should not be alone nodes.';

  -- 审计通过，新的任务的初始化工作(learn_cnt = 0)
  elsif v_cur_learn_cnt = 0
  then
    
-- -- -- debug
-- raise notice 'L298. begin: v_cur_node_nos: %;', v_cur_node_nos;
  
    update sm_sc.tb_nn_node tb_a_tar
    set 
      -- -- node_depdt_len   = 
      -- --   array 
      -- --   [
      -- --     case tb_a_tar.node_fn_type 
      -- --       when '00_full_dataset' 
      -- --         then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
      -- --       when '00_buff_slice_rand_pick' 
      -- --         then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
      -- --     end,
      -- --     v_input_len_x
      -- --   ]
      -- -- ,                        -- -- array[array_length(tb_a_sour.i_indepdt_01, 1), array_length(tb_a_sour.i_indepdt_01, 2)],
      is_fore_node = 
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then false 
          when '00_buff_slice_rand_pick' 
            then true 
        end
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type = 'input_01'
    ;
    commit;
    
    update sm_sc.tb_nn_node tb_a_tar
    set 
      is_fore_node = (select is_fore_node from sm_sc.tb_nn_node where work_no = i_work_no and tb_a_tar.node_type = 'input_01')
    where tb_a_tar.work_no = i_work_no   -- 2021082501
      and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
    ;
    commit;
    
    -- 00_const 节点的准备工作
    update sm_sc.tb_nn_node
    set 
      is_fore_node = false
    , is_back_node = false
    where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
    ;
    commit;
    
-- -- -- debug
-- raise notice 'L226. begin: node_depdt_len of input: %;', (select node_depdt_len from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01');
  
    -- 标记前向传播节点
    -- 缺省 true. 从神经网络的初始起点（包括）开始审计，至 01_prod_mx, 05_conv_2d_grp_x, rand 类算子为止（不包括），沿途皆为 false    
    -- 更新 is_fore_node
    v_cur_node_nos := 
      (
        select array_agg(node_no) from sm_sc.tb_nn_node 
        where work_no = i_work_no 
          and 
          (
            node_type in ('input_01', 'input_02', 'input_03', 'input_04')
            or node_fn_type = '00_const' and node_type <> 'weight'
          )
      );     -- -- v_input_nodes
    while array_length(v_cur_node_nos, 1) > 0
    loop 
      -- pg 暂不支持 recursive cte 使用 aggr, 先只能用循环
      with
      cte_fore_nodes as 
      (
        select 
          tb_a_fore.node_no,
          max(tb_a_fore.node_fn_type) as node_fn_type
        from sm_sc.tb_nn_path tb_a_fore_path
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
            and tb_a_fore.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_path tb_a_back_path
          on tb_a_back_path.fore_node_no = tb_a_fore.node_no
            and tb_a_back_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.node_no = tb_a_back_path.back_node_no
            and tb_a_back.work_no = i_work_no   -- 2021082501
        where tb_a_fore_path.back_node_no = any(v_cur_node_nos)
          and tb_a_fore.node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
          and tb_a_fore_path.work_no = i_work_no   -- 2021082501
        group by tb_a_fore.node_no
        having count(distinct tb_a_back.node_no) =
                 count(distinct tb_a_back.node_no) 
                   filter 
                   (
                     where tb_a_back.is_fore_node is false
                   )
      ),
      cte_upd as
      (
        update sm_sc.tb_nn_node tb_a_tar
        set 
          is_fore_node = false
        from cte_fore_nodes tb_a_sour
        where tb_a_sour.node_no = tb_a_tar.node_no
          and tb_a_tar.work_no = i_work_no   -- 2021082501
      )
      select 
        array_agg(distinct node_no) into v_cur_node_nos 
      from cte_fore_nodes
      where node_fn_type not in ('01_prod_mx', '05_conv_2d_grp_x', '05_conv', '04_rand_pick_y', '04_rand_pick_x', '04_rand_pick_x3', '04_rand_pick_x4', '04_new')
      ;
      commit;
  
-- -- -- debug
-- raise notice 'L345.: v_cur_node_nos: %;', v_cur_node_nos;
  
    end loop;
    
    -- 标记反向传播节点
    -- 缺省 false. 从 weight 算子（包括）开始传染，至神经网络的终点（包括），沿途皆为 true

    update sm_sc.tb_nn_node
    set 
      is_back_node = true
    where work_no = i_work_no   -- 2021082501
      -- and node_no = any(v_init_nodes)
      and node_type = 'weight'
    ;
    commit;

    with recursive
    cte_back_nodes as
    (
      select 
        tb_a_main.node_no
      from sm_sc.tb_nn_node tb_a_main
      where tb_a_main.work_no = i_work_no   -- 2021082501
        and tb_a_main.is_back_node is true
      union
      select
        tb_a_path.fore_node_no
      from cte_back_nodes tb_a_back
      inner join sm_sc.tb_nn_path tb_a_path
        on tb_a_path.back_node_no = tb_a_back.node_no 
      where tb_a_path.work_no = i_work_no   -- 2021082501
    )
    update sm_sc.tb_nn_node tb_a_tar
    set
      is_back_node = true
    from cte_back_nodes tb_a_sour
    where tb_a_sour.node_no = tb_a_tar.node_no
      and tb_a_tar.work_no = i_work_no   -- 2021082501
      -- and tb_a_tar.node_type <> 'output_01'
    ;
    commit;   
  end if;
  
  -- 每次启动训练，都重新计算 node_depdt_len，用于适配动态调整随机小批量的每个类别抽样数量
  update sm_sc.tb_nn_node tb_a_tar
  set 
    node_depdt_len   = 
      array 
      [
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
          when '00_buff_slice_rand_pick' 
            then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
        end
      ]
      || 
      (
        select 
          array_agg(array_length(i_indepdt_01, a_cur) order by a_cur)
        from (select i_indepdt_01 from sm_sc.tb_nn_train_input_buff where work_no = i_work_no limit 1) tb_a_buff(i_indepdt_01)
          , generate_series(1, array_ndims(tb_a_buff.i_indepdt_01)) tb_a_cur(a_cur)
      )
  where tb_a_tar.work_no = i_work_no   -- 2021082501
    and tb_a_tar.node_type = 'input_01'
  ;
  commit;
  
  -- 初始化其他模态输入节点的规格
  update sm_sc.tb_nn_node tb_a_tar
  set 
    node_depdt_len = 
      array 
      [
        case tb_a_tar.node_fn_type 
          when '00_full_dataset' 
            then (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no) 
          when '00_buff_slice_rand_pick' 
            then sm_sc.fv_aggr_slice_sum(node_fn_asso_value[3 : 3][ : ] :: int[]) 
        end
      ]
      || 
      (
        select 
          array_agg(array_length(i_indepdt_ex, a_cur) order by a_cur)
        from 
          (select 
             case tb_a_tar.node_type 
               when 'input_02'
                 then i_indepdt_02
               when 'input_03'
                 then i_indepdt_03
               when 'input_04'
                 then i_indepdt_04
             end                    
             as i_indepdt_ex 
           from sm_sc.tb_nn_train_input_buff where work_no = i_work_no limit 1
          ) tb_a_buff(i_indepdt_ex)
        , generate_series(1, array_ndims(tb_a_buff.i_indepdt_ex)) tb_a_cur(a_cur)
      )
  where tb_a_tar.work_no = i_work_no   -- 2021082501
    and tb_a_tar.node_type in ('input_02', 'input_03', 'input_04')
  ;
  commit;

  -- 对齐训练次数；训练过程中，输入、偏移量系数节点不参与传播
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = case when node_type in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_fn_type = '00_const' then i_limit_train_times else v_cur_learn_cnt end,
    learn_cnt_back = case when node_type in ('input_01', 'input_02', 'input_03', 'input_04') or (node_fn_type = '00_const' and node_type is distinct from 'weight') then i_limit_train_times else v_cur_learn_cnt end
  where work_no = i_work_no   -- 2021082501
  ;
  commit;

-- -- -- debug
-- raise notice 'L102. begin: v_cur_learn_cnt: %', v_cur_learn_cnt;

  -- 传播规划
  -- 先做个 learn_cnt_.. 的负标记
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = - 1,
    learn_cnt_back = - 1
  where work_no = i_work_no
    and ((node_type not in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') or node_type is null) and node_fn_type <> '00_const')
  ;
  commit;
  -- 前向打标
  v_cur_node_nos := v_input_nodes;
  v_cur_nn_depth := 0;

-- -- -- debug
-- raise notice 'L224. begin: v_cur_node_nos: %; v_input_nodes: %; v_cur_nn_depth: %;', v_cur_node_nos, v_input_nodes, v_cur_nn_depth;

  update sm_sc.tb_nn_node 
  set nn_depth_no = 0 
  where work_no = i_work_no 
    and (node_type in ('input_01', 'input_02', 'input_03', 'input_04') or (node_fn_type = '00_const' and node_type is distinct from 'weight'));
  commit;

  while 
    exists 
    (
      select         
      from sm_sc.tb_nn_node tb_a_fore
      where tb_a_fore.work_no = i_work_no    -- 2021082501
        and tb_a_fore.learn_cnt_fore < v_cur_learn_cnt
      limit 1
    )
  loop
    v_cur_nn_depth := v_cur_nn_depth + 1;
    v_cur_node_nos_last := v_cur_node_nos;
    with 
    -- 由 learn_cnt_fore 约束，找到与上次前向传播有关的，可前向计算的节点
    cte_y as
    (
      select 
        tb_a_fore.node_no
      -- , tb_a_fore.node_fn_type
      from sm_sc.tb_nn_path tb_a_fore_path
      inner join sm_sc.tb_nn_node tb_a_fore
        on tb_a_fore.node_no = tb_a_fore_path.fore_node_no
          and tb_a_fore.work_no = i_work_no   -- 2021082501
      inner join sm_sc.tb_nn_path tb_a_back_path
        on tb_a_back_path.fore_node_no = tb_a_fore.node_no
          and tb_a_back_path.work_no = i_work_no   -- 2021082501
      inner join sm_sc.tb_nn_node tb_a_back
        on tb_a_back.node_no = tb_a_back_path.back_node_no
          and tb_a_back.work_no = i_work_no   -- 2021082501
      where tb_a_fore_path.work_no = i_work_no   -- 2021082501
        and tb_a_fore.learn_cnt_fore < v_cur_learn_cnt 
        and tb_a_fore_path.back_node_no = any(v_cur_node_nos || v_input_nodes) 
      group by tb_a_fore.node_no -- , tb_a_fore.node_fn_type
      having count(*) filter (where tb_a_back.learn_cnt_fore > tb_a_fore.learn_cnt_fore) = count(*)
    ),
    cte_upd_fore_label as
    (
      update sm_sc.tb_nn_node tb_a_tar
      set 
        learn_cnt_fore = v_cur_learn_cnt,
        nn_depth_no = v_cur_nn_depth
      from cte_y tb_a_y
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_y.node_no = tb_a_tar.node_no
      returning tb_a_tar.node_no
    )
    select array_agg(node_no) into v_cur_node_nos from cte_upd_fore_label
    ;
    commit;

-- -- debug
-- raise notice 'L256. begin: nn_depth updated nodes = %: %', v_cur_node_nos, v_cur_nn_depth;
    if v_cur_node_nos is not distinct from v_cur_node_nos_last
    then 
      raise exception 'v_cur_node_nos is the same as v_cur_node_nos_last: %.', coalesce(v_cur_node_nos_last :: text, 'null');
    end if;

  end loop;

  v_max_nn_depth := v_cur_nn_depth;

  -- 修改 weight 节点的深度， = min(下层 01_prod_mx 的深度) - 1，保证反向传播及时传递到 weight 节点
  with
  cte_nn_depth as
  (
    select 
      tb_a_back.node_no,
      min(tb_a_fore.nn_depth_no) - 1 as nn_depth_no
    from sm_sc.tb_nn_node tb_a_back
    inner join sm_sc.tb_nn_path tb_a_path
      on tb_a_back.node_no = tb_a_path.back_node_no
        and tb_a_path.work_no = i_work_no
    inner join sm_sc.tb_nn_node tb_a_fore
      on tb_a_path.fore_node_no = tb_a_fore.node_no
        and tb_a_fore.work_no = i_work_no
    where tb_a_back.work_no = i_work_no
      and tb_a_back.node_type = 'weight'
    group by tb_a_back.node_no
  )
  update sm_sc.tb_nn_node tb_a_back
  set nn_depth_no = tb_a.nn_depth_no
  from cte_nn_depth tb_a
  where tb_a_back.work_no = i_work_no
    and tb_a.node_no = tb_a_back.node_no
    -- and tb_a_back.node_type = 'weight'
  ;
  commit;

  -- 传播规划完毕，恢复 learn_cnt 为真实标记
  update sm_sc.tb_nn_node
  set 
    learn_cnt_fore = v_cur_learn_cnt,
    learn_cnt_back = v_cur_learn_cnt
  where work_no = i_work_no   -- 2021082501
    and (node_type not in ('input_01', 'input_02', 'input_03', 'input_04', 'weight') and node_fn_type <> '00_const')
  ;
  commit;

  
  -- ---------- 填充缺省协参部分。遍历 nn_depth，每次遍历，首先更新非 weight 节点的 node_fn_asso_value，再更新 node_depdt_len，结束 nn_depth 遍历，最后更新 weight 的 node_depdt_vals  ----------------------

  -- ------------------------------- 准备数据 -----------------------------------------------
-- -- debug
raise notice 'L045. prepare data begin!!!!!!'; 

  -- ------------------------------- 审计数据部分 -----------------------------------------------
-- -- debug
raise notice 'L504. check data len begin!!!!!!';

  -- 前向通烟更新 node_depdt_len

  update sm_sc.tb_nn_node
  set node_depdt_len = node_fn_asso_value :: int[]
  where work_no = i_work_no   -- 2021082501
    and node_fn_type = '00_const'
  ;
  commit;

    
  v_cur_nn_depth = 1;
  while v_cur_nn_depth <= v_max_nn_depth
  loop

    -- np aggr 情况，化为 1p
    with
    cte_concat_dims_change as 
    (
      select 
        tb_a_fore.node_no
      , (
          array 
          [
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[1], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
              else max(coalesce(tb_a_back_pn.node_depdt_len[1], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_y')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[2], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
              else max(coalesce(tb_a_back_pn.node_depdt_len[2], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[3], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
              else max(coalesce(tb_a_back_pn.node_depdt_len[3], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x3')
            end,
            case 
              when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4' 
                then sum(coalesce(tb_a_back_pn.node_depdt_len[4], 1)) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
              else max(coalesce(tb_a_back_pn.node_depdt_len[4], 1)) filter(where tb_a_fore.node_fn_type <> '06_aggr_mx_concat_x4')
            end
          ] 
        )[1 : max(array_length(tb_a_back_pn.node_depdt_len, 1))]
          as a_pn_len
      -- 若干类型算子粒度的超参数在此自动配置填充，而不需要手动配置。参看字典表对各类算子 node_fn_asso_value 的解释
      , case tb_a_fore.node_fn_type
          when '06_aggr_mx_avg'
            then array[count(tb_a_back_pn.node_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')]
          when '06_aggr_mx_concat_y'
            then array_agg(tb_a_back_pn.node_depdt_len[1] order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
          when '06_aggr_mx_concat_x'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[2], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
          when '06_aggr_mx_concat_x3'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[3], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
          when '06_aggr_mx_concat_x4'
            then array_agg(coalesce(tb_a_back_pn.node_depdt_len[4], 1) order by tb_a_path_pn.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
        end as a_node_fn_asso_value
      from sm_sc.tb_nn_node tb_a_fore
      inner join sm_sc.tb_nn_path tb_a_path_pn
        on tb_a_path_pn.work_no = i_work_no   -- 2021082501
          and tb_a_path_pn.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.tb_nn_node tb_a_back_pn
        on tb_a_back_pn.work_no = i_work_no   -- 2021082501
          and tb_a_back_pn.node_no = tb_a_path_pn.back_node_no
      where tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
        and tb_a_fore.node_fn_type in ('06_aggr_mx_avg', '06_aggr_mx_concat_y', '06_aggr_mx_concat_x', '06_aggr_mx_concat_x3', '06_aggr_mx_concat_x4')
      group by tb_a_fore.node_no, tb_a_fore.node_fn_type
    ),
    cte_p1_p2 as 
    (
      select 
        tb_a_fore.node_no,
        coalesce(tb_a_back_pn.a_pn_len, tb_a_back_p1.node_depdt_len) as a_p1_len,
        case when tb_a_fore.node_fn_type not like '06_aggr_mx_%' then tb_a_back_p2.node_depdt_len end as a_p2_len,
        tb_a_back_pn.a_node_fn_asso_value
      from sm_sc.tb_nn_node tb_a_fore
      inner join sm_sc.tb_nn_path tb_a_path_p1
        on tb_a_path_p1.work_no = i_work_no   -- 2021082501
          and tb_a_path_p1.path_ord_no = 1 -- -- 如果与 前向节点的 path_ord_no 冲突，使用 00_none 算子隔离。
          and tb_a_path_p1.fore_node_no = tb_a_fore.node_no
      inner join sm_sc.tb_nn_node tb_a_back_p1
        on tb_a_back_p1.work_no = i_work_no   -- 2021082501
          and tb_a_back_p1.node_no = tb_a_path_p1.back_node_no
      left join sm_sc.tb_nn_path tb_a_path_p2
        on tb_a_path_p2.work_no = i_work_no   -- 2021082501
          and tb_a_path_p2.path_ord_no = 2 -- -- 如果与 前向节点的 path_ord_no 冲突，使用 00_none 算子隔离。
          and tb_a_path_p2.fore_node_no = tb_a_fore.node_no
      left join sm_sc.tb_nn_node tb_a_back_p2
        on tb_a_back_p2.work_no = i_work_no   -- 2021082501
          and tb_a_back_p2.node_no = tb_a_path_p2.back_node_no
      left join cte_concat_dims_change tb_a_back_pn
        on tb_a_back_pn.node_no = tb_a_fore.node_no
      where tb_a_fore.work_no = i_work_no   -- 2021082501
        and tb_a_fore.nn_depth_no = v_cur_nn_depth
    )
    update sm_sc.tb_nn_node tb_a_fore
    set 
      node_fn_asso_value = 
        coalesce(tb_a_fore.node_fn_asso_value, tb_a_p1_p2.a_node_fn_asso_value) 
    , node_depdt_len = 
        sm_sc.fv_lambda_arr_len
        (
          tb_a_fore.node_no,
          tb_a_fore.node_fn_type, 
          tb_a_p1_p2.a_p1_len :: int[], 
          tb_a_p1_p2.a_p2_len :: int[],
          coalesce(tb_a_fore.node_fn_asso_value, tb_a_p1_p2.a_node_fn_asso_value)
        )
    from cte_p1_p2 tb_a_p1_p2
    where tb_a_fore.work_no = i_work_no   -- 2021082501
      and tb_a_fore.node_no = tb_a_p1_p2.node_no
    ;
    commit;

    v_cur_nn_depth := v_cur_nn_depth + 1;
  end loop;

-- ---------------------------------------------------------------
  -- weight 的高宽，由前向算子的高宽反推得出
  update sm_sc.tb_nn_node tb_a_back 
  set 
    node_fn_asso_value = 
      sm_sc.fv_nn_weight_len
      (
        tb_a_fore.node_fn_type
      , tb_a_path.path_ord_no
      , tb_a_fore.node_fn_asso_value
      )
  from sm_sc.tb_nn_path tb_a_path, sm_sc.tb_nn_node tb_a_fore 
  where tb_a_back.work_no = i_work_no   -- 2021082501
    and tb_a_path.work_no = i_work_no   -- 2021082501
    and tb_a_fore.work_no = i_work_no   -- 2021082501
    and tb_a_path.back_node_no = tb_a_back.node_no
    and tb_a_fore.node_no = tb_a_path.fore_node_no
    and tb_a_back.node_type = 'weight'     
    and tb_a_back.node_fn_asso_value is null                   -- weight 节点的 tb_a_back.node_fn_type is null
    -- and tb_a_path.path_ord_no = 2
  ;
  commit;

-- ------------------------------------------------------------------------------
  -- 首轮训练之前，以 node_depdt_len 为依据，确定 weight 的矩阵规格
  if v_cur_learn_cnt = 0
  then    
    -- 初始化 w 权值，为避免鞍点，采用随机
    update sm_sc.tb_nn_node
    set 
      node_depdt_vals = 
        sm_sc.fv_new_randn
        (
          0.0 :: float, 
          1.0 / 0.6745  :: float,    -- 假定，权重（斜率）是无偏的，大于 0.0 和小于 0.0 的概率相同，都是 50%
                                     -- 参看标准正态分布表
                                     -- 误差概率分布：当统计值的误差介于±0.6745(标准误差)范围时，概率为 50%；误差介于±2范围时，概率为95%；误差介于±3范围时，概率为99.7%；
                                     -- 也可以自行修改该标准差，使用权重参数的 xaviar 初始化, He 初始化 等方法
          node_fn_asso_value :: int[]
        )
    -- , node_depdt_len = node_fn_asso_value
    where work_no = i_work_no   -- 2021082501
      and node_type = 'weight'          -- weight 节点的 tb_a_back.node_fn_type = '' 或者 is null
      and node_fn_type = '00_const'
    ;
    commit;
  end if;

-- ------------------------------------------------------------------------------
  -- 审查 nn 节点算子类型的超参数是否完善配置
  v_cur_node_nos := 
    (
      with 
      cte_fn_asso_value_range
      as 
      (
        select 
          enum_group
        , sm_sc.fa_range_or(int4range(lower(enum_range) :: int, upper(enum_range) :: int, '[]') :: int4multirange) as a_fn_asso_value_range
        from sm_sc.tb_dic_enum
        where enum_name = 'node_fn_asso_value'
        group by enum_group
      )
      select 
        array_agg(distinct tb_a_node.node_no)
      from cte_fn_asso_value_range tb_a_dic
      inner join sm_sc.tb_nn_node tb_a_node 
        on tb_a_node.node_fn_type = tb_a_dic.enum_group
      where tb_a_node.work_no = i_work_no
        and (not array_length(tb_a_node.node_fn_asso_value, 1) <@ tb_a_dic.a_fn_asso_value_range)
            or tb_a_node.node_fn_asso_value is null
    )
  ;

  delete from sm_sc.__vt_nn_node where work_no = i_work_no;
  insert into sm_sc.__vt_nn_node
  (
    work_no                      ,
    node_no                  ,        
    node_type                    ,
    node_fn_type                 ,
    node_fn_asso_value      ,
    nn_depth_no                  ,
    node_depdt_vals                  
  )
  select 
    work_no                      ,
    node_no                  ,        
    node_type                    ,
    node_fn_type                 ,
    node_fn_asso_value      ,
    nn_depth_no                  ,
    case when node_type = 'weight' or node_fn_type = '00_const' then node_depdt_vals end as node_depdt_vals                 
  from sm_sc.tb_nn_node
  where work_no = i_work_no
  ;    
  delete from sm_sc.__vt_nn_path where work_no = i_work_no;
  insert into sm_sc.__vt_nn_path
  (
    work_no     
  , fore_node_no
  , path_ord_no 
  , back_node_no
  )
  select 
    work_no     
  , fore_node_no
  , path_ord_no 
  , back_node_no
  from sm_sc.tb_nn_path
  where work_no = i_work_no
  ;
  commit;

  if array_length(v_cur_node_nos, 1) > 0 
  then 
    raise warning 'these node_fn_asso_value is null, please check nodes: %.', v_cur_node_nos;
  else 
    raise notice 'prepare well done!!!!!!';
  end if;
  
end
$$
language plpgsql;




-- -- 执行准备和检查
-- call sm_sc.prc_nn_prepare
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   2500      -- ,
--   -- 200
-- );
drop procedure if exists sm_sc.prc_nn_release_sess;
create or replace procedure sm_sc.prc_nn_release_sess
(
  i_work_no      bigint
, i_sess_id      bigint
)
as
$$
-- declare -- here  

begin
  -- set search_path to public;
  update sm_sc.__vt_nn_sess
  set sess_status = '0'
  where work_no = i_work_no
    and sess_id = i_sess_id
  ;
end
$$
language plpgsql;

-- do
-- $$
-- declare 
--   v_sess_id  bigint   :=   907;
-- begin
--   call sm_sc.prc_nn_release_sess
--   (
--     -76543
--   , v_sess_id
--   );
--   raise notice 'release sess_id: %', v_sess_id;
-- end
-- $$
-- language plpgsql
drop procedure if exists sm_sc.prc_nn_subscribe_sess;
create or replace procedure sm_sc.prc_nn_subscribe_sess
(
     in i_work_no      bigint
, inout o_sess_id      bigint
)
as
$$
-- declare -- here  

begin
  -- set search_path to public;
  select  
    sess_id into o_sess_id
  from sm_sc.__vt_nn_sess 
  where work_no = i_work_no
    and sess_status = '0'
  limit 1 
  for update
  ;
  
  if o_sess_id is null
  then   
    raise exception 'sess id are all in use.  ';
  else
    update sm_sc.__vt_nn_sess
    set sess_status = '1'
    where work_no = i_work_no
      and sess_id = o_sess_id
    ;
  end if;
end
$$
language plpgsql;

-- do
-- $$
-- declare 
--   v_sess_id  bigint;
-- begin
--   call sm_sc.prc_nn_subscribe_sess
--   (
--     -76543
--   , v_sess_id
--   );
--   raise notice 'subscribe sess_id: %', v_sess_id;
-- end
-- $$
-- language plpgsql
-- 参看设计文档 《反向传播链式求导步骤编排》

-- 初始学习率：1e-4
-- https://www.zhihu.com/question/387050717

drop procedure if exists sm_sc.prc_nn_train(bigint, float, float, boolean);
create or replace procedure sm_sc.prc_nn_train
(
  i_work_no                          bigint                          ,      -- 训练任务编号
  -- i_loss_fn_type                     varchar(32)[]                   ,      -- 损失函数选取类型。1: 最小二乘法; 2: 交叉熵
  i_learn_rate                       float    default    0.0003      ,      -- 学习率
  i_loss_delta_least_stop_threshold  float    default    0.0001      ,      -- 触发收敛的损失函数梯度阈值
  i_is_adam                          boolean           default    false              -- 是否开启 adam 优化
)
as
$$
declare -- here
  v_learn_cnt_init          int               := (select learn_cnt from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号初始值
  v_cur_learn_cnt           int               := v_learn_cnt_init;   -- 训练次数序号
  v_depdt_01                float[]  ; -- -- 训练集本轮采样之多模态 01
  v_depdt_02                float[]  ; -- -- 训练集本轮采样之多模态 02
  v_depdt_03                float[]  ; -- -- 训练集本轮采样之多模态 03
  v_depdt_04                float[]  ; -- -- 训练集本轮采样之多模态 04
  v_cur_node_nos            bigint[];  -- for debug
  v_cur_loss                float[];   -- 损失函数值之多模态 01
  v_input_nodes             bigint[]          := 
    (
      select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') -- limit 1
    );
  v_cur_nn_depth            int               := 0;   -- 深度层数游标
  v_nn_depth                int;                      -- 深度层数
  v_limit_train_times       int               := (select learn_cnt_fore from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_01' limit 1);
  v_beta_l1                 float    := 0.9;      -- Adam 梯度下降算法一阶矩等比衰减系数
  v_beta_l2                 float    := 0.999;    -- Adam 梯度下降算法二阶矩等比衰减系数
  
  -- 求导时，需要因变量的算子类型集合
  v_arr_fn_depdt_delta      varchar(64)[]   :=   (select array_agg(enum_key) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method' and (substr(enum_value, 5, 1) = '1' or substr(enum_value, 6, 1) = '1'));
  v_loss_fn_type            varchar(32)[]   := (select loss_fn_type from sm_sc.tb_classify_task where work_no = i_work_no limit 1);   -- 训练次数序号初始值

  v_is_input_02             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_02');
  v_is_input_03             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_03');
  v_is_input_04             boolean         := (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'input_04');
  
  v_is_output_01            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_01')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_01 is not null))
  ;
  v_is_output_02            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_02')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_02 is not null))
  ;
  v_is_output_03            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_03')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_03 is not null))
  ;
  v_is_output_04            boolean         := 
    (select true from sm_sc.tb_nn_node where work_no = i_work_no and node_type = 'output_04')
    and 
    (exists(select  from sm_sc.tb_nn_train_input_buff where work_no = i_work_no and i_depdt_04 is not null))
  ;
begin
  set search_path to public;
  
  -- -- -- 强制开启并行
  -- -- -- select * from pg_settings where name ~ 'paral' or name in ('max_worker_processes') limit 100
  -- -- -- show max_worker_processes
  set min_parallel_table_scan_size = 0;
  set min_parallel_index_scan_size = 0;
  set force_parallel_mode = 'off';
  set max_parallel_workers_per_gather = 64;
  set parallel_setup_cost = 0;
  set parallel_tuple_cost = 0.0;

  -- 清理旧有训练结果
  delete from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  commit;
  
  -- -- -- -- 每次训练前，调整抽样数量等策略
  -- -- -- call sm_sc.prc_nn_prepare_per_train(v_cur_learn_cnt, v_cur_loss, ...);

  -- 审计 训练是否已经满足损失函数最低阈值
  if exists (select  from sm_sc.tb_classify_task where work_no = i_work_no and sm_sc.fv_aggr_slice_sum(loss_delta) <= i_loss_delta_least_stop_threshold)
  then
    raise notice 'The loss_delta is adequately less than i_loss_delta_least_stop_threshold already!';
    return;
  end if;

  v_nn_depth := (select max(nn_depth_no) from sm_sc.tb_nn_node where work_no = i_work_no);

  raise notice 'Training begin; at %', date_trunc('milliseconds', now());

  -- 开始训练
  while 
  (
    i_loss_delta_least_stop_threshold <= sm_sc.fv_aggr_slice_sum(v_cur_loss)
    or sm_sc.fv_aggr_slice_is_exists_null(v_cur_loss)
    or v_cur_loss is null
  )
    and v_cur_learn_cnt < v_limit_train_times
  loop

-- -- debug
raise notice 'debug 001. step begin: v_cur_learn_cnt: %; at %;', v_cur_learn_cnt, date_trunc('milliseconds', now());

    -- 触发 dropout 机制，根据 dropout 概率，确定 dropout 节点清单
    update sm_sc.tb_nn_node
    set is_dropout = (dropout_ratio > random())
    where work_no = i_work_no
      and dropout_ratio > 0.0
    ;
    commit;
    
    -- 填充 dropout 节点的因变量
    update sm_sc.tb_nn_node
    set node_depdt_vals = array_fill(dropout_depdt_val, node_depdt_len)
    where work_no = i_work_no
      and is_dropout
    ;

    if exists
       (
         select  from sm_sc.tb_nn_node 
         where work_no = i_work_no 
           and node_fn_type = '00_buff_slice_rand_pick' 
           and node_type = 'input_01'
           limit 1
       )
    then
      -- 如果对训练集采用小批量随机采样，那么每轮初始化 input 值，以支持不同轮次更换训练集
      with
      cte_slice_rand_pick_agg as 
      (
        -- 要确保各模态的小批量随机采样序号集合一致。
        -- 规约：以 input_01 模态的 00_buff_slice_rand_pick 算子超参数配置为基准。而忽略其他模态的配置。
        select  
          array_agg
          (
            int4multirange
            (
              int4range
              (
                o_ord_no :: int
              , o_ord_no :: int + 1
              , '[)'
              )
            )
          )                                                                as a_pick_depdt_idx
        , array_agg(tb_a_idx.o_rand_pick_indepdt_01 order by o_ord_no)     as a_rand_pick_indepdt_01
        , case when v_is_input_02 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_02, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_02
        , case when v_is_input_03 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_03, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_03
        , case when v_is_input_04 then array_agg(coalesce(tb_a_idx.o_rand_pick_indepdt_04, array[null :: float]) order by o_ord_no) end     as a_rand_pick_indepdt_04
        from sm_sc.tb_nn_node tb_a_node,
          sm_sc.ft_nn_buff_slice_rand_pick
          (
            -- 参看 00_buff_slice_rand_pick 的 node_fn_asso_value 字典表
            i_work_no,
            (
              select 
                array_agg(int4range(tb_a_node.node_fn_asso_value[1][a_ord] :: int, tb_a_node.node_fn_asso_value[2][a_ord] :: int + 1, '[)') order by a_ord)
              from generate_series(1, array_length(tb_a_node.node_fn_asso_value, 2)) tb_a(a_ord)
            )
          , sm_sc.fv_mx_ele_2d_2_1d(tb_a_node.node_fn_asso_value[3 : 3][ : ] :: int[])
          ) tb_a_idx(o_ord_no, o_rand_pick_indepdt_01, o_rand_pick_indepdt_02, o_rand_pick_indepdt_03, o_rand_pick_indepdt_04)
        where tb_a_node.work_no = i_work_no
          and tb_a_node.node_type = 'input_01'
          and tb_a_node.node_fn_type = '00_buff_slice_rand_pick'
      )
      update sm_sc.tb_nn_node tb_a_tar
      set 
        node_depdt_vals = 
          case 
            when tb_a_tar.node_type = 'input_01'
              then a_rand_pick_indepdt_01
            when tb_a_tar.node_type = 'input_02'
              then a_rand_pick_indepdt_02
            when tb_a_tar.node_type = 'input_03'
              then a_rand_pick_indepdt_03
            when tb_a_tar.node_type = 'input_04'
              then a_rand_pick_indepdt_04
          end
      , pick_depdt_idx = a_pick_depdt_idx
      from cte_slice_rand_pick_agg tb_a_sour
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_tar.node_fn_type = '00_buff_slice_rand_pick' 
        and tb_a_tar.node_type in ('input_01', 'input_02', 'input_03', 'input_04')
      ;
      commit;
    elsif exists(select  from sm_sc.tb_nn_node where work_no = i_work_no and node_fn_type = '00_full_dataset' and node_type = 'input_01')
    then 
      -- 如果不做小批量随机采样，那么使用训练集全集
      with
      cte_slice_rand_pick as 
      (
        select 
          array_agg(i_indepdt_01)        as a_x_vals_01,
          array_agg(i_indepdt_02)        as a_x_vals_02,
          array_agg(i_indepdt_03)        as a_x_vals_03,
          array_agg(i_indepdt_04)        as a_x_vals_04,
          count(*) :: int       as a_x_cnt
        from sm_sc.tb_nn_train_input_buff
        where work_no = i_work_no
      )
      update sm_sc.tb_nn_node tb_a_tar
      set 
        node_depdt_vals = case tb_a_tar.node_type when 'input_01' then a_x_vals_01 when 'input_02' then a_x_vals_02 when 'input_03' then a_x_vals_03 when 'input_04' then a_x_vals_04 end  
      , pick_depdt_idx = array[int4multirange(int4range(1, a_x_cnt + 1, '[)'))]
      from cte_slice_rand_pick tb_a_sour
      where tb_a_tar.work_no = i_work_no   -- 2021082501
        and tb_a_tar.node_type in ('input_01', 'input_02', 'input_03', 'input_04')
      ;
      commit;
    end if;

    v_cur_nn_depth := 0;

    -- 前向传播

-- -- for debug
-- v_cur_node_nos := 
--   (
--     select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('input_01', 'input_02', 'input_03', 'input_04') -- limit 1
--   );
      
    while v_cur_nn_depth < v_nn_depth
    loop
      v_cur_nn_depth := v_cur_nn_depth + 1;

-- -- -- debug
-- raise notice 'debug 002. step fore begin: v_cur_learn_cnt: %; v_cur_nn_depth: %; v_learn_cnt_init: %; at %;', v_cur_learn_cnt, v_cur_nn_depth, v_learn_cnt_init, date_trunc('milliseconds', now());

      -- 按照 nn_depth_no，逐层前向传播。初始节点只前向传播一次
      with 
      -- 参数准备，单目、双目（三目）算子
      cte_indepdt as
      (
        select 
          tb_a_fore.node_no
        , tb_a_fore.node_fn_type
        , sm_sc.fa_range_or(tb_a_back.pick_depdt_idx) as a_pick_depdt_idx
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 1 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_1st
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 2 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_2nd
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 3 then tb_a_back.node_depdt_vals end) as a_bi_opr_input_3rd
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 1 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_1st
        , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 2 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_2nd
     -- , sm_sc.fa_mx_concat_y(case when tb_a_path.path_ord_no = 3 then tb_a_back.node_depdt_len end) as a_bi_opr_input_len_3rd
        , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 1 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_1st_back
        , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 2 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_2nd_back
     -- , (sm_sc.fa_mx_or(case when tb_a_path.path_ord_no = 3 then array[tb_a_back.is_back_node] else array[false] end))[1] as a_is_bi_opr_input_3rd_back
        from sm_sc.tb_nn_node tb_a_fore
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.work_no = i_work_no   -- 2021082501
            and tb_a_path.fore_node_no = tb_a_fore.node_no
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.work_no = i_work_no   -- 2021082501
            and tb_a_back.node_no = tb_a_path.back_node_no
        -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
        inner join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type'
            and tb_a_dic.enum_key = tb_a_fore.node_fn_type
            and tb_a_dic.enum_group in ('1_p', '2_p', '3_p')
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.nn_depth_no = v_cur_nn_depth
          and (tb_a_fore.is_fore_node or v_cur_learn_cnt = v_learn_cnt_init)
          and not tb_a_fore.is_dropout
        group by tb_a_fore.node_no, tb_a_fore.node_fn_type
      )
      ,
      -- 前向传播
      cte_upd_fore_node_indepdt as 
      (
        update sm_sc.tb_nn_node tb_a_tar_depdt
        set 
          pick_depdt_idx =
            sm_sc.fv_nn_node_pick_depdt_idx
            (
              tb_a_indepdt_fore.node_fn_type
            , tb_a_indepdt_fore.a_pick_depdt_idx
            , tb_a_tar_depdt.node_fn_asso_value
            )
        , node_depdt_vals = 
            sm_sc.fv_lambda_arr
            (
              tb_a_indepdt_fore.node_no
            , tb_a_tar_depdt.node_fn_type
            , tb_a_indepdt_fore.a_bi_opr_input_1st
            , tb_a_indepdt_fore.a_bi_opr_input_2nd
            , tb_a_tar_depdt.node_fn_asso_value
            , tb_a_indepdt_fore.a_bi_opr_input_3rd
            )  
            -- dropout 缩放
            *` tb_a_tar_depdt.dropout_rescale
        , learn_cnt_fore = v_cur_learn_cnt + 1  -- 强制对齐训练次数，避免路径跟踪混乱   -- tb_a_tar_depdt.learn_cnt_fore + 1
        from cte_indepdt tb_a_indepdt_fore
        where tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
        returning 
          tb_a_tar_depdt.node_no as a_cur_node_no
          -- 寄存、传递若干算子的因变量，对 ddepdt_dindepdt 是必要的。
        , case when tb_a_tar_depdt.node_fn_type = any(v_arr_fn_depdt_delta) then tb_a_tar_depdt.node_depdt_vals end as a_node_depdt_vals
      )
--       ,
      -- 前向传播同步求导 ddepdt_dindepdt

-- -- for debug
-- cte_upd_path as 
-- (

        update sm_sc.tb_nn_path tb_a_tar_path
        set 
          ddepdt_dindepdt =
            sm_sc.fv_lambda_arr_ddepdt_dindepdt
            (
              tb_a_indepdt_fore.node_no
            , tb_a_depdt.node_fn_type
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 1, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_1st 
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 2, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_2nd
             -- when tb_a_tar_path.path_ord_no = 3 
             --   then tb_a_indepdt_fore.a_bi_opr_input_3rd
              end 
            , tb_a_tar_path.path_ord_no
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 3, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_2nd 
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 4, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_1st
              end 
            , tb_a_depdt.node_fn_asso_value 
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 5, 1) = '1'
                  or tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 6, 1) = '1'
                  then tb_a_depdt_val.a_node_depdt_vals
              end 
            , case 
                when tb_a_tar_path.path_ord_no = 1 and substr(tb_a_dic.enum_value, 7, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_len_1st
                when tb_a_tar_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 8, 1) = '1'
                  then tb_a_indepdt_fore.a_bi_opr_input_len_2nd
             -- when tb_a_tar_path.path_ord_no = 3 
             --   then tb_a_indepdt_fore.a_bi_opr_input_len_3rd
              end 
            )
        from cte_indepdt tb_a_indepdt_fore
          , cte_upd_fore_node_indepdt tb_a_depdt_val
          , sm_sc.tb_nn_node tb_a_depdt
          , sm_sc.tb_dic_enum tb_a_dic
        where tb_a_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_depdt.node_no = tb_a_indepdt_fore.node_no
          and tb_a_tar_path.work_no = i_work_no   -- 2021082501
          and tb_a_tar_path.fore_node_no = tb_a_indepdt_fore.node_no
          -- 求导时机 = '前向传播同步'
          and tb_a_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_dic.enum_group = '0'
          and tb_a_dic.enum_key = tb_a_depdt.node_fn_type
          and tb_a_depdt_val.a_cur_node_no = tb_a_tar_path.fore_node_no
          and (tb_a_indepdt_fore.a_is_bi_opr_input_1st_back and tb_a_tar_path.path_ord_no = 1
               or tb_a_indepdt_fore.a_is_bi_opr_input_2nd_back and tb_a_tar_path.path_ord_no = 2
            -- or tb_a_indepdt_fore.a_is_bi_opr_input_3rd_back and tb_a_tar_path.path_ord_no = 3
              )
              
-- -- for debug
-- )
-- select 
--   array_agg(a_cur_node_no) -- || v_cur_node_nos
--     into v_cur_node_nos
-- from cte_upd_fore_node_indepdt

      ;      
      commit;
      -- --------------------------------------------------
      with 
      -- 参数准备，聚合算子
      cte_indepdt as
      (
        -- 聚合后的入参归并结果当作单目运算唯一参数
        -- 另一个同类型的限制是，一个CASE无法阻止其所包含的聚集表达式 的计算...取而代之的是，可以使用 一个WHERE或FILTER子句来首先阻止有问题的输入行到达 一个聚集函数。
        -- 采用 filter 过滤掉不必要的聚合输入，达到减少聚合计算以及入参宽高一致性保证等安全问题
        select 
          tb_a_fore.node_no
        , tb_a_fore.node_fn_type
        , case 
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
              then sm_sc.fa_mx_concat_y(tb_a_back.pick_depdt_idx order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
             -- 假定：如果 node_fn_type <>  '06_aggr_mx_concat_y'，那么来路的 pick_depdt_idx 要么为空，要么统一一致
            else sm_sc.fa_range_or(tb_a_back.pick_depdt_idx) filter(where tb_a_fore.node_fn_type <>  '06_aggr_mx_concat_y')
          end
          as a_pick_depdt_idx
        , case 
            when tb_a_fore.node_fn_type = '06_aggr_mx_sum'
              then sm_sc.fa_mx_sum(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_sum')
            when tb_a_fore.node_fn_type = '06_aggr_mx_prod'
              then sm_sc.fa_mx_prod(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_prod')
            when tb_a_fore.node_fn_type = '06_aggr_mx_avg'
              then sm_sc.fa_mx_avg(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_avg')
            when tb_a_fore.node_fn_type = '06_aggr_mx_max'
              then sm_sc.fa_mx_max(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_max')
            when tb_a_fore.node_fn_type = '06_aggr_mx_min'
              then sm_sc.fa_mx_min(tb_a_back.node_depdt_vals) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_min')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_y'
              then sm_sc.fa_mx_concat_y(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_y')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x'
              then sm_sc.fa_mx_concat_x(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3'
              then sm_sc.fa_mx_concat_x3(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x3')
            when tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4'
              then sm_sc.fa_mx_concat_x4(tb_a_back.node_depdt_vals order by tb_a_path.path_ord_no) filter(where tb_a_fore.node_fn_type = '06_aggr_mx_concat_x4')
          end
          as a_opr_depdt
        , sm_sc.fa_mx_max(tb_a_back.node_depdt_len) as a_opr_depdt_len
        , (sm_sc.fa_mx_or(array[tb_a_back.is_back_node]))[1] as a_is_opr_input_back
        from sm_sc.tb_nn_node tb_a_fore
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.work_no = i_work_no   -- 2021082501
            and tb_a_path.fore_node_no = tb_a_fore.node_no
        inner join sm_sc.tb_nn_node tb_a_back
          on tb_a_back.work_no = i_work_no   -- 2021082501
            and tb_a_back.node_no = tb_a_path.back_node_no
        -- 先处理单目、双目（三目）算子，聚合函数算子另行处理
        inner join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type'
            and tb_a_dic.enum_key = tb_a_fore.node_fn_type
            and tb_a_dic.enum_group = 'n_p'
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.nn_depth_no = v_cur_nn_depth
          and (tb_a_fore.is_fore_node or v_cur_learn_cnt = v_learn_cnt_init)
        group by tb_a_fore.node_no, tb_a_fore.node_fn_type
      )
      ,
      -- 前向传播
      cte_upd_fore_node_indepdt as 
      (
        update sm_sc.tb_nn_node tb_a_tar_depdt
        set 
          pick_depdt_idx     = tb_a_indepdt_fore.a_pick_depdt_idx
        , node_depdt_vals    = tb_a_indepdt_fore.a_opr_depdt
        , learn_cnt_fore     = v_cur_learn_cnt + 1  -- 强制对齐训练次数，避免路径跟踪混乱   -- tb_a_tar_depdt.learn_cnt_fore + 1
        from cte_indepdt tb_a_indepdt_fore
        where tb_a_tar_depdt.work_no = i_work_no   -- 2021082501
          and tb_a_indepdt_fore.node_no = tb_a_tar_depdt.node_no
        returning 
          tb_a_tar_depdt.node_no as a_cur_node_no
      )
--       ,
      -- 前向传播同步求导 ddepdt_dindepdt
      
-- -- for debug
-- cte_upd_path as 
-- (

        update sm_sc.tb_nn_path tb_a_tar_path
        set 
          ddepdt_dindepdt =
            case tb_a_fore.node_fn_type
              when '06_aggr_mx_sum'
                then sm_sc.fv_d_mx_sum(tb_a_indepdt_fore.a_opr_depdt_len)
              when '06_aggr_mx_prod'
                then sm_sc.fv_d_mx_prod(tb_a_back.node_depdt_vals, tb_a_fore.node_depdt_vals)
              when '06_aggr_mx_avg'
                then sm_sc.fv_d_mx_avg(tb_a_fore.node_fn_asso_value[1] :: int, tb_a_indepdt_fore.a_opr_depdt_len)
              when '06_aggr_mx_max'
                then sm_sc.fv_d_mx_max(tb_a_back.node_depdt_vals, tb_a_fore.node_depdt_vals)
              when '06_aggr_mx_min'
                then sm_sc.fv_d_mx_min(tb_a_back.node_depdt_vals, tb_a_fore.node_depdt_vals)
            end
        from cte_indepdt tb_a_indepdt_fore
          , sm_sc.tb_nn_node tb_a_fore
          , sm_sc.tb_nn_node tb_a_back
          , sm_sc.tb_dic_enum tb_a_dic
        where tb_a_fore.work_no = i_work_no   -- 2021082501
          and tb_a_fore.node_no = tb_a_indepdt_fore.node_no
          and tb_a_tar_path.work_no = i_work_no   -- 2021082501
          and tb_a_tar_path.fore_node_no = tb_a_indepdt_fore.node_no
          and tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.node_no = tb_a_tar_path.back_node_no
          -- 求导时机 = '前向传播同步'
          and tb_a_dic.enum_name = 'node_fn_type_delta_method'
          and tb_a_dic.enum_group = '0'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
          and tb_a_indepdt_fore.a_is_opr_input_back
          
-- -- for debug
-- )
-- select 
--   array_agg(a_cur_node_no) || v_cur_node_nos
--     into v_cur_node_nos
-- from cte_upd_fore_node_indepdt

      ;      
      commit;

-- -- -- debug
-- raise notice 'debug 003. step fore end: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

    end loop;
-- -----------------------------------------------------------------------
    -- 对 i_depdt_01 小批量抽样，获得本轮 v_depdt
    if v_is_output_01
    then 
      select 
        array_agg(tb_a_buff.i_depdt_01 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_01
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_01'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;
    
-- -- debug
-- raise notice 'debug 003.1. v_depdt_01: %', v_depdt_01;
    
    if v_is_output_02
    then 
       select 
         array_agg(tb_a_buff.i_depdt_02 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
       into 
         v_depdt_02
       from sm_sc.tb_nn_node tb_a_sour
       inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
         on tb_a_sour.node_type = 'output_02'
       inner join sm_sc.tb_nn_train_input_buff tb_a_buff
         on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
       where tb_a_sour.work_no = i_work_no
         and tb_a_buff.work_no = i_work_no
       ;
    end if;
    
    if v_is_output_03
    then
      select 
        array_agg(tb_a_buff.i_depdt_03 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_03
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_03'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;
    
    if v_is_output_04
    then 
      select 
        array_agg(tb_a_buff.i_depdt_04 order by tb_a_idx.a_idx, tb_a_buff.ord_no)
      into 
        v_depdt_04
      from sm_sc.tb_nn_node tb_a_sour
      inner join generate_series(1, array_length(tb_a_sour.pick_depdt_idx, 1)) tb_a_idx(a_idx)
        on tb_a_sour.node_type = 'output_04'
      inner join sm_sc.tb_nn_train_input_buff tb_a_buff
        on tb_a_buff.ord_no <@ (tb_a_sour.pick_depdt_idx[tb_a_idx.a_idx])
      where tb_a_sour.work_no = i_work_no
        and tb_a_buff.work_no = i_work_no
      ;
    end if;

    -- 损失函数
    select 
      array_agg
      (
        sm_sc.fv_lambda_loss
        (
          v_loss_fn_type[substr(node_type, 8) :: int] :: varchar(32)
        , t_a_tar.node_depdt_vals
        , case substr(node_type, 8) :: int 
            when 1
              then v_depdt_01
            when 2
              then v_depdt_02
            when 3
              then v_depdt_03
            when 4
              then v_depdt_04
          end
        ) 
        order by node_type
      )
    into 
      v_cur_loss
    from sm_sc.tb_nn_node t_a_tar 
    where node_type like 'output_%'    -- 规约：每个 work_no 全局的 input_*, output_* 节点各只有一个
      and work_no = i_work_no     -- 2021082501
    ;

-- -- debug
raise notice 'debug 004. step fore end: len of v_depdt: %; v_cur_loss: %; at %;', array_dims(v_depdt_01), v_cur_loss, date_trunc('milliseconds', now());

-- -----------------------------------------------------------------------
    -- 反向传播
    -- 反向传播起点：全局损失函数对输出 z 求导(均方差、最小二乘法)
    -- 规约：node_type in ('output_01', 'output_02', 'output_03', 'output_04') 各只有一个
    update sm_sc.tb_nn_node t_a_tar
    set 
      node_dloss_ddepdt = 
        sm_sc.fv_lambda_dloss_dz
        (
          case node_type when 'output_01' then v_loss_fn_type[1] when 'output_02' then v_loss_fn_type[2] when 'output_03' then v_loss_fn_type[3] when 'output_04' then v_loss_fn_type[4] end
        , t_a_tar.node_depdt_vals
        , case node_type when 'output_01' then v_depdt_01 when 'output_02' then v_depdt_02 when 'output_03' then v_depdt_03 when 'output_04' then v_depdt_04 end
        )
    , learn_cnt_back = v_cur_learn_cnt + 1 -- 强制对齐训练次数，避免路径计算混乱    -- t_tar.learn_cnt_back + 1
    where node_type in ('output_01', 'output_02', 'output_03', 'output_04')
      and work_no = i_work_no   -- 2021082501
    ;
    commit;

-- -- -- debug
-- raise notice 'debug 005. step fore end: v_cur_learn_cnt: %; at %;', v_cur_learn_cnt, date_trunc('milliseconds', now());

    v_cur_nn_depth := v_nn_depth;
    
-- -- for debug
-- v_cur_node_nos := (select array_agg(node_no) from sm_sc.tb_nn_node where work_no = i_work_no and node_type in ('output_01', 'output_02', 'output_03', 'output_04') limit 1);

    while 
      exists 
      (
        select  from sm_sc.tb_nn_node tb_a_back
        where tb_a_back.work_no = i_work_no    -- 2021082501
          and tb_a_back.learn_cnt_back <= v_cur_learn_cnt
          and tb_a_back.is_back_node is true
        limit 1
      )
      and v_cur_nn_depth > 1
    loop

-- -- -- debug
-- raise notice 'debug 006. step back begin: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

      with 
      cte_dloss_dindepdt as
      (
        select 
          tb_a_back.node_no,
          sm_sc.fa_mx_sum
          (
            sm_sc.fv_lambda_arr_dloss_dindepdt
            (
              tb_a_fore.node_no
            , tb_a_fore.node_fn_type
            -- 以下入参是否传入，由 字典表 enum_name = 'node_fn_type_delta_method' 控制
            -- 对于聚合算子的所有自变量，无论顺序，统一以 path_ord_no = 1 配置该控制
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 1, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 2, 1) = '1'
                  then tb_a_back.node_depdt_vals 
              end
            , tb_a_path.path_ord_no 
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 3, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 4, 1) = '1'
                  then tb_a_back_co.node_depdt_vals
              end
            , tb_a_fore.node_fn_asso_value
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 5, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 6, 1) = '1'
                  then tb_a_fore.node_depdt_vals
              end
            , tb_a_fore.node_dloss_ddepdt
            , case 
                when (tb_a_path.path_ord_no = 1 or tb_a_fore.node_fn_type like '06_aggr_%')
                     and substr(tb_a_dic.enum_value, 7, 1) = '1'
                  or tb_a_path.path_ord_no = 2 and substr(tb_a_dic.enum_value, 8, 1) = '1'
                  or tb_a_path.path_ord_no = 3 -- 当前仅 05_conv_2d 支持第三目运算符，且求导需要自变量规格
                  then tb_a_back.node_depdt_len
              end
            , tb_a_path.ddepdt_dindepdt
            )
          ) as a_dloss_dindepdt
        from sm_sc.tb_nn_node tb_a_back
        inner join sm_sc.tb_nn_path tb_a_path
          on tb_a_path.back_node_no = tb_a_back.node_no
      	  and tb_a_path.work_no = i_work_no   -- 2021082501
        inner join sm_sc.tb_nn_node tb_a_fore
          on tb_a_fore.node_no = tb_a_path.fore_node_no
          and tb_a_fore.work_no = i_work_no   -- 2021082501
        left join sm_sc.tb_dic_enum tb_a_dic
          on tb_a_dic.enum_name = 'node_fn_type_delta_method'
          -- and tb_a_dic.enum_group = '1'
          and tb_a_dic.enum_key = tb_a_fore.node_fn_type
        left join sm_sc.tb_nn_path tb_a_path_co
          on tb_a_path_co.fore_node_no = tb_a_fore.node_no
      	  and tb_a_path_co.work_no = i_work_no   -- 2021082501
          and (tb_a_path.path_ord_no = 1 and tb_a_path_co.path_ord_no = 2
               or tb_a_path.path_ord_no = 2 and tb_a_path_co.path_ord_no = 1)
        left join sm_sc.tb_nn_node tb_a_back_co
          on tb_a_back_co.node_no = tb_a_path_co.back_node_no
          and tb_a_back_co.work_no = i_work_no   -- 2021082501
        where tb_a_back.work_no = i_work_no   -- 2021082501
          and tb_a_back.nn_depth_no = v_cur_nn_depth - 1
          and tb_a_back.is_back_node
        group by tb_a_back.node_no
      )
      
-- -- for debug v_cur_node_nos
-- ,
-- cte_upd_node as
-- (

        update sm_sc.tb_nn_node t_tar
        set 
          node_dloss_ddepdt = t_sour.a_dloss_dindepdt,
          learn_cnt_back = v_cur_learn_cnt + 1 -- 强制对齐训练次数，避免路径计算混乱    -- t_tar.learn_cnt_back + 1
        from cte_dloss_dindepdt t_sour
        where t_sour.node_no = t_tar.node_no

-- -- for debug v_cur_node_nos
--   returning t_tar.node_no as a_cur_node_no
-- )
-- select 
--   array_agg(a_cur_node_no) into v_cur_node_nos
-- from cte_upd_node

      ;
      commit;
      
      -- de_broadcast, dloss_ddepdt 的广播逆运算
      update sm_sc.tb_nn_node t_tar
      set 
        node_dloss_ddepdt = sm_sc.fv_aggr_chunk_sum(node_dloss_ddepdt, node_depdt_len)
      where work_no = i_work_no   -- 2021082501
        and nn_depth_no = v_cur_nn_depth - 1
        and cardinality(node_dloss_ddepdt) / cardinality(node_depdt_vals) > 1
        and cardinality(node_dloss_ddepdt) % cardinality(node_depdt_vals) = 0
      ; 
      commit;

-- -- -- debug
-- raise notice 'debug 007. step back end: v_cur_learn_cnt: %; v_cur_node_nos: %; v_cur_nn_depth: %; at: %;', v_cur_learn_cnt, v_cur_node_nos, v_cur_nn_depth, date_trunc('milliseconds', now());

      v_cur_nn_depth := v_cur_nn_depth - 1;

    end loop;

    -- 更新 w 参数，一次训练完成
    if not i_is_adam
    then 
      update sm_sc.tb_nn_node tar
      set 
        node_depdt_vals = tar.node_depdt_vals -` (i_learn_rate *` tar.node_dloss_ddepdt)
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;

    else
      -- 采用 Adam 梯度下降算法
      update sm_sc.tb_nn_node tar
      set 
        -- node_depdt_vals = tar.node_depdt_vals -` (i_learn_rate *` tar.node_dloss_ddepdt)
        cost_delta_l1 = (v_beta_l1 *` coalesce(case when array_length(cost_delta_l1, 1) = array_length(tar.node_dloss_ddepdt, 1) then cost_delta_l1 else null end, tar.node_dloss_ddepdt)) +` ((1.0 :: float - v_beta_l1) *` tar.node_dloss_ddepdt),
        cost_delta_l2 = (v_beta_l2 *` coalesce(case when array_length(cost_delta_l2, 1) = array_length(tar.node_dloss_ddepdt, 1) then cost_delta_l2 else null end, (tar.node_dloss_ddepdt ^` 2.0 :: float))) +` ((1.0 :: float - v_beta_l2) *` (tar.node_dloss_ddepdt ^` 2.0 :: float))
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
      
      update sm_sc.tb_nn_node tar
      set 
        node_depdt_vals =
          tar.node_depdt_vals -`
          (
            (
              cost_delta_l1 *`
              (i_learn_rate / (1.0 :: float - (v_beta_l1 ^ (least(v_cur_learn_cnt, 7070) + 1))))      -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
            ) /`
            (
              sm_sc.fv_ele_replace
              (
                (
                  cost_delta_l2 /` (1.0 :: float - (v_beta_l2 ^ (least(v_cur_learn_cnt, 7070) + 1)))  -- v_cur_learn_cnt 大于 7072 之后，exp 精度不够，等同于 7071
                ) ^` 0.5 :: float
                , array[0.0 :: float]
                , 1e-128 :: float   -- 1e-128 :: float        
              )
            )
          )
      where work_no = i_work_no   -- 2021082501
        and node_type = 'weight'
      ;
      commit;
    end if;

    v_cur_learn_cnt := v_cur_learn_cnt + 1;

    -- 寄存至 sm_sc.tb_classify_task
    if v_cur_learn_cnt % 1 = 0     -- 或者 100 次， 1000 次， 10000次。。。
      or v_cur_learn_cnt >= v_limit_train_times
      or sm_sc.fv_aggr_slice_sum(v_cur_loss) < i_loss_delta_least_stop_threshold
    then 
      insert into sm_sc.tb_classify_task
      (
        work_no
      , learn_cnt
      , loss_delta
      )
      select 
        i_work_no
      , v_cur_learn_cnt
      , v_cur_loss
      on conflict(work_no) do 
      update set
        learn_cnt = v_cur_learn_cnt
      , loss_delta = v_cur_loss
      ;

      insert into sm_sc.__vt_nn_node
      (
        work_no                      ,
        node_no                  ,        
        node_type                    ,
        node_fn_type                 ,
        node_fn_asso_value      ,
        nn_depth_no                  ,
        node_depdt_vals                  
      )
      select 
        tb_a_sour.work_no                      ,
        tb_a_sour.node_no                  ,        
        tb_a_sour.node_type                    ,
        tb_a_sour.node_fn_type                 ,
        tb_a_sour.node_fn_asso_value      ,
        tb_a_sour.nn_depth_no                  ,
        case when tb_a_sour.node_type = 'weight' or tb_a_sour.node_fn_type = '00_const' then tb_a_sour.node_depdt_vals end as node_depdt_vals                 
      from sm_sc.tb_nn_node tb_a_sour
      where tb_a_sour.work_no = i_work_no
      on conflict(work_no, node_no) do 
      update set
        node_type                 = EXCLUDED.node_type                   ,
        node_fn_type              = EXCLUDED.node_fn_type                ,
        node_fn_asso_value        = EXCLUDED.node_fn_asso_value          ,
        nn_depth_no               = EXCLUDED.nn_depth_no                 ,
        node_depdt_vals                 = case when EXCLUDED.node_type = 'weight' or EXCLUDED.node_fn_type = '00_const' then EXCLUDED.node_depdt_vals end
      ;    
      commit;
      
      -- -- 谨慎开启 vacuum。如果可能有其他训练任务同时进行，那么会被 vacuum 阻塞
      -- vacuum full sm_sc.tb_nn_node;
      -- vacuum full sm_sc.tb_nn_path;
      -- vacuum full sm_sc.__vt_nn_node;

-- debug
raise notice 'debug 008. step end: v_cur_learn_cnt: %; v_cur_loss: %; at %;', v_cur_learn_cnt, v_cur_loss, date_trunc('milliseconds', now());

    end if;
  end loop;

  raise notice 'Training end report: v_cur_learn_cnt: %; v_cur_loss: %; at %;', v_cur_learn_cnt, v_cur_loss, date_trunc('milliseconds', now());

  -- 更新 0 号 sess 部署。规约: 0 号 session 为每轮训练后，默认部署。
  delete from sm_sc.__vt_tmp_nn_node
  where work_no = i_work_no
    and sess_id = 0
  ;

  insert into sm_sc.__vt_tmp_nn_node
  (
    sess_id              
  , work_no              
  , node_no              
  , node_type            
  , node_fn_type         
  , node_fn_asso_value   
  , nn_depth_no          
  , node_depdt_vals      
  )
  select 
    0                    
  , work_no              
  , node_no              
  , node_type            
  , coalesce(nullif(node_fn_type, '00_buff_slice_rand_pick'), '00_const')  
  , case node_fn_type when '00_buff_slice_rand_pick' then null else node_fn_asso_value end         
  , nn_depth_no                     
  , case 
      -- -- -- when 'prod_input' 
      -- -- --   then i_in
      -- when node_type = 'input_01' 
      --   then i_indepdt_01
      -- when node_type = 'input_02' 
      --   then i_indepdt_02
      -- when node_type = 'input_03' 
      --   then i_indepdt_03
      -- when node_type = 'input_04' 
      --   then i_indepdt_04
      -- -- when 'offset'
      -- --   then array[array[1]]
      when node_type = 'weight' or node_fn_type = '00_const'
        then node_depdt_vals
      else null
    end             
  from sm_sc.__vt_nn_node
  where work_no = i_work_no
  ;
  
end
$$
language plpgsql;

-- -- 用例 1, 二分类
-- -- -- 执行准备和检查
-- -- call sm_sc.prc_nn_prepare
-- -- (
-- --   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
-- --   2500      -- ,
-- --   -- 200
-- -- );


-- -- 开始训练
-- call sm_sc.prc_nn_train
-- (
--   pgv_get('vars', 'this_work_no_02', NULL::bigint)   ,
--   1        ,
--   0.9      ,
--   0.1
-- );

-- -- -- 观察输出
-- select * from sm_sc.tb_classify_task where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- select node_no, node_fn_type, learn_cnt_fore, learn_cnt_back, tb_a.node_depdt_vals, (tb_a.reg_w).m_vals  from sm_sc.tb_nn_node tb_a where work_no = pgv_get('vars', 'this_work_no_02', NULL::bigint)
-- select 
--   sm_sc.ft_nn_in_out
--   (
--     pgv_get('vars', 'this_work_no_02', NULL::bigint), 
--     array[array[0.0, 0.0], array[0.0, 1.0], array[1.0, 0.0], array[1.0, 1.0]] 
--       +` sm_sc.fv_new_randn(0.0, 0.1, array[4, 2])
--   )

-- -- -- 期望用例举例：
-- -- -- select sigmoid(w31 * sigmoid(w11 + w12 * x1 + w13 * x2) + w32 * sigmoid(w21 + w22 * x1 + w23 * x2))
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 0 + 20.0 * 0) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 0 + 20.0 * 0)) union all   -- 期望输出，0
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 0 + 20.0 * 1) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 0 + 20.0 * 1)) union all   -- 期望输出，1
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 1 + 20.0 * 0) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 1 + 20.0 * 0)) union all   -- 期望输出，1
-- -- select sm_sc.fv_sigmoid((-10.0) + (-20.0) * sm_sc.fv_sigmoid((-30.0) + 20.0 * 1 + 20.0 * 1) + 20.0 * sm_sc.fv_sigmoid((-10.0) + 20.0 * 1 + 20.0 * 1))             -- 期望输出，0

-- -- -----------------------------------------------------------------------------
-- -- 用例 2, 多分类
-- -- -- 执行准备和检查
-- -- call sm_sc.prc_nn_prepare
-- -- (
-- --   2021121701   ,
-- --   5000      -- ,
-- --   -- 200
-- -- );
-- 
-- 
-- -- 开始训练
-- call sm_sc.prc_nn_train
-- (
--   2021121701   ,
--   2        ,
--   0.8      ,
--   0.1
-- );
-- 
-- -- -- 观察输出
-- select 
--   sm_sc.ft_nn_in_out
--   (
--     2021121701, 
--     array[array[0.0, 0.0, 0.0], array[0.0, 1.0, 0.0], array[1.0, 0.0, 0.0], array[1.0, 1.0, 0.0]
--           , array[0.0, 0.0, 1.0], array[0.0, 1.0, 1.0], array[1.0, 0.0, 1.0], array[1.0, 1.0, 1.0]] 
--       +` sm_sc.fv_new_randn(0.0, 0.1, array[8, 3])
--   )

drop procedure if exists sm_sc.prc_dbscan_pp(typ_arr_point[], float);
create or replace procedure sm_sc.prc_dbscan_pp
(
  i_work_no                bigint                      -- 任务编号
)
as
$$
declare 
  v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)          := replace(gen_random_uuid()::char(36), '-', '')::char(32);
-- cnt of points
  v_point_cnt bigint := (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no);
-- amt of points' dimensions
  v_dimension_amt bigint;

-- for cursor of circle
  v_point_id_cur   bigint;
  v_i_cur  bigint := 1;

-- --- func body ----------
begin
  insert into sm_sc.__vt_dbscan_ods_array
  (
    sess_id  ,
    point_id ,
    point_arr
  )
  select 
    v_sess_id,
    ord_no   ,
    i_indepdt
  from sm_sc.tb_nn_train_input_buff
  where work_no = i_work_no
  ;
  -- select * from sm_sc.__vt_dbscan_ods_array limit 50

  -- calcu v_dimension_amt
  select 
    array_length(point_arr, 1) into v_dimension_amt
  from sm_sc.__vt_dbscan_ods_array
  where sess_id = v_sess_id
  limit 1
  ;

  insert into sm_sc.__vt_dbscan_nearby_point_idx
  (
    sess_id          ,
    point_id         ,
    dimension_no   ,
    point_arr_n      ,
    point_arr
  )
  -- pivot point array
  select 
    v_sess_id                                            ,
    tb_a_ods.point_id                                         ,
    a_dimension_no                                       ,
    tb_a_ods.point_arr[a_dimension_no] as point_arr_n         ,
    case when a_dimension_no = 1 then tb_a_ods.point_arr end
  from
    generate_series(1, v_dimension_amt) a_dimension_no,
    sm_sc.__vt_dbscan_ods_array tb_a_ods
  where tb_a_ods.sess_id = v_sess_id
  ;

  -- polar coordinate the arr as dimension_0
  insert into sm_sc.__vt_dbscan_nearby_point_idx
  (
    sess_id                 ,
    point_id                ,
    dimension_no          ,
    point_arr_n
  )
  select 
    v_sess_id    ,
    sour.point_id,
    0 as dimension_no,
    power(sum(power(sour.point_arr_n, 2)), 0.5) as point_arr_n
  from sm_sc.__vt_dbscan_nearby_point_idx sour
  where sess_id = v_sess_id
  group by sour.point_id
  ;
  -- select * from sm_sc.__vt_dbscan_nearby_point_idx where dimension_no = 0 limit 50

  -- ---- prepare done  ------

  -- spread
  while
    exists (select  from sm_sc.__vt_dbscan_ods_array where dbscan_grp is null and sess_id = v_sess_id limit 1)
    and v_i_cur <= v_point_cnt
  loop
    -- cursor point: v_point_id_cur
    select 
      point_id into v_point_id_cur
    from sm_sc.__vt_dbscan_ods_array
    where dbscan_grp is null
      and sess_id = v_sess_id
    limit 1
    ;

    with
    -- find all grp_labeled points of the grp which v_point_id_cur belong to
    cte_tar_point_with_cur as
    (
      select 
        incre.point_id
      from
      (
        select distinct
          t_a_guest_incre_0.point_id,
          array[max(t_a_host_incre_0.point_arr), max(t_a_guest_incre_0.point_arr)] as host_guest_arrs
        from sm_sc.__vt_dbscan_nearby_point_idx t_a_host_incre_0
        inner join sm_sc.__vt_dbscan_nearby_point_idx t_a_guest_incre_0
          on t_a_guest_incre_0.dimension_no = t_a_host_incre_0.dimension_no
            and t_a_guest_incre_0.point_arr_n between t_a_host_incre_0.point_arr_n - i_max_point_distance
                                          and t_a_host_incre_0.point_arr_n + i_max_point_distance
        where t_a_host_incre_0.point_id = v_point_id_cur
          and t_a_host_incre_0.sess_id = v_sess_id
          and t_a_guest_incre_0.sess_id = v_sess_id
        group by t_a_guest_incre_0.point_id, t_a_host_incre_0.point_id
        having count(t_a_guest_incre_0.dimension_no) = v_dimension_amt + 1 
      ) incre
      where sm_sc.fv_o_distance(incre.host_guest_arrs, v_dimension_amt) <= i_max_point_distance
    )
    -- select * from cte_tar_point_with_cur limit 50
    ,
    cte_tar_grp_with_cur as
    (
      select distinct
        tb_a_ods.dbscan_grp as old_dbscan_grp
      from cte_tar_point_with_cur t_a_tar_point
      inner join sm_sc.__vt_dbscan_ods_array tb_a_ods
        on t_a_tar_point.point_id = tb_a_ods.point_id
      where tb_a_ods.sess_id = v_sess_id
    )
    -- select * from cte_tar_grp_with_cur limit 50
    ,
    cte_tar_upd_old_grp_with_cur as
    (
      update sm_sc.__vt_dbscan_ods_array ods
      set dbscan_grp = v_point_id_cur
      from cte_tar_grp_with_cur grp_cur
      where grp_cur.old_dbscan_grp = ods.dbscan_grp
        and ods.dbscan_grp <> v_point_id_cur
        and ods.sess_id = v_sess_id
    )

    -- update dbscan_grp in ods
    update sm_sc.__vt_dbscan_ods_array ods
    set dbscan_grp = v_point_id_cur
    from cte_tar_point_with_cur point_cur
    where point_cur.point_id = ods.point_id
      and ods.dbscan_grp is null
      and ods.sess_id = v_sess_id
    ;

    -- break circle once v_i_cur > cnt of points
    select v_i_cur + 1 into v_i_cur;
  end loop;

  update sm_sc.tb_nn_train_input_buff t_a_tar
  set 
    i_depdt = tb_a_mid.i_indepdt
  from sm_sc.__vt_dbscan_ods_array tb_a_sour, sm_sc.tb_nn_train_input_buff tb_a_mid
  where tb_a_sour.sess_id = v_sess_id
    and t_a_tar.work_no = i_work_no
	and tb_a_mid.work_no = i_work_no
	and tb_a_mid.ord_no = tb_a_sour.dbscan_grp
	and t_a_tar.ord_no = tb_a_sour.point_id
  ;

  -- delete temp data
  delete from sm_sc.__vt_dbscan_ods_array where sess_id = v_sess_id;
  commit;
  delete from sm_sc.__vt_dbscan_nearby_point_idx where sess_id = v_sess_id;
  commit;
end
$$
language plpgsql;

-- ---------------

-- insert into sm_sc.tb_cluster_task
-- (
--   work_no               ,
--   cluster_cnt           ,
--   max_point_distance    ,
--   cluster_type      
-- )
-- select 
--   -2,
--   null,   -- 2,
--   5,
--   2,
-- ;
-- 
-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   ord_no,
--   i_indepdt
-- )
-- select 1, array[11.0, 22.1, 33.4]
-- union all
-- select 1, array[11.1, 22.2, 33.3]
-- union all
-- select 1, array[44.4, 55.5, 66.6]
-- union all
-- select 1, array[44.6, 55.7, 66.4]
-- union all
-- select 1, array[44.5, 55.4, 66.2]
-- return 
-- ;
-- 
-- call sm_sc.prc_kmeans_pp(-2);
drop procedure if exists sm_sc.prc_kmeans_pp(typ_arr_point[], float);
create or replace procedure sm_sc.prc_kmeans_pp
(
  i_work_no                bigint                      -- 任务编号
)
as
$$

-- -------------------- declare part --------------------------------------------------
declare
  v_sess_id   bigint        :=       lower(sm_sc.fv_get_global_seq());  -- char(32)        :=       replace(gen_random_uuid()::char(36), '-', '')::char(32);

-- cnt of points
  v_point_cnt int := (select count(*) from sm_sc.tb_nn_train_input_buff where work_no = i_work_no);
-- amt of points' demensions
  v_dimension_amt int;

-- for cursor of loop
  v_loop_cnt int := 1;

-- -------------------- func part --------------------------------------------------
begin
  insert into sm_sc.__vt_kmean_ods_array
  (
    sess_id   ,
    point_id  ,
    point_arr
  )
  -- parse to point_id, point_arr
  select 
    v_sess_id,
    ord_no   ,
    i_indepdt
  from sm_sc.tb_nn_train_input_buff
  where work_no = i_work_no
  ;
  -- select * from sm_sc.__vt_kmean_ods_array limit 50

  -- calcu v_dimension_amt
  select 
    array_length(point_arr, 1) into v_dimension_amt 
  from sm_sc.__vt_kmean_ods_array
  where sess_id = v_sess_id
  limit 1
  ;

  -- a random point as the first cluster point
  insert into sm_sc.__vt_kmean_list_cluster(sess_id, cluster_point_no)
  select 
    v_sess_id,
    v_cluster_point_no as cluster_point_no
  from generate_series(1, i_cluster_cnt) v_cluster_point_no
  ;

  update sm_sc.__vt_kmean_list_cluster tar
  set cluster_point_arr = (select point_arr from sm_sc.__vt_kmean_ods_array where sess_id = v_sess_id limit 1)
  where tar.cluster_point_no = 1
    and tar.sess_id = v_sess_id
  ;
  -- ------------------- prepare done ------------------------------------------
  
  -- find all cluster points one by one
  while v_loop_cnt <= least(i_cluster_cnt, v_point_cnt)
  loop
    with
    cte_point_cluster_distance_pre as
    (
      select 
        tb_a_ods.point_id 
        , tb_a_ods.point_arr
        , min(sm_sc.fv_o_distance(tb_a_ods.point_arr, tb_a_clusters.cluster_point_arr, v_dimension_amt)) as point_cluster_distance_pre
      from sm_sc.__vt_kmean_ods_array tb_a_ods, sm_sc.__vt_kmean_list_cluster tb_a_clusters
      where tb_a_ods.sess_id = v_sess_id
        and tb_a_clusters.sess_id = v_sess_id
      group by tb_a_ods.point_id
    )
    ,
    cte_point_cluster_weight_position as
    (
      select
        point_id,
        point_arr,
        sum(point_cluster_distance_pre) over(order by point_id rows between UNBOUNDED preceding and 0 following) as point_cluster_weight_position
      from cte_point_cluster_distance_pre
      where point_cluster_distance_pre > 0
    )
    ,
    cte_random_position as
    (
      select sum(point_cluster_distance_pre) * random() as random_position
      from cte_point_cluster_distance_pre
    )
    ,
    cte_cluster_point as
    (
      select 
        point_id
        , point_arr
        , point_cluster_weight_position
        , random_position
      from cte_point_cluster_weight_position, cte_random_position
      where point_cluster_weight_position >= random_position
      order by point_cluster_weight_position
      limit 1
    )
    update sm_sc.__vt_kmean_list_cluster tar
    set cluster_point_arr = sour.point_arr
    from cte_cluster_point sour
    where tar.cluster_point_no = v_loop_cnt
      and tar.sess_id = v_sess_id
    ;
    select v_loop_cnt + 1 into v_loop_cnt;

  end loop; 
  -- find cluster points down ---------------------------------
  
  -- reset cluster_point_no for all point_id's
  with
  cte_point_cluster_distance_post as
  (
    select
      tb_a_ods.point_id,
      tb_a_clusters.cluster_point_no,
      row_number() over(partition by tb_a_ods.point_id order by sm_sc.fv_o_distance(tb_a_ods.point_arr, tb_a_clusters.cluster_point_arr, v_dimension_amt)) as point_cluster_distance_ord
    from sm_sc.__vt_kmean_ods_array tb_a_ods
    cross join sm_sc.__vt_kmean_list_cluster tb_a_clusters
    where tb_a_ods.sess_id = v_sess_id
      and tb_a_clusters.sess_id = v_sess_id
  )
  update sm_sc.__vt_kmean_ods_array tar
  set cluster_point_no = cte_distance.cluster_point_no
  from cte_point_cluster_distance_post cte_distance
  where cte_distance.point_cluster_distance_ord = 1
    and cte_distance.point_id = tar.point_id
    and tar.sess_id = v_sess_id
  ;

  -- spread
  select 1 into v_loop_cnt;
  while
    exists(select  from sm_sc.__vt_kmean_list_cluster tb_a_clusters 
           inner join sm_sc.__vt_kmean_ods_array tb_a_ods 
             on tb_a_clusters.cluster_point_no = tb_a_ods.cluster_point_no 
           where is_loop_done is not true
             and tb_a_ods.sess_id = v_sess_id
             and tb_a_clusters.sess_id = v_sess_id
          )
    -- and v_loop_cnt <= i_loop_cnt
  loop
    -- re-calcu cluster_point_arr for all cluster_point_no
    with
    cte_cluster_dimention_arr as
    (
      select 
        cluster_point_no,
        a_cur_dim_no as dim_no,
        avg(point_arr[a_cur_dim_no]) as cluster_dimention_arr
      from sm_sc.__vt_kmean_ods_array tb_a_ods, generate_series(1, v_dimension_amt) tb_a_cur_dim_no(a_cur_dim_no)
      where tb_a_ods.sess_id = v_sess_id
      group by cluster_point_no, a_cur_dim_no
    )
    ,
    cte_cluster_point_arr as
    (
      select 
        cluster_point_no,
        array_agg(cluster_dimention_arr order by dim_no) as cluster_point_arr
      from cte_cluster_dimention_arr
      group by cluster_point_no
    )   
    update sm_sc.__vt_kmean_list_cluster tar
    set is_loop_done = case when sm_sc.fv_o_distance(tar.cluster_point_arr, cte_c_arr.cluster_point_arr, v_dimension_amt) < 0.001
                              then true
                            else false
                       end
      , cluster_point_arr = cte_c_arr.cluster_point_arr
      , loop_cnt = v_loop_cnt
    from cte_cluster_point_arr cte_c_arr
    where cte_c_arr.cluster_point_no = tar.cluster_point_no
      and tar.sess_id = v_sess_id
    ;

    -- reset cluster_point_no for all point_id
    with
    cte_point_cluster_distance as 
    (
      select 
        tb_a_ods.point_id,
        tb_a_clusters.cluster_point_no,
        row_number() over(partition by tb_a_ods.point_id order by sm_sc.fv_o_distance(tb_a_ods.point_arr, tb_a_clusters.cluster_point_arr, v_dimension_amt)) as point_cluster_distance_ord
      from sm_sc.__vt_kmean_ods_array tb_a_ods
      cross join sm_sc.__vt_kmean_list_cluster tb_a_clusters
      where tb_a_ods.sess_id = v_sess_id
        and tb_a_clusters.sess_id = v_sess_id
    )
    update sm_sc.__vt_kmean_ods_array tar
    set cluster_point_no = cte_distance.cluster_point_no
    from cte_point_cluster_distance cte_distance
    where cte_distance.point_id = tar.point_id
      and tar.sess_id = v_sess_id
    ;

    select v_loop_cnt + 1 into v_loop_cnt;
  end loop;

  -- output of func
  update sm_sc.tb_nn_train_input_buff t_a_tar
  set 
    i_depdt = tb_a_mid.i_indepdt,
    i_depdt = tb_a_clusters.cluster_point_arr
  from sm_sc.__vt_kmean_ods_array tb_a_ods
  inner join sm_sc.__vt_kmean_list_cluster tb_a_clusters
    on tb_a_ods.cluster_point_no = tb_a_clusters.cluster_point_no
  where tb_a_ods.sess_id = v_sess_id
    and tb_a_clusters.sess_id = v_sess_id
	and t_a_tar.work_no = i_work_no
	and t_a_tar.ord_no = tb_a_ods.point_id
  ;
  
  delete from sm_sc.__vt_kmean_list_cluster where sess_id = v_sess_id;
  commit;
  delete from sm_sc.__vt_kmean_ods_array where sess_id = v_sess_id;
  commit;

end
$$
language plpgsql;

-- ---------------

-- insert into sm_sc.tb_cluster_task
-- (
--   work_no               ,
--   cluster_cnt           ,
--   max_point_distance    ,
--   cluster_type      
-- )
-- select 
--   -1,
--   2,
--   null,   -- 5,
--   1,
-- ;
-- 
-- insert into sm_sc.tb_nn_train_input_buff
-- (
--   ord_no,
--   i_indepdt
-- )
-- select 1, array[11.0, 22.1, 33.4]
-- union all
-- select 1, array[11.1, 22.2, 33.3]
-- union all
-- select 1, array[44.4, 55.5, 66.6]
-- union all
-- select 1, array[44.6, 55.7, 66.4]
-- union all
-- select 1, array[44.5, 55.4, 66.2]
-- return 
-- ;
-- 
-- call sm_sc.prc_kmeans_pp(-1);


-- 初始化全局序列值
insert into sm_sc.__vt_global_seq(seq_no, cur_val) select 1, 1
on conflict (seq_no) do nothing
;
-- ---------------------------------------------------------------------------------------------------------------------

-- 初始化字典表
-- -------------------------------------------------------------------------
-- loss_fn_type  -- 损失函数类型
-- delete from sm_sc.tb_dic_enum where enum_name = 'loss_fn_type';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_order
)
with cte_loss_fn_type(enum_key, enum_value) as
(
  select '101', '最小二乘法'              union all
  select '201', '交叉熵    '              union all
  select '202', '交叉熵第二维度平均'      union all
  select '203', '交叉熵。真实值入参是 onehot 编码的编号，而不是 onehot 编码本身'
)
select 
  'loss_fn_type' as enum_name, 
  enum_key, 
  enum_value, 
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'loss_fn_type'), 0) 
  as enum_order
from cte_loss_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_order = excluded.enum_order
;

-- -------------------------------------------------------------------------
-- node_type   -- nn 节点类型
-- delete from sm_sc.tb_dic_enum where enum_name = 'node_type';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_order
)
with cte_node_type(enum_key, enum_value) as
(
  select 'input_01', 'input_01' union all   -- 规约：整个 nn 的小批量采样以该节点配置为准。缺省情况下，该配置产生的采样清单也同时更新到 input_02, input_03, input_04
  select 'input_02', 'input_02' union all   -- 规约：保持没有随机小批量采样配置即可。如果没有额外更新逻辑，该节点数据采样与 input_01 一致。不一致场景要额外编写逻辑。
  select 'input_03', 'input_03' union all   -- 规约：保持没有随机小批量采样配置即可。如果没有额外更新逻辑，该节点数据采样与 input_01 一致。不一致场景要额外编写逻辑。
  select 'input_04', 'input_04' union all   -- 规约：保持没有随机小批量采样配置即可。如果没有额外更新逻辑，该节点数据采样与 input_01 一致。不一致场景要额外编写逻辑。
  select 'output_01', 'output_01' union all
  select 'output_02', 'output_02' union all
  select 'output_03', 'output_03' union all
  select 'output_04', 'output_04' union all
  -- select 'offset', 'input' union all
  select 'weight', 'train' -- union all
  -- select 'prod_input', 'prod'
)
select 
  'node_type' as enum_name, 
  enum_key, 
  enum_value, 
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_type'), 0) 
  as enum_order
from cte_node_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_order = excluded.enum_order
;

-- -------------------------------------------------------------------------
-- node_fn_type   -- 算子
-- delete from sm_sc.tb_dic_enum where enum_name = 'node_fn_type';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as
(
  select trim('00_full_dataset        '),         trim('                                 '),            '0_p' union all
  select trim('00_const               '),         trim('                                 '),            '0_p' union all
  select trim('00_none                '),         trim('sm_sc.fv_nn_none_py              '),            '1_p' union all
  select trim('00_buff_slice_rand_pick'),         trim('sm_sc.ft_nn_buff_slice_rand_pick '),            '1_p' union all
  select trim('01_add                 '),         trim('sm_sc.fv_opr_add_py              '),            '2_p' union all
  select trim('01_mul                 '),         trim('sm_sc.fv_opr_mul_py              '),            '2_p' union all
  select trim('01_sub                 '),         trim('sm_sc.fv_opr_sub_py              '),            '2_p' union all
  select trim('01_0sub                '),         trim('sm_sc.fv_opr_sub_py              '),            '1_p' union all
  select trim('01_div                 '),         trim('sm_sc.fv_opr_div_py              '),            '2_p' union all
  select trim('01_1div                '),         trim('sm_sc.fv_opr_div_py              '),            '1_p' union all
  select trim('01_pow                 '),         trim('sm_sc.fv_opr_pow_py              '),            '2_p' union all
  select trim('01_log                 '),         trim('sm_sc.fv_opr_log_py              '),            '2_p' union all
  select trim('01_exp                 '),         trim('sm_sc.fv_opr_exp_py              '),            '1_p' union all
  select trim('01_ln                  '),         trim('sm_sc.fv_opr_ln_py               '),            '1_p' union all
  select trim('01_abs                 '),         trim('sm_sc.fv_opr_abs                 '),            '1_p' union all
  select trim('01_prod_mx             '),         trim('sm_sc.fv_opr_prod_mx_py          '),            '2_p' union all
  select trim('01_chunk_prod_mx       '),         trim('sm_sc.fv_chunk_prod_mx           '),            '2_p' union all
  select trim('02_sin                 '),         trim('sm_sc.fv_sin                     '),            '1_p' union all
  select trim('02_cos                 '),         trim('sm_sc.fv_cos                     '),            '1_p' union all
  select trim('02_tan                 '),         trim('sm_sc.fv_tan                     '),            '1_p' union all
  select trim('02_cot                 '),         trim('sm_sc.fv_cot                     '),            '1_p' union all
  select trim('02_sec                 '),         trim('sm_sc.fv_sec                     '),            '1_p' union all
  select trim('02_csc                 '),         trim('sm_sc.fv_csc                     '),            '1_p' union all
  select trim('02_asin                '),         trim('sm_sc.fv_asin                    '),            '1_p' union all
  select trim('02_acos                '),         trim('sm_sc.fv_acos                    '),            '1_p' union all
  select trim('02_atan                '),         trim('sm_sc.fv_atan                    '),            '1_p' union all
  select trim('02_acot                '),         trim('sm_sc.fv_acot                    '),            '1_p' union all
  select trim('02_sinh                '),         trim('sm_sc.fv_sinh                    '),            '1_p' union all
  select trim('02_cosh                '),         trim('sm_sc.fv_cosh                    '),            '1_p' union all
  select trim('02_tanh                '),         trim('sm_sc.fv_tanh                    '),            '1_p' union all
  -- -- select trim('02_sech                '),         trim('sm_sc.fv_sech                    '),            '1_p' union all
  -- -- select trim('02_csch                '),         trim('sm_sc.fv_csch                    '),            '1_p' union all
  select trim('02_asinh               '),         trim('sm_sc.fv_asinh                   '),            '1_p' union all
  select trim('02_acosh               '),         trim('sm_sc.fv_acosh                   '),            '1_p' union all
  select trim('02_atanh               '),         trim('sm_sc.fv_atanh                   '),            '1_p' union all
  select trim('03_sigmoid             '),         trim('sm_sc.fv_activate_sigmoid        '),            '1_p' union all
  select trim('03_relu                '),         trim('sm_sc.fv_activate_relu           '),            '1_p' union all
  select trim('03_leaky_relu          '),         trim('sm_sc.fv_activate_leaky_relu     '),            '1_p' union all
  select trim('03_elu                 '),         trim('sm_sc.fv_activate_elu            '),            '1_p' union all
  select trim('03_selu                '),         trim('sm_sc.fv_activate_selu           '),            '1_p' union all
  select trim('03_gelu                '),         trim('sm_sc.fv_activate_gelu           '),            '1_p' union all
  select trim('03_swish               '),         trim('sm_sc.fv_activate_swish          '),            '1_p' union all
  select trim('03_softplus            '),         trim('sm_sc.fv_activate_softplus       '),            '1_p' union all
  select trim('03_boxcox              '),         trim('sm_sc.fv_activate_boxcox         '),            '1_p' union all
  select trim('03_softmax             '),         trim('sm_sc.fv_redistr_softmax         '),            '1_p' union all
  select trim('03_softmax_ex          '),         trim('sm_sc.fv_redistr_softmax_ex      '),            '1_p' union all
  select trim('03_zscore              '),         trim('sm_sc.fv_redistr_zscore          '),            '1_p' union all
  select trim('04_new                 '),         trim('sm_sc.fv_new                     '),            '1_p' union all
  select trim('04_repeat_axis         '),         trim('sm_sc.fv_repeat_axis_py          '),            '1_p' union all
  select trim('04_apad                '),         trim('sm_sc.fv_apad                    '),            '2_p' union all
  select trim('04_bpad                '),         trim('sm_sc.fv_bpad                    '),            '2_p' union all
  select trim('04_lpad                '),         trim('sm_sc.fv_lpad                    '),            '2_p' union all
  select trim('04_rpad                '),         trim('sm_sc.fv_rpad                    '),            '2_p' union all
  select trim('04_transpose           '),         trim('sm_sc.fv_opr_transpose_py        '),            '1_p' union all
  select trim('04_transpose_i         '),         trim('sm_sc.fv_opr_transpose_i_py      '),            '1_p' union all
  select trim('04_chunk_transpose     '),         trim('sm_sc.fv_chunk_transpose         '),            '1_p' union all
  select trim('04_transpose_nd        '),         trim('sm_sc.fv_opr_transpose_nd_py     '),            '1_p' union all
  select trim('04_turn_90             '),         trim('sm_sc.fv_opr_turn_90_py          '),            '1_p' union all
  select trim('04_turn_180            '),         trim('sm_sc.fv_opr_turn_180_py         '),            '1_p' union all
  select trim('04_mirror              '),         trim('sm_sc.fv_opr_mirror_py           '),            '1_p' union all
  select trim('04_mx_ele_3d_2_2d      '),         trim('sm_sc.fv_mx_ele_3d_2_2d_py       '),            '1_p' union all
  select trim('04_mx_ele_2d_2_3d      '),         trim('sm_sc.fv_mx_ele_2d_2_3d_py       '),            '1_p' union all
  select trim('04_mx_ele_4d_2_3d      '),         trim('sm_sc.fv_mx_ele_4d_2_3d_py       '),            '1_p' union all
  select trim('04_mx_ele_3d_2_4d      '),         trim('sm_sc.fv_mx_ele_3d_2_4d_py       '),            '1_p' union all
  select trim('04_mx_ele_flatten_2dims'),         trim('sm_sc.fv_mx_ele_flatten_2dims_py '),            '1_p' union all
  select trim('04_mx_slice_3d_2_2d    '),         trim('sm_sc.fv_mx_slice_3d_2_2d_py     '),            '1_p' union all
  select trim('04_mx_slice_4d_2_2d    '),         trim('sm_sc.fv_mx_slice_4d_2_2d_py     '),            '1_p' union all
  select trim('04_mx_slice_4d_2_3d    '),         trim('sm_sc.fv_mx_slice_4d_2_3d_py     '),            '1_p' union all
  select trim('04_mx_ascend_dim       '),         trim('sm_sc.fv_mx_ascend_dim           '),            '1_p' union all
  select trim('04_mx_descend_dim      '),         trim('sm_sc.fv_mx_descend_dim_py       '),            '1_p' union all
  select trim('04_rand_pick_y         '),         trim('sm_sc.ft_rand_slice_y_pick       '),            '1_p' union all
  select trim('04_rand_pick_x         '),         trim('sm_sc.ft_rand_slice_x_pick       '),            '1_p' union all
  select trim('04_rand_pick_x3        '),         trim('sm_sc.ft_rand_slice_x3_pick      '),            '1_p' union all
  select trim('04_rand_pick_x4        '),         trim('sm_sc.ft_rand_slice_x4_pick      '),            '1_p' union all
  select trim('04_chunk               '),         trim('sm_sc.fv_chunk                   '),            '1_p' union all
  select trim('04_slice_y             '),         trim('sm_sc.fv_slice_y                 '),            '1_p' union all
  select trim('04_slice_x             '),         trim('sm_sc.fv_slice_x                 '),            '1_p' union all
  select trim('04_slice_x3            '),         trim('sm_sc.fv_slice_x3                '),            '1_p' union all
  select trim('04_slice_x4            '),         trim('sm_sc.fv_slice_x4                '),            '1_p' union all
  select trim('04_sample_y            '),         trim('sm_sc.fv_sample_y                '),            '1_p' union all
  select trim('04_sample_x            '),         trim('sm_sc.fv_sample_x                '),            '1_p' union all
  select trim('04_sample_x3           '),         trim('sm_sc.fv_sample_x3               '),            '1_p' union all
  select trim('04_sample_x4           '),         trim('sm_sc.fv_sample_x4               '),            '1_p' union all
  select trim('04_lower_tri_mx        '),         trim('sm_sc.fv_lower_tri_mx            '),            '1_p' union all
  select trim('04_upper_tri_mx        '),         trim('sm_sc.fv_upper_tri_mx            '),            '1_p' union all
  select trim('04_lower_tri_mx_ex     '),         trim('sm_sc.fv_lower_tri_mx_ex         '),            '1_p' union all
  select trim('04_upper_tri_mx_ex     '),         trim('sm_sc.fv_upper_tri_mx_ex         '),            '1_p' union all
  select trim('04_lmask               '),         trim('sm_sc.fv_lmask                   '),            '2_p' union all
  select trim('04_rmask               '),         trim('sm_sc.fv_rmask                   '),            '2_p' union all
  select trim('04_amask               '),         trim('sm_sc.fv_amask                   '),            '2_p' union all
  select trim('04_bmask               '),         trim('sm_sc.fv_bmask                   '),            '2_p' union all
  select trim('05_pool_max_2d_grp_x   '),         trim('sm_sc.fv_pool_max_2d_grp_x       '),            '1_p' union all
  select trim('05_pool_avg_2d_grp_x   '),         trim('sm_sc.fv_pool_avg_2d_grp_x       '),            '1_p' union all
  select trim('05_pool_max            '),         trim('sm_sc.fv_pool_max_py             '),            '1_p' union all
  select trim('05_pool_avg            '),         trim('sm_sc.fv_pool_avg_py             '),            '1_p' union all
  select trim('05_pool_none           '),         trim('sm_sc.fv_pool_none               '),            '1_p' union all
  select trim('05_conv_2d_grp_x       '),         trim('sm_sc.fv_conv_2d_grp_x           '),            '2_p' union all
  select trim('05_conv_2d             '),         trim('sm_sc.fv_conv_2d_im2col_py        '),            '3_p' union all   -- '3_p' 兼容 '2_p', 此时第三目传 null
  select trim('05_conv_add            '),         trim('sm_sc.fv_conv_add                '),            '2_p' union all
  select trim('05_conv_sub            '),         trim('sm_sc.fv_conv_sub                '),            '2_p' union all
  select trim('05_conv_mul            '),         trim('sm_sc.fv_conv_mul                '),            '2_p' union all
  select trim('05_conv_div            '),         trim('sm_sc.fv_conv_div                '),            '2_p' union all
  select trim('05_conv_pow            '),         trim('sm_sc.fv_conv_pow                '),            '2_p' union all
  select trim('05_conv_log            '),         trim('sm_sc.fv_conv_log                '),            '2_p' union all
  select trim('05_conv_prod_mx        '),         trim('sm_sc.fv_conv_prod_mx            '),            '2_p' union all
  select trim('05_conv_de_sub         '),         trim('sm_sc.fv_conv_de_sub             '),            '2_p' union all
  select trim('05_conv_de_div         '),         trim('sm_sc.fv_conv_de_div             '),            '2_p' union all
  select trim('05_conv_de_pow         '),         trim('sm_sc.fv_conv_de_pow             '),            '2_p' union all
  select trim('05_conv_de_log         '),         trim('sm_sc.fv_conv_de_log             '),            '2_p' union all
  select trim('05_conv_de_prod_mx     '),         trim('sm_sc.fv_conv_de_prod_mx         '),            '2_p' union all
  select trim('06_aggr_mx_sum         '),         trim('sm_sc.fa_mx_sum                  '),            'n_p' union all
  select trim('06_aggr_mx_prod        '),         trim('sm_sc.fa_mx_prod                 '),            'n_p' union all
  select trim('06_aggr_mx_avg         '),         trim('sm_sc.fa_mx_avg                  '),            'n_p' union all
  select trim('06_aggr_mx_max         '),         trim('sm_sc.fa_mx_max                  '),            'n_p' union all
  select trim('06_aggr_mx_min         '),         trim('sm_sc.fa_mx_min                  '),            'n_p' union all
  select trim('06_aggr_mx_concat_y    '),         trim('sm_sc.fa_mx_concat_y             '),            'n_p' union all
  select trim('06_aggr_mx_concat_x    '),         trim('sm_sc.fa_mx_concat_x             '),            'n_p' union all
  select trim('06_aggr_mx_concat_x3   '),         trim('sm_sc.fa_mx_concat_x3            '),            'n_p' union all
  select trim('06_aggr_mx_concat_x4   '),         trim('sm_sc.fa_mx_concat_x4            '),            'n_p' union all
  select trim('07_aggr_slice_sum      '),         trim('sm_sc.fv_aggr_slice_sum          '),            '1_p' union all
  select trim('07_aggr_slice_avg      '),         trim('sm_sc.fv_aggr_slice_avg          '),            '1_p' union all
  select trim('07_aggr_slice_max      '),         trim('sm_sc.fv_aggr_slice_max          '),            '1_p' union all
  select trim('07_aggr_slice_min      '),         trim('sm_sc.fv_aggr_slice_min          '),            '1_p' union all
  select trim('07_aggr_slice_prod     '),         trim('sm_sc.fv_aggr_slice_prod         '),            '1_p' union all
  select trim('07_aggr_chunk_sum      '),         trim('sm_sc.fv_aggr_chunk_sum          '),            '1_p' union all
  select trim('07_aggr_chunk_avg      '),         trim('sm_sc.fv_aggr_chunk_avg          '),            '1_p' union all
  select trim('07_aggr_chunk_max      '),         trim('sm_sc.fv_aggr_chunk_max          '),            '1_p' union all
  select trim('07_aggr_chunk_min      '),         trim('sm_sc.fv_aggr_chunk_min          '),            '1_p' union all
  select trim('07_aggr_chunk_prod     '),         trim('sm_sc.fv_aggr_chunk_prod         '),            '1_p'
)
select 
  'node_fn_type' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- -------------------------------------------------------------------------
-- node_fn_type_delta  -- 算子求导函数
-- delete from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type_delta(enum_key, enum_value, enum_group) as
(
  -- select trim('dddi_full_dataset        '), trim('                                                     '),trim('00_full_dataset        ') union all
  -- select trim('dddi_const               '), trim('                                                     '),trim('00_const               ') union all
  -- select trim('dddi_buff_slice_rand_pick'), trim('                                                     '),trim('00_buff_slice_rand_pick') union all
     select trim('dldi_none                '), trim('sm_sc.fv_d_nn_none_dloss_dindepdt                    '),trim('00_none                ') union all
     select trim('dddi_add                 '), trim('sm_sc.fv_d_add                                       '),trim('01_add                 ') union all
     select trim('dddi_mul                 '), trim('sm_sc.fv_d_mul                                       '),trim('01_mul                 ') union all
     select trim('dddi_sub_1               '), trim('sm_sc.fv_d_sub_1                                     '),trim('01_sub                 ') union all
     select trim('dddi_sub_2               '), trim('sm_sc.fv_d_sub_2                                     '),trim('01_sub                 ') union all
     select trim('dddi_0sub_1              '), trim('sm_sc.fv_d_sub_2                                     '),trim('01_0sub                ') union all
     select trim('dddi_div_1               '), trim('sm_sc.fv_d_div_1                                     '),trim('01_div                 ') union all
     select trim('dddi_div_2               '), trim('sm_sc.fv_d_div_2                                     '),trim('01_div                 ') union all
     select trim('dddi_1div_1              '), trim('sm_sc.fv_d_div_2                                     '),trim('01_1div                ') union all
     select trim('dddi_pow_1               '), trim('sm_sc.fv_d_pow_1                                     '),trim('01_pow                 ') union all
     select trim('dddi_pow_2               '), trim('sm_sc.fv_d_pow_2                                     '),trim('01_pow                 ') union all
     select trim('dddi_log_1               '), trim('sm_sc.fv_d_log_1                                     '),trim('01_log                 ') union all
     select trim('dddi_log_2               '), trim('sm_sc.fv_d_log_2                                     '),trim('01_log                 ') union all
     select trim('dddi_exp                 '), trim('sm_sc.fv_d_exp                                       '),trim('01_exp                 ') union all
     select trim('dddi_ln                  '), trim('sm_sc.fv_d_ln                                        '),trim('01_ln                  ') union all
     select trim('dddi_abs                 '), trim('sm_sc.fv_d_abs                                       '),trim('01_1div                ') union all
     select trim('dddi_prod_mx_1           '), trim('sm_sc.fv_d_prod_mx_1                                 '),trim('01_prod_mx             ') union all
     select trim('dddi_prod_mx_2           '), trim('sm_sc.fv_d_prod_mx_2                                 '),trim('01_prod_mx             ') union all
     select trim('dldi_prod_mx_1           '), trim('sm_sc.fv_d_prod_mx_dloss_dindepdt_1                  '),trim('01_prod_mx             ') union all
     select trim('dldi_prod_mx_2           '), trim('sm_sc.fv_d_prod_mx_dloss_dindepdt_2                  '),trim('01_prod_mx             ') union all
     select trim('dddi_chunk_prod_mx_1     '), trim('sm_sc.fv_d_chunk_prod_mx_1                           '),trim('01_chunk_prod_mx       ') union all
     select trim('dddi_chunk_prod_mx_2     '), trim('sm_sc.fv_d_chunk_prod_mx_2                           '),trim('01_chunk_prod_mx       ') union all
     select trim('dldi_chunk_prod_mx_1     '), trim('sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_1            '),trim('01_chunk_prod_mx       ') union all
     select trim('dldi_chunk_prod_mx_2     '), trim('sm_sc.fv_d_chunk_prod_mx_dloss_dindepdt_2            '),trim('01_chunk_prod_mx       ') union all
     select trim('dddi_sin                 '), trim('sm_sc.fv_d_sin                                       '),trim('02_sin                 ') union all
     select trim('dddi_cos                 '), trim('sm_sc.fv_d_cos                                       '),trim('02_cos                 ') union all
     select trim('dddi_tan                 '), trim('sm_sc.fv_d_tan                                       '),trim('02_tan                 ') union all
     select trim('dddi_cot                 '), trim('sm_sc.fv_d_cot                                       '),trim('02_cot                 ') union all
     select trim('dddi_sec                 '), trim('sm_sc.fv_d_sec                                       '),trim('02_sec                 ') union all
     select trim('dddi_csc                 '), trim('sm_sc.fv_d_csc                                       '),trim('02_csc                 ') union all
     select trim('dddi_asin                '), trim('sm_sc.fv_d_asin                                      '),trim('02_asin                ') union all
     select trim('dddi_acos                '), trim('sm_sc.fv_d_acos                                      '),trim('02_acos                ') union all
     select trim('dddi_atan                '), trim('sm_sc.fv_d_atan                                      '),trim('02_atan                ') union all
     select trim('dddi_acot                '), trim('sm_sc.fv_d_acot                                      '),trim('02_acot                ') union all
     select trim('dddi_sinh                '), trim('sm_sc.fv_d_sinh                                      '),trim('02_sinh                ') union all
     select trim('dddi_cosh                '), trim('sm_sc.fv_d_cosh                                      '),trim('02_cosh                ') union all
     select trim('dddi_tanh                '), trim('sm_sc.fv_d_tanh                                      '),trim('02_tanh                ') union all
     select trim('dddi_asinh               '), trim('sm_sc.fv_d_asinh                                     '),trim('02_asinh               ') union all
     select trim('dddi_acosh               '), trim('sm_sc.fv_d_acosh                                     '),trim('02_acosh               ') union all
     select trim('dddi_atanh               '), trim('sm_sc.fv_d_atanh                                     '),trim('02_atanh               ') union all
     select trim('dddi_sigmoid             '), trim('sm_sc.fv_d_activate_sigmoid                          '),trim('03_sigmoid             ') union all
     select trim('dddi_relu                '), trim('sm_sc.fv_d_activate_relu                             '),trim('03_relu                ') union all
     select trim('dddi_leaky_relu          '), trim('sm_sc.fv_d_activate_leaky_relu                       '),trim('03_leaky_relu          ') union all
     select trim('dddi_elu                 '), trim('sm_sc.fv_d_activate_elu                              '),trim('03_elu                 ') union all
     select trim('dddi_selu                '), trim('sm_sc.fv_d_activate_selu                             '),trim('03_selu                ') union all
     select trim('dddi_gelu                '), trim('sm_sc.fv_d_activate_gelu                             '),trim('03_gelu                ') union all
     select trim('dddi_swish               '), trim('sm_sc.fv_d_activate_swish                            '),trim('03_swish               ') union all
     select trim('dddi_softplus            '), trim('sm_sc.fv_d_activate_softplus                         '),trim('03_softplus            ') union all
     select trim('dddi_boxcox              '), trim('sm_sc.fv_d_activate_boxcox                           '),trim('03_boxcox              ') union all
     select trim('dldi_softmax             '), trim('sm_sc.fv_d_redistr_softmax_dloss_dindepdt            '),trim('03_softmax             ') union all
     select trim('dldi_softmax_ex          '), trim('sm_sc.fv_d_redistr_softmax_dloss_dindepdt            '),trim('03_softmax_ex          ') union all
     select trim('dddi_zscore              '), trim('sm_sc.fv_d_redistr_zscore                            '),trim('03_zscore              ') union all
     select trim('dldi_new                 '), trim('sm_sc.fv_d_new_dloss_dindepdt                        '),trim('04_new                 ') union all
     select trim('dldi_repeat_axis         '), trim('sm_sc.fv_d_repeat_axis_dloss_dindepdt                '),trim('04_repeat_axis         ') union all
     select trim('dldi_apad_1              '), trim('sm_sc.fv_d_apad_dloss_dindepdt_1                     '),trim('04_apad                ') union all
     select trim('dldi_apad_2              '), trim('sm_sc.fv_d_apad_dloss_dindepdt_2                     '),trim('04_apad                ') union all
     select trim('dldi_bpad_1              '), trim('sm_sc.fv_d_bpad_dloss_dindepdt_1                     '),trim('04_bpad                ') union all
     select trim('dldi_bpad_2              '), trim('sm_sc.fv_d_bpad_dloss_dindepdt_2                     '),trim('04_bpad                ') union all
     select trim('dldi_lpad_1              '), trim('sm_sc.fv_d_lpad_dloss_dindepdt_1                     '),trim('04_lpad                ') union all
     select trim('dldi_lpad_2              '), trim('sm_sc.fv_d_lpad_dloss_dindepdt_2                     '),trim('04_lpad                ') union all
     select trim('dldi_rpad_1              '), trim('sm_sc.fv_d_rpad_dloss_dindepdt_1                     '),trim('04_rpad                ') union all
     select trim('dldi_rpad_2              '), trim('sm_sc.fv_d_rpad_dloss_dindepdt_2                     '),trim('04_rpad                ') union all
     select trim('dldi_transpose           '), trim('sm_sc.fv_d_transpose_dloss_dindepdt                  '),trim('04_transpose           ') union all
     select trim('dldi_transpose_i         '), trim('sm_sc.fv_d_transpose_i_dloss_dindepdt                '),trim('04_transpose_i         ') union all
     select trim('dldi_chunk_transpose_i   '), trim('sm_sc.fv_d_chunk_transpose_dloss_dindepdt            '),trim('04_chunk_transpose     ') union all
     select trim('dldi_transpose_nd        '), trim('sm_sc.fv_d_transpose_nd_dloss_dindepdt               '),trim('04_transpose_nd        ') union all
     select trim('dldi_turn_90             '), trim('sm_sc.fv_d_turn_90_dloss_dindepdt                    '),trim('04_turn_90             ') union all
     select trim('dldi_turn_180            '), trim('sm_sc.fv_d_turn_180_dloss_dindepdt                   '),trim('04_turn_180            ') union all
     select trim('dldi_mirror              '), trim('sm_sc.fv_d_mirror_dloss_dindepdt                     '),trim('04_mirror              ') union all
     select trim('dldi_mx_ele_3d_2_2d      '), trim('sm_sc.fv_d_mx_ele_3d_2_2d_dloss_dindepdt             '),trim('04_mx_ele_3d_2_2d      ') union all
     select trim('dldi_mx_ele_2d_2_3d      '), trim('sm_sc.fv_d_mx_ele_2d_2_3d_dloss_dindepdt             '),trim('04_mx_ele_2d_2_3d      ') union all
     select trim('dldi_mx_ele_4d_2_3d      '), trim('sm_sc.fv_d_mx_ele_4d_2_3d_dloss_dindepdt             '),trim('04_mx_ele_4d_2_3d      ') union all
     select trim('dldi_mx_ele_3d_2_4d      '), trim('sm_sc.fv_d_mx_ele_3d_2_4d_dloss_dindepdt             '),trim('04_mx_ele_3d_2_4d      ') union all
     select trim('dldi_mx_ele_flatten_2dims'), trim('sm_sc.fv_d_mx_ele_flatten_2dims_dloss_dindepdt       '),trim('04_mx_ele_flatten_2dims') union all
     select trim('dldi_mx_slice_3d_2_2d    '), trim('sm_sc.fv_d_mx_slice_3d_2_2d_dloss_dindepdt           '),trim('04_mx_slice_3d_2_2d    ') union all
     select trim('dldi_mx_slice_4d_2_2d    '), trim('sm_sc.fv_d_mx_slice_4d_2_2d_dloss_dindepdt           '),trim('04_mx_slice_4d_2_2d    ') union all
     select trim('dldi_mx_slice_4d_2_3d    '), trim('sm_sc.fv_d_mx_slice_4d_2_3d_dloss_dindepdt           '),trim('04_mx_slice_4d_2_3d    ') union all
     select trim('dldi_mx_ascend_dim       '), trim('sm_sc.fv_d_mx_ascend_dim_dloss_dindepdt              '),trim('04_mx_ascend_dim       ') union all
     select trim('dldi_mx_descend_dim      '), trim('sm_sc.fv_d_mx_descend_dim_dloss_dindepdt             '),trim('04_mx_descend_dim      ') union all
  -- select trim('dldi_rand_pick_y         '), trim('                                                     '),trim('04_rand_pick_y         ') union all
  -- select trim('dldi_rand_pick_x         '), trim('                                                     '),trim('04_rand_pick_x         ') union all
  -- select trim('dldi_rand_pick_x3        '), trim('                                                     '),trim('04_rand_pick_x3        ') union all
  -- select trim('dldi_rand_pick_x4        '), trim('                                                     '),trim('04_rand_pick_x4        ') union all
     select trim('dldi_chunk               '), trim('sm_sc.fv_d_chunk_dloss_dindepdt                      '),trim('04_chunk               ') union all
     select trim('dldi_slice_y             '), trim('sm_sc.fv_d_slice_y_dloss_dindepdt                    '),trim('04_slice_y             ') union all
     select trim('dldi_slice_x             '), trim('sm_sc.fv_d_slice_x_dloss_dindepdt                    '),trim('04_slice_x             ') union all
     select trim('dldi_slice_x3            '), trim('sm_sc.fv_d_slice_x3_dloss_dindepdt                   '),trim('04_slice_x3            ') union all
     select trim('dldi_slice_x4            '), trim('sm_sc.fv_d_slice_x4_dloss_dindepdt                   '),trim('04_slice_x4            ') union all
     select trim('dldi_sample_y            '), trim('sm_sc.fv_d_sample_y_dloss_dindepdt_1                 '),trim('04_sample_y            ') union all
     select trim('dldi_sample_x            '), trim('sm_sc.fv_d_sample_x_dloss_dindepdt_1                 '),trim('04_sample_x            ') union all
     select trim('dldi_sample_x3           '), trim('sm_sc.fv_d_sample_x3_dloss_dindepdt_1                '),trim('04_sample_x3           ') union all
     select trim('dldi_sample_x4           '), trim('sm_sc.fv_d_sample_x4_dloss_dindepdt_1                '),trim('04_sample_x4           ') union all
     select trim('dldi_lower_tri_mx        '), trim('sm_sc.fv_d_lower_tri_mx_dloss_dindepdt               '),trim('04_lower_tri_mx        ') union all
     select trim('dldi_upper_tri_mx        '), trim('sm_sc.fv_d_upper_tri_mx_dloss_dindepdt               '),trim('04_upper_tri_mx        ') union all
     select trim('dldi_lower_tri_mx_ex     '), trim('sm_sc.fv_d_lower_tri_mx_ex_dloss_dindepdt            '),trim('04_lower_tri_mx_ex     ') union all
     select trim('dldi_upper_tri_mx_ex     '), trim('sm_sc.fv_d_upper_tri_mx_ex_dloss_dindepdt            '),trim('04_upper_tri_mx_ex     ') union all
     select trim('dldi_lmask               '), trim('sm_sc.fv_d_lmask_dloss_dindepdt_1                    '),trim('04_lmask               ') union all
     select trim('dldi_rmask               '), trim('sm_sc.fv_d_rmask_dloss_dindepdt_1                    '),trim('04_rmask               ') union all
     select trim('dldi_amask               '), trim('sm_sc.fv_d_amask_dloss_dindepdt_1                    '),trim('04_amask               ') union all
     select trim('dldi_bmask               '), trim('sm_sc.fv_d_bmask_dloss_dindepdt_1                    '),trim('04_bmask               ') union all
     select trim('dldi_pool_max_grp_x      '), trim('sm_sc.fv_d_pool_max_grp_x_dloss_dindepdt             '),trim('05_pool_max_2d_grp_x   ') union all
     select trim('dldi_pool_avg_grp_x      '), trim('sm_sc.fv_d_pool_avg_grp_x_dloss_dindepdt             '),trim('05_pool_avg_2d_grp_x   ') union all
     select trim('dldi_pool_max            '), trim('sm_sc.fv_d_pool_max_dloss_dindepdt                   '),trim('05_pool_max            ') union all
     select trim('dldi_pool_avg            '), trim('sm_sc.fv_d_pool_avg_dloss_dindepdt_ex                '),trim('05_pool_avg            ') union all
     select trim('dldi_pool_none           '), trim('sm_sc.fv_d_pool_none_dloss_dindepdt                  '),trim('05_pool_none           ') union all
     select trim('dldi_conv_2d_grp_x_1     '), trim('sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_1            '),trim('05_conv_2d_grp_x       ') union all
     select trim('dldi_conv_2d_grp_x_2     '), trim('sm_sc.fv_d_conv_2d_grp_x_dloss_dindepdt_2            '),trim('05_conv_2d_grp_x       ') union all
     select trim('dldi_conv_2d_1           '), trim('sm_sc.fv_d_conv_2d_dloss_dindepdt_1_ex               '),trim('05_conv_2d             ') union all
     select trim('dldi_conv_2d_2           '), trim('sm_sc.fv_d_conv_2d_dloss_dindepdt_2_py               '),trim('05_conv_2d             ') union all
     select trim('dldi_conv_2d_3           '), trim('sm_sc.fv_d_conv_2d_dloss_dindepdt_3                  '),trim('05_conv_2d             ') union all
     select trim('dldi_conv_add_1          '), trim('sm_sc.fv_d_conv_add_dloss_dindepdt_1                 '),trim('05_conv_add            ') union all
     select trim('dldi_conv_add_2          '), trim('sm_sc.fv_d_conv_add_dloss_dindepdt_2                 '),trim('05_conv_add            ') union all
     select trim('dldi_conv_sub_1          '), trim('sm_sc.fv_d_conv_sub_dloss_dindepdt_1                 '),trim('05_conv_sub            ') union all
     select trim('dldi_conv_sub_2          '), trim('sm_sc.fv_d_conv_sub_dloss_dindepdt_2                 '),trim('05_conv_sub            ') union all
     select trim('dldi_conv_mul_1          '), trim('sm_sc.fv_d_conv_mul_dloss_dindepdt_1                 '),trim('05_conv_mul            ') union all
     select trim('dldi_conv_mul_2          '), trim('sm_sc.fv_d_conv_mul_dloss_dindepdt_2                 '),trim('05_conv_mul            ') union all
     select trim('dldi_conv_div_1          '), trim('sm_sc.fv_d_conv_div_dloss_dindepdt_1                 '),trim('05_conv_div            ') union all
     select trim('dldi_conv_div_2          '), trim('sm_sc.fv_d_conv_div_dloss_dindepdt_2                 '),trim('05_conv_div            ') union all
     select trim('dldi_conv_pow_1          '), trim('sm_sc.fv_d_conv_pow_dloss_dindepdt_1                 '),trim('05_conv_pow            ') union all
     select trim('dldi_conv_pow_2          '), trim('sm_sc.fv_d_conv_pow_dloss_dindepdt_2                 '),trim('05_conv_pow            ') union all
     select trim('dldi_conv_log_1          '), trim('sm_sc.fv_d_conv_log_dloss_dindepdt_1                 '),trim('05_conv_log            ') union all
     select trim('dldi_conv_log_2          '), trim('sm_sc.fv_d_conv_log_dloss_dindepdt_2                 '),trim('05_conv_log            ') union all
     select trim('dldi_conv_prod_mx_1      '), trim('sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_1             '),trim('05_conv_prod_mx        ') union all
     select trim('dldi_conv_prod_mx_2      '), trim('sm_sc.fv_d_conv_prod_mx_dloss_dindepdt_2             '),trim('05_conv_prod_mx        ') union all
     select trim('dldi_conv_de_sub_1       '), trim('sm_sc.fv_d_conv_de_sub_dloss_dindepdt_1              '),trim('05_conv_de_sub         ') union all
     select trim('dldi_conv_de_sub_2       '), trim('sm_sc.fv_d_conv_de_sub_dloss_dindepdt_2              '),trim('05_conv_de_sub         ') union all
     select trim('dldi_conv_de_div_1       '), trim('sm_sc.fv_d_conv_de_div_dloss_dindepdt_1              '),trim('05_conv_de_div         ') union all
     select trim('dldi_conv_de_div_2       '), trim('sm_sc.fv_d_conv_de_div_dloss_dindepdt_2              '),trim('05_conv_de_div         ') union all
     select trim('dldi_conv_de_pow_1       '), trim('sm_sc.fv_d_conv_de_pow_dloss_dindepdt_1              '),trim('05_conv_de_pow         ') union all
     select trim('dldi_conv_de_pow_2       '), trim('sm_sc.fv_d_conv_de_pow_dloss_dindepdt_2              '),trim('05_conv_de_pow         ') union all
     select trim('dldi_conv_de_log_1       '), trim('sm_sc.fv_d_conv_de_log_dloss_dindepdt_1              '),trim('05_conv_de_log         ') union all
     select trim('dldi_conv_de_log_2       '), trim('sm_sc.fv_d_conv_de_log_dloss_dindepdt_2              '),trim('05_conv_de_log         ') union all
     select trim('dldi_conv_de_prod_mx_1   '), trim('sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_1'          ),trim('05_conv_de_prod_mx     ') union all
     select trim('dldi_conv_de_prod_mx_2   '), trim('sm_sc.fv_d_conv_de_prod_mx_dloss_dindepdt_2          '),trim('05_conv_de_prod_mx     ') union all
     select trim('dddi_aggr_mx_sum         '), trim('sm_sc.fv_d_mx_sum                                    '),trim('06_aggr_mx_sum         ') union all
     select trim('dddi_aggr_mx_prod        '), trim('sm_sc.fv_d_mx_prod                                   '),trim('06_aggr_mx_prod        ') union all
     select trim('dddi_aggr_mx_avg         '), trim('sm_sc.fv_d_mx_avg                                    '),trim('06_aggr_mx_avg         ') union all
     select trim('dddi_aggr_mx_max         '), trim('sm_sc.fv_d_mx_max                                    '),trim('06_aggr_mx_max         ') union all
     select trim('dddi_aggr_mx_min         '), trim('sm_sc.fv_d_mx_min                                    '),trim('06_aggr_mx_min         ') union all
     select trim('dldi_aggr_mx_concat_y    '), trim('sm_sc.fv_d_mx_concat_y_dloss_dindepdt_n              '),trim('06_aggr_mx_concat_y    ') union all
     select trim('dldi_aggr_mx_concat_x    '), trim('sm_sc.fv_d_mx_concat_x_dloss_dindepdt_n              '),trim('06_aggr_mx_concat_x    ') union all
     select trim('dldi_aggr_mx_concat_x3   '), trim('sm_sc.fv_d_mx_concat_x3_dloss_dindepdt_n             '),trim('06_aggr_mx_concat_x3   ') union all
     select trim('dldi_aggr_mx_concat_x4   '), trim('sm_sc.fv_d_mx_concat_x4_dloss_dindepdt_n             '),trim('06_aggr_mx_concat_x4   ') union all
     select trim('dldi_aggr_slice_sum      '), trim('sm_sc.fv_d_aggr_slice_sum_dloss_dindepdt             '),trim('07_aggr_slice_sum      ') union all
     select trim('dldi_aggr_slice_avg      '), trim('sm_sc.fv_d_aggr_slice_avg_dloss_dindepdt             '),trim('07_aggr_slice_avg      ') union all
     select trim('dldi_aggr_slice_max      '), trim('sm_sc.fv_d_aggr_slice_max_dloss_dindepdt             '),trim('07_aggr_slice_max      ') union all
     select trim('dldi_aggr_slice_min      '), trim('sm_sc.fv_d_aggr_slice_min_dloss_dindepdt             '),trim('07_aggr_slice_min      ') union all
     select trim('dldi_aggr_slice_prod     '), trim('sm_sc.fv_d_aggr_slice_prod_dloss_dindepdt            '),trim('07_aggr_slice_prod     ') union all
     select trim('dldi_aggr_chunk_sum      '), trim('sm_sc.fv_d_aggr_chunk_sum_dloss_dindepdt             '),trim('07_aggr_chunk_sum      ') union all
     select trim('dldi_aggr_chunk_avg      '), trim('sm_sc.fv_d_aggr_chunk_avg_dloss_dindepdt             '),trim('07_aggr_chunk_avg      ') union all
     select trim('dldi_aggr_chunk_max      '), trim('sm_sc.fv_d_aggr_chunk_max_dloss_dindepdt             '),trim('07_aggr_chunk_max      ') union all
     select trim('dldi_aggr_chunk_min      '), trim('sm_sc.fv_d_aggr_chunk_min_dloss_dindepdt             '),trim('07_aggr_chunk_min      ') union all
     select trim('dldi_aggr_chunk_prod     '), trim('sm_sc.fv_d_aggr_chunk_prod_dloss_dindepdt            '),trim('07_aggr_chunk_prod     ')
)
select 
  'node_fn_type_delta' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta'), 0) 
  as enum_order
from cte_node_fn_type_delta
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- -------------------------------------------------------------------------
-- node_fn_asso_value  -- 算子超参数
-- delete from sm_sc.tb_dic_enum where enum_name = 'node_fn_asso_value';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order     ,
  enum_range
)
with cte_node_fn_asso_value(a_param_name, a_cn_desc, enum_group, enum_order, enum_range) as
(
     select trim('dataset_len__1d              '), trim('训练集规格                                       '), trim('00_full_dataset        '),  1, numrange(1.0, 4.0, '[]') union all
     select trim('dataset_len__nd              '), trim('常数节点的各维度长度规格                         '), trim('00_const               '),  1, numrange(1.0, null, '[]') union all
     select trim('slice_range_lowers__2d       '), trim('若干切片范围下界                                 '), trim('00_buff_slice_rand_pick'),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_range_uppers__2d       '), trim('若干切片范围上界                                 '), trim('00_buff_slice_rand_pick'),  2, numrange(2.0, 2.0, '[]') union all
     select trim('rand_pick_cnts__2d           '), trim('在对应切片范围上随机选取训练集数量               '), trim('00_buff_slice_rand_pick'),  3, numrange(3.0, 3.0, '[]') union all
     select trim('p1_heigh__1d                 '), trim('入参一高度规格，非必要配置                       '), trim('01_prod_mx             '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('p1_width_p2_heigh__1d        '), trim('入参一宽度规格，也即入参二高度规格               '), trim('01_prod_mx             '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('p2_width__1d                 '), trim('入参二宽度规格                                   '), trim('01_prod_mx             '),  3, numrange(3.0, 3.0, '[]') union all
  -- select trim('p2_3d4d_len_y__1d            '), trim('高维广播入参二的第一维长度                       '), trim('01_prod_mx             '),  4, numrange(4.0, 4.0, '[]') union all
  -- select trim('p2_3d4d_len_x__1d            '), trim('高维广播入参二的第二维长度                       '), trim('01_prod_mx             '),  5, numrange(5.0, 5.0, '[]') union all
     select trim('p1_chunk_heigh__1d           '), trim('矩阵乘法 chunk 入参一高度规格                    '), trim('01_chunk_prod_mx       '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('p1_chunk_width_p2_heigh__1d  '), trim('矩阵乘法 chunk 入参一宽度规格，也即入参二高度规格'), trim('01_chunk_prod_mx       '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('p2_chunk_width__1d           '), trim('矩阵乘法 chunk 入参二宽度规格                    '), trim('01_chunk_prod_mx       '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('alpha__1d                    '), trim('leaky_relu 的超参数                              '), trim('03_leaky_relu          '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('alpha__1d                    '), trim('elu 的超参数                                     '), trim('03_elu                 '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('lambda__1d                   '), trim('boxcox 的超参数                                  '), trim('03_boxcox              '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('03_softmax             '),  1, numrange(1.0, 4.0, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('03_softmax_ex          '),  1, numrange(1.0, 4.0, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('03_zscore              '),  1, numrange(1.0, 4.0, '[]') union all
     select trim('dims_times__1d               '), trim('各维度重复次数                                   '), trim('04_new                 '),  1, numrange(1.0, 4.0, '[]') union all
     select trim('dims__2d                     '), trim('执行复制操作的若干维轴                           '), trim('04_repeat_axis         '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('repeat__2d                   '), trim('各维轴重复次数                                   '), trim('04_repeat_axis         '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('times__1d                    '), trim('填充重复次数                                     '), trim('04_apad                '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('times__1d                    '), trim('填充重复次数                                     '), trim('04_bpad                '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('times__1d                    '), trim('填充重复次数                                     '), trim('04_lpad                '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('times__1d                    '), trim('填充重复次数                                     '), trim('04_rpad                '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('chunk_len__1d                '), trim('块儿高宽规格                                     '), trim('04_chunk_transpose     '),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dims__1d                     '), trim('转置的两个维度                                   '), trim('04_transpose_nd        '),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dims_from_to__1d             '), trim('旋转的正方向起止两个维度                         '), trim('04_turn_90             '),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dims__1d                     '), trim('旋转的两个维度                                   '), trim('04_turn_180            '),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dim__1d                      '), trim('镜像维度，即镜面法面的维度                       '), trim('04_mirror              '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('dims_from_to__1d             '), trim('合并的两个维度                                   '), trim('04_mx_ele_3d_2_2d      '),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dim_pin_ele__1d              '), trim('定住元素顺序的维度                               '), trim('04_mx_ele_3d_2_2d      '),  2, numrange(3.0, 3.0, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('拆分分组元素个数                                 '), trim('04_mx_ele_2d_2_3d      '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('dims_from__1d                '), trim('被拆分维度                                       '), trim('04_mx_ele_2d_2_3d      '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('dims_new__1d                 '), trim('新生维度                                         '), trim('04_mx_ele_2d_2_3d      '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('dim_pin__1d                  '), trim('是否在被拆分维度保留原来的元素顺序               '), trim('04_mx_ele_2d_2_3d      '),  4, numrange(4.0, 4.0, '[]') union all
     select trim('dims_from_to__1d             '), trim('合并的两个维度                                   '), trim('04_mx_ele_4d_2_3d      '),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dim_pin_ele__1d              '), trim('定住元素顺序的维度                               '), trim('04_mx_ele_4d_2_3d      '),  2, numrange(3.0, 3.0, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('拆分分组元素个数                                 '), trim('04_mx_ele_3d_2_4d      '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('dims_from__1d                '), trim('被拆分维度                                       '), trim('04_mx_ele_3d_2_4d      '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('dims_new__1d                 '), trim('新生维度                                         '), trim('04_mx_ele_3d_2_4d      '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('dim_pin__1d                  '), trim('是否在被拆分维度保留原来的元素顺序               '), trim('04_mx_ele_3d_2_4d      '),  4, numrange(4.0, 4.0, '[]') union all
     select trim('dims_from_to__1d             '), trim('被扁平化的来源维度与跌落至维度                   '), trim('04_mx_ele_flatten_2dims'),  1, numrange(1.0, 2.0, '[]') union all
     select trim('dim_pin_ele__1d              '), trim('定住元素顺序的维度                               '), trim('04_mx_ele_flatten_2dims'),  3, numrange(3.0, 3.0, '[]') union all
     select trim('dim_sliced__1d               '), trim('切片维度                                         '), trim('04_mx_slice_3d_2_2d    '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_pos__1d                '), trim('切片位置序号                                     '), trim('04_mx_slice_3d_2_2d    '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('dim_sliced__2d               '), trim('切片维度(两个)                                   '), trim('04_mx_slice_4d_2_2d    '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_pos__2d                '), trim('切片位置序号(两个)                               '), trim('04_mx_slice_4d_2_2d    '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('dim_sliced__1d               '), trim('切片维度                                         '), trim('04_mx_slice_4d_2_3d    '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_pos__1d                '), trim('切片位置序号                                     '), trim('04_mx_slice_4d_2_3d    '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('ascend_time__1d              '), trim('加壳次数                                         '), trim('04_mx_ascend_dim       '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('descend_time__1d             '), trim('去壳次数                                         '), trim('04_mx_descend_dim      '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('pick_cnt__1d                 '), trim('随机拣取个数                                     '), trim('04_rand_pick_y         '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('pick_cnt__1d                 '), trim('随机拣取个数                                     '), trim('04_rand_pick_x         '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('pick_cnt__1d                 '), trim('随机拣取个数                                     '), trim('04_rand_pick_x3        '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('pick_cnt__1d                 '), trim('随机拣取个数                                     '), trim('04_rand_pick_x4        '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('chunk_range_lower__2d        '), trim('切块儿位置各维度下标序号下界                     '), trim('04_chunk               '),  1, numrange(1.0, null, '[]') union all
     select trim('chunk_range_upper__2d        '), trim('切块儿位置各维度下标序号上界                     '), trim('04_chunk               '),  2, numrange(1.0, null, '[]') union all
     select trim('slice_range_lower__2d        '), trim('切片位置下标序号下界                             '), trim('04_slice_y             '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_range_upper__2d        '), trim('切片位置下标序号上界，如果为空，则下界也是上界   '), trim('04_slice_y             '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('slice_range_lower__2d        '), trim('切片位置下标序号下界                             '), trim('04_slice_x             '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_range_upper__2d        '), trim('切片位置下标序号上界，如果为空，则下界也是上界   '), trim('04_slice_x             '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('slice_range_lower__2d        '), trim('切片位置下标序号下界                             '), trim('04_slice_x3            '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_range_upper__2d        '), trim('切片位置下标序号上界，如果为空，则下界也是上界   '), trim('04_slice_x3            '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('slice_range_lower__2d        '), trim('切片位置下标序号下界                             '), trim('04_slice_x4            '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('slice_range_upper__2d        '), trim('切片位置下标序号上界，如果为空，则下界也是上界   '), trim('04_slice_x4            '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('period__2d_1d                '), trim('采样周期                                         '), trim('04_sample_y            '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__2d_1d            '), trim('窗口宽度，null 表示不约束                        '), trim('04_sample_y            '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('simp_ranges_lowers__2d       '), trim('若干采样范围下界                                 '), trim('04_sample_y            '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('simp_ranges_uppers__2d       '), trim('若干采样范围上界                                 '), trim('04_sample_y            '),  4, numrange(4.0, 4.0, '[]') union all
     select trim('period__2d_1d                '), trim('采样周期                                         '), trim('04_sample_x            '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__2d_1d            '), trim('窗口宽度，null 表示不约束                        '), trim('04_sample_x            '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('simp_ranges_lowers__2d       '), trim('若干采样范围下界                                 '), trim('04_sample_x            '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('simp_ranges_uppers__2d       '), trim('若干采样范围上界                                 '), trim('04_sample_x            '),  4, numrange(4.0, 4.0, '[]') union all
     select trim('period__2d_1d                '), trim('采样周期                                         '), trim('04_sample_x3           '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__2d_1d            '), trim('窗口宽度，null 表示不约束                        '), trim('04_sample_x3           '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('simp_ranges_lowers__2d       '), trim('若干采样范围下界                                 '), trim('04_sample_x3           '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('simp_ranges_uppers__2d       '), trim('若干采样范围上界                                 '), trim('04_sample_x3           '),  4, numrange(4.0, 4.0, '[]') union all
     select trim('period__2d_1d                '), trim('采样周期                                         '), trim('04_sample_x4           '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__2d_1d            '), trim('窗口宽度，null 表示不约束                        '), trim('04_sample_x4           '),  2, numrange(2.0, 2.0, '[]') union all
     select trim('simp_ranges_lowers__2d       '), trim('若干采样范围下界                                 '), trim('04_sample_x4           '),  3, numrange(3.0, 3.0, '[]') union all
     select trim('simp_ranges_uppers__2d       '), trim('若干采样范围上界                                 '), trim('04_sample_x4           '),  4, numrange(4.0, 4.0, '[]') union all
     select trim('upper_fill_value__1d         '), trim('上三角填充值                                     '), trim('04_lower_tri_mx        '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('lower_fill_value__1d         '), trim('下三角填充值                                     '), trim('04_upper_tri_mx        '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('upper_fill_value_ex__1d      '), trim('欠上三角填充值                                   '), trim('04_lower_tri_mx_ex     '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('lower_fill_value_ex__1d      '), trim('欠下三角填充值                                   '), trim('04_upper_tri_mx_ex     '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('mask_element__1d             '), trim('遮盖填充值                                       '), trim('04_lmask               '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('mask_element__1d             '), trim('遮盖填充值                                       '), trim('04_rmask               '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('mask_element__1d             '), trim('遮盖填充值                                       '), trim('04_amask               '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('mask_element__1d             '), trim('遮盖填充值                                       '), trim('04_bmask               '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('1d_2_2d_cnt_per_grp__1d      '), trim('扁平化之前的 2d 宽度规格                         '), trim('05_pool_max_2d_grp_x   '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__1d               '), trim('滑动窗口高宽规格                                 '), trim('05_pool_max_2d_grp_x   '),  2, numrange(2.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_pool_max_2d_grp_x   '),  3, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_pool_max_2d_grp_x   '),  4, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_pool_max_2d_grp_x   '),  5, numrange(10.0, 10.0, '[]') union all
     select trim('1d_2_2d_cnt_per_grp__1d      '), trim('扁平化之前的 2d 宽度规格                         '), trim('05_pool_avg_2d_grp_x   '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__1d               '), trim('滑动窗口高宽规格                                 '), trim('05_pool_avg_2d_grp_x   '),  2, numrange(2.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_pool_avg_2d_grp_x   '),  3, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_pool_avg_2d_grp_x   '),  4, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_pool_avg_2d_grp_x   '),  5, numrange(10.0, 10.0, '[]') union all
     select trim('input_arr_len_x__1d          '), trim('扁平化之后的 1d 宽度规格                         '), trim('05_pool_avg_2d_grp_x   '),  6, numrange(11.0, 11.0, '[]') union all
     select trim('window_len__1d               '), trim('滑动窗口高宽规格                                 '), trim('05_pool_max            '),  1, numrange(2.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_pool_max            '),  2, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_pool_max            '),  3, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_pool_max            '),  4, numrange(10.0, 10.0, '[]') union all
     select trim('window_len__1d               '), trim('滑动窗口高宽规格                                 '), trim('05_pool_avg            '),  1, numrange(2.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_pool_avg            '),  2, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_pool_avg            '),  3, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_pool_avg            '),  4, numrange(10.0, 10.0, '[]') union all
     select trim('window_len__1d               '), trim('滑动窗口高宽规格                                 '), trim('05_pool_none           '),  1, numrange(2.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_pool_none           '),  2, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_pool_none           '),  3, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_pool_none           '),  4, numrange(10.0, 10.0, '[]') union all
     select trim('1d_2_2d_cnt_per_grp__1d      '), trim('扁平化之前的 2d 宽度规格                         '), trim('05_conv_2d_grp_x       '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('window_len__1d               '), trim('滑动窗口高宽规格                                 '), trim('05_conv_2d_grp_x       '),  2, numrange(2.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_2d_grp_x       '),  3, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_2d_grp_x       '),  4, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_2d_grp_x       '),  5, numrange(10.0, 10.0, '[]') union all
  -- select trim('window_bias_label__1d        '), trim('是否有卷积偏移量                                 '), trim('05_conv_2d_grp_x       '),  6, numrange(12.0, 12.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_2d             '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_2d             '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_2d             '),  3, numrange(10.0, 10.0, '[]') union all
  -- select trim('p2_3d4d_len_y__1d            '), trim('高维广播入参二的第一维长度                       '), trim('05_conv_2d             '),  4, numrange(11.0, 11.0, '[]') union all
  -- select trim('p2_3d4d_len_x__1d            '), trim('高维广播入参二的第二维长度                       '), trim('05_conv_2d             '),  5, numrange(12.0, 12.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_add            '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_add            '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_add            '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_sub            '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_sub            '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_sub            '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_mul            '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_mul            '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_mul            '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_div            '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_div            '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_div            '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_pow            '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_pow            '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_pow            '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_log            '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_log            '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_log            '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('window_len_heigh__1d         '), trim('滑动窗口高度规格                                 '), trim('05_conv_prod_mx        '),  1, numrange(2.0, 2.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_prod_mx        '),  2, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_prod_mx        '),  3, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_prod_mx        '),  4, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_de_sub         '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_de_sub         '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_de_sub         '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_de_div         '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_de_div         '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_de_div         '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_de_pow         '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_de_pow         '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_de_pow         '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_de_log         '),  1, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_de_log         '),  2, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_de_log         '),  3, numrange(10.0, 10.0, '[]') union all
     select trim('window_len_width__1d         '), trim('滑动窗口宽度规格                                 '), trim('05_conv_de_prod_mx     '),  1, numrange(3.0, 3.0, '[]') union all
     select trim('stride__1d                   '), trim('纵向和横向步长                                   '), trim('05_conv_de_prod_mx     '),  2, numrange(4.0, 5.0, '[]') union all
     select trim('padding__1d                  '), trim('上下左右补齐行数/列数                            '), trim('05_conv_de_prod_mx     '),  3, numrange(6.0, 9.0, '[]') union all
     select trim('padding_value__1d            '), trim('补齐填充元素值                                   '), trim('05_conv_de_prod_mx     '),  4, numrange(10.0, 10.0, '[]') union all
     select trim('mx_cnt__1d                   '), trim('聚合数量，也即 count                             '), trim('06_aggr_mx_avg         '),  1, numrange(1.0, 1.0, '[]') union all
     select trim('len_y__1d                    '), trim('聚合字段的 y 长度                                '), trim('06_aggr_mx_concat_y    '),  1, numrange(1.0, null, '[]') union all
     select trim('len_x__1d                    '), trim('聚合字段的 x 长度                                '), trim('06_aggr_mx_concat_x    '),  1, numrange(1.0, null, '[]') union all
     select trim('len_x3__1d                   '), trim('聚合字段的 x3 长度                               '), trim('06_aggr_mx_concat_x3   '),  1, numrange(1.0, null, '[]') union all
     select trim('len_x4__1d                   '), trim('聚合字段的 x4 长度                               '), trim('06_aggr_mx_concat_x4   '),  1, numrange(1.0, null, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('07_aggr_slice_sum      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('07_aggr_slice_avg      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('07_aggr_slice_max      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('07_aggr_slice_min      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('各维度每分组长度。维长可为 null，即缺省最大维长  '), trim('07_aggr_slice_prod     '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('切块各维度长度。维长可为 null，即缺省最大维长    '), trim('07_aggr_chunk_sum      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('切块各维度长度。维长可为 null，即缺省最大维长    '), trim('07_aggr_chunk_avg      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('切块各维度长度。维长可为 null，即缺省最大维长    '), trim('07_aggr_chunk_max      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('切块各维度长度。维长可为 null，即缺省最大维长    '), trim('07_aggr_chunk_min      '),  1, numrange(1, 4, '[]') union all
     select trim('cnt_per_grp__1d              '), trim('切块各维度长度。维长可为 null，即缺省最大维长    '), trim('07_aggr_chunk_prod     '),  1, numrange(1, 4, '[]')
)
select 
  'node_fn_asso_value' as enum_name, 
  enum_group || '_asso_' || enum_order as enum_key, 
  a_param_name || a_cn_desc as enum_value, 
  enum_group,
  enum_order,
  enum_range
from cte_node_fn_asso_value
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
, enum_range = excluded.enum_range
;

-- -------------------------------------------------------------------------
-- node_fn_type_delta_method   -- 算子求导方式
-- delete from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as 
-- enum_group: 
--   0. 前向传播后，求 ddepdt/dindepdt
--   1. 反向传播。直接求 dloss/dindepdt, 而不需显式求取 ddepdt/dindepdt
-- enum_value:
--   第一位控制：第一目求导是否需要自变量
--   第二位控制：第二目求导是否需要自变量
--   第三位控制：第一目求导是否需要另一自变量
--   第四位控制：第二目求导是否需要另一自变量
--   第五位控制：第一目求导是否需要因变量
--   第六位控制：第二目求导是否需要因变量
--   第七位控制：第一目求导是否用到自变量维长规格
--   第八位控制：第二目求导是否用到自变量维长规格
(
  -- select trim('00_full_dataset        '),         trim(''),            '' union all
  -- select trim('00_const               '),         trim(''),            '' union all
  -- select trim('00_buff_slice_rand_pick'),         trim(''),            '' union all
  select trim('00_none                '),         trim('00000010'),            '1'  union all
  select trim('01_add                 '),         trim('00001110'),            '0'  union all
  select trim('01_mul                 '),         trim('00110000'),            '0'  union all
  select trim('01_sub                 '),         trim('00001110'),            '0'  union all
  select trim('01_0sub                '),         trim('00001010'),            '0'  union all
  select trim('01_div                 '),         trim('01110000'),            '0'  union all
  select trim('01_1div                '),         trim('10000000'),            '0'  union all
  select trim('01_pow                 '),         trim('10111100'),            '0'  union all
  select trim('01_log                 '),         trim('11111000'),            '0'  union all
  select trim('01_exp                 '),         trim('00001000'),            '0'  union all
  select trim('01_ln                  '),         trim('10000000'),            '0'  union all
  select trim('01_abs                 '),         trim('10000000'),            '0'  union all
  select trim('01_prod_mx             '),         trim('00110011'),            '1'  union all
  select trim('01_chunk_prod_mx       '),         trim('00110011'),            '1'  union all
  select trim('02_sin                 '),         trim('10000000'),            '0'  union all
  select trim('02_cos                 '),         trim('10000000'),            '0'  union all
  select trim('02_tan                 '),         trim('10000000'),            '0'  union all
  select trim('02_cot                 '),         trim('10000000'),            '0'  union all
  select trim('02_sec                 '),         trim('10000000'),            '0'  union all
  select trim('02_csc                 '),         trim('10000000'),            '0'  union all
  select trim('02_asin                '),         trim('10000000'),            '0'  union all
  select trim('02_acos                '),         trim('10000000'),            '0'  union all
  select trim('02_atan                '),         trim('10000000'),            '0'  union all
  select trim('02_acot                '),         trim('10000000'),            '0'  union all
  select trim('02_sinh                '),         trim('10000000'),            '0'  union all
  select trim('02_cosh                '),         trim('10000000'),            '0'  union all
  select trim('02_tanh                '),         trim('00001000'),            '0'  union all
  -- -- select trim('02_sech                '),         trim('10001000'),            '0'  union all
  -- -- select trim('02_csch                '),         trim('10001000'),            '0'  union all
  select trim('02_asinh               '),         trim('10000000'),            '0'  union all
  select trim('02_acosh               '),         trim('10000000'),            '0'  union all
  select trim('02_atanh               '),         trim('10000000'),            '0'  union all
  select trim('03_sigmoid             '),         trim('00001000'),            '0'  union all
  select trim('03_relu                '),         trim('10000000'),            '0'  union all
  select trim('03_leaky_relu          '),         trim('10000000'),            '0'  union all
  select trim('03_elu                 '),         trim('10000000'),            '0'  union all
  select trim('03_selu                '),         trim('10000000'),            '0'  union all
  select trim('03_gelu                '),         trim('10000000'),            '0'  union all
  select trim('03_swish               '),         trim('10001000'),            '0'  union all
  select trim('03_softplus            '),         trim('10000000'),            '0'  union all
  select trim('03_boxcox              '),         trim('10000000'),            '0'  union all
  select trim('03_softmax             '),         trim('10001000'),            '1'  union all
  select trim('03_softmax_ex          '),         trim('10001000'),            '1'  union all
  select trim('03_zscore              '),         trim('10001000'),            '0'  union all
  select trim('04_new                 '),         trim('00000000'),            '1'  union all
  select trim('04_repeat_axis         '),         trim('00000000'),            '1'  union all
  select trim('04_apad                '),         trim('00000011'),            '1'  union all
  select trim('04_bpad                '),         trim('00000011'),            '1'  union all
  select trim('04_lpad                '),         trim('00000011'),            '1'  union all
  select trim('04_rpad                '),         trim('00000011'),            '1'  union all
  select trim('04_transpose           '),         trim('00000000'),            '1'  union all
  select trim('04_transpose_i         '),         trim('00000000'),            '1'  union all
  select trim('04_chunk_transpose     '),         trim('00000000'),            '1'  union all
  select trim('04_transpose_nd        '),         trim('00000000'),            '1'  union all
  select trim('04_turn_90             '),         trim('00000000'),            '1'  union all
  select trim('04_turn_180            '),         trim('00000000'),            '1'  union all
  select trim('04_mirror              '),         trim('00000000'),            '1'  union all
  select trim('04_mx_ele_3d_2_2d      '),         trim('00000010'),            '1'  union all
  select trim('04_mx_ele_2d_2_3d      '),         trim('00000000'),            '1'  union all
  select trim('04_mx_ele_4d_2_3d      '),         trim('00000010'),            '1'  union all
  select trim('04_mx_ele_3d_2_4d      '),         trim('00000000'),            '1'  union all
  select trim('04_mx_ele_flatten_2dims'),         trim('00000010'),            '1'  union all
  select trim('04_mx_slice_3d_2_2d    '),         trim('00000010'),            '1'  union all
  select trim('04_mx_slice_4d_2_2d    '),         trim('00000010'),            '1'  union all
  select trim('04_mx_slice_4d_2_3d    '),         trim('00000010'),            '1'  union all
  select trim('04_mx_ascend_dim       '),         trim('00000000'),            '1'  union all
  select trim('04_mx_descend_dim      '),         trim('00000000'),            '1'  union all
  -- select trim('04_rand_pick_y         '),         trim(''),            '' union all
  -- select trim('04_rand_pick_x         '),         trim(''),            '' union all
  -- select trim('04_rand_pick_x3        '),         trim(''),            '' union all
  -- select trim('04_rand_pick_x4        '),         trim(''),            '' union all
  select trim('04_chunk               '),         trim('00000010'),            '1'  union all
  select trim('04_slice_y             '),         trim('00000010'),            '1'  union all
  select trim('04_slice_x             '),         trim('00000010'),            '1'  union all
  select trim('04_slice_x3            '),         trim('00000010'),            '1'  union all
  select trim('04_slice_x4            '),         trim('00000010'),            '1'  union all
  select trim('04_sample_y            '),         trim('00000010'),            '1'  union all
  select trim('04_sample_x            '),         trim('00000010'),            '1'  union all
  select trim('04_sample_x3           '),         trim('00000010'),            '1'  union all
  select trim('04_sample_x4           '),         trim('00000010'),            '1'  union all
  select trim('04_lower_tri_mx        '),         trim('00000000'),            '1'  union all
  select trim('04_upper_tri_mx        '),         trim('00000000'),            '1'  union all
  select trim('04_lower_tri_mx_ex     '),         trim('00000000'),            '1'  union all
  select trim('04_upper_tri_mx_ex     '),         trim('00000000'),            '1'  union all
  select trim('04_lmask               '),         trim('00100000'),            '1'  union all
  select trim('04_rmask               '),         trim('00100000'),            '1'  union all
  select trim('04_amask               '),         trim('00100000'),            '1'  union all
  select trim('04_bmask               '),         trim('00100000'),            '1'  union all
  select trim('05_pool_max_2d_grp_x   '),         trim('10001000'),            '1'  union all
  select trim('05_pool_avg_2d_grp_x   '),         trim('00000000'),            '1'  union all
  select trim('05_pool_max            '),         trim('10001000'),            '1'  union all
  select trim('05_pool_avg            '),         trim('00000000'),            '1'  union all
  select trim('05_pool_none           '),         trim('00000010'),            '1'  union all
  select trim('05_conv_2d_grp_x       '),         trim('00110000'),            '1'  union all
  select trim('05_conv_2d             '),         trim('00110001'),            '1'  union all
  select trim('05_conv_add            '),         trim('00110011'),            '1'  union all
  select trim('05_conv_sub            '),         trim('00110011'),            '1'  union all
  select trim('05_conv_mul            '),         trim('00110011'),            '1'  union all
  select trim('05_conv_div            '),         trim('01110010'),            '1'  union all
  select trim('05_conv_pow            '),         trim('11111100'),            '1'  union all
  select trim('05_conv_log            '),         trim('11111000'),            '1'  union all
  select trim('05_conv_prod_mx        '),         trim('00110011'),            '1'  union all
  select trim('05_conv_de_sub         '),         trim('00110011'),            '1'  union all
  select trim('05_conv_de_div         '),         trim('01110010'),            '1'  union all
  select trim('05_conv_de_pow         '),         trim('11111100'),            '1'  union all
  select trim('05_conv_de_log         '),         trim('11111000'),            '1'  union all
  select trim('05_conv_de_prod_mx     '),         trim('00110011'),            '1'  union all
-- 对于聚合算子的所有自变量，无论顺序，统一以 path_ord_no = 1 配置该控制
  select trim('06_aggr_mx_sum         '),         trim('00001010'),            '0'  union all
  select trim('06_aggr_mx_prod        '),         trim('10001000'),            '0'  union all
  select trim('06_aggr_mx_avg         '),         trim('00001010'),            '0'  union all
  select trim('06_aggr_mx_max         '),         trim('10001000'),            '0'  union all
  select trim('06_aggr_mx_min         '),         trim('10001000'),            '0'  union all
  select trim('06_aggr_mx_concat_y    '),         trim('00000010'),            '1'  union all
  select trim('06_aggr_mx_concat_x    '),         trim('00000010'),            '1'  union all
  select trim('06_aggr_mx_concat_x3   '),         trim('00000010'),            '1'  union all
  select trim('06_aggr_mx_concat_x4   '),         trim('00000010'),            '1'  union all
  select trim('07_aggr_slice_sum      '),         trim('00000010'),            '1'  union all
  select trim('07_aggr_slice_prod     '),         trim('10001000'),            '1'  union all
  select trim('07_aggr_slice_avg      '),         trim('00000000'),            '1'  union all
  select trim('07_aggr_slice_max      '),         trim('10001000'),            '1'  union all
  select trim('07_aggr_slice_min      '),         trim('10001000'),            '1'  union all
  select trim('07_aggr_chunk_sum      '),         trim('00000010'),            '1'  union all
  select trim('07_aggr_chunk_prod     '),         trim('10001000'),            '1'  union all
  select trim('07_aggr_chunk_avg      '),         trim('00000010'),            '1'  union all
  select trim('07_aggr_chunk_max      '),         trim('10001000'),            '1'  union all
  select trim('07_aggr_chunk_min      '),         trim('10001000'),            '1'
)
select 
  'node_fn_type_delta_method' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- -------------------------------------------------------------------------
-- nn_sess_status   -- 模型部署的 session 状态
-- delete from sm_sc.tb_dic_enum where enum_name = 'nn_sess_status';
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_order
)
with 
cte_nn_sess_status(a_enum_key, a_enum_value) as
(
  select '0', 'idle'   union all 
  select '1', 'in-use'
) 
select 
  'nn_sess_status' as enum_name
, a_enum_key
, a_enum_value
, row_number() over()
from cte_nn_sess_status
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_order = excluded.enum_order
;

-- -- node_fn_asso_value  -- 算子超参数
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order     ,
  enum_range
)
with cte_node_fn_asso_value(a_param_name, a_cn_desc, enum_group, enum_order, enum_range) as
(
     select trim('l_w_r_h_chunk_pos__1d  '), trim('入参一高度规格，也即列的高度     '), trim('81_prod_mx_chunk       '),  1, numrange(1.0, 1.0, '[]')
)
select 
  'node_fn_asso_value' as enum_name, 
  enum_group || '_asso_' || enum_order as enum_key, 
  a_param_name || a_cn_desc as enum_value, 
  enum_group,
  enum_order,
  enum_range
from cte_node_fn_asso_value
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- node_fn_type   -- 算子
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as
(
  select trim('81_prod_mx_chunk      '),         trim('sm_sc.ufv_prod_mx_chunk          '),            '3_p'
)
select 
  'node_fn_type' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- -- node_fn_type_delta  -- 算子求导函数
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order
-- )
-- with cte_node_fn_type_delta(enum_key, enum_value, enum_group) as
-- (
--      select trim('dldi_query_from_col      '), trim('sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1      '),trim('81_prod_mx_chunk      ')
-- )
-- select 
--   'node_fn_type_delta' as enum_name, 
--   enum_key, 
--   enum_value, 
--   enum_group,
--   row_number() over() 
--     + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta'), 0) 
--   as enum_order
-- from cte_node_fn_type_delta
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;

-- ------------------------------------------------------------------------------------------

-- -- node_fn_type_delta_method   -- 算子求导方式
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order
-- )
-- with cte_node_fn_type(enum_key, enum_value, enum_group) as 
-- -- enum_group: 
-- --   0. 前向传播后，求 ddepdt/dindepdt
-- --   1. 反向传播。直接求 dloss/dindepdt, 而不需显式求取 ddepdt/dindepdt
-- -- enum_value:
-- --   第一位控制：第一目求导是否需要自变量
-- --   第二位控制：第二目求导是否需要自变量
-- --   第三位控制：第一目求导是否需要另一自变量
-- --   第四位控制：第二目求导是否需要另一自变量
-- --   第五位控制：第一目求导是否需要因变量
-- --   第六位控制：第二目求导是否需要因变量
-- --   第七位控制：第一目求导是否用到自变量维长规格
-- --   第八位控制：第二目求导是否用到自变量维长规格
-- (
--   select trim('81_prod_mx_chunk      '),         trim('00100010'),            '1'
-- )
-- select 
--   'node_fn_type_delta_method' as enum_name, 
--   enum_key, 
--   enum_value, 
--   enum_group,
--   row_number() over() 
--     + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method'), 0) 
--   as enum_order
-- from cte_node_fn_type
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;
-- -- node_fn_asso_value  -- 算子超参数
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order     ,
--   enum_range
-- )
-- with cte_node_fn_asso_value(a_param_name, a_cn_desc, enum_group, enum_order, enum_range) as
-- (
--      select trim('p1_len_col__1d  '), trim('入参一高度规格，也即列的高度     '), trim('81_prod_mx_slice_no_backward       '),  1, numrange(1.0, 1.0, '[]')
-- )
-- select 
--   'node_fn_asso_value' as enum_name, 
--   enum_group || '_asso_' || enum_order as enum_key, 
--   a_param_name || a_cn_desc as enum_value, 
--   enum_group,
--   enum_order,
--   enum_range
-- from cte_node_fn_asso_value
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;

-- ------------------------------------------------------------------------------------------

-- node_fn_type   -- 算子
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as
(
  select trim('81_prod_mx_slice_no_backward      '),         trim('sm_sc.ufv_prod_mx_slice          '),            '2_p'
)
select 
  'node_fn_type' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- -- node_fn_type_delta  -- 算子求导函数
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order
-- )
-- with cte_node_fn_type_delta(enum_key, enum_value, enum_group) as
-- (
--      select trim('dldi_query_from_col      '), trim('sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1      '),trim('81_prod_mx_slice_no_backward      ')
-- )
-- select 
--   'node_fn_type_delta' as enum_name, 
--   enum_key, 
--   enum_value, 
--   enum_group,
--   row_number() over() 
--     + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta'), 0) 
--   as enum_order
-- from cte_node_fn_type_delta
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;

-- ------------------------------------------------------------------------------------------

-- -- node_fn_type_delta_method   -- 算子求导方式
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order
-- )
-- with cte_node_fn_type(enum_key, enum_value, enum_group) as 
-- -- enum_group: 
-- --   0. 前向传播后，求 ddepdt/dindepdt
-- --   1. 反向传播。直接求 dloss/dindepdt, 而不需显式求取 ddepdt/dindepdt
-- -- enum_value:
-- --   第一位控制：第一目求导是否需要自变量
-- --   第二位控制：第二目求导是否需要自变量
-- --   第三位控制：第一目求导是否需要另一自变量
-- --   第四位控制：第二目求导是否需要另一自变量
-- --   第五位控制：第一目求导是否需要因变量
-- --   第六位控制：第二目求导是否需要因变量
-- --   第七位控制：第一目求导是否用到自变量维长规格
-- --   第八位控制：第二目求导是否用到自变量维长规格
-- (
--   select trim('81_prod_mx_slice_no_backward      '),         trim('00100010'),            '1'
-- )
-- select 
--   'node_fn_type_delta_method' as enum_name, 
--   enum_key, 
--   enum_value, 
--   enum_group,
--   row_number() over() 
--     + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method'), 0) 
--   as enum_order
-- from cte_node_fn_type
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;
-- -- node_fn_asso_value  -- 算子超参数
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order     ,
--   enum_range
-- )
-- with cte_node_fn_asso_value(a_param_name, a_cn_desc, enum_group, enum_order, enum_range) as
-- (
--      select trim('p1_len_col__1d  '), trim('入参一高度规格，也即列的高度     '), trim('81_query_from_col       '),  1, numrange(1.0, 1.0, '[]')
-- )
-- select 
--   'node_fn_asso_value' as enum_name, 
--   enum_group || '_asso_' || enum_order as enum_key, 
--   a_param_name || a_cn_desc as enum_value, 
--   enum_group,
--   enum_order,
--   enum_range
-- from cte_node_fn_asso_value
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;

-- ------------------------------------------------------------------------------------------

-- node_fn_type   -- 算子
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as
(
  select trim('81_query_from_col      '),         trim('sm_sc.ufv_query_by_width_row_idx_from_col          '),            '2_p'
)
select 
  'node_fn_type' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- node_fn_type_delta  -- 算子求导函数
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type_delta(enum_key, enum_value, enum_group) as
(
     select trim('dldi_query_from_col      '), trim('sm_sc.ufv_d_query_by_width_row_idx_from_col_dloss_dindepdt_1      '),trim('81_query_from_col      ')
)
select 
  'node_fn_type_delta' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta'), 0) 
  as enum_order
from cte_node_fn_type_delta
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- node_fn_type_delta_method   -- 算子求导方式
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as 
-- enum_group: 
--   0. 前向传播后，求 ddepdt/dindepdt
--   1. 反向传播。直接求 dloss/dindepdt, 而不需显式求取 ddepdt/dindepdt
-- enum_value:
--   第一位控制：第一目求导是否需要自变量
--   第二位控制：第二目求导是否需要自变量
--   第三位控制：第一目求导是否需要另一自变量
--   第四位控制：第二目求导是否需要另一自变量
--   第五位控制：第一目求导是否需要因变量
--   第六位控制：第二目求导是否需要因变量
--   第七位控制：第一目求导是否用到自变量维长规格
--   第八位控制：第二目求导是否用到自变量维长规格
(
  select trim('81_query_from_col      '),         trim('00100010'),            '1'
)
select 
  'node_fn_type_delta_method' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;
-- -- node_fn_asso_value  -- 算子超参数
-- insert into sm_sc.tb_dic_enum
-- (
--   enum_name      ,
--   enum_key       ,
--   enum_value     ,
--   enum_group     ,
--   enum_order     ,
--   enum_range
-- )
-- with cte_node_fn_asso_value(a_param_name, a_cn_desc, enum_group, enum_order, enum_range) as
-- (
--      select trim('p1_len_row__1d  '), trim('入参一宽度规格，也即行的宽度     '), trim('81_query_from_row       '),  1, numrange(1.0, 1.0, '[]')
-- )
-- select 
--   'node_fn_asso_value' as enum_name, 
--   enum_group || '_asso_' || enum_order as enum_key, 
--   a_param_name || a_cn_desc as enum_value, 
--   enum_group,
--   enum_order,
--   enum_range
-- from cte_node_fn_asso_value
-- on conflict(enum_name, enum_key)
-- do update
-- set 
--   enum_value = excluded.enum_value
-- , enum_group = excluded.enum_group
-- , enum_order = excluded.enum_order
-- ;

-- ------------------------------------------------------------------------------------------

-- node_fn_type   -- 算子
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as
(
  select trim('81_query_from_row      '),         trim('sm_sc.ufv_query_by_heigh_col_idx_from_row          '),            '2_p'
)
select 
  'node_fn_type' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type'), 0)
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- node_fn_type_delta  -- 算子求导函数
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type_delta(enum_key, enum_value, enum_group) as
(
     select trim('dldi_query_from_row      '), trim('sm_sc.ufv_d_query_by_heigh_col_idx_from_row_dloss_dindepdt_1      '),trim('81_query_from_row      ')
)
select 
  'node_fn_type_delta' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta'), 0)
  as enum_order
from cte_node_fn_type_delta
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;

-- ------------------------------------------------------------------------------------------

-- node_fn_type_delta_method   -- 算子求导方式
insert into sm_sc.tb_dic_enum
(
  enum_name      ,
  enum_key       ,
  enum_value     ,
  enum_group     ,
  enum_order
)
with cte_node_fn_type(enum_key, enum_value, enum_group) as 
-- enum_group: 
--   0. 前向传播后，求 ddepdt/dindepdt
--   1. 反向传播。直接求 dloss/dindepdt, 而不需显式求取 ddepdt/dindepdt
-- enum_value:
--   第一位控制：第一目求导是否需要自变量
--   第二位控制：第二目求导是否需要自变量
--   第三位控制：第一目求导是否需要另一自变量
--   第四位控制：第二目求导是否需要另一自变量
--   第五位控制：第一目求导是否需要因变量
--   第六位控制：第二目求导是否需要因变量
--   第七位控制：第一目求导是否用到自变量高宽规格
--   第八位控制：第二目求导是否用到自变量高宽规格
(
  select trim('81_query_from_row      '),         trim('00100010'),            '1'
)
select 
  'node_fn_type_delta_method' as enum_name, 
  enum_key, 
  enum_value, 
  enum_group,
  row_number() over() 
    + coalesce((select max(enum_order) from sm_sc.tb_dic_enum where enum_name = 'node_fn_type_delta_method'), 0) 
  as enum_order
from cte_node_fn_type
on conflict(enum_name, enum_key)
do update
set 
  enum_value = excluded.enum_value
, enum_group = excluded.enum_group
, enum_order = excluded.enum_order
;
drop operator if exists + (sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create operator + 
(
  leftarg     =    sm_sc.typ_l_complex           ,
  rightarg    =    sm_sc.typ_l_complex           ,
  function    =    sm_sc.fv_opr_add  ,
  commutator  =    +
);
-- select (12.3, -56.6)::sm_sc.typ_l_complex + (-2.3, 6.6)::sm_sc.typ_l_complex
-- select 12.3 + (-2.3, 6.6)::sm_sc.typ_l_complex
-- select (12.3, -56.6)::sm_sc.typ_l_complex + 6.6
-- -------------------------------------------------------------------------------------------------------
drop operator if exists - (sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create operator - 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    sm_sc.typ_l_complex           ,
  function  =    sm_sc.fv_opr_sub
);
-- select (12.3, -56.6)::sm_sc.typ_l_complex - (-2.3, 6.6)::sm_sc.typ_l_complex
-- select 12.3 - (-2.3, 6.6)::sm_sc.typ_l_complex
-- select (12.3, -56.6)::sm_sc.typ_l_complex - 6.6
  drop operator if exists - (none, sm_sc.typ_l_complex);
  create operator - 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_sub
  );
  -- select - (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select - 56.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists * (sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create operator * 
(
  leftarg     =    sm_sc.typ_l_complex           ,
  rightarg    =    sm_sc.typ_l_complex           ,
  function    =    sm_sc.fv_opr_mul  ,
  commutator  =    *
);
-- select (12.3, -56.6)::sm_sc.typ_l_complex * (-2.3, 6.6)::sm_sc.typ_l_complex
-- select 12.3 * (-2.3, 6.6)::sm_sc.typ_l_complex
-- select (12.3, -56.6)::sm_sc.typ_l_complex * 6.6
-- -------------------------------------------------------------------------------------------------------
drop operator if exists ~ (none, sm_sc.typ_l_complex);
create operator ~ 
(
  rightarg  =    sm_sc.typ_l_complex           ,
  function  =    sm_sc.fv_opr_conjugate
);
-- select ~ (-2.3, 6.6)::sm_sc.typ_l_complex
-- select ~ 56.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists @ (none, sm_sc.typ_l_complex);
create operator @ 
(
  rightarg  =    sm_sc.typ_l_complex           ,
  function  =    sm_sc.fv_opr_norm
);
-- select @ (-2.3, 6.6)::sm_sc.typ_l_complex
-- select @ 56.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists / (sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create operator / 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    sm_sc.typ_l_complex           ,
  function  =    sm_sc.fv_opr_div
);
-- select (12.3, -56.6)::sm_sc.typ_l_complex / (-2.3, 6.6)::sm_sc.typ_l_complex
-- select 12.3 / (-2.3, 6.6)::sm_sc.typ_l_complex
-- select (12.3, -56.6)::sm_sc.typ_l_complex / 6.6
  -- -- -- 不支持 / 做为单目运算符，会出现语法错误
  -- -- -- https://www.imooc.com/wenda/detail/564806
  -- -- -- https://www.linuxidc.com/Linux/2012-02/53783.htm
  -- -- drop operator if exists / (none, sm_sc.typ_l_complex);
  -- -- create operator / 
  -- -- (
  -- --   rightarg  =    sm_sc.typ_l_complex           ,
  -- --   function  =    sm_sc.fv_opr_div
  -- -- );
  -- -- -- select / (-2.3, 6.6)::sm_sc.typ_l_complex
  -- -- -- select / 56.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists ^ (sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create operator ^ 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    sm_sc.typ_l_complex           ,
  function  =    sm_sc.fv_opr_pow
);
-- select (2.3, -5.6)::sm_sc.typ_l_complex ^ (-2.3, 6.6)::sm_sc.typ_l_complex
-- select 2.3 ^ (-2.3, 2)::sm_sc.typ_l_complex
-- select (12.3, -56.6)::sm_sc.typ_l_complex ^ 3.0

  -- -- -- -- 不支持 ^ 作为单目运算符，会出现语法错误
  -- -- -- drop operator if exists ^ (none, sm_sc.typ_l_complex);
  -- -- -- create operator ^ 
  -- -- -- (
  -- -- --   rightarg  =    sm_sc.typ_l_complex           ,
  -- -- --   function  =    sm_sc.fv_opr_exp
  -- -- -- );
  -- -- -- -- select ^ (-2.3, 6.6)::sm_sc.typ_l_complex
  -- -- -- -- select ^ 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists ^! (sm_sc.typ_l_complex, sm_sc.typ_l_complex);
create operator ^! 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    sm_sc.typ_l_complex           ,
  function  =    sm_sc.fv_opr_log
);
-- select (2.3, -5.6)::sm_sc.typ_l_complex ^! (-2.3, 6.6)::sm_sc.typ_l_complex
-- select 2.3 ^! (-2.3, 2)::sm_sc.typ_l_complex
-- select (12.3, -56.6)::sm_sc.typ_l_complex ^! 3.0
  drop operator if exists ^! (none, sm_sc.typ_l_complex);
  create operator ^! 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_ln
  );
  -- select ^! (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select ^! 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
  drop operator if exists @~ (none, sm_sc.typ_l_complex);
  create operator @~ 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_real
  );
  -- select @~ (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select @~ 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
  drop operator if exists ~@ (none, sm_sc.typ_l_complex);
  create operator ~@ 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_imaginary
  );
  -- select ~@ (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select ~@ 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
  drop operator if exists ~^ (none, sm_sc.typ_l_complex);
  create operator ~^ 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_conjugate_45
  );
  -- select ~^ (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select ~^ 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists ~= (sm_sc.typ_l_complex, int);
create operator ~= 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_round
);
-- select (12.39678, -56.656466)::sm_sc.typ_l_complex ~= 3
  drop operator if exists ~= (none, sm_sc.typ_l_complex);
  create operator ~= 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_round
  );
  -- select ~= (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select ~= 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists ~< (sm_sc.typ_l_complex, int);
create operator ~< 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_floor
);
-- select (12.39678, -56.656466)::sm_sc.typ_l_complex ~< 3
  drop operator if exists ~< (none, sm_sc.typ_l_complex);
  create operator ~< 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_floor
  );
  -- select ~< (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select ~< 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists ~> (sm_sc.typ_l_complex, int);
create operator ~> 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_ceil
);
-- select (12.39678, -56.656466)::sm_sc.typ_l_complex ~> 3
  drop operator if exists ~> (none, sm_sc.typ_l_complex);
  create operator ~> 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_ceil
  );
  -- select ~> (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select ~> 6.6::sm_sc.typ_l_complex
-- -------------------------------------------------------------------------------------------------------
drop operator if exists >< (sm_sc.typ_l_complex, int);
create operator >< 
(
  leftarg   =    sm_sc.typ_l_complex           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_trunc
);
-- select (12.39678, -56.656466)::sm_sc.typ_l_complex >< 3
  drop operator if exists >< (none, sm_sc.typ_l_complex);
  create operator >< 
  (
    rightarg  =    sm_sc.typ_l_complex           ,
    function  =    sm_sc.fv_opr_trunc
  );
  -- select >< (-2.3, 6.6)::sm_sc.typ_l_complex
  -- select >< 6.6::sm_sc.typ_l_complex
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists + (anyarray, anyarray);
create operator + 
(
  leftarg     =    anyarray           ,
  rightarg    =    anyarray           ,
  function    =    sm_sc.fv_opr_add  ,
  commutator  =    +
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] + array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] + array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] + array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] + array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] + array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists + (anyelement, anyarray);
  create operator + 
  (
    leftarg     =    anyelement             ,
    rightarg    =    anyarray           ,
    function    =    sm_sc.fv_opr_add  ,
    commutator  =    +
  );
  -- select 8.8 + array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists + (anyarray, anyelement);
  create operator + 
  (
    leftarg     =    anyarray           ,
    rightarg    =    anyelement             ,
    function    =    sm_sc.fv_opr_add  ,
    commutator  =    +
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] + 8.9
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists +` (float[], float[]);
create operator +` 
(
  leftarg     =    float[]           ,
  rightarg    =    float[]           ,
  function    =    sm_sc.fv_opr_add_py  ,
  commutator  =    +`
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] +` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] +` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists +` (float, float[]);
  create operator +` 
  (
    leftarg     =    float             ,
    rightarg    =    float[]           ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
  -- select 8.8 +` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists +` (float[], float);
  create operator +` 
  (
    leftarg     =    float[]           ,
    rightarg    =    float             ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` 8.9
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists +` (decimal[], decimal[]);
create operator +` 
(
  leftarg     =    decimal[]           ,
  rightarg    =    decimal[]           ,
  function    =    sm_sc.fv_opr_add_py  ,
  commutator  =    +`
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] +` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] +` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists +` (decimal, decimal[]);
  create operator +` 
  (
    leftarg     =    decimal             ,
    rightarg    =    decimal[]           ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
  -- select 8.8 +` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists +` (decimal[], decimal);
  create operator +` 
  (
    leftarg     =    decimal[]           ,
    rightarg    =    decimal             ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] +` 8.9
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists +` (anyarray, anyarray);
create operator +` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_add
);
  drop operator if exists +` (anyarray, anyelement);
  create operator +` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_add
  );
  drop operator if exists +` (anyelement, anyarray);
  create operator +` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_add
  );
drop operator if exists +` (bigint[], bigint[]);
create operator +` 
(
  leftarg     =    bigint[]           ,
  rightarg    =    bigint[]           ,
  function    =    sm_sc.fv_opr_add_py  ,
  commutator  =    +`
);
  -- 重载
  drop operator if exists +` (bigint, bigint[]);
  create operator +` 
  (
    leftarg     =    bigint             ,
    rightarg    =    bigint[]           ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
  drop operator if exists +` (bigint[], bigint);
  create operator +` 
  (
    leftarg     =    bigint[]           ,
    rightarg    =    bigint             ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists +` (int[], int[]);
create operator +` 
(
  leftarg     =    int[]           ,
  rightarg    =    int[]           ,
  function    =    sm_sc.fv_opr_add_py  ,
  commutator  =    +`
);
  -- 重载
  drop operator if exists +` (int, int[]);
  create operator +` 
  (
    leftarg     =    int             ,
    rightarg    =    int[]           ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
  drop operator if exists +` (int[], int);
  create operator +` 
  (
    leftarg     =    int[]           ,
    rightarg    =    int             ,
    function    =    sm_sc.fv_opr_add_py  ,
    commutator  =    +`
  );
-- select
--   array[[12,9,18]] +` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] +` array[[3,3,2]]::int[]   
-- ,               12 +` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] +`              3::bigint  
-- ,               12 +` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] +`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists - (anyarray, anyarray);
create operator - 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_sub
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] - array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] - array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] - array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] - array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] - array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists - (anyarray, anyelement);
  create operator - 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_sub
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] - 2.66
  drop operator if exists - (anyelement, anyarray);
  create operator - 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_sub
  );
  -- select 2.66 - array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  drop operator if exists - (none, anyarray);
  create operator - 
  (
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_sub
  );
  -- select - array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists -` (float[], float[]);
create operator -` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_sub_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] -` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists -` (float[], float);
  create operator -` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` 2.66
  drop operator if exists -` (float, float[]);
  create operator -` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  -- select 2.66 -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  drop operator if exists -` (none, float[]);
  create operator -` 
  (
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  -- select -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists -` (decimal[], decimal[]);
create operator -` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_sub_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] -` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists -` (decimal[], decimal);
  create operator -` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] -` 2.66
  drop operator if exists -` (decimal, decimal[]);
  create operator -` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  -- select 2.66 -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  drop operator if exists -` (none, decimal[]);
  create operator -` 
  (
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  -- select -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists -` (anyarray, anyarray);
create operator -` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_sub
);
  drop operator if exists -` (anyarray, anyelement);
  create operator -` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_sub
  );
  drop operator if exists -` (anyelement, anyarray);
  create operator -` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_sub
  );
  drop operator if exists -` (none, anyarray);
  create operator -` 
  (
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_sub
  );
drop operator if exists -` (bigint[], bigint[]);
create operator -` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_sub_py
);
  -- 重载
  drop operator if exists -` (bigint[], bigint);
  create operator -` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  drop operator if exists -` (bigint, bigint[]);
  create operator -` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  drop operator if exists -` (none, bigint[]);
  create operator -` 
  (
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists -` (int[], int[]);
create operator -` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_sub_py
);
  -- 重载
  drop operator if exists -` (int[], int);
  create operator -` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  drop operator if exists -` (int, int[]);
  create operator -` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  drop operator if exists -` (none, int[]);
  create operator -` 
  (
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_sub_py
  );
  
-- select
--   array[[12,9,18]] -` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] -` array[[3,3,2]]::int[]   
-- ,               12 -` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] -`              3::bigint  
-- ,               12 -` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] -`              3::   int 
-- ,                  -` array[[3,3,2]]::   int[]
-- ,                  -` array[[12,9,18]]::bigint[]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists * (anyarray, anyarray);
create operator * 
(
  leftarg     =    anyarray           ,
  rightarg    =    anyarray           ,
  function    =    sm_sc.fv_opr_mul  ,
  commutator  =    *
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] * array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] * array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] * array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] * array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] * array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists * (anyelement, anyarray);
  create operator * 
  (
    leftarg     =    anyelement             ,
    rightarg    =    anyarray           ,
    function    =    sm_sc.fv_opr_mul  ,
    commutator  =    *
  );
  -- select 52.3 * array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists * (anyarray, anyelement);
  create operator * 
  (
    leftarg     =    anyarray           ,
    rightarg    =    anyelement             ,
    function    =    sm_sc.fv_opr_mul  ,
    commutator  =    *
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] * 2.55
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists *` (float[], float[]);
create operator *` 
(
  leftarg     =    float[]           ,
  rightarg    =    float[]           ,
  function    =    sm_sc.fv_opr_mul_py  ,
  commutator  =    *`
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] *` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] *` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists *` (float, float[]);
  create operator *` 
  (
    leftarg     =    float             ,
    rightarg    =    float[]           ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
  -- select 52.3 *` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists *` (float[], float);
  create operator *` 
  (
    leftarg     =    float[]           ,
    rightarg    =    float             ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` 2.55
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists *` (decimal[], decimal[]);
create operator *` 
(
  leftarg     =    decimal[]           ,
  rightarg    =    decimal[]           ,
  function    =    sm_sc.fv_opr_mul_py  ,
  commutator  =    *`
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] *` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] *` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists *` (decimal, decimal[]);
  create operator *` 
  (
    leftarg     =    decimal             ,
    rightarg    =    decimal[]           ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
  -- select 52.3 *` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists *` (decimal[], decimal);
  create operator *` 
  (
    leftarg     =    decimal[]           ,
    rightarg    =    decimal             ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] *` 2.55
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists *` (anyarray, anyarray);
create operator *` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_mul
);
  drop operator if exists *` (anyarray, anyelement);
  create operator *` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_mul
  );
  drop operator if exists *` (anyelement, anyarray);
  create operator *` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_mul
  );
drop operator if exists *` (bigint[], bigint[]);
create operator *` 
(
  leftarg     =    bigint[]           ,
  rightarg    =    bigint[]           ,
  function    =    sm_sc.fv_opr_mul_py  ,
  commutator  =    *`
);
  -- 重载
  drop operator if exists *` (bigint, bigint[]);
  create operator *` 
  (
    leftarg     =    bigint             ,
    rightarg    =    bigint[]           ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
  drop operator if exists *` (bigint[], bigint);
  create operator *` 
  (
    leftarg     =    bigint[]           ,
    rightarg    =    bigint             ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists *` (int[], int[]);
create operator *` 
(
  leftarg     =    int[]           ,
  rightarg    =    int[]           ,
  function    =    sm_sc.fv_opr_mul_py  ,
  commutator  =    *`
);
  -- 重载
  drop operator if exists *` (int, int[]);
  create operator *` 
  (
    leftarg     =    int             ,
    rightarg    =    int[]           ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
  drop operator if exists *` (int[], int);
  create operator *` 
  (
    leftarg     =    int[]           ,
    rightarg    =    int             ,
    function    =    sm_sc.fv_opr_mul_py  ,
    commutator  =    *`
  );
-- select
--   array[[12,9,18]] *` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] *` array[[3,3,2]]::int[]   
-- ,               12 *` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] *`              3::bigint  
-- ,               12 *` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] *`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists / (anyarray, anyarray);
create operator / 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_div
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] / array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] / array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] / array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] / array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] / array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- select array[12.5, -19.1, 1.11] / array[array[0, 0, 0], array[0, 0, 0]] :: decimal[]
  -- 重载
  drop operator if exists / (anyelement, anyarray);
  create operator / 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_div
  );
  -- select 100.66 / array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  -- select 100.66 / array[array[12.5, 0, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists / (anyarray, anyelement);
  create operator / 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_div
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` (-32.5)
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` 0
  -- -- -- drop operator if exists / (none, anyarray);
  -- -- -- create operator / 
  -- -- -- (
  -- -- --   rightarg  =    anyarray           ,
  -- -- --   function  =    sm_sc.fv_opr_div
  -- -- -- );
  -- -- -- -- 测试用例报语法错误
  -- -- -- -- select / array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- -- -- -- select / array[array[32.5, 1.26, 33.6], array[-9.1, 0, 4.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists /` (anyarray, anyarray);
create operator /` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_div
);
  drop operator if exists /` (anyarray, anyelement);
  create operator /` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_div
  );
  drop operator if exists /` (anyelement, anyarray);
  create operator /` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_div
  );
  drop operator if exists /` (none, anyarray);
  create operator /` 
  (
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_div
  );
drop operator if exists /` (float[], float[]);
create operator /` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_div_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] /` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] /` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- select array[12.5, -19.1, 1.11] /` array[array[0, 0, 0], array[0, 0, 0]]
  -- 重载
  drop operator if exists /` (float, float[]);
  create operator /` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_div_py
  );
  -- select 100.66 /` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  -- select 100.66 /` array[array[12.5, 0, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists /` (float[], float);
  create operator /` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_div_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` (-32.5)
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` 0
  drop operator if exists /` (none, float[]);
  create operator /` 
  (
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_div_py
  );
  -- select /` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- select /` array[array[32.5, 1.26, 33.6], array[-9.1, 0, 4.69]]
drop operator if exists /` (decimal[], decimal[]);
create operator /` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_div_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] /` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] /` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- select array[12.5, -19.1, 1.11] /` array[array[0, 0, 0], array[0, 0, 0]]
  -- 重载
  drop operator if exists /` (decimal, decimal[]);
  create operator /` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_div_py
  );
  -- select 100.66 /` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  -- select 100.66 /` array[array[12.5, 0, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists /` (decimal[], decimal);
  create operator /` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_div_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` (-32.5)
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] /` 0
  drop operator if exists /` (none, decimal[]);
  create operator /` 
  (
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_div_py
  );
  -- select /` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- select /` array[array[32.5, 1.26, 33.6], array[-9.1, 0, 4.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists /` (bigint[], bigint[]);
create operator /` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_div_py
);
  -- 重载
  drop operator if exists /` (bigint, bigint[]);
  create operator /` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_div_py
  );
  drop operator if exists /` (bigint[], bigint);
  create operator /` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_div_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists /` (int[], int[]);
create operator /` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_div_py
);
  -- 重载
  drop operator if exists /` (int, int[]);
  create operator /` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_div_py
  );
  drop operator if exists /` (int[], int);
  create operator /` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_div_py
  );
-- select
--   array[[12,9,18]] /` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] /` array[[3,3,2]]::int[]   
-- ,               12 /` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] /`              3::bigint  
-- ,               12 /` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] /`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists % (anyarray, anyarray);
create operator % 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_mod
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] % array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] % array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] % array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] % array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] % array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- select array[12.5, -19.1, 1.11] % array[array[0, 0, 0], array[0, 0, 0]] :: decimal(32, 4)[]
  -- 重载
  drop operator if exists % (anyelement, anyarray);
  create operator % 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_mod
  );
  -- select 100.66 % array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  -- select 100.66 % array[array[12.5, 0, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists % (anyarray, anyelement);
  create operator % 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_mod
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] % (-32.5)
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] % 0 :: decimal(32, 4)

-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists %` (anyarray, anyarray);
create operator %` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_mod
);
  drop operator if exists %` (anyarray, anyelement);
  create operator %` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_mod
  );
  drop operator if exists %` (anyelement, anyarray);
  create operator %` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_mod
  );
drop operator if exists %` (decimal(32, 4)[], decimal(32, 4)[]);
create operator %` 
(
  leftarg   =    decimal(32, 4)[]           ,
  rightarg  =    decimal(32, 4)[]           ,
  function  =    sm_sc.fv_opr_mod_posi_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] %` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %` array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %` array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] %` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- select array[12.5, -19.1, 1.11] %` array[array[0, 0, 0], array[0, 0, 0]] :: decimal(32, 4)[]
  -- 重载
  drop operator if exists %` (decimal(32, 4), decimal(32, 4)[]);
  create operator %` 
  (
    leftarg   =    decimal(32, 4)           ,
    rightarg  =    decimal(32, 4)[]           ,
    function  =    sm_sc.fv_opr_mod_posi_py
  );
  -- select 100.66 %` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  -- select 100.66 %` array[array[12.5, 0, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists %` (decimal(32, 4)[], decimal(32, 4));
  create operator %` 
  (
    leftarg   =    decimal(32, 4)[]           ,
    rightarg  =    decimal(32, 4)           ,
    function  =    sm_sc.fv_opr_mod_posi_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %` (-32.5)
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %` 0 :: decimal(32, 4)
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists %` (bigint[], bigint[]);
create operator %` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_mod_posi_py
);
  -- 重载
  drop operator if exists %` (bigint, bigint[]);
  create operator %` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_mod_posi_py
  );
  drop operator if exists %` (bigint[], bigint);
  create operator %` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_mod_posi_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists %` (int[], int[]);
create operator %` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_mod_posi_py
);
  -- 重载
  drop operator if exists %` (int, int[]);
  create operator %` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_mod_posi_py
  );
  drop operator if exists %` (int[], int);
  create operator %` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_mod_posi_py
  );
-- select
--   array[[12,9,18]] %` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] %` array[[3,3,2]]::int[]   
-- ,               12 %` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] %`              3::bigint  
-- ,               12 %` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] %`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists %`- (decimal(32, 4)[], decimal(32, 4)[]);
create operator %`- 
(
  leftarg   =    decimal(32, 4)[]           ,
  rightarg  =    decimal(32, 4)[]           ,
  function  =    sm_sc.fv_opr_mod_nega_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %`- array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[-9.1]] %`- array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %`- array[array[12.5], array[-19.1]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %`- array[12.5, -19.1, 1.11]
-- select array[12.5, -19.1, 1.11] %`- array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- select array[12.5, -19.1, 1.11] %`- array[array[0, 0, 0], array[0, 0, 0]] :: decimal(32, 4)[]
  -- 重载
  drop operator if exists %`- (decimal(32, 4), decimal(32, 4)[]);
  create operator %`- 
  (
    leftarg   =    decimal(32, 4)           ,
    rightarg  =    decimal(32, 4)[]           ,
    function  =    sm_sc.fv_opr_mod_nega_py
  );
  -- select 100.66 %`- array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  -- select 100.66 %`- array[array[12.5, 0, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists %`- (decimal(32, 4)[], decimal(32, 4));
  create operator %`- 
  (
    leftarg   =    decimal(32, 4)[]           ,
    rightarg  =    decimal(32, 4)           ,
    function  =    sm_sc.fv_opr_mod_nega_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %`- (-32.5)
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] %`- 0 :: decimal(32, 4)
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists %`- (bigint[], bigint[]);
create operator %`- 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_mod_nega_py
);
  -- 重载
  drop operator if exists %`- (bigint, bigint[]);
  create operator %`- 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_mod_nega_py
  );
  drop operator if exists %`- (bigint[], bigint);
  create operator %`- 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_mod_nega_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists %`- (int[], int[]);
create operator %`- 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_mod_nega_py
);
  -- 重载
  drop operator if exists %`- (int, int[]);
  create operator %`- 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_mod_nega_py
  );
  drop operator if exists %`- (int[], int);
  create operator %`- 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_mod_nega_py
  );
-- select
--   array[[12,9,18]] %`- array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] %`- array[[3,3,2]]::int[]   
-- ,               12 %`- array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] %`-              3::bigint  
-- ,               12 %`- array[[3,3,2]]::   int[]
-- , array[[12,9,18]] %`-              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ^ (anyarray, anyarray);
create operator ^ 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_pow
);
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^ array[array[2, 3, -1], array[-0.5, 1.5, 0]]
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^ array[array[2], array[-0.5]]
-- select array[array[32.5], array[9.1]] ^ array[array[2, 3, -1], array[-0.5, 1.5, 0]]
-- select array[32.5, 9.1, 3.2] ^ array[array[2, 3, -1], array[-0.5, 1.5, 0]]
-- select array[array[32.5, 9.1, 3.2], array[7.7, 8.2, 32.1]] ^ array[-0.5, 1.5, 0]
  -- 重载
  drop operator if exists ^ (anyelement, anyarray);
  create operator ^ 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_pow
  );
  -- select 16.88 ^ array[array[2.5, 1.26, 3.6], array[9.1, 8.6, 4.69]]
  drop operator if exists ^ (anyarray, anyelement);
  create operator ^ 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_pow
  );
  -- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^ (-1.5)
  -- -- -- -- drop operator if exists ^ (none, anyarray);
  -- -- -- -- create operator ^ 
  -- -- -- -- (
  -- -- -- --   rightarg  =    anyarray           ,
  -- -- -- --   function  =    sm_sc.fv_opr_exp
  -- -- -- -- );
  -- -- -- -- -- -- 测试用例报语法错误
  -- -- -- -- -- select ^ array[array[32.5, 1, 33.6], array[2.1, 8.6, 4.69]] 
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ^` (float[], float[]);
create operator ^` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_pow_py
);
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^` array[array[2, 3, -1], array[-0.5, 1.5, 0]]
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^` array[array[2], array[-0.5]]
-- select array[array[32.5], array[9.1]] ^` array[array[2, 3, -1], array[-0.5, 1.5, 0]]
-- select array[32.5, 9.1, 3.2] ^` array[array[2, 3, -1], array[-0.5, 1.5, 0]]
-- select array[array[32.5, 9.1, 3.2], array[7.7, 8.2, 32.1]] ^` array[-0.5, 1.5, 0]
  -- 重载
  drop operator if exists ^` (float, float[]);
  create operator ^` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_pow_py
  );
  -- select 16.88 ^` array[array[2.5, 1.26, 3.6], array[9.1, 8.6, 4.69]]
  drop operator if exists ^` (float[], float);
  create operator ^` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_pow_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^` (-1.5)
  drop operator if exists ^` (none, float[]);
  create operator ^` 
  (
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_exp_py
  );
  -- select ^` array[array[3.5, 1, 33.6], array[9.1, 8.6, 4.69]] 
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ^! (anyarray, anyarray);
create operator ^!
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_log
);
-- select array[array[32.5, 1, 33.6], array[9.1, 8.6, 4.69]] ^! array[array[2, 3, 1.26], array[0.5, 1.5, 36]] 
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^! array[array[2], array[0.5]]
-- select array[array[32.5], array[9.1]] ^! array[array[2, 3, 1.26], array[0.5, 1.5, 1.5]]
-- select array[32.5, 9.1, 1] ^! array[array[2, 3, 1.26], array[0.5, 1.5, 0.3]]
-- select array[array[32.5, 9.1, 3.2], array[7.7, 8.2, 1]] ^! array[0.5, 1.5, 0.8]
  -- 重载
  drop operator if exists ^! (anyelement, anyarray);
  create operator ^! 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_log
  );
  -- select 85.4 ^! array[array[2, 3, 1.3], array[0.5, 1.5, 36]]
  drop operator if exists ^! (anyarray, anyelement);
  create operator ^! 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_log
  );
  -- select array[array[32.5, 1, 33.6], array[9.1, 8.6, 4.69]] ^! 2.5
  drop operator if exists ^! (none, anyarray);
  create operator ^! 
  (
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_ln
  );
  -- select ^! array[array[32.5, 1, 33.6], array[9.1, 8.6, 4.69]] 
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ^!` (float[], float[]);
create operator ^!` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_log_py
);
-- select array[array[32.5, 1, 33.6], array[9.1, 8.6, 4.69]] ^!` array[array[2, 3, 1.26], array[0.5, 1.5, 36]] 
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] ^!` array[array[2], array[0.5]]
-- select array[array[32.5], array[9.1]] ^!` array[array[2, 3, 1.26], array[0.5, 1.5, 1.5]]
-- select array[32.5, 9.1, 1] ^!` array[array[2, 3, 1.26], array[0.5, 1.5, 0.3]]
-- select array[array[32.5, 9.1, 3.2], array[7.7, 8.2, 1]] ^!` array[0.5, 1.5, 0.8]
  -- 重载
  drop operator if exists ^!` (float, float[]);
  create operator ^!` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_log_py
  );
  -- select 85.4 ^!` array[array[2, 3, 1.3], array[0.5, 1.5, 36]]
  drop operator if exists ^!` (float[], float);
  create operator ^!` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_log_py
  );
  -- select array[array[32.5, 1, 33.6], array[9.1, 8.6, 4.69]] ^!` 2.5
  drop operator if exists ^!` (none, float[]);
  create operator ^!` 
  (
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_ln_py
  );
  -- select ^!` array[array[32.5, 1, 33.6], array[9.1, 8.6, 4.69]] 
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists |` (boolean[], boolean[]);
create operator |` 
(
  leftarg     =    boolean[]           ,
  rightarg    =    boolean[]           ,
  function    =    sm_sc.fv_opr_or  ,
  commutator  =    |`
);
-- select array[array[true, false, true], array[false, false, true]] |` array[array[true, true, true], array[false, true, true]]
-- select array[array[true], array[false]] |` array[array[false, true, true], array[false, true, true]]
-- select array[array[true, false, true], array[false, false, true]] |` array[array[true], array[false]]
-- select array[array[true, false, true], array[false, false, true]] |` array[true, false, false]
-- select array[true, false, false] |` array[array[true, false, true], array[false, false, true]]
  -- 重载
  drop operator if exists |` (boolean, boolean[]);
  create operator |` 
  (
    leftarg     =    boolean             ,
    rightarg    =    boolean[]           ,
    function    =    sm_sc.fv_opr_or  ,
    commutator  =    |`
  );
  -- select true |` array[array[true, false, true], array[false, true, true]]
  drop operator if exists |` (boolean[], boolean);
  create operator |` 
  (
    leftarg     =    boolean[]           ,
    rightarg    =    boolean             ,
    function    =    sm_sc.fv_opr_or  ,
    commutator  =    |`
  );
  -- select array[array[true, false, true], array[false, false, true]] |` false

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists |` (varbit[], varbit[]);
create operator |` 
(
  leftarg     =    varbit[]           ,
  rightarg    =    varbit[]           ,
  function    =    sm_sc.fv_opr_or  ,
  commutator  =    |`
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] |` array[array[B'001', B'010', B'100'], array[B'110', B'011', B'110']]
-- select array[array[B'101'], array[B'001']] |` array[array[B'100', B'101', B'010'], array[B'100', B'000', B'010']]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] |` array[array[B'100'], array[B'100']]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] |` array[B'010', B'100', B'100']
-- select array[B'010', B'100', B'100'] |` array[array[B'110', B'010', B'100'], array[B'100', B'110', B'001']]
  -- 重载
  drop operator if exists |` (varbit, varbit[]);
  create operator |` 
  (
    leftarg     =    varbit             ,
    rightarg    =    varbit[]           ,
    function    =    sm_sc.fv_opr_or  ,
    commutator  =    |`
  );
  -- select B'001' |` array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']]
  drop operator if exists |` (varbit[], varbit);
  create operator |` 
  (
    leftarg     =    varbit[]           ,
    rightarg    =    varbit             ,
    function    =    sm_sc.fv_opr_or  ,
    commutator  =    |`
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] |` B'001'
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists &` (boolean[], boolean[]);
create operator &` 
(
  leftarg     =    boolean[]           ,
  rightarg    =    boolean[]           ,
  function    =    sm_sc.fv_opr_and  ,
  commutator  =    &`
);
-- select array[array[true, false, true], array[false, false, true]] &` array[array[true, true, true], array[false, true, true]]
-- select array[array[true], array[false]] &` array[array[false, true, true], array[false, true, true]]
-- select array[array[true, false, true], array[false, false, true]] &` array[array[true], array[false]]
-- select array[array[true, false, true], array[false, false, true]] &` array[true, false, false]
-- select array[true, false, false] &` array[array[true, false, true], array[false, false, true]]
  -- 重载
  drop operator if exists &` (boolean, boolean[]);
  create operator &` 
  (
    leftarg     =    boolean             ,
    rightarg    =    boolean[]           ,
    function    =    sm_sc.fv_opr_and  ,
    commutator  =    &`
  );
  -- select true &` array[array[true, false, true], array[false, true, true]]
  drop operator if exists &` (boolean[], boolean);
  create operator &` 
  (
    leftarg     =    boolean[]           ,
    rightarg    =    boolean             ,
    function    =    sm_sc.fv_opr_and  ,
    commutator  =    &`
  );
  -- select array[array[true, false, true], array[false, false, true]] &` false

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists &` (varbit[], varbit[]);
create operator &` 
(
  leftarg     =    varbit[]           ,
  rightarg    =    varbit[]           ,
  function    =    sm_sc.fv_opr_and  ,
  commutator  =    &`
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] &` array[array[B'001', B'010', B'100'], array[B'110', B'011', B'110']]
-- select array[array[B'101'], array[B'001']] &` array[array[B'100', B'101', B'010'], array[B'100', B'000', B'010']]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] &` array[array[B'100'], array[B'100']]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] &` array[B'010', B'100', B'100']
-- select array[B'010', B'100', B'100'] &` array[array[B'110', B'010', B'100'], array[B'100', B'110', B'001']]
  -- 重载
  drop operator if exists &` (varbit, varbit[]);
  create operator &` 
  (
    leftarg     =    varbit             ,
    rightarg    =    varbit[]           ,
    function    =    sm_sc.fv_opr_and  ,
    commutator  =    &`
  );
  -- select B'001' &` array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']]
  drop operator if exists &` (varbit[], varbit);
  create operator &` 
  (
    leftarg     =    varbit[]           ,
    rightarg    =    varbit             ,
    function    =    sm_sc.fv_opr_and  ,
    commutator  =    &`
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] &` B'001'
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ~` (none, boolean[]);
create operator ~` 
(
  rightarg  =    boolean[]           ,
  function  =    sm_sc.fv_opr_not
);
-- select ~` array[array[true, true, false], array[false, true, false]] 
drop operator if exists ~` (none, varbit[]);
create operator ~` 
(
  rightarg  =    varbit[]           ,
  function  =    sm_sc.fv_opr_not
);
-- select ~` array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] 
-- -----------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ~` (text[], text[]);
create operator ~` 
(
  leftarg     =    text[]           ,
  rightarg    =    text[]           ,
  function    =    sm_sc.fv_opr_is_regexp_match  ,
  commutator  =    ~`
);
-- select array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']] ~` array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
-- select array[array['abbbbbc122223'], array['abc123']] ~` array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
-- select array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']] ~` array[array[a.c], array[1.*?3]]
-- select array[array['abbbbbc122223', 'abc123']] ~` array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
-- select array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']] ~` array[array['1.3', 'a.*?c']]
-- select array['abbbbbc122223', 'abc123', 'abc123', 'ac13'] ~` array['a.c', '1.*?3', '1.3', 'a.*?c']
  -- 重载
  drop operator if exists ~` (text, text[]);
  create operator ~` 
  (
    leftarg     =    text             ,
    rightarg    =    text[]           ,
    function    =    sm_sc.fv_opr_is_regexp_match  ,
    commutator  =    ~`
  );
  -- select 'abbbbbc122223'::text ~` array[array['a.c', '1.*?3'], array['1.3', 'a.*?c']]
  drop operator if exists ~` (text[], text);
  create operator ~` 
  (
    leftarg     =    text[]           ,
    rightarg    =    text             ,
    function    =    sm_sc.fv_opr_is_regexp_match  ,
    commutator  =    ~`
  );
  -- select array[array['abbbbbc122223', 'abc123'], array['abc123', 'ac13']] ~` 'a.c'::text
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ~` (none, sm_sc.typ_l_complex[]);
create operator ~` 
(
  rightarg  =    sm_sc.typ_l_complex[]           ,
  function  =    sm_sc.fv_opr_conjugate
);
-- select ~` array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex]
-- select ~` array[array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex], array[(12.3, 0.0)::sm_sc.typ_l_complex, (0.0, 3.25)::sm_sc.typ_l_complex]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists @~` (none, sm_sc.typ_l_complex[]);
create operator @~` 
(
  rightarg  =    sm_sc.typ_l_complex[]           ,
  function  =    sm_sc.fv_opr_real
);
-- select @~` array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex]
-- select @~` array[array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex], array[(12.3, 0.0)::sm_sc.typ_l_complex, (0.0, 3.25)::sm_sc.typ_l_complex]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ~@` (none, sm_sc.typ_l_complex[]);
create operator ~@` 
(
  rightarg  =    sm_sc.typ_l_complex[]           ,
  function  =    sm_sc.fv_opr_imaginary
);
-- select ~@` array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex]
-- select ~@` array[array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex], array[(12.3, 0.0)::sm_sc.typ_l_complex, (0.0, 3.25)::sm_sc.typ_l_complex]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ~^` (none, sm_sc.typ_l_complex[]);
create operator ~^` 
(
  rightarg  =    sm_sc.typ_l_complex[]           ,
  function  =    sm_sc.fv_opr_conjugate_45
);
-- select ~^` array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex]
-- select ~^` array[array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex], array[(12.3, 0.0)::sm_sc.typ_l_complex, (0.0, 3.25)::sm_sc.typ_l_complex]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists #` (boolean[], boolean[]);
create operator #` 
(
  leftarg     =    boolean[]           ,
  rightarg    =    boolean[]           ,
  function    =    sm_sc.fv_opr_xor  ,
  commutator  =    #`
);
-- select array[array[true, false, true], array[false, false, true]] #` array[array[true, true, true], array[false, true, true]]
-- select array[array[true], array[false]] #` array[array[false, true, true], array[false, true, true]]
-- select array[array[true, false, true], array[false, false, true]] #` array[array[true], array[false]]
-- select array[array[true, false, true], array[false, false, true]] #` array[true, false, false]
-- select array[true, false, false] #` array[array[true, false, true], array[false, false, true]]
  -- 重载
  drop operator if exists #` (boolean, boolean[]);
  create operator #` 
  (
    leftarg     =    boolean             ,
    rightarg    =    boolean[]           ,
    function    =    sm_sc.fv_opr_xor  ,
    commutator  =    #`
  );
  -- select true #` array[array[true, false, true], array[false, true, true]]
  drop operator if exists #` (boolean[], boolean);
  create operator #` 
  (
    leftarg     =    boolean[]           ,
    rightarg    =    boolean             ,
    function    =    sm_sc.fv_opr_xor  ,
    commutator  =    #`
  );
  -- select array[array[true, false, true], array[false, false, true]] #` false

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists #` (varbit[], varbit[]);
create operator #` 
(
  leftarg     =    varbit[]           ,
  rightarg    =    varbit[]           ,
  function    =    sm_sc.fv_opr_xor  ,
  commutator  =    #`
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] #` array[array[B'001', B'010', B'100'], array[B'110', B'011', B'110']]
-- select array[array[B'101'], array[B'001']] #` array[array[B'100', B'101', B'010'], array[B'100', B'000', B'010']]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] #` array[array[B'100'], array[B'100']]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] #` array[B'010', B'100', B'100']
-- select array[B'010', B'100', B'100'] #` array[array[B'110', B'010', B'100'], array[B'100', B'110', B'001']]
  -- 重载
  drop operator if exists #` (varbit, varbit[]);
  create operator #` 
  (
    leftarg     =    varbit             ,
    rightarg    =    varbit[]           ,
    function    =    sm_sc.fv_opr_xor  ,
    commutator  =    #`
  );
  -- select B'001' #` array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']]
  drop operator if exists #` (varbit[], varbit);
  create operator #` 
  (
    leftarg     =    varbit[]           ,
    rightarg    =    varbit             ,
    function    =    sm_sc.fv_opr_xor  ,
    commutator  =    #`
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] # B'001'
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists !#` (boolean[], boolean[]);
create operator !#` 
(
  leftarg     =    boolean[]           ,
  rightarg    =    boolean[]           ,
  function    =    sm_sc.fv_opr_xnor  ,
  commutator  =    !#`
);
-- select array[array[true, false, true], array[false, false, true]] !#` array[array[true, true, true], array[false, true, true]]
-- select array[array[true], array[false]] !#` array[array[false, true, true], array[false, true, true]]
-- select array[array[true, false, true], array[false, false, true]] !#` array[array[true], array[false]]
-- select array[array[true, false, true], array[false, false, true]] !#` array[true, false, false]
-- select array[true, false, false] !#` array[array[true, false, true], array[false, false, true]]
  -- 重载
  drop operator if exists !#` (boolean, boolean[]);
  create operator !#` 
  (
    leftarg     =    boolean             ,
    rightarg    =    boolean[]           ,
    function    =    sm_sc.fv_opr_xnor  ,
    commutator  =    !#`
  );
  -- select true !#` array[array[true, false, true], array[false, true, true]]
  drop operator if exists !#` (boolean[], boolean);
  create operator !#` 
  (
    leftarg     =    boolean[]           ,
    rightarg    =    boolean             ,
    function    =    sm_sc.fv_opr_xnor  ,
    commutator  =    !#`
  );
  -- select array[array[true, false, true], array[false, false, true]] !#` false

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists !#` (varbit[], varbit[]);
create operator !#` 
(
  leftarg     =    varbit[]           ,
  rightarg    =    varbit[]           ,
  function    =    sm_sc.fv_opr_xnor  ,
  commutator  =    !#`
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] !#` array[array[B'001', B'010', B'100'], array[B'110', B'011', B'110']]
-- select array[array[B'101'], array[B'001']] !#` array[array[B'100', B'101', B'010'], array[B'100', B'000', B'010']]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] !#` array[array[B'100'], array[B'100']]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] !#` array[B'010', B'100', B'100']
-- select array[B'010', B'100', B'100'] !#` array[array[B'110', B'010', B'100'], array[B'100', B'110', B'001']]
  -- 重载
  drop operator if exists !#` (varbit, varbit[]);
  create operator !#` 
  (
    leftarg     =    varbit             ,
    rightarg    =    varbit[]           ,
    function    =    sm_sc.fv_opr_xnor  ,
    commutator  =    !#`
  );
  -- select B'001' !#` array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']]
  drop operator if exists !#` (varbit[], varbit);
  create operator !#` 
  (
    leftarg     =    varbit[]           ,
    rightarg    =    varbit             ,
    function    =    sm_sc.fv_opr_xnor  ,
    commutator  =    !#`
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] !#` B'001'
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ||` (anyarray, anyarray);
create operator ||` 
(
  leftarg     =    anyarray           ,
  rightarg    =    anyarray           ,
  function    =    sm_sc.fv_opr_concat  ,
  commutator  =    ||`
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] ||` array[array[B'001', B'010', B'100'], array[B'110', B'011', B'110']]
-- select array[array[B'101'], array[B'001']] ||` array[array[B'100', B'101', B'010'], array[B'100', B'000', B'010']]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] ||` array[array[B'100'], array[B'100']]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] ||` array[B'010', B'100', B'100']
-- select array[B'010', B'100', B'100'] ||` array[array[B'110', B'010', B'100'], array[B'100', B'110', B'001']]
  -- 重载
  drop operator if exists ||` (anyelement, anyarray);
  create operator ||` 
  (
    leftarg     =    anyelement             ,
    rightarg    =    anyarray           ,
    function    =    sm_sc.fv_opr_concat  ,
    commutator  =    ||`
  );
  -- select B'001' ||` array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']]
  drop operator if exists ||` (anyarray, anyelement);
  create operator ||` 
  (
    leftarg     =    anyarray           ,
    rightarg    =    anyelement             ,
    function    =    sm_sc.fv_opr_concat  ,
    commutator  =    ||`
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] ||` B'001'
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <<` (varbit[], int[]);
create operator <<` 
(
  leftarg     =    varbit[]           ,
  rightarg    =    int[]           ,
  function    =    sm_sc.fv_opr_shift_left
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] <<` array[array[1, 2, -1], array[-2, 2, 1]]
-- select array[array[B'101'], array[B'001']] <<` array[array[1, 2, -1], array[-2, 2, 1]]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] <<` array[array[2], array[-1]]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] <<` array[1, -2, -1]
-- select array[B'010', B'100', B'100'] <<` array[array[1, 2, -1], array[-2, 2, 1]]
  -- 重载
  drop operator if exists <<` (varbit, int[]);
  create operator <<` 
  (
    leftarg     =    varbit             ,
    rightarg    =    int[]           ,
    function    =    sm_sc.fv_opr_shift_left
  );
  -- select B'001' <<` array[array[1, 2, -1], array[-2, 2, 1]]
  drop operator if exists <<` (varbit[], int);
  create operator <<` 
  (
    leftarg     =    varbit[]           ,
    rightarg    =    int             ,
    function    =    sm_sc.fv_opr_shift_left
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] <<` 2
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >>` (varbit[], int[]);
create operator >>` 
(
  leftarg     =    varbit[]           ,
  rightarg    =    int[]           ,
  function    =    sm_sc.fv_opr_shift_right
);
-- select array[array[B'101', B'110', B'011'], array[B'010', B'101', B'111']] >>` array[array[1, 2, -1], array[-2, 2, 1]]
-- select array[array[B'101'], array[B'001']] >>` array[array[1, 2, -1], array[-2, 2, 1]]
-- select array[array[B'100', B'110', B'100'], array[B'100', B'001', B'010']] >>` array[array[2], array[-1]]
-- select array[array[B'100', B'100', B'110'], array[B'001', B'010', B'100']] >>` array[1, -2, -1]
-- select array[B'010', B'100', B'100'] >>` array[array[1, 2, -1], array[-2, 2, 1]]
  -- 重载
  drop operator if exists >>` (varbit, int[]);
  create operator >>` 
  (
    leftarg     =    varbit             ,
    rightarg    =    int[]           ,
    function    =    sm_sc.fv_opr_shift_right
  );
  -- select B'001' >>` array[array[1, 2, -1], array[-2, 2, 1]]
  drop operator if exists >>` (varbit[], int);
  create operator >>` 
  (
    leftarg     =    varbit[]           ,
    rightarg    =    int             ,
    function    =    sm_sc.fv_opr_shift_right
  );
  -- select array[array[B'110', B'010', B'100'], array[B'101', B'100', B'110']] >>` 2
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists == (anyarray, anyarray);
create operator == 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_is_equal
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] == array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] == array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] == array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] == array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] == array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists == (anyarray, anyelement);
  create operator == 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_equal
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] == 12.5
  drop operator if exists == (anyelement, anyarray);
  create operator ==
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_equal
  );
  -- select (-15.6) == array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ==` (float[], float[]);
create operator ==` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_is_equal_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] ==` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] ==` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists ==` (float[], float);
  create operator ==` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] ==` 12.5
  drop operator if exists ==` (float, float[]);
  create operator ==` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
  -- select (-15.6) ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ==` (decimal[], decimal[]);
create operator ==` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_is_equal_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] ==` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] ==` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists ==` (decimal[], decimal);
  create operator ==` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] ==` 12.5
  drop operator if exists ==` (decimal, decimal[]);
  create operator ==` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
  -- select (-15.6) ==` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ==` (anyarray, anyarray);
create operator ==` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_is_equal
);
  drop operator if exists ==` (anyarray, anyelement);
  create operator ==` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_equal
  );
  drop operator if exists ==` (anyelement, anyarray);
  create operator ==` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_equal
  );
drop operator if exists ==` (bigint[], bigint[]);
create operator ==` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_is_equal_py
);
  -- 重载
  drop operator if exists ==` (bigint[], bigint);
  create operator ==` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
  drop operator if exists ==` (bigint, bigint[]);
  create operator ==` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists ==` (int[], int[]);
create operator ==` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_is_equal_py
);
  -- 重载
  drop operator if exists ==` (int[], int);
  create operator ==` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
  drop operator if exists ==` (int, int[]);
  create operator ==` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_is_equal_py
  );
-- select
--   array[[12,9,2]] ==` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] ==` array[[3,3,2]]::int[]   
-- ,               12 ==` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] ==`              3::bigint  
-- ,               12 ==` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] ==`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
-- -- -- -- 系统已经占用小于号的定义，返回 bool
-- -- -- -- drop operator if exists < (anyarray, anyarray);
-- -- -- create operator < 
-- -- -- (
-- -- --   leftarg   =    anyarray           ,
-- -- --   rightarg  =    anyarray           ,
-- -- --   function  =    sm_sc.fv_opr_is_less
-- -- -- );
-- -- -- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] < array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] < array[12.5, 21.26, 43.6]
-- -- -- -- select array[32.5, 1.26, 33.6] < array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[32.5], array[33.6]] < array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] < array[array[32.5], array[33.6]]
  -- 重载
  -- drop operator if exists < (anyarray, anyelement);
  create operator < 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_less
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] < 12.5
  -- drop operator if exists < (anyelement, anyarray);
  create operator < 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_less
  );
  -- select (-15.6) < array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <` (float[], float[]);
create operator <` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_is_less_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] <` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists <` (float[], float);
  create operator <` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <` 12.5
  drop operator if exists <` (float, float[]);
  create operator <` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
  -- select (-15.6) <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <` (decimal[], decimal[]);
create operator <` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_is_less_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] <` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists <` (decimal[], decimal);
  create operator <` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <` 12.5
  drop operator if exists <` (decimal, decimal[]);
  create operator <` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
  -- select (-15.6) <` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <` (anyarray, anyarray);
create operator <` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_is_less
);
  drop operator if exists <` (anyarray, anyelement);
  create operator <` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_less
  );
  drop operator if exists <` (anyelement, anyarray);
  create operator <` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_less
  );
drop operator if exists <` (bigint[], bigint[]);
create operator <` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_is_less_py
);
  -- 重载
  drop operator if exists <` (bigint[], bigint);
  create operator <` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
  drop operator if exists <` (bigint, bigint[]);
  create operator <` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <` (int[], int[]);
create operator <` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_is_less_py
);
  -- 重载
  drop operator if exists <` (int[], int);
  create operator <` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
  drop operator if exists <` (int, int[]);
  create operator <` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_is_less_py
  );
-- select
--   array[[12,9,18]] <` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] <` array[[3,3,2]]::int[]   
-- ,               12 <` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] <`              3::bigint  
-- ,               12 <` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] <`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
-- -- -- -- 系统已经占用大于号的定义，返回 bool
-- -- -- drop operator if exists > (anyarray, anyarray);
-- -- -- create operator > 
-- -- -- (
-- -- --   leftarg   =    anyarray           ,
-- -- --   rightarg  =    anyarray           ,
-- -- --   function  =    sm_sc.fv_opr_is_greater
-- -- -- );
-- -- -- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] > array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] > array[12.5, 21.26, 43.6]
-- -- -- -- select array[32.5, 1.26, 33.6] > array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[32.5], array[33.6]] > array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] > array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists > (anyarray, anyelement);
  create operator > 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_greater
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] > 12.5
  drop operator if exists > (anyelement, anyarray);
  create operator > 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_greater
  );
  -- select (-15.6) > array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >` (float[], float[]);
create operator >` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_is_greater_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] >` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists >` (float[], float);
  create operator >` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >` 12.5
  drop operator if exists >` (float, float[]);
  create operator >` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
  -- select (-15.6) >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >` (decimal[], decimal[]);
create operator >` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_is_greater_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] >` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists >` (decimal[], decimal);
  create operator >` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >` 12.5
  drop operator if exists >` (decimal, decimal[]);
  create operator >` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
  -- select (-15.6) >` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >` (anyarray, anyarray);
create operator >` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_is_greater
);
  drop operator if exists >` (anyarray, anyelement);
  create operator >` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_greater
  );
  drop operator if exists >` (anyelement, anyarray);
  create operator >` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_greater
  );
drop operator if exists >` (bigint[], bigint[]);
create operator >` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_is_greater_py
);
  -- 重载
  drop operator if exists >` (bigint[], bigint);
  create operator >` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
  drop operator if exists >` (bigint, bigint[]);
  create operator >` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >` (int[], int[]);
create operator >` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_is_greater_py
);
  -- 重载
  drop operator if exists >` (int[], int);
  create operator >` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
  drop operator if exists >` (int, int[]);
  create operator >` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_is_greater_py
  );
-- select
--   array[[12,9,18]] >` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] >` array[[3,3,2]]::int[]   
-- ,               12 >` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] >`              3::bigint  
-- ,               12 >` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] >`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
-- -- -- -- 系统已经占用小于等于号的定义，返回 bool
-- drop operator if exists <= (anyarray, anyarray);
-- create operator <=` 
-- (
--   leftarg   =    anyarray           ,
--   rightarg  =    anyarray           ,
--   function  =    sm_sc.fv_opr_is_less_ex
-- );
-- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <= array[12.5, 21.26, 43.6]
-- -- select array[32.5, 1.26, 33.6] <= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- select array[array[32.5], array[33.6]] <= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] <= array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists <= (anyarray, anyelement);
  create operator <= 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_less_ex
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <= 12.5
  drop operator if exists <= (anyelement, anyarray);
  create operator <= 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_less_ex
  );
  -- select (-15.6) <= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <=` (float[], float[]);
create operator <=` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_is_less_ex_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <=` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] <=` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists <=` (float[], float);
  create operator <=` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <=` 12.5
  drop operator if exists <=` (float, float[]);
  create operator <=` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
  -- select (-15.6) <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <=` (decimal[], decimal[]);
create operator <=` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_is_less_ex_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <=` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] <=` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists <=` (decimal[], decimal);
  create operator <=` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <=` 12.5
  drop operator if exists <=` (decimal, decimal[]);
  create operator <=` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
  -- select (-15.6) <=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <=` (anyarray, anyarray);
create operator <=` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_is_less_ex
);
  drop operator if exists <=` (anyarray, anyelement);
  create operator <=` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_less_ex
  );
  drop operator if exists <=` (anyelement, anyarray);
  create operator <=` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_less_ex
  );
drop operator if exists <=` (bigint[], bigint[]);
create operator <=` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_is_less_ex_py
);
  -- 重载
  drop operator if exists <=` (bigint[], bigint);
  create operator <=` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
  drop operator if exists <=` (bigint, bigint[]);
  create operator <=` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <=` (int[], int[]);
create operator <=` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_is_less_ex_py
);
  -- 重载
  drop operator if exists <=` (int[], int);
  create operator <=` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
  drop operator if exists <=` (int, int[]);
  create operator <=` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_is_less_ex_py
  );
-- select
--   array[[12,9,18]] <=` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] <=` array[[3,3,2]]::int[]   
-- ,               12 <=` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] <=`              3::bigint  
-- ,               12 <=` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] <=`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
-- -- -- -- 系统已经占用大于等于号的定义，返回 bool
-- -- -- drop operator if exists >= (anyarray, anyarray);
-- -- -- create operator >=` 
-- -- -- (
-- -- --   leftarg   =    anyarray           ,
-- -- --   rightarg  =    anyarray           ,
-- -- --   function  =    sm_sc.fv_opr_is_greater_ex
-- -- -- );
-- -- -- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >= array[12.5, 21.26, 43.6]
-- -- -- -- select array[32.5, 1.26, 33.6] >= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[32.5], array[33.6]] >= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -- -- -- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] >= array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists >= (anyarray, anyelement);
  create operator >= 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_greater_ex
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >= 12.5
  drop operator if exists >= (anyelement, anyarray);
  create operator >= 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_greater_ex
  );
  -- select (-15.6) >= array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >=` (float[], float[]);
create operator >=` 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_is_greater_ex_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >=` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] >=` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists >=` (float[], float);
  create operator >=` 
  (
    leftarg   =    float[]           ,
    rightarg  =    float           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >=` 12.5
  drop operator if exists >=` (float, float[]);
  create operator >=` 
  (
    leftarg   =    float           ,
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
  -- select (-15.6) >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >=` (decimal[], decimal[]);
create operator >=` 
(
  leftarg   =    decimal[]           ,
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_is_greater_ex_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >=` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] >=` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists >=` (decimal[], decimal);
  create operator >=` 
  (
    leftarg   =    decimal[]           ,
    rightarg  =    decimal           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] >=` 12.5
  drop operator if exists >=` (decimal, decimal[]);
  create operator >=` 
  (
    leftarg   =    decimal           ,
    rightarg  =    decimal[]           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
  -- select (-15.6) >=` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >=` (anyarray, anyarray);
create operator >=` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_is_greater_ex
);
  drop operator if exists >=` (anyarray, anyelement);
  create operator >=` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_is_greater_ex
  );
  drop operator if exists >=` (anyelement, anyarray);
  create operator >=` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_is_greater_ex
  );
drop operator if exists >=` (bigint[], bigint[]);
create operator >=` 
(
  leftarg   =    bigint[]           ,
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_is_greater_ex_py
);
  -- 重载
  drop operator if exists >=` (bigint[], bigint);
  create operator >=` 
  (
    leftarg   =    bigint[]           ,
    rightarg  =    bigint           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
  drop operator if exists >=` (bigint, bigint[]);
  create operator >=` 
  (
    leftarg   =    bigint           ,
    rightarg  =    bigint[]           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists >=` (int[], int[]);
create operator >=` 
(
  leftarg   =    int[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_is_greater_ex_py
);
  -- 重载
  drop operator if exists >=` (int[], int);
  create operator >=` 
  (
    leftarg   =    int[]           ,
    rightarg  =    int           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
  drop operator if exists >=` (int, int[]);
  create operator >=` 
  (
    leftarg   =    int           ,
    rightarg  =    int[]           ,
    function  =    sm_sc.fv_opr_is_greater_ex_py
  );
-- select
--   array[[12,9,18]] >=` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] >=` array[[3,3,2]]::int[]   
-- ,               12 >=` array[[3,3,2]]::bigint[]
-- , array[[12,9,18]] >=`              3::bigint  
-- ,               12 >=` array[[3,3,2]]::   int[]
-- , array[[12,9,18]] >=`              3::   int  
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists <>` (anyarray, anyarray);
create operator <>` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_compare
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <>` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] <>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] <>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] <>` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists <>` (anyarray, anyelement);
  create operator <>` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_compare
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] <>` 12.5
  drop operator if exists <>` (anyelement, anyarray);
  create operator <>` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_compare
  );
  -- select (-15.6) <>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]

drop operator if exists <>` (none, anyarray);
create operator <>` 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_sign
);
drop operator if exists <>` (none, anyarray);
create operator <>` 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_sign
);
-- select <>` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] 
-- select <>` - array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] 
-- select <>` (- array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] )
drop operator if exists <>` (none, decimal[]);
create operator <>` 
(
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_sign_py
);
drop operator if exists <>` (none, float[]);
create operator <>` 
(
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_sign_py
);
drop operator if exists <>` (none, bigint[]);
create operator <>` 
(
  rightarg  =    bigint[]           ,
  function  =    sm_sc.fv_opr_sign_py
);
drop operator if exists <>` (none, int[]);
create operator <>` 
(
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_sign_py
);
-- select 
--   <>` array[1,2,3] :: decimal[]
-- , <>` array[1,2,3] :: float[]
-- , <>` array[1,2,3] :: bigint[]
-- , <>` array[1,2,3] :: int[]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists @<` (anyarray, anyarray);
create operator @<` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_least  ,
  commutator  =    @<`
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] @<` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] @<` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] @<` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] @<` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] @<` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists @<` (anyarray, anyelement);
  create operator @<` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_least  ,
    commutator  =    @<`
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] @<` 12.5
  drop operator if exists @<` (anyelement, anyarray);
  create operator @<` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_least  ,
    commutator  =    @<`
  );
  -- select (-15.6) @<` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

-- 支持单列延拓情况，即 array_length(rightarg, 2) = 1
drop operator if exists @>` (anyarray, anyarray);
create operator @>` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_greatest  ,
  commutator  =    @>`
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] @>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] @>` array[12.5, 21.26, 43.6]
-- select array[32.5, 1.26, 33.6] @>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[33.6]] @>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]] @>` array[array[32.5], array[33.6]]
  -- 重载
  drop operator if exists @>` (anyarray, anyelement);
  create operator @>` 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_greatest,
    commutator  =    @>`
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] @>` 12.5
  drop operator if exists @>` (anyelement, anyarray);
  create operator @>` 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_greatest,
    commutator  =    @>`
  );
  -- select (-15.6) @>` array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists @|`  (none, anyarray);
create operator @|`  
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_abs
);
-- select @|`  array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] 
-- select @|`  -` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] 
-- select @|`  (-` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] )
-- select @|`  (-` array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] )
-- select @|`  (-` array[[row(32.5, -1.5) :: sm_sc.typ_l_complex, row(1.26, -1.5) :: sm_sc.typ_l_complex, row(33.6, -1.5) :: sm_sc.typ_l_complex]
--                    , [row(-9.1, -1.5) :: sm_sc.typ_l_complex, row(8.6, -1.5) :: sm_sc.typ_l_complex, row(4.69, -1.5) :: sm_sc.typ_l_complex]] )
-- -------------------------------------------------------------------------------------------------------

drop operator if exists @`  (none, decimal[]);
create operator @`  
(
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_norm
);
-- select @`  array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] 
-- -------------------------------------------------------------------------------------------------------

drop operator if exists @`  (none, float[]);
create operator @`  
(
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_norm
);
-- select @`  -` (array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] :: float[])

-- -------------------------------------------------------------------------------------------------------

drop operator if exists @`  (none, sm_sc.typ_l_complex[]);
create operator @`  
(
  rightarg  =    sm_sc.typ_l_complex[]           ,
  function  =    sm_sc.fv_opr_norm
);
-- select @`  (-` array[[row(32.5, -1.5) :: sm_sc.typ_l_complex, row(1.26, -1.5) :: sm_sc.typ_l_complex, row(33.6, -1.5) :: sm_sc.typ_l_complex]
--                    , [row(-9.1, -1.5) :: sm_sc.typ_l_complex, row(8.6, -1.5) :: sm_sc.typ_l_complex, row(4.69, -1.5) :: sm_sc.typ_l_complex]] )
-- -------------------------------------------------------------------------------------------------------


drop operator if exists ~=` (anyarray, int);
create operator ~=` 
(
  leftarg   =    anyarray           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_round
);
-- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ~=` 2
  -- 重载
  -- -- -- -- -- pg14 好像不支持 右单目
  -- -- drop operator if exists ~=` (anyarray, none);
  -- -- create operator ~=` 
  -- -- (
  -- --   leftarg   =    anyarray           ,
  -- --   function  =    sm_sc.fv_opr_round
  -- -- );
  -- -- -- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ~=`
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ~<` (anyarray, int);
create operator ~<` 
(
  leftarg   =    anyarray           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_floor
);
-- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ~<` 2
  -- 重载
  -- -- -- -- -- pg14 好像不支持 右单目
  -- -- drop operator if exists ~<` (decimal[], none);
  -- -- create operator ~<` 
  -- -- (
  -- --   leftarg   =    decimal[]           ,
  -- --   function  =    sm_sc.fv_opr_floor
  -- -- );
  -- -- -- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ~<`
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ~>` (anyarray, int);
create operator ~>` 
(
  leftarg   =    anyarray           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_ceil
);
-- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ~>` 2
  -- 重载
  -- -- -- -- -- pg14 好像不支持 右单目
  -- -- drop operator if exists ~>` (decimal[], none);
  -- -- create operator ~>` 
  -- -- (
  -- --   leftarg   =    decimal[]           ,
  -- --   function  =    sm_sc.fv_opr_ceil
  -- -- );
  -- -- -- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ~>`
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ><` (anyarray, int);
create operator ><` 
(
  leftarg   =    anyarray           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_trunc
);
-- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ><` 2
  -- 重载
  -- -- -- -- -- pg14 好像不支持 右单目
  -- -- drop operator if exists ><` (decimal[], none);
  -- -- create operator ><` 
  -- -- (
  -- --   leftarg   =    decimal[]           ,
  -- --   function  =    sm_sc.fv_opr_trunc
  -- -- );
  -- -- -- select array[array[32.55345, 1.264834, 33.68373], array[-9.133, 8.6387, 4.6921]] ><`
-- -------------------------------------------------------------------------------------------------------

drop operator if exists ||~| (none, anyarray);
create operator ||~| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_mirror_heigh
);
-- select ||~| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |-~| (none, anyarray);
create operator |-~| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_mirror_width
);
-- select |-~| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |+~| (anyarray, int);
create operator |+~| 
(
  leftarg   =    anyarray           ,
  rightarg  =    int                ,
  function  =    sm_sc.fv_opr_mirror
);
drop operator if exists |+~| (float[], int);
create operator |+~| 
(
  leftarg   =    float[]           ,
  rightarg  =    int                ,
  function  =    sm_sc.fv_opr_mirror_py
);
drop operator if exists |+~| (decimal[], int);
create operator |+~| 
(
  leftarg   =    decimal[]           ,
  rightarg  =    int                ,
  function  =    sm_sc.fv_opr_mirror_py
);
-- select array[[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69]], [[-32.5, -1.26, 33.6], [9.1, 8.6, -4.69]]] :: float[] |+~| 2
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |<~| (none, anyarray);
create operator |<~| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_turn_width_heigh_90
);
-- select |<~| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |>~| (none, anyarray);
create operator |>~| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_turn_heigh_width_90
);
-- select |>~| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |*~| (anyarray, int[2]);
create operator |*~| 
(
  leftarg   =    anyarray           ,
  rightarg  =    int[2]             ,
  function  =    sm_sc.fv_opr_turn_90
);
-- select array[[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69]], [[-32.5, -1.26, 33.6], [9.1, 8.6, -4.69]]] |*~| array[2, 3]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |~~| (none, anyarray);
create operator |~~| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_turn_heigh_width_180
);
-- select |~~| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |~~| (anyarray, int[2]);
create operator |~~| 
(
  leftarg   =    anyarray           ,
  rightarg  =    int[2]             ,
  function  =    sm_sc.fv_opr_turn_180
);
-- select array[[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69]], [[-32.5, -1.26, 33.6], [9.1, 8.6, -4.69]]] |~~| array[2, 3]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |^~| (none, anyarray);
create operator |^~| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_transpose
);
drop operator if exists |^~| (none, float[]);
create operator |^~| 
(
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_transpose_py
);
drop operator if exists |^~| (none, decimal[]);
create operator |^~| 
(
  rightarg  =    decimal[]           ,
  function  =    sm_sc.fv_opr_transpose_py
);
-- select |^~| (array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] :: float[])
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |^~| (anyarray, int[2]);
create operator |^~| 
(
  leftarg   =    anyarray         ,
  rightarg  =    int[2]           ,
  function  =    sm_sc.fv_opr_transpose_nd
);
-- select array[[[1.3, -8.8, 2.25],[2.2, -3.0, -0.95]],[[-1.3, 8.8, -2.25],[-2.2, 3.0, 0.95]]] |^~| array[2, 3]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |^~`| (none, sm_sc.typ_l_complex[]);
create operator |^~`| 
(
  rightarg  =    sm_sc.typ_l_complex[]           ,
  function  =    sm_sc.fv_opr_conjugate_i
);
-- -- select |^~`| array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex]
-- select |^~`| array[array[(12.3, -25.1)::sm_sc.typ_l_complex, (-2.56, 3.25)::sm_sc.typ_l_complex, (-1, 3)::sm_sc.typ_l_complex], array[(-3, 1)::sm_sc.typ_l_complex, (12.3, 0.0)::sm_sc.typ_l_complex, (0.0, 3.25)::sm_sc.typ_l_complex]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |~^| (none, anyarray);
create operator |~^| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_transpose_i
);
-- select |~^| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] 
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |-|| (anyarray, anyarray);
create operator |-|| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_concat_heigh
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |-|| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[32.5, 1.26, 33.6] |-|| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |-|| array[12.5, 21.26, 43.6]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |-|| array[array[12.5], array[21.26], array[43.6]]
-- select array[array[12.5], array[21.26], array[43.6]] |-|| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists |-|| (anyelement, anyarray);
  create operator |-|| 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_concat_heigh
  );
  -- select 12.8 |-|| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists |-|| (anyarray, anyelement);
  create operator |-|| 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_concat_heigh
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |-|| (-12.8)
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |||| (anyarray, anyarray);
create operator |||| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_concat_width
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |||| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5], array[1.26]] -| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |||| array[array[12.5], array[21.26]]
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |||| array[12.5, 21.26]
-- select array[12.5, 21.26] |||| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- 重载
  drop operator if exists |||| (anyelement, anyarray);
  create operator |||| 
  (
    leftarg   =    anyelement           ,
    rightarg  =    anyarray           ,
    function  =    sm_sc.fv_opr_concat_width
  );
  -- select 4.25 |||| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
  drop operator if exists |||| (anyarray, anyelement);
  create operator |||| 
  (
    leftarg   =    anyarray           ,
    rightarg  =    anyelement           ,
    function  =    sm_sc.fv_opr_concat_width
  );
  -- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |||| 4.25
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |><| (float[], int[]);
create operator |><| 
(
  leftarg   =    float[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_reshape_py
);
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5,7]) |><| array[5,3,7,2])
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5,7]) |><| array[7,10,3])
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5,7]) |><| array[70,3])
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5,7]) |><| array[210])

-- select array_dims(sm_sc.fv_new_rand(array[14,3,5]) |><| array[2,3,5,7])
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5]) |><| array[1,3,10])
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5]) |><| array[30,1])
-- select array_dims(sm_sc.fv_new_rand(array[2,3,5]) |><| array[30])

-- select array_dims(sm_sc.fv_new_rand(array[6,35]) |><| array[7,3,5,2])
-- select array_dims(sm_sc.fv_new_rand(array[6,5]) |><| array[3,5,2])
-- select array_dims(sm_sc.fv_new_rand(array[6,5]) |><| array[3,10])
-- select array_dims(sm_sc.fv_new_rand(array[6,5]) |><| array[30])

-- select array_dims(sm_sc.fv_new_rand(array[210]) |><| array[2,3,5,7])
-- select array_dims(sm_sc.fv_new_rand(array[210]) |><| array[6,5,7])
-- select array_dims(sm_sc.fv_new_rand(array[210]) |><| array[30,7])
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |`| (anyarray, anyarray);
create operator |`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_prod_inner  ,
  commutator  =    |`|
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |`| array[array[12.5, 21.26, 43.6], array[-19.1, 28.6, 64.69]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |**| (float[], float[]);
create operator |**| 
(
  leftarg   =    float[]           ,
  rightarg  =    float[]           ,
  function  =    sm_sc.fv_opr_prod_mx_py
);
-- select array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]] |**| array[array[12.5, 21.26], array[-19.1, 43.6], array[28.6, 64.69]]
-- select array[array[12.5, 21.26], array[-19.1, 43.6], array[28.6, 64.69]] |**| array[array[32.5, 1.26, 33.6], array[-9.1, 8.6, 4.69]]
  -- -- -- -- -- pg14 好像不支持 右单目
  -- -- drop operator if exists |**| (float[], none);
  -- -- create operator |**| 
  -- -- (
  -- --   leftarg   =    float[]           ,
  -- --   function  =    sm_sc.fv_opr_prod_mx_left
  -- -- );
  -- -- -- select array[array[1.0000,2.0000,3.0000], array[4.0000,5.0000,6.0000]] |**|
  drop operator if exists |**| (none, float[]);
  create operator |**| 
  (
    rightarg  =    float[]           ,
    function  =    sm_sc.fv_opr_prod_mx_right
  );
  -- select |**| array[array[1.0000,2.0000,3.0000], array[4.0000,5.0000,6.0000]]
-- -------------------------------------------------------------------------------------------------------

drop operator if exists |@=| (none, anyarray);
create operator |@=| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_coalesce
);
-- select |@=| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@&`| (none, boolean[]);
create operator |@&`| 
(
  rightarg  =    boolean[]           ,
  function  =    sm_sc.fv_aggr_slice_and
);
-- select |@&`| array[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]

drop operator if exists |@&`| (none, bit[]);
create operator |@&`| 
(
  rightarg  =    bit[]           ,
  function  =    sm_sc.fv_aggr_slice_and
);
-- select |@&`| array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]

drop operator if exists |@|`| (none, boolean[]);
create operator |@|`| 
(
  rightarg  =    boolean[]           ,
  function  =    sm_sc.fv_aggr_slice_or
);
-- select |@|`| array[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]]

drop operator if exists |@|`| (none, bit[]);
create operator |@|`| 
(
  rightarg  =    bit[]           ,
  function  =    sm_sc.fv_aggr_slice_or
);
-- select |@|`| array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']]

drop operator if exists |@||| (none, anyarray);
create operator |@||| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_concat
);
-- select |@||| array[[[[B'010', B'011', B'010', B'011', B'101', B'011', B'010', B'011']]]]

drop operator if exists |@/<| (none, anyarray);
create operator |@/<| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_median
);
-- select |@/<| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@/>| (none, anyarray);
create operator |@/>| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_mode
);
-- select |@/>| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@%`| (none, anyarray);
create operator |@%`| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_stddev_samp
);
-- select |@%`| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@%| (none, anyarray);
create operator |@%| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_stddev_pop
);
-- select |@%| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@#`| (none, anyarray);
create operator |@#`| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_var_samp
);
-- select |@#`| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@#| (none, anyarray);
create operator |@#| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_var_pop
);
-- select |@#| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@/=| (none, anyarray);
create operator |@/=| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_ptp
);
-- select |@/=| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@<| (none, anyarray);
create operator |@<| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_min
);
-- select |@<| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@>| (none, anyarray);
create operator |@>| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_max
);
-- select |@>| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@/| (none, anyarray);
create operator |@/| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_avg
);
-- select |@/| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@*| (none, anyarray);
create operator |@*| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_prod
);
-- select |@*| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@+| (none, anyarray);
create operator |@+| 
(
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_aggr_slice_sum
);
-- select |@+| array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]]

drop operator if exists |@=| (anyarray, int[]);
create operator |@=| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_coalesce
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@=| array[3, 1]

drop operator if exists |@&`| (boolean[], int[]);
create operator |@&`| 
(
  leftarg  =    boolean[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_and
);
-- select array[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]] |@&`| array[5, 2]

drop operator if exists |@&`| (bit[], int[]);
create operator |@&`| 
(
  leftarg  =    bit[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_and
);
-- select array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']] |@&`| array[5, 2]

drop operator if exists |@|`| (boolean[], int[]);
create operator |@|`| 
(
  leftarg  =    boolean[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_or
);
-- select array[[true, false, false, true], [false, true, false, true], [true, false, false, false], [false, false, true, true], [true, true, false, true]] |@|`| array[5, 2]

drop operator if exists |@|`| (bit[], int[]);
create operator |@|`| 
(
  leftarg  =    bit[]           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_or
);
-- select array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']] |@|`| array[5, 2]

drop operator if exists |@||| (anyarray, int[]);
create operator |@||| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_concat
);
-- select array[[B'010', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011'], [B'101', B'011', B'010', B'011']] |@||| array[5, 2]

drop operator if exists |@/<| (anyarray, int[]);
create operator |@/<| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_median
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@/<| array[3, 1]

drop operator if exists |@/>| (anyarray, int[]);
create operator |@/>| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_mode
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@/>| array[3, 1]

drop operator if exists |@%`| (anyarray, int[]);
create operator |@%`| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_stddev_samp
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@%`| array[3, 1]

drop operator if exists |@%| (anyarray, int[]);
create operator |@%| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_stddev_pop
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@%| array[3, 1]

drop operator if exists |@#`| (anyarray, int[]);
create operator |@#`| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_var_samp
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@#`| array[3, 1]

drop operator if exists |@#| (anyarray, int[]);
create operator |@#| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_var_pop
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@#| array[3, 1]

drop operator if exists |@/=| (anyarray, int[]);
create operator |@/=| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_ptp
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@/=| array[3, 1]

drop operator if exists |@<| (anyarray, int[]);
create operator |@<| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_min
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@<| array[3, 1]

drop operator if exists |@>| (anyarray, int[]);
create operator |@>| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_max
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@>| array[3, 1]

drop operator if exists |@/| (anyarray, int[]);
create operator |@/| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_avg
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@/| array[3, 1]

drop operator if exists |@*| (anyarray, int[]);
create operator |@*| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_prod
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@*| array[3, 1]

drop operator if exists |@+| (anyarray, int[]);
create operator |@+| 
(
  leftarg  =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_aggr_slice_sum
);
-- select array[[32.5, 1.26, 33.6], [-9.1, 8.6, 4.69], [-2.1, 3.6, -6.69]] |@+| array[3, 1]

-- 方阵幂运算
-- ---------------------------------------------------------------------------------------------------------------
drop operator if exists |`^| (anyarray, anyelement);
create operator |`^| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyelement           ,
  function  =    sm_sc.fv_opr_prod_inner_pow
);
-- select array[array[32.5, 1.26, 33.6], array[9.1, 8.6, 4.69]] |`^| (-1.5)

-- ---------------------------------------------------------------------------------------------------------------
drop operator if exists |*^| (float[], int);
create operator |*^| 
(
  leftarg   =    float[]           ,
  rightarg  =    int           ,
  function  =    sm_sc.fv_opr_prod_mx_pow
);
-- select array[array[32.5, 1.26], array[8.6, 4.69]] |*^| 7

-- 组播运算
-- ---------------------------------------------------------------------------------------------------------------
drop operator if exists +`| (anyarray, anyarray);
create operator +`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_add_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] +`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] +`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists -`| (anyarray, anyarray);
create operator -`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_sub_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] -`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] -`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists |-` (anyarray, anyarray);
create operator |-` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_sub_stride_1
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |-` array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]]
-- select array[0.7, 2.9, 0.68, 4.5] |-` array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists *`| (anyarray, anyarray);
create operator *`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_mul_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] *`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] *`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists /`| (anyarray, anyarray);
create operator /`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_div_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] /`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] /`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists |/` (anyarray, anyarray);
create operator |/` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_div_stride_1
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |/` array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]]
-- select array[0.7, 2.9, 0.68, 4.5] |/` array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists ^`| (anyarray, anyarray);
create operator ^`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_pow_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] ^`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] ^`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists |^` (anyarray, anyarray);
create operator |^` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_pow_stride_1
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |^` array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]]
-- select array[0.7, 2.9, 0.68, 4.5] |^` array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists ^!`| (anyarray, anyarray);
create operator ^!`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_log_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] ^!`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] ^!`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists |^!` (anyarray, anyarray);
create operator |^!` 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_log_stride_1
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |^!` array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]]
-- select array[0.7, 2.9, 0.68, 4.5] |^!` array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists %+`| (anyarray, anyarray);
create operator %+`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_add_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %+`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] %+`| array[0.7, 2.9, 0.68]

drop operator if exists %-`| (anyarray, anyarray);
create operator %-`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_sub_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %-`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69] %-`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists %*`| (anyarray, anyarray);
create operator %*`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_mul_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %*`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69] %*`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists %/`| (anyarray, anyarray);
create operator %/`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_div_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %/`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] %/`| array[0.7, 2.9, 0.68]

drop operator if exists %^`| (anyarray, anyarray);
create operator %^`| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_pow_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %^`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] %^`| array[0.7, 2.9, 4.5]

drop operator if exists %^!`| (anyarray, anyarray);
create operator %^!`|
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_log_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %^!`| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] %^!`| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists |-`% (anyarray, anyarray);
create operator |-`%
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_sub_stride_window
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |-`% array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]]
-- select array[0.7, 2.9, 0.68, 4.5] |-`% array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 4.69, 9.7]

drop operator if exists |/`% (anyarray, anyarray);
create operator |/`%
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_div_stride_window
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |/`% array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]]
-- select array[0.7, 0.68, 4.5] |/`% array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists |^`% (anyarray, anyarray);
create operator |^`%
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_pow_stride_window
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |^`% array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]]
-- select array[0.7, 2.9, 0.68, 4.5] |^`% array[2.5, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists |^!`% (anyarray, anyarray);
create operator |^!`%
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_de_log_stride_window
);
-- select array[[0.7, 2.9], [0.68, 4.5]] |^!`% array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]]
-- select array[0.7, 2.9, 4.5] |^!`% array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7]

drop operator if exists @>| (anyarray, int[]);
create operator @>|
(
  leftarg   =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_pool_max_stride_1
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] @>| array[2, 2]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] @>| array[2]

drop operator if exists @/| (anyarray, int[]);
create operator @/|
(
  leftarg   =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_pool_avg_stride_1
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] @/| array[2, 2]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] @/| array[2]

drop operator if exists @`| (anyarray, int[]);
create operator @`|
(
  leftarg   =    anyarray           ,
  rightarg  =    int[]           ,
  function  =    sm_sc.fv_opr_pool_none_stride_1
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] @`| array[2, 2]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] @`| array[2]

-- -- -- 等价于 fv_aggr_slice_max
-- -- drop operator if exists %@>| (anyarray, int[]);
-- -- create operator %@>|
-- -- (
-- --   leftarg   =    anyarray           ,
-- --   rightarg  =    int[]           ,
-- --   function  =    sm_sc.fv_opr_pool_max_stride_window
-- -- );
-- -- -- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %@>| array[2, 2]
-- -- -- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] %@>| array[2]

-- -- -- 等价于 fv_aggr_slice_avg
-- -- drop operator if exists %@/| (anyarray, int[]);
-- -- create operator %@/|
-- -- (
-- --   leftarg   =    anyarray           ,
-- --   rightarg  =    int[]           ,
-- --   function  =    sm_sc.fv_opr_pool_avg_stride_window
-- -- );
-- -- -- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %@/| array[2, 2]
-- -- -- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] %@/| array[2]

-- -- -- 等价于 fv_none 或直等于
-- -- drop operator if exists %@`| (anyarray, int[]);
-- -- create operator %@`|
-- -- (
-- --   leftarg   =    anyarray           ,
-- --   rightarg  =    int[]           ,
-- --   function  =    sm_sc.fv_opr_pool_none_stride_window
-- -- );
-- -- -- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %@`| array[2, 2]
-- -- -- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 9.7] %@`| array[2]

drop operator if exists **| (anyarray, anyarray);
create operator **| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_2d_stride_1
);
-- select array[[2.5, 1.26, 3.3], [8.6, 4.69, 9.7], [8.6, 4.69, 9.7]] **| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69, 9.7] **| array[0.7, 2.9, 0.68, 4.5]

drop operator if exists %**| (anyarray, anyarray);
create operator %**| 
(
  leftarg   =    anyarray           ,
  rightarg  =    anyarray           ,
  function  =    sm_sc.fv_opr_conv_2d_stride_window
);
-- select array[[2.5, 1.26, 3.3, 2.6], [8.6, 4.69, 9.7, 6.5], [8.6, 4.69, 9.7, 8.55]] %**| array[[0.7, 2.9], [0.68, 4.5]]
-- select array[2.5, 1.26, 3.3, 8.6, 4.69, 9.7, 8.6, 4.69] %**| array[0.7, 2.9, 0.68, 4.5]
